
	/*  1991, Bowers Development Corp.  */

#include <Types.h>
#include <Quickdraw.h>
#include <Controls.h>
#include <Dialogs.h>
#include <Events.h>
#include <Files.h>
#include <Lists.h>
#include <Menus.h>
#include <TextEdit.h>
#include <AppleEvents.h>

#include "Globals.h"	
#include "ResourceDefs.h"	
#include "Miscellany.h"	
#include "AEvent.h"	

#pragma segment AEvent


void InitializeAE()
{
	OSErr	errCode;

	errCode = AEInstallEventHandler (kCoreEventClass, kAEOpenApplication, (EventHandlerProcPtr) DoAEOpenApp, 0, false);
	errCode = AEInstallEventHandler (kCoreEventClass, kAEOpenDocuments, (EventHandlerProcPtr) DoAEOpenDoc, 0, false);
	errCode = AEInstallEventHandler (kCoreEventClass, kAEPrintDocuments, (EventHandlerProcPtr) DoAEPrintDoc, 0, false);
	errCode = AEInstallEventHandler (kCoreEventClass, kAEQuitApplication, (EventHandlerProcPtr) DoAEQuit, 0, false);

} /*InitializeAE */


void DoHighLevelEvent()
{
	OSErr	errCode;

		/*  check to see if the Apple event is of a user-defined class;	*/
		/*  if so, handle it. Otherwise, call AEProcessAppleEvent.		*/

	errCode = AEProcessAppleEvent (&curEvent);
	if( errCode == errAEEventNotHandled )
	{
		/*??	Acknowledge (CantHandleAEVTID);		??*/
	}

} /*DoHighLevelEvent*/


	/* This routine checks to verify that the caller has processed */
	/* all of the event's parameters. If no parameters are found */
	/* then it returns noErr. */

OSErr MyGotRequiredParams( const AppleEvent *theEvent )
{
	OSErr			errCode;
	Size			actualSize;
	DescType		returnedType;

	errCode = AEGetAttributePtr (theEvent, keyMissedKeywordAttr, typeWildCard,
									&returnedType, NULL, 0, &actualSize);

	if( errCode == errAEDescNotFound )
		errCode = noErr;	/*no parameters => no error*/
	else if( errCode == noErr)
	{			/*got a parameter => it wasn't handled */
		errCode = errAEEventNotHandled;
	}										/*AEGetAttributePtr failed*/

	return errCode;
} /*MyGotRequiredParams */


pascal OSErr DoAEOpenApp( AppleEvent *theEvent, AppleEvent *reply, long refCon )
{
#pragma unused (reply, refCon)

	OSErr			errCode;

	errCode = MyGotRequiredParams ( theEvent );
	if( errCode == noErr)
		OpenApp();

	return errCode;
} /*DoAEOpenApp */


pascal OSErr DoAEOpenDoc( AppleEvent *theEvent, AppleEvent *reply, long refCon )
{
#pragma unused (reply, refCon)

	OSErr			errCode;
	OSErr			ignoreErr;
	AEDescList		docList;
	long			itemsInList;
	long			index;
	AEKeyword		keyword;
	DescType		returnedType;
	Size			actualSize;
	FSSpec			myFSS;
	short			wdRefNum;

	errCode = AEGetParamDesc (theEvent, keyDirectObject, typeAEList, &docList);
	if( errCode == noErr)
	{
		errCode = MyGotRequiredParams ( theEvent );
		if( errCode == noErr)
		{
			errCode = AECountItems (&docList, &itemsInList);
			if( errCode == noErr)
			{
				for (index = 1; index <= itemsInList; index++)
				{
					errCode = AEGetNthPtr (&docList, index, typeFSS, &keyword,
							&returnedType, (Ptr) &myFSS, sizeof (myFSS), &actualSize);
					if( errCode == noErr)
					{
						errCode = OpenWD (myFSS.vRefNum, myFSS.parID, 0, &wdRefNum);
						if( errCode == noErr) {
							OpenDoc (myFSS.name, wdRefNum);
						}
					}
				} /*for*/
			}
		}
		ignoreErr = AEDisposeDesc (&docList);
	}

	return errCode;
} /*DoAEOpenDoc */


pascal OSErr DoAEPrintDoc( AppleEvent *theEvent, AppleEvent *reply, long refCon )
{
#pragma unused (reply, refCon)

	OSErr			errCode;

	errCode = MyGotRequiredParams ( theEvent );
	if( errCode == noErr)
	{
		/*??		Acknowledge (CantPrintID);	??*/
	}
	return errCode;
} /*DoAEPrintDoc */


pascal OSErr DoAEQuit( AppleEvent *theEvent, AppleEvent *reply, long refCon )
{
#pragma unused (reply, refCon)

	OSErr			errCode;

	errCode = MyGotRequiredParams ( theEvent );
	if( errCode == noErr)
		DoQuit();

	return errCode;
} /*DoAEQuit */


	/* Add an InteractWithUser as an example of how it is done.*/
	/*	Include an IdleProc which updates windows in the background.*/
