
	/*
	  PROGRAM: corqtest.c
			 (C) 1990-1992 Microsoft Corporation.  All right reserved.
	
	  PURPOSE: This is a Quick Test of the general functionality of an ODBC
			   driver.
			   Core functions are tested.
	
	  FUNCTIONS:
			CoreQuickTest() - performs the quick test
	
	  FUNCTION: CoreQuickTest(Pointer to ServerStruct)
	
	*/

#include <Types.h>	/* for Handle 	*/

#include "gator.h"
//#undef QUIET
#include "tests.h"
#include "autotest.h"
	
#define sz_empty	""		// mj
#define sz_Expected	""		// mj
#define sz_Returned	""		// mj

		/* commonly used strings */
	static char CSCONST szCreate[] = "Create Table %s (CH1 char (10),"
		"CH2 char (20))";
	static char CSCONST szSelect[] = "select CH1, CH2 from ";
	static char CSCONST szDrop[] = "drop table ";
	static char CSCONST szTag[] = "tag";
	static char CSCONST szCH1[] = "CH1";
	static char CSCONST szCH2[] = "CH2";
	static char CSCONST szField1Format[] = "%d%s";
	static char CSCONST szField2Format[] = "%d";

#define char1 10
#define char2 20 
#define MAX_DATA 70
#define MAXSTR 30
#define MAX_szSIZE 700
#define NUMSIZE   8

	pascal int frgStateCheck( HENV henv0, HDBC hdbc0, HSTMT hstmt0, char *fState[] );

typedef struct tagBindUCHAR		/* storage locations used with SQLBindCol	*/
{ 
	char	data[ MAX_DATA ];	/* for the rgbValue and pcbValue params		*/
	SDWORD	outlen;				/* this struct is for character data		*/
} BindUCHAR;

	/* this structure will be used to hold data in the bind/fetch sequence.	*/

typedef struct tagFetchedData		/* this struct consists of the pointers	*/
{		
	struct BindUCHAR   UChar0;   /* that will be used in BindCol for all	*/
	struct BindUCHAR   UChar1;   /* the data in the core quick test table	*/
} FDSTRUCT;

typedef struct	tagQtStruct		/* storage for the long buffers necessary	*/
{
	char	sz[MAX_szSIZE];    /* for storing various strings */
	char	szTableName[MAXSTR];
	char	szColName[MAXSTR];
} QTSTRUCT;

	
pascal void CoreQuickTest( struct ServerStruct *pTestSource )
{
	RETCODE		returncode;
	SDWORD		cRow;
	SDWORD		sdw;
	SWORD		cCol;
	int			i;
	int			fInt = SQL_INTEGER;
	SWORD		cbColName;
	SWORD		fSQLType;
	UDWORD		cbColDef;
	SWORD		ibScale;
	SWORD		fNullable;
	HENV		henv0		= NULL;
	HDBC		hdbc0		= NULL;
	HDBC		hdbc1		= NULL;
	HSTMT		hstmt0	= NULL;
	HSTMT		hstmt1	= NULL;
	QTSTRUCT	*lpqt;
	FDSTRUCT	*lpfd;
	int			fErrorCount = 0;
	int			fStateFailCount = 0;
//	char		szNum[ MAXSTR ];
	char		*rgStates[] = {NULL, NULL, NULL};
	BOOL		fRollback;
	HDBC		thdbc;
	
		/* initialize array used for multiple states (connecting to invalid
			servername) initializing in the declaration causes invalid
			aggregate initializer warnings */
	
	rgStates[0] = sz_00000;
	rgStates[1] = sz_01000;

	lpqt = (QTSTRUCT*)LockMemory( AllocateMemory( sizeof( QTSTRUCT ) ) );
	lpfd = (FDSTRUCT*)LockMemory( AllocateMemory( sizeof( FDSTRUCT ) ) );

		/* Test SQLAllocEnv */

	szWrite( "Testing AllocEnv: Null, valid",	TRUE );

	FUNCCALL( sz_SQLAllocEnv );
	returncode = SQLAllocEnv( NULL );	/* Call using a null pointer 
										- should return SQL_ERROR */

	if( returncode != SQL_ERROR )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_ERROR );
	}

		/* can not do a state check, because there are no handles at this
			point */

	if( *pTestSource->szValidServer0 )
	{		/* only test SQLAllocEnv if it was not passed in */

		FUNCCALL( sz_SQLAllocEnv );
		returncode = SQLAllocEnv( &henv0 );	/* Call using valid parameters 
												- should return SQL_SUCCESS */

		if( returncode != SQL_SUCCESS )
		{
			fErrorCount++;
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;	/* if SQLAllocEnv fails, cannot continue with
								the test */

		}
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	}
	else {
		szWrite( "Not testing SQLAllocEnv, using active ODBC Test handle", TRUE);
		henv0 = pTestSource->henv;
	}

		/* Test SQLAllocConnect */

	szWrite( "Testing AllocConnect: Null combinations, valid", TRUE );

	FUNCCALL( sz_SQLAllocConnect );
	returncode = SQLAllocConnect( NULL, NULL );	/* Call passing null pointer and
												handle - should return 
												SQL_INVALID_HANDLE */

	if( returncode != SQL_INVALID_HANDLE )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_INVALID_HANDLE );
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_empty );
	}

		/* do not check states on a return SQL_INVALID_HANDLE */

	FUNCCALL( sz_SQLAllocConnect );
	returncode = SQLAllocConnect(NULL, &hdbc0 ); /* Call with a null henv and
												valid pointer - should return 
												SQL_INVALID_HANDLE */

	if( returncode != SQL_INVALID_HANDLE )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_INVALID_HANDLE );
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_empty );
	}

		/* do not check for state errors on SQL_INVALID_HANDLE */

	FUNCCALL( sz_SQLAllocConnect );
	returncode = SQLAllocConnect( henv0, NULL ); /* Call with a valid henv and
												NULL pointer - should return
												SQL_ERROR the associated error
												state should be S1009 */

	if( returncode != SQL_ERROR )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_ERROR);
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_S1009);

	FUNCCALL( sz_SQLAllocConnect);
	returncode = SQLAllocConnect( henv0, &hdbc0); /* Call with valid
								parameters - should return SQL_SUCCESS */

	if( returncode != SQL_SUCCESS )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if(*pTestSource->szValidServer0 )
	{		/* only test SQLAllocConnect (second) if it was not passed in */
		FUNCCALL( sz_SQLAllocConnect );
		returncode = SQLAllocConnect( henv0, &hdbc1 ); /* Call with valid
							parameters - should return SQL_SUCCESS */

		if( returncode != SQL_SUCCESS )
		{
			fErrorCount++;
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}
		fStateFailCount += fStateCheck( henv0, hdbc1, hstmt0, sz_00000 );
	
/* Test SQLConnect */

		szWrite( "Testing Connect: valid tests only", TRUE );

			/* Valid connect - should return SQL_SUCCESS or SQL_SUCCESS_WITH_INFO
				associated state should be 00000 
				SQL_SUCCESS_WITH_INFO is returned by servers that return information
				on connect, such as the database selection message returned
				by Microsoft SQLServer */
		
		FUNCCALL( sz_SQLConnect );
		returncode = SQLConnect( hdbc0, pTestSource->szValidServer0, 
									strlen( pTestSource->szValidServer0 ),
									pTestSource->szValidLogin0,
									strlen( pTestSource->szValidLogin0 ),
									pTestSource->szValidPassword0, SQL_NTS );
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO )
		{
			fErrorCount++;
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateFailCount += frgStateCheck( henv0, hdbc0, hstmt0, rgStates );

		FUNCCALL( sz_SQLConnect );
		returncode = SQLConnect( hdbc1, pTestSource->szValidServer0, 
									strlen( pTestSource->szValidServer0 ),
									pTestSource->szValidLogin0,
									strlen( pTestSource->szValidLogin0 ),
									pTestSource->szValidPassword0, SQL_NTS );
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO )
		{
			fErrorCount++;
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateFailCount += frgStateCheck( henv0, hdbc1, hstmt0, rgStates );
	
	} else {
		szWrite( "not testing connect, using current connection",TRUE);
		thdbc = hdbc0;
		hdbc0 = pTestSource->hdbc;
	}

		/* Call with valid parameters, connected hdbc, should return SQL_ERROR
			with associated error state 08002 */
		
	FUNCCALL( sz_SQLConnect );
	returncode = SQLConnect( hdbc0, pTestSource->szValidServer0, 
								strlen( pTestSource->szValidServer0 ),
								pTestSource->szValidLogin0, 
								SQL_NTS, sz_empty, SQL_NTS );
	if( returncode != SQL_ERROR )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_ERROR );
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_08002 );

		/* Test SQLAllocStmt */

	szWrite( "Testing AllocStmt: Null combinations, valid", TRUE );
	
	FUNCCALL( sz_SQLAllocStmt );
	returncode = SQLAllocStmt(NULL, &hstmt0 ); /* Call with NULL hdbc, should
									return SQL_INVALID_HANDLE with associated
									error state 00000 */

	if( returncode != SQL_INVALID_HANDLE )
	{
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_ERROR );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		
	FUNCCALL( sz_SQLAllocStmt );
	returncode = SQLAllocStmt( hdbc0, NULL ); /* Call with a valid hdbc, but a
											NULL pointer should return
											SQL_ERROR with associated error
											state S1009 */

	if( returncode != SQL_ERROR)
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_ERROR);
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_S1009 );

	FUNCCALL( sz_SQLAllocStmt);
	returncode = SQLAllocStmt( hdbc0, &hstmt0 ); /* Valid parameters */

	if( returncode != SQL_SUCCESS )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );

		/* if SQLAllocStmt, the test cannot be completed*/

		goto ErrorRet;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if(*pTestSource->szValidServer0 )
	{		/* only Alloc second stmt if connection was not passed in */
		FUNCCALL( sz_SQLAllocStmt );
		returncode = SQLAllocStmt( hdbc1, &hstmt1 ); /* Valid parameters */

		if( returncode != SQL_SUCCESS )
		{
			fErrorCount++;
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );

			/* if SQLAllocStmt, the test cannot be completed*/
			fStateCheck( henv0, hdbc1, hstmt1, sz_00000 );

			goto ErrorRet;
		}
		fStateFailCount += fStateCheck( henv0, hdbc1, hstmt1, sz_00000 );
	}
	
		/* Create a table using only char type. The first two fields will be
		char, and the second two will be integer.  Then, insert data into the
		fields, do a few selects and make sure that the correct data is
		returned.  Not all servers allow Nullable to be specified.  If
		nothing is specified the default should be for Nullable fields If
		this is not the default for the source, the driver should take care
		of it.*/

	szWrite( "Creating test table", TRUE );

	/* the table name will be sxxxx where xxxx is the decimal
		equivalent of the hstmt allocated */

	wsprintf( lpqt->szTableName, "s%ld", (long)hstmt0 );
	lpqt->szTableName[8] = '\0';

	wsprintf( lpqt->sz, szCreate, lpqt->szTableName);

	szWrite( lpqt->sz, TRUE ); /* lpqt->sz contains the create table
		statement */

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS); /* Use 
			SQLExecDirect to process the create table statement */

	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if( returncode != SQL_SUCCESS )
	{
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet;
	}
	
	/* use a prepared statement with parameters to insert
		values into the table */

	lstrcpy( lpqt->sz, "insert into " );
	lstrcat( lpqt->sz, lpqt->szTableName );
	lstrcat( lpqt->sz, "(CH1, CH2) VALUES (?, ?)" );

		/* call SQLFreeStmt with SQL_CLOSE to re-use the handle */

	FUNCCALL( sz_SQLFreeStmt );
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE );
	if( returncode != SQL_SUCCESS )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

	FUNCCALL( sz_SQLPrepare);
	returncode = SQLPrepare( hstmt0, lpqt->sz, SQL_NTS); /* call SQLPrepare
		with the insert statement */

	if( returncode != SQL_SUCCESS )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet; /* if SQLPrepare fails, the test cannot continue	*/
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		/* use SQLSetParam for each of the 2 parameters		*/
		/* parameter 1, character string width char1		*/
	FUNCCALL( sz_SQLSetParam);
	SQLSetParam( hstmt0, 1, SQL_C_CHAR, SQL_CHAR, char1, 0,
		lpfd->UChar0.data, NULL);
	if( returncode != SQL_SUCCESS )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet; /* if SQLSetParam, the test cannot be completed */
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		/* parameter 2 a character string of length char2 */
	FUNCCALL( sz_SQLSetParam);
	SQLSetParam( hstmt0, 2, SQL_C_CHAR, SQL_CHAR, char2, 0, lpfd->UChar1.data, NULL );
	if( returncode != SQL_SUCCESS )
	{
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet; /* if SQLSetParam fails, the test cannot be completed */
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		/* call SQLExecute 5 times.  Each with different values for the
			parameters.  The values are based on row numbers so they can
			be easily reproduced for later checking.  
			The records will not necessarily be returned by the server
			in the same order they were created. */

	for(i = 0; i < 5; i++)
	{
		if(i > 0 )
				/* the first field (char) of all but the first record will	*/
				/* contain the record number followed by the characters		*/
				/* 	"tag" and several spaces								*/
			wsprintf( lpfd->UChar0.data, szField1Format, i, (char*)szTag );
			else
				/* the first record will contain only the record number
					followed by spaces */
				wsprintf( lpfd->UChar0.data, szField1Format, i, 
					(char*)sz_empty);

					/* the second field (char) will always contain twice the
						record number followed by spaces */

				wsprintf( lpfd->UChar1.data, szField2Format, i*2 );

				FUNCCALL( sz_SQLExecute );
				returncode = SQLExecute( hstmt0 );
				if( returncode != SQL_SUCCESS )
				{
					fErrorCount++;
					DisplayRetcode( TRUE, returncode );
					DisplayRetcode( FALSE, SQL_SUCCESS );
						/* if SQLExecute fails the insert the test cannot continue	*/
					goto ErrorRet;
				}
				fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
				FUNCCALL( sz_SQLNumResultCols );
				returncode = SQLNumResultCols( hstmt0, &cCol ); /* Test
							SQLNumResultCols - expect 0 because no columns
							are returned by an insert */
				if( returncode != SQL_SUCCESS )
				{
					fErrorCount++;
					DisplayRetcode( TRUE, returncode );
					DisplayRetcode( FALSE, SQL_SUCCESS );
				}
				fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
				if( cCol )
				{
					fErrorCount++;
					szWrite( szIndent, FALSE );
					szWrite( sz_Expected, FALSE );
					szWrite( itoa( 0, szNum, 10 ), TRUE );
					szWrite( szIndent, FALSE );
					szWrite( sz_Returned, FALSE );
				//mj	szWrite( itoa( cCol, szNum, 10 ), TRUE );
				}
				FUNCCALL( sz_SQLRowCount );
				returncode = SQLRowCount( hstmt0, &cRow ); /* Test SQLRowCount -
											expect to have exactly 1 row affected
											by the insert */
				if( returncode != SQL_SUCCESS ) {
					DisplayRetcode( TRUE, returncode );
					DisplayRetcode( FALSE, SQL_SUCCESS );
					fErrorCount++;
				}
				fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
				if( cRow != 1 ) {
					szWrite( szIndent, FALSE );
					szWrite( "Row Count failed", TRUE );
				}
	}

	szWrite( "Testing SQLTransact",TRUE);

		/*  Must turn off AutoCommit to allow Commit and Rollback
		after creating the table.  This is not a core function so it's
		return cannot constitute an error.  If unsuccessfull,
		do not try to Rollback. 
		This is processed after the Create statement because not all
		servers allow create's in non-autocommit mode. */

	FUNCCALL( sz_SQLSetConnectOption );
	fRollback = (SQL_SUCCESS == SQLSetConnectOption( hdbc0, SQL_AUTOCOMMIT,
		0 ));

	FUNCCALL( sz_SQLTransact );
	returncode = SQLTransact( henv0, hdbc0, SQL_COMMIT );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if(fRollback) {
		/* if able to turn off autocommit earlier, add
			a duplicate of the last row, this is going to be
				rolled back */

		/* must re-prepare, because some drivers can not retain the
			hstmt SQLString over the Transition boundry. */

		/* call SQLPrepare with the insert statement */

		FUNCCALL( sz_SQLPrepare);
		returncode = SQLPrepare( hstmt0, lpqt->sz, SQL_NTS);
		if( returncode != SQL_SUCCESS ) {
			fErrorCount++;
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			/* if SQLPrepare fails, the quick test cannot complete */
			goto ErrorRet;
		}
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		FUNCCALL( sz_SQLExecute);
		returncode = SQLExecute( hstmt0 );
		if( returncode != SQL_SUCCESS ) {
			fErrorCount++;
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet; /* if SQLExecute fails, the test cannot
								complete */
		}
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		FUNCCALL( sz_SQLTransact );
		returncode=SQLTransact( henv0, hdbc0, SQL_ROLLBACK );
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			fErrorCount++;
		}
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		/* turn autocommit back on, transaction testsing is complete */
		returncode = (SQL_SUCCESS == SQLSetConnectOption( hdbc0,
			SQL_AUTOCOMMIT, 1));
	}

	/* SQLFreeStmt with SQL_CLOSE to re-use it */
	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

/* select all the records. */

	szWrite( "Select the records, see if they are what we put in", TRUE );
	lstrcpy( lpqt->sz, szSelect);
	lstrcat( lpqt->sz, lpqt->szTableName);
	lstrcat( lpqt->sz, " order by CH2");
	szWrite( lpqt->sz, TRUE );

/* use the Prepare/Execute sequence, this time without
	params */

	FUNCCALL( sz_SQLPrepare);
	returncode = SQLPrepare( hstmt0, lpqt->sz, lstrlen( lpqt->sz));
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

/* Test SQLSetCursorName - this function must be called before 
	SQLExecute or SQLExecDirect */

	FUNCCALL( sz_SQLSetCursorName);
	returncode = SQLSetCursorName( hstmt0, lpqt->szTableName, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* executing: select * from table */
	FUNCCALL( sz_SQLExecute);
	returncode = SQLExecute( hstmt0 );
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		/* if SQLExecute, cannot complete test */
		goto ErrorRet;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	sdw = 9999;
	FUNCCALL( sz_SQLColAttributes);
	returncode = SQLColAttributes( hstmt0, 0, SQL_COLUMN_COUNT, NULL, 0,
		NULL, &sdw);
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	if(sdw != 2) {
		fErrorCount++;
		szWrite( szIndent, FALSE );
		szWrite( "SQLColAttributes failed to return correct number of columns",
			TRUE );
		szWrite( szIndent, FALSE );
		szWrite( sz_Expected, FALSE );
	//mj	szWrite( itoa( 2, szNum, 10 ), TRUE );
		szWrite( szIndent,FALSE );
		szWrite( sz_Returned, FALSE );
	//mj	szWrite( ltoa(sdw, szNum, 10 ), TRUE );
	}


	FUNCCALL( sz_SQLNumResultCols );
	returncode = SQLNumResultCols( hstmt0, &cCol ); /* Test
								SQLNumResultCols - expect 0 because no columns
								are returned by an insert */
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	if( cCol != 2 ) {
		fErrorCount++;
		szWrite( szIndent, FALSE );
		szWrite( sz_Expected, FALSE );
	//mj	szWrite( itoa( 0, szNum, 10 ), TRUE );
		szWrite( szIndent, FALSE );
		szWrite( sz_Returned, FALSE );
	//mj	szWrite( itoa( cCol, szNum, 10 ), TRUE );
	}

	/* Test SQLGetCursorName - this function can only be called
		after SQLSetCursorName or after SQLExecute or SQLExecDirect
		(in the second case, the driver assigns a default cursor
		name) */

	FUNCCALL( sz_SQLGetCursorName);
	returncode = SQLGetCursorName( hstmt0, lpqt->sz, MAXSTR, NULL);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* called SQLSetCursorName using the table name as the
		cursor name, expect to get it back */

	if(lstrcmp( lpqt->sz, lpqt->szTableName) != 0) {
		szWrite(szIndent, FALSE );
		szWrite( "GetCursorName failed", TRUE );
		fErrorCount++;
	}

	for(i = 1;i<3;i++) {

	/* Describe each of the 2 columns */

		FUNCCALL( sz_SQLDescribeCol);
		returncode = SQLDescribeCol( hstmt0, i, lpqt->szColName, MAXSTR, 
			&cbColName, &fSQLType, &cbColDef, &ibScale, &fNullable);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			fErrorCount++;
		}
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

		/* Bind the columns at this time to
			fetch the data.  Bind/Fetch is the only method of retrieving
			data from tables using only the core functions. */

		switch (i) {

			/* expect SQLDescribeCol to return the information
				used to create the table.  This switch statement has a
				case for each of the fields, so we can determine if
				the data returned is the same as the data used to
				create that field. */

			case 1:  /* character field 1 */
				if(lstrcmp( lpqt->szColName, szCH1) != 0||cbColName != 
					lstrlen(szCH1) ||
					(fSQLType != SQL_VARCHAR && fSQLType != SQL_CHAR)
					||cbColDef != char1) {
					szWrite(szIndent, FALSE );
					szWrite( "Describe col returned incorrect info", TRUE );
					fErrorCount++;
				}

				/* SQLBindCol is called with information used to create the
					table, not information returned by SQLDescribeCol */

				FUNCCALL( sz_SQLBindCol);
				returncode = SQLBindCol( hstmt0, i, SQL_C_CHAR,
					lpfd->UChar0.data, MAXSTR, &lpfd->UChar0.outlen);
				if( returncode != SQL_SUCCESS ) {
					fErrorCount++;
					DisplayRetcode( TRUE, returncode );
					DisplayRetcode( FALSE, SQL_SUCCESS );
					goto ErrorRet; /* if SQLBind fails, the test cannot
						complete */
				}
				fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0,
					sz_00000 );
				break;
			case 2:  /* character field 2 */
				if(lstrcmp( lpqt->szColName, szCH2) != 0||cbColName != 
					lstrlen(szCH2) ||
					(fSQLType != SQL_CHAR && fSQLType != SQL_VARCHAR) 
					||cbColDef != char2) {
					szWrite(szIndent, FALSE );
					szWrite( "Describe col returned incorrect info", TRUE );
					fErrorCount++;
				}

				/* SQLBindCol is called with information used to create the
					table, not information returned by SQLDescribeCol */

				FUNCCALL( sz_SQLBindCol);
                returncode = SQLBindCol( hstmt0, i, SQL_C_DEFAULT,
					lpfd->UChar1.data, MAXSTR, &lpfd->UChar0.outlen);
				if( returncode != SQL_SUCCESS ) {
					fErrorCount++;
					DisplayRetcode( TRUE, returncode );
					DisplayRetcode( FALSE, SQL_SUCCESS );
					goto ErrorRet; /* if SQLBind fails, the test cannot
						complete */
				}
				fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0,
					sz_00000 );
				break;
		}
	}

	/* ready to get data */

	FUNCCALL( sz_SQLFetch);
	for( i = 0, returncode = SQLFetch( hstmt0 )
		;returncode == SQL_SUCCESS
		;i++, returncode = SQLFetch( hstmt0 ) )
		{
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		wsprintf( lpqt->sz, szField1Format, i, i>0 ? (char*)szTag : (char*)sz_empty );
		if( _fstrncmp( lpfd->UChar0.data, lpqt->sz, lstrlen( lpfd->UChar0.data ) ) != 0 )
		{
			szWrite( szIndent, FALSE );
			szWrite( "Fetch Failed on first character string", TRUE );
			fErrorCount++;
		}
		wsprintf( lpqt->sz, szField2Format, i*2 );
		if( _fstrncmp( lpfd->UChar1.data, lpqt->sz, lstrlen( lpfd->UChar1.data)) != 0 )
		{
			szWrite(szIndent, FALSE );
			szWrite( "Fetch failed on second character string", TRUE );
			fErrorCount++;
		}
	}

	if( returncode != SQL_NO_DATA_FOUND )
	{		/* SQL_NO_DATA_FOUND should have been the last thing returned	*/
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_NO_DATA_FOUND );
		fErrorCount++;
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_empty );
	}

		/* do not check for error messages on a SQL_NO_DATA_FOUND	*/
	
		/* SQLFreeStmt with SQL_CLOSE to re-use the handle	*/

	FUNCCALL( sz_SQLFreeStmt );
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE );
	if( returncode != SQL_SUCCESS )
	{
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateCheck( henv0, hdbc0, NULL, sz_00000 );

/* SQLFreeStmt with SQL_RESET_PARAMS */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_RESET_PARAMS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateCheck( henv0, hdbc0, NULL, sz_00000 );

	szWrite( "Test SQLFreeStmt with SQL_CLOSE without fetching any data",
		TRUE);
	lstrcpy( lpqt->sz, szSelect);
	lstrcat( lpqt->sz, lpqt->szTableName);
	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* Test SQLGetCursorName - this function can only be called
		after SQLSetCursorName or after SQLExecute or SQLExecDirect
		Currently testing the second case, should get a default
		cursor name. */

	*lpqt->sz = '\0';
	FUNCCALL( sz_SQLGetCursorName);
	returncode = SQLGetCursorName( hstmt0, lpqt->sz, MAXSTR, NULL);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	if(lstrlen( lpqt->sz) < 1) {
		szWrite(szIndent, FALSE );
		szWrite( "Failed to return default Cursor Name",TRUE);
		fErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

/* Test using a <> condition */

	szWrite( "Testing result set where CH2 <> ?", TRUE );
	lstrcpy( lpqt->sz, szSelect);
	lstrcat( lpqt->sz, lpqt->szTableName);
	lstrcat( lpqt->sz, " where CH2 <> ?");
	szWrite( lpqt->sz, TRUE );

	FUNCCALL( sz_SQLSetParam);
	SQLSetParam( hstmt0, 1, SQL_C_CHAR, SQL_CHAR, 10, 0,
		&lpfd->UChar1.data, NULL);
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	lstrcpy( lpfd->UChar1.data, "0");
	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* this is the same handle used to fetch on earlier same Binds
		should still be in effect, no need
		to re-bind before fetching the new result set */

	FUNCCALL( sz_SQLFetch);
	for(cRow = 0, returncode = SQLFetch( hstmt0 )
		;returncode==SQL_SUCCESS
		;cRow++, returncode = SQLFetch( hstmt0 ))

		/* do not check the actual data returned, 
			order records are returned is undetermined */

		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	if( returncode != SQL_NO_DATA_FOUND) { /* SQL_NO_DATA_FOUND should have
											been the last thing returned */
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_NO_DATA_FOUND);
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if(cRow != 4) {
		szWrite(szIndent, FALSE );
		szWrite( "Error in number of result records in query", TRUE );
		fErrorCount++;
	}

	/* call SQLFreeStmt with SQL_CLOSE to re-use the hstmt */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);

	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

/* Testing a join condition */

	szWrite( "Testing a join clause", TRUE );
	lstrcpy( lpqt->sz, "select a.CH1, b.CH2 from ");
	lstrcat( lpqt->sz, lpqt->szTableName);
	lstrcat( lpqt->sz, " a, ");
	lstrcat( lpqt->sz, lpqt->szTableName);
	lstrcat( lpqt->sz, " b where a.CH2 < b.CH2");
	szWrite( lpqt->sz, TRUE );

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLFetch);
	for(cRow = 0, returncode = SQLFetch( hstmt0 )
		;returncode==SQL_SUCCESS
		;cRow++, returncode = SQLFetch( hstmt0 ))
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* this join should have at 18 result rows */

	if(cRow != 10) {
		szWrite(szIndent, FALSE );
		szWrite( "Incorrect number of rows", TRUE );
		szWrite(szIndent, FALSE );
		szWrite( "Rows Expected: ", FALSE );
	//mj	szWrite(itoa(10, szNum, 10), TRUE );
		szWrite(szIndent, FALSE );
		szWrite( "Actual Rows: ", FALSE );
	//mj	szWrite(ltoa(cRow, szNum, 10), TRUE );
		fErrorCount++;
	}

/* Testing > condition */

	szWrite( "Testing > ", TRUE );
	lstrcpy( lpqt->sz, szSelect);
	lstrcat( lpqt->sz, lpqt->szTableName);
	lstrcat( lpqt->sz, " where CH1 >= ?");
	szWrite( lpqt->sz, TRUE );

	/* Call SQLFreeStmt with SQL_CLOSE to re-use the hstmt */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

	FUNCCALL( sz_SQLPrepare);
	returncode = SQLPrepare( hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLSetParam);
	SQLSetParam( hstmt0, 1, SQL_C_CHAR, SQL_CHAR, 10, 0,
		&lpfd->UChar1.data, NULL);
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	lstrcpy( lpfd->UChar1.data, "1");

	FUNCCALL( sz_SQLExecute);
	returncode = SQLExecute( hstmt0 );
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLFetch);
	for(cRow = 0, returncode = SQLFetch( hstmt0 )
		;returncode==SQL_SUCCESS
		;cRow++, returncode = SQLFetch( hstmt0 ))
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* should have gotten 4 rows back */

	if(cRow != 4) {
		szWrite(szIndent, FALSE );
		szWrite( "Incorrect number of rows returned", TRUE );
		fErrorCount++;
	}

/* Test using Floating Point computation */
	szWrite( "Select with floating point", TRUE );
	lstrcpy( lpqt->sz, "select 99*342 from ");
	lstrcat( lpqt->sz, lpqt->szTableName);
	szWrite( lpqt->sz, TRUE );

	/* SQLFreeStmt with SQL_CLOSE to re-use the hstmt */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLFetch);
	for(cRow = 0, returncode = SQLFetch( hstmt0 )
		;returncode==SQL_SUCCESS
		;cRow++, returncode = SQLFetch( hstmt0 ))
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* should have gotten 5 rows with this select */

	if(cRow != 5) {
		szWrite(szIndent, FALSE );
		szWrite( "Incorrect number of rows returned", TRUE );
		fErrorCount++;
	}

	/* Test a query returning no records */

	szWrite( "select w/o records", TRUE );
	lstrcpy( lpqt->sz, szSelect);
	lstrcat( lpqt->sz, lpqt->szTableName);
	lstrcat( lpqt->sz, " where 1 = 2");
	szWrite( lpqt->sz, TRUE );

	/* SQLFreeStmt with SQL_CLOSE to re-use the hstmt */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLNumResultCols);
	returncode = SQLNumResultCols( hstmt0, &cCol);		
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}

	if(cCol != 2) {
		szWrite(szIndent, FALSE );
		szWrite( "Incorrect number of columns returned", TRUE );
		fErrorCount++;
	}
	cCol = 0;
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLRowCount);
	returncode = SQLRowCount( hstmt0, &cRow); /* SQLRowCount should not be
								valid for a SQL statement that returns
								records */
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if(cRow != -1) {
		szWrite(szIndent, FALSE );
		szWrite( "Invalid cRow return value (should have been -1)", TRUE );
		fErrorCount++;
	}
	
	FUNCCALL( sz_SQLFetch);
	for(cRow = 0, returncode = SQLFetch( hstmt0 )
		;returncode==SQL_SUCCESS
		;cRow++, returncode = SQLFetch( hstmt0 ))
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if(cRow != 0) { /* should not get any records back with this query */
		szWrite(szIndent, FALSE );
		szWrite( "Failed to return correct number of rows", TRUE );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

/* SQLFreeStmt with SQL_UNBIND */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_UNBIND);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateCheck( henv0, hdbc0, NULL, sz_00000 );

	/* Drop table */

	/* SQLFreeStmt SQL_CLOSE to re-use it */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		fErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

	lstrcpy( lpqt->sz, szDrop);
	lstrcat( lpqt->sz, lpqt->szTableName);

	szWrite( "Dropping table", TRUE );
	szWrite( lpqt->sz, TRUE );

	if(*pTestSource->szValidServer0) {
		FUNCCALL( sz_SQLDisconnect);
		returncode = SQLDisconnect(hdbc0);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			fErrorCount++;
		}
		fStateFailCount += fStateCheck( henv0, hdbc0, NULL, sz_00000 );

		hstmt0 = hstmt1;
	}

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		szWrite(szIndent, FALSE );
		szWrite( "Unable to drop table", TRUE );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt1, sz_00000 );

	/* Test SQLCancel, although this tests Core, Asynchronous functionality
		is not available.
	SQLCancel is going to work similarly to SQLFreeStmt with SQL_CLOSE */

	FUNCCALL( sz_SQLCancel);
	returncode = SQLCancel( hstmt0 );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	/* Free the hstmt */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_DROP);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	hstmt0 = NULL;
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

/* Disconnect */

	if(*pTestSource->szValidServer0) { /* if connecting was performed by 
											quicktest */
		FUNCCALL( sz_SQLDisconnect);
		returncode = SQLDisconnect(hdbc1);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			fErrorCount++;
		}
		fStateFailCount += fStateCheck( henv0, hdbc1, hstmt0, sz_00000 );

		FUNCCALL( sz_SQLFreeConnect);
		returncode = SQLFreeConnect(hdbc1);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			fErrorCount++;
		}
		fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	}
	else {
		szWrite( "Not testing SQLDisconnect, using current connection",
			TRUE);
		hdbc0 = thdbc;
	}

	/* Free the hdbc */
			
	FUNCCALL( sz_SQLFreeConnect);
	returncode = SQLFreeConnect(hdbc0);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		fErrorCount++;
	}
	hdbc0 = NULL;
	fStateFailCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	if(*pTestSource->szValidServer0) { /* if connecting was done by
											quicktest */
	/* free the henv */
	
		FUNCCALL( sz_SQLFreeEnv);
		returncode = SQLFreeEnv( henv0);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			fErrorCount++;
			fStateFailCount += fStateCheck( henv0, NULL, NULL, sz_00000 );
		}
	}
	else {
		szWrite( "Not testing SQLFreeEnv, using current connection",
			TRUE);
	}
	henv0 = NULL;

/* check out error status and report to user */
	pTestSource->cErrors = fErrorCount;
	pTestSource->cStateErrors = fStateFailCount;

	ControlSzWrite( SAVE, SAME );
	ControlSzWrite( ENABLE, SAME );

    if(fErrorCount > 0 || fStateFailCount > 0) {
		/* if there are *any* errors at all, test did not pass */
		szWrite( "=========================Noninteractive Quick Test "
			"FAILED", TRUE );
		szWrite( "Failed Tests: ", FALSE );
	//mj	szWrite(itoa(fErrorCount, szNum, 10), TRUE );
		szWrite( "Failed Error States: ", FALSE );
	//mj	szWrite(itoa(fStateFailCount, szNum, 10), TRUE );
		pTestSource->cErrors = fErrorCount;
		pTestSource->cStateErrors = fStateFailCount;
		}
	else
		szWrite( "*** Noninteractive Quick Test Passed ***", TRUE );

	ControlSzWrite( RESTORE, SAME );

	ReleaseMemory( lpfd);
	ReleaseMemory( lpqt);

	return;

ErrorRet:
	/* a failure in an ODBC function that prevents completion of the
		test - for example, connect to the server */

	ControlSzWrite( SAVE, SAME );
	ControlSzWrite( ENABLE, SAME );

	szWrite(szIndent, FALSE );
	szWrite( "  **** Unrecoverable Quick Test FAILURE ****", TRUE );
		
	fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	ControlSzWrite( RESTORE, SAME );

	pTestSource->cErrors		= -1;
	pTestSource->cStateErrors	= -1;
	
	ReleaseMemory( lpfd );
	ReleaseMemory( lpqt );

	return;
}

#define BUFSIZE 150

	/* frgStateCheck takes current handles and an array of possible states
	that would be correct if returned by SQLError.  SQLError is called and
	the returned state is compared with the array of states passed in.  If
	the state does not match one of the states input an error is displayed,
	otherwise no error.  The function continues calling SQLError in this
	manner until SQLError returns SQL_NO_DATA_FOUND.  

	frgStateCheck does not verify that all the states in the array are
	returned in the calls to SQLError, only that any state that is returned
	is among those passed in */

pascal int frgStateCheck( HENV henv0, HDBC hdbc0, HSTMT hstmt0, char *szExpState[] )
{
	UCHAR   buf[ BUFSIZE ];
	UCHAR   sz[  BUFSIZE ];
	int     fBadState;
	int     cStatesReturned = 0;
	int     cInvalidStates = 0;
	int     i;
	RETCODE returncode;
	SWORD   cch;
/*mj	UDWORD  dw;	*/		SDWORD  dw;

	for( returncode = SQLError( henv0, hdbc0, hstmt0, buf, &dw, sz,
						BUFSIZE, &cch )
		;returncode == SQL_SUCCESS || returncode == SQL_SUCCESS_WITH_INFO
		;returncode = SQLError( henv0, hdbc0, hstmt0, buf, &dw, sz, BUFSIZE,
		 	&cch ) ) {
			cStatesReturned++;
			fBadState = 1;
		for( i = 0; szExpState[i]; i++ )
			if( lstrcmp( szExpState[i], buf ) == 0)
				fBadState = 0;
			if( fBadState ) {
				szWrite(szIndent, FALSE );
				szWrite( "State Mismatch", TRUE );
				szWrite(szIndent, FALSE );
				szWrite( "State: ", FALSE );
				szWrite(buf, TRUE );
				szWrite(szIndent, FALSE );
				szWrite(sz_Expected, FALSE );
				for(i = 0;szExpState[i];i++) {
					if(i > 0)
						szWrite( " or ", FALSE );
					szWrite(szExpState[i], FALSE );
				}
				szWrite(sz_empty, TRUE );
				szWrite(szIndent, FALSE );
				szWrite( "Message: ", FALSE );
				szWrite(sz, TRUE );
			}
			cInvalidStates += fBadState;
		}
	if( returncode == SQL_ERROR)
		EchoRetcode(sz_SQLError, returncode );
	else if(lstrcmp(szExpState[0], sz_00000 ) != 0 && !cStatesReturned) {
		cInvalidStates++;
		szWrite(szIndent, FALSE );
		szWrite( "State Mismatch", TRUE );
		szWrite(szIndent, FALSE );
		szWrite( "State: 00000", TRUE );
		szWrite(szIndent, FALSE );
		szWrite(sz_Expected, FALSE );
		for(i = 0;szExpState[i];i++) {
			if(i > 0)
				szWrite( " or ", FALSE );
			szWrite(szExpState[i], FALSE );
		}
		szWrite( sz_empty, TRUE );
	}

	return cInvalidStates;
}

pascal int fStateCheck( HENV henv0, HDBC hdbc0, HSTMT hstmt0, char *szExpState )
{
	char	*rgStates[] = { NULL, NULL };

	rgStates[0] = szExpState;
	return frgStateCheck( henv0, hdbc0, hstmt0, rgStates );
}

pascal void QuickTests( struct ServerStruct *pTestSource )
{
	pTestSource->cErrors		= 0;
	pTestSource->cStateErrors	= 0;
		
	if( *pTestSource->rglMask & 1l )
	{
		ControlSzWrite( SAVE,   SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "CoreQuickTest", TRUE );
		ControlSzWrite( RESTORE, SAME );

		CoreQuickTest( pTestSource );

		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "CoreQuickTest completed", TRUE );
		ControlSzWrite( RESTORE, SAME );
	}

	if( *pTestSource->rglMask & 1l<<1 )
	{
		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "L1QuickTest", TRUE );
		ControlSzWrite( RESTORE, SAME );

		ExtQuickTestL1( pTestSource );

		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "L1QuickTest completed", TRUE );
		ControlSzWrite( RESTORE, SAME );
	}

	if( *pTestSource->rglMask & 1l<<2 )
	{
		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "L2QuickTest", TRUE );
		ControlSzWrite( RESTORE, SAME );

		ExtQuickTestL2(pTestSource);

		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "L2QuickTest completed", TRUE );
		ControlSzWrite( RESTORE, SAME );
	}

	if(*pTestSource->rglMask & 1l<<3 )
	{
		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "Custom", TRUE );
		ControlSzWrite( RESTORE, SAME );

		Custom(pTestSource);

		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "Custom completed", TRUE );
		ControlSzWrite( RESTORE, SAME );
	}

	if( *pTestSource->rglMask & 1l<<4 )
	{
		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "Multi-File Query Engine", TRUE );
		ControlSzWrite( RESTORE, SAME );

		SQLMFQueryEngine( pTestSource );

		ControlSzWrite( SAVE, SAME );
		ControlSzWrite( ENABLE, SAME );
		szWrite( "Multi-File Query Engine completed", TRUE );
		ControlSzWrite( RESTORE, SAME );
	}
}

pascal int QuickTestsDesc( int i, char *szName, char *szDesc )
{
	static char CSCONST szCOREQuick[] = "CoreQuickTest";
	static char CSCONST szExtQuickL2[] = "ExtQuickTestL2";
	static char CSCONST szExtQuickL1[] = "ExtQuickTestL1";
	static char CSCONST szCustom[] = "Custom";
	static char CSCONST szMFQEngine[] = "MultiFileQEngine";
	static char CSCONST szBQEngine[] = "Bruce Queries";

	switch( i )
	{
	case 1:
		if(szName)
			lstrcpy(szName, szCOREQuick);
		if(szDesc)
			lstrcpy(szDesc, "Tests Core level functionality");
		break;
	case 2:
		if(szName)
			lstrcpy(szName, szExtQuickL1);
		if(szDesc)
			lstrcpy(szDesc, "Tests Extension Level 1 Functionality");
		break;
	case 3:
		if(szName)
			lstrcpy(szName, szExtQuickL2);
		if(szDesc)
			lstrcpy(szDesc, "Tests Extension Level 2 Functionality");
		break;
	case 4:
		if(szName)
			lstrcpy(szName, szCustom);
		if(szDesc)
			lstrcpy(szDesc, "Your Test");
		break;
	case 5:
		if(szName)
			lstrcpy(szName, szMFQEngine);
		if(szDesc)
			lstrcpy( szDesc, "Runs multiple queries" );
		break;
	default:
		return FALSE;
	}
	return TRUE;
}


pascal void DisplayRetcode( BOOL fReturned, RETCODE retcode )
{
//#define InitFnam(name) {name, sz_ ## name}
	static char CSCONST sz_unknown[] = "an unknown value";
	static char CSCONST sz_returned[] = "        Returned: ";
	static char CSCONST sz_expected[] = "        Expected: ";

	typedef struct tagFMATCH
	{
		RETCODE fArg;
		char	*lszName;
	} FMATCH;

	static FMATCH CSCONST rgfnam[] =
	{
		{ SQL_SUCCESS,				sz_SQL_SUCCESS				},
		{ SQL_SUCCESS_WITH_INFO,	sz_SQL_SUCCESS_WITH_INFO	},
		{ SQL_STILL_EXECUTING,		sz_SQL_STILL_EXECUTING		},
		{ SQL_NO_DATA_FOUND,		sz_SQL_NO_DATA_FOUND		},
		{ SQL_ERROR,				sz_SQL_ERROR				},
		{ SQL_INVALID_HANDLE,		sz_SQL_INVALID_HANDLE		},
		{ SQL_NEED_DATA,			sz_SQL_NEED_DATA			}
	};
#define pfnamMax &rgfnam[sizeof(rgfnam)/sizeof(FMATCH)]

	char			*lsz;
	FMATCH CSCONST	*pfnam;

	szWrite( fReturned ? sz_returned : sz_expected, FALSE );
	lsz = sz_unknown;
	for (pfnam = &rgfnam[0]; pfnam < pfnamMax; pfnam++)
		if (retcode == pfnam->fArg) {
			lsz = pfnam->lszName;
			break;
		}
	szWrite(lsz, TRUE );

#undef pfnamMax
}
