
	/*  1988-91, Bowers Development Corp.	*/

#include <Types.h>
#include <Quickdraw.h>
#include <Controls.h>
#include <Desk.h>				/* for SystemTask */
#include <Dialogs.h>
#include <Events.h>
#include <Lists.h>
#include <Menus.h>
#include <TextEdit.h>

#include "EventLoop.h"
#include "Globals.h"
#include "ResourceDefs.h"
#include "Windowing.h"
#include "DoScrap.h"

#include "DialogAids.h"

#include <OSUtils.h>
#include <Packages.h>
#include <ToolUtils.h>
#include "Cursors.h"	/* cursorRgn for WaitNextEvent */

#pragma segment DialogAids

	static Boolean	sFiltered;			/* used by DoModalEvent & MovableDialog	*/


void SetUserItem( short itemNr, ProcPtr doDraw )
{
	short	itemType;
	Handle	itemHandle;
	Rect	itemRect;

	GetDItem( qd.thePort, itemNr, &itemType, &itemHandle,    &itemRect );
	SetDItem( qd.thePort, itemNr,  itemType, (Handle)doDraw, &itemRect );
	return;
}/* end SetUserItem */


void OutlineButton( short itemNr )
{
	PenState		savePen;
	short			itemType;
	Handle			itemHandle;
	Rect			itemRect;


	GetPenState( &savePen );
	PenNormal();
	PenSize( 3, 3 );
	GetDItem( qd.thePort, itemNr, &itemType, &itemHandle, &itemRect );
	InsetRect( &itemRect, -4, -4 );
	FrameRoundRect( &itemRect, 16, 16 );
	SetPenState( &savePen );
	return;
}/* end OutlineButton */


void EnableDItem( short itemNr, Boolean enable )
{
	short			itemType;
	Handle			itemHandle;
	Rect			itemRect;
	ControlHandle	control;

	GetDItem( qd.thePort, itemNr, &itemType, &itemHandle, &itemRect );
	if( enable )
		 itemType &= ~itemDisable;
	else itemType |=  itemDisable;
	SetDItem( qd.thePort, itemNr, itemType, itemHandle, &itemRect );

	if( itemType & ctrlItem )
	{		/* it's a control */
		control = (ControlHandle)itemHandle;
		HiliteScroll( control, enable );
	}/* else  do nothing? */
	return;
}/* end EnableDItem */


void SetCheckbox( short itemNr, Boolean	checked )
{
	ControlHandle	control;
	short			itemType;
	Rect			itemRect;

	GetDItem( qd.thePort, itemNr, &itemType, &(Handle)control, &itemRect );
	SetCtlMax( control, 1 );
	SetCtlValue( control, checked );
	return;
}/* end SetCheckbox */


void DoCheckbox( short itemNr, Boolean *checked )
{
	*checked = !*checked;
	SetCheckbox( itemNr, *checked );
	return;
}/* end DoCheckbox */


void DrawPopup( short itemNr, short, short )	// Dialog pops only
{
	short			itemType;
	Handle			itemHandle;
	Rect			textRect;
	PolyHandle		arrowTriangle;

	GetDItem( qd.thePort, itemNr, &itemType, &itemHandle, &textRect );

	arrowTriangle = OpenPoly();
	MoveTo( textRect.right - 17, textRect.top + 7 );
	Line( 5, 5 );
	Line( 5, -5 );
	Line( -10, 0 );
	ClosePoly();
	FillPoly( arrowTriangle, &qd.black );
	KillPoly( arrowTriangle );

	textRect.right  = textRect.right  - 1;
	textRect.bottom = textRect.bottom - 1;
	FrameRect( &textRect );
	MoveTo( textRect.right,		textRect.top + 2	);
	LineTo( textRect.right,		textRect.bottom	);
	LineTo( textRect.left + 2,	textRect.bottom	);
	return;
}/* end DrawPopup */


void InvertLabel( short itemNr )
{
	short			itemType;
	Handle			itemHandle;
	Rect			itemRect;

	GetDItem( qd.thePort, itemNr, &itemType, &itemHandle, &itemRect );
	InvertRect( &itemRect );
	return;
}/* end InvertLabel */


ListHandle NewV1List( Rect bounds, WindowPtr parentWindow )
{
	Rect			listBounds;
	Rect			dataBounds;
	Point			cSize;
	ListHandle		list;

	listBounds = bounds;
	listBounds.right = listBounds.right - 15;	/*for scroll bar		*/
	SetRect( &dataBounds, 0, 0, 1, 0 );			/*one column, no rows	*/
	SetPt( &cSize, listBounds.right - listBounds.left, 0 );
	list = LNew( &listBounds,		/* dialog item's box						*/
				 &dataBounds,		/* one column, no rows						*/
				 cSize,				/* cell size: full width, standard height	*/
				 0,					/* procid - standard text list				*/
				 parentWindow,		/* parent window							*/
				 false,				/* draw it									*/
				 false,				/*has no grow								*/
				 false,				/*no horizontal scroll						*/
				 true );			/*vertical scroll							*/
	(**list ).selFlags = lOnlyOne + lNoNilHilite;
	return list;
}/* end NewV1List */


ListHandle Vert1List( short itemNr )
{
	ListHandle		list;
	short			itemType;
	Handle			itemHandle;
	Rect			itemRect;

	GetDItem( qd.thePort, itemNr, &itemType, &itemHandle, &itemRect );
	list = NewV1List( itemRect, qd.thePort );

	return list;
}/* end Vert1List */


Boolean FilterList( EventRecord *event, ListHandle list, short listItem,
						short dblClickItem, short *itemHit )
{
	Boolean			filtered;
	Point			mousePos;

	filtered = false;
	if( (*event).what == mouseDown )
	{
		mousePos = (*event).where;
		GlobalToLocal( &mousePos );
		if( FindDItem( qd.thePort, mousePos ) + 1 == listItem )
		{
			if( LClick( mousePos, (*event).modifiers, list ) )
				 *itemHit = dblClickItem;
			else *itemHit = listItem;
			filtered = true;
		}
	}
	return filtered;
}/* end FilterList */


pascal Boolean StandardFilter( DialogPtr whichDialog, EventRecord *event,
								 short *itemHit )
{
#define returnKey		13
#define enter			3
#define esc				27
#define pushButton		(ctrlItem + btnCtrl)
	
	Boolean			filtered;
	DialogPeek		whichPeek;
	short			defItem;
	char			key;
	short			itemType;
	Handle			itemHandle;
	Rect			itemRect;
	long			finalTicks;

	filtered	= false;
	whichPeek	= (DialogPeek)whichDialog;
	defItem		= whichPeek->aDefItem;
	switch( (*event).what )
	{
	case keyDown:
	case autoKey:
		key = (*event).message & charCodeMask;
		if( (key == returnKey) || (key == enter) || (key == esc)
		|| ((key == '.') && (((*event).modifiers & cmdKey) != 0)) )
		{
			if( (key == returnKey) || (key == enter) )
				 *itemHit = defItem;
			else *itemHit = cancel;
			GetDItem( whichDialog, *itemHit, &itemType, &itemHandle, &itemRect );
			if( (itemType & itemDisable) == 0 )
			{
				if( (itemType & (255 - itemDisable)) == pushButton )
				{
					HiliteControl( (ControlHandle)itemHandle, inButton );
					Delay( 8, &finalTicks );
					HiliteControl( (ControlHandle)itemHandle, 0 );
				}
				filtered = true;
			}
		}
		else
		{
			if( (((*event).modifiers & cmdKey) != 0)
			&&  ((key == 'x') || (key == 'c') || (key == 'v')) )
			{
				*itemHit = (whichPeek)->editField + 1;
				switch( key )
				{
				case 'x':
					DlgCut( whichDialog );
					scrapDirty = (*itemHit > 0);
					break;
				case 'c':
					DlgCopy( whichDialog );
					scrapDirty = (*itemHit > 0);
					*itemHit = 0;
					break;
				case 'v':
					DlgPaste( whichDialog );
					break;
				}
				if( *itemHit > 0 )
				{
					GetDItem( whichDialog, *itemHit, &itemType, &itemHandle, &itemRect );
					if( (itemType & itemDisable) == 0 )
						filtered = true;
				}
				if( !filtered )
					(*event).what = nullEvent;
			}
		}
		break;
	case updateEvt:
		if( (WindowPtr)(*event).message == qd.thePort )
		{
			GetDItem( whichDialog, defItem, &itemType, &itemHandle, &itemRect );
			if( (itemType & (255 - itemDisable)) == pushButton )
				OutlineButton( defItem );
		}
		else if( !IsDialogEvent( event ) )
		{
			curEvent = *event;
			DoUpdate();			/* update other windows	*/
		}
		break;
	}
	return filtered;
}/* end StandardFilter */


	/* This procedure handles mouseDown in a non-dialog window.	*/

static void	DoMouseDown()
{
	long			menuChoice;
	short			menuID;
	short			itemNr;
	WindowPtr		whichWindow;
	short			whichPart;

	whichPart = FindWindow (curEvent.where, &whichWindow );
	switch( whichPart )
	{
	case inMenuBar:
			menuChoice = MenuSelect( curEvent.where );
			menuID = HiWord( menuChoice );
			itemNr = LoWord( menuChoice );
			if( (menuID != 0) && (itemNr != 0) )
			{
/* Need to handle Cut, Copy, Paste here just like StandardFilter doesnot	*/
				SysBeep(1);
			}	/* otherwise, MenuSelect returned 0. Either the user	*/
				/* chose nothing, or the user chose from a System		*/
				/* menu, and it's been taken care of					*/
		break;
	case inDrag:
			if( whichWindow == FrontWindow() )
				DoDrag( whichWindow );			/* permit drag of this dialog	*/
			else
				SysBeep(1);						/* can't drag any other window	*/
		break;
	case inSysWindow:
	case inContent:
	case inDesk:
	case inGrow:
	case inGoAway:
	case inZoomIn:
	case inZoomOut:
	default:
		SysBeep(1);
		break;
	}
	return;
}/* end DoMouseDown */
 

static Boolean GetEvent()
{
	Boolean			gotEvent;

	if( sysConfig.hasWNE )
		gotEvent = WaitNextEvent( everyEvent, &curEvent, 0L, cursorRgn );
	else
	{
		SystemTask();
		gotEvent = GetNextEvent( everyEvent, &curEvent );
	}
	return gotEvent;
}/* end GetEvent */
 

	/* This function processes the next modal dialog event and then returns. It is called		*/
	/* by MovableDialog, but it can also be called directly. Assumes that the frontmost			*/
	/* window is a modal (or movable modal) dialog window.										*/
	/*																							*/
	/* If the event is associated with an item in the current dialog, returns the item number.	*/
	/* If filterProc (or System 7.0's standard filter, or DialogSelect) handled the event		*/
	/* (and presumably set itemHit ); sets sFiltered to true, otherwise sets sFiltered to false.	*/
	/* If the event is not associated with an item in the current dialog, returns 0.			*/

short DoModalEvent( ModalFilterProcPtr filterProc )
{
	WindowPtr		eventWindow;
	short			itemHit;		/* function return value */
	GrafPtr			savePort;
	DialogPtr		theDialog;

	GetPort( &savePort );

	sFiltered	= false;
	itemHit		= 0;
	theDialog	= FrontWindow();
	if( GetEvent() || (curEvent.what == nullEvent) )
	{
			/* got an event to process	*/
		if( curEvent.what == app4Evt )
		{
				/* here so event won't get swallowed by IsDialogEvent	*/
		 		/* and so suspend/resume can be translated to deactivate/activate	*/
			DoApp4Event();
		}
		if( IsDialogEvent( &curEvent ) )
		{
				/* it's a dialog event	*/
			if( (curEvent.what == activateEvt) || (curEvent.what == updateEvt) )
				 eventWindow = (DialogPtr)curEvent.message;
			else eventWindow = FrontWindow();
			SetPort( eventWindow );
			if( filterProc )
					/* caller passed a filterProc	*/
				sFiltered = (*filterProc)( theDialog, &curEvent, &itemHit );
			else
			{
					/* filterProc == NULL	*/
/* Need to do if( gestalt then if gestalt(stdfilter available)) sFiltered = stdfilternot	*/
/* See Tech Note #304 p. 4 for GetStdFilterProc												*/
				sFiltered = StandardFilter( theDialog, &curEvent, &itemHit );
			}
			if( !sFiltered )
			{
					/* either filterProc == NULL, or filterProc
						didn't process curEvent (although filterProc
						may have altered curEvent)	*/
/*Don't need to handle Cmd-keys here, since StandardFilter does.	*/
				sFiltered = DialogSelect( &curEvent, &theDialog, &itemHit );
			}/* otherwise, sFiltered	*/
		}
		else
		{		/*it's not a dialog event	*/
			switch( curEvent.what )
			{
			case mouseDown:
				DoMouseDown();
				break;
			case mouseUp:
			case keyDown:
			case autoKey:
					/* do nothing	*/
				break;
			case updateEvt:
				DoUpdate();
				break;
			case activateEvt:
					/*do nothing, to avoid setting cur to noCur	*/
				break;
			case diskEvt:
				DoDiskEvent();
				break;
			default:
					/* Do nothing	*/
				break;
			}
		}/*if dialog event or not	*/
	}/* otherwise, not GetEvent and curEvent.what != nullEvent	*/

	SetPort( savePort );

	return itemHit;
}/* end DoModalEvent */


	/* This procedure works just like ModalDialog, case plus:							*/
	/*	- clicking and dragging in the dialog's title bar will move the dialog			*/
	/*	- application windows update													*/
	/*	- under System 7.0, uses the same standard filter proc as ModalDialog			*/
	/* Under System 7.0, the dialog window's procID should be movableDBoxProc			*/
	/* (== 5, from Windows.p).															*/
	/* Under System 6.x, procID movableDBoxProc is equivalent to dBoxProc, and the		*/
	/* user won't be able to drag the dialog. However, if the procID is noGrowDocProc,	*/
	/* then the user will be able to drag the dialog.									*/
	/* Modifies global curEvent (Globals.p).											*/

void MovableDialog( ModalFilterProcPtr filterProc, short *itemHit )
{
		/* Need to disable all of application's menus except Cut/Copy/Paste here	*/
	*itemHit = 0;
	if( FrontWindow() != NULL )
	{
		sFiltered = false;
		while( !sFiltered )
			*itemHit = DoModalEvent( filterProc );		/* also sets sFiltered	*/
	}/* otherwise, no front window */
	return;
}/* end MovableDialog */
