
	//  PROGRAM: Gator.c
	//		Copyright (C) 1988, 1989, 1990, 1991 Microsoft Corp.

	//  PURPOSE: Windows ODBC SQLCLI Library Test Application

	//  FUNCTIONS:

	//	AppInit()  - initializes app windows and global variables
	//	AboutDlg() - processes messages for About dialog box

	//  COMMENTS:

	//	Windows can have several copies of your application running at the
	//	same time.  The variable hinst keeps track of which instance this
	//	application is so that processing will be to the correct window.

	//	You only need to initialize the application once.  After it is
	//	initialized, all other copies of the application will use the same
	//	window class, and do not need to be separately initialized.


#include <memory.h>
#include <textEdit.h>		// only for TEHandle used by globals.h
#include <controls.h>		// only for ControlHandle used by globals.h
#include <events.h>			// only for EventRecord used by globals.h

#include "gator.h"
#include "adhoc.h"
#include "about.h"
#include "menus.h"
#include "globals.h"
#include <dos.h>			/* DUMMY DOS SYMBOLS - DELETE	*/
//#include <ctype.h>

#pragma segment GATOR

	extern windowPop		DBCpop;
	extern windowPop		STMTpop;

	extern char	*szLogFileName;

	INIT_ASSERT

#define MessageBox( w, x, y, z )
#define SendMessage( w, x, y, z )

#define MIN(x,y)	( (x) <= (y) ? (x) : (y) )

	// JET compatibility. START.
#ifdef JETVERSION
	unsigned _far _pascal ErrEbInitDgroup();
	unsigned _far _pascal ErrEbInit();
	void _far _pascal EbTerm();
#endif
	// JET compatibility. END.

	static BYTE CSCONST szAppName[]            = "ODBC Test";
	static BYTE CSCONST szDefExt[]             = "txt";
	static char CSCONST szGatorIni[]           = "gator.ini";
	static char CSCONST szStartup[]            = "startup";
	static char CSCONST szMaximized[]          = "maximized";
	static BYTE CSCONST sz_Error[]             = "***Error*** ";
	static char CSCONST szNullString[]         = "";
	static BYTE CSCONST szEqual[]              = " = ";
	static BYTE CSCONST szComma[]              = ", ";
	static BYTE CSCONST szPeriod[]             = ".";
	static BYTE CSCONST szExclam[]             = "!";
	static BYTE CSCONST szTilde[]              = " ~";
	static BYTE CSCONST sz_NULL[]              = "NULL";
	static BYTE CSCONST sz_NULL_CONNECTION[]   = NULL_CONNECTION;
	static BYTE CSCONST sz_NULL_STATEMENT[]    = NULL_STATEMENT;
	static BYTE CSCONST sz_App_Error[]         = "Application Error";
	static BYTE CSCONST szTimerError[]         = "unable to set timer";
	static BYTE CSCONST szASYNC[]              = "Async";
	static BYTE CSCONST szRECORD[]             = "Trace";
	static BYTE CSCONST sz_Application_error[] = "Application error";
	static BYTE CSCONST szClear[]              = "Clear";
	static BYTE CSCONST szClearInputWindow[]   = "Clear input window?";
	static BYTE CSCONST szClearOutputWindow[]  = "Clear output window?";
	static BYTE CSCONST sz_top[]               = "top";
	static BYTE CSCONST szWindowActive[]       =
								"Input or Output window must be active to clear";

	HWND	hwndMain,		hwndTop,		hwndBot;
	HWND	hwndHdbcList,	hwndHdbcName;
	HWND	hwndHstmtList,	hwndHstmtName;

	CONNINFO	*lpconninfo;		// Connection information
	DDAINFO		*lpddainfo; 		// DDA function information

	static LPAF lpafFirst = NULL; 	// for Asynchronous operation

		// The lrgserverinfo structure holds information pertaining to each
		// connection established.  Refer to the SERVERINFO typedef in the
		// header file adhoc.h.

	SERVERINFO	*lrgserverinfo;
	SERVERINFO	*lpserverinfoMax;
//	UWORD		idbcMax = 0;

	STMTINFO	*lrgstmtinfo;
	STMTINFO	*lpstmtinfoMax;
	UWORD		istmtMax = 0;

	UWORD		wMemStatus = DEBUG_STATUS_OK;

	HINSTANCE			hinst;		// Current app instance
	static HINSTANCE	hinstTop;	// Current top window instance
	static HINSTANCE	hinstBot;	// Current bottom window instance

	HENV	henv	= SQL_NULL_HENV;	// handle of current environment
	HDBC	hdbc	= SQL_NULL_HDBC;	// handle of current connection
	UWORD	idbc	= 0;				// index of current connection
	HSTMT	hstmt	= SQL_NULL_HSTMT;	// handle of current statement
	UWORD	istmt	= 0;				// index of current statement

	Str255	hdbcText	= "\pNULL";
	Str255	hstmtText	= "\p*NULL";

	int		fAsyncInterval = 0;

		// One global OutStr! (with overflow check byte)
		//
	char	OutStr[ OUTSTRLEN + 1 ];

//	HCURSOR		hSaveCursor;
//	HCURSOR		hHourGlass;
//mj	HBITMAP		hAsyncBMP;
	BOOL		fSetWinPos		= FALSE;
	BOOL		fDriverMenus	=TRUE;


	static BOOL fSeparatorNeeded = TRUE;

#define cchEditWinHardMax	64000
#define cchEditWinSoftMax	60000
#define cchEditWinTruncMin	2000

	static HWND hwnd_EN_ERRSPACE = NULL;
	
	static BOOL  fEnableStackChecking = FALSE;
	static UWORD cbIsAtLeast          = 1024;
	static UWORD dcbIncreasesBy       = 1024;
	static UWORD dcbDecreasesBy       = 1024;

	typedef struct
	{
		BOOL fChanged;
		SEL sel;
	} IWINFO;

	// rgiwinfo[ ciwinfo ]			// Input Window information

	static IWINFO	*lrgiwinfo;

	pascal VOID FreeEverything();
	pascal VOID WriteProfile( HWND hwnd );
	pascal VOID FileOpen( HWND hwnd );
	pascal VOID FileSave( HWND hwnd );
	pascal VOID ProcessPendingAsync();
	pascal VOID FetchOrGetDataAll( HWND hwnd, BOOL fGetData );
	pascal BOOL FGetDataRow( UWORD icolBoundMost, SWORD ccol );
	pascal BOOL FTerseDescribeColAll( SWORD ccol );
	pascal BOOL FTerseDisplayOutputBuf( RETCODE retcode,
									BOOL fExpectNoData, BOOL fExpectError,
									LPBYTE lpbBuf, SDWORD cbBufMax, SDWORD cbBuf );


	BOOL fAssertOn = TRUE;	// To disable asserts, use CodeView to clear this.
	SDWORD sdwError = 0;	// Optional error value displayed in assert alert.


	FNAM CSCONST rgfnam_retcode[] =
	{
		{ SQL_SUCCESS,				sz_SQL_SUCCESS				},
		{ SQL_SUCCESS_WITH_INFO,	sz_SQL_SUCCESS_WITH_INFO	},
		{ SQL_STILL_EXECUTING,		sz_SQL_STILL_EXECUTING		},
		{ SQL_NO_DATA_FOUND,		sz_SQL_NO_DATA_FOUND		},
		{ SQL_ERROR,				sz_SQL_ERROR				},
		{ SQL_NEED_DATA,			sz_SQL_NEED_DATA			},
		{ SQL_INVALID_HANDLE,		sz_SQL_INVALID_HANDLE		}
	};

	DeclareFinfo( retcode, "SQL_", FINFO_SWORD, GLOBAL );


pascal BOOL FAssertFail( LPSTR lszModule, short iLine )
{
	static BYTE CSCONST szFmt[] = "Assert failed (%ld) in %s at line %d. Continue?";
	char sz[ 256 ];

	wsprintf( sz, szFmt, sdwError, lszModule, iLine );
	sdwError = 0;

#ifdef WINDOWS
	switch( MessageBox( hwndMain, sz, sz_App_Error,
			MB_YESNOCANCEL | MB_ICONEXCLAMATION ) )
	{
	case IDYES:	return FALSE;
	case IDNO:	return TRUE;
	}
	szFmt[0] = 'A'; 	// Abort Gator by causing a GP Fault.
#endif WINDOWS
	return TRUE;
}/* end FAssertFail */


	/* Window word values for child windows */
#define GWW_HWNDEDIT	0
//#define GWW_CHANGED	2
#define GWW_WORDWRAP	4
//#define GWW_UNTITLED	6

	//  FUNCTION: AppInit( nCmdShow )

	//  PURPOSE: Initializes application windows and global variables

pascal BOOL AppInit()
{
	static BYTE CSCONST sz_STATIC[]			= "STATIC";
	static BYTE CSCONST sz_hdbc[]			= "hd&bc:";
	static BYTE CSCONST sz_COMBOBOX[]		= "COMBOBOX";
	static BYTE CSCONST sz_hstmt[]			= "hs&tmt:";
	static BYTE CSCONST sz_bottom[]			= "bottom";
	static BYTE CSCONST sz_SAGMENU[]		= "SAGMENU";
	static BYTE CSCONST sz_ASYNCBITMAP[]	= "ASYNCBITMAP";

	HANDLE		hmem;
//	LPISA		lpisa;


	OutStr[ OUTSTRLEN ] = 0xCC;	// Set buffer overflow check byte

	hmem = AllocateMemory(	sizeof( SERVERINFO ) * MAX_CONNECTIONS +
							sizeof( STMTINFO   ) * MAX_STATEMENTS	);
	if( hmem == NULL )
		return FALSE;

	lrgserverinfo   = LockMemory( hmem );
	lpserverinfoMax = &lrgserverinfo[ MAX_CONNECTIONS ];
	lrgstmtinfo     = (void*)lpserverinfoMax;
	lpstmtinfoMax   = &lrgstmtinfo[ MAX_STATEMENTS ];


	hmem = AllocateMemory( sizeof( CONNINFO ) );
	if( hmem == NULL )
		return FALSE;
	lpconninfo = LockMemory( hmem );

#ifdef CRAP
	lpisa = LpisaInitISA();
	lpconninfo->lpisaDSN		= lpisa;
	(lpconninfo->lpisaUID		= lpisa)->cuse++;
	(lpconninfo->lpisaAuthStr	= lpisa)->cuse++;
	(lpconninfo->lpisaConnStrIn	= lpisa)->cuse++;
#else
	lpconninfo->lpisaDSN		= LpisaInitISA();
	lpconninfo->lpisaUID		= LpisaInitISA();
	lpconninfo->lpisaAuthStr	= LpisaInitISA();
	lpconninfo->lpisaConnStrIn	= LpisaInitISA();
#endif CRAP

	lpconninfo->cbConnStrOutMax   = MAX_CONNECTION_STRING;
	lpconninfo->fDriverCompletion = SQL_DRIVER_COMPLETE;

		/* Allocate some input window information memory	*/

	hmem = AllocateMemory(	sizeof( DDAINFO ) + sizeof( IWINFO ) * ciwinfo );
	if( hmem == NULL )
		return FALSE;
	lpddainfo = LockMemory( hmem );		// long pointer to Dialog Data ?? info
	lrgiwinfo = (void*)(lpddainfo + 1);


#ifdef CRAP
	lpddainfo->lpisaTableQualifier		= lpisa =
												LpisaCreateISA( TRUE, 0, 0, 0 );
	(lpddainfo->lpisaTableOwner			= lpisa)->cuse++;
	(lpddainfo->lpisaTableName			= lpisa)->cuse++;
	(lpddainfo->lpisaTableType			= lpisa)->cuse++;
	(lpddainfo->lpisaColumnName			= lpisa)->cuse++;
	(lpddainfo->lpisaFkTableQualifier	= lpisa)->cuse++;
	(lpddainfo->lpisaFkTableOwner		= lpisa)->cuse++;
	(lpddainfo->lpisaFkTableName		= lpisa)->cuse++;
	(lpddainfo->lpisaProcQualifier		= lpisa)->cuse++;
	(lpddainfo->lpisaProcOwner			= lpisa)->cuse++;
	(lpddainfo->lpisaProcName			= lpisa)->cuse++;
	(lpddainfo->lpisaCursor				= lpisa)->cuse++;
#else
	lpddainfo->lpisaTableQualifier	= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaTableOwner		= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaTableName		= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaTableType		= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaColumnName		= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaFkTableQualifier= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaFkTableOwner	= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaFkTableName		= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaProcQualifier	= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaProcOwner		= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaProcName		= LpisaCreateISA( TRUE, 256, 255, 0 );
	lpddainfo->lpisaCursor			= LpisaCreateISA( TRUE, 256, 255, 0 );
#endif CRAP

	lpddainfo->fSqlType		= SQL_ALL_TYPES;
	lpddainfo->fColType		= SQL_BEST_ROWID;
	lpddainfo->fUnique		= SQL_INDEX_UNIQUE;
	lpddainfo->fAccuracy	= SQL_QUICK;
	lpddainfo->fScope		= SQL_SCOPE_CURROW;
	lpddainfo->fNullable	= SQL_NO_NULLS;

//	if( !FAllocAutoTest( TRUE ) )	// Allocate storage for AutoTest
//		return FALSE;

			// The windows were created here !!!!!!!!!!

	return TRUE;			// Initialization was successful
}/* end AppInit */


pascal VOID FreeEverything()
{
	SERVERINFO	*lpserverinfo;
	STMTINFO	*lpstmtinfo;
	BOOL		fSuccess = TRUE;

	if( !FreeParamsAndCols( FREE_ALL, SQL_NULL_HSTMT ) )
	{
		static BYTE CSCONST szMsg[] = "Could not complete deallocation"
			" of global storage for param and col information!";

		MessageBox( hwndMain, szMsg, sz_App_Error, MB_OK | MB_ICONEXCLAMATION );
	}

		// call CANCEL on all hstmts just to be sure and prevent problems
		// from half-executed data_at_exec queries
	for( lpstmtinfo = &lrgstmtinfo[0];
		 lpstmtinfo < lpstmtinfoMax; lpstmtinfo++ )
		if( lpstmtinfo->hstmt )
		{
				// Open connection found, go ahead and close it

			fSuccess &= SQLCancel( lpstmtinfo->hstmt ) == SQL_SUCCESS;
		}
		

		// Check for open connections

	for( lpserverinfo = &lrgserverinfo[0];
		 lpserverinfo < lpserverinfoMax; lpserverinfo++ )
	{
		if( lpserverinfo->hdbc )
		{
			// Open connection found, go ahead and close it

			if( lpserverinfo->fStatus == Connected )
				fSuccess &= SQLDisconnect( lpserverinfo->hdbc ) == 
					SQL_SUCCESS;
			fSuccess &= SQLFreeConnect(lpserverinfo->hdbc) == SQL_SUCCESS;
		}
	}/* end FOR */

	if( henv )
	{
		fSuccess &= SQLFreeEnv( henv ) == SQL_SUCCESS;
		henv = NULL;
	}

	if( !fSuccess )	// Didn't they all work?
	{
		static BYTE CSCONST szMsg[] = "Failed during one or more of"
			" SQLDisconnect, SQLFreeConnect, SQLFreeEnv!";

		MessageBox( hwndMain, szMsg, sz_App_Error, MB_OK | MB_ICONEXCLAMATION );
	}

		// Free globally allocated arrays and structs:

	while( lpafFirst )
		DeleteAf( &lpafFirst );

	ReleaseISA( lpddainfo->lpisaTableQualifier );
	ReleaseISA( lpddainfo->lpisaTableOwner );
	ReleaseISA( lpddainfo->lpisaTableName );
	ReleaseISA( lpddainfo->lpisaTableType );
	ReleaseISA( lpddainfo->lpisaColumnName );
	ReleaseISA( lpddainfo->lpisaFkTableQualifier );
	ReleaseISA( lpddainfo->lpisaFkTableOwner );
	ReleaseISA( lpddainfo->lpisaFkTableName );
	ReleaseISA( lpddainfo->lpisaProcQualifier );
	ReleaseISA( lpddainfo->lpisaProcOwner );
	ReleaseISA( lpddainfo->lpisaProcName );

	ReleaseMemory( lpddainfo );			// also frees lrgiwinfo

	ReleaseMemory( lrgserverinfo );		// also frees lrgstmtinfo

	ReleaseISA( lpconninfo->lpisaDSN );
	ReleaseISA( lpconninfo->lpisaUID );
	ReleaseISA( lpconninfo->lpisaAuthStr );
	ReleaseISA( lpconninfo->lpisaConnStrIn );

	ReleaseMemory( lpconninfo );
	
		// Free storage allocated for AutoTest

	if( !FAllocAutoTest( FALSE ) )
	{
		static BYTE CSCONST szMsg[] =
			"Could not complete deallocation of global storage!";

		MessageBox( hwndMain, szMsg, sz_App_Error,
			MB_OK | MB_ICONEXCLAMATION );
	}

		// Buffer overflow check:

	if( (BYTE)OutStr[ OUTSTRLEN ] != 0xCC )
	{
		static BYTE CSCONST szMsg[] = "The global output buffer,"
			" OutStr, has overflowed during this run!";

		MessageBox( hwndMain, szMsg, sz_App_Error, MB_OK | MB_ICONEXCLAMATION );
	}
	return;
}/* end FreeEverything */


pascal VOID WriteProfile( HWND hwnd )
{
	OFSTRUCT	ofs;

	OpenFile( szGatorIni, &ofs, OF_EXIST );
	WritePrivateProfileString( szStartup, szMaximized,
		itoa( 0 != (HIWORD( WS_MAXIMIZE ) &
			       HIWORD( GetWindowLong( hwnd, GWL_STYLE ) ) ),
			OutStr,10 ),
		ofs.szPathName );
	return;
}/* end WriteProfile */


pascal VOID FileOpen( HWND /*hwnd*/ )
{
	return;
}/* end FileOpen */

pascal VOID FileSave( HWND /*hwnd*/ )
{
	return;
}/* end FileSave */


#define MAXQRYDISPLAY 400


pascal VOID BuildAsyncAlertHeader( BOOL fEchoRetcode,
					TSD *ptsd, LPBYTE lszFuncName, RETCODE retcode, LPAF lpaf )
{
	if( fEchoRetcode )
		EchoRetcode( lszFuncName, retcode );

	if( ptsd->lsz == NULL )
		ptsd->lsz = LockMemory( AllocateMemory( MAXQRYDISPLAY + 100 ) );

	ptsd->lpch = ptsd->lsz;
	ptsd->hasz = NULL;
	ptsd->cbOutMax = MAXQRYDISPLAY;  // Affects only GetValueDisplay

	TakeStringDisplay( ptsd, lszFuncName );
	TakeStringDisplay( ptsd, "(#");
	TakeStringDisplay( ptsd, SzConstDisplay( NULL, lpaf->General.istmt ) );
}/* end BuildAsyncAlertHeader */


pascal VOID BuildAsyncAlertFlagArg( TSD *ptsd, FINFO *lpfinfo, SDWORD fFlag )
{
	TakeStringDisplay(	ptsd, szComma );
	TakeStringDisplay(	ptsd,
						SzConstDisplay(
								LszIfSz(	lpfinfo == NULL,
											NULL,
											LszNameFromFlagArg( lpfinfo, fFlag ) ),
								fFlag ) );
	return;
}/* end BuildAsyncAlertFlagArg */


pascal VOID BuildAsyncAlertRgbAndCbArgs( TSD *ptsd, BOOL fPutData, SWORD fCType,
											HPBYTE hsz, SDWORD cb, SDWORD cbMax )
{
	TakeStringDisplay( ptsd, szComma );
	GetValueDisplay( ptsd, fCType, hsz, cbMax );
	TakeStringDisplay( ptsd, szComma );
	TakeStringDisplay( ptsd,
		SzConstDisplay(	LszIfSz( fCType == SQL_C_CHAR && cb == SQL_NTS,
							sz_SQL_NTS,
								LszIfSz( fPutData && cb == SQL_NULL_DATA,
											sz_SQL_NULL_DATA, NULL ) ),
						cb ) );
	return;
}/* end BuildAsyncAlertRgbAndCbArgs */


pascal VOID BuildAsyncAlertISAs( TSD *ptsd, LPISA lpisa )
{
	BuildAsyncAlertRgbAndCbArgs(	ptsd, FALSE, SQL_C_CHAR,
									LszAndCbOfLpisa( lpisa ),
									lpisa->cbMax );
	return;
}/* end BuildAsyncAlertISAs */


pascal VOID BuildAsyncAlertDDAArgs(TSD * ptsd, LPAF lpaf )
{
	BuildAsyncAlertISAs( ptsd, lpaf->DDA.lpisaTableQualifier	);
	BuildAsyncAlertISAs( ptsd, lpaf->DDA.lpisaTableOwner		);
	BuildAsyncAlertISAs( ptsd, lpaf->DDA.lpisaTableName		);
	return;
}/* end BuildAsyncAlertDDAArgs */


pascal VOID BuildAsyncAlertOutputPointerArg( TSD *ptsd, SEGMENT seg, LPBYTE lsz )
{
	TakeStringDisplay( ptsd, szComma );
	TakeStringDisplay( ptsd, LszIfSz( seg == NULL, sz_NULL, lsz ) );
	return;
}/* end BuildAsyncAlertOutputPointerArg */


pascal VOID ProcessPendingAsync()
{
		// For SQLColAttributes:
	static BYTE CSCONST sz_rgbDesc[] = "rgbDesc";
	static BYTE CSCONST sz_pcbDesc[] = "pcbDesc";
	static BYTE CSCONST sz_pfDesc[]  = "pfDesc";

		// For SQLDescribeCol, SQLDescribeParam:
	static BYTE CSCONST sz_szColName[]  = "szColName";
	static BYTE CSCONST sz_pcbColName[] = "pcbColName";
	static BYTE CSCONST sz_pfSqlType[]  = "pfSqlType";
	static BYTE CSCONST sz_pcbColDef[]  = "pcbColDef";
	static BYTE CSCONST sz_pibScale[]   = "pibScale";
	static BYTE CSCONST sz_pfNullable[] = "pfNullable";

		// For SQLExtendedFetch:
	static BYTE CSCONST sz_pcrow[]        = "pcrow";
	static BYTE CSCONST sz_rgfRowStatus[] = "rgfRowStatus";

		// For SQLGetData:
	static BYTE CSCONST sz_rgbValue[] = "rgbValue";
	static BYTE CSCONST sz_pcbValue[] = "pcbValue";

		// For SQLNumResultCols:
	static BYTE CSCONST sz_pccol[] = "pccol";

		// For SQLParamData:
	static BYTE CSCONST sz_prgbValue[] = "prgbValue";

	static BYTE CSCONST sz_Invalid_async_state[] = "Invalid asynchronous state!";

	RETCODE		retcode;
	LPAF		*lplpaf = &lpafFirst;
	LPAF		lpaf;
	TSD			tsd;

	extern FINFO	finfo_fDescType;
	extern FINFO	finfo_fFetchType;
	extern FINFO	finfo_fSqlType_All;
	extern FINFO	finfo_BOOL;
	extern FINFO	finfo_fColType;
	extern FINFO	finfo_fScope;
	extern FINFO	finfo_fNullable_WithoutUnknown;
	extern FINFO	finfo_fUnique;
	extern FINFO	finfo_fAccuracy;
	extern FINFO	finfo_fCType;

	tsd.lsz = NULL;

	while( ( lpaf = *lplpaf ) != NULL )
	{
		switch( lpaf->General.ifn )
		{
		case ifnColAttributes:
			retcode = SQLColAttributes(	lpaf->ColAttributes.hstmt,
										lpaf->ColAttributes.icol,
										lpaf->ColAttributes.fDescType,
										lpaf->ColAttributes.lrgbDesc,
										lpaf->ColAttributes.cbDescMax,
										lpaf->ColAttributes.lpcbDesc,
										lpaf->ColAttributes.lpfDesc );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( FALSE, &tsd,
											sz_SQLColAttributes, retcode, lpaf );
			BuildAsyncAlertFlagArg( &tsd,
								&finfo_fDescType, lpaf->ColAttributes.fDescType );
			BuildAsyncAlertOutputPointerArg( &tsd,
								(SEGMENT)lpaf->ColAttributes.lrgbDesc, sz_rgbDesc );
			BuildAsyncAlertFlagArg( &tsd, NULL, lpaf->ColAttributes.cbDescMax );
			BuildAsyncAlertOutputPointerArg( &tsd,
								(SEGMENT)lpaf->ColAttributes.lpcbDesc, sz_pcbDesc );
			BuildAsyncAlertOutputPointerArg( &tsd,
								(SEGMENT)lpaf->ColAttributes.lpfDesc, sz_pfDesc );

			DisplayColAttributes( retcode,	lpaf->ColAttributes.icol,
											lpaf->ColAttributes.fDescType,
											lpaf->ColAttributes.lrgbDesc,
											lpaf->ColAttributes.cbDescMax,
											lpaf->ColAttributes.lpcbDesc,
											lpaf->ColAttributes.lpfDesc );
			break;

		case ifnColumnPrivileges:
			retcode = SQLColumnPrivileges(
								lpaf->ColumnPrivileges.hstmt,
								LszAndCbOfLpisa(
									lpaf->ColumnPrivileges.lpisaTableQualifier ),
								LszAndCbOfLpisa(
									lpaf->ColumnPrivileges.lpisaTableOwner ),
								LszAndCbOfLpisa(
									lpaf->ColumnPrivileges.lpisaTableName ),
								LszAndCbOfLpisa(
									lpaf->ColumnPrivileges.lpisaColumnName ));
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLColumnPrivileges, retcode, lpaf );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ColumnPrivileges.lpisaColumnName );
			break;

		case ifnColumns:
			retcode = SQLColumns(
							lpaf->Columns.hstmt,
							LszAndCbOfLpisa( lpaf->Columns.lpisaTableQualifier ),
							LszAndCbOfLpisa( lpaf->Columns.lpisaTableOwner ),
							LszAndCbOfLpisa( lpaf->Columns.lpisaTableName ),
							LszAndCbOfLpisa( lpaf->Columns.lpisaColumnName ) );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd, sz_SQLColumns, retcode, lpaf );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			BuildAsyncAlertISAs( &tsd, lpaf->Columns.lpisaColumnName );
			break;

		case ifnDescribeCol:
			retcode = SQLDescribeCol(	lpaf->DescribeCol.hstmt,
										lpaf->DescribeCol.icol,
										lpaf->DescribeCol.lszColName,
										lpaf->DescribeCol.cbColNameMax,
										lpaf->DescribeCol.lpcbColName,
										lpaf->DescribeCol.lpfSqlType,
										lpaf->DescribeCol.lpcbColDef,
										lpaf->DescribeCol.lpibScale,
										lpaf->DescribeCol.lpfNullable );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( FALSE, &tsd,
				sz_SQLDescribeCol, retcode, lpaf );
			BuildAsyncAlertFlagArg( &tsd, NULL, lpaf->DescribeCol.icol );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->DescribeCol.lszColName, sz_szColName );
			BuildAsyncAlertFlagArg( &tsd,
				NULL, lpaf->DescribeCol.cbColNameMax );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->DescribeCol.lpcbColName, sz_pcbColName );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->DescribeCol.lpfSqlType, sz_pfSqlType );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->DescribeCol.lpcbColDef, sz_pcbColDef );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->DescribeCol.lpibScale, sz_pibScale);
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->DescribeCol.lpfNullable, sz_pfNullable );

			DisplayDescribeRef( FALSE, 2, retcode,
				lpaf->DescribeCol.icol,
				lpaf->DescribeCol.lszColName,
				lpaf->DescribeCol.cbColNameMax,
				lpaf->DescribeCol.lpcbColName,
				lpaf->DescribeCol.lpfSqlType,
				lpaf->DescribeCol.lpcbColDef,
				lpaf->DescribeCol.lpibScale,
				lpaf->DescribeCol.lpfNullable );
			break;

		case ifnDescribeParam:
			retcode = SQLDescribeParam(	lpaf->DescribeParam.hstmt,
										lpaf->DescribeParam.ipar,
										lpaf->DescribeParam.lpfSqlType,
										lpaf->DescribeParam.lpcbColDef,
										lpaf->DescribeParam.lpibScale,
										lpaf->DescribeParam.lpfNullable );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( FALSE, &tsd,
											sz_SQLDescribeParam, retcode, lpaf );
			BuildAsyncAlertFlagArg( &tsd, NULL, lpaf->DescribeParam.ipar );
			BuildAsyncAlertOutputPointerArg( &tsd,
						(SEGMENT)lpaf->DescribeParam.lpfSqlType, sz_pfSqlType );
			BuildAsyncAlertOutputPointerArg( &tsd,
						(SEGMENT)lpaf->DescribeParam.lpcbColDef, sz_pcbColDef );
			BuildAsyncAlertOutputPointerArg( &tsd,
							(SEGMENT)lpaf->DescribeParam.lpibScale, sz_pibScale);
			BuildAsyncAlertOutputPointerArg( &tsd,
						(SEGMENT)lpaf->DescribeParam.lpfNullable, sz_pfNullable );

			DisplayDescribeRef( TRUE, 2, retcode,
											lpaf->DescribeParam.ipar,
											NULL,
											0,
											NULL,
											lpaf->DescribeParam.lpfSqlType,
											lpaf->DescribeParam.lpcbColDef,
											lpaf->DescribeParam.lpibScale,
											lpaf->DescribeParam.lpfNullable );
			break;

		case ifnExecDirect:
			retcode = SQLExecDirect(
					lpaf->ExecDirect.hstmt,
					lpaf->ExecDirect.hszSqlStr,
					lpaf->ExecDirect.cbSqlStr);
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLExecDirect, retcode, lpaf );
			BuildAsyncAlertRgbAndCbArgs(	&tsd, FALSE, SQL_C_CHAR,
											lpaf->ExecDirect.hszSqlStr,
											lpaf->ExecDirect.cbSqlStr,
											lpaf->ExecDirect.cbDefault + 1 );
			break;

		case ifnExecute:
			retcode = SQLExecute( lpaf->Execute.hstmt );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd, sz_SQLExecute, retcode, lpaf );
			break;

		case ifnExtendedFetch:
			retcode = SQLExtendedFetch(
					lpaf->ExtendedFetch.hstmt,
					lpaf->ExtendedFetch.fFetchType,
					lpaf->ExtendedFetch.irow,
					lpaf->ExtendedFetch.lpcrow,
					lpaf->ExtendedFetch.lrgfRowStatus );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( FALSE, &tsd, sz_SQLExtendedFetch, retcode, lpaf );
			BuildAsyncAlertFlagArg( &tsd,
							&finfo_fFetchType, lpaf->ExtendedFetch.fFetchType );
			BuildAsyncAlertFlagArg( &tsd, NULL, lpaf->ExtendedFetch.irow);
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->ExtendedFetch.lpcrow, sz_pcrow);
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->ExtendedFetch.lrgfRowStatus,
				sz_rgfRowStatus );

			DisplayExtendedFetch( retcode,
				lpaf->ExtendedFetch.fFetchType,
				lpaf->ExtendedFetch.irow,
				lpaf->ExtendedFetch.lpcrow,
				lpaf->ExtendedFetch.lrgfRowStatus );
			break;

		case ifnFetch:
			retcode = SQLFetch( lpaf->Fetch.hstmt );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd, sz_SQLFetch, retcode, lpaf );

			IcolBoundColChecks( lpaf->Fetch.hstmt, 0, retcode );
			break;

		case ifnForeignKeys:
			retcode = SQLForeignKeys(
					lpaf->ForeignKeys.hstmt,
					LszAndCbOfLpisa(
						lpaf->ForeignKeys.lpisaPkTableQualifier ),
					LszAndCbOfLpisa( lpaf->ForeignKeys.lpisaPkTableOwner ),
					LszAndCbOfLpisa( lpaf->ForeignKeys.lpisaPkTableName ),
					LszAndCbOfLpisa(
						lpaf->ForeignKeys.lpisaFkTableQualifier ),
					LszAndCbOfLpisa( lpaf->ForeignKeys.lpisaFkTableOwner ),
					LszAndCbOfLpisa( lpaf->ForeignKeys.lpisaFkTableName ) );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLForeignKeys, retcode, lpaf );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ForeignKeys.lpisaFkTableQualifier );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ForeignKeys.lpisaFkTableOwner );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ForeignKeys.lpisaFkTableName );
			break;

		case ifnGetData:
			retcode = SQLGetData(
					lpaf->GetData.hstmt,
					lpaf->GetData.icol,
					lpaf->GetData.fCType,
					lpaf->GetData.hrgbValue,
					lpaf->GetData.cbValueMax,
					lpaf->GetData.lpcbValue );
			if( retcode==SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertFlagArg( &tsd, NULL, lpaf->GetData.icol );
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_fCType, lpaf->GetData.fCType );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->GetData.hrgbValue, sz_rgbValue );
			BuildAsyncAlertFlagArg( &tsd, NULL, lpaf->GetData.cbValueMax );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->GetData.lpcbValue, sz_pcbValue );

			DisplayGetData( retcode,
				lpaf->GetData.fCType,
				lpaf->GetData.hrgbValue,
				lpaf->GetData.cbValueMax,
				lpaf->GetData.lpcbValue );
			break;

		case ifnGetTypeInfo:
			retcode = SQLGetTypeInfo(
					lpaf->GetTypeInfo.hstmt,
					lpaf->GetTypeInfo.fSqlType );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLGetTypeInfo, retcode, lpaf );
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_fSqlType_All, lpaf->GetTypeInfo.fSqlType );
			break;

		case ifnMoreResults:
			retcode = SQLMoreResults( lpaf->MoreResults.hstmt );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLMoreResults, retcode, lpaf );

			IcolBoundColChecks(
				lpaf->MoreResults.hstmt, ALWAYS_UNTOUCHED, 0 );
			break;

		case ifnNumResultCols:
			retcode = SQLNumResultCols(
				lpaf->NumResultCols.hstmt,
				lpaf->NumResultCols.lpccol);
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( FALSE, &tsd,
				sz_SQLNumResultCols, retcode, lpaf );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->NumResultCols.lpccol, sz_pccol);

			DisplayNumRefs( retcode, FALSE, lpaf->NumResultCols.lpccol);
			break;

		case ifnParamData:
			retcode = SQLParamData(
					lpaf->ParamData.hstmt,
					lpaf->ParamData.lplrgbValue );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( FALSE, &tsd,
				sz_SQLParamData, retcode, lpaf );
			BuildAsyncAlertOutputPointerArg( &tsd,
				(SEGMENT)lpaf->ParamData.lplrgbValue, sz_prgbValue );

			DisplayParamData( retcode, lpaf->ParamData.lplrgbValue );
			break;

		case ifnPrepare:
			retcode = SQLPrepare(
					lpaf->Prepare.hstmt,
					lpaf->Prepare.hszSqlStr,
					lpaf->Prepare.cbSqlStr);
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLPrepare, retcode, lpaf );
			BuildAsyncAlertRgbAndCbArgs( &tsd, FALSE, SQL_C_CHAR,
				lpaf->Prepare.hszSqlStr,
				lpaf->Prepare.cbSqlStr,
				lpaf->Prepare.cbDefault + 1);
			break;

		case ifnPrimaryKeys:
			retcode = SQLPrimaryKeys(
					lpaf->PrimaryKeys.hstmt,
					LszAndCbOfLpisa( lpaf->PrimaryKeys.lpisaTableQualifier ),
					LszAndCbOfLpisa( lpaf->PrimaryKeys.lpisaTableOwner ),
					LszAndCbOfLpisa( lpaf->PrimaryKeys.lpisaTableName ));
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLPrimaryKeys, retcode, lpaf );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			break;

		case ifnProcedureColumns:
			retcode = SQLProcedureColumns(
					lpaf->ProcedureColumns.hstmt,
					LszAndCbOfLpisa(
						lpaf->ProcedureColumns.lpisaProcQualifier ),
					LszAndCbOfLpisa(
						lpaf->ProcedureColumns.lpisaProcOwner ),
					LszAndCbOfLpisa(
						lpaf->ProcedureColumns.lpisaProcName ),
					LszAndCbOfLpisa(
						lpaf->ProcedureColumns.lpisaColumnName ));
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLProcedureColumns, retcode, lpaf );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ProcedureColumns.lpisaProcQualifier );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ProcedureColumns.lpisaProcOwner );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ProcedureColumns.lpisaProcName );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ProcedureColumns.lpisaColumnName );
			break;

		case ifnProcedures:
			retcode = SQLProcedures(
					lpaf->Procedures.hstmt,
					LszAndCbOfLpisa( lpaf->Procedures.lpisaProcQualifier ),
					LszAndCbOfLpisa( lpaf->Procedures.lpisaProcOwner ),
					LszAndCbOfLpisa( lpaf->Procedures.lpisaProcName ));
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLProcedures, retcode, lpaf );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ProcedureColumns.lpisaProcQualifier );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ProcedureColumns.lpisaProcOwner );
			BuildAsyncAlertISAs( &tsd,
				lpaf->ProcedureColumns.lpisaProcName );
			break;

		case ifnPutData:
			retcode = SQLPutData(
					lpaf->PutData.hstmt,
					lpaf->PutData.hrgbValue,
					lpaf->PutData.cbValue );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLPutData, retcode, lpaf );
			BuildAsyncAlertRgbAndCbArgs( &tsd, TRUE,
				lpaf->PutData.fCType,
				lpaf->PutData.hrgbValue,
				lpaf->PutData.cbValue,
				lpaf->PutData.cbValueMax );
			break;

		case ifnSetPos:
			retcode = SQLSetPos(
					lpaf->SetPos.hstmt,
					lpaf->SetPos.irow,
					lpaf->SetPos.fRefresh,
					lpaf->SetPos.fLock);
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLSetPos, retcode, lpaf );
			BuildAsyncAlertFlagArg( &tsd, NULL, lpaf->SetPos.irow);
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_BOOL, lpaf->SetPos.fRefresh);
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_BOOL, lpaf->SetPos.fLock);
			break;

		case ifnSpecialColumns:
			retcode = SQLSpecialColumns(
					lpaf->SpecialColumns.hstmt,
					lpaf->SpecialColumns.fColType,
					LszAndCbOfLpisa(
						lpaf->SpecialColumns.lpisaTableQualifier ),
					LszAndCbOfLpisa( lpaf->SpecialColumns.lpisaTableOwner ),
					LszAndCbOfLpisa( lpaf->SpecialColumns.lpisaTableName ),
					lpaf->SpecialColumns.fScope,
					lpaf->SpecialColumns.fNullable );
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLSpecialColumns, retcode, lpaf );
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_fColType, lpaf->SpecialColumns.fColType );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_fScope, lpaf->SpecialColumns.fScope);
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_fNullable_WithoutUnknown,
				lpaf->SpecialColumns.fNullable );
			break;

		case ifnStatistics:
			retcode = SQLStatistics(
					lpaf->Statistics.hstmt,
					LszAndCbOfLpisa( lpaf->Statistics.lpisaTableQualifier ),
					LszAndCbOfLpisa( lpaf->Statistics.lpisaTableOwner ),
					LszAndCbOfLpisa( lpaf->Statistics.lpisaTableName ),
					lpaf->Statistics.fUnique,
					lpaf->Statistics.fAccuracy);
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLStatistics, retcode, lpaf );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_fUnique, lpaf->Statistics.fUnique);
			BuildAsyncAlertFlagArg( &tsd,
				&finfo_fAccuracy, lpaf->Statistics.fAccuracy);
			break;

		case ifnTablePrivileges:
			retcode = SQLTablePrivileges(
					lpaf->TablePrivileges.hstmt,
					LszAndCbOfLpisa(
						lpaf->TablePrivileges.lpisaTableQualifier ),
					LszAndCbOfLpisa( lpaf->TablePrivileges.lpisaTableOwner ),
					LszAndCbOfLpisa( lpaf->TablePrivileges.lpisaTableName ));
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLTablePrivileges, retcode, lpaf );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			break;

		case ifnTables:
			retcode = SQLTables(
					lpaf->Tables.hstmt,
					LszAndCbOfLpisa( lpaf->Tables.lpisaTableQualifier ),
					LszAndCbOfLpisa( lpaf->Tables.lpisaTableOwner ),
					LszAndCbOfLpisa( lpaf->Tables.lpisaTableName ),
					LszAndCbOfLpisa( lpaf->Tables.lpisaTableType ));
			if( retcode == SQL_STILL_EXECUTING )
				break;

			BuildAsyncAlertHeader( TRUE, &tsd,
				sz_SQLTables, retcode, lpaf );
			BuildAsyncAlertDDAArgs( &tsd, lpaf );
			BuildAsyncAlertISAs( &tsd,
				lpaf->Tables.lpisaTableType );
			break;

		default:
			szWrite( sz_Error,FALSE );
			szWrite( sz_Invalid_async_state,TRUE );
			break;
		}
		if( retcode == SQL_STILL_EXECUTING )
			lplpaf = &lpaf->General.lpafNext;
		else
		{
			static BYTE CSCONST szTitle[] =
				"Asynchronous Operation Completed";

			TakeStringDisplay( &tsd, ")");
			MessageBox( hwndMain, tsd.lsz, szTitle, MB_OK );
			DeleteAf( lplpaf );
			}
		}
	ReleaseMemory( tsd.lsz );
}/* end ProcessPendingAsync */

#ifdef MACINTOSH
pascal BOOL FAbortCheck()
{
//debugstr( "FAbortCheck not yet implemented" );
	return FALSE;
}
#else
pascal BOOL FAbortCheck()
{
	static BYTE CSCONST sz_Continuing[] = "Continuing...";
	static BYTE CSCONST sz_Paused[]     =
		"Paused - hit any key to continue, or Esc to abort.";

	BOOL	fAbort = FALSE;
	MSG		msg;
	
	while( PeekMessage( &msg, NULL, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE ) )
		if( msg.message == WM_KEYDOWN )
			switch( msg.wParam )
			{
			case VK_ESCAPE: 	// break out of the current loop
				fAbort = TRUE;
				while( GetMessage( &msg, NULL, WM_KEYFIRST, WM_KEYLAST )
						&&  msg.message != WM_KEYUP )
					;
				break;

			case VK_PAUSE:		// wait for another keystroke to continue
				ControlSzWrite( ENABLE | SAVE, SAME );
				szWrite( sz_Paused, TRUE );
				while( GetMessage( &msg, NULL, WM_KEYFIRST, WM_KEYLAST )
						&&  msg.message != WM_KEYDOWN )
					;
				fAbort = msg.wParam == VK_ESCAPE;
				if( !fAbort )
					szWrite( sz_Continuing, TRUE );

				while( PeekMessage(
								&msg, NULL, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE ) );
				ControlSzWrite( RESTORE, SAME );
				break;
			}
	return fAbort;
}/* end FAbortCheck */
#endif MACINTOSH


pascal BOOL FChangedInputWindow( UWORD iiwinfo )
{
	SEL			sel;
	IWINFO		*lpiwinfo = &lrgiwinfo[ iiwinfo ];

sel.lresult = 0;	//mj	sel.lresult = SendMessage( hwndTop, EM_GETSEL, 0, 0 );
	if( !lpiwinfo->fChanged && lpiwinfo->sel.lresult == sel.lresult )
		return FALSE;

	lpiwinfo->fChanged    = FALSE;
	lpiwinfo->sel.lresult = sel.lresult;
	return TRUE;
}/* end FChangedInputWindow */


pascal VOID EchoRetcode( LPBYTE lszFunction, RETCODE retcode )
{
	static BYTE CSCONST sz_unknown[]  = "an unknown value";
	static BYTE CSCONST sz_returned[] = " returned ";

	LPBYTE				lsz = LszNameFromFlagArg( &finfo_retcode, retcode );


	if( (SEGMENT)lsz == NULL )
	{
		szWrite( sz_Error, FALSE );
		lsz = sz_unknown;
	}
	szWrite( lszFunction, FALSE );
	szWrite( sz_returned, FALSE );
	szWrite( SzConstDisplay( lsz, retcode ), FALSE );
	szWrite( lsz == sz_unknown ? szExclam : szPeriod, TRUE );
	return;
}


	// WinCls() - clears the displayed screen
	//
pascal VOID WinCls( HWND hwnd )
{
	HDC		hdc;

	hdc = GetDC( hwnd );			// get display context
//mj	SendMessage( hwnd, WM_ERASEBKGND, (WORD)hdc, 0 );	// always erase background
	UpdateWindow( hwnd );			// force painting of window
	ReleaseDC( hwnd, hdc );
}


// LplpafFromHstmt() - Look up the lplpaf for hstmt.
//
pascal LPAF FAR * LplpafFromHstmt(HSTMT hstmt )
{
	LPAF FAR * lplpaf;
	LPAF         lpaf;

	for( lplpaf = &lpafFirst;
		 (SEGMENT)( lpaf = *lplpaf ) != NULL && lpaf->General.hstmt != hstmt;
		 lplpaf = &lpaf->General.lpafNext)
		;

	return lplpaf;
}


// LpafAdd() - If an lpaf entry was found, check the ifn and retcode,
// and if needed, display an error message, or if the SQL function called
// is not still executing, remove any lpaf chain entry found; else if the
// function is still executing:
//   if fAdd, then alloc a new lpaf chain entry and flag that it should
//   be filled in; else just flag that SQL_STILL_EXECUTING was returned.
//
// To test a non-async function, call LpafAdd() with fAdd == FALSE,
// ifn == ifnNotAsync.
//
pascal LPAF LpafAdd(BOOL fAdd, int ifn, RETCODE retcode )
{
	static BYTE CSCONST szNoPendingError[] = "A function called"
		" with the same hstmt as the pending asynchronous execution"
		" of another function did not return SQL_ERROR.";

	static BYTE CSCONST szNonAsyncPending[] = "A function which does not"
		" support asynchronous execution returned SQL_STILL_EXECUTING!";

	static BYTE CSCONST szFormatCompleted[] =
		"The asynchronous operation on hstmt # %u is completed.";

	HANDLE haf;
	LPAF FAR * lplpaf = LplpafFromHstmt(hstmt );
	LPAF         lpaf = *lplpaf;

	if( (SEGMENT)lpaf != NULL )
		{
		if( lpaf->General.ifn == ifn)
			{
			if( retcode != SQL_STILL_EXECUTING )
				{
				wsprintf( OutStr, szFormatCompleted, istmt );
				szWrite( OutStr, TRUE );
				DeleteAf(lplpaf );
				}
			}
		else if( retcode != SQL_ERROR)
			{
			szWrite( sz_Error, FALSE );
			szWrite( szNoPendingError, TRUE );
			}
		return NULL;
		}
	if( retcode != SQL_STILL_EXECUTING )
		return NULL;

	if( ifn == ifnNotAsync)
		{
		szWrite( sz_Error, FALSE );
		szWrite( szNoPendingError, TRUE );
		}
	if( !fAdd)
		return (LPAF)TRUE;

	if( (haf = AllocateMemory(sizeof( AFSTRUCT))) == NULL )
		return NULL;

	lpaf = LockMemory( haf );
	lpaf->General.hstmt    = hstmt;
	lpaf->General.istmt    = istmt;
	lpaf->General.ifn      = ifn;
	lpaf->General.lpafNext = lpafFirst;
	return lpafFirst = lpaf;
}


	// DeleteAf() - delete an lpaf chain entry.
	// Delete any otherwise unused lsz's.

pascal VOID DeleteAf( LPAF *lplpaf )
{
	LPAF	lpaf = *lplpaf;

	if( lpaf == NULL )
		return;

	switch( lpaf->General.ifn )
	{
	case ifnColAttributes:
		ReleaseMemory( lpaf->ColAttributes.lrgbDesc	);
		ReleaseMemory( lpaf->ColAttributes.lpcbDesc	);
		ReleaseMemory( lpaf->ColAttributes.lpfDesc	);
		break;

	case ifnColumnPrivileges:
	case ifnColumns:
		ReleaseISA( lpaf->Columns.lpisaColumnName );
		goto DDA;

	case ifnDescribeCol:
	case ifnDescribeParam:
		ReleaseMemory( lpaf->DescribeCol.lszColName );
		ReleaseMemory( lpaf->DescribeCol.lpcbColName );
		ReleaseMemory( lpaf->DescribeCol.lpfSqlType );
		ReleaseMemory( lpaf->DescribeCol.lpcbColDef );
		ReleaseMemory( lpaf->DescribeCol.lpibScale );
		ReleaseMemory( lpaf->DescribeCol.lpfNullable );
		break;

	case ifnExecDirect:
		ReleaseMemory( lpaf->ExecDirect.hszSqlStr );
		break;

	case ifnExecute:
		break;

	case ifnExtendedFetch:
		ReleaseMemory( lpaf->ExtendedFetch.lpcrow);
		ReleaseMemory( lpaf->ExtendedFetch.lrgfRowStatus );
		break;

	case ifnFetch:
		break;

	case ifnForeignKeys:
		ReleaseISA( lpaf->ForeignKeys.lpisaFkTableQualifier );
		ReleaseISA( lpaf->ForeignKeys.lpisaFkTableOwner );
		ReleaseISA( lpaf->ForeignKeys.lpisaFkTableName );
		goto DDA;

	case ifnGetData:
		ReleaseMemory( lpaf->GetData.hrgbValue );
		ReleaseMemory( lpaf->GetData.lpcbValue );
		break;

	case ifnGetTypeInfo:
		break;

	case ifnMoreResults:
		break;

	case ifnNumResultCols:
		ReleaseMemory( lpaf->NumResultCols.lpccol);
		break;

	case ifnParamData:
		ReleaseMemory( lpaf->ParamData.lplrgbValue );
		break;

	case ifnPrepare:
		ReleaseMemory( lpaf->Prepare.hszSqlStr);
		break;

	case ifnPrimaryKeys:
		goto DDA;

	case ifnProcedureColumns:
		ReleaseISA( lpaf->ProcedureColumns.lpisaColumnName );
		goto DDA;

	case ifnProcedures:
		goto DDA;

	case ifnPutData:
		ReleaseMemory( lpaf->PutData.hrgbValue );
		break;

	case ifnSetPos:
		break;

	case ifnSpecialColumns:
	case ifnStatistics:
		goto DDA;

	case ifnTablePrivileges:
	case ifnTables:
		ReleaseISA( lpaf->Tables.lpisaTableType		);
DDA:
		ReleaseISA( lpaf->DDA.lpisaTableQualifier	);
		ReleaseISA( lpaf->DDA.lpisaTableOwner		);
		ReleaseISA( lpaf->DDA.lpisaTableName		);
		break;

	default:
		VPanic( sdwError = lpaf->General.ifn );
		break;
	}
	*lplpaf=lpaf->General.lpafNext;
	ReleaseMemory( lpaf );
}


pascal int HfileOpenOrCreate( LPSTR lszFile, BOOL fFirst );
pascal BOOL FWriteLogAlert( LPSTR lszAction, LPSTR lszFile );

// FWriteLogAlert() lszAction argument values:

	static BYTE CSCONST sz_create[]   = "open or create";
	static BYTE CSCONST sz_find[]     = "find end of";
	static BYTE CSCONST sz_write[]    = "write to";
	static BYTE CSCONST sz_close[]    = "close";
	static BYTE CSCONST sz_complete[] = "complete writing to";

	static BOOL fWriteScreen = TRUE;
	static BOOL fWriteLog    = FALSE;


pascal VOID ControlSzWrite(int fScreen, int fLog)
{
#define CHANGE (DISABLE | RESTORE | ENABLE)

	static BOOL fWriteScreenSav = TRUE;
	static BOOL fWriteLogSav    = FALSE;

	BOOL fTemp = fWriteScreenSav;

	if( fScreen & SAVE)
		fWriteScreenSav = fWriteScreen;

	switch( fScreen & CHANGE)
		{
	case DISABLE:
		fWriteScreen = FALSE;
		break;

	case RESTORE:
		fWriteScreen = fTemp;
		break;

	case ENABLE:
		fWriteScreen = TRUE;
		break;
		}

	fTemp = fWriteLogSav;

	if( fLog & SAVE)
		fWriteLogSav = fWriteLog;

	switch( fLog & CHANGE)
		{
	case DISABLE:
		fWriteLog = FALSE;
		break;

	case RESTORE:
		fWriteLog = fTemp;
		break;

	case ENABLE:
		fWriteLog = TRUE;
		break;
		}
}


pascal VOID WriteTab(int ctab, int cch )
{
#define cchSpaces (sizeof szSpaces - 1)
#define cchTabs   (sizeof szTabs - 1)

	static BYTE CSCONST szSpaces[] = "        ";
	static BYTE CSCONST szTabs[]   = "\t\t\t\t\t\t\t\t\t\t\t\t";

	UWORD ichSpaces = cchSpaces;

	ctab -= cch >> 3;
	if( cch >= 6 && ctab > 0 )
		{
		cch &= 7;
		if( cch >= 6)
			ctab--;

		else if( cch <= 1)
			cch += 8;

		if( cch >= 6)
			{
			ichSpaces = cchSpaces - ((10 - cch ) << 1);
			if( ctab <= 0 )
				{
				if( cch == 6)
					ctab++;

				ichSpaces = cch == 7 ? 1 : 0;
				}
			}
		}
	if( ichSpaces < cchSpaces )
		szWrite( &szSpaces[ ichSpaces ], FALSE );

	if( ctab > 0 )
	{
		UWORD ichTabs = cchTabs - ctab;

		szWrite( &szTabs[ MIN( ichTabs, cchTabs ) ], FALSE );
	}
	SeparatorWritten();
}

pascal VOID szWrite( LPBYTE lsz, BOOL fEOL )
{
	static BYTE CSCONST szEOL[] = "\n";

	if( fWriteScreen )
	{
		WriteBottomWindow( lsz, FALSE );
		if( fEOL )
			WriteBottomWindow( szEOL, TRUE );
	}
	if( fWriteLog )
	{
		WriteLog( lsz );
		if( fEOL)
			WriteLog( szEOL );
	}
	fSeparatorNeeded = !fEOL;
	return;
}


pascal BOOL FNeedSeparator()
{
	return fSeparatorNeeded;
}


pascal VOID SeparatorWritten()
{
	fSeparatorNeeded = FALSE;
}


pascal VOID TruncateEditContents( HWND /*hwnd*/, UWORD /*cchTruncate*/ )
{
	return;
}


	/**********************************************************************
	FControlWriteLog(env)  - - Enable writing strings to log file
	FControlWriteLog( NULL ) - - Disable writing strings to log file
	WriteLog(String ) - - - - - Write string to log file
	
	On each write, the file is opened, written, and closed.  This is ensure
	that all calls to Log are accurately recorded.  If this is not done,
	then it's possible that some information will remain in system buffers
	if the program (or Windows) crashes before exiting.
	***********************************************************************/

#include <setjmp.h>

static VOID * envWriteLog;


pascal BOOL FControlWriteLog( VOID * env )
{
#ifdef MACINTOSH
	return env ? TRUE : TRUE;
#else
	int hfile;

	envWriteLog = env;
	if( envWriteLog == NULL )
		return FALSE;

	if( (hfile = HfileOpenOrCreate( szLogFileName, TRUE )) == HFILE_ERROR )
		return FALSE;

	while( _lclose( hfile ) == HFILE_ERROR )
		if( FWriteLogAlert( sz_close, szLogFileName ))
			return FALSE;

	return TRUE;
#endif MACINTOSH
}/* end FControlWriteLog */


pascal VOID WriteLog( LPBYTE /*lszText*/ )
{
#ifdef MACINTOSH
	return;
#else
	int		hfile;
	WORD	cchGiven, cchWritten;

	if( envWriteLog == NULL )
		return;

	for( ;; )
	{
		if( (hfile = HfileOpenOrCreate( szLogFileName, FALSE ) )
			== HFILE_ERROR )
			return;

		while( _llseek(hfile, 0, 2) == HFILE_ERROR )
			if( FWriteLogAlert( sz_find, szLogFileName ))
				return;

		cchGiven = (WORD)lstrlen( lszText );
		while( (cchWritten = _lwrite(hfile, lszText, cchGiven)) > cchGiven )
			if( FWriteLogAlert( sz_write, szLogFileName ))
				return;

		while( _lclose( hfile ) == HFILE_ERROR )
			if( FWriteLogAlert( sz_close, szLogFileName ) )
				return;

		if( cchWritten == cchGiven ||
			FWriteLogAlert( sz_complete, szLogFileName ) )
			return;

		ControlSzWrite( ENABLE | SAVE, DISABLE );
		if( !FControlWriteLog( envWriteLog ) )
			break;

		ControlSzWrite( RESTORE, ENABLE );
		lszText += cchWritten;
	}
	ControlSzWrite( RESTORE, SAME );
	return;
#endif MACINTOSH
}


pascal int HfileOpenOrCreate(LPSTR lszFile, BOOL fFirst)
{
	static BYTE CSCONST sz_Creating[]   = "Creating '";
	static BYTE CSCONST sz_Appending[]  = "Appending to existing '";
	static BYTE CSCONST szQuotePeriod[] = "'.";

	BYTE CSCONST * szAction;
	int hfile;

	for( ;;)
		{
		if( (hfile = _lopen( lszFile, OF_READWRITE)) != HFILE_ERROR)
			{
			if( !fFirst)
				return hfile;

			szAction = sz_Appending;
			break;
			}
		if( (hfile = _lcreat( lszFile, 0 )) != HFILE_ERROR)
			{
			szAction = sz_Creating;
			break;
			}
		if( FWriteLogAlert( sz_create, szLogFileName ))
			return hfile;
		}
	ControlSzWrite(ENABLE | SAVE, DISABLE | SAVE);
	szWrite( szAction, FALSE );
	szWrite( lszFile, FALSE );
	szWrite( szQuotePeriod, TRUE );
	ControlSzWrite( RESTORE, RESTORE );
	return hfile;
}


pascal BOOL FWriteLogAlert( LPSTR lszAction, LPSTR lszFile )
{
	static BYTE CSCONST szLogDisabled[] = " - Log output disabled.";
	static BYTE CSCONST szTitle[]       = "File Access Error";
	static BYTE CSCONST szWasClosed[]   =
		"! The file has been successfully closed.";
	static BYTE CSCONST szFormat[]      =
		"Couldn't %s the file '%s'%s Abort the autotest run,"
		" Retry the operation or continue without log (Ignore)?";

	char szMsg[300];

	wsprintf( szMsg, szFormat, lszAction, lszFile,
		LszIfSz( lszAction == (LPSTR)sz_complete, szWasClosed, szExclam));

#ifdef CRAP
	switch( MessageBox( hwndBot, szMsg, szTitle, MB_ICONEXCLAMATION | MB_ABORTRETRYIGNORE))
	{
	case IDABORT:	longjmp( envWriteLog, 1);
	case IDRETRY:	return FALSE;
	}
		// case IDIGNORE:
	ControlSzWrite( ENABLE | SAVE, DISABLE );
	szWrite( szLogDisabled, TRUE );
	ControlSzWrite( RESTORE, SAME );
#endif CRAP
	return TRUE;
}


pascal SWORD CcolNumResultCols()
{
	static BYTE CSCONST sz_apccol[] = "*pccol";

	BOOL fFirst = TRUE;
	BOOL fUntouched;
	BOOL fError;
	RETCODE retcode;
	SWORD ccol = (SWORD)UNTOUCHED;

	for( ;;)
	{
		retcode = SQLNumResultCols( hstmt, &ccol );
		if( fFirst)
			LpafAdd( FALSE, ifnNumResultCols, retcode );

		if( retcode != SQL_STILL_EXECUTING )
			break;

		fFirst = FALSE;
		if( FAbortCheck() )
		{
			EchoRetcode( sz_SQLNumResultCols, retcode );
			retcode = SQLCancel(hstmt );
			EchoRetcode( sz_SQLCancel, retcode );
			return -1;
		}
	}

	fUntouched = ccol == (SWORD)UNTOUCHED;
	fError = retcode != SQL_SUCCESS || fUntouched || ccol < 0;
	if( fError )
		EchoRetcode( sz_SQLNumResultCols, retcode );

	if( retcode >= SQL_SUCCESS )
		DisplayUntouchedError( sz_apccol, fUntouched, FALSE );

	if( !fUntouched)
		{
		static BYTE CSCONST sz_ccol[] = "ccol";

		szWrite( sz_ccol, FALSE );
		szWrite( szEqual, FALSE );
		itoa(ccol, OutStr, 10 );
		szWrite( OutStr, FALSE );
		szWrite( szPeriod, TRUE );
		}
	return fError ? -1 : ccol;
}


pascal VOID DisplayBoundColAffectedMessage(SDWORD crow,
		UWORD icolTouchedMostPrev, UWORD icolTouchedMost)
{
	static BYTE CSCONST szMsgBeginning[] = "Beginning at row 1, ";
	static BYTE CSCONST szFormatAfter[]  = "After row %ld, ";
	static BYTE CSCONST szMsgNo[] = "no bound columns were affected.";
	static BYTE CSCONST szFormatHighest[] =
		"the highest bound column affected was %u.";

	if( icolTouchedMost == icolTouchedMostPrev )
		return;

	if( crow == 0 )
		szWrite( szMsgBeginning, FALSE );
	else
	{
		wsprintf( OutStr, szFormatAfter, crow );
		szWrite( OutStr, FALSE );
	}

	if( icolTouchedMost == 0 )
		szWrite( szMsgNo, TRUE );
	else
		{
		wsprintf( OutStr, szFormatHighest, icolTouchedMost );
		szWrite( OutStr, TRUE );
	}
}


pascal VOID FetchOrGetDataAll( HWND hwnd, BOOL fGetData )
{
	static BYTE CSCONST sz_Fetch_All[]   = "Fetch All";
	static BYTE CSCONST sz_GetData_All[] = "GetData All";
	static BYTE CSCONST sz_canceled[]    = " canceled.";
	static BYTE CSCONST sz_s[]           = "s";
	static BYTE CSCONST sz_apfExists[]   = "*pfExists";
	static BYTE CSCONST sz_apcrow[]      = "*pcrow";

	BOOL fExists = TRUE;
	BOOL fUntouched;
		// Flag that IcolUntouchBoundCols() needs to be called the first time:
	UWORD icolBoundMost = 1;
	RETCODE retcode;
	STMTINFO FAR * lpstmtinfo;

	SetCapture( hwnd );			// Change cursor to hourglass
//mj	hSaveCursor = SetCursor(hHourGlass);

	if( (SEGMENT)(lpstmtinfo = LpstmtinfoFromHstmt(hstmt )) != NULL )
		{
		BOOL fIllegal;
		BOOL fError;

		fExists = (BOOL)UNTOUCHED;
		retcode = SQLGetFunctions(lpstmtinfo->hdbc,
				SQL_API_SQLMORERESULTS, &fExists);
		fUntouched = fExists == (BOOL)UNTOUCHED;
		fIllegal = retcode == SQL_SUCCESS &&
			fExists != FALSE && fExists != TRUE;
		fError = retcode != SQL_SUCCESS || fIllegal;
		if(fError)
			{
			if( retcode != SQL_SUCCESS)
				szWrite( sz_Error, FALSE );

			EchoRetcode( sz_SQLGetFunctions, retcode );
			}
		if( retcode == SQL_SUCCESS)
			DisplayUntouchedError( sz_apfExists, fUntouched, FALSE );

		if( fError)
			{
			if( !fUntouched)
				{
				if( fIllegal)
					szWrite( sz_Error, FALSE );

				szWrite( sz_apfExists,	FALSE );
				szWrite( szEqual,		FALSE );
				itoa( fExists, OutStr, 10 );
				szWrite( OutStr,		FALSE );
				if( fIllegal)
					szWrite( szExclam,	TRUE );
			}
			fExists = TRUE;
		}
	}
	for(;;)
	{
		BOOL	fAbort = FALSE;
		SWORD	ccol = CcolNumResultCols();

		if( ccol < 0 )
			break;

		if( ccol > 0 )
		{
			static BYTE CSCONST		szFormat_rows[] = "%ld row%s fetched.";

			SDWORD	crow = 0;
			UWORD	icolTouchedMostPrev = 0xFFFF;

			if( fGetData && !FTerseDescribeColAll( ccol ) )
				break;

			for(;;)
			{	BOOL fSuccess;

				if( FAbortCheck() )
				{
					szWrite(	LszIfSz( fGetData, sz_GetData_All, sz_Fetch_All ),
								FALSE );
					szWrite( sz_canceled, TRUE );
					fAbort = TRUE;
					break;
				}
				if( icolBoundMost )
					icolBoundMost = IcolUntouchBoundCols( hstmt );

				while( ( retcode = SQLFetch( hstmt )) == SQL_STILL_EXECUTING )
					if( FAbortCheck() )
					{
						fAbort = TRUE;
						break;
					}
				fSuccess = retcode == SQL_SUCCESS ||
					retcode == SQL_SUCCESS_WITH_INFO;

				if( fAbort || !fSuccess && retcode != SQL_NO_DATA_FOUND)
					EchoRetcode( sz_SQLFetch, retcode );

				if( icolBoundMost)
				{	UWORD	icolTouchedMost = IcolBoundColChecks(
												hstmt, NO_ICOL_DISPLAY, retcode );

					DisplayBoundColAffectedMessage( crow,
											icolTouchedMostPrev, icolTouchedMost );

					if( (SDWORD)icolTouchedMost > (SDWORD)ccol )
					{
						static BYTE CSCONST szFormat[] = "The highest"
							" bound icol affected = %u > ccol = %d!";

						szWrite( sz_Error, FALSE );
						wsprintf( OutStr, szFormat, icolTouchedMost, ccol);
						szWrite( OutStr, TRUE );
					}
					icolTouchedMostPrev = icolTouchedMost;
				}
				if( fAbort)
				{
					retcode = SQLCancel(hstmt );
					EchoRetcode( sz_SQLCancel, retcode );
					break;
				}
				if( !fSuccess)
				{
					fAbort = retcode != SQL_NO_DATA_FOUND;
					break;
				}
				crow++;
				if( fGetData && !FGetDataRow(icolBoundMost, ccol))
				{
					fAbort = TRUE;
					break;
				}
			}
			wsprintf( OutStr, szFormat_rows, crow, (LPSTR)&sz_s[crow == 1]);
			szWrite( OutStr, TRUE );
			}
		else
		{
			static BYTE CSCONST szFormat_rows[] = "%ld row%s affected.";
			static BYTE CSCONST sz_not_available[] =
				"The number of rows affected is not available.";

			SDWORD crow = (SDWORD)UNTOUCHED;

			retcode = SQLRowCount( hstmt, &crow );
			fUntouched = crow == (SDWORD)UNTOUCHED;
			if( retcode != SQL_SUCCESS || fUntouched || crow < -1 )
			{
				EchoRetcode( sz_SQLRowCount, retcode );
				fAbort = TRUE;
			}
			if( retcode == SQL_SUCCESS)
				DisplayUntouchedError( sz_apcrow, fUntouched, FALSE );

			if( !fUntouched)
			{
				if( crow == -1)
					szWrite( sz_not_available, TRUE );
				else
				{
					wsprintf( OutStr, szFormat_rows,
						crow, (LPSTR)&sz_s[crow == 1]);
					szWrite( OutStr, TRUE );
				}
			}
		}
		if( fAbort || !fExists)
			break;

		if( icolBoundMost )
			icolBoundMost = IcolUntouchBoundCols( hstmt );

		retcode = SQLMoreResults( hstmt );

		if( icolBoundMost )
			IcolBoundColChecks(hstmt, ALWAYS_UNTOUCHED, 0 );

		if( retcode != SQL_SUCCESS)
		{
			if( retcode != SQL_NO_DATA_FOUND )
				EchoRetcode( sz_SQLMoreResults, retcode );
			break;
		}
	}
	return;
}


pascal BOOL FTerseDescribeColAll( SWORD ccol )
{
	RETCODE		retcode;
	UWORD		icol;
	SWORD		cbBuf;
	BYTE		rgbBuf[ GETDATA_ROW_CBBUF + 1 ];

	for( icol = 1; ; icol++ )
	{
		Untouch( rgbBuf, GETDATA_ROW_CBBUF + 1);
		cbBuf = (SWORD)UNTOUCHED;
		for( ;;)
		{
			retcode = SQLDescribeCol( hstmt, icol, rgbBuf,
					GETDATA_ROW_CBBUF, &cbBuf, NULL, NULL, NULL, NULL );
			if( retcode != SQL_STILL_EXECUTING )
				break;

			if( FAbortCheck() )
			{
				szWrite( szTilde, TRUE );
				EchoRetcode( sz_SQLGetData, retcode );
				retcode = SQLCancel(hstmt );
				EchoRetcode( sz_SQLCancel, retcode );
				return FALSE;
			}
		}
		FTerseDisplayOutputBuf( retcode, FALSE, FALSE,
								rgbBuf, GETDATA_ROW_CBBUF, cbBuf );

		if( icol >= (UWORD)ccol )
			break;

		szWrite( szComma, FALSE );
	}
	szWrite( szPeriod, TRUE );
	return TRUE;
}


pascal BOOL FGetDataRow( UWORD icolBoundMost, SWORD ccol )
{
	RETCODE retcode;
	UWORD icol;
	SDWORD cbBuf;
	BYTE rgbBuf[GETDATA_ROW_CBBUF + 1];

	if( ccol <= 0 )
		return FALSE;

	for( icol = 1; ; icol++ )
	{
		do
		{
			Untouch( rgbBuf, GETDATA_ROW_CBBUF + 1);
			cbBuf = UNTOUCHED;
			for(;;)
			{
				retcode = SQLGetData( hstmt, icol, SQL_CHAR,
										rgbBuf, GETDATA_ROW_CBBUF, &cbBuf );
				if( retcode != SQL_STILL_EXECUTING )
					break;

				if( FAbortCheck() )
				{
					szWrite( szTilde, TRUE );
					EchoRetcode( sz_SQLGetData, retcode );
					retcode = SQLCancel(hstmt );
					EchoRetcode( sz_SQLCancel, retcode );
					return FALSE;
				}
			}
		} while( FTerseDisplayOutputBuf(	retcode,
											TRUE, icol <= icolBoundMost,
											rgbBuf, GETDATA_ROW_CBBUF, cbBuf ) );
		if( icol >= (UWORD)ccol )
			break;

		szWrite( szComma, FALSE );
	}
	szWrite( szPeriod, TRUE );
	return TRUE;
}/* end FGetDataRow */


pascal BOOL FTerseDisplayOutputBuf( RETCODE retcode,
									BOOL fExpectNoData, BOOL fExpectError,
									LPBYTE lrgbBuf, SDWORD cbBufMax, SDWORD cbBuf )
{
	static BYTE CSCONST szSpace[]          = " ";
	static BYTE CSCONST sz_BOUND[]         = "BOUND";
	static BYTE CSCONST sz_ERROR[]         = "ERROR";
	static BYTE CSCONST sz_NO_DATA[]       = "NO DATA";
	static BYTE CSCONST sz_retcode_BAD[]   = "retcode BAD!";
	static BYTE CSCONST sz_rgb_UNTOUCHED[] = "rgb UNTOUCHED";
	static BYTE CSCONST sz_cb_UNTOUCHED[]  = " cb UNTOUCHED!";
	static BYTE CSCONST sz_cb_BAD[]        = " cb BAD!";
	static BYTE CSCONST sz_cb_MISMATCH[]   = " cb MISMATCH!";
	static BYTE CSCONST sz_TRUNC[]         = " TRUNC ";
	static BYTE CSCONST sz_INFO[]          = " INFO";
	static BYTE CSCONST sz_NO_INFO[]       = "NO INFO! ";

	SDWORD cbTouched = CbTouchedRgbCbMax(lrgbBuf, cbBufMax );
	SDWORD cch;
	BOOL fError        = retcode == SQL_ERROR;
	BOOL fNoData       = fExpectNoData && retcode == SQL_NO_DATA_FOUND;
	BOOL fInfo         = retcode == SQL_SUCCESS_WITH_INFO;
	BOOL fSuccess      = retcode == SQL_SUCCESS || fInfo;
	BOOL fCbUntouched  = cbBuf == (SDWORD)UNTOUCHED;
	BOOL fNullData     = cbBuf == SQL_NULL_DATA;
	BOOL fNoTotal      = cbBuf == SQL_NO_TOTAL;
	BOOL fTrunc        = fNoTotal || cbBuf >= cbBufMax;

	if( !fSuccess)
		szWrite(
			LszIfSz(fError,
				LszIfSz(fExpectError, sz_BOUND, sz_ERROR),
				LszIfSz(fNoData, sz_NO_DATA, sz_retcode_BAD)),
			FALSE );

	else if( fNullData)
		szWrite( sz_NULL, FALSE );

	if( cbTouched == 0 )
		{
		if( fSuccess ? !fNullData : !fError && !fNoData)
			{
			if( !fSuccess)
				szWrite( szSpace, FALSE );

			szWrite( sz_rgb_UNTOUCHED, FALSE );
			if( fSuccess)
				szWrite( szExclam, FALSE );
			}
		cch = 0;
		}
	else
		{

		if( !(fNullData && cbTouched == 1 && lrgbBuf[0] == '\0'))
			{
			if( !fSuccess || fNullData)
				szWrite( szSpace, FALSE );

			DisplayString( NULL, lrgbBuf, cbTouched, 0 );
			}
		cch = cbTouched - (lrgbBuf[cbTouched - 1] == '\0');
		}
	if( fSuccess)
		{
		if( fNullData)
			;

		else if( fCbUntouched)
			szWrite( sz_cb_UNTOUCHED, FALSE );

		else if( !fNoTotal && cbBuf < 0 )
			szWrite( sz_cb_BAD, FALSE );

		else
			{
			if( fTrunc)
				cbBuf = cbBufMax - 1;

			if( cbBuf < 0 )
				cbBuf = 0;

			if( cch != cbBuf)
				szWrite( sz_cb_MISMATCH, FALSE );
			}
		if( fTrunc)
			{
			szWrite( sz_TRUNC, FALSE );
			if( !fInfo)
				szWrite( sz_NO_INFO, FALSE );
			}
		else if( fInfo)
			szWrite( sz_INFO, FALSE );
		}
	return fTrunc;
}


pascal UWORD GetString( HWND /*hwnd*/, HANDLE * /*phaszBuf*/ )
{
debugstr( "GetString not implemented yet." );
	return 333;
}


pascal BOOL FCheckInputEdit( HWND, int, LPBYTE, BOOL )
{
	return TRUE;
}


pascal HANDLE AllocateMemory( UDWORD size )
{
	return (HANDLE)NewHandleClear( size );
}


pascal HANDLE ReAllocateMemory( HANDLE hmem, UDWORD Size )
{

	HANDLE		hNewMem;
	UDWORD		bytes_to_move;
	UCHAR		*pMem;
	UCHAR		*pNewMem;


	VAssert( hmem != NULL );

		/* The Macintosh memory reallocator does not preserve the original
		** memory. Therefore, code has been added to do this.
		*/
	hNewMem = NewHandleClear( Size );

	if( hNewMem == NULL )
		return NULL;

	bytes_to_move = GetHandleSize( hmem );

	if( bytes_to_move > Size )
		bytes_to_move = Size;

	pMem	= myLockMemory( hmem    );
	pNewMem	= myLockMemory( hNewMem );

	BlockMove( pMem, pNewMem, bytes_to_move );

	myUnlockMemory( hmem );
	myUnlockMemory( hNewMem );

	DisposHandle( hmem );
	return hNewMem;
}/* end ReAllocateMemory */


pascal HPVOID myLockMemory( HANDLE hmem )
{
	VAssert( hmem != NULL );
	HLock( hmem );
	return (HPVOID)(*hmem);
}/* end myLockMemory */


pascal VOID myUnlockMemory( HANDLE hmem )
{
	VAssert( hmem != NULL );
	HUnlock( hmem );
	return;
}/* end myUnlockMemory */



pascal VOID FreeMemory( HANDLE hmem )
{
	VAssert( hmem != NULL );
	DisposHandle( hmem );
	return;
}


	// CbMemory() - returns the number of bytes allocated
	// from hpb to the end of the (possibly HUGE) allocation.

pascal UDWORD CbMemory( HPVOID )
{
//	return GlobalSize((HANDLE)GlobalHandle( (SEGMENT)hpb) ) - LOWORD( hpb );
debugstr( "CbMemory not implemented yet." );
	return 333;
}

	// ReleaseMemory() - Checks if memory has been allocated,
	// gets the handle, unlocks and then frees it.
	// It also makes sure it isn't called with a near pointer.
	// It returns TRUE if successful.
	//
pascal void ReleaseMemory( HPVOID hpb )
{
	Handle		hmem;	// Macintosh Handle

	if( hpb == NULL )
		return;

	VAssert( hpb != &hmem );
	hmem = RecoverHandle( (Ptr)hpb );
	myUnlockMemory( hmem );
	FreeMemory( hmem );
	return;
}/* end ReleaseMemory */


pascal VOID SetDlgItemLong( HWND hdlg, int idc, long dwValue, BOOL fSigned )
{
	BYTE sz[ 20 ];

	if( fSigned )
		 ltoa(  dwValue, sz, 10 );
	else ultoa( dwValue, sz, 10 );

	SetDlgItemText( hdlg, idc, sz );
}/* end SetDlgItemLong */


pascal VOID SetDlgItemHandleIndex(HWND hdlg, int idc, UWORD ihandle)
{
	BYTE FAR * lsz = sz_NULL;

	if( ihandle > 0 )
	{
		OutStr[0] = '#';
		ltoa(ihandle, &OutStr[1], 10 );
		lsz = OutStr;
	}
	SetDlgItemText( hdlg, idc, lsz );
}


pascal BOOL FGetDlgItemSWord(HWND hdlg, int idc, SWORD FAR * lpsw)
{
	BYTE sz[8];

	return GetDlgItemText( hdlg, idc, sz, sizeof sz) < sizeof sz &&
		FAToSWord( sz, lpsw);
}


pascal BOOL FGetDlgItemUWord(HWND hdlg, int idc, UWORD FAR * lpuw)
{
	BYTE sz[8];

	return GetDlgItemText( hdlg, idc, sz, sizeof sz) < sizeof sz &&
		FAToUWord( sz, lpuw);
}


pascal BOOL FGetDlgItemSDWord(HWND hdlg, int idc, SDWORD FAR * lpsdw)
{
	BYTE sz[16];

	return GetDlgItemText( hdlg, idc, sz, sizeof sz) < sizeof sz &&
		FAToSDWord( sz, lpsdw);
}


pascal BOOL FGetDlgItemUDWord(HWND hdlg, int idc, UDWORD FAR * lpudw)
{
	BYTE sz[16];

	return GetDlgItemText( hdlg, idc, sz, sizeof sz) < sizeof sz &&
		FAToUDWord( sz, lpudw);
}


pascal int WBaseFromPrefix(
	LPBYTE * plsz, UDWORD CSCONST * * ppudwPreMultMost);

pascal BOOL FLszToUdw(LPBYTE lsz, int base,
	UDWORD udwMost, UDWORD udwPreMultMost, UDWORD FAR * lpudw);


pascal BOOL FAToSWord(LPBYTE lsz, SWORD FAR * lpsw)
{
// Allow unsigned hex or octal values to express negative numbers.
// Decimal or any explicitly negative values must not munge the sign.

	BOOL fNeg = FALSE;
	int wBase;
	UDWORD udwMost;
	UDWORD udw;

	if( *lsz == '-')
		{
		fNeg = TRUE;
		lsz++;
		}
	wBase = WBaseFromPrefix( &lsz, NULL );
	udwMost = fNeg ? 0x8000 : (wBase == 10 ? 0x7FFF : 0xFFFF);
	if( !FLszToUdw( lsz, wBase, udwMost, udwMost, &udw))
		return FALSE;

	*lpsw = fNeg ? -(SWORD)udw : (SWORD)udw;
	return TRUE;
}


pascal BOOL FAToUWord(LPBYTE lsz, UWORD FAR * lpuw)
{
	int wBase = WBaseFromPrefix( &lsz, NULL );
	UDWORD udw;

	if( !FLszToUdw( lsz, wBase, 0xFFFF, 0xFFFF, &udw))
		return FALSE;

	*lpuw = (UWORD)udw;
	return TRUE;
}


pascal BOOL FAToSDWord(LPBYTE lsz, SDWORD FAR * lpsdw)
{
// Allow unsigned hex or octal values to express negative numbers.
// Decimal or any explicitly negative values must not munge the sign.

	static UDWORD CSCONST rgudwPreMultMost[] =
		{
		0x7FFFFFFF / 10, 0xFFFFFFFF / 8, 0xFFFFFFFF / 16,
		0x80000000 / 10, 0x80000000 / 8, 0x80000000 / 16
		};

	BOOL fNeg = FALSE;
	int wBase;
	UDWORD CSCONST * pudwPreMultMost = &rgudwPreMultMost[0];
	UDWORD udwMost;

	if( *lsz == '-')
		{
		fNeg = TRUE;
		lsz++;
		}
	wBase = WBaseFromPrefix( &lsz, &pudwPreMultMost);
	if( fNeg)
		{
		pudwPreMultMost += 3;
		udwMost = 0x80000000;
		}
	else
		udwMost = wBase == 10 ? 0x7FFFFFFF : 0xFFFFFFFF;

	if( !FLszToUdw( lsz, wBase,
			udwMost, *pudwPreMultMost, (UDWORD FAR *)lpsdw))
		return FALSE;

	if( fNeg)
		*lpsdw = -*lpsdw;

	return TRUE;
}


pascal BOOL FAToUDWord(LPBYTE lsz, UDWORD FAR * lpudw)
{
	static UDWORD CSCONST rgudwPreMultMost[] =
		{0xFFFFFFFF / 10, 0xFFFFFFFF / 8, 0xFFFFFFFF / 16};

	UDWORD CSCONST * pudwPreMultMost = &rgudwPreMultMost[0];
	int wBase = WBaseFromPrefix( &lsz, &pudwPreMultMost);

	return FLszToUdw( lsz, wBase, 0xFFFFFFFF, *pudwPreMultMost, lpudw);
}


pascal int WBaseFromPrefix(
					LPBYTE * plsz, UDWORD CSCONST * * ppudwPreMultMost )
{
//	Decimal prefix: ""
//	Octal prefixes: "o", "O", "0o", "0O", "0"
//	Hex prefixes:   "x", "X", "0x", "0X"

	int ch = **plsz;	// Look at 1st char

	switch( isupper(ch) ? _tolower(ch) : ch)
		{
	case 'o':
		goto Octal;

	case 'x':
		goto Hex;

	case '0':
		ch = *++*plsz;	// Look at 2nd char
		switch( isupper(ch) ? _tolower(ch) : ch)
			{
		case '\0':
			--*plsz;	// Point to 1st char
			return 10;		// Decimal

		case 'x':
Hex:
			++*plsz;	// Point to 3rd char
			if( ppudwPreMultMost != NULL )
				*ppudwPreMultMost += 2;

			return 16;		// Hex

		case 'o':
Octal:
			++*plsz;	// Point to 3rd char
			break;
			}
		if( ppudwPreMultMost != NULL )
			*ppudwPreMultMost += 1;

		return 8;			// Octal
		}
	return 10;				// Decimal
}


pascal BOOL FLszToUdw(LPBYTE lsz, int wBase,
	UDWORD udwMost, UDWORD udwPreMultMost, UDWORD FAR * lpudw)
{
	int ch = *lsz;
	int wDigit;
	UDWORD udw = 0;

	for( ;;)
		{
		if( isdigit(ch))
			wDigit = ch - '0';

		else if( isxdigit(ch))
			wDigit = ch - (isupper(ch) ? 'A' : 'a') + 10;
		else
			return FALSE;			// Bad digit

		if( wDigit >= wBase || udw > udwMost - wDigit)
			return FALSE;			// Bad digit or overflow

		udw += wDigit;
		if( (ch = *++lsz ) == '\0')
			{
			*lpudw = udw;
			return TRUE;			// Valid number
			}
		if( udw > udwPreMultMost)
			return FALSE;			// Overflow

		udw *= wBase;
		}
}


pascal SDWORD hstrlen( HPBYTE hsz )
{
	HPBYTE	hpchFirst = &hsz[ 0 ];
	HPBYTE	hpchLast = 0;
	HPBYTE	hpchMost  = &hsz[ CbMemory( hsz ) - 1 ];
	BYTE	bLast;

	if( hpchFirst <= hpchMost )
		do
		{
#ifdef WHAT_THE_HELL	// ???
			hpchLast = (SEGMENT)hpchFirst == (SEGMENT)hpchMost ?
					hpchMost :
					((SEGMENT)hpchFirst):>(BYTE BASED( void ) *)0xFFFF;
#endif WHAT_THE_HELL
			bLast = *hpchLast;
			*hpchLast = '\0';
			hpchFirst += lstrlen( hpchFirst );
			*hpchLast = bLast;

		} while( hpchFirst == hpchLast && bLast != '\0' &&
			         (SEGMENT)(++hpchFirst) <= (SEGMENT)hpchMost );

	return hpchFirst - hsz;
}/* end hstrlen */


pascal VOID UpdateDropdownSize( HWND )
{
	return;
}/* end UpdateDropdownSize */


VOID UpdateCurrentHdbc()
{
//	SERVERINFO	*lpserverinfo;

	if(  DBCpop.choice > 1 )
	{
		idbc = DBCpop.choice - 1;					// get the index
#ifdef CRAP
		for( lpserverinfo = &lrgserverinfo[0];
			 lpserverinfo < lpserverinfoMax;	lpserverinfo++ )
		{
			if( lpserverinfo->idbc == idbc )
			{
				hdbc = lpserverinfo->hdbc;
				sprintf( hdbcText, "#%-d", idbc );
				c2pstr( hdbcText );
				return;
			}
		}
#else
		if( lrgserverinfo[ idbc-1 ].idbc == idbc )
		{
			hdbc = lrgserverinfo[ idbc-1 ].hdbc;
			sprintf( hdbcText, "#%-d", idbc );
			c2pstr( hdbcText );
			return;
		}
#endif CRAP
	}
	hdbc = SQL_NULL_HDBC;
	idbc = 0;
	strcpy( hdbcText, "\pNULL");		// ??? fix this
	return;
}/* end UpdateCurrentHdbc */


VOID UpdateCurrentHstmt()
{
	STMTINFO	*lpstmtinfo;

	if( STMTpop.choice > 1 )
	{
		istmt = STMTpop.choice - 1;
		for( lpstmtinfo = &lrgstmtinfo[0];
			 lpstmtinfo < lpstmtinfoMax;	lpstmtinfo++ )
		{
			if( lpstmtinfo->istmt == istmt )
			{
				hstmt = lpstmtinfo->hstmt;
				sprintf( hstmtText, "#%-d", istmt );
				c2pstr( hstmtText );
				return;
			}
		}
	}
	hstmt = SQL_NULL_HSTMT;
	istmt = 0;
	strcpy( hstmtText, "\pNULL" );		// ??? fix this
	return;
}/* end UpdateCurrentHstmt */


VOID UpdateStatementList()
{
	BOOL			fFound = FALSE;
	STMTINFO		*lpstmtinfo;

	extern HWND		hwndHstmtList;


//	SendMessage( hwndHstmtList, CB_RESETCONTENT, 0, 0 );
	for( lpstmtinfo = &lrgstmtinfo[0]; lpstmtinfo < lpstmtinfoMax; lpstmtinfo++ )
	{	if( lpstmtinfo->hstmt )
		{
			if( lpstmtinfo->hdbc == hdbc )
			{
				OutStr[0] = '*';
				SetItemMark( DBCpop.hMenu, DBCpop.choice, '*' );
				fFound = TRUE;
			}
			else
				OutStr[0] = ' ';

			ltoa( lpstmtinfo->istmt, &OutStr[1], 10 );
		//	SendMessage( hwndHstmtList, CB_ADDSTRING, 0, (LONG)OutStr );
		}
	}
	OutStr[0] = (char)(fFound ? ' ' : '*' );
	lstrcpy( &OutStr[1], sz_NULL_STATEMENT );
//	SendMessage( hwndHstmtList, CB_INSERTSTRING,  0, (LONG)OutStr );
//	SendMessage( hwndHstmtList, CB_SELECTSTRING, -1, (LONG)"*" );
	UpdateCurrentHstmt();
	return;
}/* end UpdateStatementList */


pascal SERVERINFO * LpserverinfoFromHdbc( HDBC hdbc )
{
	static BYTE CSCONST szMsg[] = "The current hdbc wasn't found! Operation aborted.";
	SERVERINFO		*lpserverinfo;
	extern HWND		hwndMain;

	if( hdbc == SQL_NULL_HDBC )
		return NULL;

	for( lpserverinfo = &lrgserverinfo[0];
		 lpserverinfo < lpserverinfoMax;	lpserverinfo++ )
		if( lpserverinfo->hdbc == hdbc )
			return lpserverinfo;

	MessageBox( hwndMain, szMsg, sz_Application_error, MB_OK | MB_ICONEXCLAMATION );
	return NULL;
}/* end LpserverinfoFromHdbc */


pascal STMTINFO * LpstmtinfoFromHstmt( HSTMT hstmt )
{
	static BYTE CSCONST	szMsg[] = "The current hstmt wasn't found! Operation aborted.";
	STMTINFO			*lpstmtinfo;
	extern HWND			hwndMain;

	if( hstmt == SQL_NULL_HSTMT )
		return NULL;

	for( lpstmtinfo = &lrgstmtinfo[0];
		 lpstmtinfo < lpstmtinfoMax; lpstmtinfo++ )
		if( lpstmtinfo->hstmt == hstmt )
			return lpstmtinfo;

	MessageBox(hwndMain, szMsg, sz_Application_error,
		MB_OK | MB_ICONEXCLAMATION);
	return NULL;
}/* end LpstmtinfoFromHstmt */


#ifdef MACINTOSH
pascal VOID StackCheck( BOOL )
{
	return;
}
#elif WINDOWS
pascal VOID StackCheck( BOOL fSet )
{
	// Reference byte
#define pbReference ((BYTE *)( &fSet + 1))

	// Lowest used byte (leave room for StackCheck's locals)
#define pbCheckMax (pbReference - 0x20 )

	// Lowest used UDWORD
#define pudwCheckMax ((UDWORD *)((UWORD)pbCheckMax & 0xFFFC))

	// Lowest unused UDWORD
#define pudwCheckMin	((UDWORD *)(((UWORD)&end + sizeof( UDWORD ) - 1) & 0xFFFC))

	static UDWORD CSCONST	udwUntouched = UNTOUCHED;
	static BYTE				*pbReferenceDuringSet;

	extern BYTE		end;	// static symbol at the lowest unused byte

	BYTE		*pbCheck;

	if( !fEnableStackChecking )
		return;


	if( fSet )
		pbReferenceDuringSet = pbReference;

	else if( pbReferenceDuringSet != pbReference )
	{
		static BYTE CSCONST szMsg[] =
					"StackCheck has been called from two different stack depths!";

		MessageBox( hwndMain, szMsg, sz_App_Error, MB_OK | MB_ICONEXCLAMATION );
		return;
	}
	if( fSet )
	{
		register UDWORD			*pudw = pudwCheckMin;
		register BYTE			*pb;
		register BYTE CSCONST	*pbUntouched = (BYTE CSCONST *)&udwUntouched;

		while( pudw < pudwCheckMax )
			*pudw++ = UNTOUCHED;

		pb = (BYTE *)pudw;
		while( pb < pbCheckMax )
			*pb++ = *pbUntouched++;

		return;
	}
	{
	register UDWORD			*pudw = pudwCheckMin;
	register BYTE			*pb;
	register BYTE CSCONST	*pbUntouched = (BYTE CSCONST *)&udwUntouched;

	*pbCheckMax = '\0';
	while( *pudw++ == UNTOUCHED )
		;

	pb = (BYTE*)(pudw - 1);
	while( *pb++ == *pbUntouched++ )
		;

	pbCheck = pb - 1;
	}
	{
	static BYTE CSCONST szRelOp[]    = ">= ";
	static BYTE CSCONST szIncrease[] = "an increase";
	static BYTE CSCONST szDecrease[] = "a decrease";
	static BYTE CSCONST szFormat[]   = "Stack usage has reached:"
		" %s%u bytes, %s of %s%u bytes since last time!";

	static UWORD cbUsedLastDisplayed = 0;

	UWORD			cbUsed = (UWORD)(pbReference - pbCheck);
	UWORD			dcb = cbUsed - cbUsedLastDisplayed;
	BYTE CSCONST	*szOp = &szRelOp[ sizeof szRelOp - 1 ];
	BYTE CSCONST	*szDirect = szIncrease;

	if( pbCheck == (BYTE*)pudwCheckMin )
		szOp = szRelOp;

	else if( cbUsed < cbIsAtLeast )
		return;

	else
	{
		if( cbUsed < cbUsedLastDisplayed )
		{
			dcb = cbUsedLastDisplayed - cbUsed;
			goto DecreaseTest;
		}
		if( dcb < dcbIncreasesBy )
		{
			if( dcb != 0 )
				return;
DecreaseTest:
			if( dcb < dcbDecreasesBy)
				return;

			szDirect = szDecrease;
		}
	}
	wsprintf( OutStr, szFormat, (LPBYTE)szOp, cbUsed,
				(LPBYTE)szDirect, (LPBYTE)szOp, dcb );
	szWrite( OutStr, TRUE );
	cbUsedLastDisplayed = cbUsed;
	}

#undef pbReference
#undef pbCheckMax
#undef pudwCheckMax
#undef pudwCheckMin
}/* end StackCheck */
#endif MACINTOSH elif WINDOWS


pascal VOID ShowLightning( HWND hwnd )
{
	HDC		hdc;
	HDC		hdcSrc;

	hdc		= GetDC( hwnd );
	hdcSrc	= CreateCompatibleDC( hdc );
//mj	SelectObject( hdcSrc, hAsyncBMP );
	BitBlt( hdc, 0, 0, 25, 25, hdcSrc, 0, 0, SRCCOPY );
	ReleaseDC( hwnd, hdc );
	DeleteDC( hdcSrc );
	return;
}/* end ShowLightning */


pascal VOID HideLightning( HWND hwnd )
{
	RECT	rc;

	rc.top		= 0;
	rc.left		= 0;
	rc.bottom	= 25;
	rc.right	= 25;
	InvalidateRect( hwnd, &rc, TRUE );
	return;
}/* end HideLightning */


	/****************************************************************************
	
		FUNCTION: OutOfMemory()
	
		PURPOSE:  Displays warning message
	
	****************************************************************************/
pascal VOID OutOfMemory()
{
	static BYTE CSCONST szMsg[] = "Out of Memory";

	MessageBox( GetFocus(), szMsg, NULL, MB_ICONHAND | MB_SYSTEMMODAL );
	return;
}/* end OutOfMemory */


	/****************************************************************************
	 *
	 *  FUNCTION   : SetWrap()
	 *
	 *  PURPOSE    : Changes the word wrapping in an edit control. Since this
	 *		 cannot be done by direct means, the function creates a new
	 *		 edit control, moves data from the old control to the new
	 *		 control and destroys the original control. Note that the
	 *		 function assumes that the child being modified is currently
	 *		 active.
	 *
	 ****************************************************************************/
pascal VOID SetWrap( HWND /*hwnd*/, BOOL /*fWrap*/ )
{
	return;
}/* end SetWrap */
