
	/*
	  PROGRAM: l1qtest.c
			 (C) 1990-1992 Microsoft Corporation.  All right reserved.
	
	  PURPOSE: This is a Quick Test of the general functionality of an ODBC driver
			 The level 1 extensions are tested.
	  FUNCTIONS:
			 EXTQuickTestL1() - performs the quick test
	*/

#include "gator.h"
//#undef QUIET
#include "tests.h"
#include <stdio.h>
#include "autotest.h"

#ifdef MACINTOSH
#include <dos.h>			// mj
	
#define sz_empty	""		// mj
#define sz_NULL		""		// mj
#define sz_Expected	""		// mj
#define sz_Returned	""		// mj
	LPBYTE	ltoa( long, LPBYTE, int );
	LPBYTE	itoa( int,  LPBYTE, int );
#endif MACINTOSH

#define MAXSTR 30

	/* large buffers allocated as a global used for queries and other
	returned information */
typedef struct	tagQtStruct {
	char	buf[ szSIZE ];
	char	sz[ szSIZE ];
	char	szParam[ 30 ];
	char	szDataItem[ 45 ];
	char	szTableName[ TABLENAMESIZE ];
	char	szColNames[ 200 ];
	char    szColName[ MAXSTR ];
	char	szValues[ 500 ];
}	QTSTRUCT;

	/* the storage used for data retreived using bind/fetch sequence */
typedef struct tagFetchedData {
	struct BindUCHAR   rgUChar[ 20 ];
	struct BindSDWORD  rgSDWord[ 5 ];
	struct BindSWORD   rgSWord[ 5 ];
	struct BindSDOUBLE rgSDouble[ 9 ];
	struct BindSFLOAT  rgSFloat[ 5 ];
}  FDSTRUCT;

pascal void ExtQuickTestL1( struct ServerStruct *pTestSource )
{
	RETCODE         returncode;
	SWORD           wType;
	SWORD           wNull;
	SWORD           cb;
	SWORD           wNum;
//mj	UWORD           w;
	SWORD           w;
	UWORD           i;
	UWORD           ind;
	UWORD           cTypes;
	SDWORD          isp;
	SDWORD          sdw;
	UDWORD          dwLen;
	UDWORD          udw;
	double          dNum;
	BOOL            fFoundTable = 0;
	HENV            henv0 = NULL;
	HDBC            hdbc0 = NULL;
	HDBC            hdbc1 = NULL;
	HSTMT           hstmt0 = NULL;
	HSTMT           hstmt1 = NULL;
	HSTMT           usehstmt = NULL;
	PTR             ptr;
	char FAR       *pch;
	char FAR       *rgStates[] = {NULL, NULL, NULL};
	PSTRUCT FAR    *lpParams;
	int             cTableName;
	int             fColNames;
	int             fConnectOption;
	int             iuchar;
	int             isdword;
	int             isword;
	int             isdouble;
	int             isfloat;
	int             cErrorCount = 0;
	int             cStateErrorCount = 0;
	int             cFieldName = 0;
	int             fIndex = 0;
	char	        szNum[ 30 ];	/* used with atoi and itoa conversions	*/
	QTSTRUCT FAR *lpqt		= LockMemory( AllocateMemory( sizeof( QTSTRUCT ) ) );
	FDSTRUCT FAR *lpfd		= LockMemory( AllocateMemory( sizeof( FDSTRUCT ) ) );
	TYPENUM FAR * rgTypes	= LockMemory( AllocateMemory( sizeof( TYPENUM )
																	* MAX_TYPES) );
	COLDATA FAR * rgCols = LockMemory( AllocateMemory( sizeof( COLDATA )
																* MAX_TYPES ) );

	rgStates[0] = sz_00000;
	rgStates[1] = sz_01000;		/* Driver not capable */
		
	lpParams = (PSTRUCT FAR *)LockMemory( AllocateMemory( sizeof( PSTRUCT )));

	szWrite("Testing core functions", TRUE );
	CoreQuickTest( pTestSource );
	if( pTestSource->cErrors == -1 )	/* if Core quick test did not complete,
											do not continue with this one	*/
		goto ErrorRet;

	cErrorCount			= pTestSource->cErrors;
	cStateErrorCount	= pTestSource->cStateErrors;
	
	szWrite( "Finished testing core functions", TRUE );
		
		/* Level 1 Extensions */

	if( *pTestSource->szValidServer0 ) { /* only want to SQLAllocEnv if
											Allocated handle not passed in */
		FUNCCALL( sz_SQLAllocEnv );
		returncode = SQLAllocEnv( &henv0 );
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}

		FUNCCALL( sz_SQLAllocConnect );
		returncode = SQLAllocConnect( henv0, &hdbc0 );
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}
		FUNCCALL( sz_SQLAllocConnect);
		returncode = SQLAllocConnect(henv0, &hdbc1);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}

		/* create a DriverConnect string */
		wsprintf(lpqt->sz, "dsn=%s;uid=%s;pwd=%s;",
			(char FAR *)pTestSource->szValidServer0,
			(char FAR *)pTestSource->szValidLogin0,
			(char FAR *)pTestSource->szValidPassword0);

		szWrite("szConnStrIn:  ", FALSE );
		szWrite(lpqt->sz, TRUE );

		FUNCCALL( sz_SQLDriverConnect);
		returncode = SQLDriverConnect(hdbc0, pTestSource->hwnd, lpqt->sz,
			SQL_NTS, lpqt->buf, szSIZE, NULL, SQL_DRIVER_COMPLETE);

		/* since SQL_DRIVER_COMPLETE is used, it is possible that the
			driver will bring up a dialog asking for more information. */

		if( returncode != SQL_SUCCESS && returncode !=
			SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}
		cStateErrorCount += frgStateCheck(henv0, hdbc0, hstmt0, rgStates);

		FUNCCALL( sz_SQLDriverConnect);
		returncode = SQLDriverConnect(hdbc1, pTestSource->hwnd, lpqt->sz,
			SQL_NTS, lpqt->buf, szSIZE, NULL, SQL_DRIVER_COMPLETE);
		if( returncode != SQL_SUCCESS && returncode !=
			SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}
		cStateErrorCount += frgStateCheck(henv0, hdbc0, hstmt0, rgStates);

	}
	else {
		henv0 = pTestSource->henv;
		hdbc0 = pTestSource->hdbc;
	}

	FUNCCALL( sz_SQLGetInfo);
	returncode = SQLGetInfo(hdbc0, SQL_ODBC_API_CONFORMANCE, &fIndex,
		sizeof (fIndex), &cb);
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode);
		DisplayRetcode( FALSE, SQL_SUCCESS);
		cErrorCount++;
	}

	if( fIndex != 0 && fIndex != 1 && fIndex != 2 ) {
			/* one of these two values should always be returned */
		szWrite( szIndent, FALSE );
		szWrite( "GetInfo Failed (SQL_ODBC_API_CONFORMANCE):  ", FALSE );
		szWrite( lpqt->sz, TRUE );
		cErrorCount++;
	}

	/* Set some options then get them and make sure they come back the
		same. */
	
	FUNCCALL( sz_SQLSetConnectOption );
    returncode = SQLSetConnectOption(hdbc0, SQL_ACCESS_MODE,
		SQL_MODE_READ_WRITE );
	if( returncode == SQL_SUCCESS)
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	else /* if this didn't work, the reason should be because this was
			not a valid option to use with this driver */
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_S1C00);

	FUNCCALL( sz_SQLSetConnectOption );
    returncode = SQLSetConnectOption( hdbc0, SQL_AUTOCOMMIT, 1 );
	if( returncode == SQL_SUCCESS)
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
	else /* if this didn't work, the reason should be because this was
			not a valid option to use with this driver */
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_S1C00 );

	FUNCCALL( sz_SQLSetConnectOption);
    returncode = SQLSetConnectOption(hdbc0, SQL_LOGIN_TIMEOUT, 15);
	if( returncode == SQL_SUCCESS) {
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );
		fConnectOption = TRUE;
	}
	else { /* if this didn't work, the reason should be because this was
			not a valid option to use with this driver */
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_S1C00 );
		fConnectOption = FALSE;
	}

	FUNCCALL( sz_SQLGetConnectOption );
	returncode = SQLGetConnectOption( hdbc0, SQL_LOGIN_TIMEOUT, &sdw);
	if( returncode == SQL_SUCCESS) {
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if(sdw != 15 && fConnectOption) {
			szWrite("GetConnectOption failed", TRUE );
			cErrorCount++;
		}
	}
	else /* if this didn't work, the reason should be because this was
			not a valid option to use with this driver */
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_S1C00);

	FUNCCALL( sz_SQLAllocStmt);
	returncode = SQLAllocStmt(hdbc0, &hstmt0);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		goto ErrorRet;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	if(*pTestSource->szValidServer0) { /* only want to SQLAllocEnv if
											Allocated handle not passed in */
		FUNCCALL( sz_SQLAllocStmt);
		returncode = SQLAllocStmt(hdbc1, &hstmt1);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}
		cStateErrorCount += fStateCheck( henv0, hdbc1, hstmt1, sz_00000 );
	}

	FUNCCALL( sz_SQLGetStmtOption );
	returncode = SQLGetStmtOption(hstmt0, SQL_MAX_LENGTH, &wNum );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode);
		DisplayRetcode( FALSE, SQL_SUCCESS);
		szWrite( szIndent, FALSE );
		szWrite( "Failed using SQL_MAX_LENGTH", TRUE );
		cErrorCount++;
	}
	if( returncode == SQL_SUCCESS )
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );
	else /* if this didn't work, it should be because this was
			not a valid option to use with this driver */
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_S1C00 );

	FUNCCALL( sz_SQLSetStmtOption );
	returncode = SQLSetStmtOption(hstmt0, SQL_MAX_LENGTH, 300 );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		szWrite(szIndent, FALSE );
		szWrite("Failed using SQL_MAX_LENGTH", TRUE );
		cErrorCount++;
	}

	if( returncode == SQL_SUCCESS)
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	else /* if this didn't work, it should be because this was
			not a valid option to use with this driver */
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_S1C00);

	FUNCCALL( sz_SQLGetStmtOption);
	returncode = SQLGetStmtOption(hstmt0, SQL_MAX_LENGTH, &dwLen);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		szWrite(szIndent, FALSE );
		szWrite("Failed using SQL_MAX_LENGTH", TRUE );
		cErrorCount++;
	}

	if( returncode == SQL_SUCCESS)
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	else /* if this didn't work, it should be because this was
			not a valid option to use with this driver */
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_S1C00);

	if(dwLen != 300 && returncode == SQL_SUCCESS) { /* if it did work,
								it better have returned the correct value */
		szWrite(szIndent, FALSE );
		szWrite("Failed Get/Set StmtOption", TRUE );
		cErrorCount++;
	}

/* Get the type information to use in a create statement for a table */

	FUNCCALL( sz_SQLGetInfo);
	returncode = SQLGetInfo(hdbc0, SQL_MAX_COLUMN_NAME_LEN, &cFieldName,
		sizeof cFieldName, NULL);

	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cFieldName = 10;
	}
	if(cFieldName > MAX_TYPE)
		cFieldName = MAX_TYPE;
		
	FUNCCALL( sz_SQLGetTypeInfo);
	returncode = SQLGetTypeInfo(hstmt0, SQL_ALL_TYPES);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		goto ErrorRet;
	}
		
	INITLOOP;
	for(i = 0; i < MAX_TYPES; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS)
			break;

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		*rgTypes[i].szType = *rgTypes[i].szLength = *rgTypes[i].szPrefix
			= *rgTypes[i].szSuffix = *rgTypes[i].szParams = '\0';

		rgTypes[i].iFieldNum = i;
		 
		/* get type name */
		FUNCCALL( sz_SQLGetData );
		returncode = SQLGetData(hstmt0, 1, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szType, MAX_TYPE, &sdw );

		if( returncode != SQL_SUCCESS)
			goto ErrorRet;

		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		wsprintf( rgTypes[i].szFieldName,"c%02u%s", i,
			(char FAR *)rgTypes[i].szType );

			/* null terminate at max length */
		rgTypes[i].szFieldName[ cFieldName ] = '\0';

			/* change spaces in type name to '_' */
//mj	while( (pch = _fstrchr( rgTypes[i].szFieldName, ' ' ) ) != NULL )
//mj		*pch = '_';
		
			/* get the SQLType */
		FUNCCALL( sz_SQLGetData );
        returncode = SQLGetData(hstmt0, 2, SQL_C_DEFAULT,
			(SWORD FAR *)&rgTypes[i].wSQLType, 100,
			&sdw);

		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

			/* Max length */
		FUNCCALL( sz_SQLGetData );
		returncode = SQLGetData( hstmt0, 3, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szLength, MAX_LENGTH_BUF, &sdw );

		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

			/* literal prefix and suffix are not actually used, with 
				using Prepare / Execute with params, they would be 
				necessary for a straight insert w/o params */

			/* literal prefix */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData(hstmt0, 4, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szPrefix, MAX_PREFIX, &sdw);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS)
			goto ErrorRet;

		/* literal suffix */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData(hstmt0, 5, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szSuffix, MAX_SUFFIX, &sdw);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS)
			goto ErrorRet;

		/* create params */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData(hstmt0, 6, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szParams, MAX_PARAMS, &sdw);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS)
			goto ErrorRet;

		/* nullable */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData(hstmt0, 7, SQL_C_CHAR, (char FAR *)szNum, 100,
			&sdw);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS)
			goto ErrorRet;
		rgTypes[i].nullable = atoi(szNum);

		/* searchable */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData(hstmt0, 9, SQL_C_CHAR, (char FAR *)szNum, 100,
			&sdw);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS)
			goto ErrorRet;
		rgTypes[i].searchable = atoi(szNum);

/* If a line below is compiled in, that type will not be used in creating
 the table.  (useful for when one specific type is causing problems in
 a particular test, this way the test can be run for all other types)
*/
			if(!lstrcmp(rgTypes[i].szType, "LONG RAW"))
				i--;
/*
			if(!lstrcmpi(rgTypes[i].szType, "bit"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "varbinary"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "money"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "smallint"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "text"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "smallmoney"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "int"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "datetime"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "smalldatetime"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "varchar"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "bit"))
				i--;
			if(lstrcmp(rgTypes[i].szType, "binary"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "char"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "float"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "real"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "tinyint"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "image"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "sysname"))
				i--;
			if(!lstrcmp(rgTypes[i].szType, "timestamp"))
				i--;
*/
	}

	/* if the type name contains spaces, replace them with _ because
		most servers don't allow spaces in field names */

	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	if(SQL_NO_DATA_FOUND != returncode) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	FUNCCALL( sz_SQLGetInfo);
	returncode = SQLGetInfo(hdbc0, SQL_MAX_TABLE_NAME_LEN, &cTableName,
		sizeof (int), NULL);
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	
	if(cTableName > TABLENAMESIZE)
		cTableName = TABLENAMESIZE;
	wsprintf(lpqt->szTableName, "q%ld", hstmt0);
	lpqt->szTableName[cTableName] = '\0';

	/* build create statement */
	/* the column names will be ctypname (where typename is the
		type name returned by the source in SQLGetTypeInfo) */

	*lpqt->sz = '\0';
	cTypes = i;
	lstrcpy(lpqt->sz, "Create Table ");
	lstrcat(lpqt->sz, lpqt->szTableName);
	lstrcat(lpqt->sz, " (");
	for(i = 0; i < cTypes; i++) {
		*lpqt->szParam = '\0';

		/* if SQLGetTypeInfo returned params they need to be used in
			the create statement */

		if(_fstrstr(rgTypes[i].szParams, "max length") ||
			_fstrstr(rgTypes[i].szParams, "Max Length") ||
			_fstrstr(rgTypes[i].szParams, "MAX LENGTH") ||
			_fstrstr(rgTypes[i].szParams, "width") ||
			_fstrstr(rgTypes[i].szParams, "Width") ||
			_fstrstr(rgTypes[i].szParams, "WIDTH"))
			wsprintf(lpqt->szParam, "(%s)", (char FAR *)rgTypes[i].szLength);
		if(_fstrstr(rgTypes[i].szParams, "precision, scale") ||
			_fstrstr(rgTypes[i].szParams, "Precision, Scale") ||
			_fstrstr(rgTypes[i].szParams, "PRECISION, SCALE"))
			wsprintf(lpqt->szParam, "(%d, %d)", 5, 5);
		wsprintf(&lpqt->sz[lstrlen(lpqt->sz)], " %s %s%s, ",
			(char FAR *)rgTypes[i].szFieldName,
			(char FAR *)rgTypes[i].szType, lpqt->szParam);
	}

	/* remove the last comma and space */

	lpqt->sz[lstrlen(lpqt->sz) - 2] = '\0';
	lstrcat(lpqt->sz, ")");
	szWrite(lpqt->sz, TRUE );

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		goto ErrorRet;
	}

	/* now that table was created, call SQLColumns.  Both as
		a test and to get necessary information to insert data */

	FUNCCALL( sz_SQLColumns);
	returncode = SQLColumns(hstmt0, NULL, SQL_NTS, NULL, SQL_NTS,
		lpqt->szTableName, SQL_NTS, NULL, SQL_NTS);

	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}

	INITLOOP;
	
	for(i = 0; i < MAX_TYPES; i++) {
		ITERATION(i);

		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS)
			break;

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		/* precision */
		strcpy(szNum, "0");

		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData(hstmt0, 7, SQL_C_CHAR, szNum, 100,
			&sdw)) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		rgCols[i].precision = atol(szNum);

		strcpy(szNum, "0");

		/* length */
		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData(hstmt0, 8, SQL_C_CHAR, szNum, 100,
			&sdw)) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		if(rgCols[i].precision == 0)
			rgCols[i].precision = atoi(szNum);
		else
			rgCols[i].length = atoi(szNum);

		strcpy(szNum, "0");

		/* numeric scale */
		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData(hstmt0, 9, SQL_C_CHAR, szNum, 100,
			&sdw)) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		rgCols[i].scale = atoi(szNum);

		strcpy(szNum, "0");

		/* nullable */
		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData(hstmt0, 11, SQL_C_CHAR, szNum, 100,
			&sdw)) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		rgCols[i].nullable = atoi(szNum);
	}

	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
 	if(SQL_NO_DATA_FOUND != returncode) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NO_DATA_FOUND);
		goto ErrorRet;
	}
	
	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	/* look for columns the test should not try to update by running
		a select * query and calling ColAttributes on the result fields */

	wsprintf(lpqt->sz, "select * from %s where 1 = 2", lpqt->szTableName);

	szWrite(lpqt->sz, TRUE );

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	INITLOOP;
	
	for(i = 1; i <= cTypes; i++) {
		ITERATION(i);

		FUNCCALL( sz_SQLColAttributes);
		returncode = SQLColAttributes(hstmt0, i, SQL_COLUMN_UPDATABLE, NULL,
			0, NULL, &sdw);

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
			
		rgCols[i - 1].fAutoUpdate = sdw == SQL_ATTR_READONLY;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	/* put together the insert statement, and set the parameters */
	/* parameters are only set the first time through, after which
	the contents of the pointers is changed */

	*lpqt->szColNames = NULL;
	*lpqt->szValues = NULL;
	fColNames = TRUE;

	INITLOOP;
	for(i = 0; i < cTypes; i++) {
		ITERATION(i);
		for(ind = 0, w = 0; ind < cTypes; ind++) {
			pch = MakeData(i, &rgTypes[ind], &rgCols[ind],
				lpqt->szDataItem);

			if(!pch) /* current type is REAL_ONLY) */
				continue;
			
			w++;
			/* for every nullable type, that field will be set to
			null when the row number corresponds with the field
			number */

			if(*pch) {
				lstrcpy(&lpParams->pbs[w][0], pch);
				lpParams->cbValue[w] = SQL_NTS;
			}
			else {
				lpParams->pbs[w][0] = '\0';
				lpParams->cbValue[w] = SQL_NULL_DATA;
			}
			if(fColNames) { /* the first time throught, the insert
				statement itself is created */
				lstrcat(lpqt->szColNames, rgTypes[ind].szFieldName);
				lstrcat(lpqt->szColNames, ", ");
				lstrcat(lpqt->szValues, " ?, ");

				/* and parameters are set */
				/* set all the parameters using pointers to buffers in
				the param struct. */

				FUNCCALL( sz_SQLSetParam);
				returncode = SQLSetParam(hstmt0, w,
					SQL_C_CHAR, rgTypes[ind].wSQLType,
					rgCols[ind].precision, rgCols[ind].scale,
					&lpParams->pbs[w][0],
					&lpParams->cbValue[w]);
				if( returncode != SQL_SUCCESS) {
					DisplayRetcode(TRUE, returncode);
					DisplayRetcode(FALSE, SQL_SUCCESS);
					goto ErrorRet;
				}
				cStateErrorCount += fStateCheck(henv0, hdbc0,
					hstmt0, sz_00000);
			}
		}

		if(fColNames) { /* the first time through, the insert
			statement needs to be SQLPrepare'd */
			/* remove the ", " at the end of the string */
			lpqt->szColNames[lstrlen(lpqt->szColNames) - 2] = NULL;
			lpqt->szValues[lstrlen(lpqt->szValues) - 2] = NULL;

			wsprintf(lpqt->sz, "insert into %s (%s) VALUES(%s)",
				lpqt->szTableName, lpqt->szColNames, lpqt->szValues);
			szWrite(lpqt->sz, TRUE );

			FUNCCALL( sz_SQLPrepare);
			returncode = SQLPrepare(hstmt0, lpqt->sz, SQL_NTS);
			cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0,
				sz_00000);
			if( returncode != SQL_SUCCESS) {
				DisplayRetcode(TRUE, returncode);
				DisplayRetcode(FALSE, SQL_SUCCESS);
				goto ErrorRet;
			}

			fColNames = FALSE; /* no more first times through */

		}

		FUNCCALL( sz_SQLExecute); /* this inserts a row */
		returncode = SQLExecute(hstmt0);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0,
			sz_00000);

		FUNCCALL( sz_SQLRowCount);
		returncode = SQLRowCount(hstmt0, &sdw);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0,
			sz_00000);
		if(sdw != 1) { /* the execute inserted 1 row */
			szWrite(szIndent, FALSE );
			szWrite("Failed: RowCount", TRUE );
			cErrorCount++;
		}


		/* FreeStmt SQL_CLOSE */
		FUNCCALL( sz_SQLFreeStmt);
		returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	}

	/* for the last row use SQLParamData and SQLPutData */

	sdw = SQL_DATA_AT_EXEC;

	for(ind = 0, w = 0; ind < cTypes; ind++) { /* set all the params */
		if(rgCols[ind].fAutoUpdate)
			continue;
		
		w++;
		FUNCCALL( sz_SQLSetParam);
		returncode = SQLSetParam(hstmt0, w,
			SQL_C_CHAR, rgTypes[ind].wSQLType,
			rgCols[ind].precision, rgCols[ind].scale,
			(char FAR *)(UDWORD)ind, &sdw);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0,
			sz_00000);
	}

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_NEED_DATA) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NEED_DATA);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	
	INITLOOP;
	for(ind = 0; ind <= cTypes; ind++) {
		ITERATION(ind);
		FUNCCALL( sz_SQLParamData);
		returncode = SQLParamData(hstmt0, &ptr);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0,
			sz_00000);
		if( returncode != SQL_NEED_DATA)
			break;
	
		pch = MakeData(cTypes, &rgTypes[(unsigned long)ptr],
			&rgCols[(unsigned long)ptr], lpqt->szDataItem);

		FUNCCALL( sz_SQLPutData);
		returncode = SQLPutData(hstmt0, pch, SQL_NTS);
	
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0,
			sz_00000);
	}
	
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	
	FUNCCALL( sz_SQLRowCount);
	returncode = SQLRowCount(hstmt0, &sdw);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	if(sdw != 1) {
		szWrite(szIndent, FALSE );
		szWrite("Failed: RowCount", TRUE );
		cErrorCount++;
	}
	
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	
	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_RESET_PARAMS);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	/* now that the table is created and has data, make sure it
		all comes back correctly */
	
	wsprintf(lpqt->sz, "select * from %s", lpqt->szTableName);

	szWrite(lpqt->sz, TRUE );

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	lpqt->sz[0] = '\0';

	/* the cursor name should be created by the driver since one was
		not specified */

	FUNCCALL( sz_SQLGetCursorName);
	returncode = SQLGetCursorName(hstmt0, lpqt->sz, szSIZE, &cb);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	if(cb > szSIZE) {
		szWrite(szIndent, FALSE );
		szWrite("cb too large", TRUE );
		cb = szSIZE;
		cErrorCount++;
	}

	if(!*lpqt->sz) { /* don't check the name itself, just make sure that 
		something was returned */
		szWrite(szIndent, FALSE );
		szWrite("Failed to return cursor name", TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLNumResultCols);
	returncode = SQLNumResultCols(hstmt0, &wNum);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}

	if(wNum != (SWORD)cTypes) {
		szWrite(szIndent, FALSE );
		szWrite("Columns returned != columns created", TRUE );
		cErrorCount++;
	}

	INITLOOP;
	for(i = 1; i <= cTypes; i++) {
		dwLen = 0;
		wNum = 0;
		wNull = 0;

		ITERATION(i);
		
			/* information returned by SQLDescribeCol should match info
				used to create table */

		lstrcpy( lpqt->sz, (char FAR *)rgTypes[i-1].szFieldName );
		szWrite( lpqt->sz, TRUE );

		FUNCCALL( sz_SQLDescribeCol );
		returncode = SQLDescribeCol( hstmt0, i, lpqt->buf,
			szSIZE, &w, &wType, &dwLen, &wNum, &wNull );
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}

			/* verify that column name returned is column name created */

		if( 0 != lstrcmpi( lpqt->sz, lpqt->buf ) ) {
			szWrite( szIndent, FALSE );
			szWrite( "Failed: incorrect Column Name", TRUE );
			cErrorCount++;
		}

		if(w != (UWORD)lstrlen(lpqt->buf)) {
			szWrite(szIndent, FALSE );
			szWrite("Failed: outlength != string length", TRUE );
			cErrorCount++;
		}


		if(wType != rgTypes[i-1].wSQLType) {
			szWrite( szIndent, TRUE );
			szWrite( "Failed: Incorrect Type Number", TRUE );
			cErrorCount++;
		}

		if( dwLen != rgCols[ i-1 ].precision ) {
			szWrite( szIndent, FALSE );
			szWrite( "Failed: Incorrect precision", TRUE );
			szWrite( sz_Expected, FALSE );
			szWrite( ltoa( rgCols[i-1].precision, szNum, 10 ), TRUE );
			szWrite( sz_Returned, FALSE );
			szWrite( ltoa( dwLen, szNum, 10 ), TRUE );
			cErrorCount++;
		}
		if( wNum != rgCols[i - 1].scale ) {
			cErrorCount++;
			szWrite( szIndent, FALSE );
			szWrite( "Failed: Incorrect scale", TRUE );
		}

		if( wNull != rgCols[i - 1].nullable && wNull != SQL_NULLABLE_UNKNOWN ) {
			szWrite( szIndent, FALSE );
			szWrite( "Failed: Incorrect nullable", TRUE );
			szWrite( "SQLColumns ", FALSE );
			szWrite( itoa(rgCols[i - 1].nullable, szNum, 10), TRUE );
			szWrite( "SQLGetTypeInfo ", FALSE );
			szWrite( itoa(rgTypes[i - 1].nullable, szNum, 10), TRUE );
			szWrite( sz_Returned, FALSE );
			szWrite( itoa(wNull, szNum, 10), TRUE );
			cErrorCount++;
		}
	}

/* bind all fields to a variable of the correct type for retreiving data */

	iuchar = isdword = isword = isdouble = isfloat = 0;
	for(i = 1; i <= cTypes; i++) {
		switch(rgTypes[i - 1].wSQLType) {
			case SQL_INTEGER:
				FUNCCALL( sz_SQLBindCol);
                returncode = SQLBindCol(hstmt0, i, SQL_C_DEFAULT,
					&lpfd->rgSDWord[isdword].data, sizeof(long int),
					&lpfd->rgSDWord[isdword].outlen);
				isdword++;
				break;
			case SQL_SMALLINT:
				FUNCCALL( sz_SQLBindCol);
                returncode = SQLBindCol(hstmt0, i, SQL_C_DEFAULT,
					&lpfd->rgSWord[isword].data, sizeof(short int),
					&lpfd->rgSWord[isword].outlen);
				isword++;
				break;
			case SQL_FLOAT:
			case SQL_DOUBLE:
				FUNCCALL( sz_SQLBindCol);
                returncode = SQLBindCol(hstmt0, i, SQL_C_DEFAULT,
					&lpfd->rgSDouble[isdouble].data, sizeof(double),
					&lpfd->rgSDouble[isdouble].outlen);
				isdouble++;
				break;
			case SQL_REAL:
				FUNCCALL( sz_SQLBindCol);
                returncode = SQLBindCol(hstmt0, i, SQL_C_DEFAULT,
					&lpfd->rgSFloat[isfloat].data, sizeof(float),
					&lpfd->rgSFloat[isfloat].outlen);
				isfloat++;
				break;
			case SQL_CHAR:
			case SQL_VARCHAR:
				FUNCCALL( sz_SQLBindCol);
                returncode = SQLBindCol(hstmt0, i, SQL_C_DEFAULT,
					lpfd->rgUChar[iuchar].data, szSIZE,
					&lpfd->rgUChar[iuchar].outlen);
				iuchar++;
				break;
			case SQL_NUMERIC:
			case SQL_DECIMAL:
			default:
				FUNCCALL( sz_SQLBindCol);
				returncode = SQLBindCol(hstmt0, i, SQL_C_CHAR,
					lpfd->rgUChar[iuchar].data, szSIZE,
					&lpfd->rgUChar[iuchar].outlen);
				iuchar++;
				break;
		}

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			goto ErrorRet;
		}
	}

	INITLOOP;
	for(ind = 0; ind < MAX_TYPES; ind++) {
		ITERATION(ind);

		/* Get the data back */
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS)
			break;

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		iuchar = isdword = isword = isdouble = isfloat = 0;
		for(i = 0; i < cTypes; i++) { /* make sure it's original data placed
			in that field/row location */
			pch = MakeData(ind, &rgTypes[i], &rgCols[i], lpqt->szDataItem);

			if(!pch) {
				switch(rgTypes[i].wSQLType) { /* check the outlen and data
					returned for each type. */
				case SQL_INTEGER:
					isdword++;
					break;
				case SQL_SMALLINT:
					isword++;
					break;
				case SQL_FLOAT:
				case SQL_DOUBLE:
					isdouble++;
					break;
				case SQL_REAL:
					isfloat++;
					break;
				case SQL_DECIMAL:
				case SQL_CHAR:
				case SQL_TINYINT:
				case SQL_NUMERIC:
				case SQL_VARCHAR:
				default:
					iuchar++;
					break;
				}
				continue;
			}
			switch(rgTypes[i].wSQLType) { /* check the outlen and data
				returned for each type. */
				case SQL_INTEGER:
					if(lpfd->rgSDWord[isdword].outlen != SQL_NULL_DATA) {
						if(lpfd->rgSDWord[isdword].outlen
							!= sizeof(long int)) {
							szWrite(szIndent, FALSE );
							szWrite("ERROR:  Incorrect outlen for "
								"SQL_INTEGER", TRUE );
						}
						lstrcpy(szNum, pch);
						if(atol(szNum) == lpfd->rgSDWord[isdword].data)
							lstrcpy(lpqt->sz, pch);
						else
							lstrcpy(lpqt->sz, ltoa(
								lpfd->rgSDWord[isdword].data, szNum, 10));
					}
					else {
						lstrcpy(lpqt->sz, sz_NULL );
					}
					isdword++;
					break;
				case SQL_SMALLINT:
					if(lpfd->rgSWord[isword].outlen != SQL_NULL_DATA) {
						lstrcpy(lpqt->sz, itoa(lpfd->rgSWord[isword].data,
							szNum, 10));
						if(lpfd->rgSWord[isword].outlen !=
							sizeof(short int)) {
							szWrite(szIndent, FALSE );
							szWrite("ERROR:  Incorrect outlen for "
								"SQL_SMALLINT", TRUE );
						}
					}
					else {
						lstrcpy(lpqt->sz, sz_NULL);
					}
					isword++;
					break;
				case SQL_FLOAT:
				case SQL_DOUBLE:
					if(lpfd->rgSDouble[isdouble].outlen != SQL_NULL_DATA) {
						lstrcpy(szNum, pch);
						dNum = atof(szNum);

						/* for floats verify result retrieved
							back is close to what we put in */

						if(dNum-lpfd->rgSDouble[isdouble].data < .000005
							&& dNum-lpfd->rgSDouble[isdouble].data > -.000005)
							lstrcpy(lpqt->sz, pch);
						else {
							sprintf(szNum, "%f",
								lpfd->rgSDouble[isdouble].data);
							lstrcpy(lpqt->sz, szNum);
						}

						if(lpfd->rgSDouble[isdouble].outlen != sizeof(double))
						{
							szWrite(szIndent, FALSE );
							szWrite("ERROR:  Incorrect outlen for SQL_FLOAT"
								" or SQL_DOUBLE", TRUE );
						}
					} else {
						lstrcpy(lpqt->sz, sz_NULL);
					}
					isdouble++;
					break;
				case SQL_REAL:
					if(lpfd->rgSFloat[isfloat].outlen != SQL_NULL_DATA) {
						if(lpfd->rgSFloat[isfloat].outlen != sizeof(float)) {
							szWrite(szIndent, FALSE );
							szWrite("ERROR:  Incorrect outlen for SQL_REAL",
								TRUE );
							cErrorCount++;
						}
						lstrcpy(szNum, pch);
						dNum = atof(szNum);

						/* just looking for *close* in a floating point
							number */

						if(dNum - lpfd->rgSFloat[isfloat].data < .00005
							&& dNum - lpfd->rgSFloat[isfloat].data > -.00005)
								lstrcpy(lpqt->sz, pch);
						else {
							sprintf(szNum, "%f",
								(double)lpfd->rgSFloat[isfloat].data);
							lstrcpy(lpqt->sz, szNum);
						}
					}
					else {
						lstrcpy(lpqt->sz, sz_NULL);
					}
					isfloat++;
					break;
				case SQL_DECIMAL:
					if(lpfd->rgUChar[iuchar].outlen != SQL_NULL_DATA) {
						if(lpfd->rgUChar[iuchar].outlen > szSIZE) {
							szWrite(szIndent, FALSE );
							szWrite("ERROR:  Incorrect outlen for "
								"SQL_DECIMAL", TRUE );
							cErrorCount++;
						}
						lstrcpy(szNum, pch);
						dNum = atof(szNum);
						lstrcpy(szNum, lpfd->rgUChar[iuchar].data);
						dNum -= atof(szNum);

						/* just looking for *close* in a floating point
							number */

						if(dNum < .0005	&& dNum > -.0005)
							lstrcpy(lpqt->sz, pch);
						else
							lstrcpy(lpqt->sz, lpfd->rgUChar[iuchar].data);
					}
					else {
						lstrcpy(lpqt->sz, sz_NULL);
					}
					iuchar++;
					break;
				case SQL_CHAR:
				case SQL_TINYINT:
					if(lpfd->rgUChar[iuchar].outlen != SQL_NULL_DATA) {
						if(lpfd->rgUChar[iuchar].outlen > szSIZE) {
							szWrite(szIndent, FALSE );
							szWrite("ERROR:  Incorrect outlen for SQL_CHAR, "
								" etc.", TRUE );
						}

						/* get rid of spaces used as right padding */

						for(isp = lpfd->rgUChar[iuchar].outlen - 1
							;isp >= 0 && lpfd->rgUChar[iuchar].data[isp] 
								== ' ' 
							;isp--);
						_fstrncpy(lpqt->sz, lpfd->rgUChar[iuchar].data,
							(int)isp + 1);
						lpqt->sz[isp + 1] = '\0';
					}
					else {
						lstrcpy(lpqt->sz, sz_NULL);
					}
					iuchar++;
					break;
				case SQL_NUMERIC:
				case SQL_VARCHAR:
				default:
					if(lpfd->rgUChar[iuchar].outlen != SQL_NULL_DATA) {
						lstrcpy(lpqt->sz, lpfd->rgUChar[iuchar].data);
						if(lpfd->rgUChar[iuchar].outlen > szSIZE) {
							szWrite(szIndent, FALSE );
							szWrite("ERROR:  Incorrect outlen", TRUE );
						}
					}
					else {
						lstrcpy(lpqt->sz, sz_NULL);
					}
					iuchar++;
					break;
			}
			if(*pch) {
				if(lstrcmp(lpqt->sz, pch) != 0) {
					szWrite(szIndent, FALSE );
					szWrite("Failed to return input ", FALSE );
					szWrite(rgTypes[i].szType, TRUE );
					szWrite("Data checked: ", FALSE );
					szWrite(lpqt->sz, TRUE );
	 				szWrite("Should have been: ", FALSE );
					szWrite(lpqt->szDataItem, TRUE );
					cErrorCount++;
				}
			}
		}
	}

	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}
		
	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_UNBIND);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_RESET_PARAMS);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);




	szWrite("Testing = all types ", TRUE );
	lstrcpy(lpqt->sz, "select * from ");
	lstrcat(lpqt->sz, lpqt->szTableName);
	lstrcat(lpqt->sz, " where ");

	for(i = 0, w = 0; i < cTypes; i++)
		if ((rgTypes[i].searchable == SQL_SEARCHABLE ||
			rgTypes[i].searchable == SQL_ALL_EXCEPT_LIKE) &&
			!rgCols[i].fAutoUpdate) {
			w++;
			wsprintf(&lpqt->sz[lstrlen(lpqt->sz)], "%s = ? and ",
				(char FAR *)rgTypes[i].szFieldName);
			FUNCCALL( sz_SQLSetParam);
			returncode = SQLSetParam(hstmt0, w,
				SQL_C_CHAR, rgTypes[i].wSQLType,
				rgCols[i].precision, rgCols[i].scale,
				(char FAR *)(UDWORD)i, &sdw);
			if( returncode != SQL_SUCCESS) {
				cErrorCount++;
				DisplayRetcode(TRUE, returncode);
				DisplayRetcode(FALSE, SQL_SUCCESS);
			}
			cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
			ind++;
		}
	lpqt->sz[lstrlen(lpqt->sz) - 5] = '\0';
	szWrite(lpqt->sz, TRUE );

	FUNCCALL( sz_SQLPrepare);
	returncode = SQLPrepare(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

    sdw = SQL_DATA_AT_EXEC;

	FUNCCALL( sz_SQLExecute);
	returncode = SQLExecute(hstmt0);
	if( returncode != SQL_NEED_DATA) {
		cErrorCount++;
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NEED_DATA);
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	udw = cTypes;
	
	for(i = 0; ; i++) {
		FUNCCALL( sz_SQLParamData);
		returncode = SQLParamData(hstmt0, (PTR FAR *)&udw);
		if( returncode != SQL_NEED_DATA)
			break;
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		if(udw < cTypes)
			pch = MakeData(cTypes, &rgTypes[udw], &rgCols[udw],
				lpqt->szDataItem);
		else {
			cErrorCount++;
			szWrite("ParamData returned invalid rgbValue", TRUE );
		}

		FUNCCALL( sz_SQLPutData);
		returncode = SQLPutData(hstmt0, pch, lstrlen(pch));
		if( returncode != SQL_SUCCESS) {
			cErrorCount++;
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	}
	if( returncode != SQL_SUCCESS) {
		cErrorCount++;
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	for(i = 0;; i++) {
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode != SQL_SUCCESS) {
			if( returncode != SQL_NO_DATA_FOUND) {
				cErrorCount++;
				DisplayRetcode(TRUE, returncode);
				DisplayRetcode(FALSE, SQL_SUCCESS);
			}
			break;
		}
	}
	
	/* should have gotten 1 row1 back */

	if(i != 1) {
		szWrite(szIndent, FALSE );
		szWrite("Incorrect number of rows returned", TRUE );
		cErrorCount++;
	}

	/* SQLFreeStmt with SQL_CLOSE to re-use the hstmt */

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		cErrorCount++;
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
	}

	/* verify table shows up in SQLTables */
		
	FUNCCALL( sz_SQLTables);
	returncode = SQLTables(hstmt0, NULL, 0, NULL, 0, "q%",
		SQL_NTS, "'TABLE'", SQL_NTS); /* this call may return many 
			tables, as long as the one created earlier shows up it
			will pass. */
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	INITLOOP;
	for(i = 0;; i++) {
		ITERATION(i);

		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS)
			break;

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		FUNCCALL( sz_SQLGetData); /* tablename */
		returncode = SQLGetData(hstmt0, 3, SQL_C_CHAR, lpqt->sz, szSIZE, NULL);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		fFoundTable += 0 == lstrcmpi(lpqt->sz, lpqt->szTableName);
	}

	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	if(1 != fFoundTable) {
		szWrite(szIndent, FALSE );
		szWrite("SQLTables failed to return test table", TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	/* create an index and verify that it is returned by SQLStatistics */

	lstrcpy(lpqt->szDataItem, lpqt->szTableName);
	lstrcat(lpqt->szDataItem, "i");
	for(i = 1; i < cTypes; i++)
		if(rgTypes[i].wSQLType == SQL_VARCHAR ||
			rgTypes[i].wSQLType == SQL_CHAR)
			break;
	if(i == cTypes)
		i = 0;
		
	lstrcpy(lpqt->buf, rgTypes[i].szFieldName);
	wsprintf(lpqt->sz, "create index %s on %s (%s)",
		lpqt->szDataItem, lpqt->szTableName, lpqt->buf);
	szWrite(lpqt->sz, TRUE );
		FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	fIndex = returncode == SQL_SUCCESS;

	/* this should return a keyset if one exists for the table. */
	
	FUNCCALL( sz_SQLSpecialColumns);
	returncode = SQLSpecialColumns(hstmt0, SQL_BEST_ROWID, NULL, 0, NULL, 0,
		lpqt->szTableName, lstrlen(lpqt->szTableName), SQL_SCOPE_TRANSACTION,
		SQL_NULLABLE);
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		goto ErrorRet;
	}
	
	for(i = 0; i < cTypes; i++){
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
		if( returncode == SQL_NO_DATA_FOUND)
			break;
	}

	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	/* verify the index shows up in SQLStatistics */
	FUNCCALL( sz_SQLStatistics);
	returncode = SQLStatistics(hstmt0, NULL, 0, NULL, 0, lpqt->szTableName,
        SQL_NTS, SQL_INDEX_ALL, SQL_ENSURE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	INITLOOP;
	fFoundTable = 0;
	for(i = 0; i < MAX_TYPES; i++) {
		ITERATION(i);

		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS)
			break;

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData(hstmt0, 3, SQL_C_CHAR, lpqt->sz, szSIZE,
			NULL);

		if (lstrcmp(lpqt->sz, lpqt->szTableName) == 0)
		{	//	Only test for index name if correct table name 
			FUNCCALL( sz_SQLGetData);
			returncode = SQLGetData(hstmt0, 6, SQL_C_CHAR, lpqt->sz, szSIZE,
				NULL);
			if( returncode != SQL_SUCCESS) {
				DisplayRetcode(TRUE, returncode);
				DisplayRetcode(FALSE, SQL_SUCCESS);
				cErrorCount++;
			}
			cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
			fFoundTable += 0 == lstrcmp(lpqt->sz, lpqt->szDataItem);
		}
	}

	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	if(1 != fFoundTable && fIndex) {
		szWrite(szIndent, FALSE );
		szWrite("SQLStatistics failed to find single test index",
			TRUE );
		cErrorCount++;
	}

	if(i > 2 || i < 1) { /* one row represents original table, the other represents
					the index */
		szWrite( szIndent, FALSE );
		szWrite( "Wrong number of rows", TRUE );
		szWrite( sz_Expected, FALSE );
		szWrite( itoa(2, szNum, 10), TRUE );
		szWrite( sz_Returned, FALSE );
		szWrite( ltoa(i, szNum, 10), TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt );
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode);
		DisplayRetcode( FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000);

	for( i = 0; i < cTypes; i++) {
		if( rgTypes[i].wSQLType == SQL_CHAR ) {
			lstrcpy( lpqt->buf, rgTypes[i].szFieldName );
			break;
		}
		if(rgTypes[i].wSQLType == SQL_VARCHAR) {
			lstrcpy( lpqt->buf, rgTypes[i].szFieldName );
			break;
		}
	}

	/* execute a query using like.  This query should return all records */

	wsprintf(lpqt->sz, "select * from %s where %s not like 'a' or"
		" %s is NULL", lpqt->szTableName,
		lpqt->buf, lpqt->buf);
	szWrite(lpqt->sz, TRUE );

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	INITLOOP;
	for(i = 0; i <= MAX_TYPES; i++) {
		ITERATION(i);

		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS)
			break;

		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData(hstmt0, 1, SQL_C_CHAR, lpqt->sz, szSIZE,
			NULL);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);
	}

	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	if(i != cTypes + 1) {
		szWrite(szIndent, FALSE );
		szWrite("Wrong number of rows inserted/fetched", TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	/* all done with the body of the test, time to clean up */

		/* SQLFreeStmt SQL_CLOSE to re-use it */

		FUNCCALL( sz_SQLFreeStmt);
		returncode = SQLFreeStmt(hstmt0, SQL_CLOSE);
		if( returncode != SQL_SUCCESS) {
			cErrorCount++;
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
		}

	
	lstrcpy(lpqt->sz, "drop table ");
	lstrcat(lpqt->sz, lpqt->szTableName);

	if(*pTestSource->szValidServer0) {
		szWrite("Dropping table", TRUE );
		szWrite(lpqt->sz, TRUE );

		FUNCCALL( sz_SQLDisconnect);
		returncode = SQLDisconnect(hdbc0);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, NULL, sz_00000);

		hstmt0 = hstmt1;
	}

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		szWrite(szIndent, FALSE );
		szWrite("Unable to drop table", TRUE );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	/* SQLCancel has the same functionality as SQLFreeStmt w/ SQL_CLOSE
		in a non-asynchronous environment */
	
	FUNCCALL( sz_SQLCancel);
	returncode = SQLCancel(hstmt0);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt(hstmt0, SQL_DROP);
	if( returncode != SQL_SUCCESS) {
		DisplayRetcode(TRUE, returncode);
		DisplayRetcode(FALSE, SQL_SUCCESS);
		cErrorCount++;
	}
	hstmt0 = NULL;
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

	
	if(*pTestSource->szValidServer0) { /* if the connection was made in the
										test, it should be disconnected
						in the test, otherwise it should be left connected */

		FUNCCALL( sz_SQLDisconnect);
		returncode = SQLDisconnect(hdbc1);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc1, hstmt0, sz_00000);

		FUNCCALL( sz_SQLFreeConnect);
		returncode = SQLFreeConnect(hdbc0);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		hdbc0 = NULL;
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		FUNCCALL( sz_SQLFreeConnect);
		returncode = SQLFreeConnect(hdbc1);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000);

		FUNCCALL( sz_SQLFreeEnv);
		returncode = SQLFreeEnv(henv0);
		if( returncode != SQL_SUCCESS) {
			DisplayRetcode(TRUE, returncode);
			DisplayRetcode(FALSE, SQL_SUCCESS);
			cErrorCount++;
		}

		henv0 = NULL;
	}

	ControlSzWrite( SAVE,   SAME );
	ControlSzWrite( ENABLE, SAME );

	if(cErrorCount > 0 || cStateErrorCount > 0 ) {
		szWrite( "Quick Test FAILED", TRUE );
		szWrite( "Failed Tests: ", FALSE );
		szWrite( itoa(cErrorCount, szNum, 10 ), TRUE );
		szWrite( "Failed States: ", FALSE );
		szWrite( itoa( cStateErrorCount, szNum, 10 ), TRUE );
		pTestSource->cErrors		= cErrorCount;
		pTestSource->cStateErrors	= cStateErrorCount;
	}
	else
		szWrite( "**** Quick Test PASSED ****", TRUE );

	ControlSzWrite( RESTORE, SAME );

	ReleaseMemory( lpfd );
	ReleaseMemory( lpqt );
	ReleaseMemory( lpParams );

	return;


ErrorRet:

	/* a failure in an ODBC function that prevents completion of the
		test - for example, connect to the server */

	ControlSzWrite( SAVE,   SAME );
	ControlSzWrite( ENABLE, SAME );

	fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );

	szWrite( szIndent, FALSE );
	szWrite(" *** Unrecoverable Quick Test FAILURE ***", TRUE );
		
	ControlSzWrite(RESTORE, SAME);

	pTestSource->cErrors = -1;
	pTestSource->cStateErrors = -1;
	
	ReleaseMemory(lpfd);
	ReleaseMemory(lpqt);
	ReleaseMemory(lpParams);

	return;
}
