
	/*
	  PROGRAM: l2qtest.c
			 (C) 1990-1992 Microsoft Corporation.  All right reserved.
	
	  PURPOSE: This is a Quick Test of the general functionality of a
			Level 2 ODBC driver
	
	  FUNCTIONS:
			EXTQuickTestL2() - performs the quick test
	*/

#include "gator.h"
//#undef QUIET
#include "tests.h"
#include <stdio.h>
#include "autotest.h"

#ifdef MACINTOSH
#include <dos.h>			// mj
	
#define sz_empty	""		// mj
#define sz_NULL		""		// mj
#define sz_Expected	""		// mj
#define sz_Returned	""		// mj
	LPBYTE	itoa( int,  LPBYTE, int );
#endif MACINTOSH

#define MAX_CROW 20
#define MAXSTR 45
#define MAX_PARAM 30

typedef struct tagQtStruct {
	char	buf[ OUTSTRLEN ];
	char	sz[ szSIZE ];
	char	szParam[ MAX_PARAM ];
	char	szDataItem[ MAXSTR ];
	char	szTableName[ TABLENAMESIZE ];
	char	szColNames[ 500 ];
	char	szValues[ 20 * MAX_TYPES ];
	SDWORD  cbValues[ MAX_TYPES ];
}	QTSTRUCT;

pascal int VerifyFieldName( HSTMT hstmt0, int i, char FAR *sz );

pascal void ExtQuickTestL2( struct ServerStruct *pTestSource )
{
	RETCODE     returncode;
	UDWORD      cRow;
	UWORD       i;
	UWORD       ind;
	UWORD       cTypes;
	SWORD       wNum;
	SWORD       fSqlType;
	SWORD       ibScale;
	SWORD       fNullable;
	UDWORD      cbColDef;
	HENV        henv0 = NULL;
	HDBC        hdbc0 = NULL;
	HDBC        hdbc1 = NULL;
	HSTMT       hstmt0 = NULL;
	HSTMT       hstmt1 = NULL;
	char	szNum[30];
	TYPENUM FAR * rgTypes = LockMemory( AllocateMemory( sizeof( TYPENUM )
		* MAX_TYPES));
	COLDATA FAR * rgCols = LockMemory( AllocateMemory( sizeof( COLDATA )
		* MAX_TYPES));
	UDWORD FAR *pirow = LockMemory( AllocateMemory( sizeof( UDWORD ) ) );
	int         fFoundTable;
	int         cErrorCount = 0;
	int         cStateErrorCount=0;
	QTSTRUCT FAR	*lpqt = LockMemory( AllocateMemory( sizeof( QTSTRUCT ) ) );
	UWORD			rgbRowStatus[ MAX_CROW ];
	SDWORD			cbValue;
	char FAR		*rgStates[] = {NULL, NULL, NULL};
	char FAR		*pch;

		/* initialize array used for multiple states (connecting to invalid
			servername) initializing in the declaration causes invalid
			aggregate initializer warnings */
	
	rgStates[0] = sz_00000;
	rgStates[1] = sz_01000;

	szWrite( "Testing Level 1 functions", TRUE );
	ExtQuickTestL1(pTestSource);
	if(pTestSource->cErrors == -1) /* if Level 1 quick test did not complete, 
										do not continue with this one */
		return;

	szWrite( "Finished testing Level 1 functions", TRUE );

		/* initialize connection and stmt handle */
	
	if(*pTestSource->szValidServer0) { /* only want to SQLAllocEnv if
											Allocated handle not passed in */
		FUNCCALL( sz_SQLAllocEnv);
		returncode = SQLAllocEnv( &henv0 );
		cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE,  returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}

		FUNCCALL( sz_SQLAllocConnect );
		returncode = SQLAllocConnect(henv0, &hdbc0 );
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}

		FUNCCALL( sz_SQLAllocConnect);
		returncode = SQLAllocConnect(henv0, &hdbc1);
		cStateErrorCount += fStateCheck(henv0, hdbc1, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}

		wsprintf(lpqt->sz, "dsn=%s;uid=%s;pwd=%s;", 
			(char FAR *)pTestSource->szValidServer0, 
			(char FAR *)pTestSource->szValidLogin0, 
			(char FAR *)pTestSource->szValidPassword0);

		szWrite( "szConnStrIn:  ", FALSE );
		szWrite(lpqt->sz, TRUE );

		FUNCCALL( sz_SQLDriverConnect);
		returncode = SQLDriverConnect(hdbc0, pTestSource->hwnd, lpqt->sz, 
			SQL_NTS, lpqt->buf, szSIZE, NULL, SQL_DRIVER_COMPLETE);
		if( returncode != SQL_SUCCESS && returncode !=
			SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}
		cStateErrorCount += frgStateCheck(henv0, hdbc0, hstmt0, rgStates);

		FUNCCALL( sz_SQLDriverConnect);
		returncode = SQLDriverConnect(hdbc1, pTestSource->hwnd, lpqt->sz, 
			SQL_NTS, lpqt->buf, szSIZE, NULL, SQL_DRIVER_COMPLETE);
		if( returncode != SQL_SUCCESS && returncode !=
			SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}
		cStateErrorCount += frgStateCheck(henv0, hdbc1, hstmt0, rgStates);
	}
	else {
		henv0 = pTestSource->henv;
		hdbc0 = pTestSource->hdbc;
	}

	cTypes = MakeTable(hdbc0, lpqt->szTableName, rgTypes, rgCols);
		
	if(cTypes < 1) {
		pTestSource->cErrors = -1;
		if(cTypes == -1)
			goto ErrorRet;	
		else
			goto ErrorDrop;
	}
		
	/* create a query to test with SQLNativeSql */
	
	wsprintf(lpqt->sz, "insert into %s (cchar) values (\"pepermint stick"
		" ice cream\")", lpqt->szTableName);
	lstrcpy( lpqt->buf, "No string returned");
	lstrcpy( lpqt->szValues, lpqt->buf);
	FUNCCALL( sz_SQLNativeSql);
	returncode = SQLNativeSql(hdbc0, lpqt->sz, SQL_NTS, 
		lpqt->buf, OUTSTRLEN, NULL);
	if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
	szWrite( "Native SQL string: ", FALSE );
	szWrite(lpqt->buf, TRUE );

	if(lstrcmp(lpqt->buf, lpqt->szValues) == 0) { /* can't verify the sting
			returned, but can make sure that something is returned. */
		szWrite(szIndent, TRUE );
		szWrite( "error, no string returned", TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLAllocStmt);
	returncode = SQLAllocStmt(hdbc0, &hstmt0);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	FUNCCALL( sz_SQLAllocStmt);
	returncode = SQLAllocStmt(hdbc1, &hstmt1);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

/* test SQLPrimaryKeys */
/* there is no way to know if data will be returned by this call.  Can only
verify that the call returns SQL_SUCCESS, and the correct fields are returned.
*/
	FUNCCALL( sz_SQLPrimaryKeys);
	returncode=SQLPrimaryKeys(hstmt0, NULL, 0, NULL, 0, lpqt->szTableName, 
			SQL_NTS);
	if(returncode!=SQL_SUCCESS) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	FUNCCALL( sz_SQLNumResultCols);
	returncode = SQLNumResultCols(hstmt0, &wNum);
	if(returncode!=SQL_SUCCESS) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	if(wNum < 5) {
		szWrite(szIndent, FALSE );
		szWrite( "Too few cols returned", TRUE );
		cErrorCount++;
	}
	
	cErrorCount += VerifyFieldName(hstmt0, 1, "TABLE_QUALIFIER");
	cErrorCount += VerifyFieldName(hstmt0, 2, "TABLE_OWNER");
	cErrorCount += VerifyFieldName(hstmt0, 3, "TABLE_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 4, "COLUMN_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 5, "KEY_SEQ");
	
	/* fetch any data that is returned */
	INITLOOP;
	for( i = 0;; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO)
			break;
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}
		
	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}

/* test SQLForeignKeys */
	FUNCCALL( sz_SQLForeignKeys);
	returncode=SQLForeignKeys(hstmt0, NULL, 0, NULL, 0, lpqt->szTableName, 
		SQL_NTS, NULL, 0, NULL, 0, lpqt->szTableName, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	FUNCCALL( sz_SQLNumResultCols);
	returncode = SQLNumResultCols(hstmt0, &wNum);
	if(returncode!=SQL_SUCCESS) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	if(wNum < 11) {
		szWrite(szIndent, FALSE );
		szWrite( "Too few cols returned", TRUE );
		cErrorCount++;
	}
	
	cErrorCount += VerifyFieldName(hstmt0, 1, "PKTABLE_QUALIFIER");
	cErrorCount += VerifyFieldName(hstmt0, 2, "PKTABLE_OWNER");
	cErrorCount += VerifyFieldName(hstmt0, 3, "PKTABLE_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 4, "PKCOLUMN_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 5, "FKTABLE_QUALIFIER");
	cErrorCount += VerifyFieldName(hstmt0, 6, "FKTABLE_OWNER");
	cErrorCount += VerifyFieldName(hstmt0, 7, "FKTABLE_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 8, "FKCOLUMN_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 9, "KEY_SEQ");
	cErrorCount += VerifyFieldName(hstmt0, 10, "UPDATE_RULE");
	cErrorCount += VerifyFieldName(hstmt0, 11, "DELETE_RULE");

	INITLOOP;
	for( i = 0;; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode=SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO)
			break;
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
	if(returncode!=SQL_NO_DATA_FOUND) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}
		
	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if(returncode!=SQL_SUCCESS) {
		cErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

/* test SQLTablePrivileges */
	FUNCCALL( sz_SQLTablePrivileges);
	returncode=SQLTablePrivileges(hstmt0, NULL, 0, NULL, 0,
		lpqt->szTableName, SQL_NTS);
	if(returncode!=SQL_SUCCESS) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
		
	FUNCCALL( sz_SQLNumResultCols);
	returncode = SQLNumResultCols(hstmt0, &wNum);
	if(returncode!=SQL_SUCCESS) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	if(wNum < 7) {
		szWrite(szIndent, FALSE );
		szWrite( "Too few cols returned", TRUE );
		cErrorCount++;
	}
	
	cErrorCount += VerifyFieldName(hstmt0, 1, "TABLE_QUALIFIER");
	cErrorCount += VerifyFieldName(hstmt0, 2, "TABLE_OWNER");
	cErrorCount += VerifyFieldName(hstmt0, 3, "TABLE_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 4, "GRANTOR");
	cErrorCount += VerifyFieldName(hstmt0, 5, "GRANTEE");
	cErrorCount += VerifyFieldName(hstmt0, 6, "PRIVILEGE");
	cErrorCount += VerifyFieldName(hstmt0, 7, "IS_GRANTABLE");

	fFoundTable = FALSE;
	INITLOOP;
	for( i = 0;; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS 
			&& returncode != SQL_SUCCESS_WITH_INFO)
			break;
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
		FUNCCALL( sz_SQLGetData);
		returncode=SQLGetData( hstmt0, 3, SQL_C_CHAR, lpqt->sz,
			szSIZE, NULL);
		if( returncode != SQL_SUCCESS &&
			returncode != SQL_SUCCESS_WITH_INFO ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
		fFoundTable += 0 == lstrcmp(lpqt->sz, lpqt->szTableName);
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_NO_DATA_FOUND);
		cErrorCount++;
	}
		
	if(1 != fFoundTable) {
		szWrite(szIndent, FALSE );
		szWrite( "SQLTablePrivileges failed to find test table", TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		cErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

/* test SQLColumnPrivileges */
	FUNCCALL( sz_SQLColumnPrivileges);
	returncode=SQLColumnPrivileges(hstmt0, NULL, 0, NULL, 0,
		lpqt->szTableName, SQL_NTS, NULL, 0);
	if( returncode != SQL_SUCCESS ) {
		cErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	FUNCCALL( sz_SQLNumResultCols);
	returncode = SQLNumResultCols(hstmt0, &wNum);
	if(returncode!=SQL_SUCCESS) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	if(wNum < 8) {
		szWrite(szIndent, FALSE );
		szWrite( "Too few cols returned", TRUE );
		cErrorCount++;
	}
	
	cErrorCount += VerifyFieldName(hstmt0, 1, "TABLE_QUALIFIER");
	cErrorCount += VerifyFieldName(hstmt0, 2, "TABLE_OWNER");
	cErrorCount += VerifyFieldName(hstmt0, 3, "TABLE_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 4, "COLUMN_NAME");
	cErrorCount += VerifyFieldName(hstmt0, 5, "GRANTOR");
	cErrorCount += VerifyFieldName(hstmt0, 6, "GRANTEE");
	cErrorCount += VerifyFieldName(hstmt0, 7, "PRIVILEGE");
	cErrorCount += VerifyFieldName(hstmt0, 8, "IS_GRANTABLE");

	fFoundTable = FALSE;
	INITLOOP;
	for( i = 0;; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS 
			&& returncode != SQL_SUCCESS_WITH_INFO)
			break;
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 3, SQL_C_CHAR, lpqt->sz,
			szSIZE, NULL);
		if( returncode != SQL_SUCCESS &&
			returncode != SQL_SUCCESS_WITH_INFO) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
		fFoundTable += 0 == lstrcmp(lpqt->sz, lpqt->szTableName);
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");
	if( returncode != SQL_NO_DATA_FOUND) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
		
	if(1 != fFoundTable) {
		szWrite(szIndent, FALSE );
		szWrite( "SQLColumnPrivileges failed to find test table", TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	/* This will be the ExtendedFetch test when SetScrollOptions
		can be called. */
	
	FUNCCALL( sz_SQLSetScrollOptions);
	returncode = SQLSetScrollOptions(hstmt0, SQL_CONCUR_VALUES, 
			SQL_SCROLL_FORWARD_ONLY, 4);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	wsprintf(lpqt->sz, "select * from %s", lpqt->szTableName);
	szWrite(lpqt->sz, TRUE );
	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	for( i = 0; i < cTypes; i++){
		if(rgTypes[i].wSQLType == SQL_CHAR)
			lstrcpy( lpqt->buf, rgTypes[i].szType);
		if(rgTypes[i].wSQLType == SQL_VARCHAR) {
			lstrcpy( lpqt->buf, rgTypes[i].szType);
			break;
		}
	}

	FUNCCALL( sz_SQLBindCol);
	returncode = SQLBindCol(hstmt0, i+1, SQL_C_CHAR, lpqt->sz, szSIZE,
		&cbValue);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck( henv0, hdbc0, hstmt0, "00000" );

	FUNCCALL( sz_SQLExtendedFetch );
	returncode = SQLExtendedFetch( hstmt0, SQL_FETCH_FIRST, 1, 
		&cRow, rgbRowStatus);

	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000");

	FUNCCALL( sz_SQLSetPos);
	returncode = SQLSetPos(hstmt0, 0, TRUE, TRUE );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, "00000" );

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE );
	if( returncode != SQL_SUCCESS ) {
		cErrorCount++;
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}


		/* ParamOptions:  Create an array with the cchar field entries of all but the
		first record, set this array as the param, and attempt to access each of
		the records individually using ParamOptions. */
	
	for( i = 0; i < cTypes; i++ ) {
		if( rgTypes[i].wSQLType == SQL_CHAR 
			|| rgTypes[i].wSQLType == SQL_VARCHAR ) {
			lstrcpy( lpqt->buf, rgTypes[i].szFieldName );
			ind = i;
			break;
		}
	}

	lpqt->szValues[0] = '\0';
	for( i = 0; i <= cTypes; i++ ) {
		pch = MakeData( i, &rgTypes[ ind ], &rgCols[ ind ], &lpqt->szValues[
			lstrlen( lpqt->szValues )] );
		lpqt->cbValues[i] = pch ? lstrlen( pch ) : 0;
	}

	FUNCCALL( sz_SQLSetParam );
	returncode = SQLSetParam( hstmt0, 1, SQL_C_CHAR, rgTypes[ind].wSQLType, 
		rgCols[ind].precision, rgCols[ind].scale, lpqt->szValues,
		lpqt->cbValues);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLParamOptions);
	returncode = SQLParamOptions(hstmt0, cTypes, pirow);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	wsprintf(lpqt->sz, "select %s from %s where %s = ?",
		lpqt->buf, lpqt->szTableName, lpqt->buf);
	szWrite(lpqt->sz, TRUE );
	
	*pirow = 2;
	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	if(*pirow != i) {
		szWrite(szIndent, FALSE );
		szWrite( "row count was not increased", TRUE );
		cErrorCount++;
		*pirow = i;
	}

	FUNCCALL( sz_SQLNumParams);
	returncode = SQLNumParams(hstmt0, &wNum);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	if(wNum != 1) {
		szWrite(szIndent, FALSE );
		szWrite( "Incorrect number of params", TRUE );
		cErrorCount++;
	}

	fSqlType = ibScale = fNullable = 99;
	cbColDef = 99;
	lpqt->szDataItem[0] = '\0';

	FUNCCALL( sz_SQLDescribeParam);
	returncode = SQLDescribeParam(hstmt0, 1, 
		&fSqlType, &cbColDef, &ibScale, &fNullable);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	if(fNullable != rgTypes[ind].nullable) {
		szWrite(szIndent, FALSE );
		szWrite( "Failed to match nullable", TRUE );
		cErrorCount++;
	}

	if(ibScale != rgCols[ind].scale) {
		szWrite(szIndent, FALSE );
		szWrite( "Failed to match scale", TRUE );
		cErrorCount++;
	}

	if(cbColDef != rgCols[ind].precision) {
		szWrite(szIndent, FALSE );
		szWrite( "Failed to match cbColDef", TRUE );
		cErrorCount++;
	}

	if(fSqlType != rgTypes[ind].wSQLType) {
		szWrite(szIndent, FALSE );
		szWrite( "Failed to match SQLType", TRUE );
		cErrorCount++;
	}

	if(0 != lstrcmp(lpqt->szDataItem, lpqt->buf)) {
		szWrite(szIndent, FALSE );
		szWrite( "Failed to return correct column name", TRUE );
		cErrorCount++;
	}

	INITLOOP;
	for( i = 1; i <= cTypes; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

		if( returncode != SQL_SUCCESS )
			break;
		
		lpqt->szDataItem[0] = '\0';
		
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 1, SQL_C_CHAR, lpqt->szDataItem, 
			MAXSTR, &cbValue);
		
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

		pch = MakeData(i, &rgTypes[ind], &rgCols[ind], lpqt->szParam);
		
		if(cbValue != SQL_NULL_DATA && !pch)
			/* returned data is OK */;
		else if(lstrcmp(lpqt->szDataItem, pch ? pch : sz_NULL) != 0) {
			szWrite(szIndent, FALSE );
			szWrite( "Failed to return expected data", TRUE );
			szWrite(sz_Returned, FALSE );
			szWrite(lpqt->szDataItem, TRUE );
			szWrite(sz_Expected, FALSE );
			szWrite(pch, TRUE );
			cErrorCount++;
		}
	}

	if(i != cTypes) {
		szWrite( "Failed to return all the rows", TRUE );
		cErrorCount++;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	ErrorDrop:
		
	lstrcpy( lpqt->sz, "drop table ");
	lstrcat(lpqt->sz, lpqt->szTableName);

	if(*pTestSource->szValidServer0) {
		szWrite( "Dropping table", TRUE );
		szWrite(lpqt->sz, TRUE );

		FUNCCALL( sz_SQLDisconnect);
		returncode = SQLDisconnect(hdbc0);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, NULL, sz_00000 );

		hstmt0 = hstmt1;
	}

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		szWrite(szIndent, FALSE );
		szWrite( "Unable to drop table", TRUE );
		cErrorCount++;
	}
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_DROP);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	hstmt0 = NULL;
	cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	if(*pTestSource->szValidServer0) { /* if the test connected, it should
			disconnect, otherwise it should leave the connection */
		
		FUNCCALL( sz_SQLDisconnect);
		returncode = SQLDisconnect(hdbc1);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc1, hstmt0, sz_00000 );

		FUNCCALL( sz_SQLFreeConnect);
		returncode = SQLFreeConnect(hdbc0);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		hdbc0 = NULL;
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

		FUNCCALL( sz_SQLFreeConnect);
		returncode = SQLFreeConnect(hdbc1);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}
		cStateErrorCount += fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

		FUNCCALL( sz_SQLFreeEnv);
		returncode = SQLFreeEnv(henv0);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			cErrorCount++;
		}

		henv0 = NULL;
	}

	ControlSzWrite( SAVE, SAME );
	ControlSzWrite( ENABLE, SAME );

	if( cErrorCount > 0 || cStateErrorCount > 0 ) {
		szWrite( "Quick Test FAILED", TRUE );
		szWrite( "Failed Tests: ", FALSE );
		szWrite( itoa( cErrorCount, szNum, 10 ), TRUE );
		szWrite( "Failed States: ", FALSE );
		szWrite( itoa( cStateErrorCount, szNum, 10 ), TRUE );
	}
	else
		szWrite( "**** Quick Test PASSED ****", TRUE );

	ControlSzWrite( RESTORE, SAME );

	ReleaseMemory( lpqt );

	return;


ErrorRet:

		/* a failure in an ODBC function that prevents completion of the
			test - for example, connect to the server */

	szWrite(szIndent, FALSE );
	szWrite( " *** Unrecoverable Quick Test FAILURE ***", TRUE );
		
	fStateCheck(henv0, hdbc0, hstmt0, sz_00000 );

	return;
}


pascal int VerifyFieldName(HSTMT hstmt0, int i, char FAR *sz)
{
	RETCODE returncode;
	char buf[50];
	int cErrorCount = 0;
	
	FUNCCALL( sz_SQLDescribeCol);
	returncode = SQLDescribeCol(hstmt0, i, buf, OUTSTRLEN, NULL, NULL,
		NULL, NULL, NULL);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cErrorCount++;
	}
	fStateCheck( NULL, NULL, hstmt0, "00000");

	if(lstrcmpi(buf, sz) != 0) {
		szWrite(szIndent, FALSE );
		wsprintf(buf, "Field %d incorrect", i);
		szWrite(buf, TRUE );
		cErrorCount++;
	}
	return cErrorCount;
}

pascal char FAR * MakeData( int nRow, struct TypeNum FAR *TypeInfo, 
	struct ColData FAR *ColInfo, char FAR * sz )
{
	UDWORD		precision;
	UDWORD		scale;
	UDWORD		i;
	char		szCol[10];
	char		szDecimal[30];
	char FAR	*pch;

	if(ColInfo->fAutoUpdate)
		return NULL;

	if(nRow == TypeInfo->iFieldNum && ColInfo->nullable == SQL_NULLABLE) {
		*sz = '\0';
		return sz;
	}


	wsprintf(szCol, "%d", nRow);
	
	switch (TypeInfo->wSQLType) {
		case SQL_FLOAT:
		case SQL_REAL:
		case SQL_DOUBLE:
			scale = nRow > MAX_SCALE ? MAX_SCALE : nRow;
			break;
		default:
			scale = ColInfo->scale;
		}
		
	*sz = NULL;

	if(ColInfo->precision > 20 || ColInfo->precision <= 0) {
		if(scale > 20)
			scale = 20;
		precision = 20;
	} else
		precision = ColInfo->precision;
		if(precision > 5)
			switch (TypeInfo->wSQLType) {
			case SQL_NUMERIC:
			case SQL_DECIMAL:
			case SQL_FLOAT:
			case SQL_DOUBLE:
			case SQL_INTEGER:
				precision--;
				break;
			case SQL_REAL:
				precision = 5;
				break;
			}

	switch(TypeInfo->wSQLType) {
		case SQL_TINYINT:
			precision = sizeof(signed char);
			break;
		case SQL_SMALLINT:
			precision = sizeof(short int);
			break;
		default:
			;
		}

	*szDecimal = NULL;
	switch(TypeInfo->wSQLType) {
		case SQL_NUMERIC  :
		case SQL_DECIMAL  :
		case SQL_FLOAT    :
		case SQL_REAL     :
		case SQL_DOUBLE   :
			lstrcat(szDecimal, ".");
			for( i = 0; i < scale; i += strlen(szCol))
				lstrcat(szDecimal, szCol);
			break;
		default:
			scale = 0;
		}

	for( i = 0
		; i < (precision >= scale ? precision - scale : precision)
		; i += strlen(szCol))
		lstrcat(sz, (char FAR *)szCol);

	lstrcat(sz, (char FAR *)szDecimal);

	/* if we had an odd 2 digit numbers, we may have ended up with an even
		number of characters, one too many. */
	
	switch(TypeInfo->wSQLType)
		{
			/* types with a decimal point shouldn't have this problem */
		case SQL_NUMERIC  :
		case SQL_DECIMAL  :
		case SQL_FLOAT    :
		case SQL_REAL     :
		case SQL_DOUBLE   :
			break;
		default:
			sz[precision] = '\0';

		}

	if(TypeInfo->wSQLType == SQL_BINARY ||
		TypeInfo->wSQLType == SQL_VARBINARY) {
		for( i = lstrlen(sz) - 1; i > 1 && sz[i] == '0'; i--)
			sz[i] = '\0'; 
		/* binary numbers right justify everything */
		if(lstrlen(sz) > 2 && (lstrlen(sz) / 2 == (lstrlen(sz) - 1) / 2))
		{
			lstrcpy(szDecimal, sz);
			sz[0] = '0';
			lstrcpy(&sz[1], szDecimal);
		}
	}


	switch(TypeInfo->wSQLType) {
		case SQL_BIT:
			lstrcpy(sz, nRow ? "1" : "0");
			break;
			

		case SQL_DATE: {
			char buf[30];

			sprintf(buf, "1991-10-%02d", nRow + 1);

			lstrcpy(sz, buf);

			break;
		}
		
		case SQL_TIMESTAMP: {
			UWORD day;
			UWORD hour = 0;
			UWORD minute = 0;
			UWORD second = 0;
			UDWORD fraction = 0;
			char buf[30];
		
			switch(ColInfo->precision) {
				case 21:
				case 22:
				case 23:
				case 24:
				case 25:
				case 26:
				case 27:
				case 28:
				case 29:
					fraction = nRow + 1;
				case 19:
					second = nRow + 1;
				case 16:
					minute = nRow + 1;
				case 13:
					hour = nRow + 1;
				case 10:
					day = nRow + 1;
					break;
				default:
					day = 0;
				}

			if(fraction != 0)
				sprintf(buf, "1991-10-%02d %02d:%02d:%02d.%0*d0", day, hour,
					minute, second, ColInfo->scale - 1, fraction);
			else
				sprintf(buf, "1991-10-%02d %02d:%02d:%02d", day, hour,
					minute, second);
			lstrcpy(sz, buf);
			break;
		}

		case SQL_TIME: {
			UWORD hour = 0;
			UWORD minute = 0;
			UWORD second = 0;
			UDWORD fraction = 0;
			char buf[30];
		
			switch(ColInfo->precision) {
				case 18:
				case 17:
				case 16:
				case 15:
				case 14:
				case 12:
				case 11:
				case 10:
					fraction = nRow + 1;
				case 8:
					second = nRow + 1;
				case 5:
					minute = nRow + 1;
				case 2:
					hour = nRow + 1;
					break;
				default:
					hour = 0;
				}

			if(fraction != 0)
				sprintf(buf, "%02d:%02d:%02d.%0*d0", hour,
					minute, second, ColInfo->scale - 1, fraction);
			else
				sprintf(buf, "%02d:%02d:%02d", hour,
					minute, second, ColInfo->scale - 1);
			lstrcpy(sz, buf);
			break;
		}
	}

	/* make strings one smaller if they are less than 10 so we don't
		end up with dupes on multiples of 11 */
	if(nRow < 10 && (TypeInfo->wSQLType == SQL_CHAR ||
		TypeInfo->wSQLType == SQL_VARCHAR))
		sz[lstrlen(sz) - 1] = '\0';
	
	pch = sz;
	if( !nRow )
		switch( TypeInfo->wSQLType ) {
			case SQL_SMALLINT:
			case SQL_TINYINT:
			case SQL_INTEGER:
				lstrcpy( sz, "0" );
				break;
			case SQL_DECIMAL:
				pch = (char*)(_fstrchr(sz, '.') - 1);
				break;
			case SQL_VARBINARY:
			case SQL_LONGVARBINARY:
				lstrcpy( sz, "00" );
				break;
		}

	return pch;
}

pascal int MakeTable( HDBC hdbc0, char FAR * szCreatedTable, 
	TYPENUM FAR *rgTypes, COLDATA FAR *rgCols )
{
	HSTMT           hstmt0;
	RETCODE         returncode;
	UWORD           i;
	UWORD           w;
	SDWORD          sdw;
	UWORD           cTypes;
	UWORD           ind;
	PSTRUCT FAR    *lpParams;
	char FAR       *pch;
	int             fColNames;
	int             fTableExists = 0;
	int             cTableName;
	int             cFieldName;
	char	        szNum[ 30 ];
	QTSTRUCT FAR   *lpqt;

	lpqt = (QTSTRUCT FAR *)LockMemory( AllocateMemory( sizeof( QTSTRUCT ) ) );
	lpParams = (PSTRUCT FAR *)LockMemory( AllocateMemory( sizeof( PSTRUCT ) ) );

	FUNCCALL( sz_SQLAllocStmt );
	returncode = SQLAllocStmt( hdbc0, &hstmt0 );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet;
	}
	
	
/* Get the type information so we can use it in a create statement for a
	table */

	FUNCCALL( sz_SQLGetInfo);
	returncode = SQLGetInfo(hdbc0, SQL_MAX_COLUMN_NAME_LEN, &cFieldName,
		sizeof cFieldName, NULL);

	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		cFieldName = 10;
	}
	if(cFieldName > MAX_TYPE)
		cFieldName = MAX_TYPE;
		
	FUNCCALL( sz_SQLGetTypeInfo);
	returncode = SQLGetTypeInfo(hstmt0, SQL_ALL_TYPES);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet;
	}
		
	INITLOOP;
	for( i = 0; i < MAX_TYPES; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS )
			break;

		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

		*rgTypes[i].szType = *rgTypes[i].szLength = *rgTypes[i].szPrefix
			= *rgTypes[i].szSuffix = *rgTypes[i].szParams = '\0';

		rgTypes[i].iFieldNum = i;
		 
		/* get type name */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 1, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szType, MAX_TYPE, &sdw );
		wsprintf(rgTypes[i].szFieldName,"c%02u%s", i,
			(char FAR *)rgTypes[i].szType);

			/* null terminate at max length */
		rgTypes[i].szFieldName[cFieldName] = NULL;

			/* change spaces in type name to '_' */
		while( (pch = (char*)(_fstrchr( rgTypes[i].szFieldName, ' ' ) ) ) != NULL )
			*pch = '_';
		
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

			/* get the SQLType */
		FUNCCALL( sz_SQLGetData );
        returncode = SQLGetData( hstmt0, 2, SQL_C_DEFAULT,
			(SWORD FAR *)&rgTypes[i].wSQLType, 100, 
			&sdw );

		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

			/* Max length */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 3, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szLength, MAX_LENGTH_BUF, &sdw );

		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

			/* don't actually use literal prefix or suffix, 
				using Prepare/ Execute with params */

			/* literal prefix */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 4, SQL_C_CHAR, 
			(char FAR *)rgTypes[i].szPrefix, MAX_PREFIX, &sdw );
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

			/* literal suffix */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 5, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szSuffix, MAX_SUFFIX, &sdw );
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

			/* create params */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 6, SQL_C_CHAR,
			(char FAR *)rgTypes[i].szParams, MAX_PARAMS, &sdw );
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;

			/* nullable */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 7, SQL_C_CHAR, (char FAR *)szNum, 100,
			&sdw );
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;
		rgTypes[i].nullable = atoi( szNum );

			/* searchable */
		FUNCCALL( sz_SQLGetData);
		returncode = SQLGetData( hstmt0, 9, SQL_C_CHAR, (char FAR *)szNum, 100,
			&sdw );
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if( returncode != SQL_SUCCESS )
			goto ErrorRet;
		rgTypes[i].searchable = atoi(szNum);

// If a line below is compiled in, that type will not be used in creating
// the table.  (useful for when one specific type is causing problems in
// a particular test, this way the test can be run for all other types)

/*
			if( !lstrcmp( rgTypes[i].szType, "LONG RAW"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "datetime"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "smalldatetime"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "money"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "smallmoney"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "varchar"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "varbinary"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "bit"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "binary"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "char"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "float"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "real"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "int"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "smallint"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "text"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "tinyint"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "image"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "sysname"))
				i--;
			if( !lstrcmp( rgTypes[i].szType, "timestamp"))
				i--;
*/
	}

	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
	if(SQL_NO_DATA_FOUND != returncode ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_NO_DATA_FOUND );
	}

	FUNCCALL( sz_SQLFreeStmt );
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
	
	FUNCCALL( sz_SQLGetInfo);
	returncode = SQLGetInfo(hdbc0, SQL_MAX_TABLE_NAME_LEN, &cTableName,
		sizeof (int), NULL);
	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

	if( returncode != SQL_SUCCESS && returncode != SQL_SUCCESS_WITH_INFO) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	
	if(cTableName > TABLENAMESIZE)
		cTableName = TABLENAMESIZE;
	wsprintf(lpqt->szTableName, "a%ld", hstmt0);
	lpqt->szTableName[cTableName] = '\0';

		/* build create statement */
		/* the column names will be ctypname (where typename is the
			type name returned by the source in SQLGetTypeInfo) */

	*lpqt->sz = '\0';
	cTypes = i;
	lstrcpy( lpqt->sz, "Create Table " );
	lstrcat( lpqt->sz, lpqt->szTableName );
	lstrcat( lpqt->sz, " (");
	for( i = 0; i < cTypes; i++ ) {
		*lpqt->szParam = '\0';
		if( _fstrstr( rgTypes[i].szParams, "max length") ||
			_fstrstr( rgTypes[i].szParams, "Max Length") ||
			_fstrstr( rgTypes[i].szParams, "MAX LENGTH") ||
			_fstrstr( rgTypes[i].szParams, "width") ||
			_fstrstr( rgTypes[i].szParams, "Width") ||
			_fstrstr( rgTypes[i].szParams, "WIDTH"))
			wsprintf( lpqt->szParam, "(%s)", (char FAR *)rgTypes[i].szLength);
		if(_fstrstr( rgTypes[i].szParams, "precision, scale") ||
			_fstrstr( rgTypes[i].szParams, "Precision, Scale") ||
			_fstrstr( rgTypes[i].szParams, "PRECISION, SCALE")) {
			rgTypes[i].precision = 5;
			rgTypes[i].scale = 5;
			wsprintf( lpqt->szParam, "(%d, %d)",
				rgTypes[i].precision, rgTypes[i].scale );
		}
		wsprintf( &lpqt->sz[lstrlen(lpqt->sz)], " %s %s%s, ",
			(char FAR *)rgTypes[i].szFieldName,
			(char FAR *)rgTypes[i].szType, lpqt->szParam );
	}

		/* remove the last comma and space */
	lpqt->sz[ lstrlen( lpqt->sz) - 2 ] = '\0';
	lstrcat( lpqt->sz, ")");
	szWrite( lpqt->sz, TRUE );

	FUNCCALL( sz_SQLExecDirect );
	returncode = SQLExecDirect( hstmt0, lpqt->sz, SQL_NTS );
	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
		goto ErrorRet;
	}

	fTableExists = TRUE;

		/* now that table was created, call SQLColumns.  Both as
			a test and to get necessary information to insert data */

	FUNCCALL( sz_SQLColumns);
	returncode = SQLColumns(hstmt0, NULL, SQL_NTS, NULL, SQL_NTS,
		lpqt->szTableName, SQL_NTS, NULL, SQL_NTS );

	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

	INITLOOP;
	for( i = 0; i < MAX_TYPES; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLFetch);
		returncode = SQLFetch(hstmt0);
		if( returncode != SQL_SUCCESS )
			break;

		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

		/* precision */
		strcpy(szNum, "0");

		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData( hstmt0, 7, SQL_C_CHAR, szNum, 100, 
			&sdw)) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		rgCols[i].precision = atol(szNum);

		strcpy(szNum, "0");

		/* length */
		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData( hstmt0, 8, SQL_C_CHAR, szNum, 100, 
			&sdw)) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

		if(rgCols[i].precision == 0)
			rgCols[i].precision = atol(szNum);
		else
			rgCols[i].length = atol(szNum);

		strcpy(szNum, "0");

		/* numeric scale */
		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData( hstmt0, 9, SQL_C_CHAR, szNum, 100, 
			&sdw)) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		rgCols[i].scale = atoi(szNum);

		strcpy(szNum, "0");

		/* nullable */
		FUNCCALL( sz_SQLGetData);
		if(SQL_SUCCESS != SQLGetData( hstmt0, 11, SQL_C_CHAR, szNum, 100, 
			&sdw)) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		rgCols[i].nullable = atoi(szNum);
	}

	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
 	if(SQL_NO_DATA_FOUND != returncode ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_NO_DATA_FOUND);
		goto ErrorRet;
	}
	
	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}

	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

	/* look for columns the test should not try to update by running
		a select * query and calling ColAttributes on the result fields */

	wsprintf(lpqt->sz, "select * from %s where 1 = 2", lpqt->szTableName);

	szWrite(lpqt->sz, TRUE );

	FUNCCALL( sz_SQLExecDirect);
	returncode = SQLExecDirect(hstmt0, lpqt->sz, SQL_NTS);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

	INITLOOP;
	for( i = 1; i <= cTypes; i++) {
		ITERATION(i);
		FUNCCALL( sz_SQLColAttributes);
		returncode = SQLColAttributes(hstmt0, i, SQL_COLUMN_UPDATABLE, NULL,
			0, NULL, &sdw );

		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
			
		rgCols[i - 1].fAutoUpdate = sdw == SQL_ATTR_READONLY;

		FUNCCALL( sz_SQLColAttributes);
		returncode = SQLColAttributes(hstmt0, i, SQL_COLUMN_SEARCHABLE, NULL,
			0, NULL, &sdw );

		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
			
		rgCols[i - 1].fSearchable = sdw;
	}

	FUNCCALL( sz_SQLFreeStmt);
	returncode = SQLFreeStmt( hstmt0, SQL_CLOSE);
	if( returncode != SQL_SUCCESS ) {
		DisplayRetcode( TRUE, returncode );
		DisplayRetcode( FALSE, SQL_SUCCESS );
	}
	fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

	/* put together the insert statement, and set the parameters */
	/* parameters are only set the first time through, after which
		the contents of the pointers is changed, using the prepare
		setparam method */

	*lpqt->szColNames = NULL;
	*lpqt->szValues = NULL;
	fColNames = TRUE;

	INITLOOP;
	for( i = 0; i <= cTypes; i++) {
		ITERATION(i);
		for( ind = 0, w = 0; ind < cTypes; ind++) {
			pch = MakeData(i, &rgTypes[ind], &rgCols[ind],
				lpqt->szDataItem);

			if(!pch) /* current type is REAL_ONLY) */
				continue;
			
			w++;
			/* for every nullable type, that field will be set to
			null when the row number corresponds with the field
			number */

			if(*pch) {
				lstrcpy(&lpParams->pbs[w][0], pch);
				lpParams->cbValue[w] = SQL_NTS;
			}
			else {
				lpParams->pbs[w][0] = '\0';
				lpParams->cbValue[w] = SQL_NULL_DATA;
			}

			if(fColNames) { /* the first time throught, the insert
				statement itself is created */
				lstrcat(lpqt->szColNames, rgTypes[ind].szFieldName);
				lstrcat(lpqt->szColNames, ", ");
				lstrcat(lpqt->szValues, " ?, ");

				/* and parameters are set */
				/* set all the parameters using pointers to buffers in
				the param struct. */

				FUNCCALL( sz_SQLSetParam);
				returncode = SQLSetParam( hstmt0, w,
					SQL_C_CHAR, rgTypes[ind].wSQLType,
					rgCols[ind].precision, rgCols[ind].scale,
					&lpParams->pbs[w][0],
					&lpParams->cbValue[w] );
				if( returncode != SQL_SUCCESS ) {
					DisplayRetcode( TRUE, returncode );
					DisplayRetcode( FALSE, SQL_SUCCESS );
					goto ErrorRet;
				}
				fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
			}
		}

		if( fColNames ) { /* the first time through, the insert
			statement needs to be SQLPrepare'd */
				/* remove the ", " at the end of the string */
			lpqt->szColNames[ lstrlen( lpqt->szColNames ) - 2 ] = NULL;
			lpqt->szValues[   lstrlen( lpqt->szValues    ) - 2] = NULL;

			wsprintf( lpqt->sz, "insert into %s (%s) VALUES(%s)",
				lpqt->szTableName, lpqt->szColNames, lpqt->szValues );
			szWrite( lpqt->sz, TRUE );

			FUNCCALL( sz_SQLPrepare );
			returncode = SQLPrepare( hstmt0, lpqt->sz, SQL_NTS );
			fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
			if( returncode != SQL_SUCCESS ) {
				DisplayRetcode( TRUE, returncode );
				DisplayRetcode( FALSE, SQL_SUCCESS );
				goto ErrorRet;
			}

			fColNames = FALSE; /* no more first times through */

		}

		FUNCCALL( sz_SQLExecute); /* this inserts a row */
		returncode = SQLExecute(hstmt0);
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
			goto ErrorRet;
		}
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

		FUNCCALL( sz_SQLRowCount);
		returncode = SQLRowCount(hstmt0, &sdw );
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		if(sdw != 1) { /* the execute inserted 1 row */
			szWrite(szIndent, FALSE );
			szWrite( "Failed: RowCount", TRUE );
		}


			/* FreeStmt SQL_CLOSE */
		FUNCCALL( sz_SQLFreeStmt );
		returncode = SQLFreeStmt( hstmt0, SQL_CLOSE );
		if( returncode != SQL_SUCCESS ) {
			DisplayRetcode( TRUE, returncode );
			DisplayRetcode( FALSE, SQL_SUCCESS );
		}
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );

	}

	SQLFreeStmt( hstmt0, SQL_DROP );

	lstrcpy(szCreatedTable, lpqt->szTableName);
	szWrite( "Table sucessfully created", TRUE );
	ReleaseMemory(lpqt);
	ReleaseMemory(lpParams);
	
	return cTypes;

	ErrorRet:
		ReleaseMemory( lpqt );
		ReleaseMemory( lpParams );
		fStateCheck( NULL, hdbc0, hstmt0, sz_00000 );
		SQLFreeStmt( hstmt0, SQL_DROP );
		if( fTableExists ){
			szWrite( "MakeTable function unable to complete",TRUE );
			return 0;
		}
		else{
			szWrite( "Table creation failed", TRUE );
			return -1;
		}
}
