
	/*****************************************************************
	 ODBCSTR.H - ODBC string declarations
	
		Each ODBC function name, manifest constant name, and SQLSTATE string
		is available as a global BYTE FAR array, for example, the string
		"SQLAllocConnect" is available under the name 'sz_SQLAllocConnect',
		"SQL_ERROR" is available as 'sz_SQL_ERROR', and "IM001" is available
		as 'sz_IM001'. The strings are defined in auto.c (auto.obj), which
		defines the symbol ODBC_STRING_DEFINITIONS before including this
		header file, so that the following macro calls produce global
		BYTE array definitions, for example:
	
			BYTE CSCONST sz_SQLAllocConnect[] = "SQLAllocConnect";
	
		Normally, the following macro calls produce external declarations,
		for example:
	
			extern BYTE FAR sz_SQLAllocConnect[];
	
			 (C) 1990-1992 Microsoft Corporation.  All right reserved.
	
	*********************************************************************/
#ifndef NO_ODBC_STRINGS

#ifdef ODBC_STRING_DEFINITIONS
//#define DeclareString(name) BYTE CSCONST sz_ ## name[] = #name;
#else
//#define DeclareString(name) extern BYTE FAR sz_ ## name[];
#endif

	/*
	** Named constants from sql.h:
	*/

	/* generally useful constants */
BYTE sz_SQL_NTS[]					= "SQL_NTS";					/* NTS = Null Terminated String  */
BYTE sz_SQL_SQLSTATE_SIZE[]			= "SQL_SQLSTATE_SIZE";		/* size of SQLSTATE              */
BYTE sz_SQL_MAX_MESSAGE_LENGTH[]	= "SQL_MAX_MESSAGE_LENGTH";	/* message buffer size           */
BYTE sz_SQL_MAX_DSN_LENGTH[]		= "SQL_MAX_DSN_LENGTH";		/* maximum data source name size */

	/* RETCODEs */
BYTE sz_SQL_ERROR[]	= "SQL_ERROR";
BYTE sz_SQL_INVALID_HANDLE[]	= "SQL_INVALID_HANDLE";
BYTE sz_SQL_NEED_DATA[]			= "SQL_NEED_DATA";
BYTE sz_SQL_NO_DATA_FOUND[]		= "SQL_NO_DATA_FOUND";
BYTE sz_SQL_SUCCESS[]			= "SQL_SUCCESS";
BYTE sz_SQL_SUCCESS_WITH_INFO[]	= "SQL_SUCCESS_WITH_INFO";

	/* SQLFreeStmt defines */
BYTE sz_SQL_CLOSE[]			= "SQL_CLOSE";
BYTE sz_SQL_DROP[]			= "SQL_DROP";
BYTE sz_SQL_UNBIND[]		= "SQL_UNBIND";
BYTE sz_SQL_RESET_PARAMS[]	= "SQL_RESET_PARAMS";

	/* SQLSetParam defines */
BYTE sz_SQL_C_DEFAULT[]		= "SQL_C_DEFAULT";

	/* SQLTransact defines */
BYTE sz_SQL_COMMIT[]		= "SQL_COMMIT";
BYTE sz_SQL_ROLLBACK[]		= "SQL_ROLLBACK";

	/* Standard SQL datatypes, using ANSI type numbering */
BYTE sz_SQL_CHAR[]		= "SQL_CHAR";
BYTE sz_SQL_NUMERIC[]	= "SQL_NUMERIC";
BYTE sz_SQL_DECIMAL[]	= "SQL_DECIMAL";
BYTE sz_SQL_INTEGER[]	= "SQL_INTEGER";
BYTE sz_SQL_SMALLINT[]	= "SQL_SMALLINT";
BYTE sz_SQL_FLOAT[]		= "SQL_FLOAT";
BYTE sz_SQL_REAL[]		= "SQL_REAL";
BYTE sz_SQL_DOUBLE[]	= "SQL_DOUBLE";
BYTE sz_SQL_VARCHAR[]	= "SQL_VARCHAR";

BYTE sz_SQL_TYPE_MIN[]	= "SQL_TYPE_MIN";
BYTE sz_SQL_TYPE_NULL[]	= "SQL_TYPE_NULL";
BYTE sz_SQL_TYPE_MAX[]	= "SQL_TYPE_MAX";

	/* C datatype to SQL datatype mapping    SQL types 
                                         ----------------------- */
BYTE sz_SQL_C_CHAR[]	= "SQL_C_CHAR";		/* CHAR, VARCHAR, DECIMAL, NUMERIC */
BYTE sz_SQL_C_LONG[]	= "SQL_C_LONG";		/* INTEGER          */
BYTE sz_SQL_C_SHORT[]	= "SQL_C_SHORT";	/* SMALLINT         */
BYTE sz_SQL_C_FLOAT[]	= "SQL_C_FLOAT";	/* REAL             */
BYTE sz_SQL_C_DOUBLE[]	= "SQL_C_DOUBLE";	/* FLOAT, DOUBLE    */

	/* NULL status constants.  These are used in SQLColumns, SQLColAttributes,
SQLDescribeCol, and SQLSpecialColumns to describe the nullablity of a
column in a table.  SQL_NULLABLE_UNKNOWN can be returned only by 
SQLDescribeCol or SQLColAttributes.  It is used when the DBMS's meta-data
does not contain this info.  */
BYTE sz_SQL_NO_NULLS[]			= "SQL_NO_NULLS";
BYTE sz_SQL_NULLABLE[]			= "SQL_NULLABLE";
BYTE sz_SQL_NULLABLE_UNKNOWN[]	= "SQL_NULLABLE_UNKNOWN";

	/* Special length values */
BYTE sz_SQL_NULL_DATA[]		= "SQL_NULL_DATA";
BYTE sz_SQL_DATA_AT_EXEC[]	= "SQL_DATA_AT_EXEC";

	/* SQLColAttributes defines */
BYTE sz_SQL_COLUMN_COUNT[]			= "SQL_COLUMN_COUNT";
BYTE sz_SQL_COLUMN_NAME[]			= "SQL_COLUMN_NAME";
BYTE sz_SQL_COLUMN_TYPE[]			= "SQL_COLUMN_TYPE";
BYTE sz_SQL_COLUMN_LENGTH[]			= "SQL_COLUMN_LENGTH";
BYTE sz_SQL_COLUMN_PRECISION[]		= "SQL_COLUMN_PRECISION";
BYTE sz_SQL_COLUMN_SCALE[]			= "SQL_COLUMN_SCALE";
BYTE sz_SQL_COLUMN_DISPLAY_SIZE[]	= "SQL_COLUMN_DISPLAY_SIZE";
BYTE sz_SQL_COLUMN_NULLABLE[]		= "SQL_COLUMN_NULLABLE";
BYTE sz_SQL_COLUMN_UNSIGNED[]		= "SQL_COLUMN_UNSIGNED";
BYTE sz_SQL_COLUMN_MONEY[]			= "SQL_COLUMN_MONEY";
BYTE sz_SQL_COLUMN_UPDATABLE[]		= "SQL_COLUMN_UPDATABLE";
BYTE sz_SQL_COLUMN_AUTO_INCREMENT[]	= "SQL_COLUMN_AUTO_INCREMENT";
BYTE sz_SQL_COLUMN_CASE_SENSITIVE[]	= "SQL_COLUMN_CASE_SENSITIVE";
BYTE sz_SQL_COLUMN_SEARCHABLE[]		= "SQL_COLUMN_SEARCHABLE";
BYTE sz_SQL_COLUMN_TYPE_NAME[]		= "SQL_COLUMN_TYPE_NAME";

	/* SQLColAttributes subdefines for SQL_COLUMN_UPDATABLE */
BYTE sz_SQL_ATTR_READONLY[]				= "SQL_ATTR_READONLY";
BYTE sz_SQL_ATTR_WRITE[]				= "SQL_ATTR_WRITE";
BYTE sz_SQL_ATTR_READWRITE_UNKNOWN[]	= "SQL_ATTR_READWRITE_UNKNOWN";

	/* SQLColAttributes subdefines for SQL_COLUMN_SEARCHABLE */
	/* These are also used by SQLGetInfo                     */
BYTE sz_SQL_UNSEARCHABLE[]		= "SQL_UNSEARCHABLE";
BYTE sz_SQL_LIKE_ONLY[]			= "SQL_LIKE_ONLY";
BYTE sz_SQL_ALL_EXCEPT_LIKE[]	= "SQL_ALL_EXCEPT_LIKE";
BYTE sz_SQL_SEARCHABLE[]		= "SQL_SEARCHABLE";

	/* SQLError defines */
BYTE sz_SQL_NULL_HENV[]			= "SQL_NULL_HENV";
BYTE sz_SQL_NULL_HDBC[]			= "SQL_NULL_HDBC";
BYTE sz_SQL_NULL_HSTMT[]		= "SQL_NULL_HSTMT";

	/* environment specific definitions */
BYTE sz_SQL_API[]				= "SQL_API";

	/* Core Function Prototypes */
BYTE sz_SQLAllocConnect[]	= "SQLAllocConnect";
BYTE sz_SQLAllocEnv[]		= "SQLAllocEnv";
BYTE sz_SQLAllocStmt[]		= "SQLAllocStmt";
BYTE sz_SQLBindCol[]		= "SQLBindCol";
BYTE sz_SQLCancel[]			= "SQLCancel";
BYTE sz_SQLColAttributes[]	= "SQLColAttributes";
BYTE sz_SQLConnect[]		= "SQLConnect";
BYTE sz_SQLDescribeCol[]	= "SQLDescribeCol";
BYTE sz_SQLDisconnect[]		= "SQLDisconnect";
BYTE sz_SQLError[]			= "SQLError";
BYTE sz_SQLExecDirect[]		= "SQLExecDirect";
BYTE sz_SQLExecute[]		= "SQLExecute";
BYTE sz_SQLFetch[]			= "SQLFetch";
BYTE sz_SQLFreeConnect[]	= "SQLFreeConnect";
BYTE sz_SQLFreeEnv[]		= "SQLFreeEnv";
BYTE sz_SQLFreeStmt[]		= "SQLFreeStmt";
BYTE sz_SQLGetCursorName[]	= "SQLGetCursorName";
BYTE sz_SQLNumResultCols[]	= "SQLNumResultCols";
BYTE sz_SQLPrepare[]		= "SQLPrepare";
BYTE sz_SQLRowCount[]		= "SQLRowCount";
BYTE sz_SQLSetCursorName[]	= "SQLSetCursorName";
BYTE sz_SQLSetParam[]		= "SQLSetParam";
BYTE sz_SQLTransact[]		= "SQLTransact";

	/*
	** Named constants from sqlext.h:
	*/

	/* Defines used by both Level 1 and Level 2 functions */

	/* Additional return codes */
BYTE sz_SQL_STILL_EXECUTING[]	= "SQL_STILL_EXECUTING";

	/* SQL extended datatypes */
BYTE sz_SQL_DATE[]			= "SQL_DATE";
BYTE sz_SQL_TIME[]			= "SQL_TIME";
BYTE sz_SQL_TIMESTAMP[]		= "SQL_TIMESTAMP";
BYTE sz_SQL_LONGVARCHAR[]	= "SQL_LONGVARCHAR";
BYTE sz_SQL_BINARY[]		= "SQL_BINARY";
BYTE sz_SQL_VARBINARY[]		= "SQL_VARBINARY";
BYTE sz_SQL_LONGVARBINARY[]	= "SQL_LONGVARBINARY";
BYTE sz_SQL_BIGINT[]		= "SQL_BIGINT";
BYTE sz_SQL_TINYINT[]		= "SQL_TINYINT";
BYTE sz_SQL_BIT[]			= "SQL_BIT";

	/* C datatype to SQL datatype mapping */
BYTE sz_SQL_C_DATE[]		= "SQL_C_DATE";
BYTE sz_SQL_C_TIME[]		= "SQL_C_TIME";
BYTE sz_SQL_C_TIMESTAMP[]	= "SQL_C_TIMESTAMP";
BYTE sz_SQL_C_BINARY[]		= "SQL_C_BINARY";
BYTE sz_SQL_C_BIT[]			= "SQL_C_BIT";
BYTE sz_SQL_C_TINYINT[]		= "SQL_C_TINYINT";

	/* from SQL.h -- extended data types change this */
	/*        (sz_SQL_TYPE_MIN was defined above) */
BYTE sz_SQL_ALL_TYPES[]		= "SQL_ALL_TYPES";



	/* Level 1 Functions                             */

	/* Access modes                 */
BYTE sz_SQL_MODE_READ_WRITE[]	= "SQL_MODE_READ_WRITE";
BYTE sz_SQL_MODE_READ_ONLY[]	= "SQL_MODE_READ_ONLY";

	/* Options for SQLDriverConnect */
BYTE sz_SQL_DRIVER_NOPROMPT[]			= "SQL_DRIVER_NOPROMPT";
BYTE sz_SQL_DRIVER_COMPLETE[]			= "SQL_DRIVER_COMPLETE";
BYTE sz_SQL_DRIVER_PROMPT[]				= "SQL_DRIVER_PROMPT";
BYTE sz_SQL_DRIVER_COMPLETE_REQUIRED[]	= "SQL_DRIVER_COMPLETE_REQUIRED";

	/* Special return values for SQLGetData */
BYTE sz_SQL_NO_TOTAL[]				= "SQL_NO_TOTAL";

	/* Defines for SQLGetFunctions */
BYTE sz_SQL_API_SQLALLOCCONNECT[]	= "SQL_API_SQLALLOCCONNECT";		/* Core Functions           */
BYTE sz_SQL_API_SQLALLOCENV[]		= "SQL_API_SQLALLOCENV";
BYTE sz_SQL_API_SQLALLOCSTMT[]		= "SQL_API_SQLALLOCSTMT";
BYTE sz_SQL_API_SQLBINDCOL[]		= "SQL_API_SQLBINDCOL";
BYTE sz_SQL_API_SQLCANCEL[]			= "SQL_API_SQLCANCEL";
BYTE sz_SQL_API_SQLCOLATTRIBUTES[]	= "SQL_API_SQLCOLATTRIBUTES";
BYTE sz_SQL_API_SQLCONNECT[]		= "SQL_API_SQLCONNECT";
BYTE sz_SQL_API_SQLDESCRIBECOL[]	= "SQL_API_SQLDESCRIBECOL";
BYTE sz_SQL_API_SQLDISCONNECT[]		= "SQL_API_SQLDISCONNECT";
BYTE sz_SQL_API_SQLERROR[]			= "SQL_API_SQLERROR";
BYTE sz_SQL_API_SQLEXECDIRECT[]		= "SQL_API_SQLEXECDIRECT";
BYTE sz_SQL_API_SQLEXECUTE[]		= "SQL_API_SQLEXECUTE";
BYTE sz_SQL_API_SQLFETCH[]			= "SQL_API_SQLFETCH";
BYTE sz_SQL_API_SQLFREECONNECT[]	= "SQL_API_SQLFREECONNECT";
BYTE sz_SQL_API_SQLFREEENV[]		= "SQL_API_SQLFREEENV";
BYTE sz_SQL_API_SQLFREESTMT[]		= "SQL_API_SQLFREESTMT";
BYTE sz_SQL_API_SQLGETCURSORNAME[]	= "SQL_API_SQLGETCURSORNAME";
BYTE sz_SQL_API_SQLNUMRESULTCOLS[]	= "SQL_API_SQLNUMRESULTCOLS";
BYTE sz_SQL_API_SQLPREPARE[]		= "SQL_API_SQLPREPARE";
BYTE sz_SQL_API_SQLROWCOUNT[]		= "SQL_API_SQLROWCOUNT";
BYTE sz_SQL_API_SQLSETCURSORNAME[]	= "SQL_API_SQLSETCURSORNAME";
BYTE sz_SQL_API_SQLSETPARAM[]		= "SQL_API_SQLSETPARAM";
BYTE sz_SQL_API_SQLTRANSACT[]		= "SQL_API_SQLTRANSACT";

BYTE sz_SQL_NUM_FUNCTIONS[]			= "SQL_NUM_FUNCTIONS";

BYTE sz_SQL_EXT_API_START[]			= "SQL_EXT_API_START";

BYTE sz_SQL_API_SQLCOLUMNS[]			= "SQL_API_SQLCOLUMNS";			/* Level 1 Functions        */
BYTE sz_SQL_API_SQLDRIVERCONNECT[]		= "SQL_API_SQLDRIVERCONNECT";
BYTE sz_SQL_API_SQLGETCONNECTOPTION[]	= "SQL_API_SQLGETCONNECTOPTION";
BYTE sz_SQL_API_SQLGETDATA[]			= "SQL_API_SQLGETDATA";
BYTE sz_SQL_API_SQLGETFUNCTIONS[]		= "SQL_API_SQLGETFUNCTIONS";
BYTE sz_SQL_API_SQLGETINFO[]			= "SQL_API_SQLGETINFO";
BYTE sz_SQL_API_SQLGETSTMTOPTION[]		= "SQL_API_SQLGETSTMTOPTION";
BYTE sz_SQL_API_SQLGETTYPEINFO[]		= "SQL_API_SQLGETTYPEINFO";
BYTE sz_SQL_API_SQLPARAMDATA[]			= "SQL_API_SQLPARAMDATA";
BYTE sz_SQL_API_SQLPUTDATA[]			= "SQL_API_SQLPUTDATA";
BYTE sz_SQL_API_SQLSETCONNECTOPTION[]	= "SQL_API_SQLSETCONNECTOPTION";
BYTE sz_SQL_API_SQLSETSTMTOPTION[]		= "SQL_API_SQLSETSTMTOPTION";
BYTE sz_SQL_API_SQLSPECIALCOLUMNS[]		= "SQL_API_SQLSPECIALCOLUMNS";
BYTE sz_SQL_API_SQLSTATISTICS[]			= "SQL_API_SQLSTATISTICS";
BYTE sz_SQL_API_SQLTABLES[]				= "SQL_API_SQLTABLES";

BYTE sz_SQL_API_SQLBROWSECONNECT[]		= "SQL_API_SQLBROWSECONNECT"; 	/* Level 2 Functions        */
BYTE sz_SQL_API_SQLCOLUMNPRIVILEGES[]	= "SQL_API_SQLCOLUMNPRIVILEGES";
BYTE sz_SQL_API_SQLDATASOURCES[]		= "SQL_API_SQLDATASOURCES";
BYTE sz_SQL_API_SQLDESCRIBEPARAM[]		= "SQL_API_SQLDESCRIBEPARAM";
BYTE sz_SQL_API_SQLEXTENDEDFETCH[]		= "SQL_API_SQLEXTENDEDFETCH";
BYTE sz_SQL_API_SQLFOREIGNKEYS[]		= "SQL_API_SQLFOREIGNKEYS";
BYTE sz_SQL_API_SQLMORERESULTS[]		= "SQL_API_SQLMORERESULTS";
BYTE sz_SQL_API_SQLNATIVESQL[]			= "SQL_API_SQLNATIVESQL";
BYTE sz_SQL_API_SQLNUMPARAMS[]			= "SQL_API_SQLNUMPARAMS";
BYTE sz_SQL_API_SQLPARAMOPTIONS[]		= "SQL_API_SQLPARAMOPTIONS";
BYTE sz_SQL_API_SQLPRIMARYKEYS[]		= "SQL_API_SQLPRIMARYKEYS";
BYTE sz_SQL_API_SQLPROCEDURECOLUMNS[]	= "SQL_API_SQLPROCEDURECOLUMNS";
BYTE sz_SQL_API_SQLPROCEDURES[]			= "SQL_API_SQLPROCEDURES";
BYTE sz_SQL_API_SQLSETPOS[]				= "SQL_API_SQLSETPOS";
BYTE sz_SQL_API_SQLSETSCROLLOPTIONS[]	= "SQL_API_SQLSETSCROLLOPTIONS";
BYTE sz_SQL_API_SQLTABLEPRIVILEGES[]	= "SQL_API_SQLTABLEPRIVILEGES";

BYTE sz_SQL_EXT_API_LAST[]				= "SQL_EXT_API_LAST";

BYTE sz_SQL_NUM_EXTENSIONS[]			= "SQL_NUM_EXTENSIONS";

	/* Defines for SQLGetInfo */
BYTE sz_SQL_INFO_FIRST[]				= "SQL_INFO_FIRST";
BYTE sz_SQL_ACTIVE_CONNECTIONS[]		= "SQL_ACTIVE_CONNECTIONS";
BYTE sz_SQL_ACTIVE_STATEMENTS[]			= "SQL_ACTIVE_STATEMENTS";
BYTE sz_SQL_DATA_SOURCE_NAME[]			= "SQL_DATA_SOURCE_NAME";
BYTE sz_SQL_DRIVER_HDBC[]				= "SQL_DRIVER_HDBC";
BYTE sz_SQL_DRIVER_HENV[]				= "SQL_DRIVER_HENV";
BYTE sz_SQL_DRIVER_HSTMT[]				= "SQL_DRIVER_HSTMT";
BYTE sz_SQL_DRIVER_NAME[]				= "SQL_DRIVER_NAME";
BYTE sz_SQL_DRIVER_VER[]				= "SQL_DRIVER_VER";
BYTE sz_SQL_FETCH_DIRECTION[]			= "SQL_FETCH_DIRECTION";
BYTE sz_SQL_ODBC_API_CONFORMANCE[]		= "SQL_ODBC_API_CONFORMANCE";
BYTE sz_SQL_ODBC_VER[]					= "SQL_ODBC_VER";
BYTE sz_SQL_ROW_UPDATES[]				= "SQL_ROW_UPDATES";
BYTE sz_SQL_ODBC_SAG_CLI_CONFORMANCE[]	= "SQL_ODBC_SAG_CLI_CONFORMANCE";
BYTE sz_SQL_SERVER_NAME[]				= "SQL_SERVER_NAME";
BYTE sz_SQL_SEARCH_PATTERN_ESCAPE[]		= "SQL_SEARCH_PATTERN_ESCAPE";
BYTE sz_SQL_ODBC_SQL_CONFORMANCE[]		= "SQL_ODBC_SQL_CONFORMANCE";

BYTE sz_SQL_DATABASE_NAME[]				= "SQL_DATABASE_NAME";
BYTE sz_SQL_DBMS_NAME[]					= "SQL_DBMS_NAME";
BYTE sz_SQL_DBMS_VER[]					= "SQL_DBMS_VER";

BYTE sz_SQL_ACCESSIBLE_TABLES[]			= "SQL_ACCESSIBLE_TABLES";
BYTE sz_SQL_ACCESSIBLE_PROCEDURES[]		= "SQL_ACCESSIBLE_PROCEDURES";
BYTE sz_SQL_PROCEDURES[]				= "SQL_PROCEDURES";
BYTE sz_SQL_CONCAT_NULL_BEHAVIOR[]		= "SQL_CONCAT_NULL_BEHAVIOR";
BYTE sz_SQL_CURSOR_COMMIT_BEHAVIOR[]	= "SQL_CURSOR_COMMIT_BEHAVIOR";
BYTE sz_SQL_CURSOR_ROLLBACK_BEHAVIOR[]	= "SQL_CURSOR_ROLLBACK_BEHAVIOR";
BYTE sz_SQL_DATA_SOURCE_READ_ONLY[]		= "SQL_DATA_SOURCE_READ_ONLY";
BYTE sz_SQL_DEFAULT_TXN_ISOLATION[]		= "SQL_DEFAULT_TXN_ISOLATION";
BYTE sz_SQL_EXPRESSIONS_IN_ORDERBY[]	= "SQL_EXPRESSIONS_IN_ORDERBY";
BYTE sz_SQL_IDENTIFIER_CASE[]			= "SQL_IDENTIFIER_CASE";
BYTE sz_SQL_IDENTIFIER_QUOTE_CHAR[]		= "SQL_IDENTIFIER_QUOTE_CHAR";
BYTE sz_SQL_MAX_COLUMN_NAME_LEN[]		= "SQL_MAX_COLUMN_NAME_LEN";
BYTE sz_SQL_MAX_CURSOR_NAME_LEN[]		= "SQL_MAX_CURSOR_NAME_LEN";
BYTE sz_SQL_MAX_OWNER_NAME_LEN[]		= "SQL_MAX_OWNER_NAME_LEN";
BYTE sz_SQL_MAX_PROCEDURE_NAME_LEN[]	= "SQL_MAX_PROCEDURE_NAME_LEN";
BYTE sz_SQL_MAX_QUALIFIER_NAME_LEN[]	= "SQL_MAX_QUALIFIER_NAME_LEN";
BYTE sz_SQL_MAX_TABLE_NAME_LEN[]		= "SQL_MAX_TABLE_NAME_LEN";
BYTE sz_SQL_MULT_RESULT_SETS[]			= "SQL_MULT_RESULT_SETS";
BYTE sz_SQL_MULTIPLE_ACTIVE_TXN[]		= "SQL_MULTIPLE_ACTIVE_TXN";
BYTE sz_SQL_OUTER_JOINS[]				= "SQL_OUTER_JOINS";
BYTE sz_SQL_OWNER_TERM[]				= "SQL_OWNER_TERM";
BYTE sz_SQL_PROCEDURE_TERM[]			= "SQL_PROCEDURE_TERM";
BYTE sz_SQL_QUALIFIER_NAME_SEPARATOR[]	= "SQL_QUALIFIER_NAME_SEPARATOR";
BYTE sz_SQL_QUALIFIER_TERM[]			= "SQL_QUALIFIER_TERM";
BYTE sz_SQL_SCROLL_CONCURRENCY[]		= "SQL_SCROLL_CONCURRENCY";
BYTE sz_SQL_SCROLL_OPTIONS[]			= "SQL_SCROLL_OPTIONS";
BYTE sz_SQL_TABLE_TERM[]				= "SQL_TABLE_TERM";
BYTE sz_SQL_TXN_CAPABLE[]				= "SQL_TXN_CAPABLE";
BYTE sz_SQL_USER_NAME[]					= "SQL_USER_NAME";

BYTE sz_SQL_CONVERT_FUNCTIONS[]			= "SQL_CONVERT_FUNCTIONS";
BYTE sz_SQL_NUMERIC_FUNCTIONS[]			= "SQL_NUMERIC_FUNCTIONS";
BYTE sz_SQL_STRING_FUNCTIONS[]			= "SQL_STRING_FUNCTIONS";
BYTE sz_SQL_SYSTEM_FUNCTIONS[]			= "SQL_SYSTEM_FUNCTIONS";
BYTE sz_SQL_TIMEDATE_FUNCTIONS[]		= "SQL_TIMEDATE_FUNCTIONS";

BYTE sz_SQL_CONVERT_BIGINT[]			= "SQL_CONVERT_BIGINT";
BYTE sz_SQL_CONVERT_BINARY[]			= "SQL_CONVERT_BINARY";
BYTE sz_SQL_CONVERT_BIT[]				= "SQL_CONVERT_BIT";
BYTE sz_SQL_CONVERT_CHAR[]				= "SQL_CONVERT_CHAR";
BYTE sz_SQL_CONVERT_DATE[]				= "SQL_CONVERT_DATE";
BYTE sz_SQL_CONVERT_DECIMAL[]			= "SQL_CONVERT_DECIMAL";
BYTE sz_SQL_CONVERT_DOUBLE[]			= "SQL_CONVERT_DOUBLE";
BYTE sz_SQL_CONVERT_FLOAT[]				= "SQL_CONVERT_FLOAT";
BYTE sz_SQL_CONVERT_INTEGER[]			= "SQL_CONVERT_INTEGER";
BYTE sz_SQL_CONVERT_LONGVARCHAR[]		= "SQL_CONVERT_LONGVARCHAR";
BYTE sz_SQL_CONVERT_NUMERIC[]			= "SQL_CONVERT_NUMERIC";
BYTE sz_SQL_CONVERT_REAL[]				= "SQL_CONVERT_REAL";
BYTE sz_SQL_CONVERT_SMALLINT[]			= "SQL_CONVERT_SMALLINT";
BYTE sz_SQL_CONVERT_TIME[]				= "SQL_CONVERT_TIME";
BYTE sz_SQL_CONVERT_TIMESTAMP[]			= "SQL_CONVERT_TIMESTAMP";
BYTE sz_SQL_CONVERT_TINYINT[]			= "SQL_CONVERT_TINYINT";
BYTE sz_SQL_CONVERT_VARBINARY[]			= "SQL_CONVERT_VARBINARY";
BYTE sz_SQL_CONVERT_VARCHAR[]			= "SQL_CONVERT_VARCHAR";
BYTE sz_SQL_CONVERT_LONGVARBINARY[]		= "SQL_CONVERT_LONGVARBINARY";

BYTE sz_SQL_TXN_ISOLATION_OPTION[]		= "SQL_TXN_ISOLATION_OPTION";
BYTE sz_SQL_ODBC_SQL_OPT_IEF[]			= "SQL_ODBC_SQL_OPT_IEF";

BYTE sz_SQL_INFO_LAST[]					= "SQL_INFO_LAST";
BYTE sz_SQL_INFO_DRIVER_START[]			= "SQL_INFO_DRIVER_START";

	/* "SQL_CONVERT_" return value bitmasks */

BYTE sz_SQL_CVT_CHAR[]				= "SQL_CVT_CHAR";
BYTE sz_SQL_CVT_NUMERIC[]			= "SQL_CVT_NUMERIC";
BYTE sz_SQL_CVT_DECIMAL[]			= "SQL_CVT_DECIMAL";
BYTE sz_SQL_CVT_INTEGER[]			= "SQL_CVT_INTEGER";
BYTE sz_SQL_CVT_SMALLINT[]			= "SQL_CVT_SMALLINT";
BYTE sz_SQL_CVT_FLOAT[]				= "SQL_CVT_FLOAT";
BYTE sz_SQL_CVT_REAL[]				= "SQL_CVT_REAL";
BYTE sz_SQL_CVT_DOUBLE[]			= "SQL_CVT_DOUBLE";
BYTE sz_SQL_CVT_VARCHAR[]			= "SQL_CVT_VARCHAR";
BYTE sz_SQL_CVT_LONGVARCHAR[]		= "SQL_CVT_LONGVARCHAR";
BYTE sz_SQL_CVT_BINARY[]			= "SQL_CVT_BINARY";
BYTE sz_SQL_CVT_VARBINARY[]			= "SQL_CVT_VARBINARY";
BYTE sz_SQL_CVT_BIT[]				= "SQL_CVT_BIT";
BYTE sz_SQL_CVT_TINYINT[]			= "SQL_CVT_TINYINT";
BYTE sz_SQL_CVT_BIGINT[]			= "SQL_CVT_BIGINT";
BYTE sz_SQL_CVT_DATE[]				= "SQL_CVT_DATE";
BYTE sz_SQL_CVT_TIME[]				= "SQL_CVT_TIME";
BYTE sz_SQL_CVT_TIMESTAMP[]			= "SQL_CVT_TIMESTAMP";
BYTE sz_SQL_CVT_LONGVARBINARY[]		= "SQL_CVT_LONGVARBINARY";

	/* Conversion functions */
BYTE sz_SQL_FN_CVT_CONVERT[]	= "SQL_FN_CVT_CONVERT";

	/* String functions */

BYTE sz_SQL_FN_STR_CONCAT[]	= "SQL_FN_STR_CONCAT";
BYTE sz_SQL_FN_STR_INSERT[]	= "SQL_FN_STR_INSERT";
BYTE sz_SQL_FN_STR_LEFT[]	= "SQL_FN_STR_LEFT";
BYTE sz_SQL_FN_STR_LTRIM[]	= "SQL_FN_STR_LTRIM";
BYTE sz_SQL_FN_STR_LENGTH[]	= "SQL_FN_STR_LENGTH";
BYTE sz_SQL_FN_STR_LOCATE[]	= "SQL_FN_STR_LOCATE";
BYTE sz_SQL_FN_STR_LCASE[]	= "SQL_FN_STR_LCASE";
BYTE sz_SQL_FN_STR_REPEAT[]	= "SQL_FN_STR_REPEAT";
BYTE sz_SQL_FN_STR_REPLACE[]	= "SQL_FN_STR_REPLACE";
BYTE sz_SQL_FN_STR_RIGHT[]	= "SQL_FN_STR_RIGHT";
BYTE sz_SQL_FN_STR_RTRIM[]	= "SQL_FN_STR_RTRIM";
BYTE sz_SQL_FN_STR_SUBSTRING[]	= "SQL_FN_STR_SUBSTRING";
BYTE sz_SQL_FN_STR_UCASE[]	= "SQL_FN_STR_UCASE";
BYTE sz_SQL_FN_STR_ASCII[]	= "SQL_FN_STR_ASCII";
BYTE sz_SQL_FN_STR_CHAR[]	= "SQL_FN_STR_CHAR";

	/* Numeric functions */

BYTE sz_SQL_FN_NUM_ABS[]	= "SQL_FN_NUM_ABS";
BYTE sz_SQL_FN_NUM_ACOS[]	= "SQL_FN_NUM_ACOS";
BYTE sz_SQL_FN_NUM_ASIN[]	= "SQL_FN_NUM_ASIN";
BYTE sz_SQL_FN_NUM_ATAN[]	= "SQL_FN_NUM_ATAN";
BYTE sz_SQL_FN_NUM_ATAN2[]	= "SQL_FN_NUM_ATAN2";
BYTE sz_SQL_FN_NUM_CEILING[]	= "SQL_FN_NUM_CEILING";
BYTE sz_SQL_FN_NUM_COS[]	= "SQL_FN_NUM_COS";
BYTE sz_SQL_FN_NUM_COT[]	= "SQL_FN_NUM_COT";
BYTE sz_SQL_FN_NUM_EXP[]	= "SQL_FN_NUM_EXP";
BYTE sz_SQL_FN_NUM_FLOOR[]	= "SQL_FN_NUM_FLOOR";
BYTE sz_SQL_FN_NUM_LOG[]	= "SQL_FN_NUM_LOG";
BYTE sz_SQL_FN_NUM_MOD[]	= "SQL_FN_NUM_MOD";
BYTE sz_SQL_FN_NUM_SIGN[]	= "SQL_FN_NUM_SIGN";
BYTE sz_SQL_FN_NUM_SIN[]	= "SQL_FN_NUM_SIN";
BYTE sz_SQL_FN_NUM_SQRT[]	= "SQL_FN_NUM_SQRT";
BYTE sz_SQL_FN_NUM_TAN[]	= "SQL_FN_NUM_TAN";
BYTE sz_SQL_FN_NUM_PI[]	= "SQL_FN_NUM_PI";
BYTE sz_SQL_FN_NUM_RAND[]	= "SQL_FN_NUM_RAND";

	/* Time/date functions */

BYTE sz_SQL_FN_TD_NOW[]	= "SQL_FN_TD_NOW";
BYTE sz_SQL_FN_TD_CURDATE[]	= "SQL_FN_TD_CURDATE";
BYTE sz_SQL_FN_TD_DAYOFMONTH[]	= "SQL_FN_TD_DAYOFMONTH";
BYTE sz_SQL_FN_TD_DAYOFWEEK[]	= "SQL_FN_TD_DAYOFWEEK";
BYTE sz_SQL_FN_TD_DAYOFYEAR[]	= "SQL_FN_TD_DAYOFYEAR";
BYTE sz_SQL_FN_TD_MONTH[]	= "SQL_FN_TD_MONTH";
BYTE sz_SQL_FN_TD_QUARTER[]	= "SQL_FN_TD_QUARTER";
BYTE sz_SQL_FN_TD_WEEK[]	= "SQL_FN_TD_WEEK";
BYTE sz_SQL_FN_TD_YEAR[]	= "SQL_FN_TD_YEAR";
BYTE sz_SQL_FN_TD_CURTIME[]	= "SQL_FN_TD_CURTIME";
BYTE sz_SQL_FN_TD_HOUR[]	= "SQL_FN_TD_HOUR";
BYTE sz_SQL_FN_TD_MINUTE[]	= "SQL_FN_TD_MINUTE";
BYTE sz_SQL_FN_TD_SECOND[]	= "SQL_FN_TD_SECOND";

	/* System functions */

BYTE sz_SQL_FN_SYS_USERNAME[]	= "SQL_FN_SYS_USERNAME";
BYTE sz_SQL_FN_SYS_DBNAME[]	= "SQL_FN_SYS_DBNAME";
BYTE sz_SQL_FN_SYS_IFNULL[]	= "SQL_FN_SYS_IFNULL";

	/* Scroll option masks */

BYTE sz_SQL_SO_FORWARD_ONLY[]	= "SQL_SO_FORWARD_ONLY";
BYTE sz_SQL_SO_KEYSET_DRIVEN[]	= "SQL_SO_KEYSET_DRIVEN";
BYTE sz_SQL_SO_DYNAMIC[]	= "SQL_SO_DYNAMIC";
BYTE sz_SQL_SO_MIXED[]	= "SQL_SO_MIXED";

	/* Scroll concurrency option masks */

BYTE sz_SQL_SCCO_READ_ONLY[]	= "SQL_SCCO_READ_ONLY";
BYTE sz_SQL_SCCO_LOCK[]	= "SQL_SCCO_LOCK";
BYTE sz_SQL_SCCO_OPT_TIMESTAMP[]	= "SQL_SCCO_OPT_TIMESTAMP";
BYTE sz_SQL_SCCO_OPT_VALUES[]	= "SQL_SCCO_OPT_VALUES";

	/* Fetch direction option masks */

BYTE sz_SQL_FD_FETCH_NEXT[]	= "SQL_FD_FETCH_NEXT";
BYTE sz_SQL_FD_FETCH_FIRST[]	= "SQL_FD_FETCH_FIRST";
BYTE sz_SQL_FD_FETCH_LAST[]	= "SQL_FD_FETCH_LAST";
BYTE sz_SQL_FD_FETCH_PREV[]	= "SQL_FD_FETCH_PREV";
BYTE sz_SQL_FD_FETCH_ABSOLUTE[]	= "SQL_FD_FETCH_ABSOLUTE";
BYTE sz_SQL_FD_FETCH_RELATIVE[]	= "SQL_FD_FETCH_RELATIVE";
BYTE sz_SQL_FD_FETCH_RESUME[]	= "SQL_FD_FETCH_RESUME";

	/* Transaction isolation option masks */

BYTE sz_SQL_TXN_READ_UNCOMMITTED[]	= "SQL_TXN_READ_UNCOMMITTED";
BYTE sz_SQL_TXN_READ_COMMITTED[]	= "SQL_TXN_READ_COMMITTED";
BYTE sz_SQL_TXN_REPEATABLE_READ[]	= "SQL_TXN_REPEATABLE_READ";
BYTE sz_SQL_TXN_SERIALIZABLE[]	= "SQL_TXN_SERIALIZABLE";
BYTE sz_SQL_TXN_VERSIONING[]	= "SQL_TXN_VERSIONING";

	/* options for SQLGetStmtOption/SQLSetStmtOption */
BYTE sz_SQL_QUERY_TIMEOUT[]	= "SQL_QUERY_TIMEOUT";
BYTE sz_SQL_MAX_ROWS[]	= "SQL_MAX_ROWS";
BYTE sz_SQL_NOSCAN[]	= "SQL_NOSCAN";
BYTE sz_SQL_MAX_LENGTH[]	= "SQL_MAX_LENGTH";
BYTE sz_SQL_ASYNC_ENABLE[]	= "SQL_ASYNC_ENABLE";
BYTE sz_SQL_BIND_TYPE[]	= "SQL_BIND_TYPE";

BYTE sz_SQL_BIND_BY_COLUMN[]	= "SQL_BIND_BY_COLUMN";		/* Suboption for SQL_BIND_TYPE */

	/* options for SQLSetConnectOption/SQLGetConnectOption */
BYTE sz_SQL_ACCESS_MODE[]	= "SQL_ACCESS_MODE";
BYTE sz_SQL_AUTOCOMMIT[]	= "SQL_AUTOCOMMIT";
BYTE sz_SQL_LOGIN_TIMEOUT[]	= "SQL_LOGIN_TIMEOUT";
BYTE sz_SQL_OPT_TRACE[]	= "SQL_OPT_TRACE";
BYTE sz_SQL_OPT_TRACEFILE[]	= "SQL_OPT_TRACEFILE";
BYTE sz_SQL_TRANSLATE_DLL[]	= "SQL_TRANSLATE_DLL";
BYTE sz_SQL_TRANSLATE_OPTION[]	= "SQL_TRANSLATE_OPTION";
BYTE sz_SQL_TXN_ISOLATION[]	= "SQL_TXN_ISOLATION";
BYTE sz_SQL_CONNECT_OPT_DRVR_START[]	= "SQL_CONNECT_OPT_DRVR_START";

	/*  Column types and scopes in SQLSpecialColumns.  */
BYTE sz_SQL_BEST_ROWID[]	= "SQL_BEST_ROWID";
BYTE sz_SQL_ROWVER[]	= "SQL_ROWVER";

BYTE sz_SQL_SCOPE_CURROW[]	= "SQL_SCOPE_CURROW";
BYTE sz_SQL_SCOPE_TRANSACTION[]	= "SQL_SCOPE_TRANSACTION";
BYTE sz_SQL_SCOPE_SESSION[]	= "SQL_SCOPE_SESSION";

	/* Level 1 Prototypes */
BYTE sz_SQLColumns[]	= "SQLColumns";
BYTE sz_SQLDriverConnect[]	= "SQLDriverConnect";
BYTE sz_SQLGetConnectOption[]	= "SQLGetConnectOption";
BYTE sz_SQLGetData[]	= "SQLGetData";
BYTE sz_SQLGetFunctions[]	= "SQLGetFunctions";
BYTE sz_SQLGetInfo[]	= "SQLGetInfo";
BYTE sz_SQLGetStmtOption[]	= "SQLGetStmtOption";
BYTE sz_SQLGetTypeInfo[]	= "SQLGetTypeInfo";
BYTE sz_SQLParamData[]	= "SQLParamData";
BYTE sz_SQLPutData[]	= "SQLPutData";
BYTE sz_SQLSetConnectOption[]	= "SQLSetConnectOption";
BYTE sz_SQLSetStmtOption[]	= "SQLSetStmtOption";
BYTE sz_SQLSpecialColumns[]	= "SQLSpecialColumns";
BYTE sz_SQLStatistics[]	= "SQLStatistics";
BYTE sz_SQLTables[]	= "SQLTables";


	/* Level 2 Functions                             */

	/* SQLExtendedFetch "fFetchType" values */
BYTE sz_SQL_FETCH_NEXT[]	= "SQL_FETCH_NEXT";
BYTE sz_SQL_FETCH_FIRST[]	= "SQL_FETCH_FIRST";
BYTE sz_SQL_FETCH_LAST[]	= "SQL_FETCH_LAST";
BYTE sz_SQL_FETCH_PREV[]	= "SQL_FETCH_PREV";
BYTE sz_SQL_FETCH_ABSOLUTE[]	= "SQL_FETCH_ABSOLUTE";
BYTE sz_SQL_FETCH_RELATIVE[]	= "SQL_FETCH_RELATIVE";
BYTE sz_SQL_FETCH_RESUME[]	= "SQL_FETCH_RESUME";

	/* SQLExtendedFetch "rgfRowStatus" element values */
BYTE sz_SQL_ROW_SUCCESS[]	= "SQL_ROW_SUCCESS";
BYTE sz_SQL_ROW_DELETED[]	= "SQL_ROW_DELETED";
BYTE sz_SQL_ROW_UPDATED[]	= "SQL_ROW_UPDATED";
BYTE sz_SQL_ROW_NOROW[]	= "SQL_ROW_NOROW";

	/* Defines for SQLForeignKeys (returned in result set) */
BYTE sz_SQL_CASCADE[]	= "SQL_CASCADE";
BYTE sz_SQL_RESTRICT[]	= "SQL_RESTRICT";
BYTE sz_SQL_SET_NULL[]	= "SQL_SET_NULL";

	/* Defines for SQLProcedureColumns (returned in the result set) */
BYTE sz_SQL_PARAM_TYPE_UNKNOWN[]	= "SQL_PARAM_TYPE_UNKNOWN";
BYTE sz_SQL_PARAM_INPUT[]	= "SQL_PARAM_INPUT";
BYTE sz_SQL_PARAM_INPUT_OUTPUT[]	= "SQL_PARAM_INPUT_OUTPUT";
BYTE sz_SQL_RESULT_COL[]	= "SQL_RESULT_COL";

	/* Defines for SQLSetScrollOptions */
BYTE sz_SQL_CONCUR_READ_ONLY[]	= "SQL_CONCUR_READ_ONLY";
BYTE sz_SQL_CONCUR_LOCK[]	= "SQL_CONCUR_LOCK";
BYTE sz_SQL_CONCUR_TIMESTAMP[]	= "SQL_CONCUR_TIMESTAMP";
BYTE sz_SQL_CONCUR_VALUES[]	= "SQL_CONCUR_VALUES";

BYTE sz_SQL_SCROLL_FORWARD_ONLY[]	= "SQL_SCROLL_FORWARD_ONLY";
BYTE sz_SQL_SCROLL_KEYSET_DRIVEN[]	= "SQL_SCROLL_KEYSET_DRIVEN";
BYTE sz_SQL_SCROLL_DYNAMIC[]	= "SQL_SCROLL_DYNAMIC";

	/* Defines for SQLStatistics */
BYTE sz_SQL_INDEX_UNIQUE[]	= "SQL_INDEX_UNIQUE";
BYTE sz_SQL_INDEX_ALL[]	= "SQL_INDEX_ALL";
BYTE sz_SQL_ENSURE[]	= "SQL_ENSURE";
BYTE sz_SQL_QUICK[]	= "SQL_QUICK";

	/* Defines for SQLStatistics (returned in the result set) */
BYTE sz_SQL_TABLE_STAT[]	= "SQL_TABLE_STAT";
BYTE sz_SQL_INDEX_CLUSTERED[]	= "SQL_INDEX_CLUSTERED";
BYTE sz_SQL_INDEX_HASHED[]	= "SQL_INDEX_HASHED";
BYTE sz_SQL_INDEX_OTHER[]	= "SQL_INDEX_OTHER";

	/* Defines for SQLSetPos */
BYTE sz_SQL_ENTIRE_ROWSET[]	= "SQL_ENTIRE_ROWSET";

	/* Level 2 Prototypes */
BYTE sz_SQLBrowseConnect[]	= "SQLBrowseConnect";
BYTE sz_SQLColumnPrivileges[]	= "SQLColumnPrivileges";
BYTE sz_SQLDataSources[]	= "SQLDataSources";
BYTE sz_SQLDescribeParam[]	= "SQLDescribeParam";
BYTE sz_SQLExtendedFetch[]	= "SQLExtendedFetch";
BYTE sz_SQLForeignKeys[]	= "SQLForeignKeys";
BYTE sz_SQLMoreResults[]	= "SQLMoreResults";
BYTE sz_SQLNativeSql[]	= "SQLNativeSql";
BYTE sz_SQLNumParams[]	= "SQLNumParams";
BYTE sz_SQLParamOptions[]	= "SQLParamOptions";
BYTE sz_SQLPrimaryKeys[]	= "SQLPrimaryKeys";
BYTE sz_SQLProcedureColumns[]	= "SQLProcedureColumns";
BYTE sz_SQLProcedures[]	= "SQLProcedures";
BYTE sz_SQLSetPos[]	= "SQLSetPos";
BYTE sz_SQLSetScrollOptions[]	= "SQLSetScrollOptions";
BYTE sz_SQLTablePrivileges[]	= "SQLTablePrivileges";

	/*
	** SQLSTATE strings:
	*/

BYTE sz_00000[]	= "00000";
BYTE sz_01000[]	= "01000";
BYTE sz_01002[]	= "01002";
BYTE sz_01004[]	= "01004";
BYTE sz_01006[]	= "01006";
BYTE sz_01S00[]	= "01S00";
BYTE sz_07001[]	= "07001";
BYTE sz_07003[]	= "07003";
BYTE sz_07005[]	= "07005";
BYTE sz_07006[]	= "07006";
BYTE sz_07008[]	= "07008";
BYTE sz_08001[]	= "08001";
BYTE sz_08002[]	= "08002";
BYTE sz_08003[]	= "08003";
BYTE sz_08004[]	= "08004";
BYTE sz_08005[]	= "08005";
BYTE sz_08006[]	= "08006";
BYTE sz_08007[]	= "08007";
BYTE sz_08S01[]	= "08S01";
BYTE sz_21001[]	= "21001";
BYTE sz_21002[]	= "21002";
BYTE sz_21S01[]	= "21S01";
BYTE sz_21S02[]	= "21S02";
BYTE sz_22001[]	= "22001";
BYTE sz_22003[]	= "22003";
BYTE sz_22005[]	= "22005";
BYTE sz_22012[]	= "22012";
BYTE sz_23000[]	= "23000";
BYTE sz_24000[]	= "24000";
BYTE sz_25000[]	= "25000";
BYTE sz_26000[]	= "26000";
BYTE sz_28000[]	= "28000";
BYTE sz_34000[]	= "34000";
BYTE sz_37000[]	= "37000";
BYTE sz_3C000[]	= "3C000";
BYTE sz_40001[]	= "40001";
BYTE sz_42000[]	= "42000";
BYTE sz_70100[]	= "70100";
BYTE sz_IM001[]	= "IM001";
BYTE sz_IM002[]	= "IM002";
BYTE sz_IM003[]	= "IM003";
BYTE sz_IM004[]	= "IM004";
BYTE sz_IM005[]	= "IM005";
BYTE sz_IM006[]	= "IM006";
BYTE sz_IM007[]	= "IM007";
BYTE sz_IM008[]	= "IM008";
BYTE sz_IM009[]	= "IM009";
BYTE sz_S0001[]	= "S0001";
BYTE sz_S0002[]	= "S0002";
BYTE sz_S0011[]	= "S0011";
BYTE sz_S0012[]	= "S0012";
BYTE sz_S0021[]	= "S0021";
BYTE sz_S0022[]	= "S0022";
BYTE sz_S1000[]	= "S1000";
BYTE sz_S1001[]	= "S1001";
BYTE sz_S1002[]	= "S1002";
BYTE sz_S1003[]	= "S1003";
BYTE sz_S1004[]	= "S1004";
BYTE sz_S1005[]	= "S1005";
BYTE sz_S1006[]	= "S1006";
BYTE sz_S1007[]	= "S1007";
BYTE sz_S1008[]	= "S1008";
BYTE sz_S1009[]	= "S1009";
BYTE sz_S1010[]	= "S1010";
BYTE sz_S1012[]	= "S1012";
BYTE sz_S1013[]	= "S1013";
BYTE sz_S1014[]	= "S1014";
BYTE sz_S1015[]	= "S1015";
BYTE sz_S1090[]	= "S1090";
BYTE sz_S1091[]	= "S1091";
BYTE sz_S1092[]	= "S1092";
BYTE sz_S1093[]	= "S1093";
BYTE sz_S1094[]	= "S1094";
BYTE sz_S1095[]	= "S1095";
BYTE sz_S1096[]	= "S1096";
BYTE sz_S1097[]	= "S1097";
BYTE sz_S1098[]	= "S1098";
BYTE sz_S1099[]	= "S1099";
BYTE sz_S1100[]	= "S1100";
BYTE sz_S1101[]	= "S1101";
BYTE sz_S1102[]	= "S1102";
BYTE sz_S1103[]	= "S1103";
BYTE sz_S1104[]	= "S1104";
BYTE sz_S1106[]	= "S1106";
BYTE sz_S1107[]	= "S1107";
BYTE sz_S1108[]	= "S1108";
BYTE sz_S1109[]	= "S1109";
BYTE sz_S1110[]	= "S1110";
BYTE sz_S1C00[]	= "S1C00";
BYTE sz_S1LD0[]	= "S1LD0";
BYTE sz_S1T00[]	= "S1T00";

#undef DeclareString

#endif /* NO_ODBC_STRINGS */
