
	/*
		FILE: tests.h
	
		PURPOSE: Defines values internal to Gator.EXE,
				 the ODBC SQLCLI Test Application for Microsoft Windows
	
			 (C) 1990-1992 Microsoft Corporation.  All right reserved.
	
	*/
#include "strings.h"

	/*
	//	OutStr - This global temporary output buffer is automatically
	//	checked when ODBC Test is exited to make sure bytes haven't
	//	been written past OutStr[OUTSTRLEN - 1].
	*/
extern char OutStr[ OUTSTRLEN ];

	/*
	//		Examples of writing output to standard output:
	//	{
	//		static char CSCONST szStringConst1[] = "Any string constant";
	//		static char CSCONST szStringConst2[] = " or variable";
	//		static char CSCONST szStringConst3[] = "(must be zero terminated).";
	//		static char CSCONST szFormat[]       = "Format %s, %d";
	//		static char CSCONST sz_string[]      = "string";
	//
	//		BYTE CSCONST * pszConst;
	//		BYTE         * pszLocal;
	//		BYTE FAR     * lszEither = szStringConst3;
	//		int value = 1;
	//
	//		szWrite(szStringConst1, FALSE);	// Write part of a line
	//
	//		pszConst = szStringConst2;
	//		szWrite(pszConst, FALSE);		// Write part of a line
	//
	//		pszLocal = " ";
	//		szWrite(pszLocal, FALSE);		// Write part of a line
	//
	//		szWrite(lszEither, TRUE); 		// Finish a line
	//
	//		wsprintf(OutStr, szFormat, (LPSTR)sz_string, value);
	//		szWrite(OutStr, TRUE);			// Write using OutStr
	//	}
	*/

#ifndef QUIET

#define INITLOOP    szWrite( sz_loop, TRUE );

	/* displays iteration: i to the output */
#define ITERATION(i) { \
		szWrite(sz_iteration, FALSE); \
		szWrite(itoa(i, OutStr, 10), TRUE); \
		}
		
	/* displays function name and line number for the current call, should
	be used immediately before an SQLFunction call and the parameter should be
	a CSCONST string in the format sz_SQLFunctionName */
		
#define FUNCCALL(str) { \
	wsprintf(OutStr,sz_Formatline, \
		(char FAR *)str,__LINE__,(char FAR *)__FILE__); \
	szWrite(OutStr,TRUE); \
	}

#else

#define INITLOOP /**/
#define ITERATION(i) /**/
#define FUNCCALL(str) /**/

#endif

	/* this mask is used by the individual auto/quick tests to determine which
	of the sub-tests should be run */
#define MASKSIZE 2
#define TOTAL_TESTS (MASKSIZE * 8 * sizeof(long))
#define getbit(rglMask,ibit) (rglMask[((ibit)-1)/(8*sizeof(long))]\
	& (1l << (((ibit)-1) % (8*sizeof(long)))))

	/* indention before all error messages in quick tests */
static BYTE CSCONST szIndent[] = "        ";

typedef struct ServerStruct  /* information read from the .ini file 
								when an autotest is selected */
	{
	char szSource[20];
	char szValidServer0[20]; /* if this is an empty string then hdbc should be
								used */
	char szValidServer1[20];
	char szValidLogin0[20];
	char szValidLogin1[20];
	char szValidPassword0[20];
	char szValidPassword1[20];
	char szValidQuery0[200];
	char szParamQuery0[300];
	char szParam0[20];
	long FAR *rglMask;
//mjHANDLE hwnd;    /* this can be used as the window handle for
	HWND	hwnd;	/* this can be used as the window handle for
                       DriverConnect*/
	HENV henv;		/* these are used when the connection being tested */
	HDBC hdbc;      /* is the current connection in ODBC Test */
	int  cErrors;   /* contains the number of errors encountered during 
						the test run */
	int  cStateErrors; /* contains the number of error states encountered
						during the test run */
	} SERVERSTRUCT;

#define MAX_TYPE 50    /* max length of buffer for typename */
#define MAX_PREFIX 10  /* max size of a prefix string returned by
							SQLGetTypeInfo */
#define MAX_SUFFIX 10  /* max size of a suffix string returned by 
							SQLGetTypeInfo */
#define MAX_LENGTH_BUF 11  /* storage size of length buffer */
#define MAX_PARAMS 50  /* storage for params when creating insert stmt */
#define MAX_TYPES 50   /* max number of types the tests can handle */
#define szSIZE 700     /* size of sz buffer */
#define TABLENAMESIZE 31

	/* holds information returned for SQLGetTypeInfo */
typedef struct TypeNum {
		char szType[MAX_TYPE];
		char szFieldName[MAX_TYPE + 1];
		int  iFieldNum;
		SWORD  wSQLType;
		char szPrefix[MAX_PREFIX];
		char szSuffix[MAX_SUFFIX];
		char szParams[MAX_PARAMS];
		char szLength[MAX_LENGTH_BUF];
		UDWORD precision;
		SWORD scale;
		int  nullable;
		int  searchable;
		} TYPENUM;

#define MAX_SCALE 5
	/* holds data returned from SQLColumns */
typedef struct ColData {
		UDWORD precision;
		SWORD scale;
		SWORD nullable;
		UDWORD length;
		int   fAutoUpdate;
		SDWORD fSearchable;
		} COLDATA;

	/* the following hold data received in a BindFetch loop */
typedef struct BindUCHAR {
		char data[szSIZE];
		SDWORD outlen;
		} BINDUCHAR;

typedef struct BindSDWORD {
		SDWORD data;
		SDWORD outlen;
		} BINDSDWORD;

typedef struct BindSWORD {
		SWORD data;
		SDWORD outlen;
		} BINDSWORD;

typedef struct BindSDOUBLE {
		SDOUBLE data;
		SDWORD outlen;
		} BINDSDOUBLE;

typedef struct BindSFLOAT {
		SFLOAT data;
		SDWORD outlen;
		} BINDSFLOAT;

	/* holds the parameter information used with SetParam */
typedef struct tagParamS {
	BYTE pbs[ MAX_TYPES ][ 25 ];
	SDWORD cbValue[ MAX_TYPES ];
}	PSTRUCT;

	/* create a table with all types returned by SQLGetTypeInfo and populate
	the table.  Each field gets data based on the row number. */
pascal int MakeTable( HDBC hdbc0, char FAR *szCreatedTable,
						TYPENUM FAR *rgTypes, COLDATA FAR *rgCols );

	/* create data for a particular row of a particular field.  Used during
	table creation, and when retreiving data to verify that correct data is
	returned */
pascal char * MakeData( int nCol, TYPENUM FAR *TypeInfo,
							COLDATA *ColInfo, char FAR * sz );

	/* display "returned: SQL_WHATEVER" or "expected: SQL_WHATEVER" the first
if fReturned is TRUE, false otherwise */
pascal void DisplayRetcode(BOOL fReturned, RETCODE retcode);

	/* verify that these state returned by SQLError is the state passed
	in, display an error message otherwise.  The state passed in should be the
	CSCONST in the form sz_00000 */
pascal int fStateCheck( HENV henv0, HDBC hdbc0, HSTMT hstmt0,
							char FAR * fState );

	/* check an array of states and verify that each state returned in SQLError
		is a member of the array passed in, otherwise display an error message */
pascal int frgStateCheck( HENV henv0, HDBC hdbc0, HSTMT hstmt0,
								char FAR * szExpState[] );

