
	/*  1988-91, Bowers Development Corp. */
	/* EventLoop.c */

#include <Types.h>
#include <Quickdraw.h>
#include <Controls.h>
#include <Desk.h>
#include <Dialogs.h>
#include <DiskInit.h>
#include <Editions.h>
#include <EPPC.h>
#include <Events.h>
#include <Fonts.h>
#include <GestaltEqu.h>
#include <Lists.h>
#include <Menus.h>
#include <SegLoad.h>
#include <OSEvents.h>
#include <TextEdit.h>
#include <ToolUtils.h>
#include <Traps.h>
#include <AppleEvents.h>
#include <LibraryManager.h>

#include "Globals.h"
#include "ResourceDefs.h"
#include "Cursors.h"
#include "DoScrap.h"
#include "Miscellany.h"
#include "Windowing.h"
#include "Dispatcher.h"
#include "MainMenu.h"

#include "EventLoop.h"
 
#include "SQL.h"
#include "SQLEXT.h"

#include "adhoc.h"
#include "odbcstr.h"

#pragma segment ODBCmain

	HENV	henv	= SQL_NULL_HENV;	// handle of current environment
	HDBC	hdbc	= SQL_NULL_HDBC;	// handle of current connection
	UWORD	idbc	= 0;				// index of current connection
	HSTMT	hstmt	= SQL_NULL_HSTMT;	// handle of current statement
	UWORD	istmt	= 0;				// index of current statement

	BOOL	fSeparatorNeeded	= TRUE;
	BOOL	fWriteScreen		= TRUE;
	BOOL	fWriteLog			= FALSE;

	char	OutStr[ 255 ];	// One global OutStr!

	void DoMouseDown();
	void DoMouseUp();
	void DoKeystroke();
	void DoActivate();

	Boolean			TrapAvailable( short tNumber, TrapType tType );
	void			SetStackSize( long stackSize );
	Boolean			DoEvent();
	long			GetSleep();

	extern void		_DataInit();

	short		finderMessage;
	short		nrDocs;

	short	main();
	void	PrintDocuments();
	void	OpenDocuments();


short main()
{
	Initialize();
	CountAppFiles( &finderMessage, &nrDocs );
	if( nrDocs > 0 )
	{
		if( finderMessage == appPrint )
		{
			PrintDocuments();
		}
		else
		{
			OpenDocuments();
			MainLoop();
		}
	}
	else
	{
		OpenApp();
		MainLoop();
	}
	return 0;
}/* end main */


void PrintDocuments()
{
	Acknowledge( CantPrintID );
}/* end PrintDocuments */


void OpenDocuments()
{
	short			n;
	short			nrOpened;
	AppFile			docInfo;

	nrOpened = 0;
	for( n = 1; n <= nrDocs; n++ )
	{
		GetAppFiles( n, &docInfo );
		if( OkToOpen( docInfo.fType ) )
		{
			OpenDoc( docInfo.fName, docInfo.vRefNum );
			nrOpened++;
		}
		else
		{
			ParamText( docInfo.fName, "\p", "\p", "\p" );
			Acknowledge( WrongTypeID );
		}
		ClrAppFiles( n );
	}
	if( nrOpened == 0 )
	{
		OpenApp();
	}
}/* end OpenDocuments */


Boolean TrapAvailable( short tNumber, TrapType tType )
{
	return( NGetTrapAddress( tNumber, tType ) != GetTrapAddress( _Unimplemented ) );
}/* end TrapAvailable */


void SetStackSize( long stackSize )
{
	SetApplLimit( (Ptr)((long)&stackSize - stackSize) );
}/* end SetStackSize */


static void	GetSysConfig( void )
{
	OSErr			ignoreError;
	long			tempLong;		/* for Gestalt	*/
	SysEnvRec		environs;

	sysConfig.hasGestalt = (Gestalt( gestaltVersion, &tempLong ) == noErr );
	if( sysConfig.hasGestalt )
	{
		sysConfig.hasWNE = TrapAvailable( _WaitNextEvent, ToolTrap );
		ignoreError = Gestalt( gestaltQuickdrawVersion, &tempLong );
		sysConfig.hasColorQD		= (tempLong != gestaltOriginalQD );
		sysConfig.hasAppleEvents	= (Gestalt( gestaltAppleEventsAttr, &tempLong ) == noErr );
		sysConfig.hasEditionMgr		 = (Gestalt( gestaltEditionMgrAttr, &tempLong ) == noErr );
		if( sysConfig.hasEditionMgr )
		{
			if( !CheckOS( InitEditionPack() ) )
			{
				sysConfig.hasEditionMgr = false;	/* unable to load Edition Manager package*/
			}
		}
	}
	else
	{
		ignoreError = SysEnvirons ( curSysEnvVers, &environs );
		if( environs.machineType < 0 )
		{
			sysConfig.hasWNE = false;
		}
		else
		{
			sysConfig.hasWNE = TrapAvailable( _WaitNextEvent, ToolTrap );
		}
		sysConfig.hasColorQD		= environs.hasColorQD;
		sysConfig.hasAppleEvents	= false;
		sysConfig.hasEditionMgr		= false;
	}
	return;
}/* end GetSysConfig */


void Initialize()
{
		OSErr	 rc;


	UnloadSeg ( (Ptr)_DataInit );

	SetStackSize( 15000 );
	MaxApplZone();
	MoreMasters();
	MoreMasters();
	MoreMasters();

	InitGraf( &qd.thePort );
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( NULL );

	InitLibraryManager( 0, kSystemZone, kLockMemory );	// pool sz, zone, memType
#ifdef NONSLM
#else
	rc = LoadLibraries( TRUE, FALSE );	/* force all dependent libs, do self	*/

	if( rc )
	{
	//	put up a dialog box here
		CleanupLibraryManager();
	}
#endif NONSLM

	FlushEvents( everyEvent - diskMask - app4Mask, 0 );

	GetSysConfig();	
	inBackground = false;

	LoadMenus();
	LoadCursors();
	InitScrap(); 

	InitGlobals();
	initglobs();

	AppInit();					// Do the Microsoft initialization
	return;
}/* end Initialize */


void DoMouseDown()
{
	WindowPtr		whichWindow;
	short			whichPart;

	whichPart = FindWindow (curEvent.where, &whichWindow );
	switch( whichPart )
	{
	case inDesk:		/* Do nothing */;						break;
	case inMenuBar:		DoMenu( MenuSelect( curEvent.where ) );	break;
	case inSysWindow:	SystemClick( &curEvent, whichWindow );	break;
	case inContent:		DoContent( whichWindow );				break;
	case inDrag:		DoDrag( whichWindow );					break;
	case inGrow:		DoGrow( whichWindow );					break;
	case inGoAway:		DoGoAway( whichWindow );				break;
	case inZoomIn:		DoZoom( whichWindow, inZoomIn );		break;
	case inZoomOut:		DoZoom( whichWindow, inZoomOut );		break;
	}
}/* end DoMouseDown */


void DoMouseUp()
{
	WindowPtr		whichWindow;
	short			whichPart;

	whichPart = FindWindow( curEvent.where, &whichWindow );
		/* save event time to check later for double click	*/
}/* end DoMouseUp*/
 

void DoKeystroke()
{
	short			charCode;
	char			ch;

	charCode = curEvent.message & charCodeMask;
	ch = charCode;
	
	if( curEvent.modifiers & cmdKey )
	{
		if( curEvent.what != autoKey )
			DoMenu( MenuKey( ch ) );
	}
	else
	{
		if( curWindow == NULL )
			SysBeep(1);
		else
			TypeInWindow(ch);
	}
	return;
}/* end DoKeystroke */


void DoUpdate()
{
	GrafPtr			savePort;
	WindowPtr		saveWindow;
	WindowPtr		whichWindow;

	GetPort( &savePort );
	whichWindow = (WindowPtr)curEvent.message;
	SetPort( whichWindow );
	saveWindow = curWindow;
	SetInfo( whichWindow );
	BeginUpdate( whichWindow );
		EraseRect( &(whichWindow->portRect) );
		DrawControls( whichWindow );
		UpdateContent();
	EndUpdate( whichWindow );
	SetInfo( saveWindow );	
	SetPort( savePort );
	return;
}/* end DoUpdate */


void DoActivate()
{
	Boolean			activate;
	WindowPtr		whichWindow;

	whichWindow = (WindowPtr)curEvent.message;
	SetPort( whichWindow );
	SetInfo( whichWindow );

	activate = ((curEvent.modifiers & activeFlag) != 0);
	if( activate )
	{
		if( cur->text )
			TEActivate( cur->text );
		ReadDeskScrap();
	}

	HiliteScroll( cur->Scroll5Handle,	activate );
	HiliteScroll( cur->vScroll,			activate );
	HiliteScroll( cur->hScroll,			activate );

	ActivateContent( activate );

	if( !activate )
	{
		WriteDeskScrap();
		if( cur->text )
		{
			TEDeactivate( cur->text );
		}
		SetInfo( NULL );
	}
	return;
}/* end DoActivate */
 

void DoDiskEvent()
{
#define dlgTop		75                                   
#define dlgLeft		100

	OSErr			ignore;
	Point			where;

	if( HiWord( curEvent.message ) != noErr )
	{
		DILoad();
		SetPt( &where, dlgLeft, dlgTop ); 
		ignore = DIBadMount( where, curEvent.message );
		DIUnload();
	}
	return;
}/* end DoDiskEvent */
 

void DoApp4Event()
{
#define suspendResumeMsg	1
#define resumeMask			1
	
	Boolean			resume;
	WindowPtr		front;

	curEvent.what = nullEvent;
	if( BitShift( curEvent.message, -24) == suspendResumeMsg )
	{
		resume = ((curEvent.message & resumeMask) != 0 );
		if( resume )
		{
			ReadDeskScrap();
		}
		else
		{
			WriteDeskScrap();
		}
		inBackground = !resume;
		front = FrontWindow();
		if( front )
		{
			curEvent.what		= activateEvt;
			curEvent.modifiers	= LoWord( curEvent.message );
			curEvent.message	= (long)front;
		}
	}
	return;
}/* end DoApp4Event */
 

long GetSleep()
{
	long	sleep;

	sleep = GetCaretTime();		/* if long time, some Inits won't run	*/
	if( !inBackground && FrontWindow() )
	{
		if( FrontWindow() == curWindow )
		{
			if( cur->text )
			{	if( (**( cur->text )).selStart == (**( cur->text )).selEnd )
					sleep = GetCaretTime();
			}
		}
		else
		{		/* DA or modeless dialog */
			sleep = GetCaretTime();
		}
	}
	return sleep;
}/* end GetSleep */
 

static Boolean GetEvent()
{
	Boolean		gotEvent;

	if( sysConfig.hasWNE )
		gotEvent = WaitNextEvent( everyEvent, &curEvent, GetSleep(), cursorRgn );
	else
	{
		SystemTask();
		gotEvent = GetNextEvent( everyEvent, &curEvent );
	}
	return gotEvent;
}/* end GetEvent */
 

Boolean DoEvent()
{
	Boolean			gotEvent;
	DialogPtr		whichDialog;

	gotEvent = GetEvent();
	if( curEvent.what == app4Evt )
	{
			/* here so event won't get swallowed by IsDialogEvent	*/
	  		/* and so suspend/resume can be translated to activate	*/
		DoApp4Event();
	}
	if( IsDialogEvent( &curEvent ) )
	{
		if( (curEvent.what == activateEvt)
		||  (curEvent.what == updateEvt) )
			 whichDialog = (DialogPtr)curEvent.message;
		else whichDialog = FrontWindow();
		SetPort( whichDialog );
		if( (curEvent.what == keyDown) && (curEvent.modifiers & cmdKey) )
			DoKeystroke();		/* => calls MenuKey */
	}
	else if( gotEvent )
	{
		switch( curEvent.what )
		{
		case mouseDown:		DoMouseDown();	break;
		case mouseUp:		DoMouseUp();	break;
		case keyDown:		DoKeystroke();	break;
		case updateEvt:		DoUpdate();		break;
		case activateEvt:	DoActivate();	break;
		case diskEvt:		DoDiskEvent();	break;
	//	case kHighLevelEvent:
	//		if( sysConfig.hasAppleEvents )
	//			DoHighLevelEvent();
	//		break;
		default:			DoIdle();		break;
		}
	}
	return gotEvent;
}/* end DoEvent */
 

void MainLoop()
{
	UpdateMenus();

	quittingTime = false;	
	while( !quittingTime )
	{
		ShapeCursor();
		if( cur->text )
			TEIdle( cur->text );
		if( DoEvent() )
			UpdateMenus();
	}
	WriteDeskScrap();
	return;
}/* end MainLoop */


void messageBox( char *msg, char *title, int type )
{
	switch( type )
	{
	case stopIcon:
		ParamText( title, msg, 0, 0 );
		StopAlert( StopID, 0 );
	}
	return;
}/* end messageBox */
