/*
 *	ODBCdataTranslate.c
 *
 *	(c) Apple Computer, Inc 1993
 */

#include <Dialogs.h>
#include <Fonts.h>
#include <LibraryManagerUtilities.h>
#include <Menus.h>
#include <QuickDraw.h>
#include <TextEdit.h>
#include <Windows.h>

#include "DataTrans.h"

#include "ConfigTranslator.h"

/*
 *	Globals
 */

TLibraryFile	*gLibraryFile;

/*
 *	Prototypes
 */

/*
 *	Public functions
 */

void
ODBCdataTranslateInit(void)
{
	/*
	 *	Shared library init function
	 */
	
	GlobalWorld	saved;
	
	/*
	 *	switch to shared library's A5
	 */
	
	saved = OpenGlobalWorld();
	
	/*
	 *	save the local library file so we can quickly preflight later
	 */
	
	gLibraryFile = GetLocalLibraryFile();
	if (gLibraryFile == NULL)
	{
		DebugStr("\p GetLocalLibraryFile failed");
	}
	
	/*
	 *	since we may be posting dialogs the QuickDraw globals, et al. must be initialized
	 */
	
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	
	/*
	 *	always remember to restore original A5
	 */
	
	CloseGlobalWorld(saved);
}

void
ODBCdataTranslateCleanUp(void)
{
	/*
	 *	Shared library cleanup function
	 */
}

TRANSAPI
ConfigTranslator(GrafPtr	pGrafPort,			/* window to center over	*/
				 DWORD		*pvOption)			/* translation option		*/
{
#pragma unused( pGrafPort )
	/*
	 *	Shared lilbrary entry point for configuring a data translator.
	 */
	
	TRANSAPI		success;
	long			savedrefnum;
	OSErr			err;
	GlobalWorld		saved;
	
	/*
	 *	setup for using shared library globals, resources
	 */
	
	saved = OpenGlobalWorld();
	if( err = Preflight( gLibraryFile, &savedrefnum ) ) 
	{
		DebugStr("\p Preflight failed");
		CloseGlobalWorld(saved);
		return FALSE;
	}
	
	/*
	 *	here's where the work gets done
	 */
	
	success = (TRANSAPI) DoConfigTranslator(pvOption);
	
	/*
	 *	restore resource path, A5 world
	 */
	
	Postflight(gLibraryFile, savedrefnum);
	CloseGlobalWorld(saved);
	
	return success;
}

TRANSAPI
SQLDataSourceToDriver(UDWORD	fOption,
					  SWORD		fCType,
					  PTR		rgbValueIn,
					  SDWORD	cbValueIn,
					  PTR		rgbValueOut,
					  SDWORD	cbValueOutMax,
					  SDWORD	*pcbValueOut,
					  UCHAR		*szErrorMsg,
					  SWORD		cbErrorMsgMax,
					  SWORD		*pcbErrorMsg)
{
#pragma unused(	fOption, fCType, rgbValueIn, cbValueIn, rgbValueOut, cbValueOutMax, pcbValueOut, szErrorMsg, cbErrorMsgMax, pcbErrorMsg )
	/*
	 *	Shared lilbrary entry point for ??.
	 */
	
	TRANSAPI		success;
	long			savedrefnum;
	OSErr			err;
	GlobalWorld		saved;
	
	/*
	 *	setup for using shared library globals, resources
	 */
	
	saved = OpenGlobalWorld();
	if( err = Preflight( gLibraryFile, &savedrefnum ) ) 
	{
		DebugStr("\p Preflight failed");
		CloseGlobalWorld(saved);
		return FALSE;
	}
	
	/*
	 *	here's where the work gets done
	 */
	
	success = TRUE;	// do something, stupid!
	
	/*
	 *	restore resource path, A5 world
	 */
	
	Postflight(gLibraryFile, savedrefnum);
	CloseGlobalWorld(saved);
	
	return success;
}

TRANSAPI
SQLDriverToDataSource(UDWORD	fOption,
					  SWORD		fCType,
					  PTR		rgbValueIn,
					  SDWORD	cbValueIn,
					  PTR		rgbValueOut,
					  SDWORD	cbValueOutMax,
					  SDWORD	*pcbValueOut,
					  UCHAR		*szErrorMsg,
					  SWORD		cbErrorMsgMax,
					  SWORD		*pcbErrorMsg)
{
#pragma unused(	fOption, fCType, rgbValueIn, cbValueIn, rgbValueOut, cbValueOutMax, pcbValueOut, szErrorMsg, cbErrorMsgMax, pcbErrorMsg )
	/*
	 *	Shared lilbrary entry point for ??.
	 */
	
	TRANSAPI		success;
	long			savedrefnum;
	OSErr			err;
	GlobalWorld		saved;
	
	/*
	 *	setup for using shared library globals, resources
	 */
	
	saved = OpenGlobalWorld();
	if( err = Preflight( gLibraryFile, &savedrefnum ) ) 
	{
		DebugStr("\p Preflight failed");
		CloseGlobalWorld(saved);
		return FALSE;
	}
	
	/*
	 *	here's where the work gets done
	 */
	
	success = TRUE;	// do something, stupid!
	
	/*
	 *	restore resource path, A5 world
	 */
	
	Postflight(gLibraryFile, savedrefnum);
	CloseGlobalWorld(saved);
	
	return success;
}

