/*
 **************************************************************************

	ODBCsharedLibrary.h
	
	Contains the prototypes and data structure definitions necessary to 
	interface to the Configuration Manager through the Shared Library
	Manager.
	
	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.
	
 **************************************************************************
 */


#include "LibraryManager.h"

/*
 *
 *	Configuration Manager Shared Library Manager definitions.
 *
 */

#define kODBCconfigManagerLibID			"appl:ODBC$ConfigManagerLibrary"
#define kODBCconfigManagerFunctionSet	"appl:ODBC$ConfigManagerFunctionSet"
#define kODBCparentID					"appl:ODBC$ParentId"

									/* The following defines are for use in calling 	*/
									/** the Configuration Manager functions by name.	*/
									
#define kSQLManageDataSourcesId			"SQLManageDataSources"
#define kSQLCreateDataSourceId			"SQLCreateDataSource"
#define kSQLGetTranslatorId				"SQLGetTranslator"
#define kSQLSQLGetInstalledDriversId	"SQLGetInstalledDrivers"
#define kSQLRemoveDefaultDataSourceId	"SQLRemoveDefaultDataSource"
#define kSQLWriteDSNId					"SQLWriteDSN"
#define kSQLRemoveDSNId					"SQLRemoveDSN"
#define kSQLGetKeywordValueId			"SQLGetKeywordValue"
#define kSQLSetKeywordValueId			"SQLSetKeywordValue"



/*
 *
 *	Driver Manager Shared Library Manager definitions.
 *
 */

#define kODBCdriverManagerID			"appl:ODBC$DriverManagerLibrary"
#define kODBCdriverManagerFunctionSet	"appl:ODBC$DriverManagerFunctionSet"
									/* The Driver Manager's Parent ID is the same as	*/
									/** the configuration managers.						*/

									/* The following defines are for use in calling 	*/
									/** the Driver Manager functions by name.			*/

#define kSQLAllocConnectId			"SQLAllocConnect"
#define kSQLAllocEnvId				"SQLAllocEnv"
#define kSQLAllocStmtId				"SQLAllocStmt"
#define kSQLBindColId				"SQLBindCol"
#define kSQLCancelId				"SQLCancel"
#define kSQLColAttributesId			"SQLColAttributes"
#define kSQLConnectId				"SQLConnect"
#define kSQLDescribeColId			"SQLDescribeCol"
#define kSQLDisconnectId			"SQLDisconnect"
#define kSQLErrorId					"SQLError"
#define kSQLExecDirectId			"SQLExecDirect"
#define kSQLExecuteId				"SQLExecute"
#define kSQLFetchId					"SQLFetch"
#define kSQLFreeConnectId			"SQLFreeConnect"
#define kSQLFreeEnvId				"SQLFreeEnv"
#define kSQLFreeStmtId				"SQLFreeStmt"
#define kSQLGetCursorNameId			"SQLGetCursorName"
#define kSQLNumResultColsId			"SQLNumResultCols"
#define kSQLPrepareId				"SQLPrepare"
#define kSQLRowCountId				"SQLRowCount"
#define kSQLSetCursorNameId			"SQLSetCursorName"
#define kSQLSetParamId				"SQLSetParam"
#define kSQLTransactId				"SQLTransact"
#define kSQLColumnsId				"SQLColumns"
#define kSQLDriverConnectId			"SQLDriverConnect"
#define kSQLGetConnectOptionId		"SQLGetConnectOption"
#define kSQLGetDataId				"SQLGetData"
#define kSQLGetFunctionsId			"SQLGetFunctions"
#define kSQLGetInfoId				"SQLGetInfo"
#define kSQLGetStmtOptionId			"SQLGetStmtOption"
#define kSQLGetTypeInfoId			"SQLGetTypeInfo"
#define kSQLParamDataId				"SQLParamData"
#define kSQLPutDataId				"SQLPutData"
#define kSQLSetConnectOptionId		"SQLSetConnectOption"
#define kSQLSetStmtOptionId			"SQLSetStmtOption"
#define kSQLSpecialColumnsId		"SQLSpecialColumns"
#define kSQLStatisticsId			"SQLStatistics"
#define kSQLTablesId				"SQLTables"
#define kSQLBrowseConnectId			"SQLBrowseConnect"
#define kSQLColumnPrivilegesId		"SQLColumnPrivileges"
#define kSQLDataSourcesId			"SQLDataSources"
#define kSQLDescribeParamId			"SQLDescribeParam"
#define kSQLExtendedFetchId			"SQLExtendedFetch"
#define kSQLForeignKeysId			"SQLForeignKeys"
#define kSQLMoreResultsId			"SQLMoreResults"
#define kSQLNativeSqlId				"SQLNativeSql"
#define kSQLNumParamsId				"SQLNumParams"
#define kSQLParamOptionsId			"SQLParamOptions"
#define kSQLPrimaryKeysId			"SQLPrimaryKeys"
#define kSQLProcedureColumnsId		"SQLProcedureColumns"
#define kSQLProceduresId			"SQLProcedures"
#define kSQLSetPosId				"SQLSetPos"
#define kSQLSetScrollOptionsId		"SQLSetScrollOptions"
#define kSQLTablePrivilegesId		"SQLTablePrivileges"

/*
 *
 *	DBMS Driver Shared Library Manager definitions.
 *
 */
									/* The following define sets the DBMS Setup Parent	*/
									/** Id for all DBMS setup libraries.				*/

#define kODBCdbmsDriverParentID		"appl:ODBC$DBMSdriverParentId"

									/* The defines for the calling the DBMS driver		*/
									/** functions by name are the same as for the 		*/
									/** Driver Manager functions except that a DBMS		*/
									/** Driver does not support the SQLGetFunctions and	*/
									/** SQLDataSources function calls.					*/
									
									/* Defines the resource type used by configuration	*/
									/** manager to find the DBMS vendor type for the 	*/
									/** DBMS setup library.								*/
#define kODBCdbmsDriverResId		'oddb'

/*
 *
 *	DBMS Setup Shared Library Manager definitions.
 *
 */
									/* The following define sets the DBMS Setup Parent	*/
									/** Id for all DBMS setup libraries.				*/

#define kODBCdbmsSetupParentID		"appl:ODBC$DBMSsetupParentId"

									/* The following defines are for use in calling 	*/
									/** the DBMS Setup functions by name.				*/
									/* AT LEAST FOR NOW, IT SEEMS THAT IF THE ROUTINE	*/
									/** IS DECLARED AS pascal THEN THE NAMES HAVE TO BE	*/
									/** UPPER CASE.										*/
//#define kConfigDSNId				"ConfigDSN"
#define kConfigDSNId				"CONFIGDSN"

									/* Defines the resource type used by configuration	*/
									/** manager to find the DBMS vendor type for the 	*/
									/** DBMS setup library.								*/
#define kODBCdbmsSetupResId			'odds'

/*
 *
 *	Data Translate Shared Library Manager definitions.
 *
 */
									/* The following define sets the DBMS Setup Parent	*/
									/** Id for all DBMS setup libraries.				*/

#define kODBCdataTranslateParentID	"appl:ODBC$DataTranslateParentId"

									/* The following defines are for use in calling 	*/
									/** the Data Translate functions by name.			*/

//#define kSQLDataSourceToDriverId	"SQLDataSourceToDriver"
//#define kSQLDriverToDataSourceId	"SQLDriverToDataSource"
//#define kConfigTranslatorId		"ConfigTranslator"
#define kSQLDataSourceToDriverId	"SQLDATASOURCETODRIVER"
#define kSQLDriverToDataSourceId	"SQLDRIVERTODATASOURCE"
#define kConfigTranslatorId			"CONFIGTRANSLATOR"

									/* Defines the resource type used by configuration	*/
									/** manager to find the descriptor for the 			*/
									/** Data Translate library.							*/
#define kODBCdbmsTranslateResId		'oddt'


