/*
	File:		HelloPart.cpp

	Contains:	HelloPart OpenDoc part class

	Written by:	Richard Clark, Eric Soldan, Tantek elik

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	  4/14/94	T		Update to work with a6.
		 <2>	  4/12/94	ERS		Rewrite, based on original.
		 <1>	  4/12/94	RDC		Original version.
	To Do:
*/


//#define 	WeBeHere(StringPtr) DebugStr(StringPtr)
#define 	WeBeHere(StringPtr)

// To help you understand what is being called when, I've added DebugStr
// statements to all of the methods in this file, in the form of the
// macro WeBeHere.  Above you will see two lines defining WeBeHere, one of
// which is commented out.  If you want break points at the start of each
// method, you may wish to enable WeBeHere.  It sometimes helps a lot in
// terms of understanding what is going on in your part.



#ifndef _XMPTYPES_
#include "XMPTypes.h"		// MUST BE BEFORE TOOLBOX INCLUDES
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRIPT__
#include <script.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef _ARBITRAT_
#include "Arbitrat.h"
#endif

#ifndef _CANVAS_
#include "Canvas.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _FOCI_
#include "Foci.h"
#endif

#ifndef _INFO_
#include "Info.h"
#endif

#ifndef _PSTOBJ_
#include "PstObj.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _STDPROPS_
#include "StdProps.h"
#endif

#ifndef _STDTYPES_
#include "StdTypes.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _WINSTAT_
#include "WinStat.h"
#endif

#ifndef _XMPSESSM_
#include "XMPSessM.h"
#endif

#ifndef _HELLOPART_
#include "HelloPart.h"
#endif


#pragma segment HelloPart

//====================================================================
// Constants
//====================================================================

const short kContentStringResID = 5000;
const short kContentStringID    = 1;

//====================================================================
// Local Classes
//====================================================================

class CFocus {
	public:
	CFocus(XMPFacet*);
	~CFocus();

	protected:
		CFocus()	{ } // Just to make sure nobody constructs
						// without passing in a facet
		GrafPtr		fFacetPort;
		GrafPtr		fOldPort;
		Point		fOldOrigin;
		RgnHandle	fOldClip;
};

//====================================================================
// Function Prototypes
//====================================================================

static pascal StringPtr 	PLstrcpy(StringPtr d, ConstStr255Param s);
static StringPtr			IntlToPStr(XMPIText* intlText, StringPtr pstr);

static short		BeginUsingEditorResFile(void);
static void			EndUsingEditorResFile(short oldRes);



//====================================================================
// HelloPart
//====================================================================

//--------------------------------------------------------------------
// HelloPart::HelloPart
//--------------------------------------------------------------------
// Here we are -- happy in our constructor.  We have to at least
// init our fields so others are happy, too.

HelloPart::HelloPart() 
{
	WeBeHere("\pHelloPart::HelloPart");

	fTextData[0] = 0;
	fDirty       = kXMPFalse;
	fWindowID    = 0;
	fSession     = kXMPNULL;
}



//--------------------------------------------------------------------
// HelloPart::~HelloPart
//--------------------------------------------------------------------

HelloPart::~HelloPart()
{
	WeBeHere("\pHelloPart::~HelloPart");

	// We're going away, and nobody cares.
}



//--------------------------------------------------------------------
// HelloPart::InitPart
//--------------------------------------------------------------------

void HelloPart::InitPart(XMPStorageUnit* storageUnit)
{
	WeBeHere("\pHelloPart::InitPart");

	XMPPersistentObject::InitPersistentObject(storageUnit);
	this->CommonInitPart();
		// We say "this" everywhere just to be explicit.

	storageUnit->AddProperty(kXMPPropContents)->AddValue(kHelloPartKind);
		// For your edification, AddProperty & AddValue 
		// implicitly focus.  Check the API for more info.

	short curRes = BeginUsingEditorResFile();
	GetIndString(fTextData, kContentStringResID, kContentStringID);
	EndUsingEditorResFile(curRes);
		// Note that to get to the resources of your part, you simply
		// bracket your resource manager calls with BeginUsingEditorResFile
		// andEndUsingEditorResFile.

	// Initialize your other fields here.

	fDirty = true;
	this->Externalize();
		// This sample starts up with some initial content.  This is
		// actually a tricky situation.  Unless you cause the initial
		// content to persist, if the user saves and then opens the
		// document at a later time, the initial content won't be
		// there.
		// Note that even if you have no initial content, this still
		// doesn't hurt.  You will officially save nothing, instead
		// of implicitly saving nothing.  Therefore this is the
		// correct way to handle the initial content situation in
		// all cases.
		// So, the correct and good thing to do is to make sure that
		// all of your writeMyData code is in one place.  The correct
		// OpenDoc place for that code is in your Externalize method.
		// By setting fDirty true, we can then call Externalize to
		// write the data out.
}



//--------------------------------------------------------------------
// HelloPart::InitPartFromStorage
//--------------------------------------------------------------------

void HelloPart::InitPartFromStorage(XMPStorageUnit* storageUnit)
{
	WeBeHere("\pHelloPart::InitPartFromStorage");

	XMPPersistentObject::InitPersistentObjectFromStorage(storageUnit);

	this->CommonInitPart();
	storageUnit->Focus(kXMPPropContents, kXMPPosSame,
					   kHelloPartKind, 0, kXMPPosSame);
	fTextData[0] = (unsigned char)storageUnit->GetSize();
	storageUnit->GetValue(fTextData[0], fTextData + 1);
}



//--------------------------------------------------------------------
// HelloPart::CommonInitPart
//--------------------------------------------------------------------

void HelloPart::CommonInitPart()
{
	WeBeHere("\pHelloPart::CommonInitPart");

	fSession = this->GetStorageUnit()->GetSession();
		// As your part grows, add your initialization that is common
		// between InitPart and InitPartFromStorage here.
		// Note that GetStorageUnit can be used, since just prior to
		// calling this method, InitPart calledInitPersistentObject or
		// InitPartFromStorage called InitPersistentObjectFromStorage.
}



//--------------------------------------------------------------------
// HelloPart::Externalize
//--------------------------------------------------------------------

void	HelloPart::Externalize()
{
	WeBeHere("\pHelloPart::Externalize");

	if (fDirty)
	{
		XMPPersistentObject::Externalize();
			// First do standard OpenDoc externalization.

		XMPStorageUnit* storageUnit = this->GetStorageUnit();
			// Get the reference to where we are writing to.

		storageUnit->Focus(kXMPPropContents, kXMPPosSame,
						   kHelloPartKind, 0, kXMPPosSame);
			// First we focus on the property we want to write out.

		storageUnit->SetValue(fTextData[0], fTextData + 1);
			// Now we write out the property.

		fDirty = kXMPFalse;
			// Flag our part as no longer being dirty.
	}
}



//--------------------------------------------------------------------
// HelloPart::Open
//--------------------------------------------------------------------
// This function is responsible for opening the part into a
// window if a source frame is supplied.  If no frame is
// supplied, then a window needs to be created, for sure.
// If a frame is supplied, then we may or may not need to
// create a window.

XMPID  HelloPart::Open(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::Open");

	XMPWindow*	window = kXMPNULL;

	if (frame)
		window = fSession->GetWindowState()->GetWindow(fWindowID);
			// If we are doing a View As Window...
			// ...then get the window via our window ID.

	if (!window)	// If we have no window, for whatever reason...
	{
		window = this->MakeWindow(frame);			// Make one.
		fWindowID = window->GetID();				// Save the ID.
		window->Open();		// Do stuff with it.
		window->Show();
	}

	window->Select();			// We should be pretty happy by now.

	return window->GetID();
}



//--------------------------------------------------------------------
// HelloPart::MakeWindow
//--------------------------------------------------------------------

XMPWindow*	HelloPart::MakeWindow(XMPFrame* sourceFrame)
{
	WeBeHere("\pHelloPart::MakeWindow");

	Rect				windRect;
	XMPPlatformWindow	platformWindow = kXMPNULL;
	XMPWindow*			window = kXMPNULL;
	
	if (sourceFrame)
	{
		SetRect(&windRect, 100, 100, 300, 300);	
	}
	else
	{
		const XMPSShort		kOnePageWidth = 600;
		XMPSShort			mbHeight = GetMBarHeight();
			// Get the menu bar height, for figuring the top of our windows

		SetRect(&windRect, 4, mbHeight + 20,
				XMPASLMQDGlobals.screenBits.bounds.right - 64,
				XMPASLMQDGlobals.screenBits.bounds.bottom - 4);		
		
		if (windRect.right - windRect.left > kOnePageWidth)
			windRect.right = windRect.left + kOnePageWidth;
	}

	Str255		windowTitleStr;
	XMPULong	maxsize  = 32 + sizeof(XMPName);
	XMPName*	partName = (XMPName*)NewPtr(maxsize);

	XMPGetPOName(this, partName, &maxsize);
	IntlToPStr(partName, (StringPtr)&windowTitleStr);
	DisposePtr((Ptr)partName);

	platformWindow = NewCWindow
	(
		kXMPNULL,
		&windRect,
		windowTitleStr,
		false,
		zoomDocProc,
		(WindowPtr)-1L,
		true,
		kXMPNULL
	);

	window = fSession->GetWindowState()->CreateWindow
	(
		platformWindow,
		(sourceFrame == kXMPNULL),		// Keeps draft open
		kXMPTrue,						// Is resizable
		kXMPFalse,						// Is floating
		kXMPTrue,						// should save
		this,
		kXMPNullTypeToken,
		kXMPNullTypeToken,
		sourceFrame
	);

	return window;
}



//--------------------------------------------------------------------
// HelloPart::Draw
//--------------------------------------------------------------------

void HelloPart::Draw(XMPFacet* facet, XMPShape* invalidShape)
{
	WeBeHere("\pHelloPart::Draw");
	XMPUnused(invalidShape);
	
	CFocus	qdPort(facet);
		// Set port, origin, clip, plus preserve the QDPort's
		// state for the destructor to reset, plus set the
		// QDPort to a known state.  (Good stuff, huh?)

	MoveTo(2, 12);
	TextFont(0);	// Default, default, default.  (Boring.)
	TextSize(0);
	TextFace(0);

	DrawString(fTextData);

	// Remember that when qdPort goes out of scope (here),
	// the destructor is called, which puts all of the port
	// stuff back the way it was.
}



//====================================================================
//====================================================================
// From here down we have some utility stuff to make life easier.
//====================================================================
//====================================================================



//--------------------------------------------------------------------
// IntlToPStr
//--------------------------------------------------------------------

static StringPtr IntlToPStr(XMPIText* intlText, StringPtr pstr)
{
	BlockMove(intlText->theText, &(pstr[1]), (size_t)intlText->textLength);
	pstr[0] = (unsigned char)intlText->textLength;
	return pstr;
}



//--------------------------------------------------------------------
// PLstrcpy
//--------------------------------------------------------------------

pascal StringPtr 	PLstrcpy(StringPtr d, ConstStr255Param s)
{
	short	i;

	i = *s;
	do {
		d[i] = s[i];
	} while (i--);

	return d;
}



//====================================================================
// CFocus
//====================================================================

CFocus::CFocus(XMPFacet *facet)
{
	XMPShape	*newClip;
	Point		offset;
			
	fFacetPort = (GrafPtr)facet->GetCanvas()->GetPlatformCanvas();
	newClip    = facet->GetAggregateClipShape();
	offset     = facet->GetContentTransform()->GetQDOffset();

	GetPort(&fOldPort);
	SetPort(fFacetPort);

	fOldOrigin.h = fFacetPort->portRect.left;
	fOldOrigin.v = fFacetPort->portRect.top;
	SetOrigin(-offset.h, -offset.v);
	
	GetClip(fOldClip = NewRgn());
	SetClip(newClip->GetQDRegion());
}

CFocus::~CFocus()
{
	SetPort(fFacetPort);
	SetClip(fOldClip);
	DisposeRgn(fOldClip);
	SetOrigin(fOldOrigin.h, fOldOrigin.v);
	SetPort(fOldPort);
}



//--------------------------------------------------------------------
// BeginUsingEditorResFile
//--------------------------------------------------------------------
// Use this when you need to set the CurResFile to the part's
// resource file.  It returns what the CurResFile was before,
// in case you care.  To properly balance this call, you need to
// call EndUsingEditorResFile.  This call resets the CurResFile
// to what it was prior to calling BeginUsingEditorResFile.
// Don't assume any behaviors beyond what is shown here, as the
// internal behavior may change when the conversion to SOM occurs.

short	BeginUsingEditorResFile(void)
{
	short			curRes;
	TLibraryFile*	lib;
	OSErr			err;
	long			libRes;

	curRes = CurResFile();

#ifdef ASLMBUILD
	lib = GetLocalLibraryFile();
	err = Preflight(lib, &libRes);
		// if (err) throw exception here, I guess.
	UseResFile((short)libRes);
#endif

	return curRes;
}

//--------------------------------------------------------------------
// EndUsingEditorResFile
//--------------------------------------------------------------------
// Use this balance your call to BeginUsingEditorResFile.  Note that
// the CurResFile that BeginUsingEditorResFile MUST be the CurResFile
// when you call this function.  You can change the CurResFile
// between these two calls if you want, but you need to set it back
// to what BeginUsingEditorResFile set it to prior to calling this.

void	EndUsingEditorResFile(short oldRes)
{
	long			libRes;
	TLibraryFile*	lib;

#ifdef ASLMBUILD
	libRes = CurResFile();
	lib    = GetLocalLibraryFile();
	Postflight(lib, libRes);
#endif

	UseResFile(oldRes);
}



//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//
// Below are all of the stub methods for an OpenDoc part that HelloPart
// doesn't need.  As you part grows, you will need to implement many of
// the below methods.  Use the other sample parts to determine what it is
// that you need to do.
// To help you understand what is being called when, I've added DebugStr
// statements to all of the methods in this file, in the form of the
// macro WeBeHere.  At the top of the file you will find two lines defining
// WeBeHere, one of which is commented out.  If you want break points at
// the start of each method, you may wish to enable WeBeHere.  It sometimes
// helps a lot in terms of understanding what is going on in your part.
//
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------



// STUBS:

//-------------------------------------------------------------------------
// Overridden Methods
//-------------------------------------------------------------------------

// XMPObject
XMPSize HelloPart::Purge(XMPSize size)
{
	WeBeHere("\pHelloPart::Purge");

	XMPUnused(size);

	// If HelloPart had any caches, or any uses of memory which could be
	// dumped and then regenerated later on demand, it should dispose of them
	// until either it has disposed of all such caches, or at least 'size'
	// bytes have been freed up (not necessarily contiguous).
	// After disposing, return how much space has been freed up.

	return 0;
}


void HelloPart::Release()
{
	WeBeHere("\pHelloPart::Release");

	XMPRefCntObject::Release();
	if (this->GetRefCount() == 0)
		this->GetStorageUnit()->GetDraft()->ReleasePart(this);
}



//-------------------------------------------------------------------------
// From DragAndDrop protocol
//-------------------------------------------------------------------------

// --- FulfillPromise ---

void HelloPart::FulfillPromise(XMPStorageUnitView *promiseSUView)
{
	WeBeHere("\pHelloPart::FulfillPromise");

	XMPUnused(promiseSUView);
}

// --- DropCompleted ---

void HelloPart::DropCompleted(XMPPart* destPart, XMPDropResult dropResult)
{
	WeBeHere("\pHelloPart::DropCompleted");

	XMPUnused(destPart);
	XMPUnused(dropResult);
}

// --- DragEnter ---

void HelloPart::DragEnter(XMPDragItemIterator* dragInfo,
						  XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pHelloPart::DragEnter");

	XMPUnused(dragInfo);
	XMPUnused(facet);
	XMPUnused(where);
}

// --- DragWithin ---

void HelloPart::DragWithin(XMPDragItemIterator* dragInfo,
						   XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pHelloPart::DragWithin");

	XMPUnused(dragInfo);
	XMPUnused(facet);
	XMPUnused(where);
}

// --- DragLeave ---

void HelloPart::DragLeave(XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pHelloPart::DragLeave");

	XMPUnused(facet);
	XMPUnused(where);
}

// --- Drop ---

XMPDropResult HelloPart::Drop(XMPDragItemIterator* dropInfo,
							  XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pHelloPart::Drop");

	XMPUnused(dropInfo);
	XMPUnused(facet);
	XMPUnused(where);

	return kXMPDropFail;
}



//-------------------------------------------------------------------------
// From Embedding protocol
//-------------------------------------------------------------------------

// --- ContainingPartPropertiesChanged ---

void HelloPart::ContainingPartPropertiesChanged(XMPFrame* displayFrame,
												XMPStorageUnit* propertyUnit)
{
	WeBeHere("\pHelloPart::ContainingPartPropertiesChanged");

	XMPUnused(displayFrame);
	XMPUnused(propertyUnit);
}

	// ********* for containing parts *********

// --- GetContainingPartProperties ---

XMPStorageUnit* HelloPart::GetContainingPartProperties(XMPFrame* displayFrame)
{
	WeBeHere("\pHelloPart::GetContainingPartProperties");

	XMPUnused(displayFrame);

	THROW(kXMPErrCannotEmbed);
	return nil;
}

// --- RevealFrame ---

void HelloPart::RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape)
{
	WeBeHere("\pHelloPart::RevealFrame");

	XMPUnused(embeddedFrame);
	XMPUnused(revealShape);

	THROW(kXMPErrCannotEmbed);
}

// --- EmbeddedFrameSpec ---

void HelloPart::EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec)
{
	WeBeHere("\pHelloPart::EmbeddedFrameSpec");

	XMPUnused(embeddedFrame);
	XMPUnused(spec);

	THROW(kXMPErrCannotEmbed);
}

// --- CreateEmbeddedFramesIterator ---

XMPEmbeddedFramesIterator* HelloPart::CreateEmbeddedFramesIterator(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::CreateEmbeddedFramesIterator");

	XMPUnused(frame);

	THROW(kXMPErrCannotEmbed);

	return nil;
		// return new XMPEmbeddedFramesIterator(this);
}


//-------------------------------------------------------------------------
// from Frame protocol:
//-------------------------------------------------------------------------

// --- AddDisplayFrame ---

void HelloPart::AddDisplayFrame(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::AddDisplayFrame");

	XMPUnused(frame);
}

// --- AttachSourceFrame ---

void HelloPart::AttachSourceFrame(XMPFrame* frame, XMPFrame* sourceFrame)
{
	WeBeHere("\pHelloPart::AttachSourceFrame");

	XMPUnused(frame);
	XMPUnused(sourceFrame);
}

// --- RemoveDisplayFrame ---

void HelloPart::RemoveDisplayFrame(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::RemoveDisplayFrame");

	XMPUnused(frame);
}

// --- CloseDisplayFrame ---

void HelloPart::CloseDisplayFrame(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::CloseDisplayFrame");

	XMPUnused(frame);
}

// --- FrameShapeChanged ---

void HelloPart::FrameShapeChanged(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::FrameShapeChanged");

	XMPUnused(frame);
}

// --- ViewTypeChanged ---

void HelloPart::ViewTypeChanged(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::ViewTypeChanged");

	XMPUnused(frame);
}

// --- PresentationChanged ---

void HelloPart::PresentationChanged(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::PresentationChanged");

	XMPUnused(frame);
}

// --- SequenceChanged ---

void HelloPart::SequenceChanged(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::SequenceChanged");

	XMPUnused(frame);
}

// --- WritePartInfo ---

void HelloPart::WritePartInfo(XMPPtr partInfo,
							   XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pHelloPart::WritePartInfo");

	XMPUnused(partInfo);
	XMPUnused(storageUnitView);
}

// --- ReadPartInfo ---

XMPPtr HelloPart::ReadPartInfo(XMPFrame* frame, XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pHelloPart::ReadPartInfo");

	XMPUnused(frame);
	XMPUnused(storageUnitView);

	return nil;
}

	// ********* for containing parts *********

// --- CreateEmbeddedFrame ---

XMPFrame* HelloPart::RequestEmbeddedFrame(XMPFrame* containingFrame,
										   XMPFrame* baseFrame,
										   XMPShape* frameShape,
										   XMPPart* embedPart,
										   XMPTypeToken viewType,
										   XMPTypeToken presentation,
										   XMPBoolean isOverlaid)
{
	WeBeHere("\pHelloPart::RequestEmbeddedFrame");

	XMPUnused(containingFrame);
	XMPUnused(baseFrame);
	XMPUnused(frameShape);
	XMPUnused(embedPart);
	XMPUnused(viewType);
	XMPUnused(presentation);
	XMPUnused(isOverlaid);

	THROW(kXMPErrCannotEmbed);
	return nil;
}
	
// --- RemoveEmbeddedFrame ---

void HelloPart::RemoveEmbeddedFrame(XMPFrame* embeddedFrame)
{
	WeBeHere("\pHelloPart::RemoveEmbeddedFrame");

	XMPUnused(embeddedFrame);

	THROW(kXMPErrCannotEmbed);
}

// --- RequestFrameShape ---

XMPShape* HelloPart::RequestFrameShape(XMPFrame* embeddedFrame, XMPShape* frameShape)
{
	WeBeHere("\pHelloPart::RequestFrameShape");

	XMPUnused(embeddedFrame);
	XMPUnused(frameShape);

	THROW(kXMPErrCannotEmbed);
	return nil;
}

// --- UsedShapeChanged ---

void HelloPart::UsedShapeChanged(XMPFrame* embeddedFrame)
{
	WeBeHere("\pHelloPart::UsedShapeChanged");

	XMPUnused(embeddedFrame);
}

// --- AdjustBorderShape ---

XMPShape* HelloPart::AdjustBorderShape(XMPFrame* embeddedFrame, XMPShape* shape)
{
	WeBeHere("\pHelloPart::AdjustBorderShape");

	XMPUnused(embeddedFrame);

	return shape;
}

//-------------------------------------------------------------------------
// From Facet protocol
//-------------------------------------------------------------------------

// --- FacetAdded ---

void HelloPart::FacetAdded(XMPFacet* facet)
{
	WeBeHere("\pHelloPart::FacetAdded");

	XMPUnused(facet);
}

// --- FacetRemoved ---

void HelloPart::FacetRemoved(XMPFacet* facet)
{
	WeBeHere("\pHelloPart::FacetRemoved");

	XMPUnused(facet);
}

// --- CanvasChanged ---

void HelloPart::CanvasChanged(XMPFacet* facet)
{
	WeBeHere("\pHelloPart::CanvasChanged");

	XMPUnused(facet);
}

// --- GeometryChanged ---

void HelloPart::GeometryChanged(XMPFacet* facet)
{
	WeBeHere("\pHelloPart::GeometryChanged");

	XMPUnused(facet);
}

//-------------------------------------------------------------------------
// From Imaging protocol
//-------------------------------------------------------------------------

// --- CanvasUpdated ---

void HelloPart::CanvasUpdated(XMPCanvas* canvas)
{
	WeBeHere("\pHelloPart::CanvasUpdated");

	XMPUnused(canvas);
}

// --- HighlightChanged ---

void HelloPart::HighlightChanged(XMPFacet* facet)
{
	WeBeHere("\pHelloPart::HighlightChanged");

	XMPUnused(facet);
}

// --- GetPrintResolution ---

XMPULong HelloPart::GetPrintResolution(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::GetPrintResolution");

	XMPUnused(frame);

	return 0;
}

//-------------------------------------------------------------------------
// From Linking protocol
//-------------------------------------------------------------------------

// --- CreateLink ---

XMPLink* HelloPart::CreateLink(XMPPtr data, XMPULong size)
{
	WeBeHere("\pHelloPart::CreateLink");

	XMPUnused(data);
	XMPUnused(size);

	return nil;
}

// --- LinkUpdated ---

void HelloPart::LinkUpdated(XMPLink* updatedLink, XMPChangeID id)
{
	WeBeHere("\pHelloPart::LinkUpdated");

	XMPUnused(updatedLink);
	XMPUnused(id);
}

// --- RevealLink ---

void HelloPart::RevealLink(XMPLinkSource* linkSource)
{
	WeBeHere("\pHelloPart::RevealLink");

	XMPUnused(linkSource);
}

void HelloPart::EmbeddedFrameChanged(XMPFrame* frame, XMPChangeID change)
{
	WeBeHere("\pHelloPart::EmbeddedFrameChanged");

	XMPUnused(change);
	XMPUnused(frame);
}

void HelloPart::LinkStatusChanged(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::LinkStatusChanged");

	XMPUnused(frame);
}

//-------------------------------------------------------------------------
// From Memory Management protocol
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
// From Part Activation protocol
//-------------------------------------------------------------------------

// --- BeginRelinquishFocus ---

XMPBoolean HelloPart::BeginRelinquishFocus(XMPTypeToken focus,
										   XMPFrame* ownerFrame,
										   XMPFrame* proposedFrame)
{
	WeBeHere("\pHelloPart::BeginRelinquishFocus");

	XMPUnused(focus);
	XMPUnused(ownerFrame);
	XMPUnused(proposedFrame);

	return kXMPTrue;
}
	
// --- CommitRelinquishFocus ---

void HelloPart::CommitRelinquishFocus(XMPTypeToken focus,
									   XMPFrame* ownerFrame,
									   XMPFrame* proposedFrame)
{
	WeBeHere("\pHelloPart::CommitRelinquishFocus");

	XMPUnused(focus);
	XMPUnused(ownerFrame);
	XMPUnused(proposedFrame);
}
							
// --- AbortRelinquishFocus ---

void HelloPart::AbortRelinquishFocus(XMPTypeToken focus,
									  XMPFrame* ownerFrame,
									  XMPFrame* proposedFrame)
{
	WeBeHere("\pHelloPart::AbortRelinquishFocus");

	XMPUnused(focus);
	XMPUnused(ownerFrame);
	XMPUnused(proposedFrame);
}
							  
// --- FocusAcquired ---

void HelloPart::FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	WeBeHere("\pHelloPart::FocusAcquired");

	XMPUnused(focus);
	XMPUnused(ownerFrame);
}	

// --- FocusLost ---

void HelloPart::FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	WeBeHere("\pHelloPart::FocusLost");

	XMPUnused(focus);
	XMPUnused(ownerFrame);
}

//-------------------------------------------------------------------------
// From Storage protocol
//-------------------------------------------------------------------------


// --- CloneTo ---

void HelloPart::CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU)
{
	WeBeHere("\pHelloPart::CloneInto");

	XMPStorageUnit*	su = this->GetStorageUnit();
	XMPDraft*		draft = su->GetDraft();

	XMPVolatile(draft);

	this->Externalize();
	TRY
	
		su->CloneInto(key, storageUnit, initiatingFrameSU);
		
	CATCH_ALL
	
		draft->AbortClone(key);
		
	ENDTRY
}


//-------------------------------------------------------------------------
// From Binding protocol
//-------------------------------------------------------------------------

// --- ExternalizeKinds ---

void HelloPart::ExternalizeKinds(XMPTypeList* kindList)
{
	WeBeHere("\pHelloPart::ExternalizeKinds");

	XMPUnused(kindList);

	this->Externalize();
}

// --- ChangeKind ---

void HelloPart::ChangeKind(XMPType kind)
{
	WeBeHere("\pHelloPart::ChangeKind");

	XMPUnused(kind);
}

//-------------------------------------------------------------------------
// From UI Events protocol
//-------------------------------------------------------------------------


// --- HandleEvent ---

XMPBoolean HelloPart::HandleEvent(XMPEventData event, XMPFrame* frame, XMPFacet* facet)
{
	WeBeHere("\pHelloPart::HandleEvent");

	XMPUnused(event);
	XMPUnused(frame);
	XMPUnused(facet);

	return kXMPFalse;
}

// --- HandleEventInEmbedded ---

XMPBoolean HelloPart::HandleEventInEmbedded(XMPEventData event,
											XMPFrame* frame, XMPFacet* facet,
											XMPFrame* embeddedFrame,
											XMPFacet* embeddedFacet)
{
	WeBeHere("\pHelloPart::HandleEventInEmbedded");

	XMPUnused(event);
	XMPUnused(frame);
	XMPUnused(facet);
	XMPUnused(embeddedFrame);
	XMPUnused(embeddedFacet);

	return kXMPFalse;
}

// --- MouseEnter ---

void HelloPart::MouseEnter(XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pHelloPart::MouseEnter");

	XMPUnused(facet);
	XMPUnused(where);
}

// --- MouseWithin ---

void HelloPart::MouseWithin(XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pHelloPart::MouseWithin");

	XMPUnused(facet);
	XMPUnused(where);
}

// --- MouseLeave ---

void HelloPart::MouseLeave(XMPFacet* facet)
{
	WeBeHere("\pHelloPart::MouseLeave");

	XMPUnused(facet);
}

// --- AdjustMenus ---

void HelloPart::AdjustMenus(XMPFrame* frame)
{
	WeBeHere("\pHelloPart::AdjustMenus");

	XMPUnused(frame);
}

//-------------------------------------------------------------------------
// From Undo protocol
//-------------------------------------------------------------------------

// --- UndoAction ---

void HelloPart::UndoAction(XMPActionData actionState)
{
	WeBeHere("\pHelloPart::UndoAction");

	XMPUnused(actionState);
}

// --- RedoAction ---

void HelloPart::RedoAction(XMPActionData actionState)
{
WeBeHere("\pHelloPart::RedoAction");
XMPUnused(actionState);
}

// --- DisposeAction ---

void HelloPart::DisposeActionState(XMPActionData actionState,
								   XMPDoneState doneState)
{
	WeBeHere("\pHelloPart::DisposeActionState");

	XMPUnused(actionState);
	XMPUnused(doneState);
}


// --- WriteAction ---

void HelloPart::WriteActionState(XMPPtr actionState,
								 XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pHelloPart::WriteActionState");

	XMPUnused(actionState);
	XMPUnused(storageUnitView);
}

// --- ReadAction ---

XMPPtr HelloPart::ReadActionState(XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pHelloPart::ReadActionState");

	XMPUnused(storageUnitView);

	return kXMPNULL;
}


