/*
	File:		PartMakerSample.cpp

	Contains:	PartMakerSample OpenDoc part editor class implementation

	Written by:	PartMaker
*/

#ifndef _XMPTYPES_
#include "XMPTypes.h"		// MUST BE BEFORE TOOLBOX INCLUDES
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRIPT__
#include <script.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef _ARBITRAT_
#include "Arbitrat.h"
#endif

#ifndef _CANVAS_
#include "Canvas.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef _FOCI_
#include "Foci.h"
#endif

#ifndef _INFO_
#include "Info.h"
#endif

#ifndef _PSTOBJ_
#include "PstObj.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _STDPROPS_
#include "StdProps.h"
#endif

#ifndef _STDTYPES_
#include "StdTypes.h"
#endif

#ifndef _STDDEFS_
#include "StdDefs.h"
#endif

#ifndef _CMDDEFS_
#include "CmdDefs.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _FOCUSSET_
#include "FocusSet.h"
#endif

#ifndef _FOCI_
#include "Foci.h"
#endif

#ifndef _DISPTCH_
#include "Disptch.h"
#endif

#ifndef _MENUBAR_
#include "MenuBar.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _WINSTAT_
#include "WinStat.h"
#endif

#ifndef _XMPSESSM_
#include "XMPSessM.h"
#endif

#ifndef _PARTMAKERSAMPLE_
#include "PartMakerSample.h"
#endif


#pragma segment PartMakerSample

static XMPSession* gSession;

//====================================================================
// Constants
//====================================================================

const short kContentStringID    = 1;

//====================================================================
// Local Classes
//====================================================================

class CFocus {
	public:
	CFocus(XMPFacet*);
	~CFocus();

	protected:
		CFocus()	{ }
		GrafPtr		fFacetPort;
		GrafPtr		fOldPort;
		Point		fOldOrigin;
		RgnHandle	fOldClip;
};

//====================================================================
// Function Prototypes
//====================================================================

//#define 	WeBeHere(StringPtr) DebugStr(StringPtr)
#define 	WeBeHere(StringPtr)

static pascal StringPtr 	PLstrcpy(StringPtr d, ConstStr255Param s);
static StringPtr			IntlToPStr(XMPIText* intlText, StringPtr pstr);

static short		BeginUsingEditorResFile(void);
static void			EndUsingEditorResFile(short oldRes);

//====================================================================
// PartMakerSample
//====================================================================

//--------------------------------------------------------------------
// PartMakerSample::PartMakerSample
//--------------------------------------------------------------------

PartMakerSample::PartMakerSample() 
{
	WeBeHere("\pPartMakerSample::PartMakerSample");

	fTextData[0] = 0;
	fDirty       = kXMPFalse;
	fWindowID    = 0;
	fSession     = kXMPNULL;
	gSession 	 = kXMPNULL;

	fPartsBinWin = 0;
	fPartsBinOpen = kXMPFalse;
}


//--------------------------------------------------------------------
// PartMakerSample::~PartMakerSample
//--------------------------------------------------------------------

PartMakerSample::~PartMakerSample()
{
	WeBeHere("\pPartMakerSample::~PartMakerSample");
	fMenuBar->Release();
}

//--------------------------------------------------------------------
// PartMakerSample::InitPart
//--------------------------------------------------------------------

void PartMakerSample::InitPart(XMPStorageUnit* storageUnit)
{
	WeBeHere("\pPartMakerSample::InitPart");

	XMPPersistentObject::InitPersistentObject(storageUnit);
	this->CommonInitPartMakerSample();

	storageUnit->AddProperty(kXMPPropContents)->AddValue(kPartMakerSampleKind);

	short curRes = BeginUsingEditorResFile();
	GetIndString(fTextData, kContentStringResID, kContentStringID);
	EndUsingEditorResFile(curRes);

	fDirty = true;
	this->Externalize();
}



//--------------------------------------------------------------------
// PartMakerSample::InitPartFromStorage
//--------------------------------------------------------------------

void PartMakerSample::InitPartFromStorage(XMPStorageUnit* storageUnit)
{
	WeBeHere("\pPartMakerSample::InitPartFromStorage");

	XMPPersistentObject::InitPersistentObjectFromStorage(storageUnit);

	this->CommonInitPartMakerSample();
	storageUnit->Focus(kXMPPropContents, kXMPPosSame,
					   kPartMakerSampleKind, 0, kXMPPosSame);
	fTextData[0] = (unsigned char)storageUnit->GetSize();
	storageUnit->GetValue(fTextData[0], fTextData + 1);
}



//--------------------------------------------------------------------
// PartMakerSample::CommonInitPartMakerSample
//--------------------------------------------------------------------

void PartMakerSample::CommonInitPartMakerSample()
{
	WeBeHere("\pPartMakerSample::CommonInitPartMakerSample");
	fSession = this->GetStorageUnit()->GetSession();
	gSession = fSession;

	fSelectionFocus = fSession->Tokenize(kXMPSelectionFocus);
	fMenuFocus = fSession->Tokenize(kXMPMenuFocus);
	fKeyFocus = fSession->Tokenize(kXMPKeyFocus);
	fModalFocus = fSession->Tokenize(kXMPModalFocus);

	fFocusSet = new XMPFocusSet();
	fFocusSet->InitFocusSet();
	fFocusSet->Add(fKeyFocus);
	fFocusSet->Add(fMenuFocus);
	fFocusSet->Add(fSelectionFocus);

	fMenuBar = fSession->GetWindowState()->CopyBaseMenuBar();
}

//--------------------------------------------------------------------
// PartMakerSample::Externalize
//--------------------------------------------------------------------

void	PartMakerSample::Externalize()
{
	WeBeHere("\pPartMakerSample::Externalize");

	if (fDirty)
	{
		XMPPersistentObject::Externalize();

		XMPStorageUnit* storageUnit = this->GetStorageUnit();

		storageUnit->Focus(kXMPPropContents, kXMPPosSame,
						   kPartMakerSampleKind, 0, kXMPPosSame);

		storageUnit->SetValue(fTextData[0], fTextData + 1);

		fDirty = kXMPFalse;
	}
}

//--------------------------------------------------------------------
// PartMakerSample::Open
//--------------------------------------------------------------------

XMPID  PartMakerSample::Open(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::Open");

	XMPWindow*	window = kXMPNULL;

	if (frame)
		window = fSession->GetWindowState()->GetWindow(fWindowID);

	if (!window)
	{
		window = this->MakeWindow(frame);
		fWindowID = window->GetID();
		window->Open();
		window->Show();
	}

	window->Select();

	return window->GetID();
}



//--------------------------------------------------------------------
// PartMakerSample::MakeWindow
//--------------------------------------------------------------------

XMPWindow*	PartMakerSample::MakeWindow(XMPFrame* sourceFrame)
{
	WeBeHere("\pPartMakerSample::MakeWindow");

	Rect				windRect;
	XMPPlatformWindow	platformWindow = kXMPNULL;
	XMPWindow*			window = kXMPNULL;
	
	if (sourceFrame)
	{
		SetRect(&windRect, 100, 100, 300, 300);	
	}
	else
	{
		const XMPSShort		kOnePageWidth = 600;
		XMPSShort			mbHeight = GetMBarHeight();
			// Get the menu bar height, for figuring the top of our windows

		SetRect(&windRect, 4, mbHeight + 20,
				XMPASLMQDGlobals.screenBits.bounds.right - 64,
				XMPASLMQDGlobals.screenBits.bounds.bottom - 4);		
		
		if (windRect.right - windRect.left > kOnePageWidth)
			windRect.right = windRect.left + kOnePageWidth;
	}

	Str255		windowTitleStr;
	XMPULong	maxsize  = 32 + sizeof(XMPName);
	XMPName*	partName = (XMPName*)NewPtr(maxsize);

	XMPGetPOName(this, partName, &maxsize);
	IntlToPStr(partName, (StringPtr)&windowTitleStr);
	DisposePtr((Ptr)partName);

	platformWindow = NewCWindow
	(
		kXMPNULL,
		&windRect,
		windowTitleStr,
		false,
		zoomDocProc,
		(WindowPtr)-1L,
		true,
		kXMPNULL
	);

	window = fSession->GetWindowState()->CreateWindow
	(
		platformWindow,
		(sourceFrame == kXMPNULL),
		kXMPTrue,
		kXMPFalse,
		kXMPTrue,
		this,
		kXMPNullTypeToken,
		kXMPNullTypeToken,
		sourceFrame
	);

	return window;
}

//--------------------------------------------------------------------
// PartMakerSample::Draw
//--------------------------------------------------------------------

void PartMakerSample::Draw(XMPFacet* facet, XMPShape* invalidShape)
{
	WeBeHere("\pPartMakerSample::Draw");
XMPUnused(invalidShape);
	
	CFocus	qdPort(facet);

	MoveTo(2, 12);
	TextFont(0);
	TextSize(0);
	TextFace(0);

	DrawString(fTextData);
}

//====================================================================
// PartsBin
//====================================================================

void PartMakerSample::ShowPartsBin(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::ShowPartsBin");
	XMPPart*			partsbin;
	XMPStorageUnit*		su = this->GetStorageUnit();
	XMPStorageUnit*		pbSU;
	XMPStorageUnitRef	pbSURef;
	XMPStorageUnitID	pbSUID;
	
	if (!su->Exists(kXMPPropPartsBin,kXMPStrongStorageUnitRef,0))
	{
		su->AddProperty(kXMPPropPartsBin)->AddValue(kXMPStrongStorageUnitRef);

		partsbin = su->GetDraft()->CreatePart(kXMPKindPartsBin, kXMPNULL);
		ASSERTM(partsbin != kXMPNULL, kXMPErrCannotCreatePart, "Cannot Create Parts Bin");
		
		if (partsbin!=kXMPNULL)
		{
			pbSU = partsbin->GetStorageUnit();
			pbSURef = su->GetStrongStorageUnitRef(pbSU);
			su->SetValue(sizeof(kXMPStrongStorageUnitRef), (XMPValue)&pbSURef);
		}
	}
	else
	{
		su->Focus(kXMPPropPartsBin,kXMPPosSame,kXMPStrongStorageUnitRef,1,kXMPPosFirstSib);
		su->SetOffset(0);
		su->GetValue(sizeof(kXMPStrongStorageUnitRef), (XMPValue)&pbSURef);
		
		pbSUID = su->GetIDFromStorageUnitRef(pbSURef);
		partsbin = su->GetDraft()->GetPart(pbSUID);
	}
	
	fPartsBinWin = partsbin->Open(frame);
	partsbin->Release();
	fPartsBinOpen = kXMPTrue;
}

void PartMakerSample::HidePartsBin()
{
	WeBeHere("\pPartMakerSample::HidePartsBin");
	if (fPartsBinOpen && fPartsBinWin)
	{
		XMPWindow* pbWin = fSession->GetWindowState()->GetWindow(fPartsBinWin);
		ASSERTM(pbWin != kXMPNULL, kXMPErrCannotGetWindow, "Cannot Hide Parts Bin");

		if (pbWin!=kXMPNULL) 
			pbWin->Hide();
		fPartsBinOpen = kXMPFalse;
	}
}

//--------------------------------------------------------------------
// MyDialogFilter
//--------------------------------------------------------------------

pascal Boolean MyDialogFilter(DialogPtr dialog,
							  EventRecord* event,
							  short* itemHit)
{
	XMPBoolean handled = kXMPFalse;
	
	if (event->what == nullEvent ||
		event->what == updateEvt && (WindowPtr) event->message != dialog ||
		event->what == activateEvt && (WindowPtr) event->message != dialog)
	{
		gSession->GetDispatcher()->Dispatch(event);
	}
	else if ((event->what == keyDown) || (event->what == autoKey))
	{
		#define kMyButtonDelay 8
		long	waitTicks;
		short	itemKind;
		Handle	itemHandle;
		Rect	itemRect;
		char key = (char)(event->message & charCodeMask);
		const char kReturnKey = 13;
		const char kEnterKey = 3;
		const char kEscKey = 27;
		
		switch(key)
		{
			case kReturnKey:
			case kEnterKey:
				/* change whatever is the current item is to the OK item */
				*itemHit = ok;
				GetDItem(dialog,ok,&itemKind,&itemHandle,&itemRect);
				HiliteControl((ControlHandle)itemHandle,inButton);
				Delay(kMyButtonDelay,&waitTicks);/* wait about 8 ticks */
				HiliteControl((ControlHandle)itemHandle,kXMPFalse);
				handled = kXMPTrue;
				break;
			case kEscKey:
				*itemHit = cancel;
				GetDItem(dialog,cancel,&itemKind,&itemHandle,&itemRect);
				HiliteControl((ControlHandle)itemHandle,inButton);
				Delay(kMyButtonDelay,&waitTicks);
				HiliteControl((ControlHandle)itemHandle,kXMPFalse);
				handled = kXMPTrue;
				break;
		}
	}

	return handled;
}

//--------------------------------------------------------------------
// PartMakerSample::HandleMenuEvent
//--------------------------------------------------------------------

XMPBoolean PartMakerSample::HandleMenuEvent(XMPFrame*		frame,
										XMPEventData 	event)
{
	WeBeHere("\pPartMakerSample::HandleMenuEvent");
XMPUnused(frame);
	long menuResult = event->message;
	short menu = HiWord(menuResult);
	short item = LoWord(menuResult);
	XMPCommandID command = fMenuBar->GetCommand(menu, item);
	
	if (command == kXMPCommandAbout) 
	{
		short curRes = BeginUsingEditorResFile();

		if (fSession->GetArbitrator()->RequestFocus(fModalFocus,frame))
		{
			ParamText("\pPartMakerSample Part",0,0,0);
			SetCursor(&XMPASLMQDGlobals.arrow);
			ModalFilterUPP modalFilter = NewModalFilterProc(MyDialogFilter);
			Alert(kPartMakerSampleAboutBoxID, modalFilter);
			DisposeRoutineDescriptor(modalFilter);
			fSession->GetArbitrator()->RelinquishFocus(fModalFocus,frame);
		}
		else
			SysBeep(2);
		
		EndUsingEditorResFile(curRes);
	}
	else if (command == kXMPCommandShowPartsBin)
	{
		if (fPartsBinOpen)
			this->HidePartsBin();
		else
			this->ShowPartsBin(frame);
	}
	else if (command == kXMPCommandViewAsWin)
	{
		this->Open(frame);
	}
	else if (command == kXMPCommandGetPartInfo) 
	{
		ShowPartFrameInfo(frame);
	}
	else
	{			
		return kXMPFalse;
	}
	
	return kXMPTrue;
}

//------------------------------------------------------------------------------
// PartMakerSample::HandleMouseDown
//------------------------------------------------------------------------------

XMPBoolean PartMakerSample::HandleMouseDown( XMPEventData event, XMPFrame* frame, XMPFacet* facet )
{
	WeBeHere("\pPartMakerSample::HandleMouseDown");
XMPUnused(frame);
	XMPBoolean tWasHandled = kXMPFalse;
	
	if (facet == kXMPNULL) // Outside modal focus
	{
		SysBeep(2);
		tWasHandled = kXMPTrue;
	}
	else if (IsDialogEvent(event))
	{
		short itemHit;
		DialogPtr dialog;
		tWasHandled = DialogSelect(event, &dialog, &itemHit);
		XMPWindow* window = fSession->GetWindowState()->GetXMPWindow(dialog);
	}
	else
	{
		XMPBoolean tWasHandled = kXMPTrue;
		if (!facet->GetWindow()->IsActive())
			facet->GetWindow()->Select();
		else
		{
			XMPFrame* frame = facet->GetFrame();
			XMPBoolean succeeded = kXMPFalse;
			succeeded = fSession->GetArbitrator()->RequestFocusSet(fFocusSet,frame);
			if (succeeded)
			{
				this->FocusAcquired(fSelectionFocus, frame);
				this->FocusAcquired(fMenuFocus, frame);
				this->FocusAcquired(fKeyFocus, frame);
			}
		}
	}
	return tWasHandled;
}

//-------------------------------------------------------------------------
// UI Events protocol
//-------------------------------------------------------------------------

XMPBoolean PartMakerSample::HandleEvent(XMPEventData event, XMPFrame* frame, XMPFacet* facet)
{
	WeBeHere("\pPartMakerSample::HandleEvent");

	XMPBoolean	tWasHandled = kXMPFalse;
	
	switch ( event->what ) {
		case nullEvent:
			if (IsDialogEvent(event))
			{
				short itemHit;
				DialogPtr dialog;
				tWasHandled = DialogSelect(event, &dialog, &itemHit);
				return tWasHandled;
			}
			tWasHandled = kXMPTrue;
			break;
			
		case mouseDown:
			tWasHandled = this->HandleMouseDown(event, frame, facet);
			break;
			
		case kXMPEvtMouseDownEmbedded:
		case mouseUp:
		case keyDown:
		case autoKey:
		case activateEvt:
			tWasHandled = true;
			break;
		case kXMPEvtMenu:
			tWasHandled = this->HandleMenuEvent( frame, event );
			break;
		default:
			break;
	}	
	return tWasHandled;
}


XMPBoolean PartMakerSample::HandleEventInEmbedded(XMPEventData event,
											XMPFrame* frame, XMPFacet* facet,
											XMPFrame* embeddedFrame,
											XMPFacet* embeddedFacet)
{
	WeBeHere("\pPartMakerSample::HandleEventInEmbedded");
XMPUnused(event);
XMPUnused(frame);
XMPUnused(facet);
XMPUnused(embeddedFrame);
XMPUnused(embeddedFacet);
	return kXMPFalse;
}

void PartMakerSample::MouseEnter(XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pPartMakerSample::MouseEnter");
XMPUnused(facet);
XMPUnused(where);
}


void PartMakerSample::MouseWithin(XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pPartMakerSample::MouseWithin");
XMPUnused(facet);
XMPUnused(where);
}


void PartMakerSample::MouseLeave(XMPFacet* facet)
{
	WeBeHere("\pPartMakerSample::MouseLeave");
XMPUnused(facet);
}


void PartMakerSample::AdjustMenus(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::AdjustMenus");
XMPUnused(frame);
	fMenuBar->EnableCommand(kXMPCommandViewAsWin, !frame->IsRoot());
	fMenuBar->EnableCommand(kXMPCommandGetPartInfo, kXMPTrue);
	fMenuBar->EnableCommand(kXMPCommandShowPartsBin, kXMPTrue);
	fMenuBar->SetItemString(kXMPCommandAbout, "\pAbout PartMakerSample");
	
	if (fPartsBinWin)
	{
		XMPWindow* window = fSession->GetWindowState()->GetWindow(fPartsBinWin);
		if (window) 
			fPartsBinOpen = window->IsShown();
		else
			fPartsBinOpen = kXMPFalse;
	}
	
	if (fPartsBinOpen)
		fMenuBar->SetItemString(kXMPCommandShowPartsBin, "\pHide Parts Bin");
	else
		fMenuBar->SetItemString(kXMPCommandShowPartsBin, "\pShow Parts Bin");
}


//====================================================================
// Static Functions
//====================================================================
//--------------------------------------------------------------------
// IntlToPStr
//--------------------------------------------------------------------

static StringPtr IntlToPStr(XMPIText* intlText, StringPtr pstr)
{
	BlockMove(intlText->theText, &(pstr[1]), (size_t)intlText->textLength);
	pstr[0] = (unsigned char)intlText->textLength;
	return pstr;
}

//--------------------------------------------------------------------
// PLstrcpy
//--------------------------------------------------------------------

pascal StringPtr 	PLstrcpy(StringPtr d, ConstStr255Param s)
{
	short	i;

	i = *s;
	do {
		d[i] = s[i];
	} while (i--);

	return d;
}

//--------------------------------------------------------------------
// BeginUsingEditorResFile
//--------------------------------------------------------------------

short	BeginUsingEditorResFile(void)
{
	short			curRes;
	TLibraryFile*	lib;
	OSErr			err;
	long			libRes;

	curRes = CurResFile();

#ifdef ASLMBUILD
	lib = GetLocalLibraryFile();
	err = Preflight(lib, &libRes);
		// if (err) throw exception here, I guess.
	UseResFile((short)libRes);
#endif

	return curRes;
}

//--------------------------------------------------------------------
// EndUsingEditorResFile
//--------------------------------------------------------------------

void	EndUsingEditorResFile(short oldRes)
{
	long			libRes;
	TLibraryFile*	lib;

#ifdef ASLMBUILD
	libRes = CurResFile();
	lib    = GetLocalLibraryFile();
	Postflight(lib, libRes);
#endif

	UseResFile(oldRes);
}

//====================================================================
// CFocus
//====================================================================

CFocus::CFocus(XMPFacet *facet)
{
	XMPShape	*newClip;
	Point		offset;
			
	fFacetPort = (GrafPtr)facet->GetCanvas()->GetPlatformCanvas();
	newClip    = facet->GetAggregateClipShape();
	offset     = facet->GetContentTransform()->GetQDOffset();

	GetPort(&fOldPort);
	SetPort(fFacetPort);

	fOldOrigin.h = fFacetPort->portRect.left;
	fOldOrigin.v = fFacetPort->portRect.top;
	SetOrigin(-offset.h, -offset.v);
	
	GetClip(fOldClip = NewRgn());
	SetClip(newClip->GetQDRegion());
}

CFocus::~CFocus()
{
	SetPort(fFacetPort);
	SetClip(fOldClip);
	DisposeRgn(fOldClip);
	SetOrigin(fOldOrigin.h, fOldOrigin.v);
	SetPort(fOldPort);
}

//-------------------------------------------------------------------------
// PartMakerSample stub methods
//-------------------------------------------------------------------------

XMPSize PartMakerSample::Purge(XMPSize size)
{
XMPUnused(size);
	WeBeHere("\pPartMakerSample::Purge");
	return 0;
}

void PartMakerSample::Release()
{
	WeBeHere("\pPartMakerSample::Release");
	XMPRefCntObject::Release();
	if (this->GetRefCount() == 0)
		this->GetStorageUnit()->GetDraft()->ReleasePart(this);
}

//-------------------------------------------------------------------------
// DragAndDrop protocol
//-------------------------------------------------------------------------

void PartMakerSample::FulfillPromise(XMPStorageUnitView *promiseSUView)
{
	WeBeHere("\pPartMakerSample::FulfillPromise");
XMPUnused(promiseSUView);
}

void PartMakerSample::DropCompleted(XMPPart* destPart, XMPDropResult dropResult)
{
	WeBeHere("\pPartMakerSample::DropCompleted");
XMPUnused(destPart);
XMPUnused(dropResult);
}

void PartMakerSample::DragEnter(XMPDragItemIterator* dragInfo,
						  XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pPartMakerSample::DragEnter");
XMPUnused(dragInfo);
XMPUnused(facet);
XMPUnused(where);
}

void PartMakerSample::DragWithin(XMPDragItemIterator* dragInfo,
						   XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pPartMakerSample::DragWithin");
XMPUnused(dragInfo);
XMPUnused(facet);
XMPUnused(where);
}

void PartMakerSample::DragLeave(XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pPartMakerSample::DragLeave");
XMPUnused(facet);
XMPUnused(where);
}

XMPDropResult PartMakerSample::Drop(XMPDragItemIterator* dropInfo,
							  XMPFacet* facet, XMPPoint where)
{
	WeBeHere("\pPartMakerSample::Drop");
XMPUnused(dropInfo);
XMPUnused(facet);
XMPUnused(where);

	return kXMPDropFail;
}


//-------------------------------------------------------------------------
// Embedding protocol
//-------------------------------------------------------------------------

void PartMakerSample::ContainingPartPropertiesChanged(XMPFrame* displayFrame,
												XMPStorageUnit* propertyUnit)
{
	WeBeHere("\pPartMakerSample::ContainingPartPropertiesChanged");
XMPUnused(displayFrame);
XMPUnused(propertyUnit);
}

XMPStorageUnit* PartMakerSample::GetContainingPartProperties(XMPFrame* displayFrame)
{
	WeBeHere("\pPartMakerSample::GetContainingPartProperties");
XMPUnused(displayFrame);
	THROW(kXMPErrCannotEmbed);
	return kXMPNULL;
}

void PartMakerSample::RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape)
{
	WeBeHere("\pPartMakerSample::RevealFrame");
XMPUnused(embeddedFrame);
XMPUnused(revealShape);
	THROW(kXMPErrCannotEmbed);
}

void PartMakerSample::EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec)
{
	WeBeHere("\pPartMakerSample::EmbeddedFrameSpec");
XMPUnused(embeddedFrame);
XMPUnused(spec);
	THROW(kXMPErrCannotEmbed);
}

XMPEmbeddedFramesIterator* PartMakerSample::CreateEmbeddedFramesIterator(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::CreateEmbeddedFramesIterator");
XMPUnused(frame);
	THROW(kXMPErrCannotEmbed);
	return kXMPNULL;
}


//-------------------------------------------------------------------------
// Frame protocol
//-------------------------------------------------------------------------

void PartMakerSample::AddDisplayFrame(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::AddDisplayFrame");
XMPUnused(frame);
}

void PartMakerSample::AttachSourceFrame(XMPFrame* frame, XMPFrame* sourceFrame)
{
	WeBeHere("\pPartMakerSample::AttachSourceFrame");
XMPUnused(frame);
XMPUnused(sourceFrame);
}

void PartMakerSample::RemoveDisplayFrame(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::RemoveDisplayFrame");
XMPUnused(frame);
}

void PartMakerSample::CloseDisplayFrame(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::CloseDisplayFrame");
XMPUnused(frame);
}

void PartMakerSample::FrameShapeChanged(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::FrameShapeChanged");
XMPUnused(frame);
}

void PartMakerSample::ViewTypeChanged(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::ViewTypeChanged");
XMPUnused(frame);
}

void PartMakerSample::PresentationChanged(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::PresentationChanged");
XMPUnused(frame);
}

void PartMakerSample::SequenceChanged(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::SequenceChanged");
XMPUnused(frame);
}

void PartMakerSample::WritePartInfo(XMPPtr partInfo,
							   XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pPartMakerSample::WritePartInfo");
XMPUnused(partInfo);
XMPUnused(storageUnitView);
}

XMPPtr PartMakerSample::ReadPartInfo(XMPFrame* frame, XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pPartMakerSample::ReadPartInfo");
XMPUnused(frame);
XMPUnused(storageUnitView);
	return kXMPNULL;
}

XMPFrame* PartMakerSample::RequestEmbeddedFrame(XMPFrame* containingFrame,
										   XMPFrame* baseFrame,
										   XMPShape* frameShape,
										   XMPPart* embedPart,
										   XMPTypeToken viewType,
										   XMPTypeToken presentation,
										   XMPBoolean isOverlaid)
{
	WeBeHere("\pPartMakerSample::RequestEmbeddedFrame");
XMPUnused(containingFrame);
XMPUnused(baseFrame);
XMPUnused(frameShape);
XMPUnused(embedPart);
XMPUnused(viewType);
XMPUnused(presentation);
XMPUnused(isOverlaid);
	THROW(kXMPErrCannotEmbed);
	return kXMPNULL;
}
	
void PartMakerSample::RemoveEmbeddedFrame(XMPFrame* embeddedFrame)
{
	WeBeHere("\pPartMakerSample::RemoveEmbeddedFrame");
XMPUnused(embeddedFrame);
	THROW(kXMPErrCannotEmbed);
}

XMPShape* PartMakerSample::RequestFrameShape(XMPFrame* embeddedFrame, XMPShape* frameShape)
{
	WeBeHere("\pPartMakerSample::RequestFrameShape");
XMPUnused(embeddedFrame);
XMPUnused(frameShape);
	THROW(kXMPErrCannotEmbed);
	return kXMPNULL;
}

void PartMakerSample::UsedShapeChanged(XMPFrame* embeddedFrame)
{
	WeBeHere("\pPartMakerSample::UsedShapeChanged");
XMPUnused(embeddedFrame);
}

XMPShape* PartMakerSample::AdjustBorderShape(XMPFrame* embeddedFrame, XMPShape* shape)
{
	WeBeHere("\pPartMakerSample::AdjustBorderShape");
XMPUnused(embeddedFrame);
	return shape;
}

//-------------------------------------------------------------------------
// Facet protocol
//-------------------------------------------------------------------------

void PartMakerSample::FacetAdded(XMPFacet* facet)
{
	WeBeHere("\pPartMakerSample::FacetAdded");
	XMPFrame* frame = facet->GetFrame();
	if (frame->IsRoot()) 
	{
		XMPBoolean succeeded = kXMPFalse;
		succeeded = fSession->GetArbitrator()->RequestFocusSet(fFocusSet,frame);
		if (succeeded)
		{
			this->FocusAcquired(fSelectionFocus, frame);
			this->FocusAcquired(fMenuFocus, frame);
			this->FocusAcquired(fKeyFocus, frame);
		}
	}
}

void PartMakerSample::FacetRemoved(XMPFacet* facet)
{
	WeBeHere("\pPartMakerSample::FacetRemoved");
XMPUnused(facet);
}

void PartMakerSample::CanvasChanged(XMPFacet* facet)
{
	WeBeHere("\pPartMakerSample::CanvasChanged");
XMPUnused(facet);
}

void PartMakerSample::GeometryChanged(XMPFacet* facet)
{
	WeBeHere("\pPartMakerSample::GeometryChanged");
XMPUnused(facet);
}

//-------------------------------------------------------------------------
// Imaging protocol
//-------------------------------------------------------------------------

void PartMakerSample::CanvasUpdated(XMPCanvas* canvas)
{
	WeBeHere("\pPartMakerSample::CanvasUpdated");
XMPUnused(canvas);
}

void PartMakerSample::HighlightChanged(XMPFacet* facet)
{
	WeBeHere("\pPartMakerSample::HighlightChanged");
XMPUnused(facet);
}

XMPULong PartMakerSample::GetPrintResolution(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::GetPrintResolution");
XMPUnused(frame);
	return 0;
}

//-------------------------------------------------------------------------
// From Linking protocol
//-------------------------------------------------------------------------

XMPLink* PartMakerSample::CreateLink(XMPPtr data, XMPULong size)
{
	WeBeHere("\pPartMakerSample::CreateLink");
XMPUnused(data);
XMPUnused(size);
	return kXMPNULL;
}

void PartMakerSample::LinkUpdated(XMPLink* updatedLink, XMPChangeID id)
{
	WeBeHere("\pPartMakerSample::LinkUpdated");
XMPUnused(updatedLink);
XMPUnused(id);
}


void PartMakerSample::RevealLink(XMPLinkSource* linkSource)
{
	WeBeHere("\pPartMakerSample::RevealLink");
XMPUnused(linkSource);
}

void PartMakerSample::EmbeddedFrameChanged(XMPFrame* frame, XMPChangeID change)
{
	WeBeHere("\pPartMakerSample::EmbeddedFrameChanged");
XMPUnused(change);
XMPUnused(frame);
}

void PartMakerSample::LinkStatusChanged(XMPFrame* frame)
{
	WeBeHere("\pPartMakerSample::LinkStatusChanged");
XMPUnused(frame);
}


//-------------------------------------------------------------------------
// Part Activation protocol
//-------------------------------------------------------------------------

XMPBoolean PartMakerSample::BeginRelinquishFocus(XMPTypeToken focus,
										   XMPFrame* ownerFrame,
										   XMPFrame* proposedFrame)
{
	WeBeHere("\pPartMakerSample::BeginRelinquishFocus");
XMPUnused(ownerFrame);
	if ((focus == fModalFocus) && (proposedFrame->GetPart() != this))
		return kXMPFalse;
	else
		return kXMPTrue;
}
	
void PartMakerSample::CommitRelinquishFocus(XMPTypeToken focus,
									   XMPFrame* ownerFrame,
									   XMPFrame* proposedFrame)
{
	WeBeHere("\pPartMakerSample::CommitRelinquishFocus");
	if (proposedFrame->GetPart() != this)
		this->FocusLost(focus, ownerFrame);
}
							
void PartMakerSample::AbortRelinquishFocus(XMPTypeToken focus,
									  XMPFrame* ownerFrame,
									  XMPFrame* proposedFrame)
{
	WeBeHere("\pPartMakerSample::AbortRelinquishFocus");
XMPUnused(focus);
XMPUnused(ownerFrame);
XMPUnused(proposedFrame);
}
							  
void PartMakerSample::FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	WeBeHere("\pPartMakerSample::FocusAcquired");
	if (focus == fMenuFocus) 
		if ( ownerFrame && fMenuBar)
		{	
			fMenuBar->Display();
		}
}	

void PartMakerSample::FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	WeBeHere("\pPartMakerSample::FocusLost");
XMPUnused(ownerFrame);
	if (focus == fMenuFocus) 
	{
	}
}

//-------------------------------------------------------------------------
// Storage protocol
//-------------------------------------------------------------------------

void PartMakerSample::CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU)
{
	WeBeHere("\pPartMakerSample::CloneInto");

	XMPStorageUnit*	su = this->GetStorageUnit();
	XMPDraft*		draft = su->GetDraft();

	XMPVolatile(draft);

	this->Externalize();
	
	TRY
		su->CloneInto(key, storageUnit, initiatingFrameSU);
	CATCH_ALL
		draft->AbortClone(key);
	ENDTRY
}


//-------------------------------------------------------------------------
// Binding protocol
//-------------------------------------------------------------------------

void PartMakerSample::ExternalizeKinds(XMPTypeList* kindList)
{
	WeBeHere("\pPartMakerSample::ExternalizeKinds");
XMPUnused(kindList);
	this->Externalize();
}

void PartMakerSample::ChangeKind(XMPType kind)
{
	WeBeHere("\pPartMakerSample::ChangeKind");
XMPUnused(kind);
}

//-------------------------------------------------------------------------
// Undo protocol
//-------------------------------------------------------------------------

void PartMakerSample::UndoAction(XMPActionData actionState)
{
	WeBeHere("\pPartMakerSample::UndoAction");
XMPUnused(actionState);
}

void PartMakerSample::RedoAction(XMPActionData actionState)
{
	WeBeHere("\pPartMakerSample::RedoAction");
XMPUnused(actionState);
}

void PartMakerSample::DisposeActionState(XMPActionData actionState,
								   XMPDoneState doneState)
{
	WeBeHere("\pPartMakerSample::DisposeActionState");
XMPUnused(actionState);
XMPUnused(doneState);
}

void PartMakerSample::WriteActionState(XMPPtr actionState,
								 XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pPartMakerSample::WriteActionState");
XMPUnused(actionState);
XMPUnused(storageUnitView);
}

XMPPtr PartMakerSample::ReadActionState(XMPStorageUnitView* storageUnitView)
{
	WeBeHere("\pPartMakerSample::ReadActionState");
XMPUnused(storageUnitView);
	return kXMPNULL;
}
