/*
	File:		PartMakerSample.h

	Contains:	PartMakerSample OpenDoc part editor class definition

	Written by:	PartMaker

*/

#ifndef _PARTMAKERSAMPLE_
#define _PARTMAKERSAMPLE_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _PART_
#include "Part.h"
#endif

#ifndef _PARTMAKERSAMPLEDEF_
#include "PartMakerSampleDef.h"
#endif


//==============================================================================
// PartMakerSample
//==============================================================================

class PartMakerSample : public XMPPart
{
public:
						PartMakerSample();
	virtual				~PartMakerSample();
	virtual void		Draw(XMPFacet* facet, XMPShape* invalidShape);
	virtual XMPID		Open(XMPFrame* frame);
	virtual void		InitPart(XMPStorageUnit* storageUnit);
	virtual void		InitPartFromStorage(XMPStorageUnit* storageUnit);
	virtual void		Externalize();

	//  UI Events 
	virtual XMPBoolean	HandleEvent(XMPEventData event, XMPFrame* frame,
									XMPFacet* facet);
	virtual XMPBoolean	HandleEventInEmbedded(XMPEventData event,
											  XMPFrame* frame,
											  XMPFacet* facet,
											  XMPFrame* embeddedFrame,
											  XMPFacet* embeddedFacet);
	virtual void		MouseEnter(XMPFacet* facet, XMPPoint where);
	virtual void		MouseWithin(XMPFacet* facet, XMPPoint where);
	virtual void		MouseLeave(XMPFacet* facet);
	virtual void		AdjustMenus(XMPFrame* frame);

protected:

	void				CommonInitPartMakerSample();
	XMPWindow*			MakeWindow(XMPFrame* sourceFrame);

	virtual void		ShowPartsBin(XMPFrame* frame);
	virtual void		HidePartsBin();
	XMPBoolean			HandleMenuEvent(XMPFrame*		frame,
										XMPEventData 	event);
	XMPBoolean			HandleMouseDown(XMPEventData event,
									 	XMPFrame* frame,
									 	XMPFacet* facet);
private:

	Str255				fTextData;		// All fields are initialized
	Boolean				fDirty;			// to something by constructor,
	XMPID				fWindowID;		// even those that are also
										// initialized elsewhere.

	XMPSession*			fSession;

	XMPID				fPartsBinWin;
	XMPBoolean			fPartsBinOpen;

	XMPFocusSet* 		fFocusSet;
	XMPTypeToken		fSelectionFocus;
	XMPTypeToken		fMenuFocus;
	XMPTypeToken		fKeyFocus;
	XMPTypeToken		fModalFocus;
	
	XMPMenuBar* 		fMenuBar;

public:
// Stub Overrides

	virtual XMPSize		Purge(XMPSize size);

	// XMPRefCntObject
	XMPMethod void		Release();
	
	//  DragAndDrop 
	virtual void			FulfillPromise(XMPStorageUnitView *promiseSUView);
	virtual void			DropCompleted(XMPPart* destPart,
										  XMPDropResult dropResult);
	virtual void			DragEnter(XMPDragItemIterator* dragInfo,
									  XMPFacet* facet, XMPPoint where);
	virtual void			DragWithin(XMPDragItemIterator* dragInfo,
									   XMPFacet* facet, XMPPoint where);
	virtual void			DragLeave(XMPFacet* facet, XMPPoint where);
	virtual XMPDropResult	Drop(XMPDragItemIterator* dropInfo,
								 XMPFacet* facet, XMPPoint where);

	//  Embedding 
	virtual void			ContainingPartPropertiesChanged(XMPFrame* frame,
											XMPStorageUnit* propertyUnit);
	virtual XMPStorageUnit*	GetContainingPartProperties(XMPFrame* frame);
	virtual void			RevealFrame(XMPFrame* embeddedFrame,
										XMPShape* revealShape);
	virtual void			EmbeddedFrameSpec(XMPFrame* embeddedFrame,
											  XMPObjectSpec* spec);
	virtual XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(
											XMPFrame* frame);

	//  Frame 
	virtual void		AddDisplayFrame(XMPFrame* frame);
	virtual void		AttachSourceFrame(XMPFrame* frame,
										  XMPFrame* sourceFrame);
	virtual void		RemoveDisplayFrame(XMPFrame* frame);
	virtual void		CloseDisplayFrame(XMPFrame* frame);
	virtual void		FrameShapeChanged(XMPFrame* frame);
	virtual void		ViewTypeChanged(XMPFrame* frame);
	virtual void		PresentationChanged(XMPFrame* frame);
	virtual void		SequenceChanged(XMPFrame* frame);
	virtual void		WritePartInfo(XMPPtr partInfo,
									  XMPStorageUnitView* storageUnitView);
	virtual XMPPtr		ReadPartInfo(XMPFrame* frame,
									 XMPStorageUnitView* storageUnitView);
	virtual XMPFrame*	RequestEmbeddedFrame(XMPFrame* containingFrame,
											 XMPFrame* baseFrame,
											 XMPShape* frameShape,
											 XMPPart* embedPart,
											 XMPTypeToken viewType,
											 XMPTypeToken presentation,
											 XMPBoolean isOverlaid);
	virtual void		RemoveEmbeddedFrame(XMPFrame* embeddedFrame);
	virtual XMPShape*	RequestFrameShape(XMPFrame* embeddedFrame,
										  XMPShape* frameShape);
	virtual void		UsedShapeChanged(XMPFrame* embeddedFrame);
	virtual XMPShape*	AdjustBorderShape(XMPFrame* embeddedFrame,
										  XMPShape* shape);

	//  Facet 
	virtual void		FacetAdded(XMPFacet* facet);
	virtual void		FacetRemoved(XMPFacet* facet);
	virtual void		CanvasChanged(XMPFacet* facet);
	virtual void		GeometryChanged(XMPFacet* facet);

	//  Imaging 
	virtual void		CanvasUpdated(XMPCanvas* canvas);
	virtual void		HighlightChanged(XMPFacet* facet);
	virtual XMPULong	GetPrintResolution(XMPFrame* frame);

	//  Linking 
	virtual XMPLink*	CreateLink(XMPPtr data, XMPULong size);
	virtual void		LinkUpdated(XMPLink* updatedLink, XMPChangeID id);
	virtual void		RevealLink(XMPLinkSource* linkSource);
	virtual void		EmbeddedFrameChanged(XMPFrame* frame,
											 XMPChangeID change);
	virtual void		LinkStatusChanged(XMPFrame* frame);

	//  Memory Management 

	//  Part Activation 
	virtual XMPBoolean	BeginRelinquishFocus(XMPTypeToken focus,
											 XMPFrame* ownerFrame,
											 XMPFrame* proposedFrame);
	virtual void		CommitRelinquishFocus(XMPTypeToken focus, 
											  XMPFrame* ownerFrame,
											  XMPFrame* proposedFrame);
	virtual void		AbortRelinquishFocus(XMPTypeToken focus,
											 XMPFrame* ownerFrame,
											 XMPFrame* proposedFrame);
	virtual void		FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame);
	virtual void		FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame);

	//  Storage 
	virtual void		CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit,
								  XMPStorageUnit* initiatingFrameSU);

	//  Binding 
	virtual void		ExternalizeKinds(XMPTypeList* kindList);
	virtual void		ChangeKind(XMPType kind);

	//  Undo 
	virtual void		UndoAction(XMPActionData actionState);
	virtual void		RedoAction(XMPActionData actionState);
	virtual void		DisposeActionState(XMPActionData actionState,
										   XMPDoneState doneState);
	virtual void		WriteActionState(XMPPtr actionState,
										 XMPStorageUnitView* storageUnitView);
	virtual XMPPtr		ReadActionState(XMPStorageUnitView* storageUnitView);

};

#endif
