/*
	File:		SimplPrt.h

	Contains:	Definition for class SimplePart

	Written by:	Joshua Susser

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	  4/6/94	JBS		1155477, 1155480
		 <4>	  4/5/94	CC		ExternalizeKinds() parameter changed from
									XMPTypeSet to XMPTypeList. (1153046)
		 <3>	 3/25/94	JA		JBS: Updated HighlightChanged, RevealFrame,
									CreateEmbeddedFrameIterator, CanvasChanged.
		 <2>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <1>	 2/28/94	CG		first checked in
		<21>	 2/25/94	T		override all methods from superclasses
		<20>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame();
									added AttachSourceFrame()
		<19>	  2/2/94	NP		Added methods for new linking protocol
									APIs.
		<18>	  2/1/94	CC		Added RevealLink method.
		<17>	 1/31/94	JBS		added GetPrintResolution()
		<16>	 1/27/94	VL		Changed XMPPart::CloneTo to
									XMPPart::CloneInto.
		<15>	 1/24/94	CC		Added XMPChangeID parameter to
									LinkUpdated().
		<14>	 1/20/94	SS		::Open returns a window id
		<13>	 1/11/94	T		Init... changes
		<12>	12/21/93	JBS		added ViewTypeChanged() &
									PresentationChanged(), renamed
									FrameClosed() to CloseDisplayFrame()
		<11>	12/16/93	JBS		added RevealFrame(), EmbeddedFrameSpec(),
									CanvasUpdated()
		<10>	12/13/93	T		InitPart changes
		 <9>	12/12/93	T		Added ExternalizeKinds, and ChangeKind
		 <8>	11/30/93	LM		Added an XMPFrame parameter to ReadPartInfo
		 <7>	11/16/93	RR		Added AdjustMenus (not yet called), and
									MouseWithin. Removed point from MouseLeave
		 <6>	11/15/93	CC		Changed parameter list of CreateLink()
		 <5>	11/12/93	JBS		use Facets
		 <4>	10/21/93	PH		Change FulfillPromise call
		 <3>	10/14/93	RR		Add sourceFrame parameter to AddDisplayFrame
		 <2>	 10/6/93	T		merge-in Joshua's changes
		 <1>	 10/6/93	JBS		add fSession field
		<20>	 9/30/93	PH		Support new Drag and Drop protocol
		<19>	 9/29/93	VL		Added InitializePersistent.
		<18>	 8/10/93	VL		Added IncrementRefCount and Release.
		<17>	  8/9/93	NP		Removed class id strings.
		<16>	  8/6/93	JBS		add HighlightChanged() method
		<14>	 5/19/93	JBS		updated for namesmithing
		<13>	 4/29/93	JBS		8.3 file names
		<12>	 4/28/93	JBS		8.3 file names
		<10>	  4/2/93	TI		add Initialize and ClipShapeChanged
		 <9>	 3/19/93	SF		Removed Shape parameter from ChangedUsedShape and
									ChangedFrameShape;
		 <8>	 3/18/93	SF		Changed names: NotifyReshape->ChangedUsedShape;
									Reshape->ChangedFrameShape; Added: RequestFrameShape
		 <7>	 3/11/93	SF		NotifyReshape has a Shape parameter as well
	   	 <6>	 3/11/93	RCR		Foci are owned by frames, not part/frame pairs
		 <5>	 3/11/93	JBS		now descends from XMPPart, not XMPContainingPart; remove
									Extension protocol methods
		 <4>	 3/10/93	JBS		change XMPMapping to XMPTransform in AddFrame()
		 <3>	  3/4/93	JBS		remove fContents field
		 <2>	  3/3/93	JBS		update
		 <1>	  2/1/93	JBS		first checked in
		 <0>	  2/1/93	JBS		split from SimplePart.h
*/


#ifndef _SIMPLPRT_
#define _SIMPLPRT_

#ifndef _PART_
#include "Part.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"	// For XMPDraftKey
#endif


//==============================================================================
// Classes defined in this interface
//==============================================================================

class SimplePart;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPPart;

//==============================================================================
// SimplePart
//==============================================================================

#define kSimplePartID "appl:simplepart$class,1.0.0"

class SimplePart : public XMPPart
{

	friend class XMPDraft;

public:

	// Overridden Methods

	// XMPObject
	XMPVMethod XMPSize Purge(XMPSize size);

	// XMPRefCntObject
	XMPMethod void Release();
	
	//  DragAndDrop 
	XMPVMethod void FulfillPromise(XMPStorageUnitView *promiseSUView);
	XMPVMethod void DropCompleted(XMPPart* destPart, XMPDropResult dropResult);
	XMPVMethod void DragEnter(XMPDragItemIterator* dragInfo,
							  XMPFacet* facet,
							  XMPPoint where);
	XMPVMethod void DragWithin(XMPDragItemIterator* dragInfo,
							   XMPFacet* facet,
							   XMPPoint where);
	XMPVMethod void DragLeave(XMPFacet* facet,
							  XMPPoint where);
	XMPVMethod XMPDropResult Drop(XMPDragItemIterator* dropInfo,
								  XMPFacet* facet,
								  XMPPoint where);

	//  Embedding 
	XMPVMethod void ContainingPartPropertiesChanged(XMPFrame* frame,
													XMPStorageUnit* propertyUnit);
	XMPVMethod XMPStorageUnit* GetContainingPartProperties(XMPFrame* frame);
	XMPVMethod void RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape);
	XMPVMethod void EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec);
	XMPVMethod XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(XMPFrame* frame);

	//  Frame 
	XMPVMethod void AddDisplayFrame(XMPFrame* frame);
	XMPVMethod void AttachSourceFrame(XMPFrame* frame, XMPFrame* sourceFrame);
	XMPVMethod void RemoveDisplayFrame(XMPFrame* frame);
	XMPVMethod void CloseDisplayFrame(XMPFrame* frame);
	XMPVMethod void FrameShapeChanged(XMPFrame* frame);
	XMPVMethod void ViewTypeChanged(XMPFrame* frame);
	XMPVMethod void PresentationChanged(XMPFrame* frame);
	XMPVMethod void SequenceChanged(XMPFrame* frame);
	XMPVMethod void WritePartInfo(XMPPtr partInfo,
								  XMPStorageUnitView* storageUnitView);
	XMPVMethod XMPPtr ReadPartInfo(XMPFrame* frame, XMPStorageUnitView* storageUnitView);
	XMPVMethod XMPID  Open(XMPFrame* frame);
	XMPVMethod XMPFrame* RequestEmbeddedFrame(XMPFrame* containingFrame,
											  XMPFrame* baseFrame,
											  XMPShape* frameShape,
											  XMPPart* embedPart,
											  XMPTypeToken viewType,
											  XMPTypeToken presentation,
											  XMPBoolean isOverlaid);
	XMPVMethod void RemoveEmbeddedFrame(XMPFrame* embeddedFrame);
	XMPVMethod XMPShape* RequestFrameShape(XMPFrame* embeddedFrame, XMPShape* frameShape);
	XMPVMethod void UsedShapeChanged(XMPFrame* embeddedFrame);
	XMPVMethod XMPShape* AdjustBorderShape(XMPFrame* embeddedFrame, XMPShape* shape);

	//  Facet 
	XMPVMethod void FacetAdded(XMPFacet* facet);
	XMPVMethod void FacetRemoved(XMPFacet* facet);
	XMPVMethod void CanvasChanged(XMPFacet* facet);
	XMPVMethod void GeometryChanged(XMPFacet* facet);

	//  Imaging 
	XMPVMethod void Draw(XMPFacet* facet, XMPShape* invalidShape);
	XMPVMethod void CanvasUpdated(XMPCanvas* canvas);
	XMPVMethod void HighlightChanged(XMPFacet* facet);
	XMPVMethod XMPULong GetPrintResolution(XMPFrame* frame);

	//  Linking 
	XMPVMethod XMPLink* CreateLink(XMPPtr data, XMPULong size);
	XMPVMethod void LinkUpdated(XMPLink* updatedLink, XMPChangeID id);
	XMPVMethod void RevealLink(XMPLinkSource* linkSource);
	XMPVMethod void EmbeddedFrameChanged(XMPFrame* frame, XMPChangeID change);
	XMPVMethod void LinkStatusChanged(XMPFrame* frame);

	//  Memory Management 

	//  Part Activation 
	XMPVMethod XMPBoolean BeginRelinquishFocus(XMPTypeToken focus,
											   XMPFrame* ownerFrame,
											   XMPFrame* proposedFrame);
	XMPVMethod void CommitRelinquishFocus(XMPTypeToken focus, 
										  XMPFrame* ownerFrame,
										  XMPFrame* proposedFrame);
	XMPVMethod void AbortRelinquishFocus(XMPTypeToken focus,
										 XMPFrame* ownerFrame,
										 XMPFrame* proposedFrame);
	XMPVMethod void FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame);
	XMPVMethod void FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame);

	//  Storage 
	XMPVMethod void CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU);

	//  Binding 
	XMPVMethod void ExternalizeKinds(XMPTypeList* kindList);
	XMPVMethod void ChangeKind(XMPType kind);

	//  UI Events 
	XMPVMethod XMPBoolean HandleEvent(XMPEventData event, XMPFrame* frame, XMPFacet* facet);
	XMPVMethod XMPBoolean HandleEventInEmbedded(XMPEventData event,
												XMPFrame* frame, XMPFacet* facet,
												XMPFrame* embeddedFrame, XMPFacet* embeddedFacet);
	XMPVMethod void MouseEnter(XMPFacet* facet, XMPPoint where);
	XMPVMethod void MouseWithin(XMPFacet* facet, XMPPoint where);
	XMPVMethod void MouseLeave(XMPFacet* facet);
	XMPVMethod void AdjustMenus(XMPFrame* frame);

	//  Undo 
	XMPVMethod void UndoAction(XMPActionData actionState);
	XMPVMethod void RedoAction(XMPActionData actionState);
	XMPVMethod void DisposeActionState(XMPActionData actionState,
									   XMPDoneState doneState);
	XMPVMethod void WriteActionState(XMPPtr actionState,
									 XMPStorageUnitView* storageUnitView);
	XMPVMethod XMPPtr ReadActionState(XMPStorageUnitView* storageUnitView);

protected:

	SimplePart();
	virtual ~CDECL SimplePart();

	XMPVMethod	void InitPart(XMPStorageUnit* storageUnit);
	XMPVMethod	void InitPartFromStorage(XMPStorageUnit* storageUnit);

	// data members
	
	XMPSession* fSession;
};

#endif // _SIMPLPRT_
