/*
	File:		PropAcsr.h

	Contains:	Abstract class for Apple Event object model property accessing
				class.

	Written by:	Nick Pilch

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	  2/9/94	NP		Tiger Team cleanup.
		 <4>	  2/7/94	NP		Tiger Team doings.
		 <3>	10/18/93	CG		Added constructor def for ASLM builds.
		 <2>	10/11/93	NP		Methods no longer pure virtual so that
									client doesn't have to implement, but if
									they get called, an exception will be
									signalled.
		 <1>	 9/10/93	NP		first checked in

	To Do:
*/

#ifndef _PROPACSR_
#define _PROPACSR_

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This a simple abstract class to aid in the processing of Apple Events that
	are requests to get or set properties. Since OpenDoc does the resolution
	first, the object accessor routines can allocate one of these classes
	according to the property for which they have been asked to create a token.
	When passed back to the event handler, the event handler need only call the
	the appropriate member function of this class, GetData or SetData, to
	process the event. The event handler must then delete the object.
*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Classes defined in this interface
//==============================================================================

class PropAccessor;

//==============================================================================
// Classes used by this interface
//==============================================================================

//==============================================================================
// PropAccessor
//==============================================================================

//#define kPropAccessorID "appl:propaccessor$class,1.0.0"

class PropAccessor
{
  public:

	PropAccessor();
//		= 0;
//	XMPVMethod void Initialize()
//		= 0;
// 	virtual ~PropAccessor()
//		= 0;

	XMPVMethod void SetData(AppleEvent* message);
//		= 0;
		
		// Call this function inside a SetData event handler.
	
	XMPVMethod void GetData(AppleEvent* reply);
//		= 0;
		
		// Call this function inside a GetData event handler.
};

#endif // _PROPACSR_