/*
	File:		DictList.h

	Contains:	Definition of class DictionaryList

	Written by:	Richard Rodseth

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <5>	  2/7/94	JA		Tiger Team Makeover!
		 <4>	 11/4/93	RR		Moved class Association to .cp
		 <3>	10/28/93	RR		Removed ambiguous constructor
		 <1>	 8/13/93	RCR		first checked in

	To Do:
*/

#ifndef _DICTLIST_
#define _DICTLIST_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _LIST_
#include "List.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

// OrdereCollection is an ordered collection of elements of type void* (since
// we can't use templates)
// Duplicates are allowed.

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

typedef void* KeyType;
typedef void* ValueType;

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class DictionaryList;	

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class Association; // Defined in implementation

//=====================================================================================
// Global Variables
//=====================================================================================


//=====================================================================================
// Class DictionaryList
//=====================================================================================

#define kDictionaryListID "appl:dictionarylist$class,1.0.0"

class DictionaryList
{
	ASLM_COMPATIBLE

public:

	DictionaryList();
	virtual ~ CDECL DictionaryList();

	XMPVMethod void	AddPair(KeyType key, ValueType value);
	
	XMPVMethod XMPULong Count();
	XMPVMethod XMPBoolean ContainsKey(KeyType key);
	XMPVMethod ValueType ValueAtKey(KeyType key);

	XMPVMethod void	RemoveKey(KeyType key);
	XMPVMethod void	RemoveAll();			
	XMPVMethod void	DeleteKeys();
	XMPVMethod void	DeleteValues();
	
protected:

	 XMPVMethod Association* CreateNewAssociation(KeyType key, ValueType value) const;
	 XMPVMethod XMPBoolean	KeysMatch(KeyType key1,KeyType key2) const;
	 
	 	// Does a straight == comparison by default 

private:

	LinkedList		fImplementation;
};

#endif // _DICTLIST_
