/*
	File:		Except.cpp

	Contains:	Exception-handling, assertion and debugging utilities

	Written by:	Richard Rodseth, Steve Smith, Jens Alfke

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 4/18/94	VL		(JA) 1158103: AssertionFailed uses DebugStr
									instead of SysBreak.
		 <2>	 4/12/94	eeh		bug #1146751: remove WriteToFile
		<17>	  2/7/94	JA		Capitalized THROW functions.
		<16>	  2/7/94	JA		Tiger Team Makeover!
		<15>	 1/31/94	JA		Don't compile Warning/Assert stuff in
									non-debug builds. Added log stuff.
		<14>	12/19/93	VL		Put in ASSERT code again.
		<13>	12/17/93	VL		Turned off AssertionFailed for code drop.
		<12>	12/10/93	CG		Cleaned up warnings.
		<11>	10/11/93	JA		Improved Assert formatting some more. THINK
									compatibility.
		<10>	 10/7/93	PH		Add some formatting to _AssertionFailed
		 <9>	 9/22/93	JA		Changed ASSERT to _AssertionFailed.
		 <8>	 9/21/93	CG		WriteToFile crashes in ASLM builds.
									#ifdef-ed it out for now.
		 <7>	 9/16/93	CG		Removed error logging for ASLM builds
									because of bug.
		 <6>	 9/15/93	JA		Assert now fails if condition is FALSE.
									(Yow!)
		 <5>	  9/8/93	SS		Moved some includes from .h
		 <4>	 8/25/93	SS		Add logging functionality to ThrowXXX
									functions - Removed INLINE keywords
		 <8>	  6/10/93	RCR		Change to new Exception macros
		 <2>	 4/29/93	RCR		Rename includes.
		 <1>	 4/8/93 	RCR		first checked in

	To Do:
*/

#ifndef __EXCEPT__
#include "Except.h"
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef THINK_CPLUS
#ifndef __STRINGS__
#include <strings.h>
#endif
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __LIBRARYMANAGER__
#include <LibraryManager.h>
#endif

#ifdef __THINK__
#include <pascal.h>
#endif

#include <stdarg.h>


#ifdef __THINK__
#define USE_SYSBREAK 0	// THINK debugger uses DebugStr instead
#else
#define USE_SYSBREAK 0	// 1 to use SysBreak, 0 to use DebugStr. SysBreak appears to have probs.
#endif


#pragma segment XMPExceptions

//=================================================================================== 
// ASSERTION-FAILED
//=================================================================================== 

#if XMPDebug

void _AssertionFailed( char *cond, char *filename,
						XMPError error, char *msg /*=NULL*/ )
{
	XMPUnused(filename);
	char dbg[256];
	dbg[0] = 0;
	if( msg ) {
		strcat(dbg,msg);
		strcat(dbg,": ");
	}
	strcat(dbg,"Assertion ");
	strcat(dbg, cond);
	strcat(dbg, " failed.\n");
#if USE_SYSBREAK
		SysBreakStr(c2pstr(dbg));
#else
		DebugStr(c2pstr(dbg));
#endif
	
#ifndef ASLMBUILD
//		WriteToFile("Assertion failed",error,msg);
#endif // ASLMBUILD
	
	if( error!=0 )
		Fail(error, msg ?msg :"Assertion failed");
}
	
#endif /*XMPDebug*/
	
	
//=================================================================================== 
// LOG
//=================================================================================== 

#if XMPDebug

void _Log( char *fmt, ... )
{
	char msg[512];
	msg[0] = '\0';
	va_list args;
	va_start(args,fmt);
	vsprintf(msg+strlen(msg),fmt,args);
	va_end(args);
	
#if USE_SYSBREAK
	SysBreakStr(c2pstr(msg));
#else
	DebugStr(c2pstr(msg));
#endif

#ifndef ASLMBUILD
//	WriteToFile("Logged: ",0,p2cstr((unsigned char*)msg));
#endif // ASLMBUILD
}
	
#endif /*XMPDebug*/


//=================================================================================== 
// WARN
//=================================================================================== 

#if XMPDebug

void _Warn( char *fmt, ... )
{
	char msg[512];
	strcpy(msg, "XMPWarning: ");
	va_list args;
	va_start(args,fmt);
	vsprintf(msg+strlen(msg),fmt,args);
	va_end(args);
	strcat(msg,"\n");
	
#if USE_SYSBREAK
	SysBreakStr(c2pstr(msg));
#else
	DebugStr(c2pstr(msg));
#endif

#ifndef ASLMBUILD
//	WriteToFile("",0,p2cstr((unsigned char*)msg));
#endif // ASLMBUILD
}
	
#endif /*XMPDebug*/


//=================================================================================== 
// THROW, ETC.
//=================================================================================== 

void THROW_IF_ERROR(XMPError error, char* msg)
{
	if (error) {
#ifndef ASLMBUILD
//		WriteToFile("ThrowIfError",error,msg);
#endif // ASLMBUILD
		Fail(error, msg);
	}
}

void THROW(XMPError error, char* msg)
{
#ifndef ASLMBUILD
//	WriteToFile("Throw",error,msg);
#endif // ASLMBUILD
	Fail(error, msg);
}

void THROW_IF_NULL(void* value, char* msg)
{
	if (value == kXMPNULL) {
#ifndef ASLMBUILD
//		WriteToFile("ThrowIfNULL",kXMPErrOutOfMemory,msg);
#endif // ASLMBUILD
		Fail(kXMPErrOutOfMemory, msg);
	}
}
