/*
	File:		ISOString.cp

	Contains:	Routines for manipulating XMPISOStr

	Written by:	Steve Smith, Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <8>	  2/7/94	JA		Tiger Team Makeover!
		 <7>	11/23/93	VL		Added overloaded XMPISOStrCopy, overloaded
									XMPISOStrCompare and XMPISOStrConcat.
		 <6>	 11/5/93	NP		Added ISOStrEqual.
		 <5>	 8/20/93	VL		Used XMPMem.h.
		 <4>	 7/23/93	VL		Added XMPISOStrFromCStr.
		 <3>	 6/15/93	RCR		Rename #include
		 <2>	 6/11/93	RCR		#include String.h

	To Do:
*/

/************************************************************

	ISOString.h
	ISO String handling
	
	Copyright Apple Computer,Inc. 1993
	All rights reserved

************************************************************/

#ifndef _ISOSTR_
#include "ISOStr.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef __STRING__
#include <String.h>
#endif

XMPISOStr XMPISOStrCopy (XMPISOStr dest, const XMPISOStr source)
{
	strcpy(dest, source);
	
	return dest;
}

XMPISOStr XMPISOStrCopy (XMPISOStr dest, const XMPISOStr source, XMPULong length)
{
	strncpy(dest, source, (size_t)length);
	
	return dest;
}

XMPISOStr XMPISOStrConcat (XMPISOStr dest, const XMPISOStr source)
{
	strcat(dest, source);
	
	return dest;
}

XMPISOStr XMPISOStrConcat (XMPISOStr dest, const XMPISOStr source, XMPULong length)
{
	strncat(dest, source, (size_t)length);
	
	return dest;
}

XMPSLong XMPISOStrCompare (const XMPISOStr iso1, const XMPISOStr iso2)
{
	return strcmp(iso1, iso2);
}

XMPSLong XMPISOStrCompare (const XMPISOStr iso1, const XMPISOStr iso2, XMPULong length)
{
	return strncmp(iso1, iso2, (size_t)length);
}

XMPBoolean XMPISOStrEqual (const XMPISOStr iso1, const XMPISOStr iso2)
{
	return (strcmp(iso1, iso2) == 0);
}

XMPULong XMPISOStrLength (const XMPISOStr iso)
{
	return strlen(iso);
}

XMPISOStr XMPISOStrFromCStr (const char* cstring)
{
	XMPULong	cstringSize;
	XMPISOStr	isoStr;
	
	cstringSize = strlen(cstring);
	isoStr = (XMPISOStr) XMPNewPtr(cstringSize + 1);
	strcpy(isoStr, cstring);
	
	return isoStr;
}

