/*
	File:		XMPPascalString.cp

	Contains:	Pascal string manipulation routines

	Written by:	Nick Pilch

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 3/31/94	eeh		bug #1153999: make PStrToIntl work with
									preallocated XMPITexts
		 <2>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<12>	  2/7/94	JA		Tiger Team Makeover!
		<11>	 1/28/94	NP		Changed PStrToIntl to create roman, english
									text only.
		<10>	 1/19/94	NP		Added PStrToIntl.
		 <9>	 1/10/94	NP		Modified routines to work with new XMPIText
									type.
		 <8>	 11/2/93	NP		Cast for CFront
		 <7>	 11/2/93	NP		Use XMP mem functions instead of platform
									ones.
		 <6>	10/11/93	NP		Added EqualPascalStrings.
		 <5>	 10/7/93	CG		Added CopyISOStr2PStr.
		 <4>	  7/7/93	VL		Fixed non-pointer to theText in
									CToPascalString.
		 <2>	 4/28/93	NP		File name changes.
		 <1>	 4/15/93	NP		first checked in

	To Do:
*/

#ifndef _PASCLSTR_
#include "PasclStr.h"
#endif

#ifndef _ISOSTR_
#include "ISOStr.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __STRING__
#include <string.h>
#endif

#ifndef __CTYPE__
#include <ctype.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifdef __SC__
	#ifndef __LANGUAGE__
	#include <Language.h>
	#endif
#endif // __SC__

#pragma segment XMPPascalString

//==============================================================================
// Functions
//==============================================================================

void CopyPascalString(Str255 destString, ConstStr255Param srcString)
{
	XMPBlockMove((XMPPtr)srcString, (XMPPtr)destString, srcString[0] + 1L);
}

void CopyISOStr2PStr(Str255 destString, const XMPISOStr srcString)
{
	XMPSize len = XMPISOStrLength(srcString);
	XMPBlockMove((XMPPtr)srcString, &(destString[1]), (XMPULong)len);
	destString[0] = (unsigned char) len;
}

void ConcatPascalStrings(Str255 destString, ConstStr255Param srcString)
{
    XMPSShort charsToCopy;
    
    /* Truncate if concatenated string would be longer than 255 chars.  */
    
    charsToCopy = (srcString[0] < 255 - destString[0])
					? (srcString[0]) : (255 - destString[0]);
    XMPBlockMove((XMPPtr)(srcString + 1),
					(XMPPtr)(destString + destString[0] + 1), charsToCopy);
    destString[0] += charsToCopy;
}

XMPBoolean EqualPascalStrings(ConstStr255Param str1, ConstStr255Param str2)
{
	if (str1[0] != str2[0])
		return kXMPFalse;
	for (int i = 1; i < str1[0]; i++)
	{
		if (tolower(str1[i]) != tolower(str2[i]))
			return kXMPFalse;
	}
	return kXMPTrue;
}

StringPtr IntlToPStr(XMPIText* intlText, StringPtr pstr)
{
	memcpy(&pstr[1], intlText->theText, (size_t)intlText->textLength);
	pstr[0] = (unsigned char)intlText->textLength;
	return pstr;
}

XMPIText* PStrToIntl(StringPtr pstr, XMPIText** intlText)
{
	XMPIText* result = kXMPNULL ;
	if ( intlText )
		result = *intlText ;
	if ( !result )
		result = (XMPIText*)XMPNewPtr(sizeof(XMPIText)
										- sizeof((**intlText).theText)
										+ pstr[0]);

	result->theScriptCode = smRoman;
	result->theLangCode = langEnglish;
	result->textLength = pstr[0];
	memcpy(result->theText, pstr + 1, pstr[0]);
	if ( intlText ) 
		*intlText = result ;
	return result ;
}

