/*
	File:		PasclStr.h

	Contains:	Pascal string manipulation routines

	Written by:	Nick Pilch

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<11>	 3/31/94	eeh		bug #1153999: change PStrToIntl decl.
		<10>	  2/7/94	JA		Tiger Team Makeover!
		 <9>	 1/28/94	NP		Changed comment for PSTrToIntl to reflect
									code change.
		 <8>	 1/19/94	NP		Added PStrToIntl.
		 <7>	 1/10/94	NP		Added some comments.
		 <6>	10/11/93	NP		Added EqualPascalStrings.
		 <5>	 10/7/93	CG		Added CopyISOStr2PStr.
		 <3>	 4/28/93	NP		File name changes.
		 <2>	 4/27/93	NP		Included types.h.
		 <1>	 4/15/93	NP		first checked in

	To Do:
*/

#ifndef _PASCLSTR_
#define _PASCLSTR_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifdef THINK_CPLUS
	#ifndef __pascal__
	#include <pascal.h>
	#endif
#else // THINK_CPLUS
	#ifndef __STRINGS__
	#include <strings.h>
	#endif
#endif // THINK_CPLUS

#ifndef __TYPES__
#include <types.h>
#endif


// These routines convert the string in place.

inline StringPtr CToPascalString(char* string)
{
	return c2pstr(string);
}

inline char* PascalToCString(StringPtr string)
{
	return p2cstr(string);
}

void ConcatPascalStrings(Str255 destString, ConstStr255Param srcString);
	// Error checking is done to ensure that the limit of 256 chars is not
	//	exceeded.

void CopyPascalString(Str255 destString, ConstStr255Param srcString);

XMPBoolean EqualPascalStrings(ConstStr255Param str1, ConstStr255Param str2);
	// Returns kXMPTrue if strings are equal, kXMPFalse otherwise. Does a CASE-
	//	INSENSITIVE compare!

void CopyISOStr2PStr(Str255 destString, const XMPISOStr srcString);

StringPtr IntlToPStr(XMPIText* intlText, StringPtr pstr);
	// Caller must allocate storage for pstr. No error checking is done for
	//	strings longer than 255 chars. Simply grabs the text from the theText
	//	field of XMPIText.

XMPIText* PStrToIntl(StringPtr pstr, XMPIText** intlText);
	// Caller must dispose storage for intlText when done with it. The XMPIText
	//	is created with theScriptCode = smScriptRoman and
	//	theLangCode = langEnglish.  Storage for the XMPIText is allocated only if
	//  intlText == NULL; otherwise it's assumed that *intlText is a valid ptr to
	//  an XMPIText big enough to hold the string.

#endif // _PASCLSTR_