/*
	File:		XMPFile.cpp

	Contains:	Implmentation for XMPFile class

	Written by:	Vincent Lo

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <8>	 3/24/94	T		(no bug# yet) added
									SetContainerType(XMPContainerType
									containerType);
		 <7>	 3/17/94	EL		Add SetEndOfFile. #1150044
		 <6>	 3/16/94	eeh		bug #1149649: make CopyFrom work on open
									files
		 <5>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <4>	 3/10/94	eeh		bug #1149754
		 <3>	 2/17/94	VL		#1142972, #1144024: Fixed GetCreator.
		 <2>	 2/17/94	T		fix #1144608. IsStationery was not
									returning the correct value.
		<22>	  2/7/94	JA		Capitalized THROW.
		<21>	  2/7/94	JA		Tiger Team Makeover!
		<20>	  2/3/94	CG		Added fHeap.
		<19>	 1/13/94	eeh		moved Exists from ShellM.cp
		<18>	11/18/93	CG		With eeh, fixed dirID param in Move to
									long.
		<17>	11/15/93	eeh		added CopyFrom, MoveRename, Move, and
									Rename
		<16>	 11/8/93	NP		Added GetFileRefNum.
		<15>	 11/5/93	NP		Removed compiler warnings.
		<14>	10/27/93	eeh		added GetCreator()
		<13>	  9/3/93	NP		Fixed MPW compile problem.
		<12>	 8/27/93	T		modified GetContainerType() to not take
									session, and implemented it's type
									generation
		<11>	 8/19/93	T		
		<10>	 8/19/93	T		use XMPMem.h XMPNewPtr
		 <9>	  8/4/93	VL		Close raises an exception if there is no
									refnum.
		 <8>	  7/2/93	T		change GetName to return XMPName* and add
									GetAsciiName
		 <7>	 6/30/93	VL		Typecasted XMPName to char* in strcpy.
		 <6>	 5/24/93	T		remove CopyFrom(), add GetType(),
									GetContainerType()
		 <5>	  5/7/93	T		Add IsStationery(), and CopyFrom(XMPFile*)
									(not implemented yet)
		 <4>	  5/7/93	VL		Moved parameters from Constructor to
									Create.
		 <3>	  5/3/93	NP		Fixed #include statements.
		 <2>	 4/27/93	VL		Rewrote GetName to do typecast from Str255
									to char* because THINK C doesn't like the
									other way.
		 <1>	 4/19/93	VL		first checked in

	To Do:
	
	$$$$$ 1) fPermission is set to Read/Write. Should it be more restrictive?
*/

#ifndef _XMPFILE_
#include "XMPFile.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _PASCLSTR_
#include "PasclStr.h"
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __FINDER__
#include <Finder.h>
#endif

#ifndef __STRING__
#include <string.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#pragma segment XMPFile

//==============================================================================
// Constants
//==============================================================================
const XMPError	kXMPErrFileClosed = 1341;	// $$$$$ move to ErrorDef.h
const XMPError	kXMPErrFileOpened = 1342;	// $$$$$ move to ErrorDef.h

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Local Classes
//==============================================================================

//==============================================================================
// Global Variables
//==============================================================================

//==============================================================================
// Function Prototype
//==============================================================================

//==============================================================================
// XMPFile
//==============================================================================

ASLM_SCDECLARATION(XMPFile)

const XMPOSType kBogusOSType = 0x21212121; // '!!!!'
// $$$$$ Instead of using this, should throw an exception if the fileType/Creator
// is unknown.

//------------------------------------------------------------------------------
// XMPFile::XMPFile
//------------------------------------------------------------------------------
XMPFile::XMPFile()
{
	fHeap = kXMPNULL;
	fCreator = kBogusOSType;
	fFileType = kBogusOSType;
	fScriptTag = 0;
	
	fFileSpec.vRefNum = 0;
	fFileSpec.parID = 0;
	fFileSpec.name[0] = 0;
	fDataRefNum = 0;
	fPermission = fsRdWrPerm;				// a convenience
}

XMPFile::XMPFile(XMPHeap where)
{
	fHeap = where;
	fCreator = kBogusOSType;
	fFileType = kBogusOSType;
	fScriptTag = 0;
	
	fFileSpec.vRefNum = 0;
	fFileSpec.parID = 0;
	fFileSpec.name[0] = 0;
	fDataRefNum = 0;
	fPermission = fsRdWrPerm;				// a convenience
}

//------------------------------------------------------------------------------
// XMPFile::~XMPFile
//------------------------------------------------------------------------------
XMPFile::~XMPFile()
{
}

//------------------------------------------------------------------------------
// XMPFile::Specify
//------------------------------------------------------------------------------
void XMPFile::Specify(XMPFileSpec* fileSpec)
{
	fFileSpec = *fileSpec;
}

//------------------------------------------------------------------------------
// XMPFile::IsStationery
//------------------------------------------------------------------------------
XMPBoolean XMPFile::IsStationery()
{
	FInfo finderInfo;
	FSpGetFInfo(&fFileSpec,&finderInfo);
	return (((finderInfo.fdFlags)&kIsStationary)!=0);
}

//------------------------------------------------------------------------------
// XMPFile::SetPermission
//------------------------------------------------------------------------------
void XMPFile::SetPermission(XMPFilePermission permission)
{
	fPermission = permission;
}


//------------------------------------------------------------------------------
// XMPFile::Create
//------------------------------------------------------------------------------
void XMPFile::Create(XMPOSType creator, XMPOSType fileType, XMPScriptCode scriptCode)
{	
	fCreator = creator;
	fFileType = fileType;
	fScriptTag = scriptCode;
	
	if (fDataRefNum == kXMPNoFileRefNum) {
		THROW_IF_ERROR(FSpCreate(&fFileSpec, fCreator, fFileType, fScriptTag));
	}
}

//------------------------------------------------------------------------------
// XMPFile::Exists
//------------------------------------------------------------------------------
XMPBoolean XMPFile::Exists()
{
	XMPFileSpec filespec = this->GetFileSpec() ;
	FInfo ignore ;
	OSErr err = FSpGetFInfo( &filespec, &ignore ) ;
	if ( err == noErr )
		return kXMPTrue ;
	else {
		if ( err != fnfErr )
			THROW( err ) ;
		return kXMPFalse ;
	}
}

//------------------------------------------------------------------------------
// XMPFile::Open
//------------------------------------------------------------------------------
void XMPFile::Open()
{	
	if (fDataRefNum == kXMPNoFileRefNum) {
		THROW_IF_ERROR(FSpOpenDF(&fFileSpec, fPermission, &fDataRefNum));
	}
	else
		THROW(kXMPErrFileOpened);
}

//------------------------------------------------------------------------------
// XMPFile::Close
//------------------------------------------------------------------------------
void XMPFile::Close()
{	
	if (fDataRefNum != kXMPNoFileRefNum) {
		THROW_IF_ERROR(FSClose(fDataRefNum));
		fDataRefNum = kXMPNoFileRefNum;
	}
	else
		THROW(kXMPErrFileClosed);
}

//------------------------------------------------------------------------------
// XMPFile::Delete
//------------------------------------------------------------------------------
void XMPFile::Delete()
{	
	THROW_IF_ERROR(FSpDelete(&fFileSpec));
}


//------------------------------------------------------------------------------
// XMPFile::FlushVolume
//------------------------------------------------------------------------------
void XMPFile::FlushVolume()
{	
	THROW_IF_ERROR(FlushVol(kXMPNULL, fFileSpec.vRefNum));
}

//------------------------------------------------------------------------------
// XMPFile::SetFilePos
//------------------------------------------------------------------------------
void XMPFile::SetFilePos(XMPSShort posMode, XMPSLong posOff)
{	
	THROW_IF_ERROR(SetFPos(fDataRefNum, posMode, posOff));
}

//------------------------------------------------------------------------------
// XMPFile::GetFilePos
//------------------------------------------------------------------------------
XMPSLong XMPFile::GetFilePos()
{
	XMPSLong	posOff;
	
	THROW_IF_ERROR(GetFPos(fDataRefNum, &posOff));
	return posOff;
}

//------------------------------------------------------------------------------
// XMPFile::Read
//------------------------------------------------------------------------------
void XMPFile::Read(XMPSByte* buffer, XMPSLong* count)
{	
	THROW_IF_ERROR(FSRead(fDataRefNum, count, buffer));
}

//------------------------------------------------------------------------------
// XMPFile::Write
//------------------------------------------------------------------------------
void XMPFile::Write(const XMPSByte* buffer, XMPSLong* count)
{	
	THROW_IF_ERROR(FSWrite(fDataRefNum, count, buffer));
}

//------------------------------------------------------------------------------
// XMPFile::GetEndOfFile
//------------------------------------------------------------------------------
XMPSLong XMPFile::GetEndOfFile()
{	
	XMPSLong	length;
	
	THROW_IF_ERROR(GetEOF(fDataRefNum, &length));
	return length;
}

//------------------------------------------------------------------------------
// XMPFile::SetEndOfFile
//------------------------------------------------------------------------------
void  XMPFile::SetEndOfFile(XMPSLong length)
{	
	THROW_IF_ERROR(SetEOF(fDataRefNum, length));
}

//------------------------------------------------------------------------------
// XMPFile::GetName
//------------------------------------------------------------------------------
XMPName* XMPFile::GetName()
{
	Str255 tmp;
	
	CopyPascalString(tmp, fFileSpec.name);
	PascalToCString(tmp);
	XMPName* name = (XMPName*)(XMPNewPtr(sizeof(XMPName)+fFileSpec.name[0]));
	name->theScriptCode = 0;	//$$$$$ Need script code?
	name->theLangCode = 0;		//$$$$$ Need a language code?
	strcpy(&(name->theText[0]),(char*)tmp);
	return name;
}

void XMPFile::GetAsciiName(char* name, XMPULong maxLength)
{
	Str255 tmp;
	CopyPascalString(tmp, fFileSpec.name);
	PascalToCString(tmp);
	strncpy(name, (char*)tmp, (size_t)maxLength);
}

//------------------------------------------------------------------------------
// XMPFile::XMPOSType
//------------------------------------------------------------------------------
XMPOSType XMPFile::GetCreator()
{
	if (fCreator == kBogusOSType) {
		FInfo	fileInfo;
		THROW_IF_ERROR(FSpGetFInfo(&fFileSpec, &fileInfo));
		fCreator = fileInfo.fdCreator;
		fFileType = fileInfo.fdType;
	}
	return fCreator;
}	

//------------------------------------------------------------------------------
// XMPFile::XMPOSType
//------------------------------------------------------------------------------
XMPOSType XMPFile::GetType()
{
	if (fFileType == kBogusOSType) {
		FInfo	fileInfo;
		THROW_IF_ERROR(FSpGetFInfo(&fFileSpec, &fileInfo));
		fCreator = fileInfo.fdCreator;
		fFileType = fileInfo.fdType;
	}
	return fFileType;
}	

//------------------------------------------------------------------------------
// XMPFile::GetContainerType
//------------------------------------------------------------------------------
XMPContainerType XMPFile::GetContainerType()
{
// $$$$$ SKELETON
return kXMPNULL;
}

//------------------------------------------------------------------------------
// XMPFile::SetContainerType
//------------------------------------------------------------------------------
void XMPFile::SetContainerType(XMPContainerType containerType)
{
// $$$$$ SKELETON
XMPUnused(containerType);
}


//------------------------------------------------------------------------------
// XMPFile::GetFileSpec
//------------------------------------------------------------------------------
XMPFileSpec	XMPFile::GetFileSpec()
{
	return fFileSpec;
}

//------------------------------------------------------------------------------
// XMPFile::GetFileRefNum
//------------------------------------------------------------------------------
XMPFileRefNum	XMPFile::GetFileRefNum()
{
	return fDataRefNum;
}

//------------------------------------------------------------------------------
// XMPFile::CopyFrom
//------------------------------------------------------------------------------
void XMPFile::CopyFrom( XMPFile* src )
{
	XMPBoolean scrIsOpen = (src->GetFileRefNum() != kXMPNoFileRefNum) ;
	if ( !scrIsOpen )
		src->Open() ;
	this->SetPermission( fsRdWrPerm ) ;		// currently this is the default value
	this->Open() ;
	
	src->SetFilePos( fsFromStart, 0 ) ;
	this->SetFilePos( fsFromStart, 0 ) ;
	
	const XMPSShort kBufferSize = 1000 ;
	XMPSByte buffer[kBufferSize] ;
	XMPSLong bytesLeft = src->GetEndOfFile() ;
	XMPSLong count ;

	while( bytesLeft > 0 )
	{
		count = bytesLeft < kBufferSize? bytesLeft: kBufferSize ;
		src->Read( buffer, &count ) ;
		this->Write( buffer, &count ) ;
		bytesLeft -= count ;
	}
	
	if ( !scrIsOpen )
		src->Close() ;
	this->Close() ;
}	// CopyFrom()

//------------------------------------------------------------------------------
// XMPFile::MoveRename
//------------------------------------------------------------------------------
void XMPFile::MoveRename( XMPFileSpec* newSpec, XMPBoolean isDuplicate )
{
//	OSErr err ;
	if ( isDuplicate )
	{
		THROW_IF_ERROR( FSpDelete( newSpec ) ) ; 
	}

//	XMPFileSpec oldSpec = this->GetFileSpec() ;	// remember it before changed
	this->Move( newSpec->parID ) ;

	this->Rename( newSpec->name ) ;

#ifdef TO_BE_DELETED
	err = HRename( newSpec->vRefNum, newSpec->parID, oldSpec.name,
 			newSpec->name ) ; 
	THROW_IF_ERROR( err ) ;
#endif //TO_BE_DELETED
}	// MoveRename()


//------------------------------------------------------------------------------
// XMPFile::Move
//------------------------------------------------------------------------------

void XMPFile::Move( XMPSLong targetParID )
{
	CMovePBRec paramBlock ;
	memset( &paramBlock, 0, sizeof(paramBlock) ) ;
	
	// $$$$$ <eeh> should I use this method, or just write to fFileSpec?
	XMPFileSpec src = this->GetFileSpec() ;

	paramBlock.ioNamePtr = src.name ;
	paramBlock.ioVRefNum = src.vRefNum ;
	paramBlock.ioNewDirID = targetParID ;
	paramBlock.ioDirID = src.parID ;
	
	src.parID = targetParID ;
	FSpDelete( &src ) ; 				// ignore error (it may not exist)
	OSErr err = PBCatMove( &paramBlock, false );
	THROW_IF_ERROR( err ) ;

	this->Specify( &src ) ;
}	// Move()

//------------------------------------------------------------------------------
// XMPFile::Rename
//		This routine will not yet delete a file that will prevent the
//			renaming (as Move() does above.)
//------------------------------------------------------------------------------
void XMPFile::Rename( Str63 name )
{
	XMPFileSpec src = this->GetFileSpec() ;
	OSErr err = HRename( src.vRefNum, src.parID, src.name, name ) ; 
	THROW_IF_ERROR( err ) ;
	BlockMove( name, src.name, name[0]+1 ) ;
	this->Specify( &src ) ;
}	//  Rename()

//------------------------------------------------------------------------------
// XMPFile::ForceSpecify
//------------------------------------------------------------------------------
#ifdef TO_BE_DELETED
void XMPFile::ForceSpecify( XMPFileSpec *newSpec )
{
	XMPFileSpec curFileSpec = this->GetFileSpec() ;
	WASSERT( curFileSpec != *newSpec ) ;	// if they're the same, don't delete! 

	WASSERT( curFileSpec.vRefNum==newSpec->vRefNum ) ;
	if ( curFileSpec.parID != newSpec->parID )
		this->Move( newSpec->parID ) ;
	if ( !PStrEq( curFileSpec.name, newSpec->name )
		this->Rename( newSpec->name ) ;
}
#endif TO_BE_DELETED
