/*
	File:		XMPFile.h

	Contains:	Template for XMPFile header

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<21>	 3/24/94	T		(no bug# yet) added
									SetContainerType(XMPContainerType
									containerType);
		<20>	 3/17/94	EL		Add SetEndOfFile. #1150044
		<19>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<18>	  2/7/94	JA		Tiger Team Makeover!
		<17>	  2/4/94	T		class not Class
		<16>	  2/4/94	T		DONT #include header files for Class
									Definitions
		<15>	  2/3/94	CG		Added fHeap.
		<14>	 1/13/94	eeh		added Exists()
		<13>	11/18/93	CG		with eeh, fixed dirID param in Move to
									long.
		<12>	11/15/93	eeh		added CopyFrom, MoveRename, Move, and
									Rename
		<11>	 11/8/93	NP		Added GetFileRefNum.
		<10>	10/27/93	eeh		added GetCreator()
		 <9>	 8/27/93	T		modified GetContainerType() to not take
									session
		 <8>	  8/9/93	NP		Added class id string.
		 <7>	  8/2/93	NP		Added virtual to class def to allow ASLM
									compilation.
		 <6>	  7/2/93	T		change GetName to return XMPName* and add
									GetAsciiName
		 <5>	 5/24/93	T		remove CopyFrom(), add GetType(), Delete(),
									GetContainerType()
		 <4>	  5/7/93	T		Add IsStationery(), and CopyFrom(XMPFile*)
		 <3>	  5/7/93	VL		Moved parameters from Constructor to
									Create.
		 <2>	 4/29/93	T		change names of includefiles
		 <1>	 4/19/93	VL		first checked in

	To Do:
*/

#ifndef _XMPFILE_
#define _XMPFILE_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

//==============================================================================
// Constants
//==============================================================================
const	XMPSShort kXMPNoFileRefNum = 0;

//==============================================================================
// Scalar Types
//==============================================================================
typedef XMPSByte XMPFilePermission;

//==============================================================================
// Classes defined in this interface
//==============================================================================
class XMPFile;

//==============================================================================
// Classes used by this interface
//==============================================================================

//==============================================================================
// XMPFile
//==============================================================================

#define kXMPFileID "appl:xmpfile$class,1.0.0"

class XMPFile
{
	ASLM_COMPATIBLE
	
public:

	XMPFile();
	XMPFile(XMPHeap where);
	virtual ~ CDECL XMPFile();
		
	XMPMethod void Specify(XMPFileSpec* fileSpec);

	XMPMethod XMPBoolean IsStationery();
	
	XMPMethod void SetPermission(XMPFilePermission filePermission);
	
	XMPMethod void Create(XMPOSType creator, XMPOSType fileType, XMPScriptCode scriptCode);
	
	XMPMethod XMPBoolean Exists();
	
	XMPMethod void Open();
	
	XMPMethod void Close();
	
	XMPMethod void Delete();

	XMPMethod void FlushVolume();
	
	XMPMethod void SetFilePos(XMPSShort posMode, XMPSLong posOff);
	
	XMPMethod XMPSLong GetFilePos();
	
	XMPMethod void Read(XMPSByte* buffer, XMPSLong* count);
	
	XMPMethod void Write(const XMPSByte* buffer, XMPSLong* count);
	
	XMPMethod XMPSLong GetEndOfFile();
	
	XMPMethod void SetEndOfFile(XMPSLong length);
	
	XMPMethod XMPName*	GetName();
	XMPMethod void		GetAsciiName(char* name,XMPULong maxLength);
	
	XMPMethod XMPOSType GetCreator();
	XMPMethod XMPOSType GetType();

	XMPMethod XMPContainerType GetContainerType();
	XMPMethod void SetContainerType(XMPContainerType containerType);
	
	XMPMethod XMPFileSpec	GetFileSpec();

	XMPMethod XMPSShort	GetFileRefNum();

	XMPMethod void CopyFrom( XMPFile* src );
	XMPMethod void MoveRename( XMPFileSpec* newSpec, XMPBoolean isDuplicate ) ;
	XMPMethod void Move( XMPSLong targetParID ) ;
	XMPMethod void Rename( Str63 name ) ;

private:

	XMPFileSpec			fFileSpec;
	XMPSShort			fDataRefNum;
	XMPFilePermission	fPermission;
	XMPOSType			fCreator;
	XMPOSType			fFileType;
	XMPScriptCode		fScriptTag;
	XMPHeap 			fHeap;
};

#endif // _XMPFILE_
