/*
	File:		XMPMem.h

	Contains:	Memory allocation routines

	Written by:	Nick Pilch

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<13>	 2/22/94	VL		JA: Added calls to catch memory errors.
		<12>	 2/15/94	T		fix 1143649 and some cleanup
		<11>	  2/7/94	JA		Tiger Team Makeover!
		<10>	  2/3/94	CG		Added XMPNewPtrClear that takes a heap.
		 <9>	  2/2/94	NP		Added XMPGetSystemHeap, XMPNewPtrSys and
									XMPGetPool -> XMPGetHeap.
		 <8>	 1/12/94	CG		Added XMPNewPtrClear.
		 <7>	 12/9/93	VL		Added XMPSetHandleSize.
		 <6>	11/24/93	VL		Added overloaded XMPUnlockHandle(void*).
		 <5>	11/23/93	VL		Added overloaded NewPtr calls and handle
									calls.
		 <4>	11/23/93	JA		Made source of XMPBlockMove const.
		 <3>	10/14/93	JA		Added XMPCopyHandle.
		 <2>	 8/20/93	VL		Added XMPBlockMove. Changed void* to XMPPtr
									and void** to XMPHandle.
		 <1>	 8/16/93	NP		first checked in

	To Do:
*/

#ifndef _XMPMEM_
#define _XMPMEM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	Abstract memory routines.
*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

typedef const void *ConstXMPPtr;

//==============================================================================
// Classes defined in this interface
//==============================================================================

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPObject;

//==============================================================================
// Routines
//==============================================================================

XMPPtr XMPNewPtr(XMPULong howBig);
XMPPtr XMPNewPtr(XMPULong howBig, XMPHeap heap);
		// Create a pointer in the specified heap
XMPPtr XMPNewPtrClear(XMPULong howBig);
XMPPtr XMPNewPtrClear(XMPULong howBig, XMPHeap heap);
XMPPtr XMPNewPtrSys(XMPULong howBig);
		// Create the pointer in the system heap
void XMPDisposePtr(XMPPtr pointer);
XMPHeap	XMPGetHeap(XMPPtr pointer);
		// Get the heap from which the pointer was allocated
XMPHeap XMPGetSystemHeap();
		// Get a reference to the system heap.
void XMPBlockMove(ConstXMPPtr from, XMPPtr to, XMPULong size);
		// Move data. !!Parameters are in opposite order from memcpy/memmove!!
XMPHandle XMPNewHandle(XMPULong howBig);
void XMPDisposeHandle(XMPHandle handle);
XMPHandle XMPCopyHandle( XMPHandle handle );
XMPULong XMPGetHandleSize(XMPHandle handle);
void XMPSetHandleSize(XMPHandle handle, XMPULong size);
void* XMPLockHandle(XMPHandle handle);
void XMPUnlockHandle(void* ptr);
void XMPUnlockHandle(XMPHandle handle);

#if XMPDebug
	// Debugging Utilities:
	
	XMPULong	XMPGetPtrSize( XMPPtr );
	
	XMPBoolean	XMPValidateHeap( XMPHeap heap = kXMPNULL );
	XMPBoolean	XMPValidateAllHeaps( );
	XMPBoolean	XMPValidateAllObjects( );
	XMPBoolean	XMPValidatePtr( XMPPtr block );
	XMPBoolean	XMPValidateObject( XMPObject *obj );
	XMPBoolean	XMPValidateHandle( XMPHandle h );
	
	void		XMPZapMem( XMPPtr anywhere, XMPULong size );
	XMPBoolean	XMPZapBlock( XMPPtr block );
	
	// Wrap these two calls around code you want to torture-test for memory problems.
	//  * All input pointers and handles will be checked for validity.
	//  * ASLM heaps will be checked for validity on every memory call.
	//  * New & disposed blocks will be filled with a repeating pattern of 0x50FF8001.
	//  * Warning if XMPUnlockHandle(void*) called on already-unlocked handle.
	void		XMPBeginMemValidation( );
	void		XMPEndMemValidation( );
#endif

#endif // _XMPMEM_