/*
	File:		XMPNew.cpp

	Contains:	Implementation of a new operator for OpenDoc

	Written by:	Nick Pilch

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	  3/1/94	JA		Added includes (1147588)
		 <2>	 2/22/94	VL		JA: Used XMP Memory calls instead of ASLM
									memory calls.
		 <5>	  2/7/94	JA		Capitalized THROW.
		 <4>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		 <3>	11/24/93	VL		Made delete work for NULL.
		 <2>	11/23/93	VL		Added heap-aware overloaded operator new.
		 <1>	11/19/93	NP		first checked in

	To Do:
*/

#ifndef _PLFMTYPE_
#include "PlfmType.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef __LIBRARYMANAGER__
#include <LibraryManager.h>
#endif

#pragma segment XMPNew

/*
These implementations of operator new and operator delete supplement the inline
versions found in XMPNew.h. The inline versions cannot be used sometimes. For
example, new ClassName[10] will cause some compilers to generate code inline
that calls new for each class instance.

The object code from this file must appear before any library in the link line.

The ASLM default pool is used for memory allocations.
*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Local Classes
//==============================================================================

//==============================================================================
// Global Variables
//==============================================================================

//==============================================================================
// Function Prototype
//==============================================================================

//==============================================================================
// Global new and delete operators
//==============================================================================

//------------------------------------------------------------------------------
// operator new
//------------------------------------------------------------------------------

void* operator new(size_t size)
{
	return XMPNewPtr(size);
}

//------------------------------------------------------------------------------
// operator new
//------------------------------------------------------------------------------

void* operator new(size_t size, XMPHeap heap)
{
	return XMPNewPtr(size,heap);
}

//------------------------------------------------------------------------------
// operator delete
//------------------------------------------------------------------------------

void operator delete(void* object)
{
	XMPDisposePtr(object);
}

