/*
	File:		XMPNew.h

	Contains:	Inline versions of operator new and operator delete for OpenDoc

	Written by:	Nick Pilch

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 2/22/94	VL		JA: Used XMP Memory calls instead of ASLM
									Memory calls.
		 <2>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		 <1>	11/19/93	NP		first checked in

	To Do:
*/

#ifndef _XMPNEW_
#define _XMPNEW_

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef __LIBRARYMANAGER__
#include <LibraryManager.h>
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
These inline versions are supplemented by "out-line" versions in XMPNew.cp. You
only have to use this file if you want to use the new operator before
InitLibraryManager is called.

The ASLM default pool is used for memory allocations.
*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Classes defined in this interface
//==============================================================================

//==============================================================================
// Classes used by this interface
//==============================================================================

XMPPtr	XMPNewPtr( XMPULong );
void	XMPDisposePtr( XMPPtr );

//==============================================================================
// Global new and delete operators
//==============================================================================

//void* operator new(size_t size, TMemoryPool*);	// default size, from a pool
//void* operator new(size_t);						// from global pool
//void operator delete(void*);						// return memory to pool 

inline void* operator new(size_t size)
{
	return XMPNewPtr(size);
}

inline void operator delete(void* theMem)
{
	XMPDisposePtr(theMem);
}

#endif // _XMPNEW_