/*
	File:		XMPUtils.cpp

	Contains:	utility functions

	Written by:	Tantek elik

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 2/23/94	JA		Switched to XMP memory fns.
		 <1>	  2/7/94	JA		first checked in
		----------------------------Renamed from Utility.cp to XMPUtils.cpp
		 <9>	12/21/93	VL		Changed XMPStorageUnit::GetValueSize to
									XMPStorageUnit::GetSize.
		 <8>	 11/7/93	T		add XMPGetValuePtr
		 <7>	 8/19/93	T		use XMPMem.h XMPNewPtr
		 <6>	 7/16/93	T		#include "SUView.h"
		 <5>	  7/2/93	T		add XMPName i/o calls
		 <4>	 6/27/93	T		add XMPReleaseRefCntObject
		 <3>	  6/8/93	T		change XMPDeleteField to XMPDeleteObject
		 <2>	  5/3/93	NP		Fixed #include statement.
		 <1>	 4/27/93	T		first checked in

	To Do:
*/

#ifndef _XMPUTILS_
#include "XMPUtils.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _SUView_
#include "SUView.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif


XMPName*	XMPBlankName()
{
	XMPName* name = new XMPName();
	name->theScriptCode = kXMPNULL;
	name->theLangCode = kXMPNULL;
	name->theText[0] = kXMPNULL;
	return name;
}


XMPName*	XMPGetName(XMPStorageUnitView* suView)
// $$$$$ Assumes length of name <= 255 chars
{
	ScriptCode	aScriptCode;
	LangCode	aLangCode;
	char 		buffer[256];
	
	XMPSShort length = -1;
	suView->GetValue(sizeof(aScriptCode),(XMPValue)&aScriptCode);
	suView->GetValue(sizeof(aLangCode),(XMPValue)&aLangCode);
	do {
		suView->GetValue(sizeof(char),&(buffer[++length]));
	} while (buffer[length]!=kXMPNULL);
	WASSERTM(length<256,"Oh #@*&, overflowed buffer in XMPGetName");
	
	XMPName* name = (XMPName*)(XMPNewPtr(sizeof(XMPName)+length*sizeof(char)));
	name->theScriptCode = aScriptCode;
	name->theLangCode = aLangCode;
	while (length>=0) {
		name->theText[length] = buffer[length];
		--length;
	}
	return name;
}

void		XMPSetName(XMPStorageUnitView* suView, XMPName* name)
{
XMPUnused(suView);
XMPUnused(name);
}

void		XMPInsertName(XMPStorageUnitView* suView, XMPName* name)
{
XMPUnused(suView);
XMPUnused(name);
}

void	XMPDeleteName(XMPName** name)
{
	if (*name != kXMPNULL) {
		XMPDisposePtr(*name);
		*name = kXMPNULL;
	}
}


XMPPtr		XMPGetValuePtr(XMPStorageUnit* su)
{
	XMPPtr ptr = XMPNewPtr(su->GetSize());
	if (ptr) su->GetValue(su->GetSize(),(XMPValueType)ptr);
	return ptr;
}
