/*
	File:		XMPUtils.h

	Contains:	utility functions

	Written by:	Tantek elik

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <1>	  2/7/94	JA		first checked in
		----------------------------Renamed from Utility.h to XMPUtils.h
		 <8>	11/18/93	JA		Added a comment explaining the purpose of
									the "do...while"s in the macros.
		 <7>	 11/7/93	T		add XMPGetValuePtr
		 <6>	 9/08/93	jpa		Wrapped "do{...}while(0)" around macros to avoid ultra-nasty
		 								problems with any "else" statement following the macro.
		 <5>	  7/2/93	T		add XMPName i/o calls
		 <4>	 6/27/93	T		add XMPReleaseRefCntObject
		 <3>	  6/8/93	T		change XMPDeleteField to XMPDeleteObject
		 <2>	 4/29/93	T		change names of includefiles
		 <1>	 4/27/93	T		first checked in

	To Do:
*/

#ifndef _XMPUTILS_
#define _XMPUTILS_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif


class XMPStorageUnit;
class XMPStorageUnitView;


// The "do...while(0)" blocks wrapped around these macros are there to prevent syntactic
// problems if a macro call is immediately followed by an "else" statement, to keep
// the "else" from binding to the "if" in the macro body. The "while" loop is optimized
// out by the compiler and has no effect on the flow of control or code quality.  --Jens


#define	XMPDeleteObject(object)	\
	do{							\
		if (object!=kXMPNULL) {	\
			delete object;		\
			object = kXMPNULL;	\
		}						\
	}while(0)

#define	XMPReleaseObject(object) \
	do{							\
		if (object!=kXMPNULL) {	\
			object->Release();	\
			object = kXMPNULL;	\
		}						\
	}while(0)


XMPName*	XMPBlankName();
XMPName*	XMPGetName(XMPStorageUnitView* suView);
void		XMPSetName(XMPStorageUnitView* suView, XMPName* name);
void		XMPInsertName(XMPStorageUnitView* suView, XMPName* name);

void		XMPDeleteName(XMPName** name);

XMPPtr		XMPGetValuePtr(XMPStorageUnit* su);

#endif //_XMPUTILS_

