/*
	File:		ClipbdM.h

	Contains:	Class definition for XMPMacClipboard.

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<16>	  4/4/94	CC		SetPlatformClipboard() &
									ExportPlatformClipboard(): Parameter
									changed from XMPTypeSet to XMPTypeList
									(1153046).
		<15>	 3/18/94	CC		Added private members ValidateKey() and
									fLockCount. (1151853)
		<14>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<13>	 2/15/94	CC		Bug #1142933 - Added key parameter to
									SetPlatformClipboard.
		<12>	  2/4/94	VL		Moved to PPC Header and code cleanup.
		<11>	 1/28/94	CC		Added fNextChangeID; removed include of
									Clipbd.h.
		<10>	 1/21/94	CC		Replaced GetTimeStamp()with GetChangeID;
									Lock() returns boolean result and returns
									key in second parameter.
		 <9>	 1/14/94	CC		Added type XMPClipboardKey, added key
									parameter to various routines, replaced
									GetGeneration() with GetTimeStamp, replaced
									GetStorageUnit() with
									GetContentStorageUnit().
		 <8>	 1/11/94	T		Init... changes
		 <7>	12/15/93	T		more InitObject changes, remove
									Initialize() method
		 <6>	 12/2/93	CC		Added Lock(), Unlock(), and GetGeneration()
		 <5>	10/27/93	CC		Added methods to transfer to/from the
									platform clipboard.
		 <4>	10/20/93	CC		Added session parameter to XMPClipboard
									constructor; added ExportClipboard
									method; added code to create the clipboard
									container.
		 <3>	  8/9/93	NP		Removed wierd character from file.
		 <2>	  8/9/93	NP		Added class id string.
		 <1>	  7/1/93	VL		first checked in

	To Do:
*/

#ifndef _CLIPBDM_
#define _CLIPBDM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================
class XMPMacClipboard;

//=====================================================================================
// Classes used by this interface
//=====================================================================================
class XMPAbsClipboard;
class XMPStorageUnit;
class XMPDraft;
class XMPDocument;
class XMPContainer;
class XMPSession;
class XMPSimpleFocusModule;
class XMPPlatformTypeList;

//=====================================================================================
// XMPClipboard
//=====================================================================================

#define kXMPMacClipboardID "appl:xmpmacclipboard$class,1.0.0"

class XMPMacClipboard : public XMPAbsClipboard
{
public:
		
	XMPMacClipboard();
	
	XMPVMethod ~ CDECL XMPMacClipboard();

	XMPNVMethod void InitClipboard(XMPSession* session);
		
	XMPVMethod XMPSize Purge(XMPSize size);

	XMPVMethod XMPBoolean Lock(XMPULong wait, XMPClipboardKey* key);
                              
	XMPVMethod void Unlock(XMPClipboardKey key);

	XMPVMethod XMPChangeID GetChangeID();
                               
	XMPVMethod void Clear(XMPClipboardKey key);
		
	XMPVMethod XMPStorageUnit* GetContentStorageUnit(XMPClipboardKey key);
		
	XMPVMethod void SetPlatformClipboard(
						XMPPlatformTypeList* typeList,
						XMPClipboardKey key);

	XMPVMethod void ExportClipboard(XMPClipboardKey key);
		
private:

	XMPMethod void ValidateKey(XMPClipboardKey key);
	XMPMethod void NewClipboard ();
	XMPMethod void DiscardClipboard ();
	XMPMethod void ImportContent();
	XMPMethod void PutContentOnPlatformClipboard();
	XMPMethod void ExportPlatformTypes(XMPPlatformTypeList* typeList);
	XMPMethod void PutClipboardOnPlatformClipboard();
	XMPMethod Boolean ImportClipboard();

	XMPSession*				fSession;
	Handle					fContainerHandle;
	XMPContainer*			fContainer;
	XMPDocument*			fDocument;
	XMPDraft*				fDraft;
	XMPStorageUnit*			fSU;
	
	XMPClipboardKey			fLockCount;
	XMPChangeID				fChangeID;
	XMPChangeID				fNextChangeID;

	short					fScrapCount;			// Mac desk scrap counter of last import
	short					fScrapCountLastChange;	// Need to update platform clipboard
};

#endif // _CLIPBDM_