/*
	File:		XMPDispatcher.h

	Contains:	Interface to XMPMacDispatcher class.

	Written by:	Richard Rodseth

	Copyright:	 1992-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<22>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<21>	  2/9/94	NP		Tiger Team cleanup.
		<20>	  2/2/94	RR		Made GetMouseRegion public
		<19>	  2/1/94	RR		Added AddMonitor and RemoveMonitor
		<18>	 1/26/94	RR		Added InvalidateFacetUnderMouse
		<17>	 1/21/94	RR		Updated method visibility. Misc. cleanup
		<16>	 1/15/94	RR		WinState.h -> WinStat.h, Dispatch.h ->
									Disptch.h
		<15>	 1/14/94	RR		Removed FindFacetUnderPoint
		<14>	 1/11/94	T		Init... changes
		<13>	12/22/93	RR		XMPMethod
		<12>	12/20/93	RR		New init strategy
		<11>	 12/8/93	RR		XMPMethod -> XMPVMethod and other cleanup
		<10>	 12/2/93	RR		Use new eventType definition
		 <9>	11/16/93	RR		Added GetSleepTime, SetIdleFrequency and
									FindFacetUnderPoint
		 <8>	11/12/93	JBS		use Facets
		 <7>	10/27/93	RR		Added private methods
									MouseRegionIsInvalid(),
									ComputeMouseRegion(), and
									CheckFrameUnderMouse
		 <6>	 8/17/93	RR		Delete unused fields fSEDispatchModule etc.
		 <5>	 8/11/93	RR		Restored support for idle frames using
									FrameSet class
		 <7>	  8/10/93	RCR		Use FrameSet class for idle frames
		 <3>	  8/9/93	NP		Added class id string.
	    <22>	 5/27/93	RCR		Added Yield
	    <21>	 5/27/93	RCR		Separated from Disptch.h
	    <20>	 5/27/93	RCR		Introduce prototype dispatch table
	    <19>	 5/18/93	RCR		Added Initialize/Purge
		<18>	  4/29/93	RCR		Rename Includes 
		<17>	 4/23/93	RCR		Added fMouseRegion and fFrameUnderMouse + Getters/Setters.
		<16>	 3/29/93	RCR		Added fIdleFrames, RenoveIdleFrame and HandleNullEvent.
		<15>	 3/13/93	RCR		Added kMouseDownInborder event.
		<15>	  3/10/93	RCR		Derive from XMPObject
		<14>	  3/8/93	RCR		Added kMenuEvent
		<13>	  3/3/93	NP		Added fields for dispatch modules.
		<12>	 2/25/93	RCR		SleepArea -> MouseRegion
		<11>	 2/17/93	RCR		Added support for event monitors, in response to review feedback
		<10>	 2/10/93	RCR		Remove canpatch,canremove from dispatchmodule constructor
		 <9>	  2/8/93	RCR		Rename RegisterDispatchModule to InstallDispatchModule
		 <8>	 1/28/93	RCR		Remove globals. Integrate with SystemInterface
		 <7>	 1/26/93	VL		Changed __DISPATCHER__ to _ XMPDISPATCHER_.
		 <6>	 1/21/93	RCR		Removed RedispatchMenu
		 <5>	 1/18/93	RCR		Minor changes from review
		 <4>	 1/12/93	RCR		Added Terminate and IsTerminated so shell can check if main
									event loop should continue
		 <3>	  1/7/93	RCR		Mostly formatting
		 <2>	  1/4/93	RCR		First cut
		 <1>	12/18/92	RCR		first checked in

	To Do:
*/

#ifndef _DISPATCHM_
#define _DISPATCHM_

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPMacDispatcher;

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPFacet;
class XMPFrame;
class XMPPart;
class XMPSession;
class IdleList;
class DispatchTable;

//=====================================================================================
// XMPMacDispatcher
//=====================================================================================

#define kXMPMacDispatcherID "appl:xmpmacdispatcher$class,1.0.0"

class XMPMacDispatcher : public XMPAbsDispatcher
{
public:
	
	XMPMacDispatcher();
	
	XMPVMethod ~ CDECL XMPMacDispatcher();
	
	XMPNVMethod void InitDispatcher(XMPSession* session); 
	
	XMPMethod	XMPSize	Purge(XMPSize size); 
	
	XMPMethod void AddDispatchModule(XMPEventType eventType, 
										XMPDispatchModule* dispatchModule);
	
	XMPMethod void RemoveDispatchModule(XMPEventType eventType);
	
	XMPMethod void AddMonitor(XMPEventType eventType,XMPDispatchModule* dispatchModule);
	
	XMPVMethod void RemoveMonitor(XMPEventType eventType, XMPDispatchModule* dispatchModule);
	
	XMPMethod XMPBoolean Dispatch(XMPEventData event);
	
	XMPMethod void RegisterIdle(XMPPart* part, XMPFrame* frame, XMPIdleFrequency frequency);	
	
	XMPMethod void UnregisterIdle(XMPPart* part, XMPFrame* frame);

	XMPMethod void SetIdleFrequency(XMPPart* part, XMPFrame* frame, XMPIdleFrequency frequency);	
	
	XMPMethod	void TerminateEventLoop();
	
	XMPMethod XMPBoolean ShouldTerminateEventLoop();
	
	XMPMethod void Yield(XMPFrame* frame);

public: // Macintosh-specific

	XMPMethod void SetMouseRegion(XMPRgnHandle area);
	
	XMPMethod XMPRgnHandle GetMouseRegion();
	
	XMPMethod void InvalidateFacetUnderMouse();

public: // Private to UI Subsystem

	XMPMethod void InvalidateMouseRegion();
	
	XMPMethod XMPFacet* GetFacetUnderMouse();
	
	XMPMethod void SetFacetUnderMouse(XMPFacet* facet);
	
	XMPMethod XMPBoolean HandleNullEvent(XMPEventData event, XMPFrame* skipFrame);
	
	XMPMethod XMPSLong GetSleepTime();

	
protected:

	XMPMethod XMPBoolean MouseRegionIsInvalid();
	XMPMethod void ComputeMouseRegion();
	XMPMethod void CheckFacetUnderMouse();

protected:

	XMPSession* fSession;	// Access to globals
	
private:

	XMPBoolean	fTerminateEventLoop; //	This is set to TRUE when the dispatcher thinks
									 // the shell application ought to Quit.

	DispatchTable* fDispatchTable;
	IdleList* fIdleList;	

	RgnHandle fMouseRegion;			
	RgnHandle fPartMouseRegion;
	XMPFacet* fFacetUnderMouse;
};


#endif // _DISPATCHM_
