/*
	File:		Draft.h

	Contains:	definition for XMPDraft

	Written by:	Vincent Lo, Tantek I. Celik

	Copyright:	 1992 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<59>	  4/6/94	JBS		1155480
		<58>	 3/25/94	CC		Added kXMPCloneDropCopy and
									kXMPCloneDropMove to XMPCloneKind
									(1153424).
		<57>	 3/25/94	MB		Symantec ASLM fixes. #1150864
		<56>	 3/25/94	VL		#1147173, #1144732: made this class
																		platform independent.
		<55>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<54>	 3/14/94	CC		Added internal methods FixReusedLinks() and
									ReferencedStorageUnitCloned(); added
									parameter to PrepareLinksForClone().
		<53>	 2/18/94	VL		#1142972, #1144024: Added
									GetDragDropStorageUnit to enable importing
									data from a foreign source (e.g. the
									Finder).
		<52>	  2/4/94	VL		Moved to PPC Header and began code cleanup.
		<51>	  2/2/94	RR		Added isRoot to CreateFrame
		<50>	  2/1/94	CC		Added the enumerated type XMPCloneKind;
									Added XMPCloneKind argument to BeginClone;
									added internal field fCloneKind and two
									private methods.
		<49>	 1/28/94	CC		Removed UniqueChangeID(),
									ShowLinkBorders(), and SetLinkBorders().
		<48>	 1/27/94	VL		Added IsValidDraftKey.
		<47>	 1/25/94	CC		Added fShowLinkBorders field.
		<46>	 1/21/94	CC		Added CreateLinkSource(), GetLinkSource(),
									RemoveLinkSource(),
									GetLinkSourceIterator(), and
									UniqueChangeID().
		<45>	 1/18/94	CC		Added ShowLinkBorders() and
									SetLinkBorders().
		<44>	12/21/93	VL		Used new Init strategy. Added support for
									persistent link id. Removed friends for
									Document and StorageUnit.
		<43>	12/21/93	JBS		CreateFrame with viewType & presentation
		<42>	12/20/93	VL		Removed Window-related calls. XMPWindow is
									no longer a persistent object.
		<41>	12/16/93	T		CreatePart now takes an optional Editor
									parameter
		<40>	 12/8/93	CC		GetLink: Renamed 2nd parameter from
									theLinkSpec to linkSpec
		<39>	11/30/93	VL		Changed frameGroup from XMPUShort to
									XMPULong.
		<38>	11/23/93	VL		Added fHeap, GetHeap().
		<37>	11/15/93	CC		Added CreateLinkSpec() method
									CreateLink() takes XMPPart* parameter
		<36>	11/12/93	JBS		remove Layout methods
		<35>	11/11/93	PH		Add forward declaration of XMPLinkSpec
		<34>	 11/9/93	PH		Added CreateDragDropStorageUnit
		<33>	 11/1/93	VL		Added BeginClone, EndClone, AbortClone.
		<32>	10/21/93	RR		CreateWindow takes new window flags (these
									are Mac-specific and Draft isn't, but we
									need a new Initialize strategy anyway for
									SOM. Stay tuned)
		<31>	 10/7/93	RR		Added root part to CreateWindow
		<30>	 10/7/93	VL		Removed InitSemanticInterface, fIsFinished
									and added SetChangedFromPrev.
		<29>	 9/30/93	VL		Removed SetFinished.
		<28>	 9/23/93	VL		Added GetPermissions.
		<27>	 9/21/93	VL		Moved ChangedFromPrev method to private by
									convention and added another parameter to
									it.
		<26>	 9/14/93	VL		Added GetDraftGut to make calls reentrant.
		<25>	  9/8/93	VL		Support Flushing.
		<24>	  9/3/93	JBS		updated to support new Layout class
		<23>	 8/26/93	VL		Added Flush and FlushVersion in preparation
									for Bento Flush functionality.
		<22>	 8/23/93	VL		Added private fExternalized, private
									NeedExternalizing, FailIfNotExclusiveWrite
									and changed return value to void for
									SetChangedFromPrev.
		<21>	 8/20/93	VL		Added private ExternalizeCollections.
		<20>	 8/18/93	VL		ReleaseStorageUnit takes an ID now.
		<19>	  8/9/93	VL		Removed bogus types.
		<18>	  8/9/93	NP		Added class id string.
		<17>	  8/6/93	VL		Added CreateCollections and
									DeleteCollections.
		<16>	  8/6/93	PH		Storage integration
		<15>	  8/6/93	VL		Added support for getting
									PersistentObjects.
		<14>	 7/29/93	VL		Removed getting persistent objects by name.
		<13>	 7/14/93	VL		Fixed problem with fStorageUnits defined
									twice.
		<12>	 7/14/93	VL		Used AEHashTable for fStorageUnits.
		<11>	  7/2/93	T		fix IntlText compile bugs
		<10>	  7/1/93	VL		CreateSU should take XMPType, not XMPName.
		 <9>	 6/30/93	VL		Added Reinitialize, Open, Close, Abort,
									IsNewDraft and utility calls like
									Create/Open/Close/AbortVersion.
		 <8>	 6/22/93	VL		Removed fDraftRefCount. Added
									fChangedFromPrev, fEmbeddedCotnainer and
									SetChangedFromPrev().
		 <7>	 6/15/93	VL		Added fCMContainer to keep track of the
									embedded container.
		 <6>	 6/10/93	NP		Changed some parameter type for XMPType.
		 <5>	  6/2/93	VL		Added fDraftRefCount.
		 <4>	 4/29/93	VL		8.3 Name Change.
		 <3>	  4/9/93	VL		XMPDraft should be subclassed from
									XMPRefCntObject.
		 <2>	  4/7/93	VL		Moved parameters from Initialize to
									constructor.
		 <1>	  4/6/93	VL		first checked in

	To Do:
		1) Take out const names.
		2) Do we need fEmbeddedContainer or fCMContainer? Should we always get it from fDocument?
		3) XMPPositionCode is included only for XMPPositionCode.
		4) XMPDraftKey is defined in StorageU.h.
*/

#ifndef _DRAFT_
#define _DRAFT_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _REFCTOBJ_
#include "RefCtObj.h"
#endif

//==============================================================================
// Constants
//==============================================================================

enum {
	kDPNone 			= 0,
	kDPTransient		= 1,
	kDPReadOnly			= 2,
	kDPSharedWrite		= 3,
	kDPExclusiveWrite	= 4
};

typedef XMPULong XMPDraftPermissions;

typedef const enum {
	kInvisibleBlocks	= 1,
	kAllBlocks			= 2,
	kVisibleBlocks		= 3
}	XMPPurgePriority;

typedef enum {
	kXMPCloneCopy	= 0,
	kXMPCloneCut,
	kXMPClonePaste,
	kXMPCloneDuplicate,
	kXMPCloneDropCopy,
	kXMPCloneDropMove,
	kXMPCloneAll				// For private use only
} XMPCloneKind;

//==============================================================================
// Scalar Types
//==============================================================================
typedef XMPULong	XMPVersionID;
typedef	XMPID		XMPDraftID;
typedef	XMPID		XMPLinkID;
typedef	XMPISOStr	XMPDraftName;

//==============================================================================
// Classes defined in this interface
//==============================================================================
class XMPDraft;

//==============================================================================
// Classes used by this interface
//==============================================================================
class	XMPDocument;
class	XMPFrame;
class	XMPLink;
class	XMPLinkSource;
class	XMPLinkSpec;
class	XMPPart;
class	XMPShape;
class	XMPEmbeddedContainer;
class	XMPStorageUnit;

//==============================================================================
// XMPDraft
//==============================================================================

class XMPDraft : public XMPRefCntObject
{
public:

	XMPVMethod	XMPDocument*	GetDocument()
		= 0;
	
	XMPVMethod	XMPDraftID		GetID()
		= 0;
	
	XMPVMethod	XMPDraftName	GetName()
		= 0;

	XMPVMethod	void			SetName(XMPDraftName name)
		= 0;

	XMPVMethod	XMPStorageUnit*	GetDraftProperties()
		= 0;
	
	XMPVMethod	void IncrementRefCount()
		= 0;
		
	XMPVMethod	void Release()
		= 0;
		
	XMPVMethod	XMPDraftPermissions	GetPermissions()
		= 0;
		
	XMPVMethod	XMPStorageUnit*	CreateStorageUnit()
		= 0;
		
	XMPVMethod	XMPStorageUnit*	GetStorageUnit(XMPStorageUnitID id)
		= 0;
		
	XMPVMethod	void			RemoveStorageUnit(XMPStorageUnit* storageUnit)
		= 0;
	
	XMPVMethod	XMPDraftKey		BeginClone(XMPCloneKind kind)
		= 0;
	
	XMPVMethod	void			EndClone(XMPDraftKey key)
		= 0;
		
	XMPVMethod	void			AbortClone(XMPDraftKey key)
		= 0;

	XMPVMethod	XMPBoolean		ChangedFromPrev()
		= 0;

	XMPVMethod	void			SetChangedFromPrev()
		= 0;

	XMPVMethod	void			RemoveFromDocument()
		= 0;

	XMPVMethod	XMPDraft*		RemoveChanges()
		= 0;

	XMPVMethod	XMPDraft*		Externalize()
		= 0;
	
	XMPVMethod 	XMPDraft*		SaveToAPrevious(XMPDraft* to)
		= 0;

	XMPVMethod XMPFrame*	CreateFrame(
							XMPFrame*		containingFrame,
							XMPShape* 		frameShape,
							XMPPart* 		part,
							XMPTypeToken	viewType,
							XMPTypeToken	presentation,
							XMPBoolean		isRoot,
							XMPBoolean 		isOverlaid)
		= 0;
			 
	XMPVMethod XMPFrame*	GetFrame(XMPStorageUnitID id)
		= 0;
		
	XMPVMethod void		RemoveFrame(XMPFrame* frame)
		= 0;

	XMPVMethod XMPPart*	CreatePart(XMPType partType,XMPEditor optionalEditor)
		= 0;
		
	XMPVMethod XMPPart*	GetPart(XMPStorageUnitID id)
		= 0;
		
	XMPVMethod void		RemovePart(XMPPart* part)
		= 0;

	XMPVMethod XMPLinkSpec*		CreateLinkSpec (XMPPart* part, XMPPtr data, XMPULong size)
		= 0;
		
	XMPVMethod XMPLinkSource*	CreateLinkSource(XMPPart* part)
		= 0;
		
	XMPVMethod XMPLinkSource* 	GetLinkSource(XMPStorageUnitID id)
		= 0;
		
	XMPVMethod XMPLink*			GetLink(XMPStorageUnitID id,XMPLinkSpec* linkSpec)
		= 0;
		
	XMPVMethod void				RemoveLink(XMPLink* link)
		= 0;
		
	XMPVMethod void				RemoveLinkSource(XMPLinkSource* link)
		= 0;

public: // private by convention

	XMPDraft() {}
	
	XMPVMethod ~ CDECL XMPDraft() {}

	XMPVMethod void InitDraft(XMPDocument* document, XMPDraftID id, XMPDraftPermissions perms)
		= 0;
		
	XMPVMethod void	ReleaseFrame(XMPFrame* frame)
		= 0;
		
	XMPVMethod void	ReleasePart(XMPPart* part)
		= 0;
		
	XMPVMethod void	ReleaseLink(XMPLink* link)
		= 0;
		
	XMPVMethod XMPDraft*		ReleaseStorageUnit(XMPStorageUnitID id)
		= 0;
};

#endif	// _