/*
	File:		DragDrop.h

	Contains:	Class definition for Drag and Drop.

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<14>	 3/25/94	CC		Renamed GetStorageUnit() to
									GetContentStorageUnit() for consistency
									with links and clipboard (1153347).
		<13>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<12>	  2/4/94	VL		Moved to PPC Header and code cleanup.
		<11>	10/12/93	PH		Change DragItemIter constructor
		 <9>	 9/30/93	PH		Add support for multiple drag items
		 <8>	  7/1/93	VL		Included XMPObj.h.
		 <7>	  7/1/93	PH		Separate Abstract/Concrete classes
		 <6>	 6/29/93	RCR		Removed #include Container
		 <5>	 4/29/93	VL		8.3 Name Change.
		 <4>	 4/23/93	VL		Purge needs a ;
		 <3>	 4/23/93	VL		Added Initialize and Purge.
		 <2>	  4/9/93	VL		Updated comments.
		 <1>	  4/9/93	VL		first checked in

	To Do:
*/

#ifndef _DRAGDROP_
#define _DRAGDROP_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================
class XMPAbsDragAndDrop;

//=====================================================================================
// Classes used by this interface
//=====================================================================================
class XMPPart;
class XMPFrame;
class XMPSession;
class XMPStorageUnit;

//=====================================================================================
// XMPAbsDragAndDrop
//=====================================================================================

class XMPAbsDragAndDrop : public XMPObject
{
public:
	
	XMPAbsDragAndDrop(XMPSession* session) {XMPUnused(session);}
		
	XMPVMethod ~ CDECL XMPAbsDragAndDrop() {}
		
	XMPVMethod void InitDragAndDrop()
		= 0;

	XMPVMethod XMPSize Purge(XMPSize size)
		= 0;
	
	XMPVMethod void Clear()
		= 0;
		
	XMPVMethod XMPStorageUnit* GetContentStorageUnit()
		= 0;
		
	XMPVMethod XMPDropResult StartDrag(XMPFrame* srcFrame,
									XMPValueType imageType,
									XMPPtr image,
									XMPPart** destPart,
									XMPPtr refCon)
		= 0;
};

#ifdef PLATFORM_MACINTOSH
#ifndef _DRAGDRPM_
#include "DragDrpM.h"
#endif
#endif

#endif // _DRAGDROP_ 