/*
	File:		DragDrop.h

	Contains:	Class definition for Drag and Drop.

	Written by:	Paul Hartenstine

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<27>	 4/12/94	VL		1143573, 1153424: StartDrag() returns
									XMPDropResult which indicates copy or move
									or fail. 1153569: DragDrpM.h includes
									Bento-specific File (DDCMSUM.h).
		<26>	  4/1/94	T		#1154941: Get OpenDoc source to actually
									compile for PowerPC
		<25>	 3/25/94	CC		Renamed GetStorageUnit() to
									GetContentStorageUnit() for consistency
									with links and clipboard (1153347).
		<24>	 3/25/94	VL		#1147173, #1144732:Used DDCMSUM.h instead
									of DrgDrSUM.h.
		<23>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<22>	  2/4/94	VL		More code cleanup.
		<21>	  2/4/94	VL		Moved to PPC Header and code cleanup.
		<20>	11/23/93	PH		Rework for new promise and ASLM stuff
		<19>	11/19/93	PH		Added GetPromiseFromDragManager
		<18>	11/12/93	JBS		use Facets for dragging
		<17>	 11/9/93	PH		GetDataFromDragManager
		<16>	10/27/93	PH		Change statics for Kurt's problem list
		<15>	 10/6/93	JA		Fwd-ref XMPPoint struct since it's not in
									PlfmType.h anymore.
		<14>	 9/30/93	PH		Add support for multiple drag items
		<13>	 9/24/93	JA		Minor syntactic tweaks for THINK C++.
		<12>	 9/21/93	PH		Remove DragInfo struct and add GetDragRef
		<11>	 9/16/93	PH		Add API changes for highlighting
		<10>	 9/10/93	PH		Clean up draginfo structure
		 <9>	 8/17/93	PH		Add fSession
		 <8>	 8/17/93	PH		Remove old storage code
		 <7>	  8/9/93	NP		Added class id string.
		 <6>	  8/6/93	PH		Storage integration
		 <5>	  8/2/93	T		Fix for XMP build
		 <4>	 7/28/93	PH		Changes for new Drag Manager
		 <3>	  7/1/93	VL		Removed constants as they are already
									defined in DragDrop.h
		 <2>	  7/1/93	PH		First Checked In

	To Do:
*/

#ifndef _DRAGDRPM_
#define _DRAGDRPM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _DRAGDROP_
#include "DragDrop.h"
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================
class XMPMacDragAndDrop;

//=====================================================================================
// Classes used by this interface
//=====================================================================================
class XMPSession;
class XMPPart;
class XMPStorageUnit;
class XMPFacet;
class XMPFrame;
class XMPStorageSystem;
class XMPWindow;
class XMPWindowState;
class XMPDragItemIterator;
class XMPStorageUnitView;
class LinkedList;

struct XMPPoint;

//=====================================================================================
// Constants
//=====================================================================================
#define kXMPDragFlavor 					0x50415254	// 'PART'

#define kXMPdragHasLeftSourceFrame		0x00000001
#define kXMPdragIsInSourcePart			0x00000002
#define kXMPdragIsInSourceFrame			0x00000004

//=====================================================================================
// XMPMacDragAndDrop
//=====================================================================================

#define kXMPMacDragAndDropID "appl:xmpmacdraganddrop$class,1.0.0"

class XMPMacDragAndDrop : public XMPAbsDragAndDrop
{
public:
	
	XMPMacDragAndDrop(XMPSession* session);
		
	XMPVMethod ~ CDECL XMPMacDragAndDrop();
		
	XMPNVMethod void	InitDragAndDrop();
	
	XMPMethod	XMPSize	Purge(XMPSize size);

	XMPMethod void Clear();
		
	XMPMethod XMPStorageUnit* GetContentStorageUnit();
		
	XMPMethod XMPDropResult StartDrag(XMPFrame* srcFrame,
									XMPValueType imageType,
									XMPPtr image,
									XMPPart** destPart,
									XMPPtr refCon);
	
	// Macintosh specific methods
		
	XMPMethod XMPULong GetDragAttributes();
		
	XMPMethod DragReference GetDragReference();

private:

	static OSErr pascal MyDragTrackingHandler(short message, WindowPtr theWindow,
									          void *handlerRefCon, DragReference theDrag);

	static OSErr pascal MyReceiveDropHandler(WindowPtr theWindow, void *handlerRefCon,
                                             DragReference theDrag);

	static OSErr pascal MySendDataProc(FlavorType theType, void *refCon,
	                                   ItemReference theItem, DragReference theDrag);

	XMPFacet* FindTargetFacet(XMPPlatformWindow theWindow, XMPPoint mouse, XMPPoint* local);

	static OSErr pascal PromiseAEHandler(AppleEvent* message, AppleEvent* reply, long refCon);

	XMPSession*			 fSession;
	XMPWindowState*		 fWindowState;
	XMPStorageSystem*	 fStorageSystem;

	LinkedList			*fDragItemList;
	XMPBoolean			 fListFromHandler;
	XMPDragItemIterator *fDragItemIterator;

	XMPULong			 fAttributes;
	DragReference		 fDragReference;
	
	XMPFacet*			 fPrevFacet;
	XMPFrame*			 fSourceFrame;
	XMPPart*			 fSourcePart;
	XMPFacet*			 fDestFacet;
	
	AEEventHandlerUPP	fPromiseAEHandler;

	DragReceiveHandlerUPP	fReceiveHandler;
	DragTrackingHandlerUPP	fTrackingHandler;
	DragSendDataUPP			fSendDataHandler;
	
	XMPULong				fDropResult;

public:	// Private by convention

	void GetPromiseFromDragManager(XMPPart *sourcePart, XMPStorageUnitView *destSUView);
	void GetDataFromDragManager(XMPStorageUnitView *theSUView);
};

#endif // _DRAGDRPM_