/*
	File:		Facet.h

	Contains:	Definition of class XMPAbsFacet

	Written by:	Joshua Susser

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<13>	  4/6/94	JBS		1155477
		<12>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<11>	 2/16/94	JA		Stop including Point.h.
		<10>	  2/3/94	JA		Tiger Team Makeover!
		 <9>	 1/24/94	SS		Checkin for Joshua (see below)
		 <8+>	 1/13/94	JBS		active border displays fActiveShape
	   								added offscreen canvas support
		 <8>	12/21/93	JBS		removed Initialize()
		 <7>	12/15/93	JBS		add fActiveShape & fWindow, plus accessors
		 <6>	 12/3/93	T		return XMPFacet* from CreateEmebeddedFacet
		 <5>	11/19/93	PH		Add virtual keyword and fix for ASLM
		 <4>	11/18/93	JBS		remove fWindow, add mouse tracking
		 <3>	11/16/93	JBS		add child drawing and moving methods
		 <2>	11/16/93	JBS		add fIsSelected and fHighlight
		 <1>	11/12/93	JBS		first checked in
*/

#ifndef _FACET_
#define _FACET_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

//==============================================================================
// Types
//==============================================================================

typedef enum {
	kXMPNoHighlight = 0,
	kXMPFullHighlight,
	kXMPDimHighlight
} XMPHighlight;

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPAbsFacet;
class XMPFacetIterator;

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPFacet;
class FacetNode;
class List;
class NodeTraverser;
class XMPCanvas;
class XMPDragItemIterator;
class XMPFrame;
class XMPObject;
class XMPShape;
class XMPTransform;
class XMPWindow;

//=====================================================================================
// XMPAbsFacet
//=====================================================================================

class XMPAbsFacet : public XMPObject
{
	friend class XMPFacetIterator;

public:
	
	//---------------------------------
	// constructor

	XMPAbsFacet() {}
			
	XMPVMethod ~ CDECL XMPAbsFacet() {}
	
	XMPVMethod XMPSize Purge(XMPSize size)
		= 0;
	
	//---------------------------------
	// frames

	XMPVMethod XMPFrame* GetFrame()
		= 0;

	XMPVMethod XMPFacet* CreateEmbeddedFacet(XMPFrame* frame,
							XMPShape* clipShape, XMPTransform* externalTransform,
							XMPFacet* siblingFacet, XMPFramePosition position)
		= 0;
				
	XMPVMethod void RemoveFacet(XMPFacet* facet)
		= 0;

	XMPVMethod void MoveBefore(XMPFacet* child, XMPFacet* sibling)
		= 0;

	XMPVMethod void MoveBehind(XMPFacet* child, XMPFacet* sibling)
		= 0;

	XMPVMethod XMPFacet* GetContainingFacet()
		= 0;
	
	XMPVMethod XMPFacetIterator* CreateFacetIterator(XMPTraversalType traversalType, 
													 XMPSiblingOrder siblingOrder)
		= 0;


	//---------------------------------
	// clip shape & external transform
	
	XMPVMethod void ChangeGeometry(XMPShape* clipShape, XMPTransform* transform)
		= 0;

	//---------------------------------
	// clip shape
	
	XMPVMethod XMPShape* GetClipShape()
		= 0;

	XMPVMethod XMPShape* GetAggregateClipShape()
		= 0;

	XMPVMethod XMPShape* GetWindowAggregateClipShape()
		= 0;

	XMPVMethod void InvalidateAggregateClipShape()
		= 0;

	//---------------------------------
	// active shape
	
	XMPVMethod XMPShape* GetActiveShape()
		= 0;

	XMPVMethod void ChangeActiveShape(XMPShape* activeShape)
		= 0;

	//---------------------------------
	// external transform

	XMPVMethod XMPTransform* GetExternalTransform()
		= 0;

	XMPVMethod void InternalTransformChanged()
		= 0;

	XMPVMethod XMPTransform* GetFrameTransform()
		= 0;

	XMPVMethod XMPTransform* GetContentTransform()
		= 0;

	XMPVMethod XMPTransform* GetWindowFrameTransform()
		= 0;

	XMPVMethod XMPTransform* GetWindowContentTransform()
		= 0;

	XMPVMethod void InvalidateAggregateTransforms()
		= 0;

	//---------------------------------
	// canvas

	XMPVMethod XMPBoolean HasCanvas()
		= 0;

	XMPVMethod XMPCanvas* GetCanvas()
		= 0;

	XMPVMethod void SetCanvas(XMPCanvas* canvas)
		= 0;

	//---------------------------------
	// window

	XMPVMethod XMPWindow* GetWindow()
		= 0;

	XMPVMethod void SetWindow(XMPWindow* window)
		= 0;

	//---------------------------------
	// PartInfo

	XMPVMethod XMPInfoType GetPartInfo()
		= 0;

	XMPVMethod void SetPartInfo(XMPInfoType partInfo)
		= 0;

	//---------------------------------
	// drag&drop

	XMPVMethod void DragEnter(XMPPoint point, XMPDragItemIterator* dragInfo)
		= 0;

	XMPVMethod void DragWithin(XMPPoint point, XMPDragItemIterator* dragInfo)
		= 0;

	XMPVMethod void DragLeave(XMPPoint point)
		= 0;

	XMPVMethod XMPDropResult Drop(XMPPoint point, XMPDragItemIterator* dropInfo)
		= 0;

	//---------------------------------
	// drawing

	XMPVMethod void Update(XMPShape* invalidShape)
		= 0;

	XMPVMethod void Invalidate(XMPShape* invalidShape)
		= 0;

	XMPVMethod void Validate(XMPShape* validShape)
		= 0;

	XMPVMethod void Draw(XMPShape* invalidShape)
		= 0;

	XMPVMethod void DrawChildren(XMPShape* invalidShape)
		= 0;

	XMPVMethod void DrawChildrenAlways(XMPShape* invalidShape)
		= 0;

	XMPVMethod void DrawnIn(XMPShape* shape)
		= 0;

	XMPVMethod void InvalidateActiveBorder()
		= 0;

	XMPVMethod void DrawActiveBorder()
		= 0;

	//---------------------------------
	// mouse tracking and hit detection:

	XMPVMethod void MouseEnter(XMPPoint where)
		= 0;

	XMPVMethod void MouseWithin(XMPPoint where)
		= 0;
			
	XMPVMethod void MouseLeave()
		= 0;

	XMPVMethod XMPBoolean ContainsPoint(XMPPoint point)
		= 0;

	XMPVMethod XMPBoolean ActiveBorderContainsPoint(XMPPoint point)
		= 0;

	//---------------------------------
	// selection and highlighting:

	XMPVMethod XMPBoolean IsSelected()
		= 0;

	XMPVMethod void SetSelected(XMPBoolean isSelected)
		= 0;

	XMPVMethod XMPHighlight	GetHighlight()
		= 0;
	
	XMPVMethod void ChangeHighlight(XMPHighlight highlight)
		= 0;


protected:

	XMPVMethod FacetNode* GetNode()
		= 0;

	XMPVMethod XMPBoolean NeedsUpdate()
		= 0;

	XMPVMethod void SetNeedsUpdate(XMPBoolean needsUpdate)
		= 0;
};


//=====================================================================================
// XMPFacetIterator
//=====================================================================================

#define kXMPFacetIteratorID "appl:xmpfacetiterator$class,1.0.0"

class XMPFacetIterator
{
	ASLM_COMPATIBLE

public:

	XMPFacetIterator(XMPFacet* facet,
					 XMPTraversalType traversalType, 
					 XMPSiblingOrder siblingOrder);

	virtual ~ CDECL XMPFacetIterator();
	
	XMPMethod XMPFacet* First();
	
	XMPMethod XMPFacet* Next();
	
	XMPMethod void SkipChildren();
	
	XMPMethod XMPBoolean IsNotComplete();

private:

	XMPFacet*			fFacet;
 	XMPTraversalType	fTraversalType;
 	XMPSiblingOrder		fSiblingOrder;
	NodeTraverser*		fNodeTraverser;
};

#if PLATFORM_MACINTOSH
#ifndef _FACETM_
#include "FacetM.h"
#endif
#endif

#endif // _FACET_