/*
	File:		FacetM.h

	Contains:	Definition of class XMPMacFacet

	Written by:	Joshua Susser

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<14>	  4/6/94	JBS		1155477
		<13>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<12>	 2/16/94	JA		Stop including Point.h.
		<11>	  2/3/94	JA		Tiger Team Makeover!
		<10>	 1/24/94	SS		Checkin for Joshua (see below)
		 <9+>	 1/13/94	JBS		active border displays fActiveShape
	   								added offscreen canvas support
		 <9>	 1/11/94	T		Init... changes
		 <8>	12/21/93	JBS		InitFacet changes
		 <7>	12/15/93	JBS		add fActiveShape & fWindow, plus accessors
		 <6>	 12/3/93	T		return XMPFacet* from CreateEmebeddedFacet
		 <5>	11/19/93	PH		Add class id and fix for ASLM
		 <4>	11/18/93	JBS		remove fWindow, add mouse tracking
		 <3>	11/16/93	JBS		add child drawing and moving methods
		 <2>	11/16/93	JBS		add fIsSelected, fHighlight
		 <1>	11/12/93	JBS		first checked in
*/

#ifndef _FACETM_
#define _FACETM_

#ifndef _FACET_
#include "Facet.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPMacFacet;

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPAbsFacet;
class XMPFacetIterator;
class FacetNode;
class List;
class NodeTraverser;
class XMPCanvas;
class XMPDragItemIterator;
class XMPFrame;
class XMPShape;
class XMPTransform;
class XMPWindow;

//=====================================================================================
// XMPFacet
//=====================================================================================

#define kXMPMacFacetID "appl:xmpmacfacet$class,1.0.0"

class XMPMacFacet : public XMPAbsFacet
{
	friend class XMPFacetIterator;

public:
	
	//---------------------------------
	// constructor

	XMPMacFacet();
			
	virtual ~ CDECL XMPMacFacet();
	
	XMPNVMethod void	 InitFacet(XMPFrame* frame,
									XMPShape* clipShape,
									XMPTransform* externalTransform);

	XMPVMethod XMPSize Purge(XMPSize size);
	
	//---------------------------------
	// frames

	XMPVMethod XMPFrame* GetFrame();

	XMPVMethod XMPFacet* CreateEmbeddedFacet(XMPFrame* frame,
							XMPShape* clipShape, XMPTransform* externalTransform,
							XMPFacet* siblingFacet, XMPFramePosition position);
				
	XMPVMethod void RemoveFacet(XMPFacet* facet);

	XMPVMethod void MoveBefore(XMPFacet* child, XMPFacet* sibling);

	XMPVMethod void MoveBehind(XMPFacet* child, XMPFacet* sibling);

	XMPVMethod XMPFacet* GetContainingFacet();
	
	XMPVMethod XMPFacetIterator* CreateFacetIterator(XMPTraversalType traversalType, 
													 XMPSiblingOrder siblingOrder);

	//---------------------------------
	// clip shape & external transform
	
	XMPVMethod void ChangeGeometry(XMPShape* clipShape, XMPTransform* transform);

	//---------------------------------
	// clip shape
	
	XMPVMethod XMPShape* GetClipShape();

	XMPVMethod XMPShape* GetAggregateClipShape();

	XMPVMethod XMPShape* GetWindowAggregateClipShape();

	XMPVMethod void InvalidateAggregateClipShape();

	//---------------------------------
	// active shape
	
	XMPVMethod XMPShape* GetActiveShape();

	XMPVMethod void ChangeActiveShape(XMPShape* activeShape);

	//---------------------------------
	// transformations:

	XMPVMethod XMPTransform* GetExternalTransform();

	XMPVMethod void InternalTransformChanged();

	XMPVMethod XMPTransform* GetFrameTransform();

	XMPVMethod XMPTransform* GetContentTransform();

	XMPVMethod XMPTransform* GetWindowFrameTransform();

	XMPVMethod XMPTransform* GetWindowContentTransform();

	XMPVMethod void InvalidateAggregateTransforms();

	//---------------------------------
	// canvas

	XMPVMethod XMPBoolean HasCanvas();

	XMPVMethod XMPCanvas* GetCanvas();

	XMPVMethod void SetCanvas(XMPCanvas* canvas);

	//---------------------------------
	// window

	XMPVMethod XMPWindow* GetWindow();

	XMPVMethod void SetWindow(XMPWindow* window);

	//---------------------------------
	// PartInfo

	XMPVMethod XMPInfoType GetPartInfo();

	XMPVMethod void SetPartInfo(XMPInfoType partInfo);

	//---------------------------------
	// drag&drop

	XMPVMethod void DragEnter(XMPPoint point, XMPDragItemIterator* dragInfo);

	XMPVMethod void DragWithin(XMPPoint point, XMPDragItemIterator* dragInfo);

	XMPVMethod void DragLeave(XMPPoint point);

	XMPVMethod XMPDropResult Drop(XMPPoint point, XMPDragItemIterator* dropInfo);

	//---------------------------------
	// drawing

	XMPVMethod void Update(XMPShape* invalidShape);

	XMPVMethod void Invalidate(XMPShape* invalidShape);

	XMPVMethod void Validate(XMPShape* validShape);

	XMPVMethod void Draw(XMPShape* invalidShape);

	XMPVMethod void DrawChildren(XMPShape* invalidShape);

	XMPVMethod void DrawChildrenAlways(XMPShape* invalidShape);

	XMPVMethod void DrawnIn(XMPShape* shape);

	XMPVMethod void InvalidateActiveBorder();

	XMPVMethod void DrawActiveBorder();

	//---------------------------------
	// mouse tracking and hit detection:

	XMPVMethod void MouseEnter(XMPPoint where);

	XMPVMethod void MouseWithin(XMPPoint where);
			
	XMPVMethod void MouseLeave();

	XMPVMethod XMPBoolean ContainsPoint(XMPPoint point);

	XMPVMethod XMPBoolean ActiveBorderContainsPoint(XMPPoint point);

	//---------------------------------
	// selection and highlighting:

	XMPVMethod XMPBoolean IsSelected();

	XMPVMethod void SetSelected(XMPBoolean isSelected);

	XMPVMethod XMPHighlight	GetHighlight();
	
	XMPVMethod void ChangeHighlight(XMPHighlight highlight);


protected:

	XMPVMethod FacetNode* GetNode();

	XMPVMethod XMPBoolean NeedsUpdate();
	XMPVMethod void SetNeedsUpdate(XMPBoolean needsUpdate);

protected:

	XMPFrame*		fFrame;
	XMPShape*		fClipShape;
	XMPShape*		fActiveShape;
	XMPTransform*	fExternalTransform;
	XMPCanvas*		fCanvas;
	XMPWindow*		fWindow;
	XMPInfoType		fPartInfo;
	XMPBoolean		fIsSelected;
	XMPHighlight	fHighlight;

	FacetNode*		fNode;
	XMPBoolean		fNeedsUpdate;

	// cached values

	XMPShape*		fAggregateClipShape;
	XMPShape*		fWindowAggregateClipShape;
	XMPTransform*	fFrameTransform;
	XMPTransform*	fContentTransform;
	XMPTransform*	fWindowFrameTransform;
	XMPTransform*	fWindowContentTransform;
};

#endif // _FACETM_