/*
	File:		XMPFrame.h

	Contains:	Definition of class XMPFrame

	Written by:	Joshua Susser & Sepp Friedrich

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<32>	  4/6/94	JBS		1155479
		<31>	 3/25/94	VL		#1147173: Moved constructor, destructor and
									init methods to private by convention.
		<30>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<29>	 2/14/94	JBS		RADAR bug# 1143608 - remove SetRoot(),
									SetOverlaid(), fUsesCachedImage & accessors
		<28>	  2/2/94	RR		Added isRoot to InitFrame
		<27>	  2/2/94	NP		Added methods and XMPLinkStatus type for
									new linking protocol APIs.
		<26>	 1/27/94	VL		Added CloneTo.
		<25>	 1/15/94	VL		Added fGettingPart to prevent recursive
									GetPart.
		<24>	 1/13/94	JBS		add fIsSubframe, IsSubframe(),
									SetSubframe()
		<23>	 1/11/94	T		Init... changes
		<22>	12/21/93	JBS		InitFrame with viewType & presentation
		<21>	12/15/93	JBS		remove fActiveShape, fWindow
		<20>	12/14/93	T		InitFrame changes
		<19>	11/19/93	PH		Add virtual keyword and fix for ASLM
		<18>	11/18/93	JBS		add fWindow and accessors
		<17>	11/16/93	JBS		remove fHighlight, add fViewType and
									get/set
		<16>	11/12/93	JBS		use Facets
		<15>	10/12/93	PH		Remove DragDrop.h
		<14>	 9/30/93	PH		Support new Drag and Drop protocol
		<13>	 9/29/93	T		with RCR, added Close()
		<12>	 9/29/93	VL		Added InitializePersistent.
		<11>	 9/21/93	PH		Remove De/RegisterDroppableFrame
		<10>	 9/17/93	JBS		added fIsDroppable & fIsDragging &
									get/setters
		 <9>	  9/3/93	JBS		removed fWindow, fCanvas; added fLayout
		 <8>	 8/10/93	PH		Add release
		 <7>	  8/9/93	NP		Added class id string.
		 <6>	  8/6/93	JBS		add fHighlight and related methods
		 <5>	  7/8/93	JBS		replaced with prototype version

		<45>	 5/13/93	JBS		add ChangeActiveShape()
		<44>	  5/7/93	JBS		namesmith
		<43>	 4/29/93	JBS		8.3 file names
		<42>	 4/28/93	JBS		8.3 file names
		<39>	 4/19/93	NP		NameSmithing.
		<38>	 4/19/93	NP		NameSmithing.
		<37>	 4/19/93	NP		Namesmithing.
		<36>	  4/1/93	SF		privatized constructor/destructor/initialize					
		<35>	  4/1/93	SF		Updated "Theory of Operation"
		<34>	 3/30/93	SF		Correct constructor/Internalize usage and removal of Restore
		<33>	 3/29/93	SF		new GetAggregateClipShape().
		<32>	 3/29/93	JBS		add IntersectClipShape()
		<31>	 3/23/93	SF		subclass of XMPObject
		<30>	 3/22/93	SF		Creation broke up into XMPFrame(), Initialize / Restore
		<29>	 3/19/93	SF		Changed ChangedFrameShape to ChangeFrameShape ChangedUsedShape
									to ChangeUseShape; ChangeFrameShape returns void now.
		<28>	 3/18/93	SF		Changed names: NotifyReshape->ChangedUsedShape;
									Reshape->ChangedFrameShape; Added: RequestFrameShape
		<27>	 3/11/93	SF		Removed fCachedPresentation, XMPImage, fInvalidShape, and
									Get/SetCachedRepresentation
		<26>	 3/11/93	SF		Reshape returns a Shape
		<25>	 3/10/93	SF		don't know any more
		<24>	  3/2/93	SF		Removed 2 creation function; made remaining creators temporarly
									public for Window.cp and ContainingPartRCR.cp
		<23>	  3/2/93	SF		update with March 1 API documentation
		<22>	 2/26/93	TI		remove partInfo from XMPFrame constructor
		<21>	 2/25/93	VL		XMPFrame::DragLeave takes the mouse position now.
		<20>	  2/9/93	SF		Added creation: XMPFrame(XMPStorageUnit* su); changed
									GetFrameID() to GetID();
		<19>	  2/3/93	SF		Fixed comments.
		<17>	  2/3/93	SF		Added "Theory of Operation"
		<16>	  2/2/93	SF		remove XMPDisplayFramesIterator()
		<15>	  2/2/93	SF		Added protected GetStorageUnit() and GetFrameID(); removed class
									XMPEmbeddedFrameIterator;
		<14>	  2/1/93	RCR		Merge with prototype shell
		<13>	 1/29/93	SF		Changed fInvalid to fCachedTransfomValid + extended usage text
									for InvalidateCachedTransform();
		<12>	 1/28/93	SF		Changed "...FrameIterator" to "...FramesIterator"
		<11>	 1/27/93	SF		Adopt to Richard's JediShell: temporary create
		<10>	 1/27/93	SF		return value of Purge: XMPSize; cached transform is now the
									composite transform of contained frames with the contained
									frame's Inner and this frame's External transfom.
		 <9>	 1/26/93	VL		Changed __XXXX__ to _XXXX_.
		 <8>	 1/21/93	SF		Added comments about what to change
		 <7>	 1/20/93	SF		Added Internalize/Externalize, simple Draw(),
		 							fFrozen, fDisplayMyImage ("as is" and don't
									call partHandler). creation/destruction
									became protected. friend class XMPDraft.
									StorageUnit added to creation parameters.
		 <6>	 1/16/93	SF		Added creation parameters. Removed get/set
		 							for embedded frames.
									Added size parameter for purging. Updated
									XMPEmbeddedFrameIterator class.
		 <5>	 1/12/93	SF		Added class XMPEmbeddedFrameIterator and 
									creator function in XMPFrame.
		 <4>	 1/11/93	SF		Moved XMPXform to platform types.
		 <3>	 1/10/93	SF		Cosmetics
		 <2>	  1/7/93	SF		Changed headers, names ...
		 <1>	12/22/92	SF		first checked in
*/

#ifndef _FRAME_
#define _FRAME_

#ifndef _PSTOBJ_
#include "PstObj.h"	// base class
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"	// For XMPDraftKey
#endif


//==============================================================================
// Types
//==============================================================================

enum XMPLinkStatus
{
	kXMPInLinkDestination,
	kXMPInLinkSource,
	kXMPNotInLink
};

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPFrame;
class XMPFrameFacetIterator;

//==============================================================================
// Classes used by this interface
//==============================================================================

class OrderedCollection;
class OrderedCollectionIterator;
class XMPCanvas;
class XMPDraft;
class XMPFacet;
class XMPPart;
class XMPPersistentObject;
class XMPShape;
class XMPStorageUnit;
class XMPTransform;
class XMPDragItemIterator;
class XMPWindow;

struct XMPPoint;

//==============================================================================
// XMPFrame
//==============================================================================

#define kXMPFrameID "appl:xmpframe$class,1.0.0"

class XMPFrame : public XMPPersistentObject
{
	friend class XMPDraft;
	friend class XMPFrameFacetIterator;

public:

// getters/setters:

	XMPVMethod XMPFrame*	GetContainingFrame();
	XMPVMethod void 		SetContainingFrame(XMPFrame* frame);

	XMPVMethod XMPULong		GetFrameGroup();
	XMPVMethod void			SetFrameGroup(XMPULong groupID);

	XMPVMethod XMPULong		GetSequenceNumber();
	XMPVMethod void			SetSequenceNumber(XMPULong sequenceNumber);

	XMPVMethod XMPBoolean	IsRoot();

	XMPVMethod XMPBoolean	IsSubframe();
	XMPVMethod void			SetSubframe(XMPBoolean isSubframe);

	XMPVMethod XMPBoolean	IsOverlaid();

	XMPVMethod XMPBoolean	IsFrozen();
	XMPVMethod void			SetFrozen(XMPBoolean isFrozen);

	XMPVMethod XMPBoolean	DoesPropagateEvents();
	XMPVMethod void			SetPropagateEvents(XMPBoolean doesPropagateEvents);


// part

	XMPVMethod XMPPart* GetPart();

	XMPVMethod void ChangePart(XMPPart* part);

	XMPVMethod XMPInfoType	GetPartInfo();
	XMPVMethod void			SetPartInfo(XMPInfoType partInfo);

	XMPVMethod XMPTypeToken GetViewType();
	XMPVMethod void			SetViewType(XMPTypeToken viewType);
	XMPVMethod void			ChangeViewType(XMPTypeToken viewType);

	XMPVMethod XMPTypeToken GetPresentation();
	XMPVMethod void			SetPresentation(XMPTypeToken presentation);
	XMPVMethod void			ChangePresentation(XMPTypeToken presentation);

// facets

	XMPVMethod void FacetAdded(XMPFacet* facet);

	XMPVMethod void FacetRemoved(XMPFacet* facet);

	XMPVMethod XMPFrameFacetIterator* CreateFacetIterator();
	
	XMPVMethod XMPShape* GetFrameShape();

	XMPVMethod void ChangeFrameShape(XMPShape* shape);

	XMPVMethod XMPShape* RequestFrameShape(XMPShape* shape);

	
	XMPVMethod XMPShape* GetUsedShape();

	XMPVMethod void ChangeUsedShape(XMPShape* shape);

// transformations:

	XMPVMethod XMPTransform* GetInternalTransform();

	XMPVMethod void ChangeInternalTransform(XMPTransform* transform);

// drag&drop:

	XMPVMethod XMPBoolean	IsDroppable();
	XMPVMethod void			SetDroppable(XMPBoolean isDroppable);

	XMPVMethod XMPBoolean	IsDragging();
	XMPVMethod void			SetDragging(XMPBoolean isDragging);

// linking:

	XMPVMethod void				ContentChanged(XMPChangeID change);
	XMPVMethod void				ChangeLinkStatus(XMPLinkStatus status);
	XMPVMethod XMPLinkStatus	GetLinkStatus();

// invalidation/draw:

	XMPVMethod void Invalidate(XMPShape* invalidShape);

	XMPVMethod void Validate(XMPShape* validShape);

	XMPVMethod void InvalidateActiveBorder();

	XMPVMethod void DrawActiveBorder();

// memory management:

	XMPVMethod XMPSize Purge(XMPSize numBytes);		// override

	XMPVMethod XMPStorageUnit*	CloneTo(XMPDraftKey key, XMPDraft* destDraft);
	
	XMPVMethod void Externalize();					// override

// ref counting
	
	XMPVMethod void Release();						// override

	XMPVMethod void Close();

	XMPVMethod void Remove();
		

protected:	// only to be called by XMPFrameFacetIterator

	XMPVMethod OrderedCollection* GetFacets();

protected:

	void	CommonInitFrame();

	XMPFrame*			fContainingFrame;	// parent in frame hierarchy
	OrderedCollection*	fFacets;			// list of current facets

	XMPShape*		fFrameShape;			// basic shape of frame
	XMPShape*		fUsedShape;				// region to wrap around

	XMPTransform*	fInternalTransform;		// viewing transform

	XMPPart*		fPart;					// part the frame is displaying
	XMPInfoType		fPartInfo;				// data type of part
	XMPTypeToken	fViewType;				// type of view of part (icon, etc.)
	XMPTypeToken	fPresentation;			// kind of frame (content, palette, etc.)

	XMPULong		fFrameGroup;			// ID of group this frame is in
	XMPULong		fSequenceNumber;		// number of this frame in its group

	XMPBoolean		fIsRoot;				// is this a window's root frame?
	XMPBoolean		fIsSubframe;			// is this subframe of conglomerate frame?
	XMPBoolean		fIsOverlaid;			// is this an overlaid frame?
	XMPBoolean		fIsFrozen;				// mouse events go to my containing frame
	XMPBoolean		fIsDroppable;			// accepts drag/drop events
	XMPBoolean		fIsDragging;			// is being dragged
	XMPBoolean		fDoesPropagateEvents;	// frame can't handle delegated events
	
#ifdef DEBUG
	XMPBoolean		fGettingPart;			// Semaphore to ensure that GetPart is not called recursively.
#endif

public:	// public by convention.
		// only to be called by XMPDraft

	XMPFrame();
	XMPVMethod ~ CDECL XMPFrame();

	XMPNVMethod void	InitFrame(
				XMPStorageUnit*	storageUnit,
				XMPFrame* 		containingFrame,
				XMPShape* 		frameShape,
				XMPPart* 		part,
				XMPTypeToken	viewType,
				XMPTypeToken	presentation,
				XMPBoolean		isRoot,
				XMPBoolean 		isOverlaid);
				
	XMPNVMethod void	InitFrameFromStorage(XMPStorageUnit*	storageUnit);


};

//==============================================================================
// XMPFrameFacetIterator
//==============================================================================

#define kXMPFrameFacetIteratorID "appl:xmpframefacetiterator$class,1.0.0"

class XMPFrameFacetIterator
{
	ASLM_COMPATIBLE

public:

	XMPFrameFacetIterator(XMPFrame* frame);

	virtual ~ CDECL XMPFrameFacetIterator();

	XMPVMethod XMPFacet* First();
	
	XMPVMethod XMPFacet* Next();
	
	XMPVMethod XMPBoolean IsNotComplete();

	XMPFrame*					fFrame;
	OrderedCollectionIterator*	fIter;
};

#endif // _XMPFRAME_