/*
	File:		Info.h

	Contains:	function declarations for Part Frame Info dialog and getsetters

	Written by:	Tantek elik

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<12>	 3/31/94	eeh		bug #1153999: change defs of XMPGetPOName,
									XMPGetPOComments, and XMPGetPOModifiedBy
		<11>	 3/28/94	CG		1153547: Changed parameter list for
									XMPGetPOModifiedBy.
		<10>	 3/24/94	T		(no bug# yet) ShowPartFrameInfo returns a
									boolean indicating whether or not the User
									pressed OK
		 <9>	  3/1/94	JA		Made string consts extern. (1147295)
		 <8>	 2/15/94	T		fix 1143649
		 <7>	 1/11/94	T		add XMPType	XMPGetKind(XMPPart* part);
		 <6>	12/14/93	T		added kXMPViewAs constants
		 <5>	12/11/93	T		changed XMPGetPOModifiedBy to use Str32
									instead of IntlText because Str32 is the
									only type of username info we can extract
									from Mac
		 <4>	 12/8/93	T		#include "XMPMem.h"
		 <3>	 12/8/93	T		added Getters & Setters for IconFamily on
									Part
		 <2>	 12/7/93	T		added Part & Frame Property getters and
									setters
		 <1>	11/18/93	T		first checked in

	To Do:
*/


#ifndef _INFO_
#define _INFO_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

//==============================================================================
// Constants
//==============================================================================

// ViewAs ISOStr constants

extern const	XMPType	kXMPViewAsSmallIcon		; // = "OpenDoc:ViewAs:SmallIcon";
extern const	XMPType	kXMPViewAsLargeIcon		; // = "OpenDoc:ViewAs:LargeIcon";
extern const	XMPType	kXMPViewAsThumbnail		; // = "OpenDoc:ViewAs:Thumbnail";
extern const	XMPType	kXMPViewAsFrame			; // = "OpenDoc:ViewAs:Frame";

//==============================================================================
// Scalar Types
//==============================================================================

typedef XMPHandle	XMPIconFamily;

//==============================================================================
// Classes used by this interface
//==============================================================================

class	XMPFrame;
class	XMPStorageUnit;
class	XMPCategorySet;
class	XMPPart;
class	XMPPersistentObject;

//==============================================================================
// Info functions
//==============================================================================

//-------------------------------------------------------------------------------------
// Part & Frame Info Dialog
//-------------------------------------------------------------------------------------
// public

XMPBoolean	ShowPartFrameInfo(XMPFrame* frame);

//-------------------------------------------------------------------------------------
// Part & Frame Property getters and setters
// Note: 
//  all Getters expect you to pass in a buffer to be filled along 
// with the maxsize if the value being gotten is of variable size
// the actual size is returned in maxsize as well
//  all Setters do NOT consume what you pass in, so it is safe to pass in
// a reference to your internal structures.  They are only accessed for the duration
// of the Setter API call.
// These conventions were adopted to reduce memory allocation/deallocation.
//-------------------------------------------------------------------------------------
// private by convention

XMPName*	XMPGetPOName(XMPPersistentObject* pstobj,XMPName* name,XMPULong* maxsize);
void		XMPSetPOName(XMPPersistentObject* pstobj,XMPName* name);

XMPName*	XMPGetPOComments(XMPPersistentObject* pstobj,XMPIText* comments,XMPULong* maxsize);
void		XMPSetPOComments(XMPPersistentObject* pstobj,XMPIText* comments);

XMPULong	XMPGetPOSize(XMPPersistentObject* pstobj);
XMPULong	XMPGetPOCreationDate(XMPPersistentObject* pstobj);
XMPULong	XMPGetPOModificationDate(XMPPersistentObject* pstobj);
XMPName*	XMPGetPOModifiedBy(XMPPersistentObject* pstobj,XMPIText* comments,XMPULong* maxsize);

//-------------------------------------------------------------------------------------
// Part only Property getters and setters
//-------------------------------------------------------------------------------------
// private by convention
void	XMPGetCategories(XMPPart* part,XMPCategorySet* comments,XMPULong* maxsize);

XMPType	XMPGetKind(XMPPart* part);

void	XMPGetIconFamily(XMPPart* part,XMPIconFamily iconFamily);
void	XMPSetIconFamily(XMPPart* part,XMPIconFamily iconFamily);

XMPBoolean	XMPGetIsStationery(XMPPart* part);
void		XMPSetIsStationery(XMPPart* part,XMPBoolean isStationery);

//-------------------------------------------------------------------------------------
// Frame only Property getters and setters
//-------------------------------------------------------------------------------------
// see Frame.h

#endif // _INFO_

