/*
	File:		LinkDlgs.h

	Contains:	function declarations for Paste As and Link Info dialogs

	Written by:	Craig Carper

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	  4/6/94	CC		XMPTypeInfo: Changed field "creationDate"
									to "creationTime" and "modificationDate" to
									"changeTime". (1154722)
		 <5>	  4/5/94	CC		Removed unused declaration of class
									XMPTypeSet. (1153046)
		 <4>	 3/25/94	JA		Added missing #include (1147588)
		 <3>	 3/24/94	CC		ShowLinkSourceInfo() takes XMPChangeID
									parameter; XMPPasteAsResult specifies
									original kind when selected kind is a
									translation (1153068).
		 <2>	 3/17/94	CC		Added ShowLinkSourceInfo(),
									ShowLinkDestinationInfo(), XMPLinkAction,
									XMPLinkResult, and XMPLinkInfo. (1151329)
		 <1>	 3/14/94	CC		first checked in
		 <0>	 3/14/94	CC		Declaration of ShowPasteAsDialog (1144253).

	To Do:
*/


#ifndef _LINKDLGS_
#define _LINKDLGS_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"		// jpa: Needed 'cause inline fn uses it
#endif

#ifndef __TIME_H__
#include <time.h>
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPStorageUnit;
class XMPFrame;
class XMPLinkSource;
class XMPLink;

//==============================================================================
// Data Interchange functions
//==============================================================================

//-------------------------------------------------------------------------------------
// Paste As Dialog
//-------------------------------------------------------------------------------------
// public

class XMPPasteAsResult {
public:
	XMPPasteAsResult()
		{
			fSelectedKind = (XMPType) kXMPNULL;
			fTranslateKind = (XMPType) kXMPNULL;
		};

	~XMPPasteAsResult()
		{
			XMPDisposePtr((XMPPtr) fSelectedKind);
			XMPDisposePtr((XMPPtr) fTranslateKind);
		};

	XMPBoolean		fPasteLinkSetting;
	XMPBoolean		fAutoUpdateSetting;
	XMPBoolean		fMergeSetting;
	XMPTypeToken	fSelectedView;
	XMPType			fSelectedKind;
	XMPType			fTranslateKind;
};

XMPBoolean ShowPasteAsDialog(
				XMPBoolean			canPasteLink,
				XMPBoolean			canEmbed,
				XMPStorageUnit*		contentSU,
				XMPFrame*			activeFrame,
				XMPTypeToken		viewType,
				XMPPasteAsResult*	result);

//-------------------------------------------------------------------------------------
// Link Info Dialogs
//-------------------------------------------------------------------------------------
// public

typedef enum {
	kXMPLinkInfoCancel	= 0,
	kXMPLinkInfoOk,
	kXMPLinkInfoBreakLink,
	kXMPLinkInfoUpdateNow,
	kXMPLinkInfoFindSource
} XMPLinkInfoAction;

struct XMPLinkInfoResult {
	XMPLinkInfoAction	action;
	XMPBoolean			autoUpdate;
};

XMPBoolean ShowLinkSourceInfo(
				XMPLinkSource*		linkSource,
				XMPChangeID			change,
				XMPLinkInfoResult*	infoResult);
				
struct XMPLinkInfo {
	XMPType				kind;
	XMPULong			size;
	time_t				creationTime;
	time_t				changeTime;
	XMPChangeID			change;
	XMPBoolean			autoUpdate;
};

XMPBoolean ShowLinkDestinationInfo(
				XMPLink*			link,
				XMPLinkInfo* 		info,
				XMPLinkInfoResult*	infoResult);

#endif // _LINKDLGS_

