/*
	File:		RefCtObj.h

	Contains:	Class definition for XMPRefCntObject

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<10>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <9>	 1/11/94	T		Init... changes
		 <8>	12/15/93	T		InitObject changes
		 <7>	  8/9/93	NP		Added class id string.
		 <6>	  8/6/93	VL		Added GetRefCount.
		 <5>	 7/21/93	NP		Added a virtual keyword (temporary for
									ASLM).
		 <4>	 6/16/93	VL		Added protected field fRefCount.
		 <3>	 4/29/93	VL		8.3 Name Change.
		 <1>	  4/8/93	VL		first checked in

	To Do:
*/

#ifndef _REFCTOBJ_
#define _REFCTOBJ_

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This file defines class XMPRefCntObject.  This class is the common base class
	for all XMP classes whose objects require ref counting.
*/

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPRefCntObject;

//==============================================================================
// XMPObject
//==============================================================================

#define kXMPRefCntObjectID "appl:refcntobject$class,1.0.0"

class XMPRefCntObject : public XMPObject
{
public:

	XMPRefCntObject();
	
	virtual ~ CDECL XMPRefCntObject();
	
	XMPNVMethod void	InitRefCntObject();
	
	//-------------------------------------------------------------------------
	// From ObjectStorage protocol
	//-------------------------------------------------------------------------

	XMPVMethod	void	IncrementRefCount();
	
		// Increments the ref count for the object.
	
	XMPVMethod	void	Release();
	
		// Decrements the ref count for the object.
		
	XMPVMethod	XMPULong	GetRefCount();
		
protected:

	XMPULong	fRefCount;
		
		
};

#endif // _REFCTOBJ_
