/*
	File:		StdProps.h

	Contains:	Standard Properties

	Written by:	OpenDoc Engineering

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<40>	  4/6/94	JBS		1155477
		<39>	 3/27/94	T		#1153523.  Adjust name of
									kXMPPropRootPartSU to more clearly indicate
									what it is supposed to be used for.
		<38>	 3/24/94	CC		Added kXMPPropChangeTime (1153068); changed
									kXMPPropAutoExport to kXMPPropAutoUpdate
									(1142938).
		<37>	  3/1/94	JA		Made string consts extern. (1147295)
		<36>	  2/2/94	RR		Added IsRoot property for frame
		<35>	  2/1/94	CC		Property name changed to
									kXMPPropShouldShowLinks.
		<34>	  2/1/94	SS		Added content property for printing
		<33>	  2/1/94	CC		Added kXMPPropOriginalID,
									kXMPPropOriginalDraft, kXMPPropCloneKind,
									and kXMPPropShowLinkBorders.
		<32>	 1/27/94	VL		Added kXMPPropStandInFrame.
		<31>	 1/26/94	RR		Subrame -> SubFrame
		<30>	 1/21/94	CC		Removed kXMPPropLinkState and
									kXMPPropTimeStamp.  Added kXMPPropLink,
									kXMPPropLinkSource, and kXMPPropChangeID.
		<29>	 1/20/94	SS		Added PartsBin prop
		<28>	 1/14/94	T		removed some Draft Properties, renamed
									others
		<27>	 1/14/94	CC		Added kXMPPropLinkContentSU,
									kXMPPropTimeStamp, kXMPPropAutoExport
		<26>	 1/13/94	JBS		added Frame: kXMPPropIsSubframe
		<25>	12/21/93	JBS		added kXMPPropPresentation
		<24>	12/16/93	CC		Added document properties for linking.
		<23>	12/12/93	T		added kXMPPropPreferredEditor
		<22>	12/11/93	T		added kXMPPropIsStationery
		<21>	 12/8/93	T		added kXMPPropIconFamily for parts and
									kXMPPropName and kXMPPropComments for
									persistent objects
		<20>	11/19/93	PH		Move kXMPPropCurrentSURef here
		<19>	11/18/93	T		added props for Creation, Mod Date and Mod
									User
		<18>	11/17/93	SS		Added Props for Parts
		<17>	11/16/93	EL		Change OpenDocProp to OpenDoc:Property.
		<16>	11/12/93	JBS		add Facet, remove Layout
		<15>	11/10/93	CC		Added kXMPPropLinkSpec and
									kXMPPropSourcePart
		<14>	 11/9/93	PH		Add extra colon
		<13>	10/27/93	eeh		added kXMPPropWindowHasMailer
		<12>	10/21/93	RR		Added window properties isResizable,
									isFloating and isRootWindow
		<11>	10/12/93	eeh		added kXMPPropWindowIsMailerAware
		<10>	 10/4/93	RR		Added window properties
		 <9>	 9/24/93	PH		Added kXMPPropContents
		 <8>	 9/23/93	VL		Added StorageUnitType.
		 <7>	  9/3/93	JBS		added Layout props, fixed some window props
		 <6>	 8/13/93	PH		Name all properties with OpenDocProp to
									avoid conflicts with values
		 <5>	 8/12/93	T		unify property constants
		 <4>	 8/11/93	T		add properties for persistent objects
		 <3>	 8/10/93	T		add properties for WindowState
		 <2>	  8/9/93	VL		Added new storage types.
		 <1>	  7/7/93	VL		first checked in

	To Do:
*/

#ifndef _STDPROPS_
#define _STDPROPS_

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This file is used to store all the standard properties.
*/

//==============================================================================
// Constants
//==============================================================================

// Storage
extern const	XMPPropertyName kXMPPropVersionList 		; // = "OpenDoc:Property:DraftList";
extern const	XMPPropertyName kXMPPropDraftProperties 	; // = "OpenDoc:Property:DraftProperties";
extern const	XMPPropertyName kXMPPropRootPartSU 			; // = "OpenDoc:Property:Draft:RootPartStorageUnit";

extern const 	XMPPropertyName	kXMPPropStorageUnitType		; // = "OpenDoc:Property:StorageUnit:Type";
extern const	XMPPropertyName kXMPPropStorageUnitName 	; // = "OpenDoc:Property:StorageUnitName";
extern const	XMPPropertyName	kXMPPropCurrentSURef 		; // = "OpenDoc:Property:StorageUnit:CurrentSURef";
extern const	XMPPropertyName kXMPPropDocumentName 		; // = "OpenDoc:Property:DocumentName";
extern const	XMPPropertyName kXMPPropDraftName 			; // = "OpenDoc:Property:DraftName";

extern const	XMPPropertyName	kXMPPropValueProperty 		; // = "OpenDoc:Property:Property";

// Persistent Object
extern const	XMPPropertyName kXMPPropName				; // = "OpenDoc:Property:Name";
extern const	XMPPropertyName kXMPPropComments			; // = "OpenDoc:Property:Comments";
extern const	XMPPropertyName	kXMPPropCreateDate 			; // = "OpenDoc:Property:CreationDate";
extern const	XMPPropertyName	kXMPPropModDate 			; // = "OpenDoc:Property:ModifiedDate";
extern const	XMPPropertyName	kXMPPropModUser 			; // = "OpenDoc:Property:ModifiedUser";

// WindowState
extern const	XMPPropertyName kXMPPropWindowList 			; // = "OpenDoc:Property:WindowList";

// Window
extern const	XMPPropertyName kXMPPropWindow 				; // = "OpenDoc:Property:Window";
extern const	XMPPropertyName kXMPPropWindowRect			; // = "OpenDoc:Property:WindowRect";
extern const	XMPPropertyName kXMPPropWindowTitle			; // = "OpenDoc:Property:WindowTitle";
extern const	XMPPropertyName kXMPPropWindowProcID		; // = "OpenDoc:Property:WindowProcID";
extern const	XMPPropertyName kXMPPropWindowIsVisible		; // = "OpenDoc:Property:WindowVisible";
extern const	XMPPropertyName kXMPPropWindowHasCloseBox	; // = "OpenDoc:Property:WindowHasCloseBox";
extern const	XMPPropertyName kXMPPropWindowHasZoomBox	; // = "OpenDoc:Property:WindowHasZoomBox";
extern const	XMPPropertyName kXMPPropWindowIsResizable	; // = "OpenDoc:Property:WindowIsResizable";
extern const	XMPPropertyName kXMPPropWindowIsRootWindow	; // = "OpenDoc:Property:WindowIsRootWindow";
extern const	XMPPropertyName kXMPPropWindowIsFloating	; // = "OpenDoc:Property:WindowIsFloating";
extern const	XMPPropertyName kXMPPropWindowHasMailer		; // = "OpenDoc:Property:WindowHasMailer";
extern const	XMPPropertyName kXMPPropWindowIsMailerAware	; // = "OpenDoc:Property:WindowIsMailerAware";
extern const	XMPPropertyName kXMPPropWindowRefCon		; // = "OpenDoc:Property:WindowRefCon";
extern const	XMPPropertyName kXMPPropRootFrame			; // = "OpenDoc:Property:RootFrame";
extern const	XMPPropertyName kXMPPropSourceFrame			; // = "OpenDoc:Property:SourceFrame";
extern const	XMPPropertyName kXMPPropShouldShowLinks		; // = "OpenDoc:Property:ShouldShowLinks";

// Frame
extern const	XMPPropertyName kXMPPropContainingFrame 	; // = "OpenDoc:Property:ContainingFrame";
extern const	XMPPropertyName kXMPPropGraphicsSystem 		; // = "OpenDoc:Property:GraphicsSystem";
extern const	XMPPropertyName kXMPPropFrameShape	 		; // = "OpenDoc:Property:FrameShape";
extern const	XMPPropertyName kXMPPropUsedShape			; // = "OpenDoc:Property:UsedShape";
extern const	XMPPropertyName	kXMPPropInternalTransform	; // = "OpenDoc:Property:InternalTransform";
extern const	XMPPropertyName kXMPPropPart				; // = "OpenDoc:Property:Part";
extern const	XMPPropertyName kXMPPropPartInfo			; // = "OpenDoc:Property:PartInfo";
extern const	XMPPropertyName kXMPPropViewType			; // = "OpenDoc:Property:ViewType";
extern const	XMPPropertyName kXMPPropPresentation		; // = "OpenDoc:Property:Presentation";
extern const	XMPPropertyName kXMPPropFrameGroup			; // = "OpenDoc:Property:FrameGroup";
extern const	XMPPropertyName kXMPPropSequenceNumber		; // = "OpenDoc:Property:SequenceNumber";
extern const	XMPPropertyName kXMPPropIsRoot				; // = "OpenDoc:Property:IsRoot";
extern const	XMPPropertyName kXMPPropIsSubframe			; // = "OpenDoc:Property:IsSubFrame";
extern const	XMPPropertyName kXMPPropIsOverlaid			; // = "OpenDoc:Property:IsOverlaid";
extern const	XMPPropertyName kXMPPropIsFrozen			; // = "OpenDoc:Property:IsFrozen";
extern const	XMPPropertyName kXMPPropDoesPropagateEvents ; // = "OpenDoc:Property:DoesPropagateEvents";

// Draft
extern const XMPPropertyName	kXMPPropDraftNumber 		; // = "OpenDoc:Property:Draft:Number";
extern const XMPPropertyName	kXMPPropDraftComment 		; // = "OpenDoc:Property:Draft:Comment";

// Link
extern const	XMPPropertyName kXMPPropLink				; // = "OpenDoc:Property:Link";
extern const	XMPPropertyName kXMPPropLinkSource			; // = "OpenDoc:Property:LinkSource";
extern const	XMPPropertyName	kXMPPropLinkSpec			; // = "OpenDoc:Property:LinkSpec";
extern const	XMPPropertyName kXMPPropSourcePart			; // = "OpenDoc:Property:SourcePart";
extern const	XMPPropertyName kXMPPropEditionAlias		; // = "OpenDoc:Property:EditionAlias";
extern const	XMPPropertyName kXMPPropLinkSection			; // = "OpenDoc:Property:LinkSection";
extern const	XMPPropertyName kXMPPropLinkContentSU		; // = "OpenDoc:Property:LinkContentSU";
extern const	XMPPropertyName kXMPPropAutoUpdate			; // = "OpenDoc:Property:AutoUpdate";
extern const	XMPPropertyName kXMPPropChangeID			; // = "OpenDoc:Property:ChangeID";
extern const	XMPPropertyName kXMPPropChangeTime			; // = "OpenDoc:Property:ChangeTime";
extern const	XMPPropertyName kXMPPropOriginalID			; // = "OpenDoc:Property:OriginalID";
extern const	XMPPropertyName kXMPPropOriginalDraft		; // = "OpenDoc:Property:OriginalDraft";
extern const	XMPPropertyName kXMPPropOriginalCloneKind	; // = "OpenDoc:Property:OriginalCloneKind";

// Drag&Drop + Clipboard
extern const XMPPropertyName	kXMPPropContents 			; // = "OpenDoc:Property:Contents";
extern const XMPPropertyName	kXMPPropExternalTransform	; // = "OpenDoc:Property:ExternalTransform";
extern const	XMPPropertyName kXMPPropStandInFrame 		; // = "OpenDoc:Property:StandInFrame";

// Document Properties
extern const	XMPPropertyName kXMPPropEditionID			; // = "OpenDoc:Property:Document:EditionID";
extern const	XMPPropertyName kXMPPropSectionID			; // = "OpenDoc:Property:Document:SectionID";

// Part
// annotations
extern const	XMPPropertyName kXMPPropIconFamily			; // = "OpenDoc:Property:IconFamily";
extern const	XMPPropertyName kXMPPropIsStationery	 	; // = "OpenDoc:Property:IsStationery";
extern const	XMPPropertyName	kXMPPropPreferredEditor 	; // = "OpenDoc:Property:PreferredEditor";
extern const	XMPPropertyName	kXMPPropPartsBin		 	; // = "OpenDoc:Property:PartsBin";

// content
extern const	XMPPropertyName kXMPPropDisplayFrames		; // = "OpenDoc:Property:DisplayFrames";
extern const	XMPPropertyName kXMPPropEmbeddedFrames 		; // = "OpenDoc:Property:EmbeddedFrames";
extern const	XMPPropertyName kXMPPropPrintingInfo 		; // = "OpenDoc:Property:PrintingInfo";

#endif	// _STDPROPS_