/*
	File:		TranslaM.h

	Contains:	Class definition for XMPMacTranslation.

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <9>	  4/4/94	CC		GetTranslationOf() returns XMPTypeList*
									(1153046).
		 <8>	 3/25/94	CC		VL: New translation interface (Radar # not
									available).
		 <7>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <6>	  2/4/94	VL		Moved to PPC Header and began code cleanup.
		 <5>	 12/9/93	VL		Added toData and toDataSize parameters to
									Translate. Added toView parameter to the
									overloaded Translate.
		 <4>	11/23/93	VL		Added flag for MEO existence.
		 <3>	 9/16/93	CC		Changed XMPOSType to XMPType. Added
									GetISOTypeFromPlatformType and
									GetPlatformTypeFromISOType.
		 <2>	  8/9/93	NP		Added class id string.
		 <1>	  7/1/93	VL		first checked in

	To Do:
*/

#ifndef _TRANSLAM_
#define _TRANSLAM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _TRANSLAT_
#include "Translat.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================
class XMPMacTranslation;

//=====================================================================================
// Classes used by this interface
//=====================================================================================
class XMPTypeList;
class XMPStorageUnitView;

//=====================================================================================
// Class XMPMacTranslation
//=====================================================================================

#define kXMPMacTranslationID "appl:xmpmactranslation$class,1.0.0"

class XMPMacTranslation : public XMPAbsTranslation
{
public:
	
	XMPMacTranslation();
		
	XMPVMethod ~ CDECL XMPMacTranslation();
	
	XMPMethod void InitTranslation();

	XMPMethod XMPSize Purge(XMPSize size);
	
	XMPMethod XMPTranslateResult CanTranslate(XMPType fromType);
	
	XMPMethod XMPTypeList* GetTranslationOf(XMPType fromType);
											
	XMPMethod XMPTranslateResult Translate(XMPStorageUnitView* fromView,
							XMPStorageUnitView* toView);
		
	XMPVMethod XMPTranslateResult Translate(XMPPtr fromData, 
							XMPULong fromDataSize,
							XMPType fromType,
							XMPType	toType,
							XMPPtr* toData,
							XMPULong* toDataSize);
		
	XMPMethod XMPType GetISOTypeFromPlatformType(XMPPlatformType platformType,
												 XMPPlatformTypeSpace typeSpace);

	XMPMethod XMPPlatformType GetPlatformTypeFromISOType(XMPType type);
		
private:

	XMPBoolean fMEOExists;
};

#endif // _TRANSLAM_