/*
	File:		Trnsform.h

	Contains:	Definition of class XMPAbsTransform

	Written by:	Joshua Susser & Sepp Friedrich & Jens Alfke

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<14>	 3/25/94	JA		Changed XMPTransform API (1153438).
		<13>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<12>	 2/16/94	JA		Stop including Point.h.
		<11>	  2/3/94	JA		Tiger Team Makeover!
		<10>	10/14/93	JA		Rewrote theory (get/setter copying now
									optional.)
		 <9>	 10/6/93	JA		Include new "Point.h"
		 <8>	 9/23/93	JA		Restored MoveBy.
		 <7>	 9/16/93	JA		Added Read/WritePlatformTransform. Removed
									ExtractPlatformTransform and constructor
									with params.
		 <6>	 9/08/93	jpa		Removed ExtractPlatformTransform, added more comments.
		 <5>	  7/6/93	JBS		separate into abstract/concrete
		 <4>	 4/30/93	JBS		8.3 file names
		 <3>	 4/19/93	JBS		further clean-ups
		 <2>	 4/14/93	JBS		clean up; add ExtractPlatformTransform()
		 <1>	 4/14/93	JBS		first checked in
*/

#ifndef _TRNSFORM_
#define _TRNSFORM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"			// base class
#endif


//==============================================================================
// Constants & types defined in this interface
//==============================================================================

enum {
	kXMPIdentityXform,
	kXMPTranslateXform,
	kXMPScaleXform,
	kXMPScaleTranslateXform,
	kXMPLinearXform,
	kXMPLinearTranslateXform,
	kXMPPerspectiveXform
};
typedef XMPUShort XMPTransformType;


//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPAbsTransform;

//==============================================================================
// Classes used in this interface
//==============================================================================

class XMPTransform;
class XMPStorageUnit;
class XMPShape;

//==============================================================================
// XMPAbsTransform
//==============================================================================

class XMPAbsTransform : public XMPObject
{
	
public:

	//---------------------------------
	// overrides

//	XMPVMethod XMPSize Purge(XMPSize size);		// override from XMPObject

	//---------------------------------
	// constructor/destructor
	
	XMPAbsTransform() {}

	XMPVMethod ~ CDECL XMPAbsTransform() {}
	
	//---------------------------------
	// getters:

	XMPMethod	XMPTransformType  GetType( )  =0;
	XMPMethod	void		GetOffset( XMPPoint *offset )  =0;
	XMPMethod	void		GetPreScaleOffset( XMPPoint *offset )  =0;
	XMPMethod	void		GetScale( XMPPoint *scale )  =0;
	XMPMethod	void		GetMatrix( XMPMatrix *matrix )  =0;
	XMPMethod	XMPBoolean	HasMatrix( )  =0;

	XMPMethod	XMPBoolean	IsSameAs( XMPTransform* compareTransform )  =0;

	//---------------------------------
	// setters:

	XMPMethod	XMPTransform*	Reset( )  =0;
	XMPMethod	XMPTransform*	SetMatrix( XMPMatrix *matrix )  =0;
	XMPMethod	XMPTransform*	Copy( )  =0;
	XMPMethod	XMPTransform*	CopyFrom( XMPTransform* sourceTransform )  =0;
	
	//---------------------------------
	// geometric operations:
	
	XMPMethod	XMPTransform*	SetOffset( XMPPoint )  =0;
	XMPMethod	XMPTransform*	MoveBy( XMPPoint )  =0;
	XMPMethod	XMPTransform*	ScaleBy( XMPPoint scale )  =0;
	XMPMethod	XMPTransform*	ScaleDownBy( XMPPoint scale )  =0;

	XMPMethod	XMPTransform*	Invert( )  =0;
	XMPMethod	XMPTransform*	PreCompose( XMPTransform* transform )  =0;
	XMPMethod	XMPTransform*	PostCompose( XMPTransform* transform )  =0;

	//---------------------------------
	// geometry operations with points & shapes:

	XMPMethod	XMPPoint		TransformPoint( XMPPoint )  =0;
	XMPMethod	XMPPoint		InvertPoint( XMPPoint )  =0;
	
	XMPMethod	void			TransformShape( XMPShape* )  =0;
	XMPMethod	void			InvertShape( XMPShape* )  =0;
	
	//---------------------------------
	// input / output:
	
	XMPMethod	void		WriteTo( XMPStorageUnit* )  =0;
	XMPMethod	void		ReadFrom( XMPStorageUnit* )  =0;
};

#if PLATFORM_MACINTOSH
#ifndef _TRNSFRMM_
#include "TrnsfrmM.h"
#endif
#endif

#endif // _TRNSFORM_