/*
	File:		TrnsfrmM.h

	Contains:	Definition of class XMPMacTransform

	Written by:	Jens Alfke & Joshua Susser & Sepp Friedrich

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):
			
		<10>	 3/25/94	JA		Changed XMPTransform API to support zooming
									(1153438).
		 <9>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <8>	  2/3/94	JA		Tiger Team Makeover!
		 <7>	 1/31/94	JA		More tweaks to comments.
		 <6>	10/14/93	JA		Rewrote theory and comments.
		 <5>	 9/23/93	JA		Restored MoveBy. Added Mac-specific
									convenience methods.
		 <4>	 9/16/93	JA		Added Read/WritePlatformTransform. Removed
									constructor w/params: it could fail.
		 <3>	 9/08/93	jpa		Added private ::InitTransform and ::Compose.
		 							Deleted Mac-specific methods (scale, skew, etc.)
		 <2>	  8/9/93	NP		Added class id string.
		 <1>	  7/6/93	JBS		first checked in
		<12>	  5/7/93	JBS		namesmith
		<11>	  5/3/93	JBS		back up one version
		 <9>	 4/28/93	JBS		8.3 file names
		 <8>	 3/23/93	SF		subclass of XMPObject
		 <7>	 3/10/93	SF		Added InversePoint(..) which handles internally
									creation/destruction of Inverse transform.
		 <6>	 1/29/93	SF		editorial
		 <5>	 1/29/93	SF		debugging; removed Transform(aRect)
		 <4>	 1/28/93	SF		some return types changed
		 <3>	 1/27/93	SF		change XMPType to XMPUShort
		 <2>	 1/26/93	SF		add TransformType, identity check
		 <1>	 1/16/93	SF		first checked in
*/

#ifndef _TRNSFRMM_
#define _TRNSFRMM_

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif


//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPMacTransform;

//==============================================================================
// XMPMacTransform
//==============================================================================

#define kXMPMacTransformID "appl:xmpmactransform$class,1.0.0"

class XMPMacTransform : public XMPAbsTransform
{
	
public:

	//---------------------------------
	// overrides

	XMPVMethod XMPSize Purge(XMPSize size);		// override from XMPObject

	//---------------------------------
	// constructor/destructor
	
	XMPMacTransform( );

	XMPVMethod ~ CDECL XMPMacTransform();
	
	//---------------------------------
	// getters:

	XMPMethod	XMPTransformType GetType( );
	XMPMethod	void		GetOffset( XMPPoint *offset );
	XMPMethod	void		GetPreScaleOffset( XMPPoint *offset );
	XMPMethod	void		GetScale( XMPPoint *scale );
	XMPMethod	void		GetMatrix( XMPMatrix *matrix );
	XMPMethod	XMPBoolean	HasMatrix( );

	XMPMethod	XMPBoolean	IsSameAs( XMPTransform* compareTransform );

	//---------------------------------
	// setters:

	XMPMethod	XMPTransform*	Reset( );
	XMPMethod	XMPTransform*	SetMatrix( XMPMatrix *matrix );
	XMPMethod	XMPTransform*	Copy( );
	XMPMethod	XMPTransform*	CopyFrom( XMPTransform* sourceTransform );
	
	//---------------------------------
	// geometric operations:
	
	XMPMethod	XMPTransform*	SetOffset( XMPPoint );
	XMPMethod	XMPTransform*	MoveBy( XMPPoint );
	XMPMethod	XMPTransform*	ScaleBy( XMPPoint scale );
	XMPMethod	XMPTransform*	ScaleDownBy( XMPPoint scale );

	XMPMethod	XMPTransform*	Invert( );
	XMPMethod	XMPTransform*	PreCompose( XMPTransform* transform );
	XMPMethod	XMPTransform*	PostCompose( XMPTransform* transform );

	//---------------------------------
	// geometry operations with points and shapes:

	XMPMethod	XMPPoint		TransformPoint( XMPPoint );
	XMPMethod	XMPPoint		InvertPoint( XMPPoint );
	
	XMPMethod	void			TransformShape( XMPShape* );
	XMPMethod	void			InvertShape( XMPShape* );
	
	//---------------------------------
	// input / output:
	
	XMPMethod	void			WriteTo( XMPStorageUnit* );
	XMPMethod	void			ReadFrom( XMPStorageUnit* );

	//---------------------------------
	// Mac-specific API extensions:
	
	XMPMethod	XMPBoolean		IsQDOffset( );			// Pure integer offset?
	XMPMethod	Point			GetQDOffset( );
	XMPMethod	XMPTransform*	SetQDOffset( Point );

private:

						XMPTransform( XMPMatrix *m );	// QuickTime matrix
	XMPNVMethod	void	ClearInverse( );
	XMPMethod	void	GetRawMatrix( void *m );

	XMPMatrix			fMatrix;			// Not exactly a real XMPMatrix!!
	XMPMatrix*			fInverseMatrix;		// Cached inverse matrix (or NULL)
};

#endif // _TRNSFRMM_
