/*
	File:		XMPWindowState.h

	Contains:	Interface to XMPAbsWindowState class.

	Written by:	Richard Rodseth

	Copyright:	 1992-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <5>	  2/9/94	NP		Tiger Team cleanup.
		 <4>	  2/7/94	NP		Tiger Team doings.
		 <3>	  2/2/94	RR		Added SetDefaultWindowTitles
		 <2>	 1/21/94	RR		Added shouldSave to CreateWindow
		 <1>	 1/15/94	RR		first checked in
		<16>	 1/10/94	RR		Added CreateWindowIterator
		<15>	12/21/93	JBS		CreateWindow with viewType & presentation
		<14>	12/20/93	VL		Added CreateWindow and GetWindow.
		<13>	12/20/93	RR		New Init Strategy
		<12>	 12/8/93	RR		Cleanup XMPMethod and forward declarations.
									Move iterator to separate file
		<11>	11/16/93	RR		Added AdjustPartMenus. Remove
									SetMenuUpdateProc and UpdateBaseMenuBar
		<10>	10/21/93	RR		Added GetRootWindowCount and
									GetTotalRootWindowCount. GetWindowCount now
									returns an unsigned short.
		 <9>	 10/8/93	RR		Added IsXMPWindow
		 <8>	 10/7/93	RR		Removed CreateWindow (use Draft call)
		 <7>	 9/22/93	RR		Added OpenWindows and CloseWindows
		 <6>	 9/22/93	RR		GetBaseMenuBar -> CopyBaseMenuBar
		 <5>	 9/10/93	RR		Get/SetBaseMenuBar now take an XMPMenuBar*
		<3>	 	 6/30/93	RCR		Separate Abstract and Concrete classes.
		<3>	 	 4/29/93	RCR		Rename Includes.
		 <2>	 4/23/93	RCR		Added Initialize and Purge
		 <1>	 4/9/93		RCR		first checked in

	To Do:
*/

#ifndef _WINSTAT_
#define _WINSTAT_

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPWindowState;				// The list of XMPWindows

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPSession;
class XMPDraft;
class XMPWindow;
class XMPMenuBar;
class XMPWindowIterator;				
class XMPFrame;

//=====================================================================================
// XMPWindowState
//=====================================================================================

class XMPAbsWindowState : public XMPObject
{
  public:

	XMPAbsWindowState() {} 

	XMPVMethod ~ CDECL XMPAbsWindowState() {}		
			
	XMPVMethod	XMPSize	Purge(XMPSize size)
		= 0;
	
	XMPVMethod XMPWindow* 	CreateWindow(XMPPlatformWindow newWindow, 
									 XMPBoolean isRootWindow,
									 XMPBoolean isResizable,
									 XMPBoolean isFloating,
									 XMPBoolean shouldSave,
									 XMPPart* rootPart,
									 XMPTypeToken viewType,
									 XMPTypeToken presentation,
									 XMPFrame* sourceFrame)
		= 0;

	XMPVMethod XMPWindow*	GetWindow(XMPID id)
		= 0;
		
	XMPVMethod void Internalize(XMPDraft* draft)
		= 0;
		
	XMPVMethod void Externalize(XMPDraft* draft)
		= 0;
		
	XMPMethod void SetDefaultWindowTitles(XMPDraft* draft)
		= 0;

	XMPVMethod void OpenWindows(XMPDraft* draft)
		= 0;
		
	XMPVMethod void CloseWindows(XMPDraft* draft)
		= 0;
		
	XMPVMethod XMPUShort GetWindowCount()
		= 0;
	
	XMPVMethod XMPUShort GetRootWindowCount(XMPDraft* draft)
		= 0;
	
	XMPVMethod XMPUShort GetTotalRootWindowCount()
		= 0;

	XMPVMethod XMPBoolean IsXMPWindow(XMPPlatformWindow aWindow)
		= 0;
		
	XMPVMethod XMPWindow* GetXMPWindow(XMPPlatformWindow aWindow)
		= 0;
	
	XMPVMethod XMPWindowIterator* CreateWindowIterator()
		= 0;
		
	XMPVMethod XMPWindow* GetActiveWindow()
		= 0;
	
	XMPVMethod void SetBaseMenuBar(XMPMenuBar* theMenuBar)
		= 0;
	
	XMPVMethod XMPMenuBar* CopyBaseMenuBar()
		= 0;

	XMPVMethod void AdjustPartMenus()
	 	= 0;
};

#ifdef PLATFORM_MACINTOSH
#ifndef _WINSTATM_
#include "WinStatM.h"
#endif
#endif

#endif // _WINSTAT_