/*
	File:		WinStatM.h

	Contains:	Interface to XMPMacWindowState class.

	Written by:	Richard Rodseth

	Copyright:	 1992-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<39>	 3/16/94	RR		#1151142 Renamed private method
									GetFrontPlatformWindow ->
									GetFrontNonFloatingWindow
		<38>	 3/16/94	eeh		bug #1149649: added MakeWindowIntoReply;
									fMailerReplyInfo and 2nd param to
									SetTempMailerWindow
		<37>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<36>	 2/14/94	RR		#1143609. Made GetFrontWindow etc. public.
		<35>	  2/9/94	NP		Tiger Team cleanup.
		<34>	  2/2/94	RR		Made SetDefaultWindowTitles public.
		<33>	 1/21/94	RR		Clarify method visibility. Add shouldSave
									to CreateWindow
		<32>	 1/20/94	RR		Added private methods for AOCE Mailer
		<31>	 1/19/94	RR		Cleanup
		<30>	 1/18/94	RR		Remove const from Drag. Reorder methods.
									Remove GetWindowVisible. Change _WinStateM_
									to _WinStatM_
		<29>	 1/14/94	RR		Added utility FindXMPWindow (Private to UI
									subsystem)
		<28>	 1/11/94	T		Init... changes
		<27>	 1/10/94	RR		Added CreateWindowIterator. Added private
									method SetWindowTitles, called during
									'ternalization
		<26>	12/22/93	RR		Added fNextID
		<25>	12/22/93	RR		XMPMethod Usage
		<24>	12/21/93	JBS		CreateWindow with viewType & presentation
		<23>	12/20/93	VL		Added CreateWindow and GetWindow. Also
									added ReleaseWindow.
		<22>	12/20/93	RR		Added InitWindowState, removed Initialize
		<21>	 12/8/93	RR		Cleanup XMPMethod and forward declarations.
									Move some methods to public section for VL
		<20>	 12/8/93	VL		Added DeactivateFrontWindows,
									ActivateFrontWindows and a bunch of
									GetWindow utility calls.
		<19>	 12/6/93	RR		Added SuspendResume
		<18>	 12/1/93	VL		Added support for floating windows (Drag,
									Select, Show, Hide
		<17>	11/16/93	RR		Added AdjustPartMenus. Remove
									SetMenuUpdateProc and UpdateBaseMenuBar
		<16>	11/15/93	eeh		added GetTempMailerWindow
		<15>	 11/8/93	NP		Comment text following an #endif.
		<14>	 11/1/93	eeh		added ClearMailerWindow()
		<13>	10/29/93	T		changed FixUpMailerWindow to use a Str255
									not a Str255* to pacify CFront
		<12>	10/27/93	eeh		added FixUpMailerWindow,
									SetTempMailerWindow, Set- &
									GetMailerWindow, fTempMailerWindow and
									fMailerWindow
		<11>	10/21/93	RR		Added GetRootWindowCount and
									GetTotalRootWindowCount. GetWindowCount now
									returns an unsigned short
		<10>	 10/8/93	RR		Added IsXMPWindow
		 <9>	 10/7/93	RR		Removed CreateWindow (use Draft call)
		 <8>	 9/22/93	RR		Added OpenWindows and CloseWindows
		 <7>	 9/22/93	RR		GetBaseMenuBar -> CopyBaseMenuBar
		 <6>	 9/10/93	RR		Added fCurrentMenuBar + Set/Get
		 <5>	  8/9/93	NP		Added class id string.
		 <4>	 7/21/93	NP		Added a virtual keyword (temporary for
									ASLM).
		 <5>	 7/13/93	RCR		Change implementation to linked list.
		 <4>	 6/30/93	RCR		Change iterator base class to XMPAbs.
		 <3>	 6/30/93	RCR		Separate Abstract and Concrete classes.
		 <1>	 4/9/93		RCR		first checked in

	To Do:
*/

#ifndef _WINSTATM_
#define _WINSTATM_

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPWindowState;				

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPSession;
class XMPDraft;
class XMPWindow;
class XMPFrame;
class XMPPart;
class LinkedList;
class XMPWindowIterator;				

//=====================================================================================
// XMPWindowState
//=====================================================================================

typedef enum MailerTempInfo { kNotForeign, kReplyToOne, kReplyToAll } ;

#define kXMPMacWindowStateID "appl:xmpmacwindowstate$class,1.0.0"

class XMPMacWindowState : public XMPAbsWindowState
{
  public:

	XMPMacWindowState();
		
	XMPVMethod ~ CDECL XMPMacWindowState();
	
	XMPNVMethod void InitWindowState(XMPSession* session); 

	XMPMethod	XMPSize	Purge(XMPSize size); 
	
	XMPMethod XMPWindow* 	CreateWindow(XMPPlatformWindow newWindow, 
									 XMPBoolean isRootWindow,
									 XMPBoolean isResizable,
									 XMPBoolean isFloating,
									 XMPBoolean shouldSave,
									 XMPPart* rootPart,
									 XMPTypeToken viewType,
									 XMPTypeToken presentation,
									 XMPFrame* sourceFrame);

	XMPMethod XMPWindow*	GetWindow(XMPID id);
	
	XMPMethod void Internalize(XMPDraft* draft);
		
	XMPMethod void Externalize(XMPDraft* draft);
		
	XMPMethod void SetDefaultWindowTitles(XMPDraft* draft);
	
	XMPMethod void OpenWindows(XMPDraft* draft);
		
	XMPMethod void CloseWindows(XMPDraft* draft);
		
	XMPMethod XMPUShort GetWindowCount();
	
	XMPMethod XMPUShort GetRootWindowCount(XMPDraft* draft);
	
	XMPMethod XMPUShort GetTotalRootWindowCount();

	XMPBoolean IsXMPWindow(XMPPlatformWindow aWindow);

	XMPMethod XMPWindow* GetXMPWindow(XMPPlatformWindow aWindow);	
	
	XMPMethod XMPWindowIterator* CreateWindowIterator();
		
	XMPMethod XMPWindow* GetActiveWindow();
	
	XMPMethod void SetBaseMenuBar(XMPMenuBar* theMenuBar);
	
	XMPMethod XMPMenuBar* CopyBaseMenuBar();

	XMPMethod void AdjustPartMenus();

  public: // Macintosh-specific

	XMPMethod	void		DeactivateFrontWindows();
	XMPMethod	void		ActivateFrontWindows();
	XMPMethod	XMPWindow*	GetFrontWindow();
	XMPMethod	XMPWindow* 	GetFrontFloatingWindow();
	XMPMethod	XMPWindow*	GetFrontRootWindow();
		
  public: // Private to UI Subsystem

	XMPMethod void SetCurrentMenuBar(XMPMenuBar* theMenuBar);
	XMPMethod XMPMenuBar* GetCurrentMenuBar();
	XMPMethod XMPWindow* AddWindow(XMPWindow* window);
	XMPMethod void RemoveWindow(XMPWindow* oldWindow);
		// Removes and disposes of the specified window
	XMPMethod XMPBoolean HandleAOCEMailerEvent(XMPEventData event);
	XMPMethod XMPBoolean HandleAOCEMailerCommand(XMPCommandID command);
	XMPMethod void UpdateAOCEMailerMenus();
	XMPMethod XMPWindow* FindXMPWindow(Point globalPoint);
	XMPMethod void ReleaseWindow(XMPWindow* window);
	XMPMethod void SuspendResume(XMPEventData event);
	XMPMethod void SelectXMPWindow(XMPWindow* window);
	XMPMethod void DragXMPWindow(XMPWindow* window, Point startPoint, Rect* draggingBounds);
	XMPMethod void ShowXMPWindow(XMPWindow* window);
	XMPMethod void HideXMPWindow(XMPWindow* window);
	XMPMethod LinkedList*	GetWindowList(); // for XMPWindowIterator
	
	// AOCE Mailer
	
	XMPMethod void SetTempMailerWindow( XMPPlatformWindow platWin,
			MailerTempInfo replyInfo ) ;
	XMPMethod void MakeWindowIntoReply( XMPMacWindow* newWindow ) ;
	XMPMethod XMPPlatformWindow GetTempMailerWindow() ;	
	XMPMethod void SetMailerWindow( XMPWindow* window ) ;	
	XMPMethod void ClearMailerWindow() ;	
	XMPMethod XMPWindow* GetMailerWindow() ;
	XMPMethod XMPPlatformWindow FixUpMailerWindow( Rect* bounds,
							Str255 title,
							XMPSShort procID,
							XMPBoolean hasCloseBox,
							XMPSLong refCon) ;

#ifdef TO_BE_DELETED
	XMPMethod XMPPlatformWindow GetMailerWindow() ;	
#endif // TO_BE_DELETED

  protected:
	
	void ActivateWindow(WindowPtr platformWindow);
	void DeactivateWindow(WindowPtr platformWindow);
	void HighlightAndActivateWindow(WindowPtr platformWindow, XMPBoolean activate);
	WindowPtr	GetLastFloatingPlatformWindow();
	WindowPtr	GetFrontNonFloatingPlatformWindow();

  protected:

	XMPSession* fSession;
	LinkedList* fWindowList;
	XMPID		fNextID;
	
	XMPMenuBar*	fBaseMenuBar;
	XMPMenuBar* fCurrentMenuBar;
	
	XMPPlatformWindow	fTempMailerWindow ;
	MailerTempInfo		fMailerReplyInfo ;
	XMPWindow*			fMailerWindow ;	// the one window that has the mailer

};

#endif // _WINSTATM_