/*
	File:		XMPObj.h

	Contains:	Class definition for XMPObject

	Written by:	Joshua Susser

	Protocols:	Extensions

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<16>	 3/25/94	MB		Symantec ASLM fixes. #1150864
		<15>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<14>	  3/1/94	JA		Removed bogus "XMPRootObject" superclass.
		<13>	 2/22/94	CC		VL: Moved implementation of GetFirstObject
									to .cpp file.
		<12>	 2/22/94	VL		JA: Added support for object list.
		<11>	 1/11/94	T		Init... changes
		<10>	12/15/93	T		InitObject changes
		 <9>	  8/9/93	NP		Added class id string.
		 <8>	 7/21/93	NP		Added a virtual keyword (temporary for
									ASLM).
		 <7>	 6/10/93	NP		Changed XMPName to XMPType.
		 <6>	  5/4/93	NP		Unvirtualed Initialize.
		 <5>	 4/29/93	VL		8.3 Name Change.
		 <3>	 4/12/93	VL		Made functions public.
		 <2>	  4/8/93	VL		Added Purge and Initialize.
		 <1>	  4/6/93	VL		first checked in

*/

#ifndef _XMPOBJ_
#define _XMPOBJ_

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This file defines class XMPObject.  This class is the common base class
	for most XMP classes.  It provides for extensibility through the
	Extensions protocol.
*/

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPObject;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPExtension;

//==============================================================================
// XMPObject
//==============================================================================

#define kXMPObjectID "appl:object$class,1.0.0"

class XMPObject
{
	ASLM_COMPATIBLE

public:

	XMPObject();
	
	virtual ~ CDECL XMPObject();

	XMPNVMethod	void	InitObject();
	
	//-------------------------------------------------------------------------
	// From Extensions protocol
	//-------------------------------------------------------------------------

	XMPVMethod XMPBoolean	HasExtension(XMPType extensionName);

		// answer whether this object supports the specified extension interface
	
	XMPVMethod XMPExtension*	GetExtension(XMPType extensionName);
	
		// return the specified extension interface

	XMPVMethod void			ReleaseExtension(XMPExtension* extension);

		// release resources associated with the specified extension interface.
		// may count references if necessary.

	//-------------------------------------------------------------------------
	// From Memory protocol
	//-------------------------------------------------------------------------
	
	XMPVMethod	XMPSize	Purge(XMPSize size)
		= 0;
	
		// Get rid of any unnecessary memory.
		
protected:
	XMPBoolean fInitialized;

#if XMPDebug

private:

	static	XMPObject*	fFirst;
	XMPObject*			fNext;
	XMPObject*			fPrev;

public:	// private by convention

	XMPVMethod	XMPBoolean ValidateObject();
	XMPMethod	XMPObject*	GetNextObject();
	XMPMethod	XMPObject*	GetPrevObject();
	static		XMPObject*	GetFirstObject();
	
#endif

};


#endif // _XMPOBJ_
