/*
	File:		FWBaseFr.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 4/11/94	DS		Change call to ChangeExternalTransform to
									ChangeGeometry.

	To Do:
*/

//	File:		FWBaseFr.cpp
//	Contains:	Implementation for class CBaseFrame
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWBASEFR_
#include "FWBaseFr.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes
#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _PART_
#include <Part.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

// ----- Macintosh Includes
#ifdef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#pragma segment FWFrameSeg

//==============================================================================
//  Class CBaseFrame
//==============================================================================

//------------------------------------------------------------------------------
//  CBaseFrame::CBaseFrame
//------------------------------------------------------------------------------

CBaseFrame::CBaseFrame()
{
	fXMPFrame = kXMPNULL;	
}

//------------------------------------------------------------------------------
//  CBaseFrame::IBaseFrame
//------------------------------------------------------------------------------

void CBaseFrame::IBaseFrame(XMPFrame* xmpFrame)
{
	fXMPFrame = xmpFrame;
	fXMPFrame->IncrementRefCount();
}

//------------------------------------------------------------------------------
//  CBaseFrame::~CBaseFrame
//------------------------------------------------------------------------------

CBaseFrame::~CBaseFrame()
{
	if (fXMPFrame)
		fXMPFrame->Release();
}

//------------------------------------------------------------------------------
//  CBaseFrame::GetFrameShape
//------------------------------------------------------------------------------

void CBaseFrame::GetFrameShape(XMPShape* shape) const
{
	shape->CopyFrom(this->GetXMPFrame()->GetFrameShape());
}

//------------------------------------------------------------------------------
//  CBaseFrame::GetFrameShapeBounds
//------------------------------------------------------------------------------

void CBaseFrame::GetFrameShapeBounds(XMPRect* rect) const
{
	XMPShape* shape = this->GetXMPFrame()->GetFrameShape();
	shape->GetBoundingBox(rect);
}

//------------------------------------------------------------------------------
//  CBaseFrame::OffsetFrame
//------------------------------------------------------------------------------

void CBaseFrame::OffsetFrame(const XMPPoint& offset)
{	
	if (offset.x != 0 || offset.y!=0)
	{
		XMPFrameFacetIterator i(fXMPFrame);
		
		for (XMPFacet* xmpFacet = i.First(); i.IsNotComplete(); xmpFacet = i.Next())
		{
			XMPTransform *transform = ::NewXMPTransform(xmpFacet->GetExternalTransform());
			
			transform->MoveBy(offset);
			
			xmpFacet->ChangeGeometry(kXMPNULL, transform);
		}

	}
}

//------------------------------------------------------------------------------
//  CBaseFrame::PlaceFrame
//------------------------------------------------------------------------------

void CBaseFrame::PlaceFrame(const XMPPoint& position)
{	
	XMPFrameFacetIterator i(fXMPFrame);
	
	for (XMPFacet* xmpFacet = i.First(); i.IsNotComplete(); xmpFacet = i.Next())
	{
		XMPTransform *transform = ::NewXMPTransform(xmpFacet->GetExternalTransform());
		
		transform->Reset();
		transform->MoveBy(position);
		
		xmpFacet->ChangeGeometry(kXMPNULL, transform);
	}
}

//------------------------------------------------------------------------------
//  CBaseFrame::ResizeFrame
//------------------------------------------------------------------------------

void CBaseFrame::ResizeFrame(XMPShape* newShape)
{	
	XMPRect frameBounds;
	GetFrameShapeBounds(&frameBounds);
	
	XMPRect shapeBox;
	newShape->GetBoundingBox(&shapeBox);
	
	XMPPoint offset(-shapeBox.left + frameBounds.left, -shapeBox.top + frameBounds.top);
	XMPTransform *transform = ::NewXMPTransform(offset);
	newShape->Transform(transform);
	delete transform;
	
	offset.x *= -1;
	offset.y *= -1;	
	PlaceFrame(offset);
	
//	return fXMPFrame->RequestFrameShape(newShape);	// newShape is kept by the frame
	fXMPFrame->ChangeFrameShape(newShape);
}

//------------------------------------------------------------------------------
//  CBaseFrame::DrawAllFacets
//------------------------------------------------------------------------------

void CBaseFrame::DrawAllFacets(XMPShape* invalidShape)
{
	XMPFrameFacetIterator i(fXMPFrame);
	for (XMPFacet* xmpFacet = i.First(); i.IsNotComplete(); xmpFacet = i.Next())
	{
		xmpFacet->Draw(invalidShape);
	}
}

//------------------------------------------------------------------------------
//  CBaseFrame::Invalidate
//------------------------------------------------------------------------------

void CBaseFrame::Invalidate(XMPShape* invalidShape)
{
	GetXMPFrame()->Invalidate(invalidShape);
}

//------------------------------------------------------------------------------
//  CBaseFrame::Validate
//------------------------------------------------------------------------------

void CBaseFrame::Validate(XMPShape* validShape)
{
	GetXMPFrame()->Validate(validShape);
}


