/*
	File:		FWBaseFr.h

	Contains:	Definition for class  
				 CBaseFrame

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<1+>	 3/17/94	DS		Add CDECL to Base frame dtor

	To Do:
*/



#ifndef _FWBASEFR_
#define _FWBASEFR_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _FRAME_
#include <Frame.h>
#endif




//==============================================================================
//  Forward Declaration
//==============================================================================

class XMPShape;
class XMPWindow;
class XMPStorageUnit;
struct XMPPoint;
struct XMPRect;




//==============================================================================
//  CBaseFrame Definition
//==============================================================================

class CBaseFrame
{
//------------------------------------------------------------------------------
//	 CBaseFrame Initialization/Destruction
//
public:
	CBaseFrame();
	void IBaseFrame(XMPFrame* frame);
	virtual ~ CDECL CBaseFrame();
	
//------------------------------------------------------------------------------
//	 CBaseFrame New API
//
public:
	// ----- Geometry -----
	void				GetFrameShape(XMPShape* shape) const;
	void				GetFrameShapeBounds(XMPRect* rect) const;
	
	virtual void		PlaceFrame(const XMPPoint& position);
	virtual void		OffsetFrame(const XMPPoint& offset);
	virtual void		ResizeFrame(XMPShape* newShape);
	
	// ----- Drawing -----
	void				DrawAllFacets(XMPShape *invalidShape);
	
	virtual void 		Invalidate(XMPShape *invalidShape);
	virtual void 		Validate(XMPShape *validShape);
	
	// ----- Getters/Setters -----
	XMPFrame*			GetXMPFrame() const;
	XMPPart*			GetXMPPart() const;
	
	// ----- From XMPFrame (Inlines) -----
	XMPULong			GetFrameGroup();
	void				SetFrameGroup(XMPULong groupID);
						
	XMPBoolean			IsRoot();

	XMPBoolean			IsOverlaid();

	XMPBoolean			IsFrozen();
	void				SetFrozen(XMPBoolean isFrozen);

	XMPBoolean			DoesPropagateEvents();
	void				SetPropagateEvents(XMPBoolean doesPropagateEvents);

	// ----- viewType/Presentation -----
	XMPTypeToken		GetPresentation() const;
	XMPTypeToken		GetViewType() const;
		
	// ----- Drag & Drop -----
	void 					SetDragging(XMPBoolean dragging);
	void					SetDroppable(XMPBoolean droppable);
	
//------------------------------------------------------------------------------
//	 CBaseFrame Data Members
//
private:
	XMPFrame			*fXMPFrame;
};




//==============================================================================
//  CBaseFrame Inlines
//==============================================================================

//------------------------------------------------------------------------------
//  CBaseFrame::GetXMPFrame
//------------------------------------------------------------------------------
inline XMPFrame* CBaseFrame::GetXMPFrame() const
{
	return fXMPFrame;
}

//------------------------------------------------------------------------------
//  CBaseFrame::GetXMPPart
//------------------------------------------------------------------------------
inline XMPPart* CBaseFrame::GetXMPPart() const
{
	return fXMPFrame->GetPart();
}

//------------------------------------------------------------------------------
//  CBaseFrame::GetFrameGroup
//------------------------------------------------------------------------------
inline XMPULong CBaseFrame::GetFrameGroup()
{
	return fXMPFrame->GetFrameGroup();
}

//------------------------------------------------------------------------------
//  CBaseFrame::SetFrameGroup
//------------------------------------------------------------------------------
inline void CBaseFrame::SetFrameGroup(XMPULong groupID)
{
	fXMPFrame->SetFrameGroup(groupID);
}

//------------------------------------------------------------------------------
//  CBaseFrame::IsRoot
//------------------------------------------------------------------------------
inline XMPBoolean CBaseFrame::IsRoot()
{
	return fXMPFrame->IsRoot();
}

//------------------------------------------------------------------------------
//  CBaseFrame::IsOverlaid
//------------------------------------------------------------------------------
inline XMPBoolean CBaseFrame::IsOverlaid()
{
	return fXMPFrame->IsOverlaid();
}

//------------------------------------------------------------------------------
//  CBaseFrame::IsFrozen
//------------------------------------------------------------------------------
inline XMPBoolean CBaseFrame::IsFrozen()
{
	return fXMPFrame->IsFrozen();
}

//------------------------------------------------------------------------------
//  CBaseFrame::SetFrozen
//------------------------------------------------------------------------------
inline void CBaseFrame::SetFrozen(XMPBoolean isFrozen)
{
	fXMPFrame->SetFrozen(isFrozen);
}

//------------------------------------------------------------------------------
//  CBaseFrame::DoesPropagateEvents
//------------------------------------------------------------------------------
inline XMPBoolean CBaseFrame::DoesPropagateEvents()
{
	return fXMPFrame->DoesPropagateEvents();
}

//------------------------------------------------------------------------------
//  CBaseFrame::SetPropagateEvents
//------------------------------------------------------------------------------
inline void CBaseFrame::SetPropagateEvents(XMPBoolean doesPropagateEvents)
{
	fXMPFrame->SetPropagateEvents(doesPropagateEvents);
}

//----------------------------------------------------------------------------------------
//	 CBaseFrame::GetViewType
//----------------------------------------------------------------------------------------
inline XMPTypeToken CBaseFrame::GetViewType() const
{
	return fXMPFrame->GetViewType();
}

//----------------------------------------------------------------------------------------
//	 CBaseFrame::GetPresentation
//----------------------------------------------------------------------------------------
inline XMPTypeToken CBaseFrame::GetPresentation() const
{
	return fXMPFrame->GetPresentation();
}


//----------------------------------------------------------------------------------------
//	 CBaseFrame::SetDragging
//----------------------------------------------------------------------------------------
inline void CBaseFrame::SetDragging(XMPBoolean dragging)
{
	fXMPFrame->SetDragging(dragging);
}

//----------------------------------------------------------------------------------------
//	 CBaseFrame::SetDroppable
//----------------------------------------------------------------------------------------
inline void CBaseFrame::SetDroppable(XMPBoolean droppable)
{
	fXMPFrame->SetDroppable(droppable);
}

#endif
