//	File:		FRBorder.cpp
//	Contains:	Implementation of class CProxyBorder
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWBORDER_
#include "FWBorder.h"
#endif

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _XMPMEM_
#include <XMPMem.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

// ----- Macintosh Includes -----
#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#pragma segment FWFrameSeg

//==============================================================================
//	 Class CBaseProxyBorder
//==============================================================================

//------------------------------------------------------------------------------
//	 CBaseProxyBorder::CBaseProxyBorder
//------------------------------------------------------------------------------

CBaseProxyBorder::CBaseProxyBorder()
{
	fProxyRun = kXMPNULL;
	fBorderHasChanged = kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CBaseProxyBorder::InitBaseProxyBorder
//------------------------------------------------------------------------------

void CBaseProxyBorder::InitBaseProxyBorder(CProxyRun *proxyRun)
{
	fProxyRun = proxyRun;
}

//------------------------------------------------------------------------------
//	 CBaseProxyBorder::~CBaseProxyBorder
//------------------------------------------------------------------------------

CBaseProxyBorder::~CBaseProxyBorder()
{
}

//==============================================================================
//	 class CProxyBorder
//==============================================================================

//------------------------------------------------------------------------------
//	 CProxyBorder::CProxyBorder
//------------------------------------------------------------------------------

CProxyBorder::CProxyBorder()
{
	fSelectRgn 			= kXMPNULL;
	fCornerHandleRgn 	= kXMPNULL;
	fEdgeHandleRgn 		= kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CProxyBorder::InitProxyBorder
//------------------------------------------------------------------------------

void CProxyBorder::InitProxyBorder(CProxyRun *proxyRun)
{
	InitBaseProxyBorder(proxyRun);
}

//------------------------------------------------------------------------------
//	 CProxyBorder::~CProxyBorder
//------------------------------------------------------------------------------

CProxyBorder::~CProxyBorder()
{
	DisposeSelectionRgn();
}

//------------------------------------------------------------------------------
//	 CProxyBorder::DisposeSelectionRgn
//------------------------------------------------------------------------------

void CProxyBorder::DisposeSelectionRgn()
{
	if (fSelectRgn)
		DisposeRgn(fSelectRgn);
	if (fCornerHandleRgn)
		DisposeRgn(fCornerHandleRgn);
	if (fEdgeHandleRgn)
		DisposeRgn(fEdgeHandleRgn);
	
	fSelectRgn = kXMPNULL;
	fCornerHandleRgn = kXMPNULL;
	fEdgeHandleRgn = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CProxyBorder::CalcProxyBorder
//------------------------------------------------------------------------------

void CProxyBorder::CalcProxyBorder(CFacet* facet, XMPFacet *embeddedXMPFacet)
{
	DisposeSelectionRgn();
	
	//	 get frame shape to draw border
	XMPRgnHandle border = NewRgn();
	
	{
		XMPShape* frameShape = ::NewXMPShape(embeddedXMPFacet->GetFrame()->GetFrameShape());
		
		frameShape->Transform(embeddedXMPFacet->GetExternalTransform());	// now in enclosure content coordinate
		
		Rect qdRect;
		::GetXMPShapeQDBox(frameShape, qdRect);
		delete frameShape;
		
		::RectRgn(border, &qdRect);
	}
	
	
	XMPRgnHandle insideBorder = ::NewRgn();
	::CopyRgn(border, insideBorder);
	::InsetRgn(border, -kXMPBorderWidth, -kXMPBorderWidth);
	::DiffRgn(border, insideBorder, border);
	::DisposeRgn(insideBorder);
	
	Rect borderRect = (**border).rgnBBox;
	
	GrafPtr svPort;
	::GetPort(&svPort);
	
	::SetPort(embeddedXMPFacet->GetWindow()->GetPlatformWindow());
	
	::OpenRgn();

	// top left handle
	::MoveTo(borderRect.left, borderRect.top);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.top);

	// top right handle
	::MoveTo(borderRect.right, borderRect.top);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.top);

	// bottom right handle
	::MoveTo(borderRect.right, borderRect.bottom);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.bottom);

	// bottom left handle
	::MoveTo(borderRect.left, borderRect.bottom);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.bottom);

	fCornerHandleRgn = NewRgn();
	::CloseRgn(fCornerHandleRgn);
	
	::UnionRgn(border,fCornerHandleRgn, border);
	
	::OpenRgn();

	if ((borderRect.right - borderRect.left) >= (5 * kXMPHandleLenMultiplier * kXMPBorderWidth)) {
	
		XMPSShort hcenter = (borderRect.right - borderRect.left) / 2;
		XMPSShort hdist = (kXMPHandleLenMultiplier * kXMPBorderWidth) / 2;
		
		// top middle handle
		::MoveTo(borderRect.left + hcenter - hdist, borderRect.top);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.top);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.top + kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.top + kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.top);
		
		// bottom middle handle
		::MoveTo(borderRect.left + hcenter - hdist, borderRect.bottom);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.bottom);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.bottom - kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.bottom - kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.bottom);
	}
	
	if ((borderRect.bottom - borderRect.top) >= (5 * kXMPHandleLenMultiplier * kXMPBorderWidth)) {
	
		XMPSShort vcenter = (borderRect.bottom - borderRect.top) / 2;
		XMPSShort vdist = (kXMPHandleLenMultiplier * kXMPBorderWidth) / 2;
		
		// left middle handle
		::MoveTo(borderRect.left, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.left, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.left, borderRect.top + vcenter - vdist);
		
		// right middle handle
		::MoveTo(borderRect.right, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.right, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.right, borderRect.top + vcenter - vdist);
	}
	
	fEdgeHandleRgn = NewRgn();
	::CloseRgn(fEdgeHandleRgn);
	
	::SetPort(svPort);
	
	::UnionRgn(border,fEdgeHandleRgn, border);
	fSelectRgn = border;
	
	XMPRect xmpRect((*fSelectRgn)->rgnBBox);
	XMPShape* clipShape = ::NewXMPShape();
	GetContentClip(facet, xmpRect, clipShape);
	if (!clipShape->IsEmpty())
	{
		::DiffRgn(fSelectRgn, clipShape->GetQDRegion(), fSelectRgn);
		::DiffRgn(fCornerHandleRgn, clipShape->GetQDRegion(), fCornerHandleRgn);
		::DiffRgn(fEdgeHandleRgn, clipShape->GetQDRegion(), fEdgeHandleRgn);
	}
	delete clipShape;
	
	BorderHasChanged(kXMPFalse);
}

//------------------------------------------------------------------------------
//	 CProxyBorder::GetContenClip
//------------------------------------------------------------------------------

void CProxyBorder::GetContentClip(CFacet* facet, const XMPRect& borderBox, XMPShape* clip)
{
	clip->Subtract(clip);

	XMPRect shapeRect;
	XMPShape* clipShape = ::NewXMPShape();
	XMPFacetIterator* ite = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (XMPFacet* xmpFacet = ite->First(); ite->IsNotComplete(); xmpFacet = ite->Next())
	{
		if (xmpFacet->GetFrame()->GetPart() == GetProxyRun()->GetXMPPart())
			break;
		clipShape->CopyFrom(xmpFacet->GetClipShape());
		clipShape->Transform(xmpFacet->GetExternalTransform());
		clipShape->GetBoundingBox(&shapeRect);
		if (shapeRect.Intersects(borderBox))
		{
			clip->Union(clipShape);
		}
	}
	delete ite;
	delete clipShape;
}

//------------------------------------------------------------------------------
//	 CProxyBorder::DrawProxyBorder
//------------------------------------------------------------------------------
//	Enclosure must be prepared for drawing

void CProxyBorder::DrawProxyBorder(CFacet* facet, XMPFacet* embeddedFacet)
{
XMPUnused(facet);

	if (GetProxyRun()->IsSelected())
	{
		if (HasBorderChanged())
			CalcProxyBorder(facet, embeddedFacet);
		
#ifdef THINK_CPLUS
		::FillRgn(fSelectRgn, XMPASLMQDGlobals.gray);
		::FillRgn(fCornerHandleRgn, XMPASLMQDGlobals.white);
		::FillRgn(fEdgeHandleRgn, XMPASLMQDGlobals.white);
#else
		::FillRgn(fSelectRgn, &XMPASLMQDGlobals.gray);
		::FillRgn(fCornerHandleRgn, &XMPASLMQDGlobals.white);
		::FillRgn(fEdgeHandleRgn, &XMPASLMQDGlobals.white);
#endif

		::FrameRgn(fCornerHandleRgn);
		::FrameRgn(fEdgeHandleRgn);
	}
}

//------------------------------------------------------------------------------
//	 CProxyBorder::EraseProxyBorder
//------------------------------------------------------------------------------
//	Enclosure must be prepared for drawing

void CProxyBorder::EraseProxyBorder(CFacet* facet, XMPFacet* embeddedFacet)
{	
XMPUnused(embeddedFacet);

	if (fSelectRgn != kXMPNULL)
	{
		XMPShape* borderShape = ::NewXMPShape((RgnHandle)XMPCopyHandle((XMPHandle)fSelectRgn));
		::EraseRgn(fSelectRgn);
		facet->Invalidate(borderShape);
		delete borderShape;
	}
}

//------------------------------------------------------------------------------
//	 CProxyBorder::WhereInProxyBorder
//------------------------------------------------------------------------------
//	1--5--2
//	|	  |
//	8	  6
//	|	  |
//  3--7--4

XMPSShort CProxyBorder::WhereInProxyBorder(CFacet* facet, Point mouse) const
{
XMPUnused(facet);

	if (GetProxyRun()->IsSelected())
	{
		if (::PtInRgn(mouse, fCornerHandleRgn))
		{
			Rect rect = (*fSelectRgn)->rgnBBox;
			
			Point middle;
			middle.h  = (rect.right + rect.left) / 2;
			middle.v  = (rect.top + rect.bottom) / 2;
			
			if (mouse.h < middle.h)
			{
				if (mouse.v < middle.v)
					return kInTopLeftCorner;
				else
					return kInBottomLeftCorner;
			}
			else
			{
				if (mouse.v < middle.v)
					return kInTopRightCorner;
				else
					return kInBottomRightCorner;		
			}
		}
		else if (::PtInRgn(mouse, fEdgeHandleRgn))
		{
			return kInDrag; // [HLX] for now
		}
		else if (::PtInRgn(mouse, fSelectRgn))
			return kInDrag;
	}
	
	return 0;
}

