//	File:		FWBorder.h
//
//	Contains:	Definition for classes
//					 CBaseProxyBorder
//					 CProxyBorder
//
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWBORDER_
#define _FWBORDER_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class CProxyRun;
class XMPPoint;
class XMPShape;
class CFacet;
class XMPFacet;




//==============================================================================
//	 Constants
//==============================================================================

const XMPSShort kXMPBorderWidth = 5;
const XMPSShort kXMPHandleLenMultiplier = 3;

const XMPSShort kInTopLeftCorner = 1;
const XMPSShort kInTopRightCorner = 2;
const XMPSShort kInBottomLeftCorner = 3;
const XMPSShort kInBottomRightCorner = 4;

const XMPSShort kInTopEdge = 5;
const XMPSShort kInRightEdge = 6;
const XMPSShort kInBottomEdge = 7;
const XMPSShort kInLeftEdge = 8;

const XMPSShort kInDrag = 100;




//==============================================================================
//	 class CBaseProxyBorder
//==============================================================================

class CBaseProxyBorder
{
//------------------------------------------------------------------------------
//	 CBaseProxyBorder Initialization/Destruction
//
public:
	CBaseProxyBorder();
	void InitBaseProxyBorder(CProxyRun* proxyRun);
	virtual ~CBaseProxyBorder();

//------------------------------------------------------------------------------
//	 CBaseProxyBorder New API
//
public:
	// ----- Drawing -----
	virtual void		DrawProxyBorder(CFacet* facet, XMPFacet* embeddedFacet) = 0;
	virtual void		EraseProxyBorder(CFacet* facet, XMPFacet* embeddedFacet) = 0;
	
	// ----- Hit Test -----
	virtual XMPSShort	WhereInProxyBorder(CFacet* facet, Point mouse) const = 0;
	
	// ----- Invalid Border Cache -----
	void				BorderHasChanged(XMPBoolean state);
	XMPBoolean			HasBorderChanged();
	
	// ----- Getters/Setters -----
	CProxyRun*			GetProxyRun() const;
							
//------------------------------------------------------------------------------
//	 CBaseProxyBorder Data Members
//
private:	
	CProxyRun			*fProxyRun;
	XMPBoolean			fBorderHasChanged;
};




//==============================================================================
//	 class CProxyBorder
//==============================================================================

class CProxyBorder : public CBaseProxyBorder
{
//------------------------------------------------------------------------------
//	 CProxyBorder Initialization/Destruction
//
public:
	CProxyBorder();
	void InitProxyBorder(CProxyRun* proxyRun);
	virtual ~CProxyBorder();

//------------------------------------------------------------------------------
//	 CProxyBorder Inherited API
//
public:
	// ----- Drawing -----
	void				DrawProxyBorder(CFacet* facet, XMPFacet* embeddedFacet);
	void				EraseProxyBorder(CFacet* facet, XMPFacet* embeddedFacet);
	
	// ----- Hit Test -----
	XMPSShort			WhereInProxyBorder(CFacet* facet, Point mouse) const;
	
//------------------------------------------------------------------------------
//	 CProxyBorder New API
//
public:
	// ----- Calc/Dispose -----
	void				CalcProxyBorder(CFacet* facet, XMPFacet *embeddedXMPFacet);
	virtual void 		GetContentClip(CFacet* facet, const XMPRect& borderBox, XMPShape* clip);

private:
	void				DisposeSelectionRgn();

//------------------------------------------------------------------------------
//	 CProxyBorder Data Members
//
private:	
	RgnHandle			fSelectRgn;
	RgnHandle			fCornerHandleRgn;
	RgnHandle			fEdgeHandleRgn;
};




//==============================================================================
//	 CBaseProxyBorder Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CBaseProxyBorder::BorderHasChanged
//------------------------------------------------------------------------------
inline void CBaseProxyBorder::BorderHasChanged(XMPBoolean state)
{
	fBorderHasChanged = state;
}

//------------------------------------------------------------------------------
//	 CBaseProxyBorder::HasBorderChanged
//------------------------------------------------------------------------------
inline XMPBoolean CBaseProxyBorder::HasBorderChanged()
{
	return fBorderHasChanged;
}
	
//------------------------------------------------------------------------------
//	 CBaseProxyBorder::GetProxyRun
//------------------------------------------------------------------------------
inline CProxyRun* CBaseProxyBorder::GetProxyRun() const
{
	return  fProxyRun;
}

#endif

