//	File:		FWCtlMgr.cpp
//	Contains:	Implementation of classes CCtlMgrPart, CCtlMgrFrame, CCtlMgrFacet
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWCTLMGR_
#include "FWCtlMgr.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

// ----- Macintosh Includes

#ifndef mathRoutinesIncludes
#include <math routines.h>
#endif

#pragma segment framework

//==============================================================================
//	 Static
//==============================================================================

// Doesn't work with ASLM -> wait to get globals
// WindowPtr CCtlMgrPart::gScratchWindow = kXMPNULL;

//==============================================================================
//	 class CCtlMgrPart
//==============================================================================

//------------------------------------------------------------------------------
//	 CCtlMgrPart::CCtlMgrPart
//------------------------------------------------------------------------------

CCtlMgrPart::CCtlMgrPart():
	fControlSize(ff(80), ff(20))
{
	fControlHandle = kXMPNULL;
	gScratchWindow = kXMPNULL;	// To remove when we have globals
}

//------------------------------------------------------------------------------
//	 CCtlMgrPart::~CCtlMgrPart
//------------------------------------------------------------------------------

CCtlMgrPart::~CCtlMgrPart()
{
	if (fControlHandle)
	{
		::DisposeControl(fControlHandle);
		fControlHandle = kXMPNULL;
	}
	
	if (gScratchWindow)
	{
		XMPSLong count;
		::SetWRefCon(gScratchWindow,  count = (::GetWRefCon(gScratchWindow) - 1));
		if (count == 0)
		{
			::DisposeWindow(gScratchWindow);
			gScratchWindow = kXMPNULL;
		}
	}
}

//------------------------------------------------------------------------------
//	 CCtlMgrPart::Initialize
//------------------------------------------------------------------------------

void CCtlMgrPart::Initialize()
{
	CPart::Initialize();
	
	if (gScratchWindow == kXMPNULL)
	{
		Rect rBounds;
		::SetRect(&rBounds, 0, 0, 10 , 10);
		gScratchWindow = ::NewCWindow(kXMPNULL,
										&rBounds,
										"\p",
										kXMPFalse,
										documentProc,
										kXMPNULL,
										kXMPFalse,
										0);
	}

	::SetWRefCon(gScratchWindow, ::GetWRefCon(gScratchWindow) + 1);
	
	XMPRect xmpRect(0, 0, fControlSize.x, fControlSize.y);
	fControlHandle = CreateControlHandle(gScratchWindow, xmpRect);
	
	// ----- Remove it from the window -----
	((WindowPeek)gScratchWindow)->controlList = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CCtlMgrPart::NewFrame
//------------------------------------------------------------------------------

CFrame* CCtlMgrPart::NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation)
{
	XMPUnused(presentation);
	
	CCtlMgrFrame *frame = new CCtlMgrFrame;
	frame->InitCtlMgrFrame(xmpFrame, this);
	return frame;
}

//------------------------------------------------------------------------------
//	 CCtlMgrPart::SetControlSize
//------------------------------------------------------------------------------

void CCtlMgrPart::SetControlSize(const XMPPoint& controlSize)
{
	if (controlSize != fControlSize)
	{
		fControlSize = controlSize;
		
		XMPRect xmpRect(0, 0, controlSize.x, controlSize.y);
		XMPShape controlShape;
		controlShape.SetRectangle(&xmpRect);
		
		CPartDisplayFrameIterator ite(this);
		for (CCtlMgrFrame *frame = (CCtlMgrFrame*)ite.First(); ite.IsNotComplete(); frame = (CCtlMgrFrame*)ite.Next())
		{
			XMPShape *shape = ::NewXMPShape(&controlShape);
			frame->RequestFrameShape(shape);
			frame->UpdateUsedAndActiveShapes();
		}
	}
}

//------------------------------------------------------------------------------
//	 CCtlMgrPart::DoClick
//------------------------------------------------------------------------------

void CCtlMgrPart::DoClick(const XMPPoint& where, XMPEventData event)
{
	XMPUnused(where);
	XMPUnused(event);
}

//==============================================================================
//	 class CCtlMgrFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 CCtlMgrFrame::CCtlMgrFrame
//------------------------------------------------------------------------------

CCtlMgrFrame::CCtlMgrFrame()
{
}

//------------------------------------------------------------------------------
//	 CCtlMgrFrame::InitCtlMgrFrame
//------------------------------------------------------------------------------

void CCtlMgrFrame::InitCtlMgrFrame(XMPFrame* frame, CPart* part)
{
	IFrame(frame, part);
	SetFrontClicks(kXMPTrue);	// I want the first click
}

//------------------------------------------------------------------------------
//	 CCtlMgrFrame::~CCtlMgrFrame
//------------------------------------------------------------------------------

CCtlMgrFrame::~CCtlMgrFrame()
{
}

//------------------------------------------------------------------------------
//	 CCtlMgrFrame::NewFacet
//------------------------------------------------------------------------------

CFacet* CCtlMgrFrame::NewFacet(XMPFacet* xmpFacet)
{
	CCtlMgrFacet* facet = new CCtlMgrFacet;
	facet->InitCtlMgrFacet(xmpFacet);
	return facet;
}

//------------------------------------------------------------------------------
//	 CCtlMgrFrame::FacetAdded
//------------------------------------------------------------------------------

void CCtlMgrFrame::FacetAdded(CFacet* facet)
{
	XMPUnused(facet);
	
	XMPPoint controlSize;
	((CCtlMgrPart*)GetPart())->GetControlSize(controlSize);
	XMPRect xmpRect(0, 0, controlSize.x, controlSize.y);
	XMPShape* controlShape = ::NewXMPShape(&xmpRect);

	this->RequestFrameShape(controlShape);
	this->UpdateUsedAndActiveShapes();
}

//------------------------------------------------------------------------------
//	 CCtlMgrFrame::FrameShapeChanged
//------------------------------------------------------------------------------

void CCtlMgrFrame::FrameShapeChanged()
{
	XMPRect rect;
	GetXMPFrame()->GetFrameShape()->GetBoundingBox(&rect);
	
	XMPPoint controlSize(rect.right - rect.left, rect.bottom - rect.top);
	((CCtlMgrPart*)GetPart())->SetControlSize(controlSize);
	
	CFrame::FrameShapeChanged();
}

//==============================================================================
//	 class CCtlMgrFacet
//==============================================================================

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::CCtlMgrFacet
//------------------------------------------------------------------------------

CCtlMgrFacet::CCtlMgrFacet()
{
}

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::InitCtlMgrFacet
//------------------------------------------------------------------------------

void CCtlMgrFacet::InitCtlMgrFacet(XMPFacet* xmpFacet)
{
	InitFacet(xmpFacet);
}

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::~CCtlMgrFacet
//------------------------------------------------------------------------------

CCtlMgrFacet::~CCtlMgrFacet()
{
}

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::Draw
//------------------------------------------------------------------------------

void CCtlMgrFacet::Draw(XMPShape* invalidShape)
{
XMPUnused(invalidShape);

	ControlHandle hControl = SetupControl();
		
	::Draw1Control(hControl);
	
	RestoreControl();
}

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::DoMouseDown
//------------------------------------------------------------------------------

XMPBoolean CCtlMgrFacet::DoMouseDown(const XMPPoint& where, XMPEventData event)
{
XMPUnused(event);

	CDrawInitiator di(this);
	
	ControlHandle hControl = SetupControl();
	
	Point pt = where.AsQDPoint();
	
	if (::TrackControl(hControl, pt, kXMPNULL))
		((CCtlMgrPart*)this->GetFrame()->GetPart())->DoClick(where, event);

	RestoreControl();
	
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::SetupControl
//------------------------------------------------------------------------------

ControlHandle CCtlMgrFacet::SetupControl()
{
	ControlHandle hControl = GetControlHandle();

	if (hControl)
	{
		XMPRect xmpRect;
		XMPShape* frameShape = ::NewXMPShape();
		GetFrame()->GetFrameShape(frameShape);
		frameShape->Transform(GetExternalTransform());
		frameShape->GetBoundingBox(&xmpRect);
		delete frameShape;
		
		Rect rect;
		xmpRect.AsQDRect(rect);
		::OffsetRect(&rect, -rect.left, -rect.top);	// because of the origin
		(*hControl)->contrlRect = rect;	
		(*hControl)->contrlOwner = GetXMPWindow()->GetPlatformWindow();
	}
	
	return hControl;
}

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::RestoreControl
//------------------------------------------------------------------------------

void CCtlMgrFacet::RestoreControl()
{
	ControlHandle hControl = GetControlHandle();
	if (hControl)
		(*hControl)->contrlOwner = ((CCtlMgrPart*)this->GetFrame()->GetPart())->gScratchWindow;	//CCtlMgrPart::gScratchWindow;
}
