/*
	File:		FWCtlMgr.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<2+>	 3/17/94	DS		Add CDECL to CtrlMgrPart and CtrlMgrFrame
									dtor
		 <2>	  3/8/94	DS		Make DoClick Virtual

	To Do:
*/

//	File:		FWCtlMgr.h
//
//	Contains:	Definition for classes 
//						 CCtlMgrPart
//						 CCtlMgrFrame
//						 CCtlMgrFacet
//
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWCTLMGR_
#define _FWCTLMGR_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

// ----- Macintosh Includes -----

#ifndef __CONTROLS__
#include <Controls.h>
#endif

//==============================================================================
//	 class CCtlMgrPart
//==============================================================================

#define kCCtlMgrPartID "appl:cctlmgrpart$class,1.0.0"

class CCtlMgrPart : public CPart
{

//------------------------------------------------------------------------------
//	 CCtlMgrPart Initialization/Destruction
//
public:
	virtual ~ CDECL CCtlMgrPart();

protected:
	CCtlMgrPart();
	
	virtual void Initialize();

//------------------------------------------------------------------------------
//	 CCtlMgrPart Inherited API
//
public:
	virtual CFrame*			NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	
//------------------------------------------------------------------------------
//	 CCtlMgrPart New API
//
public:
	ControlHandle				GetControlHandle() const;
									
	void						SetControlSize(const XMPPoint& controlSize);
	void						GetControlSize(XMPPoint& controlSize) const;

	void						SetAsDefaultButton(XMPBoolean state);
	XMPBoolean					IsDefaultButton() const;
	virtual void				DoClick(const XMPPoint& where, XMPEventData event);

protected:
	virtual ControlHandle		CreateControlHandle(XMPPlatformWindow window, const XMPRect& controlRect) = 0;
									
//------------------------------------------------------------------------------
//	 CCtlMgrPart Data Members
//
public:
	WindowPtr 	gScratchWindow;
//	static WindowPtr 	gScratchWindow;		Doesn't work with ASLM

private:
	ControlHandle		fControlHandle;
	XMPPoint			fControlSize;
	XMPBoolean 			fDefaultButton;
};




//==============================================================================
//	 class CCtlMgrFrame
//==============================================================================

#define kCCtlMgrFrameID "appl:cctlmgrframe$class,1.0.0"

class CCtlMgrFrame : public CFrame
{
//------------------------------------------------------------------------------
//	 CCtlMgrFrame Initialization/Destruction
//
public:
	CCtlMgrFrame();
	virtual void InitCtlMgrFrame(XMPFrame* frame, CPart* part);
	virtual ~ CDECL CCtlMgrFrame();
		
//------------------------------------------------------------------------------
//	 CCtlMgrFrame Inherited API
//
public:
	virtual CFacet*		NewFacet(XMPFacet* xmpFacet);
	virtual void			FacetAdded(CFacet* facet);
	virtual void			FrameShapeChanged();	
};




//==============================================================================
//	 class CCtlMgrFacet
//==============================================================================

#define kCCtlMgrFacetID "appl:cctlmgrfacet$class,1.0.0"

class CCtlMgrFacet : public CFacet
{

//------------------------------------------------------------------------------
//	 CCtlMgrFacet Initialization/Destruction
//
public:
	CCtlMgrFacet();
	virtual void InitCtlMgrFacet(XMPFacet* xmpFacet);
	virtual ~CCtlMgrFacet();
		

//------------------------------------------------------------------------------
//	 CCtlMgrFacet Inherited API
//
public:
	virtual void		Draw(XMPShape* invalidShape);
	virtual XMPBoolean DoMouseDown(const XMPPoint& where, XMPEventData event);


//------------------------------------------------------------------------------
//	 CCtlMgrFacet New API
//
public:
	ControlHandle		GetControlHandle() const;

protected:
	ControlHandle		SetupControl();
	void				RestoreControl();
};




//==============================================================================
//	 class CCtlMgrPart inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CCtlMgrPart::GetControlHandle
//------------------------------------------------------------------------------

inline ControlHandle CCtlMgrPart::GetControlHandle() const
{
	return fControlHandle;
}
									
//------------------------------------------------------------------------------
//	 CCtlMgrPart::GetControlSize
//------------------------------------------------------------------------------

inline void CCtlMgrPart::GetControlSize(XMPPoint& controlSize) const
{
	controlSize = fControlSize;
}

//------------------------------------------------------------------------------
//	 CCtlMgrPart::SetAsDefaultButton
//------------------------------------------------------------------------------
inline void CCtlMgrPart::SetAsDefaultButton(XMPBoolean state)
{
	fDefaultButton = state;
}

//------------------------------------------------------------------------------
//	 CCtlMgrPart::IsDefaultButton
//------------------------------------------------------------------------------
inline XMPBoolean CCtlMgrPart::IsDefaultButton() const
{
	return fDefaultButton;
}



//==============================================================================
//	 class CCtlMgrFacet inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CCtlMgrFacet::GetControlHandle
//------------------------------------------------------------------------------
inline ControlHandle CCtlMgrFacet::GetControlHandle() const
{
	return ((CCtlMgrPart*)GetFrame()->GetPart())->GetControlHandle();
}


#endif
