/*
	File:		FWFacet.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 4/12/94	DS		Replace ExternalTransformChanged and
									ClipShapeChange with GeometryChanged.
		 <4>	 4/11/94	DS		Change ChangeClipShape to ChangeGeometry
		 <3>	  4/4/94	DS		Change to ::drop
		 <2>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h

	To Do:
*/

//	File:		FWFacet.cpp
//	Contains:	Implementation of class CFacet
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

// ----- Framework Includes -----

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _FWWINDOW_
#include "FWWindow.h"
#endif

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif

#ifndef _FWDROPTR_
#include "FWDropTr.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _XMPSESSN_
#include "XMPSessM.h"
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _DGITMITR_
#include <DgItmItr.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

// ----- Macintosh Includes -----

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#pragma segment FWFacetSeg

//==============================================================================
//	 class CFacet
//==============================================================================

//------------------------------------------------------------------------------
//	 CFacet::CFacet
//------------------------------------------------------------------------------

CFacet::CFacet()
{
	fXMPFacet = kXMPNULL;
	fRedraw = kXMPTrue;
	fCursorInContent = kXMPFalse;
	fCanAcceptDrop = kXMPFalse;
	fDropTracker = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CFacet::InitFacet
//------------------------------------------------------------------------------

void CFacet::InitFacet(XMPFacet* xmpFacet)
{
	fXMPFacet = xmpFacet;

	xmpFacet->SetPartInfo((XMPInfoType)this);
	
	fFrame = CFrame::XMPtoFWFrame(xmpFacet->GetFrame());
	
	fDropTracker = this->NewDropTracker(this);
}

//------------------------------------------------------------------------------
//	 CFacet::~CFacet
//------------------------------------------------------------------------------

CFacet::~CFacet()
{
	XMPFacet *xmpFacet = GetXMPFacet();
	if (xmpFacet)
		xmpFacet->SetPartInfo((XMPInfoType)kXMPNULL);
}

//------------------------------------------------------------------------------
//	 CFacet::XMPtoFWFacet
//------------------------------------------------------------------------------

CFacet* CFacet::XMPtoFWFacet(XMPFacet* xmpFacet)
{
	return xmpFacet != kXMPNULL ? (CFacet*)xmpFacet->GetPartInfo() : (CFacet*)kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CFacet::CalcDragHiliteRgn
//------------------------------------------------------------------------------

RgnHandle CFacet::CalcDragHiliteRgn()
{
	return this->GetXMPFacet()->GetActiveShape()->CopyQDRegion();
}

//------------------------------------------------------------------------------
//	 CFacet::DragEnter
//------------------------------------------------------------------------------
// 'where' is in window coordinate

void CFacet::DragEnter(XMPDragItemIterator* dragInfo, XMPPoint where)
{
	XMPStorageUnit *dragSU;
	HFSFlavor *valueData;
	long valueSize;
	CPart *thePart = this->GetFrame()->GetPart();
	FW_CSelection* selection = thePart->GetSelection();
	
	fCanAcceptDrop = kXMPFalse;

	for (dragSU = dragInfo->First(); dragSU; dragSU = dragInfo->Next())
		if (thePart->IsEmbeddingPart() || dragSU->Exists(kXMPPropContents, thePart->GetContentPropertyValueType(), 0))
			fCanAcceptDrop = kXMPTrue;
		else if (dragSU->Exists(kXMPPropContents, kXMPApplehfs, 0))
		{
			dragSU->Focus(kXMPPropContents, kXMPPosUndefined, kXMPApplehfs, 0, kXMPPosUndefined);		
			valueSize = dragSU->GetSize();
			valueData = (HFSFlavor*)NewPtr(valueSize);
			if (valueData)
			{
				dragSU->GetValue(valueSize, (XMPValue)valueData);
				if (valueData->fileType == 'bnto')
					fCanAcceptDrop = kXMPTrue;
			}
			DisposePtr((Ptr)valueData);
		}
	
	fCursorInContent = kXMPFalse;
	
	if (fCanAcceptDrop && fDropTracker)
		fDropTracker->HandleBeginTracking(where);
}

//------------------------------------------------------------------------------
//	 CFacet::DragWithin
//------------------------------------------------------------------------------
// 'where' is in window coordinate

void CFacet::DragWithin(XMPDragItemIterator* dragInfo, XMPPoint where)
{
	XMPUnused(dragInfo);
	
	if (!fCanAcceptDrop)
		return;
		
	DragReference theDrag;
	XMPULong attributes;
	XMPSession* session = this->GetFrame()->GetPart()->GetSession();
	
	CDrawInitiator di(this->GetXMPWindow());		// focus drawing to the window

	attributes = session->GetDragAndDrop()->GetDragAttributes();
	theDrag = session->GetDragAndDrop()->GetDragReference();
	
	if (this->GetXMPFacet()->ContainsPoint(where))
	{
		if (!fCursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
		{
			RgnHandle hiliteRgn = this->CalcDragHiliteRgn();
			Point offset = this->GetWindowContentTransform()->GetQDOffset();
			::OffsetRgn(hiliteRgn, offset.h, offset.v);
			::ShowDragHilite(theDrag, hiliteRgn, this->DrawDragHiliteInside());
			fCursorInContent = kXMPTrue;
		}
	}
	else
	{
		if (fCursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
			::HideDragHilite(theDrag);
		fCursorInContent = kXMPFalse;
	}
	
	if (fDropTracker)
		fDropTracker->HandleContinueTracking(where);
}

//------------------------------------------------------------------------------
//	 CFacet::DragLeave
//------------------------------------------------------------------------------
// 'where' is in window coordinate

void CFacet::DragLeave(XMPPoint where)
{
	if (!fCanAcceptDrop)
		return;
	
	DragReference theDrag;
	XMPULong attributes;
	XMPSession* session = this->GetFrame()->GetPart()->GetSession();
	
	CDrawInitiator di(this->GetXMPWindow());	// focus drawing on the window

	attributes = session->GetDragAndDrop()->GetDragAttributes();
	theDrag = session->GetDragAndDrop()->GetDragReference();
	
	if (fCursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
		HideDragHilite(theDrag);
		
	fCursorInContent = kXMPFalse;
	
	if (fDropTracker)
		fDropTracker->HandleEndTracking(where);
}

//------------------------------------------------------------------------------
//	 CFacet::Drop
//------------------------------------------------------------------------------

XMPDropResult CFacet::Drop(XMPDragItemIterator* dropInfo, XMPPoint where)
{
	XMPStorageUnit *dropSU;
	DragReference theDrag;
	XMPULong attributes;
	XMPPoint dragPoint, pinnedPoint;
	Point qd;
	short mouseDownModifiers, mouseUpModifiers, movePart;
	
	if (!fCanAcceptDrop)
		return kXMPDropFail;
	
	CDrawInitiator di(this->GetXMPWindow());		// focus drawing to the window

	// Get the attributes for this drag
	XMPSession* session = this->GetFrame()->GetPart()->GetSession();
	attributes = session->GetDragAndDrop()->GetDragAttributes();
	theDrag = session->GetDragAndDrop()->GetDragReference();
	::GetDragModifiers(theDrag, 0L, &mouseDownModifiers, &mouseUpModifiers);
	movePart = (attributes & kXMPdragIsInSourceFrame) && (!((mouseDownModifiers & optionKey) | (mouseUpModifiers & optionKey)));

	XMPBoolean acceptedDrop = kXMPFalse;
	for (dropSU = dropInfo->First(); dropSU && !acceptedDrop; dropSU = dropInfo->Next())
	{		
		FW_CSelection* selection = this->GetFrame()->GetPart()->GetSelection();
		if (movePart)
		{
			GetDragOrigin(theDrag, &qd);
			::GlobalToLocal(&qd);
			dragPoint = qd;
			
			GetDragMouse(theDrag, NULL, &qd);
			::GlobalToLocal(&qd);
			pinnedPoint = qd;
			
			acceptedDrop = selection->DoDroppedInSameFrame(dropSU, this, dragPoint, pinnedPoint);
		}
		else
		{
			Point mouseDownOffset = {0,0};
			if (dropSU->Exists(kXMPPropMouseDownOffset, kQDPoint, 0))
			{
				dropSU->Focus(kXMPPropMouseDownOffset, kXMPPosUndefined, kQDPoint, 0, kXMPPosFirstSib);
				dropSU->GetValue(sizeof(Point), (XMPValue) &mouseDownOffset);
			}
			XMPPoint originPoint(mouseDownOffset);
			acceptedDrop = selection->DoDrop(dropSU, this, originPoint, where);
		}
	}
	this->GetFrame()->GetPart()->Changed();
	
	return ( movePart ? kXMPDropMove : kXMPDropCopy);
}

//------------------------------------------------------------------------------
//	 CFacet::DrawDragHiliteInside
//------------------------------------------------------------------------------

XMPBoolean CFacet::DrawDragHiliteInside()
{
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CFacet::NewDropTracker
//------------------------------------------------------------------------------

FW_CDropTracker* CFacet::NewDropTracker(CFacet* facet)
{
XMPUnused(facet);

	return kXMPNULL;
}

//-------------------------------------------------------------------------
// 	 CFacet::HandleMouseDown
//-------------------------------------------------------------------------

XMPBoolean CFacet::HandleMouseDown(XMPFacet* embeddedXMPFacet, XMPEventData event)
{
	XMPWindow* window = GetXMPWindow();

	if (!IsInFloatingWindow())
	{
		XMPBoolean firstClick = kXMPFalse;

		if (!window->IsActive())
		{
			window->Select();
			firstClick = kXMPTrue;
		}
		
		if (!fFrame->IsActive())
		{
			fFrame->ActivateFrame(this);
			firstClick = kXMPTrue;
		}
				
		if (firstClick && !fFrame->GetFrontClicks())
			return kXMPTrue;
	}
	
	Point mousePos = event->where;
	::SetPort(window->GetPlatformWindow());
	::SetOrigin(0,0);
	::GlobalToLocal(&mousePos);
	XMPPoint windowXMPPoint(mousePos);	
	windowXMPPoint = GetWindowContentTransform()->InvertPoint(windowXMPPoint);

	if (event->what == mouseDown)
	{
		return DoMouseDown(windowXMPPoint, event);
	}
	else
	{
		return DoMouseDownInEmbeddedFrame(embeddedXMPFacet, windowXMPPoint, event);
	}
}

//-------------------------------------------------------------------------
//	 CFacet::DoMouseDown
//-------------------------------------------------------------------------

XMPBoolean CFacet::DoMouseDown(const XMPPoint& where, XMPEventData event)
{
XMPUnused(where);
XMPUnused(event);
	return kXMPFalse;
}

//-------------------------------------------------------------------------
//	 CFacet::HandleMouseUp
//-------------------------------------------------------------------------

XMPBoolean CFacet::HandleMouseUp(XMPEventData event)
{
	Point mousePos = event->where;
	::SetPort(GetXMPWindow()->GetPlatformWindow());
	::SetOrigin(0,0);
	::GlobalToLocal(&mousePos);
	XMPPoint windowXMPPoint(mousePos);	
	windowXMPPoint = GetWindowContentTransform()->InvertPoint(windowXMPPoint);

	return DoMouseUp(windowXMPPoint, event);
}

//-------------------------------------------------------------------------
//	 CFacet::DoMouseUp
//-------------------------------------------------------------------------

XMPBoolean CFacet::DoMouseUp(const XMPPoint& where, XMPEventData event)
{
XMPUnused(where);
XMPUnused(event);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
//	 CFacet::WindowActivated
//------------------------------------------------------------------------------

void CFacet::WindowActivated()
{
	CFrame *frame = GetFrame();
	if (frame->NeedsActivating() && !IsInFloatingWindow()) 	// Floating window always active
	{
		frame->ActivateFrame(this);
		frame->SetNeedsActivating(kXMPFalse);
	}
}

//------------------------------------------------------------------------------
//	 CFacet::WindowDeActivated
//------------------------------------------------------------------------------

void CFacet::WindowDeActivated()
{
	if (!IsInFloatingWindow())
	{
		CFrame *frame = GetFrame();
		frame->SetNeedsActivating(frame->IsActive());
	}
}

//------------------------------------------------------------------------------
//	 CFacet::GeometryChanged
//------------------------------------------------------------------------------

void CFacet::GeometryChanged()
{
	ClipEmbeddedFacets();
}

//------------------------------------------------------------------------------
//	 CFacet::ClipEmbeddedFacets
//------------------------------------------------------------------------------

void CFacet::ClipEmbeddedFacets()
{
	XMPFacet* xmpFacet = GetXMPFacet();
	
	XMPShape* workingClip = NewXMPShape(xmpFacet->GetClipShape());
	
	XMPMacTransform* wclipTrans = ::NewXMPTransform(GetWindowFrameTransform());
	workingClip->Transform(wclipTrans);		// so now in window coordinate
	
	XMPFacetIterator* iter = xmpFacet->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (XMPFacet* embeddedFacet = iter->First();
		 iter->IsNotComplete();
		 embeddedFacet = iter->Next())
	{
		// ----- Set the clip to the used shape -----
		XMPShape* embeddedClipShape = ::NewXMPShape(embeddedFacet->GetFrame()->GetUsedShape());
		
		// ----- 
		wclipTrans->CopyFrom(embeddedFacet->GetWindowFrameTransform());
		wclipTrans->Invert();
		workingClip->Transform(wclipTrans);		// now in frame coordinate
		
		// ----- Set the clip of embFrame -----
		embeddedClipShape->Intersect(workingClip);
		embeddedFacet->ChangeGeometry(embeddedClipShape, kXMPNULL);
				
		// ----- remove embFrame clip from the working clip -----
		workingClip->Subtract(embeddedFacet->GetClipShape());
		
		// ----- Put back working clip in window coordinate -----
		wclipTrans->Invert();
		workingClip->Transform(wclipTrans);
	}
	delete iter;
	
	// loop to clip for part content here
	
	delete workingClip;
	delete wclipTrans;
}

//------------------------------------------------------------------------------
//	 CFacet::UpdateActiveShape
//------------------------------------------------------------------------------

void CFacet::UpdateActiveShape()
{
	XMPShape* scratch = ::NewXMPShape(this->GetFrame()->GetXMPFrame()->GetFrameShape());
	this->GetXMPFacet()->ChangeActiveShape(scratch);
}

//------------------------------------------------------------------------------
//	 CFacet::CreateEmbeddedFacet
//------------------------------------------------------------------------------

XMPFacet* CFacet::CreateEmbeddedFacet(CProxyFrame *itsFrame, XMPShape* clipShape, XMPTransform* externalTransform, 
						   		 		XMPFacet* siblingFacet, XMPFramePosition position)
{
	return GetXMPFacet()->CreateEmbeddedFacet(itsFrame->GetXMPFrame(), clipShape, externalTransform,
												siblingFacet, position);
}

//------------------------------------------------------------------------------
//	 CFacet::GetWindow
//------------------------------------------------------------------------------

CWindow* CFacet::GetWindow() const
{
	return GetFrame()->GetPart()->XMPtoFWWindow(GetXMPFacet()->GetWindow());
}

//------------------------------------------------------------------------------
//	 CFacet::Draw
//------------------------------------------------------------------------------

void CFacet::Draw(XMPShape* invalidShape)
{
XMPUnused(invalidShape);
}

//-------------------------------------------------------------------------
//	 CFacet::MouseEnter
//-------------------------------------------------------------------------

void CFacet::MouseEnter(const XMPPoint& where)
{
XMPUnused(where);
	::InitCursor();
}

//-------------------------------------------------------------------------
//	 CFacet::MouseWithin
//-------------------------------------------------------------------------

void CFacet::MouseWithin(const XMPPoint& where)
{
XMPUnused(where);
}

//-------------------------------------------------------------------------
//	 CFacet::MouseLeave
//-------------------------------------------------------------------------

void CFacet::MouseLeave()
{
}

//----------------------------------------------------------------------------------------
//	 CFacet::DoMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

XMPBoolean CFacet::DoMouseDownInEmbeddedFrame(XMPFacet* embeddedXMPFacet, const XMPPoint& where, XMPEventData event)
{
XMPUnused(embeddedXMPFacet);

	return DoMouseDown(where, event);
}

//----------------------------------------------------------------------------------------
//	 CFacet::HandleMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

XMPBoolean CFacet::HandleMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, XMPEventData event)
{
	XMPWindow* window = GetXMPWindow();
	
	if (window->IsActive())
	{
		fFrame->ActivateFrame(this);
	}
	else
	{
		window->Select();
		return kXMPTrue;
	}
	
	Point mousePos = event->where;
	::SetPort(window->GetPlatformWindow());
	::SetOrigin(0,0);
	::GlobalToLocal(&mousePos);
	XMPPoint windowXMPPoint(mousePos);	
	windowXMPPoint = GetWindowContentTransform()->InvertPoint(windowXMPPoint);

	return DoMouseDownInEmbeddedFrameBorder(embeddedXMPFacet,
											 windowXMPPoint, 
											 event);
}

//----------------------------------------------------------------------------------------
//	 CFacet::DoMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

XMPBoolean CFacet::DoMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, const XMPPoint& where, XMPEventData event)
{
XMPUnused(embeddedXMPFacet);
XMPUnused(where);
XMPUnused(event);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
//	 CFacet::RemoveEmbeddedFacets
//------------------------------------------------------------------------------

void CFacet::RemoveEmbeddedFacets()
{
	// ----- We need to do it in two steps
	OrderedCollection* children = new OrderedCollection;
	XMPFacetIterator* ite = GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (XMPFacet* xmpChildFacet = ite->First();
		ite->IsNotComplete();
		xmpChildFacet = ite->Next())
	{
		children->AddLast(xmpChildFacet);
	}
	delete ite;
	
	// ----- Now we can remove them
	OrderedCollectionIterator iter(children);
	for (xmpChildFacet = (XMPFacet*)iter.First();
		iter.IsNotComplete();
		xmpChildFacet =  (XMPFacet*)iter.Next() )
	{
		GetXMPFacet()->RemoveFacet(xmpChildFacet);
		delete xmpChildFacet;
	}
	
	delete children;
}

//----------------------------------------------------------------------------------------
//	 CFacet::InvalidateAllFacets
//----------------------------------------------------------------------------------------
// invalidShape is in content coordinate

void CFacet::InvalidateAllFacets(XMPShape* invalidShape, XMPBoolean skipThis)
{
	XMPShape* shape = kXMPNULL;
	if (invalidShape)
	{
		shape = ::NewXMPShape(invalidShape);
		shape->Transform(fFrame->GetXMPFrame()->GetInternalTransform()); 	// content -> frame coordinate
	}

	fFrame->GetPart()->InvalidateAllFrames(fFrame->GetIdentifier(), skipThis ? this : (CFacet*)kXMPNULL, shape);
	
	delete shape;
}

//----------------------------------------------------------------------------------------
//	 CFacet::ValidateAllFacets
//----------------------------------------------------------------------------------------
// validShape is in content coordinate

void CFacet::ValidateAllFacets(XMPShape* validShape, XMPBoolean skipThis)
{
	XMPShape* shape = kXMPNULL;
	if (validShape)
	{
		shape = ::NewXMPShape(validShape);
		shape->Transform(fFrame->GetXMPFrame()->GetInternalTransform());	// content -> frame coordinate
	}

	fFrame->GetPart()->ValidateAllFrames(fFrame->GetIdentifier(), skipThis ? this :  (CFacet*)kXMPNULL, shape);
	
	delete shape;
}

//------------------------------------------------------------------------------
//	 CFacet::GetEmbeddedXMPFacet
//------------------------------------------------------------------------------
//	Return the <first> facet belonging to proxy embedded in this 

XMPFacet* CFacet::GetEmbeddedXMPFacet(CProxyRun* proxy)
{
	XMPFacetIterator i(GetXMPFacet(), kXMPChildrenOnly, kXMPFrontToBack);

	for (XMPFacet* xmpFacet = i.First(); i.IsNotComplete(); xmpFacet = i.Next())
	{
		if (xmpFacet->GetFrame()->GetPart() == proxy->GetXMPPart())
			return xmpFacet;
	}
	
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CFacet::Invalidate
//------------------------------------------------------------------------------
// invalidShape is in frame coordinate

void CFacet::Invalidate(XMPShape *invalidShape)
{
	if (fRedraw)
		GetXMPFacet()->Invalidate(invalidShape);
}

//------------------------------------------------------------------------------
//	 CFacet::Validate
//------------------------------------------------------------------------------
// validShape is in frame coordinate

void CFacet::Validate(XMPShape *validShape)
{
	if (fRedraw)
		GetXMPFacet()->Validate(validShape);
}

//------------------------------------------------------------------------------
//	 CFacet::SetRedraw
//------------------------------------------------------------------------------

XMPBoolean CFacet::SetRedraw(XMPBoolean state)
{
	XMPBoolean temp = fRedraw;
	fRedraw = state;
	return temp;
}
