/*
	File:		FWFacet.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 4/12/94	DS		Remove external transform changed
		 <3>	 4/12/94	DS		Replace ExternalTransformChanged and
									ClipShapeChange with GeometryChanged.
		 <2>	 3/21/94	DS		Add constant to this file that was removed
									from OpenDoc. This Sucks.

	To Do:
*/

//	File:		FWFacet.h
//
//	Contains:	Definition for class 
//						 CFacet
//
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWFACET_
#define _FWFACET_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPWindow;
class XMPShape;
class CFrame;
class CProxyFrame;
class CProxyRun;
class CWindow;
class FW_CDropTracker;

//==============================================================================
//	 Constants
//==============================================================================
const	XMPValueType	kXMPApplehfs	= "Apple:OSType:Scrap:hfs ";

//==============================================================================
//	 class CFacet
//==============================================================================

#define kCFacetID "appl:cfacet$class,1.0.0"

class CFacet
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CFacet();
	void InitFacet(XMPFacet* xmpFacet);
	virtual ~CFacet();

//------------------------------------------------------------------------------
//	 New API
//
public:
	virtual void 			Draw(XMPShape* invalidShape);

	void					Invalidate(XMPShape *invalidShape);
	void					Validate(XMPShape *validShape);
	
	void					InvalidateAllFacets(XMPShape* invalidShape, XMPBoolean skipThis);
	void					ValidateAllFacets(XMPShape* validShape, XMPBoolean skipThis);
	
	// ----- Drag And Drop -----
	virtual RgnHandle 		CalcDragHiliteRgn();
	virtual XMPBoolean		DrawDragHiliteInside();
	virtual void 			DragEnter(XMPDragItemIterator* dragInfo,  
									XMPPoint where);
	virtual void 			DragWithin(XMPDragItemIterator* dragInfo,
									XMPPoint where);
	virtual void 			DragLeave(XMPPoint where);
	virtual XMPDropResult 	Drop(XMPDragItemIterator* dropInfo,
								  XMPPoint where);
	virtual FW_CDropTracker* NewDropTracker(CFacet* facet);

	// ----- Events -----
	virtual XMPBoolean 	HandleMouseDown(XMPFacet* embeddedXMPFacet, XMPEventData event);
	virtual XMPBoolean 	DoMouseDown(const XMPPoint& where,
										XMPEventData event);
	
	virtual XMPBoolean 	HandleMouseUp(XMPEventData event);
	virtual XMPBoolean 	DoMouseUp(const XMPPoint& where,
										XMPEventData event);

	virtual void			MouseEnter(const XMPPoint& where);
	virtual void			MouseWithin(const XMPPoint& where);
	virtual void			MouseLeave();

	virtual XMPBoolean		DoMouseDownInEmbeddedFrame(XMPFacet* embeddedXMPFacet, 
														const XMPPoint& where, 
														XMPEventData event);
	virtual XMPBoolean		HandleMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, 
																 XMPEventData event);
	virtual XMPBoolean		DoMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, 
															const XMPPoint& where, 
															XMPEventData event);
	// ----- Imaging -----
	virtual void			GeometryChanged();
	virtual void 			ClipEmbeddedFacets();
	virtual void 			UpdateActiveShape();


	// ----- Activation -----
	virtual void 			WindowActivated();
	virtual void			WindowDeActivated();
	
	// ----- Getters/Setters -----
	CFrame*					GetFrame() const;
	XMPShape*				GetClipShape() const;
	
	// ----- XMP/CFacet mapping -----
	static CFacet* 		XMPtoFWFacet(XMPFacet* xmpFacet);
	
	// ----- Facet Hierarchy -----
	XMPFacet* 				GetEmbeddedXMPFacet(CProxyRun* proxyRun);
	void					RemoveEmbeddedFacets();
	virtual XMPFacet*		CreateEmbeddedFacet(CProxyFrame *itsFrame, XMPShape* clipShape, XMPTransform* externalTransform, 
						   		 				XMPFacet* siblingFacet, XMPFramePosition position);
	
	// ----- Transforms -----
	XMPTransform* 			GetExternalTransform() const;
	XMPTransform* 			GetFrameTransform() const;
	XMPTransform* 			GetContentTransform() const;
	XMPTransform* 			GetWindowFrameTransform() const;
	XMPTransform* 			GetWindowContentTransform() const;
	
	// ----- Getters/Setters -----
	CWindow*				GetWindow() const;
	XMPWindow*				GetXMPWindow() const;
	XMPFacet*				GetXMPFacet() const;
	XMPBoolean				IsInFloatingWindow() const;
	XMPBoolean				GetCursorInContent() const;
	void					SetCursorInContent(XMPBoolean inContent);
	XMPBoolean				GetCanAcceptDrop() const;
	void					SetCanAcceptDrop(XMPBoolean acceptDrop);
	
	// ----- Redraw -----
	XMPBoolean				GetRedraw() const;
	XMPBoolean				SetRedraw(XMPBoolean state);
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	XMPFacet*				fXMPFacet;	
	CFrame*					fFrame;
	XMPBoolean				fRedraw;
	XMPBoolean				fCursorInContent;		// Used for drag hiliting
	XMPBoolean				fCanAcceptDrop;
	FW_CDropTracker*		fDropTracker;
};




//==============================================================================
//	 CFacet Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CFacet::GetRedraw
//------------------------------------------------------------------------------
inline XMPBoolean CFacet::GetRedraw() const
{
	return fRedraw;
}

//------------------------------------------------------------------------------
//	 CFacet::GetClipShape
//------------------------------------------------------------------------------
inline XMPShape* CFacet::GetClipShape() const
{
	return fXMPFacet->GetClipShape();
}

//------------------------------------------------------------------------------
//	 CFacet::GetFrame
//------------------------------------------------------------------------------
inline CFrame* CFacet::GetFrame() const
{
	return fFrame;
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetXMPFacet
//----------------------------------------------------------------------------------------
inline XMPFacet* CFacet::GetXMPFacet() const
{
	return fXMPFacet;
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetXMPWindow
//----------------------------------------------------------------------------------------
inline XMPWindow* CFacet::GetXMPWindow() const
{
	return fXMPFacet->GetWindow();
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetExternalTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* CFacet::GetExternalTransform() const
{
	return fXMPFacet->GetExternalTransform();
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetFrameTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* CFacet::GetFrameTransform() const
{
	return fXMPFacet->GetFrameTransform();
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetContentTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* CFacet::GetContentTransform() const
{
	return fXMPFacet->GetContentTransform();
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetWindowFrameTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* CFacet::GetWindowFrameTransform() const
{
	return fXMPFacet->GetWindowFrameTransform();
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetWindowContentTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* CFacet::GetWindowContentTransform() const
{
	return fXMPFacet->GetWindowContentTransform();
}

//----------------------------------------------------------------------------------------
//	 CFacet::IsInFloattingWindow()
//----------------------------------------------------------------------------------------
inline XMPBoolean CFacet::IsInFloatingWindow() const
{
	return GetXMPWindow()->IsFloating();
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetCursorInContent()
//----------------------------------------------------------------------------------------
inline XMPBoolean CFacet::GetCursorInContent() const
{
	return fCursorInContent;
}

//----------------------------------------------------------------------------------------
//	 CFacet::SetCursorInContent()
//----------------------------------------------------------------------------------------
inline void CFacet::SetCursorInContent(XMPBoolean inContent)
{
	fCursorInContent = inContent;
}

//----------------------------------------------------------------------------------------
//	 CFacet::GetCanAcceptDrop()
//----------------------------------------------------------------------------------------
inline XMPBoolean CFacet::GetCanAcceptDrop() const
{
	return fCanAcceptDrop;
}

//----------------------------------------------------------------------------------------
//	 CFacet::SetCanAcceptDrop()
//----------------------------------------------------------------------------------------
inline void CFacet::SetCanAcceptDrop(XMPBoolean acceptDrop)
{
	fCanAcceptDrop = acceptDrop;
}

#endif