/*
	File:		FWFrame.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h
		 <2>	 3/30/94	DS		Add NewEmbeddedFacet method

	To Do:
*/

//	File:		FWFrame.cpp
//
//	Contains:	Implementation for class 
//						 CFrame
//						 CFrameFacetIterator
//
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWWINDOW_
#include "FWWindow.h"
#endif

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _FWPART_
#include <FWPart.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _FOCUSSET_
#include <FocusSet.h>
#endif

#ifndef _FOCUSITR_
#include <FocusItr.h>
#endif

// ----- Macintosh Includes -----

#ifndef mathRoutinesIncludes
#include <math routines.h>
#endif

#pragma segment FWFrameSeg

//==============================================================================
//	 class CFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 CFrame::CFrame
//------------------------------------------------------------------------------

CFrame::CFrame()
{
	fFocusSet = kXMPNULL;
	fGetFrontClicks = kXMPTrue;
	fNeedsActivating = kXMPFalse;
	fActiveFacet = kXMPNULL;
	fCanBeActiveFrame = kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CFrame::IFrame
//
//	Should be called FIRST in CYourFrame::IYourFrame
//------------------------------------------------------------------------------

void CFrame::IFrame(XMPFrame* xmpFrame, CPart* part)
{
	IBaseFrame(xmpFrame);
	SetNextEventHandler(part);
	SetIdentifier(xmpFrame->GetPresentation());
	
	fFocusSet = new XMPFocusSet;
	fFocusSet->InitFocusSet();
	
	xmpFrame->SetPartInfo((XMPInfoType)this);
	
	fNeedsActivating = IsRoot();
}

//------------------------------------------------------------------------------
//	 CFrame::~CFrame
//------------------------------------------------------------------------------

CFrame::~CFrame()
{
	XMPFrame *xmpFrame = GetXMPFrame();
	if (xmpFrame)
		xmpFrame->SetPartInfo((XMPInfoType)kXMPNULL);
	
	delete fFocusSet;
	fFocusSet = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CFrame::CloseDisplayFrame
//
//	When overridden, call inherited::CloseDisplayFrame LAST
//------------------------------------------------------------------------------

void CFrame::CloseDisplayFrame()
{
	// Relinquish what ever focus we have
	RelinquishFocusSet();
}

//------------------------------------------------------------------------------
//	 CFrame::RemoveDisplayFrame
//
//	When overridden, call inherited::RemoveDisplayFrame LAST
//------------------------------------------------------------------------------

void CFrame::RemoveDisplayFrame()
{
	// Relinquish what ever focus we have
	RelinquishFocusSet();
}

//------------------------------------------------------------------------------
//	 CFrame::AddedToDisplayList
//------------------------------------------------------------------------------

void CFrame::AddedToDisplayList()
{
}

//-------------------------------------------------------------------------
//	 CFrame::HandleKeyDown
//-------------------------------------------------------------------------

XMPBoolean CFrame::HandleKeyDown(XMPEventData event)
{
XMPUnused(event);
	
	return kXMPFalse;
}

//-------------------------------------------------------------------------
//	 CFrame::DoMenuEvent
//-------------------------------------------------------------------------

XMPBoolean CFrame::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	XMPBoolean result = kXMPTrue;
	FW_CSelection* selection = GetPart()->GetSelection();
	
	switch (commandID)
	{
		case kXMPCommandViewAsWin:
			GetPart()->Open(this->GetXMPFrame());
			break;
		
		case kXMPCommandCut:
			if (selection && selection->DoCut(this))
				GetPart()->Changed();
			break;
			
		case kXMPCommandCopy:
			if (selection)
				selection->DoCopy(this);
			break;
		
		default:
			result = GetPart()->DoMenuEvent(menuBar, commandID);	
	}
			
	return result;
}

//-------------------------------------------------------------------------
//	 CFrame::DoIdle
//-------------------------------------------------------------------------

XMPBoolean CFrame::DoIdle()
{
	return GetPart()->DoIdle();
}

//-------------------------------------------------------------------------
//	 CFrame::InstallMenus
//
// 	By default call the part with a null menuBar
//-------------------------------------------------------------------------

void CFrame::InstallMenus()
{
	GetPart()->InstallMenus(kXMPNULL);	// will install the part menubar
}

//-------------------------------------------------------------------------
//	CFrame::DoAdjustMenus
//
//	When overridden, call inherited::DoAdjustMenus FIRST
//-------------------------------------------------------------------------

void CFrame::DoAdjustMenus(XMPMenuBar* menuBar)
{
	GetPart()->DoAdjustMenus(menuBar);

	menuBar->EnableCommand(kXMPCommandViewAsWin, !IsRoot());
}

//-------------------------------------------------------------------------
//	 CFrame::FocusStateChanged
//
//	When overridden, call inherited::FocusStateChanged FIRST
//-------------------------------------------------------------------------

void CFrame::FocusStateChanged(XMPTypeToken focus, XMPBoolean newState)
{
	CPart *itsPart = GetPart();
	if (newState)
	{
		if (focus == itsPart->GetSelectionFocusToken())
		{
			Activate();
			itsPart->SetActiveFrame(this);
		}
		if (focus == itsPart->GetMenuFocusToken())
		{
			InstallMenus();
		}
	}
	else
	{
		if (focus == itsPart->GetSelectionFocusToken())
		{
			Deactivate();
			itsPart->SetActiveFrame(kXMPNULL);
		}
	}
}

//-------------------------------------------------------------------------
// 	CFrame::ActivateFrame
//-------------------------------------------------------------------------
//	Should not need to be override. Override instead FocusStateChanged.

XMPBoolean CFrame::ActivateFrame(CFacet* activatedFacet)
{ 
	XMPBoolean result = kXMPTrue;
	
	SetActiveFacet(activatedFacet);
	
	if (!IsActive())
	{
		if (RequestFocusSet())
		{
			XMPFocusSetIterator* ite = fFocusSet->CreateIterator();
			for (XMPTypeToken token = ite->First(); ite->IsNotComplete(); token = ite->Next())
				GetPart()->FocusAcquired(token, GetXMPFrame());
			delete ite;
		}
		else
		{
			result = kXMPFalse;	//	 Couln't activate it
		}
	}
	
	return result;
}

//-------------------------------------------------------------------------
//	 CFrame::DeactivateFrame
//-------------------------------------------------------------------------
//	Should not need to be override. Override instead FocusStateChanged.

void CFrame::DeactivateFrame()
{
	if (IsActive())
	{
		RelinquishFocusSet();
		
		XMPFocusSetIterator* ite = fFocusSet->CreateIterator();
		for (XMPTypeToken token = ite->First(); ite->IsNotComplete(); token = ite->Next())
			GetPart()->FocusLost(token, GetXMPFrame());
		delete ite;
	}
}

//-------------------------------------------------------------------------
//	 CFrame::XMPtoFWFrame
//-------------------------------------------------------------------------

CFrame* CFrame::XMPtoFWFrame(XMPFrame* xmpFrame)
{
	return xmpFrame ? (CFrame*)xmpFrame->GetPartInfo() : (CFrame*)kXMPNULL;
}

//------------------------------------------------------------------------------
// 	CFrame::FrameShapeChanged
//
//	When overridden, call inherited::FrameShapeChanged
//------------------------------------------------------------------------------

void CFrame::FrameShapeChanged()
{
	UpdateUsedAndActiveShapes();
}

//----------------------------------------------------------------------------------------
//	 CFrame::SetActiveFacet
//----------------------------------------------------------------------------------------
void CFrame::SetActiveFacet(CFacet*  activeFacet)
{
	fActiveFacet = activeFacet;
}

//------------------------------------------------------------------------------
//	 CFrame::UpdateUsedAndActiveShapes
//------------------------------------------------------------------------------

void CFrame::UpdateUsedAndActiveShapes()
{	
	UpdateUsedShape();

	CFrameFacetIterator i(this);		
	for (CFacet* facet = i.First(); i.IsNotComplete(); facet = i.Next())
		facet->UpdateActiveShape();
}

//------------------------------------------------------------------------------
//	 CFrame::UpdateUsedShape
//------------------------------------------------------------------------------

void CFrame::UpdateUsedShape()
{	
	XMPShape* scratch = ::NewXMPShape();
	GetFrameShape(scratch);
	this->GetXMPFrame()->ChangeUsedShape(scratch);
}

//------------------------------------------------------------------------------
//	 CFrame::AddToFocusSet
//------------------------------------------------------------------------------

void CFrame::AddToFocusSet(XMPTypeToken focus)
{
	fFocusSet->Add(focus);
}

//------------------------------------------------------------------------------
//	 CFrame::RemoveFromFocusSet
//------------------------------------------------------------------------------

void CFrame::RemoveFromFocusSet(XMPTypeToken focus)
{
	fFocusSet->Remove(focus);
}

//------------------------------------------------------------------------------
//	 CFrame::RelinquishFocusSet
//
//	Just for practical purpose (this is a private method)
//------------------------------------------------------------------------------

void CFrame::RelinquishFocusSet()
{
	GetPart()->GetSession()->GetArbitrator()->RelinquishFocusSet(fFocusSet, GetXMPFrame());
}

//------------------------------------------------------------------------------
//	 CFrame::RequestFocusSet
//
//	Just for practical purpose (this is a private method)
//------------------------------------------------------------------------------

XMPBoolean CFrame::RequestFocusSet()
{
	return GetPart()->GetSession()->GetArbitrator()->RequestFocusSet(fFocusSet, GetXMPFrame());
}

//------------------------------------------------------------------------------
// 	CFrame::NewEmbeddedFacet
//
//	Override to support nested facets
//------------------------------------------------------------------------------

CFacet* CFrame::NewEmbeddedFacet(XMPFacet* xmpFacet)
{
	// You must override this method if you ( or someone else ) calls CreateEmbeddedFacet
	// on a facet contained in your derivative of CFrame. 
	
	WASSERTM(false, "This method must be implemented.");
	
	return kXMPNULL;
}

//------------------------------------------------------------------------------
// 	CFrame::FacetAdded
//
//	When overridden call inherited::FacetAdded FIRST
//------------------------------------------------------------------------------

void CFrame::FacetAdded(CFacet* facet)
{
	// Install the frame with the newest facet as the active frame
	if (this->CanBeActiveFrame())
		this->GetPart()->SetActiveFrame(this);
		
	// Install the newest facet as the active facet
	this->SetActiveFacet(facet);
}

//------------------------------------------------------------------------------
// 	CFrame::FacetRemoved
//
//	When overridden call inherited::FacetAdded LAST
//------------------------------------------------------------------------------

void CFrame::FacetRemoved(CFacet* facet)
{
	if (this->GetActiveFacet() == facet)
	{
		CFrameFacetIterator iter(this);
		this->SetActiveFacet(iter.First());
	}
}

//------------------------------------------------------------------------------
//	 CFrame::CreateChildWindow
//------------------------------------------------------------------------------

XMPWindow* CFrame::CreateChildWindow(XMPISOStr windowTitle)
{
	XMPRect windRect(ff(100), ff(100), ff(300), ff(300));
	
	return GetPart()->CreateXMPWindow(this,
									 GetPart()->GetSession()->Tokenize(kXMPViewAsFrame),
									 GetPresentation(),
									 windowTitle,
									 windRect,
									 kXMPTrue,		// close box
									 kXMPTrue,		// resizable
									 kXMPFalse,		// floating
									 kXMPTrue,		// should save
									 zoomDocProc);
}

//-------------------------------------------------------------------------
//	 CFrame::InvalidateAllFacets
//-------------------------------------------------------------------------
//	invalidateShape must be in frame coordinate

void CFrame::InvalidateAllFacets(CFacet* skipFacet, XMPShape* invalidShape)
{
	CFrameFacetIterator ite(this);
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		if (facet != skipFacet)
			facet->Invalidate(invalidShape);
	}
}

//-------------------------------------------------------------------------
//	 CFrame::ValidateAllFacets
//-------------------------------------------------------------------------
//	validShape must be in frame coordinate

void CFrame::ValidateAllFacets(CFacet* skipFacet, XMPShape* validShape)
{
	CFrameFacetIterator ite(this);
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		if (facet != skipFacet)
			facet->Validate(validShape);
	}
}

//-------------------------------------------------------------------------
//	 CFrame::Invalidate
//-------------------------------------------------------------------------
//	invalidateShape must be in frame coordinate

void CFrame::Invalidate(XMPShape* invalidShape)
{
	CFrameFacetIterator ite(this);
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		facet->Invalidate(invalidShape);
	}
}

//-------------------------------------------------------------------------
//	 CFrame::Validate
//-------------------------------------------------------------------------
//	validShape must be in frame coordinate

void CFrame::Validate(XMPShape* validShape)
{
	CFrameFacetIterator ite(this);
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		facet->Validate(validShape);
	}
}

//----------------------------------------------------------------------------------------
//	 CFrame::HasSelectionFocus()
//----------------------------------------------------------------------------------------

XMPBoolean CFrame::HasSelectionFocus() const
{	
	return GetXMPFrame() == GetPart()->GetSession()->GetArbitrator()->GetFocusOwner(GetPart()->GetSelectionFocusToken());
}

//----------------------------------------------------------------------------------------
//	 CFrame::ViewTypeChanged
//----------------------------------------------------------------------------------------

void CFrame::ViewTypeChanged()
{	
}

//----------------------------------------------------------------------------------------
//	 CFrame::PresentationChanged()
//----------------------------------------------------------------------------------------

void CFrame::PresentationChanged()
{	
	SetIdentifier(GetPresentation());
}

//-------------------------------------------------------------------------
//	 CFrame::ChangeSelectionState
//-------------------------------------------------------------------------

void CFrame::ChangeSelectionState(SelectionState state)
{
	SelectionStateChanged(fSelectionState, state);
	fSelectionState = state;
}

//-------------------------------------------------------------------------
//	 CFrame::SelectionStateChanged
//-------------------------------------------------------------------------

void CFrame::SelectionStateChanged(SelectionState oldState, SelectionState newState)
{
XMPUnused(oldState);
XMPUnused(newState);
}

//==============================================================================
//	 class CFrameFacetIterator
//==============================================================================

//-------------------------------------------------------------------------
//	 CFrameFacetIterator::CFrameFacetIterator
//-------------------------------------------------------------------------

CFrameFacetIterator::CFrameFacetIterator(CFrame* frame):
	fIte(frame->GetXMPFrame())
{
}

//-------------------------------------------------------------------------
//	 CFrameFacetIterator::~CFrameFacetIterator
//-------------------------------------------------------------------------

CFrameFacetIterator::~CFrameFacetIterator()
{
}

//-------------------------------------------------------------------------
//	 CFrameFacetIterator::First
//-------------------------------------------------------------------------

CFacet* CFrameFacetIterator::First()
{
	return CFacet::XMPtoFWFacet(fIte.First());
}

//-------------------------------------------------------------------------
//	 CFrameFacetIterator::Next
//-------------------------------------------------------------------------

CFacet* CFrameFacetIterator::Next()
{
	return CFacet::XMPtoFWFacet(fIte.Next());
}
