/*
	File:		FWFrame.h

	Contains:	Definition for class 
						 CFrame
						 CFrameFacetIterator

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 3/30/94	DS		Add NewEmbeddedFacet method
		<1+>	 3/17/94	DS		Add CDECL to CFrame dtor

	To Do:
*/


#ifndef _FWFRAME_
#define _FWFRAME_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _FWBASEFR_
#include "FWBaseFr.h"
#endif

#ifndef _FWEVENTH_
#include "FWEventH.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

//==============================================================================
//	 Forward Declaration
//==============================================================================

class CPart;
class CFacet;
class XMPFacet;
class XMPFocusSet;
class XMPMenuBar;




//==============================================================================
//	 Defines
//==============================================================================

#define kNoSelection 0
#define kInactiveSelection 1
#define kActiveSelection 2




//==============================================================================
//	 Type Definitions
//==============================================================================

typedef XMPUShort SelectionState;




//==============================================================================
//	 class CFrame
//==============================================================================

#define kCFrameID "appl:cframe$class,1.0.0"

class CFrame : public CBaseFrame, public MEventHandler
{
//------------------------------------------------------------------------------
//	 New API
//
public:
	CFrame();
	void IFrame(XMPFrame* frame, CPart* part);
	virtual ~ CDECL CFrame();

//------------------------------------------------------------------------------
//	 New API
//
public:
	static CFrame* XMPtoFWFrame(XMPFrame* xmpFrame);

	// ----- Events -----
	virtual XMPBoolean 	HandleKeyDown(XMPEventData event);
	virtual XMPBoolean 	DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	virtual XMPBoolean 	DoIdle();

	// ----- Facets -----
	virtual CFacet* 		NewEmbeddedFacet(XMPFacet* xmpFacet);
		//	Same as NewFacet. Wait for FacetAdded. This an optional method to
		//	derive from if you wish to support nested facets ( necessary for scrolling ).
	
	virtual CFacet* 		NewFacet(XMPFacet* xmpFacet) = 0;
		// 	In NewFacet just create your subclass of CFacet. Wait for 
		//	FacetAdded before doing anything else.
		
	virtual void 			FacetAdded(CFacet* facet);
	virtual void 			FacetRemoved(CFacet* facet);
		
	void					InvalidateAllFacets(CFacet* skipFacet, XMPShape* invalidShape);
	void					ValidateAllFacets(CFacet* skipFacet, XMPShape* validShape);

	virtual void			Invalidate(XMPShape* invalidShape);
	virtual void			Validate(XMPShape* validShape);

	// ----- Add/Remove frame notification -----
	virtual void			AddedToDisplayList();
	virtual void			CloseDisplayFrame();
	virtual void			RemoveDisplayFrame();
	
	// ----- Focus Set -----
	XMPFocusSet*			GetFocusSet() const;
	void 					AddToFocusSet(XMPTypeToken focus);	
	void 					RemoveFromFocusSet(XMPTypeToken focus);	
	
	// ----- Menus -----
	virtual void			DoAdjustMenus(XMPMenuBar* menuBar);
	virtual void			InstallMenus();
	
	// ----- Focus -----
	virtual void 			FocusStateChanged(XMPTypeToken focus, XMPBoolean newState);
	virtual XMPBoolean 	ActivateFrame(CFacet* activatedFacet);
	virtual void			DeactivateFrame();
	XMPBoolean				HasSelectionFocus() const;
	
	// ----- ViewType/Presentation -----
	virtual void 			ViewTypeChanged();
	virtual void 			PresentationChanged();

	// ----- Frame Negotiation -----
	virtual void 			FrameShapeChanged();
	XMPShape*				RequestFrameShape(XMPShape* shape);
	
	virtual void 			UpdateUsedAndActiveShapes();
	virtual void 			UpdateUsedShape();

	// ----- Persistence -----
	virtual XMPStorageUnit*	GetXMPStorageUnit() const;
	
	// ----- Window -----
	virtual XMPWindow*		CreateChildWindow(XMPISOStr windowTitle);
	
	// ----- Selection State -----
	void 					ChangeSelectionState(SelectionState state);
	virtual void			SelectionStateChanged(SelectionState oldState, SelectionState newState);
	SelectionState			GetSelectionState() const;
	
	// ----- Getter/Setter -----
	CPart* 					GetPart() const;
	XMPBoolean				GetFrontClicks() const;
	void					SetFrontClicks(XMPBoolean getFrontClicks);
	XMPTransform* 			GetInternalTransform() const;
	CFacet* 				GetActiveFacet() const;
	virtual void			SetActiveFacet(CFacet* activeFacet);
	XMPBoolean				NeedsActivating() const;
	void					SetNeedsActivating(XMPBoolean state);
	XMPBoolean				CanBeActiveFrame() const;
	void					SetCanBeActiveFrame(XMPBoolean beActive);
	
protected:
	// ----- Relinquish/Request Focus Set -----
	void					RelinquishFocusSet();
	XMPBoolean				RequestFocusSet();
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	XMPFocusSet* 			fFocusSet;
	XMPBoolean				fGetFrontClicks;
	XMPBoolean				fNeedsActivating;	// should this be activated when its window is activated?
	CFacet*					fActiveFacet;
	SelectionState			fSelectionState;
	XMPBoolean				fCanBeActiveFrame;
};

//========================================================================================
//	 CFrame Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	 CFrame::RequestFrameShape()
//----------------------------------------------------------------------------------------
inline XMPShape* CFrame::RequestFrameShape(XMPShape* shape)
{
	return GetXMPFrame()->RequestFrameShape(shape);
}

//----------------------------------------------------------------------------------------
//	 CFrame::GetInternalTransform()
//----------------------------------------------------------------------------------------
inline XMPTransform* CFrame::GetInternalTransform() const
{
	return GetXMPFrame()->GetInternalTransform();
}

//----------------------------------------------------------------------------------------
//	 CFrame::GetFrontClicks()
//----------------------------------------------------------------------------------------
inline XMPBoolean CFrame::GetFrontClicks() const
{
	return fGetFrontClicks;
}

//----------------------------------------------------------------------------------------
//	 CFrame::SetFrontClicks()
//----------------------------------------------------------------------------------------
inline void CFrame::SetFrontClicks(XMPBoolean getFrontClicks)
{
	fGetFrontClicks = getFrontClicks;
}

//----------------------------------------------------------------------------------------
//	 CFrame::GetPart()
//----------------------------------------------------------------------------------------
inline CPart* CFrame::GetPart() const
{
	return (CPart*)GetXMPFrame()->GetPart();
}

//----------------------------------------------------------------------------------------
//	 CFrame::GetFocusSet()
//----------------------------------------------------------------------------------------
inline XMPFocusSet* CFrame::GetFocusSet() const
{
	return fFocusSet;
}

//----------------------------------------------------------------------------------------
//	 CFrame::GetXMPStorageUnit
//----------------------------------------------------------------------------------------

inline XMPStorageUnit* CFrame::GetXMPStorageUnit() const
{
	return GetXMPFrame()->GetStorageUnit();
}

//----------------------------------------------------------------------------------------
//	 CFrame::GetActiveFacet
//----------------------------------------------------------------------------------------
inline CFacet* CFrame::GetActiveFacet() const
{
	return fActiveFacet;
}

//----------------------------------------------------------------------------------------
//	 CFrame::CanBeActiveFrame
//----------------------------------------------------------------------------------------
inline XMPBoolean CFrame::CanBeActiveFrame() const
{
	return fCanBeActiveFrame;
}

//----------------------------------------------------------------------------------------
//	 CFrame::SetCanBeActiveFrame
//----------------------------------------------------------------------------------------
inline void CFrame::SetCanBeActiveFrame(XMPBoolean beActive)
{
	fCanBeActiveFrame = beActive;
}

//----------------------------------------------------------------------------------------
//	 CFrame::GetSelectionState
//----------------------------------------------------------------------------------------
inline SelectionState CFrame::GetSelectionState() const
{
	return fSelectionState;
}

//----------------------------------------------------------------------------------------
//	 CFrame::NeedsActivating
//----------------------------------------------------------------------------------------
inline XMPBoolean CFrame::NeedsActivating() const
{
	return fNeedsActivating;
}

//----------------------------------------------------------------------------------------
//	 CFrame::SetNeedsActivating
//----------------------------------------------------------------------------------------
inline void CFrame::SetNeedsActivating(XMPBoolean state)
{
	fNeedsActivating = state;
}




//========================================================================================
//	 class CFrameFacetIterator
//========================================================================================

#define kCFrameFacetIteratorID "appl:cframefacetiterator$class,1.0.0"

class CFrameFacetIterator
{

public:
	CFrameFacetIterator(CFrame* frame);

	virtual ~CFrameFacetIterator();

	CFacet* First();
		// Returns the first visible facet
	
	CFacet* Next();
		// Returns the next visible facet

	XMPBoolean IsNotComplete()
				{return fIte.IsNotComplete();}
		// Returns TRUE as long as there are more facets

private:
	XMPFrameFacetIterator 	fIte;
};

#endif

