/*
	File:		FWFrming.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 4/11/94	DS		Remove frameGroupID parameter from call to
									CreateFrame as per bug 1155480 and example
									code found in Drawpart
		 <3>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h
		<1+>	 3/17/94	DS		Make the constructor for the iterator for
									this file take CEmbeddingFrame * as a const
									parameter. SC seems to care about this. It
									compiles without it under CFront just fine.

	To Do:
*/

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _FWPARTING_
#include "FWParting.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _INFO_
#include "Info.h"
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _FWPART_
#include <FWPart.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

// ----- Macintosh Includes

#ifndef mathRoutinesIncludes
#include <math routines.h>				// for ff()
#endif

#pragma segment FWFrameSeg

//==============================================================================
//	 class CEmbeddingFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::CEmbeddingFrame
//------------------------------------------------------------------------------

CEmbeddingFrame::CEmbeddingFrame()
{
	fProxyFrameList = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::IEmbeddingFrame
//------------------------------------------------------------------------------

void CEmbeddingFrame::IEmbeddingFrame(XMPFrame* xmpFrame, 
										CPart* part)
{
	IFrame(xmpFrame, part);
	fProxyFrameList = new OrderedCollection;
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::~CEmbeddingFrame
//------------------------------------------------------------------------------

CEmbeddingFrame::~CEmbeddingFrame()
{
	if (fProxyFrameList)
	{
		fProxyFrameList->DeleteAll();
		delete fProxyFrameList;
	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::CreateEmbeddedFrame
//------------------------------------------------------------------------------

CProxyFrame* CEmbeddingFrame::CreateEmbeddedFrame(XMPPart *embeddedXMPPart,
												 CProxyRun* proxyRun,
												 XMPShape* frameShape,												
												 XMPTypeToken viewType,
												 XMPTypeToken presentation,
												 XMPID frameGroupID,
												 XMPBoolean isOverlaid)
{
	XMPStorageUnit* su = GetPart()->GetStorageUnit();
		
	// ----- create the new frame
	XMPFrame* embeddedXMPFrame = su->GetDraft()->CreateFrame(GetXMPFrame(), frameShape, embeddedXMPPart, 
															viewType, presentation, // frameGroupID, ???? DCS a5 change 
															kXMPFalse, isOverlaid);
		
	// ----- Create the proxy Frame
	CProxyFrame *proxyFrame = proxyRun->NewProxyFrame(embeddedXMPFrame);
	this->AddProxyFrame(proxyFrame);
	
	embeddedXMPFrame->Release();	// Release one of the references since CreateFrame gives
									// you a frame with a reference count of one instead of zero.
	
	return proxyFrame;
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::AdddToDisplayList
//------------------------------------------------------------------------------

void CEmbeddingFrame::AddedToDisplayList()
{
	// ----- If it is the first display frame look if we don't have orphan embeddedframes
	if (GetPart()->CountDisplayFrame() == 1)
	{
		((CEmbeddingPart*)GetPart())->SetEmbeddedFramesContainingFrame(this);
	}
	
	// ----- Call inherited -----
	CFrame::AddedToDisplayList();
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::AttachSourceFrame
//------------------------------------------------------------------------------
//	CEmbeddingFrame::AttachSourceFrame won't be called if sourceFrame == kXMPNULL

void CEmbeddingFrame::AttachSourceFrame(CEmbeddingFrame* sourceFrame)
{
	// ----- Duplicate all the embedded frames of sourceFrame -----
	// ----- CreateEmbeddedFrame doesn't create facets, wait for FacetAdded to duplicate my facet -----
	CEmbeddingFrameCProxyFrameIterator iter(sourceFrame);
	for (CProxyFrame* proxyFrame = iter.First();
		 iter.IsNotComplete();
		 proxyFrame = iter.Next())
	{
		XMPShape* frameShape = ::NewXMPShape();
		proxyFrame->GetFrameShape(frameShape);
		
		XMPPart* embeddedXMPPart = proxyFrame->GetXMPPart();
		CProxyRun* proxyRun = proxyFrame->GetProxyRun();
		
		// ----- Create the embedded Frame (proxyFrame) -----
		CProxyFrame* newProxyFrame = CreateEmbeddedFrame(embeddedXMPPart,
														proxyRun,
														frameShape,
														proxyFrame->GetViewType(),
														proxyFrame->GetPresentation(),
														proxyFrame->GetFrameGroup(),
														proxyFrame->IsOverlaid());

		// ----- Add the embedded frame to the embedded part -----
		embeddedXMPPart->AttachSourceFrame(newProxyFrame->GetXMPFrame(), proxyFrame->GetXMPFrame());
	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::RemoveEmbeddedFrame
//------------------------------------------------------------------------------

void CEmbeddingFrame::RemoveEmbeddedFrame(CProxyFrame* proxyFrame)
{
	XMPFrame *xmpEmbeddedFrame = proxyFrame->GetXMPFrame();
	
	// ----- Increment the refcount so at the end we can call Remove -----
	xmpEmbeddedFrame->IncrementRefCount();
	
	// ----- Delete its facet -----
	CFrameFacetIterator ite(this);
	OrderedCollection* tempCollect = new OrderedCollection;
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		XMPFacetIterator ite2(facet->GetXMPFacet(), kXMPChildrenOnly, kXMPFrontToBack);
		for (XMPFacet *xmpFacet = ite2.First(); ite2.IsNotComplete(); xmpFacet = ite2.Next())
		{
			if (xmpFacet->GetFrame() == xmpEmbeddedFrame)
			{
				tempCollect->AddLast((ElementType)xmpFacet);
			}
		}
		OrderedCollectionIterator ite3(tempCollect);
		for (XMPFacet* xmpFacet2 = (XMPFacet*)ite3.First(); ite3.IsNotComplete(); xmpFacet2 = (XMPFacet*)ite3.Next())
		{
			facet->GetXMPFacet()->RemoveFacet(xmpFacet2);
			delete xmpFacet2;
		}
		
		tempCollect->RemoveAll();
	}
	delete tempCollect;
	
	// ----- Remove it from the list -----
	this->RemoveProxyFrame(proxyFrame);
	delete proxyFrame;
		
	// ----- Tell the frame to remove itself -----
	// ----- Note: The frame will be automatically -----
	// ----- removed from the draft if its refcount is 1 -----
	xmpEmbeddedFrame->Remove();
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::FrameShapeChanged
//------------------------------------------------------------------------------
//	When overridden, call inherited::FrameShapeChanged

void CEmbeddingFrame::FrameShapeChanged()
{
	CFrame::FrameShapeChanged();
	ClipEmbeddedFrames();
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::UsedShapeChanged
//------------------------------------------------------------------------------

void CEmbeddingFrame::UsedShapeChanged(CProxyRun* proxyRun, XMPFrame* embeddedXMPFrame)
{
XMPUnused(proxyRun);
XMPUnused(embeddedXMPFrame);

	ClipEmbeddedFrames();
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::XMPtoProxyFrame
//------------------------------------------------------------------------------

CProxyFrame* CEmbeddingFrame::XMPtoProxyFrame(XMPFrame* xmpFrame) const
{
	CEmbeddingFrameCProxyFrameIterator ite(this);
	for (CProxyFrame* proxy = ite.First(); ite.IsNotComplete(); proxy = ite.Next())
	{
		if (proxy->GetXMPFrame() == xmpFrame)
			return proxy;
	}
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::ClipEmbeddedFrames
//------------------------------------------------------------------------------

void CEmbeddingFrame::ClipEmbeddedFrames()
{
	CFrameFacetIterator iter(this);
	for (CFacet* facet = iter.First(); iter.IsNotComplete(); facet = iter.Next())
	{
		facet->ClipEmbeddedFacets();
	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::CloseDisplayFrame
//------------------------------------------------------------------------------

void CEmbeddingFrame::CloseDisplayFrame()
{
	// ----- Close all my embedded frames -----
	CProxyFrame* proxyFrame;
	while ((proxyFrame = (CProxyFrame*)fProxyFrameList->First()) != kXMPNULL) {
		proxyFrame->GetXMPFrame()->Close();
		this->RemoveProxyFrame(proxyFrame);
		delete proxyFrame;
	}

	// ----- Call inherited -----
	CFrame::CloseDisplayFrame();
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::RemoveDisplayFrame
//------------------------------------------------------------------------------

void CEmbeddingFrame::RemoveDisplayFrame()
{
	// ----- Remove all my embedded frames -----
	CProxyFrame* proxyFrame;
	while ((proxyFrame = (CProxyFrame*)fProxyFrameList->First()) != kXMPNULL) {
		XMPFrame *embeddedXMPFrame = proxyFrame->GetXMPFrame();
		
		// ----- Increment the refcount to be able to call XMPFrame->Remove() -----
		embeddedXMPFrame->IncrementRefCount();
		
		// ----- Remove it from the list of proxyFrame -----
		this->RemoveProxyFrame(proxyFrame);
		delete proxyFrame;

		// ----- Remove it from the draft -----
		embeddedXMPFrame->Remove();
	}

	// ----- Call inherited -----
	CFrame::RemoveDisplayFrame();
}


//------------------------------------------------------------------------------
//	 CEmbeddingFrame::FacetAdded
//------------------------------------------------------------------------------
//	When overridden call inherited::FacetAdded FIRST

void CEmbeddingFrame::FacetAdded(CFacet* facet)
{
	// ----- Call inherited first
	CFrame::FacetAdded(facet);
	
	CEmbeddingPartProxyRunIterator ite((CEmbeddingPart*)GetPart());
	for (CProxyRun* proxyRun = ite.First(); ite.IsNotComplete(); proxyRun = ite.Next())
	{
		CProxyRunCProxyFrameIterator ite2(proxyRun);
		for (CProxyFrame* proxyFrame = ite2.First();  ite2.IsNotComplete(); proxyFrame = ite2.Next())
		{
			if (proxyFrame->GetContainingFrame() == this)
			{
				XMPShape* clipShape = ::NewXMPShape();
				proxyFrame->GetFrameShape(clipShape);
				XMPTransform* externalTransform = ::NewXMPTransform();
				proxyRun->GetFacetExternalTransform(proxyFrame->GetXMPFrame(), externalTransform);
				facet->CreateEmbeddedFacet(proxyFrame,
										   clipShape, externalTransform,
											kXMPNULL, kXMPFrameBehind);
			}
		}
	}
		
	// ----- Recalculate the clip of my embedded facets -----
	facet->ClipEmbeddedFacets();
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::FacetRemoved
//------------------------------------------------------------------------------
//	When overridden call inherited::FacetRemoved LAST

void CEmbeddingFrame::FacetRemoved(CFacet* facet)
{
	// ----- Remove the child facets of my facet
	facet->RemoveEmbeddedFacets();
	
	// ----- Call inherited
	CFrame::FacetRemoved(facet);
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::DoMenuEvent
//------------------------------------------------------------------------------

XMPBoolean CEmbeddingFrame::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	if (commandID == kXMPCommandGetPartInfo)
	{
		ShowPartInfo();
		return kXMPTrue;
	}
			
	return CFrame::DoMenuEvent(menuBar, commandID);
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::ShowPartInfo
//------------------------------------------------------------------------------

void CEmbeddingFrame::ShowPartInfo()
{
	CEmbeddingFrameCProxyFrameIterator ite(this);
	for (CProxyFrame* proxyFrame = ite.First(); ite.IsNotComplete(); proxyFrame = ite.Next())
	{
		if (proxyFrame->GetProxyRun()->IsSelected())
			::ShowPartFrameInfo(proxyFrame->GetXMPFrame());
	}		
}

//==============================================================================
//	 CEmbeddingFrameCProxyFrameIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddingFrameCProxyFrameIterator::CEmbeddingFrameCProxyFrameIterator
//------------------------------------------------------------------------------

CEmbeddingFrameCProxyFrameIterator::CEmbeddingFrameCProxyFrameIterator(const CEmbeddingFrame* embeddingFrame) :
	fIterator(embeddingFrame->fProxyFrameList)
{
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrameCProxyFrameIterator::~CEmbeddingFrameCProxyFrameIterator
//------------------------------------------------------------------------------

CEmbeddingFrameCProxyFrameIterator::~CEmbeddingFrameCProxyFrameIterator()
{
}
