/*
	File:		FWFrming.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<1+>	 3/17/94	DS		Add CDECL to Embedding frame dtor

	To Do:
*/


#ifndef _FWFRMING_
#define _FWFRMING_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class CPart;
class CProxyFrame;
class CProxyRun;

//==============================================================================
//	 class CEmbeddingFrame
//==============================================================================

class CEmbeddingFrame : public CFrame
{
	friend CEmbeddingFrameCProxyFrameIterator;
	
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CEmbeddingFrame();
	void IEmbeddingFrame(XMPFrame* xmpFrame, CPart* part);
	virtual ~ CDECL CEmbeddingFrame();
	
//------------------------------------------------------------------------------
//	 New API
//
public:
	// ----- Facets -----
	virtual void 			FacetAdded(CFacet* facet);
	virtual void 			FacetRemoved(CFacet* facet);

	// ----- Frame Negotiation -----
	virtual void			FrameShapeChanged();
	virtual void			UsedShapeChanged(CProxyRun* proxyRun, XMPFrame* embeddedXMPFrame);

	// ----- Frame Hierarchy -----
	void					AttachSourceFrame(CEmbeddingFrame* sourceFrame);
	void					RemoveEmbeddedFrame(CProxyFrame* proxyFrame);	
	CProxyFrame*			CreateEmbeddedFrame(XMPPart *embeddedXMPPart,
												 CProxyRun* proxyRun,
												XMPShape* frameShape,											
												XMPTypeToken viewType,
												XMPTypeToken presentation,
												XMPID frameGroupID,
												XMPBoolean isOverlaid);
	CProxyFrame*			XMPtoProxyFrame(XMPFrame* xmpFrame) const;
	
	virtual void			AddedToDisplayList();
	virtual void			CloseDisplayFrame();
	virtual void			RemoveDisplayFrame();
	
	// ----- Menu Events -----
	virtual XMPBoolean		DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);

	// ----- Part Info -----
	void					ShowPartInfo();

	// ----- Implementation methods
	void					ClipEmbeddedFrames();

	// ----- Embedded frames -----
	void					AddProxyFrame(CProxyFrame* proxyFrame);
	void					RemoveProxyFrame(CProxyFrame* proxyFrame);
	XMPULong				CountEmbeddedFrames() const;
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	OrderedCollection 		*fProxyFrameList;
};

//==============================================================================
//	 Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::CountEmbeddedFrames
//------------------------------------------------------------------------------
inline XMPULong CEmbeddingFrame::CountEmbeddedFrames() const
{
	return fProxyFrameList->Count();
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::AddProxyFrame
//------------------------------------------------------------------------------
inline void CEmbeddingFrame::AddProxyFrame(CProxyFrame* proxyFrame)
{
	fProxyFrameList->AddLast(proxyFrame);
}

//------------------------------------------------------------------------------
//	 CEmbeddingFrame::RemoveProxyFrame
//------------------------------------------------------------------------------
inline void CEmbeddingFrame::RemoveProxyFrame(CProxyFrame* proxyFrame)
{
	fProxyFrameList->Remove(proxyFrame);
}

//==============================================================================
//	 CEmbeddingFrameCProxyFrameIterator
//==============================================================================

class CEmbeddingFrameCProxyFrameIterator
{
public:
	CEmbeddingFrameCProxyFrameIterator(const CEmbeddingFrame* embeddingFrame);
	virtual ~CEmbeddingFrameCProxyFrameIterator();

	CProxyFrame*	First()
						{return (CProxyFrame*)fIterator.First();}
	CProxyFrame*	Next()
						{return (CProxyFrame*)fIterator.Next();}
	XMPBoolean		IsNotComplete()
						{return fIterator.IsNotComplete();}
						
private:
	OrderedCollectionIterator	fIterator;
};

#endif

