/*
	File:		FWPart.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<11>	 4/12/94	DS		Change CreateEmbeddedFrame to
									RequestEmbeddedFrame
		<10>	 4/12/94	DS		Fix GeometryChanged def
		 <9>	 4/12/94	DS		Fix GeometryChanged def
		 <8>	 4/12/94	DS		Replace ExternalTransformChanged and
									ClipShapeChanged with GeometryChanged.
		 <7>	 4/11/94	DS		Change XMPTypeSet to XMPTypeList. Also make
									ExternalizeKinds call externalize by
									default.
		 <6>	  4/4/94	DS		Made IsFacetNested return FALSE if the
									facet is the root facet thereby preventing
									a BUS ERROR
		 <5>	 3/31/94	DS		Respond to API changes to RevealFrame,
									CreateMebeddedFramesIterator, and
									Highlight..
		 <4>	 3/30/94	DS		Make CPart::FacetAdded check if the facet
									that was being added is a nested facet. If
									so, call AddNested.. protocol.
		 <3>	  3/8/94	DS		Ack!
		 <2>	  3/8/94	DS		implement GetContentPropertyValueType with
									a debugstr instead of having it be pure
									virtual. The compiler isn't notifying that
									we haven't implemented this method.

	To Do:
*/

//	File:		FWPart.cpp
//	Contains:	Implementation of class CPart
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWWINDOW_
#include "FWWindow.h"
#endif

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _CLIPBD_
#include <Clipbd.h>
#endif

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _STORAGE_
#include <Storage.h>
#endif

#ifndef _DOCUMENT_
#include <Document.h>
#endif

#ifndef _SUCURSOR_
#include <SUCursor.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _WINSTAT_
#include <WinStat.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _PASCLSTR_
#include <PasclStr.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _FOCI_
#include <Foci.h>
#endif

#ifndef _LINKSRC_
#include <LinkSrc.h>
#endif

#ifndef _ISOSTR_
#include <ISOStr.h>
#endif

// ----- Macintosh Includes -----

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __EVENTS__
#include <Events.h>		// GetMouse, StillDown.
#endif

#ifndef __STDLIB__
#include <StdLib.h>		// Abs
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>		// See MoveTransformBy
#endif

#pragma segment FWPartSeg

//==============================================================================
// Constants
//==============================================================================

#define kOrderedList "Apple:OpenDoc:Framework:kOrderedList"
#define kUserPropertiesAdded "Apple:OpenDoc:Framework:kUserPropertiesAdded"

//==============================================================================
//	 class CPart
//==============================================================================

CPart::CPart()
{
	fPartMenuBar = kXMPNULL;
	fMenuBar = kXMPNULL;
	fActiveFrame = kXMPNULL;
	fDisplayFrames = kXMPNULL;
	fWindowList = kXMPNULL;
	fSelection = kXMPNULL;
	
	// ----- tokenize 
	fSelectionFocusToken = 0;
	fMenuFocusToken = 0;
	fKeyFocusToken = 0;
	
	// ----- linking 
	fPendingPublishLink = kXMPNULL;
	fPublishLinkList = kXMPNULL;
	fSubscribLinkList = kXMPNULL;
	
	// ----- User properties added -----
	fUserPropertiesAdded = kXMPFalse;
}

CPart::~CPart()
{	
	if (fPartMenuBar)
		fPartMenuBar->Release();
	
	delete fDisplayFrames;
	fDisplayFrames = kXMPNULL;

	delete fWindowList;
	fWindowList = kXMPNULL;
}

void CPart::Release()
{	
	XMPRefCntObject::Release();

	if (this->GetRefCount() == 0)
	{
		fDisplayFrames->DeleteAll();

		fWindowList->DeleteAll();
	
		if (fPublishLinkList)
		{
			fPublishLinkList->DeleteAll();
			delete fPublishLinkList;
			fPublishLinkList = kXMPNULL;
		}
		
		if (fSubscribLinkList)
		{
			fSubscribLinkList->DeleteAll();
			delete fSubscribLinkList;
			fSubscribLinkList = kXMPNULL;
		}
	}
}

void CPart::ReleaseAll()
{	
	XMPPersistentObject::ReleaseAll();
}

//-------------------------------------------------------------------------
//	 CPart::Initialize
//-------------------------------------------------------------------------

void CPart::Initialize()
{
	XMPSession* session = GetSession();
	
	// ----- tokenize 
	fSelectionFocusToken = session->Tokenize(kXMPSelectionFocus);
	fMenuFocusToken = session->Tokenize(kXMPMenuFocus);
	fKeyFocusToken = session->Tokenize(kXMPKeyFocus);
	
	// ----- menu bar
	fPartMenuBar = session->GetWindowState()->CopyBaseMenuBar();
	fMenuBar = fPartMenuBar;
	
	// ----- display collections
	fDisplayFrames = new OrderedCollection;

	// ----- Window List
	fWindowList = new OrderedCollection;
	
	// ----- Selection -----
	fSelection = NewSelection();
}

//-------------------------------------------------------------------------
//	 CPart::InitPart
//-------------------------------------------------------------------------

void CPart::InitPart(XMPStorageUnit* storageUnit)
{
	if (fInitialized)
		return;
	
	XMPPersistentObject::InitPersistentObject(storageUnit);

	// Common initialization
	this->Initialize();
	
	this->CheckAndAddProperties(storageUnit);
}

//-------------------------------------------------------------------------
//	 CPart::AddProperties
//-------------------------------------------------------------------------

void CPart::AddProperties(XMPStorageUnit* storageUnit)
{
XMPUnused(storageUnit);
}

//-------------------------------------------------------------------------
//	 CPart::CheckAndAddProperties
//-------------------------------------------------------------------------

void CPart::CheckAndAddProperties(XMPStorageUnit* storageUnit)
{
	// ----- Check and add content property -----
	if (!storageUnit->Exists(kXMPPropContents, this->GetContentPropertyValueType(), 0))
		storageUnit->AddProperty(kXMPPropContents)->AddValue(this->GetContentPropertyValueType());

	// ----- Check and add Display frames property -----
	if (!storageUnit->Exists(kXMPPropDisplayFrames, kOrderedList, 0))
	{
		storageUnit->AddProperty(kXMPPropDisplayFrames)->AddValue(kOrderedList);
		XMPULong count = 0;
		storageUnit->SetValue(sizeof(count), (XMPValue)&count);
	}
	
	// ----- Check and add User properties -----
	if (!storageUnit->Exists(kUserPropertiesAdded, kXMPBoolean, 0))
	{
		this->AddProperties(storageUnit);
		storageUnit->AddProperty(kUserPropertiesAdded)->AddValue(kXMPBoolean);
		fUserPropertiesAdded = kXMPTrue;
	}

}

//-------------------------------------------------------------------------
//	 CPart::AddProperties
//-------------------------------------------------------------------------

XMPValueType CPart::GetContentPropertyValueType() const
{
	// You must implement this method
	// In this method you must return your content kind string
	// for example, as in CClockPart:
	//	return kSampleClockKind
	//
	// Which translates into:
	//	return "Apple:Kind:Framework:Sample:Clock"
	
	WASSERTM(false, "This method must be implemented.");
	
	return "Apple:Kind:YourContentKindHere";
}


//-------------------------------------------------------------------------
//	 CPart::InitPartFromStorage
//-------------------------------------------------------------------------

void CPart::InitPartFromStorage(XMPStorageUnit* storageUnit)
{
	if (fInitialized)
		return;
	
	XMPPersistentObject::InitPersistentObjectFromStorage(storageUnit);

	// ----- Common initialization -----
	this->Initialize();
	
	// ----- Check/add our properties -----
	CheckAndAddProperties(storageUnit);
	
	// ----- Read Framework Specific data -----
	InternalizeDisplayFrames(storageUnit);
	
	// ----- Read Part Specific data -----
	InternalizeContent(storageUnit);	
}

//-------------------------------------------------------------------------
//	 CPart::Externalize
//-------------------------------------------------------------------------

void CPart::Externalize()
{
	XMPPersistentObject::Externalize();

	XMPStorageUnit* partStorageUnit = this->GetStorageUnit();
	
	// ----- Externalize framework specific data -----
	ExternalizeDisplayFrames(partStorageUnit);
	
	// ----- Externalize part specific data -----
	ExternalizeContent(partStorageUnit);
}

//-------------------------------------------------------------------------
//	 CPart::InternalizeContent
//-------------------------------------------------------------------------

void CPart::InternalizeContent(XMPStorageUnit* storageUnit)
{
XMPUnused(storageUnit);
}

//-------------------------------------------------------------------------
//	 CPart::ExternalizeContent
//-------------------------------------------------------------------------

void CPart::ExternalizeContent(XMPStorageUnit* storageUnit)
{
XMPUnused(storageUnit);
}

//-------------------------------------------------------------------------
//	 CPart::ExternalizeDisplayFrames
//-------------------------------------------------------------------------

void CPart::ExternalizeDisplayFrames(XMPStorageUnit* storageUnit)
{
	// ----- Write list of displayFrames -----
	storageUnit->Focus(kXMPPropDisplayFrames, kXMPPosUndefined, kOrderedList, 0, kXMPPosUndefined);
	storageUnit->Remove();
	storageUnit->AddValue(kOrderedList);
		
	XMPULong count = fDisplayFrames->Count();
	storageUnit->SetValue(sizeof(count), (XMPValue)&count);
	
	CPartDisplayFrameIterator ite1(this);
	for (CFrame* frame = ite1.First(); ite1.IsNotComplete(); frame = ite1.Next())
	{		
		// We get a weak reference because we don't want the frame to be cloned when
		// we clone the part
		XMPStorageUnitRef aSURef = storageUnit->GetWeakStorageUnitRef(frame->GetXMPStorageUnit());
		storageUnit->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
	}
}

//-------------------------------------------------------------------------
//	 CPart::InternalizeDisplayFrames
//-------------------------------------------------------------------------

void CPart::InternalizeDisplayFrames(XMPStorageUnit* storageUnit)
{
 	XMPStorageUnitRef aSURef;
	XMPFrame* xmpFrame;
	XMPVolatile(xmpFrame);
	
	// ----- Read the Display frames -----
	// We don't have to test because already done in CheckAndAddProperties -----
 	storageUnit->Focus(kXMPPropDisplayFrames, kXMPPosUndefined, kOrderedList, 0, kXMPPosUndefined);
	XMPULong count;
	storageUnit->GetValue(sizeof(count), (XMPValue)&count);
	for (XMPULong i = 0; i<count; i++)
	{
		storageUnit->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		
		TRY
		
			xmpFrame = storageUnit->GetDraft()->GetFrame(storageUnit->GetIDFromStorageUnitRef(aSURef));
		
			CFrame* frame = NewFrame(xmpFrame, xmpFrame->GetPresentation());
			xmpFrame->Release();			// This is needed to balance an additional reference created when GetFrame
											// was called.  We keep our own reference in CBaseFrame
	
			fDisplayFrames->AddLast((ElementType)frame);
			
		CATCH_ALL
			
			xmpFrame = kXMPNULL;
			
		ENDTRY
	}
}

//-------------------------------------------------------------------------
//	 CPart::CloneInto
//-------------------------------------------------------------------------

void CPart::CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU)
{
	XMPStorageUnit*	su = this->GetStorageUnit();
	
	XMPDraft* draft = su->GetDraft();
	XMPVolatile(draft);

	this->Externalize();
	
#ifndef ASLMBUILD
	storageUnit->AddProperty(kXMPPropPart)->AddValue(kXMPISOStr);
	XMPISOStr partName = GetContentPropertyValueType();
	XMPULong size = XMPISOStrLength(partName) + 1;
	storageUnit->SetValue(size, partName);
#endif

	TRY
	
		su->CloneInto(key, storageUnit, initiatingFrameSU);
		
	CATCH_ALL
	
		draft->AbortClone(key);
		
	ENDTRY
}

//-------------------------------------------------------------------------
// From DragAndDrop protocol
//-------------------------------------------------------------------------

// --- FulfillPromise ---

void CPart::FulfillPromise(XMPStorageUnitView *promiseSUView)
{
XMPUnused(promiseSUView);
}

// --- DropCompleted ---

void CPart::DropCompleted(XMPPart* destPart,
							   XMPDropResult dropResult)
{
XMPUnused(destPart);
XMPUnused(dropResult);
}

// --- DragEnter ---

void CPart::DragEnter(XMPDragItemIterator* dragInfo,  
						XMPFacet* xmpFacet,  
						XMPPoint where)
{
	CFacet *facet = CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	XMPPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	facet->DragEnter(dragInfo, windowPoint);
}

// --- DragWithin ---

void CPart::DragWithin(XMPDragItemIterator* dragInfo,
							XMPFacet* xmpFacet,
							XMPPoint where)
{
	CFacet *facet = CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	XMPPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	facet->DragWithin(dragInfo, windowPoint);
}

// --- DragLeave ---

void CPart::DragLeave(XMPFacet* xmpFacet,
						   XMPPoint where)
{
	CFacet *facet = CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	XMPPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	facet->DragLeave(windowPoint);
}

// --- Drop ---

XMPDropResult CPart::Drop(XMPDragItemIterator* dropInfo,
							   XMPFacet* xmpFacet,
							   XMPPoint where)
{
	CFacet *facet = CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	XMPPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	return facet->Drop(dropInfo, windowPoint);
}

//-------------------------------------------------------------------------
// From Embedding protocol
//-------------------------------------------------------------------------

// --- ContainingPartPropertiesChanged ---

void CPart::ContainingPartPropertiesChanged(XMPFrame* displayFrame,
												 XMPStorageUnit* propertyUnit)
{
XMPUnused(displayFrame);
XMPUnused(propertyUnit);
}

	// ********* for containing parts *********

// --- GetContainingPartProperties ---

XMPStorageUnit* CPart::GetContainingPartProperties(XMPFrame* displayFrame)
{
XMPUnused(displayFrame);
	THROW_IF_ERROR(kXMPErrCannotEmbed);
	return nil;
}

// --- CreateEmbeddedFramesIterator ---

XMPEmbeddedFramesIterator* CPart::CreateEmbeddedFramesIterator(XMPFrame* frame)
{
	return kXMPNULL;
}

//-------------------------------------------------------------------------
// from Facet protocol:
//-------------------------------------------------------------------------

//
// --- IsFacetNested ---
//
// Tests the given facet to see if it is nested within another facet in the SAME frame.
// This is significant if the developer wishes to nest facets ( e.g in scrolling ).
//
XMPBoolean CPart::IsFacetNested(XMPFacet* xmpFacet)
{
	
	// Get the frame for the passed in facet
	XMPFrame* frame = xmpFacet->GetFrame();
	
	// Get the containing facet of the passed in facet
	// And get its frame also
	XMPFacet* containingFacet = xmpFacet->GetContainingFacet();
	
	// Check to see if this facet is the root facet
	if (containingFacet == kXMPNULL)
		return kXMPFalse;
	
	XMPFrame* containedFacetFrame = containingFacet->GetFrame();
	
	// If they are different then the facet is not a nested facet
	// BTW: ???? What is the correct way to compare object pointers?
	// I assume this will be different for SOM
	if (frame != containedFacetFrame)
		return kXMPFalse;
	else
		return kXMPTrue;
}

// --- FacetAdded ---

void CPart::FacetAdded(XMPFacet* xmpFacet)
{

	if (IsFacetNested(xmpFacet)==kXMPTrue)
	{
		CFrame* frame = CFrame::XMPtoFWFrame(xmpFacet->GetFrame());
		CFacet* facet = frame->NewEmbeddedFacet(xmpFacet);	
		frame->FacetAdded(facet);
	}
	else
	{
		CFrame* frame = CFrame::XMPtoFWFrame(xmpFacet->GetFrame());
		CFacet* facet = frame->NewFacet(xmpFacet);	
		frame->FacetAdded(facet);
	}
}

// --- FacetRemoved ---

void CPart::FacetRemoved(XMPFacet* xmpFacet)
{
	CFrame* frame = CFrame::XMPtoFWFrame(xmpFacet->GetFrame());
	CFacet* facet = CFacet::XMPtoFWFacet(xmpFacet);
	frame->FacetRemoved(facet);
	delete facet;
}

// --- CanvasChanged ---

void CPart::CanvasChanged(XMPFacet* facet)
{
	XMPUnused(facet);
}

// --- GeometryChanged ---

void CPart::GeometryChanged(XMPFacet* xmpFacet)
{
	CFacet* facet = CFacet::XMPtoFWFacet(xmpFacet);
	facet->GeometryChanged();
}

//-------------------------------------------------------------------------
// from Frame protocol:
//-------------------------------------------------------------------------

// --- AddDisplayFrame ---

void CPart::AddDisplayFrame(XMPFrame* xmpFrame)
{
	if (xmpFrame->GetPresentation() == kNoPresentation)
		xmpFrame->SetPresentation(GetDefaultPresentation());
	
	CFrame* frame = NewFrame(xmpFrame, xmpFrame->GetPresentation());
	fDisplayFrames->AddLast(frame);
	
	frame->AddedToDisplayList();
}

// --- AttachSourceFrame ---

void CPart::AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame)
{
XMPUnused(xmpFrame);
XMPUnused(xmpSourceFrame);
}

// --- RemoveDisplayFrame ---

void CPart::RemoveDisplayFrame(XMPFrame* xmpFrame)
{
	CFrame* frame = CFrame::XMPtoFWFrame(xmpFrame);
	
	// ----- If the frame we remove is the activeFrame set it to the
	// ----- next activable frame
	if (frame == this->GetActiveFrame())
	{
		CPartDisplayFrameIterator ite1(this);
		for (CFrame* frame = ite1.First(); ite1.IsNotComplete(); frame = ite1.Next())
			if (frame->CanBeActiveFrame())
			{
				this->SetActiveFrame(frame);
				break;
			}
	}
	
	fDisplayFrames->Remove(frame);	
	frame->RemoveDisplayFrame();
	delete frame;
}

// --- CloseDisplayFrame ---

void CPart::CloseDisplayFrame(XMPFrame* xmpFrame)
{
	CFrame* frame = CFrame::XMPtoFWFrame(xmpFrame);
	fDisplayFrames->Remove(frame);	
	frame->CloseDisplayFrame();
	delete frame;
}

// --- RevealFrame ---

void CPart::RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape)
{
XMPUnused(embeddedFrame);
XMPUnused(revealShape);

	DebugStr("\pCPart::RevealFrame(XMPFrame* embeddedFrame) called");
}

// --- FrameShapeChanged ---

void CPart::FrameShapeChanged(XMPFrame* xmpFrame)
{
	CFrame* frame = CFrame::XMPtoFWFrame(xmpFrame);
	frame->FrameShapeChanged();
}

// --- ChangePresentation ---

void CPart::ChangePresentation(XMPFrame* xmpFrame, XMPName* viewType)
{
XMPUnused(xmpFrame);
XMPUnused(viewType);
}

// --- WritePartInfo ---

void CPart::WritePartInfo(XMPPtr partInfo,
							XMPStorageUnitView* storageUnitView)
{
XMPUnused(partInfo);
XMPUnused(storageUnitView);
}

// --- ReadPartInfo ---

XMPPtr CPart::ReadPartInfo(XMPFrame* xmpFrame, XMPStorageUnitView* storageUnitView)
{
	XMPUnused(storageUnitView);
	
	return (XMPPtr)xmpFrame->GetPartInfo();
}

// --- Open ---

XMPID CPart::Open(XMPFrame* xmpFrame)
{
XMPUnused(xmpFrame);

	XMPWindow* window;
	
	XMPISOStr title = "Untitled-1";
	if (xmpFrame)
	{
		CFrame* frame = CFrame::XMPtoFWFrame(xmpFrame);
		window = frame->CreateChildWindow(title);
	}
	else
	{
		window = CreateRootWindow(title);
	}
	
	XMPID windowID = 0;
	
	if (window)
	{
		// ATTENTION: the order is very important [HLX]
		window->Open();
		window->Show();
		window->Select();
	
		windowID =  window->GetID();
	}
	
	return windowID;
}

// --- Open ---

void CPart::FrameClosed(XMPFrame* xmpFrame)
{
XMPUnused(xmpFrame);
}

	// ********* for containing parts *********

// --- CreateEmbeddedFrame ---

XMPFrame* CPart::CreateEmbeddedFrame(XMPFrame* containingFrame,
									 XMPShape* frameShape,
									 XMPPart* embeddedXMPPart,
									 XMPTypeToken viewType,
									 XMPTypeToken presentation,
									 XMPID frameGroupID,
									 XMPBoolean isOverlaid)
{
	XMPUnused(containingFrame);
	XMPUnused(frameShape);
	XMPUnused(embeddedXMPPart);
	XMPUnused(viewType);
	XMPUnused(presentation);
	XMPUnused(frameGroupID);
	XMPUnused(isOverlaid);

	THROW_IF_ERROR(kXMPErrCannotEmbed);
	return kXMPNULL;
}
	
// --- RequestEmbeddedFrame ---

XMPFrame* CPart::RequestEmbeddedFrame(XMPFrame* containingFrame,
											  XMPFrame* baseFrame,
											  XMPShape* frameShape,
											  XMPPart* embedPart,
											  XMPTypeToken viewType,
											  XMPTypeToken presentation,
											  XMPBoolean isOverlaid)
{
	return CreateEmbeddedFrame(containingFrame, frameShape, embedPart, viewType, presentation, 0, isOverlaid);
}
	
// --- RemoveEmbeddedFrame ---

void CPart::RemoveEmbeddedFrame(XMPFrame* xmpFrame)
{
	XMPUnused(xmpFrame);
	THROW_IF_ERROR(kXMPErrCannotEmbed);
}

// --- RequestFrameShape ---

XMPShape* CPart::RequestFrameShape(XMPFrame* embeddedXMPFrame, XMPShape* frameShape)
{
XMPUnused(embeddedXMPFrame);
	return frameShape;
}

// --- UsedShapeChanged ---

void CPart::UsedShapeChanged(XMPFrame* xmpEmbeddedFrame)
{
XMPUnused(xmpEmbeddedFrame);
	THROW_IF_ERROR(kXMPErrCannotEmbed);
}

// --- AdjustBorderShape ---

XMPShape* CPart::AdjustBorderShape(XMPFrame* xmpFrame, XMPShape* shape)
{
XMPUnused(xmpFrame);

	return shape;
}

// --- EmbeddedFrameSpec ---

void CPart::EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec)
{
XMPUnused(embeddedFrame);
XMPUnused(spec);
}

// --- ViewTypeChanged ---

void CPart::ViewTypeChanged(XMPFrame* xmpFrame)
{
XMPUnused(xmpFrame);
}
	
// --- PresentationChanged ---

void CPart::PresentationChanged(XMPFrame* xmpFrame)
{
	CFrame* frame = CFrame::XMPtoFWFrame(xmpFrame);
	frame->PresentationChanged();
}

//-------------------------------------------------------------------------
// From Binding protocol
//-------------------------------------------------------------------------

void CPart::ExternalizeKinds(XMPTypeList* kindlist)
{
XMPUnused(kindlist);

	// Default behavior is to externalize everything
	this->Externalize();
}

void CPart::ChangeKind(XMPType kind)
{
XMPUnused(kind);
}

//-------------------------------------------------------------------------
// From Imaging protocol
//-------------------------------------------------------------------------

// --- CanvasUpdate ---

void CPart::CanvasUpdated(XMPCanvas* canvas)
{
XMPUnused(canvas);

	DebugStr("\pCPart::CanvasUpdate(XMPCanvas* canvas) called");
}

// --- Draw ---

void CPart::Draw(XMPFacet* xmpFacet, XMPShape* invalidShape)
{
	CFacet* facet = CFacet::XMPtoFWFacet(xmpFacet);

	XMPShape* testShape = kXMPNULL;
	if (invalidShape)
	{
		testShape = ::NewXMPShape(invalidShape);
		testShape->Intersect(xmpFacet->GetAggregateClipShape());
	}	

	CDrawInitiator di(facet, testShape);
	facet->Draw(testShape);
	
	delete testShape;
}

// --- HighlightChanged ---

void CPart::HighlightChanged(XMPFacet* facet)
{
XMPUnused(facet);
}

// --- GetPrintResolution ---

XMPULong CPart::GetPrintResolution(XMPFrame* xmpFrame)
{
XMPUnused(xmpFrame);
	return 0;
}

//==============================================================================
//	 Linking protocol
//

//------------------------------------------------------------------------------
//	 CPart::CreateLink
//------------------------------------------------------------------------------

XMPLink* CPart::CreateLink(XMPPtr data, XMPULong size)
{
XMPUnused(size);

	XMPChangeID changeID = *((XMPChangeID*)data);
	
	XMPLinkSource* xmpLinkSource = this->GetStorageUnit()->GetDraft()->CreateLinkSource(this);
	
	WASSERTM(fPendingPublishLink->GetChangeID() == changeID, "Error in CreateLink");
	
	fPendingPublishLink->SetXMPLinkSource(xmpLinkSource);
	
	AddToPublishLinkList(fPendingPublishLink);
	fPendingPublishLink->Publish();

	fPendingPublishLink = kXMPNULL;
		
	return xmpLinkSource->GetLink();
}

//------------------------------------------------------------------------------
//	 CPart::LinkUpdated
//------------------------------------------------------------------------------

void CPart::LinkUpdated(XMPLink* updatedLink, XMPChangeID id)
{
	CSubscribLink *link = XMPtoSubscribLink(updatedLink);
	if (link)
		link->LinkUpdated(id);
}

//------------------------------------------------------------------------------
//	 CPart::RevealLink
//------------------------------------------------------------------------------

void CPart::RevealLink(XMPLinkSource* xmpLinkSource)
{
XMPUnused(xmpLinkSource);
}

//------------------------------------------------------------------------------
//	 CPart::EmbeddedFrameChanged
//------------------------------------------------------------------------------

void CPart::EmbeddedFrameChanged(XMPFrame* xmpFrame, XMPChangeID change)
{
XMPUnused(xmpFrame);
XMPUnused(change);
}

//------------------------------------------------------------------------------
//	 CPart::LinkStatusChanged
//------------------------------------------------------------------------------

void CPart::LinkStatusChanged(XMPFrame* xmpFrame)
{
XMPUnused(xmpFrame);
}

//------------------------------------------------------------------------------
//	 CPart::AddToPublishLinkList
//------------------------------------------------------------------------------

void CPart::AddToPublishLinkList(CPublishLink* publishLink)
{
	if (fPublishLinkList == kXMPNULL)
	{
		fPublishLinkList = new OrderedCollection;
	}
	
	fPublishLinkList->AddLast((ElementType)publishLink);
}

//------------------------------------------------------------------------------
//	 CPart::RemoveFromPublishLinkList
//------------------------------------------------------------------------------

void CPart::RemoveFromPublishLinkList(CPublishLink* publishLink)
{
	fPublishLinkList->Remove((ElementType)publishLink);
	
	if (fPublishLinkList->Count() == 0)
	{
		delete fPublishLinkList;
		fPublishLinkList = kXMPNULL;
	}
}

//------------------------------------------------------------------------------
//	 CPart::AddToSubscribLinkList
//------------------------------------------------------------------------------

void CPart::AddToSubscribLinkList(CSubscribLink* subscribLink)
{
	if (fSubscribLinkList == kXMPNULL)
	{
		fSubscribLinkList = new OrderedCollection;
	}
	
	fSubscribLinkList->AddLast((ElementType)subscribLink);
}

//------------------------------------------------------------------------------
//	 CPart::RemoveFromSubscribLinkList
//------------------------------------------------------------------------------

void CPart::RemoveFromSubscribLinkList(CSubscribLink* subscribLink)
{
	fSubscribLinkList->Remove((ElementType)subscribLink);
	
	if (fSubscribLinkList->Count() == 0)
	{
		delete fSubscribLinkList;
		fSubscribLinkList = kXMPNULL;
	}
}

//------------------------------------------------------------------------------
//	 CPart::XMPtoSubscribLink
//------------------------------------------------------------------------------

CSubscribLink* CPart::XMPtoSubscribLink(XMPLink* xmpLink)
{
	OrderedCollectionIterator ite(fSubscribLinkList);
	for (CSubscribLink* link = (CSubscribLink*)ite.First(); ite.IsNotComplete(); link = (CSubscribLink*)ite.Next())
	{
		if (link->GetXMPLink() == xmpLink)
			return link;
	}
	return kXMPNULL;
}

//==============================================================================
//	 Memory Management protocol
//

//------------------------------------------------------------------------------
//	 CPart::Purge
//------------------------------------------------------------------------------

XMPSize CPart::Purge(XMPSize size)
{
	return size; // return fStorageUnit->Purge(size);
}

//==============================================================================
//	 Part Activation protocol
//

// --- BeginRelinquishFocus ---

XMPBoolean CPart::BeginRelinquishFocus(XMPTypeToken focus,
											XMPFrame* ownerFrame,
											XMPFrame* proposedFrame)
{
XMPUnused(focus);
XMPUnused(ownerFrame);
XMPUnused(proposedFrame);

	return kXMPTrue;
}
	
// --- CommitRelinquishFocus ---

void CPart::CommitRelinquishFocus(XMPTypeToken focus,
									   XMPFrame* ownerFrame,
									   XMPFrame* proposedFrame)
{
XMPUnused(proposedFrame);

	FocusLost(focus, ownerFrame);
	
	if (proposedFrame == kXMPNULL || proposedFrame->GetPart() != this)
	{
		if (fSelection)
			fSelection->CloseSelection();
		HideShowFloatingWindows(kXMPFalse);
	}
}
							
// --- AbortRelinquishFocus ---

void CPart::AbortRelinquishFocus(XMPTypeToken focus,
									  XMPFrame* ownerFrame,
									  XMPFrame* proposedFrame)
{
XMPUnused(focus);
XMPUnused(ownerFrame);
XMPUnused(proposedFrame);
}
							  
// --- FocusAcquired ---

void CPart::FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	CFrame* frame = CFrame::XMPtoFWFrame(ownerFrame);
	frame->FocusStateChanged(focus, kXMPTrue);
	HideShowFloatingWindows(kXMPTrue);
}	

// --- FocusLost ---

void CPart::FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	CFrame* frame = CFrame::XMPtoFWFrame(ownerFrame);
	frame->FocusStateChanged(focus, kXMPFalse);
}

//-------------------------------------------------------------------------
// From UI Events protocol
//-------------------------------------------------------------------------


// --- HandleEvent ---

XMPBoolean CPart::HandleEvent(XMPEventData event,  XMPFrame* xmpFrame,  XMPFacet* xmpFacet)
{
	XMPBoolean handled = false;
	CFrame* frame = CFrame::XMPtoFWFrame(xmpFrame);
	CFacet* facet = CFacet::XMPtoFWFacet(xmpFacet);
	
	switch (event->what)
	{
		case nullEvent:
			handled = HandleIdle(frame);
			break;
		
		case mouseDown:
			handled = facet->HandleMouseDown(kXMPNULL, event);
			break;
		case mouseUp:
			handled = facet->HandleMouseUp(event);
			break;
			
		case kXMPEvtMouseDownEmbedded:
		case kXMPEvtMouseUpEmbedded:
		case kXMPEvtMouseDownBorder:
		case kXMPEvtMouseUpBorder:
			break;
		
		case keyDown:
		case autoKey:
			handled = frame->HandleKeyDown(event);
			break;
	
		case activateEvt:
			handled = HandleActivate(frame, facet, (event->modifiers & activeFlag ) != 0);
			break;
		
		case kXMPEvtWindow: 
			{
				CWindow* window = XMPtoFWWindow(facet->GetXMPWindow());
				if (window)
				{
					switch (event->message)
					{
						case inGoAway:
							if (window->IsFloating()) {						
								window->HideWindow();
								handled = kXMPTrue;
							}
							break;
						case inZoomIn:
						case inZoomOut:
							SysBeep(2); 
							handled = kXMPTrue;
							break;
						default:
							break;
					}
				}
			}
			break;
		
		case keyUp:
			break;
						
		case osEvt:
			break;
	
		case kHighLevelEvent:
			break;

		case kXMPEvtMenu:
			handled = HandleMenuEvent(frame, (XMPCommandID)event->message);
			break;
			
		default:
			return false;
	}
	
	return handled;
}

// --- MouseEnter ---

void CPart::MouseEnter(XMPFacet* xmpFacet, XMPPoint where)
{
	CFacet *facet = CFacet::XMPtoFWFacet(xmpFacet);
	facet->MouseEnter(where);
}

// ----- MouseWithin

void CPart::MouseWithin(XMPFacet* xmpFacet, XMPPoint where)
{
	CFacet *facet = CFacet::XMPtoFWFacet(xmpFacet);
	facet->MouseWithin(where);
}

// --- MouseLeave ---

void CPart::MouseLeave(XMPFacet* xmpFacet)
{
	CFacet *facet = CFacet::XMPtoFWFacet(xmpFacet);
	facet->MouseLeave();
}

// --- AdjustMenus ---

void CPart::AdjustMenus(XMPFrame* xmpFrame)
{
	CFrame *frame = CFrame::XMPtoFWFrame(xmpFrame);
	frame->DoAdjustMenus(GetMenuBar());
}

//-------------------------------------------------------------------------
// From Undo protocol
//-------------------------------------------------------------------------

// --- UndoAction ---

void CPart::UndoAction(XMPActionData actionState)
{
XMPUnused(actionState);
}

// --- RedoAction ---

void CPart::RedoAction(XMPActionData actionState)
{
XMPUnused(actionState);
}

// --- DisposeAction ---

void CPart::DisposeActionState(XMPActionData actionState,
									XMPDoneState doneState)
{
XMPUnused(actionState);
XMPUnused(doneState);
}


// --- WriteAction ---

void CPart::WriteActionState(XMPPtr actionState,
								  XMPStorageUnitView* storageUnitView)
{
XMPUnused(actionState);
XMPUnused(storageUnitView);
}

// --- ReadAction ---

XMPPtr CPart::ReadActionState(XMPStorageUnitView* storageUnitView)
{
XMPUnused(storageUnitView);
	return kXMPNULL;
}

//==============================================================================
// New API
//==============================================================================

//------------------------------------------------------------------------------
//	 CPart::HandleMenuEvent
//------------------------------------------------------------------------------

XMPBoolean CPart::HandleMenuEvent(CFrame* frame, XMPSLong message)
{
	XMPMenuBar* menuBar = GetMenuBar();
	
	XMPMenuID menu = HiWord(message);
	XMPMenuItemID item = LoWord(message);
	XMPCommandID commandID = menuBar->GetCommand(menu, item);

	XMPBoolean result = frame->DoMenuEvent(menuBar, commandID);
	
	::HiliteMenu(0);
	
	return result;
}

//------------------------------------------------------------------------------
//	 CPart::DoMenuEvent
//------------------------------------------------------------------------------

XMPBoolean CPart::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
XMPUnused(menuBar);

	XMPBoolean result = kXMPTrue;

	switch (commandID)
	{
		case kXMPCommandSelectAll:
			if (fSelection)
				fSelection->SelectAll();
			break;
			
		case  kXMPCommandClear:
			if (fSelection && fSelection->DoClear())
				this->Changed();
			break;
						
		case kXMPCommandPaste:
			if (fSelection && fSelection->DoPaste())
				this->Changed();
			break;
		
		case kXMPCommandPasteAs:
			if (fSelection && fSelection->DoPasteWithLink())
				this->Changed();
			break;

		default:
			result = kXMPFalse;
	};
	
	return result;
}

//------------------------------------------------------------------------------
//	 CPart::HandleIdle
//------------------------------------------------------------------------------

XMPBoolean CPart::HandleIdle(CFrame* frame)
{
	if (frame)
		return frame->DoIdle();
	else
		return this->DoIdle();
}

//------------------------------------------------------------------------------
//	 CPart::DoIdle
//------------------------------------------------------------------------------

XMPBoolean CPart::DoIdle()
{
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CPart::InstallMenus
//------------------------------------------------------------------------------

void CPart::InstallMenus(XMPMenuBar* menuBar)
{
	if (menuBar == kXMPNULL)
		menuBar = GetPartMenuBar();
		
	if (menuBar)
		menuBar->Display();
}

//------------------------------------------------------------------------------
//	 CPart::DoAdjustMenus
//------------------------------------------------------------------------------

void CPart::DoAdjustMenus(XMPMenuBar* menuBar)
{
	XMPBoolean hasSelection = fSelection && !fSelection->IsEmpty();

	// ----- Edit Menu -----
	menuBar->EnableCommand(kXMPCommandClear, hasSelection);
	menuBar->EnableCommand(kXMPCommandCut, hasSelection);
	menuBar->EnableCommand(kXMPCommandCopy, hasSelection);
	menuBar->EnableCommand(kXMPCommandPaste, kXMPFalse);
	menuBar->EnableCommand(kXMPCommandPasteAs, HasPropertyOnClipboard(kXMPPropLinkSpec, kXMPLinkSpec));
}

//------------------------------------------------------------------------------
//	 CPart::HandleActivate
//------------------------------------------------------------------------------

XMPBoolean CPart::HandleActivate(CFrame *frame, CFacet* facet, XMPBoolean state)
{
	XMPUnused(frame);
	
	if (state)
		facet->WindowActivated();
	else
		facet->WindowDeActivated();

	return kXMPTrue;		// we just don't have anuthing to do
}

//------------------------------------------------------------------------------
//	 CPart::AddWindow
//------------------------------------------------------------------------------

void CPart::AddWindow(CWindow* window)
{
	fWindowList->AddLast((ElementType)window);
}

//------------------------------------------------------------------------------
//	 CPart::RemoveWindow
//------------------------------------------------------------------------------

void CPart::RemoveWindow(CWindow* window)
{
	fWindowList->Remove((ElementType)window);
}

//------------------------------------------------------------------------------
//	 CPart::XMPtoFWWindow
//------------------------------------------------------------------------------

CWindow* CPart::XMPtoFWWindow(XMPWindow *xmpWindow) const
{
	OrderedCollectionIterator ite(fWindowList);
	for (CWindow* window = (CWindow*)ite.First(); ite.IsNotComplete(); window = (CWindow*)ite.Next())
	{
		if (window->GetXMPWindow() == xmpWindow)
			return window;
	}
	
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CPart::HideShowFloatingWindows();
//------------------------------------------------------------------------------

void CPart::HideShowFloatingWindows(XMPBoolean state)
{
	OrderedCollectionIterator ite(fWindowList);
	for (CWindow* window = (CWindow*)ite.First(); ite.IsNotComplete(); window = (CWindow*)ite.Next())
	{
		if (window->GetXMPWindow() && window->IsFloating() && window->IsShown())
		{
			if (state)
				window->GetXMPWindow()->Show();
			else
				window->GetXMPWindow()->Hide();
		}
	}
}

//------------------------------------------------------------------------------
//	 CPart::CreateRootWindow
//------------------------------------------------------------------------------

XMPWindow* CPart::CreateRootWindow(XMPISOStr windowTitle)
{
	// ----- Taken from the shell. Should be replaced -----
	// Get the menu bar height, for figuring the top of our windows
	XMPSShort	mbHeight	= 24; //(*(XMPSShort*)MBarHeight);	//get it from lomem
	XMPRect windRect(ff(4), ff(mbHeight+16), 
					 ff(XMPASLMQDGlobals.screenBits.bounds.right - 64), ff(XMPASLMQDGlobals.screenBits.bounds.bottom - 4));
	if (windRect.right-windRect.left > ff(600))
	{
		windRect.right = windRect.left + ff(600);
	}

	return CreateXMPWindow(kXMPNULL,
						 	GetSession()->Tokenize(kXMPViewAsFrame),
							kNoPresentation,
							windowTitle,
							windRect,
							kXMPTrue,		// close box
							kXMPTrue,		// resizable
							kXMPFalse,		// floating
							kXMPTrue,		// should save
							zoomDocProc);
}

//------------------------------------------------------------------------------
//	 CPart::CreateXMPWindow
//------------------------------------------------------------------------------

XMPWindow* CPart::CreateXMPWindow(CFrame* itsFrame,
								XMPTypeToken viewType,
								XMPTypeToken presentation,
								XMPISOStr windowTitle,
								const XMPRect& windowRect,
								XMPBoolean hasCloseBox,
								XMPBoolean isResizable,
								XMPBoolean isFloating,
								XMPBoolean shouldSave,
								XMPSShort procID)
{
	Rect qdWinRect;
	windowRect.AsQDRect(qdWinRect);
	
	Str255 title;
	CopyISOStr2PStr(title, windowTitle);
	
	XMPPlatformWindow platformWindow = ::NewCWindow(kXMPNULL, 
													&qdWinRect, 
													title, 
													kXMPFalse, 
													procID, 
													(WindowPtr)-1L, 
													hasCloseBox, 
													kXMPNULL);

	return GetSession()->GetWindowState()->CreateWindow(platformWindow, 
														itsFrame == kXMPNULL && !isFloating,		// root frame
														isResizable,
														isFloating,
														shouldSave,
														this,
														viewType, 
														presentation,
														itsFrame ? itsFrame->GetXMPFrame() : (XMPFrame*)kXMPNULL);
}

//------------------------------------------------------------------------------
//	 CPart::InvalidateAllFrames
//------------------------------------------------------------------------------

void CPart::InvalidateAllFrames(XMPTypeToken whichFrames, CFacet* skipFacet, XMPShape* invalidShape)
{
	CPartDisplayFrameIterator ite(this, whichFrames);
	for (CFrame* frame = ite.First(); ite.IsNotComplete(); frame = ite.Next())
	{
		frame->InvalidateAllFacets(skipFacet, invalidShape);
	}
}

//------------------------------------------------------------------------------
//	 CPart::ValidateAllFrames
//------------------------------------------------------------------------------

void CPart::ValidateAllFrames(XMPTypeToken whichFrames, CFacet* skipFacet, XMPShape* validShape)
{
	CPartDisplayFrameIterator ite(this, whichFrames);
	for (CFrame* frame = ite.First(); ite.IsNotComplete(); frame = ite.Next())
	{
		frame->ValidateAllFacets(skipFacet, validShape);
	}
}

//------------------------------------------------------------------------------
//	 CPart::Changed
//------------------------------------------------------------------------------

void CPart::Changed()
{
	this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
	
	DeletePendingPublish();
}

//------------------------------------------------------------------------------
//	 CPart::GetDefaultPresentation
//------------------------------------------------------------------------------

XMPTypeToken CPart::GetDefaultPresentation() const
{
	return kNoPresentation;
}

//----------------------------------------------------------------------------------------
//	 CPart::SetActiveFrame
//----------------------------------------------------------------------------------------
void CPart::SetActiveFrame(CFrame*  activeFrame)
{
	fActiveFrame = activeFrame;
}

//----------------------------------------------------------------------------------------
//	 CPart::GetActiveFacet
//----------------------------------------------------------------------------------------
CFacet* CPart::GetActiveFacet() const
{
	CFacet* facet = kXMPNULL;
	if (fActiveFrame)
		facet = fActiveFrame->GetActiveFacet();
	return facet;
}

//------------------------------------------------------------------------------
//	 CPart::NewSelection
//------------------------------------------------------------------------------

FW_CSelection* CPart::NewSelection()
{
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CPart::HasPropertyOnClipboard
//------------------------------------------------------------------------------

XMPBoolean CPart::HasPropertyOnClipboard(XMPPropertyName propertyName, XMPValueType type)
{
	XMPClipboard* clipboard = GetSession()->GetClipboard();
	XMPBoolean result = kXMPFalse;

	XMPClipboardKey key;
	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* su = clipboard->GetContentStorageUnit(key);
			result =  su->Exists(propertyName, type, 0);
		CATCH_ALL
		ENDTRY
		clipboard->Unlock(key);
	}
	return result;
}

//------------------------------------------------------------------------------
//	 CPart::DeletePendingPublish
//------------------------------------------------------------------------------
// Remove the link spec from the clipboard

void CPart::DeletePendingPublish()
{
	if (fPendingPublishLink != kXMPNULL)
	{
		XMPClipboard* clipboard = GetSession()->GetClipboard();
		
		if (fPendingPublishLink->GetChangeID() == clipboard->GetChangeID()) 
		{
			XMPClipboardKey key;
			if (clipboard->Lock(0, &key)) 
			{
				TRY
					XMPStorageUnit* clipboardSU = clipboard->GetContentStorageUnit(key);
					if (clipboardSU->Exists(kXMPPropLinkSpec, (XMPValueType) kXMPNULL, 0) == kXMPTrue) 
					{
						clipboardSU->Focus(kXMPPropLinkSpec, kXMPPosUndefined, (XMPValueType) kXMPNULL, 0, kXMPPosUndefined);
						clipboardSU->Remove();
					}
				CATCH_ALL
				ENDTRY
				clipboard->Unlock(key);
			}
		}
		
		delete fPendingPublishLink;
		fPendingPublishLink = kXMPNULL;
	}
}

//------------------------------------------------------------------------------
//	 CPart::Dragging
//------------------------------------------------------------------------------

void CPart::Dragging(CFacet* facet, XMPBoolean state)
{
XMPUnused(facet);
XMPUnused(state);
}

//------------------------------------------------------------------------------
//	 CPart::ClonePartFrom
//------------------------------------------------------------------------------
//	A CPart can't embed anything

XMPPart* CPart::ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind)
{
XMPUnused(storageUnit);
XMPUnused(kind);
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CPart::IsEmbeddingPart
//------------------------------------------------------------------------------
//	This needs to be removed

XMPBoolean CPart::IsEmbeddingPart() const
{
	return kXMPFalse;
}

//==============================================================================
//	 class CPartDisplayFrameIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 CPartDisplayFrameIterator::CPartDisplayFrameIterator
//------------------------------------------------------------------------------

CPartDisplayFrameIterator::CPartDisplayFrameIterator(CPart* part, XMPTypeToken presentation) :
	fIterator(part->GetDisplayFrameList()),
	fPresentation(presentation)
{
	fCurrentFrame = (CFrame*)fIterator.First();
	FindNext();
}

//------------------------------------------------------------------------------
//	 CPartDisplayFrameIterator::~CPartDisplayFrameIterator
//------------------------------------------------------------------------------

CPartDisplayFrameIterator::~CPartDisplayFrameIterator()
{
}

//------------------------------------------------------------------------------
//	 CPartDisplayFrameIterator::First
//------------------------------------------------------------------------------

CFrame* CPartDisplayFrameIterator::First()
{
	return fCurrentFrame;
}
	
//------------------------------------------------------------------------------
//	 CPartDisplayFrameIterator::Next
//------------------------------------------------------------------------------

CFrame* CPartDisplayFrameIterator::Next()
{
	fCurrentFrame = (CFrame*)fIterator.Next();
	FindNext();
	return fCurrentFrame;
}

//------------------------------------------------------------------------------
//	 CPartDisplayFrameIterator::IsNotComplete
//------------------------------------------------------------------------------

XMPBoolean CPartDisplayFrameIterator::IsNotComplete()
{
	return fCurrentFrame != kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CPartDisplayFrameIterator::FindNext
//------------------------------------------------------------------------------

void CPartDisplayFrameIterator::FindNext()
{
	while (fCurrentFrame != kXMPNULL)
	{
		if (fPresentation == kNoPresentation || fCurrentFrame->GetPresentation() == fPresentation)
		{
			return;
		}
		fCurrentFrame = (CFrame*)fIterator.Next();
	} 
}

//========================================================================================
//	 class CPartFacetIterator
//========================================================================================

//---------------------------------------------------------------------------------------
//	 CPartFacetIterator::CPartFacetIterator
//---------------------------------------------------------------------------------------

CPartFacetIterator::CPartFacetIterator(CPart* part, XMPTypeToken presentation) :
	fFrameIterator(part, presentation),
	fCurrentFacet(kXMPNULL),
	fFacetIterator(kXMPNULL)	
{
	fCurrentFrame = fFrameIterator.First();
}

//---------------------------------------------------------------------------------------
//	 CPartFacetIterator::~CPartFacetIterator
//---------------------------------------------------------------------------------------

CPartFacetIterator::~CPartFacetIterator()
{
	delete fFacetIterator;
}

//---------------------------------------------------------------------------------------
//	 CPartFacetIterator::First
//---------------------------------------------------------------------------------------

CFacet* CPartFacetIterator::First()
{
	if (fFacetIterator == kXMPNULL)
	{
		NewFacetIterator();		
	}
	
	return fCurrentFacet;
}
	
//---------------------------------------------------------------------------------------
//	 CPartFacetIterator::Next
//---------------------------------------------------------------------------------------

CFacet* CPartFacetIterator::Next()
{
	if (fFacetIterator)
	{
		fCurrentFacet = fFacetIterator->Next();
		if (fCurrentFacet == kXMPNULL)
		{
			delete fFacetIterator;
			fFacetIterator = kXMPNULL;
			fCurrentFrame = fFrameIterator.Next();
			NewFacetIterator();
		}
	}
	return fCurrentFacet;
}

//---------------------------------------------------------------------------------------
//	 CPartFacetIterator::IsNotComplete
//---------------------------------------------------------------------------------------

XMPBoolean CPartFacetIterator::IsNotComplete()
{
	return fCurrentFacet != kXMPNULL;
}

//---------------------------------------------------------------------------------------
//	 CPartFacetIterator::NewFacetIterator
//---------------------------------------------------------------------------------------

void CPartFacetIterator::NewFacetIterator()
{
	if (fCurrentFrame != kXMPNULL)
	{
		fFacetIterator = new CFrameFacetIterator(fCurrentFrame);
		fCurrentFacet = fFacetIterator->First();		
	}
}

