/*
	File:		FWPart.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<10>	 4/12/94	DS		Implement RequestFrameShape
		 <9>	 4/12/94	DS		Replace ExternalTransformChanged and
									ClipShapeChanged with GeometryChanged.
		 <8>	 4/11/94	DS		Change XMPTypeSet to XMPTypeList
		 <7>	 3/31/94	DS		Respond to API changes to RevealFrame,
									CreateMebeddedFramesIterator, and
									Highlight..
		 <6>	 3/31/94	DS		Remove reference to PartType.h
		 <5>	 3/30/94	DS		Add IsFacetEmbedded protocol
		<3+>	 3/17/94	DS		Add CDECL to CPart dtor
		 <3>	  3/8/94	DS		Ack!
		 <2>	  3/8/94	DS		implement GetContentPropertyValueType with
									a debugstr instead of having it be pure
									virtual. The compiler isn't notifying that
									we haven't implemented this method.

	To Do:
*/

//	File:		FWPart.h
//	Contains:	Definition for class CPart
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWPART_
#define _FWPART_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _FWEVENTH_
#include "FWEventH.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _PART_
#include <Part.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _DOCUMENT_
#include <Document.h>
#endif

#ifndef _STORAGE_
#include <Storage.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif




//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPFrame;
class XMPMenuBar;
class XMPWindow;
class CFrame;
class CFacet;
class CWindow;
class CFrameFacetIterator;
class FW_CSelection;
class CPublishLink;
class CSubscribLink;
class XMPSession;

//==============================================================================
//	 Defines
//==============================================================================

#define kNoPresentation 0
#define kContentAnnotationProp		"Apple:Framework:Proposition:ContentAnnotation"
#define kContentAnnotationValue		"Apple:Framework:Value:ContentAnnotation"



//==============================================================================
//	 class CPart
//==============================================================================

#define kCPartID "appl:CPart$class,1.0.0"

class CPart : public XMPPart, public MEventHandler
{

	friend class XMPDraft;

//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
protected:

	CPart();
	virtual ~ CDECL CPart();

	virtual void Initialize();
	XMPVMethod void InitPart(XMPStorageUnit* storageUnit);
	XMPVMethod void InitPartFromStorage(XMPStorageUnit* storageUnit);

//------------------------------------------------------------------------------
//	 Inherited API
//
public:

	//  ref counting 
	XMPMethod void Release();
	XMPMethod void ReleaseAll();
	
	//  DragAndDrop 
	XMPVMethod void FulfillPromise(XMPStorageUnitView *promiseSUView);
	XMPVMethod void DropCompleted(XMPPart* destPart, XMPDropResult dropResult);
	XMPVMethod void DragEnter(XMPDragItemIterator* dragInfo,  
								XMPFacet* facet,  
								XMPPoint where);
	XMPVMethod void DragWithin(XMPDragItemIterator* dragInfo,
									XMPFacet* xmpFacet,
									XMPPoint where);
	XMPVMethod void DragLeave(XMPFacet* xmpFacet,
						   		XMPPoint where);
	XMPVMethod XMPDropResult Drop(XMPDragItemIterator* dropInfo,
								  XMPFacet* xmpFacet,
								  XMPPoint where);

	//  Embedding 
	XMPVMethod void ContainingPartPropertiesChanged(XMPFrame* displayFrame,
													XMPStorageUnit* propertyUnit);
	XMPVMethod XMPStorageUnit* GetContainingPartProperties(XMPFrame* displayFrame);
	XMPVMethod XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(XMPFrame* frame);
		
	//  Facet 
	XMPNVMethod XMPBoolean IsFacetNested(XMPFacet* xmpFacet);
	XMPVMethod void FacetAdded(XMPFacet* xmpFacet);
	XMPVMethod void FacetRemoved(XMPFacet* xmpFacet);
	XMPVMethod void CanvasChanged(XMPFacet* facet);
	XMPVMethod void GeometryChanged(XMPFacet* facet);


	//  Frame 
	XMPVMethod void AddDisplayFrame(XMPFrame* xmpFrame);
	XMPVMethod void AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame);
	XMPVMethod void RemoveDisplayFrame(XMPFrame* xmpFrame);
	XMPVMethod void CloseDisplayFrame(XMPFrame* frame);
	XMPVMethod void RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape);
	
	XMPVMethod void FrameShapeChanged(XMPFrame* xmpFrame);
	XMPVMethod void UsedShapeChanged(XMPFrame* xmpEmbeddedFrame);
	
	XMPVMethod void ChangePresentation(XMPFrame* xmpFrame, XMPName* viewType);
	XMPVMethod void WritePartInfo(XMPPtr partInfo,
								  XMPStorageUnitView* storageUnitView);
	XMPVMethod XMPPtr ReadPartInfo(XMPFrame* xmpFrame, XMPStorageUnitView* storageUnitView);
	XMPVMethod XMPID Open(XMPFrame* xmpFrame);
	XMPVMethod void FrameClosed(XMPFrame* xmpFrame);
	XMPVMethod XMPFrame* CreateEmbeddedFrame(XMPFrame* containingFrame,
											 XMPShape* frameShape,
											 XMPPart* embedPart,
											 XMPTypeToken viewType,
											 XMPTypeToken presentation,
											 XMPID frameGroupID,
											 XMPBoolean isOverlaid);
											 
	XMPVMethod XMPFrame* RequestEmbeddedFrame(XMPFrame* containingFrame,
											  XMPFrame* baseFrame,
											  XMPShape* frameShape,
											  XMPPart* embedPart,
											  XMPTypeToken viewType,
											  XMPTypeToken presentation,
											  XMPBoolean isOverlaid);
	XMPVMethod void RemoveEmbeddedFrame(XMPFrame* xmpFrame);
	XMPVMethod XMPShape* RequestFrameShape(XMPFrame* xmpFrame, XMPShape* frameShape);
	XMPVMethod XMPShape* AdjustBorderShape(XMPFrame* xmpFrame, XMPShape* shape);
	XMPVMethod void EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec);
	XMPVMethod void ViewTypeChanged(XMPFrame* frame);
	XMPVMethod void PresentationChanged(XMPFrame* frame);
	
	//  Binding 

	XMPVMethod void ExternalizeKinds(XMPTypeList* kindlist);
	XMPVMethod void ChangeKind(XMPType kind);

	//  Imaging 
	XMPVMethod void CanvasUpdated(XMPCanvas* canvas);
	XMPVMethod void Draw(XMPFacet* xmpFacet, XMPShape* invalidShape);
	XMPVMethod void HighlightChanged(XMPFacet* facet);
	XMPVMethod XMPULong GetPrintResolution(XMPFrame* xmpFrame);

	//  Linking 
	XMPVMethod XMPLink* CreateLink(XMPPtr data, XMPULong size);
	XMPVMethod void LinkUpdated(XMPLink* updatedLink, XMPChangeID id);
	XMPVMethod void RevealLink(XMPLinkSource* xmpLinkSource);

	XMPVMethod void	EmbeddedFrameChanged(XMPFrame* xmpFrame, XMPChangeID change);
	XMPVMethod void	LinkStatusChanged(XMPFrame* xmpFrame);

	//  Memory Management 
	XMPVMethod XMPSize Purge(XMPSize size);

	//  Part Activation 
	XMPVMethod XMPBoolean BeginRelinquishFocus(XMPTypeToken focus,
											   XMPFrame* ownerFrame,
											   XMPFrame* proposedFrame);
	XMPVMethod void CommitRelinquishFocus(XMPTypeToken focus, 
										  XMPFrame* ownerFrame,
										  XMPFrame* proposedFrame);
	XMPVMethod void AbortRelinquishFocus(XMPTypeToken focus,
										 XMPFrame* ownerFrame,
										 XMPFrame* proposedFrame);
	XMPVMethod void FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame);
	XMPVMethod void FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame);

	//  Storage 
	XMPVMethod void Externalize();
	XMPVMethod void CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU);

	//  UI Events 
	XMPVMethod XMPBoolean HandleEvent(XMPEventData event,  XMPFrame* xmpFrame,  XMPFacet* xmpFacet);
	XMPVMethod void MouseEnter(XMPFacet* xmpFacet, XMPPoint where);
	XMPVMethod void MouseWithin(XMPFacet* xmpFacet, XMPPoint where);
	XMPVMethod void MouseLeave(XMPFacet* xmpFacet);
	XMPVMethod void AdjustMenus(XMPFrame* xmpFrame);

	//  Undo 
	XMPVMethod void UndoAction(XMPActionData actionState);
	XMPVMethod void RedoAction(XMPActionData actionState);
	XMPVMethod void DisposeActionState(XMPActionData actionState,
									   XMPDoneState doneState);
	XMPVMethod void WriteActionState(XMPPtr actionState,
									 XMPStorageUnitView* storageUnitView);
	XMPVMethod XMPPtr ReadActionState(XMPStorageUnitView* storageUnitView);

//------------------------------------------------------------------------------
//	 New API
//
public:
	// ----- Storage Properties -----
	virtual void 			AddProperties(XMPStorageUnit* storageUnit);
	void					CheckAndAddProperties(XMPStorageUnit* storageUnit);
	
	// ----- Events -----
	virtual XMPBoolean 	HandleActivate(CFrame* frame, CFacet* facet, XMPBoolean state);

	virtual XMPBoolean 	HandleMenuEvent(CFrame* frame, XMPSLong message);
	virtual XMPBoolean 	DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	
	XMPVMethod XMPBoolean HandleIdle(CFrame* frame);
	XMPVMethod XMPBoolean DoIdle();

	// ----- Frame -----
	virtual CFrame* 		NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation) = 0;

	// ----- Menus -----
	virtual void 			InstallMenus(XMPMenuBar* menuBar);
	virtual void 			DoAdjustMenus(XMPMenuBar* menuBar);

	// ----- Validation/Invalidation -----
	void					InvalidateAllFrames(XMPTypeToken whichFrames, CFacet* skipFacet, XMPShape* invalidShape);
	void					ValidateAllFrames(XMPTypeToken whichFrames, CFacet* skipFacet, XMPShape* validShape);
	
	// ----- Presentation -----
	virtual XMPTypeToken	GetDefaultPresentation() const;
	
	// ----- Getters/Setters -----
	CFrame* 				GetActiveFrame() const;
	virtual void			SetActiveFrame(CFrame* activeFrame);
	CFacet* 				GetActiveFacet() const;
	XMPMenuBar* 			GetMenuBar() const;
	XMPMenuBar* 			GetPartMenuBar() const;
	OrderedCollection* 		GetDisplayFrameList() const;
	XMPULong				CountDisplayFrame() const;
	XMPSession* 			GetSession() const;
	
	XMPTypeToken			GetSelectionFocusToken() const;
	XMPTypeToken			GetMenuFocusToken() const;
	XMPTypeToken			GetKeyFocusToken() const;

	virtual XMPValueType	GetContentPropertyValueType() const;
	
	// ----- Window -----
	void					AddWindow(CWindow *window);
	void					RemoveWindow(CWindow *window);
	CWindow*				XMPtoFWWindow(XMPWindow *xmpWindow) const;
	void					HideShowFloatingWindows(XMPBoolean state);
	
	virtual XMPWindow*		CreateRootWindow(XMPISOStr windowTitle);
	
	XMPWindow*				CreateXMPWindow(CFrame* itsFrame,
											XMPTypeToken viewType,
											XMPTypeToken presentation,
											XMPISOStr windowTitle,
											const XMPRect& windowRect,
											XMPBoolean hasCloseBox,
											XMPBoolean isResizable,
											XMPBoolean isFloating,
											XMPBoolean shouldSave,
											XMPSShort procID);
											
	// ----- Saving -----
	void					Changed();
	
	// ----- Selection -----
	FW_CSelection*			GetSelection() const;
	virtual FW_CSelection*	NewSelection();
	
	// ----- Utilities -----
	XMPBoolean				HasPropertyOnClipboard(XMPPropertyName propertyName, XMPValueType type);

	// ----- Linking -----
	void					SetPendingPublish(CPublishLink* publishLink);
	void					DeletePendingPublish();
	
	void					AddToPublishLinkList(CPublishLink* publishLink);
	void					RemoveFromPublishLinkList(CPublishLink* publishLink);

	void					AddToSubscribLinkList(CSubscribLink* subscribLink);
	void					RemoveFromSubscribLinkList(CSubscribLink* subscribLink);
	
	// ----- Drag&Drop -----
	virtual void			Dragging(CFacet *facet, XMPBoolean dragging);
	
	// ----- Externalize Content -----
	virtual void			ExternalizeContent(XMPStorageUnit* storageUnit);
	virtual void			InternalizeContent(XMPStorageUnit* storageUnit);

	// ----- Cloning -----
	virtual XMPPart*		ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind);
	
	// ----- Can I embed? -----
	// This is temporary. It needs to be removed
	virtual XMPBoolean		IsEmbeddingPart() const;
	
protected:
	// ----- Framework data specific Ex/Internalize -----
	void					ExternalizeDisplayFrames(XMPStorageUnit* storageUnit);
	void					InternalizeDisplayFrames(XMPStorageUnit* storageUnit);
	
	// ----- Linking -----
	CSubscribLink*			XMPtoSubscribLink(XMPLink* xmpLink);

//------------------------------------------------------------------------------
//	 Data members
//
private:
	XMPMenuBar*			fPartMenuBar;
	XMPMenuBar*			fMenuBar;			// This is the current menubar 
	CFrame* 			fActiveFrame;
	OrderedCollection 	*fDisplayFrames;	
	OrderedCollection 	*fWindowList;
	
	XMPTypeToken		fSelectionFocusToken;
	XMPTypeToken		fMenuFocusToken;
	XMPTypeToken		fKeyFocusToken;
	
	FW_CSelection*		fSelection;

	CPublishLink*		fPendingPublishLink;
	OrderedCollection*	fPublishLinkList;
	OrderedCollection*	fSubscribLinkList;
	
	XMPBoolean			fUserPropertiesAdded;
};



//========================================================================================
//	 class CPartDisplayFrameIterator
//========================================================================================

class CPartDisplayFrameIterator
{
public:
	CPartDisplayFrameIterator(CPart* part, XMPTypeToken presentation = 0);
	virtual ~CPartDisplayFrameIterator();

	CFrame*			First();
	CFrame*			Next();
	XMPBoolean		IsNotComplete();

private:
	void			FindNext();
	
private:
	OrderedCollectionIterator	fIterator;
	XMPTypeToken				fPresentation;
	CFrame*						fCurrentFrame;
};



//========================================================================================
//	 class CPartFacetIterator
//========================================================================================

class CPartFacetIterator
{
public:
	CPartFacetIterator(CPart* part, XMPTypeToken presentation = 0);
	virtual ~CPartFacetIterator();

	CFacet*			First();
	CFacet*			Next();
	XMPBoolean		IsNotComplete();

private:
	void			NewFacetIterator();
	
private:
	CPartDisplayFrameIterator	fFrameIterator;
	CFrameFacetIterator*		fFacetIterator;
	CFrame*						fCurrentFrame;
	CFacet*						fCurrentFacet;
};


//========================================================================================
//	 Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	 CPart::GetActiveFrame
//----------------------------------------------------------------------------------------
inline CFrame* CPart::GetActiveFrame() const
{
	return fActiveFrame;
}

//----------------------------------------------------------------------------------------
//	 CPart::GetMenuBar
//----------------------------------------------------------------------------------------
inline XMPMenuBar* CPart::GetMenuBar() const
{
	return fMenuBar;
}

//----------------------------------------------------------------------------------------
//	 CPart::GetMenuBar
//----------------------------------------------------------------------------------------
inline XMPMenuBar* CPart::GetPartMenuBar() const
{
	return fPartMenuBar;
}

//----------------------------------------------------------------------------------------
//	 CPart::GetDisplayFrameList
//----------------------------------------------------------------------------------------
inline OrderedCollection* CPart::GetDisplayFrameList() const
{
	return fDisplayFrames;
}

//----------------------------------------------------------------------------------------
//	 CPart::GetSession
//----------------------------------------------------------------------------------------
inline XMPSession* CPart::GetSession() const
{
	XMPPart* self = (XMPPart *) this;
	return self->GetStorageUnit()->GetSession();
}

//----------------------------------------------------------------------------------------
//	 CPart::GetSelectionFocusToken
//----------------------------------------------------------------------------------------
inline XMPTypeToken CPart::GetSelectionFocusToken() const
{
	return fSelectionFocusToken;
}

//----------------------------------------------------------------------------------------
//	 CPart::GetMenuFocusToken
//----------------------------------------------------------------------------------------
inline XMPTypeToken CPart::GetMenuFocusToken() const
{
	return fMenuFocusToken;
}

//----------------------------------------------------------------------------------------
//	 CPart::GetKeyFocusToken 
//----------------------------------------------------------------------------------------
inline XMPTypeToken CPart::GetKeyFocusToken() const
{
	return fKeyFocusToken;
}

//------------------------------------------------------------------------------
//	 CPart::GetSelection
//------------------------------------------------------------------------------
inline FW_CSelection* CPart::GetSelection() const
{
	return fSelection;
}

//------------------------------------------------------------------------------
//	 CPart::SetPendingPublish
//------------------------------------------------------------------------------
inline void CPart::SetPendingPublish(CPublishLink* publishLink)
{
	fPendingPublishLink = publishLink;
}

//------------------------------------------------------------------------------
//	 CPart::CountDisplayFrame
//------------------------------------------------------------------------------
inline XMPULong CPart::CountDisplayFrame() const
{
	return fDisplayFrames->Count();
}

#endif



