/*
	File:		FWParting.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 4/12/94	DS		Implement RequestFrameShape
		 <4>	 3/31/94	DS		Respond to API changes to RevealFrame,
									CreateMebeddedFramesIterator, and
									Highlight..
		<2+>	 3/17/94	DS		Make the constructor for the iterator for
									this file take CEmbeddingPart * as a const
									parameter. SC seems to care about this. It
									compiles without it under CFront just fine.
		 <2>	 3/14/94	DS		Add support for the insert menu item..

	To Do:
*/

//	File:		FWParting.cpp
//	Contains:	Implementation for class CEmbeddingPart
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWPARTING_
#include "FWParting.h"
#endif

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _SUCURSOR_
#include <SUCursor.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _POINT_
#include <Point.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _WINSTAT_
#include <WinStat.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

// ----- Macintosh Includes -----

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>
#endif

#pragma segment FWPartSeg

//==============================================================================
//	 class CPart
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddingPart::CEmbeddingPart
//------------------------------------------------------------------------------

CEmbeddingPart::CEmbeddingPart()
{
	fProxyRunList = kXMPNULL;
	fProxyRunCount = 0;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::~CEmbeddingPart
//------------------------------------------------------------------------------

CEmbeddingPart::~CEmbeddingPart()
{
	delete fProxyRunList;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::Release
//------------------------------------------------------------------------------

void CEmbeddingPart::Release()
{
	CPart::Release();	
	
	if (this->GetRefCount() == 0)
	{
		WASSERTM(fProxyRunCount == 0, "All proxyRuns should have been deleted");
		
		// Can't call fProxyRunList->DeleteAll because proxyRuns remove themself 
		// automatically from the list
		CProxyRun* proxyRun;
		while ((proxyRun = (CProxyRun*)fProxyRunList->First()) != kXMPNULL)
		{
			delete proxyRun;
		}
		fProxyRunCount = 0;
	}	
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::Initialize
//------------------------------------------------------------------------------

void CEmbeddingPart::Initialize()
{
	CPart::Initialize();
	
	fProxyRunList = new OrderedCollection;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::AttachSourceFrame
//------------------------------------------------------------------------------

void CEmbeddingPart::AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame)
{
	if (xmpSourceFrame && xmpFrame)
	{
		CEmbeddingFrame* sourceFrame = (CEmbeddingFrame*)CFrame::XMPtoFWFrame(xmpSourceFrame);	
		CEmbeddingFrame* frame = (CEmbeddingFrame*)CFrame::XMPtoFWFrame(xmpFrame);
		frame->AttachSourceFrame(sourceFrame);
	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::CreateEmbeddedFramesIterator
//------------------------------------------------------------------------------

XMPEmbeddedFramesIterator* CEmbeddingPart::CreateEmbeddedFramesIterator(XMPFrame* frame)
{
	return new CEmbeddedXMPFramesIterator(this);
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::UsedShapeChanged
//------------------------------------------------------------------------------

void CEmbeddingPart::UsedShapeChanged(XMPFrame* embeddedXMPFrame)
{
	CEmbeddingFrame *containingFrame = (CEmbeddingFrame*)CFrame::XMPtoFWFrame(embeddedXMPFrame->GetContainingFrame());	
	CProxyRun *proxyRun = XMPFrametoProxyRun(embeddedXMPFrame);	

	containingFrame->UsedShapeChanged(proxyRun, embeddedXMPFrame);
	proxyRun->UsedShapeChanged(embeddedXMPFrame);
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::EmbedPart
//------------------------------------------------------------------------------

void CEmbeddingPart::EmbedPart(XMPPart *embeddedXMPPart,
								CProxyRun* proxyRun, 
								XMPTypeToken inWhichPresentation,
								XMPTypeToken viewType,
								XMPTypeToken presentation,
								XMPTransform* externalXForm, 
								XMPShape* frameShape,
								XMPBoolean invalidate)
{	
	// ----- Create a shape if neccessary -----
	XMPShape *invalidShape = ::NewXMPShape();

	// ----- Create an embedded frame for each of my display frame -----		
	CPartDisplayFrameIterator ite(this);
	XMPFrame* sourceFrame = kXMPNULL;
	for (CEmbeddingFrame* frame = (CEmbeddingFrame*)ite.First(); ite.IsNotComplete(); frame = (CEmbeddingFrame*)ite.Next())
	{
		if (inWhichPresentation == kNoPresentation || frame->GetPresentation() == inWhichPresentation)
		{
			// ----- Create the frame -----
			XMPShape *itsShape = ::NewXMPShape(frameShape);
			CProxyFrame* proxyFrame = frame->CreateEmbeddedFrame(embeddedXMPPart, proxyRun, itsShape, viewType, presentation, 0, kXMPFalse);
						
			// ----- Attach source frames if needed -----
			if (sourceFrame)
				embeddedXMPPart->AttachSourceFrame(proxyFrame->GetXMPFrame(), sourceFrame);
			sourceFrame = proxyFrame->GetXMPFrame();
	
			// ----- Create its facets -----
			CFrameFacetIterator ite2(frame);
			for (CFacet* facet = ite2.First(); ite2.IsNotComplete(); facet = ite2.Next())
			{
				XMPShape* clip = ::NewXMPShape();
				proxyFrame->GetFrameShape(clip);
				XMPTransform* xform = ::NewXMPTransform(externalXForm);
				
				XMPBoolean svRedraw = facet->SetRedraw(invalidate);
				
				facet->CreateEmbeddedFacet(proxyFrame, clip, xform, kXMPNULL, kXMPFrameInFront);
				facet->ClipEmbeddedFacets();

				proxyFrame->GetFrameShape(invalidShape); 	// May have changed when the facet was added
				invalidShape->Transform(externalXForm);
				facet->Invalidate(invalidShape);

				facet->SetRedraw(svRedraw);
			}
			
		}
	}
	
	delete invalidShape;
		
	delete frameShape;
	delete externalXForm;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::EmbedFrame
//------------------------------------------------------------------------------

void CEmbeddingPart::EmbedFrame(XMPFrame* embeddedXMPFrame,
								CProxyRun* proxyRun, 
								XMPTypeToken inWhichPresentation, 		
								XMPTransform* externalXForm, 
								XMPBoolean invalidate)
{	
	XMPPart *embeddedXMPPart = embeddedXMPFrame->GetPart();
	
	// ----- Create a shape if neccessary -----
	XMPShape *invalidShape = ::NewXMPShape();
	
	// ----- Get Info from embeddedXMPFrame -----
	XMPShape* frameShape = ::NewXMPShape(embeddedXMPFrame->GetFrameShape());
	XMPTypeToken viewType = embeddedXMPFrame->GetViewType();
	XMPTypeToken presentation = embeddedXMPFrame->GetPresentation();
	
	// ----- Create an embedded frame for each of my display frame -----		
	CPartDisplayFrameIterator ite(this);
	XMPFrame* sourceFrame = kXMPNULL;
	for (CEmbeddingFrame* frame = (CEmbeddingFrame*)ite.First(); ite.IsNotComplete(); frame = (CEmbeddingFrame*)ite.Next())
	{
		if (inWhichPresentation == kNoPresentation || frame->GetPresentation() == inWhichPresentation)
		{
			CProxyFrame* proxyFrame;
			
			// ----- Create the frame -----
			if (embeddedXMPFrame)
			{
				proxyFrame = proxyRun->NewProxyFrame(embeddedXMPFrame);
				embeddedXMPFrame->Release();
				embeddedXMPFrame->SetContainingFrame(frame->GetXMPFrame());
				frame->AddProxyFrame(proxyFrame);
				embeddedXMPFrame = kXMPNULL;
			}
			else
			{
				XMPShape *itsShape = ::NewXMPShape(frameShape);
				proxyFrame = frame->CreateEmbeddedFrame(embeddedXMPPart, proxyRun, 
														itsShape, viewType, presentation, 
														0, kXMPFalse);
			}
			
			// ----- Attach source frames if needed -----
			if (sourceFrame)
				embeddedXMPPart->AttachSourceFrame(proxyFrame->GetXMPFrame(), sourceFrame);
			sourceFrame = proxyFrame->GetXMPFrame();
	
			// ----- Create its facets -----
			CFrameFacetIterator ite2(frame);
			for (CFacet* facet = ite2.First(); ite2.IsNotComplete(); facet = ite2.Next())
			{
				XMPShape* clip = ::NewXMPShape();
				proxyFrame->GetFrameShape(clip);
				XMPTransform* xform = ::NewXMPTransform(externalXForm);
				
				XMPBoolean svRedraw = facet->SetRedraw(invalidate);
				
				facet->CreateEmbeddedFacet(proxyFrame, clip, xform, kXMPNULL, kXMPFrameInFront);
				facet->ClipEmbeddedFacets();

				proxyFrame->GetFrameShape(invalidShape); 	// May have changed when the facet was added
				invalidShape->Transform(externalXForm);
				facet->Invalidate(invalidShape);

				facet->SetRedraw(svRedraw);
			}
			
		}
	}
	
	delete invalidShape;
		
	delete frameShape;
	delete externalXForm;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::CreateEmbeddedFrame
//------------------------------------------------------------------------------

XMPFrame* CEmbeddingPart::CreateEmbeddedFrame(XMPFrame* containingFrame,
												XMPShape* frameShape,
												XMPPart* embeddedXMPPart,
												XMPTypeToken viewType,
												XMPTypeToken presentation,
												XMPID frameGroupID,
												XMPBoolean isOverlaid)
{
	CEmbeddingFrame* frame = (CEmbeddingFrame*)CFrame::XMPtoFWFrame(containingFrame);
	
	
	// ----- Create ProxyRun -----
	CProxyRun* proxyRun = NewProxyRun();
	
	// ----- Create and return the embedded frame -----
	CProxyFrame* proxyFrame = frame->CreateEmbeddedFrame(embeddedXMPPart, proxyRun, frameShape, 
													     viewType, presentation, frameGroupID, 
														 isOverlaid);
	return proxyFrame->GetXMPFrame();
}
	
//------------------------------------------------------------------------------
//	 CEmbeddingPart::RemoveEmbeddedFrame
//------------------------------------------------------------------------------

void CEmbeddingPart::RemoveEmbeddedFrame(XMPFrame* xmpFrame)
{
	XMPFrame* containingXMPFrame = xmpFrame->GetContainingFrame();
	CEmbeddingFrame* containingFrame = (CEmbeddingFrame*)CFrame::XMPtoFWFrame(containingXMPFrame);
	CProxyFrame* proxyFrame = containingFrame->XMPtoProxyFrame(xmpFrame);
	
//	CProxyRun* proxyRun = proxyFrame->GetProxyRun();
	
	containingFrame->RemoveEmbeddedFrame(proxyFrame);	// Note: will remove the XMPEmbeddedFrame from the draft
	
	// Note: the proxyRun will be automatically removed (and deleted) when the last proxyFrame
	// is deleted
//	// ----- if was the last proxyFrame delete the proxyRun -----
//	if (proxyRun->CountProxyFrames() == 0)
//	{
//		fProxyRunList->Remove(proxyRun);
//		delete proxyRun;
//	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::HandleEvent
//------------------------------------------------------------------------------

XMPBoolean CEmbeddingPart::HandleEvent(XMPEventData event, XMPFrame* xmpFrame, XMPFacet* xmpFacet)
{
	XMPBoolean handled = false;
	CFacet* facet = CFacet::XMPtoFWFacet(xmpFacet);
	
	switch (event->what)
	{
		case kXMPEvtMouseUpEmbedded:
			break;
			
		case kXMPEvtMouseUpBorder:
			break;

		case kXMPEvtMouseDownEmbedded:
			handled = facet->HandleMouseDown((XMPFacet*)event->message, event);
			break;
			
		case kXMPEvtMouseDownBorder:
			handled = facet->HandleMouseDownInEmbeddedFrameBorder((XMPFacet*)event->message, event);
			break;
		
		default:
			return CPart::HandleEvent(event, xmpFrame, xmpFacet);
	}
	
	return handled;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::DoAdjustMenus
//------------------------------------------------------------------------------

void CEmbeddingPart::DoAdjustMenus(XMPMenuBar* menuBar)
{
	CPart::DoAdjustMenus(menuBar);

	menuBar->EnableCommand(kXMPCommandInsert, kXMPTrue);	// Should this really happen here? Maybe should have flag..	
	menuBar->EnableCommand(kXMPCommandGetPartInfo, HasSelectedProxyRun());	
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::DoMenuEvent
//------------------------------------------------------------------------------

XMPBoolean CEmbeddingPart::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	XMPBoolean result = kXMPTrue;
	FW_CSelection* selection = kXMPNULL;

	switch (commandID)
	{
		case kXMPCommandInsert:
			selection = GetSelection();
			if (selection != kXMPNULL)
				result = selection->InsertNewPart();
			else
				SysBeep(1);
			break;
			
		default:
			result = CPart::DoMenuEvent(menuBar, commandID);
	};
	
	return result;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::ClipEmbeddedFrames
//------------------------------------------------------------------------------

void CEmbeddingPart::ClipEmbeddedFrames(XMPTypeToken whichFrames)
{
	CPartDisplayFrameIterator ite(this, whichFrames);
	for (CEmbeddingFrame* frame = (CEmbeddingFrame*)ite.First(); ite.IsNotComplete(); frame = (CEmbeddingFrame*)ite.Next())
	{
		frame->ClipEmbeddedFrames();
	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::CloneFrameFrom
//------------------------------------------------------------------------------

XMPFrame* CEmbeddingPart::CloneFrameFrom(XMPStorageUnit* storageUnit, XMPStorageUnitRef frameSURef, XMPCloneKind kind)
{
	XMPDraft* myDraft = GetStorageUnit()->GetDraft();
	XMPStorageUnit* newSU = myDraft->CreateStorageUnit();
	
	XMPDraft* srcDraft = storageUnit->GetDraft();
	XMPStorageUnit* srcSU = srcDraft->GetStorageUnit(storageUnit->GetIDFromStorageUnitRef(frameSURef));

	XMPDraftKey key = srcDraft->BeginClone(kind);
	srcSU->CloneInto(key, newSU, kXMPNULL);
	srcDraft->EndClone(key);

	srcSU->Release();

	XMPFrame* xmpFrame = myDraft->GetFrame(newSU->GetID());
	
	newSU->Release();
	
	return xmpFrame;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::XMPFrametoProxyRun
//------------------------------------------------------------------------------

CProxyRun* CEmbeddingPart::XMPFrametoProxyRun(XMPFrame* embeddedXMPFrame) const
{
	CEmbeddingPartProxyRunIterator ite(this);
	for (CProxyRun* proxyRun = ite.First(); ite.IsNotComplete(); proxyRun = ite.Next())
	{
		if (proxyRun->GetXMPPart() == embeddedXMPFrame->GetPart())
		{
			CProxyRunCProxyFrameIterator ite2(proxyRun);
			for (CProxyFrame* proxyFrame = ite2.First(); ite2.IsNotComplete(); proxyFrame = ite2.Next())
			{
				if (proxyFrame->GetXMPFrame() == embeddedXMPFrame)
					return proxyRun;
			}
		}
	}
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::XMPtoProxyFrame
//------------------------------------------------------------------------------

CProxyFrame* CEmbeddingPart::XMPtoProxyFrame(XMPFrame* embeddedXMPFrame) const
{
	CEmbeddingPartProxyRunIterator ite(this);
	for (CProxyRun* proxyRun = ite.First(); ite.IsNotComplete(); proxyRun = ite.Next())
	{
		if (proxyRun->GetXMPPart() == embeddedXMPFrame->GetPart())
		{
			CProxyRunCProxyFrameIterator ite2(proxyRun);
			for (CProxyFrame* proxyFrame = ite2.First(); ite2.IsNotComplete(); proxyFrame = ite2.Next())
			{
				if (proxyFrame->GetXMPFrame() == embeddedXMPFrame)
					return proxyFrame;
			}
		}
	}
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::HasSelectedProxyRun
//------------------------------------------------------------------------------

XMPBoolean CEmbeddingPart::HasSelectedProxyRun() const
{
	XMPBoolean result = kXMPFalse;
	CEmbeddingPartProxyRunIterator ite(this);
	for (CProxyRun* proxyRun = ite.First(); ite.IsNotComplete(); proxyRun = ite.Next())
	{
		if (proxyRun->IsSelected())
		{
			result = kXMPTrue;
			break;
		}
	}
	return result;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::SetEmbeddedFramesContainingFrame
//------------------------------------------------------------------------------
//	This is call by AddDisplayFrame when the first display frame is added. The part
//	may have orphaned embedded frames that we have to attach to the frame passed as
// 	parameter

void CEmbeddingPart::SetEmbeddedFramesContainingFrame(CEmbeddingFrame* frame)
{
	CEmbeddedXMPFramesIterator ite(this);
	for (XMPFrame* xmpFrame = ite.First(); ite.IsNotComplete(); xmpFrame = ite.Next())
	{
		if (xmpFrame->GetContainingFrame() == kXMPNULL)
		{
			xmpFrame->SetContainingFrame(frame->GetXMPFrame());
			CProxyFrame* proxyFrame = XMPtoProxyFrame(xmpFrame);
			frame->AddProxyFrame(proxyFrame);
		}
	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::RemoveEmbeddedFrames
//------------------------------------------------------------------------------

void CEmbeddingPart::RemoveEmbeddedFrames(CProxyRun* proxyRun)
{
	//  We have to collect first all the embedded frames
	//  Because the proxyRun will be deleted when the last proxyFrame
	//  will be
	OrderedCollection* tempCollect = new OrderedCollection;
	
	CProxyRunCProxyFrameIterator ite(proxyRun);
	for (CProxyFrame* proxyFrame = ite.First(); ite.IsNotComplete(); proxyFrame = ite.Next())
	{
		tempCollect->AddLast(proxyFrame);
	}
	
	// ----- Now we can call RemoveEmbeddedFrame -----
	OrderedCollectionIterator ite2(tempCollect);
	for (proxyFrame = (CProxyFrame*)ite2.First(); ite2.IsNotComplete(); proxyFrame = (CProxyFrame*)ite2.Next())
	{
		this->RemoveEmbeddedFrame(proxyFrame->GetXMPFrame());
	}

	tempCollect->RemoveAll();
	delete tempCollect;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::StartDragging
//------------------------------------------------------------------------------

void CEmbeddingPart::Dragging(CFacet* facet, XMPBoolean state)
{
	CFrame* frame = facet->GetFrame();
	
	CEmbeddingPartProxyRunIterator ite1(this);
	for (CProxyRun* proxyRun = ite1.First(); ite1.IsNotComplete(); proxyRun = ite1.Next())
	{
		if (proxyRun->IsSelected())
		{
			CProxyRunCProxyFrameIterator ite(proxyRun);
			for (CProxyFrame *proxyFrame = ite.First(); ite.IsNotComplete(); proxyFrame = ite.Next())
			{
				if (proxyFrame->GetContainingFrame() == frame)
					proxyFrame->SetDragging(state);
			}
		}
	}
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::ClonePartFrom
//------------------------------------------------------------------------------

XMPPart* CEmbeddingPart::ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind)
{
	XMPDraft* myDraft = GetStorageUnit()->GetDraft();
	XMPDraft* srcDraft = storageUnit->GetDraft();

	XMPDraftKey key = srcDraft->BeginClone(kind);
	XMPStorageUnit* newSU = storageUnit->CloneTo(key, myDraft, kXMPNULL);
	srcDraft->EndClone(key);
	
	XMPPart* xmpPart = myDraft->GetPart(newSU->GetID());
	
	newSU->Release();
	
	return xmpPart;	
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::IsEmbeddingPart
//------------------------------------------------------------------------------
//	This needs to be removed

XMPBoolean CEmbeddingPart::IsEmbeddingPart() const
{
	return kXMPTrue;
}

//==============================================================================
//	 CEmbeddedXMPFramesIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFramesIterator::First
//------------------------------------------------------------------------------

XMPFrame* CEmbeddedXMPFramesIterator::First()
{
	delete fProxyRunIter;
	fProxyRunIter = kXMPNULL;
	
	delete fProxyFrameIter;
	fProxyFrameIter = kXMPNULL;

	fProxyRunIter = new CEmbeddingPartProxyRunIterator(fPart);
	fCurrentProxyRun = fProxyRunIter->First();

	if (fCurrentProxyRun)
	{
		fProxyFrameIter = new CProxyRunCProxyFrameIterator(fCurrentProxyRun);
		fCurrentProxyFrame = fProxyFrameIter->First();
	}

	return fCurrentProxyFrame ? fCurrentProxyFrame->GetXMPFrame() : (XMPFrame*)kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFramesIterator::Next
//------------------------------------------------------------------------------

XMPFrame* CEmbeddedXMPFramesIterator::Next()
{
	if (fProxyFrameIter)
	{
		fCurrentProxyFrame = fProxyFrameIter->Next();
		if (fCurrentProxyFrame == kXMPNULL)
		{
			delete fProxyFrameIter;
			fProxyFrameIter = kXMPNULL;
			
			fCurrentProxyFrame = kXMPNULL;
			
			fCurrentProxyRun = fProxyRunIter->Next();
			if (fCurrentProxyRun)
			{
				fProxyFrameIter = new CProxyRunCProxyFrameIterator(fCurrentProxyRun);
				fCurrentProxyFrame = fProxyFrameIter->First();
			}
		}
	}
	
	return fCurrentProxyFrame ? fCurrentProxyFrame->GetXMPFrame() : (XMPFrame*)kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFramesIterator::IsNotComplete
//------------------------------------------------------------------------------

XMPBoolean	CEmbeddedXMPFramesIterator::IsNotComplete()
{
	return fCurrentProxyFrame != kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFramesIterator::CEmbeddedXMPFramesIterator
//------------------------------------------------------------------------------

CEmbeddedXMPFramesIterator::CEmbeddedXMPFramesIterator(CEmbeddingPart* part)
{
	fPart = part;
	fCurrentProxyRun = kXMPNULL;
	fCurrentProxyFrame = kXMPNULL;
	fProxyRunIter = kXMPNULL;
	fProxyFrameIter = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFramesIterator::~CEmbeddedXMPFramesIterator
//------------------------------------------------------------------------------

CEmbeddedXMPFramesIterator::~CEmbeddedXMPFramesIterator()
{
	delete fProxyRunIter;
	delete fProxyFrameIter;
}

//==============================================================================
//	 CEmbeddingPartProxyRunIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddingPartProxyRunIterator::CEmbeddingPartProxyRunIterator
//------------------------------------------------------------------------------

CEmbeddingPartProxyRunIterator::CEmbeddingPartProxyRunIterator(const CEmbeddingPart* part) :
	fIterator(part->GetProxyRunList())
{
}

//------------------------------------------------------------------------------
//	 CEmbeddingPartProxyRunIterator::~CEmbeddingPartProxyRunIterator
//------------------------------------------------------------------------------

CEmbeddingPartProxyRunIterator::~CEmbeddingPartProxyRunIterator()
{
}
