/*
	File:		FWParting.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 4/12/94	DS		Implement RequestFrameShape
		 <4>	 3/31/94	DS		Respond to API changes to RevealFrame,
									CreateMebeddedFramesIterator, and
									Highlight..
		<2+>	 3/17/94	DS		Add CDECL to Embedding part dtor
		 <2>	 3/14/94	DS		add DoMenuEvent

	To Do:
*/

//	File:		FWParting.h
//	Contains:	Definition for class CEmbeddingPart
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWPARTING_
#define _FWPARTING_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

//==============================================================================
// Forward Declaration
//==============================================================================

class XMPFrame;
class XMPMenuBar;
class XMPFocusSet;
class CEmbeddingFrame;
class CProxyFrame;
class CProxyRun;
class CEmbeddingPartProxyRunIterator;
class CProxyRunCProxyFrameIterator;

//==============================================================================
//	 class CEmbeddingPart
//==============================================================================

#define kCEmbeddingPartID "appl:CEmbeddingPart$class,1.0.0"

class CEmbeddingPart : public CPart
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
protected:
	CEmbeddingPart();
	virtual ~ CDECL CEmbeddingPart();

	virtual void Initialize();
	
	virtual void		Release();

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	// ----- Embedding ----- 
	XMPVMethod XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(XMPFrame* frame);

	// ----- Frame ----- 	
	XMPVMethod XMPFrame* CreateEmbeddedFrame(XMPFrame* containingFrame,
	 											XMPShape* frameShape,
	 											XMPPart* embedPart,
												XMPTypeToken viewType,
												XMPTypeToken presentation,
	 											XMPID frameGroupID,
											 	XMPBoolean isOverlaid);
																	
	XMPVMethod void 		RemoveEmbeddedFrame(XMPFrame* frame);

	XMPVMethod void		UsedShapeChanged(XMPFrame* xmpEmbeddedFrame);
	XMPVMethod void 		AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame);
	
	// ----- UI Events ----- 
	XMPVMethod void			DoAdjustMenus(XMPMenuBar* menuBar);
	XMPVMethod XMPBoolean 	DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	XMPVMethod XMPBoolean 	HandleEvent(XMPEventData event, XMPFrame* xmpFrame, XMPFacet* xmpFacet);

//------------------------------------------------------------------------------
//	 New API
//
public:
	// ----- Embedding -----
	void						EmbedPart(XMPPart *embeddedXMPPart,
											CProxyRun* proxyRun, 
											XMPTypeToken inWhichPresentation, 		
											XMPTypeToken viewType,
											XMPTypeToken presentation,
											XMPTransform* externalXForm, 
											XMPShape* frameShape,
											XMPBoolean invalidate);
	void						EmbedFrame(XMPFrame* embeddedXMPFrame,
										  CProxyRun* proxyRun, 
										  XMPTypeToken inWhichPresentation, 		
										  XMPTransform* externalXForm, 
										  XMPBoolean invalidate);
											
	void 						ClipEmbeddedFrames(XMPTypeToken whichFrames);
	void						RemoveEmbeddedFrames(CProxyRun *proxyRun);
	
	// ----- Cloning -----
	XMPFrame*					CloneFrameFrom(XMPStorageUnit* storageUnit, XMPStorageUnitRef frameSURef, XMPCloneKind kind);
	virtual XMPPart*			ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind);
	
	
	// ----- Conversion -----
	CProxyRun*					XMPFrametoProxyRun(XMPFrame* embeddedXMPFrame) const;
	CProxyFrame*				XMPtoProxyFrame(XMPFrame* embeddedXMPFrame) const;
	
	XMPBoolean					HasSelectedProxyRun() const;
	
	// ----- ProxyRun List -----
	virtual CProxyRun* 		NewProxyRun() = 0;	
	void						AddProxyRun(CProxyRun *proxyRun);
	void						RemoveProxyRun(CProxyRun *proxyRun);
	OrderedCollection*			GetProxyRunList() const
									{return fProxyRunList;}											

	// -----
	void						SetEmbeddedFramesContainingFrame(CEmbeddingFrame* frame);
	
	// ----- Drag&Drop -----
	virtual void				Dragging(CFacet *facet, XMPBoolean state);

	// ----- Can I embed? -----
	// This is temporary. It needs to be removed
	virtual XMPBoolean			IsEmbeddingPart() const;

//------------------------------------------------------------------------------
//	 Data Members
//
private:
	OrderedCollection		*fProxyRunList;
	XMPULong				fProxyRunCount;
};

//==============================================================================
//	 class CEmbeddedXMPFramesIterator
//==============================================================================

class CEmbeddedXMPFramesIterator : public XMPEmbeddedFramesIterator
{
public:
	CEmbeddedXMPFramesIterator(CEmbeddingPart* part);
	virtual ~CEmbeddedXMPFramesIterator();

	virtual XMPFrame*		First();
	virtual XMPFrame*		Next();
	virtual XMPBoolean		IsNotComplete();

	CProxyFrame*			GetCurrentProxyFrame() const
									{return fCurrentProxyFrame;}
	
private:
	CEmbeddingPart*					fPart;
	CProxyRun* 						fCurrentProxyRun;
	CProxyFrame*					fCurrentProxyFrame;
	CEmbeddingPartProxyRunIterator	*fProxyRunIter;
	CProxyRunCProxyFrameIterator	*fProxyFrameIter;
};

//==============================================================================
//	 CEmbeddingPartProxyRunIterator
//==============================================================================

class CEmbeddingPartProxyRunIterator
{
public:
	CEmbeddingPartProxyRunIterator(const CEmbeddingPart* part);
	virtual ~CEmbeddingPartProxyRunIterator();

	CProxyRun*		First()
						{return (CProxyRun*)fIterator.First();}
	CProxyRun*		Next()
						{return (CProxyRun*)fIterator.Next();}
	XMPBoolean		IsNotComplete()
						{return fIterator.IsNotComplete();}

private:
	OrderedCollectionIterator	fIterator;
};

//==============================================================================
//	 CEmbeddingPart Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddingPart::AddProxyRun
//------------------------------------------------------------------------------
inline void CEmbeddingPart::AddProxyRun(CProxyRun* proxyRun)
{
	fProxyRunList->AddLast(proxyRun);
	fProxyRunCount++;
}

//------------------------------------------------------------------------------
//	 CEmbeddingPart::RemoveProxyRun
//------------------------------------------------------------------------------
inline void CEmbeddingPart::RemoveProxyRun(CProxyRun* proxyRun)
{
	fProxyRunList->Remove(proxyRun);
	fProxyRunCount--;
}


#endif
