//	File:		FRProxy.cpp
//	Contains:	Implementation of class CProxyFrame
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

#ifndef _FWPARTING_
#include "FWParting.h"
#endif

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWBORDER_
#include "FWBorder.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes
#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _XMPMEM_
#include <XMPMem.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#pragma segment FWFrameSeg

//==============================================================================
//	 class CProxyRun
//==============================================================================

//------------------------------------------------------------------------------
//	 CProxyRun::CProxyRun
//------------------------------------------------------------------------------

CProxyRun::CProxyRun()
{
	fProxyFrameList 	= kXMPNULL;
	fProxyBorder		= kXMPNULL;
	fSelected			= kXMPFalse;
	fCachedProxyFrame 	= kXMPNULL;
	fCount				= 0;
}

//------------------------------------------------------------------------------
//	 CProxyRun::InitProxyRun
//------------------------------------------------------------------------------

void CProxyRun::InitProxyRun(CEmbeddingPart* embeddingPart)
{
	fEmbeddingPart = embeddingPart;
	fEmbeddingPart->AddProxyRun(this);
	
	fProxyFrameList = new OrderedCollection;
	
	fProxyBorder = NewProxyBorder();
}

//------------------------------------------------------------------------------
//	 CProxyRun::~CProxyRun
//------------------------------------------------------------------------------

CProxyRun::~CProxyRun()
{
	if (fEmbeddingPart)
		fEmbeddingPart->RemoveProxyRun(this);
		
	delete fProxyBorder;
	
	if (fProxyFrameList)
	{
		WASSERTM(fCount == 0, "All proxyFrame should be already deleted");
		fProxyFrameList->DeleteAll();
		fCount = 0;
		delete fProxyFrameList;
	}
}

//------------------------------------------------------------------------------
//	 CProxyRun::AddProxyFrame
//------------------------------------------------------------------------------

void CProxyRun::AddProxyFrame(CProxyFrame* proxyFrame)
{
	if (fCachedProxyFrame == kXMPNULL)
		fCachedProxyFrame = proxyFrame;
		
	fProxyFrameList->AddLast(proxyFrame);
	
	fCount++;
}

//------------------------------------------------------------------------------
//	 CProxyRun::RemoveProxyFrame
//------------------------------------------------------------------------------

void CProxyRun::RemoveProxyFrame(CProxyFrame* proxyFrame)
{
	if (fCachedProxyFrame == proxyFrame)
		fCachedProxyFrame = kXMPNULL;
		
	fProxyFrameList->Remove(proxyFrame);
	
	fCount--;
	
	if (fCount == 0)
		delete this;
}

//------------------------------------------------------------------------------
//	 CProxyRun::UsedShapeChanged
//------------------------------------------------------------------------------

void CProxyRun::UsedShapeChanged(XMPFrame* xmpFrame)
{
XMPUnused(xmpFrame);

	fProxyBorder->BorderHasChanged(kXMPTrue);
}

//------------------------------------------------------------------------------
//	 CProxyRun::WhichEmbeddedFacet
//------------------------------------------------------------------------------
// mouse in window coordinate system

XMPFacet* CProxyRun::WhichEmbeddedFacet(CFacet* embeddingFacet, const XMPPoint& mouse)
{	
	XMPFacet *result = kXMPNULL;
	CEmbeddedXMPFacetsIterator ite(this, embeddingFacet, kXMPFrontToBack);
	for (XMPFacet* xmpFacet = ite.First(); ite.IsNotComplete(); xmpFacet = ite.Next())
	{
		if (xmpFacet->ContainsPoint(mouse))
		{
			result = xmpFacet;
			break;
		}
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 CProxyRun::NewProxyBorder
//------------------------------------------------------------------------------

CBaseProxyBorder* CProxyRun::NewProxyBorder()
{
	CProxyBorder* border = new CProxyBorder;
	border->InitProxyBorder(this);
	return border;
}

//------------------------------------------------------------------------------
//	 CProxyRun::SetSelectState
//------------------------------------------------------------------------------

void CProxyRun::SetSelectState(XMPBoolean state)
{
	fSelected = state;
}

//------------------------------------------------------------------------------
//	 CProxyRun::GetXMPPart
//------------------------------------------------------------------------------
//	Can't be const because of the cache

XMPPart* CProxyRun::GetXMPPart()
{
	if (fCachedProxyFrame == kXMPNULL)
	{		
		CProxyRunCProxyFrameIterator ite(this);
		fCachedProxyFrame = ite.First();
	}
	
	return fCachedProxyFrame->GetXMPPart();
}

//------------------------------------------------------------------------------
//	 CProxyRun::WhichProxyFrame
//------------------------------------------------------------------------------

CProxyFrame* CProxyRun::WhichProxyFrame(CEmbeddingFrame* embeddingFrame)
{
	CProxyFrame *result = kXMPNULL;
	
	CProxyRunCProxyFrameIterator ite(this);
	for (CProxyFrame* proxyFrame = ite.First(); ite.IsNotComplete(); proxyFrame = ite.Next())
	{
		if (proxyFrame->GetContainingFrame() == embeddingFrame)
		{
			result = proxyFrame;
			break;
		}
	}
	
	return result;
}

//----------------------------------------------------------------------------------------
// CProxyRun::NewProxyFrame
//----------------------------------------------------------------------------------------

CProxyFrame* CProxyRun::NewProxyFrame(XMPFrame* xmpFrame)
{
	CProxyFrame* proxyFrame =  new CProxyFrame;
	proxyFrame->InitProxyFrame(this, xmpFrame);
	return proxyFrame;
}

//------------------------------------------------------------------------------
//	 CProxyRun::Externalize
//------------------------------------------------------------------------------

void CProxyRun::Externalize(XMPStorageUnit* storageUnit)
{
	XMPULong count = fProxyFrameList->Count();	
	storageUnit->SetValue(sizeof(XMPULong), &count);
	
	CProxyRunCProxyFrameIterator ite(this);
	for (CProxyFrame* proxyFrame = ite.First(); ite.IsNotComplete(); proxyFrame = ite.Next())
	{
		XMPStorageUnitRef aSURef = storageUnit->GetStrongStorageUnitRef(proxyFrame->GetXMPFrame()->GetStorageUnit());		
		storageUnit->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);			
	}
}

//------------------------------------------------------------------------------
//	 CProxyRun::CloneTo
//------------------------------------------------------------------------------

void CProxyRun::CloneTo(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
	XMPStorageUnit* newSU = storageUnit->GetDraft()->CreateStorageUnit();

	// ----- Look for the frame embedded in commandFrame -----
	CProxyFrame *proxyFrame = WhichProxyFrame((CEmbeddingFrame*)commandFrame);
	XMPStorageUnit* InitiatingFrameSU = proxyFrame ? proxyFrame->GetXMPFrame()->GetStorageUnit() : kXMPNULL;

	XMPDraft* fromDraft = fEmbeddingPart->GetStorageUnit()->GetDraft();
	XMPDraftKey	key = fromDraft->BeginClone(cloneKind);
	
	// ----- Clone the embedded part -----
	GetXMPPart()->CloneInto(key, newSU, InitiatingFrameSU);

	// ----- Clone the initiating frame -----
	XMPStorageUnit* frameSU = proxyFrame->GetXMPFrame()->CloneTo(key, storageUnit->GetDraft());
		
	fromDraft->EndClone(key);
	
	newSU->Release();
	
	XMPStorageUnitRef frameSURef = storageUnit->GetStrongStorageUnitRef(frameSU);
	frameSU->Release();
	
	// ----- Write out -----
	XMPULong count = 1;
	storageUnit->SetValue(sizeof(count), (XMPValue)&count);		
	storageUnit->SetValue(sizeof(frameSURef), (XMPValue)&frameSURef);	
}

//------------------------------------------------------------------------------
//	 CProxyRun::CloneInto
//------------------------------------------------------------------------------

void CProxyRun::CloneInto(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
	CProxyFrame *proxyFrame = WhichProxyFrame((CEmbeddingFrame*)commandFrame);
	XMPStorageUnit* InitiatingFrameSU = proxyFrame ? proxyFrame->GetXMPFrame()->GetStorageUnit() : kXMPNULL;

	XMPDraft* fromDraft = fEmbeddingPart->GetStorageUnit()->GetDraft();
	
	XMPDraftKey	key = fromDraft->BeginClone(cloneKind);
	GetXMPPart()->CloneInto(key, storageUnit, InitiatingFrameSU);
	fromDraft->EndClone(key);
	
	// ----- Add the shape of the frame -----
	if (proxyFrame)
	{
		XMPShape *shape = proxyFrame->GetXMPFrame()->GetFrameShape();
		storageUnit->AddProperty(kXMPPropFrameShape)->AddValue(kXMPFrame);
		shape->WriteShape(storageUnit);
	}
}

//------------------------------------------------------------------------------
//	 CProxyRun::CloneFrom
//------------------------------------------------------------------------------

XMPFrame* CProxyRun::CloneFrom(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	XMPStorageUnitRef ref;
	XMPULong count;
	storageUnit->GetValue(sizeof(count), (XMPValue)&count);
	storageUnit->GetValue(sizeof(ref), (XMPValue)&ref);
	
	return fEmbeddingPart->CloneFrameFrom(storageUnit, ref, cloneKind);
}

//------------------------------------------------------------------------------
//	 CProxyRun::Internalize
//------------------------------------------------------------------------------

void CProxyRun::Internalize(XMPStorageUnit* storageUnit)
{
	XMPFrame* xmpEmbeddedFrame;
	XMPVolatile(xmpEmbeddedFrame);
	
	XMPULong count;	
	storageUnit->GetValue(sizeof(XMPULong), (XMPValue)&count);
	
	XMPULong nbProxyFrame = 0;
	for (XMPULong i = 0; i<count; i++)
	{
		XMPStorageUnitRef aSURef;
		storageUnit->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		TRY
			XMPStorageUnitID id = storageUnit->GetIDFromStorageUnitRef(aSURef);
			xmpEmbeddedFrame = storageUnit->GetDraft()->GetFrame(id);
		CATCH_ALL
			xmpEmbeddedFrame = kXMPNULL;
		ENDTRY
		
		if (xmpEmbeddedFrame)
		{
			nbProxyFrame++;
			
			CProxyFrame* proxyFrame = NewProxyFrame(xmpEmbeddedFrame);
			CEmbeddingFrame* embeddingFrame = (CEmbeddingFrame*)CFrame::XMPtoFWFrame(xmpEmbeddedFrame->GetContainingFrame());
			if (embeddingFrame)
				embeddingFrame->AddProxyFrame(proxyFrame);
			
			xmpEmbeddedFrame->Release();			// This is needed to balance an additional reference created when GetFrame
													// was called.  We keep our own reference in proxyFrame
		}
	}
}

//==============================================================================
//	 class CProxyFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 CProxyFrame::CProxyFrame
//------------------------------------------------------------------------------

CProxyFrame::CProxyFrame()
{
	fProxyRun = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CProxyFrame::IProxyFrame
//------------------------------------------------------------------------------

void CProxyFrame::InitProxyFrame(CProxyRun *proxyRun, XMPFrame *xmpFrame)
{
	IBaseFrame(xmpFrame);
	
	fProxyRun = proxyRun;
	fProxyRun->AddProxyFrame(this);
}

//------------------------------------------------------------------------------
//	 CProxyFrame::~CProxyFrame
//------------------------------------------------------------------------------

CProxyFrame::~CProxyFrame()
{
	if (fProxyRun)
		fProxyRun->RemoveProxyFrame(this);
}

//------------------------------------------------------------------------------
//	 CProxyFrame::GetContainingFrame
//------------------------------------------------------------------------------

CEmbeddingFrame* CProxyFrame::GetContainingFrame() const
{
	XMPFrame* containingFrame = GetXMPFrame()->GetContainingFrame();
	if (containingFrame)
		return (CEmbeddingFrame*)CFrame::XMPtoFWFrame(containingFrame);
	else
		return kXMPNULL;
}

//==============================================================================
//	 CEmbeddedXMPFacetIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFacetsIterator::CEmbeddedXMPFacetsIterator
//------------------------------------------------------------------------------

CEmbeddedXMPFacetsIterator::CEmbeddedXMPFacetsIterator(CProxyRun* proxyRun, 
													CFacet* facet, 
													XMPSiblingOrder siblingOrder) :
	fXMPFacetIterator(facet->GetXMPFacet(),
					  kXMPChildrenOnly,
					  siblingOrder)
{
	fProxyRun = proxyRun;
	fXMPFacet = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFacetsIterator::~CEmbeddedXMPFacetsIterator
//------------------------------------------------------------------------------

CEmbeddedXMPFacetsIterator::~CEmbeddedXMPFacetsIterator()
{
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFacetsIterator::First
//------------------------------------------------------------------------------

XMPFacet* CEmbeddedXMPFacetsIterator::First()
{
	fXMPFacet = fXMPFacetIterator.First();
	FindNext();
	return fXMPFacet;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFacetsIterator::Next
//------------------------------------------------------------------------------

XMPFacet* CEmbeddedXMPFacetsIterator::Next()
{
	fXMPFacet = fXMPFacetIterator.Next();
	FindNext();
	return fXMPFacet;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFacetsIterator::IsNotComplete
//------------------------------------------------------------------------------

XMPBoolean CEmbeddedXMPFacetsIterator::IsNotComplete()
{
	return fXMPFacet != kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CEmbeddedXMPFacetsIterator::FindNext
//------------------------------------------------------------------------------

void CEmbeddedXMPFacetsIterator::FindNext()
{
	while (fXMPFacet != kXMPNULL)
	{
		if (fXMPFacet->GetFrame()->GetPart() == fProxyRun->GetXMPPart())
		{
			return;
		}
		fXMPFacet = fXMPFacetIterator.Next();
	} 
}

//==============================================================================
//	 class CProxyRunCProxyFrameIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 CProxyRunCProxyFrameIterator::CProxyRunCProxyFrameIterator
//------------------------------------------------------------------------------

CProxyRunCProxyFrameIterator::CProxyRunCProxyFrameIterator(CProxyRun* proxyRun) :
	fIterator(proxyRun->fProxyFrameList)
{
}

//------------------------------------------------------------------------------
//	 CProxyRunCProxyFrameIterator::~CProxyRunCProxyFrameIterator
//------------------------------------------------------------------------------

CProxyRunCProxyFrameIterator::~CProxyRunCProxyFrameIterator()
{
}
