/*
	File:		FWProxy.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<1+>	 3/17/94	DS		Add CDECL to CProxyFrame dtor

	To Do:
*/

#ifndef _FWPROXY_
#define _FWPROXY_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _FWBASEFR_
#include "FWBaseFr.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

// ----- OpenDoc Includes
#ifndef _PLFMTYPE_
#include <PlfmType.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef _PART_
#include <Part.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

// ----- Macintosh Includes
#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPFrame;
class XMPShape;
class CEmbeddingFrame;
class CProxyFrame;
class CEmbeddingPart;
class CBaseProxyBorder;

//==============================================================================
//	 class CProxyRun
//==============================================================================

#define kCProxyRunID "appl:cproxyrun$class,1.0.0"

class CProxyRun
{
	friend	CProxyRunCProxyFrameIterator;
	
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CProxyRun();
	void InitProxyRun(CEmbeddingPart* embeddingPart);
	virtual ~CProxyRun();
	
//------------------------------------------------------------------------------
//	 New API
//
public:
	virtual CProxyFrame* 	NewProxyFrame(XMPFrame *xmpFrame);
	virtual void			AddProxyFrame(CProxyFrame* proxyFrame);
	virtual void			RemoveProxyFrame(CProxyFrame* proxyFrame);
	XMPULong				CountProxyFrames() const
									{return fCount;}
	
	// ----- Query -----
	XMPFacet*				WhichEmbeddedFacet(CFacet* embeddingFacet, const XMPPoint& mouse);	
	CProxyFrame*			WhichProxyFrame(CEmbeddingFrame* embeddingFrame);
	
	// ----- Notification -----
	virtual void 			UsedShapeChanged(XMPFrame* xmpFrame);
	
	// ----- Persistence -----
	void					Externalize(XMPStorageUnit* storageUnit);
	void					Internalize(XMPStorageUnit* storageUnit);
	void					CloneTo(XMPStorageUnit* storageUnit, 
									  CFrame* commandFrame, 
									  XMPCloneKind cloneKind);
	void					CloneInto(XMPStorageUnit* storageUnit, 
									  CFrame* commandFrame, 
									  XMPCloneKind cloneKind);
	XMPFrame*				CloneFrom(XMPStorageUnit* storageUnit, 
									  XMPCloneKind cloneKind);
									  
	// ----- Selection state -----
	XMPBoolean				IsSelected() const
								{return fSelected;}
	virtual void			SetSelectState(XMPBoolean state);

	// ----- Selection Border -----
	virtual CBaseProxyBorder*	NewProxyBorder();
	
	// ----- Facet External Transform -----
	virtual void 				GetFacetExternalTransform(XMPFrame* xmpFrame, 
														  XMPTransform* facetExternalTranform) = 0;
	
	// ----- Getters/Setters -----
	XMPPart*					GetXMPPart();
	CEmbeddingPart*				GetEmbeddingPart() const
									{return fEmbeddingPart;}
	CBaseProxyBorder*			GetProxyBorder() const
									{return fProxyBorder;}
	
										
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	CProxyFrame*			fCachedProxyFrame;
	OrderedCollection*		fProxyFrameList;
	CEmbeddingPart*			fEmbeddingPart;
	CBaseProxyBorder*		fProxyBorder;
	XMPBoolean				fSelected;
	XMPULong				fCount;
};

//==============================================================================
//	 class CProxyFrame
//==============================================================================
// CProxyFrame is used as a wrapper for an embedded frame

#define kCProxyFrameID "appl:cproxyframe$class,1.0.0"

class CProxyFrame : public CBaseFrame
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CProxyFrame();
	void InitProxyFrame(CProxyRun* proxyRun,
					 	XMPFrame *xmpFrame);
	virtual ~ CDECL CProxyFrame();
	
//------------------------------------------------------------------------------
//	 New API
//
	// ----- Presistence -----
	void					Externalize(XMPStorageUnit* storageUnit);
	
	// ----- Getters/Setters
	CEmbeddingFrame* 		GetContainingFrame() const;
	
	CBaseProxyBorder*		GetProxyBorder() const;
	CProxyRun*				GetProxyRun() const
								{return fProxyRun;}

//------------------------------------------------------------------------------
//	 Data Members
//
private:
	CProxyRun*				fProxyRun;
};

//==============================================================================
//	 class CProxyRunCProxyFrameIterator
//==============================================================================

class CProxyRunCProxyFrameIterator
{
public:
	CProxyRunCProxyFrameIterator(CProxyRun* proxyRun);
	virtual ~CProxyRunCProxyFrameIterator();

	CProxyFrame*	First()
						{return (CProxyFrame*)fIterator.First();}
	CProxyFrame*	Next()
						{return (CProxyFrame*)fIterator.Next();}
	XMPBoolean		IsNotComplete()
						{return fIterator.IsNotComplete();}

private:
	OrderedCollectionIterator	fIterator;
};

//==============================================================================
//	 CEmbeddedXMPFacetIterator
//==============================================================================
// Iterate trought all the facet of proxyRun embedded in facet
// If facet is Null returns all the embedded facets

class CEmbeddedXMPFacetsIterator
{
public:
	CEmbeddedXMPFacetsIterator(CProxyRun* proxyRun, CFacet* facet, XMPSiblingOrder siblingOrder);
	virtual ~CEmbeddedXMPFacetsIterator();

	XMPFacet*		First();
	XMPFacet*		Next();
	XMPBoolean		IsNotComplete();

private:
	void			FindNext();
	
private:
	XMPFacetIterator	fXMPFacetIterator;
	CProxyRun*			fProxyRun;
	XMPFacet*			fXMPFacet;
};

#endif

