//	File:		FWSclBar.cp
//	Contains:	Definition for methods of CScrollBar
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

// ----- Framework Includes

#ifndef _FWSCLBAR_
#include "FWSclBar.h"
#endif

#ifndef _FWCONTRH_
#include "FWContrH.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

// ----- OpenDoc Includes

#ifndef _POINT_
#include "Point.h"
#endif

#ifndef _RECT_
#include "Rect.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

// ----- Macintosh Includes

#ifndef __CONTROLS__
#include <Controls.h>
#endif


//========================================================================================
// CLASS FW_CScrollBar
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CScrollBar::FW_CScrollBar
//----------------------------------------------------------------------------------------

FW_CScrollBar::FW_CScrollBar() :
	fControlMgrHelper(kXMPNULL)
{
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::~FW_CScrollBar
//----------------------------------------------------------------------------------------

FW_CScrollBar::~FW_CScrollBar()
{
	delete fControlMgrHelper;
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::InitScrollBar
//----------------------------------------------------------------------------------------

void FW_CScrollBar::InitScrollBar(CFacet* facet, FW_CView* enclosure,
								  XMPID identifier,
								  XMPTransform* transform, XMPShape* shape)
{	
	this->InitView(facet, enclosure, identifier, transform, shape);
	
	ASSERT(shape->IsRectangular(), 0);
	
	XMPRect rect;
	shape->GetBoundingBox(&rect);
	
	Rect qdRect;
	rect.AsQDRect(qdRect);
	
	fControlMgrHelper = new FW_CControlMgrHelper;
	fControlMgrHelper->InitControlMgrHelper(qdRect, "\p", 0, 0, 1, scrollBarProc);	
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::Draw
//----------------------------------------------------------------------------------------

void FW_CScrollBar::Draw(XMPShape* area)
{
XMPUnused(area);

	XMPPlatformWindow window = GetFacet()->GetXMPWindow()->GetPlatformWindow();
	fControlMgrHelper->Draw(window);
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::MouseDown
//----------------------------------------------------------------------------------------

XMPBoolean FW_CScrollBar::MouseDown(const XMPPoint& where, XMPEventData event)
{
XMPUnused(event);

	FW_CViewDrawInitiator di(this);

	XMPPlatformWindow window = GetFacet()->GetXMPWindow()->GetPlatformWindow();
	Point qdPoint = where.AsQDPoint();
	
	fControlMgrHelper->MouseDown(window, qdPoint);
	
	return kXMPTrue;
}
