/*
	File:		FWScroll.cpp

	Contains:	Definition for FW_CScrollFacet

	Written by:	David Stafford

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/30/94	DS		Make FWScrollFacet class

	To Do:
*/

//	File:		FWScroll.cpp
//	Contains:	Implementation of class CScrollerFrame
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWSCROLL_
#include "FWScroll.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

// ----- Toolbox Includes -----

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

//==============================================================================
// FW_CScrollFacet
//==============================================================================

//------------------------------------------------------------------------------
// FW_CScrollFacet::FW_CScrollFacet
//------------------------------------------------------------------------------

FW_CScrollFacet::FW_CScrollFacet()
{
	for ( short index = horizontal; index++; index < vertical )
	{
		fScrollBars[index] = kXMPNULL;
		fScrollConstrain[index] = kXMPFalse;
		fHasScrollbars[index] = kXMPFalse;
		fScrollBarOffsets[index] = kDefaultScrollbarOffset;
	}
	fScrollUnits = kDefaultScrollUnits;
	fEmbeddedFacet = kXMPNULL;
}

//------------------------------------------------------------------------------
// FW_CScrollFacet::FW_CScrollFacet
//------------------------------------------------------------------------------

FW_CScrollFacet::~FW_CScrollFacet()
{
	for ( short index = horizontal; index++; index < vertical )
	{
		if (fScrollBars[index] == kXMPNULL)
			delete fScrollBars[index];
	}
}

//------------------------------------------------------------------------------
// FW_CScrollFacet::FW_CScrollFacet
//------------------------------------------------------------------------------

FW_CView* FW_CScrollFacet::CreateViewHierarchy()
{
	// Create 		scroll bars
	XMPFacet* 		facet = GetXMPFacet();
	XMPShape* 		shape = facet->GetClipShape();
	XMPTransform*	transform = facet->GetExternalTransform();
	XMPRect	 		shapeRect;
	
	shape->GetBoundingBox(&shapeRect);
	
	// Create Root View
	XMPShape* 		viewShape = new XMPShape;
	XMPTransform* 	viewTransform = new XMPTransform;
	FW_CView* 		rootView = new FW_CView;
	
	viewShape->CopyFrom(shape);
	viewTransform->CopyFrom(transform);
	
	rootView->InitView(this, kXMPNULL, XMPID('SCLR'), viewTransform, viewShape );
	
	// Create horizontal scroll bar
	if (GetHasScrollbar(horizontal)==kXMPTrue)
	{
		viewShape = new XMPShape;
		viewTransform = new XMPTransform;
		FW_CScrollBar* 	horzScrollbar = new FW_CScrollBar;
		XMPRect viewRect;
		
		viewShape->CopyFrom(shape);
		viewTransform->CopyFrom(transform);
		
		viewShape->GetBoundingBox(&viewRect);
		viewRect.SetInt(viewRect.Width()-GetScrollOffset(horizontal), 0, viewRect.Width(), viewRect.Height());
		viewShape->SetRectangle(&viewRect);
	
		horzScrollbar->InitScrollBar(rootView, XMPID('SCB1'), viewTransform, viewShape );
	}
	
	
	// Create horizontal scroll bar
	if (GetHasScrollbar(vertical)==kXMPTrue)
	{
		// Create vertical scroll bar
		viewShape = new XMPShape;
		viewTransform = new XMPTransform;
		FW_CScrollBar* 	horzScrollbar = new FW_CScrollBar;
		XMPRect viewRect;
		
		viewShape->CopyFrom(shape);
		viewTransform->CopyFrom(transform);
		
		viewShape->GetBoundingBox(&viewRect);
		viewRect.SetInt(viewRect.Width()-GetScrollOffset(horizontal), 0, viewRect.Width(), viewRect.Height());
		viewShape->SetRectangle(&viewRect);
	
		horzScrollbar->InitScrollBar(rootView, XMPID('SCB1'), viewTransform, viewShape );
	}
	
	return rootView;
	
}

//------------------------------------------------------------------------------
// FW_CScrollFacet::FW_CScrollFacet
//------------------------------------------------------------------------------

void FW_CScrollFacet::InitScrollerFacet( XMPFacet* xmpFacet, 
											XMPBoolean hasHorzScrollbar, 
											XMPBoolean hasVertScrollbar,
											XMPSShort horzScrollbarOffset,
											XMPSShort vertScrollbarOffset,
											XMPSShort scrollUnits )
{
	InitViewFacet(xmpFacet);
	
	// Set instance data
	SetScrollOffset( horizontal, horzScrollbarOffset );
	SetScrollOffset( vertical, vertScrollbarOffset );
		
	SetScrollUnits(scrollUnits);
	
	SetHasScrollbar( horizontal, hasHorzScrollbar );
	SetHasScrollbar( vertical, hasVertScrollbar );

	
	// Calculate the parameters for the embedded facet and call CreateEmbeddedFacet.
	// CPart should recognize that the facet is an embedded one and call the right method of
	//
	XMPFrame*		theFrame = kXMPNULL; //= xmpFacet->GetFrame();
	XMPRect			oldRect;
	XMPRect			newRect;
	XMPShape*		newShape = new XMPShape;
	XMPTransform*	transform = new XMPTransform;
	XMPFacet*		embeddedFacet;
	
	// Get and adjust shape
	newShape->CopyFrom(xmpFacet->GetClipShape());
	newShape->GetBoundingBox(&oldRect);
	newRect.Set(oldRect.TopLeft(), oldRect.Width()-horzScrollbarOffset, oldRect.Height()-vertScrollbarOffset);
	newShape->SetRectangle(&newRect);
	
	// Get transform from parent facet
	transform->CopyFrom(xmpFacet->GetExternalTransform());
	
	xmpFacet->CreateEmbeddedFacet(theFrame, newShape, transform, xmpFacet, kXMPFrameInFront);
	
}


//------------------------------------------------------------------------------
// FW_CScrollFacet::DoMouseDown
//------------------------------------------------------------------------------

XMPBoolean FW_CScrollFacet::DoMouseDown(const XMPPoint& where,
										XMPEventData event)
{

	WASSERTM(false, "This method must be implemented.");
	
	
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// FW_CScrollFacet::AdjustScrollbars
//------------------------------------------------------------------------------

void FW_CScrollFacet::AdjustScrollbars()
{

	WASSERTM(false, "This method must be implemented.");

}


