/*
	File:		FWScroll.h

	Contains:	Definition for FW_CScrollFacet class

	Written by:	David Stafford

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 3/30/94	DS		Make FWScrollFacet class
		<1+>	 3/17/94	DS		Add CDECL to CSCrollerFrame dtor

	To Do:
*/

//	File:		FWScroll.h
//	Contains:	Definition for class CScrollerFrame
//									 CScrollerFacet
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWSCROLL_
#define _FWSCROLL_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

#ifndef _PLFMTYPE_
#include "PlfmType.h"		// For XMPSShort
#endif

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _FWVIEWFA_
#include "FWViewFa.h"
#endif

#ifndef _FWVIEW_
#include "FWView.h"
#endif

#ifndef _FWSCLBAR_
#include "FWSclBar.h"
#endif

//==============================================================================
// Types
//==============================================================================
const XMPSShort kDefaultScrollbarOffset = 16;
const XMPSShort kDefaultScrollUnits = 32;

//==============================================================================
// Types
//==============================================================================
typedef enum
{
	horizontal 		= 1,
	vertical		= 2
} VHSelector;

//==============================================================================
// Forward Declarations
//==============================================================================

//==============================================================================
// FW_CScrollFacet
//==============================================================================

#define kFW_CScrollFacetID "appl:fw_cscrollfacet$class,1.0.0"

class FW_CScrollFacet : public FW_CViewFacet
{
public:
// ----- Initialization/destruction
					FW_CScrollFacet();
	void 			InitScrollerFacet(XMPFacet* xmpFacet, 
										XMPBoolean hasHorzScrollbar, 
										XMPBoolean hasVertScrollbar,
										XMPSShort horzScrollbarOffset,
										XMPSShort vertScrollbarOffset,
										XMPSShort scrollUnits );
	
	virtual 		~FW_CScrollFacet();
	

// ----- Accessors
FW_CView* 		GetScrollbar(VHSelect which) const;
void 			SetScrollbar(VHSelect which, FW_CScrollBar* scrollBar);
XMPBoolean 		GetScrollConstrain(VHSelect which) const;
void 			SetScrollConstrain(VHSelect which, XMPBoolean constrain);
XMPBoolean 		GetHasScrollbar(VHSelect which) const;
void 			SetHasScrollbar(VHSelect which, XMPBoolean has);
XMPBoolean 		GetScrollOffset(VHSelect which) const;
void 			SetScrollOffset(VHSelect which, XMPBoolean offset);
XMPBoolean 		GetScrollUnits() const;
void 			SetScrollUnits(XMPSShort units);



// ----- Derived API
	virtual XMPBoolean 		DoMouseDown(const XMPPoint& where,
										XMPEventData event);
										
	virtual FW_CView*		CreateViewHierarchy();
	
// ----- New API
	virtual void			AdjustScrollbars();

//	To be added:
// 		ScrollTo
//		RevealRect


// ----- Fields
private:

	CFacet* 				fEmbeddedFacet;
	FW_CScrollBar* 			fScrollBars[2];
	XMPBoolean				fHasScrollbars[2];
	XMPBoolean				fScrollConstrain[2];
	XMPSShort				fScrollBarOffsets[2];
	XMPSShort				fScrollUnits;

};

FW_CView* FW_CScrollFacet::GetScrollbar(VHSelect which) const
{
	return fScrollBars[which];
}

void FW_CScrollFacet::SetScrollbar(VHSelect which, FW_CScrollBar* scrollBar)
{
	fScrollBars[which] = scrollBar;
}

XMPBoolean FW_CScrollFacet::GetScrollConstrain(VHSelect which) const
{
	return fScrollConstrain[which];
}

void FW_CScrollFacet::SetScrollConstrain(VHSelect which, XMPBoolean constrain)
{
	fScrollConstrain[which] = constrain;
}

XMPBoolean FW_CScrollFacet::GetHasScrollbar(VHSelect which) const
{
	return fHasScrollbars[which];
}

void FW_CScrollFacet::SetHasScrollbar(VHSelect which, XMPBoolean has)
{
	fHasScrollbars[which] = has;
}

XMPBoolean FW_CScrollFacet::GetScrollOffset(VHSelect which) const
{
	return fScrollBarOffsets[which];
}

void FW_CScrollFacet::SetScrollOffset(VHSelect which, XMPBoolean offset)
{
	fScrollBarOffsets[which] = offset;
}

XMPBoolean FW_CScrollFacet::GetScrollUnits() const
{
	return fScrollUnits;
}

void FW_CScrollFacet::SetScrollUnits(XMPSShort units)
{
	fScrollUnits = units;
}



#endif