/*
	File:		FWSelect.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	 4/14/94	DS		Fixed Insert
		 <5>	 4/13/94	DS		InsertNewPart problem
		 <4>	 4/12/94	DS		Re-Implement Insert to follow the correct
									recipe.
		 <3>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h
		 <2>	 3/14/94	DS		Add InsertNewPart.

	To Do:
*/

//	File:		FWSelect.cpp
//	Contains:	Implementation for class FW_CSelection
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _FWPARTING_
#include "FWParting.h"
#endif

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _XMPFILE_
#include "XMPFile.h"
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _CLIPBD_
#include <Clipbd.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _STORAGE_
#include <Storage.h>
#endif

#ifndef _STORGDEF_
#include "StorgDef.h"		// For kXMPDefaultFileContainer
#endif

#ifndef _WINSTAT_
#include <WinStat.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _DOCUMENT_
#include <Document.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _XMPCTR_
#include <XMPCtr.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _LINK_
#include <Link.h>
#endif

#ifndef _LINKSPEC_
#include <LinkSpec.h>
#endif

#ifndef _LINKSRC_
#include <LinkSrc.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _ISOSTR_
#include <ISOStr.h>
#endif

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>		// See MoveTransformBy
#endif


#pragma segment FWSelection

//==============================================================================
//	 class FW_CSelection
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CSelection::FW_CSelection
//------------------------------------------------------------------------------

FW_CSelection::FW_CSelection()
{
}

//------------------------------------------------------------------------------
//	 FW_CSelection::InitSelection
//------------------------------------------------------------------------------

void FW_CSelection::InitSelection(CPart* part,  XMPBoolean allowPublish, XMPBoolean allowSubscrib)
{
	fPart = part;
	fAllowPublish = allowPublish;
	fAllowSubscrib = allowSubscrib;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::~FW_CSelection
//------------------------------------------------------------------------------

FW_CSelection::~FW_CSelection()
{
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoCut
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::DoCut(CFrame* frame)
{
	Copy(kXMPFalse, frame, kXMPCloneCut);	// we dont't allow linking for a cut
	return DoClear();
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoCopy
//------------------------------------------------------------------------------

void FW_CSelection::DoCopy(CFrame* frame)
{
	Copy(fAllowPublish, frame, kXMPCloneCopy);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::IsSelectionOnlyOneProxyRun
//------------------------------------------------------------------------------

CProxyRun* FW_CSelection::IsSelectionOnlyOneProxyRun() const
{
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::Copy
//------------------------------------------------------------------------------

void FW_CSelection::Copy(XMPBoolean allowPublish, CFrame* frame, XMPCloneKind cloneKind)
{	
	XMPClipboard* clipboard = fPart->GetSession()->GetClipboard();
	XMPClipboardKey key;
	if (clipboard->Lock(0, &key))
	{
		clipboard->Clear(key);
		XMPStorageUnit* clipBSU = clipboard->GetContentStorageUnit(key);
		
		this->WriteSelection(clipBSU, frame, cloneKind);			
		
		// ----- Write out link Spec if there is one -----
		if (allowPublish && IsSelectionPublishable())
		{
			XMPChangeID changeID = clipboard->GetChangeID();

			CPublishLink* publishLink = NewPublishLink(changeID);
			fPart->SetPendingPublish(publishLink);	
	
			XMPLinkSpec* linkSpec = fPart->GetStorageUnit()->GetDraft()->CreateLinkSpec(fPart, &changeID, sizeof(XMPChangeID));

			clipBSU->AddProperty(kXMPPropLinkSpec);
			linkSpec->WriteLinkSpec(clipBSU);
			delete linkSpec;
		}
		
		clipboard->Unlock(key);
	}
}

//------------------------------------------------------------------------------
//	 FW_CSelection::GetSelectionShape
//------------------------------------------------------------------------------

XMPShape* FW_CSelection::GetSelectionShape()
{
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::IsSelectionPublishable
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::IsSelectionPublishable()
{
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::NewPublishLink
//------------------------------------------------------------------------------
//	Must overridden if supporting publishing

CPublishLink* FW_CSelection::NewPublishLink(XMPChangeID changeID)
{
XMPUnused(changeID);

	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::InsertNewPart
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::InsertNewPart()
{
	XMPBoolean result = kXMPFalse;
	XMPStorageUnit* su = GetPart()->GetStorageUnit();
	XMPSession* session = su->GetSession();
	XMPWindowState* windowState = session->GetWindowState();
	XMPContainer* insertContainer = kXMPNULL;
	XMPDocument* insertDocument = kXMPNULL;
	
	XMPType	partkind = kXMPNULL;

	// StdGetFile please select an editor for the root part
	StandardFileReply sfReply;
	SFTypeList sfTypes;
	
	windowState->DeactivateFrontWindows();
	StandardGetFile((FileFilterUPP)kXMPNULL,-1,sfTypes,&sfReply);
	windowState->ActivateFrontWindows();
	
	if (sfReply.sfGood) 
	{
		XMPFile* file = new XMPFile();
		file->Specify(&(sfReply.sfFile));
		
		TRY
			insertContainer = session->GetStorageSystem()->GetContainer(kXMPDefaultFileContainer, file);
			// IF file->GetContainerType doesn't work then use kXMPDefaultFileContainer
		CATCH_ALL
			insertContainer = kXMPNULL;
			result = kXMPFalse;
		ENDTRY
	
		// If the user chose an OpenDoc XMPContainer
		if (insertContainer != kXMPNULL)
		{
			// ???? Should use a document method to get the top draft? What does that mean?
			insertDocument = insertContainer->GetDocument(kXMPDefaultDocument);
			XMPDraft* nextDraft = kXMPNULL;
			XMPDraft* insertDraft = insertDocument->GetBaseDraft(kDPExclusiveWrite);
		
			{
			TRY
				while(kXMPTrue)
				{
				nextDraft = insertDocument->GetDraft(kDPReadOnly, kXMPNULL, insertDraft, kXMPPosLastAbove, kXMPTrue);
				insertDraft = nextDraft;
				}
			CATCH_ALL
			nextDraft = kXMPNULL;
			ENDTRY
			}
		
			XMPStorageUnit* draftSU = insertDraft->GetDraftProperties();
			XMPStorageUnitRef suRef;
			draftSU->Focus(kXMPPropRootPartSU, kXMPPosUndefined, kXMPStorageUnit, 0, kXMPPosUndefined);
			draftSU->GetValue(sizeof(XMPStorageUnitRef), &suRef);
			XMPStorageUnit* insertSU = insertDraft->GetStorageUnit(draftSU->GetIDFromStorageUnitRef(suRef));
		
			// Clone the root part from the insertDraft to this Draft. As per Insert recipe.
			XMPStorageUnit* newSU = kXMPNULL;
			XMPDraftKey key = insertDraft->BeginClone(kXMPClonePaste);
			{
			TRY
				newSU = insertSU->CloneTo(key, su->GetDraft(), kXMPNULL);
			CATCH_ALL
				// If an exception was raised while cloning, abort the insert
				insertDraft->AbortClone(key);
				RERAISE;
			ENDTRY
			}
			insertDraft->EndClone(key);
			
			XMPPart* newPart = su->GetDraft()->GetPart(newSU->GetID());
			
			// Get the frame shape
			XMPShape* frameShape = new XMPShape;
			if (insertSU->Exists(kXMPPropFrameShape, kXMPNULL, 0))
			{
				insertSU->Focus(kXMPPropFrameShape, kXMPPosUndefined, kXMPNULL, 0, kXMPPosUndefined);
				frameShape->ReadShape(insertSU);
			}
			else
			{
				XMPRect frameRect(0, 0, ff(80), ff(80));
				frameShape->SetRectangle(&frameRect);
			}
			
			// Reset selection
			CloseSelection();
			
			result = EmbedSelection(newPart, frameShape);
			
			// ----- Release -----
			newPart->Release();
			
			newSU->Release();
			insertSU->Release();
			draftSU->Release();
			insertDraft->Release();
			if (insertDocument)
				insertDocument->Release();
			insertContainer->Release();
			
			// ----- Release Shape -----
			delete frameShape;
			
			result = kXMPTrue;
			
			delete file;
			
		}
		
	}
	
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::NewSubscribLink
//------------------------------------------------------------------------------
//	Must overridden if supporting subscribing

CSubscribLink* FW_CSelection::NewSubscribLink(XMPLink *xmpLink)
{
XMPUnused(xmpLink);

	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoPaste
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::DoPaste()
{
	return Paste(kXMPFalse, kXMPClonePaste);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoPasteWithLink
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::DoPasteWithLink()
{
	return Paste(fAllowSubscrib, kXMPClonePaste);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoPaste
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::Paste(XMPBoolean allowSubscrib, XMPCloneKind cloneKind)
{
	XMPBoolean result = kXMPFalse;
	
	XMPClipboard* clipboard = fPart->GetSession()->GetClipboard();
	XMPClipboardKey key;
	if (clipboard->Lock(0, &key))
	{
		XMPStorageUnit* clipBSU = clipboard->GetContentStorageUnit(key);

		result = this->ReadSelection(clipBSU, cloneKind);
				
		// ----- Link -----
		if (result && allowSubscrib && clipBSU->Exists(kXMPPropLinkSpec, (XMPValueType)kXMPNULL, 0))
		{
			XMPLinkSpec* linkSpec = new XMPLinkSpec;
			clipBSU->Focus(kXMPPropLinkSpec, kXMPPosUndefined, (XMPValueType)kXMPNULL, 0, kXMPPosUndefined);
			linkSpec->ReadLinkSpec(clipBSU);
			XMPLink *xmpLink = fPart->GetStorageUnit()->GetDraft()->GetLink((XMPStorageUnitID)kXMPNULL, linkSpec);	
			
			CSubscribLink *subscribLink = NewSubscribLink(xmpLink);
			fPart->AddToSubscribLinkList(subscribLink);
			subscribLink->Subscrib();
			
			xmpLink->RegisterDependent(fPart, 0);
			delete linkSpec;
		}
		
		clipboard->Unlock(key);
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::Drag
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::Drag(CFacet* facet, XMPEventData event)
{
	XMPPart* destPart;
	XMPID frameID;
	XMPDragAndDrop*	drag;
	XMPStorageUnit*	dragSU;
	XMPDropResult dropResult;

	if (!::WaitMouseMoved(event->where))
		return kXMPFalse;
	
	// ----- In the following block we focus on the window -----
	{
		CDrawInitiator di(facet->GetXMPWindow());
	
		XMPRgnHandle dragRgn = this->CalcDragRgn(facet);
	
		Point translation = {0,0};
		translation = facet->GetWindowFrameTransform()->TransformPoint(translation).AsQDPoint();
		
		::LocalToGlobal(&translation);
		
		::OffsetRgn(dragRgn, translation.h, translation.v);
		
		//  Fill Drag and Drop SU with Props n' Vals
		drag = fPart->GetSession()->GetDragAndDrop();
		drag->Clear();
		dragSU = drag->GetContentStorageUnit();
		
		this->WriteSelection(dragSU, facet->GetFrame(), kXMPCloneCut);
		
		// save ID of frame being dragged
		frameID = facet->GetFrame()->GetXMPFrame()->GetID();
		dragSU->AddProperty(kXMPPropFrameInfo)->AddValue(kXMPID);
		dragSU->SetValue(sizeof(XMPID), (XMPValue) &frameID);
		
		// save offset between mousedown pt and topLeft pt of selection
		Point mouseOffset = {0,0};
		Point topLeft;
		
		// Top left of selection
		topLeft.v = (*dragRgn)->rgnBBox.top;
		topLeft.h = (*dragRgn)->rgnBBox.left;
		
		mouseOffset = event->where;	
		::SubPt(topLeft,&mouseOffset);
		
		dragSU->AddProperty(kXMPPropMouseDownOffset)->AddValue(kQDPoint);
		dragSU->SetValue(sizeof(Point), (XMPValue) &mouseOffset);
		
		fPart->Dragging(facet, kXMPTrue);
		
		dropResult = drag->StartDrag(facet->GetFrame()->GetXMPFrame(), "RGNH", dragRgn, &destPart, event);
		
		if ((dropResult == kXMPDropMove)||(dropResult == kXMPDropCopy))
		{
			if ((destPart != fPart) && (!::TestOptionKey(event) || ::TestControlKey(event)))
				this->DoClear();
		}
		
		fPart->Dragging(facet, kXMPFalse);
	
		::DisposeRgn(dragRgn);
	}
	
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::WriteSelection
//------------------------------------------------------------------------------
//	Here we have two cases: 
//		1) The selection consist of only one embedded part. In this case
//			we just want to clone it into the clipboard
//		2) The selection is two or more embedded parts or one or more
//			embedded parts plus content. In this case we externalize the selection

void FW_CSelection::WriteSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
	CProxyRun *selectedProxyRun = IsSelectionOnlyOneProxyRun();
	if (selectedProxyRun)		// if the selection is only one embedded part clone it into the clipboard 
	{
		selectedProxyRun->CloneInto(storageUnit, commandFrame, cloneKind);
	}
	else
	{		
#ifndef ASLMBUILD
		// ----- Add a kXMPPropPart Property -----
		storageUnit->AddProperty(kXMPPropPart)->AddValue(kXMPISOStr);
		XMPISOStr partName = fPart->GetContentPropertyValueType();
		XMPULong size = XMPISOStrLength(partName) + 1;
		storageUnit->SetValue(size, partName);
#endif
	
		this->ExternalizeSelection(storageUnit, commandFrame, cloneKind);

		// ----- If supported, write a shape property -----
		XMPShape* selectionShape = GetSelectionShape();
		if (selectionShape)
		{
			// ----- Note: We only add a property, no value because WriteShape will do it
			storageUnit->AddProperty(kXMPPropFrameShape);
			selectionShape->WriteShape(storageUnit);
		}
	}
}

//------------------------------------------------------------------------------
//	 FW_CSelection::ReadSelection
//------------------------------------------------------------------------------
//	ReadSelection tries first to internalize the selection. If the selection is
//	not of a kind that this part can read, an embedding part is read.
//	Note: If the part is a non embedding part ClonePartFrom will return NULL so no
//	embedding will happended.

XMPBoolean FW_CSelection::ReadSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	XMPBoolean result = kXMPFalse;
	
	if ((result = InternalizeSelection(storageUnit, cloneKind)) == kXMPFalse)
	{
		XMPPart *xmpPart = fPart->ClonePartFrom(storageUnit, cloneKind);
		if ((result = (xmpPart != kXMPNULL)) == kXMPTrue)
		{
			// ----- Look for a frame shape property -----
			XMPShape* frameShape = kXMPNULL;
			if (storageUnit->Exists(kXMPPropFrameShape, (XMPValueType)kXMPNULL, 0))
			{
				// ----- Note: we focus on the property, ReadShape will focus on the value
				storageUnit->Focus(kXMPPropFrameShape, kXMPPosUndefined, 
							   		(XMPValueType)kXMPNULL, (XMPValueIndex)1, kXMPPosUndefined);
				frameShape = ::NewXMPShape(storageUnit);
			}
			
			result = EmbedSelection(xmpPart, frameShape);
			xmpPart->Release();
			delete frameShape;
		}
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::CalcDragRgn
//------------------------------------------------------------------------------

XMPRgnHandle FW_CSelection::CalcDragRgn(CFacet* facet)
{
	RgnHandle dragRgn = facet->GetXMPFacet()->GetActiveShape()->CopyQDRegion();
	RgnHandle tempRgn = ::NewRgn();
	
	::CopyRgn(dragRgn, tempRgn);
	::InsetRgn(tempRgn, 1, 1);
	::DiffRgn(dragRgn, tempRgn, dragRgn);
	::DisposeRgn(tempRgn);
	
	return dragRgn;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoDroppedInSameFrame
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::DoDroppedInSameFrame(XMPStorageUnit* dropSU, CFacet* facet,
											   XMPPoint originPoint, XMPPoint dropPoint)
{
	XMPUnused(dropSU);
	XMPUnused(facet);
	XMPUnused(originPoint);
	XMPUnused(dropPoint);
	
	return kXMPFalse;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoDrop
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::DoDrop(XMPStorageUnit* dropSU, CFacet* facet, XMPPoint originPoint, XMPPoint dropPoint)
{
	XMPUnused(facet);
	XMPUnused(originPoint);
	XMPUnused(dropPoint);
	
	return this->ReadSelection(dropSU, kXMPClonePaste);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::EmbedSelection
//------------------------------------------------------------------------------

XMPBoolean FW_CSelection::EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape)
{
	XMPUnused(xmpPart);
	XMPUnused(frameShape);
	
	return kXMPFalse;
}

//========================================================================================
//	 class CPublishLink
//========================================================================================

//---------------------------------------------------------------------------------------
//	 CPublishLink::CPublishLink
//---------------------------------------------------------------------------------------

CPublishLink::CPublishLink(XMPChangeID changeID)
{
	fXMPLinkSource = kXMPNULL;
	fChangeID = changeID;
}

//---------------------------------------------------------------------------------------
//	 CPublishLink::SetXMPLinkSource
//---------------------------------------------------------------------------------------

void CPublishLink::SetXMPLinkSource(XMPLinkSource* xmpLinkSource)
{
	WASSERTM(xmpLinkSource!=kXMPNULL, "xmpLinkSource can't be null");
	WASSERTM(fXMPLinkSource==kXMPNULL, "fXMPLinkSource must be null");
	
	fXMPLinkSource = xmpLinkSource;
	xmpLinkSource->IncrementRefCount();
}

//---------------------------------------------------------------------------------------
//	 CPublishLink::~CPublishLink
//---------------------------------------------------------------------------------------

CPublishLink::~CPublishLink()
{
	if (fXMPLinkSource)
		fXMPLinkSource->Release();
}

//---------------------------------------------------------------------------------------
//	 CPublishLink::ContentChanged
//---------------------------------------------------------------------------------------

void CPublishLink::ContentChanged(XMPChangeID changeID)
{
	XMPLinkKey key;
	if (fXMPLinkSource->Lock(0, &key)) 
	{
		XMPStorageUnit* linkSU = fXMPLinkSource->GetContentStorageUnit(key);
		if (!linkSU->Exists(kXMPPropContents, (XMPValueType) kXMPNULL, 0)) 
		{
			linkSU->AddProperty(kXMPPropContents);
		}
		
		if (!linkSU->Exists(kXMPPropContents, GetPublishFormat(), 0)) 
		{
			linkSU->AddValue(GetPublishFormat());
		}
		else 
		{
			XMPULong valueSize;
			linkSU->Focus(kXMPPropContents, kXMPPosUndefined, GetPublishFormat(), 0, kXMPPosUndefined);
			valueSize = linkSU->GetSize();
			linkSU->DeleteValue(valueSize);
		}
		
		linkSU->Focus(kXMPPropContents, kXMPPosUndefined, GetPublishFormat(), 0, kXMPPosUndefined);

		ExternalizeLinkContent(linkSU);
		
		fXMPLinkSource->Unlock(key);		
		fXMPLinkSource->ContentChanged(changeID);	
	}
}

//========================================================================================
//	 class CSubscribLink
//========================================================================================

//---------------------------------------------------------------------------------------
//	 CSubscribLink::CSubscribLink
//---------------------------------------------------------------------------------------

CSubscribLink::CSubscribLink(XMPLink *xmpLink)
{
	fXMPLink = xmpLink;
	fXMPLink->IncrementRefCount();
}

//---------------------------------------------------------------------------------------
//	 CSubscribLink::~CSubscribLink
//---------------------------------------------------------------------------------------

CSubscribLink::~CSubscribLink()
{
	if (fXMPLink)
		fXMPLink->Release();
}

//---------------------------------------------------------------------------------------
//	 CSubscribLink::~CSubscribLink
//---------------------------------------------------------------------------------------

void CSubscribLink::LinkUpdated(XMPChangeID id)
{
XMPUnused(id);
}

