/*
	File:		FWSelect.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/14/94	DS		Add support for the insert menu item..

	To Do:
*/

//	File:		FWSelect.h
//	Contains:	Definition for class FW_CSelection
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWSELECT_
#define _FWSELECT_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPStorageUnit;
class XMPPoint;
class CPart;
class CFacet;
class CFrame;
class XMPLink;
class XMPLinkSource;
class XMPShape;
class CProxyRun;

//==============================================================================
//	 constants
//==============================================================================

const XMPPropertyName kXMPPropFrameInfo = "Framework:Property:FrameInfo";
const XMPPropertyName kXMPPropMouseDownOffset = "Framework:Property:MouseDownOffset";

const XMPValueType kQDPoint = "Framework:ValueType:Point";

//==============================================================================
//	 class CPublishLink
//==============================================================================

class CPublishLink
{
public:
	CPublishLink(XMPChangeID changeID);
	virtual ~CPublishLink();
	
	void			SetXMPLinkSource(XMPLinkSource* xmpLinkSource);
	XMPLinkSource*	GetXMPLinkSource() const
						{return fXMPLinkSource;}
	XMPBoolean		IsPending() const
						{return fXMPLinkSource == kXMPNULL;}
	XMPChangeID		GetChangeID() const
						{return fChangeID;}
	
	void			ContentChanged(XMPChangeID changeID);
	
	virtual void	Publish() = 0;	
	virtual void	ExternalizeLinkContent(XMPStorageUnit* linkSU) = 0;
	virtual XMPValueType GetPublishFormat() = 0;
 
private:
	XMPLinkSource	*fXMPLinkSource;
	XMPChangeID		fChangeID;
};

//==============================================================================
//	 class CSubscribLink
//==============================================================================

class CSubscribLink
{
public:
	CSubscribLink(XMPLink *xmpLink);
	virtual ~CSubscribLink();
	
	XMPLink* 		GetXMPLink() const
						{return fXMPLink;}
						
	virtual void	LinkUpdated(XMPChangeID id);
						
	virtual void	Subscrib() = 0;

private:
	XMPLink *fXMPLink;
};

//==============================================================================
//	 class FW_CSelection
//==============================================================================

class FW_CSelection
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	FW_CSelection();
	void		InitSelection(CPart* part, XMPBoolean allowPublish, XMPBoolean allowSubscrib);
	virtual 	~FW_CSelection();

//------------------------------------------------------------------------------
//	 New API
//
public:
	virtual XMPBoolean	IsEmpty() const = 0;	
	virtual void		CloseSelection() = 0;
	virtual void		SelectAll() = 0;
	virtual XMPShape*	GetSelectionShape();
	virtual CProxyRun*	IsSelectionOnlyOneProxyRun() const;
	
	// ----- Cut/Copy/Paste/Clear -----
	virtual XMPBoolean	DoCut(CFrame* frame);
	virtual void		DoCopy(CFrame* frame);
	virtual XMPBoolean	DoPaste();
	virtual XMPBoolean	DoPasteWithLink();
	virtual XMPBoolean	DoClear() = 0;
	
	// ----- Drag & Drop -----
	virtual XMPBoolean		Drag(CFacet* facet, XMPEventData event);
	virtual XMPRgnHandle 	CalcDragRgn(CFacet* facet);
	virtual XMPBoolean 	DoDroppedInSameFrame(XMPStorageUnit* dropSU, CFacet* facet,
											 XMPPoint originPoint, XMPPoint dropPoint);
 	virtual XMPBoolean		DoDrop(XMPStorageUnit* dropSU, CFacet* facet,
							   	XMPPoint originPoint, XMPPoint dropPoint);
 
 	// ----- Linking -----
	virtual CPublishLink* 	NewPublishLink(XMPChangeID changeID);
	virtual CSubscribLink*	NewSubscribLink(XMPLink *xmpLink);
	virtual XMPBoolean		IsSelectionPublishable();
	
	// ----- Externalize/Internalize Selection -----
	virtual void		ExternalizeSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind) = 0;
	virtual XMPBoolean	InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind) = 0;
	virtual XMPBoolean	EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape);

	virtual XMPBoolean	InsertNewPart();

	// ----- Getters/Setters -----
	CPart*				GetPart() const;
	
protected:
	void				Copy(XMPBoolean allowLinking, CFrame* frame, XMPCloneKind cloneKind);
	XMPBoolean			Paste(XMPBoolean withLink, XMPCloneKind cloneKind);
	XMPPart*			ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind);
	void				WriteSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind);
	XMPBoolean			ReadSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	CPart*				fPart;
	XMPBoolean			fAllowPublish;
	XMPBoolean			fAllowSubscrib;
};

//==============================================================================
//	 FW_CSelection Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CSelection::GetPart
//------------------------------------------------------------------------------
inline CPart* FW_CSelection::GetPart() const
{
	return fPart;
}

#endif