//	File:		FWUtil.cpp
//	Contains:	Some Utilities
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

// ----- Macintosh Includes -----

#ifndef mathRoutinesIncludes
#include <math routines.h>				// for ff()
#endif

#ifndef __LIBRARYMANAGERUTILITIES__
#include <LibraryManagerUtilities.h>
#endif

#ifndef __LIBRARYMANAGERCLASSES__
#include <LibraryManagerClasses.h>
#endif

#pragma segment FWUtilSeg

//----------------------------------------------------------------------------------------
// SortRect
//----------------------------------------------------------------------------------------

void SortRect(Rect& rect)
{
	if (rect.left > rect.right)
	{
		short temp = rect.left;
		rect.left = rect.right;
		rect.right = temp;
	}
	if (rect.top > rect.bottom)
	{
		short temp = rect.top;
		rect.top = rect.bottom;
		rect.bottom = temp;
	}
}

//----------------------------------------------------------------------------------------
// Abs
//----------------------------------------------------------------------------------------

XMPSLong Abs(XMPSLong value)
{
	return value<0 ? -1 * value : value;
}

//----------------------------------------------------------------------------------------
// GetPortTextStyle: 
//----------------------------------------------------------------------------------------

void GetPortTextStyle(TextStyle* theTextStyle)
{
	theTextStyle->tsFont = XMPASLMQDGlobals.thePort->txFont;
	theTextStyle->tsFace = XMPASLMQDGlobals.thePort->txFace;
	theTextStyle->tsSize = XMPASLMQDGlobals.thePort->txSize;
}


//----------------------------------------------------------------------------------------
// SetPortTextStyle: 
//----------------------------------------------------------------------------------------

void SetPortTextStyle(const TextStyle* theTextStyle)
{
	if (XMPASLMQDGlobals.thePort->txFont != theTextStyle->tsFont)
		TextFont(theTextStyle->tsFont);
	if (XMPASLMQDGlobals.thePort->txFace != theTextStyle->tsFace)
		TextFace(theTextStyle->tsFace);
	if (XMPASLMQDGlobals.thePort->txSize != theTextStyle->tsSize)
		TextSize(theTextStyle->tsSize);
} 

//----------------------------------------------------------------------------------------
// ::InsetXMPShape 
//----------------------------------------------------------------------------------------

void InsetXMPShape(XMPShape *shape, XMPSShort xInset, XMPSShort yInset)
{
// [HLX] needs to implement GX
	if (shape->IsRectangular())
	{
		XMPRect rect;
		shape->GetBoundingBox(&rect);
		rect.Inset(ff(xInset), ff(yInset));
		shape->SetRectangle(&rect);
	}
	else
	{
		RgnHandle shapeRgn = shape->GetQDRegion();
		::InsetRgn(shapeRgn, xInset, yInset);
		shape->SetQDRegion(shapeRgn);
	}
} 

//----------------------------------------------------------------------------------------
// ::GetXMPShapeQDBox
//----------------------------------------------------------------------------------------

void GetXMPShapeQDBox(XMPShape *shape, Rect& box)
{
	XMPRect rect;
	shape->GetBoundingBox(&rect);
	rect.AsQDRect(box);
}

//----------------------------------------------------------------------------------------
// ::NewXMPShape
//----------------------------------------------------------------------------------------

XMPShape* NewXMPShape(const Rect& rect)
{
	XMPRect xmpRect(rect);
	return NewXMPShape(&xmpRect);
}

//----------------------------------------------------------------------------------------
// ::NewXMPShape
//----------------------------------------------------------------------------------------

XMPShape* NewXMPShape(XMPRect* rect)
{
	XMPShape *shape  =  kXMPNULL;
	XMPVolatile(shape);
	
	TRY
		shape = new XMPShape;
		FailNULL(shape, kXMPErrOutOfMemory);
		shape->SetRectangle(rect);
	
	CATCH_ALL
		
		if (shape!=kXMPNULL)
			delete shape;
			
		RERAISE;
		
	ENDTRY
	
	return shape;
}

//----------------------------------------------------------------------------------------
// ::NewXMPShape
//----------------------------------------------------------------------------------------

XMPShape* NewXMPShape(RgnHandle rgn)
{
	XMPShape *shape  =  kXMPNULL;
	XMPVolatile(shape);
	
	TRY
		shape = new XMPShape;
		FailNULL(shape, kXMPErrOutOfMemory);
		shape->SetPlatformShape(kXMPQuickDraw, rgn);
	
	CATCH_ALL
		
		if (shape!=kXMPNULL)
			delete shape;
			
		RERAISE;
		
	ENDTRY
	
	return shape;
}

//----------------------------------------------------------------------------------------
// ::NewXMPShape
//----------------------------------------------------------------------------------------

XMPShape* NewXMPShape()
{
	XMPShape *shape = new XMPShape;
	FailNULL(shape, kXMPErrOutOfMemory);
	
	XMPRect rect(0,0,0,0);
	shape->SetRectangle(&rect);
	
	return shape;
}

//----------------------------------------------------------------------------------------
// ::NewXMPShape
//----------------------------------------------------------------------------------------
// The storage unit must have been focused first

XMPShape* NewXMPShape(XMPStorageUnit* su)
{
	XMPShape *shape  =  kXMPNULL;
	XMPVolatile(shape);

	TRY
		shape = new XMPShape;
		FailNULL(shape, kXMPErrOutOfMemory);
		shape = shape->ReadShape(su);
		
	CATCH_ALL
		
		if (shape!=kXMPNULL)
			delete shape;
			
		RERAISE;
		
	ENDTRY
	
	return shape;
}

//----------------------------------------------------------------------------------------
// ::NewXMPShape
//----------------------------------------------------------------------------------------

XMPShape* NewXMPShape(XMPShape* otherShape)
{
	XMPShape *shape  =  kXMPNULL;
	XMPVolatile(shape);
	
	TRY
		shape = new XMPShape;
		FailNULL(shape, kXMPErrOutOfMemory);
		shape->CopyFrom(otherShape);
	
	CATCH_ALL
		
		if (shape!=kXMPNULL)
			delete shape;
			
		RERAISE;
		
	ENDTRY
	
	return shape;
}

//----------------------------------------------------------------------------------------
// ::NewXMPTransform
//----------------------------------------------------------------------------------------

XMPTransform* NewXMPTransform()
{
	XMPTransform *transform = new XMPTransform;
	FailNULL(transform, kXMPErrOutOfMemory);
	
	return transform;
}

//----------------------------------------------------------------------------------------
// ::NewXMPTransform
//----------------------------------------------------------------------------------------

XMPTransform* NewXMPTransform(XMPTransform* otherTransform)
{
	XMPTransform *transform  =  kXMPNULL;
	XMPVolatile(transform);
	
	TRY
		transform = new XMPTransform;
		FailNULL(transform, kXMPErrOutOfMemory);
		transform->CopyFrom(otherTransform);
	
	CATCH_ALL
		
		if (transform!=kXMPNULL)
			delete transform;
			
		RERAISE;
		
	ENDTRY
	
	return transform;
}

//----------------------------------------------------------------------------------------
// ::NewXMPTransform
//----------------------------------------------------------------------------------------

XMPTransform* NewXMPTransform(Point offset)
{
	XMPPoint xmpOffset(offset);
	return NewXMPTransform(xmpOffset);
}

//----------------------------------------------------------------------------------------
// ::NewXMPTransform
//----------------------------------------------------------------------------------------

XMPTransform* NewXMPTransform(const XMPPoint& offset)
{
	XMPTransform *transform  =  kXMPNULL;
	XMPVolatile(transform);
	
	TRY
		transform = new XMPTransform;
		FailNULL(transform, kXMPErrOutOfMemory);
		transform->MoveBy(offset);
	
	CATCH_ALL
		
		if (transform!=kXMPNULL)
			delete transform;
			
		RERAISE;
		
	ENDTRY
	
	return transform;
}

//------------------------------------------------------------------------------
// ::IsKeyPressed
//------------------------------------------------------------------------------

XMPBoolean IsKeyPressed(XMPUShort theKey)
{
	unsigned char theKeys[16];
	::GetKeys((long*)theKeys);
	return ((theKeys[theKey >> 3] >> (theKey & 7)) & 1);
}

//==============================================================================
//	 class FW_CAcquireASLMResourceAccess
//==============================================================================

//----------------------------------------------------------------------------------------
// FW_CAcquireASLMResourceAccess::FW_CAcquireASLMResourceAccess
//----------------------------------------------------------------------------------------

FW_CAcquireASLMResourceAccess::FW_CAcquireASLMResourceAccess()
{
#ifdef ASLMBUILD
	fShellLibFile = GetLocalLibraryFile();
	fShellLibFile->Preflight(fShellLibRefNum);
#endif // ASLMBUILD
}

//----------------------------------------------------------------------------------------
// FW_CAcquireASLMResourceAccess::~FW_CAcquireASLMResourceAccess
//----------------------------------------------------------------------------------------

FW_CAcquireASLMResourceAccess::~FW_CAcquireASLMResourceAccess()
{
#ifdef ASLMBUILD
 	fShellLibFile->Postflight(fShellLibRefNum);
#endif // ASLMBUILD
}

//========================================================================================
//	 class CDrawInitiator
//========================================================================================

//----------------------------------------------------------------------------------------
// CDrawInitiator::CDrawInitiator
//----------------------------------------------------------------------------------------

CDrawInitiator::CDrawInitiator(CFacet *facet, XMPShape* clipShape)
{
	XMPFacet *xmpFacet = facet->GetXMPFacet();
	
	::GetPort(&fCurPort);

	::SetPort(facet->GetXMPWindow()->GetPlatformWindow());

	fCurOrigin.h = XMPASLMQDGlobals.thePort->portRect.left;
	fCurOrigin.v = XMPASLMQDGlobals.thePort->portRect.top;

	fClip = NewRgn();
	::GetClip(fClip);
	
	::SetClip(clipShape ? clipShape->GetQDRegion() : xmpFacet->GetAggregateClipShape()->GetQDRegion());

	XMPPoint offset(0,0);
	offset = xmpFacet->GetContentTransform()->TransformPoint(offset);
	::SetOrigin(-(offset.IntX()), -(offset.IntY()));
}

//----------------------------------------------------------------------------------------
// CDrawInitiator::CDrawInitiator
//----------------------------------------------------------------------------------------

CDrawInitiator::CDrawInitiator(XMPWindow* xmpWindow)
{
	::GetPort(&fCurPort);

	::SetPort(xmpWindow->GetPlatformWindow());

	fCurOrigin.h = XMPASLMQDGlobals.thePort->portRect.left;
	fCurOrigin.v = XMPASLMQDGlobals.thePort->portRect.top;

	fClip = ::NewRgn();
	::GetClip(fClip);

	::SetOrigin(0, 0);
	
	::ClipRect(&XMPASLMQDGlobals.thePort->portRect);
}

//----------------------------------------------------------------------------------------
// CDrawInitiator::~CDrawInitiator
//----------------------------------------------------------------------------------------

CDrawInitiator::~CDrawInitiator()
{
	::SetOrigin(fCurOrigin.h, fCurOrigin.v);
	::SetClip(fClip);
	::DisposeRgn(fClip);
	fClip = kXMPNULL;

	::SetPort(fCurPort);
}

