//	File:		FWPart.h
//	Contains:	Some Utilities
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWUTIL_
#define _FWUTIL_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

// ----- Macintosh Includes -----

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __LIBRARYMANAGERUTILITIES__
#include <LibraryManagerUtilities.h>
#endif

#ifndef __LIBRARYMANAGERCLASSES__
#include <LibraryManagerClasses.h>
#endif

//==============================================================================
// Forward Declarations
//==============================================================================

class CFrame;
class CFacet;
class XMPShape;
class XMPTransform;
class XMPWindow;
class XMPStorageUnit;

//==============================================================================
// Global Utilities
//==============================================================================

void 		SortRect(Rect& rect);
XMPSLong 	Abs(XMPSLong value);
void 		GetPortTextStyle(TextStyle* theTextStyle);
void 		SetPortTextStyle(const TextStyle* theTextStyle);

// ----- Key pressed
XMPBoolean IsShiftKeyPressed();
XMPBoolean IsOptionKeyPressed();
XMPBoolean IsCommandKeyPressed();
XMPBoolean IsControlKeyPressed();
XMPBoolean IsKeyPressed(XMPUShort theKey);

// ----- XMPShape utilities
void		InsetXMPShape(XMPShape *shape, XMPSShort xInset, XMPSShort yInset);
void 		GetXMPShapeQDBox(XMPShape *shape, Rect& box);

XMPShape*	NewXMPShape();
XMPShape*	NewXMPShape(XMPShape* otherShape);
XMPShape*	NewXMPShape(XMPRect* rect);
XMPShape*	NewXMPShape(const Rect& rect);
XMPShape*	NewXMPShape(RgnHandle rgn);
XMPShape*	NewXMPShape(XMPStorageUnit* su);

// ----- XMPTransform utilities
XMPTransform* 	NewXMPTransform();
XMPTransform* 	NewXMPTransform(XMPTransform* otherTransform);
XMPTransform* 	NewXMPTransform(Point offset);
XMPTransform* 	NewXMPTransform(const XMPPoint& offset);

//==============================================================================
//	 class FW_CAcquireASLMResourceAccess
//==============================================================================

class FW_CAcquireASLMResourceAccess
{
public:
	FW_CAcquireASLMResourceAccess();
	virtual ~FW_CAcquireASLMResourceAccess();

private:
	XMPSLong		fShellLibRefNum;
	TLibraryFile*	fShellLibFile;
};

//==============================================================================
//	 class CDrawInitiator
//==============================================================================

class CDrawInitiator
{
public:
	CDrawInitiator(CFacet *facet, XMPShape* clipShape = kXMPNULL);
	CDrawInitiator(XMPWindow* xmpWindow);
	
	~CDrawInitiator();

private:
	GrafPtr		fCurPort;
	Point		fCurOrigin;
	RgnHandle	fClip;
};

//==============================================================================
//	 Global Inline Methods
//==============================================================================

//------------------------------------------------------------------------------
// TestShiftKey
//------------------------------------------------------------------------------

inline XMPBoolean TestShiftKey(EventRecord* event)
{
	return (event->modifiers & 0x0200) != 0;
}

//------------------------------------------------------------------------------
// TestCommandKey
//------------------------------------------------------------------------------

inline XMPBoolean TestCommandKey(EventRecord* event)
{
	return (event->modifiers & 0x0100) != 0;
}

//------------------------------------------------------------------------------
// TestOptionKey
//------------------------------------------------------------------------------

inline XMPBoolean TestOptionKey(EventRecord* event)
{
	return (event->modifiers & 0x0800) != 0;
}

//------------------------------------------------------------------------------
// TestControlKey
//------------------------------------------------------------------------------

inline XMPBoolean TestControlKey(EventRecord* event)
{
	return (event->modifiers & 0x1000) != 0;
}

//------------------------------------------------------------------------------
// ::IsShiftKeyPressed
//------------------------------------------------------------------------------

inline XMPBoolean IsShiftKeyPressed()
{
	return ::IsKeyPressed(0x38);
}

//------------------------------------------------------------------------------
// ::IsOptionKeyPressed
//------------------------------------------------------------------------------

inline XMPBoolean IsOptionKeyPressed()
{
	return ::IsKeyPressed(0x3A);
}

//------------------------------------------------------------------------------
// ::IsCommandKeyPressed
//------------------------------------------------------------------------------

inline XMPBoolean IsCommandKeyPressed()
{
	return ::IsKeyPressed(0x37);
}

//------------------------------------------------------------------------------
// ::IsControlKeyPressed
//------------------------------------------------------------------------------

inline XMPBoolean IsControlKeyPressed()
{
	return ::IsKeyPressed(0x3B);
}


#endif