/*
	File:		FWView.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<3+>	 3/17/94	DS		Add CDECL to dtor for CView
		 <3>	 3/10/94	DS		Remove bogus method

	To Do:
*/

//	File:		FWView.h
//	Contains:	Definition for class FW_CView
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWVIEW_
#define _FWVIEW_

// ----- Framework Includes -----

#ifndef _FWEVENTH_
#include "FWEventH.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _PLFMTYPE_
#include "PlfmType.h"
#endif


//========================================================================================
// Forward class declarations
//========================================================================================

class XMPPoint;
class XMPRect;
class XMPShape;
class XMPTransform;
class CFacet;
class OrderedCollection;


//========================================================================================
// CLASS FW_CView
//========================================================================================

class FW_CView : public MEventHandler
{
// ----- Initialization/destruction
public:
						FW_CView();
	virtual				~ CDECL FW_CView();
	
	void				InitView(CFacet* facet, FW_CView* enclosure,
								 XMPID identifier,
								 XMPTransform* transform, XMPShape* shape);
	void				InitView(FW_CView* enclosure,
								 XMPID identifier,
								 XMPTransform* transform, XMPShape* shape);

// ----- Update
public:
	void 				HandleDraw(XMPShape* area);
	virtual void 		Draw(XMPShape* area);

		
// ----- Mouse events
public:
	XMPBoolean			HandleMouseDown(const XMPPoint& where, XMPEventData event);
							// In this view coordinate
	virtual XMPBoolean	MouseDown(const XMPPoint& where, XMPEventData event);
							// In this view coordinate
	XMPBoolean			HandleMouseUp(const XMPPoint& where, XMPEventData event);
							// In this view coordinate
	virtual XMPBoolean	MouseUp(const XMPPoint& where, XMPEventData event);
							// In this view coordinate
	XMPBoolean			IsPointInView(const XMPPoint &point);
							// In enclosure coordinates

// ----- View Shape
public:
	XMPShape*			GetShape() const;
	void				SetShape(XMPShape* shape);
	void				EnclosureChangedShape(XMPShape* oldShape,
											  XMPShape* newShape);

// ----- conversion
public:
	void				ViewToEnclosure(XMPShape& shape) const;
	void				EnclosureToView(XMPShape& shape) const;	
	void				ViewToEnclosure(XMPPoint& point) const;
	void				EnclosureToView(XMPPoint& point) const;
	
	XMPTransform*		ComputeViewTransform() const;

// ----- View List Management
public:
	void				AddSubview(FW_CView *subview);
	void				RemoveSubview(FW_CView *subview);
	
	unsigned long		CountSubviews() const;
	XMPBoolean			HasSubviews() const;
	XMPBoolean			IsInSubviewList(FW_CView *subview) const;
	
	FW_CView*			GetFirstSubview() const;
	FW_CView*			GetSubviewAt(unsigned long index) const;
	FW_CView*			GetNextSubview(FW_CView* subview,
									   XMPBoolean circle = kXMPFalse) const;
	FW_CView*			GetLastSubview() const;
	
	FW_CView*			FindSubview(XMPID identifier) const;

// ----- Facet
public:
	CFacet*				GetFacet() const;
	virtual void		SetFacet(CFacet* facet);
	
// ----- Fields
private:
	CFacet*				fFacet;
	FW_CView*			fEnclosure;
	XMPShape*			fShape;
	XMPTransform*		fTransform;
	OrderedCollection*	fSubviewList;
};

//----------------------------------------------------------------------------------------
// FW_CView::GetFacet
//----------------------------------------------------------------------------------------

inline CFacet* FW_CView::GetFacet() const
{
	return fFacet;
}

//----------------------------------------------------------------------------------------
// FW_CView::InitView
//----------------------------------------------------------------------------------------

inline void FW_CView::InitView(FW_CView* enclosure,
							   XMPID identifier,
							   XMPTransform* transform,
							   XMPShape* shape)
{
	InitView(enclosure->GetFacet(), enclosure, identifier, transform, shape);
}


//========================================================================================
// CLASS FW_CViewDrawInitiator
//========================================================================================

class FW_CViewDrawInitiator
{
// ----- Initialization/destruction
public:
						FW_CViewDrawInitiator(FW_CView* view);
	virtual				~FW_CViewDrawInitiator();

private:
	CDrawInitiator		fDrawInitiator;
	GrafPtr				fCurPort;
	Point				fCurOrigin;
	RgnHandle			fClip;
};

#endif
