/*
	File:		FWWindow.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 4/18/94	DS		Pass false to CreateXMPWindow's Should save
									parameter.
		 <2>	 3/31/94	DS		Remove reference to XMPSessn.h

	To Do:
*/

//	File:		FWWindow.cpp
//	Contains:	Implementation of CWindow, CPalette
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWWINDOW_
#include "FWWindow.h"
#endif

#ifndef _FWEVENTH_
#include "FWEventH.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _XMPSESSN_
#include "XMPSessM.h"
#endif


#pragma segment framework

//==============================================================================
// CLASS CWindow
//==============================================================================

//------------------------------------------------------------------------------
// CWindow::CWindow
//------------------------------------------------------------------------------

CWindow::CWindow() :
	fXMPWindow(kXMPNULL)
{
}

//------------------------------------------------------------------------------
// CWindow::InitWindow
//------------------------------------------------------------------------------

void CWindow::InitWindow(CPart* part, const XMPRect& windowRect)
{
	fPart = part;
	fWindowRect = windowRect;
}

//------------------------------------------------------------------------------
// CWindow::InitWindow
//------------------------------------------------------------------------------

void CWindow::InitWindow(XMPWindow* xmpWindow)
{
	fXMPWindow = xmpWindow;
	fXMPWindow->IncrementRefCount();
}

//------------------------------------------------------------------------------
// CWindow::~CWindow
//------------------------------------------------------------------------------

CWindow::~CWindow()
{
	if (fXMPWindow) {
		fXMPWindow->Release();
		fXMPWindow = kXMPNULL;
	}
}

//------------------------------------------------------------------------------
// CWindow::CloseWindow
//
// Never call delete myWindow to close a window, call CloseWindow
//------------------------------------------------------------------------------

void CWindow::CloseWindow()
{
	if (fXMPWindow)
	{
		HideWindow();
		fXMPWindow->Close();	// 'this' will be deleted automatically
	}		
}

//------------------------------------------------------------------------------
// CWindow::ShowWindow
//------------------------------------------------------------------------------

void CWindow::ShowWindow()
{
	if (fXMPWindow)
		fXMPWindow->Show();
}

//------------------------------------------------------------------------------
// CWindow::HideWindow
//------------------------------------------------------------------------------

void CWindow::HideWindow()
{
	if (fXMPWindow)
		fXMPWindow->Hide();
}

//------------------------------------------------------------------------------
//	CWindow::IsShown
//------------------------------------------------------------------------------

XMPBoolean CWindow::IsShown() const
{
	return fXMPWindow ? fXMPWindow->IsShown() : kXMPFalse;
}

//==============================================================================
// CLASS CFloatingWindow
//==============================================================================

//------------------------------------------------------------------------------
// CFloatingWindow::CFloatingWindow
//------------------------------------------------------------------------------

CFloatingWindow::CFloatingWindow()
{
	fFloatingShown = kXMPFalse;
}

//------------------------------------------------------------------------------
// CFloatingWindow::InitFloatingWindow
//------------------------------------------------------------------------------

void CFloatingWindow::InitFloatingWindow(CPart* part,
										 XMPTypeToken presentation,
										 const XMPRect& windowRect)
{
	fPresentation = presentation;
	InitWindow(part, windowRect);
}

//------------------------------------------------------------------------------
// CFloatingWindow::InitFloatingWindow
//------------------------------------------------------------------------------

void CFloatingWindow::InitFloatingWindow(XMPWindow* xmpWindow)
{
	InitWindow(xmpWindow);
	fFloatingShown = xmpWindow->GetWasVisible();
}

//------------------------------------------------------------------------------
// CFloatingWindow::~CFloatingWindow
//------------------------------------------------------------------------------

CFloatingWindow::~CFloatingWindow()
{
}

//------------------------------------------------------------------------------
// CFloatingWindow::CreateFloatingWindow
//------------------------------------------------------------------------------

void CFloatingWindow::CreateFloatingWindow(XMPTypeToken presentation)
{

	fXMPWindow = fPart->CreateXMPWindow(kXMPNULL,
									  fPart->GetSession()->Tokenize(kXMPViewAsFrame),
									  presentation,
									  "",
									  fWindowRect,
									  kXMPTrue,			// close box
									  kXMPFalse,		// resizable
									  kXMPTrue,			// floating
									  kXMPFalse,		// shouldn't save ?????
									  kWindoidWDEF + kWindoidNotResizable);
	
	fXMPWindow->IncrementRefCount();
	
	fXMPWindow->Open();
}

//------------------------------------------------------------------------------
// CFloatingWindow::ShowWindow
//------------------------------------------------------------------------------

void CFloatingWindow::ShowWindow()
{
	if (!fFloatingShown)
	{
		if (fXMPWindow == kXMPNULL)
			CreateFloatingWindow(fPresentation);
		
		CWindow::ShowWindow();
		fFloatingShown = kXMPTrue;
	}
}

//------------------------------------------------------------------------------
// CFloatingWindow::HideWindow
//------------------------------------------------------------------------------

void CFloatingWindow::HideWindow()
{
	if (fFloatingShown)
	{		
		CWindow::HideWindow();
		fFloatingShown = kXMPFalse;
	}
}

//------------------------------------------------------------------------------
// CFloatingWindow::HideShowFloating
//------------------------------------------------------------------------------

void CFloatingWindow::HideShowFloating()
{
	if (fFloatingShown)
		HideWindow();	
	else
		ShowWindow();
}

//------------------------------------------------------------------------------
//	CFloatingWindow::IsShown
//------------------------------------------------------------------------------

XMPBoolean CFloatingWindow::IsShown() const
{
	return fFloatingShown;
}

