//	File:		FWWindow.h
//	Contains:	Interface for CWindow, CFloatingWindow
//	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

#ifndef _FWWINDOW_
#define _FWWINDOW_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

//==============================================================================
// Forward Declarations
//==============================================================================

class 	CPart;
class  	CFacet;
class	CFrame;


//==============================================================================
// defines
//==============================================================================

#define kWindoidWDEF			48				// the default windoid WDEF proc ID
#define kWindoidHilite			 1				// add this to kWindoidWDEF if the title bar can be unhilited
#define kWindoidTallTitleBar	 2				// add this to kWindoidWDEF if the title bar has a title
#define kWindoidNotResizable	 4				// add this to kWindoidWDEF if the windoid is NOT resizable
#define kWindoidZoomable		 8				// add this to kWindoidWDEF if the windoid is zoomable

//==============================================================================
// Class CWindow
//==============================================================================

class CWindow
{
public:
	CWindow();
	void InitWindow(CPart* part, const XMPRect& windowRect);
	void InitWindow(XMPWindow* xmpWindow);
	virtual ~CWindow();
	
	XMPWindow*		GetXMPWindow() const;

	void			CloseWindow();	
	virtual void 	ShowWindow();
	virtual void 	HideWindow();
	
	virtual XMPBoolean IsShown() const;
	XMPBoolean 		IsFloating() const;
	
	void			InClose();
	
protected:
	XMPWindow*		fXMPWindow;
	CPart*			fPart;
	XMPRect			fWindowRect;
};

//------------------------------------------------------------------------------
//	CWindow::GetXMPWindow
//------------------------------------------------------------------------------
inline XMPWindow* CWindow::GetXMPWindow() const
{
	return fXMPWindow;
}

//------------------------------------------------------------------------------
//	CWindow::IsFloating
//------------------------------------------------------------------------------
inline XMPBoolean CWindow::IsFloating() const
{
	return fXMPWindow ? fXMPWindow->IsFloating() : kXMPFalse;
}

//==============================================================================
// Class CFloatingWindow
//==============================================================================

class CFloatingWindow : public CWindow
{
public:
	CFloatingWindow();
	void InitFloatingWindow(CPart* part,
							XMPTypeToken presentation,
							const XMPRect& windowRect);
		// AMB - May just need presentation since the root frame/facet can't be an
		// icon or thumbnail, can it?
	void InitFloatingWindow(XMPWindow* xmpWindow);
		
	virtual ~CFloatingWindow();
	
	virtual void 	ShowWindow();
	virtual void 	HideWindow();

	void			HideShowFloating();
	virtual XMPBoolean IsShown() const;

private:
	void			CreateFloatingWindow(XMPTypeToken presentation);
		
private:
	XMPBoolean		fFloatingShown;
	XMPTypeToken	fPresentation;
};

#endif
