/*
	File:		ButtonPart.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	 4/19/94	DS		Fix
		 <5>	 4/10/94	DS		Change SetButtonSize to AdjustButton. Also
									Set the content property during part
									initialization.
		 <4>	  4/7/94	DS		Change SetButtonSize to AdjustButton. You
									must call AdjustButton after calling
									SetButtonName.
		 <3>	 3/31/94	DS		Remove typecast of this pointer in call to
									InitSemanticInterface.

	To Do:
*/

//	File:		ScriptButtonPart.cp
//	Contains:	Implementation of classes 
//					- CScriptButtonPart
//					- CButtonFrame
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.
//	Author:		Henri Lamiraux

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include <FWUtil.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

// ----- Macintosh Includes -----

#ifndef mathRoutinesIncludes
#include <math routines.h>
#endif

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef __COMPONENTS__
#include <Components.h>
#endif

#ifndef __OSA__
#include <OSA.h>
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif


#ifndef _ButtonPart_
#include "ButtonPart.h"
#endif

#ifndef _BUTTONPARTAE_
#include "ButtonPartAE.h"
#endif



//==============================================================================
//	Defines
//==============================================================================
#define kButtonNameProp	"appl:cscriptbuttonpart:button name"



#pragma segment ScriptButtonPart

//==============================================================================
//	 class CScriptButtonPart
//==============================================================================

//------------------------------------------------------------------------------
//	 CScriptButtonPart::CScriptButtonPart
//------------------------------------------------------------------------------

CScriptButtonPart::CScriptButtonPart()
{
	fOSAID = kXMPNULL;		
	fComponent = kXMPNULL;	
	fSemtIntf = kXMPNULL;
	fSession = kXMPNULL;
	fCScriptButtonFrame = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CScriptButtonPart::~CScriptButtonPart
//------------------------------------------------------------------------------

CScriptButtonPart::~CScriptButtonPart()
{
	if(fOSAID != 0)
		OSADispose(fComponent, fOSAID);
	
	if (fSemtIntf != kXMPNULL)	
		delete fSemtIntf;
		
	if (fComponent != kXMPNULL)
		CloseComponent(fComponent);
}

//----------------------------------------------------------------------------------------
// CScriptButtonPart::Initialize
//----------------------------------------------------------------------------------------

void CScriptButtonPart::Initialize()
{	
	{	// load defaults
		FW_CAcquireASLMResourceAccess aq;
		
		fScriptH = ::GetResource(kOSAScriptResourceType, 128);
		if (fScriptH)
			::DetachResource(fScriptH);

		::GetIndString(fButtonName, 128, 1);
	}

	fButtonSize.x = 80;	fButtonSize.y = 20;

	// initialize component
	if (fComponent == kXMPNULL) {
		fComponent = ::OpenDefaultComponent(kOSAComponentType, kOSAGenericScriptingComponentSubtype);
		if(((long)fComponent == (long)badComponentInstance) || 
			((long)fComponent == (long)badComponentSelector)) {
			fComponent = kXMPNULL; 	
		}
	} 

	// initialize semantic interface 
	fSemtIntf = new XMPSemanticInterface();
	fSemtIntf->InitSemanticInterface(this);

	fSession = (XMPSession*)GetStorageUnit()->GetSession();
	InstallObjectAccessors();


	CCtlMgrPart::Initialize();	// must call AFTER loading the defaults!!
} 

//------------------------------------------------------------------------------
//  CScriptButtonPart::InternalizeContent
//------------------------------------------------------------------------------

void CScriptButtonPart::InternalizeContent(XMPStorageUnit* storageUnit)
{
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);

	XMPULong scriptSize = storageUnit->GetSize();
	if (scriptSize > 0) {
		Handle scriptHandle = ::XMPNewHandle(scriptSize);
		
		::HLock(scriptHandle);
		storageUnit->GetValue(scriptSize, (XMPValue)*scriptHandle);
		::HUnlock(scriptHandle);
		
		this->SetScriptHandle(scriptHandle);

		// ----- Look for an annotation property -----
		if (storageUnit->Exists(kButtonNameProp, kXMPISOStr, 0))
		{
			Str255	tmpStr;
			
			storageUnit->Focus(kButtonNameProp, kXMPPosUndefined, kXMPISOStr, (XMPValueIndex)0, kXMPPosUndefined);
			
			tmpStr[0] = (unsigned char)storageUnit->GetSize() - 1;	// -1 for NULL at end
			storageUnit->GetValue(tmpStr[0], (XMPValue)&tmpStr[1]);
			SetButtonName((char *)tmpStr);
			
		}
	}
}

//----------------------------------------------------------------------------------------
//  CScriptButtonPart::ExternalizeContent
//----------------------------------------------------------------------------------------

void CScriptButtonPart::ExternalizeContent(XMPStorageUnit* storageUnit)
{	
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);
	storageUnit->Remove();
	storageUnit->AddValue(this->GetContentPropertyValueType());

	::HLock(fScriptH);
	storageUnit->SetValue(::XMPGetHandleSize(fScriptH), (XMPValue)*fScriptH);
	::HUnlock(fScriptH);

	{
		Str255		tmpStr;

		storageUnit->AddProperty(kButtonNameProp)->AddValue(kXMPISOStr);
		storageUnit->Focus(kButtonNameProp, kXMPPosUndefined, kXMPISOStr, (XMPValueIndex)0, kXMPPosUndefined);
	
		BlockMove(&fButtonName[1], tmpStr, fButtonName[0]);
		tmpStr[fButtonName[0]+1] = 0;	// set to null
		
		storageUnit->SetValue(fButtonName[0]+1, (XMPValue)&tmpStr);
	}
}

//------------------------------------------------------------------------------
//	 CScriptButtonPart::CreateControlHandle
//------------------------------------------------------------------------------

ControlHandle CScriptButtonPart::CreateControlHandle(XMPPlatformWindow window, const XMPRect& controlRect)
{
	Rect rect;
	controlRect.AsQDRect(rect);
	
	return ::NewControl(window, &rect, fButtonName, kXMPTrue, 0, 0, 1, pushButProc, (XMPSLong)this);
}

//----------------------------------------------------------------------------------------
//	 CScriptButtonPart::GetContentPropertyValueType
//----------------------------------------------------------------------------------------

XMPValueType CScriptButtonPart::GetContentPropertyValueType() const
{
	return kSampleButtonKind;
}

//------------------------------------------------------------------------------
//	 CScriptButtonPart::NewFrame
//------------------------------------------------------------------------------

CFrame* CScriptButtonPart::NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation)
{
XMPUnused(presentation);
	
	CScriptButtonFrame *frame = new CScriptButtonFrame;
	frame->InitButtonFrame(xmpFrame, this);
	fCScriptButtonFrame = frame;
	return frame;
}

//----------------------------------------------------------------------------------------
//  CScriptButtonPart::NewSelection
//----------------------------------------------------------------------------------------

FW_CSelection* CScriptButtonPart::NewSelection()
{
	fDragger = new CButtonDragObj;
	fDragger->InitButtonDragger(this);
	return fDragger;
}

//------------------------------------------------------------------------------
// CScriptButtonPart::HasExtension
//------------------------------------------------------------------------------

XMPBoolean	CScriptButtonPart::HasExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	if (!XMPISOStrCompare( (XMPISOStr)extensionName, semtIntf ))
		return kXMPTrue;
	else
		return kXMPFalse;
}
	
//------------------------------------------------------------------------------
// CScriptButtonPart::GetExtension
//------------------------------------------------------------------------------

XMPExtension*	CScriptButtonPart::GetExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	if (!XMPISOStrCompare( (XMPISOStr)extensionName, semtIntf ))
		return fSemtIntf;
	else
		return kXMPNULL;
}


// ** Internal stuff! **
//----------------------------------------------------------------------------------------
// CScriptButtonPart::GetButtonFrame
//----------------------------------------------------------------------------------------


CScriptButtonFrame* CScriptButtonPart::GetButtonFrame()
{
	return fCScriptButtonFrame;
}

//------------------------------------------------------------------------------
//	 CScriptButtonPart::AdjustButton
//------------------------------------------------------------------------------

void CScriptButtonPart::AdjustButton()
{
	XMPPoint controlSize;
	GetControlSize(controlSize);
	
	ControlHandle	theButton = GetControlHandle();
	XMPShape controlShape;
	XMPRect xmpRect(0, 0, controlSize.x, controlSize.y);
	controlShape.SetRectangle(&xmpRect);
	Str255 buttonName;

	// and adjust the size of the control to match the name
	::GetControlTitle(theButton, buttonName);
	{
		XMPUShort	ctrlWidth;
		
		ctrlWidth = StringWidth((ConstStr255Param)buttonName) + 20;	// 10 pixels on each side
		XMPPoint xmpPoint(ff(ctrlWidth), ff(20));	// always leave 20 pixels high
		SetControlSize(xmpPoint);
	}
	
	
	CPartDisplayFrameIterator ite(this);
	for (CCtlMgrFrame *frame = (CCtlMgrFrame*)ite.First(); ite.IsNotComplete(); frame = (CCtlMgrFrame*)ite.Next())
	{
		frame->Invalidate(&controlShape);
	}	

/*
	XMPRect xmpRect(0, 0, controlSize.x, controlSize.y);
	XMPShape controlShape;
	controlShape.SetRectangle(&xmpRect);
	
	fButtonSize = controlSize;	// store it off!
	
	CPartDisplayFrameIterator ite(this);
	for (CCtlMgrFrame *frame = (CCtlMgrFrame*)ite.First(); ite.IsNotComplete(); frame = (CCtlMgrFrame*)ite.Next())
	{
		XMPShape *shape = ::NewXMPShape(&controlShape);
		frame->RequestFrameShape(shape);
		frame->UpdateUsedAndActiveShapes();
		frame->Invalidate(shape);
	}
*/
}

//------------------------------------------------------------------------------
//	 CScriptButtonPart::DoClick
//------------------------------------------------------------------------------

void CScriptButtonPart::DoClick(const XMPPoint& where, XMPEventData event)
{
	XMPUnused(where);
	
	if (event->modifiers & optionKey)	AskForButtonName();
	else {
		if (this->GetScriptID() != kXMPNULL)
			this->ExecuteScriptID();
		else if (fScriptH)
			ExecuteScriptHandle();
		else
			SysBeep(0);
	}
}


//----------------------------------------------------------------------------------------
// 	 CScriptButtonPart::GetButtonName
//----------------------------------------------------------------------------------------
char* CScriptButtonPart::GetButtonName()
{
	Str255			name;
	ControlHandle	theButton = GetControlHandle();
	
	::GetCTitle(theButton, name);
	return((char *)name);
}

//----------------------------------------------------------------------------------------
// 	 CScriptButtonPart::SetButtonName
//----------------------------------------------------------------------------------------

void CScriptButtonPart::SetButtonName(char *buttonName)
{
	ControlHandle	theButton = GetControlHandle();
	
	BlockMove(buttonName, fButtonName, 255);	// copy into our data member
	::SetControlTitle(theButton, (ConstStr255Param) fButtonName);	
	
	// and marked it changed for saving
	Changed();
}

//----------------------------------------------------------------------------------------
// 	 CScriptButtonPart::SetScriptHandle
//----------------------------------------------------------------------------------------

void CScriptButtonPart::SetScriptHandle(Handle scriptHandle)
{
	::DisposeHandle(fScriptH);
	fScriptH = scriptHandle;

	ScriptHandleToID();	// convert to new form
	// fOSAID = kXMPNULL;
	
	Changed();
}

//------------------------------------------------------------------------------
// CScriptButtonPart::GetScriptID
//------------------------------------------------------------------------------

OSAID CScriptButtonPart::GetScriptID()
{
	return fOSAID;
}
	
//------------------------------------------------------------------------------
// CScriptButtonPart::SetScriptID
//------------------------------------------------------------------------------

void  CScriptButtonPart::SetScriptID(OSAID theOSAID)
{
	OSAError	theOSAErr;
	AEDesc	tmpDesc;
	
	fOSAID = theOSAID;
	
	if (fScriptH) {
		::DisposeHandle(fScriptH);
		fScriptH = kXMPNULL;
	}

	theOSAErr = OSACoerceToDesc(fComponent, fOSAID, kOSAScriptResourceType, kOSANullScript, &tmpDesc);
	fScriptH = tmpDesc.dataHandle;
	
	Changed();
}

//------------------------------------------------------------------------------
// CScriptButtonPart::GetComponent
//------------------------------------------------------------------------------

ComponentInstance	CScriptButtonPart::GetComponent() 
{
	return fComponent; 
}

//----------------------------------------------------------------------------------------
// 	 CScriptButtonPart::AskForButtonName
//----------------------------------------------------------------------------------------

void CScriptButtonPart::AskForButtonName()
{
	DialogPtr		dlog;
	short			item;

	{
		FW_CAcquireASLMResourceAccess aq;		// sets up ASLM resource access
	
		dlog = GetNewDialog(128, NULL, (WindowPtr) -1);
	}
	
	if (dlog) {
		ShowWindow(dlog);
		
		while ((item != 1) && (item != 2)) {
			ModalDialog((ModalFilterUPP)NULL, &item);
		}
		
		if (item == 1) {
			Str255	str;
			Handle h;	Rect itembox;	short itype;
	
			GetDItem(dlog, 3, &itype, &h, &itembox);
			GetIText(h, str);
			
			SetButtonName((char *)str);
			AdjustButton();
		}
		
		DisposeDialog(dlog);
	}
}

//------------------------------------------------------------------------------
// CScriptButtonPart::ScriptHandleToID
//------------------------------------------------------------------------------

void CScriptButtonPart::ScriptHandleToID() 
{
   OSErr				theOSAErr = noErr;
	AEDesc				source;
	
   // Put our text into an AEDesc
   HLock(fScriptH);
   
   theOSAErr = AECreateDesc(kOSAScriptResourceType, (Ptr)*fScriptH, (Size)(GetHandleSize(fScriptH)), &source);
   if(!theOSAErr) {
	   #if 1
			theOSAErr = OSACoerceFromDesc(fComponent, &source, kOSANullScript, &fOSAID);
	   #else
			theOSAErr = OSALoad(fComponent, &source, kOSANullScript, &fOSAID);
	   #endif
   }
   
   HUnlock(fScriptH);
   
	if(source.dataHandle != nil) 	AEDisposeDesc(&source);
}

//------------------------------------------------------------------------------
// CScriptButtonPart::ExecuteScriptID
//------------------------------------------------------------------------------

OSAError CScriptButtonPart::ExecuteScriptID() 
{
	OSAError 	theOSAErr = noErr;
	long		modeFlags = 0;
	OSAID 		resultingScriptValueID;
	
	theOSAErr = OSAExecute(	fComponent,
							fOSAID,
							kOSANullScript,
							modeFlags,
							&resultingScriptValueID);
	return theOSAErr;
}

//----------------------------------------------------------------------------------------
// 	 CScriptButtonPart::ExecuteScriptHandle
//----------------------------------------------------------------------------------------

OSErr CScriptButtonPart::ExecuteScriptHandle()
{
   OSErr				theOSAErr = noErr;
   OSErr				theErr = noErr;
   OSAID				resultingID = 0, executeID = 0;
   long					modeFlags = kOSAModeNull;
   AEDesc				source, result;
   ComponentInstance	gASComponent = 0;
 	
	
   // open generic component
 
   gASComponent = OpenDefaultComponent(kOSAComponentType, kOSAGenericScriptingComponentSubtype);
 
   // Check errors here!!
   if(!gASComponent /*(gASComponent == badComponentInstance) || (gASComponent == badComponentSelector)*/)
		return invalidComponentID;  // Is this the right error??
 

   // Put our text into an AEDesc
   HLock(fScriptH);
   theErr = AECreateDesc(kOSAScriptResourceType, (Ptr)*fScriptH, (Size)(GetHandleSize(fScriptH)), &source);
   if(theErr != noErr) {
	   theErr = theOSAErr;
	   goto BAILLABEL;
   }
   
   
	theOSAErr = OSALoad(gASComponent, &source, kOSAModeNull, &resultingID);
	
	if (!theOSAErr) {
		theOSAErr = OSAExecute(gASComponent, resultingID, kOSANullScript, modeFlags, &executeID);
	}
   
   if(theOSAErr){
		OSErr scriptErr;
	   theErr = theOSAErr;
	   
		/*
			Use OSAScriptError to get a valid error string!
		*/
		scriptErr = OSAScriptError(gASComponent, kOSAErrorMessage, typeChar, &result);
								   
	   goto BAILLABEL;
   } else  {
		theOSAErr = OSACoerceToDesc(gASComponent, resultingID, typeWildCard, modeFlags, &result);
   }
 
   theErr = theOSAErr;

   BAILLABEL:;
   // Clean up time
   if(resultingID != 0) 		OSADispose( gASComponent, resultingID);
   if(source.dataHandle != nil) AEDisposeDesc(&source);
   if(gASComponent != 0) 		CloseComponent(gASComponent);

   HUnlock(fScriptH);

	
   return theErr;
}


//==============================================================================
//	 class CScriptButtonFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 CScriptButtonFrame::CScriptButtonFrame
//------------------------------------------------------------------------------

CScriptButtonFrame::CScriptButtonFrame()
{
}

//------------------------------------------------------------------------------
//	 CScriptButtonFrame::InitScriptButtonFrame
//------------------------------------------------------------------------------

void CScriptButtonFrame::InitButtonFrame(XMPFrame* frame, CPart* part)
{
	InitCtlMgrFrame(frame, part);
	
	SetDroppable(kXMPTrue);
}

//------------------------------------------------------------------------------
//	 CScriptButtonFrame::~CScriptButtonFrame
//------------------------------------------------------------------------------

CScriptButtonFrame::~CScriptButtonFrame()
{
}

//------------------------------------------------------------------------------
//	 CScriptButtonFrame::NewFacet
//------------------------------------------------------------------------------

CFacet* CScriptButtonFrame::NewFacet(XMPFacet* xmpFacet)
{
	CScriptButtonFacet* facet = new CScriptButtonFacet;
	facet->InitButtonFacet(xmpFacet);
	return facet;
}

//------------------------------------------------------------------------------
//	 CScriptButtonFrame::UpdateUsedShape
//------------------------------------------------------------------------------

void CScriptButtonFrame::UpdateUsedShape()
{	
	XMPPoint controlSize;
	((CScriptButtonPart*)GetPart())->GetControlSize(controlSize);
	Rect controlRect;
	::SetRect(&controlRect, 0, 0, FixedToInt(controlSize.x), FixedToInt(controlSize.y));
	
	
	RgnHandle rgn = ::NewRgn();
	::InsetRect(&controlRect, -1, -1);
	::OpenRgn();
	::FrameRoundRect(&controlRect, 16, 16);
	::CloseRgn(rgn);
	
	XMPShape* scratch = ::NewXMPShape(rgn);
	scratch->Intersect(GetXMPFrame()->GetFrameShape());
	GetXMPFrame()->ChangeUsedShape(scratch);
}

#if 0
//------------------------------------------------------------------------------
//	 CScriptButtonFrame::FacetAdded
//------------------------------------------------------------------------------

void CScriptButtonFrame::FacetAdded(CFacet* facet)
{
	XMPUnused(facet);
	
	XMPPoint controlSize;
	((CScriptButtonPart*)GetPart())->GetButtonSize(controlSize);
	XMPRect xmpRect(0, 0, controlSize.x, controlSize.y);
	XMPShape* controlShape = ::NewXMPShape(&xmpRect);

	this->RequestFrameShape(controlShape);
	this->UpdateUsedAndActiveShapes();
}

//------------------------------------------------------------------------------
//	 CScriptButtonFrame::FrameShapeChanged
//------------------------------------------------------------------------------

void CScriptButtonFrame::FrameShapeChanged()
{
	XMPRect rect;
	GetXMPFrame()->GetFrameShape()->GetBoundingBox(&rect);
	
	XMPPoint controlSize(rect.right - rect.left, rect.bottom - rect.top);
	((CScriptButtonPart*)GetPart())->SetControlSize(controlSize);
	
	CFrame::FrameShapeChanged();
}
#endif

//==============================================================================
//	 class CScriptButtonFacet
//==============================================================================

//------------------------------------------------------------------------------
//	 CScriptButtonFacet::CScriptButtonFacet
//------------------------------------------------------------------------------

CScriptButtonFacet::CScriptButtonFacet()
{
}

//------------------------------------------------------------------------------
//	 CScriptButtonFacet::InitScriptButtonFacet
//------------------------------------------------------------------------------

void CScriptButtonFacet::InitButtonFacet(XMPFacet* Facet)
{
	InitCtlMgrFacet(Facet);
}

//------------------------------------------------------------------------------
//	 CScriptButtonFacet::~CScriptButtonFacet
//------------------------------------------------------------------------------

CScriptButtonFacet::~CScriptButtonFacet()
{
}


//------------------------------------------------------------------------------
//	 CScriptButtonFacet::DoMouseDown
//------------------------------------------------------------------------------

XMPBoolean CScriptButtonFacet::DoMouseDown(const XMPPoint& where, XMPEventData event)
{
XMPUnused(event);

	CDrawInitiator di(this);
	
	ControlHandle hControl = SetupControl();
	
	Point pt = where.AsQDPoint();
	
	if (::TrackControl(hControl, pt, kXMPNULL))	
		((CScriptButtonPart*)this->GetFrame()->GetPart())->DoClick(where, event);

	RestoreControl();
	
	return kXMPTrue;
}



//==============================================================================
//	 class CButtonDragObj
//==============================================================================

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::CButtonDragObj
//---------------------------------------------------------------------------------------

CButtonDragObj::CButtonDragObj()
{
	fButtonPart = kXMPNULL;
}

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::InitPICTSelection
//---------------------------------------------------------------------------------------

void CButtonDragObj::InitButtonDragger(CScriptButtonPart* thePart)
{
	InitSelection(thePart, kXMPFalse, kXMPFalse);
	
	fButtonPart = thePart;
}

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::~CButtonDragObj
//---------------------------------------------------------------------------------------

CButtonDragObj::~CButtonDragObj()
{	
}


//---------------------------------------------------------------------------------------
//	 CButtonDragObj::CloseSelection
//---------------------------------------------------------------------------------------

void CButtonDragObj::CloseSelection()
{
}

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::DoClear
//---------------------------------------------------------------------------------------
// We don't support clear selection. Just return false because the document
// hasn't changed.

XMPBoolean CButtonDragObj::DoClear()
{
	return kXMPFalse;
}

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::SelectAll
//---------------------------------------------------------------------------------------

void CButtonDragObj::SelectAll()
{
}

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::IsEmpty
//---------------------------------------------------------------------------------------

XMPBoolean CButtonDragObj::IsEmpty() const
{
	return kXMPTrue;	// we never have one!
}

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::ExternalizeSelection
//---------------------------------------------------------------------------------------

void CButtonDragObj::ExternalizeSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
XMPUnused(storageUnit);
XMPUnused(commandFrame);
XMPUnused(cloneKind);
}

//---------------------------------------------------------------------------------------
//	 CButtonDragObj::InternalizeSelection
//---------------------------------------------------------------------------------------

XMPBoolean CButtonDragObj::InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	XMPUnused(cloneKind);

	HFSFlavor	draggedItem;
	XMPBoolean 	internalized = kXMPFalse;

	// ----- If there is a property content -----
	if (storageUnit->Exists(kXMPPropContents, kXMPApplehfs, 0))
	{
		internalized = kXMPTrue;	// set here, but we may need to override later

		storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, kXMPApplehfs, (XMPValueIndex)0, kXMPPosUndefined);
					   
		XMPULong flavorSize = storageUnit->GetSize();	// should be sizeof(HFSFlavor)

		storageUnit->GetValue(flavorSize, (XMPValue)&draggedItem);
		
		{	// open up the res file and check for the resource we want!
			XMPSShort	refNum = ::FSpOpenResFile(&draggedItem.fileSpec, fsRdPerm);
			if (refNum > 0) {
				Handle scriptH;
				
				if (scriptH = Get1Resource(kOSAScriptResourceType, 128)) {
					DetachResource(scriptH);
					fButtonPart->SetScriptHandle(scriptH);

					fButtonPart->SetButtonName((char *)draggedItem.fileSpec.name);
				} else
					internalized = kXMPFalse;	// no resource, nothing happened!
				
				CloseResFile(refNum);
			} else
				internalized = kXMPFalse;	// no resource, nothing happened!
		}
	}
		
	return internalized;
}
