/*
	File:		ButtonPart.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	  4/7/94	DS		Change SetButtonSize to AdjustButton. You
									must call AdjustButton after calling
									SetButtonSize.
		<2+>	 3/17/94	DS		Add CDECL to Button Frame and Button Part
									dtors

	To Do:
*/


#ifndef _ButtonPart_
#define _ButtonPart_

#ifndef _BUTTONDEF_
#include "ButtonDef.h"
#endif

// ----- Framework Includes -----

#ifndef _FWCTLMGR_
#include "FWCtlMgr.h"
#endif

#ifndef _FWSELECT_
#include <FWSelect.h>
#endif

// ---- shouldn't need this but...

#ifndef __OSA__
#include <OSA.h>
#endif


//==============================================================================
//	 Forward Declarations
//==============================================================================

class CButtonDragObj;
class CScriptButtonFrame;
class XMPSemanticInterface;


//==============================================================================
//	 class CScriptButtonPart
//==============================================================================

class CScriptButtonPart : public CCtlMgrPart
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CScriptButtonPart();
	virtual ~ CDECL CScriptButtonPart();
	virtual void Initialize();
	
//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual CFrame*				NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	virtual FW_CSelection* 		NewSelection();

	virtual void 				InternalizeContent(XMPStorageUnit* storageUnit);
	virtual void 				ExternalizeContent(XMPStorageUnit* storageUnit);
	
protected:
	virtual ControlHandle		CreateControlHandle(XMPPlatformWindow window, 
													const XMPRect& controlRect);
	virtual XMPValueType		GetContentPropertyValueType() const;

	XMPMethod void 				InstallObjectAccessors();
									
//------------------------------------------------------------------------------
//	 New API
//
public:

	void						DoClick(const XMPPoint& where, XMPEventData event);

	void						AskForButtonName();


	// Run the script
	OSErr 						ExecuteScriptHandle();
	virtual OSAError			ExecuteScriptID();


	// -Extensions/Semantic Interface-	
	XMPMethod XMPBoolean		HasExtension(XMPType extensionName);
	XMPMethod XMPExtension*		GetExtension(XMPType extensionName);


	// Getters and Setters
	virtual OSAID 				GetScriptID();
	virtual void  				SetScriptID(OSAID theOSAID);
	Handle						GetScriptHandle();
	void						SetScriptHandle(Handle scriptHandle);
	void						ScriptHandleToID();

	virtual ComponentInstance	GetComponent();

	virtual CScriptButtonFrame*	GetButtonFrame();
	
	char* 						GetButtonName();
	void 						SetButtonName(char *buttonName);

	void 						AdjustButton();
	void						GetButtonSize(XMPPoint& controlSize)	const;


//------------------------------------------------------------------------------
//	 Data Members
//
private:
	Str255						fButtonName;	// persistant store items
	XMPPoint					fButtonSize;	
	Handle						fScriptH;		
	
	OSAID						fOSAID;		// Script to run
	ComponentInstance 			fComponent;	// Scripting system connection
	XMPSession*					fSession;	// needed for ObjectAccessors

	CButtonDragObj				*fDragger;
	XMPSemanticInterface*		fSemtIntf;
	CScriptButtonFrame*			fCScriptButtonFrame;
};

//------------------------------------------------------------------------------
//	 CScriptButtonPart::GetButtonSize
//------------------------------------------------------------------------------

inline void CScriptButtonPart::GetButtonSize(XMPPoint& controlSize) const
{
	controlSize = fButtonSize;
}



//==============================================================================
//	 class CScriptButtonFrame
//==============================================================================

#define kCScriptButtonFrameID "appl:cscriptbuttonframe$class,1.0.0"

class CScriptButtonFrame : public CCtlMgrFrame
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CScriptButtonFrame();
	virtual void InitButtonFrame(XMPFrame* frame, CPart* part);	
	virtual ~ CDECL CScriptButtonFrame();

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual CFacet*			NewFacet(XMPFacet* xmpFacet);
	virtual void			UpdateUsedShape();
	#if 0
	virtual void			FacetAdded(CFacet* facet);
	virtual void			FrameShapeChanged();	
	#endif
};


//==============================================================================
//	 class CScriptButtonFacet
//==============================================================================

#define kCScriptButtonFacetID "appl:cscriptbuttonfacet$class,1.0.0"

class CScriptButtonFacet : public CCtlMgrFacet
{

//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CScriptButtonFacet();
	virtual void InitButtonFacet(XMPFacet* xmpFacet);
	virtual ~CScriptButtonFacet();
		

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	// virtual void		Draw(XMPShape* invalidShape);
	virtual XMPBoolean 	DoMouseDown(const XMPPoint& where, XMPEventData event);

};


//==============================================================================
//	 class CButtonDragObj
//==============================================================================

class CButtonDragObj : public FW_CSelection
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CButtonDragObj();
	void			InitButtonDragger(CScriptButtonPart* thePart);
	virtual 		~CButtonDragObj();
	
//------------------------------------------------------------------------------
//	 Inherited API
//
	virtual void		CloseSelection();
	virtual void		SelectAll();
	virtual XMPBoolean	IsEmpty() const;
	virtual XMPBoolean	DoClear();
	
	virtual void		ExternalizeSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual XMPBoolean	InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);

//------------------------------------------------------------------------------
//	 New API
//
// public:
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	CScriptButtonPart*		fButtonPart;
};

#endif
