/*
	File:		ButtonPartAE.cp

	Contains:	Object accessor implementation for the ButtonPart

	Written by:	Donald Olson

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	  4/8/94	DS		Call AdjustButton after SetButtonName
		 <2>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h
		 <1>	1/2/94	DO		first started
	To Do:
*/


#ifndef __ALTPOINT__
#include "AltPoint.h"
#endif

#include "ButtonPartAE.h"

#ifndef _BUTTONPART_
#include "ButtonPart.h"
#endif

 
#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

#ifndef _NAMRSLVR_
#include "NamRslvr.h"
#endif

#ifndef _XMPSESSN_
#include "XMPSessM.h"
#endif

#ifndef __AEOBJECTS__
#include "AEObjects.h"
#endif

#ifndef __AEPACKOBJECT__
#include "AEPackObject.h"
#endif

#ifndef __AEREGISTRY__
#include "AERegistry.h"
#endif

#ifndef _MSSGINTF_
#include "MssgIntf.h"
#endif

#ifndef _SEUTILS_
#include "SEUtils.h"
#endif

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif


#ifndef mathRoutinesIncludes
#include <math routines.h>
#endif


//==============================================================================
// Temporary constants
//==============================================================================

#define cDrawPart 'draw'
#define cDragPart 'drag'
#define cClockPart 'clck'
#define cTestPart 'test'

//==============================================================================
// Semantic Event Handlers and utility Functions
//==============================================================================

// This might be moved into a common header file, but there's something to
// be said for having it right here.

#define XMPOJBECT_ACCESSOR_PARAMS	XMPPart*			thisPart,			\
									DescType			desiredClass,		\
									XMPOSLToken*	container,			\
									DescType			containerClass,		\
									DescType			form,				\
									AEDesc*		selectionData,		\
									XMPOSLToken*		value,				\
									XMPSLong			refCon


static XMPError GetPropertyFromPart( XMPOJBECT_ACCESSOR_PARAMS );
static XMPError GetPropertyFromNULL( XMPOJBECT_ACCESSOR_PARAMS );

static XMPError HandleSetData(XMPPart*		thisPart,
						AppleEvent*	theAppleEvent,
						AppleEvent*	reply,
						XMPSLong	handlerRefcon);
						
static XMPError HandleGetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon);



//------------------------------------------------------------------------------
// CScriptButtonPart::InstallObjectAccessors
//------------------------------------------------------------------------------

void CScriptButtonPart::InstallObjectAccessors()
{
	fSemtIntf->InstallObjectAccessor(cProperty, typeNull, GetPropertyFromNULL,
										(XMPSLong)fSession);

 	fSemtIntf->InstallObjectAccessor(cProperty, cPart, GetPropertyFromPart,
										(XMPSLong)fSession);
										
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAESetData,
												HandleSetData,
												(XMPSLong)fSession);
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAEGetData,
												HandleGetData,
												(XMPSLong)fSession);
}

//------------------------------------------------------------------------------
// GetPropertyFromPart
//------------------------------------------------------------------------------

static XMPError GetPropertyFromPart(XMPPart*		thisPart,
								DescType			desiredClass,
								XMPOSLToken*		container,
								DescType			containerClass,
								DescType			form,
								AEDesc*				selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(desiredClass);
XMPUnused(container);
XMPUnused(containerClass);

	//DebugStr("\pGetPropertyFromPart");
	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();
	DescType				propID;
	ScriptButtonPartPropAccessor*	accessorObj;
	XMPError	error = noErr;

	TRY

	if (form != formPropertyID)
		THROW(errAEWrongDataType);

	switch( propID = **(DescType**)(selectionData->dataHandle) )
	{
		case pBounds:
		case pScript:
		case pName:
			resolver->CreateToken(value, cProperty, thisPart);
			accessorObj = new ScriptButtonPartPropAccessor(propID, (CScriptButtonPart*)thisPart);
			ReallocAndStuff2nd4Bytes(value, (XMPULong)accessorObj);
			break;
		default:
			THROW(errAECantSupplyType);
			break;
	}
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetPropertyFromPart()


//------------------------------------------------------------------------------
// HandleSetData
//------------------------------------------------------------------------------

static XMPError HandleSetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
	XMPUnused(thisPart);
	XMPUnused(reply);
	XMPUnused(handlerRefcon);

	
	AEDesc theData ;
	XMPError	error = noErr;
	
	//DebugStr("\pHandleSetData");

	TRY
	
	THROW_IF_ERROR( AEGetKeyDesc( theAppleEvent, keyAEData,
              typeWildCard, &theData ) ) ;

	AEDesc theToken;
	OSErr err =  AEGetParamDesc( theAppleEvent, keyDirectObject, typeWildCard,
			&theToken ) ;
	if ( err == noErr )
	{
		switch (theToken.descriptorType)
		{
			case typeProperty:
				ScriptButtonPartPropAccessor* btnPropAccessorObj
						= (ScriptButtonPartPropAccessor*)GetSecondFourBytesFromDataHandle
								((XMPOSLToken*)&theToken);
					btnPropAccessorObj->SetData( &theData );
				delete btnPropAccessorObj;
				break;
			
			default:
				ASSERTM( false, errAECantSupplyType, "You shouldn't be here" ) ;
			
		}
	}
	AEDisposeDesc( &theData ) ;
	THROW_IF_ERROR( err ) ;

	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// HandleSetData()


//------------------------------------------------------------------------------
// HandleGetData
//------------------------------------------------------------------------------

static XMPError HandleGetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
XMPUnused(thisPart);
XMPUnused(handlerRefcon);

	DescType		reqType;
	AEDesc			theToken;
	Size			theSize;
	AEDesc			objectData;
	XMPError		result;
	XMPError		error;

	// DebugStr("\pHandleGetData");
	TRY

	result = AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard,
								&theToken);
	THROW_IF_ERROR(result);

			//	Next, get the requested return type, if it exists.
	result = AEGetParamPtr(theAppleEvent,keyAERequestedType,typeType,
					&reqType,(Ptr)&reqType,sizeof(reqType),&theSize);
	if (result == errAEDescNotFound)		// Not an error if return type is not found.
	{
		result = noErr;
		reqType = typeWildCard;
	}
	THROW_IF_ERROR(result);

	switch (theToken.descriptorType)
	{
		case typeProperty:
			ScriptButtonPartPropAccessor* btnPropAccessorObj
					= (ScriptButtonPartPropAccessor*)GetSecondFourBytesFromDataHandle
							((XMPOSLToken*)&theToken);
			btnPropAccessorObj->GetData(&objectData); //, reqType);
			delete btnPropAccessorObj;
			break;
		default:
			ASSERTM( false, errAECantSupplyType, "You shouldn't be here" ) ;
		
	}
	
	result = AEPutParamDesc(reply, keyAEResult, &objectData);
	THROW_IF_ERROR( result);

	CATCH_ALL
		error = ErrorCode();
	ENDTRY
	
	return error;
}	// HandleGetData()

//------------------------------------------------------------------------------
// GetPropertyFromNULL
//------------------------------------------------------------------------------

static XMPError GetPropertyFromNULL(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(desiredClass);
XMPUnused(container);
XMPUnused(containerClass);

	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();
	DescType				propID;
	ScriptButtonPartPropAccessor*	accessorObj;
	XMPError	error = noErr;

	TRY

	if (form != formPropertyID)
		THROW(errAEWrongDataType);

	switch( propID = **(DescType**)(selectionData->dataHandle) )
	{
		case pBounds:
		case pScript:
		case pName:
			resolver->CreateToken(value, cProperty, thisPart);
			accessorObj = new ScriptButtonPartPropAccessor(propID, (CScriptButtonPart*)thisPart);
			ReallocAndStuff2nd4Bytes(value, (XMPULong)accessorObj);
			break;
		default:
			THROW(errAECantSupplyType);
			break;
	}
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetPropertyFromNULL()


//==============================================================================
// class ScriptButtonPartPropAccessor
//==============================================================================

//------------------------------------------------------------------------------
// ScriptButtonPartPropAccessor::ScriptButtonPartPropAccessor
//------------------------------------------------------------------------------

ScriptButtonPartPropAccessor::ScriptButtonPartPropAccessor(DescType property, CScriptButtonPart* part)
{
	fProperty = property;
	fPart = part;
} 

//------------------------------------------------------------------------------
// ScriptButtonPartPropAccessor::DescListToRect
//------------------------------------------------------------------------------

OSErr ScriptButtonPartPropAccessor::DescListToRect(AEDesc* objectHandle, XMPRect* controlRect)
{
	OSErr 		theErr = noErr;
	DescType	typeCode;
	AEKeyword  	theAEKeyword;
	long 		actualSize, index, theCount;
	short 		theValue;
	
	theErr = AECountItems((AEDescList*)objectHandle, &theCount);
	if(theErr) return theErr;
	if(theCount != 4) return errAECoercionFail;
	for(index = 1; index <= theCount; index++) {
		theErr = AEGetNthPtr(objectHandle, index, typeShortInteger, &theAEKeyword, 
								&typeCode, &theValue, sizeof(theValue), &actualSize);
		if(index == 1) {
			controlRect->top = theValue;
		} else	if(index == 2) {
			controlRect->left = theValue;
		} else if(index == 3) {
			controlRect->bottom = theValue;
		} else if(index == 4) {
			controlRect->right = theValue;
		}
	}
	return theErr;
}

//------------------------------------------------------------------------------
// ScriptButtonPartPropAccessor::SetData
//------------------------------------------------------------------------------

void ScriptButtonPartPropAccessor::SetData(AEDesc* objectData)
{
	switch(fProperty)
	{ 
		case pScript:
		{ 
			OSAID 				tempID  = 0;
			ComponentInstance	scriptingComponent = fPart->GetComponent();

			THROW_IF_ERROR( OSACoerceFromDesc(scriptingComponent, objectData,  0,  &tempID) );
			if((fPart->GetScriptID()) != kXMPNULL) 
			{
				OSADispose(scriptingComponent,(OSAID)fPart->GetScriptID());
			}
			fPart->SetScriptID(tempID);
			break ;
		}
		case pBounds: 
		{

			XMPRect controlRect;

			THROW_IF_ERROR( DescListToRect(objectData, &controlRect) );
			XMPPoint	controlPoint(controlRect.right-controlRect.left, controlRect.bottom-controlRect.top);
			fPart->SetControlSize(controlPoint);
			fPart->AdjustButton();
			break ;
		}
		
		case pName: 
		{
			Str255 name;	//name;
			long	hdlSize;
			hdlSize = GetHandleSize((Handle) objectData->dataHandle);
			BlockMove(*objectData->dataHandle, &name[1], hdlSize);
			name[0] = (unsigned char)hdlSize;
			fPart->SetButtonName((char *)name);
			break ;
		}
		// read-only cases go here:
		// case readOnly:
		//THROW_IF_ERROR(errAENotModifiable);
			
		default:
			THROW(errAECantSupplyType);
			break;
	}
}

//------------------------------------------------------------------------------
// ScriptButtonPartPropAccessor::GetData
//------------------------------------------------------------------------------

void ScriptButtonPartPropAccessor::GetData(AEDesc* objectData)
{
	switch(fProperty)
	{
		case pName:
		{
			Str255	bName;

			BlockMove(fPart->GetButtonName(), bName, 255);
			THROW_IF_ERROR( AECreateDesc(typeChar, (Ptr)&bName[1], (Size)bName[0], objectData)) ;
			break ;
		}
		case pScript:
			OSAID 				theOSAID = fPart->GetScriptID() ;
			ComponentInstance	scriptingComponent = fPart->GetComponent();
			
			if( theOSAID != kXMPNULL) 
			{
	
				THROW_IF_ERROR( OSACoerceToDesc(scriptingComponent, theOSAID,
								typeScript,
								0,
								objectData) );
			}
			else
			{
				(*objectData).descriptorType = typeNull;
				(*objectData).dataHandle = kXMPNULL;
			}
			break ;
			
		case pBounds: 
		{
			OSErr theErr;
			
			ControlHandle theCtlHdl = fPart->GetControlHandle();
			
			THROW_IF_ERROR( AECreateList(kXMPNULL, 0, false, objectData) );
			theErr = AEPutPtr(objectData, 0, typeShortInteger, &((*theCtlHdl)->contrlRect.top), sizeof(long));
			theErr = AEPutPtr(objectData, 0, typeShortInteger, &((*theCtlHdl)->contrlRect.left), sizeof(long));
			theErr = AEPutPtr(objectData, 0, typeShortInteger, &((*theCtlHdl)->contrlRect.bottom), sizeof(long));
			theErr = AEPutPtr(objectData, 0, typeShortInteger, &((*theCtlHdl)->contrlRect.right), sizeof(long));
					
			break ;
		}	
			
		//case pLocation: 
		//	break ;
			
		default:
			THROW(errAECantSupplyType);
			break;
	}
}
#undef XMPOJBECT_ACCESSOR_PARAMS


