//	File:		ClockFac.cp
//
//	Contains:	implementation of class CClockFacet
//
//	Written by:	Lonnie Millett
//
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.

#ifndef _CLOCKFAC_
#include "ClockFac.h"
#endif

#ifndef _CLOCKFRA_
#include "ClockFra.h"
#endif

#ifndef _CLOCKPAR_
#include "ClockPar.h"
#endif

// ----- FrameWork Includes -----

#ifndef _FWUTIL_
#include <FWUtil.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

// ----- Macintosh Includes -----

#ifndef __STDTYPES__
#include <StdTypes.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#pragma segment cclockfacet

//----------------------------------------------------------------------------------------
// The new math libraries for the universal headers have bugs so we're just including
// definitions from the old headers and linking with the old library till the numerics
// guys get the bugs worked out.
//
extern "C" {
	extended rint(extended x);
	extended sin(extended x);
	extended cos(extended x);
	extended pi();
}
//
//#ifndef __FP__
//#include <Fp.h>
//#endif
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// CClockFacet::CClockFacet
//----------------------------------------------------------------------------------------
CClockFacet::CClockFacet()
{
}

//----------------------------------------------------------------------------------------
// CClockFacet::~CClockFrame
//----------------------------------------------------------------------------------------
CClockFacet::~CClockFacet()
{
}

//----------------------------------------------------------------------------------------
// CClockFacet::InitClockFacet
//----------------------------------------------------------------------------------------
void CClockFacet::InitClockFacet(XMPFacet* xmpFacet, CClockFrame* clockFrame)
{
	this->InitFacet(xmpFacet);
	fClockFrame = clockFrame;
}

//----------------------------------------------------------------------------------------
// CClockFacet::Draw
//----------------------------------------------------------------------------------------
void CClockFacet::Draw(XMPShape* invalidShape)
{
	XMPUnused(invalidShape);

	XMPULong tickCount = fClockFrame->GetClockPart()->GetLastTime();
	
	if (fClockFrame->GetClockPart()->GetClockType() == kAnalogClock)
	{
		this->DrawClockFace();
		this->DrawClockHands(tickCount);
	}
	else
		DrawDigitalClock(tickCount);
}

//----------------------------------------------------------------------------------------
// CClockFacet::DrawClockFace
//----------------------------------------------------------------------------------------
void CClockFacet::DrawClockFace()
{
	Point currentPenSize = XMPASLMQDGlobals.thePort->pnSize;
	TextStyle currentTextStyle;
	Point centerPoint;
	Point newPoint;
	Point tickPoint;
	XMPSShort radius;
	XMPSShort angle = 0;
	XMPSShort fiveMinute = 0;
	extended radians;
	
	::GetPortTextStyle(&currentTextStyle);
	
	XMPRgnHandle shapeRegion = fClockFrame->GetXMPFrame()->GetFrameShape()->GetQDRegion();
	Rect clockRect = (*shapeRegion)->rgnBBox;
	
	centerPoint.h = (clockRect.right - clockRect.left) / 2;
	centerPoint.v = (clockRect.bottom - clockRect.top) / 2;
	radius = centerPoint.h;

	::EraseOval(&clockRect);

	::PenSize(2,2);
	::FrameOval(&clockRect);
	::PenSize(1,1);
	
	::TextFont(helvetica);
	::TextFace(italic);
	::TextSize(radius / 10);
	
	Str255 clockFaceString;
	FW_CAcquireASLMResourceAccess aq;
	::GetIndString(clockFaceString, kClockPartStrings, kClockFaceString);
	
	::MoveTo(((clockRect.right - clockRect.left) - ::StringWidth(clockFaceString)) / 2, (clockRect.bottom - clockRect.top) / 4);
	::DrawString(clockFaceString);

	// Use the symmetry of the clock face to speed up the calculations for drawing the face
	// We only need to calculate points for 45 degrees of the circle.  The remaining points can
	// be inferred from these points
	while (angle < 45)
	{
		// convert angle to radians
		radians = (90 - angle) / (360 / (2 * pi()));
		newPoint.h = (short)::rint((radius * cos(radians)) + centerPoint.h);
		newPoint.v = (short)::rint((radius * sin(radians)) + centerPoint.v);
		
		if (fiveMinute == 0)
		{
			short hourTick = radius - (radius / 10);
			tickPoint.h = (short)::rint((hourTick * cos(radians)) + centerPoint.h);
			tickPoint.v = (short)::rint((hourTick * sin(radians)) + centerPoint.v);
			fiveMinute = 4;
		}
		else
		{
			short minuteTick = radius - (radius / 20);
			tickPoint.h = (short)::rint((minuteTick * cos(radians)) + centerPoint.h);
			tickPoint.v = (short)::rint((minuteTick * sin(radians)) + centerPoint.v);
			fiveMinute--;
		}

		::MoveTo(newPoint.h - 1, newPoint.v - 1);
		::LineTo(tickPoint.h - 1, tickPoint.v - 1);
		
		::MoveTo(clockRect.right - newPoint.h, newPoint.v - 1);
		::LineTo(clockRect.right - tickPoint.h, tickPoint.v - 1);
		
		::MoveTo(newPoint.h - 1, clockRect.bottom - newPoint.v);
		::LineTo(tickPoint.h - 1, clockRect.bottom - tickPoint.v);

		::MoveTo(clockRect.right - newPoint.h, clockRect.bottom - newPoint.v);
		::LineTo(clockRect.right - tickPoint.h, clockRect.bottom - tickPoint.v);

		::MoveTo(newPoint.v - 1, newPoint.h - 1);
		::LineTo(tickPoint.v - 1, tickPoint.h - 1);
		
		::MoveTo(clockRect.bottom - newPoint.v, newPoint.h - 1);
		::LineTo(clockRect.bottom - tickPoint.v, tickPoint.h - 1);
		
		::MoveTo(newPoint.v - 1, clockRect.right - newPoint.h);
		::LineTo(tickPoint.v - 1, clockRect.right - tickPoint.h);

		::MoveTo(clockRect.bottom - newPoint.v, clockRect.right - newPoint.h);
		::LineTo(clockRect.bottom - tickPoint.v, clockRect.right - tickPoint.h);

		angle += 6;
	}
	
	::PenSize(currentPenSize.h, currentPenSize.v);
	::SetPortTextStyle(&currentTextStyle);
}

//----------------------------------------------------------------------------------------
// CClockFacet::DrawClockHands
//----------------------------------------------------------------------------------------
void CClockFacet::DrawClockHands(XMPULong tickCount)
{
	XMPSShort currentPenMode = XMPASLMQDGlobals.thePort->pnMode;
	Point centerPoint;
	DateTimeRec time;
	
	XMPRgnHandle shapeRegion = fClockFrame->GetXMPFrame()->GetFrameShape()->GetQDRegion();
	Rect clockRect = (*shapeRegion)->rgnBBox;
	
	centerPoint.h = (clockRect.right - clockRect.left) / 2;
	centerPoint.v = (clockRect.bottom - clockRect.top) / 2;
	
	::Secs2Date(tickCount, &time);
	
	PenMode(patXor);
	this->DrawSecondHand(&clockRect, &centerPoint, centerPoint.h, time.second);
	this->DrawMinuteHand(&clockRect, &centerPoint, centerPoint.h, time.minute);

	if (time.hour > 11)
	this->DrawHourHand(&clockRect, &centerPoint, centerPoint.h, time.hour - 12, time.minute);
	else
	this->DrawHourHand(&clockRect, &centerPoint, centerPoint.h, time.hour, time.minute);

	::PenMode(currentPenMode);
}

//----------------------------------------------------------------------------------------
// CClockFacet::UpdateClockHands
//----------------------------------------------------------------------------------------
void CClockFacet::UpdateClockHands(XMPULong tickCount)
{
	XMPSShort currentPenMode = XMPASLMQDGlobals.thePort->pnMode;
	Point centerPoint;
	DateTimeRec time;
	DateTimeRec lastTime;
	
	CDrawInitiator di(this);

 	::GetDateTime(&tickCount);
	::Secs2Date(tickCount, &time);
	::Secs2Date(fClockFrame->GetClockPart()->GetLastTime(), &lastTime);
	
	XMPRgnHandle shapeRegion = fClockFrame->GetXMPFrame()->GetFrameShape()->GetQDRegion();
	Rect clockRect = (*shapeRegion)->rgnBBox;
	
	centerPoint.h = (clockRect.right - clockRect.left) / 2;
	centerPoint.v = (clockRect.bottom - clockRect.top) / 2;
	
	::PenMode(patXor);
	
	// Since we're called every second go ahead and update the second hand
	this->DrawSecondHand(&clockRect, &centerPoint, centerPoint.h, lastTime.second);
	this->DrawSecondHand(&clockRect, &centerPoint, centerPoint.h, time.second);

	if (lastTime.minute != time.minute)
	{
		this->DrawMinuteHand(&clockRect, &centerPoint, centerPoint.h, lastTime.minute);
		this->DrawMinuteHand(&clockRect, &centerPoint, centerPoint.h, time.minute);
		
		if (lastTime.hour > 11)
			this->DrawHourHand(&clockRect, &centerPoint, centerPoint.h, lastTime.hour - 12, lastTime.minute);
		else
			this->DrawHourHand(&clockRect, &centerPoint, centerPoint.h, lastTime.hour, lastTime.minute);

		if (time.hour > 11)
			this->DrawHourHand(&clockRect, &centerPoint, centerPoint.h, time.hour - 12, time.minute);
		else
			this->DrawHourHand(&clockRect, &centerPoint, centerPoint.h, time.hour, time.minute);
	}
	
	::PenMode(currentPenMode);
}

//----------------------------------------------------------------------------------------
// CClockFacet::DrawHourHand
//----------------------------------------------------------------------------------------
void CClockFacet::DrawHourHand(Rect* clockRect, Point* centerPoint, XMPSShort radius, XMPSShort hour, XMPSShort minute)
{
	Point currentPenSize = XMPASLMQDGlobals.thePort->pnSize;
	Point newPoint;
	extended radians;
	XMPSShort hourRadius = radius / 2;		// Make the hour hand short and stubby
	
	// Divide the clock face into degrees.  Hour hand falls on every five minutes or every 30 degrees
	radians = ((hour * 30) + (minute / 2)) / (360 / (2 * pi()));
	newPoint.h = (short)::rint((hourRadius * ::sin(radians)) + centerPoint->h);
	newPoint.v = clockRect->bottom - ((short)::rint((hourRadius * ::cos(radians)) + centerPoint->v));

	::PenSize(3,3);
	::MoveTo(centerPoint->h, centerPoint->v);
	::LineTo(newPoint.h - 1, newPoint.v);
	::PenSize(currentPenSize.h, currentPenSize.v);
}

//----------------------------------------------------------------------------------------
// CClockFacet::DrawMinuteHand
//----------------------------------------------------------------------------------------
void CClockFacet::DrawMinuteHand(Rect* clockRect, Point* centerPoint, XMPSShort radius, XMPSShort minute)
{
	Point currentPenSize = XMPASLMQDGlobals.thePort->pnSize;
	Point newPoint;
	extended radians;
	XMPSShort minuteRadius = radius - (radius / 20);  // Make the minute hand longer than the hour hand

	// Divide the clock face into degrees.  (360 degrees divided by 60 minutes is 6 degrees per minute)
	radians = (minute * 6) / (360 / (2 * pi()));
	newPoint.h = (short)::rint((minuteRadius * ::sin(radians)) + centerPoint->h);
	newPoint.v = clockRect->bottom - ((short)::rint((minuteRadius * ::cos(radians)) + centerPoint->v));

	::PenSize(2,2);
	::MoveTo(centerPoint->h, centerPoint->v);
	::LineTo(newPoint.h - 1, newPoint.v);
	::PenSize(currentPenSize.h, currentPenSize.v);
}

//----------------------------------------------------------------------------------------
// CClockFacet::DrawSecondHand
//----------------------------------------------------------------------------------------
void CClockFacet::DrawSecondHand(Rect* clockRect, Point* centerPoint, XMPSShort radius, XMPSShort second)
{
	Point currentPenSize = XMPASLMQDGlobals.thePort->pnSize;
	Point newPoint;
	extended radians;
	XMPSShort secondRadius = radius - (radius / 40);	// A nice sweeping second hand

	// Divide the clock face into degrees.  (360 degrees divided by 60 seconds is 6 degrees per second)
	radians = (second * 6) / (360 / (2 * pi()));
	newPoint.h = (short)::rint((secondRadius * ::sin(radians)) + centerPoint->h);
	newPoint.v = clockRect->bottom - ((short)::rint((secondRadius * ::cos(radians)) + centerPoint->v));

	::PenSize(1,1);
	::MoveTo(centerPoint->h, centerPoint->v);
	::LineTo(newPoint.h - 1, newPoint.v);
	::PenSize(currentPenSize.h, currentPenSize.v);
}

//----------------------------------------------------------------------------------------
// CClockFacet::DrawDigitalClock
//----------------------------------------------------------------------------------------
void CClockFacet::DrawDigitalClock(XMPULong tickCount)
{
	Str255 time;
	TextStyle currentTextStyle;
	::GetPortTextStyle(&currentTextStyle);

	CDrawInitiator di(this);

	::TextFont(GetSysFont());
	::TextFace(normal);
	::TextSize(GetDefFontSize());
	
	FontInfo clockInfo;
	::GetFontInfo(&clockInfo);
	
	::IUTimeString(tickCount, kXMPTrue, time);
	
	XMPRgnHandle shapeRegion = fClockFrame->GetXMPFrame()->GetFrameShape()->GetQDRegion();
	Rect clockRect = (*shapeRegion)->rgnBBox;

	::EraseRect(&clockRect);
	::FrameRect(&clockRect);
	::MoveTo(5, clockInfo.ascent + 5);
	::DrawString(time);
	
	::SetPortTextStyle(&currentTextStyle);
}

