//	File:		ClockFra.cp
//
//	Contains:	implementation of class CClockFrame
//
//	Written by:	Lonnie Millett
//
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.

#ifndef _CLOCKFRA_
#include "ClockFra.h"
#endif

#ifndef _CLOCKFAC_
#include "ClockFac.h"
#endif

#ifndef _CLOCKPAR_
#include "ClockPar.h"
#endif

// ----- FrameWork Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _SESSION_
#include <Session.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

// ----- Macintosh Includes -----

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#pragma segment cclockframe

//==============================================================================
// Constants
//==============================================================================

const XMPSShort kClockMinimumSize = 120;

//----------------------------------------------------------------------------------------
// CClockFrame::CClockFrame
//----------------------------------------------------------------------------------------
CClockFrame::CClockFrame()
					  : CFrame()
{
	fClockPart = kXMPNULL;
}

//----------------------------------------------------------------------------------------
// CClockFrame::~CClockFrame
//----------------------------------------------------------------------------------------
CClockFrame::~CClockFrame()
{
}

//----------------------------------------------------------------------------------------
// CClockFrame::InitClockFrame
//----------------------------------------------------------------------------------------
void CClockFrame::InitClockFrame(XMPFrame* xmpFrame, CClockPart* clockPart)
{
	this->IFrame(xmpFrame, clockPart);
	
	// ----- By default we put all of them -----
	AddToFocusSet(clockPart->GetMenuFocusToken());
	AddToFocusSet(clockPart->GetSelectionFocusToken());

	fClockPart = clockPart;
	
	// Calculate a default rectangle size for a digital clock
	TextStyle currentTextStyle;
	GetPortTextStyle(&currentTextStyle);
	
	TextFont(GetSysFont());
	TextFace(normal);
	TextSize(GetDefFontSize());
	
	FontInfo clockInfo;
	
	GetFontInfo(&clockInfo);
	
	Str255 digitalWidth;
	GetIndString(digitalWidth, kClockPartStrings, kClockDigitalWidthString);
	SetRect(&fDigitalClockRect, 0, 0, (StringWidth(digitalWidth)+10), ((clockInfo.ascent+clockInfo.descent)+10));

	SetPortTextStyle(&currentTextStyle);
}

//----------------------------------------------------------------------------------------
// CClockFrame::GetNewFrameShape
//----------------------------------------------------------------------------------------
XMPShape* CClockFrame::GetNewFrameShape(CFacet* facet)
{
	CDrawInitiator di(facet);

	XMPRgnHandle frameRegion = NewRgn();
	Rect frameRect;

	if (fClockPart->GetClockType() == kAnalogClock)
	{
		XMPRgnHandle shapeRegion = this->GetXMPFrame()->GetFrameShape()->GetQDRegion();
		XMPSShort size = (*shapeRegion)->rgnBBox.bottom - (*shapeRegion)->rgnBBox.top;
		if (size > (*shapeRegion)->rgnBBox.right - (*shapeRegion)->rgnBBox.left)
			size = (*shapeRegion)->rgnBBox.right - (*shapeRegion)->rgnBBox.left;
		
		if (size < kClockMinimumSize)
			size = kClockMinimumSize;
		
		SetRect(&frameRect, 0, 0, size, size);
		OpenRgn();
		FrameOval(&frameRect);
		CloseRgn(frameRegion);
	}
	else
		RectRgn(frameRegion, &fDigitalClockRect);

	XMPShape* newFrameShape = new XMPShape;
	newFrameShape->SetQDRegion(frameRegion);
	
	return newFrameShape;
}
	
//----------------------------------------------------------------------------------------
// CClockFrame::NewFacet
//----------------------------------------------------------------------------------------
CFacet* CClockFrame::NewFacet(XMPFacet* xmpFacet)
{
	CClockFacet* facet = new CClockFacet;
	facet->InitClockFacet(xmpFacet, this);
	return facet;
}

//----------------------------------------------------------------------------------------
// CClockFrame::FacetAdded
//----------------------------------------------------------------------------------------
void CClockFrame::FacetAdded(CFacet* facet)
{
	CFrame::FacetAdded(facet);
	
	this->GetXMPFrame()->RequestFrameShape(this->GetNewFrameShape(facet));
	this->UpdateUsedAndActiveShapes();
}

//----------------------------------------------------------------------------------------
// CClockFrame::FacetRemoved
//----------------------------------------------------------------------------------------
void CClockFrame::FacetRemoved(CFacet* facet)
{
	CFrame::FacetRemoved(facet);

	this->GetXMPFrame()->RequestFrameShape(this->GetNewFrameShape(facet));
	this->UpdateUsedAndActiveShapes();
}

//----------------------------------------------------------------------------------------
// CClockFrame::FrameShapeChanged
//----------------------------------------------------------------------------------------
void CClockFrame::FrameShapeChanged()
{
	this->GetXMPFrame()->RequestFrameShape(this->GetNewFrameShape(this->GetActiveFacet()));
	this->UpdateUsedAndActiveShapes();

	XMPShape oldShape;
	this->GetFrameShape(&oldShape);
	this->Invalidate(&oldShape);
	this->GetXMPFrame()->InvalidateActiveBorder();
}

//----------------------------------------------------------------------------------------
// CClockFrame::UpdateClock
//----------------------------------------------------------------------------------------
void CClockFrame::UpdateClock(XMPULong tickCount)
{
	CFrameFacetIterator facets(this);
	for (CClockFacet* clockFacet = (CClockFacet*)facets.First(); facets.IsNotComplete(); clockFacet = (CClockFacet*)facets.Next())
	{
		if (fClockPart->GetClockType() == kAnalogClock)
 			clockFacet->UpdateClockHands(tickCount);
		else
			clockFacet->DrawDigitalClock(tickCount);
		clockFacet->Validate(this->GetXMPFrame()->GetUsedShape());
	}
}

