/*
	File:		ClockPar.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 4/19/94	DS		Remove Add Contents property fix
		 <3>	 4/10/94	DS		Set the content property in during part
									initialization
		 <2>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h

	To Do:
*/

//	File:		ClockPar.cp
//
//	Contains:	implementation of class CClockPart
//
//	Written by:	Lonnie Millett
//
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.

#ifndef _CLOCKPAR_
#include "ClockPar.h"
#endif

#ifndef _CLOCKFRA_
#include "ClockFra.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _FOCI_
#include <Foci.h>
#endif

#ifndef _FOCUSSET_
#include <FocusSet.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

// ----- Macintosh Includes -----

#ifndef __STDTYPES__
#include <StdTypes.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SOUND__
#include <Sound.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

//==============================================================================
// Constants
//==============================================================================

const XMPSShort kClockTypeItemID = 1;

const XMPSShort kClockSoundsNoneItemID = 1;
const XMPSShort kClockSoundsTickItemID = 2;
const XMPSShort kClockSoundsChimeItemID = 3;
const XMPSShort kClockSoundsBothItemID = 4;

const XMPSShort kClockSoundsNone = 1;
const XMPSShort kClockSoundsTick = 2;
const XMPSShort kClockSoundsChime = 3;
const XMPSShort kClockSoundsBoth = 4;

const XMPFocusType kClockAudioFocus = "Audio";

const XMPSShort kClockIdleFreq = 15;

#pragma segment cclockpart

//----------------------------------------------------------------------------------------
// CClockPart::CClockPart
//----------------------------------------------------------------------------------------
CClockPart::CClockPart()
{
	fLastTime = 0;
	fClockType = kAnalogClock;
	fClockSounds = kClockSoundsChime;
	fAudioFocusToken = kXMPNullFocus;
	fAudioFocusSet = kXMPNULL;
	fAudioFocusFrame = kXMPNULL;
}

//----------------------------------------------------------------------------------------
// CClockPart::~CClockPart
//----------------------------------------------------------------------------------------
CClockPart::~CClockPart()
{
	delete fAudioFocusSet;
	fAudioFocusSet = kXMPNULL;
}

//----------------------------------------------------------------------------------------
// CClockPart::Release
//----------------------------------------------------------------------------------------
void CClockPart::Release()
{
	CPart::Release();
	
	if (this->GetRefCount() == 1)
	{
		this->GetSession()->GetDispatcher()->UnregisterIdle(this, kXMPNULL);
	}
}

//----------------------------------------------------------------------------------------
// CClockPart::Initialize
//----------------------------------------------------------------------------------------
void CClockPart::Initialize()
{
	CPart::Initialize();
	
	XMPSession* session = this->GetSession();
	
	// Register a special focus for audio so the arbitrator can arbitrate the audio focus
	fAudioFocusToken = session->Tokenize(kClockAudioFocus);
	
	if (!session->GetArbitrator()->IsFocusRegistered(fAudioFocusToken))
		session->GetArbitrator()->RegisterFocus(fAudioFocusToken,kXMPNULL);

	FW_CAcquireASLMResourceAccess aq;

	XMPMenuBar *menuBar = this->GetMenuBar();
	menuBar->AddMenuLast(kClockTypeMenu, GetMenu(kClockTypeMenu), this);
	menuBar->AddMenuLast(kClockSoundsMenu, GetMenu(kClockSoundsMenu), this);

	menuBar->RegisterCommand(cClockType, kClockTypeMenu, kClockTypeItemID);
	menuBar->RegisterCommand(cClockSoundsNone, kClockSoundsMenu, kClockSoundsNoneItemID);
	menuBar->RegisterCommand(cClockSoundsTick, kClockSoundsMenu, kClockSoundsTickItemID);
	menuBar->RegisterCommand(cClockSoundsChime, kClockSoundsMenu, kClockSoundsChimeItemID);
	menuBar->RegisterCommand(cClockSoundsBoth, kClockSoundsMenu, kClockSoundsBothItemID);
	menuBar->CheckCommand(cClockSoundsChime, kXMPTrue);
	
	// Create a new focus set which just contains the speaker.  This focus will be
	// aquired before playing the clock sounds
	fAudioFocusSet = new XMPFocusSet;
	fAudioFocusSet->InitFocusSet();
	fAudioFocusSet->Add(this->GetAudioFocusToken());

	XMPULong time;
	GetDateTime(&time);
	this->SetLastTime(time);

	session->GetDispatcher()->RegisterIdle(this, kXMPNULL, kClockIdleFreq);
}

//------------------------------------------------------------------------------
// CClockPart::InternalizeContent
//------------------------------------------------------------------------------
void CClockPart::InternalizeContent(XMPStorageUnit* storageUnit)
{
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);
	
	storageUnit->GetValue(sizeof(fClockType), (XMPValue)&fClockType);	
	storageUnit->GetValue(sizeof(fClockSounds), (XMPValue)&fClockSounds);
}

//----------------------------------------------------------------------------------------
// CClockPart::ExternalizeContent
//----------------------------------------------------------------------------------------
void CClockPart::ExternalizeContent(XMPStorageUnit* storageUnit)
{
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);

	storageUnit->SetValue(sizeof(fClockType), (XMPValue)&fClockType);
	storageUnit->SetValue(sizeof(fClockSounds), (XMPValue)&fClockSounds);
}

//----------------------------------------------------------------------------------------
// CClockPart::GetContentPropertyValueType
//----------------------------------------------------------------------------------------
XMPValueType CClockPart::GetContentPropertyValueType() const
{
	return kSampleClockKind;
}

//----------------------------------------------------------------------------------------
// CClockPart::NewFrame
//----------------------------------------------------------------------------------------
CFrame* CClockPart::NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation)
{
	XMPUnused(presentation);
	
	CClockFrame *frame = new CClockFrame();
	frame->InitClockFrame(xmpFrame, this);
	return frame;
}

//----------------------------------------------------------------------------------------
// CClockPart::DoIdle
//----------------------------------------------------------------------------------------
XMPBoolean CClockPart::DoIdle()
{
	XMPULong tickCount;
	DateTimeRec time;
	DateTimeRec lastTime;
	
 	::GetDateTime(&tickCount);
	::Secs2Date(tickCount, &time);
	::Secs2Date(this->GetLastTime(), &lastTime);
	
 	if (lastTime.second != time.second)
	{
		XMPSShort clockSound = this->GetClockSounds();
		if (clockSound != kClockSoundsNone)
		{
			// Ask for the audio focus so we can play sound and keep it for as long as we can
			// It would be nice if focus abitration could be at the part level just like idling
			// but for now it belongs to the frame
			XMPBoolean hasSoundFocus = kXMPFalse;
			XMPArbitrator* arbitrator = this->GetSession()->GetArbitrator();
			if (arbitrator->GetFocusOwner(this->GetAudioFocusToken()) != fAudioFocusFrame)
				hasSoundFocus = arbitrator->RequestFocusSet(fAudioFocusSet, fAudioFocusFrame);
			else
				hasSoundFocus = kXMPTrue;
				
			if (hasSoundFocus)
			{
				if ((clockSound == kClockSoundsTick) || (clockSound == kClockSoundsBoth))
					this->PlayTickSound();
				if (((clockSound == kClockSoundsChime) || (clockSound == kClockSoundsBoth)) &&
						(time.minute == 0) && (time.second == 0))
					this->PlayChimeSound();
			}
		}
		
		// Update all the display frames of the same type
		OrderedCollectionIterator iter(this->GetDisplayFrameList());
		for (CClockFrame* clockFrame = (CClockFrame*)iter.First(); iter.IsNotComplete(); clockFrame = (CClockFrame*)iter.Next())
			clockFrame->UpdateClock(tickCount);

		this->SetLastTime(tickCount);
	}
	return kXMPTrue;
}

//------------------------------------------------------------------------------
// CClockPart::DoMenuEvent
//------------------------------------------------------------------------------
XMPBoolean CClockPart::DoMenuEvent(XMPMenuBar* menuBar, XMPCommandID commandID)
{
	XMPBoolean menuHandled = kXMPFalse;
	
	switch (commandID)
	{
		case cClockType:
				{
					if (this->GetClockType() == kAnalogClock)
						this->SetClockType(kDigitalClock);
					else
						this->SetClockType(kAnalogClock);
						
					CPartDisplayFrameIterator iter(this, this->GetDefaultPresentation());
					for (CFrame* frame = iter.First(); iter.IsNotComplete(); frame = iter.Next())
						this->FrameShapeChanged(frame->GetXMPFrame());
				}
					
				menuHandled = kXMPTrue;
				break;
				
		case cClockSoundsNone:
				this->SetClockSounds(kClockSoundsNone);
				menuHandled = kXMPTrue;
				break;

		case cClockSoundsTick:
				this->SetClockSounds(kClockSoundsTick);
				menuHandled = kXMPTrue;
				break;

		case cClockSoundsChime:
				this->SetClockSounds(kClockSoundsChime);
				menuHandled = kXMPTrue;
				break;

		case cClockSoundsBoth:
				this->SetClockSounds(kClockSoundsBoth);
				menuHandled = kXMPTrue;
				break;

		default:
			menuHandled = CPart::DoMenuEvent(menuBar, commandID);
	}			
	
	return menuHandled;
}

//----------------------------------------------------------------------------------------
// CClockPart::DoAdjustMenus
//----------------------------------------------------------------------------------------
void CClockPart::DoAdjustMenus(XMPMenuBar* menuBar)
{
	Str255 menuString;
	
	FW_CAcquireASLMResourceAccess aq;

	if (this->GetClockType() == kAnalogClock)
		GetIndString(menuString, kClockPartStrings, kClockDigitalMenuString);
	else
		GetIndString(menuString, kClockPartStrings, kClockAnalogMenuString);
	menuBar->SetItemString(cClockType, menuString);
	
	// Uncheck them all then check the correct one
	menuBar->CheckCommand(cClockSoundsNone, kXMPFalse);
	menuBar->CheckCommand(cClockSoundsTick, kXMPFalse);
	menuBar->CheckCommand(cClockSoundsChime, kXMPFalse);
	menuBar->CheckCommand(cClockSoundsBoth, kXMPFalse);
	
	switch(this->GetClockSounds())
	{
		case kClockSoundsNone:
			menuBar->CheckCommand(cClockSoundsNone, kXMPTrue);
			break;
		case kClockSoundsTick:
			menuBar->CheckCommand(cClockSoundsTick, kXMPTrue);
			break;
		case kClockSoundsChime:
			menuBar->CheckCommand(cClockSoundsChime, kXMPTrue);
			break;
		case kClockSoundsBoth:
			menuBar->CheckCommand(cClockSoundsBoth, kXMPTrue);
			break;
	}
}


//----------------------------------------------------------------------------------------
// CClockPart::AddDisplayFrame
//----------------------------------------------------------------------------------------
void CClockPart::AddDisplayFrame(XMPFrame* xmpFrame)
{
	CPart::AddDisplayFrame(xmpFrame);
	
	if (!fAudioFocusFrame)
		fAudioFocusFrame = xmpFrame;
}

//----------------------------------------------------------------------------------------
// CClockPart::RemoveDisplayFrame
//----------------------------------------------------------------------------------------
void CClockPart::RemoveDisplayFrame(XMPFrame* xmpFrame)
{
	if (xmpFrame == fAudioFocusFrame)
	{
		XMPArbitrator* arbitrator = this->GetSession()->GetArbitrator();
		arbitrator->RelinquishFocusSet(fAudioFocusSet, fAudioFocusFrame);
		fAudioFocusFrame = kXMPNULL;
	}
	CPart::RemoveDisplayFrame(xmpFrame);
}

//----------------------------------------------------------------------------------------
// CClockPart::PlayTickSound
//----------------------------------------------------------------------------------------
void CClockPart::PlayTickSound()
{
	FW_CAcquireASLMResourceAccess aq;

	Handle tick = GetResource(soundListRsrc, kClockTick);
	if (tick != kXMPNULL)
		SndPlay(kXMPNULL, tick, kXMPTrue);
}

//----------------------------------------------------------------------------------------
// CClockPart::PlayChimeSound
//----------------------------------------------------------------------------------------
void CClockPart::PlayChimeSound()
{
	FW_CAcquireASLMResourceAccess aq;
	
	Handle chime = GetResource(soundListRsrc, kClockChime);
	if (chime != kXMPNULL)
		SndPlay(kXMPNULL, chime, kXMPTrue);
}

