/*
	File:		ClockPar.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<1+>	 3/17/94	DS		Add CDECL to Clock Part dtor

	To Do:
*/

#ifndef _CLOCKPAR_
#define _CLOCKPAR_

#ifndef _CCLOCKDEF_
#include "CClockDef.h"
#endif

#ifndef _FWPART_
#include <FWPart.h>
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class CClockPart;

//==============================================================================
// Classes used by this interface
//==============================================================================

class CPart;
class XMPStorageUnit;
class XMPSession;
class XMPFrame;
class XMPFocusSet;

//==============================================================================
// Constants
//==============================================================================

const XMPCommandID cClockType = 10000;
const XMPCommandID cClockSoundsNone = 10001;
const XMPCommandID cClockSoundsTick = 10002;
const XMPCommandID cClockSoundsChime = 10003;
const XMPCommandID cClockSoundsBoth = 10004;

const XMPSShort kAnalogClock = 1;
const XMPSShort kDigitalClock = 2;

//==============================================================================
// CClockPart
//==============================================================================

class CClockPart : public CPart
{

public:
	CClockPart();
	virtual ~ CDECL CClockPart();

	XMPMethod void Release();

	virtual void Initialize();
	
	virtual XMPValueType GetContentPropertyValueType() const; 
	
	XMPULong GetLastTime();
	void SetLastTime(XMPULong lastTime);
	
	XMPSShort GetClockType();
	void SetClockType(XMPSShort clockType);
	
	XMPSShort GetClockSounds();
	void SetClockSounds(XMPSShort clockSounds);

	XMPTypeToken GetAudioFocusToken();
	
	virtual CFrame* NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	virtual void AddDisplayFrame(XMPFrame* xmpFrame);
	virtual void RemoveDisplayFrame(XMPFrame* xmpFrame);
	
	virtual XMPBoolean DoIdle();
	virtual XMPBoolean DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	virtual void DoAdjustMenus(XMPMenuBar* menuBar);
	
	virtual void InternalizeContent(XMPStorageUnit* storageUnit);
	virtual void ExternalizeContent(XMPStorageUnit* storageUnit);
	
	void PlayTickSound();
	void PlayChimeSound();
	
private:
	XMPULong fLastTime;
	XMPSShort fClockType;
	XMPSShort fClockSounds;
	XMPTypeToken fAudioFocusToken;
	XMPFocusSet* fAudioFocusSet;
	XMPFrame* fAudioFocusFrame;
};

//----------------------------------------------------------------------------------------
// CClockPart::GetLastTime
//----------------------------------------------------------------------------------------
inline XMPULong CClockPart::GetLastTime()
{
	return fLastTime;
}

//----------------------------------------------------------------------------------------
// CClockPart::SetLastTime
//----------------------------------------------------------------------------------------
inline void CClockPart::SetLastTime(XMPULong lastTime)
{
	fLastTime = lastTime;
}

//----------------------------------------------------------------------------------------
// CClockPart::GetClockType
//----------------------------------------------------------------------------------------
inline XMPSShort CClockPart::GetClockType()
{
	return fClockType;
}

//----------------------------------------------------------------------------------------
// CClockPart::SetClockType
//----------------------------------------------------------------------------------------
inline void CClockPart::SetClockType(XMPSShort clockType)
{
	fClockType = clockType;
}

//----------------------------------------------------------------------------------------
// CClockPart::GetClockSounds
//----------------------------------------------------------------------------------------
inline XMPSShort CClockPart::GetClockSounds()
{
	return fClockSounds;
}

//----------------------------------------------------------------------------------------
// CClockPart::SetClockSounds
//----------------------------------------------------------------------------------------
inline void CClockPart::SetClockSounds(XMPSShort clockSounds)
{
	fClockSounds = clockSounds;
}

//----------------------------------------------------------------------------------------
// CClockPart::GetAudioFocusToken
//----------------------------------------------------------------------------------------
inline XMPTypeToken CClockPart::GetAudioFocusToken()
{
	return fAudioFocusToken;
}

#endif
