/*
	File:		DrawFacet.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 4/11/94	DS		Change call to ChangeExternalTransform to
									ChangeGeometry.
		 <3>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h
		 <2>	 3/10/94	DS		Derive DrawFacet from ViewFacet. Implement
									CreateViewHierarchy.

	To Do:
*/

/*
	File:		DrawFacet.cpp

	Contains:	implementation of class CDrawFacet

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _DRAWFACET_
#include "DrawFacet.h"
#endif

#ifndef _QDRAWPART_
#include "QDrawPart.h"
#endif

#ifndef _DRAWSELECTION_
#include "DrawSelection.h"
#endif

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _DRAWFRAME_
#include "DrawFrame.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

// ----- Macintosh Includes -----

#ifndef __EVENTS__
#include <Events.h>		// GetMouse, StillDown.
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>	// HiWord etc.
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>		// Abs
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>		// See MoveTransformBy
#endif

#pragma segment qdrawpart

//=========================================================================
// CDrawFacet
//=========================================================================

//------------------------------------------------------------------------------
// CDrawFacet::CDrawFacet
//------------------------------------------------------------------------------

CDrawFacet::CDrawFacet()
{
}

//------------------------------------------------------------------------------
// CDrawFacet::IDrawFacet
//------------------------------------------------------------------------------

void CDrawFacet::IDrawFacet(XMPFacet* xmpFacet, CDrawPart* drawPart)
{
	InitViewFacet(xmpFacet);
	fDrawPart = drawPart;
}

//------------------------------------------------------------------------------
// CDrawFacet::~CDrawFacet
//------------------------------------------------------------------------------

CDrawFacet::~CDrawFacet()
{
}

//----------------------------------------------------------------------------------------
// FW_CView::InitView
//----------------------------------------------------------------------------------------
FW_CView* CDrawFacet::CreateViewHierarchy()
{
	// You must implement this method
	
	return ((FW_CView*)NULL);
}



//------------------------------------------------------------------------------
// CDrawFacet::Draw
//------------------------------------------------------------------------------

void CDrawFacet::Draw(XMPShape* invalidShape)
{
	XMPShape* clipShape = kXMPNULL;
	RgnHandle svClip = kXMPNULL;
	XMPShape* shapeClipShape = kXMPNULL;
	
	// Call inherited draw first
	FW_CViewFacet::Draw(invalidShape);
	
	svClip = ::NewRgn();
	::GetClip(svClip);
	shapeClipShape = ::NewXMPShape();
	
	if (invalidShape)
		clipShape = ::NewXMPShape(invalidShape);
	else
		clipShape = ::NewXMPShape(this->GetClipShape());
	
	::EraseRgn(clipShape->GetQDRegion());

	::SetClip(clipShape->GetQDRegion());

	OrderedCollectionIterator ite(fDrawPart->GetShapeList());
	for (CBaseShape* shape = (CBaseShape*)ite.First(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Next())
	{
		shape->DrawShape(this, clipShape);
		::SetClip(clipShape->GetQDRegion());
	}

	// Removed for Doug Hill so that MI5 can do its pattern matching thing..
	// DrawGrid();
	//

	delete shapeClipShape;
	delete clipShape;

	// ----- Draw selections Handles -----
	if (GetFrame()->IsActive())
	{
		if (invalidShape)
			::SetClip(invalidShape->GetQDRegion());
		else
			::SetClip(this->GetClipShape()->GetQDRegion());
			
		((CDrawSelection*)fDrawPart->GetSelection())->DrawSelectionHandles(this, kXMPTrue);
	}

	::SetClip(svClip);
	::DisposeRgn(svClip);		
}

//------------------------------------------------------------------------------
// CDrawFacet::MouseEnterFrame
//------------------------------------------------------------------------------

void CDrawFacet::MouseEnter(const XMPPoint& /*where*/)
{
	if (fDrawPart->GetActiveFrame() != GetFrame() || fDrawPart->GetTool() == kSelectTool)
		::InitCursor();
	else
		::SetCursor(*::GetCursor(crossCursor));
}

//------------------------------------------------------------------------------
// CDrawFacet::DoMouseDownInEmbeddedFrame
//------------------------------------------------------------------------------

XMPBoolean CDrawFacet::DoMouseDownInEmbeddedFrame(XMPFacet* embeddedXMPFacet, 
												const XMPPoint& where, 
												XMPEventData event)
{
	CDrawProxyRun *drawProxyRun = (CDrawProxyRun*)fDrawPart->XMPFrametoProxyRun(embeddedXMPFacet->GetFrame());
	if (drawProxyRun->IsSelected())
	{
		if (!::WaitMouseMoved(event->where))
		{
			drawProxyRun->SetSelectState(kXMPFalse);
			event->what = mouseDown;
			fDrawPart->GetSession()->GetDispatcher()->Dispatch(event);
			if (GetFrame()->IsActive())
			{
				drawProxyRun->SetSelectState(kXMPTrue);
			}
			return kXMPTrue;
		}
	}
	
	return DoMouseDown(where, event);
}

//------------------------------------------------------------------------------
// CDrawFacet::DoMouseDownInEmbeddedFrameBorder
//------------------------------------------------------------------------------

XMPBoolean CDrawFacet::DoMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, 
														const XMPPoint& where, 
														XMPEventData event)
{
XMPUnused(embeddedXMPFacet);

	CDrawInitiator di(this);

	fDrawPart->SetTool(kSelectTool);
	
	CDrawProxyRun *drawProxyRun = (CDrawProxyRun*)fDrawPart->XMPFrametoProxyRun(embeddedXMPFacet->GetFrame());
	CProxyShape *theShape = drawProxyRun->GetShape();
	CDrawSelection* selection = (CDrawSelection*)fDrawPart->GetSelection();

	if (::TestShiftKey(event))
	{
		if (theShape->IsSelected())
		{
			selection->RemoveFromSelection(theShape, kXMPTrue);
			return kXMPTrue;
		}
		else
			selection->AddToSelection(theShape, kXMPTrue);
		
		// ----- Clear the shift key flag
		event->modifiers -= 0x0200;
	}
	else if (!theShape->IsSelected())
	{
		selection->CloseSelection();
		selection->AddToSelection(theShape, kXMPTrue);
	}

	// ----- if a shape is selected start dragging
	if (theShape->IsSelected())
	{
		Point anchorPoint = where.AsQDPoint();
		short whichHandle = theShape->WhichHandle(this, anchorPoint);
		selection->SetAnchorShape(theShape);
		if (whichHandle != 0)
			selection->Resize(whichHandle, this, anchorPoint, event);
		else 
			selection->Drag(this, event);
	}

	return kXMPTrue;	
//	return DoMouseDown(where, event);

}

//------------------------------------------------------------------------------
// CDrawFacet::DoMouseDown
//------------------------------------------------------------------------------

XMPBoolean CDrawFacet::DoMouseDown(const XMPPoint& where, XMPEventData event)
{
	CDrawInitiator di(this);

	Point anchorPoint = where.AsQDPoint();
	
	if (fDrawPart->GetTool() == kSelectTool)
	{
		CDrawSelection* selection = (CDrawSelection*)fDrawPart->GetSelection();

		// ----- Click in an handle
		short whichHandle;
		CBaseShape* shape = selection->WhichHandle(this, anchorPoint, whichHandle);
		if (shape != kXMPNULL)
		{
			selection->SetAnchorShape(shape);
			selection->Resize(whichHandle, this, anchorPoint, event);
			fDrawPart->Changed();
			return kXMPTrue;
		}
		
		// ----- if not in an handle, look for a shape
		shape = WhichShape(anchorPoint, event);
		
		// ----- 
		if (shape == kXMPNULL)
		{
			// ----- Draw a selection rectangle
			selection->SelectWithRectangle(this, anchorPoint, event);
		}
		else
		{
			if (::TestShiftKey(event))
			{
				if (shape->IsSelected())
					selection->RemoveFromSelection(shape, kXMPTrue);
				else
					selection->AddToSelection(shape, kXMPTrue);
			}
			else if (!shape->IsSelected())
			{
				selection->CloseSelection();
				selection->AddToSelection(shape, kXMPTrue);
			}			
			
			// ----- if a shape is selected start dragging
			if (shape->IsSelected())
			{
				selection->SetAnchorShape(shape);
				selection->Drag(this, event);
				fDrawPart->Changed();
			}
		}
			
		return kXMPTrue;
	}
	
	CBaseShape *theShape;
	
	switch (fDrawPart->GetTool()) {
		case kSelectTool:
			break;
		case kLine:
			theShape = new CLineShape();
			break;
		case kRectangle:
			theShape = new CRectShape();
			break;
		case kOval:
			theShape = new COvalShape();
			break;
		case kRoundRect:
			theShape = new CRoundRectShape();
			break;
		case kArc:
			theShape = new CRectShape();
			break;
	}
	
	Point penSize;
	penSize.h = fDrawPart->GetPenSize();
	penSize.v = penSize.h;
	
	CShapeStyle trackStyle(penSize, XMPASLMQDGlobals.black, kRGBBlack);
	
	if (theShape->Track(trackStyle, anchorPoint, event))
	{		
		fDrawPart->AddShape(theShape);
		fDrawPart->Changed();
		
		CShapeStyle shapeStyle(fDrawPart->GetFillFrame(), penSize, XMPASLMQDGlobals.black, XMPASLMQDGlobals.black, kRGBBlack, kRGBWhite);
		theShape->SetShapeStyle(shapeStyle);

		fDrawPart->ClipEmbeddedFrames(fDrawPart->GetMainPresentation());
		theShape->DrawShape(this, kXMPNULL);
		
		XMPShape *invalidShape = ::NewXMPShape();
		theShape->GetUpdateShape(invalidShape);
		this->InvalidateAllFacets(invalidShape, kXMPTrue);
		delete invalidShape;
	}
	else
	{
		delete theShape;
	}
	
	return kXMPTrue;
}

//------------------------------------------------------------------------------
// CDrawFacet::WhichShape
//------------------------------------------------------------------------------

CBaseShape* CDrawFacet::WhichShape(Point mouse, XMPEventData event)
{
	OrderedCollectionIterator ite(fDrawPart->GetShapeList());

	for (CBaseShape* shape = (CBaseShape*)ite.First(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Next())
	{
		if (shape->HitTest(this, mouse, event))
			return shape;	
	}
	
	return kXMPNULL;
}

//------------------------------------------------------------------------------
// CDrawFacet::DrawGrid
//------------------------------------------------------------------------------

void CDrawFacet::DrawGrid()
{
	Rect extend;
	((CDrawFrame*)GetFrame())->GetExtendRect(extend);
	
	::PenNormal();	
	Pattern pattern1 = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF};
	::PenPat(&pattern1);
	
	for (short x = -1; x <= extend.right; x+=72)
	{
		::MoveTo(x, 0);
		::LineTo(x, extend.bottom-1);
	}
	
	Pattern pattern2 = {0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01};
	::PenPat(&pattern2);
	for (short y = -1; y <= extend.bottom; y+=72)
	{
		::MoveTo(0, y);
		::LineTo(extend.right-1, y);
	}
	
	::PenNormal();
}

//------------------------------------------------------------------------------
// CDrawFacet::ClipEmbeddedFacets
//------------------------------------------------------------------------------

void CDrawFacet::ClipEmbeddedFacets()
{
	XMPFacet* xmpFacet = GetXMPFacet();
	
	XMPShape* clipShape = ::NewXMPShape();
	XMPTransform* wclipTrans = ::NewXMPTransform();
	
	XMPShape* workingClip = ::NewXMPShape(xmpFacet->GetClipShape());
	workingClip->Transform(xmpFacet->GetWindowFrameTransform());		// so now in window coordinate
	
	OrderedCollectionIterator iter(fDrawPart->GetShapeList());
	for (CBaseShape* shape = (CBaseShape*)iter.First();
		 iter.IsNotComplete();
		 shape = (CBaseShape*)iter.Next())
	{	
		if (shape->GetShapeType() == kProxyShape)
		{	
			CProxyRunCProxyFrameIterator ite2(((CProxyShape*)shape)->GetProxyRun());
			for (CProxyFrame* proxyFrame = ite2.First(); ite2.IsNotComplete(); proxyFrame = ite2.Next())
			{
				if (proxyFrame->GetContainingFrame() == (CEmbeddingFrame*)this->GetFrame())
				{
					XMPFrameFacetIterator ite3(proxyFrame->GetXMPFrame());
					for (XMPFacet *embeddedXMPFacet = ite3.First(); ite3.IsNotComplete(); embeddedXMPFacet = ite3.Next())
					{
						if (embeddedXMPFacet->GetContainingFacet() == this->GetXMPFacet())
						{
							XMPShape* embeddedClip = ::NewXMPShape(embeddedXMPFacet->GetFrame()->GetFrameShape());
							embeddedClip->Intersect(embeddedXMPFacet->GetFrame()->GetUsedShape());
														
							// ----- Transform workingClip into embeddedXMPFacet frame coordinate -----
							wclipTrans->CopyFrom(embeddedXMPFacet->GetWindowFrameTransform());
							wclipTrans->Invert();
							workingClip->Transform(wclipTrans);
							
							// ----- Set the clip of embeddedXMPFacet -----
							embeddedClip->Intersect(workingClip);
							workingClip->Subtract(embeddedClip);
							embeddedXMPFacet->ChangeGeometry(embeddedClip, kXMPNULL);
				
							// ----- put back workingClip to window coordinate
							wclipTrans->Invert();
							workingClip->Transform(wclipTrans);
						}
					}					
				}
			}						
		}
		else
		{
			// ----- remove the clipShape of the shape -----
			clipShape->CopyFrom(shape->GetClipShape());
			clipShape->Transform(xmpFacet->GetWindowContentTransform());	// in window coordinate
			workingClip->Subtract(clipShape);
		}
	}
	
	delete clipShape;
	delete workingClip;
	delete wclipTrans;
}

//------------------------------------------------------------------------------
// CDrawFacet::CreateEmbeddedFacet
//------------------------------------------------------------------------------
//	We need to set the selected flag in the newly created embedded facet

XMPFacet* CDrawFacet::CreateEmbeddedFacet(CProxyFrame *itsFrame, XMPShape* clipShape, XMPTransform* externalTransform, 
						   		 		XMPFacet* siblingFacet, XMPFramePosition position)
{
	XMPFacet* embeddedXMPFacet = CFacet::CreateEmbeddedFacet(itsFrame, 
															clipShape, 
															externalTransform,
															siblingFacet, 
															position);
	
	embeddedXMPFacet->SetSelected(itsFrame->GetProxyRun()->IsSelected());
	
	return embeddedXMPFacet;
}

