/*
	File:		DrawFrame.cpp

	Contains:	implementation of class CDrawFrame

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _DRAWFRAME_
#include "DrawFrame.h"
#endif

#ifndef _QDRAWPART_
#include "QDrawPart.h"
#endif

#ifndef _DRAWSELECTION_
#include "DrawSelection.h"
#endif

#ifndef _UTILITIES_
#include "Utilities.h"
#endif

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _DRAWDACET_
#include "DrawFacet.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _MENUBAR_
#include "MenuBar.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

// ----- Macintosh Includes -----

#ifndef __EVENTS__
#include <Events.h>		// GetMouse, StillDown.
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>	// HiWord etc.
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>		// Abs
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>		// See MoveTransformBy
#endif

#pragma segment qdrawpart

//=========================================================================
// CDrawFrame
//=========================================================================

//------------------------------------------------------------------------------
// CDrawFrame::CDrawFrame
//------------------------------------------------------------------------------

CDrawFrame::CDrawFrame()
{
}

//------------------------------------------------------------------------------
// CDrawFrame::IDrawFrame
//------------------------------------------------------------------------------

void CDrawFrame::IDrawFrame(XMPFrame* xmpFrame, CDrawPart* drawPart)
{
	IEmbeddingFrame(xmpFrame, drawPart);
	
	// ----- By default we put all of them
	AddToFocusSet(drawPart->GetKeyFocusToken());
	AddToFocusSet(drawPart->GetMenuFocusToken());
	AddToFocusSet(drawPart->GetSelectionFocusToken());
	
	fDrawPart = drawPart;

	this->SetDroppable(kXMPTrue);
}

//------------------------------------------------------------------------------
// CDrawFrame::~CDrawFrame
//------------------------------------------------------------------------------

CDrawFrame::~CDrawFrame()
{
}

//------------------------------------------------------------------------------
// CDrawFrame::RemoveEmbeddedFrame
//------------------------------------------------------------------------------

void CDrawFrame::RemoveEmbeddedFrame(CProxyFrame* proxy)
{
	// ----- Invalidate the shape of the old frame -----
	XMPShape shape;
	proxy->GetFrameShape(&shape);
	proxy->Invalidate(&shape);
	
	// ----- Call inherited -----
	CEmbeddingFrame::RemoveEmbeddedFrame(proxy);
	
	// ------ Clip -----
	ClipEmbeddedFrames();
}

//-------------------------------------------------------------------------
// CDrawFrame::HandleKeyDown
//-------------------------------------------------------------------------

XMPBoolean CDrawFrame::HandleKeyDown(XMPEventData event)
{
	return kXMPTrue;
}

//------------------------------------------------------------------------------
// CDrawFrame::ActiveStateChanged
//------------------------------------------------------------------------------

void CDrawFrame::FocusStateChanged(XMPTypeToken focus, XMPBoolean newState)
{	
	CEmbeddingFrame::FocusStateChanged(focus, newState);

	if (focus == GetPart()->GetSelectionFocusToken())
	{
		CDrawSelection *selection = (CDrawSelection*)fDrawPart->GetSelection();
		
		selection->DrawAllHandles(this, newState);
	}
}

//------------------------------------------------------------------------------
// CDrawFrame::GetExtend
//------------------------------------------------------------------------------

void CDrawFrame::GetExtend(XMPShape* extent) const
{
	extent->CopyFrom(GetXMPFrame()->GetUsedShape());
	
	XMPTransform frameToContent;
	frameToContent.CopyFrom(GetXMPFrame()->GetInternalTransform());
	frameToContent.Invert();
	extent->Transform(&frameToContent);
}

//------------------------------------------------------------------------------
// CDrawFrame::GetExtendRect
//------------------------------------------------------------------------------

void CDrawFrame::GetExtendRect(Rect &rect) const
{
	XMPShape extentShape;
	GetExtend(&extentShape);
	rect = (*extentShape.GetQDRegion())->rgnBBox;
}

//------------------------------------------------------------------------------
// CDrawFrame::NewFacet
//------------------------------------------------------------------------------

CFacet* CDrawFrame::NewFacet(XMPFacet* xmpFacet)
{
	CDrawFacet* facet = new CDrawFacet;
	facet->IDrawFacet(xmpFacet, fDrawPart);
	return facet;
}

//------------------------------------------------------------------------------
// 	CDrawFrame::UsedShapeChanged
//
//------------------------------------------------------------------------------

void CDrawFrame::UsedShapeChanged(CProxyRun* proxyRun, XMPFrame* embeddedXMPFrame)
{
	XMPShape *oldUpdateShape = ::NewXMPShape();
	CProxyShape* shape = ((CDrawProxyRun*)proxyRun)->GetShape();
	shape->GetUpdateShape(oldUpdateShape);
	oldUpdateShape->Transform(GetInternalTransform());
	
	CEmbeddingFrame::UsedShapeChanged(proxyRun, embeddedXMPFrame);
	
	XMPShape *updateShape = ::NewXMPShape();
	shape->GetUpdateShape(updateShape);
	updateShape->Transform(GetInternalTransform());

	updateShape->Union(oldUpdateShape);
	delete oldUpdateShape;
	
	InvalidateAllFacets(kXMPNULL, updateShape);
	delete updateShape;
}
