/*
	File:		DrawFrame.h

	Contains:	Definition for class CDrawFrame

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<1+>	 3/17/94	DS		Add CDECL to QDraw Frame dtor
*/

#ifndef _DRAWFRAME_
#define _DRAWFRAME_

// ----- Framework Includes -----
#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _SHAPE_
#include <Shape.h>
#endif

//==============================================================================
// Forward Declaration
//==============================================================================

class CDrawPart;
class XMPFrame;
class CProxyShape;
class CBaseShape;
class XMPWindow;

//==============================================================================
// CDrawFrame
//==============================================================================

class CDrawFrame : public CEmbeddingFrame
{
//------------------------------------------------------------------------------
// Initialization/destruction
//------------------------------------------------------------------------------
public:		
	CDrawFrame();
	void IDrawFrame(XMPFrame* xmpFrame, CDrawPart* drawPart);
	virtual ~ CDECL CDrawFrame();

//------------------------------------------------------------------------------
// Inherited
//------------------------------------------------------------------------------
public:		
	virtual void			RemoveEmbeddedFrame(CProxyFrame* proxy);
	
	virtual CFacet* 		NewFacet(XMPFacet* xmpFacet);

	virtual void			FocusStateChanged(XMPTypeToken focus, XMPBoolean newState);

	virtual void 			UsedShapeChanged(CProxyRun* proxyRun, XMPFrame* embeddedXMPFrame);

	virtual XMPBoolean		HandleKeyDown(XMPEventData event);

//------------------------------------------------------------------------------
// New API
//------------------------------------------------------------------------------
public:		
	void 					GetExtend(XMPShape* extent) const;
	void 					GetExtendRect(Rect& rect) const;
	
//------------------------------------------------------------------------------
// Data Members
//------------------------------------------------------------------------------
private:	
	CDrawPart*				fDrawPart;
};

#endif
