/*
	File:		DrawProxy.cpp

	Contains:	implementation of class CDrawProxy

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _DRAWPROXY_
#include "DrawProxy.h"
#endif

#ifndef _QDRAWPART_
#include "QDrawPart.h"
#endif

#ifndef _DRAWFRAME_
#include "DrawFrame.h"
#endif

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _DRAWPROXYBORDER_
#include "DrawProxyBorder.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _FWPARTING_
#include "FWParting.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

// ----- Macintosh Includes -----

#ifndef mathRoutinesIncludes
#include <math routines.h>
#endif

#pragma segment qdrawpart

//=========================================================================
//	 CDrawProxyRun
//=========================================================================

//------------------------------------------------------------------------------
//	 CDrawProxyRun::CDrawProxyRun
//------------------------------------------------------------------------------

CDrawProxyRun::CDrawProxyRun()
{
	fShape = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::InitDrawProxyRun
//------------------------------------------------------------------------------

void CDrawProxyRun::InitDrawProxyRun(CDrawPart* drawPart)
{
	InitProxyRun(drawPart);
	fDrawPart = drawPart;
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::~CDrawProxyRun
//------------------------------------------------------------------------------

CDrawProxyRun::~CDrawProxyRun()
{
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::GetFacetExternalTransform
//------------------------------------------------------------------------------

void CDrawProxyRun::GetFacetExternalTransform(XMPFrame* xmpFrame, XMPTransform* facetTransform)
{
XMPUnused(xmpFrame);

	Rect box;
	fShape->GetBoundingBox(box);
	facetTransform->Reset();
	XMPPoint offset(ff(box.left), ff(box.top));
	facetTransform->MoveBy(offset);
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::UsedShapeChanged
//------------------------------------------------------------------------------

void CDrawProxyRun::UsedShapeChanged(XMPFrame* xmpFrame)
{
	if (fShape)		// fShape might be null if UsedShapeChanged called early
	{
		// We assume here that all embedded facets have the same
		// externalTransform
		
		XMPFrameFacetIterator* ite = xmpFrame->CreateFacetIterator();
		XMPFacet *xmpFacet = ite->First();
		delete ite;
		
		if (xmpFacet)
		{
			XMPShape *shape = ::NewXMPShape(xmpFrame->GetFrameShape());
			shape->Transform(xmpFacet->GetExternalTransform());
			XMPRect rect;
			shape->GetBoundingBox(&rect);
			delete shape;
			fShape->ProxyShapeChanged(rect);
		}
	}
	
	CProxyRun::UsedShapeChanged(xmpFrame);
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::MoveBefore
//------------------------------------------------------------------------------

void CDrawProxyRun::MoveBefore(CDrawProxyRun* before)
{
	CPartFacetIterator ite(fDrawPart, fDrawPart->GetMainPresentation());
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		XMPFacetIterator* ite3 = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
		for (XMPFacet* xmpFacet = ite3->First(); ite3->IsNotComplete(); xmpFacet = ite3->Next())
		{
			if (xmpFacet->GetFrame()->GetPart() == 	GetXMPPart())
			{
				XMPFacetIterator* ite4 = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
				for (XMPFacet* xmpFacet2 = ite4->First(); ite4->IsNotComplete(); xmpFacet2 = ite4->Next())
				{
					if (xmpFacet2->GetFrame()->GetPart() == before->GetXMPPart())
					{
						facet->GetXMPFacet()->MoveBefore(xmpFacet, xmpFacet2);
						break;
					}
				}
				delete ite4;
			}
		}
		delete ite3;
	}
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::MoveBehind
//------------------------------------------------------------------------------

void CDrawProxyRun::MoveBehind(CDrawProxyRun* behind)
{	
	CPartFacetIterator ite(fDrawPart, fDrawPart->GetMainPresentation());
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		XMPFacetIterator* ite3 = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
		for (XMPFacet* xmpFacet = ite3->First(); ite3->IsNotComplete(); xmpFacet = ite3->Next())
		{
			if (xmpFacet->GetFrame()->GetPart() == 	GetXMPPart())
			{
				XMPFacetIterator* ite4 = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
				for (XMPFacet* xmpFacet2 = ite4->First(); ite4->IsNotComplete(); xmpFacet2 = ite4->Next())
				{
					if (xmpFacet2->GetFrame()->GetPart() == behind->GetXMPPart())
					{
						facet->GetXMPFacet()->MoveBehind(xmpFacet, xmpFacet2);
						break;
					}
				}
				delete ite4;
			}
		}
		delete ite3;
	}
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::MoveFirst
//------------------------------------------------------------------------------

void CDrawProxyRun::MoveFirst()
{
	CPartFacetIterator ite(fDrawPart, fDrawPart->GetMainPresentation());
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		XMPFacetIterator* ite3 = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPBackToFront);
		for (XMPFacet* xmpFacet = ite3->First(); ite3->IsNotComplete(); xmpFacet = ite3->Next())
		{
			if (xmpFacet->GetFrame()->GetPart() == 	GetXMPPart())
			{
				facet->GetXMPFacet()->MoveBefore(xmpFacet, kXMPNULL);
			}
		}
		delete ite3;
	}
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::MoveLast
//------------------------------------------------------------------------------

void CDrawProxyRun::MoveLast()
{
	CPartFacetIterator ite(fDrawPart, fDrawPart->GetMainPresentation());
	for (CFacet* facet = ite.First(); ite.IsNotComplete(); facet = ite.Next())
	{
		XMPFacetIterator* ite3 = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
		for (XMPFacet* xmpFacet = ite3->First(); ite3->IsNotComplete(); xmpFacet = ite3->Next())
		{
			if (xmpFacet->GetFrame()->GetPart() == 	GetXMPPart())
			{
				facet->GetXMPFacet()->MoveBehind(xmpFacet, kXMPNULL);
			}
		}
		delete ite3;
	}
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::NewProxyBorder
//------------------------------------------------------------------------------

CBaseProxyBorder* CDrawProxyRun::NewProxyBorder()
{
	CDrawProxyBorder* border = new CDrawProxyBorder;
	border->InitDrawProxyBorder(this);
	return border;
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::SetSelectState
//------------------------------------------------------------------------------

void CDrawProxyRun::SetSelectState(XMPBoolean state)
{
	CProxyRun::SetSelectState(state);
	
	// ----- Set the selected flag in facet -----
	CProxyRunCProxyFrameIterator ite(this);
	for (CProxyFrame* proxyFrame = ite.First(); ite.IsNotComplete(); proxyFrame = ite.Next())
	{
		XMPFrameFacetIterator ite2(proxyFrame->GetXMPFrame());
		for (XMPFacet* xmpFacet = ite2.First(); ite2.IsNotComplete(); xmpFacet = ite2.Next())
			xmpFacet->SetSelected(state);
	}
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::OffsetProxyFrames
//------------------------------------------------------------------------------

void CDrawProxyRun::OffsetProxyFrames(const XMPPoint& offset)
{
	CProxyRunCProxyFrameIterator ite(this);
	for (CProxyFrame *proxy = ite.First(); ite.IsNotComplete(); proxy = ite.Next())
	{
		proxy->OffsetFrame(offset);
	}
	GetProxyBorder()->BorderHasChanged(kXMPTrue);
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::PlaceProxyFrames
//------------------------------------------------------------------------------

void CDrawProxyRun::PlaceProxyFrames(const XMPPoint& position)
{
	CProxyRunCProxyFrameIterator ite(this);
	for (CProxyFrame *proxy = ite.First(); ite.IsNotComplete(); proxy = ite.Next())
	{
		proxy->PlaceFrame(position);
	}
	GetProxyBorder()->BorderHasChanged(kXMPTrue);
}

//------------------------------------------------------------------------------
//	 CDrawProxyRun::ResizeProxyFrames
//------------------------------------------------------------------------------

void CDrawProxyRun::ResizeProxyFrames(XMPShape* newShape)
{
	CProxyRunCProxyFrameIterator ite(this);
	for (CProxyFrame *proxy = ite.First(); ite.IsNotComplete(); proxy = ite.Next())
	{
		XMPShape *shape = ::NewXMPShape(newShape);
		proxy->ResizeFrame(shape);
	}
	delete newShape;
	GetProxyBorder()->BorderHasChanged(kXMPTrue);
}

