/*
	File:		DrawProxy.h

	Contains:	Definition for class CDrawProxy

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

*/

#ifndef _DRAWPROXY_
#define _DRAWPROXY_

// ----- Framework Includes -----
#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWPROXY_
#include "FWProxy.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _SHAPE_
#include <Shape.h>
#endif

//==============================================================================
// Forward Declaration
//==============================================================================

class XMPFrame;
class CProxyShape;
class XMPTransform;
class CDrawPart;

//==============================================================================
//	 class CDrawProxyRun
//==============================================================================

class CDrawProxyRun : public CProxyRun
{
//--------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CDrawProxyRun();
	void InitDrawProxyRun(CDrawPart* drawPart);
	virtual ~CDrawProxyRun();
	
//-------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual void 				UsedShapeChanged(XMPFrame* xmpFrame);
	virtual CBaseProxyBorder* 	NewProxyBorder();
	
	virtual void				SetSelectState(XMPBoolean state);
	virtual void 				GetFacetExternalTransform(XMPFrame* xmpFrame, XMPTransform* facetTransform);
	
//--------------------------------------------------------------------------------
//	 New API
//
public:
	void						OffsetProxyFrames(const XMPPoint& offset);
	void						PlaceProxyFrames(const XMPPoint& position);
	void						ResizeProxyFrames(XMPShape* newShape);
	
	void 						MoveBefore(CDrawProxyRun* before);
	void 						MoveBehind(CDrawProxyRun* behind);
	void						MoveLast();
	void						MoveFirst();

	void						SetShape(CProxyShape* shape)
									{fShape = shape;}
	CProxyShape* 				GetShape() const
									{return fShape;}
	
//--------------------------------------------------------------------------------
//	 Data Members
//--------------------------------------------------------------------------------
private:
	CProxyShape					*fShape;
	CDrawPart					*fDrawPart;
};

#endif
