/*
	File:		DrawSelection.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h
		<2+>	 3/17/94	DS		Make CSelectionIterator's constructor take
									constant parms
		 <2>	 3/14/94	DS		Make EmbedSelection pass true to
									NewProxyShape so that Insert... will work.

	To Do:
*/

/*
	File:		DrawSelection.cpp

	Contains:	implementation of class CDrawSelection

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _DRAWSELECTION_
#include "DrawSelection.h"
#endif

#ifndef _QDRAWPART_
#include "QDrawPart.h"
#endif

#ifndef _DRAWFRAME_
#include "DrawFrame.h"
#endif

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _QDRAWDEF_
#include "QDrawDef.h"
#endif

// ----- Framework Includes -----

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWEVENTH_
#include "FWEventH.h"
#endif

#ifndef _FWBORDER_
#include "FWBorder.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _LINK_
#include <Link.h>
#endif

#ifndef _LINKSRC_
#include <LinkSrc.h>
#endif

// ----- Macintosh Includes -----

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>				// for ff()
#endif

#pragma segment qdrawpart

//==============================================================================
//	 class CDrawSelection
//==============================================================================

//------------------------------------------------------------------------------
//	 CDrawSelection::CDrawSelection
//------------------------------------------------------------------------------

CDrawSelection::CDrawSelection()
{
	fDrawPart = kXMPNULL;
	fUpdateShape = kXMPNULL;
	::SetRect(&fDragRect, 0,0,0,0);
	fProxyShapeCount = 0;
	fFrozenCount = 0;
	fCount = 0;
	fCollection = kXMPNULL;
	fSelectionShape = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::InitDrawSelection
//------------------------------------------------------------------------------

void CDrawSelection::InitDrawSelection(CDrawPart* drawPart)
{
	InitSelection(drawPart, kXMPTrue, kXMPTrue);	// Allow both publish and subscrib
	
	fDrawPart = drawPart;
	fCollection = new OrderedCollection;
	
	fSelectionShape = ::NewXMPShape();
}

//------------------------------------------------------------------------------
//	 CDrawSelection::~CDrawSelection
//------------------------------------------------------------------------------

CDrawSelection::~CDrawSelection()
{	
	delete fUpdateShape;
	delete fSelectionShape;
	
	fCollection->RemoveAll();
	delete fCollection;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::WhichHandle
//------------------------------------------------------------------------------

CBaseShape* CDrawSelection::WhichHandle(CFacet* facet, Point mouse, short& whichHandle) const
{
	whichHandle = 0;
	
	if (fCount != 0)
	{
		CSelectionIterator ite(this);
		for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
		{
			whichHandle = shape->WhichHandle(facet, mouse);
			if (whichHandle != 0)
				return shape;
		}
	}
	
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DrawSelectionHandles
//------------------------------------------------------------------------------

void CDrawSelection::DrawSelectionHandles(CFacet* facet, XMPBoolean turOn)
{
	CSelectionIterator ite(this);
	for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->DrawShapeHandles(facet, turOn);	
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DrawAllHandles
//------------------------------------------------------------------------------

void CDrawSelection::DrawAllHandles(CFrame* frame, XMPBoolean turOn)
{
	if (fCount != 0)
	{
		CFrameFacetIterator i(frame);		
		for (CFacet* facet = i.First(); i.IsNotComplete(); facet = i.Next())
		{
			CDrawInitiator di(facet);
			DrawSelectionHandles(facet, turOn);
		}
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DrawHandles
//------------------------------------------------------------------------------

void CDrawSelection::DrawHandles(CBaseShape* shape, XMPBoolean turOn)
{
	CPartDisplayFrameIterator ite(fDrawPart, fDrawPart->GetMainPresentation());
	for (CFrame* frame = ite.First(); ite.IsNotComplete(); frame = ite.Next())
	{
		if (frame->IsActive())
		{
			CFrameFacetIterator i(frame);			
			for (CFacet* facet = i.First(); i.IsNotComplete(); facet = i.Next())
			{
				CDrawInitiator di(facet);
				shape->DrawShapeHandles(facet, turOn);
			}
		}
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::CloseSelection
//------------------------------------------------------------------------------

void CDrawSelection::CloseSelection()
{
	CBaseShape *shape;	
	while ((shape = (CBaseShape*)fCollection->First()) != kXMPNULL)
	{
		DoRemove(shape);
		DrawHandles(shape, kXMPFalse);		// turn off
	}

	CalcDragRect();
	CalcUpdateShape();
}

//------------------------------------------------------------------------------
//	 CDrawSelection::AddToSelection
//------------------------------------------------------------------------------

void CDrawSelection::AddToSelection(CBaseShape* newSelection, XMPBoolean drawHandles)
{				
	if (newSelection != kXMPNULL)
	{
		DoAdd(newSelection);
		if (drawHandles)
			DrawHandles(newSelection, kXMPTrue);	// Turn on	
	}		
			
	CalcDragRect();
	CalcUpdateShape();
}

//------------------------------------------------------------------------------
//	 CDrawSelection::RemoveFromSelection
//------------------------------------------------------------------------------

void CDrawSelection::RemoveFromSelection(CBaseShape* shape, XMPBoolean drawHandles)
{				
	if (shape != kXMPNULL)
	{
		DoRemove(shape);
		if (drawHandles)
			DrawHandles(shape, kXMPFalse);		// Turn Off
	}		
		
	CalcDragRect();
	CalcUpdateShape();
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DoAdd
//------------------------------------------------------------------------------

void CDrawSelection::DoAdd(CBaseShape *shape)
{
	shape->SelectShape(kXMPTrue);
	fCollection->AddLast(shape);
	fCount++;
	if (shape->GetShapeType() == kProxyShape)
		fProxyShapeCount++;
	if (shape->IsFrozen())
		fFrozenCount++;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DoRemove
//------------------------------------------------------------------------------

void CDrawSelection::DoRemove(CBaseShape *shape)
{
	shape->SelectShape(kXMPFalse);
	fCollection->Remove(shape);
	fCount--;
	if (shape->GetShapeType() == kProxyShape)
		fProxyShapeCount--;
	if (shape->IsFrozen())
		fFrozenCount--;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DoClear
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::DoClear()
{
	CBaseShape *shape;
	while ((shape = (CBaseShape*)fCollection->First()) != kXMPNULL)
	{
		DoRemove(shape);				// Remove from Selection list
		fDrawPart->DeleteShape(shape);	// Remove from shape list
	}
	
	fDrawPart->ClipEmbeddedFrames(fDrawPart->GetMainPresentation());
	fDrawPart->InvalidateAllFrames(fDrawPart->GetMainPresentation(), kXMPNULL, fUpdateShape);
	
	CalcDragRect();
	CalcUpdateShape();
	
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::SelectAll
//------------------------------------------------------------------------------

void CDrawSelection::SelectAll()
{				
	DrawAllHandles(fDrawPart->GetActiveFrame(), kXMPFalse);

	OrderedCollectionIterator ite(fDrawPart->GetShapeList());
	for (CBaseShape *shape = (CBaseShape*)ite.First(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Next())
	{
		if (!shape->IsSelected())
			DoAdd(shape);
	}
	
	DrawAllHandles(fDrawPart->GetActiveFrame(), kXMPTrue);

	CalcDragRect();
	CalcUpdateShape();
	
	fDrawPart->SetTool(kSelectTool);
}

//------------------------------------------------------------------------------
//	 CDrawSelection::IsEmpty
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::IsEmpty() const
{	
	return fCount == 0;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::IsSelectionOnlyOneProxyRun
//------------------------------------------------------------------------------

CProxyRun* CDrawSelection::IsSelectionOnlyOneProxyRun() const
{
	if (fCount == 1 && fProxyShapeCount == 1)
	{
		return ((CProxyShape*)fCollection->First())->GetProxyRun();
	}
	
	return kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::ExternalizeSelection
//------------------------------------------------------------------------------

void CDrawSelection::ExternalizeSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
	storageUnit->AddProperty(kXMPPropContents)->AddValue(GetPart()->GetContentPropertyValueType());
	
	// ----- Write number of shapes -----
	storageUnit->SetValue(sizeof(fCount), (XMPValue)&fCount);
	
	// ----- Write top left offset -----
	XMPCoordinate coordinate = ff(-fDragRect.left);
	storageUnit->SetValue(sizeof(coordinate), (XMPValue)&coordinate);
	coordinate = ff(-fDragRect.top);
	storageUnit->SetValue(sizeof(coordinate), (XMPValue)&coordinate);
	
	// ----- Write out the contents of the selection -----
	CSelectionIterator ite(this);
	for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->CloneTo(storageUnit, commandFrame, cloneKind);
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::InternalizeSelection
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	if (storageUnit->Exists(kXMPPropContents, GetPart()->GetContentPropertyValueType(), 0))
	{
		// ----- Close First the current selection -----
		CloseSelection();
		
		// ----- Focus on content property -----
		storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, 
					   		GetPart()->GetContentPropertyValueType(), (XMPValueIndex)0, kXMPPosUndefined);
	
		// ----- Read number of shapes -----
		XMPULong count;
		storageUnit->GetValue(sizeof(count), (XMPValue)&count);
	
		// ----- Read top left offset -----
		XMPCoordinate xTopLeft, yTopLeft;
		storageUnit->GetValue(sizeof(xTopLeft), (XMPValue)&xTopLeft);
		storageUnit->GetValue(sizeof(yTopLeft), (XMPValue)&yTopLeft);
		
		// ----- Then read in the new selection -----
		CBaseShape *after = kXMPNULL;
		for (XMPUShort i = 0; i<count; i++)
		{
			XMPUShort shapeType;
			CBaseShape* theShape = kXMPNULL;
			storageUnit->GetValue(sizeof(XMPUShort), (XMPValue)&shapeType);	
			theShape = fDrawPart->NewShape(shapeType);
			
			if (theShape)
			{
				theShape->CloneFrom(fDrawPart, storageUnit, cloneKind);
				
				if (after)
					fDrawPart->AddShapeAfter(after, theShape);
				else
					fDrawPart->AddShape(theShape);
				after = theShape;
				
				this->AddToSelection(theShape, kXMPFalse);
			}
		}
	
		this->OffsetSelection(xTopLeft, yTopLeft);

		return kXMPTrue;
	}
	
	return kXMPFalse;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::EmbedSelection
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape)
{
	XMPRect shapeRect;
	if (frameShape)
	{
		frameShape->GetBoundingBox(&shapeRect);
		shapeRect.Offset(ff(-shapeRect.left), ff(-shapeRect.top));
	}
	else
	{
		shapeRect.Set(ff(0), ff(0), ff(80), ff(80));
	}
	
	CProxyShape *proxyShape = fDrawPart->NewProxyShape(xmpPart, shapeRect, kXMPTrue);	// ??? DS Was False, made true to support insert..
	
	AddToSelection(proxyShape, kXMPFalse);
	
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DoPaste
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::DoPaste()
{
	XMPBoolean result = FW_CSelection::DoPaste();
	
	if (result)
		AdjustSelectionAfterPaste();
	
	return result;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DoPasteWithLink
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::DoPasteWithLink()
{
	XMPBoolean result = FW_CSelection::DoPasteWithLink();
	
	if (result)
		AdjustSelectionAfterPaste();
	
	return result;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::AdjustSelectionAfterPaste
//------------------------------------------------------------------------------

void CDrawSelection::AdjustSelectionAfterPaste()
{
	fDrawPart->SetTool(kSelectTool);
	
	fDrawPart->InvalidateAllFrames(fDrawPart->GetMainPresentation(), kXMPNULL, fUpdateShape);
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DoDroppedInSameFrame
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::DoDroppedInSameFrame(XMPStorageUnit* dropSU, CFacet* facet,
												XMPPoint originPoint, XMPPoint dropPoint)
{
	XMPUnused(dropSU);
	
	XMPPoint delta;
	delta.x = dropPoint.x - originPoint.x;
	delta.y = dropPoint.y - originPoint.y;
	if (delta.x != ff(0) || delta.y != ff(0))
	{
		facet->InvalidateAllFacets(fUpdateShape, kXMPFalse);
		OffsetSelection(delta.x, delta.y);
		facet->InvalidateAllFacets(fUpdateShape, kXMPFalse);
	}
	
	return kXMPTrue;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::DoDrop
//------------------------------------------------------------------------------

XMPBoolean CDrawSelection::DoDrop(XMPStorageUnit* dropSU, CFacet* facet, XMPPoint originPoint, XMPPoint dropPoint)
{
	CloseSelection();
	
	XMPBoolean result = FW_CSelection::DoDrop(dropSU, facet, originPoint, dropPoint);
	
	if (result)
	{
		fDrawPart->SetTool(kSelectTool);
		
		XMPPoint newOffset = dropPoint;
		XMPPoint frameOffset(facet->GetWindowFrameTransform()->GetQDOffset());
		
		newOffset -= frameOffset;
		newOffset -= originPoint;
		
		XMPRect box;
		fUpdateShape->GetBoundingBox(&box);
		XMPPoint currentOffset = box.TopLeft();
		OffsetSelection(-currentOffset.x + newOffset.x, -currentOffset.y + newOffset.y);
		
		// ----- Invalidate -----
		fDrawPart->InvalidateAllFrames(fDrawPart->GetMainPresentation(), kXMPNULL, fUpdateShape);
	}
	
	return result;
}

/*
//------------------------------------------------------------------------------
//	 CDrawSelection::Drag
//------------------------------------------------------------------------------

void CDrawSelection::Drag(CFacet* facet, Point anchorPoint, XMPEventData event)
{
	if (!::WaitMouseMoved(event->where))
		return;
		
	XMPRgnHandle dragRgn = CalcDragRgn();
	
	::PenNormal();
	::PenPat(&XMPASLMQDGlobals.gray);
	::PenMode(patXor);

	Point currentLoc;
	Point prevLoc =  anchorPoint;
	short xDelta = 0;
	short yDelta = 0;
	
	XMPBoolean erase = kXMPFalse;
	
	XMPBoolean stillDown = kXMPTrue;
	while (stillDown)
	{
		::GetMouse(&currentLoc);	

		if ((prevLoc.h != currentLoc.h) || (prevLoc.v != currentLoc.v))
		{
			if (erase)
				DragFeedback(dragRgn, xDelta, yDelta);
			
			xDelta = currentLoc.h - anchorPoint.h;
			yDelta = currentLoc.v - anchorPoint.v;
			DragFeedback(dragRgn, xDelta, yDelta);
			
			erase = kXMPTrue;
			prevLoc = currentLoc;
		}
		
		stillDown = ::StillDown();
	}
	
	if (erase)
		DragFeedback(dragRgn, xDelta, yDelta);
	
	::PenNormal();

	if (xDelta != 0 || yDelta != 0)
	{
		facet->InvalidateAllFacets(fUpdateShape, kXMPFalse);
		OffsetSelection(ff(xDelta), ff(yDelta));
		facet->InvalidateAllFacets(fUpdateShape, kXMPFalse);
	}
}
*/

//------------------------------------------------------------------------------
//	 CDrawSelection::DragFeedback
//------------------------------------------------------------------------------

void CDrawSelection::DragFeedback(RgnHandle dragRgn, short xDelta, short yDelta)
{
	::OffsetRgn(dragRgn, xDelta, yDelta);
	::PaintRgn(dragRgn);
	::OffsetRgn(dragRgn, -xDelta, -yDelta);
}

//------------------------------------------------------------------------------
//	 CDrawSelection::Resize
//------------------------------------------------------------------------------

void CDrawSelection::Resize(short whichHandle, CFacet* facet, Point firstPoint, XMPEventData event)
{	
	if (!::WaitMouseMoved(event->where))
		return;
	
	CBaseShape* anchorShape = GetAnchorShape();
	
	Point handleCenter;
	anchorShape->GetHandleCenter(whichHandle, handleCenter);
	anchorShape->DrawHandle(whichHandle);	// erase handle
	
	// ----- Calcul delta between the mouse click and the
	// ----- handle center
	short xDelta = firstPoint.h - handleCenter.h;
	short yDelta = firstPoint.v - handleCenter.v;
	
	Point currentLoc;
	Point prevLoc = handleCenter;

	XMPBoolean firstMove = kXMPTrue;
	
	::PenNormal();	
	::PenPat(&XMPASLMQDGlobals.gray);
	::PenMode(patXor);
	Point penSize;
	anchorShape->GetPenSize(penSize);
	::PenSize(penSize.h, penSize.v);
	
	Rect mapRect;
	anchorShape->GetBoundingBox(mapRect);
	Rect orgRect = mapRect;
	XMPBoolean erase = kXMPFalse;
	XMPBoolean stillDown = kXMPTrue;
	while (stillDown)
	{
		::GetMouse(&currentLoc);	
		currentLoc.h -= xDelta;
		currentLoc.v -= yDelta;
		
		if ((prevLoc.h != currentLoc.h) || (prevLoc.v != currentLoc.v))
		{
			if (erase)
				anchorShape->ResizeFeedback(orgRect, mapRect);		// erase
			
			mapRect = orgRect;
			CalcMapRect(whichHandle, currentLoc, penSize, mapRect);
			
			anchorShape->ResizeFeedback(orgRect, mapRect);			// draw
			
			erase = kXMPTrue;				
			prevLoc = currentLoc;
		}
		
		stillDown = ::StillDown();
	}
	
	if (erase)
		anchorShape->ResizeFeedback(orgRect, mapRect);				// Erase
		
	::PenNormal();
	
	if (firstPoint.h != currentLoc.h || firstPoint.v != currentLoc.v)
	{
		facet->InvalidateAllFacets(fUpdateShape, kXMPFalse);
		MapSelection(orgRect, mapRect);
		facet->InvalidateAllFacets(fUpdateShape, kXMPFalse);
		
		// ----- Clip all embedding frames -----
		((CEmbeddingPart*)facet->GetFrame()->GetPart())->ClipEmbeddedFrames(facet->GetFrame()->GetIdentifier());
		
		SelectionChanged();
	}
	else
	{
		anchorShape->DrawHandle(whichHandle);	// redraw the handle
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::CalcMapRect
//------------------------------------------------------------------------------

void CDrawSelection::CalcMapRect(XMPSShort whichHandle, Point currentPos, Point penSize, Rect& mapRect)
{
	switch (whichHandle)
	{
		case kInTopLeftCorner:
			mapRect.left = currentPos.h;
			mapRect.top = currentPos.v;
			break;
		case kInTopRightCorner:
			mapRect.right = currentPos.h+1;
			mapRect.top = currentPos.v;
			break;
		case kInBottomLeftCorner:
			mapRect.left = currentPos.h;
			mapRect.bottom = currentPos.v+1;
			break;
		case kInBottomRightCorner:
			mapRect.right = currentPos.h+1;
			mapRect.bottom = currentPos.v+1;
			break;
	}
	
	if (mapRect.left > mapRect.right)
	{
		switch (whichHandle)
		{
			case 1:
			case 3:
				mapRect.right -= penSize.h;
				break;
			case 2:
			case 4:
				mapRect.left += penSize.h;
				break;
		}		
	}
	if (mapRect.top > mapRect.bottom)
	{
		switch (whichHandle)
		{
			case 1:
			case 2:
				mapRect.bottom -= penSize.v;
				break;
			case 3:
			case 4:
				mapRect.top += penSize.v;
				break;
		}		
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::CalcDragRect
//------------------------------------------------------------------------------

void CDrawSelection::CalcDragRect()
{
	::SetRect(&fDragRect,0,0,0,0);
	
	if (fCount == 0)
		return;
		
	CSelectionIterator ite(this);
	Rect tempRect;
	XMPBoolean first = kXMPTrue;
	
	for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->GetBoundingBox(tempRect);
		if (first)
			fDragRect = tempRect;
		else
			::UnionRect(&fDragRect, &tempRect, &fDragRect);
		first = kXMPFalse;
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::CalcDragRgn
//------------------------------------------------------------------------------

XMPRgnHandle CDrawSelection::CalcDragRgn(CFacet* facet)
{
	XMPUnused(facet);
	CBaseShape* anchorShape = GetAnchorShape();
	XMPRgnHandle dragRgn = ::NewRgn();

	RgnHandle tempRgn = ::NewRgn();

	if (fCount > 1)
	{
		::RectRgn(dragRgn, &fDragRect);
		::CopyRgn(dragRgn, tempRgn);
		::InsetRgn(tempRgn, 1, 1);
		::DiffRgn(dragRgn, tempRgn, dragRgn);
	}
	
	anchorShape->GetDragRgn(tempRgn);
	::UnionRgn(dragRgn, tempRgn, dragRgn);
	
	::DisposeRgn(tempRgn);
		
	return dragRgn;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::CalcUpdateShape
//------------------------------------------------------------------------------

void CDrawSelection::CalcUpdateShape()
{
	delete fUpdateShape;
	fUpdateShape = ::NewXMPShape();
	
	if (fCount == 0)
		return;

	CSelectionIterator ite(this);
	XMPShape tempShape;
	for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->GetUpdateShape(&tempShape);
		fUpdateShape->Union(&tempShape);
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::MapSelection
//------------------------------------------------------------------------------

void CDrawSelection::MapSelection(const Rect& srcRect, const Rect& dstRect)
{
	CSelectionIterator ite(this);	
	for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->MapShape(srcRect, dstRect);
	}
	
	fDrawPart->ClipEmbeddedFrames(fDrawPart->GetMainPresentation());
	
	CalcDragRect();
	CalcUpdateShape();
}

//------------------------------------------------------------------------------
//	 CDrawSelection::OffsetSelection
//------------------------------------------------------------------------------

void CDrawSelection::OffsetSelection(XMPCoordinate xDelta, XMPCoordinate yDelta)
{
	CSelectionIterator ite(this);
	for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->OffsetShape(xDelta, yDelta);
	}

	fDrawPart->ClipEmbeddedFrames(fDrawPart->GetMainPresentation());
	
	CalcDragRect();
	CalcUpdateShape();
}

//------------------------------------------------------------------------------
//	 CDrawSelection::SelectWithRectangle
//------------------------------------------------------------------------------

void CDrawSelection::SelectWithRectangle(CFacet* facet, Point anchorPoint, XMPEventData event)
{
	CRectShape* rectShape = new CRectShape;

	Point penSize = {1, 1};
	Pattern framePat = {0xCC, 0x66, 0x33, 0x99, 0xCC, 0x66, 0x33, 0x99};
	CShapeStyle trackStyle(penSize, framePat, kRGBBlack);
	
	CDrawInitiator di(facet);
	if (rectShape->Track(trackStyle, anchorPoint, event))
	{
		XMPBoolean isShift = IsShiftKeyPressed();
		Rect selectRect;
		rectShape->GetBoundingBox(selectRect);
		OrderedCollectionIterator ite(fDrawPart->GetShapeList());
		for (CBaseShape *shape = (CBaseShape*)ite.First(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Next())
		{
			if (shape->InSelectionRect(selectRect))
			{
				if (shape->IsSelected())
				{
					if (isShift)
					{
						DoRemove(shape);
						DrawHandles(shape, kXMPFalse);		// Turn Off
					}
				}
				else
				{
					DoAdd(shape);
					DrawHandles(shape, kXMPTrue);	// Turn on	
				}
			}
			else if (shape->IsSelected() && !isShift)
			{
				DoRemove(shape);
				DrawHandles(shape, kXMPFalse);		// Turn Off
			}
		}			
		CalcDragRect();
		CalcUpdateShape();
	}
	else
	{
		CloseSelection();
	}
	
	delete rectShape;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::SetSelectedShapesPenSize
//------------------------------------------------------------------------------

void CDrawSelection::SetSelectedShapesPenSize(XMPUShort newPenSize)
{	
	Point penSize;
	penSize.h = newPenSize;
	penSize.v = newPenSize;

	CShapeStyle shapeStyle;
	shapeStyle.SetPenSize(penSize);

	CSelectionIterator ite(this);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->GetShapeStyle(shapeStyle);
		shapeStyle.SetPenSize(penSize);
		shape->SetShapeStyle(shapeStyle);
		this->RedrawShape(shape);
	}
	
	fDrawPart->ClipEmbeddedFrames(fDrawPart->GetMainPresentation());
}

//------------------------------------------------------------------------------
//	 CDrawSelection::SetSelectedShapesStyle
//------------------------------------------------------------------------------

void CDrawSelection::SetSelectedShapesStyle(XMPUShort newStyle)
{
	CShapeStyle shapeStyle;

	CSelectionIterator ite(this);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->GetShapeStyle(shapeStyle);
		shapeStyle.SetFrameFillStyle(newStyle);
		shape->SetShapeStyle(shapeStyle);
		this->RedrawShape(shape);
	}
	
	fDrawPart->ClipEmbeddedFrames(fDrawPart->GetMainPresentation());
	
	SelectionChanged();
}

//------------------------------------------------------------------------------
//	 CDrawSelection::RedrawShape
//------------------------------------------------------------------------------

void CDrawSelection::RedrawShape(CBaseShape *shape)
{
	XMPShape* invalidShape = ::NewXMPShape();
	shape->GetUpdateShape(invalidShape);
	fDrawPart->InvalidateAllFrames(fDrawPart->GetMainPresentation(), kXMPNULL, invalidShape);
	delete invalidShape;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::SetFrozen
//------------------------------------------------------------------------------

void CDrawSelection::SetFrozen(XMPBoolean state)
{
	CSelectionIterator ite(this);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		if (shape->SetFrozen(state))
		{
			state ? fFrozenCount++ : fFrozenCount--;
		}
	}
}

//------------------------------------------------------------------------------
//	 CDrawSelection::NewPublishLink
//------------------------------------------------------------------------------

CPublishLink* CDrawSelection::NewPublishLink(XMPChangeID changeID)
{
	return new CDrawPublishLink(changeID, this);
}

//------------------------------------------------------------------------------
//	 CDrawSelection::NewSubscribLink
//------------------------------------------------------------------------------

CSubscribLink* CDrawSelection::NewSubscribLink(XMPLink *xmpLink)
{
	return new CDrawSubscribLink(xmpLink, this);
}

//------------------------------------------------------------------------------
//	 CDrawSelection::SelectionChanged
//------------------------------------------------------------------------------

void CDrawSelection::SelectionChanged()
{
	OrderedCollection *temp = new OrderedCollection;
	
	CSelectionIterator ite(this);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		if (shape->IsPublished())
		{
			XMPBoolean add = kXMPTrue;
			OrderedCollectionIterator ite2(temp);
			for (CDrawPublishLink* link = (CDrawPublishLink*)ite2.First(); ite2.IsNotComplete(); link = (CDrawPublishLink*)ite2.Next())
			{
				if (shape->GetPublishLink() == link)
				{
					add = kXMPFalse;
					break;
				}
			}
			if (add)
				temp->AddLast((ElementType)shape->GetPublishLink());
		}
	}
	
	OrderedCollectionIterator ite3(temp);
	for (CDrawPublishLink* link = (CDrawPublishLink*)ite3.First(); ite3.IsNotComplete(); link = (CDrawPublishLink*)ite3.Next())
	{
		link->ContentChanged(fDrawPart->GetSession()->UniqueChangeID());
	}
	
	delete temp;	// call removeAll
}

//------------------------------------------------------------------------------
//	 CDrawSelection::IsSelectionPublishable
//------------------------------------------------------------------------------
//	We don't allow linking if one of the shape is already published

XMPBoolean CDrawSelection::IsSelectionPublishable()
{
	XMPBoolean result = kXMPTrue;
	
	CSelectionIterator ite(this);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{	
		if (shape->IsPublished())
		{
			result = kXMPFalse;
			break;
		}
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 CDrawSelection::GetSelectionShape
//------------------------------------------------------------------------------

XMPShape* CDrawSelection::GetSelectionShape()
{
	XMPRect selectionRect(fDragRect);
	selectionRect.Offset(-selectionRect.left, -selectionRect.top);
	fSelectionShape->SetRectangle(&selectionRect);
	return fSelectionShape;
}

//==============================================================================
//	 class CDrawPublishLink
//==============================================================================

//------------------------------------------------------------------------------
//	 CDrawPublishLink::CDrawPublishLink
//------------------------------------------------------------------------------

CDrawPublishLink::CDrawPublishLink(XMPChangeID changeID, CDrawSelection *drawSelection) :
	CPublishLink(changeID)
{
	fDrawSelection = drawSelection;
	fCollection = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawPublishLink::Publish
//------------------------------------------------------------------------------

void CDrawPublishLink::Publish()
{
	fCollection = new OrderedCollection;
	
	CSelectionIterator ite(fDrawSelection);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		fCollection->AddLast(shape);
		shape->SetPublishLink(this);
	}
}

//------------------------------------------------------------------------------
//	 CDrawPublishLink::~CDrawPublishLink
//------------------------------------------------------------------------------

CDrawPublishLink::~CDrawPublishLink()
{
	delete fCollection;		// delete calls RemoveAll
}

//------------------------------------------------------------------------------
//	 CDrawPublishLink::GetPublishFormat
//------------------------------------------------------------------------------

XMPValueType CDrawPublishLink::GetPublishFormat()
{	
	return kPublishFormat;
}

//------------------------------------------------------------------------------
//	 CDrawPublishLink::ExternalizeLinkContent
//------------------------------------------------------------------------------

void CDrawPublishLink::ExternalizeLinkContent(XMPStorageUnit* linkSU)
{	
	fDrawSelection->ExternalizeSelection(linkSU, kXMPNULL, kXMPCloneCopy);
}

//==============================================================================
//	 class CDrawSubscribLink
//==============================================================================

//------------------------------------------------------------------------------
//	 CDrawSubscribLink::CDrawSubscribLink
//------------------------------------------------------------------------------

CDrawSubscribLink::CDrawSubscribLink(XMPLink *xmpLink, CDrawSelection *drawSelection) :
	CSubscribLink(xmpLink)
{
	fDrawSelection = drawSelection;
	fCollection = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawSubscribLink::~CDrawSubscribLink
//------------------------------------------------------------------------------

CDrawSubscribLink::~CDrawSubscribLink()
{
	delete fCollection;		// delete calls RemoveAll
}

//---------------------------------------------------------------------------------------
//	 CDrawSubscribLink::Subscrib
//---------------------------------------------------------------------------------------

void CDrawSubscribLink::Subscrib()
{
	fCollection = new OrderedCollection;
	
	CSelectionIterator ite(fDrawSelection);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		fCollection->AddLast(shape);
		shape->SetSubscribLink(this);
	}
}

//===============================================================================
//	 class CSelectionIterator
//===============================================================================

//-------------------------------------------------------------------------------
//	 CSelectionIterator::CSelectionIterator
//-------------------------------------------------------------------------------

CSelectionIterator::CSelectionIterator(const CDrawSelection* selection) :
	fIterator(selection->fCollection)
{
}

//-------------------------------------------------------------------------------
//	 CSelectionIterator::~CSelectionIterator
//-------------------------------------------------------------------------------

CSelectionIterator::~CSelectionIterator()
{
}

