/*
	File:		PaletteFacet.cpp

	Contains:	implementation of class CPaletteFacet

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _PALETTEFACET_
#include "PaletteFacet.h"
#endif

#ifndef _QDRAWPART_
#include "QDrawPart.h"
#endif

#ifndef _DRAWSELECTION_
#include "DrawSelection.h"
#endif

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _DRAWFRAME_
#include "DrawFrame.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWFRMING_
#include "FWFrming.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

// ----- Macintosh Includes -----

#ifndef __EVENTS__
#include <Events.h>		// GetMouse, StillDown.
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>	// HiWord etc.
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>		// Abs
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>		// See MoveTransformBy
#endif

#pragma segment qdrawpart

//=========================================================================
// CPaletteFacet
//=========================================================================

//------------------------------------------------------------------------------
// CPaletteFacet::CPaletteFacet
//------------------------------------------------------------------------------

CPaletteFacet::CPaletteFacet()
{
}

//------------------------------------------------------------------------------
// CPaletteFacet::InitPaletteFacet
//------------------------------------------------------------------------------

void CPaletteFacet::InitPaletteFacet(XMPFacet* xmpFacet, CDrawPart* drawPart)
{
	InitFacet(xmpFacet);
	fDrawPart = drawPart;
	
	fRects[0].Set(ff(2), ff(2), ff(24), ff(24));
	fRects[1].Set(ff(23), ff(2), ff(45), ff(24));
	
	fRects[2].Set(ff(2), ff(23), ff(24), ff(45));
	fRects[3].Set(ff(23), ff(23), ff(45), ff(45));
	
	fRects[4].Set(ff(2), ff(44), ff(24), ff(66));
	fRects[5].Set(ff(23), ff(44), ff(45), ff(66));

	fRects[6].Set(ff(2), ff(69), ff(17), ff(81));
	fRects[7].Set(ff(16), ff(69), ff(31), ff(81));
	fRects[8].Set(ff(30), ff(69), ff(45), ff(81));
}

//------------------------------------------------------------------------------
// CPaletteFacet::~CPaletteFacet
//------------------------------------------------------------------------------

CPaletteFacet::~CPaletteFacet()
{
}

//------------------------------------------------------------------------------
// CPaletteFacet::Draw
//------------------------------------------------------------------------------

void CPaletteFacet::Draw(XMPShape* invalidShape)
{
XMPUnused(invalidShape);

	DrawTools();
	
	FW_CAcquireASLMResourceAccess aq;
	
	Rect r;
	PicHandle hPic = ::GetPicture(131);
	::SetRect(&r, 2, 2 + kToolHeight + 3, kToolWidth + 2, 2 + kToolHeight + 3 + kSelectorHeight);
	::DrawPicture(hPic, &r);
	
	DrawFillFrame(fDrawPart->GetFillFrame(), XMPASLMQDGlobals.black);
}

//------------------------------------------------------------------------------
// CPaletteFacet::DrawTools
//------------------------------------------------------------------------------

void CPaletteFacet::DrawTools()
{
	FW_CAcquireASLMResourceAccess aq;
	
	Rect r;
	::SetRect(&r, 2, 2, kToolWidth + 2, kToolHeight  + 2);
	PicHandle hPic = ::GetPicture(128 + fDrawPart->GetFillFrame() - 1);
	::DrawPicture(hPic, &r);

	InvertTool(fDrawPart->GetTool());
}

//------------------------------------------------------------------------------
// CPaletteFacet::DoMouseDown
//------------------------------------------------------------------------------

XMPBoolean CPaletteFacet::DoMouseDown(const XMPPoint& where, XMPEventData event)
{
XMPUnused(event);

	for (XMPSShort i = 0; i < kNbRects; i++)
	{
		if (fRects[i].Contains(where))
		{
			if (i<=5)
				fDrawPart->SetTool(kSelectTool + i);
			else
				fDrawPart->SetFillFrame(i-5);
			return kXMPTrue;
		}
	}

	return kXMPFalse;
}

//------------------------------------------------------------------------------
// CPaletteFacet::CheckTool
//------------------------------------------------------------------------------

void CPaletteFacet::CheckTool(XMPUShort oldTool, XMPUShort newTool)
{
	CDrawInitiator i(this);
	InvertTool(oldTool);
	InvertTool(newTool);
}

//------------------------------------------------------------------------------
// CPaletteFacet::InvertTool
//------------------------------------------------------------------------------

void CPaletteFacet::InvertTool(XMPUShort theTool)
{
	if (theTool>0)
	{
		Rect rect;
		fRects[theTool -1].AsQDRect(rect);
		::InsetRect(&rect, 1, 1);
		::InvertRect(&rect);
	}
}

//------------------------------------------------------------------------------
// CPaletteFacet::CheckFillFrame
//------------------------------------------------------------------------------

void CPaletteFacet::CheckFillFrame(XMPUShort oldFillFrame, XMPUShort newFillFrame)
{
	CDrawInitiator i(this);
	DrawFillFrame(oldFillFrame, XMPASLMQDGlobals.white);
	DrawFillFrame(newFillFrame, XMPASLMQDGlobals.black);
	DrawTools();
}

//------------------------------------------------------------------------------
// CPaletteFacet::DrawFillFrame
//------------------------------------------------------------------------------

void CPaletteFacet::DrawFillFrame(XMPUShort fillFrame, const Pattern& pattern)
{
	if (fillFrame>0)
	{
		Rect rect;
		fRects[5 + fillFrame].AsQDRect(rect);
		::InsetRect(&rect, 1, 1);
		::OffsetRect(&rect, 0, 2);
		::PenPat(&pattern);
		::PenSize(2, 2);
		::MoveTo(rect.left, rect.bottom);
		::LineTo(rect.right - 2, rect.bottom);
	}
}


