/*
	File:		QDrawPart.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<11>	 4/19/94	DS		Fix
		<10>	 4/19/94	DS		Fix bus error
		 <8>	 4/12/94	DS		Add embed... menu item to arrange menu
		 <7>	 4/10/94	DS		set content property during part
									initialization
		 <6>	 3/31/94	DS		Change ref to XMPSessn.h to XMPSessnM.h
		 <5>	 3/14/94	DS		Remove Embed Menu field ( woops )
		 <4>	 3/14/94	DS		Remove Embed Menu
		<2+>	  3/8/94	DS		Ack!
		 <2>	  3/8/94	DS		implement GetContentPropertyValueType

	To Do:
*/

/*
	File:		QDrawPart.cpp

	Contains:	implementation of class CDrawPart

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _QDRAWPART_
#include "QDrawPart.h"
#endif

#ifndef _DRAWFRAME_
#include "DrawFrame.h"
#endif

#ifndef _SHAPES_
#include "Shapes.h"
#endif

#ifndef _DRAWSELECTION_
#include "DrawSelection.h"
#endif

#ifndef _PALETTEFRAME_
#include "PaletteFrame.h"
#endif

#ifndef _PALETTEFACET_
#include "PaletteFacet.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FWWINDOW_
#include "FWWindow.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _WINSTAT_
#include <WinStat.h>
#endif


#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _ISOSTRING_
#include <ISOStr.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _SUVIEW_
#include <SUView.h>
#endif

#ifndef _SUCURSOR_
#include <SUCursor.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _WINSTAT_
#include <WinStat.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _SEMTINTF_
#include <SemtIntf.h>
#endif

#ifndef _NAMRSLVR_
#include <NamRslvr.h>
#endif

#ifndef _SEUTILS_
#include <SEUtils.h>
#endif

#ifndef _PRTPRPAC_
#include <PrtPrpAc.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _LINKSRC_
#include <LinkSrc.h>
#endif

#ifndef _LINK_
#include <Link.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

// ----- Macintosh Includes -----

#ifndef __EVENTS__
#include <Events.h>		// GetMouse, StillDown.
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __PICKER__
#include <Picker.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>	// HiWord etc.
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>		// Abs
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>		// See MoveTransformBy
#endif

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif


#pragma segment qdrawpart

//=========================================================================
//	 class CDrawPart
//=========================================================================

//------------------------------------------------------------------------------
//	 CDrawPart::CDrawPart
//------------------------------------------------------------------------------

CDrawPart::CDrawPart()
{
	fPenSizeMenu = kXMPNULL;
	fArrangeMenu = kXMPNULL;
	fWindowMenu = kXMPNULL;
	
	fShapeList = kXMPNULL;
	fCurrentPenSize = 1;
	fFillFrame = kFrameFill;
	
	fNbEmbedded = 0;
	
	fPalette = kXMPNULL;
	fTool = kRectangle;
	
	fDrawSelection = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawPart::~CDrawPart
//------------------------------------------------------------------------------

CDrawPart::~CDrawPart()
{
	if (fPenSizeMenu) 
		::DisposeMenu(fPenSizeMenu);
	
	if (fArrangeMenu) 
		::DisposeMenu(fArrangeMenu);

	if (fWindowMenu) 
		::DisposeMenu(fWindowMenu);
	
	delete fShapeList;
	fShapeList = kXMPNULL;
}

//------------------------------------------------------------------------------
//	 CDrawPart::Initialize
//------------------------------------------------------------------------------

void CDrawPart::Initialize()
{
	CEmbeddingPart::Initialize();
	
	fShapeList = new OrderedCollection;

	fArrangeMenu = ::NewMenu(kArrangeMenu,"\pArrange");
	fPenSizeMenu = ::NewMenu(kPenSizeMenu,"\pSize");
	fWindowMenu = ::NewMenu(kWindowMenu,"\pWindows");
	
	::AppendMenu(fPenSizeMenu,"\p1 Pixel");
	::AppendMenu(fPenSizeMenu,"\p2 Pixels");
	::AppendMenu(fPenSizeMenu,"\p3 Pixels");
	::AppendMenu(fPenSizeMenu,"\p4 Pixels");
	::AppendMenu(fPenSizeMenu,"\p5 Pixels");

	::AppendMenu(fArrangeMenu,"\pMove Forward /F");
	::AppendMenu(fArrangeMenu,"\pMove To Front");
	::AppendMenu(fArrangeMenu,"\pMove Backward /B");
	::AppendMenu(fArrangeMenu,"\pMove To Back");
	::AppendMenu(fArrangeMenu,"\p-");
	::AppendMenu(fArrangeMenu,"\pFreeze");
	::AppendMenu(fArrangeMenu,"\pDefrost");
	::AppendMenu(fArrangeMenu,"\pEmbed");

	::AppendMenu(fWindowMenu,"\pabc/T");	// will be changed in AdjustMenus

	XMPMenuBar *menuBar = GetMenuBar();

	menuBar->AddMenuLast(kArrangeMenu, fArrangeMenu, this);
	menuBar->AddMenuLast(kPenSizeMenu, fPenSizeMenu, this);
	menuBar->AddMenuLast(kWindowMenu, fWindowMenu, this);

	menuBar->RegisterCommand(cMoveForward, kArrangeMenu, 1);
	menuBar->RegisterCommand(cMoveToFront, kArrangeMenu, 2);
	menuBar->RegisterCommand(cMoveBackward, kArrangeMenu, 3);
	menuBar->RegisterCommand(cMoveToBack, kArrangeMenu, 4);
	menuBar->RegisterCommand(cFreeze, kArrangeMenu, 6);
	menuBar->RegisterCommand(cDefrost, kArrangeMenu, 7);
	menuBar->RegisterCommand(cEmbed, kArrangeMenu, 8);
	
	menuBar->RegisterCommand(cPen1, kPenSizeMenu, 1);
	menuBar->RegisterCommand(cPen2, kPenSizeMenu, 2);
	menuBar->RegisterCommand(cPen3, kPenSizeMenu, 3);
	menuBar->RegisterCommand(cPen4, kPenSizeMenu, 4);
	menuBar->RegisterCommand(cPen5, kPenSizeMenu, 5);

	menuBar->RegisterCommand(cHideShowPalette, kWindowMenu, 1);
	
	// ----- Tokenize -----
	fPalettePresentation = GetSession()->Tokenize(kPalettePresentation);
	fMainPresentation = GetSession()->Tokenize(kMainPresentation);
}

//------------------------------------------------------------------------------
//	 CDrawPart::AddProperties
//------------------------------------------------------------------------------

void CDrawPart::AddProperties(XMPStorageUnit* storageUnit)
{
	storageUnit->AddProperty(kContentAnnotationProp)->AddValue(kContentAnnotationValue);
}

//------------------------------------------------------------------------------
//	 CDrawPart::InternalizeContent
//------------------------------------------------------------------------------

void CDrawPart::InternalizeContent(XMPStorageUnit* storageUnit)
{
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);

	// ----- Read number of shapes -----
	XMPULong count;
	storageUnit->GetValue(sizeof(count), (XMPValue)&count);

	// ----- Read top left offset -----
	XMPCoordinate xTopLeft, yTopLeft;
	storageUnit->GetValue(sizeof(xTopLeft), (XMPValue)&xTopLeft);
	storageUnit->GetValue(sizeof(yTopLeft), (XMPValue)&yTopLeft);

	for (XMPULong i = 0; i<count; i++)
	{
		XMPUShort shapeType;
		storageUnit->GetValue(sizeof(XMPUShort), (XMPValue)&shapeType);	
		CBaseShape* theShape = NewShape(shapeType);
		if (theShape)
		{
			theShape->Internalize(this, storageUnit);			
			fShapeList->AddLast(theShape);
			
			if (theShape->IsSelected())
				fDrawSelection->AddToSelection(theShape, kXMPTrue);
				
			if (xTopLeft != 0 || yTopLeft != 0)
			{
				theShape->OffsetShape(xTopLeft, yTopLeft);
			}
		}
	}

	// ----- Read general information -----
	if (storageUnit->Exists(kContentAnnotationProp, (XMPValueType)kContentAnnotationValue, 0))
	{
		storageUnit->Focus(kContentAnnotationProp, kXMPPosUndefined, kContentAnnotationValue, (XMPValueIndex)0, kXMPPosUndefined);
		storageUnit->GetValue(sizeof(XMPUShort), (XMPValue)&fTool);
		storageUnit->GetValue(sizeof(XMPUShort), (XMPValue)&fCurrentPenSize);
		storageUnit->GetValue(sizeof(XMPUShort), (XMPValue)&fFillFrame);
	}
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetContentPropertyValueType
//------------------------------------------------------------------------------
XMPValueType CDrawPart::GetContentPropertyValueType() const
{
	return kQDrawKind;
}


//------------------------------------------------------------------------------
//	 CDrawPart::ExternalizeContent
//------------------------------------------------------------------------------

void CDrawPart::ExternalizeContent(XMPStorageUnit* storageUnit)
{
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);
	storageUnit->Remove();
	storageUnit->AddValue(this->GetContentPropertyValueType());
	
	// ----- Write number of shapes -----
	XMPULong count = fShapeList->Count();
	storageUnit->SetValue(sizeof(count), (XMPValue)&count);
	
	// ----- Write top left offset -----
	XMPCoordinate coordinate = 0;
	storageUnit->SetValue(sizeof(coordinate), (XMPValue)&coordinate);
	storageUnit->SetValue(sizeof(coordinate), (XMPValue)&coordinate);

	// ----- Write shapes -----
	OrderedCollectionIterator ite(fShapeList);
	for (CBaseShape *shape = (CBaseShape*)ite.First(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Next())
	{
		shape->Externalize(storageUnit);
	}	
		
	// ----- Write general information -----
	storageUnit->Focus(kContentAnnotationProp, kXMPPosUndefined, kContentAnnotationValue, (XMPValueIndex)0, kXMPPosUndefined);
	storageUnit->SetValue(sizeof(XMPUShort), (XMPValue)&fTool);
	storageUnit->SetValue(sizeof(XMPUShort), (XMPValue)&fCurrentPenSize);
	storageUnit->SetValue(sizeof(XMPUShort), (XMPValue)&fFillFrame);
}

//------------------------------------------------------------------------------
//	 CDrawPart::NewShape
//------------------------------------------------------------------------------

CBaseShape* CDrawPart::NewShape(XMPUShort shapeType)
{	
	CBaseShape *theShape = kXMPNULL;
	
	switch (shapeType) {
		case kLineShape:
			theShape = new CLineShape();
			break;
		case kRectShape:
			theShape = new CRectShape();
			break;
		case kOvalShape:
			theShape = new COvalShape();
			break;
		case kRRectShape:
			theShape = new CRoundRectShape();
			break;
		case kProxyShape:
			theShape = new CProxyShape();
			break;
	}
	
	return theShape;
}

//------------------------------------------------------------------------------
//	 CDrawPart::Release
//------------------------------------------------------------------------------

void CDrawPart::Release()
{	
	if (fShapeList && this->GetRefCount() == 1)
	{
		CBaseShape* shape;
		while ((shape = (CBaseShape*)fShapeList->First()) != kXMPNULL) {
			fShapeList->Remove(shape);
			delete shape;
		}
	}
	
	CEmbeddingPart::Release();
}


//------------------------------------------------------------------------------
//	 CDrawPart::NewSelection
//------------------------------------------------------------------------------
//	We keep a reference to the selection around to avoid
//	typecast. fDrawSelection will be deleted by CPart

FW_CSelection* CDrawPart::NewSelection()
{	
	fDrawSelection = new CDrawSelection;
	fDrawSelection->InitDrawSelection(this);
	return fDrawSelection;
}

//------------------------------------------------------------------------------
//	 CDrawPart::DeleteShape
//------------------------------------------------------------------------------
void CDrawPart::DeleteShape(CBaseShape* shape)
{
	fShapeList->Remove(shape);
	shape->Removed();
	delete shape;
}

//------------------------------------------------------------------------------
//	 CDrawPart::AddShape
//------------------------------------------------------------------------------
void CDrawPart::AddShape(CBaseShape* shape)
{
	fShapeList->AddFirst(shape);
}

//------------------------------------------------------------------------------
//	 CDrawPart::AddShapeAfter
//------------------------------------------------------------------------------
void CDrawPart::AddShapeAfter(CBaseShape* after, CBaseShape* shape)
{
	fShapeList->AddAfter(after, shape);
}

//------------------------------------------------------------------------------
//	 CDrawPart::HandleEmbed
//
// Handle user selection of the embed menu item.
//
//------------------------------------------------------------------------------
XMPBoolean CDrawPart::HandleEmbed()
{
	XMPPart* embeddedPart;
	XMPBoolean result = kXMPFalse;
	XMPStorageUnit* su = GetStorageUnit();
	XMPSession* session = su->GetSession();
	XMPWindowState* windowState = session->GetWindowState();

	XMPType	partkind = kXMPNULL;

	// StdGetFile please select an editor for the root part
	StandardFileReply sfReply;
	SFTypeList sfTypes;
	sfTypes[0] = 'libr';
	sfTypes[1] = 'libr';
	sfTypes[2] = 'libr';
	sfTypes[3] = 'libr';
	
	windowState->DeactivateFrontWindows();
	StandardGetFile((FileFilterUPP)kXMPNULL,1,sfTypes,&sfReply);
	windowState->ActivateFrontWindows();
	
	if (sfReply.sfGood) 
	{
		XMPSShort saveResFile = CurResFile();
		XMPSShort resRefNum = FSpOpenResFile(&(sfReply.sfFile),fsRdPerm);

		Handle NMAPHdl = Get1Resource('NMAP',kNMAPid + 1);
		if (kXMPNULL != NMAPHdl)
		{

			XMPUByte offset = sizeof(XMPUShort) + sizeof(kXMPKind) + sizeof(XMPUShort);
			XMPUByte length = (*(char**)(NMAPHdl))[offset];
			partkind = (XMPValueType)XMPNewPtrClear(1 + length);
			XMPISOStrCopy(partkind,&((*(char**)(NMAPHdl))[offset+1]),(XMPULong)length);
		}
		ReleaseResource(NMAPHdl);
		CloseResFile(resRefNum);
		UseResFile(saveResFile);

		embeddedPart = su->GetDraft()->CreatePart(partkind, kXMPNULL);
		
		// ----- Create new Shape -----
		XMPRect frameRect(0, 0, ff(80), ff(80));
		XMPShape* frameShape = ::NewXMPShape(&frameRect);
		
		result = fDrawSelection->EmbedSelection(embeddedPart, frameShape);
		
		// ----- Release Shape -----
		delete frameShape;
		
		// ----- Release Part -----
		embeddedPart->Release();
	}
	return result;

}

//------------------------------------------------------------------------------
//	 CDrawPart::DoMenuEvent
//------------------------------------------------------------------------------
XMPBoolean CDrawPart::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	XMPBoolean result = kXMPTrue;
	
	if (cPen1 <= commandID && commandID <= cPen5)
	{
		XMPSShort penSize = (XMPSShort)(commandID - cPen1 + 1);
		if (!fDrawSelection->IsEmpty())
			fDrawSelection->SetSelectedShapesPenSize(penSize);
		else
			fCurrentPenSize = penSize;
	}
	else
	{
		switch (commandID)
		{
			case cHideShowPalette:
				fPalette->HideShowFloating();
				break;
			case cMoveForward:
				MoveForward();
				this->Changed();
				break;
			case cMoveToFront:
				MoveToFront();
				this->Changed();
				break;
			case cMoveBackward:
				MoveBackward();
				this->Changed();
				break;
			case cMoveToBack:
				MoveToBack();
				this->Changed();
				break;
			case cFreeze:
				fDrawSelection->SetFrozen(kXMPTrue);
				this->Changed();
				break;
			case cDefrost:
				fDrawSelection->SetFrozen(kXMPFalse);
				this->Changed();
				break;
			case cEmbed:
				result = HandleEmbed();
				this->Changed();
				break;
			default:
				result = CEmbeddingPart::DoMenuEvent(menuBar, commandID);
		}	
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 CDrawPart::SetTool
//------------------------------------------------------------------------------
void CDrawPart::SetTool(XMPUShort newTool)
{
	if (fTool != newTool)
	{
		if (fTool == kSelectTool && !fDrawSelection->IsEmpty())
		{
			fDrawSelection->CloseSelection();
		}
		
		CheckTool(fTool, newTool);
		fTool = newTool;		
	}
}

//------------------------------------------------------------------------------
//	 CDrawPart::SetPalette
//------------------------------------------------------------------------------
void CDrawPart::SetPalette(CFloatingWindow* paletteWindow)
{
	fPalette = paletteWindow;
	this->AddWindow(fPalette);
}

//------------------------------------------------------------------------------
//	 CDrawPart::SetFillFrame
//------------------------------------------------------------------------------
void CDrawPart::SetFillFrame(XMPUShort fillFrame)
{
	if (fFillFrame != fillFrame)
	{		
		if (!fDrawSelection->IsEmpty())
			fDrawSelection->SetSelectedShapesStyle(fillFrame);

		XMPUShort oldFillFrame = fFillFrame;
		fFillFrame = fillFrame;		
		CheckFillFrame(oldFillFrame, fFillFrame);
	}
}

//------------------------------------------------------------------------------
//	 CDrawPart::NewFrame
//------------------------------------------------------------------------------
CFrame* CDrawPart::NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation)
{
	if (presentation == fPalettePresentation)
	{
		CPaletteFrame *frame = new CPaletteFrame();
		frame->InitPaletteFrame(xmpFrame, this);
		return frame;
	}
	
	CDrawFrame *frame = new CDrawFrame();
	frame->IDrawFrame(xmpFrame, this);
	return frame;
}

//------------------------------------------------------------------------------
//	 CDrawPart::DoAdjustMenus
//------------------------------------------------------------------------------

void CDrawPart::DoAdjustMenus(XMPMenuBar* menuBar)
{
	CEmbeddingPart::DoAdjustMenus(menuBar);
	
	XMPCommandID command;

	CShapeStyle shapeStyle;
	XMPBoolean useSelection = kXMPFalse;
	if (fDrawSelection->Count() >= 1)
	{
		CSelectionIterator ite(fDrawSelection);
		CBaseShape* shape = ite.First();
		shape->GetShapeStyle(shapeStyle);
		useSelection = kXMPTrue;
	}
	
	// ----- PenSize Menu -----
	if (useSelection)
		command = shapeStyle.GetPenWidth() + cPen1 - 1;
	else
		command = fCurrentPenSize + cPen1 - 1;
	
	for (XMPCommandID p = cPen1; p <= cPen5; p++)
		menuBar->CheckCommand(p, p == command);
	
	XMPBoolean hasSelection = fDrawSelection->Count()>0;
	
	// ----- Edit Menu -----
	menuBar->EnableCommand(kXMPCommandSelectAll, fShapeList->Count()>0);
	menuBar->EnableCommand(kXMPCommandPaste, HasPropertyOnClipboard(kXMPPropContents, kXMPNULL)); // we don't care about the type
	
	// ----- Arrange Menu -----
	menuBar->EnableCommand(cMoveForward, hasSelection);
	menuBar->EnableCommand(cMoveToFront, hasSelection);
	menuBar->EnableCommand(cMoveBackward, hasSelection);
	menuBar->EnableCommand(cMoveToBack, hasSelection);
	menuBar->EnableCommand(cFreeze, fDrawSelection->HasNotFrozen());
	menuBar->EnableCommand(cDefrost, fDrawSelection->HasFrozen());
	menuBar->EnableCommand(cEmbed, !hasSelection);
	
	// ----- Window Menu
	if (!fPalette)
	{
		//	 Create the palette object first
		XMPRect paletteRect(ff(100), ff(100), ff(100+kPaletteWidth), ff(100 + kPaletteHeight));
		fPalette = new CFloatingWindow;
		fPalette->InitFloatingWindow(this, fPalettePresentation, paletteRect);
		this->AddWindow(fPalette);
	}
	
	if (fPalette->IsShown())
		menuBar->SetItemString(cHideShowPalette, "\pHide Palette");
	else
		menuBar->SetItemString(cHideShowPalette, "\pShow Palette");
		
}

//------------------------------------------------------------------------------
//	 CDrawPart::NewProxyShape
//------------------------------------------------------------------------------

CProxyShape* CDrawPart::NewProxyShape(XMPPart* xmpPart, const XMPRect& shapeRect, XMPBoolean invalidate)
{	
	// ----- decide on a position -----
	fNbEmbedded++;
	XMPSShort offset = 40 + fNbEmbedded*20;
	Point ptTemp;
	::SetPt(&ptTemp, offset, offset);
	XMPTransform* externalXForm = ::NewXMPTransform(ptTemp);
	
	XMPRect tempRect = shapeRect;
	XMPShape *frameShape = ::NewXMPShape(&tempRect);
	
	// ----- Create the shape
	tempRect.Offset(ff(offset), ff(offset));
	CProxyShape* theShape = new CProxyShape(tempRect);

	// ----- Add the shape to the list of shape -----
	AddShape(theShape);
	
	// ----- Create the proxyRun -----
	CDrawProxyRun* proxyRun = (CDrawProxyRun*)NewProxyRun();
	
	// ----- Set the proxyRun of the shape and the shape of the proxyRun -----
	theShape->SetProxyRun(proxyRun);
	proxyRun->SetShape(theShape);
	
	// ----- Embed the part -----
	EmbedPart(xmpPart, proxyRun, fMainPresentation, 
			  GetSession()->Tokenize(kXMPViewAsFrame), kNoPresentation,
			  externalXForm, frameShape, invalidate);

	// ----- Select the selection tool -----
	SetTool(kSelectTool);
	
	return theShape;
}

//------------------------------------------------------------------------------
//	 CDrawPart::NewProxyRun
//------------------------------------------------------------------------------

CProxyRun* CDrawPart::NewProxyRun()
{
	CDrawProxyRun *proxyRun = new CDrawProxyRun();
	proxyRun->InitDrawProxyRun(this);
	
	return proxyRun;
}

//------------------------------------------------------------------------------
//	 CDrawPart::CheckTool
//------------------------------------------------------------------------------

void CDrawPart::CheckTool(XMPUShort oldTool, XMPUShort newTool)
{
	OrderedCollectionIterator ite(GetDisplayFrameList());
	for (CFrame *frame = (CFrame*)ite.First(); ite.IsNotComplete(); frame = (CFrame*)ite.Next())
	{
		if (frame->GetPresentation() == fPalettePresentation)
		{
			CFrameFacetIterator ite2(frame);
			for (CPaletteFacet *facet = (CPaletteFacet*)ite2.First(); ite2.IsNotComplete(); facet = (CPaletteFacet*)ite2.Next())
			{
				facet->CheckTool(oldTool, newTool);
			}
		}
	}
}

//------------------------------------------------------------------------------
//	 CDrawPart::CheckFillFrame
//------------------------------------------------------------------------------

void CDrawPart::CheckFillFrame(XMPUShort oldFillFrame, XMPUShort newFillFrame)
{
	OrderedCollectionIterator ite(GetDisplayFrameList());
	for (CFrame *frame = (CFrame*)ite.First(); ite.IsNotComplete(); frame = (CFrame*)ite.Next())
	{
		if (frame->GetPresentation() == fPalettePresentation)
		{
			CFrameFacetIterator ite2(frame);
			for (CPaletteFacet *facet = (CPaletteFacet*)ite2.First(); ite2.IsNotComplete(); facet = (CPaletteFacet*)ite2.Next())
			{
				facet->CheckFillFrame(oldFillFrame, newFillFrame);
			}
		}
	}
}

//------------------------------------------------------------------------------
//	 CDrawPart::MoveForward
//------------------------------------------------------------------------------

void CDrawPart::MoveForward()
{
	XMPBoolean didIt = kXMPFalse;
	CSelectionIterator ite(fDrawSelection);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		CBaseShape* previous = (CBaseShape*)fShapeList->Before((ElementType)shape);
		if (previous != kXMPNULL)
		{
			fShapeList->Remove((ElementType)shape);
			fShapeList->AddBefore((ElementType)previous, (ElementType)shape);
			previous->MovedBefore(shape);
			didIt = kXMPTrue;
		}
	}

	if (didIt)
	{
		ClipEmbeddedFrames(fMainPresentation);
		InvalidateAllFrames(fMainPresentation, kXMPNULL, fDrawSelection->GetUpdateShape());
	}
}

//------------------------------------------------------------------------------
//	 CDrawPart::MoveBackward
//------------------------------------------------------------------------------

void CDrawPart::MoveBackward()
{
	XMPBoolean didIt = kXMPFalse;
	CSelectionIterator ite(fDrawSelection);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		CBaseShape* next = (CBaseShape*)fShapeList->After((ElementType)shape);
		if (next != kXMPNULL)
		{
			fShapeList->Remove((ElementType)shape);
			fShapeList->AddAfter((ElementType)next, (ElementType)shape);
			next->MovedAfter(shape);
			didIt = kXMPTrue;
		}
	}
	
	if (didIt)
	{
		ClipEmbeddedFrames(fMainPresentation);
		InvalidateAllFrames(fMainPresentation, kXMPNULL, fDrawSelection->GetUpdateShape());
	}
}

//------------------------------------------------------------------------------
//	 CDrawPart::MoveToFront
//------------------------------------------------------------------------------

void CDrawPart::MoveToFront()
{
	OrderedCollection *extract = ExtractSelection();
	
	CBaseShape* shape;
	OrderedCollectionIterator ite(extract);
	for (shape = (CBaseShape*)ite.Last(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Previous())
	{
		fShapeList->AddFirst(shape);
		shape->MovedFirst();
	}
	
	extract->RemoveAll();
	delete extract;

	ClipEmbeddedFrames(fMainPresentation);
	InvalidateAllFrames(fMainPresentation, kXMPNULL, fDrawSelection->GetUpdateShape());
}

//------------------------------------------------------------------------------
//	 CDrawPart::MoveToBack
//------------------------------------------------------------------------------

void CDrawPart::MoveToBack()
{
	OrderedCollection *extract = ExtractSelection();
	
	OrderedCollectionIterator ite(extract);
	for (CBaseShape* shape = (CBaseShape*)ite.First(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Next())
	{
		fShapeList->AddLast(shape);
		shape->MovedLast();
	}
	
	extract->RemoveAll();
	delete extract;

	ClipEmbeddedFrames(fMainPresentation);
	InvalidateAllFrames(fMainPresentation, kXMPNULL, fDrawSelection->GetUpdateShape());
}

//------------------------------------------------------------------------------
//	 CDrawPart::ExtractSelection
//------------------------------------------------------------------------------

OrderedCollection* CDrawPart::ExtractSelection()
{
	OrderedCollection* extract = new OrderedCollection;
	
	// ----- Create the extract list -----
	OrderedCollectionIterator ite(fShapeList);
	for (CBaseShape* shape = (CBaseShape*)ite.First(); ite.IsNotComplete(); shape = (CBaseShape*)ite.Next())
	{
		if (shape->IsSelected())
			extract->AddLast((ElementType)shape);
	}
	
	// ----- Remove then from the shape list -----
	OrderedCollectionIterator ite2(extract);
	for (shape = (CBaseShape*)ite2.First(); ite2.IsNotComplete(); shape = (CBaseShape*)ite2.Next())
	{
		fShapeList->Remove((ElementType)shape);
	}
	
	return extract;
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetDefaultPresentation
//------------------------------------------------------------------------------

XMPTypeToken CDrawPart::GetDefaultPresentation() const
{
	return fMainPresentation;
}
