/*
	File:		QDrawPart.cp

	Contains:	Definition for class CDrawPart

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <7>	 4/12/94	DS		Add cEmbed commands number
		<5+>	 3/17/94	DS		Add CDECL to QDraw Part dtor
		 <5>	 3/14/94	DS		Remove Embed Menu field ( woops )
		 <4>	 3/14/94	DS		Remove Embed because we are removing the
									embed menu
		<2+>	  3/8/94	DS		Ack!
		 <2>	  3/8/94	DS		implement GetContentPropertyValueType
*/

#ifndef _QDRAWPART_
#define _QDRAWPART_

#ifndef _QDRAWDEF_
#include "QDrawDef.h"
#endif

#ifndef _STYLE_
#include "Style.h"
#endif

// ----- Framework Includes -----

#ifndef _FWPARTING_
#include "FWParting.h"
#endif


//==============================================================================
//	 Forward Declarations
//==============================================================================

class CDrawSelection;
class CBaseShape;
class CFloatingWindow;
class CLink;
class CProxyShape;

//==============================================================================
//	 Constants
//==============================================================================

const short kArrangeMenu = 4;
const short kPenSizeMenu = kArrangeMenu + 1;
const short kWindowMenu = kPenSizeMenu + 1;

// ----- Commands -----
const XMPCommandID cMoveForward		= 1;
const XMPCommandID cMoveToFront		= cMoveForward + 1;
const XMPCommandID cMoveBackward	= cMoveToFront + 1;
const XMPCommandID cMoveToBack 		= cMoveBackward + 1;
const XMPCommandID cFreeze 			= cMoveToBack + 1;
const XMPCommandID cDefrost 		= cFreeze + 1;
const XMPCommandID cEmbed	 		= cDefrost + 1;

const XMPCommandID cPen1 		= cEmbed + 1;
const XMPCommandID cPen2 		= cPen1+1;
const XMPCommandID cPen3 		= cPen2+1;
const XMPCommandID cPen4 		= cPen3+1;
const XMPCommandID cPen5 		= cPen4+1;

const XMPCommandID cHideShowPalette = cPen5+1;

const XMPUShort kToolWidth = 43;
const XMPUShort kToolHeight = 64;
const XMPUShort kSelectorHeight = 12;
const XMPUShort kPaletteWidth = 2 + kToolWidth + 2;
const XMPUShort kPaletteHeight = 2 + kToolHeight + 4 + kSelectorHeight + 4 + 2;

const XMPUShort kSelectTool = 1;
const XMPUShort kLine = 2; 
const XMPUShort kRectangle = 3; 
const XMPUShort kOval = 4; 
const XMPUShort kRoundRect = 5;
const XMPUShort kArc = 6;

//==============================================================================
//	 class CDrawPart
//==============================================================================

class CDrawPart : public CEmbeddingPart
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:		
	CDrawPart();
	virtual ~ CDECL CDrawPart();

	virtual void			Initialize();
	virtual void			AddProperties(XMPStorageUnit* storageUnit);
	virtual void			Release();
	
	virtual XMPValueType 	GetContentPropertyValueType() const; 

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual CFrame* 		NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
									
	virtual void 			ExternalizeContent(XMPStorageUnit* storageUnit);
	virtual void 			InternalizeContent(XMPStorageUnit* storageUnit);

	virtual XMPBoolean		DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	virtual void			DoAdjustMenus(XMPMenuBar* menuBar);
	
	virtual CProxyRun*		NewProxyRun();
	
	virtual XMPTypeToken	GetDefaultPresentation() const;
	
//------------------------------------------------------------------------------
//	 New API
//
public:	
	CProxyShape*			NewProxyShape(XMPPart* xmpPart, const XMPRect& shapeRect, XMPBoolean invalidate);

	void					DeleteShape(CBaseShape* shape);
	void					AddShape(CBaseShape* shape);
	void					AddShapeAfter(CBaseShape* after, CBaseShape* shape);

	OrderedCollection*		GetShapeList() const;
	XMPUShort				GetPenSize() const;
	XMPUShort				GetFillFrame() const;
	
	XMPBoolean				HandleEmbed();

	void					CheckTool(XMPUShort oldTool, XMPUShort newTool);
	void					CheckFillFrame(XMPUShort oldFillFrame, XMPUShort newFillFrame);
	void					SetFillFrame(XMPUShort fillFrame);
	
	void					MoveForward();
	void					MoveBackward();
	void					MoveToFront();
	void					MoveToBack();

	void					SetTool(XMPUShort newTool);
	XMPUShort				GetTool() const;
								
	virtual FW_CSelection*	NewSelection();
	
	virtual void			SetPalette(CFloatingWindow* paletteWindow);
	CFloatingWindow*		GetPalette() const;

	XMPTypeToken			GetMainPresentation() const;
	XMPTypeToken			GetPalettePresentation() const;

	CBaseShape*				NewShape(XMPUShort shapeType);
	
private:
	OrderedCollection*		ExtractSelection();

//------------------------------------------------------------------------------
//	 Data Members
//------------------------------------------------------------------------------
private:	
	MenuHandle			fArrangeMenu;
	MenuHandle			fPenSizeMenu;
	MenuHandle			fWindowMenu;
	
	CFloatingWindow*	fPalette;
	
	OrderedCollection*	fShapeList;
	XMPUShort			fTool;
	XMPUShort			fCurrentPenSize;
	XMPUShort			fFillFrame;
	
	XMPUShort			fNbEmbedded;

	XMPTypeToken		fPalettePresentation;
	XMPTypeToken		fMainPresentation;
	
	CDrawSelection*		fDrawSelection;
	
	OrderedCollection*	fSrcLinks;
	OrderedCollection*	fDestLinks;
};

//==============================================================================
//	 Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 CDrawPart::GetShapeList
//------------------------------------------------------------------------------
inline OrderedCollection* CDrawPart::GetShapeList() const
{
	return fShapeList;
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetPenSize
//------------------------------------------------------------------------------
inline XMPUShort CDrawPart::GetPenSize() const
{
	return fCurrentPenSize;
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetFillFrame
//------------------------------------------------------------------------------
inline XMPUShort CDrawPart::GetFillFrame() const
{
	return fFillFrame;
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetTool
//------------------------------------------------------------------------------
inline XMPUShort CDrawPart::GetTool() const
{
	return fTool;
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetPalette
//------------------------------------------------------------------------------
inline CFloatingWindow* CDrawPart::GetPalette() const
{
	return fPalette;
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetMainPresentation
//------------------------------------------------------------------------------
inline XMPTypeToken CDrawPart::GetMainPresentation() const
{
	return fMainPresentation;
}

//------------------------------------------------------------------------------
//	 CDrawPart::GetPalettePresentation
//------------------------------------------------------------------------------
inline XMPTypeToken CDrawPart::GetPalettePresentation() const
{
	return fPalettePresentation;
}

#endif
