/*
	File:		Style.cpp

	Contains:	Implementation of classes:
						CShapeStyle

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.
*/

#ifndef _STYLE_
#include "Style.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#pragma segment qdrawpart

//==============================================================================
// class CShapeStyle
//==============================================================================

//-------------------------------------------------------------------------
// CShapeStyle::CShapeStyle
//-------------------------------------------------------------------------

CShapeStyle::CShapeStyle()
{
	fFrameFill = kFrameOnly;
	fPenSize.h = fPenSize.v = 1;
	fFillPattern = XMPASLMQDGlobals.black;
	fFramePattern = XMPASLMQDGlobals.black;
	fFillColor = kRGBWhite;
	fFrameColor = kRGBBlack;
}

//-------------------------------------------------------------------------
// CShapeStyle::CShapeStyle
//-------------------------------------------------------------------------

CShapeStyle::CShapeStyle(const CShapeStyle& otherStyle)
{
	fFrameFill = otherStyle.fFrameFill;
	fPenSize.h = otherStyle.fPenSize.h;
	fPenSize.v = otherStyle.fPenSize.v;
	fFillPattern = otherStyle.fFillPattern;
	fFramePattern = otherStyle.fFramePattern;
	fFillColor = otherStyle.fFillColor;
	fFrameColor = otherStyle.fFrameColor;
}

//-------------------------------------------------------------------------
// CShapeStyle::CShapeStyle
//-------------------------------------------------------------------------

CShapeStyle::CShapeStyle(XMPUShort frameFill, Point penSize, 
						 const Pattern& framePattern, const Pattern& fillPattern,
						 const RGBColor& frameColor, const RGBColor& fillColor)
{
	fFrameFill = frameFill;
	fPenSize = penSize;
	fFillPattern = fillPattern;
	fFramePattern = framePattern;
	fFillColor= fillColor;
	fFrameColor = frameColor;
}

//-------------------------------------------------------------------------
// CShapeStyle::CShapeStyle
//-------------------------------------------------------------------------

CShapeStyle::CShapeStyle(Point penSize, 
						 const Pattern& framePattern,
						 const RGBColor& frameColor)
{
	fFrameFill = kFrameOnly;
	fPenSize = penSize;
	fFramePattern = framePattern;
	fFrameColor = frameColor;
}

//-------------------------------------------------------------------------
// CShapeStyle::CShapeStyle
//-------------------------------------------------------------------------

CShapeStyle::CShapeStyle(const Pattern& fillPattern,
						 const RGBColor& fillColor)
{
	fFrameFill = kFillOnly;
	fFillPattern = fillPattern;
	fFillColor = fillColor;
}

//-------------------------------------------------------------------------
// CShapeStyle::~CShapeStyle
//-------------------------------------------------------------------------

CShapeStyle::~CShapeStyle()
{
}

//-------------------------------------------------------------------------
// CShapeStyle::SetGrafPortFrameStyle
//-------------------------------------------------------------------------

void CShapeStyle::SetGrafPortFrameStyle() const
{
	::PenSize(fPenSize.h, fPenSize.v);
	::PenPat(&fFramePattern);
	::RGBForeColor(&fFrameColor);
}

//-------------------------------------------------------------------------
// CShapeStyle::SetGrafPortFillStyle
//-------------------------------------------------------------------------

void CShapeStyle::SetGrafPortFillStyle() const
{
	::PenPat(&fFillPattern);
	::RGBForeColor(&fFillColor);
}

//-------------------------------------------------------------------------
// CShapeStyle::Externalize
//-------------------------------------------------------------------------

void CShapeStyle::Externalize(XMPStorageUnit* storage)
{
	storage->SetValue(sizeof(fFrameFill), (XMPValue)&fFrameFill);
	storage->SetValue(sizeof(fPenSize), (XMPValue)&fPenSize);
	storage->SetValue(sizeof(fFillPattern), (XMPValue)&fFillPattern);
	storage->SetValue(sizeof(fFramePattern), (XMPValue)&fFramePattern);
	storage->SetValue(sizeof(fFillColor), (XMPValue)&fFillColor);
	storage->SetValue(sizeof(fFrameColor), (XMPValue)&fFrameColor);
}

//-------------------------------------------------------------------------
// CShapeStyle::Internalize
//-------------------------------------------------------------------------

void CShapeStyle::Internalize(XMPStorageUnit* storage)
{
	storage->GetValue(sizeof(fFrameFill), (XMPValue)&fFrameFill);
	storage->GetValue(sizeof(fPenSize), (XMPValue)&fPenSize);
	storage->GetValue(sizeof(fFillPattern), (XMPValue)&fFillPattern);
	storage->GetValue(sizeof(fFramePattern), (XMPValue)&fFramePattern);
	storage->GetValue(sizeof(fFillColor), (XMPValue)&fFillColor);
	storage->GetValue(sizeof(fFrameColor), (XMPValue)&fFrameColor);
}

