/*
	File:		Style.h

	Contains:	Definition for class CShapeStyle

	Written by:	Henri Lamiraux

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.
*/

#ifndef _STYLE_
#define _STYLE_

#ifndef _ALTPOINT_			// Just to be sure to be included before PlfmType.h
#include "AltPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _PLFMTYPE_
#include <PlfmType.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

// ----- Macintosh Includes -----

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

//==============================================================================
// Forward Declarations
//==============================================================================

class XMPStorageUnit;

//==============================================================================
// Constants
//==============================================================================

const RGBColor kRGBWhite = {0xFFFF, 0xFFFF, 0xFFFF};
const RGBColor kRGBBlack = {0x0000, 0x0000, 0x0000};
const RGBColor kRGBRed = {0xFFFF, 0x0000, 0x0000};
const RGBColor kRGBGreen = {0x0000, 0xFFFF, 0x0000};
const RGBColor kRGBBlue = {0x0000, 0x0000, 0xFFFF};

const XMPUShort kFrameOnly = 1;
const XMPUShort kFillOnly = 2;
const XMPUShort kFrameFill = 3;

//==============================================================================
// class CShapeStyle
//==============================================================================

class CShapeStyle
{		
public:
	CShapeStyle();
	CShapeStyle(const CShapeStyle& otherStyle);
	CShapeStyle(XMPUShort frameFill,
				Point penSize, 
				const Pattern& framePattern, 
				const Pattern& fillPattern,
				const RGBColor& frameColor,
				const RGBColor& fillColor);
	CShapeStyle(Point penSize, 
				const Pattern& framePattern, 
				const RGBColor& frameColor);
	
	CShapeStyle(const Pattern& fillPattern, 
				const RGBColor& fillColor);
		
	~CShapeStyle();
	
	XMPBoolean		HasFillStyle() const
						{return (fFrameFill & kFillOnly) != 0;}
	XMPBoolean		HasFrameStyle() const
						{return (fFrameFill & kFrameOnly) != 0;}
	XMPBoolean		HasFrameAndFillStyle() const
						{return (fFrameFill == kFrameFill);}
	void			SetFrameFillStyle(XMPUShort style)
						{fFrameFill = style;}
	XMPUShort		GetFrameFillStyle() const
						{return fFrameFill;}
	
	void			GetPenSize(Point &penSize) const
						{penSize = fPenSize;}
	short			GetPenWidth() const
						{return fPenSize.h;}
	short			GetPenHeight() const
						{return fPenSize.v;}
						
	void			SetPenSize(const Point &penSize)
						{fPenSize = penSize;}
	
	void			GetFillPattern(Pattern& fillPattern) const
						{fillPattern = fFillPattern;}
	void			SetFillPattern(const Pattern& fillPattern)
						{fFillPattern = fillPattern;}
						
	void			GetFramePattern(Pattern& framePattern) const
						{framePattern = fFramePattern;}
	void			SetFramePattern(const Pattern& framePattern)
						{fFramePattern = framePattern;}
	
	void			GetFillColor(RGBColor& fillColor) const
						{fillColor = fFillColor;}
	void			SetFillColor(const RGBColor& fillColor)
						{fFillColor = fillColor;}
						
	void			GetFrameColor(RGBColor& frameColor) const
						{frameColor = fFrameColor;}
	void			SetFrameColor(const RGBColor& frameColor)
						{fFrameColor = frameColor;}
	
	void			SetGrafPortFrameStyle() const;
	void			SetGrafPortFillStyle() const;
	
	// ----- Persistence -----
	void			Externalize(XMPStorageUnit* storage);
	void			Internalize(XMPStorageUnit* storage);

private:
	XMPUShort		fFrameFill;
	Point			fPenSize;
	Pattern			fFillPattern;
	Pattern			fFramePattern;
	RGBColor		fFillColor;
	RGBColor		fFrameColor;
};


#endif