/*
	File:		MovieFac.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 4/13/94	DS		Remove AdjustMovieBox call. Don't know why.
		 <2>	 4/12/94	DS		Replace ExternalTransformChanged and
									ClipShapeChanged with GeometryChanged

	To Do:
*/

//	File:		CMovieFac.cp
//
//	Contains:	implementation of class CMovieFacet
//
//	Written by:	Lonnie Millett
//
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.

#ifndef _MOVIEFAC_
#include "MovieFac.h"
#endif

#ifndef _MOVIEFRA_
#include "MovieFra.h"
#endif

#ifndef _MOVIEPAR_
#include "MoviePar.h"
#endif

#ifndef _MOVIESEL_
#include "MovieSel.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _TRNSFRMM_
#include <TrnsfrmM.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _DGITMITR_
#include <DgItmItr.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#pragma segment MoviePart

//----------------------------------------------------------------------------------------
// CMovieFacet::CMovieFacet
//----------------------------------------------------------------------------------------
CMovieFacet::CMovieFacet()
{
	fMovie = kXMPNULL;
	fMovieController = kXMPNULL;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::~CClockFrame
//----------------------------------------------------------------------------------------
CMovieFacet::~CMovieFacet()
{
	if (fMovieController)
	{
		DisposeMovieController(fMovieController);
		fMovieController = kXMPNULL;
	}
	
	if (fMovie)
	{
		DisposeMovie(fMovie);
		fMovie = kXMPNULL;
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::InitMovieFacet
//----------------------------------------------------------------------------------------
void CMovieFacet::InitMovieFacet(XMPFacet* xmpFacet, CMovieFrame* movieFrame)
{
	this->InitFacet(xmpFacet);
	fMovieFrame = movieFrame;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::GetMovieBoundingBox
//----------------------------------------------------------------------------------------
void CMovieFacet::GetMovieBoundingBox(XMPRect* boundingBox)
{
	if (fMovie)
	{
		Rect movieBox;
		::GetMovieBox(fMovie, &movieBox);
		
		if (this->GetMovieFrame()->IsActive())
			movieBox.bottom += 16;
		*boundingBox = movieBox;
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::SetMovie
//----------------------------------------------------------------------------------------
void CMovieFacet::SetMovie(Movie newMovie)
{
	if (newMovie)
	{
		if (fMovieController)
		{
			DisposeMovieController(fMovieController);
			fMovieController = kXMPNULL;
		}
		
		if (fMovie)
		{
			DisposeMovie(fMovie);
			fMovie = kXMPNULL;
		}
			
		fMovie = newMovie;
		
		::SetMovieGWorld(fMovie,(CGrafPtr)this->GetXMPWindow()->GetPlatformWindow(), kXMPNULL);
		
		Rect movieBox;
		::GetMovieBox(fMovie, &movieBox);
		if (::EmptyRect(&movieBox))
			::SetRect(&movieBox, 0, 0, 100, 100);
		fMovieController = NewMovieController(fMovie, &movieBox, mcTopLeftMovie+mcWithBadge+mcNotVisible);
		THROW_IF_NULL(fMovieController);
		::MCDoAction(fMovieController, mcActionSetKeysEnabled, (Ptr)kXMPTrue);
		::MCDoAction(fMovieController, mcActionSetUseBadge, (Ptr)kXMPTrue);
		::MCEnableEditing(fMovieController, kXMPTrue);
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::AdjustMovieBox
//----------------------------------------------------------------------------------------
void CMovieFacet::AdjustMovieBox()
{
	if (fMovie)
	{
		XMPRect boundingBox;
		Rect movieBox;
		
		fMovieFrame->GetFrameShapeBounds(&boundingBox);
		boundingBox.AsQDRect(movieBox);
		if (fMovieFrame->IsRoot())
			movieBox.bottom -= 16;
			
		::OffsetRect(&movieBox, -movieBox.left, -movieBox.top);
		::SetMovieBox(fMovie, &movieBox);
		THROW_IF_ERROR(::MCMovieChanged(fMovieController, fMovie), "::MCMovieChanged");
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::StartMovie
//----------------------------------------------------------------------------------------
void CMovieFacet::StartMovie()
{
	if (fMovieController)
	{
		Fixed playRate = ::GetMoviePreferredRate(fMovie);
		::MCDoAction(fMovieController, mcActionPlay, (Ptr)playRate);
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::StopMovie
//----------------------------------------------------------------------------------------
void CMovieFacet::StopMovie()
{
	if (fMovieController)
		::MCDoAction(fMovieController, mcActionPlay, (Ptr)0);
}

//----------------------------------------------------------------------------------------
// CMovieFacet::SetMovieLooping
//----------------------------------------------------------------------------------------
void CMovieFacet::SetMovieLooping(XMPBoolean loop)
{
	if (fMovieController)
		::MCDoAction(fMovieController, mcActionSetLooping, (Ptr)loop);
}

//----------------------------------------------------------------------------------------
// CMovieFacet::Draw
//----------------------------------------------------------------------------------------
void CMovieFacet::Draw(XMPShape* invalidShape)
{
	XMPUnused(invalidShape);

	if (fMovieController != kXMPNULL)
		::MCDraw(fMovieController, this->GetXMPFacet()->GetWindow()->GetPlatformWindow());
}

//----------------------------------------------------------------------------------------
// CMovieFacet::GeometryChanged
//----------------------------------------------------------------------------------------
void CMovieFacet::GeometryChanged()
{
	// Was in ClipShapeChanged
	if (fMovieController)		
	{
		RgnHandle movieClipRegion;
		RgnHandle clipRegion;
		
		clipRegion = NewRgn();
		CopyRgn(this->GetXMPFacet()->GetClipShape()->GetQDRegion(), clipRegion);
		
		movieClipRegion = GetMovieDisplayBoundsRgn(fMovie);
		SectRgn(clipRegion, movieClipRegion, movieClipRegion);
		
		THROW_IF_ERROR(::MCSetClip(fMovieController, clipRegion, movieClipRegion), "::MCSetClip");
		
		DisposeRgn(clipRegion);
		DisposeRgn(movieClipRegion);
		
	// Was in externalTransformChanged
	// this->AdjustMovieBox();	
	}
}

//------------------------------------------------------------------------------
// CMovieFacet::WindowActivated
//------------------------------------------------------------------------------

void CMovieFacet::WindowActivated()
{
	this->DoMovieActivate(kXMPTrue);
}

//------------------------------------------------------------------------------
// CMovieFacet::WindowDeActivated
//------------------------------------------------------------------------------

void CMovieFacet::WindowDeActivated()
{
	this->DoMovieActivate(kXMPFalse);
}

//----------------------------------------------------------------------------------------
// CMovieFacet::DoMouseDown
//----------------------------------------------------------------------------------------
XMPBoolean CMovieFacet::DoMouseDown(const XMPPoint& where, XMPEventData event)
{
	XMPBoolean eventHandled = kXMPFalse;
	
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		Point originalWhere;
		::SetPt(&originalWhere, event->where.h, event->where.v);
		
		XMPPoint offset(0,0);
		offset = this->GetContentTransform()->TransformPoint(offset);
		::SetOrigin(-(offset.IntX()), -(offset.IntY()));
		
		event->where.h = where.IntX(); event->where.v = where.IntY();
		eventHandled = (XMPBoolean)::MCClick(fMovieController, this->GetXMPFacet()->GetWindow()->GetPlatformWindow(),
										event->where, event->when, event->modifiers);
		
		::SetPt(&event->where, originalWhere.h, originalWhere.v);
										
		this->GetFrame()->GetPart()->GetSelection()->Drag(this, event);
	}
	
	return eventHandled;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::DoMovieKey
//----------------------------------------------------------------------------------------
XMPBoolean CMovieFacet::DoMovieKey(XMPEventData event)
{
	XMPBoolean eventHandled = kXMPFalse;
	
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		eventHandled = (XMPBoolean)::MCKey(fMovieController, (char)event->message, event->modifiers);
	}
	
	return eventHandled;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::DoMovieSetVisible
//----------------------------------------------------------------------------------------
void CMovieFacet::DoMovieSetVisible(XMPBoolean visible)
{
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		THROW_IF_ERROR(::MCSetVisible(fMovieController, visible), "::MCSetVisible");
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::DoMovieIdle
//----------------------------------------------------------------------------------------
XMPBoolean CMovieFacet::DoMovieIdle()
{
	XMPBoolean eventHandled = kXMPFalse;
	
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		eventHandled = (XMPBoolean)::MCIdle(fMovieController);
	}
	
	return eventHandled;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::DoMovieActivate
//----------------------------------------------------------------------------------------
void CMovieFacet::DoMovieActivate(XMPBoolean activate)
{
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		::MCActivate(fMovieController, this->GetXMPWindow()->GetPlatformWindow(), activate);
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::IsControllerVisible
//----------------------------------------------------------------------------------------
XMPBoolean CMovieFacet::IsControllerVisible()
{
	if (fMovieController)
		return ::MCGetVisible(fMovieController);
	else
		return kXMPFalse;
}

//------------------------------------------------------------------------------
// CMovieFacet::GetCurrentSelection
//------------------------------------------------------------------------------
void CMovieFacet::GetCurrentSelection(TimeValue* currentTime, TimeValue* duration)
{
	if (fMovie)
		::GetMovieSelection(fMovie, currentTime, duration);
}

//------------------------------------------------------------------------------
// CMovieFacet::SetCurrentSelection
//------------------------------------------------------------------------------
void CMovieFacet::SetCurrentSelection(TimeValue currentTime, TimeValue duration)
{
	if (fMovie)
	{
		CDrawInitiator di(this);
		
		::SetMovieSelection(fMovie, currentTime, duration);
		THROW_IF_ERROR(::MCMovieChanged(fMovieController, fMovie), "::MCMovieChanged");
	}
}

//------------------------------------------------------------------------------
// CMovieFacet::IsEmpty
//------------------------------------------------------------------------------
XMPBoolean CMovieFacet::IsEmpty()
{
	XMPBoolean empty = kXMPTrue;
	if (fMovie)
	{
		TimeValue currentTime, duration;
		
		::GetMovieSelection(fMovie, &currentTime, &duration);
		empty = duration == (TimeValue)0;
	}
	return empty;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::CloseSelection
//----------------------------------------------------------------------------------------
void CMovieFacet::CloseSelection()
{
	this->SetCurrentSelection(0, 0);
}

//------------------------------------------------------------------------------
// CMovieFacet::ClearSelection
//------------------------------------------------------------------------------
void CMovieFacet::ClearSelection(TimeValue currentTime, TimeValue duration)
{
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		this->SetCurrentSelection(currentTime, duration);
		THROW_IF_ERROR(::MCClear(fMovieController));
	}
}

//------------------------------------------------------------------------------
// CMovieFacet::SelectAll
//------------------------------------------------------------------------------
void CMovieFacet::SelectAll()
{
	if (fMovie)
	{
		CDrawInitiator di(this);
		
		this->SetCurrentSelection((TimeValue)0, ::GetMovieDuration(fMovie));
	}
}

//------------------------------------------------------------------------------
// CMovieFacet::CopySelection
//------------------------------------------------------------------------------
Handle CMovieFacet::CopySelection()
{
	Handle movieHandle = kXMPNULL;
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		Movie newMovie = ::MCCopy(fMovieController);
		movieHandle = XMPNewHandle(0);
		THROW_IF_ERROR(::PutMovieIntoHandle(newMovie, movieHandle));
		DisposeMovie(newMovie);
	}
	return movieHandle;
}

//------------------------------------------------------------------------------
// CMovieFacet::DragEnter
//------------------------------------------------------------------------------
void CMovieFacet::DragEnter(XMPDragItemIterator* dragInfo, XMPPoint where)
{
	CFacet::DragEnter(dragInfo, where);
	
	if (!this->GetCanAcceptDrop())
	{
		XMPStorageUnit *dragSU;
		
		for (dragSU = dragInfo->First(); dragSU; dragSU = dragInfo->Next())
			if (dragSU->Exists(kXMPPropContents, kXMPApplePICT, 0) ||
					dragSU->Exists(kXMPPropContents, kXMPAppleTEXT, 0) ||
					dragSU->Exists(kXMPPropContents, kXMPApplesnd, 0))
				this->SetCanAcceptDrop(kXMPTrue);
	}
}

//------------------------------------------------------------------------------
// CMovieFacet::DrawDragHiliteInside
//------------------------------------------------------------------------------
XMPBoolean CMovieFacet::DrawDragHiliteInside()
{
	if (this->GetFrame()->IsRoot())
		return kXMPTrue;
	else
		return kXMPFalse;
}

//------------------------------------------------------------------------------
// CMovieFacet::GetMovieAsPict
//------------------------------------------------------------------------------
PicHandle CMovieFacet::GetMovieAsPict()
{
	PicHandle moviePict = kXMPNULL;
	if (fMovie)
		moviePict = ::GetMoviePict(fMovie, ::GetMovieTime(fMovie, kXMPNULL));
	return moviePict;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::PasteMovieToSelection
//----------------------------------------------------------------------------------------
void CMovieFacet::PasteMovieToSelection(Movie newMovie, TimeValue currentTime, TimeValue duration)
{
	if (fMovieController)
	{
		CDrawInitiator di(this);
		
		this->SetCurrentSelection(currentTime, duration);
		THROW_IF_ERROR(::MCPaste(fMovieController, newMovie));
	}
}

//----------------------------------------------------------------------------------------
// CMovieFacet::PasteHandleToSelection
//----------------------------------------------------------------------------------------
void CMovieFacet::PasteHandleToSelection(Handle newHandle, OSType handleType, TimeValue currentTime, TimeValue duration)
{
	if (fMovie)
	{
		CDrawInitiator di(this);
		
		this->SetCurrentSelection(currentTime, duration);
		THROW_IF_ERROR(::PasteHandleIntoMovie(newHandle, handleType, fMovie, 0, (ComponentInstance)0));
		THROW_IF_ERROR(::MCMovieChanged(fMovieController, fMovie), "::MCMovieChanged");
	}
}

