/*
	File:		MovieFra.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 4/12/94	DS		Change call to ClipShapeChanged with call
									to GeometryChanged
		 <2>	 3/31/94	DS		Fix include of XMPSession

	To Do:
*/

//	File:		MovieFra.cp
//
//	Contains:	implementation of class CMovieFrame
//
//	Written by:	Lonnie Millett
//
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.

#ifndef _MOVIEFRA_
#include "MovieFra.h"
#endif

#ifndef _MOVIEFAC_
#include "MovieFac.h"
#endif

#ifndef _MOVIEPAR_
#include "MoviePar.h"
#endif

// ----- Framework Includes -----
#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __MOVIES__
#include <Movies.h>
#endif

#ifndef mathRoutinesIncludes
#include <Math Routines.h>
#endif

#pragma segment MoviePart

//==============================================================================
// Constants
//==============================================================================

const XMPID kCMovieFrameID = 'movi';

//----------------------------------------------------------------------------------------
// CMovieFrame::CMovieFrame
//----------------------------------------------------------------------------------------
CMovieFrame::CMovieFrame()
					  : CFrame()
{
	fMoviePart = kXMPNULL;
}

//----------------------------------------------------------------------------------------
// CMovieFrame::~CMovieFrame
//----------------------------------------------------------------------------------------
CMovieFrame::~CMovieFrame()
{
}

//----------------------------------------------------------------------------------------
// CMovieFrame::InitMovieFrame
//----------------------------------------------------------------------------------------
void CMovieFrame::InitMovieFrame(XMPFrame* xmpFrame, CMoviePart* moviePart)
{
	this->IFrame(xmpFrame, moviePart);
	
	fMoviePart = moviePart;
	
	AddToFocusSet(fMoviePart->GetKeyFocusToken());
	AddToFocusSet(fMoviePart->GetMenuFocusToken());
	AddToFocusSet(fMoviePart->GetSelectionFocusToken());

	this->SetDroppable(kXMPTrue);
}

//----------------------------------------------------------------------------------------
// CMovieFrame::NewFacet
//----------------------------------------------------------------------------------------
CFacet* CMovieFrame::NewFacet(XMPFacet* xmpFacet)
{
	CMovieFacet* facet = new CMovieFacet;
	facet->InitMovieFacet(xmpFacet, this);

	return facet;
}

//----------------------------------------------------------------------------------------
// CMovieFrame::FacetAdded
//----------------------------------------------------------------------------------------
void CMovieFrame::FacetAdded(CFacet* facet)
{
	CFrame::FacetAdded(facet);
	
	Movie newMovie;
	
	// Determine from where we should get our movie
	
	// Get any existing movie and use it since all movies are supposed to be in-synch
	Movie currentMovie = ((CMovieFacet*)fMoviePart->GetAnyExistingFacet())->GetMovie();
	if (currentMovie)
	{
		XMPBoolean dataRefChanged;
		Handle movieHandle = XMPNewHandle(0);
		
		THROW_IF_ERROR(::PutMovieIntoHandle(currentMovie, movieHandle));
		THROW_IF_ERROR(::NewMovieFromHandle(&newMovie, movieHandle, newMovieActive, &dataRefChanged));
		
		DisposeHandle(movieHandle);
	}
	// Otherwise look at the part to see if we have internalized a movie from storage
	else if (fMoviePart->GetMovieHandle())
	{
		newMovie = fMoviePart->GetNewMovieFromMovieHandle();
	}
	// Looks like we're just starting out.  Ask the user to choose a movie
	else
	{
		AliasHandle movieAlias = fMoviePart->GetMovieFileFromUser();
		if (movieAlias)
			newMovie = fMoviePart->GetMovieFromAlias(movieAlias);
		else		// if no movie was chosen than make a new empty movie 
			newMovie = ::NewMovie(newMovieActive);
	}
	
	((CMovieFacet*)facet)->SetMovie(newMovie);
	((CMovieFacet*)facet)->SetMovieLooping(fMoviePart->GetMovieLooping());
	
	XMPRect movieBox;
	
	if (this->IsRoot())
		this->GetFrameShapeBounds(&movieBox);
	else
		((CMovieFacet*)facet)->GetMovieBoundingBox(&movieBox);
		
	this->AdjustFrameSize(&movieBox);
	
	if (this->IsRoot())
	{
		((CMovieFacet*)facet)->AdjustMovieBox();
		((CMovieFacet*)facet)->DoMovieSetVisible(kXMPTrue);
	}
	
	this->UpdateUsedAndActiveShapes();
}

//----------------------------------------------------------------------------------------
// CMovieFrame::CreateChildWindow
//----------------------------------------------------------------------------------------
XMPWindow* CMovieFrame::CreateChildWindow(XMPISOStr windowTitle)
{
	XMPRect windRect;
	CMovieFacet* movieFacet = (CMovieFacet*)this->GetActiveFacet();
	movieFacet->GetMovieBoundingBox(&windRect);
	windRect.Offset(ff(100), ff(100));

	return GetPart()->CreateXMPWindow(this,
									 GetPart()->GetSession()->Tokenize(kXMPViewAsFrame),
									 GetPresentation(),
									 windowTitle,
									 windRect,
									 kXMPTrue,		// close box
									 kXMPFalse,		// resizable
									 kXMPFalse,		// floating
									 kXMPTrue,		// should save
									 zoomNoGrow);
}

//----------------------------------------------------------------------------------------
// CMovieFrame::FrameShapeChanged
//----------------------------------------------------------------------------------------
void CMovieFrame::FrameShapeChanged()
{
	this->UpdateUsedAndActiveShapes();

	CFrameFacetIterator facets(this);
	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->AdjustMovieBox();
	
	XMPShape oldShape;
	this->GetFrameShape(&oldShape);
	this->Invalidate(&oldShape);
	this->GetXMPFrame()->InvalidateActiveBorder();
}

//----------------------------------------------------------------------------------------
// CMovieFrame::FocusStateChanged
//----------------------------------------------------------------------------------------
void CMovieFrame::FocusStateChanged(XMPTypeToken focus, XMPBoolean newState)
{
	CFrame::FocusStateChanged(focus, newState);

	if (focus == this->GetPart()->GetSelectionFocusToken() && !this->IsRoot())
	{
		XMPRect movieBox;
		((CMovieFacet*)this->GetActiveFacet())->GetMovieBoundingBox(&movieBox);
		this->AdjustFrameSize(&movieBox);
		
		// This doesn't quite make sense to me. Should we change the geometry via
		// an OD call and have it notifiy us that the geometry has changed? ????
		((CMovieFacet*)this->GetActiveFacet())->GeometryChanged();
		((CMovieFacet*)this->GetActiveFacet())->DoMovieSetVisible(newState);
	}
}

//------------------------------------------------------------------------------
// CMovieFrame::HandleKeyDown
//------------------------------------------------------------------------------
XMPBoolean CMovieFrame::HandleKeyDown(XMPEventData event)
{
	return ((CMovieFacet*)this->GetActiveFacet())->DoMovieKey(event);
}

//------------------------------------------------------------------------------
// CMovieFrame::DoMenuEvent
//------------------------------------------------------------------------------
XMPBoolean CMovieFrame::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	XMPBoolean menuHandled = kXMPFalse;
	CFrameFacetIterator facets(this);
	CMovieFacet* movieFacet;
	
	switch (commandID)
	{
		case cStartMovie:
				for (movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
					movieFacet->StartMovie();
				
				menuHandled = kXMPTrue;
				break;
				
		case cStopMovie:
				for (movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
					movieFacet->StopMovie();
				
				menuHandled = kXMPTrue;
				break;
				
		default:
			menuHandled = CFrame::DoMenuEvent(menuBar, commandID);
	}			
	
	return menuHandled;
}

//----------------------------------------------------------------------------------------
// CMovieFrame::AdjustFrameSize
//----------------------------------------------------------------------------------------
void CMovieFrame::AdjustFrameSize(XMPRect* movieBox)
{
	XMPShape* newFrameShape = new XMPShape;
	newFrameShape->SetRectangle(movieBox);
	
	this->GetXMPFrame()->RequestFrameShape(newFrameShape);
	this->UpdateUsedAndActiveShapes();
	
	XMPShape oldShape;
	this->GetFrameShape(&oldShape);
	this->Invalidate(&oldShape);
	this->GetXMPFrame()->InvalidateActiveBorder();
}

//----------------------------------------------------------------------------------------
// CMovieFrame::UpdateMovie
//----------------------------------------------------------------------------------------
void CMovieFrame::UpdateMovie()
{
	CFrameFacetIterator facets(this);
	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->DoMovieIdle();
}

//----------------------------------------------------------------------------------------
// CMovieFrame::SetNewMovie
//----------------------------------------------------------------------------------------
void CMovieFrame::SetNewMovie(Movie newMovie)
{
	XMPRect movieBox;
	CFrameFacetIterator facets(this);
	CMovieFacet* movieFacet = (CMovieFacet*)facets.First();
	
	// First the first facet just use the movie we were passed
	movieFacet->SetMovie(newMovie);
	movieFacet->GetMovieBoundingBox(&movieBox);
	this->AdjustFrameSize(&movieBox);
	
	if (this->IsRoot())
	{
		Rect windowRect;
		XMPPlatformWindow platformWindow;
		
		movieBox.AsQDRect(windowRect);
		// AMB - It may be a bad assumption that there is an active facet at this point.
		platformWindow = this->GetActiveFacet()->GetXMPWindow()->GetPlatformWindow();
		SizeWindow((WindowPtr)platformWindow, windowRect.right, windowRect.bottom, kXMPTrue);
		this->GetActiveFacet()->GetXMPWindow()->Resized();
	}

	if (this->IsActive() || this->IsRoot())
		movieFacet->DoMovieSetVisible(kXMPTrue);
		
//	if (this->IsRoot() && this->IsActive())
//		movieFacet->DoMovieActivate(kXMPTrue);
//	else
//		movieFacet->DoMovieActivate(kXMPFalse);

		
	// If there are more facets then create a handle to clone movies from
	Handle movieHandle = kXMPNULL;
	if (facets.IsNotComplete())
	{
		movieHandle = XMPNewHandle(0);
		THROW_IF_ERROR(::PutMovieIntoHandle(newMovie, movieHandle));
		
		for (movieFacet = (CMovieFacet*)facets.Next(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		{
			XMPBoolean dataRefChanged;
			THROW_IF_ERROR(::NewMovieFromHandle(&newMovie, movieHandle, newMovieActive, &dataRefChanged));
			movieFacet->SetMovie(newMovie);
			
			if (this->IsActive() || this->IsRoot())
				movieFacet->DoMovieSetVisible(kXMPTrue);
				
	//		if (this->IsRoot() && this->IsActive())
	//			movieFacet->DoMovieActivate(kXMPTrue);
	//		else
	//			movieFacet->DoMovieActivate(kXMPFalse);
	
		}
	}
	
	if (movieHandle)
		DisposeHandle(movieHandle);
}

//----------------------------------------------------------------------------------------
// CMovieFrame::CloseSelection
//----------------------------------------------------------------------------------------
void CMovieFrame::CloseSelection()
{
	CFrameFacetIterator facets(this);
	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->CloseSelection();
}

//----------------------------------------------------------------------------------------
// CMovieFrame::ClearSelection
//----------------------------------------------------------------------------------------
void CMovieFrame::ClearSelection(TimeValue currentTime, TimeValue duration)
{
	CFrameFacetIterator facets(this);
	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->ClearSelection(currentTime, duration);
}

//----------------------------------------------------------------------------------------
// CMovieFrame::SelectAll
//----------------------------------------------------------------------------------------
void CMovieFrame::SelectAll()
{
	CFrameFacetIterator facets(this);
	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->SelectAll();
}

//----------------------------------------------------------------------------------------
// CMovieFrame::PasteMovieToSelection
//----------------------------------------------------------------------------------------
void CMovieFrame::PasteMovieToSelection(Movie newMovie, TimeValue currentTime, TimeValue duration)
{
	CFrameFacetIterator facets(this);
	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->PasteMovieToSelection(newMovie, currentTime, duration);
}

//----------------------------------------------------------------------------------------
// CMovieFrame::PasteHandleToSelection
//----------------------------------------------------------------------------------------
void CMovieFrame::PasteHandleToSelection(Handle newHandle, OSType handleType, TimeValue currentTime, TimeValue duration)
{
	CFrameFacetIterator facets(this);
	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->PasteHandleToSelection(newHandle, handleType, currentTime, duration);
}

//----------------------------------------------------------------------------------------
// CMovieFrame::ToggleLooping
//----------------------------------------------------------------------------------------
void CMovieFrame::ToggleLooping(XMPBoolean looping)
{
	CFrameFacetIterator facets(this);

	for (CMovieFacet* movieFacet = (CMovieFacet*)facets.First(); facets.IsNotComplete(); movieFacet = (CMovieFacet*)facets.Next())
		movieFacet->SetMovieLooping(looping);
}
