/*
	File:		MoviePar.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 4/19/94	DS		Remove AddContents Property fix
		 <3>	 4/10/94	DS		Set the content property during part
									initialization
		 <2>	 3/31/94	DS		Fix include of XMPSession

	To Do:
*/

//	File:		MoviePar.cp
//
//	Contains:	implementation of class CMoviePart
//
//	Written by:	Lonnie Millett
//
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.

#ifndef _MOVIEPAR_
#include "MoviePar.h"
#endif

#ifndef _MOVIEFRA_
#include "MovieFra.h"
#endif

#ifndef _MOVIEFAC_
#include "MovieFac.h"
#endif

#ifndef _MOVIESEL_
#include "MovieSel.h"
#endif

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _WINSTAT_
#include <WinStat.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef __GESTALTEQU__
#include <GestaltEqu.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#pragma segment MoviePart

//==============================================================================
// Constants
//==============================================================================

const XMPSShort kChooseMovieItemID = 1;
const XMPSShort kStartMovieItemID = 3;
const XMPSShort kStopMovieItemID = 4;
const XMPSShort kSetMovieLoopingItemID = 6;

const XMPSShort kMovieIdleFreq = 1;

// Property for annotating our external movie information
const XMPPropertyName	kPropQTMovieLooping  = "Apple:OpenDoc:Quality:kQTMovieLooping";

//----------------------------------------------------------------------------------------
// CMoviePart::CMoviePart
//----------------------------------------------------------------------------------------
CMoviePart::CMoviePart()
{
	fQuickTimeInstalled = kXMPFalse;
	fMovieHandle = kXMPNULL;
	fLooping = kXMPFalse;
}

//----------------------------------------------------------------------------------------
// CMoviePart::~CMoviePart
//----------------------------------------------------------------------------------------
CMoviePart::~CMoviePart()
{
	if (fMovieHandle)
	{
		DisposeHandle(fMovieHandle);
		fMovieHandle = kXMPNULL;
	}
		
	if (fQuickTimeInstalled)
		ExitMovies();
}

//----------------------------------------------------------------------------------------
// CMoviePart::Release
//----------------------------------------------------------------------------------------
void CMoviePart::Release()
{
	CPart::Release();
	
	if (this->GetRefCount() == 1)
	{
		this->GetSession()->GetDispatcher()->UnregisterIdle(this, kXMPNULL);
	}
}

//----------------------------------------------------------------------------------------
// CMoviePart::Initialize
//----------------------------------------------------------------------------------------
void CMoviePart::Initialize()
{
	CPart:: Initialize();
	
	XMPSShort error;
	XMPSLong	result;
	
	XMPSession* session = this->GetSession();
	
	error = Gestalt(gestaltQuickTime, &result);
	fQuickTimeInstalled = (error == noErr);
	
	if (fQuickTimeInstalled)
		THROW_IF_ERROR(EnterMovies());
	else
		THROW_IF_ERROR(envNotPresent, "MoviePart:Init:Quicktime not available");   

	FW_CAcquireASLMResourceAccess aq;
	
	XMPMenuBar *menuBar = this->GetMenuBar();
	menuBar->AddMenuLast(kMovieMenu, GetMenu(kMovieMenu), this);

	menuBar->RegisterCommand(cChooseMovie, kMovieMenu, kChooseMovieItemID);
	menuBar->RegisterCommand(cStartMovie, kMovieMenu, kStartMovieItemID);
	menuBar->RegisterCommand(cStopMovie, kMovieMenu, kStopMovieItemID);
	menuBar->RegisterCommand(cSetMovieLooping, kMovieMenu, kSetMovieLoopingItemID);
	
	this->GetSession()->GetDispatcher()->RegisterIdle(this, kXMPNULL, kMovieIdleFreq);
}

//------------------------------------------------------------------------------
// CMoviePart::InternalizeContent
//------------------------------------------------------------------------------
void CMoviePart::InternalizeContent(XMPStorageUnit* storageUnit)
{
	XMPULong movieSize;
	Handle movieHandle = XMPNewHandle(0);
	
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);
	
	movieSize = storageUnit->GetSize();
	SetHandleSize((Handle)movieHandle, movieSize);
	HLock((Handle)movieHandle);
	storageUnit->GetValue(movieSize, (XMPValue)*movieHandle);
	HUnlock((Handle)movieHandle);
	this->SetMovieHandle(movieHandle);

	storageUnit->Focus(kContentAnnotationProp, kXMPPosUndefined, kContentAnnotationValue, (XMPValueIndex)0, kXMPPosFirstSib);
	storageUnit->GetValue(sizeof(XMPBoolean), (XMPValue)&fLooping);
}

//----------------------------------------------------------------------------------------
// CMoviePart::ExternalizeContent
//----------------------------------------------------------------------------------------
void CMoviePart::ExternalizeContent(XMPStorageUnit* storageUnit)
{
	CMovieFacet* facet = (CMovieFacet*)this->GetActiveFacet();
	if (facet)
	{
		Handle movieHandle = XMPNewHandle(0);
		THROW_IF_ERROR(::PutMovieIntoHandle(facet->GetMovie(), movieHandle));
		
		HLock((Handle)movieHandle);
		storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, this->GetContentPropertyValueType(), 0, kXMPPosUndefined);
		storageUnit->DeleteValue(storageUnit->GetSize());
		storageUnit->SetValue(::GetHandleSize((Handle)movieHandle), (XMPValue)*movieHandle);
		HUnlock((Handle)movieHandle);
		DisposeHandle(movieHandle);
		
		storageUnit->Focus(kContentAnnotationProp, kXMPPosUndefined, kContentAnnotationValue, (XMPValueIndex)0, kXMPPosFirstSib);
		storageUnit->SetValue(sizeof(XMPBoolean), (XMPValue)&fLooping);
	}
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetContentPropertyValueType
//----------------------------------------------------------------------------------------
void CMoviePart::AddProperties(XMPStorageUnit* storageUnit)
{
	storageUnit->AddProperty(kContentAnnotationProp)->AddValue(kContentAnnotationValue);
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetContentPropertyValueType
//----------------------------------------------------------------------------------------
XMPValueType CMoviePart::GetContentPropertyValueType() const
{
	// Our content type is the standard type for movies
	return kSampleMovieKind;
}

//----------------------------------------------------------------------------------------
// CMoviePart::SetMovieHandle
//----------------------------------------------------------------------------------------
void CMoviePart::SetMovieHandle(Handle movieHandle)
{
	if (fMovieHandle)
		::DisposeHandle(fMovieHandle);
		
	fMovieHandle = movieHandle;
}

//----------------------------------------------------------------------------------------
// CMoviePart::SetMovieLooping
//----------------------------------------------------------------------------------------
void CMoviePart::SetMovieLooping(XMPBoolean loop)
{
	fLooping = loop;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetNewMovieFromMovieHandle
//----------------------------------------------------------------------------------------
Movie CMoviePart::GetNewMovieFromMovieHandle()
{
	Movie newMovie;
	XMPBoolean dataRefChanged;
	
	THROW_IF_ERROR(::NewMovieFromHandle(&newMovie, fMovieHandle, newMovieActive, &dataRefChanged));
	
	return newMovie;
}

//----------------------------------------------------------------------------------------
// CMoviePart::NewFrame
//----------------------------------------------------------------------------------------
CFrame* CMoviePart::NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation)
{
	XMPUnused(presentation);
	
	CMovieFrame *frame = new CMovieFrame();
	frame->InitMovieFrame(xmpFrame, this);
	return frame;
}

//----------------------------------------------------------------------------------------
// CMoviePart::NewSelection
//----------------------------------------------------------------------------------------
FW_CSelection* CMoviePart::NewSelection()
{
	CMovieSelection* movieSelection = new CMovieSelection;
	movieSelection->InitMovieSelection(this);
	return movieSelection;
}

//------------------------------------------------------------------------------
// CMoviePart::DoMenuEvent
//------------------------------------------------------------------------------

XMPBoolean CMoviePart::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	XMPUnused(menuBar);
	XMPBoolean menuHandled = kXMPFalse;
	
	switch (commandID)
	{
		case cChooseMovie:
			{
				AliasHandle movieAlias = this->GetMovieFileFromUser();
				if (movieAlias)
				{
					CPartDisplayFrameIterator iter(this, this->GetDefaultPresentation());
					for (CMovieFrame* frame = (CMovieFrame*)iter.First(); iter.IsNotComplete(); frame = (CMovieFrame*)iter.Next())
					{
						Movie newMovie = this->GetMovieFromAlias(movieAlias);
						frame->SetNewMovie(newMovie);
					}
				}
			}
			menuHandled = kXMPTrue;
			break;
				
		case cSetMovieLooping:
				this->ToggleLooping();
				menuHandled = kXMPTrue;
				break;
				
		default:
			menuHandled = CPart::DoMenuEvent(menuBar, commandID);
	}			
	
	return menuHandled;
}

//----------------------------------------------------------------------------------------
// CMoviePart::DoAdjustMenus
//----------------------------------------------------------------------------------------
void CMoviePart::DoAdjustMenus(XMPMenuBar* menuBar)
{
	CPart::DoAdjustMenus(menuBar);
	
	menuBar->EnableCommand(kXMPCommandSelectAll, kXMPTrue);
	menuBar->EnableCommand(kXMPCommandPaste, kXMPTrue);
	menuBar->CheckCommand(cSetMovieLooping, fLooping);
}

//----------------------------------------------------------------------------------------
// CMoviePart::DoIdle
//----------------------------------------------------------------------------------------
XMPBoolean CMoviePart::DoIdle()
{
	// Update all the display frames of the same type
	CPartDisplayFrameIterator iter(this, this->GetDefaultPresentation());
	for (CMovieFrame* movieFrame = (CMovieFrame*)iter.First(); iter.IsNotComplete(); movieFrame = (CMovieFrame*)iter.Next())
		movieFrame->UpdateMovie();

	return kXMPTrue;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetMovieFromUser
//----------------------------------------------------------------------------------------
AliasHandle CMoviePart::GetMovieFileFromUser()
{
	SFTypeList typeList;
	StandardFileReply reply;
	AliasHandle movieFileAlias = kXMPNULL;

	typeList[0] = MovieFileType;
	
	this->GetSession()->GetWindowState()->DeactivateFrontWindows();
	::StandardGetFilePreview ((FileFilterProcPtr)kXMPNULL, 1, typeList, &reply);
	this->GetSession()->GetWindowState()->ActivateFrontWindows();

	if (reply.sfGood == true)
		THROW_IF_ERROR(::NewAlias(kXMPNULL, &reply.sfFile, &movieFileAlias));
	return  movieFileAlias;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetMovieFromAlias
//----------------------------------------------------------------------------------------
Movie CMoviePart::GetMovieFromAlias(AliasHandle movieAlias)
{
	Movie aMovie;
	XMPSShort movieResFile;
	XMPSShort movieResID;
	Str255 movieName;
	XMPBoolean wasChanged;
	FSSpec movieFile;

	aMovie = kXMPNULL;
	THROW_IF_ERROR(::ResolveAlias(kXMPNULL, movieAlias, &movieFile, &wasChanged));
	THROW_IF_ERROR(::OpenMovieFile(&movieFile, &movieResFile, fsRdPerm));
	movieResID = 0;	// First movie found
	THROW_IF_ERROR(::NewMovieFromFile(&aMovie, movieResFile,
									&movieResID,
									movieName, newMovieActive,
									&wasChanged));
	THROW_IF_ERROR(::CloseMovieFile(movieResFile));
	return  aMovie;
}

//----------------------------------------------------------------------------------------
// CMoviePart::ToggleLooping
//----------------------------------------------------------------------------------------
void CMoviePart::ToggleLooping()
{
	fLooping = !fLooping;
	
	CPartDisplayFrameIterator iter(this, this->GetDefaultPresentation());
	for (CMovieFrame* frame = (CMovieFrame*)iter.First(); iter.IsNotComplete(); frame = (CMovieFrame*)iter.Next())
		frame->ToggleLooping(fLooping);
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetAnyExistingFacet
//----------------------------------------------------------------------------------------
CFacet* CMoviePart::GetAnyExistingFacet()
{
	CFacet* facet = kXMPNULL;
	CPartDisplayFrameIterator iter1(this, this->GetDefaultPresentation());
	for (CFrame* frame = iter1.First(); iter1.IsNotComplete(); frame = iter1.Next())
	{
		CFrameFacetIterator iter2(frame);		
		for (facet = iter2.First(); iter2.IsNotComplete(); facet = iter2.Next())
			return facet;
	}
	return facet;
}
