/*
	File:		MoviePar.h

	Contains:	Definition for class CMoviePart

	Written by:	Lonnie Millett

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<1+>	 3/17/94	DS		Add CDECL to MoviePart dtor

	To Do:
*/


#ifndef _MOVIEPAR_
#define _MOVIEPAR_

#ifndef _FWPART_
#include "FWPart.h"
#endif

#ifndef _MOVIEDEF_
#include "MovieDef.h"
#endif

#ifndef __MOVIES__
#include <Movies.h>
#endif

#ifndef __ALIASES__
#include <Aliases.h>
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class CMoviePart;

//==============================================================================
// Classes used by this interface
//==============================================================================

class CPart;
class CMovieFrame;
class CMovieFacet;
class XMPStorageUnit;
class XMPSession;
class XMPFrame;
class XMPFacet;
class FW_CSelection;

//==============================================================================
// Constants
//==============================================================================

const XMPCommandID cChooseMovie = 10100;
const XMPCommandID cStartMovie = 10101;
const XMPCommandID cStopMovie = 10102;
const XMPCommandID cSetMovieLooping = 10103;

//==============================================================================
// CClockPart
//==============================================================================

class CMoviePart : public CPart
{

public:
	CMoviePart();
	virtual ~ CDECL CMoviePart();

	XMPMethod void Release();

	virtual void Initialize();
	
	virtual void InternalizeContent(XMPStorageUnit* storageUnit);
	virtual void ExternalizeContent(XMPStorageUnit* storageUnit);
	
	virtual void AddProperties(XMPStorageUnit* storageUnit);
	virtual XMPValueType GetContentPropertyValueType() const;
	
	XMPBoolean IsQuickTimeInstalled();
	XMPHandle GetMovieHandle() const;
	void SetMovieHandle(Handle movieHandle);
	XMPBoolean GetMovieLooping() const;
	void SetMovieLooping(XMPBoolean loop);
	Movie GetNewMovieFromMovieHandle();
	
	virtual CFrame* NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	virtual FW_CSelection* NewSelection();


	virtual XMPBoolean DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	virtual XMPBoolean DoIdle();
	virtual void DoAdjustMenus(XMPMenuBar* menuBar);

	AliasHandle GetMovieFileFromUser();
	Movie GetMovieFromAlias(AliasHandle movieAlias);
	
	void ToggleLooping();
	
	CFacet* GetAnyExistingFacet();
	
private:
	XMPBoolean fQuickTimeInstalled;
	XMPHandle fMovieHandle;
	XMPBoolean fLooping;
};

//----------------------------------------------------------------------------------------
// CMoviePart::IsQuickTimeInstalled
//----------------------------------------------------------------------------------------
inline XMPBoolean CMoviePart::IsQuickTimeInstalled()
{
	return fQuickTimeInstalled;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetMovieHandle
//----------------------------------------------------------------------------------------
inline XMPHandle CMoviePart::GetMovieHandle() const
{
	return fMovieHandle;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetMovieLooping
//----------------------------------------------------------------------------------------
inline XMPBoolean CMoviePart::GetMovieLooping() const
{
	return fLooping;
}

#endif
