/*
	File:		MovieSel.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/31/94	DS		Fix include of XMPSession

	To Do:
*/

//	File:		MovieSel.cp
//
//	Contains:	implementation of class CMovieSelection
//
//	Written by:	Lonnie Millett
//
//	Copyright:	 1993-1994 by Apple Computer, Inc., all rights reserved.

#ifndef _MOVIESEL_
#include "MovieSel.h"
#endif

#ifndef _MOVIEPAR_
#include "MoviePar.h"
#endif

#ifndef _MOVIEFRA_
#include "MovieFra.h"
#endif

#ifndef _MOVIEFAC_
#include "MovieFac.h"
#endif

// ----- Framework Includes -----

#ifndef _FWPART_
#include "FWPart.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

// ----- Macintosh Includes -----

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef __MOVIES__
#include <Movies.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>				// for ff()
#endif

#pragma segment MoviePart

//==============================================================================
//	 class CMovieSelection
//==============================================================================

//---------------------------------------------------------------------------------------
//	 CMovieSelection::CMovieSelection
//---------------------------------------------------------------------------------------

CMovieSelection::CMovieSelection()
{
	fMoviePart = kXMPNULL;
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::InitMovieSelection
//---------------------------------------------------------------------------------------

void CMovieSelection::InitMovieSelection(CMoviePart* moviePart)
{
	InitSelection(moviePart, kXMPFalse, kXMPFalse);
	
	fMoviePart = moviePart;
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::~CMovieSelection
//---------------------------------------------------------------------------------------

CMovieSelection::~CMovieSelection()
{	
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::CloseSelection
//---------------------------------------------------------------------------------------

void CMovieSelection::CloseSelection()
{
		CPartDisplayFrameIterator iter(fMoviePart, fMoviePart->GetDefaultPresentation());
		for (CMovieFrame* frame = (CMovieFrame*)iter.First(); iter.IsNotComplete();
					frame = (CMovieFrame*)iter.Next())
			frame->CloseSelection();
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::DoClear
//---------------------------------------------------------------------------------------

XMPBoolean CMovieSelection::DoClear()
{
		TimeValue currentTime, duration;
		((CMovieFacet*)fMoviePart->GetActiveFacet())->GetCurrentSelection(&currentTime, &duration);
		
		CPartDisplayFrameIterator iter(fMoviePart, fMoviePart->GetDefaultPresentation());
		for (CMovieFrame* frame = (CMovieFrame*)iter.First(); iter.IsNotComplete();
					frame = (CMovieFrame*)iter.Next())
			frame->ClearSelection(currentTime, duration);
	return kXMPFalse;
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::SelectAll
//---------------------------------------------------------------------------------------

void CMovieSelection::SelectAll()
{
		CPartDisplayFrameIterator iter(fMoviePart, fMoviePart->GetDefaultPresentation());
		for (CMovieFrame* frame = (CMovieFrame*)iter.First(); iter.IsNotComplete();
					frame = (CMovieFrame*)iter.Next())
			frame->SelectAll();
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::IsEmpty
//---------------------------------------------------------------------------------------

XMPBoolean CMovieSelection::IsEmpty() const
{
	return ((CMovieFacet*)fMoviePart->GetActiveFacet())->IsEmpty();
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::ExternalizeSelection
//---------------------------------------------------------------------------------------

void CMovieSelection::ExternalizeSelection(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
	XMPUnused(commandFrame);
	XMPUnused(cloneKind);
	
	CMovieFacet* movieFacet = (CMovieFacet*)fMoviePart->GetActiveFacet();
	if (movieFacet)
	{
		// Our content type is the standard 'moov' type
		storageUnit->AddProperty(kXMPPropContents)->AddValue(fMoviePart->GetContentPropertyValueType());
		Handle movieHandle = movieFacet->CopySelection();
		HLock(movieHandle);
		storageUnit->SetValue(::GetHandleSize((Handle)movieHandle), (XMPValue)*movieHandle);
		HUnlock(movieHandle);
		DisposeHandle(movieHandle);
		
		// Add a PICT standard type for parts that don't understand movies
		storageUnit->AddProperty(kXMPPropContents)->AddValue(kXMPApplePICT);
		PicHandle pictHandle = ((CMovieFacet*)fMoviePart->GetActiveFacet())->GetMovieAsPict();
		::HLock((Handle)pictHandle);
		storageUnit->SetValue(::GetHandleSize((Handle)pictHandle), (XMPValue)*pictHandle);
		::HUnlock((Handle)pictHandle);
		::KillPicture(pictHandle);
	}
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::InternalizeSelection
//---------------------------------------------------------------------------------------

XMPBoolean CMovieSelection::InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	XMPUnused(cloneKind);

	XMPBoolean internalized = kXMPFalse;
	Handle movieHandle;
	
	if (storageUnit->Exists(kXMPPropContents, fMoviePart->GetContentPropertyValueType(), 0))
	{
		storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, fMoviePart->GetContentPropertyValueType(), 0, kXMPPosUndefined);
					   
		XMPULong movieSize;
		movieHandle = XMPNewHandle(0);
		
		movieSize = storageUnit->GetSize();
		SetHandleSize(movieHandle, movieSize);
		::HLock(movieHandle);
		storageUnit->GetValue(movieSize, (XMPValue)*movieHandle);
		::HUnlock(movieHandle);
		
		Movie newMovie;
		TimeValue currentTime, duration;
		XMPBoolean dataRefChanged;
		CMovieFacet* movieFacet = (CMovieFacet*)fMoviePart->GetActiveFacet();
		
		THROW_IF_ERROR(::NewMovieFromHandle(&newMovie, movieHandle, newMovieActive, &dataRefChanged));
		movieFacet->GetCurrentSelection(&currentTime, &duration);
		
		CPartDisplayFrameIterator iter(fMoviePart, fMoviePart->GetDefaultPresentation());
		for (CMovieFrame* frame = (CMovieFrame*)iter.First(); iter.IsNotComplete();
					frame = (CMovieFrame*)iter.Next())
			frame->PasteMovieToSelection(newMovie, currentTime, duration);
		
		::DisposeHandle(movieHandle);
		::DisposeMovie(newMovie);
			
		internalized = kXMPTrue;
	}
	else if (storageUnit->Exists(kXMPPropContents, kXMPApplePICT, 0))
	{
		storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, kXMPApplePICT, (XMPValueIndex)0, kXMPPosUndefined);
					   
		XMPULong pictSize;
		PicHandle pictHandle = (PicHandle)XMPNewHandle(0);
		
		pictSize = storageUnit->GetSize();
		SetHandleSize((Handle)pictHandle, pictSize);
		::HLock((Handle)pictHandle);
		storageUnit->GetValue(pictSize, (XMPValue)*pictHandle);
		::HUnlock((Handle)pictHandle);
		
		this->PasteOSTypeToMovies((Handle)pictHandle, 'PICT');
		
		::KillPicture(pictHandle);

		internalized = kXMPTrue;
	}
	else if (storageUnit->Exists(kXMPPropContents, kXMPAppleTEXT, 0))
	{
		storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT, (XMPValueIndex)0, kXMPPosUndefined);
					   
		XMPULong textSize;
		Handle textHandle = XMPNewHandle(0);
		
		textSize = storageUnit->GetSize();
		SetHandleSize(textHandle, textSize);
		::HLock(textHandle);
		storageUnit->GetValue(textSize, (XMPValue)*textHandle);
		::HUnlock(textHandle);
		
		this->PasteOSTypeToMovies(textHandle, 'TEXT');
		
		::DisposeHandle(textHandle);

		internalized = kXMPTrue;
	}
	else if (storageUnit->Exists(kXMPPropContents, kXMPApplesnd, 0))
	{
		storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, kXMPApplesnd, (XMPValueIndex)0, kXMPPosUndefined);
					   
		XMPULong sndSize;
		Handle sndHandle = XMPNewHandle(0);
		
		sndSize = storageUnit->GetSize();
		SetHandleSize(sndHandle, sndSize);
		::HLock(sndHandle);
		storageUnit->GetValue(sndSize, (XMPValue)*sndHandle);
		::HUnlock(sndHandle);
		
		this->PasteOSTypeToMovies(sndHandle, 'snd ');
		
		::DisposeHandle(sndHandle);

		internalized = kXMPTrue;
	}
		
	return internalized;
}

//---------------------------------------------------------------------------------------
//	 CMovieSelection::PasteOSTypeToMovies
//---------------------------------------------------------------------------------------

void CMovieSelection::PasteOSTypeToMovies(Handle newHandle, OSType handleType)
{
		TimeValue currentTime, duration;
		((CMovieFacet*)fMoviePart->GetActiveFacet())->GetCurrentSelection(&currentTime, &duration);
		
		CPartDisplayFrameIterator iter(fMoviePart, fMoviePart->GetDefaultPresentation());
		for (CMovieFrame* frame = (CMovieFrame*)iter.First(); iter.IsNotComplete();
					frame = (CMovieFrame*)iter.Next())
			frame->PasteHandleToSelection(newHandle, handleType, currentTime, duration);
}
