/*
	File:		ODText.cp

	Contains:	Implementation for class COpenDocText

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

// ----- TextPart Includes -----

#ifndef _ODTEXT_
#include "ODText.h"
#endif

#ifndef _EMBEDRUN_
#include "EmbedRun.h"
#endif

//========================================================================================
// CLASS COpenDocText
//========================================================================================

//----------------------------------------------------------------------------------------
// COpenDocText::COpenDocText
//----------------------------------------------------------------------------------------

COpenDocText::COpenDocText()
{
}

//----------------------------------------------------------------------------------------
// COpenDocText::~COpenDocText
//----------------------------------------------------------------------------------------

COpenDocText::~COpenDocText()
{
}

//----------------------------------------------------------------------------------------
// COpenDocText::COpenDocText
//----------------------------------------------------------------------------------------

OSErr COpenDocText::InitOpenDocText(GrafPtr textPort,
									TTextensionHandlers* handlers,
									TSize sizeInfo)
{
	return CTextension::ITextension(textPort, handlers, sizeInfo);
}

//----------------------------------------------------------------------------------------
// COpenDocText::NotifyEmbeddedPartsOfPositionChange
//----------------------------------------------------------------------------------------

void COpenDocText::NotifyEmbeddedPartsOfPositionChange(long firstFormatLine,
													   long lastFormatLine)
{
	// ----- Notify embedded parts that their position may have changed

	long runLength;
	CRunObject* run;
	
	long lineStart = fFormatter->GetLineStart(firstFormatLine);
	CRunsRanges* runsRanges = this->GetRunsRanges();
		 
	while (runsRanges->GetNextRun(lineStart, &run, &runLength) >= 0)
	{
		if (run->GetClassId() == kEmbeddedPartClassId)
		{
			Point pt;
			short hite;
			long top, edge;
			short ascent, descent, leading;
			
			short selectionWidth = ((CEmbeddedRun *) run)->GetSelectionWidth();
			
			run->GetHiteInfo(&ascent, &descent, &leading);
			this->Char2Point(lineStart, &top, &edge, &hite);
			pt.v = (short) top + hite - (ascent + descent + leading) + selectionWidth;
			pt.h = (short) edge + selectionWidth;
			
			((CEmbeddedRun *) run)->ChangePosition(pt);
		}
		lineStart += runLength;
	}	
}

//----------------------------------------------------------------------------------------
// COpenDocText::COpenDocText
//----------------------------------------------------------------------------------------

void COpenDocText::EndEdit(const TEditInfo* editInfo,
						   long firstFormatLine, long lastFormatLine,
						   TOffset* selOffset, Boolean updateKeyScript)
{
	this->NotifyEmbeddedPartsOfPositionChange(firstFormatLine, lastFormatLine);	
	CTextension::EndEdit(editInfo, firstFormatLine, lastFormatLine,
						 selOffset, updateKeyScript);
}
