/*
	File:		TextFacet.cp

	Contains:	implementation of class CTextFacet

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

// ----- Macintosh Includes -----

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef mathRoutinesIncludes
#include <Math Routines.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _DGITMITR_
#include <DgItmItr.h>
#endif

// ----- Textension Includes -----

#ifndef _Textension_
#include "Textension.h"
#endif

// ----- TextPart Includes -----

#ifndef _TEXTUTIL_
#include "TextUtil.h"
#endif

#ifndef _TEXTFACET_
#include "TextFacet.h"
#endif

#ifndef _TEXTFRAME_
#include "TextFrame.h"
#endif

#ifndef _TEXTPART_
#include "TextPart.h"
#endif

#ifndef _RULERFRAME_
#include "RulerFrame.h"
#endif

#ifndef _TEXTSELC_
#include "TextSelc.h"
#endif

#ifndef _ODTEXT_
#include "ODText.h"
#endif

#ifndef _ALTPOINT_
#include "AltPoint.h"
#endif

#ifndef _FWFRAME_
#include <FWFrame.h>
#endif

#pragma segment TextPartSegment


//========================================================================================
// CLASS CTextFacet
//========================================================================================

//----------------------------------------------------------------------------------------
// CTextFacet::CTextFacet
//----------------------------------------------------------------------------------------

CTextFacet::CTextFacet() :
	CFacet(),
	fTextPart(kXMPNULL)
{
}

//----------------------------------------------------------------------------------------
// CTextFacet::InitTextFacet
//----------------------------------------------------------------------------------------

void CTextFacet::InitTextFacet(XMPFacet* facet, CTextPart* textPart)
{
	InitFacet(facet);
	fTextPart = textPart;
}

//----------------------------------------------------------------------------------------
// CTextFacet::~CTextFacet
//----------------------------------------------------------------------------------------

CTextFacet::~CTextFacet()
{
}

//----------------------------------------------------------------------------------------
// CTextFacet::Draw
//----------------------------------------------------------------------------------------

void CTextFacet::Draw(XMPShape* invalidShape)
{
	CTextDrawInitiator di(this);
	
	XMPRgnHandle region;
	if (invalidShape)
		region = invalidShape->GetQDRegion();
	else
		region = GetClipShape()->GetQDRegion();		
	
	::EraseRgn(region);
	
	Rect bounds = (**region).rgnBBox;
	
	fTextPart->SetEmbeddedRunFrame((CTextFrame *) GetFrame());
	fTextPart->SetEmbeddedRunFacet(this);

	(fTextPart->GetEditText())->Draw(&bounds, kNoDrawFlags);
}

//----------------------------------------------------------------------------------------
// CTextFacet::DoMouseDown
//----------------------------------------------------------------------------------------

XMPBoolean CTextFacet::DoMouseDown(const XMPPoint& where,
								   XMPEventData event)
{
XMPUnused(where);

	CTextDrawInitiator di(this);

	EventRecord localEvent = *event;
	
	XMPRect sbBounds;
	
	fTextPart->SetEmbeddedRunFrame((CTextFrame *) GetFrame());
	fTextPart->SetEmbeddedRunFacet(this);
	fTextPart->SetEmbeddedRunEvent(event);

	// The following statement delegates the responsibility of dragging to an embedded
	// part if the selection consists solely of the part. This gives us a chance to see
	// if the mouse is in the resize handle.
	
	XMPBoolean didDrag = kXMPFalse;
	if (((CTextSelection *) fTextPart->GetSelection())->IsPointInSelection(where)
		&& !((CTextSelection *) fTextPart->GetSelection())->IsSelectionOnlyOneProxyRun())
		didDrag = fTextPart->GetSelection()->Drag(this, event);
	
	if (!didDrag)
	{
		TClickCommandInfo clickCommandInfo;
		(fTextPart->GetEditText())->Click(&localEvent, &clickCommandInfo, NULL, NULL);
	}
	
	return true;
}

//----------------------------------------------------------------------------------------
// CTextFacet::MouseEnter
//----------------------------------------------------------------------------------------

void CTextFacet::MouseEnter(const XMPPoint& where)
{
XMPUnused(where);
	SetCursor(*GetCursor(iBeamCursor)); 
}

//----------------------------------------------------------------------------------------
// CTextFacet::NewDropTracker
//----------------------------------------------------------------------------------------

FW_CDropTracker* CTextFacet::NewDropTracker(CFacet* facet)
{
	CTextDropTracker* dropTracker = new CTextDropTracker;
	dropTracker->InitTextDropTracker((CTextFacet *) facet);
	return dropTracker;
}

//----------------------------------------------------------------------------------------
// CTextFacet::DragEnter
//----------------------------------------------------------------------------------------

void CTextFacet::DragEnter(XMPDragItemIterator* dragInfo, XMPPoint where)
{
	CFacet::DragEnter(dragInfo, where);
	
	if (!this->GetCanAcceptDrop())
	{
		XMPStorageUnit *dragSU;
		
		for (dragSU = dragInfo->First(); dragSU; dragSU = dragInfo->Next())
			if (dragSU->Exists(kXMPPropContents, kXMPAppleTEXT, 0))
				this->SetCanAcceptDrop(kXMPTrue);
	}
}

//----------------------------------------------------------------------------------------
// CTextFacet::DrawDragHiliteInside
//----------------------------------------------------------------------------------------

XMPBoolean CTextFacet::DrawDragHiliteInside()
{
	return kXMPFalse;
}


//========================================================================================
// CLASS CTextDropTracker
//========================================================================================

//----------------------------------------------------------------------------------------
// CTextDropTracker::CTextDropTracker
//----------------------------------------------------------------------------------------

CTextDropTracker::CTextDropTracker()
{
}

//----------------------------------------------------------------------------------------
// CTextDropTracker::~CTextDropTracker
//----------------------------------------------------------------------------------------

CTextDropTracker::~CTextDropTracker()
{
}

//----------------------------------------------------------------------------------------
// CTextDropTracker::InitTextDropTracker
//----------------------------------------------------------------------------------------

void CTextDropTracker::InitTextDropTracker(CTextFacet* facet)
{
	FW_CDropTracker::InitDropTracker(facet);
}

//----------------------------------------------------------------------------------------
// CTextDropTracker::BeginTracking
//----------------------------------------------------------------------------------------

void CTextDropTracker::BeginTracking(XMPPoint where)
{
#if 0
	char str[80];
	sprintf(str, (const char *) "\pBeginTracking: where=(%d,%d)\n", FixedToInt(where.x), FixedToInt(where.y));
	SysBreakFunc((unsigned const char *) str);
#endif
	
	fCharOffset = this->GetCharOffset(where);
	this->DrawCaret(fCharOffset);
}

//----------------------------------------------------------------------------------------
// CTextDropTracker::ContinueTracking
//----------------------------------------------------------------------------------------

void CTextDropTracker::ContinueTracking(XMPPoint where)
{
	TOffset newOffset = this->GetCharOffset(where);
	if (newOffset != fCharOffset)
	{
#if 0
		char str[80];
		sprintf(str, (const char *) "\pContinueTracking: where=(%d,%d)\n", FixedToInt(where.x), FixedToInt(where.y));
		SysBreakFunc((unsigned const char *) str);
#endif
	
		this->DrawCaret(fCharOffset);
		this->DrawCaret(newOffset);
		fCharOffset = newOffset;
	}
}

//----------------------------------------------------------------------------------------
// CTextDropTracker::EndTracking
//----------------------------------------------------------------------------------------

void CTextDropTracker::EndTracking(XMPPoint where)
{
XMPUnused(where);

#if 0
	char str[80];
	sprintf(str, (const char *) "\pEndTracking: where=(%d,%d)\n", FixedToInt(where.x), FixedToInt(where.y));
	SysBreakFunc((unsigned const char *) str);
#endif
	
	this->DrawCaret(fCharOffset);
}

//----------------------------------------------------------------------------------------
// CTextDropTracker::GetCharOffset
//----------------------------------------------------------------------------------------

TOffset CTextDropTracker::GetCharOffset(XMPPoint where) const
{
	TOffsetRange range;
	Boolean outside;
	
	Point pt = where.AsQDPoint();
	
	this->GetTextPart()->GetEditText()->Point2Char(pt, &range, &outside);
	
	return range.Start();
}

//----------------------------------------------------------------------------------------
// CTextDropTracker::DrawCaret
//----------------------------------------------------------------------------------------

void CTextDropTracker::DrawCaret(TOffset charOffset)
{
#if 0
	char str[80];
	sprintf(str, (const char *) "\pDrawCaret: offset=%ld\n", charOffset.offset);
	SysBreakFunc((unsigned const char *) str);
#endif

	short hite;
	long top, trailEdge;
	
	this->GetTextPart()->GetEditText()->Char2Point(charOffset, &top, &trailEdge, &hite);

	PenMode(patXor);
	
	::MoveTo((short) trailEdge, (short) top);
	::Line(0, hite);
	
	PenNormal();
}

