/*
	File:		TextFacet.h

	Contains:	Definition for class CTextFacet

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _TEXTFACET_
#define _TEXTFACET_

// ----- Framework Includes -----

#ifndef _FWFACET_
#include "FWFacet.h"
#endif

#ifndef _FWDROPTR_
#include "FWDropTr.h"
#endif

//========================================================================================
// Forward class declarations
//========================================================================================

class CTextPart;


//========================================================================================
// CLASS CTextFacet
//========================================================================================

class CTextFacet : public CFacet
{
// ----- Construction/destruction
public:
	CTextFacet();
	void InitTextFacet(XMPFacet* facet,
					   CTextPart* textPart);
	virtual ~CTextFacet();

// ----- Inherited API
public:
	virtual void 			Draw(XMPShape* invalidShape);
	virtual XMPBoolean		DoMouseDown(const XMPPoint& where,
							 			XMPEventData event);
	virtual void			MouseEnter(const XMPPoint& where);

// ----- Drag and drop
public:
	virtual FW_CDropTracker*NewDropTracker(CFacet* facet);
	virtual void			DragEnter(XMPDragItemIterator* dragInfo, XMPPoint where);
	virtual XMPBoolean		DrawDragHiliteInside();

// ----- Accessors
	CTextPart* GetTextPart() const;

// ----- Fields
private:
	CTextPart*				fTextPart;
};

//----------------------------------------------------------------------------------------
// CTextFacet::GetTextPart
//----------------------------------------------------------------------------------------

CTextPart* CTextFacet::GetTextPart() const
{
	return fTextPart;
}


//========================================================================================
// CLASS CTextDropTracker
//========================================================================================

class CTextDropTracker : public FW_CDropTracker
{
// ----- Initialization/destruction
public:
	CTextDropTracker();
	virtual ~CTextDropTracker();
	void InitTextDropTracker(CTextFacet* facet);
	
// ----- Tracking
	virtual void BeginTracking(XMPPoint where);
	virtual void ContinueTracking(XMPPoint where);
	virtual void EndTracking(XMPPoint where);

// ----- New Methods
	TOffset GetCharOffset(XMPPoint pt) const;
	void DrawCaret(TOffset charOffset);
	CTextPart* GetTextPart() const;
	
// ----- Fields
private:
	TOffset fCharOffset;
};

//----------------------------------------------------------------------------------------
// CTextDropTracker::GetTextPart
//----------------------------------------------------------------------------------------#endif
 
CTextPart* CTextDropTracker::GetTextPart() const
{
	return ((CTextFacet *) fFacet)->GetTextPart();
}

 #endif
 