/*
	File:		TextPart.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 3/31/94	DS		Remove embed menu
		 <3>	 3/31/94	DS		Remove HandleEmbedMenuEvent
		 <2>	 3/23/94	DS		Add kXMPAppleTEXT to header. This is a
									platform specific type that was removed
									from OpenDoc. There is a call in the
									translation object that should be used, but
									for now, this is simplest.

	To Do:
*/

/*
	File:		TextPart.h

	Contains:	Definition for class CTextPart

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _TEXTPART_
#define _TEXTPART_

#ifndef _FWPARTING_
#include "FWParting.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _TEXTDEF_
#include "TextDef.h"
#endif


//========================================================================================
// Forward class declarations
//========================================================================================

class CFrame;
class XMPSemanticInterface;
class COpenDocText;
class CTextFrame;
class CTextFacet;
class FW_CSelection;
class CTextSelection;


//========================================================================================
// Constants
//========================================================================================
const	XMPValueType	kXMPAppleTEXT	= "Apple:OSType:Scrap:TEXT";

//========================================================================================
// CTextPart
//========================================================================================

class CTextPart : public CEmbeddingPart
{
	friend CTextSelection;
	
// ----- Types
public:
	typedef char Justification;
	
// ----- Constants
public:
	enum { kFontMenu = 5, kSizeMenu = 6, kEmbedMenu = 7 };
	enum { kRootFrameId = 'root', kContentFrameId = 'cont', kRulerFrameId = 'rulr' };
	enum { kNumberOfTools = 4 };
	enum { kMarginPictHeight = 9, kMarginPictWidth = 5 };
	enum { kToolHeights = 13, kToolWidths = 19 };
	enum { kRulerHeight = 16, kRulerWidth = 614 };
	enum { kRulerRulerHeight = 20, kRulerToolsHeight = kToolHeights + 9 }; 
	
private:
	static const XMPPropertyName kTextPropertyName;
	static const XMPValueType kTextensionTextType;
	
// ----- Initialization and termination
public:		
	CTextPart();
	virtual ~CTextPart();

	virtual void Initialize();

// ----- from CEmbeddingPart
public:
	virtual void			Draw(XMPFacet* xmpFacet, XMPShape* invalidShape);
//	virtual CFrame*			NewScrollerFrameTree(XMPFrame* xmpFrame);
	virtual CFrame* 		NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	virtual FW_CSelection*	NewSelection();
	virtual void 			InternalizeContent(XMPStorageUnit* storageUnit);
	virtual void			ExternalizeContent(XMPStorageUnit* storageUnit);
	virtual void			DoAdjustMenus(XMPMenuBar* menuBar);
	virtual XMPBoolean		DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	virtual XMPValueType GetContentPropertyValueType() const;
	virtual CProxyRun*		NewProxyRun();
	virtual XMPTypeToken	GetDefaultPresentation() const;

	
// ----- New Methods
public:
	COpenDocText*				GetEditText()
								{ return fEditText; }
	
	// The following methods apply to the current selection.
	virtual void			SetFontSize(short fontSize);
	virtual Boolean			GetFontSize(short& fontSize) const;
	virtual void			SetFont(const Str255 fontName);
	virtual Boolean			GetFont(Str255 fontName) const;
	virtual void			SetSelectionJustification(Justification justify) const;
	virtual Justification	GetSelectionJustification() const;
	CTextFacet*				GetAnyFacet(XMPTypeToken presentation = 0);
	

	void					HandleEmbedPart(XMPPart* xmpPart, const XMPRect& frameShape);
	
	XMPTypeToken			GetMainPresentation() const
								{ return fMainPresentation; }
	XMPTypeToken			GetRulerPresentation() const
								{ return fRulerPresentation; }
										  
	Boolean					IsDrawing()
								{ return fIsDrawing; }
	void					SetEmbeddedRunFrame(CTextFrame *frame)
								{ fEmbeddedRunFrame = frame; }
	CTextFrame*				GetEmbeddedRunFrame()
								{ return fEmbeddedRunFrame; }
	void					SetEmbeddedRunFacet(CTextFacet *facet)
								{ fEmbeddedRunFacet = facet; }
	CTextFacet*				GetEmbeddedRunFacet()
								{ return fEmbeddedRunFacet; }
	void					SetEmbeddedRunEvent(XMPEventData event)
								{ fEvent = event; }
	XMPEventData			GetEmbeddedRunEvent()
								{ return fEvent; }
	static CTextPart*		GetCurrentPart()
								{ return gCurrentPart; }				
	
private:
	static void			InitializeTextension();
	
	void					CreateEditText();
	void					StuffInTheText(Handle text, Size textSize);
	void					StuffInTheStyles(Handle styles, Size stylesSize);
			
// ----- Data Members
private:
	MenuHandle				fFontMenu;
	MenuHandle				fSizeMenu;
	
	XMPTypeToken			fRulerPresentation;
	XMPTypeToken			fMainPresentation;
	XMPTypeToken			fScrollerPresentation;

	COpenDocText*			fEditText;
	
	// Hack. When my Textension run object's draw method is called by Textension I need to
	// be able to pass it some parameters. I store those parameters here before calling methods
	// on Textension.
	
	Boolean					fIsDrawing;
	CTextFrame				*fEmbeddedRunFrame;
	CTextFacet				*fEmbeddedRunFacet;
	XMPEventData			fEvent;
	
	// Global variable used to pass current part through to Textension during
	// internalization and cloning.
	
	static CTextPart*		gCurrentPart;
};

#endif
