/*
	File:		TextProxy.cp

	Contains:	implementation of class CTextProxyRun

	Written by:	Michael Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

// ----- Macintosh Includes -----

#ifndef __FIXMATH__
#include <FixMath.h>
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include "Shape.h"
#endif

// ----- Framework Includes -----

#ifndef _FWUTIL_
#include "FWUtil.h"
#endif

// ----- TextPart Includes -----

#ifndef _TEXTPROXY_
#include "TextProxy.h"
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#pragma segment TextPartSegment


//=============================================================================
// CLASS CTextProxyRun
//=============================================================================

//-----------------------------------------------------------------------------
// CTextProxyRun::CTextProxyRun
//-----------------------------------------------------------------------------

CTextProxyRun::CTextProxyRun()
{
}

//-----------------------------------------------------------------------------
// CTextProxyRun::InitTextProxyRun
//-----------------------------------------------------------------------------

void CTextProxyRun::InitTextProxyRun(CEmbeddingPart* embeddingPart)
{
	this->InitProxyRun(embeddingPart);
}

//-----------------------------------------------------------------------------
// CTextProxyRun::~CTextProxyRun
//-----------------------------------------------------------------------------

CTextProxyRun::~CTextProxyRun()
{
}

//-----------------------------------------------------------------------------
// CTextProxyRun::UsedShapeChanged
//-----------------------------------------------------------------------------

void CTextProxyRun::UsedShapeChanged(XMPFrame* xmpFrame)
{
	// We assume here that all embedded facets have the same
	// externalTransform
	
	XMPFrameFacetIterator* ite = xmpFrame->CreateFacetIterator();
	XMPFacet *xmpFacet = ite->First();
	delete ite;
	
	if (xmpFacet)
	{
		XMPRect rect;
		XMPShape *shape = ::NewXMPShape(xmpFrame->GetFrameShape());
		shape->Transform(xmpFacet->GetExternalTransform());
		shape->GetBoundingBox(&rect);
		rect.AsQDRect(fBoundingBox);
		delete shape;
	}
	
	CProxyRun::UsedShapeChanged(xmpFrame);
}

//-----------------------------------------------------------------------------
// CTextProxyRun::GetFacetExternalTransform
//-----------------------------------------------------------------------------

void CTextProxyRun::GetFacetExternalTransform(XMPFrame* xmpFrame,
											   XMPTransform* facetTransform)
{
XMPUnused(xmpFrame);

	facetTransform->Reset();
	XMPPoint offset(ff(fBoundingBox.left), ff(fBoundingBox.top));
	facetTransform->MoveBy(offset);
}

//----------------------------------------------------------------------------------------
// CTextProxyRun::SetSelectState
//----------------------------------------------------------------------------------------

void CTextProxyRun::SetSelectState(XMPBoolean state)
{
	CProxyRun::SetSelectState(state);
	
	// ----- Set the selected flag in facet -----
	CProxyRunCProxyFrameIterator ite(this);
	for (CProxyFrame* proxyFrame = ite.First(); ite.IsNotComplete(); proxyFrame = ite.Next())
	{
		XMPFrameFacetIterator ite2(proxyFrame->GetXMPFrame());
		for (XMPFacet* xmpFacet = ite2.First(); ite2.IsNotComplete(); xmpFacet = ite2.Next())
			xmpFacet->SetSelected(state);
	}
}

//----------------------------------------------------------------------------------------
// CTextProxyRun::SetBoundingBox
//----------------------------------------------------------------------------------------

void CTextProxyRun::SetBoundingBox(const Rect& rect)
{
	fBoundingBox = rect;
}

//----------------------------------------------------------------------------------------
// CTextProxyRun::GetBoundingBox
//----------------------------------------------------------------------------------------

void CTextProxyRun::GetBoundingBox(Rect& rect) const
{
	rect = fBoundingBox;
}
