/*
	File:		TextSelc.cpp

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 4/18/94	DS		Turn off linking by passing false, false to
									initselection

	To Do:
*/

/*
	File:		TextSelc.cp

	Contains:	implementation of class CTextSelection

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

// ----- Macintosh Includes -----

#ifndef __FIXMATH__
#include <FixMath.h>
#endif

#ifndef mathRoutinesIncludes
#include <Math Routines.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _TRNSFORM_
#include <TrnsForm.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

// ----- Framework Includes -----

#ifndef _FWFRAME_
#include "FWFrame.h"
#endif

// ----- Textension Includes -----

#ifndef _TextensionCommon_
#include "TextensionCommon.h"
#endif

#ifndef _Textension_
#include "Textension.h"
#endif

// ----- TextPart Includes -----

#ifndef _TEXTPART_
#include "TextPart.h"
#endif

#ifndef _TEXTSELC_
#include "TextSelc.h"
#endif

#ifndef _VALUESTREAM_
#include "ValueStream.h"
#endif

#ifndef _TEXTUTIL_
#include "TextUtil.h"
#endif

#ifndef _TEXTFACET_
#include "TextFacet.h"
#endif

#ifndef _TEXTPROXY_
#include "TextProxy.h"
#endif

#ifndef _EMBEDRUN_
#include "EmbedRun.h"
#endif

#ifndef _ODTEXT_
#include "ODText.h"
#endif

#pragma segment TextPartSegment

//========================================================================================
// CLASS CTextSelection
//========================================================================================

//----------------------------------------------------------------------------------------
// CTextSelection::CTextSelection
//----------------------------------------------------------------------------------------

CTextSelection::CTextSelection():
	fTextPart(kXMPNULL),
	fIsCloning(kXMPFalse),
	fCommandFrame(kXMPNULL)
{
}

//----------------------------------------------------------------------------------------
// CTextSelection::InitTextSelection
//----------------------------------------------------------------------------------------

void CTextSelection::InitTextSelection(CTextPart* textPart)
{
	InitSelection(textPart, kXMPFalse, kXMPFalse);// Was true, true ????? DISABLE Linking cause its not finished
	fTextPart = textPart;
}

//----------------------------------------------------------------------------------------
// CTextSelection::~CTextSelection
//----------------------------------------------------------------------------------------

CTextSelection::~CTextSelection()
{
}

//----------------------------------------------------------------------------------------
// CTextSelection::CloseSelection
//----------------------------------------------------------------------------------------

void CTextSelection::CloseSelection()
{
	CTextFacet* facet = fTextPart->GetAnyFacet(fTextPart->GetMainPresentation());
	
	CTextDrawInitiator di((CTextFacet *) facet);
	TOffsetRange selRange((long) 0, (long) 0);
	fTextPart->GetEditText()->SetSelectionRange(selRange);
}

//----------------------------------------------------------------------------------------
// CTextSelection::SelectAll
//----------------------------------------------------------------------------------------

void CTextSelection::SelectAll()
{
	TOffsetRange selRange(0, fTextPart->GetEditText()->CountChars());
	fTextPart->GetEditText()->SetSelectionRange(selRange);
}

//----------------------------------------------------------------------------------------
// CTextSelection::IsEmpty
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::IsEmpty() const
{
	TOffsetRange range;
	fTextPart->GetEditText()->GetSelectionRange(&range);
	
	return range.IsEmpty();
}

//----------------------------------------------------------------------------------------
// CTextSelection::DoClear
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::DoClear()
{
	CTextFacet* facet = fTextPart->GetAnyFacet(fTextPart->GetMainPresentation());
	CTextDrawInitiator di(facet);
	
	TOffsetRange range;
	fTextPart->GetEditText()->GetSelectionRange(&range);

	TReplaceParams replaceParams;
	fTextPart->GetEditText()->ReplaceRange(range.Start(), range.End(),
										   &replaceParams, false);
										   
	fTextPart->InvalidateAllFrames(fTextPart->GetMainPresentation(), facet, kXMPNULL);
	
	return kXMPTrue;
}

//----------------------------------------------------------------------------------------
// CTextSelection::IsSelectionOnlyOneProxyRun
//----------------------------------------------------------------------------------------

CProxyRun* CTextSelection::IsSelectionOnlyOneProxyRun() const
{
	CProxyRun* proxyRun = kXMPNULL;
	CRunsRanges* runsRanges = fTextPart->GetEditText()->GetRunsRanges();
	
	TOffsetRange range;
	fTextPart->GetEditText()->GetSelectionRange(&range);

	if (range.Len() == 1)
	{
		long runLength;
		CRunObject* run;
		 
		runsRanges->GetNextRun(range.Start(), &run, &runLength);
		if (run->GetClassId() == kEmbeddedPartClassId)
			proxyRun = ((CEmbeddedRun *) run)->GetProxyRun();
	}
	
	return proxyRun;
}

//----------------------------------------------------------------------------------------
// CTextSelection::EmbedSelection
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape)
{
	XMPRect shapeRect;
	if (frameShape)
	{
		frameShape->GetBoundingBox(&shapeRect);
		shapeRect.Offset(ff(-shapeRect.left), ff(-shapeRect.top));
	}
	else
	{
		shapeRect.Set(ff(0), ff(0), ff(80), ff(80));
	}
	
	fTextPart->HandleEmbedPart(xmpPart, shapeRect);
	
	return kXMPTrue;
}

//----------------------------------------------------------------------------------------
// CTextSelection::ExternalizeSelection_ContentPropertyValueType
//----------------------------------------------------------------------------------------

void CTextSelection::ExternalizeSelection_ContentPropertyValueType
	(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind)
{
XMPUnused(cloneKind);

	storageUnit->AddValue(this->GetPart()->GetContentPropertyValueType());

	CValueStream *stream = new CValueStream;
	stream->InitValueStream(storageUnit);
	
	CTextensionIOSuite *ioSuite = new CTextensionIOSuite;
	ioSuite->ITextensionIOSuite(false, stream);
	
	CTextPart::gCurrentPart = fTextPart;
	fIsCloning = kXMPTrue;
	fCommandFrame = commandFrame;
	
	TOffsetRange range2Save;
	fTextPart->GetEditText()->GetSelectionRange(&range2Save);
	OSErr err = fTextPart->GetEditText()->GetRangeIOSuite(&range2Save, ioSuite);

	fCommandFrame = kXMPNULL;
	fIsCloning = kXMPFalse;
	CTextPart::gCurrentPart = NULL;

	ioSuite->Free();
	delete ioSuite;
	
	stream->Free();
	delete stream;
}

//----------------------------------------------------------------------------------------
// CTextSelection::ExternalizeSelection_TEXT
//----------------------------------------------------------------------------------------

void CTextSelection::ExternalizeSelection_TEXT(XMPStorageUnit* storageUnit,
											   CFrame* commandFrame,
											   XMPCloneKind cloneKind)
{
XMPUnused(cloneKind);
XMPUnused(commandFrame);

	storageUnit->AddValue(kXMPAppleTEXT);

	CValueStream *stream = new CValueStream;
	stream->InitValueStream(storageUnit);
	
	CTextensionIOSuite *ioSuite = new CTextensionIOSuite;
	ioSuite->ITextensionIOSuite(false, stream);
	
	TOffsetRange range2Save;
	fTextPart->GetEditText()->GetSelectionRange(&range2Save);
	OSErr err = fTextPart->GetEditText()->GetRangeIOSuite(&range2Save, ioSuite, kIOText);

	ioSuite->Free();
	delete ioSuite;
	
	stream->Free();
	delete stream;
}

//----------------------------------------------------------------------------------------
// CTextSelection::ExternalizeSelection
//----------------------------------------------------------------------------------------

void CTextSelection::ExternalizeSelection(XMPStorageUnit* storageUnit,
										  CFrame* commandFrame, XMPCloneKind cloneKind)
{
	storageUnit->AddProperty(kXMPPropContents);
	
	this->ExternalizeSelection_ContentPropertyValueType(storageUnit, commandFrame, cloneKind);
	this->ExternalizeSelection_TEXT(storageUnit, commandFrame, cloneKind);
}

//----------------------------------------------------------------------------------------
// CTextSelection::InternalizeSelection_ContentPropertyValueType
//----------------------------------------------------------------------------------------
 
XMPBoolean CTextSelection::InternalizeSelection_ContentPropertyValueType
			(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
XMPUnused(cloneKind);

	// ----- Focus on content property -----
	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, 
				   	   this->GetPart()->GetContentPropertyValueType(),
					   (XMPValueIndex)0, kXMPPosUndefined);

	CTextFacet* facet = fTextPart->GetAnyFacet(fTextPart->GetMainPresentation());
	CTextDrawInitiator di(facet);

	CValueStream *stream = new CValueStream;
	stream->InitValueStream(storageUnit);
	
	CTextensionIOSuite *ioSuite = new CTextensionIOSuite;
	ioSuite->ITextensionIOSuite(false, stream);
	
	CTextPart::gCurrentPart = fTextPart;
	fIsCloning = kXMPTrue;
	
	TOffsetRange range2Replace;
	fTextPart->GetEditText()->GetSelectionRange(&range2Replace);
	TReplaceParams replaceParams(ioSuite);
	OSErr err = fTextPart->GetEditText()->ReplaceRange(range2Replace.Start(),
													   range2Replace.End(),
													   &replaceParams, false);

	fTextPart->InvalidateAllFrames(fTextPart->GetMainPresentation(), facet, kXMPNULL);
	
	fIsCloning = kXMPFalse;
	CTextPart::gCurrentPart = NULL;
		
	ioSuite->Free();
	delete ioSuite;
	
	stream->Free();
	delete stream;
	
	return kXMPTrue;
}

//----------------------------------------------------------------------------------------
// CTextSelection::InternalizeSelection_TEXT
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::InternalizeSelection_TEXT
			(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
XMPUnused(cloneKind);

	storageUnit->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT,
					   (XMPValueIndex)0, kXMPPosUndefined);
				   
	CTextFacet* facet = fTextPart->GetAnyFacet(fTextPart->GetMainPresentation());
	CTextDrawInitiator di(facet);

	XMPULong textSize;
	Handle textHandle = XMPNewHandle(0);
	
	textSize = storageUnit->GetSize();
	SetHandleSize(textHandle, textSize);
	::HLock(textHandle);
	storageUnit->GetValue(textSize, (XMPValue)*textHandle);
	
	TOffsetRange range2Replace;
	fTextPart->GetEditText()->GetSelectionRange(&range2Replace);
	TReplaceParams replaceParams(TTextDescriptor((unsigned char *) *textHandle, textSize));
	OSErr err = fTextPart->GetEditText()->ReplaceRange(range2Replace.Start(),
													   range2Replace.End(),
													   &replaceParams, false);

	fTextPart->InvalidateAllFrames(fTextPart->GetMainPresentation(), facet, kXMPNULL);

	::HUnlock(textHandle);
	::DisposeHandle(textHandle);

	return kXMPTrue;
}

//----------------------------------------------------------------------------------------
// CTextSelection::InternalizeSelection
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::InternalizeSelection(XMPStorageUnit* storageUnit,
												XMPCloneKind cloneKind)
{
XMPUnused(cloneKind);

	XMPBoolean success = kXMPFalse;

	if (storageUnit->Exists(kXMPPropContents, this->GetPart()->GetContentPropertyValueType(), 0))
		success = this->InternalizeSelection_ContentPropertyValueType(storageUnit, cloneKind);
	else if (storageUnit->Exists(kXMPPropContents, kXMPAppleTEXT, 0))
		success = this->InternalizeSelection_TEXT(storageUnit, cloneKind);
	
	return success;
}

//----------------------------------------------------------------------------------------
// CTextSelection::CalcDragRgn
//----------------------------------------------------------------------------------------

XMPRgnHandle CTextSelection::CalcDragRgn(CFacet* facet)
{
XMPUnused(facet);

	// Just temporary until I get a Textension that provides me with an easy method to
	// get the region of the current selection.

	TOffsetRange selection;
	fTextPart->GetEditText()->GetSelectionRange(&selection);
	
	long top, trailEdge;
	short hite;
	Rect rt;
	
	fTextPart->GetEditText()->Char2Point(selection.Start(), &top, &trailEdge, &hite);
	
	rt.top = (short) top;
	rt.left = (short) trailEdge;
	
	fTextPart->GetEditText()->Char2Point(selection.End(), &top, &trailEdge, &hite);
	
	rt.bottom = (short) top + hite;
	rt.right = (short) trailEdge;
	
	XMPRgnHandle dragRgn = ::NewRgn();
	RgnHandle tempRgn = ::NewRgn();

	if (selection.Len() > 0)
	{
		::RectRgn(dragRgn, &rt);
		::CopyRgn(dragRgn, tempRgn);
		::InsetRgn(tempRgn, 1, 1);
		::DiffRgn(dragRgn, tempRgn, dragRgn);
	}
	
	::DisposeRgn(tempRgn);
	
	return dragRgn;
}

//----------------------------------------------------------------------------------------
// CTextSelection::DoDrop
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::DoDrop(XMPStorageUnit* dropSU, CFacet* facet,
								  XMPPoint originPoint, XMPPoint dropPoint)
{
	// Set the Textension selection
	
	TOffsetRange range;
	Boolean outside;
	
	XMPPoint where = facet->GetWindowContentTransform()->InvertPoint(dropPoint);
	Point pt = where.AsQDPoint();
	
	CTextDrawInitiator di((CTextFacet *) facet);
	
	fTextPart->GetEditText()->Point2Char(pt, &range, &outside);
	fTextPart->GetEditText()->SetSelectionRange(range);
	
	return FW_CSelection::DoDrop(dropSU, facet, originPoint, dropPoint);
}

//----------------------------------------------------------------------------------------
// CTextSelection::DoDroppedInSameFrame
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::DoDroppedInSameFrame(XMPStorageUnit* dropSU, CFacet* facet,
												XMPPoint originPoint, XMPPoint dropPoint)
{
	return this->DoDrop(dropSU, facet, originPoint, dropPoint);
}

//----------------------------------------------------------------------------------------
// CTextSelection::IsPointInSelection
//----------------------------------------------------------------------------------------

XMPBoolean CTextSelection::IsPointInSelection(const XMPPoint& xmpPt)
{
	Point pt = xmpPt.AsQDPoint();
	
	TOffsetRange selectionRange;
	fTextPart->GetEditText()->GetSelectionRange(&selectionRange);
	
	Boolean outside;
	TOffsetRange charRange;
	fTextPart->GetEditText()->Point2Char(pt, &charRange, &outside);
	
	if (charRange.Start() >= selectionRange.Start()
		&& charRange.End() <= selectionRange.End())
		return kXMPTrue;
	else
		return kXMPFalse;
}

