/*
	File:		TextSelc.h

	Contains:	Definition of class CTextSelection

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _TEXTSELC_
#define _TEXTSELC_

#ifndef _FWSELECT_
#include "FWSelect.h"
#endif


//========================================================================================
// Forward class declarations
//========================================================================================

class CTextPart;


//========================================================================================
// CLASS CTextSelection
//========================================================================================

class CTextSelection : public FW_CSelection
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
						CTextSelection();
	void				InitTextSelection(CTextPart* textPart);
	virtual				~CTextSelection();
	
//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual void		CloseSelection();
	virtual void		SelectAll();
	virtual XMPBoolean	IsEmpty() const;

	// ----- Cut/Copy/Paste/Clear -----
	virtual XMPBoolean	DoClear();

	virtual CProxyRun*	IsSelectionOnlyOneProxyRun() const;

	virtual XMPBoolean	EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape);

	virtual void		ExternalizeSelection_ContentPropertyValueType
							(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual void		ExternalizeSelection_TEXT
							(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual void		ExternalizeSelection
							(XMPStorageUnit* storageUnit, CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual XMPBoolean	InternalizeSelection_ContentPropertyValueType
							(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);
	virtual XMPBoolean	InternalizeSelection_TEXT
							(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);
	virtual XMPBoolean	InternalizeSelection
							(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);

	// ----- Drag & Drop -----
	virtual XMPRgnHandle CalcDragRgn(CFacet* facet);
	virtual XMPBoolean	DoDrop(XMPStorageUnit* dropSU, CFacet* facet, XMPPoint originPoint, XMPPoint dropPoint);
	virtual XMPBoolean	DoDroppedInSameFrame(XMPStorageUnit* dropSU, CFacet* facet, XMPPoint originPoint, XMPPoint dropPoint);
	
	// ----- Parameter accessors -----
	XMPBoolean			IsCloning() const
							{ return fIsCloning; }
	CFrame*				GetCommandFrame() const
							{ return fCommandFrame; }
	XMPCloneKind		GetCloneKind() const
							{ return fCloneKind; }

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual XMPBoolean	IsPointInSelection(const XMPPoint& xmpPt);
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	CTextPart*			fTextPart;

	// The following fields are used to pass parameters from a text part through
	// textension to an embedded part.
	
	XMPBoolean			fIsCloning;
	CFrame*				fCommandFrame;
	XMPCloneKind		fCloneKind;
};

#endif
