/*
	File:		ValueStream.cp

	Contains:	Definition for class CValueStream

	Written by:	Mike Burbidge

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _VALUESTREAM_
#include "ValueStream.h"
#endif

#pragma segment TextPartSegment

//========================================================================================
// CLASS CValueStream
//========================================================================================

//----------------------------------------------------------------------------------------
// CValueStream::CValueStream
//----------------------------------------------------------------------------------------

CValueStream::CValueStream() :
	fSU(NULL)
{
}
	
//----------------------------------------------------------------------------------------
// CValueStream::InitValueStream
//----------------------------------------------------------------------------------------

void CValueStream::InitValueStream(XMPStorageUnit *su)
{
	CStream::IStream();
	fSU = su;
}

//----------------------------------------------------------------------------------------
// CValueStream::~CValueStream
//----------------------------------------------------------------------------------------

CValueStream::~CValueStream()
{
}

//----------------------------------------------------------------------------------------
// CValueStream::WriteBytes
//----------------------------------------------------------------------------------------

OSErr CValueStream::WriteBytes(const void* theBytes, long bytesCount)
{
	fSU->SetValue(bytesCount, theBytes);	
	return noErr;
}

//----------------------------------------------------------------------------------------
// CValueStream::ReadBytes
//----------------------------------------------------------------------------------------

OSErr CValueStream::ReadBytes(void* theBytes, long bytesCount)
{
	fSU->GetValue(bytesCount, theBytes);
	return noErr;
}


//----------------------------------------------------------------------------------------
// CValueStream::GetPosition
//----------------------------------------------------------------------------------------

long CValueStream::GetPosition() const
{
	return fSU->GetOffset();
}

//----------------------------------------------------------------------------------------
// CValueStream::SetPosition
//----------------------------------------------------------------------------------------

void CValueStream::SetPosition(long newPosition)
{
	fSU->SetOffset(newPosition);
}

//----------------------------------------------------------------------------------------
// CValueStream::Skip
//----------------------------------------------------------------------------------------

void CValueStream::Skip(long count)
{
	fSU->SetOffset(fSU->GetOffset() + count);
}

//----------------------------------------------------------------------------------------
// CValueStream::GetSize
//----------------------------------------------------------------------------------------

long CValueStream::GetSize() const
{
	return fSU->GetSize();
}

//----------------------------------------------------------------------------------------
// CValueStream::Append
//----------------------------------------------------------------------------------------

OSErr CValueStream::Append(long count)
{
XMPUnused(count);

	DebugStr("\pCValueStream::Append(long count)");
	
	return noErr;
}
	
//----------------------------------------------------------------------------------------
// CValueStream::Empty
//----------------------------------------------------------------------------------------

void CValueStream::Empty()
{
	fSU->DeleteValue(0);
}

//----------------------------------------------------------------------------------------
// CValueStream::Load
//----------------------------------------------------------------------------------------

OSErr CValueStream::Load(long size, Ptr* data)
{
XMPUnused(size);
XMPUnused(data);

	DebugStr("\pCValueStream::Load(long size, Ptr* data)");
	
	return noErr;
}
	
//----------------------------------------------------------------------------------------
// CValueStream::Unload
//----------------------------------------------------------------------------------------

void CValueStream::Unload(Ptr data)
{
XMPUnused(data);

	DebugStr("\pCValueStream::Unload(Ptr data)");
}
	