/*
	File:		Clock.cpp

	Contains:	Source code for the sample "Clock" part

	Written by:	Steve Smith

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<10>	 4/16/94	T		#1157828 Move kXMPKindPartsBin to a public
									header
		 <9>	  4/8/94	RR		Handle kXMPCommandClose for settings window
		 <8>	  4/1/94	T		#1154941: Get OpenDoc source to actually
									compile for PowerPC
		 <7>	 3/28/94	T		#1145834.  Remove PrtType.h & .cpp from the
									project
		 <6>	 3/28/94	CG		1153547: include XMPSessM.h instead of
									XMPSessn.h.
		 <5>	 3/25/94	CC		Replaced kXMPDropSucceed with kXMPDropCopy
									(1153424); use
									XMPDragAndDrop::GetContentStorageUnit()
									(1153043).
		 <4>	 2/23/94	VL		114????, fixed nasty linking problem
									(PartInfoRec::~PartInfoRec linked to the
									one in DrawPart.) by changing PartInfoRec
									to ClockPartInfoRec.
		 <3>	 2/16/94	T		fix #1144525
		 <2>	 2/16/94	JA		Include new AltPoint.h.
		<54>	  2/8/94	JA		Added missing #includes.
		<53>	  2/7/94	JA		Utility.h --> XMPUtils.h
		<52>	  2/7/94	T		fixes to compile with PPC Headers
		<51>	  2/3/94	CG		Session organization changes.
		<50>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame()
		<49>	  2/1/94	SS		PartsBin menu item handling
		<48>	 1/28/94	RR		macro for ASLM QD globals
		<47>	 1/27/94	RR		Enable Show Parts Bin
		<46>	 1/27/94	RR		Added CreateWindow. Reworked Open.
									Implementated activation recipe
		<45>	 1/26/94	RR		Removed calls to invalidate facet under
									mouse when dismissing dialogs
		<44>	 1/25/94	RR		Call window->Open,Show,Select
		<43>	 1/24/94	SS		Additional work on Open method,
		<42>	 1/24/94	CC		Added XMPChangeID parameter to
									LinkUpdated().
		<41>	 1/21/94	RR		Added shouldSave to CreateWindow
		<40>	 1/20/94	SS		::Open returns a window id
		<39>	 1/20/94	SS		Updated partsbin support
		<38>	 1/15/94	RR		WinState.h->WinStat.h,
									Dispatch.h->Disptch.h
		<37>	 1/15/94	RR		Updated HandleMenuEvent (command no. is no
									longer in event)
		<36>	 1/14/94	RR		Entirely formatting
		<35>	 1/11/94	T		Init... changes
		<34>	 1/10/94	RR		Removed HiliteMenu calls. Used new
									TransferFocus calls
		<33>	  1/7/94	EL		Remove extra declaration of tEventHandled
									in HandleMenuEvent.
		<32>	12/23/93	RR		Call InitFocusSet
		<31>	12/22/93	RR		Save reference to fSettingsWindow as an ID
		<30>	12/21/93	VL		Changed XMPStorageUnit::GetValueSize to
									XMPStorageUnit::GetSize.
		<29>	12/21/93	JBS		CreateFrame with viewType & presentation
		<28>	12/20/93	SS		Added partsbin support
		<27>	12/20/93	VL		Used WindowState to create window.
		<26>	12/20/93	RR		RegisterIdleFrame -> RegisterIdle
		<25>	12/16/93	RR		Added a variety of dialogs
		<24>	12/15/93	JBS		add FrameClosed
		<23>	12/13/93	T		InitPart changes
		<22>	12/13/93	RR		Implement About box from resources. Added
									gSession global for use by dialog filter.
		<21>	 12/8/93	RR		Fix syntax error
		<20>	 12/8/93	RR		Added (inactive) code for testing modal
									dialogs
		<19>	11/30/93	LM		Added an XMPFrame parameter to ReadPartInfo
		<18>	11/23/93	JA		Updated for new Shape API.
		<17>	11/23/93	T		fTestClockSU = kXMPNULL; in constructor
		<16>	11/22/93	T		modifications to use the correct part
									storage model
		<15>	11/19/93	RR		Use correct event constants from
									PlfTypes.h.
		<14>	11/18/93	T		call inherited::InitializePersistent() and
									::Externalize()
		<13>	11/18/93	EL		Use standard property and types.
		<12>	11/16/93	SS		Fixed bug that caused menubar font/size to
									change
		<11>	11/16/93	RR		Added AdjustMenus (not yet called), and
									MouseWithin. Removed point from
									MouseLeave
		<10>	11/15/93	CC		Added stubs for CreateLink() and
									LinkUpdated()
		 <9>	11/12/93	JBS		use Facets
		 <8>	 11/1/93	VL		Used BeginClone, CloneTo and EndClone.
		 <7>	10/27/93	RR		Set Cursor in
									MouseEnterFrame/MouseLeaveFrame
		 <6>	10/21/93	PH		Change FulfillPromise call
		 <5>	10/14/93	RR		Add source frame parameter to
									AddDisplayFrame
		 <4>	10/14/93	JA		XMPShape fu: No longer returns copies from
									GetPlatformShape. Deal w/it.
		 <2>	 10/8/93	T		call SetChangedFromPrev when content
									changes
		<47>	 10/6/93	JA		Use new fixed-point XMPPoint.
		<46>	 10/5/93	T		In Initialize set fSemanticInterface =
									kXMPNULL.   Use XMPDeleteObject and
									XMPReleaseObject on appropriate fields
									inside destructor.
		<45>	 10/5/93	T		RelinquishFocusSet inside
									RemoveDisplayFrame
		<44>	 10/5/93	T		fixed InstallMenu to DisplayMenu before
									annotating it
		<43>	 10/5/93	T		fixed InstallMenu to DisplayMenu before
									annotating it
		<42>	 9/30/93	PH		Support new Drag and Drop protocol
		<41>	 9/30/93	RR		Added fFocusSet
		<40>	 9/29/93	VL		Added InitializePersistent.
		<39>	 9/23/93	JA		Use new convenience methods from Shape and
									Transform.
		<38>	 9/22/93	RR		Release menu bar, instead of deleting it
		<37>	 9/16/93	RR		Added fMenuBar
		<36>	 9/16/93	JA		Dispose rgn returned from GetPlatformShape.
		<35>	 9/10/93	RR		Use new menubar calls
		<34>	 8/23/93	PH		Fix in/externalize
		<33>	 8/20/93	PH		Fix ref counting
		<32>	 8/20/93	CG		Removed SemtIntf test methods.
		<31>	 8/13/93	RR		Use OrdColl.h instead of Collect.h
		<30>	 7/29/93	NP		Global variable change for ASLM.
		<29>	 7/28/93	PH		Move menu setup to constructor
		<28>	 7/26/93	CG		Added Reply handler to SemtItnf test code.
		<27>	 7/21/93	PH		Fix menus
		<26>	 7/21/93	CG		Sending a reply after AEvent is handled
									does not work.
		<25>	 7/20/93	CG		Added SemanticIntf test code.
		<24>	 7/16/93	PH		Call Get/PutOrderedCollection in DrawPart
		<23>	 7/13/93	PH		Comments on internalize code
		<22>	 7/13/93	PH		RegisterIdleFrame on internalize
		<21>	 7/12/93	PH		Externalize/Internalize fDisplayFrames
		<20>	  7/6/93	PH		Fix MinimizeFrameShape
		<19>	  7/6/93	PH		Fix FrameShapeChanged
		<16>	 6/24/93	RCR		Added #include Disptch.h
		<16>	 6/23/93	RCR		Use tokens for foci
		<15>	 6/15/93	RCR		Compare foci using XMPISOStrCompare
		<14>	 5/26/93	JBS		update Active and Used shapes in
									FrameShapeChanged()
		<13>	 5/21/93	JBS		fix typo
		 <11>	 5/19/93	RCR		Part.h NameSmith.
		 <10>	 5/13/93	RCR		Adjust to Shape/Transform changes.
		 <9>	 4/29/93	VL		8.3 Name Changes.
		 <3>	  4/1/93	SES		Implemented "Font","Size", and "Style" menus. Added menu and
									selection focus/handling code.
		 <2>	 3/29/93	SES		Changed the text drawing mode to (srcCopy) so that updating
									works
		 <1>	 3/29/93	SES		first checked in
	To Do:
*/


#ifndef _ALTPOINT_
#include "AltPoint.h"			// Use C++ savvy XMPPoint and XMPRect
#endif


#include "Clock.h"

#ifndef _FOCI_
#include "Foci.h"
#endif

#ifndef _FOCUSSET_
#include "FocusSet.h"
#endif

#ifndef _STDPROPS_
#include "StdProps.h"
#endif

#ifndef _STDTYPES_
#include "StdTypes.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _STDDEFS_
#include "StdDefs.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _DOCUMENT_
#include "Document.h"
#endif

#ifndef _XMPCTR_
#include "XMPCtr.h"
#endif

#ifndef _SUVIEW_
#include "SUView.h"
#endif

#ifndef _SUCURSOR_
#include "SUCursor.h"
#endif

#ifndef _STORAGE_
#include "Storage.h"
#endif

#ifndef _XMPSESSM_
#include "XMPSessM.h"
#endif

#ifndef _WINSTAT_
#include "WinStat.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _DISPTCH_
#include "Disptch.h"
#endif

#ifndef _XMPUTILS_
#include "XMPUtils.h"
#endif

#ifndef _MENUBAR_
#include "MenuBar.h"
#endif

#ifndef _CMDDEFS_
#include "CmdDefs.h"
#endif

#ifndef _INFO_
#include "Info.h"
#endif

#ifndef _ISOSTRING_
#include "ISOStr.h"
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

#ifndef _MSSGINTF_
#include "MssgIntf.h"
#endif

#ifndef _AEOBJECTS_
#include "AEObjects.h"
#endif

#ifndef _FOCUSLIB_
#include "FocusLib.h"
#endif

#ifndef _ARBITRAT_
#include "Arbitrat.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __LIBRARYMANAGERUTILITIES__
#include <LibraryManagerUtilities.h>
#endif

#ifndef __LIBRARYMANAGERCLASSES__
#include <LibraryManagerClasses.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __PACKAGES__
#include <Packages.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __DIALOGS__
#include "Dialogs.h"
#endif

#ifdef THINK_CPLUS

#ifndef __BDC__
#include <bdc.h>
#endif

#ifndef __pascal__
#include <pascal.h>
#endif

#else // THINK_CPLUS

#ifndef __PACKAGES__
#include <packages.h>
#endif

#ifndef __STRINGS__
#include <strings.h>
#endif

#endif // THINK_CPLUS



#pragma segment ClockPart

static XMPSession* gSession;

//=========================================================================
// Constants
//=========================================================================

const	XMPPropertyName	kXMPPropClockFont   = "ClockPart:Property:Font";
const	XMPPropertyName	kXMPPropClockStyle  = "ClockPart:Property:Style";
const	XMPPropertyName	kXMPPropClockSize   = "ClockPart:Property:Size";

// To identify different frames (should be using viewType, not frameGroup)
const XMPULong kXMPClockFrame = 1000;
const XMPULong kXMPSynchronizeFrame = 2000;
const XMPULong kXMPAlarmSettingsFrame = 3000;

//=========================================================================
// Local Classes
//=========================================================================

class ClockPartInfoRec 
{
public:
	ClockPartInfoRec() { fIsActive = kXMPFalse; fNeedsActivating = kXMPFalse; }
	virtual ~ClockPartInfoRec() { }
	XMPBoolean	fIsActive;	// Does this frame have its focus set?
	XMPBoolean	fNeedsActivating;	// should this be activated when its window is activated?
};

//=========================================================================
// Function Prototype
//=========================================================================

pascal Boolean MyDialogFilter(DialogPtr dialog,
							  EventRecord* event,
							  short* itemHit);
							  
//=========================================================================
// ClockPart
//=========================================================================

//-------------------------------------------------------------------------
// Constructor/Destructor
//-------------------------------------------------------------------------
	
ClockPart::ClockPart() 
{		
	gSession = kXMPNULL;
	fWindowID = 0;
	fDisplayFrames = NULL;						// Create an empty collection for storing the list 
												// of frames to display	the "clock" in.	
	fFrameViewWindow = nil;						// WindowPtr storage - used by the ::Open method
	
	fFont   = GetSysFont();						// Script Mgr: get the current system font
	fStyle  = normal;							// Font Mgr: enum for plain style
	fMode   = srcCopy;							// Quickdraw: set the text drawing mode to "source copy"
	fSize   = GetDefFontSize();					// Script Mgr: get the current default font size
	fJustification = GetSysJust();				// Script Mgr: get the current system justification

	fAscent = 0;								// Ascent of the fFont font - used in positioning for drawing
	
	fCrossCursor = (XMPASLMQDGlobals.arrow);					// Store the cursors used by the part
	fArrowCursor = (XMPASLMQDGlobals.arrow);					// handler - "arrow" is the std pointer
												// defined by Quickdraw		
	fFontMenu = kXMPNULL;
	fSizeMenu = kXMPNULL;
	fStyleMenu = kXMPNULL;
	fMenuBar = kXMPNULL;
	fFocusSet = kXMPNULL;
	fSemanticInterface = kXMPNULL;
	fTestClockSU = kXMPNULL;
	
	fSettingsWindow = 0;
	fSettingsWindowWasShown = kXMPFalse;
	fAlarmOn = kXMPFalse;
	
	fPartsBinWin = 0;
	fPartsBinOpen = kXMPFalse;
}

void ClockPart::InitPart(XMPStorageUnit* storageUnit)
{
	if (fInitialized)
		return;
	
	SimplePart::InitPart(storageUnit);
	
	this->CommonInitClock();
	
	storageUnit->AddProperty(kXMPPropContents)->AddValue(kXMPKindTestClock);	

	fTestClockSU = storageUnit->GetDraft()->CreateStorageUnit();

	fTestClockSU->AddProperty(kXMPPropClockFont)->AddValue(kXMPUShort);	
	fTestClockSU->AddProperty(kXMPPropClockStyle)->AddValue(kXMPUShort);			
	fTestClockSU->AddProperty(kXMPPropClockSize)->AddValue(kXMPUShort);		
	fTestClockSU->AddProperty(kXMPPropDisplayFrames)->AddValue(kXMPID);			
}

void ClockPart::InitPartFromStorage(XMPStorageUnit* storageUnit)
{
	if (fInitialized)
		return;
	
	SimplePart::InitPartFromStorage(storageUnit);
	
	this->CommonInitClock();

	XMPULong valueSize, offset;
	XMPStorageUnitRef aSURef;
	XMPFrame* aFrame;
	XMPStorageUnit* su;

XMPVolatile(su);
XMPVolatile(aFrame);
	
	su = this->GetStorageUnit();
	su->Focus(kXMPPropContents,kXMPPosSame,kXMPKindTestClock,1,kXMPPosFirstSib);
	su->GetValue(sizeof(XMPStorageUnitRef),&aSURef);
	fTestClockSU = su->GetDraft()->GetStorageUnit(su->GetIDFromStorageUnitRef(aSURef));
	su = fTestClockSU;
	
	su->Focus(kXMPPropClockFont,kXMPPosSame,kXMPSShort,1,kXMPPosFirstSib);
	su->GetValue(sizeof(XMPSShort), (XMPValue)&fFont);
	su->Focus(kXMPPropClockStyle,kXMPPosSame,kXMPSShort,1,kXMPPosFirstSib);
	su->GetValue(sizeof(XMPSShort), (XMPValue)&fStyle);
	su->Focus(kXMPPropClockSize,kXMPPosSame,kXMPSShort,1,kXMPPosFirstSib);
	su->GetValue(sizeof(XMPSShort), (XMPValue)&fSize);
	
	su->Focus(kXMPPropDisplayFrames,kXMPPosUndefined,0,1,kXMPPosFirstSib);
	valueSize = su->GetSize();
	for (offset = 0; offset < valueSize; offset += sizeof(XMPStorageUnitRef))
	{
		su->SetOffset(offset);
		su->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		
		TRY
		
			aFrame = su->GetDraft()->GetFrame(su->GetIDFromStorageUnitRef(aSURef));
			fDisplayFrames->AddLast((ElementType)aFrame);
			fSession->GetDispatcher()->RegisterIdle(this, aFrame, kCPIdleFrequency);
		
		CATCH_ALL
		
			aFrame = kXMPNULL;

		ENDTRY
	}
	
	// We should minimize the frame shape here, but we can't because I think that
	// the fPart and fPartInfo for the frame hasn't been set up yet.
	// this->MinimizeFrameShape(dispFrame);	
}


void ClockPart::CommonInitClock()
{				
	gSession = fSession;

	fSelectionFocus = fSession->Tokenize(kXMPSelectionFocus);
	fMenuFocus = fSession->Tokenize(kXMPMenuFocus);
	fKeyFocus = fSession->Tokenize(kXMPKeyFocus);
	fModalFocus = fSession->Tokenize(kXMPModalFocus);
	
	fDisplayFrames = new OrderedCollection;		// Create an empty collection for storing the list 
												// of frames to display	the "clock" in.	
	fCrossCursor = **GetCursor(crossCursor);	// Store the cursors used by the part

#ifdef ASLMBUILD
XMPSLong		clockLibRefNum;
TLibraryFile*	clockLibFile = GetLocalLibraryFile();
clockLibFile->Preflight(clockLibRefNum);
#endif // ASLMBUILD

	fFontMenu = NewMenu(kXMPClockPart_FontMenuID,"\pFont");
	fSizeMenu = NewMenu(kXMPClockPart_SizeMenuID,"\pSize");
	fStyleMenu = GetMenu(kXMPClockPart_StyleMenuID);
	DetachResource((Handle) fStyleMenu);
	fSettingsMenu = GetMenu(kXMPClockPart_SettingsMenuID);
	DetachResource((Handle) fSettingsMenu);
	if (!fFontMenu || !fSizeMenu || !fStyleMenu || !fSettingsMenu)
		DebugStr("\pError creating menus");
		
	AddResMenu( fFontMenu, 'FOND' );

	AppendMenu(fSizeMenu,"\p9");				//   Move to resources for localization
	AppendMenu(fSizeMenu,"\p10");
	AppendMenu(fSizeMenu,"\p11");
	AppendMenu(fSizeMenu,"\p12");
	AppendMenu(fSizeMenu,"\p14");
	AppendMenu(fSizeMenu,"\p18");
	AppendMenu(fSizeMenu,"\p24");
	AppendMenu(fSizeMenu,"\p36");

	fMenuBar = fSession->GetWindowState()->CopyBaseMenuBar();
	fMenuBar->AddMenuLast(kXMPClockPart_FontMenuID, fFontMenu, this);
	fMenuBar->AddMenuLast(kXMPClockPart_SizeMenuID, fSizeMenu, this);
	fMenuBar->AddMenuLast(kXMPClockPart_StyleMenuID, fStyleMenu, this);
	fMenuBar->AddMenuLast(kXMPClockPart_SettingsMenuID, fSettingsMenu, this);

#ifdef ASLMBUILD
clockLibFile->Postflight(clockLibRefNum);
#endif // ASLMBUILD

	fFocusSet = new XMPFocusSet();
	fFocusSet->InitFocusSet();
	fFocusSet->Add(fKeyFocus);
	fFocusSet->Add(fMenuFocus);
	fFocusSet->Add(fSelectionFocus);
}

	
ClockPart::~ClockPart()
{
	XMPDeleteObject(fDisplayFrames);// Release the display frames collection.
									// Deletion of memory used by the object will be
									// reclaimed when the shell deletes the
									// instantiation.
									
	XMPDeleteObject(fSemanticInterface);		// Release the SemanticInterface extension.
	XMPReleaseObject(fMenuBar);
	XMPReleaseObject(fTestClockSU);
}
	
	

//-------------------------------------------------------------------------
// Getters/Setters
//-------------------------------------------------------------------------
	
void ClockPart::SetFont( XMPSShort fontNum )
{
	if (fFont != fontNum) {
		fFont = fontNum;
		this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
	}
}

void ClockPart::SetFontSize( XMPSShort fontSize )
{
	if (fSize != fontSize) {
		fSize = fontSize;
		this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
	}
}

void ClockPart::SetFontStyle( XMPUByte fontStyle )
{
	if (fStyle != fontStyle) {
		fStyle = fontStyle;
		this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
	}
}


//-------------------------------------------------------------------------
// Implementation for DragAndDrop protocol:
//
//		The Drag and Drop protocol provides facilities for interaction with
//		the user during the dragging of Jedi objects. Only part handlers 
//		that support embedding of content need to implement the following
//		methods.
//
//		The Clock Part implements the ::DragEnter, ::DragLeave, and ::Drop
//		methods only to provide the user a bit of feedback while dragging
//		objects.
//
//-------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ClockPart::FulfillPromise
//------------------------------------------------------------------------------


void ClockPart::FulfillPromise(XMPStorageUnitView *promiseSUView)
{
XMPUnused(promiseSUView);
	// Clock Part does not allow embedding of data.
	// This method is not needed.
}


//------------------------------------------------------------------------------
// ClockPart::DropCompleted
//------------------------------------------------------------------------------


void ClockPart::DropCompleted( XMPPart* 		destPart, 
							   XMPDropResult 	dropResult )
{
XMPUnused(destPart);
XMPUnused(dropResult);

	// Clock Part does not allow embedding of data.
	// This method is not needed.
}


//------------------------------------------------------------------------------
// ClockPart::DragEnter 
//------------------------------------------------------------------------------


void ClockPart::DragEnter( XMPDragItemIterator* dragInfo, 
					  XMPFacet*            facet, 
					  XMPPoint 			   where ) 
{
XMPUnused(dragInfo);
XMPUnused(facet);
XMPUnused(where);

	SetCursor(&fCrossCursor);
}


//------------------------------------------------------------------------------
// ClockPart::DragWithin 
//------------------------------------------------------------------------------


void ClockPart::DragWithin( XMPDragItemIterator*	dragInfo, 
					   XMPFacet*            facet,
					   XMPPoint 		    where ) 
{
XMPUnused(dragInfo);
XMPUnused(facet);
XMPUnused(where);

	// Clock Part does not allow embedding of data.
	// This method is not needed.
}


//------------------------------------------------------------------------------
// ClockPart::DragLeave  
//------------------------------------------------------------------------------


void ClockPart::DragLeave( XMPFacet*     facet, 
					  XMPPoint		where ) 
{
XMPUnused(facet);
XMPUnused(where);

	SetCursor(&fArrowCursor);
}


//------------------------------------------------------------------------------
// ClockPart::Drop 
//------------------------------------------------------------------------------


XMPDropResult
ClockPart::Drop( XMPDragItemIterator* dropInfo, 
				 XMPFacet*            facet,
				 XMPPoint 			  where ) 
{
XMPUnused(dropInfo);
XMPUnused(facet);
XMPUnused(where);

	SysBeep(1);
	
	return kXMPDropFail;
}


//-------------------------------------------------------------------------
// Implementation for Frame protocol:
//
//		The Frame protocol...
//
//-------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ClockPart::RemoveDisplayFrame 
//------------------------------------------------------------------------------


void ClockPart::RemoveDisplayFrame( XMPFrame* oldFrame )
{
	if ( fDisplayFrames -> Contains( oldFrame ))
	{	
		ClockPartInfoRec* pInfo = (ClockPartInfoRec*) oldFrame->GetPartInfo();
		oldFrame->SetPartInfo((XMPInfoType) kXMPNULL);
		delete pInfo;

		fSession->GetArbitrator()->RelinquishFocusSet(fFocusSet,oldFrame);
		fDisplayFrames -> Remove( oldFrame );

		fSession -> GetDispatcher() -> UnregisterIdle( this, oldFrame );
		oldFrame -> Release();
	}
}

//------------------------------------------------------------------------------
// ClockPart::AddDisplayFrame 
//------------------------------------------------------------------------------



void ClockPart::AddDisplayFrame(XMPFrame* frame)
{
	if ( frame -> GetPart() == this ) 
	{	
		ClockPartInfoRec* pInfo = new ClockPartInfoRec;
		if (frame->IsRoot())
			pInfo->fNeedsActivating = kXMPTrue;
		frame->SetPartInfo((XMPInfoType) pInfo);
		frame->SetFrameGroup(kXMPClockFrame); //!!! Should use viewType. Gets changed later for dialogs
		fDisplayFrames -> AddLast( frame );
		
		frame -> IncrementRefCount();
		
		fSession -> GetDispatcher() -> RegisterIdle( this, frame, kCPIdleFrequency );
	}
}


//------------------------------------------------------------------------------
// ClockPart::CloseDisplayFrame
//------------------------------------------------------------------------------


void ClockPart::CloseDisplayFrame(XMPFrame* frame)
{
	if ( fDisplayFrames -> Contains( frame ))
	{	
		ClockPartInfoRec* pInfo = (ClockPartInfoRec*) frame->GetPartInfo();
		frame->SetPartInfo((XMPInfoType) kXMPNULL);
		delete pInfo;
		fSession->GetArbitrator()->RelinquishFocusSet(fFocusSet, frame);
		fDisplayFrames -> Remove( frame );

		fSession -> GetDispatcher() -> UnregisterIdle( this, frame );
		frame -> Release();
	}
}

//------------------------------------------------------------------------------
// ClockPart::FacetAdded 
//------------------------------------------------------------------------------


void ClockPart::FacetAdded(XMPFacet* facet)
{
	this -> MinimizeFrameShape( facet->GetFrame() );
}

//------------------------------------------------------------------------------
// ClockPart::FrameShapeChanged 
//------------------------------------------------------------------------------


void ClockPart::FrameShapeChanged(XMPFrame* frame)
{
	// don't need to change used if same as frameShape
	// - just pass null to notify dependents
	frame->ChangeUsedShape(kXMPNULL);
}


//------------------------------------------------------------------------------
// ClockPart::WritePartInfo
//------------------------------------------------------------------------------


void ClockPart::WritePartInfo( XMPPtr 			partInfo, 
						  XMPStorageUnitView*	storageUnitView)
{
	if (partInfo)
	{
		XMPBoolean needsActivating = ((ClockPartInfoRec*)partInfo)->fNeedsActivating 
									|| ((ClockPartInfoRec*)partInfo)->fIsActive;
		storageUnitView->SetValue(sizeof(XMPBoolean),
									(XMPValue)&needsActivating);
	}

//XMPSShort		fFont;
//XMPUByte		fStyle;
//XMPSShort		fMode;
//XMPSShort		fSize;
//XMPSShort		fJustification;
}


//------------------------------------------------------------------------------
// ClockPart::ReadPartInfo
//------------------------------------------------------------------------------


XMPPtr ClockPart::ReadPartInfo(XMPFrame* frame, XMPStorageUnitView*	storageUnitView)
{
XMPUnused(frame);
	if (storageUnitView->GetSize())
	{
		ClockPartInfoRec* partInfo = new ClockPartInfoRec;
		
		XMPBoolean needsActivating;
		storageUnitView->GetValue(sizeof(XMPBoolean),
									(XMPValue)&(needsActivating));
		partInfo->fNeedsActivating = needsActivating;
										
		return partInfo;
	}
	else
		return ((XMPPtr)kXMPNULL);
}


//------------------------------------------------------------------------------
// ClockPart::Open
//------------------------------------------------------------------------------
 
	
XMPID ClockPart::Open( XMPFrame*  frame )
{	
	XMPWindow* window = kXMPNULL;

	if (frame) // Doing a View As Window
	{
		window = fSession->GetWindowState()->GetWindow(fWindowID);
		if (window)
			window->Select();
		else
		{
			window = this->CreateWindow(frame);
			fWindowID = window->GetID();
			window->Open();
			window->Show();
			window->Select();			
		}
	}
	else
	{
		window = this->CreateWindow(frame);
		fWindowID = window->GetID();
		window->Open();
		window->Show();
		window->Select();			
	}
	return window->GetID();
}

XMPWindow* ClockPart::CreateWindow(XMPFrame* sourceFrame)
{
	Rect windRect;
	XMPPlatformWindow platformWindow = kXMPNULL;
	XMPWindow* window = kXMPNULL;
	
	/*if (sourceFrame)
	{
		RgnHandle framergn = sourceFrame->GetFrameShape()->GetQDRegion();
		windRect = (**framergn).rgnBBox;
	}
	else
	*/
	SetRect(&windRect, 4, 40, 204, 100);	
	
	platformWindow = NewCWindow(kXMPNULL, &windRect, "\pClock", false, zoomDocProc, (WindowPtr)-1L, true, kXMPNULL);
	window =  fSession->GetWindowState()->
				CreateWindow(platformWindow, 
							(sourceFrame==kXMPNULL),	// is root
							kXMPTrue,			// Is resizable
							kXMPFalse,			// Is floating
							kXMPTrue,			// should save
							this, 
							(fSession->Tokenize(kXMPViewAsFrame)), 
							kXMPNullTypeToken, 
							sourceFrame);
	
	if (!sourceFrame)
	{
		XMPRect* bounds = kXMPNULL;
		XMPShape* wantShape = this->CalcActualClockShape(platformWindow);
		wantShape->GetBoundingBox(bounds);
		
		bounds->AsQDRect(windRect);
		
		SizeWindow(platformWindow,windRect.right, windRect.bottom,kXMPFalse);
		
		delete wantShape;
		delete bounds;
	}
	return window;
}


//-------------------------------------------------------------------------
// Implementation for Imaging protocol
//-------------------------------------------------------------------------
	
//------------------------------------------------------------------------------
// ClockPart::Draw
//------------------------------------------------------------------------------
 
	
void ClockPart::Draw( XMPFacet* facet, 
				 XMPShape* invalShape )
{
XMPUnused(invalShape);

	if (facet->GetFrame()->GetFrameGroup() != kXMPClockFrame) // Should be using viewType
		return;

	XMPULong		tTickCount;		// ticks since 1/1/04
	Str255			tTimeString;	// pascal string holding date to draw
	
	CFocus			focus(facet);	// Set port, origin, clip
	
	TextFont( fFont );
	TextFace( fStyle );
	TextMode( fMode );
	TextSize( fSize );
	
	GetDateTime( &tTickCount );
	
	fLastTime = tTickCount;
	
	IUTimeString( tTickCount, kXMPTrue, tTimeString );
	
	if (!fAscent)
	{
			FontInfo* fontIRec = (FontInfo *)NewPtr(sizeof(FontInfo));
	
			GetFontInfo( fontIRec );
			fAscent = fontIRec->ascent;
	}

	MoveTo( 2, (fAscent + 1) );
	DrawString( tTimeString );
	
	DrawString("\p    ");
	
	TextFace( normal );
	TextFont( geneva );
	TextMode( srcOr );
	TextSize( 12 );
}


//-------------------------------------------------------------------------
// Implementation for Linking protocol
//-------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ClockPart::CreateLink
//------------------------------------------------------------------------------


XMPLink*
ClockPart::CreateLink(XMPPtr data, XMPULong size)
{
XMPUnused(data);
XMPUnused(size);

	return nil;
}


//------------------------------------------------------------------------------
// ClockPart::LinkUpdated
//------------------------------------------------------------------------------


void ClockPart::LinkUpdated( XMPLink*  updatedLink, XMPChangeID id )
{
XMPUnused(updatedLink);
XMPUnused(id);

	// does not support linking
}



//-------------------------------------------------------------------------
// Implementation for Memory Management protocol
//-------------------------------------------------------------------------

XMPSize
ClockPart::Purge( XMPSize  size )
{
	XMPStorageUnit* su = this->GetStorageUnit();
	return su -> Purge( size );
}


//-------------------------------------------------------------------------
// Implementation for Storage Management protocol
//-------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ClockPart:: GetOrderedFrameCollection
//------------------------------------------------------------------------------


extern void GetOrderedFrameCollection(OrderedCollection* aCollection,XMPStorageUnit* vfSU);

//------------------------------------------------------------------------------
// ClockPart:: PutOrderedFrameCollection
//------------------------------------------------------------------------------


extern void PutOrderedFrameCollection(OrderedCollection* aCollection,XMPStorageUnit* vfSU);

//------------------------------------------------------------------------------
// ClockPart:: Externalize
//------------------------------------------------------------------------------



void ClockPart::Externalize()
{
	SimplePart::Externalize();

	XMPFrame* aFrame;
	XMPStorageUnitRef aSURef;
	XMPULong offset, offsetLimit;
	XMPStorageUnit* su = fTestClockSU;

	su->Focus(kXMPPropClockFont,kXMPPosSame,kXMPSShort,1,kXMPPosFirstSib);
	su->SetValue(sizeof(XMPSShort), (XMPValue)&fFont);
	su->Focus(kXMPPropClockStyle,kXMPPosSame,kXMPSShort,1,kXMPPosFirstSib);
	su->SetValue(sizeof(XMPSShort), (XMPValue)&fStyle);
	su->Focus(kXMPPropClockSize,kXMPPosSame,kXMPSShort,1,kXMPPosFirstSib);
	su->SetValue(sizeof(XMPSShort), (XMPValue)&fSize);
	
	OrderedCollectionIterator aIter(fDisplayFrames);
	su->Focus(kXMPPropDisplayFrames,kXMPPosUndefined,0,1,kXMPPosFirstSib);
	offsetLimit = su->GetSize();
	offset = 0;
	for (aFrame = (XMPFrame*)aIter.First(); aIter.IsNotComplete();
											aFrame = (XMPFrame*)aIter.Next(),
											offset+=sizeof(XMPStorageUnitRef))
	{
		aSURef = su->GetWeakStorageUnitRef(aFrame->GetStorageUnit());
		su->SetOffset(offset);
		su->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
	}
	if (offset < offsetLimit)
		su->DeleteValue(offsetLimit - offset);
		
	su = this->GetStorageUnit();
	su->Focus(kXMPPropContents,kXMPPosSame,kXMPKindTestClock,1,kXMPPosFirstSib);
	aSURef = su->GetStrongStorageUnitRef(fTestClockSU);
	su->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
}


//-------------------------------------------------------------------------
// Implementation for Part Activation protocol
//-------------------------------------------------------------------------
	
//------------------------------------------------------------------------------
// ClockPart::BeginRelinquishFocus
//------------------------------------------------------------------------------


XMPBoolean ClockPart::BeginRelinquishFocus( XMPTypeToken 	focus,
									 XMPFrame* 		currentFrame,
									 XMPFrame* 		proposedFrame )
{
XMPUnused(focus);
XMPUnused(currentFrame);
XMPUnused(proposedFrame);

	if ((focus == fModalFocus) && (proposedFrame->GetPart() != this))
		return kXMPFalse;
	else
		return kXMPTrue;
}
	
	
//------------------------------------------------------------------------------
// ClockPart::CommitRelinquishFocus
//------------------------------------------------------------------------------


void ClockPart::CommitRelinquishFocus( XMPTypeToken 	focus,
							XMPFrame* 		currentFrame,
							XMPFrame*		proposedFrame )
{
	if (proposedFrame->GetPart() != this)
	this->FocusLost(focus, currentFrame);
}
	
							
//------------------------------------------------------------------------------
// ClockPart::AbortRelinquishFocus
//------------------------------------------------------------------------------


void ClockPart::AbortRelinquishFocus( XMPTypeToken	focus,
								  XMPFrame*		currentFrame,
								  XMPFrame*		proposedFrame )
{
XMPUnused(focus);
XMPUnused(currentFrame);
XMPUnused(proposedFrame);
	// default is do nothing
}
			
							  
//------------------------------------------------------------------------------
// ClockPart::FocusAcquired
//------------------------------------------------------------------------------


void ClockPart::FocusAcquired( XMPTypeToken	focus, 
						  XMPFrame*		newOwner )
{
	if (focus == fSelectionFocus) 
	{
		ClockPartInfoRec* pInfo = (ClockPartInfoRec*) newOwner->GetPartInfo();
		pInfo->fIsActive = kXMPTrue;
	}
	else if (focus == fMenuFocus) 
		this->InstallMenus(newOwner);
}	


//------------------------------------------------------------------------------
// ClockPart::FocusLost
//------------------------------------------------------------------------------


void ClockPart::FocusLost( XMPTypeToken	focus, 
					  XMPFrame* 	oldOwner )
{
	if (focus == fSelectionFocus) 
	{	// !!! unhighlight selection...
		ClockPartInfoRec* pInfo = (ClockPartInfoRec*) oldOwner->GetPartInfo();
		pInfo->fIsActive = kXMPFalse;

		XMPWindow* settingsWindow = fSession->GetWindowState()->GetWindow(fSettingsWindow);
		if (settingsWindow && settingsWindow->IsShown())
		{
			fSettingsWindowWasShown = kXMPTrue;
			settingsWindow->Hide();
		}
	}
	else if (focus == fMenuFocus) 
		this->RemoveMenus( oldOwner );

}	


//-------------------------------------------------------------------------
// Implementation for UI Events protocol
//-------------------------------------------------------------------------
	
//------------------------------------------------------------------------------
// ClockPart::HandleEvent
//------------------------------------------------------------------------------


XMPBoolean ClockPart::HandleEvent( XMPEventData event,
						XMPFrame* frame,
						XMPFacet* facet )
{
	XMPBoolean	tWasHandled = kXMPFalse;
	
	switch ( event -> what ) {
		case nullEvent:
			{
				this -> Idle( frame );
				if (IsDialogEvent(event))
				{
					short itemHit;
					DialogPtr dialog;
					tWasHandled = DialogSelect(event, &dialog, &itemHit);
					
					return tWasHandled;
				}
				tWasHandled = kXMPTrue;
				break;
			}
			
		case mouseDown:
			{
				tWasHandled = this -> HandleMouseDown(event, frame, facet );
			}
			break;
			
		case kXMPEvtMouseDownEmbedded:
		case mouseUp:
		case keyDown:
		case autoKey:
		case activateEvt:
			tWasHandled = true; // actually ignored by dispatcher
			if ((event->modifiers & activeFlag ) != 0)
				this->ActivatingWindow(frame);
			else
				this->DeActivatingWindow(frame);
			break;
		case keyUp:
		case osEvt:
		case kHighLevelEvent:
			break;
		case kXMPEvtMenu:
			{
				tWasHandled = this -> HandleMenuEvent( frame, event );
				break;
			}
		case kXMPEvtWindow: 
			{
				switch (event->message)
				{
					case inGoAway:
						XMPWindow*	window = facet->GetWindow();
						XMPWindow* settingsWindow = fSession->GetWindowState()->GetWindow(fSettingsWindow);
						if (window == settingsWindow) {
							settingsWindow->Hide();
						}
						break;
					case inZoomIn:
					case inZoomOut:
						SysBeep(2); 
						tWasHandled = kXMPTrue;
						break;
					default:
						break;
				}
			}
			break;
		default:
			break;
	}
	
	return tWasHandled;
}

		
//------------------------------------------------------------------------------
// ClockPart::MouseEnterFrame
//------------------------------------------------------------------------------


void ClockPart::MouseEnter( XMPFacet* facet, 
							XMPPoint  where )
{
	XMPUnused(facet);
	XMPUnused(where);
	//
	SetCursor(&fCrossCursor);
}

//------------------------------------------------------------------------------
// ClockPart::MouseWithin
//------------------------------------------------------------------------------


void ClockPart::MouseWithin( XMPFacet* facet, 
							XMPPoint  where )
{
	XMPUnused(facet);
	XMPUnused(where);
}

		
//------------------------------------------------------------------------------
// ClockPart::MouseLeaveFrame
//------------------------------------------------------------------------------


void ClockPart::MouseLeave( XMPFacet* facet)
{
	XMPUnused(facet);
	
	SetCursor(&fArrowCursor);
	//
}

//------------------------------------------------------------------------------
// ClockPart::AdjustMenus
//------------------------------------------------------------------------------


void ClockPart::AdjustMenus( XMPFrame* frame)
{
	XMPUnused(frame);
	fMenuBar->EnableCommand(kXMPCommandViewAsWin, !frame->IsRoot());
	fMenuBar->EnableCommand(kXMPCommandShowPartsBin, kXMPTrue);
	fMenuBar->SetItemString(kXMPCommandAbout, "\pAbout Clock");
	
	if (fPartsBinWin)
	{
		XMPWindow* window = fSession->GetWindowState()->GetWindow(fPartsBinWin);
		if (window) 
			fPartsBinOpen = window->IsShown();
		else
			fPartsBinOpen = kXMPFalse;		// $$$$$ SHOULD NEVER HAPPEN - BAD!!!!
	}
	
	if (fPartsBinOpen)
		fMenuBar->SetItemString(kXMPCommandShowPartsBin, "\pHide Parts Bin");
	else
		fMenuBar->SetItemString(kXMPCommandShowPartsBin, "\pShow Parts Bin");
}


//------------------------------------------------------------------------------
// ClockPart::Idle
//------------------------------------------------------------------------------


pascal Boolean MyDialogFilter(DialogPtr dialog,
							  EventRecord* event,
							  short* itemHit)
{
	XMPBoolean handled = kXMPFalse;
	
	if (event->what == nullEvent)
	{
		gSession->GetDispatcher()->Dispatch(event);
	}
	else if (event->what == updateEvt &&
				(WindowPtr) event->message != dialog)		
	{
		gSession->GetDispatcher()->Dispatch(event);
	}
	else if (event->what == activateEvt &&
				(WindowPtr) event->message != dialog)		
	{
		gSession->GetDispatcher()->Dispatch(event);
	}
	else if ((event->what == keyDown) || (event->what == autoKey))
	{
		#define kMyButtonDelay 8
		long	waitTicks;
		short	itemKind;
		Handle	itemHandle;
		Rect	itemRect;
		char key = (char)(event->message & charCodeMask);
		const char kReturnKey = 13;
		const char kEnterKey = 3;
		const char kEscKey = 27;
		
		switch(key)
		{
			case kReturnKey:
			case kEnterKey:
				/* change whatever is the current item is to the OK item */
				*itemHit = ok;
				GetDItem(dialog,ok,&itemKind,&itemHandle,&itemRect);
				HiliteControl((ControlHandle)itemHandle,inButton);
				Delay(kMyButtonDelay,&waitTicks);/* wait about 8 ticks */
				HiliteControl((ControlHandle)itemHandle,kXMPFalse);
				handled = kXMPTrue;
				break;
			case kEscKey:
				*itemHit = cancel;
				GetDItem(dialog,cancel,&itemKind,&itemHandle,&itemRect);
				HiliteControl((ControlHandle)itemHandle,inButton);
				Delay(kMyButtonDelay,&waitTicks);
				HiliteControl((ControlHandle)itemHandle,kXMPFalse);
				handled = kXMPTrue;
				break;
		}
	}

	return handled;
}

void ClockPart::Idle( XMPFrame*  frame )
{
	XMPBoolean	tWasHandled = kXMPFalse;
	XMPULong	tTickCount = 0;

	GetDateTime( &tTickCount );
	if ( (tTickCount + 60) > fLastTime )
	{
		XMPFrameFacetIterator* facets = frame->CreateFacetIterator();
		for ( XMPFacet* facet = facets->First();
				facets->IsNotComplete();
				facet = facets->Next() )
		{
			this -> Draw( facet, kXMPNULL );
		}
		delete facets;
	}


	if (fAlarmOn &&  (tTickCount % 10 == 0) && (frame->GetFrameGroup() == kXMPClockFrame)) // Should be using viewType
	{
		XMPFrame* focusFrame = fSession->GetArbitrator()->GetFocusOwner(fModalFocus);
		/*
		if (focusFrame)
			SysBeep(2);
		else if (fSession->GetArbitrator()->RequestFocus(fModalFocus,frame))
		{
			ParamText("\pWake Up!",0,0,0);
			ModalFilterUPP modalFilter = NewModalFilterProc(MyDialogFilter);
			Alert(kXMPClockPart_AboutBoxID, modalFilter);
			DisposeRoutineDescriptor(modalFilter);
			fSession->GetArbitrator()->RelinquishFocus(fModalFocus,frame);			
		}
		else
			SysBeep(2);
		*/
		if (focusFrame == frame)
			SysBeep(2);
		else if (fSession->GetArbitrator()->RequestFocus(fModalFocus,frame))
		{
			ParamText("\pWake Up!",0,0,0);
			ModalFilterUPP modalFilter = NewModalFilterProc(MyDialogFilter);
			Alert(kXMPClockPart_AboutBoxID, modalFilter);
			DisposeRoutineDescriptor(modalFilter);
			fSession->GetArbitrator()->TransferFocus(fModalFocus,frame, focusFrame);			
		}
		else
			SysBeep(2);
		
	}
}


//------------------------------------------------------------------------------
// ClockPart::HandleMouseDown
//------------------------------------------------------------------------------


XMPBoolean ClockPart::HandleMouseDown( XMPEventData event, XMPFrame* frame, XMPFacet* facet )
{
	XMPBoolean tWasHandled = kXMPFalse;
	
	if (facet == kXMPNULL) // Outside modal focus
	{
		SysBeep(2);
		tWasHandled = kXMPTrue;
	}
	else if (IsDialogEvent(event))
	{
		short itemHit;
		DialogPtr dialog;
		tWasHandled = DialogSelect(event, &dialog, &itemHit);
		XMPWindow* window = fSession->GetWindowState()->GetXMPWindow(dialog);
		
		if (frame->GetFrameGroup() == kXMPSynchronizeFrame)
		{
			if (itemHit == cancel)
			{
				fSession->GetArbitrator()->RelinquishFocus(fModalFocus, frame);
				window->CloseAndRemove();
				//!!! Reenable entire Menus. MenuBar object needs more support for this
				
				fMenuBar->EnableCommand( kXMPCommandAbout, kXMPTrue ) ;
				EnableItem(GetMHandle(129), 0);
				EnableItem(GetMHandle(130), 0);
				EnableItem(GetMHandle(kXMPClockPart_FontMenuID), 0);
				EnableItem(GetMHandle(kXMPClockPart_SizeMenuID), 0);
				EnableItem(GetMHandle(kXMPClockPart_StyleMenuID), 0);
				EnableItem(GetMHandle(kXMPClockPart_SettingsMenuID), 0);
				InvalMenuBar();
			}
		}
		else if(frame->GetFrameGroup() == kXMPAlarmSettingsFrame)
		{
			XMPSShort itemType,checkbox;
			Handle		itemHandle;
			Rect		scratchRect;
			
			if (itemHit == kXMPClockPart_SettingsCheckBoxID)
			{
				GetDItem(dialog, itemHit, &itemType, &itemHandle, &scratchRect);
				checkbox = GetCtlValue((ControlHandle)itemHandle);
				checkbox = 1 - checkbox;
				SetCtlValue((ControlHandle)itemHandle,checkbox);
				fAlarmOn = !fAlarmOn;
			}
		}
	}
	else
	{
		XMPBoolean tWasHandled = kXMPTrue;
		if (!facet->GetWindow()->IsActive())
			facet->GetWindow()->Select();
		else
		{
			this->ActivateFrame(facet->GetFrame());

			XMPWindow* settingsWindow = fSession->GetWindowState()->GetWindow(fSettingsWindow);
			if (settingsWindow && fSettingsWindowWasShown)
				settingsWindow->Show();
		}
				
	}
	return tWasHandled;
}


//------------------------------------------------------------------------------
// ClockPart::HandleMenuEvent
//------------------------------------------------------------------------------


XMPBoolean ClockPart::HandleMenuEvent( XMPFrame*		frame,
							XMPEventData 	event )
{
	XMPBoolean tEventHandled = kXMPTrue;

	long menuResult = event->message;
	short menu = HiWord(menuResult);
	short item = LoWord(menuResult);
	XMPCommandID command = fMenuBar->GetCommand(menu, item);
	
	if (command == kXMPCommandAbout) 
	{
#ifdef ASLMBUILD
XMPSLong		clockLibRefNum;
TLibraryFile*	clockLibFile = GetLocalLibraryFile();
clockLibFile->Preflight(clockLibRefNum);
#endif // ASLMBUILD
		if (fSession->GetArbitrator()->RequestFocus(fModalFocus,frame))
		{
			ParamText("\pClock Part",0,0,0);
			SetCursor(&XMPASLMQDGlobals.arrow);
			ModalFilterUPP modalFilter = NewModalFilterProc(MyDialogFilter);
			Alert(kXMPClockPart_AboutBoxID, modalFilter);
			DisposeRoutineDescriptor(modalFilter);
			//HiliteMenu(0);
			fSession->GetArbitrator()->RelinquishFocus(fModalFocus,frame);
		}
		else
			SysBeep(2);
#ifdef ASLMBUILD
clockLibFile->Postflight(clockLibRefNum);
#endif // ASLMBUILD
		return kXMPTrue;
	}
	else if (command == kXMPCommandShowPartsBin)
	{
		if (fPartsBinOpen)
			this->HidePartsBin();
		else
			this->ShowPartsBin(frame);
	}
	else if (command == kXMPCommandViewAsWin)
	{
		this->Open(frame);
	}
	else if (command == kXMPCommandClose)
	{
		XMPWindow* settingsWindow = fSession->GetWindowState()->GetWindow(fSettingsWindow);
		if (settingsWindow->GetRootFrame() == frame) {
			settingsWindow->Hide();
		}
	}
	else
	{			
		tEventHandled = kXMPTrue;
		
		switch ( menu ) {
			case kXMPClockPart_FontMenuID:	this -> HandleFontMenu( frame, item ); break;
			case kXMPClockPart_SizeMenuID:	this -> HandleSizeMenu( item ); break;
			case kXMPClockPart_StyleMenuID:	this -> HandleStyleMenu(item ); break;
			case kXMPClockPart_SettingsMenuID:	this -> HandleSettingsMenu(item ); break;
			default:
				tEventHandled = kXMPFalse;
		}
		
		if (tEventHandled)							// if change made, resize the frame
			this -> MinimizeFrameShape( frame );
	}
	
	return tEventHandled;
}


//------------------------------------------------------------------------------
// ClockPart::HandleFontMenu
//------------------------------------------------------------------------------


void ClockPart::HandleFontMenu( XMPFrame*	frame,
						   XMPUShort	item )
{
XMPUnused(frame);

	Str255		fontName;
	XMPSShort	fontNum = 0;

	MenuHandle clockMenuH = GetMHandle( kXMPClockPart_FontMenuID );

	GetFontName( fFont, fontName );
	this -> CheckMarkMenu( clockMenuH, fontName, kXMPFalse );
	
	GetItem( clockMenuH, item, fontName );
	
	GetFNum( fontName, &fontNum );
	this -> SetFont( fontNum );
	this -> CheckMarkMenu( clockMenuH, fontName, kXMPTrue );
	this -> OutlineSizeMenu();
}


//------------------------------------------------------------------------------
// ClockPart::HandleSizeMenu
//------------------------------------------------------------------------------


void ClockPart::HandleSizeMenu( XMPUShort  item )
{
	XMPSShort	fontSize = 0;
	
	MenuHandle clockMenuH = GetMHandle( kXMPClockPart_SizeMenuID );
	Str255	tSizeStr;
	
	NumToString( (XMPULong)fSize, tSizeStr );
	this -> CheckMarkMenu( clockMenuH, tSizeStr, kXMPFalse );
	
	switch (item) {
		case kXMPClockPart_Size_9Pt:	fontSize =  9; break;
		case kXMPClockPart_Size_10Pt:	fontSize = 10; break;
		case kXMPClockPart_Size_11Pt:	fontSize = 11; break;
		case kXMPClockPart_Size_12Pt:	fontSize = 12; break;
		case kXMPClockPart_Size_14Pt:	fontSize = 14; break;
		case kXMPClockPart_Size_18Pt:	fontSize = 18; break;
		case kXMPClockPart_Size_24Pt:	fontSize = 24; break;
		case kXMPClockPart_Size_36Pt:	fontSize = 36; break;
		default:
			fontSize = 12;
	}
	
	this -> SetFontSize( fontSize );

	NumToString( (XMPULong)fontSize, tSizeStr );
	this -> CheckMarkMenu( clockMenuH, tSizeStr, kXMPTrue );
	this -> OutlineSizeMenu();
}


//------------------------------------------------------------------------------
// ClockPart::HandleStyleMenu
//------------------------------------------------------------------------------


void ClockPart::HandleStyleMenu( XMPUShort  item )
{
	XMPUByte	fontStyle = normal;
	
	MenuHandle clockMenuH = GetMHandle( kXMPClockPart_StyleMenuID );
	Str255	tStyleStr;
	
	StyleToString( fStyle, (Str255 *)&tStyleStr);
	this -> CheckMarkMenu( clockMenuH, tStyleStr, kXMPFalse );
	
	switch (item) {
		case kXMPClockPart_Style_Plain:		fontStyle = normal; break;
		case kXMPClockPart_Style_Bold:		fontStyle = bold; break;
		case kXMPClockPart_Style_Italic:	fontStyle = italic; break;
		case kXMPClockPart_Style_Underline:	fontStyle = underline; break;
		case kXMPClockPart_Style_Outline:	fontStyle = outline; break;
		case kXMPClockPart_Style_Shadow:	fontStyle = shadow; break;
		case kXMPClockPart_Style_Condense:	fontStyle = condense; break;
		case kXMPClockPart_Style_Extend:	fontStyle = extend; break;
		default:
			;
	}
	
	this -> SetFontStyle( fontStyle );

	StyleToString( fontStyle, (Str255 *)&tStyleStr);
	this -> CheckMarkMenu( clockMenuH, tStyleStr, kXMPTrue );
}

//------------------------------------------------------------------------------
// ClockPart::HandleSettingsMenu
//------------------------------------------------------------------------------


void ClockPart::HandleSettingsMenu( XMPUShort  item )
{		
	switch (item) 
	{
		case 1:	
			{
			#ifdef ASLMBUILD
				XMPSLong		clockLibRefNum;
				TLibraryFile*	clockLibFile = GetLocalLibraryFile();
				clockLibFile->Preflight(clockLibRefNum);
			#endif // ASLMBUILD
				
				DialogPtr synchDialog = GetNewDialog(kXMPClockPart_SynchronizeDialogID ,0,(WindowPtr)-1L);
				XMPWindow* synchWindow = this->GetStorageUnit()->GetSession()->GetWindowState()->CreateWindow(synchDialog, 
																 kXMPFalse,	// Keeps draft open
																 kXMPFalse,	// Is resizable
																 kXMPFalse,	// Is floating
																 kXMPFalse, // don't save
																 this, kXMPNullTypeToken, kXMPNullTypeToken, kXMPNULL);
			#ifdef ASLMBUILD
				clockLibFile->Postflight(clockLibRefNum);
			#endif // ASLMBUILD
				
				if (synchDialog 
					&& (fSession->GetArbitrator()->RequestFocus(fModalFocus,
								synchWindow->GetRootFrame())))
			
				{
				
					//!!! Disable entire Menus. MenuBar object needs more support for this
					
					fMenuBar->EnableCommand( kXMPCommandAbout, kXMPFalse ) ;
					DisableItem(GetMHandle(129), 0);
					DisableItem(GetMHandle(130), 0);
					DisableItem(GetMHandle(kXMPClockPart_FontMenuID), 0);
					DisableItem(GetMHandle(kXMPClockPart_SizeMenuID), 0);
					DisableItem(GetMHandle(kXMPClockPart_StyleMenuID), 0);
					DisableItem(GetMHandle(kXMPClockPart_SettingsMenuID), 0);
					InvalMenuBar();
					
					
					synchWindow->SetShouldSave(kXMPFalse);
					SetCursor(&XMPASLMQDGlobals.arrow);
					synchWindow->GetRootFrame()->SetFrameGroup(kXMPSynchronizeFrame); // Should use viewType
					synchWindow->Open();
					synchWindow->Show();
					synchWindow->Select();
					
				}
			}
			break;
		case 2:	
			{
				XMPWindow* settingsWindow = fSession->GetWindowState()->GetWindow(fSettingsWindow);

				if (settingsWindow)
				{
					settingsWindow->Select();
					settingsWindow->Show();
				}
				else
				{
				#ifdef ASLMBUILD
					XMPSLong		clockLibRefNum;
					TLibraryFile*	clockLibFile = GetLocalLibraryFile();
					clockLibFile->Preflight(clockLibRefNum);
				#endif // ASLMBUILD
					
					DialogPtr settingsDialog = GetNewDialog(kXMPClockPart_SettingsDialogID ,0,(WindowPtr)-1L);
					settingsWindow = this->GetStorageUnit()->GetSession()->GetWindowState()->CreateWindow(settingsDialog, 
																	 kXMPFalse,	// Keeps draft open
																	 kXMPFalse,	// Is resizable
																	 kXMPFalse,	// Is floating
																	 kXMPFalse,	// don't save
																	 this, kXMPNullTypeToken, kXMPNullTypeToken, kXMPNULL);
				#ifdef ASLMBUILD
					clockLibFile->Postflight(clockLibRefNum);
				#endif // ASLMBUILD
					
					if (settingsDialog)
					{
						settingsWindow->SetShouldSave(kXMPFalse);
						settingsWindow->GetRootFrame()->SetFrameGroup(kXMPAlarmSettingsFrame); // Should use viewType
						settingsWindow->Open();
						settingsWindow->Show();
						settingsWindow->Select();
						fSettingsWindow = settingsWindow->GetID();
					}
				}
			}
			break;
		default:
			break;
	}
}


//-------------------------------------------------------------------------
// SemanticInterface
//-------------------------------------------------------------------------

XMPExtension*
ClockPart::GetExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	if (!XMPISOStrCompare( (XMPISOStr)extensionName, semtIntf ))
		return ( (XMPExtension*)fSemanticInterface );
	else
		return nil;
}


XMPBoolean ClockPart::HasExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	if (!XMPISOStrCompare( (XMPISOStr)extensionName, semtIntf ))
		return ( fHasSemanticInterface );
	else
		return false;
}



//-------------------------------------------------------------------------
// Utilities
//-------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ClockPart::InstallMenus
//------------------------------------------------------------------------------


void ClockPart::InstallMenus( XMPFrame*  frame )
{	
	if ( frame && fMenuBar)
	{	
		fMenuBar->Display();
		fMenuBar->EnableCommand(kXMPCommandShowPartsBin , kXMPTrue);

		this -> OutlineSizeMenu();
		
		Str255	tFontName;
		
		GetFontName( fFont, tFontName );
		this -> CheckMarkMenu( fFontMenu, tFontName, kXMPTrue );
	
		Str255	tSizeStr;
		
		NumToString( (XMPULong)fSize, tSizeStr );
		this -> CheckMarkMenu( fSizeMenu, tSizeStr, kXMPTrue);
		
		Str255	tStyleStr;
		
		StyleToString( fStyle, (Str255 *)&tStyleStr);
		this -> CheckMarkMenu( fStyleMenu, tStyleStr, kXMPTrue );

	}
}


//------------------------------------------------------------------------------
// ClockPart::RemoveMenus
//------------------------------------------------------------------------------


void ClockPart::RemoveMenus(XMPFrame* frame)
{
	XMPUnused(frame);

	fMenuBar->EnableCommand(kXMPCommandShowPartsBin , kXMPFalse);
}



//-------------------------------------------------------------------------
// Utilities
//-------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ClockPart::CheckMarkMenu
//------------------------------------------------------------------------------


void ClockPart::CheckMarkMenu( MenuHandle	menu,
						  Str255		checkItem,
						  XMPBoolean	checkIt )
{
	
	XMPSShort  	numMenuItems = CountMItems( menu );
	Str255  	menuItemString;
	
	for (XMPSShort menuItemNum = 1; menuItemNum <= numMenuItems; menuItemNum++)
	{
		GetItem( menu, menuItemNum, menuItemString );
		
		if ( IUEqualString( checkItem, menuItemString) == 0)
		{
			if ( checkIt )
				CheckItem( menu, menuItemNum, kXMPTrue );
			else
				CheckItem( menu, menuItemNum, kXMPFalse );
		}
	}
}


//------------------------------------------------------------------------------
// ClockPart::StyleToString
//------------------------------------------------------------------------------


void ClockPart::StyleToString( XMPUByte	style,
						  Str255*	styleName )
{
	char tStyleStr[12];
	
	switch (style) {
		case normal:	strcpy((char *)&tStyleStr ,"Plain\0"); break;				//   Move to resource
		case bold:		strcpy((char *)&tStyleStr ,"Bold\0"); break;				//   for localization
		case italic:	strcpy((char *)&tStyleStr ,"Italic\0"); break;				//   purposes
		case underline:	strcpy((char *)&tStyleStr ,"Underline\0"); break;
		case outline:	strcpy((char *)&tStyleStr ,"Outline\0"); break;
		case shadow:	strcpy((char *)&tStyleStr ,"Shadow\0"); break;
		case condense:	strcpy((char *)&tStyleStr ,"Condense\0"); break;
		case extend:	strcpy((char *)&tStyleStr ,"Extend\0"); break;
		default:
			strcpy((char *)&tStyleStr ,"Plain\0");
	}
	
	BlockMove((Ptr)c2pstr((char *)&tStyleStr),(Ptr)styleName,12);
}


//------------------------------------------------------------------------------
// ClockPart::OutlineSizeMenu
//------------------------------------------------------------------------------


void ClockPart::OutlineSizeMenu()
{
	MenuHandle menu = GetMHandle( kXMPClockPart_SizeMenuID );
	
	// () Outline the sizes in the font menu that are available
	
	if (RealFont(fFont, 9))
		SetItemStyle( menu, kXMPClockPart_Size_9Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_9Pt, normal );

	if (RealFont(fFont, 10))
		SetItemStyle( menu, kXMPClockPart_Size_10Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_10Pt, normal );

	if (RealFont(fFont, 11))
		SetItemStyle( menu, kXMPClockPart_Size_11Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_11Pt, normal );

	if (RealFont(fFont, 12))
		SetItemStyle( menu, kXMPClockPart_Size_12Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_12Pt, normal );

	if (RealFont(fFont, 14))
		SetItemStyle( menu, kXMPClockPart_Size_14Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_14Pt, normal );

	if (RealFont(fFont, 18))
		SetItemStyle( menu, kXMPClockPart_Size_18Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_18Pt, normal );

	if (RealFont(fFont, 24))
		SetItemStyle( menu, kXMPClockPart_Size_24Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_24Pt, normal );

	if (RealFont(fFont, 36))
		SetItemStyle( menu, kXMPClockPart_Size_36Pt, outline );
	else
		SetItemStyle( menu, kXMPClockPart_Size_36Pt, normal );
}


//------------------------------------------------------------------------------
// ClockPart::CalcActualClockSize
//------------------------------------------------------------------------------


#define ff(N)		( (XMPCoordinate)(N) <<16 )	// Converts integer to fixed (XMPCoordinate)
	
XMPShape* ClockPart::CalcActualClockShape(XMPPlatformWindow window)
{
	GrafPtr	savePort;
	GetPort(&savePort);
	SetPort(window);

		TextFont( fFont );
		TextFace( fStyle );
		TextMode( fMode );
		TextSize( fSize );
		
		FontInfo fontIRec;
		GetFontInfo( &fontIRec );
		fAscent = fontIRec.ascent;
										//   move to resource for localization
		XMPRect rect( 0, 0,
					  ff(StringWidth("\p12:59:59 MM")+4),
					  ff((fontIRec.ascent+fontIRec.descent+fontIRec.leading)+2) );
		XMPShape *wantShape = new XMPShape;
		wantShape->SetRectangle(&rect);
		
		TextFace( normal );
		TextFont( geneva );
		TextMode( srcOr );
		TextSize( 12 );
					
		SetPort(savePort);
	
	return wantShape;
}

//------------------------------------------------------------------------------
// ClockPart:: MinimizeFrameShape
//------------------------------------------------------------------------------


void ClockPart::MinimizeFrameShape( XMPFrame*  frame )
{
	if (frame->GetFrameGroup() != kXMPClockFrame) //!!! Should use viewType
		return;
	
	XMPFrameFacetIterator* fiter = frame->CreateFacetIterator();
	
	XMPPlatformWindow window = fiter->First()->GetWindow()->GetPlatformWindow();

	XMPShape* wantShape = this->CalcActualClockShape(window);
	
	frame-> Invalidate(NULL);
	frame-> InvalidateActiveBorder();
	
	XMPShape* gotShape = frame -> RequestFrameShape( wantShape );
	frame-> Invalidate(NULL);
	frame-> InvalidateActiveBorder();
	
	frame->ChangeUsedShape(kXMPNULL);
	
	delete fiter;
}
	
//------------------------------------------------------------------------------
// ClockPart::ShowPartsBin
//------------------------------------------------------------------------------


void ClockPart::ShowPartsBin(XMPFrame* frame)
{
	XMPPart*			partsbin;
	XMPStorageUnit*		su = this->GetStorageUnit();
	XMPStorageUnit*		pbSU;
	XMPStorageUnitRef	pbSURef;
	XMPStorageUnitID	pbSUID;
	
	if (!su->Exists(kXMPPropPartsBin,kXMPStrongStorageUnitRef,0))
	{
		su->AddProperty(kXMPPropPartsBin)->AddValue(kXMPStrongStorageUnitRef);

		partsbin = su->GetDraft()->CreatePart(kXMPKindPartsBin, kXMPNULL);
		ASSERTM(partsbin != kXMPNULL, kXMPErrCannotCreatePart, "Cannot Create Parts Bin");
		
		pbSU = partsbin->GetStorageUnit();
		pbSURef = su->GetStrongStorageUnitRef(pbSU);
		su->SetValue(sizeof(kXMPStrongStorageUnitRef), (XMPValue)&pbSURef);
	}
	else
	{
		su->Focus(kXMPPropPartsBin,kXMPPosSame,kXMPStrongStorageUnitRef,1,kXMPPosFirstSib);
		su->SetOffset(0);
		su->GetValue(sizeof(kXMPStrongStorageUnitRef), (XMPValue)&pbSURef);
		
		pbSUID = su->GetIDFromStorageUnitRef(pbSURef);
		partsbin = su->GetDraft()->GetPart(pbSUID);
	}
	
	fPartsBinWin = partsbin->Open(frame);
	partsbin->Release();
	fPartsBinOpen = kXMPTrue;
}

void ClockPart::HidePartsBin()
{
	if (fPartsBinOpen && fPartsBinWin)
	{
		XMPWindow* pbWin = fSession->GetWindowState()->GetWindow(fPartsBinWin);
		ASSERTM(pbWin != kXMPNULL, kXMPErrCannotGetWindow, "Cannot Hide Parts Bin");

		pbWin->Hide();
		fPartsBinOpen = kXMPFalse;
	}
}

	
void ClockPart::ActivateFrame(XMPFrame* frame)
{
	ClockPartInfoRec* pInfo = (ClockPartInfoRec*) frame->GetPartInfo();
	if (!(pInfo->fIsActive))
	{
		XMPBoolean succeeded = kXMPFalse;
							
		succeeded = fSession->GetArbitrator()->RequestFocusSet(fFocusSet,frame);
				
		if (succeeded)
		{
			this->FocusAcquired(fSelectionFocus, frame);
			this->FocusAcquired(fMenuFocus, frame);
			this->FocusAcquired(fKeyFocus, frame);
		}
	}
}

void ClockPart::DeActivateFrame(XMPFrame* frame)
{
	fSession->GetArbitrator()->RelinquishFocusSet(fFocusSet,frame);
	this->FocusLost(fSelectionFocus, frame);
	this->FocusLost(fMenuFocus, frame);
	this->FocusLost(fKeyFocus, frame);
}

void ClockPart::ActivatingWindow(XMPFrame* frame)
{
	ClockPartInfoRec* pInfo = (ClockPartInfoRec*) frame->GetPartInfo();
	if (pInfo->fNeedsActivating)
	{
		this->ActivateFrame(frame);
	}
}

void ClockPart::DeActivatingWindow(XMPFrame* frame)
{
	ClockPartInfoRec* pInfo = (ClockPartInfoRec*) frame->GetPartInfo();
	if (frame == fSession->GetArbitrator()->GetFocusOwner(fSelectionFocus))
	{
		pInfo->fNeedsActivating = kXMPTrue;
	}
	else
		pInfo->fNeedsActivating = kXMPFalse;
}
