/*
	File:		ClockPart.h

	Contains:	Class definition for ClockPart

	Written by:	Steve Smith

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<28>	 3/25/94	VL		#1147173: Moved constructor, destructor and
									init methods to private by convention.
		<27>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<26>	  2/4/94	T		class XMPMacSemanticInterface;
		<25>	  2/4/94	T		DONT #include header files for Class
									Definitions
		<24>	  2/4/94	VL		Removed Shape.h
		<23>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame()
		<22>	 1/27/94	RR		Added private CreateWindow
		<21>	 1/24/94	SS		Additional work on Open method
		<20>	 1/24/94	CC		Added XMPChangeID parameter to
									LinkUpdated().
		<19>	 1/20/94	SS		::Open returns a window id
		<18>	 1/15/94	RR		WinState.h->WinStat.h,
									Dispatch.h->Disptch.h
		<17>	 1/11/94	T		Init... changes
		<16>	12/22/93	RR		Changed fSettingsWindow to type XMPID
		<15>	12/21/93	JBS		CreateFrame with viewType & presentation
		<14>	12/20/93	SS		Added PartsBin support
		<13>	12/16/93	RR		Added fields for settings dialogs etc.
		<12>	12/13/93	T		InitPart changes
		<11>	12/13/93	RR		Added fields for wake-up dialog
									(Implementation not yet checked in)
		<10>	11/30/93	LM		Added an XMPFrame parameter to ReadPartInfo
		 <9>	11/23/93	JA		Updated for new Shape API.
		 <8>	11/22/93	T		modifications to use correct part storage
									model and moved defines to ClockDef.h
		 <7>	11/16/93	SS		Added "FacetAdded" method
		 <6>	11/16/93	RR		Added AdjustMenus (not yet called), and
									MouseWithin. Removed point from MouseLeave
		 <5>	11/15/93	CC		Added CreateLink() and LinkUpdated()
		 <4>	11/12/93	JBS		use Facets
		 <3>	10/21/93	PH		Change FulfillPromise call
		 <2>	10/14/93	RR		Added sourceFrame parameter to
									AddDisplayFrame
		<19>	 9/30/93	PH		Support new Drag and Drop protocol
		<18>	 9/30/93	RR		Added fFocusSet
		<17>	 9/29/93	VL		Added InitializePersistent and removed
									InitStructFields.
		<16>	 9/16/93	RR		Added fMenuBar
		<15>	 8/20/93	CG		Removed all SemtIntf test methods.
		<14>	  8/9/93	NP		Added class id string.
		<13>	 7/26/93	CG		Added Reply handler to SemtIntf test
									methods.
		<12>	 7/20/93	PH		Fix menus
		<11>	 7/20/93	CG		Added SemtIntf test methods and fields.
		<42>	 6/23/93	RCR		Use tokens for foci
		 <11>	 5/19/93	RCR		Part.h NameSmith.
		 <7>	 4/29/93	VL		8.3 Name Changes.
		 <2>	  4/1/93	SES		Added new methods for menu handling and did some general
									housecleaning.
		 <1>	 3/29/93	SES		first checked in
*/


#ifndef _CLOCK_
#define _CLOCK_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#include "ClockDef.h"

#include "SimplPrt.h"

//==============================================================================
// #Defines for the ClockPart class
//==============================================================================

#define kCPIdleFrequency				20

//==============================================================================
// Typedefs for the ClockPart class
//==============================================================================

//   typedef CursHandle	XMPCursorHandle;

//==============================================================================
// Classes defined in this interface
//==============================================================================

class ClockPart;

//==============================================================================
// Classes used by this interface
//==============================================================================

class OrderedCollection;
class OrderedCollectionIterator;
class XMPEmbeddedFramesIterator;
class XMPFacet;
class XMPFrame;
class XMPMenuBar;
class XMPFocusSet;
class XMPShape;
class XMPStorageUnit;
class XMPSession;
class XMPWindow;
class XMPMacSemanticInterface;

//==============================================================================
// XMPClockPart
//==============================================================================

class ClockPart : public SimplePart
{
	//--------------------------------------------------------------------------
	// Class Friends
	//--------------------------------------------------------------------------

	friend class XMPDraft;

	//--------------------------------------------------------------------------
	// Class Types
	//--------------------------------------------------------------------------

	// none
	
	
protected:

	XMPNVMethod	void CommonInitClock();

	virtual void	InstallMenus( XMPFrame*  frame );
	
	virtual void	RemoveMenus( XMPFrame*  frame );
	
	virtual	void	CheckMarkMenu( MenuHandle	theMenu, 
								   Str255		itemText,
								   XMPBoolean	checkIt );
	
	virtual	void	StyleToString( XMPUByte style,
								   Str255*	styleName );
	
	virtual	void	SetFont( XMPSShort  fontNum ); 
	
	virtual	void	SetFontSize( XMPSShort  fontSize ); 
	
	virtual	void	SetFontStyle( XMPUByte  fontStyle ); 

	virtual void	OutlineSizeMenu();
	
	virtual	void		MinimizeFrameShape( XMPFrame* frame );
	virtual XMPShape*	CalcActualClockShape(XMPPlatformWindow window);
	
	virtual void	ShowPartsBin(XMPFrame* frame);
	virtual void	HidePartsBin();
	
	
public:

	//  DragAndDrop 
	
	virtual void	FulfillPromise(XMPStorageUnitView *promiseSUView);
	
	virtual void	DropCompleted(XMPPart* destPart,
								  XMPDropResult 	dropResult );
	
	virtual void	DragEnter( XMPDragItemIterator* dragInfo, 
							   XMPFacet*            facet, 
							   XMPPoint 		    where ); 
							   
	virtual void	DragWithin( XMPDragItemIterator* dragInfo, 
								XMPFacet*            facet,
								XMPPoint 		     where ); 
								
	virtual void	DragLeave( XMPFacet*    facet, 
							   XMPPoint 	where );
							   
	virtual XMPDropResult
					Drop( XMPDragItemIterator* dropInfo, 
						  XMPFacet*            facet,
						  XMPPoint 			   where );

	//  Frame 

	virtual void		RemoveDisplayFrame( XMPFrame*  oldFrame );
	
	virtual void		AddDisplayFrame(XMPFrame* frame);
										 
	virtual void		CloseDisplayFrame(XMPFrame* frame);

	virtual void		FrameShapeChanged(XMPFrame* target);
	
	virtual void 		FacetAdded(XMPFacet* facet);
	
	virtual void		WritePartInfo( XMPPtr			partInfo,
									   XMPStorageUnitView*	storageUnitView );
	
	virtual XMPPtr		ReadPartInfo(XMPFrame* frame, XMPStorageUnitView* storageUnitView );
	
	virtual XMPID		Open( XMPFrame*	frame );
	
									   	    
	//  Imaging 

	virtual void		Draw( XMPFacet* facet,
							  XMPShape*	invalRegion );


	//  Linking 

	virtual XMPLink*	CreateLink(XMPPtr data, XMPULong size);
	
	virtual void		LinkUpdated(XMPLink*  updatedLink, XMPChangeID id);


	//  Memory Management 

	virtual XMPSize		Purge(XMPSize size);


	//  Storage Management 

	virtual void		Externalize();


	//  Part Activation 

	virtual XMPBoolean	BeginRelinquishFocus( XMPTypeToken 	focus,
												  XMPFrame* 	currentFrame,
											 	  XMPFrame* 	proposedFrame );
												  
	virtual void		CommitRelinquishFocus( XMPTypeToken 	focus,
										 XMPFrame* 		currentFrame,
										 XMPFrame*		 proposedFrame );
										 
	virtual void		AbortRelinquishFocus( XMPTypeToken 	focus,
											   XMPFrame* 		currentFrame,
											   XMPFrame* 		proposedFrame );
											   
	virtual void		FocusAcquired( XMPTypeToken	focus, 
									   XMPFrame*	newOwner );	
									   
	virtual void		FocusLost( XMPTypeToken	focus, 
								   XMPFrame* 	oldOwner );	

	//  UI Events 

	virtual XMPBoolean	HandleEvent( XMPEventData event,
									 XMPFrame* frame,
									 XMPFacet* facet );
	
	virtual void		MouseEnter( XMPFacet* facet,
									XMPPoint  where );
	
	virtual void		MouseWithin( XMPFacet* facet,
									XMPPoint  where );

	virtual void		MouseLeave( XMPFacet* facet);
	
	virtual void		AdjustMenus(XMPFrame* frame);

	virtual void		Idle( XMPFrame*  frame );
	
	virtual XMPBoolean	HandleMouseDown(XMPEventData event,
									 	XMPFrame* frame,
									 	XMPFacet* facet);
	
	virtual XMPBoolean	HandleMenuEvent( XMPFrame*		frame,
										 XMPEventData	event );

	virtual void		HandleFontMenu( XMPFrame*	frame,
										XMPUShort	item );
	
	virtual void		HandleSizeMenu( XMPUShort  item );
	
	virtual void		HandleStyleMenu( XMPUShort  item );

	virtual void		HandleSettingsMenu( XMPUShort  item );

	
	//  SemanticInterface 

	XMPVMethod XMPExtension*	GetExtension( XMPType extensionName );
	
	XMPVMethod XMPBoolean 		HasExtension( XMPType extensionName );

	XMPMethod XMPWindow* CreateWindow(XMPFrame* sourceFrame);
	XMPMethod void ActivateFrame(XMPFrame* frame);
	XMPMethod void DeActivateFrame(XMPFrame* frame);
	XMPMethod void ActivatingWindow(XMPFrame* frame);
	XMPMethod void DeActivatingWindow(XMPFrame* frame);
	
private:

	OrderedCollection*		fDisplayFrames;
	XMPID					fWindowID;			// ID of View As Window window
	
	//   XMPUShort				fFrameGroupIDCounter;
	XMPPlatformWindow		fFrameViewWindow;			// window for ::Open method
		
	XMPSShort 				fFont;						// text font
 	XMPUByte 				fStyle;						// text style
 	//   XMPUByte 				fFiller;			// 
 	XMPSShort 				fMode;						// text draw mode (srcOr/srcXor/srcBic)
 	XMPSShort 				fSize;						// text size
 	XMPSShort				fJustification;				// text justification
	XMPSShort 				fAscent;					// font ascent

	XMPSLong				fLastTime; 					// tick count last time we drew time
	
	Cursor					fCrossCursor;				// cross cursor
	Cursor					fArrowCursor;				// pointer cursor
	
	XMPMacSemanticInterface* fSemanticInterface;		// the Semantic interface for this part
	XMPBoolean				fHasSemanticInterface;
	
	MenuHandle				fFontMenu;
	MenuHandle				fSizeMenu;
	MenuHandle				fStyleMenu;
	MenuHandle				fSettingsMenu;

	XMPMenuBar* 			fMenuBar;
	XMPFocusSet* 			fFocusSet;
	XMPTypeToken			fSelectionFocus;
	XMPTypeToken			fMenuFocus;
	XMPTypeToken			fKeyFocus;
	XMPTypeToken			fModalFocus;
	
	XMPStorageUnit*			fTestClockSU;
	XMPID					fSettingsWindow;
	XMPBoolean				fSettingsWindowWasShown;
	XMPBoolean				fAlarmOn;
	
	XMPID					fPartsBinWin;
	XMPBoolean				fPartsBinOpen;
	
public:	// private by convention

	ClockPart();
	virtual ~ CDECL ClockPart();
	
	XMPVMethod	void InitPart(XMPStorageUnit* storageUnit);
	XMPVMethod	void InitPartFromStorage(XMPStorageUnit* storageUnit);
};

#endif // _CLOCK_
