/*
	File:		ClockPart.r

	Contains:	Resources for ClockPart

	Written by:	Caia Grisar

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <8>	 3/11/94	CG		Bug #1150241 - changed smSystemScript to
									smRoman and smScriptLang to langEnglish.
		 <7>	  2/8/94	JA		Moved 'kNMAPIDxx' consts here from
									ClockDef.h
		 <6>	 1/18/94	CG		Added new NMAP, changed format of NMAPs.
		 <5>	 1/14/94	CG		Added NMAPs for user strings.
		 <4>	 1/12/94	CG		Updated NMAP resources.
		 <3>	12/16/93	RR		Added alerts, dialogs amnd menus
		 <2>	12/13/93	RR		Added Alert resource for About box
		 <1>	 12/6/93	CG		first checked in

	To Do:
*/


#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#include "Types.r"
#include "SysTypes.r"

#ifndef __STDTYPES_R__
	#include "StdTypes.r"
#endif __STDTYPES_R__

#ifndef _STDDEFS_
	#include "StdDefs.h"
#endif _STDDEFS_

#ifndef _CLOCKDEF_
	#include "ClockDef.h"
#endif //_CLOCKDEF_


#ifdef ASLMBUILD
#define kNMAPid1 kNMAPid+1
#define kNMAPid2 kNMAPid+2
#define kNMAPid3 kNMAPid+3
#define kNMAPid4 kNMAPid+4
#define kNMAPid5 kNMAPid+5
#define kNMAPid6 kNMAPid+6
#else
#define kNMAPid1 kNMAPid+13
#define kNMAPid2 kNMAPid+14
#define kNMAPid3 kNMAPid+15
#define kNMAPid4 kNMAPid+16
#define kNMAPid5 kNMAPid+17
#define kNMAPid6 kNMAPid+18
#endif ASLMBUILD


//-------------------------------------------------------------------------------------
// NMAP Resources
//-------------------------------------------------------------------------------------

resource kNameMappings (kNMAPid1) {
	kXMPKind,
	{	/* array KeyList: 1 elements */
		/* [1] */
		kXMPKindTestClock,
		kXMPIsAnISOStringList {
			/* array ValueList: 1 elements */
			/* [1] */
			{
				kXMPCategoryTime
			}
		}
	}
};

resource kNameMappings (kNMAPid2) {
	kXMPEditorKinds,
	{	/* array KeyList: 1 elements */
		/* [1] */
		kClockPartID,
		kXMPIsAnISOStringList {
			/* array ValueList: 1 elements */
			/* [1] */
			{
				kXMPKindTestClock
			}
		}
	}
};


// User visible strings must be International Text
resource kNameMappings (kNMAPid3) {
	kXMPEditorUserString,
	{	/* array KeyList: 1 elements */
		/* [1] */
		kClockPartID,
		kXMPIsINTLText {
			kXMPRomanScript,
			kXMPEnglishLang,
			kClockEditorUserString
		}
	}
};

resource kNameMappings (kNMAPid4) {
	kXMPKindUserString,
	{	/* array KeyList: 1 elements */
		/* [1] */
		kXMPKindTestClock,
		kXMPIsINTLText {
			kXMPRomanScript,
			kXMPEnglishLang,
			kClockKindUserString
		}
	}
};

resource kNameMappings (kNMAPid5) {
	kXMPCategoryUserString,
	{	/* array KeyList: 1 elements */
		/* [1] */
		kXMPCategoryTime,
		kXMPIsINTLText {
			kXMPRomanScript,
			kXMPEnglishLang,
			kClockCategoryUserString
		}
	}
};

resource kNameMappings (kNMAPid6) {
	kXMPKindOldMacOSType,
	{	/* array KeyList: 1 elements */
		/* [1] */
		kXMPKindTestClock,
		kXMPIsMacOSType {
			kClockOSType
		}
	}
};

//-------------------------------------------------------------------------------------
// Menus
//-------------------------------------------------------------------------------------

resource 'MENU' (kXMPClockPart_StyleMenuID, preload) {
	kXMPClockPart_StyleMenuID, textMenuProc,
	0x7FFFFFFF,	/* all enabled */
	enabled, "Style",
	 {
		"Plain",
			noicon, nokey, nomark, plain;
		"Bold",
			noicon, nokey, nomark, plain;
		"Italic",
			noicon, nokey, nomark, plain;
		"Underline",
			noicon,  nokey, nomark, plain;
		"Outline",
			noicon,  nokey, nomark, plain;
		"Shadow",
			noicon,  nokey, nomark, plain;
		"Condense",
			noicon, nokey, nomark, plain;
		"Extend",
			noicon, nokey, nomark, plain
	}
};

resource 'MENU' (kXMPClockPart_SettingsMenuID, preload) {
	kXMPClockPart_SettingsMenuID, textMenuProc,
	0x7FFFFFFF,	/* all enabled */
	enabled, "Settings",
	 {
		"Synchronize",
			noicon, nokey, nomark, plain;
		"Show Alarm Settings",
			noicon, nokey, nomark, plain
	}
};

//-------------------------------------------------------------------------------------
// Dialogs and Alerts
//-------------------------------------------------------------------------------------

resource 'ALRT' (kXMPClockPart_AboutBoxID) {
	{62, 145, 210, 352},
	kXMPClockPart_AboutBoxID,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	centerParentWindow
};

resource 'DITL' (kXMPClockPart_AboutBoxID) {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{82, 72, 102, 130},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{26, 28, 46, 149},
		StaticText {
			disabled,
			"^0"
		}
	}
};

resource 'DLOG' (kXMPClockPart_SynchronizeDialogID) {
	{87, 123, 225, 325},
	movableDBoxProc,
	invisible,
	goAway,
	0x0,
	kXMPClockPart_SynchronizeDialogID,
	"Synchronize to GMT",
	centerParentWindow
};


resource 'DITL' (kXMPClockPart_SynchronizeDialogID) {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{23, 30, 40, 190},
		StaticText {
			disabled,
			"Synchronizing"
		},
		/* [2] */
		{74, 69, 94, 127},
		Button {
			enabled,
			"Cancel"
		}
	}
};

resource 'DLOG' (kXMPClockPart_SettingsDialogID) {
	{87, 123, 225, 325},
	noGrowDocProc,
	invisible,
	goAway,
	0x0,
	kXMPClockPart_SettingsDialogID,
	"Alarm Settings",
	centerParentWindow
};


resource 'DITL' (kXMPClockPart_SettingsDialogID) {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{23, 30, 40, 190},
		CheckBox {
			enabled,
			"Alarm On"
		},
	}
};
