/*
	File:		PrtPrpAc.cp

	Contains:	Implemementation of DragPartPropAccessor class.

	Written by:	Eric House

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	  2/8/94	T		Throw -> THROW & some code clean up
	To Do:
*/

#ifndef _DPPRPAC_
#include "DPPrpAc.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef _SEUTILS_
#include "SEUtils.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#pragma segment DPPrpAc

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Local Classes
//==============================================================================

//==============================================================================
// Global Variables
//==============================================================================

//==============================================================================
// Function Prototype
//==============================================================================

//==============================================================================
// DrawPartPropAccessor
//==============================================================================

//------------------------------------------------------------------------------
// DragPartPropAccessor::DragPartPropAccessor
//------------------------------------------------------------------------------

DragPartPropAccessor::DragPartPropAccessor(DescType property, DragPart* part)
{
	fProperty = property;
	fPart = part;
}

//------------------------------------------------------------------------------
// DragPartPropAccessor::SetData
//------------------------------------------------------------------------------

void DragPartPropAccessor::SetData(AppleEvent* message)
{
//	DescType	gotType;
//	RGBColor	color;
//	Size		actualSize;
//	XMPError	result;

	switch(fProperty)
	{
		case pContents:
		{
			AEDesc contents;
			THROW_IF_ERROR(AEGetKeyDesc(message, keyAEData, typeChar,
					&contents));
			TEHandle teh = fPart->GetTEH();
			TEDelete(teh);
			HLock(contents.dataHandle);
			TESetText(*(contents.dataHandle),
					GetHandleSize(contents.dataHandle), teh);
			AEDisposeDesc(&contents);
			TECalText(teh);
		//	Rect teRect = (*teh)->destRect;	// or is it viewRect?
		//	TEUpdate(&teRect, teh);

			fPart->InvalAllFrames();
			break;
		}

		// read-only cases go here:
		case pName:
			THROW(errAENotModifiable);
		default:
			THROW(errAECantSupplyType);
			break;
	}
}

//------------------------------------------------------------------------------
// DragPartPropAccessor::GetData
//------------------------------------------------------------------------------

void DragPartPropAccessor::GetData(AEDesc* objectData)
{
//	XMPError	result;
	
	switch(fProperty)
	{
		case pName:
			{
				// if we got here, we're in the Drag part!
				unsigned char* name = "\pDragPart";
				THROW_IF_ERROR(AECreateDesc(typeChar, (Ptr)&name[1], name[0],
						objectData));
			}
			break;

		case pContents:
			TEHandle teh = fPart->GetTEH();
			AEDesc dummy;
			dummy.descriptorType = typeChar;
			dummy.dataHandle = (*teh)->hText;
			THROW_IF_ERROR(AEDuplicateDesc(&dummy, objectData));
			break;

		default:
			THROW(errAECantSupplyType);
			break;
	}
}

