/*
	File:		DPPrpAc.h

	Contains:	Interface to class that handles accessing part properties for
				the OpenDoc shell.

	Written by:	Eric House

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

	To Do:
*/

#ifndef _DPPRPAC_
#define _DPPRPAC_

#ifndef _DRAGPART_
#include "DragPart.h"
#endif

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _PROPACSR_
#include "PropAcsr.h"
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This a simple abstract class to aid in the processing of Apple Events that
	are requests to get or set properties. Since OpenDoc does the resolution
	first, the object accessor routines can allocate one of these classes
	according to the property for which they have been asked to create a token.
	When passed back to the event handler, the event handler need only call the
	the appropriate member function of this class, GetData or SetData, to
	process the event. The event handler must then delete the object.
*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Classes defined in this interface
//==============================================================================

class DragPartPropAccessor;

//==============================================================================
// Classes used by this interface
//==============================================================================

class DragPart;

//==============================================================================
// PropAccessor
//==============================================================================

class DragPartPropAccessor : public PropAccessor
{
  public:

	DragPartPropAccessor(DescType property, DragPart* part);

	XMPVMethod void SetData(AppleEvent* message);
	
	XMPVMethod void GetData(AppleEvent* reply);

  private:

	DescType	fProperty;
	DragPart*	fPart;
};

#endif // _PRTPRPAC_