/*
	File:		DragObj.cpp

	Contains:	Object accessor implementation for the DragPart

	Written by:	Eric House

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	 3/28/94	CG		1153547: include XMPSessM.h instead of
									XMPSessn.h.
		 <5>	 3/26/94	NP		1153509. Messaging not PowerPC ready.
		 <4>	 3/26/94	NP		1153053-native ready.
		 <3>	 3/25/94	eeh		bug #1153053: Changes for PPC nativity
		 <2>	 2/16/94	JA		Fixed includes.
		 <8>	  2/8/94	T		Throw -> THROW & some code clean up
		 <7>	  2/7/94	T		fixes to compile with PPC Headers
		 <6>	 1/17/94	NP		Removed consts from public API.
		 <5>	 1/17/94	NP		Semantic Events callbacks API changes.
		 <4>	 1/11/94	eeh		removed "frame" from objspec vocab; other
									changes for scripting?
		 <3>	12/20/93	NP		Changed calls to CreateSwapToken.
		 <2>	12/15/93	NP		CreatePartToken -> CreateSwapToken.
		 <1>	11/16/93	CG		first checked in

	To Do:
*/

#ifndef _DRAGOBJ_
#include "DragObj.h"
#endif

#ifndef _DRAGPART_
#include "DragPart.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef _NAMRSLVM_
#include "NamRslvr.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _PART_
#include "Part.h"
#endif

#ifndef _DPPRPAC_
#include "DPPrpAc.h"
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

#ifndef _XMPSESSM_
#include "XMPSessM.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _SEUTILS_
#include "SEUtils.h"
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __AEOBJECTS__
#include <AEObjects.h>
#endif

#pragma segment TextObj

//==============================================================================
// Semantic Event Handlers and utility Functions
//==============================================================================

static XMPError GetPropertyFromNULL(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon);
								
								
static XMPError GetWildcardFromPart(XMPPart*			thisPart,
									DescType			desiredClass,
									XMPOSLToken*	container,
									DescType			containerClass,
									DescType			form,
									AEDesc*		selectionData,
									XMPOSLToken*		value,
									XMPSLong			refCon);
									
								
static XMPError SetData(XMPPart*		thisPart,
						AppleEvent*	theAppleEvent,
						AppleEvent*	reply,
						XMPSLong	handlerRefcon);
						
static XMPError GetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon);
					
static XMPError DragCompare(XMPPart* thePart, DescType oper,
		XMPOSLToken* obj1, XMPOSLToken*	obj2, XMPBoolean* result,
		XMPSLong refCon);

static XMPError DragCount(XMPPart* thePart, DescType desiredType,
		DescType containerClass, XMPOSLToken* container,
		XMPSLong* result, XMPSLong refCon);
static XMPError DragDisposeToken(XMPPart* thePart, XMPOSLToken* unneededToken,
		XMPSLong refCon);

void DragPart::InstallObjectAccessors()
{
	XMPObjectAccessorUPP accessorUPP ;
	
	accessorUPP = NewXMPObjectAccessorProc(GetPropertyFromNULL) ;
	fSemtIntf->InstallObjectAccessor(cProperty, typeNull, accessorUPP,
										(XMPSLong)fSession);

	accessorUPP = NewXMPObjectAccessorProc(GetWildcardFromPart) ;
 	fSemtIntf->InstallObjectAccessor(typeWildCard, cPart, accessorUPP,
										(XMPSLong)fSession);

	XMPEventHandlerUPP handlerUPP ;
	handlerUPP = NewXMPEventHandlerProc(SetData) ;
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAESetData,
												handlerUPP,
												(XMPSLong)fSession);

	handlerUPP = NewXMPEventHandlerProc(GetData) ;
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAEGetData,
												handlerUPP,
												(XMPSLong)fSession);


	XMPCompareUPP compUPP = NewXMPCompareProc( DragCompare ) ;
	fSemtIntf->InstallCompareProc(compUPP, (XMPSLong)fSession);

	XMPCountUPP countUPP = NewXMPCountProc( DragCount ) ;
	fSemtIntf->InstallCountProc(countUPP, (XMPSLong)fSession);

	XMPDisposeTokenUPP disposeTokenUPP
			= NewXMPDisposeTokenProc( DragDisposeToken ) ;
	fSemtIntf->InstallDisposeTokenProc(disposeTokenUPP, (XMPSLong)fSession);
}

//------------------------------------------------------------------------------
// GetPropertyFromNULL
//------------------------------------------------------------------------------

static XMPError GetPropertyFromNULL(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(desiredClass);
XMPUnused(container);
XMPUnused(containerClass);

	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();
	DescType				propID;
	DragPartPropAccessor*	accessorObj;
	XMPError				error = noErr;

	TRY

	if (form != formPropertyID)
		THROW(errAEWrongDataType);
	propID = **(DescType**)(selectionData->dataHandle);
	switch (propID)
	{
		case pName:
		case pContents:
			resolver->CreateToken(value, cProperty, thisPart);
			accessorObj = new DragPartPropAccessor(propID, (DragPart*)thisPart);
			ReallocAndStuff2nd4Bytes(value, (XMPULong)accessorObj);
			break;
		default:
			THROW(errAECantSupplyType);
			break;
	}
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}

								
//------------------------------------------------------------------------------
// GetWildcardFromPart
//------------------------------------------------------------------------------

static XMPError GetWildcardFromPart(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
	XMPUnused(thisPart);
	XMPUnused(desiredClass);
	XMPUnused(container);
	XMPUnused(containerClass);
	XMPUnused(form);
	XMPUnused(selectionData);

	XMPNameResolver*	resolver = ((XMPSession*)refCon)->GetNameResolver();
	XMPError			error = noErr;

	TRY

	DragPart* part = (DragPart*)GetSecondFourBytesFromDataHandle(container);
	resolver->CreateSwapToken(value, part->GetDragTextFrame());
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}


//------------------------------------------------------------------------------
// SetData
//------------------------------------------------------------------------------

static XMPError SetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
	XMPUnused(thisPart);
	XMPUnused(reply);
	XMPUnused(handlerRefcon);

	AEDesc	theToken;
	XMPError	error = noErr;

	TRY

	THROW_IF_ERROR(AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard,
								&theToken));
	DragPartPropAccessor* propAccessorObj
			= (DragPartPropAccessor*)GetSecondFourBytesFromDataHandle
					((XMPOSLToken*)&theToken);
	propAccessorObj->SetData(theAppleEvent);
	delete propAccessorObj;
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}


//------------------------------------------------------------------------------
// GetData
//------------------------------------------------------------------------------

static XMPError GetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
	XMPUnused(thisPart);
	XMPUnused(handlerRefcon);

	DescType		reqType;
	AEDesc			theToken;
	Size			theSize;
	AEDesc			objectData;
	XMPError		result;
	XMPError		error = noErr;

	TRY

	result = AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard,
								&theToken);
	THROW_IF_ERROR(result);
	result = AEGetParamPtr(theAppleEvent, keyAERequestedType,typeType,
								&reqType,(Ptr)&reqType,sizeof(reqType),&theSize);
	if (result == errAEDescNotFound)		// Not an error if return type is not found.
	{
		reqType = typeWildCard;
	}

	DragPartPropAccessor* propAccessorObj
			= (DragPartPropAccessor*)GetSecondFourBytesFromDataHandle
					((XMPOSLToken*)&theToken);
	propAccessorObj->GetData(&objectData);
	delete propAccessorObj;
	
	if ((reqType!=typeWildCard)&&(reqType!=typeBest)&&(reqType!=objectData.descriptorType))
	{
		AEDesc			tempDesc;
		THROW_IF_ERROR(AECoerceDesc(&objectData, reqType, &tempDesc));

		THROW_IF_ERROR(AEDisposeDesc(&objectData));

		objectData = tempDesc;
	}

	THROW_IF_ERROR(AEPutParamDesc(reply, keyAEResult, &objectData));
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetData()

//------------------------------------------------------------------------------
// DragCompare
// For now, just handle what we need to handle
//------------------------------------------------------------------------------
static XMPError DragCompare(	XMPPart*			thePart,
							DescType			oper,
							XMPOSLToken*	obj1,
							XMPOSLToken*	obj2,
							XMPBoolean*			result,
							XMPSLong			refCon)
{
	XMPUnused(thePart);
	XMPUnused(obj1);
	XMPUnused(result);
	XMPUnused(refCon);

	XMPError	error = noErr;

	switch(oper)
	{
		case kAEEquals:
			if (((AEDesc*)obj2)->descriptorType == typeChar)
				break;
		default:
			error = errAEEventNotHandled;
	}
	
	return error;
}

//------------------------------------------------------------------------------
// DragCount
//------------------------------------------------------------------------------
static XMPError DragCount(		XMPPart*			thePart,
							DescType			desiredType,
							DescType			containerClass,
							XMPOSLToken*	container,
							XMPSLong*			result,
							XMPSLong			refCon)
{
	XMPUnused(thePart);
	XMPUnused(desiredType);
	XMPUnused(containerClass);
	XMPUnused(container);
	XMPUnused(result);
	XMPUnused(refCon);
	return errAENoSuchObject;
}

//------------------------------------------------------------------------------
// DragDisposeToken
//------------------------------------------------------------------------------
static XMPError DragDisposeToken(XMPPart*			thePart,
							XMPOSLToken*	unneededToken,
							XMPSLong		refCon)
{
	XMPUnused(thePart);
	XMPUnused(unneededToken);
	XMPUnused(refCon);
	return errAEEventNotHandled;
}

