/*
	File:		DragPart.cpp

	Contains:	DragPart class definition.

	Written by:	Paul Hartenstine

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<24>	 4/18/94	T		#1144379 Stationery model & generation have
									been fixed to enable dragging & dropping of
									stationery onto open documents.  DragPart
									no longer assumes fDocument has been
									allocated inside its Externalize method.
		<23>	 4/13/94	SS		Fixed port swapping during printing in
									::Draw (#1144419)
		<22>	  4/7/94	CC		IncorporateTestDragContent: Set info.kind
									of a copied destination link correctly.
									(1155840)
		<21>	  4/6/94	CC		XMPTypeInfo: Changed field "creationDate"
									to "creationTime" and "modificationDate" to
									"changeTime". (1154722)
		<20>	  4/4/94	CC		Removed unused #include of TypeSet.h
									(1153046).
		<19>	  4/4/94	CC		Maintain XMPLinkInfo for destination links
									(1154127); Implemented RevealLink()
									(1154069); Implement BreakLink for
									destination links (1154858).
		<18>	 3/28/94	CG		1153547: include XMPSessM.h instead of
									XMPSessn.h.
		<17>	 3/27/94	T		#1153523 Merge PrtRCR into DragPart
		<16>	 3/26/94	NP		1153054. InitSemanticInterface.
		<15>	 3/25/94	CC		Replaced kXMPDropSucceed with kXMPDropCopy
									(1153424); use
									XMPDragAndDrop::GetContentStorageUnit()
									(1153043); call ShowSourceContent()
									(1153347).
		<14>	 3/25/94	JA		JPA: GetPlatformTransform -> GetQDOffset
									(1153438).
		<13>	 3/25/94	eeh		bug #1153053: Changes for PPC nativity
		<12>	 3/24/94	CC		Changed WriteToProperty() to
									WriteLinkSpec(), ReadFromProperty() to
									ReadLinkSpec(). (1153049)
		<11>	 3/24/94	CC		Use XMPLinkSource::CloneTo() and
									XMPLink::CloneTo (1153038); pass
									XMPChangeID parameter to
									ShowLinkSourceInfo() (1153068).
		<10>	 3/18/94	CC		Added constants defining ISO equivalents
									for Apple scrap types. (1151636)
		 <9>	 3/17/94	CG		#1151392. Commented out IntlToPStr.
		 <8>	 3/17/94	CC		Revised to maintain Part Info menu item and
									display link info dialogs. (1151329)
		 <7>	 3/14/94	CC		Write content correctly. (1147874)  Many
									revisions to support Paste As dialog.
									(1144421)
		 <6>	  3/2/94	CC		DisposeActionState(): Commented out
									erroneous WASSERTs.  UndoAction(),
									RedoAction(): correct test in WASSERT.
									(1145487)  Drop(): Call
									SetChangedFromPrev() if successful.
									(1147876).
		 <5>	 2/28/94	CC		Replaced toolbox mem calls with XMP calls;
									declared all local functions static.
		 <4>	 2/17/94	VL		#1142972, #1144024: Fixed many memory
									problems. Enabled receiving drops from the
									Finder.
		 <3>	 2/17/94	JBS		#114???? fix inconsistent facet coords
		 <2>	 2/16/94	JA		Include new AltPoint.h.
		<70>	  2/8/94	JA		Added missing #includes.
		<69>	  2/8/94	T		Throw -> THROW & some code clean up
		<68>	  2/7/94	JA		Utility.h --> XMPUtils.h
		<67>	  2/7/94	T		fixes to compile with PPC Headers
		<66>	  2/2/94	VL		Externalize link source and link before
									externalizing itself.
		<65>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame()
		<64>	  2/1/94	CC		Added XMPCloneKind argument to
									ExternalizeSelection and BeginClone.
		<63>	 1/28/94	NP		Fixed creation of Undo text so that menu
									key for Undo menu item is preserved.
		<62>	 1/28/94	CC		Added ReleaseAll(); fixed bug in
									ExternalizeSelection(); use UniqueChangeID
									method of session.
		<61>	 1/27/94	RR		Remove redundant disabling of menus
		<60>	 1/27/94	RR		Added CreateWindow. Reworked ::Open
		<59>	 1/27/94	VL		Changed XMPPart::CloneTo to
									XMPPart::CloneInto.
		<58>	 1/25/94	RR		Call window->Open,Show,Select
		<57>	 1/25/94	NP		Added AdjustMenus, Removed calls to
									SyncMenuBar.
		<56>	 1/24/94	NP		Enabled Paste As. Fixed menu handling bug.
		<55>	 1/24/94	CC		Added XMPChangeID parameter to
									LinkUpdated().
		<54>	 1/22/94	CC		Don't release objects in the destructor.
		<53>	 1/21/94	RR		Added shouldSave to CreateWindow
		<52>	 1/21/94	CC		Adapted to split of links into XMPLink and
									XMPLinkSource, conform to new link and
									clipboard Lock() method.  Added code to
									write LinkedText value format.
		<51>	 1/20/94	SS		::Open returns a window id
		<50>	 1/19/94	NP		Fixed some undo-related stuff.
		<49>	 1/15/94	RR		WinState.h->WinStat.h,
									Dispatch.h->Disptch.h
		<48>	 1/15/94	RR		Updated HandleMenuEvent (command no. is no
									longer in event)
		<47>	 1/14/94	NP		Init changes.
		<46>	 1/14/94	CC		Update to latest link and clipboard api.
		<45>	 1/14/94	RR		Cursor tracking coordinates now in local
									coords
		<44>	 1/13/94	NP		Mods for new Undo API.
		<43>	 1/11/94	eeh		added GetDragTextFrame
		<42>	 1/11/94	T		Init... changes
		<41>	 1/10/94	NP		Modified for new Undo stuff.
		<40>	12/22/93	CC		Lots of calls to
									XMPDraft::SetChangedFromPrev(); Call
									XMPLink::GetStorageUnit() for link's
									persistent su, call
									XMPLink::GetContentStorageUnit() for the
									link's content su.  Many changes to keep
									links consistent with the TextEdit record.
		<39>	12/21/93	VL		Changed XMPStorageUnit::GetValueSize to
									XMPStorageUnit::GetSize.
		<38>	12/21/93	JBS		CreateFrame with viewType & presentation,
									rename FrameClosed to CloseDisplayFrame
		<37>	12/20/93	VL		Used WindowState to CreateWindow.
		<36>	12/20/93	RR		RegisterIdleFrame -> RegisterIdle
		<35>	12/16/93	CC		Save links persistently.
		<34>	12/15/93	JBS		add FrameClosed
		<33>	12/13/93	T		InitPart changes
		<32>	12/13/93	RR		Remove extra parameter from MouseLeave
		<31>	 12/9/93	eeh		#include ISOStr.h
		<30>	 12/9/93	eeh		scripting support focussed around fSemtIntf
		<29>	 12/8/93	CC		Rename methods on LinkSpec objects.
		<28>	 12/2/93	CC		Use Lock() and Unlock() methods of the
									clipboard object instead of the clipboard
									focus.
		<27>	11/24/93	CC		Permanent fix to SyncMenuBar frame
									reference; removed most CFront warnings.
		<26>	11/24/93	T		modifications to use the correct part
									storage model
		<25>	11/23/93	JA		Updated for new Shape API.
		<24>	11/23/93	T		fix/comment out a couple of out of context
									references to 'frame' which were being
									bound to the quickdraw constant
		<23>	11/19/93	NP		Changed parameters to AddActionToHistory to
									conform to new XMPUndo interface.
		<22>	11/18/93	T		call inherited::InitializePersistent() and
									::Externalize()
		<21>	11/18/93	EL		Use "OpenDoc" instead of "kXMP".
		<20>	11/18/93	CC		Many changes to support links; actually
									pasting a link temporarily inhibited.
		<19>	11/18/93	eeh		commented out kXMPPropDisplayFrames
		<18>	11/16/93	SS		Updated to use Facets - also, began support
									for Undo (incomplete)
		<17>	11/10/93	SS		Added command numbers for all menu items
									and methods to support them
		<16>	 11/8/93	SS		Added Clipboard Object support - fixed
									clipping problem in DragWithin - fixed
									intermittant BusError in Drop
		<15>	 11/4/93	PH		Remove call to SetResolvePromise
		<14>	 11/1/93	VL		Changed Internalize code to handle bogus
									references.
		<13>	10/29/93	CC		Cut/Copy/Paste synchronize OpenDoc
									clipboard with desk scrap
		<12>	10/28/93	PH		Add MouseEnter/Leave
		<11>	10/26/93	SS		Added two new methods to simplify clipping;
									also added invalidation in
									frameshapechanged
		<10>	10/24/93	PH		Use promise for 'styl' information in drag
		 <9>	10/21/93	PH		Change FulfillPromise call
		 <8>	10/21/93	RR		CreateWindow takes new flags
		 <7>	10/21/93	CC		Correct constants for items on the Edit
									menu
		 <6>	10/20/93	PH		Add Open method
		 <5>	10/14/93	RR		Add source frame parameter to
									AddDisplayFrame
		 <4>	10/14/93	JA		XMPShape fu: No longer returns copies from
									GetPlatformShape. Deal w/it.
		 <3>	10/12/93	PH		Add fTextHandle and fStylHandle
		<19>	 10/6/93	JA		Use new fixed-point XMPPoint.
		<18>	 10/5/93	PH		Don't use clipboard in ex/internalize and
									export styl information in the drag
		<17>	 10/5/93	T		added #ifdef SSREMOVEWORKAROUND until
									su->Remove() is fixed
		<16>	 10/4/93	PH		Fix Externalize and RemoveDisplayFrame
		<15>	 9/30/93	PH		Support new Drag and Drop protocol
		<14>	 9/30/93	RR		Call inherited Initialize
		<13>	 9/29/93	VL		Added InitializePersistent.
		<12>	 9/27/93	PH		Remove draghilite
		<11>	 9/24/93	PH		Add segment call
		<10>	 9/24/93	PH		Use new content properties
		 <9>	 9/23/93	JA		Use new convenience methods from Shape and
									Transform.
		 <8>	 9/22/93	RR		Release menu bar, instead of deleting it
		 <7>	 9/21/93	PH		Use new DragAndDrop and Frame API's
		 <6>	 9/16/93	PH		Use new Drag and Drop API
		 <5>	 9/16/93	RR		Added fMenuBar
		 <4>	 9/16/93	JA		Dispose regions returned by
									GetPlatformShape.
		 <3>	 9/13/93	PH		Fix some update and dragging bugs
		 <2>	 9/10/93	PH		Handle dragging text and fix casts
		 <1>	  9/7/93	PH		first checked in
*/


#ifndef _ALTPOINT_
#include "AltPoint.h"			// Use C++ savvy XMPPoint and XMPRect
#endif


#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __STRING__
#include <String.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>	// SysBeep etc.
#endif

#ifndef __OSEVENTS__
#include <OSEvents.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>	// HiWord etc.
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifdef __SC__

#ifndef __LANGUAGE__
#include <Language.h>
#endif

#endif // __SC__

#ifndef _DRAGPART_
#include "DragPart.h"
#endif

#ifndef _STDPROPS_
#include "StdProps.h"
#endif

#ifndef _STDTYPES_
#include "StdTypes.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _DOCUMENT_
#include "Document.h"
#endif

#ifndef _XMPCTR_
#include "XMPCtr.h"
#endif

#ifndef _STORAGE_
#include "Storage.h"
#endif

#ifndef _XMPSESSM_
#include "XMPSessM.h"
#endif

#ifndef _WINSTAT_
#include "WinStat.h"
#endif

#ifndef _ARBITRAT_
#include "Arbitrat.h"
#endif

#ifndef _DISPTCH_
#include "Disptch.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _CANVAS_
#include "Canvas.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _MENUBAR_
#include "MenuBar.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _DRAGDROP_
#include "DragDrop.h"
#endif

#ifndef _DGITMITR_
#include "DgItmItr.h"
#endif

#ifndef _TRANSLAT_
#include "Translat.h"
#endif

#ifndef _SUVIEW_
#include "SUView.h"
#endif

#ifndef _CMDDEFS_
#include "CmdDefs.h"
#endif

#ifndef _UNDO_
#include "Undo.h"
#endif

#ifndef _FOCUSSET_
#include "FocusSet.h"
#endif

#ifndef _FOCI_
#include "Foci.h"
#endif

#ifndef _XMPUTILS_
#include "XMPUtils.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _CLIPBD_
#include "Clipbd.h"
#endif

#ifndef _LINK_
#include "Link.h"
#endif

#ifndef _LINKSRC_
#include "LinkSrc.h"
#endif

#ifndef _LINKSPEC_
#include "LinkSpec.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

#ifndef _ISOSTR_
#include "ISOStr.h"
#endif

#ifndef _LINKDLGS_
#include "LinkDlgs.h"
#endif

#ifndef _NMSPCMGR_
#include "NmSpcMgr.h"
#endif

#ifndef _NAMSPACE_
#include "NamSpace.h"
#endif

#ifndef _STDDEFS_
#include "StdDefs.h"	//jpa
#endif

#ifndef _INFO_
#include "Info.h"		// for kXMPViewAsFrame, etc.
#endif

#ifndef _FOCUSLIB_
#include "FocusLib.h"	// for drawing/printing
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>			// gx header; used only for ff() macro
#endif

#pragma segment DragPart


//==============================================================================
// Constants
//==============================================================================

#define SSREMOVEWORKAROUND

#define	MaxDocumentCount		8

#define	InitialH				16
#define	InitialV				42

#define	TopMargin				6
#define	LeftMargin				6
#define	RightMargin				6
#define	BottomMargin			6

#define	ScrollResolution		12

const char kRightArrow	= 0x1D;
const char kLeftArrow	= 0x1C;
const char kUpArrow		= 0x1E;
const char kDownArrow	= 0x1F;

const	XMPPropertyName	kXMPPropText = "DragPart:Property:Text";		// Text
const	XMPPropertyName	kXMPPropStyl = "DragPart:Property:Style";		// Style record
const	XMPPropertyName	kXMPPropSourceLink	= "DragPart:Property:SourceLink";
const	XMPPropertyName	kXMPPropDestLink	= "DragPart:Property:DestLink";

const XMPPropertyName	kXMPUndoInfo	= "DragPart:UndoInfo";

const XMPValueType		kXMPFrameRef	= "OpenDoc:Type:FrameRef";
const XMPValueType		kXMPCommandID	= "OpenDoc:Type:CommandID";
const XMPValueType		kXMPStart		= "DragPart:TextStart";
const XMPValueType		kXMPEnd			= "DragPart:TextEnd";

const	XMPValueType	kXMPAppleTEXT	= "Apple:OSType:Scrap:TEXT";
const	XMPValueType	kXMPApplestyl	= "Apple:OSType:Scrap:styl";
const	XMPValueType	kXMPApplehfs	= "Apple:OSType:Scrap:hfs ";

// const XMPValueType		kXMPLinkedText	= "OpenDoc:LinkedText";


//==============================================================================
// DragPart Command Numbers
//==============================================================================

#define cDTSampleOne	4001
#define cDTSampleTwo	4002
#define cDTTyping		9999
#define cDTStyling		9998

#define kUndoIt			1
#define kRedoIt			2

//==============================================================================
// Private functions
//==============================================================================

static XMPBoolean ValueOnClipboard(XMPValueType type, XMPSession* session);
static XMPBoolean LinkSpecOnClipboard(XMPSession* session);
static XMPBoolean IncludedInRange(XMPSShort start, XMPSShort end, XMPSShort startRange, XMPSShort endRange);

//==============================================================================
// DragPart
//==============================================================================


DragPart::DragPart()
{
	fIsActive = false;
	fFocusSet = kXMPNULL;

	fWindowID = 0;
	fDocument = NULL;
	fTextHandle = NULL;
	fStylHandle = NULL;
	fExtTransStore.x = 0;		//  hack! so we can tell if frame has been dragged
	fExtTransStore.y = 0;
	fLastFont = geneva;
	fLastSize = 12;
	fFrameShapeChanged = true;
	fDisplayFrames = kXMPNULL;
	fUndoCount = 0;
	fRedoCount = 0;
	fSrcLink.linkSource = (XMPLinkSource*) kXMPNULL;
	fSrcLink.start = -1;
	fSrcLink.end = -1;
	fDstLink.link = (XMPLink*) kXMPNULL;
	fDstLink.start = -1;
	fDstLink.end = -1;
	fDstLink.info.kind = (XMPType) kXMPNULL;
	fDraft = kXMPNULL;
	fTestDragSU = kXMPNULL;

	fSemtIntf = kXMPNULL;
}

DragPart::~DragPart()
{
	delete fDisplayFrames;
	if (fDocument)
	{
		if (fDocument->theTE)
			TEDispose(fDocument->theTE);
		XMPDisposePtr((Ptr)fDocument);
	}
	if (fTextHandle)
		XMPDisposeHandle(fTextHandle);
	if (fStylHandle)
		XMPDisposeHandle((Handle)fStylHandle);
	if (fTextMenu) DisposeMenu(fTextMenu);
	if (fFontMenu) DisposeMenu(fFontMenu);
	if (fSizeMenu) DisposeMenu(fSizeMenu);

	delete fSemtIntf;
	XMPDeleteObject(fFocusSet);
}

void DragPart::InitPart(XMPStorageUnit* storageUnit)	// Override
{
	if (fInitialized)
		return;
	
	SimplePart::InitPart(storageUnit);

	this->CommonInitDragPart();

	storageUnit->AddProperty(kXMPPropContents)->AddValue(kXMPKindTestDrag);	

	fDraft = storageUnit->GetDraft();
	fTestDragSU = fDraft->CreateStorageUnit();
	
	fTestDragSU->AddProperty(kXMPPropText)->AddValue(kXMPAppleTEXT);			
	fTestDragSU->AddProperty(kXMPPropStyl)->AddValue(kXMPApplestyl);			
	fTestDragSU->AddProperty(kXMPPropDisplayFrames)->AddValue(kXMPID);
	fTestDragSU->AddProperty(kXMPPropSourceLink)->AddValue(kXMPValue);
	fTestDragSU->AddProperty(kXMPPropDestLink)->AddValue(kXMPValue);
}

void DragPart::InitPartFromStorage(XMPStorageUnit* storageUnit)	// Override
{
	if (fInitialized)
		return;
	
	SimplePart::InitPartFromStorage(storageUnit);
	
	this->CommonInitDragPart();

	Size              textSize, stylSize;
	XMPStorageUnit   *su;
 	XMPStorageUnitRef aSURef;
	
XMPVolatile(su);
		
	su = this->GetStorageUnit();
	fDraft = su->GetDraft();
	
	su->Focus(kXMPPropContents,kXMPPosSame,kXMPKindTestDrag,1,kXMPPosFirstSib);
	su->GetValue(sizeof(XMPStorageUnitRef),&aSURef);
	fTestDragSU = fDraft->GetStorageUnit(su->GetIDFromStorageUnitRef(aSURef));
	su = fTestDragSU;

	su->Focus(kXMPPropText, kXMPPosUndefined, 0, 1, kXMPPosFirstSib);			// read in TEXT
	textSize = su->GetSize();
	if (textSize > 0)
	{
		fTextHandle = XMPNewHandle(textSize);
		XMPLockHandle(fTextHandle);
		su->GetValue(textSize, (XMPValue)*fTextHandle);
		XMPUnlockHandle(fTextHandle);
	}
	
	su->Focus(kXMPPropStyl, kXMPPosUndefined, 0, 1, kXMPPosFirstSib);			// read in styl
	stylSize = su->GetSize();
	if (stylSize > 0)
	{
		fStylHandle = (StScrpHandle)XMPNewHandle(stylSize);
		XMPLockHandle((Handle)fStylHandle);
		su->GetValue(stylSize, (XMPValue)*fStylHandle);
		XMPUnlockHandle((Handle)fStylHandle);
	}
	
	//
	//  Read in the display frame; there can be only one but there may be none.
	//
		
	TRY
	
		su->Focus(kXMPPropDisplayFrames, kXMPPosUndefined, 0, 1, kXMPPosFirstSib);
		su->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		XMPFrame* aFrame = fDraft->GetFrame(su->GetIDFromStorageUnitRef(aSURef));
		fDisplayFrames->AddLast((ElementType)aFrame);
		
	CATCH_ALL
	
	ENDTRY

	//
	// Read in the source and destination links
	//
	su->Focus(kXMPPropSourceLink, kXMPPosUndefined, kXMPValue, (XMPValueIndex)0, kXMPPosUndefined);
	if (su->GetSize() > 0) 
	{
		TRY
			su->GetValue(sizeof(XMPStorageUnitRef),&aSURef);
			fSrcLink.linkSource = fDraft->GetLinkSource(su->GetIDFromStorageUnitRef(aSURef));
			su->GetValue(sizeof(XMPSShort), (XMPValue) &fSrcLink.start);
			su->GetValue(sizeof(XMPSShort), (XMPValue) &fSrcLink.end);
		CATCH_ALL
			su->SetOffset(0);
			su->DeleteValue(su->GetSize());
			fSrcLink.linkSource = (XMPLinkSource*) kXMPNULL;
			fSrcLink.start = -1;
			fSrcLink.end = -1;
		ENDTRY
	}

	su->Focus(kXMPPropDestLink, kXMPPosUndefined, kXMPValue, (XMPValueIndex)0, kXMPPosUndefined);
	if (su->GetSize() > 0) 
	{
		TRY
			su->GetValue(sizeof(XMPStorageUnitRef),&aSURef);
			fDstLink.link = fDraft->GetLink(su->GetIDFromStorageUnitRef(aSURef), (XMPLinkSpec*) kXMPNULL);
			su->GetValue(sizeof(XMPSShort), (XMPValue) &fDstLink.start);
			su->GetValue(sizeof(XMPSShort), (XMPValue) &fDstLink.end);
			su->GetValue(sizeof(XMPLinkInfo), (XMPValue) &fDstLink.info);
			XMPType kind = (XMPType) XMPNewPtr((XMPULong) fDstLink.info.kind);
			su->GetValue((XMPULong) fDstLink.info.kind, (XMPValue) kind);
			fDstLink.info.kind = kind;
		CATCH_ALL
			su->SetOffset(0);
			su->DeleteValue(su->GetSize());
			fDstLink.link = (XMPLink*) kXMPNULL;
			fDstLink.start = -1;
			fDstLink.end = -1;
		ENDTRY
	}
}

void DragPart::CommonInitDragPart()
{
	fFocusSet = new XMPFocusSet();
	fFocusSet->InitFocusSet();
	fFocusSet->Add(fSession->Tokenize(kXMPKeyFocus));
	fFocusSet->Add(fSession->Tokenize(kXMPMenuFocus));
	fFocusSet->Add(fSession->Tokenize(kXMPSelectionFocus));

	fDisplayFrames = new OrderedCollection;
	fTextMenu = NewMenu(140,"\pText");
	fFontMenu = NewMenu(141,"\pFont");
	fSizeMenu = NewMenu(142,"\pSize");
	
	if (!fTextMenu || !fFontMenu || !fSizeMenu)
		DebugStr("\pError creating menus");
		
	fMenuBar = fSession->GetWindowState()->CopyBaseMenuBar();

	AppendMenu(fTextMenu,"\pSample One");
	fMenuBar->RegisterCommand(cDTSampleOne, 140, 1);
	AppendMenu(fTextMenu,"\pSample Two");
	fMenuBar->RegisterCommand(cDTSampleTwo, 140, 2);
	
	AddResMenu(fFontMenu,'FOND');
	
	AppendMenu(fSizeMenu,"\p9");
	AppendMenu(fSizeMenu,"\p10");
	AppendMenu(fSizeMenu,"\p11");
	AppendMenu(fSizeMenu,"\p12");
	AppendMenu(fSizeMenu,"\p14");
	AppendMenu(fSizeMenu,"\p18");
	AppendMenu(fSizeMenu,"\p24");
	AppendMenu(fSizeMenu,"\p36");

	fMenuBar->AddMenuLast(140, fTextMenu, this);
	fMenuBar->AddMenuLast(141, fFontMenu, this);
	fMenuBar->AddMenuLast(142, fSizeMenu, this);

	fSemtIntf = new XMPSemanticInterface();
	fSemtIntf->InitSemanticInterface(this);
	this->InstallObjectAccessors();
}

void DragPart::ReleaseAll()
{
	if (fSrcLink.linkSource != (XMPLinkSource*) kXMPNULL)
	{
		fSrcLink.linkSource->Release();
		fSrcLink.linkSource = (XMPLinkSource*) kXMPNULL;
	}
		
	if (fDstLink.link != (XMPLink*) kXMPNULL)
	{
		fDstLink.link->Release();
		fDstLink.link = (XMPLink*) kXMPNULL;
	}

	XMPReleaseObject(fMenuBar);
	XMPReleaseObject(fTestDragSU);
}

void  DragPart::Externalize()
{
	SimplePart::Externalize();

	XMPStorageUnit   *su; 
	XMPULong          suOffset;
	long              offset;
	XMPStorageUnitRef aSURef;
	XMPFrame         *aFrame;
	StScrpHandle      stylHandle;
	Handle            textHandle;
	Size              textSize, stylSize, oldSize;
	char              textState;
	short             oldSelStart, oldSelEnd;

XMPUnused(offset);
XMPUnused(stylSize);
	
	su = fTestDragSU;

	if (fDocument!=kXMPNULL)
	{
		textHandle = (**(fDocument->theTE)).hText;
		textSize = (**(fDocument->theTE)).teLength;
		textState = HGetState(textHandle);
		su->Focus(kXMPPropText, kXMPPosUndefined, 0, 1, kXMPPosFirstSib);
		oldSize = su->GetSize();
		if (oldSize != XMPGetHandleSize(textHandle))
			su->DeleteValue(oldSize);
		XMPLockHandle(textHandle);
		su->SetValue(textSize, (XMPValue)*textHandle);
		HSetState(textHandle, textState);
	
		oldSelStart = (**(fDocument->theTE)).selStart;
		oldSelEnd   = (**(fDocument->theTE)).selEnd;
		TESetSelect(0, 32767, fDocument->theTE);
		stylHandle = GetStylScrap(fDocument->theTE);	
		su->Focus(kXMPPropStyl, kXMPPosUndefined, 0, 1, kXMPPosFirstSib);
		oldSize = su->GetSize();
		if (oldSize != XMPGetHandleSize((XMPHandle)stylHandle))
			su->DeleteValue(oldSize);
		XMPLockHandle((XMPHandle)stylHandle);
		su->SetValue(XMPGetHandleSize((Handle)stylHandle), (XMPValue)*stylHandle);
		XMPUnlockHandle((XMPHandle)stylHandle);
		XMPDisposeHandle((XMPHandle)stylHandle);
		TESetSelect(oldSelStart, oldSelEnd, fDocument->theTE);
	}
	
	//
	// Write out the display frame
	//
	OrderedCollectionIterator aIter(fDisplayFrames);
 	su->Focus(kXMPPropDisplayFrames, kXMPPosUndefined, 0, 1, kXMPPosFirstSib);
#ifndef SSREMOVEWORKAROUND
	su->Remove();
	su->AddProperty(kXMPPropDisplayFrames);
	su->AddValue(kXMPID);
#else
	XMPULong oldValueSize = su->GetSize();;
#endif
	suOffset = 0;
	for (aFrame = (XMPFrame*)aIter.First(); aIter.IsNotComplete();
											aFrame = (XMPFrame*)aIter.Next(),
											suOffset+=sizeof(XMPStorageUnitRef))
	{
		aSURef = su->GetWeakStorageUnitRef(aFrame->GetStorageUnit());
		su->SetOffset(suOffset);
		su->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
	}
#ifdef SSREMOVEWORKAROUND
	if (oldValueSize > suOffset)
		su->DeleteValue(oldValueSize-suOffset);
#endif

	//
	// Write out the source and destination links
	//
	su->Focus(kXMPPropSourceLink, kXMPPosUndefined, kXMPValue, (XMPValueIndex)0, kXMPPosUndefined);
	if (fSrcLink.linkSource == (XMPLinkSource*) kXMPNULL)
		su->DeleteValue(su->GetSize());
	else {
		fSrcLink.linkSource->Externalize();
		aSURef = su->GetStrongStorageUnitRef(fSrcLink.linkSource->GetStorageUnit());
		su->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
		su->SetValue(sizeof(XMPSShort), (XMPValue) &fSrcLink.start);
		su->SetValue(sizeof(XMPSShort), (XMPValue) &fSrcLink.end);
		}

	su->Focus(kXMPPropDestLink, kXMPPosUndefined, kXMPValue, (XMPValueIndex)0, kXMPPosUndefined);
	if (fDstLink.link == (XMPLink*) kXMPNULL)
	{
		su->DeleteValue(su->GetSize());
	}
	else 
	{
		fDstLink.link->Externalize();
		aSURef = su->GetStrongStorageUnitRef(fDstLink.link->GetStorageUnit());
		su->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
		su->SetValue(sizeof(XMPSShort), (XMPValue) &fDstLink.start);
		su->SetValue(sizeof(XMPSShort), (XMPValue) &fDstLink.end);

		XMPType kind = fDstLink.info.kind;
		fDstLink.info.kind = (XMPType) XMPISOStrLength(kind) + 1;
		su->SetValue(sizeof(XMPLinkInfo), (XMPValue) &fDstLink.info);
		su->SetValue((XMPULong) fDstLink.info.kind, (XMPValue) kind);
		fDstLink.info.kind = kind;
	}

	su = this->GetStorageUnit();
	su->Focus(kXMPPropContents,kXMPPosSame,kXMPKindTestDrag,1,kXMPPosFirstSib);
	aSURef = su->GetStrongStorageUnitRef(fTestDragSU);
	su->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
}

static XMPBoolean IncludedInRange(XMPSShort start, XMPSShort end, XMPSShort startRange, XMPSShort endRange)
{
	return (startRange <= start) && (endRange >= end);
}

void DragPart::ExternalizeSelection(XMPStorageUnit* su, XMPCloneKind cloneKind)
{
	// Write out the current selection to a new storage unit

	XMPStorageUnitRef	aSURef;
	StScrpHandle		stylHandle;
	Handle				textHandle;
	char				textState;

	XMPSShort			selStart	= (**(fDocument->theTE)).selStart;
	XMPSShort			selEnd		= (**(fDocument->theTE)).selEnd;
	XMPULong			textSize	= (selEnd < selStart ? 0 : (XMPULong) (selEnd - selStart));

	XMPBoolean			srcLinkIn	= kXMPFalse;
	XMPBoolean			dstLinkIn	= kXMPFalse;
	XMPULong			linkCount	= 0;

	if ( fSrcLink.linkSource != (XMPLinkSource*) kXMPNULL )
		srcLinkIn = IncludedInRange(fSrcLink.start, fSrcLink.end, selStart, selEnd);

	if ( fDstLink.link != (XMPLink*) kXMPNULL )
		dstLinkIn = IncludedInRange(fDstLink.start, fDstLink.end, selStart, selEnd);

	if ( srcLinkIn )
		linkCount += 1;
	if ( dstLinkIn )
		linkCount += 1;

	XMPDraft*			toDraft = su->GetDraft();
	XMPStorageUnit*		testDragSU = toDraft->CreateStorageUnit();

	// First make the storage unit a legitimate part by adding a part property
	su->AddProperty(kXMPPropPart)->AddValue(kXMPISOStr);
	su->SetValue(XMPISOStrLength(kXMPKindTestDrag) + 1 , (XMPValue) kXMPKindTestDrag);

	// Add the contents property to argument property
	su->AddProperty(kXMPPropContents);

	// Add the TestDrag content value (a reference to another storage unit)
	su->AddValue(kXMPKindTestDrag);	
	aSURef = su->GetStrongStorageUnitRef(testDragSU);
	su->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
	
	// Write out the selected text
	textHandle = (**(fDocument->theTE)).hText;
	textState = HGetState(textHandle);
	XMPLockHandle(textHandle);

	// to the content property
	su->AddValue(kXMPAppleTEXT);			
	su->SetValue(textSize, (XMPValue) ((*(char **)(textHandle)) + selStart));
	
	// and to the TestDrag storage unit
	testDragSU->AddProperty(kXMPPropText)->AddValue(kXMPAppleTEXT);			
	testDragSU->SetValue(textSize, (XMPValue) ((*(char **)(textHandle)) + selStart));

	HSetState(textHandle, textState);

	// Write out the style info
	stylHandle = TEGetStyleScrapHandle(fDocument->theTE);	
	XMPLockHandle((XMPHandle)stylHandle);

	// to the content property
	su->AddValue(kXMPApplestyl);			
	su->SetValue(XMPGetHandleSize((XMPHandle)stylHandle), (XMPValue)*stylHandle);

	// and to the TestDrag storage unit
	testDragSU->AddProperty(kXMPPropStyl)->AddValue(kXMPApplestyl);			
	testDragSU->SetValue(XMPGetHandleSize((XMPHandle)stylHandle), (XMPValue)*stylHandle);

	XMPUnlockHandle((XMPHandle)stylHandle);
	XMPDisposeHandle((XMPHandle)stylHandle);

	if ( linkCount )
	{
		XMPDraftKey draftKey = fDraft->BeginClone(cloneKind);
		TRY
			// Write out the source and destination links
			if ( srcLinkIn )
			{
				XMPStorageUnit* srcLinkCloneSU = fSrcLink.linkSource->CloneTo(draftKey, toDraft, su);

				// The link source should now reference the storage unit in the destination draft
				//srcLinkCloneSU->AddProperty(kXMPPropSourcePart)->AddValue(kXMPWeakStorageUnitRef);
				//aSURef = srcLinkCloneSU->GetWeakStorageUnitRef(su);
				//srcLinkCloneSU->SetValue(sizeof(XMPStorageUnitRef), (XMPValue) &aSURef);

				testDragSU->AddProperty(kXMPPropSourceLink)->AddValue(kXMPValue);
				aSURef = testDragSU->GetStrongStorageUnitRef(srcLinkCloneSU);
				testDragSU->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
				XMPSShort srcLinkStart = fSrcLink.start - selStart;
				testDragSU->SetValue(sizeof(XMPSShort), (XMPValue) &srcLinkStart);
				XMPSShort srcLinkEnd = fSrcLink.end - selStart;
				testDragSU->SetValue(sizeof(XMPSShort), (XMPValue) &srcLinkEnd);

				srcLinkCloneSU->Release();
			}
		
			if ( dstLinkIn )
			{
				XMPStorageUnit* dstLinkCloneSU = fDstLink.link->CloneTo(draftKey, toDraft);

				testDragSU->AddProperty(kXMPPropDestLink)->AddValue(kXMPValue);
				aSURef = testDragSU->GetStrongStorageUnitRef(dstLinkCloneSU);
				testDragSU->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
				XMPSShort dstLinkStart = fDstLink.start - selStart;
				testDragSU->SetValue(sizeof(XMPSShort), (XMPValue) &dstLinkStart);
				XMPSShort dstLinkEnd = fDstLink.end - selStart;
				testDragSU->SetValue(sizeof(XMPSShort), (XMPValue) &dstLinkEnd);
				
				XMPType kind = fDstLink.info.kind;
				fDstLink.info.kind = (XMPType) XMPISOStrLength(kind) + 1;
				testDragSU->SetValue(sizeof(XMPLinkInfo), (XMPValue) &fDstLink.info);
				testDragSU->SetValue((XMPULong) fDstLink.info.kind, (XMPValue) kind);
				fDstLink.info.kind = kind;

				dstLinkCloneSU->Release();
			}
		CATCH_ALL
			fDraft->AbortClone(draftKey);
			RERAISE;
		ENDTRY
		fDraft->EndClone(draftKey);
	}

	testDragSU->Release();
}

//-------------------------------------------------------------------------
// From DragAndDrop protocol
//-------------------------------------------------------------------------

void DragPart::FulfillPromise(XMPStorageUnitView *promiseSUView)
{
	StScrpHandle theStyl;

	theStyl = GetStylScrap(fDocument->theTE);
	if (theStyl)
	{
		XMPLockHandle((Handle)theStyl);
		promiseSUView->SetValue(XMPGetHandleSize((Handle)theStyl), (XMPValue)*theStyl);
		XMPUnlockHandle((Handle)theStyl);
		XMPDisposeHandle((Handle)theStyl);
	}
}

void DragPart::DropCompleted(XMPPart* destPart, XMPDropResult dropResult)
{
XMPUnused(destPart);
XMPUnused(dropResult);
}

void DragPart::DragEnter(XMPDragItemIterator* dragInfo, XMPFacet* facet, XMPPoint where)
{
XMPUnused(facet);
XMPUnused(where);

	XMPStorageUnit *dragSU;
	HFSFlavor	   *valueData;
	long			valueSize;
	
	fDocument->canAccept = true;
	for (dragSU = dragInfo->First(); dragSU; dragSU = dragInfo->Next())
		if (!dragSU->Exists(kXMPPropContents, kXMPAppleTEXT, 0))
		{
			if (!dragSU->Exists(kXMPPropContents, kXMPApplehfs, 0))
			{
				// check for Bento file
				fDocument->canAccept = false;
			}
			else
			{
				dragSU->Focus(kXMPPropContents, kXMPPosUndefined, kXMPApplehfs, 0, kXMPPosUndefined);		
				valueSize = dragSU->GetSize();
				valueData = (HFSFlavor*) XMPNewPtr(valueSize);
				if (!valueData)
				{
					fDocument->canAccept = false;
				}
				else
				{
					dragSU->GetValue(valueSize, (XMPValue)valueData);
					if ((valueData->fileType != 'TEXT') && (valueData->fileType != 'sEXT') &&
						(valueData->fileType != 'bnto'))					// sEXT = TeachText stationery
						fDocument->canAccept = false;
				}
				XMPDisposePtr(valueData);
			}
		}
	fDocument->caretTime = TickCount();
	fDocument->caretOffset = fDocument->lastOffset = fDocument->insertPosition = -1;
	fDocument->caretShow = true;
	fDocument->cursorInContent = false;
}

void DragPart::DragWithin(XMPDragItemIterator* dragInfo, XMPFacet* facet, XMPPoint where)
{
	XMPUnused(dragInfo);

	short offset;
	long theTime = TickCount();
	GrafPtr	savedPort;
	XMPULong attributes;
	DragReference theDrag;
	RgnHandle dragHilite;
	XMPPoint windowWhere = facet->GetWindowFrameTransform()->TransformPoint(where);
	
	if (!fDocument->canAccept)
		return;

	GetPort(&savedPort);
	SetPort(fDocument->theWindow);
	
	this->ClipThis(facet);
	
	attributes = fSession->GetDragAndDrop()->GetDragAttributes();
	theDrag = fSession->GetDragAndDrop()->GetDragReference();

	if (PtInRect(windowWhere.AsQDPoint(), &(**(fDocument->theTE)).viewRect))
	{
		if (!fDocument->cursorInContent && (attributes & kXMPdragHasLeftSourceFrame)) 
		{
			dragHilite = NewRgn();
			RectRgn(dragHilite, &(**(fDocument->theTE)).viewRect);
			ShowDragHilite(theDrag, dragHilite, true);
		}
		fDocument->cursorInContent = true;
		
		offset = TEGetOffset(windowWhere.AsQDPoint(), fDocument->theTE);

		if ((TEIsFrontOfLine(offset, fDocument->theTE)) && (offset) &&			
			((*((char **)((**(fDocument->theTE)).hText)))[offset - 1] != 0x0D) &&
			(ff(TEGetPoint(offset - 1, fDocument->theTE).h) < windowWhere.x))
			offset--;
		
		// If this is the source frame, do not allow tracking through the selection
		if ((attributes & kXMPdragIsInSourceFrame) &&
		    (offset >= (**(fDocument->theTE)).selStart) &&
			(offset <= (**(fDocument->theTE)).selEnd))
			offset = -1;
		
		fDocument->insertPosition = offset;

		//
		//	Reset flashing counter if the offset has moved.
		//
		if (offset != fDocument->lastOffset) 
		{
			fDocument->caretTime = theTime;
			fDocument->caretShow = true;
		}
		fDocument->lastOffset = offset;

		//
		//	Flash caret.
		//
		if (theTime - fDocument->caretTime > GetCaretTime())
		{
			fDocument->caretShow = !fDocument->caretShow;
			fDocument->caretTime = theTime;
		}
		if (!fDocument->caretShow)
			offset = -1;

		//
		//	If offset has changed, move caret on screen.
		//
		if (offset != fDocument->caretOffset) 
		{
			if (offset != -1)
				DrawCaret(offset, fDocument->theTE, 0L);
				
			if (fDocument->caretOffset != -1)
				DrawCaret(fDocument->caretOffset, fDocument->theTE, 0L);
		}
		
		fDocument->caretOffset = offset;
	}
	else
	{
		if (fDocument->cursorInContent && (attributes & kXMPdragHasLeftSourceFrame)) 
		{
			HideDragHilite(theDrag);
		}
		fDocument->cursorInContent = false;
		fDocument->lastOffset = -1;
		fDocument->insertPosition = -1;
	}
	
	SetPort(savedPort);
}

void DragPart::DragLeave(XMPFacet* facet, XMPPoint where)
{
XMPUnused(facet);
XMPUnused(where);

	GrafPtr	savedPort;
	XMPULong attributes;
	DragReference theDrag;
	
	if (!fDocument->canAccept)
		return;

	GetPort(&savedPort);
	SetPort(fDocument->theWindow);
	
	attributes = fSession->GetDragAndDrop()->GetDragAttributes();
	theDrag = fSession->GetDragAndDrop()->GetDragReference();

	//
	//	If the caret is on the screen, remove it.
	//
	if (fDocument->caretOffset != -1) 
	{
		DrawCaret(fDocument->caretOffset, fDocument->theTE, 0L);
		fDocument->caretOffset = -1;
	}

	//
	//	Remove frame highlighting, if showing.
	//
	if (fDocument->cursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
	{
		HideDragHilite(theDrag);
	}
	
	SetPort(savedPort);
}

XMPDropResult DragPart::Drop(XMPDragItemIterator* dropInfo, XMPFacet* facet, XMPPoint where)
{
XMPUnused(where);
XMPUnused(facet);

	GrafPtr				savedPort;
	Size				textSize, stylSize;
	short				selStart, selEnd, mouseDownModifiers, mouseUpModifiers, moveText;
	//WindowOffscreen		*theOffscreen;
	Rect				theRect, srcRect;
	Point				thePoint;
	XMPULong            attributes;
	DragReference       theDrag;
	XMPStorageUnit     *dropSU;
	OSErr				error = noErr;
	short				fileRefNum = 0;
	long				fileSize;
	XMPPtr				fileData = kXMPNULL;
	XMPPtr				textData = kXMPNULL;
	TEHandle			tempTE = kXMPNULL;
	StScrpHandle        stylHandle = kXMPNULL;
	XMPDropResult		dropResult = kXMPDropCopy;
	XMPBoolean			notDone = kXMPTrue;
	
XMPVolatile(fileData);
XMPVolatile(textData);
XMPVolatile(stylHandle);
XMPVolatile(tempTE);
XMPVolatile(fileRefNum);
XMPVolatile(exit);


	if ((!fDocument->canAccept) || (fDocument->insertPosition == -1))
		return(kXMPDropFail);

	GetPort(&savedPort);
	SetPort(fDocument->theWindow);
	
	attributes = fSession->GetDragAndDrop()->GetDragAttributes();
	theDrag = fSession->GetDragAndDrop()->GetDragReference();
	GetDragModifiers(theDrag, 0L, &mouseDownModifiers, &mouseUpModifiers);
	moveText = (attributes & kXMPdragIsInSourceFrame) &&
			   (!((mouseDownModifiers & optionKey) | (mouseUpModifiers & optionKey)));

	//
	//	Loop through all of the drag items contained in this drag and collect the text
	//	into the tempTE record.
	//

	SetRect(&theRect, 0, 0, 0, 0);
	tempTE = TEStylNew(&theRect, &theRect);

	TRY
		for (dropSU = dropInfo->First(); dropSU && notDone; dropSU = dropInfo->Next())
	{
		//
		//  If canAccept is true then 'TEXT' or 'hfs' must be present in all items
		//
		if (dropSU->Exists(kXMPPropContents, kXMPAppleTEXT, 0))
		{
			dropSU->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT, 0, kXMPPosUndefined);
			textSize = dropSU->GetSize();
				textData = XMPNewPtr(textSize);
			dropSU->GetValue(textSize, (XMPValue)textData);
	
			//
			//	Check for optional styl data for the TEXT.
			//
			if (dropSU->Exists(kXMPPropContents, kXMPApplestyl, 0))
			{
				dropSU->Focus(kXMPPropContents, kXMPPosUndefined, kXMPApplestyl, 0, kXMPPosUndefined);
				stylSize = dropSU->GetSize();
					stylHandle = (StScrpHandle) XMPNewHandle(stylSize);
					dropSU->GetValue(stylSize, (XMPValue) XMPLockHandle((XMPHandle) stylHandle));
					XMPUnlockHandle((XMPHandle) stylHandle);
				}
				notDone = kXMPFalse;
			}
		else if (dropSU->Exists(kXMPPropContents, kXMPApplehfs, 0))
		{
			dropSU->Focus(kXMPPropContents, kXMPPosUndefined, kXMPApplehfs, 0, kXMPPosUndefined);		
			fileSize = dropSU->GetSize();		// really hfsflavor size
				fileData = XMPNewPtr(fileSize);			// really hfsflavor data
				dropSU->GetValue(fileSize, (XMPValue)fileData);
				
				THROW_IF_ERROR(FSpOpenDF((FSSpec*)&(((HFSFlavor*)fileData)->fileSpec), fsRdPerm, &fileRefNum));
				THROW_IF_ERROR(GetEOF(fileRefNum,&textSize));
				if (textSize == 0)
					THROW(kXMPErrUndefined);
					
				textData = XMPNewPtr(textSize);
				
				THROW_IF_ERROR(SetFPos(fileRefNum,fsFromStart,0));
				THROW_IF_ERROR(FSRead(fileRefNum,&textSize,textData));
				THROW_IF_ERROR(FSClose(fileRefNum));
				notDone = kXMPFalse;
		}
		else  if (dropSU->Exists(kXMPPropStorageUnitType, kXMPPart, 0))
		{
			if (dropSU->Exists(kXMPPropText, kXMPAppleTEXT, 0))
			{
				dropSU->Focus(kXMPPropText, kXMPPosUndefined, kXMPAppleTEXT, 0, kXMPPosUndefined);
				textSize = dropSU->GetSize();
					textData = XMPNewPtr(textSize);
				dropSU->GetValue(textSize, (XMPValue)textData);
				
				if (dropSU->Exists(kXMPPropStyl, kXMPApplestyl, 0))
				{
					dropSU->Focus(kXMPPropStyl, kXMPPosUndefined, kXMPApplestyl, 0, kXMPPosUndefined);
					stylSize = dropSU->GetSize();
						stylHandle = (StScrpHandle) XMPNewHandle(stylSize);
						dropSU->GetValue(stylSize, (XMPValue) XMPLockHandle((XMPHandle) stylHandle));
						XMPUnlockHandle((XMPHandle) stylHandle);
					}
					notDone = kXMPFalse;
				}
			}
		//
		//	Insert this drag item's text into the tempTE.
		//
		TESetSelect(32767, 32767, tempTE);
		TEStylInsert(textData, textSize, stylHandle, tempTE);

			if (textData)
				XMPDisposePtr(textData);
			if (stylHandle)
				XMPDisposeHandle((XMPHandle) stylHandle);
		}
		
	CATCH_ALL
		WASSERTM(kXMPFalse, "Die in Drop.");
		if (tempTE != kXMPNULL)
			TEDispose(tempTE);
		if (fileData != kXMPNULL)
			XMPDisposePtr(fileData);
		if (stylHandle != kXMPNULL)
			XMPDisposeHandle((XMPHandle) stylHandle);
		if (fileRefNum != 0)
			FSClose(fileRefNum);
		dropResult = kXMPDropFail;
	ENDTRY
	
	if ( (dropResult == kXMPDropCopy) || (dropResult == kXMPDropMove) ) {

	//
	//	Pull the TEXT and styl data out of the tempTE handle.
	//
	textSize = (**tempTE).teLength;
	textData = XMPNewPtr(textSize);
	if (!textData) 
	{
		DebugStr("\pNewPtr error");
		TEDispose(tempTE);
		return(kXMPDropFail);
	}
	XMPBlockMove(*(**tempTE).hText, textData, textSize);
	TESetSelect(0, 32767, tempTE);
	stylHandle = GetStylScrap(tempTE);
	TEDispose(tempTE);

	if (textSize)
	{
		//
		//	If the caret or highlighting is on the screen, remove it/them.
		//
		if (fDocument->caretOffset != -1) 
		{
			DrawCaret(fDocument->caretOffset, fDocument->theTE, 0L);
			fDocument->caretOffset = -1;
		}
		if (fDocument->cursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
		{
			HideDragHilite(theDrag);
		}

		//
		//	If the frame is not active, must activate TE before inserting
		//	text or the background hilite will not update correctly.
		//
		if (!fDocument->frameIsActive)
			TEActivate(fDocument->theTE);

		//
		//	If this window is also the sender, delete source selection if no option key.
		//	
		if (moveText)
		{
			selStart = (**(fDocument->theTE)).selStart;
			selEnd   = (**(fDocument->theTE)).selEnd;
			if ( WhiteSpaceAtOffset(selStart - 1, fDocument->theTE) &&
				!WhiteSpaceAtOffset(selStart, fDocument->theTE) &&
				!WhiteSpaceAtOffset(selEnd - 1, fDocument->theTE) &&
				 WhiteSpaceAtOffset(selEnd, fDocument->theTE)) {
				 
				 if (GetCharAtOffset(selEnd, fDocument->theTE) == ' ')
					(**(fDocument->theTE)).selEnd++;
			}
			if (fDocument->insertPosition > selStart) {
				fDocument->insertPosition -= ((**(fDocument->theTE)).selEnd -
											  (**(fDocument->theTE)).selStart);
			}
			srcRect = (**fDocument->hiliteRgn).rgnBBox;
			TEDelete(fDocument->theTE);
		}
		
		InsertTextAtOffset(fDocument->insertPosition, (char*) textData, textSize, stylHandle, fDocument->theTE);
		
		TEGetHiliteRgn(fDocument->hiliteRgn, fDocument->theTE);
		
		if (moveText)
		{
			theRect = (**fDocument->hiliteRgn).rgnBBox;
			SetPt(&thePoint,0,0);
			LocalToGlobal(&thePoint);
			OffsetRect(&srcRect, thePoint.h, thePoint.v);
			OffsetRect(&theRect, thePoint.h, thePoint.v);
			ZoomRects(&srcRect, &theRect, 12, zoomDecelerate);
		}		

		fDraft->SetChangedFromPrev();
	}

		if (textData)
			XMPDisposePtr(textData);
		if (fileData)
			XMPDisposePtr(fileData);
	
		if (stylHandle)
			XMPDisposeHandle((XMPHandle) stylHandle);
	
	//
	//	Undo the TEActivate, if needed.
	//
	if (!fDocument->frameIsActive)
		TEDeactivate(fDocument->theTE);

	//
	//  Signal the draw method to update the scroll bars
	//
	fFrameShapeChanged = true;

	facet->Invalidate(kXMPNULL);
	}

	SetPort(savedPort);
	return dropResult;
}

void DragPart::FacetAdded(XMPFacet* facet)
{
	this->CreateDocument(facet->GetFrame());
}

void DragPart::AddDisplayFrame(XMPFrame* newFrame)
{
	fDisplayFrames->AddLast(newFrame);
	newFrame->IncrementRefCount();
}

void DragPart::RemoveDisplayFrame(XMPFrame* oldFrame)
{
	if (fDisplayFrames->Contains(oldFrame))
	{
		if (fDocument->frameIsActive)
			fSession->GetDispatcher()->UnregisterIdle(this, oldFrame);
		this->DeActivateFrame(oldFrame);
		fDisplayFrames->Remove(oldFrame);
		oldFrame->Release();
	}
	else
	{
		THROW_IF_ERROR(kXMPErrInvalidFrame);
	}
}

void DragPart::CloseDisplayFrame(XMPFrame* frame)
{
	if (fDisplayFrames->Contains(frame))
	{
		if (fDocument->frameIsActive)
			fSession->GetDispatcher()->UnregisterIdle(this, frame);
		this->DeActivateFrame(frame);
		fDisplayFrames->Remove(frame);
		frame->Release();
	}
	else
	{
		THROW_IF_ERROR(kXMPErrInvalidFrame);
	}
}

void DragPart::ExternalTransformChanged(XMPFacet* facet)
{
	XMPUnused(facet);
	
	this->PositionDocumentParts();
	this->AdjustScrollBar();
	TEGetHiliteRgn(fDocument->hiliteRgn, fDocument->theTE);
	fFrameShapeChanged = false;
}

void DragPart::Draw(XMPFacet* facet, XMPShape* invalidShape)
{
	XMPUnused(invalidShape);
	
	XMPPoint xForm;
	RgnHandle clipRgn, saveClip, frameShape;
	Rect bounds;
	GrafPtr	savedQDPort, oldTEPort;
	PrInfoPtr prInfo;
	
	XMPBoolean drawing = facet->GetCanvas()->IsDynamic();
	
	if (facet)
	{
		GetPort(&savedQDPort);
		if (drawing)
		{
			SetPort(facet->GetCanvas()->GetPlatformCanvas());
		}
		else
		{
			prInfo = (PrInfoPtr) facet->GetCanvas()->GetPlatformCanvas();
			
			oldTEPort = (**(fDocument->theTE)).inPort;
			(**(fDocument->theTE)).inPort = (GrafPtr) prInfo->printingPort;
		}
		SetOrigin(0,0);

		xForm = this->GetAggregateTranslation(facet);

		frameShape = facet->GetFrame()->GetFrameShape()->GetQDRegion();
		clipRgn = facet->GetClipShape()->CopyQDRegion();
		saveClip = NewRgn();
		
		GetClip(saveClip);
		bounds = (**frameShape).rgnBBox;
		OffsetRect(&bounds, xForm.IntX(), xForm.IntY());
		OffsetRgn(clipRgn, xForm.IntX(), xForm.IntY());
		SetClip(clipRgn);
		EraseRect(&(**clipRgn).rgnBBox);

		if( drawing )
			DrawControls(fDocument->theWindow);
		
		TEUpdate(&bounds, fDocument->theTE);

		if (!drawing)									// restore the window port
			(**(fDocument->theTE)).inPort = oldTEPort;

		SetPort(savedQDPort);	
		SetClip(saveClip);
		
		DisposeRgn(saveClip);
		DisposeRgn(clipRgn);
	}
}

XMPBoolean 	DragPart::HandleEvent(XMPEventData event, XMPFrame* frame, XMPFacet* facet)
{
	XMPBoolean handled = kXMPFalse;
	
	switch (event->what)
	{
		case nullEvent:
			this->Idle(frame);
			break;
		
		case mouseDown:
			{
				Point windowPoint = event->where;
				GrafPtr curPort;
				GetPort(&curPort);
//				SetPort(facet->GetWindow()->GetPlatformWindow());
				SetOrigin(0,0);
				GlobalToLocal(&windowPoint);
				handled = this->HandleMouseDown(facet,windowPoint,event);
				SetPort(curPort);
			}
			break;
			
		case kXMPEvtMouseDownEmbedded:
			{
				Point windowPoint = event->where;
				GrafPtr curPort;
				GetPort(&curPort);
//				SetPort(facet->GetWindow()->GetPlatformWindow());
				SetOrigin(0,0);
				GlobalToLocal(&windowPoint);
				handled = this->HandleMouseDownInBorder(facet, 
					(XMPFacet*) event->message, windowPoint,event);
				SetPort(curPort);
			}
			break;
		
		case mouseUp:
			break;

		case keyDown:
		case autoKey:
			handled = this->HandleKeyDown(frame, event);
			break;
	
		case activateEvt:
			handled = kXMPTrue;
			if ((event->modifiers & activeFlag) != 0)
				this->ActivatingWindow(frame);
			else
				this->DeActivatingWindow(frame);
			break;

		case keyUp:
			break;

		case osEvt:
			break;
	
		case kHighLevelEvent:
			break;

		case kXMPEvtMenu:
			handled = this->HandleMenuEvent(frame, event);
			break;

		default:
			return kXMPFalse;
	}
	return handled;
}


XMPBoolean	DragPart::HandleMouseDown(	XMPFacet* facet,
										XMPPoint where,
										XMPEventData event)
{
	GrafPtr			curPort;
	short			thePart;
	Point			thePoint;
	ControlHandle	theControl;
	Point	 		xForm, localPoint;
	RgnHandle		saveClip, clipRgn;
	
	if (!facet->GetWindow()->IsActive())
		SelectWindow(facet->GetWindow()->GetPlatformWindow());
	else 
	{
		this->ActivateFrame(facet->GetFrame());

		GetPort(&curPort);
		SetPort(fDocument->theWindow);
		thePoint = event->where;
		GlobalToLocal(&thePoint);

		xForm = this->GetAggregateTranslation(facet).AsQDPoint();
		localPoint = where.AsQDPoint();
		SubPt(xForm, &localPoint);
		
		clipRgn = facet->GetClipShape()->CopyQDRegion();
		saveClip = NewRgn();
		GetClip(saveClip);
		
		OffsetRgn(clipRgn, xForm.h, xForm.v);
		SetClip(clipRgn);

		// <eeh> better to put this on the stack, but I can't get it to work
		ControlActionUPP scrollProcUPP = NewControlActionProc( ScrollProc ) ;

		thePart = FindControl(thePoint, fDocument->theWindow, &theControl);
		if (thePart) 
		{
			if (theControl == fDocument->vScroll)
			{
				if (thePart == inThumb) 
				{
					TrackControl(theControl, thePoint, 0L);
					AdjustDocumentView(fDocument);
				}
				else 
				{
					TrackControl( theControl, thePoint, scrollProcUPP ) ;
				}
				this->AdjustScrollBar();
			}
		}
		else if (PtInRgn(thePoint, fDocument->hiliteRgn))
		{
			if (!this->DragText(event)) 
			{
				TEClick(thePoint, false, fDocument->theTE);
			}
		}
		else if (PtInRect(thePoint, &(**(fDocument->theTE)).viewRect))
		{
			TEClick(thePoint, (event->modifiers & shiftKey) != 0, fDocument->theTE);
		}
		DisposeRoutineDescriptor( scrollProcUPP ) ;

		TEGetHiliteRgn(fDocument->hiliteRgn, fDocument->theTE);
		
		SetClip(saveClip);
		
		DisposeRgn(saveClip);
		DisposeRgn(clipRgn);

		SetPort(curPort);
		
		// Prevent selection of less than an entire destination link
		this->ExpandSelectionAroundLink();
	}
	return true;
}


XMPLink* DragPart::CreateLink(XMPPtr data, XMPULong size)
{
XMPUnused(data);
XMPUnused(size);
	TEHandle myTE = fDocument->theTE;
	XMPULong textSize;

	if (fSrcLink.linkSource == (XMPLinkSource*) kXMPNULL) {
		textSize = fSrcLink.end-fSrcLink.start;
		if (textSize > 0) {
			fSrcLink.linkSource = fDraft->CreateLinkSource(this);
			this->LinkChanged(&fSrcLink, fSession->UniqueChangeID());
			}
		}

	XMPLink* link = fSrcLink.linkSource->GetLink();
	link->IncrementRefCount();
	return link;
}

void DragPart::LinkChanged(SrceOfLink* srcLink, XMPChangeID changeID)
{
	TEHandle myTE = fDocument->theTE;
	XMPULong textSize;
	CharsHandle text;

	if (srcLink->linkSource != ((XMPLinkSource*) kXMPNULL)) {
		XMPLinkKey key;
		if (srcLink->linkSource->Lock(0, &key)) {
			XMPStorageUnit* su = srcLink->linkSource->GetContentStorageUnit(key);
			if (su->Exists(kXMPPropContents, (XMPValueType) kXMPNULL, 0) == kXMPFalse) {
				su->AddProperty(kXMPPropContents);
				}
			if (su->Exists(kXMPPropContents, kXMPAppleTEXT, 0) == kXMPFalse) {
				su->AddValue(kXMPAppleTEXT);
				}
			else {
				XMPULong valueSize;
				su->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT, 0, kXMPPosUndefined);
				valueSize = su->GetSize();
				su->DeleteValue(valueSize);
				}
			su->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT, 0, kXMPPosUndefined);
			textSize = srcLink->end - srcLink->start;
			text = (CharsHandle) (*myTE)->hText;
			XMPLockHandle((Handle)text);
			su->SetValue(textSize, (XMPValue) ((*text) + srcLink->start));
			XMPUnlockHandle((Handle)text);
			srcLink->linkSource->Unlock(key);
			srcLink->linkSource->ContentChanged(changeID);
			}
		}
}

void DragPart::LinkUpdated(XMPLink* updatedLink, XMPChangeID id)
{
	if (updatedLink == fDstLink.link) {
		XMPSShort		oldSize = fDstLink.end - fDstLink.start;
		XMPSShort		newSize;
		XMPPtr			text;
		XMPSShort		selStart = (**(fDocument->theTE)).selStart;
		XMPSShort		selEnd = (**(fDocument->theTE)).selEnd;
		
		// Delete the existing linked text
		TESetSelect(fDstLink.start, fDstLink.end, fDocument->theTE);
		TEDelete(fDocument->theTE);
		
		// Insert the new linked text
		XMPLinkKey key;
		if (fDstLink.link->Lock(0, &key)) {
			XMPStorageUnit*	su = fDstLink.link->GetContentStorageUnit(key);
			su->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT, 0, kXMPPosUndefined);
			newSize = (XMPSShort) su->GetSize();
			if (newSize > 0) {
				text = XMPNewPtr(newSize);
				su->GetValue(newSize, (XMPValue) text);
				TEInsert(text, newSize, fDocument->theTE);
				XMPDisposePtr (text);
				}
			
			fDstLink.info.change = fDstLink.link->GetChangeID();
			fDstLink.info.changeTime = fDstLink.link->GetChangeTime();

			// Adjust other links due to this change
			// Special cased for source links only for now
			// Assume no overlapping links or recursion problems
			AdjustSrcLinks(fDstLink.start, fDstLink.end, fDstLink.start + newSize, id);
			// Can we let AdjustDstLinks take care of this one too???
			fDstLink.end = fDstLink.start + newSize;

			fDstLink.link->Unlock(key);
			}
			
		// Restore the original selection; Doesn't handle overlapping 
		if (selStart > fDstLink.start) {
			selStart -= (oldSize - newSize);
			}
		if (selEnd > fDstLink.start) {
			selEnd -= (oldSize - newSize);
			}
		TESetSelect(selStart, selEnd, fDocument->theTE);

		// Don't panic if the link's content is now empty
		//if (newSize == 0) {
			// Can't call UnregisterDependent inside LinkUpdated
			// Undo may cause us to re-create this link, anyway
			// if (fDstLink.link) fDstLink.link->UnregisterDependent(this);
		//	fDstLink.link->Release();
		//	fDstLink.link = (XMPLink*) kXMPNULL;
		//	fDstLink.start = -1;
		//	fDstLink.end = -1;
		//	}
		}
	else {
		DebugStr("\pUnknown link passed to LinkUpdated()!");
		}
}

void DragPart::ExpandSelectionAroundLink()
// If the current selection includes a portion of a link destination,
// expand the selection to encompas the entire link

{
	XMPSShort selStart = (**(fDocument->theTE)).selStart;
	XMPSShort selEnd = (**(fDocument->theTE)).selEnd;
	
	// Make sure the deletion includes the entire link destination
	if ((selStart > fDstLink.start) && (selStart < fDstLink.end))
		selStart = fDstLink.start;
	if ((selEnd > fDstLink.start) && (selEnd < fDstLink.end))
		selEnd = fDstLink.end;
	TESetSelect(selStart, selEnd, fDocument->theTE);
}

void DragPart::CheckDstLinkDeletion (XMPSShort start, XMPSShort endOld)
{
	if ((start <= fDstLink.start) && (endOld >= fDstLink.end)) {
		// Destination link has been removed.
		this->RemoveDstLink();
		}
}

void DragPart::AdjustLinks(
						XMPSShort 	start,
						XMPSShort 	endOld,
						XMPSShort 	endNew,
						XMPChangeID changeID)
// Adjust source and destination links following a change to the text edit record
{
	this->AdjustDstLinks(start, endOld, endNew);
	this->AdjustSrcLinks(start, endOld, endNew, changeID);
}

void DragPart::AdjustDstLinks(XMPSShort start, XMPSShort endOld, XMPSShort endNew)
// Adjust destination links following a change to the text edit record
{
	if (endOld > start) {
		// Some content was replaced, so see if the destination link
		// was deleted
		this->CheckDstLinkDeletion (start, endOld);
		}
	
	if (start >= fDstLink.end) {
		// No change to destination link
		}
	else {
		// Must be entirely before the destination link
		if (fDstLink.link) {
			fDstLink.start += (endNew - endOld);
			fDstLink.end += (endNew - endOld);
			}
		}
}
			
void DragPart::AdjustSrcLinks(
						XMPSShort 	start,
						XMPSShort 	endOld,
						XMPSShort 	endNew,
						XMPChangeID changeID)
// Adjust source link offsets following a change to the text edit record
// Text in the range start..endOld was replaced by the range start..endNew.
// Note that this routine calls ContentChanged(); this can cause the LinkUpdated
// method of this object to be called, which calls this method.
// On entry, fSrcLink contains old offsets.  On exit, fSrcLink contains new offsets.
{
	if ((start >= fSrcLink.end) && (endOld > fSrcLink.end)) {
		// Source link not affected
		// Note that we don't fall in here if text is inserted immediately after
		// the link, that is, if start == endOld == fSrcLink.end
		return;
		}
	if (((start < fSrcLink.start) && (endOld >= fSrcLink.end)) ||
		((start <= fSrcLink.start) && (endOld > fSrcLink.end)) ||
		((endNew == start) && (start == fSrcLink.start) && (endOld == fSrcLink.end))) {
		// Fall in here if
		//  The text range start..endOld included more than the entire link, or
		//  Exactly the link was deleted with no replacement
		// In either case, delete the link
		this->RemoveSrcLink();
		return;
		}
	if ((endOld <= fSrcLink.start) && (start < fSrcLink.start)) {
		// Changes occurred ahead of the source link, so just move it accordingly
		// (Treat the case of an insertion point at the beginning of the link as
		// inserting into the link; handled below).
		fSrcLink.start += (endNew - endOld);
		fSrcLink.end += (endNew - endOld);
		return;
		}
	if (start == endOld) {
		// Additions occurred at the insertion point after the last character in
		// the link
		fSrcLink.end += (endNew - endOld);
		}
	else if (start < fSrcLink.start) {
		// Only that portion of link unaffected remains within the link
		fSrcLink.start = start + endNew; 
		fSrcLink.end += (endNew - endOld);
		}
	else if (endOld > fSrcLink.end) {
		// Only that portion of link unaffected remains within the link
		// The replaced text starts in the selection and continues beyond
		fSrcLink.end = start;
		}
	else {
		// Replaced text lies entirely within the link, so all new text
		// is included in the link
		fSrcLink.end += (endNew - endOld);
		}

	this->LinkChanged(&fSrcLink, changeID);
	this->RemoveLinkSpec();
}

void DragPart::RemoveDstLink()
{
	if (fDstLink.link != ((XMPLink*) kXMPNULL)) {
		fDstLink.link->UnregisterDependent(this);
		fDstLink.link->Release();
		}
	fDstLink.link = (XMPLink*) kXMPNULL;
	fDstLink.start = -1;
	fDstLink.end = -1;
	fDstLink.info.kind = (XMPType) kXMPNULL;
	fDraft->SetChangedFromPrev();
}

void DragPart::RemoveSrcLink()
{
	XMPLinkSource* linkSource = fSrcLink.linkSource;
	
	// Null the source link prior to calling ContentChanged() to prevent
	// indefinite recursion.
	fSrcLink.linkSource = (XMPLinkSource*) kXMPNULL;

	if (linkSource != ((XMPLinkSource*) kXMPNULL)) {
		XMPLinkKey key;
		if (linkSource->Lock(0, &key)) {
			XMPStorageUnit* su = linkSource->GetContentStorageUnit(key);
			XMPULong size;
			su->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT, 0, kXMPPosUndefined);
			size = su->GetSize();
			su->DeleteValue(size);
			linkSource->Unlock(key);
			// $$$$$ Need to use the change id associated with the operation that
			//  caused the source link to be removed
			linkSource->ContentChanged(kXMPUnknownChange);
			linkSource->Release();
			}
		}
	RemoveLinkSpec();
	fSrcLink.start = -1;
	fSrcLink.end = -1;
}

void DragPart::RemoveLinkSpec()
{
	// Remove the link spec from the clipboard
	XMPClipboard* clipboard = fSession->GetClipboard();
	if (fClipboardChangeID == clipboard->GetChangeID()) {
		XMPClipboardKey key;
		if (clipboard->Lock(0, &key)) {
			TRY
				XMPStorageUnit* clipRootSU = clipboard->GetContentStorageUnit(key);
				if (clipRootSU->Exists(kXMPPropLinkSpec, (XMPValueType) kXMPNULL, 0) == kXMPTrue) {
					clipRootSU->Focus(kXMPPropLinkSpec, kXMPPosUndefined, (XMPValueType) kXMPNULL, 0, kXMPPosUndefined);
					clipRootSU->Remove();
					}
			CATCH_ALL
			ENDTRY
			clipboard->Unlock(key);
			}
		}
}

XMPBoolean	DragPart::HandleMouseDownInBorder(XMPFacet* container,
								XMPFacet* facet,
								XMPPoint where,
								XMPEventData event)
{
XMPUnused(container);
XMPUnused(facet);
XMPUnused(where);
XMPUnused(event);
	SysBeep(2);
	return kXMPFalse;
}

XMPBoolean	DragPart::HandleKeyDown(XMPFrame* focusFrame, XMPEventData event)
{
	XMPBoolean handled = kXMPFalse;
	
	if (/*!ParentPrtRCR::HandleKeyDown(focusFrame, event) &&*/ focusFrame)
	{
		if (event->modifiers & (0xFFFFFFFF & cmdKey))	// was cmd key down?
		{
			XMPULong mcValue = MenuKey((short)(event->message & charCodeMask));
			XMPCommandID command = fMenuBar->GetCommand(HiWord(mcValue),LoWord(mcValue));
			handled = this->DoCommand(focusFrame,command);
		}
		else
		{
			XMPSShort delStart;
			XMPSShort delEnd;
			XMPSShort newEnd;
			char key = (char) (event->message) & charCodeMask;
			
			delStart = (**(fDocument->theTE)).selStart;
			delEnd = (**(fDocument->theTE)).selEnd;
			
			XMPFrameFacetIterator* fiter = focusFrame->CreateFacetIterator();
			this->ClipThis(fiter->First());
	
			TEKey(key, fDocument->theTE);
			this->AdjustScrollBar();
			TEGetHiliteRgn(fDocument->hiliteRgn, fDocument->theTE);
	
			this->ClipToRoot(fiter->First());
			handled = kXMPTrue;
			delete fiter;
			
			if ((key == kRightArrow) || (key == kLeftArrow) || (key == kUpArrow) || (key == kDownArrow)) {
				ExpandSelectionAroundLink();
				}
			else {
				newEnd = (**(fDocument->theTE)).selEnd;
				if ((delEnd > delStart) || (newEnd != delEnd)) {
					// Some change occurred
					if ((delEnd == delStart) && (newEnd < delStart)) {
						// Special case for delete key
						delStart = newEnd;
						}
					this->AdjustLinks(delStart, delEnd, newEnd, fSession->UniqueChangeID());
					}
				fDraft->SetChangedFromPrev();
				}
		}
	}
	return (handled);
}

XMPBoolean DragPart::DoCut(XMPFrame* frame)
{
	XMPClipboardKey key;
	XMPClipboard* clipboard = fSession->GetClipboard();
	if (clipboard->Lock(0, &key))
	{
		XMPStorageUnit* clipBSU = kXMPNULL;
		
		XMPSShort delStart = (**(fDocument->theTE)).selStart;
		XMPSShort delEnd = (**(fDocument->theTE)).selEnd;

		XMPPtr text;
		XMPHandle style;
		
		TRY
			clipboard->Clear(key);
			clipBSU = clipboard->GetContentStorageUnit(key);
			this->ExternalizeSelection(clipBSU, kXMPCloneCut);
			fClipboardChangeID = clipboard->GetChangeID();
		CATCH_ALL
			WASSERTM(kXMPFalse, "Cut to clipboard failed");
		ENDTRY
			
		text = XMPNewPtr(delEnd-delStart);
		XMPBlockMove((XMPPtr)((*(char **)((**(fDocument->theTE)).hText)) + (**(fDocument->theTE)).selStart),text,delEnd-delStart);
		
		TEDelete(fDocument->theTE);

		this->AdjustLinks(delStart, delEnd, delStart, fSession->UniqueChangeID());
		
		style = (XMPHandle)GetStylScrap(fDocument->theTE);

		this->PushUndoAction(frame,kXMPCommandCut,delStart,delEnd,text,style,
								"Undo Cut", "Redo Cut");
		clipboard->Unlock(key);

		fDraft->SetChangedFromPrev();
	}
	return kXMPTrue;
}

void DragPart::UndoRedoCut(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

static XMPBoolean RangesOverlap(XMPSShort start1, XMPSShort end1,XMPSShort start2,XMPSShort end2)
// Returns kXMPTrue if the two argument text offset ranges overlap
{
	if (start1 >= end2)
		return kXMPFalse;
	else if (start2 >= end1)
		return kXMPFalse;
	else if (start1 < start2) {
		if (end1 <= start2)
			return kXMPFalse;
		else
			return kXMPTrue;
		}
	else
		return kXMPTrue;
}

XMPBoolean DragPart::DoCopy(XMPFrame* frame)
{
	XMPUnused(frame);
	
	XMPClipboardKey key;
	XMPClipboard* clipboard = fSession->GetClipboard();
	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* clipBSU = kXMPNULL;
			XMPBoolean makeLinkSpec = kXMPFalse;

			clipboard->Clear(key);
			clipBSU = clipboard->GetContentStorageUnit(key);
			
			// Add a property indicating the part that wrote this storage unit
			//clipBSU->AddProperty(kXMPPropPart);
			//clipBSU->AddValue(kXMPISOStr);
			//clipBSU->SetValue(???);
			
			this->ExternalizeSelection(clipBSU, kXMPCloneCopy);
			fClipboardChangeID = clipboard->GetChangeID();
		
			// Add a link spec to the clipboard if we don't already have a source link
			// (or the source link is selected exactly),
			// and as long as the selection does not include a link destination.
			if (fSrcLink.linkSource == (XMPLinkSource*) kXMPNULL) {
				if (fDstLink.link == (XMPLink*) kXMPNULL) {
					makeLinkSpec = kXMPTrue;
					}
				else {
					XMPSShort selStart = (**(fDocument->theTE)).selStart;
					XMPSShort selEnd = (**(fDocument->theTE)).selEnd;
					makeLinkSpec = !RangesOverlap(selStart,selEnd,fDstLink.start,fDstLink.end);
					}
				}
			else {
				XMPSShort selStart = (**(fDocument->theTE)).selStart;
				XMPSShort selEnd = (**(fDocument->theTE)).selEnd;
				makeLinkSpec = (selStart==fSrcLink.start) && (selEnd==fSrcLink.end);
				}
			if (makeLinkSpec) {
				XMPLinkSpec* ls = fDraft->CreateLinkSpec(this, kXMPNULL, 0);
				clipBSU->AddProperty(kXMPPropLinkSpec);
				ls->WriteLinkSpec(clipBSU);
				delete ls;
				
				// Save away the range for the potential link
				if (fSrcLink.linkSource == (XMPLinkSource*) kXMPNULL) {
					fSrcLink.start = (**(fDocument->theTE)).selStart;
					fSrcLink.end = (**(fDocument->theTE)).selEnd;
					}
				}
		CATCH_ALL
			WASSERTM(kXMPFalse, "Copy to clipboard failed");
		ENDTRY
		clipboard->Unlock(key);
	}
	return kXMPTrue;
}

void DragPart::IncorporateTestDragContent(XMPStorageUnit* su)
{
	XMPSShort			delStart = (**(fDocument->theTE)).selStart;
	XMPSShort			delEnd = (**(fDocument->theTE)).selEnd;
	XMPSShort			newEnd;
	XMPValue			text = kXMPNULL;
	XMPULong			size = 0;
	XMPULong			stsize = 0;
	StScrpHandle		style = kXMPNULL;
	XMPStorageUnit*		newSU;
	XMPStorageUnit*		testDragSU;
	XMPStorageUnitRef	aSURef;
	XMPDraft*			fromDraft = su->GetDraft();

	XMPDraftKey draftKey = fromDraft->BeginClone(kXMPClonePaste);
	TRY
		newSU = su->CloneTo(draftKey, fDraft, kXMPNULL);
	CATCH_ALL
		fromDraft->AbortClone(draftKey);
		RERAISE;
	ENDTRY
	fromDraft->EndClone(draftKey);

	newSU->Focus(kXMPPropContents, kXMPPosUndefined, kXMPKindTestDrag, (XMPValueIndex)0, kXMPPosUndefined);
	newSU->GetValue(sizeof(XMPStorageUnitRef),&aSURef);
	testDragSU = fDraft->GetStorageUnit(newSU->GetIDFromStorageUnitRef(aSURef));

	testDragSU->Focus(kXMPPropText, kXMPPosUndefined, 0, 1, kXMPPosFirstSib);
	size = testDragSU->GetSize();
	text = (XMPValue)XMPNewPtr(size);
	
	testDragSU->SetOffset(0);
	testDragSU->GetValue(size,text);
	
	testDragSU->Focus(kXMPPropStyl, kXMPPosUndefined, 
				   kXMPApplestyl, (XMPValueIndex)0, kXMPPosUndefined);
	
	stsize = testDragSU->GetSize();
	style = (StScrpHandle)XMPNewHandle(stsize);
	
	XMPLockHandle((Handle)style);
	testDragSU->SetOffset(0);
	testDragSU->GetValue(stsize,(XMPValue)(*style));
	XMPUnlockHandle((Handle)style);

	TEDelete(fDocument->theTE);
	TEStylInsert((void*)text, size, style, fDocument->theTE);
	
	newEnd = (**(fDocument->theTE)).selEnd;
	this->AdjustLinks(delStart, delEnd, newEnd, fSession->UniqueChangeID());

	if ( fSrcLink.linkSource == (XMPLinkSource*) kXMPNULL )
	{
		if ( testDragSU->Exists(kXMPPropSourceLink, kXMPValue, kXMPPosAll) )
		{
			testDragSU->Focus(kXMPPropSourceLink, kXMPPosUndefined, kXMPValue, (XMPValueIndex)0, kXMPPosUndefined);
			if ( testDragSU->GetSize() > 0 ) 
			{
				// A try block here is essential because the clone operation will not copy
				// link source objects that have no destinations in the clone.
				// This will cause XMPDraft::GetLinkSource to throw an exception.
				// This is NOT an error; this part should forget about the link.
				TRY
					testDragSU->GetValue(sizeof(XMPStorageUnitRef),&aSURef);

					// XMPStorageUnitID linkSourceID = testDragSU->GetIDFromStorageUnitRef(aSURef);
					// XMPStorageUnit* linkSourceSU = fDraft->GetStorageUnit(linkSourceID);
					// linkSourceSU->Focus(kXMPPropSourcePart, kXMPPosUndefined, kXMPWeakStorageUnitRef, (XMPValueIndex)0, kXMPPosUndefined);
					// aSURef = linkSourceSU->GetWeakStorageUnitRef(this->GetStorageUnit());
					// linkSourceSU->SetValue(sizeof(XMPStorageUnitRef), (XMPValue) &aSURef);
					// linkSourceSU->Release();

					fSrcLink.linkSource = fDraft->GetLinkSource(testDragSU->GetIDFromStorageUnitRef(aSURef));
					testDragSU->GetValue(sizeof(XMPSShort), (XMPValue) &fSrcLink.start);
					testDragSU->GetValue(sizeof(XMPSShort), (XMPValue) &fSrcLink.end);
					fSrcLink.start += delStart;
					fSrcLink.end += delStart;

					// Change the link source to reference this part's storage unit
					fSrcLink.linkSource->SetSourcePart(this->GetStorageUnit());

					// Now notify destinations; necessary if this source link was cut
					//  but some destinations were not
					this->LinkChanged(&fSrcLink, fSession->UniqueChangeID());
				CATCH_ALL
					fSrcLink.linkSource = (XMPLinkSource*) kXMPNULL;
					fSrcLink.start = -1;
					fSrcLink.end = -1;
				ENDTRY
			}
		}
	}

	if ( fDstLink.link == (XMPLink*) kXMPNULL )
	{
		if ( testDragSU->Exists(kXMPPropDestLink, kXMPValue, kXMPPosAll) )
		{
			testDragSU->Focus(kXMPPropDestLink, kXMPPosUndefined, kXMPValue, (XMPValueIndex)0, kXMPPosUndefined);
			if ( testDragSU->GetSize() > 0 ) 
			{
				TRY
					testDragSU->GetValue(sizeof(XMPStorageUnitRef),&aSURef);
					fDstLink.link = fDraft->GetLink(testDragSU->GetIDFromStorageUnitRef(aSURef), kXMPNULL);
					testDragSU->GetValue(sizeof(XMPSShort), (XMPValue) &fDstLink.start);
					testDragSU->GetValue(sizeof(XMPSShort), (XMPValue) &fDstLink.end);
					fDstLink.start += delStart;
					fDstLink.end += delStart;

					XMPDisposePtr(fDstLink.info.kind);
					testDragSU->GetValue(sizeof(XMPLinkInfo), (XMPValue) &fDstLink.info);
					XMPType kind = (XMPType) XMPNewPtr((XMPULong) fDstLink.info.kind);
					testDragSU->GetValue((XMPULong) fDstLink.info.kind, (XMPValue) kind);
					fDstLink.info.kind = kind;

					// Now register as a dependent of the link
					fDstLink.link->RegisterDependent(this, fDstLink.info.change);
				CATCH_ALL
					fDstLink.link = (XMPLink*) kXMPNULL;
					fDstLink.start = -1;
					fDstLink.end = -1;
				ENDTRY
			}
		}
	}

	testDragSU->Release();
	//	fDraft->RemoveStorageUnit(newSU);	$$$$$ Why doesn't this work?
		newSU->Release();

	fDraft->SetChangedFromPrev();
}

void DragPart::IncorporateStyledTextContent(XMPStorageUnit* su)
{
	XMPSShort		delStart = (**(fDocument->theTE)).selStart;
	XMPSShort		delEnd = (**(fDocument->theTE)).selEnd;
	XMPSShort		newEnd;
	XMPValue		text = kXMPNULL;
	XMPULong		size = 0;
	XMPULong		stsize = 0;
	StScrpHandle	style = kXMPNULL;
	
	su->Focus(kXMPPropContents, kXMPPosUndefined, 
				   kXMPAppleTEXT, (XMPValueIndex)0, kXMPPosUndefined);
	
	size = su->GetSize();
	text = (XMPValue)XMPNewPtr(size);
	
	su->SetOffset(0);
	su->GetValue(size,text);
	
	su->Focus(kXMPPropContents, kXMPPosUndefined, 
				   kXMPApplestyl, (XMPValueIndex)0, kXMPPosUndefined);
	
	stsize = su->GetSize();
	style = (StScrpHandle)XMPNewHandle(stsize);
	
	XMPLockHandle((Handle)style);
	su->SetOffset(0);
	su->GetValue(stsize,(XMPValue)(*style));
	XMPUnlockHandle((Handle)style);

	TEDelete(fDocument->theTE);
	TEStylInsert((void*)text, size, style, fDocument->theTE);
		
	newEnd = (**(fDocument->theTE)).selEnd;
	this->AdjustLinks(delStart, delEnd, newEnd, fSession->UniqueChangeID());
	fDraft->SetChangedFromPrev();
}

void DragPart::IncorporateTextContent(XMPStorageUnit* su)
{
	XMPSShort		delStart = (**(fDocument->theTE)).selStart;
	XMPSShort		delEnd = (**(fDocument->theTE)).selEnd;
	XMPSShort		newEnd;
	XMPValue		text = kXMPNULL;
	XMPULong		size = 0;
	
	su->Focus(kXMPPropContents, kXMPPosUndefined, kXMPAppleTEXT, (XMPValueIndex)0, kXMPPosUndefined);
	
	size = su->GetSize();
	text = (XMPValue)XMPNewPtr(size);
	
	su->SetOffset(0);
	su->GetValue(size,text);
	
	TEDelete(fDocument->theTE);
	TEInsert((void*)text, size, fDocument->theTE);
		
	newEnd = (**(fDocument->theTE)).selEnd;
	this->AdjustLinks(delStart, delEnd, newEnd, fSession->UniqueChangeID());
	fDraft->SetChangedFromPrev();
}

XMPStatic XMPBoolean IsKind(XMPType type, XMPISOStr kind)
{
	return !XMPISOStrCompare( (XMPISOStr) type, kind );
}

XMPStatic XMPBoolean PasteThisKind(XMPStorageUnit* su, XMPType aKind, XMPType specifiedKind)
// Returns true if aKind exists in the contents property of the argument storage unit
// and specifiedKind is either nil (don't care) or the same as aKind.
{
	return ( su->Exists(kXMPPropContents, aKind, kXMPPosAll)
		&& ((specifiedKind == (XMPType) kXMPNULL) || (IsKind(aKind, specifiedKind))) );
}

XMPBoolean DragPart::DoPaste(XMPFrame* frame, XMPType desiredKind)
{
	XMPUnused(frame);
	
	XMPClipboardKey key;
	XMPClipboard* clipboard = fSession->GetClipboard();
	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* clipBSU = clipboard->GetContentStorageUnit(key);

			if ( PasteThisKind (clipBSU, kXMPKindTestDrag, desiredKind) )
			{
				IncorporateTestDragContent(clipBSU);
			}
			else if ( PasteThisKind (clipBSU, kXMPAppleTEXT, desiredKind) )
			{
				IncorporateTextContent(clipBSU);
			}
			else if ( PasteThisKind (clipBSU, kXMPApplestyl, desiredKind) )
			{
				IncorporateStyledTextContent(clipBSU);
			}
		CATCH_ALL
		ENDTRY
		clipboard->Unlock(key);
	}
	return kXMPTrue;
}

void DragPart::UndoRedoPaste(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

XMPBoolean DragPart::DoPasteAs(XMPFrame* frame)
{
	XMPPasteAsResult	pasteAsRslt;
	XMPBoolean			result = kXMPTrue;

	XMPClipboardKey key;
	XMPClipboard* clipboard = fSession->GetClipboard();
	if ( clipboard->Lock(0, &key) )
	{
		TRY
			XMPStorageUnit* clipContentSU = clipboard->GetContentStorageUnit(key);
		
			if ( ShowPasteAsDialog(
						this->CanPasteLink(),
						kXMPFalse,
						clipContentSU,
						frame,
						fSession->Tokenize(kXMPViewAsFrame),
						&pasteAsRslt) )
			{
				if ( pasteAsRslt.fPasteLinkSetting == kXMPTrue )
					result = DoPasteLink(frame, pasteAsRslt.fSelectedKind);
				else
					result = DoPaste(frame, pasteAsRslt.fSelectedKind);
			}
		CATCH_ALL
			
		ENDTRY
		clipboard->Unlock(key);
	}

	return result;
}

XMPBoolean DragPart::CanPasteLink()
{
	XMPBoolean result = (fDstLink.link == (XMPLink*) kXMPNULL) && LinkSpecOnClipboard(fSession);
	if (result) {
		// Do not allow pasting a destination link into a source link (even a potential one!)
		if (fSrcLink.end > fSrcLink.start) {
			XMPSShort selStart = (**(fDocument->theTE)).selStart;
			XMPSShort selEnd = (**(fDocument->theTE)).selEnd;
			result = !RangesOverlap(selStart,selEnd,fSrcLink.start,fSrcLink.end);
			}
		}
	return result;
}

XMPBoolean DragPart::DoPasteLink(XMPFrame* frame, XMPType desiredKind)
{
	XMPUnused(frame);
	XMPUnused(desiredKind);
	
	XMPClipboardKey key;
	XMPClipboard* clipboard = fSession->GetClipboard();
	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* clipBSU = clipboard->GetContentStorageUnit(key);;
		
			if (clipBSU->Exists(kXMPPropLinkSpec, (XMPValueType) kXMPNULL, 0) == kXMPTrue) {
				XMPLinkSpec* ls = new XMPLinkSpec;
				clipBSU->Focus(kXMPPropLinkSpec, kXMPPosUndefined, (XMPValueType) kXMPNULL, 0, kXMPPosUndefined);
				ls->ReadLinkSpec(clipBSU);
				TRY
					fDstLink.link = fDraft->GetLink((XMPStorageUnitID) kXMPNULL, ls);
					if (fDstLink.link != (XMPLink*) kXMPNULL) {
						XMPSShort delStart = (**(fDocument->theTE)).selStart;
						XMPSShort delEnd = (**(fDocument->theTE)).selEnd;

						// Delete the existing text and wait for LinkUpdated() to be
						// called to get the text
						TEDelete(fDocument->theTE);
						
						fDstLink.info.change = fDstLink.link->GetChangeID();
						GetDateTime(&fDstLink.info.creationTime);
						fDstLink.info.changeTime = fDstLink.info.creationTime;
						fDstLink.info.kind = (XMPType) XMPISOStrFromCStr(desiredKind);
						fDstLink.info.autoUpdate = kXMPTrue;

						// Adjust other links due to this change
						// Special cased for source links only for now
						// Assume no overlapping links or recursion problems
						AdjustSrcLinks(delStart, delEnd, delStart, fDstLink.info.change);
						fDstLink.start = delStart;
						fDstLink.end = delStart;
						// Now register as a dependent of the link
						// Specify kXMPUnknownChange to force getting the text.
						fDstLink.link->RegisterDependent(this, kXMPUnknownChange);
						// Put the insertion point after the text
						TESetSelect(fDstLink.end, fDstLink.end, fDocument->theTE); 

						fDraft->SetChangedFromPrev();
						}
				CATCH(kXMPErrDocNotSaved)
					DebugStr("\pSource document not saved!");
				CATCH_ALL
				ENDTRY
				delete ls;
				}
		CATCH_ALL
		ENDTRY
		clipboard->Unlock(key);
	}
	return kXMPTrue;
}

void DragPart::UndoRedoPasteLink(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

XMPBoolean DragPart::SelectionInLinkSource()
{
	XMPBoolean inSourceLink = kXMPFalse;
	XMPSShort selStart = (**(fDocument->theTE)).selStart;
	XMPSShort selEnd = (**(fDocument->theTE)).selEnd;
	
	if (fSrcLink.linkSource == ((XMPLinkSource*) kXMPNULL))
		inSourceLink = kXMPFalse;
	if ((selStart >= fSrcLink.start) && (selStart < fSrcLink.end))
		inSourceLink = kXMPTrue;
	else if ((selEnd > fSrcLink.start) && (selEnd <= fSrcLink.end))
		inSourceLink = kXMPTrue;
	else if ((selStart <= fSrcLink.start) && (selEnd >= fSrcLink.end))
		inSourceLink = kXMPTrue;
	else
		inSourceLink = kXMPFalse;

	return inSourceLink;
}

XMPBoolean DragPart::SelectionInLinkDestination()
{
	XMPBoolean inDestinationLink = kXMPFalse;
	XMPSShort selStart = (**(fDocument->theTE)).selStart;
	XMPSShort selEnd = (**(fDocument->theTE)).selEnd;
	
	if ((selStart >= fDstLink.start) && (selStart < fDstLink.end))
		inDestinationLink = kXMPTrue;
	else if ((selEnd > fDstLink.start) && (selEnd <= fDstLink.end))
		inDestinationLink = kXMPTrue;
	else if ((selStart <= fDstLink.start) && (selEnd >= fDstLink.end))
		inDestinationLink = kXMPTrue;

	return inDestinationLink;
}

XMPBoolean DragPart::DoLinkInfo(XMPFrame* frame)
{
	XMPUnused(frame);

	XMPBoolean			result = kXMPTrue;
	XMPLinkInfoResult	infoResult;

	if (this->SelectionInLinkSource())
	{
		if ( ShowLinkSourceInfo(fSrcLink.linkSource, fSrcLink.linkSource->GetChangeID(), &infoResult) )
		{
			switch (infoResult.action) {
			case kXMPLinkInfoBreakLink:
				fSrcLink.linkSource->SetSourcePart((XMPStorageUnit*) kXMPNULL);
				fSrcLink.linkSource->Release();
				fSrcLink.linkSource = (XMPLinkSource*) kXMPNULL;
				fSrcLink.start = -1;
				fSrcLink.end = -1;
				fDraft->SetChangedFromPrev();
				break;
			default:
				break;
			}
		}
	}
	else if (this->SelectionInLinkDestination())
	{
		fDstLink.info.size = fDstLink.end - fDstLink.start;

		if ( ShowLinkDestinationInfo(fDstLink.link, &fDstLink.info, &infoResult) )
		{
			switch (infoResult.action) {
			case kXMPLinkInfoFindSource:
				fDstLink.link->ShowSourceContent();
				break;
			case kXMPLinkInfoBreakLink:
				this->RemoveDstLink();
				break;
			default:
				break;
			}
		}
	}

	return kXMPTrue;	// Always handle the event
}

#define kDragPartErrInvalidLink				7787

void DragPart::RevealLink(XMPLinkSource* linkSource)
{
	if ( fSrcLink.linkSource != linkSource )
		THROW_IF_ERROR(kDragPartErrInvalidLink);

	XMPFrame* frame = (XMPFrame*) fDisplayFrames->First();
	
	if ( frame != (XMPFrame*) kXMPNULL )
	{
		XMPFrame* containingFrame = frame->GetContainingFrame();

		if ( containingFrame != (XMPFrame*) kXMPNULL )
		{
			// Sample parts currently THROW so catch and ignore
			TRY
				containingFrame->GetPart()->RevealFrame(frame, (XMPMacShape*) kXMPNULL);
			CATCH_ALL
			ENDTRY
		}

		ActivateFrame(frame);
		if (fIsActive)
		{
			TESetSelect(fSrcLink.start, fSrcLink.end, fDocument->theTE); 
		}
	}
	else
	{
		// Otherwise open a new window
	}
}

XMPBoolean DragPart::DoClear(XMPFrame* frame)
{
	XMPUnused(frame);
	
	XMPSShort delStart = (**(fDocument->theTE)).selStart;
	XMPSShort delEnd = (**(fDocument->theTE)).selEnd;

	TEDelete(fDocument->theTE);

	this->AdjustLinks(delStart, delEnd, delStart, fSession->UniqueChangeID());

	if (delEnd > delStart)
		fDraft->SetChangedFromPrev();

	return kXMPTrue;
}

void DragPart::UndoRedoClear(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

XMPBoolean DragPart::DoSelectAll(XMPFrame* frame)
{
	XMPUnused(frame);
	
	TESetSelect(0,32767,fDocument->theTE);
	
	return kXMPTrue;
}

XMPBoolean DragPart::DoSampleOne(XMPFrame* frame)
{
	char* defOne = "As I sailed into Shadow, a white bird of my desire came and sat upon my right shoulder, and I wrote a note and tied it to its leg and sent in on its way.\n";
	TESetText(defOne, strlen(defOne), fDocument->theTE);
	RemoveSrcLink();
	RemoveDstLink();
	XMPFrameFacetIterator* facet = frame->CreateFacetIterator();
	facet->First()->Invalidate(kXMPNULL);
	delete facet;
	fFrameShapeChanged = true;
	fDraft->SetChangedFromPrev();
	return kXMPTrue;
}

void DragPart::UndoRedoSampleOne(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

XMPBoolean DragPart::DoSampleTwo(XMPFrame* frame)
{
	char* defTwo = "A black bird of my desire came and sat on my left shoulder, and I wrote a note and tied it to its leg and sent it off into the west.\n";
	TESetText(defTwo, strlen(defTwo), fDocument->theTE);
	RemoveSrcLink();
	RemoveDstLink();
	XMPFrameFacetIterator* facet = frame->CreateFacetIterator();
	facet->First()->Invalidate(kXMPNULL);
	delete facet;
	fFrameShapeChanged = true;
	fDraft->SetChangedFromPrev();
	return kXMPTrue;
}

void DragPart::UndoRedoSampleTwo(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

void DragPart::UndoRedoTyping(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

void DragPart::UndoRedoStyling(UndoInfo* action, XMPUShort selector)
{
	 XMPUnused(action);
	 XMPUnused(selector);
}

XMPBoolean DragPart::DoCommand(XMPFrame* frame, XMPCommandID command)
{
	XMPBoolean handled = kXMPFalse;
	short menu = 0;
	short item = 0;

	if (!fMenuBar->IsCommandSynthetic(command))
	{
		switch (command) {
			case kXMPCommandCut: 		handled = DoCut(frame); break;
			case kXMPCommandCopy:		handled = DoCopy(frame); break;
			case kXMPCommandPaste:		handled = DoPaste(frame, kXMPNULL); break;
			case kXMPCommandPasteAs:	handled = DoPasteAs(frame); break;
			case kXMPCommandClear:		handled = DoClear(frame); break;
			case kXMPCommandSelectAll:	handled = DoSelectAll(frame); break;
			case kXMPCommandGetPartInfo:handled = DoLinkInfo(frame); break;
			case cDTSampleOne:			handled = DoSampleOne(frame); break;
			case cDTSampleTwo: 			handled = DoSampleTwo(frame); break;
			default:
				break;
		}
	}
	else
	{
		fMenuBar->GetMenuAndItem(command, menu, item);	
		switch (menu)
		{
			case 141:
				this->HandleFontMenu(item);
				handled = kXMPTrue;
				break;
			case 142:
				this->HandleSizeMenu(item);
				handled = kXMPTrue;
				break;
			default:
				break;
		}
	}

	HiliteMenu(0);	
	return (handled);
}

XMPBoolean DragPart::HandleMenuEvent(XMPFrame* frame, XMPEventData event)
{
	long menuResult = event->message;
	short menu = HiWord(menuResult);
	short item = LoWord(menuResult);
	XMPCommandID command = fMenuBar->GetCommand(menu, item);
	XMPBoolean handled = DoCommand(frame,command);
	return (handled);
}

//-------------------------------------------------------------------------
// From Undo Protocol 
//-------------------------------------------------------------------------

void DragPart::PushUndoAction(XMPFrame* frame, XMPCommandID command, XMPULong start,
							  XMPULong end, XMPPtr text, XMPHandle style,
							  char* undoText, char* redoText)
{
	// create action object for Undo stack
	UndoInfo* uaction = (UndoInfo*)XMPNewPtr(sizeof(UndoInfo));
	uaction->displayFrame = frame;
	uaction->command = command;
	uaction->start = start;
	uaction->end = end;
	uaction->text = text;
	uaction->style = style;
	
	// THERE ARE HELPER FUNCTIONS FOR THIS NOW IN PASCLSTR.H
	// create Undo and Redo menu item text
	XMPName* undoMenuText = (XMPName*)XMPNewPtr(sizeof(XMPIText)+strlen(undoText));
	undoMenuText->theScriptCode = smRoman;
	undoMenuText->theLangCode = langEnglish;
	undoMenuText->textLength = strlen(undoText);
	memcpy(undoMenuText->theText, undoText, strlen(undoText));
	
	// THERE ARE HELPER FUNCTIONS FOR THIS NOW IN PASCLSTR.H
	// create Redo menu item text
	XMPName* redoMenuText = (XMPName*)XMPNewPtr(sizeof(XMPIText)+strlen(redoText));
	redoMenuText->theScriptCode = smRoman;
	redoMenuText->theLangCode = langEnglish;
	redoMenuText->textLength = strlen(redoText);
	memcpy(redoMenuText->theText, redoText, strlen(redoText));
	
	fSession->GetUndo()->AddActionToHistory(this, (XMPActionData)uaction,
											kXMPSingleAction, undoMenuText,
											redoMenuText);
	fUndoCount++;
	
	frame->IncrementRefCount();
}
								
void DragPart::UndoAction(XMPActionData action)
{
	WASSERTM((fUndoCount != 0), "Undo unexpected, internal count is zero");
	
	XMPBoolean ok = kXMPFalse;
	UndoInfo* uaction = (UndoInfo*)action;
	
	OrderedCollectionIterator dispIter(fDisplayFrames);
	for (XMPFrame* frame = (XMPFrame*)dispIter.First();
		 dispIter.IsNotComplete();
		 frame = (XMPFrame*)dispIter.Next())
	{
		if (frame == (*uaction).displayFrame) ok = kXMPTrue;
	}
	
	if (ok)
	{
		switch ((*uaction).command)
		{
			case kXMPCommandCut: 	UndoRedoCut(uaction, kUndoIt); break;
			case kXMPCommandPaste:	UndoRedoPaste(uaction, kUndoIt); break;
			case kXMPCommandPasteAs:UndoRedoPasteLink(uaction, kUndoIt); break;
			case kXMPCommandClear:	UndoRedoClear(uaction, kUndoIt); break;
			case cDTSampleOne:		UndoRedoSampleOne(uaction, kUndoIt); break;
			case cDTSampleTwo: 		UndoRedoSampleTwo(uaction, kUndoIt); break;
			case cDTTyping: 		UndoRedoTyping(uaction, kUndoIt); break;
			case cDTStyling: 		UndoRedoStyling(uaction, kUndoIt); break;
			default:
				break;
		}
		
		fUndoCount--;
		fRedoCount++;
	}
}

void DragPart::RedoAction(XMPActionData action)
{
	WASSERTM((fRedoCount != 0), "Redo unexpected, internal count is zero");
	
	XMPBoolean ok = kXMPFalse;
	UndoInfo* uaction = (UndoInfo*)action;
	
	OrderedCollectionIterator dispIter(fDisplayFrames);
	for (XMPFrame* frame = (XMPFrame*)dispIter.First();
		 dispIter.IsNotComplete();
		 frame = (XMPFrame*)dispIter.Next())
	{
		if (frame == (*uaction).displayFrame) ok = kXMPTrue;
	}
	
	if (ok)
	{
		switch ((*uaction).command)
		{
			case kXMPCommandCut: 	UndoRedoCut(uaction, kRedoIt); break;
			case kXMPCommandPaste:	UndoRedoPaste(uaction, kRedoIt); break;
			case kXMPCommandPasteAs:UndoRedoPasteLink(uaction, kRedoIt); break;
			case kXMPCommandClear:	UndoRedoClear(uaction, kRedoIt); break;
			case cDTSampleOne:		UndoRedoSampleOne(uaction, kRedoIt); break;
			case cDTSampleTwo: 		UndoRedoSampleTwo(uaction, kRedoIt); break;
			case cDTTyping: 		UndoRedoTyping(uaction, kRedoIt); break;
			case cDTStyling: 		UndoRedoStyling(uaction, kRedoIt); break;
			default:
				break;
		}
		
		fUndoCount++;
		fRedoCount--;
	}
}

void DragPart::DisposeActionState(XMPActionData action, XMPDoneState doneState)
{
	// CC: The tests here should be logically inverted; but its not necessary
	//     that both of these counts be non zero.  So remove for now.
	// WASSERTM((fUndoCount == 0), "Undo unexpected, internal count is zero");
	// WASSERTM((fRedoCount == 0), "Redo unexpected, internal count is zero");

	UndoInfo* uaction = (UndoInfo*)action;
	
	(*uaction).displayFrame->Release();
//	if ((*uaction).text)
//		XMPDisposePtr((Ptr)(*uaction).text);
	if ((*uaction).style)
		XMPDisposeHandle((Handle)(*uaction).style);
	
	XMPDisposePtr((XMPPtr)uaction);
	
	switch (doneState) {
		case kXMPDone:
		case kXMPRedone:
			fUndoCount--; break;
		case kXMPUndone:
			fRedoCount--; break;
		default:
			;
	}
}

void DragPart::WriteActionState(XMPPtr action, XMPStorageUnitView* storageUnitView)
{
	UndoInfo* uaction = (UndoInfo*)action;
	
	XMPStorageUnitRef suRef;
	suRef = storageUnitView->GetWeakStorageUnitRef((*uaction).displayFrame->GetStorageUnit());
	
	storageUnitView->AddProperty(kXMPUndoInfo);
	storageUnitView->AddValue(kXMPFrameRef);
	storageUnitView->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&suRef);
	
	storageUnitView->AddValue(kXMPCommandID);
	storageUnitView->SetValue(sizeof(XMPCommandID), (XMPValue)&(*uaction).command);
	
	storageUnitView->AddValue(kXMPStart);
	storageUnitView->SetValue(sizeof(XMPULong), (XMPValue)&(*uaction).start);
	
	storageUnitView->AddValue(kXMPEnd);
	storageUnitView->SetValue(sizeof(XMPULong), (XMPValue)&(*uaction).end);
	
	storageUnitView->AddValue(kXMPAppleTEXT);
	storageUnitView->SetValue(GetPtrSize((Ptr)(*uaction).text), (XMPValue)(*uaction).text);
	
	XMPLockHandle((Handle)(*uaction).style);
	storageUnitView->AddValue(kXMPApplestyl);
	storageUnitView->SetValue(XMPGetHandleSize((Handle)(*uaction).style), (XMPValue)(*(*uaction).style));
	XMPUnlockHandle((Handle)(*uaction).style);
}

XMPPtr DragPart::ReadActionState(XMPStorageUnitView* storageUnitView)
{
	XMPStorageUnitRef suRef;
	XMPStorageUnit* su = storageUnitView->GetStorageUnit();
	XMPCommandID command;
	XMPULong start, end;
	XMPPtr text;
	XMPULong size;
	XMPHandle style;
	
	su->Focus(kXMPUndoInfo, kXMPPosSame, kXMPFrameRef, 1, kXMPPosUndefined);
	su->SetOffset(0);
	su->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&suRef);
	
	su->Focus(kXMPUndoInfo, kXMPPosSame, kXMPCommandID, 1, kXMPPosUndefined);
	su->SetOffset(0);
	su->GetValue(sizeof(XMPCommandID), (XMPValue)&command);
	
	su->Focus(kXMPUndoInfo, kXMPPosSame, kXMPStart, 1, kXMPPosUndefined);
	su->SetOffset(0);
	su->GetValue(sizeof(XMPULong), (XMPValue)&start);
	
	su->Focus(kXMPUndoInfo, kXMPPosSame, kXMPEnd, 1, kXMPPosUndefined);
	su->SetOffset(0);
	su->GetValue(sizeof(XMPULong), (XMPValue)&end);
	
	su->Focus(kXMPUndoInfo, kXMPPosSame, kXMPAppleTEXT, 1, kXMPPosUndefined);
	su->SetOffset(0);
	size = su->GetSize();
	text = XMPNewPtr(size);
	THROW_IF_NULL(text);
	su->GetValue(size, (XMPValue)text);

	su->Focus(kXMPUndoInfo, kXMPPosSame, kXMPApplestyl, 1, kXMPPosUndefined);
	su->SetOffset(0);
	size = su->GetSize();
	style = XMPNewHandle(size);
	THROW_IF_NULL(style);
	XMPLockHandle((Handle)style);
	su->GetValue(size, (XMPValue)(*style));
	XMPUnlockHandle((Handle)style);

	UndoInfo* uaction = (UndoInfo*) XMPNewPtr(sizeof(UndoInfo));
	THROW_IF_NULL(uaction);
	
	(*uaction).displayFrame = su->GetDraft()->GetFrame(su->GetIDFromStorageUnitRef(suRef));
	(*uaction).command = command;
	(*uaction).start = start;
	(*uaction).end = end;
	(*uaction).text = text;
	
	return ((XMPPtr)uaction);
}

//-------------------------------------------------------------------------
// Utility Methods 
//-------------------------------------------------------------------------

void DragPart::CommitRelinquishFocus(XMPTypeToken focus, 
									XMPFrame* currentFrame,
									XMPFrame* proposedFrame)
{
XMPUnused(proposedFrame);

	if (focus == fSession->Tokenize(kXMPMenuFocus)) 
	{
		this->RemoveMenus(currentFrame);
		fIsActive = false;

	}
}

void DragPart::InstallMenus(XMPFrame* frame)
{
	fMenuBar->Display();

	fSession->GetDispatcher()->RegisterIdle(this, frame, 0);
	XMPFrameFacetIterator* fiter = frame->CreateFacetIterator();
	this->ClipThis(fiter->First());
	TEActivate(fDocument->theTE);
	HiliteControl(fDocument->vScroll, 0);
	HiliteControl(fDocument->hScroll, 0);
	TEGetHiliteRgn(fDocument->hiliteRgn, fDocument->theTE);
	fDocument->frameIsActive = true;
	this->ClipToRoot(fiter->First());
	delete fiter;
}

static XMPBoolean LinkSpecOnClipboard(XMPSession* session)
{
	XMPBoolean		result = kXMPFalse;
	XMPClipboard*	clipboard = session->GetClipboard();
	XMPClipboardKey key;

	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* su = clipboard->GetContentStorageUnit(key);
			result = su->Exists(kXMPPropLinkSpec, (XMPValueType) kXMPLinkSpec, 0);
		CATCH_ALL
		ENDTRY
		clipboard->Unlock(key);
	}
	return result;
}

static XMPBoolean ValueOnClipboard(XMPValueType type, XMPSession* session)
{
	XMPBoolean		result = kXMPFalse;
	XMPClipboard*	clipboard = session->GetClipboard();
	XMPClipboardKey key;

	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* su = clipboard->GetContentStorageUnit(key);
			result =  su->Exists(kXMPPropContents, type, 0);
		CATCH_ALL
		ENDTRY
		clipboard->Unlock(key);
	}
	return result;
}

XMPBoolean DragPart::UserCategoryString(XMPName** name)
{
	XMPULong	valueLen;
	XMPBoolean	result  = kXMPFalse ;

	// look it up in the spaceName namespace
	XMPNameSpace* userStringNameSpace = 
		fSession->GetNameSpaceManager()->HasNameSpace(kXMPCategoryUserString);

	if (userStringNameSpace)
		result = userStringNameSpace->GetValue(kXMPCategoryText, (XMPPtr*) name, &valueLen);
	return result ;
}

//XMPStatic StringPtr IntlToPStr(XMPIText* intlText, StringPtr pstr)
//{
//	memcpy(&pstr[1], intlText->theText, (size_t)intlText->textLength);
//	pstr[0] = (unsigned char)intlText->textLength;
//	return pstr;
//}

void DragPart::SyncMenuBar()
{
	MenuHandle	editmenu = GetMHandle(130);
	TEHandle	myTE = fDocument->theTE;
	XMPBoolean	isSelection;
	XMPBoolean	canPaste;
	XMPBoolean	canSelectAll;
	XMPBoolean	canShowLinkInfo;

	isSelection = (*myTE)->selEnd > (*myTE)->selStart;
	fMenuBar->EnableCommand(kXMPCommandCut, isSelection);
	fMenuBar->EnableCommand(kXMPCommandCopy, isSelection);
	fMenuBar->EnableCommand(kXMPCommandClear, isSelection);
	
	canPaste = ValueOnClipboard(kXMPKindTestDrag, fSession)
				|| ValueOnClipboard(kXMPAppleTEXT, fSession);
	fMenuBar->EnableCommand(kXMPCommandPaste, canPaste);
	fMenuBar->EnableCommand(kXMPCommandPasteAs, canPaste);

	canSelectAll = ((*myTE)->teLength > 0) && (((*myTE)->selStart > 0) || ((*myTE)->selEnd < (*myTE)->teLength));
	fMenuBar->EnableCommand(kXMPCommandSelectAll, canSelectAll);

	canShowLinkInfo = SelectionInLinkSource() || SelectionInLinkDestination();
	if (canShowLinkInfo)
		fMenuBar->SetItemString(kXMPCommandGetPartInfo, "\pGet Link Info");
	else
		fMenuBar->SetItemString(kXMPCommandGetPartInfo, "\pGet Part Info");
	fMenuBar->EnableCommand(kXMPCommandGetPartInfo, canShowLinkInfo);
}

void DragPart::RemoveMenus(XMPFrame* frame)
{
	DeleteMenu(140);
	DeleteMenu(141);
	DeleteMenu(142);
	
	InvalMenuBar();
	fSession->GetDispatcher()->UnregisterIdle(this, frame);
	XMPFrameFacetIterator* fiter = frame->CreateFacetIterator();
	this->ClipThis(fiter->First());
	TEDeactivate(fDocument->theTE);
	HiliteControl(fDocument->vScroll, 255);
	HiliteControl(fDocument->hScroll, 255);
	fDocument->frameIsActive = false;
	this->ClipToRoot(fiter->First());
	delete fiter;
}

XMPPoint DragPart::GetAggregateTranslation(XMPFacet* facet)
{
	XMPPoint translation(0,0);
	XMPTransform* total = facet->GetWindowContentTransform();
	translation = total->TransformPoint(translation);
	return translation;
}

void DragPart::ExternalShapeToWindowShape(XMPFacet* facet, XMPShape* inShape, XMPShape* outShape)
{
	XMPTransform* localToGlobal = new XMPTransform();
	localToGlobal->CopyFrom(facet->GetWindowFrameTransform());		
	// localToGlobal->Invert();
	outShape->CopyFrom(inShape);
	outShape->Transform(localToGlobal);
	delete localToGlobal;
}

XMPPoint DragPart::InternalPointToWindowPoint(XMPFacet* facet)
{
	XMPTransform* globalToLocal = facet->GetWindowContentTransform();
	XMPPoint offset(0,0);
	offset = globalToLocal->TransformPoint(offset);
	return offset;
}

void DragPart::ActivatingWindow(XMPFrame* frame)
{
	if (fDocument->frameIsActive)
	{
		XMPFrameFacetIterator* fiter = frame->CreateFacetIterator();
		this->ClipThis(fiter->First());
		TEActivate(fDocument->theTE);
		HiliteControl(fDocument->vScroll, 0);
		HiliteControl(fDocument->hScroll, 0);
		TEGetHiliteRgn(fDocument->hiliteRgn, fDocument->theTE);
		this->ClipToRoot(fiter->First());
		delete fiter;
	}
}

void DragPart::DeActivatingWindow(XMPFrame* frame)
{
	if (fDocument->frameIsActive)
	{
		XMPFrameFacetIterator* fiter = frame->CreateFacetIterator();
		this->ClipThis(fiter->First());
		TEDeactivate(fDocument->theTE);
		HiliteControl(fDocument->vScroll, 255);
		HiliteControl(fDocument->hScroll, 255);
		this->ClipToRoot(fiter->First());
		delete fiter;
	}
}

void DragPart::ClipThis(XMPFacet* facet)
{
	SetPort(facet->GetWindow()->GetPlatformWindow());
	
	Point xForm = this->GetAggregateTranslation(facet).AsQDPoint();
	RgnHandle clipRgn = facet->GetClipShape()->CopyQDRegion();
		
	OffsetRgn(clipRgn, xForm.h, xForm.v);
	SetClip(clipRgn);
	DisposeRgn(clipRgn);
}

void DragPart::ClipToRoot(XMPFacet* facet)
{
	XMPFacet* pointer = facet;
	while (pointer->GetContainingFacet() != kXMPNULL)
		pointer = pointer->GetContainingFacet();
	
	SetPort(pointer->GetWindow()->GetPlatformWindow());
	SetClip(pointer->GetClipShape()->GetQDRegion());
}

void DragPart::Idle(XMPFrame* frame)
{
	XMPFrameFacetIterator* fiter = frame->CreateFacetIterator();
	this->ClipThis(fiter->First());

	TEIdle(fDocument->theTE);
	
	this->ClipToRoot(fiter->First());
	delete fiter;
}

void DragPart::InvalidateFacet(XMPFacet* facet)
{
	facet->Invalidate(NULL);
}

void DragPart::FrameShapeChanged(XMPFrame* frame)
{
	frame->Invalidate(frame->GetUsedShape());

	frame->ChangeUsedShape(kXMPNULL);
		
	this->PositionDocumentParts();
	this->AdjustScrollBar();
	TEGetHiliteRgn(fDocument->hiliteRgn, fDocument->theTE);
	fFrameShapeChanged = false;

}

XMPBoolean DragPart::ActivateFrame(XMPFrame* frame)
{
	if (!fIsActive)
	{
		XMPBoolean succeeded = false;
							
		succeeded = fSession->GetArbitrator()->RequestFocusSet(fFocusSet,frame);
		
		if (succeeded)
		{
			this->InstallMenus(frame);
			fIsActive = true;
		}
	}
	return true;
}

void DragPart::DeActivateFrame(XMPFrame* frame)
{
	if (fIsActive)
	{
		XMPArbitrator* arbitrator = fSession->GetArbitrator();
									
		arbitrator->RelinquishFocusSet(fFocusSet,frame);
		fIsActive = false;
	}
}


void DragPart::HandleFontMenu( XMPUShort  item )
{
	Str255		fontName;
	XMPSShort	fontNum = 0;
	
	MenuHandle clockMenuH = GetMHandle( 141 );

	GetFontName( fLastFont, fontName );
	this -> CheckMarkMenu( clockMenuH, fontName, kXMPFalse );
	
	GetItem( clockMenuH, item, fontName );
	
	this -> CheckMarkMenu( clockMenuH, fontName, kXMPTrue );
	this -> OutlineSizeMenu();
	
	GetFNum( fontName, &fontNum );
	
	TextStyle *newStyle = (TextStyle *) XMPNewPtr(sizeof(TextStyle));
	
	(*newStyle).tsFont = fontNum;	
	
	TESetStyle(doFont, newStyle, kXMPTrue, fDocument->theTE);
	
	XMPDisposePtr((Ptr)newStyle);
	
	fLastFont = fontNum;
}


void DragPart::HandleSizeMenu( XMPUShort  item )
{
	XMPSShort	fontSize = 0;
	
	MenuHandle clockMenuH = GetMHandle( 142 );
	Str255	tSizeStr;
	
	NumToString( (XMPULong)fLastSize, tSizeStr );
	this -> CheckMarkMenu( clockMenuH, tSizeStr, kXMPFalse );
	
	switch (item) {
		case 1:	fontSize =  9; break;
		case 2:	fontSize = 10; break;
		case 3:	fontSize = 11; break;
		case 4:	fontSize = 12; break;
		case 5:	fontSize = 14; break;
		case 6:	fontSize = 18; break;
		case 7:	fontSize = 24; break;
		case 8:	fontSize = 36; break;
		default:
			fontSize = 12;
	}
	
	NumToString( (XMPULong)fontSize, tSizeStr );
	this -> CheckMarkMenu( clockMenuH, tSizeStr, kXMPTrue );

	fLastSize = fontSize;
	this -> OutlineSizeMenu();
	
	TextStyle *newStyle = (TextStyle *) XMPNewPtr(sizeof(TextStyle));
	
	(*newStyle).tsSize = fontSize;	
	
	TESetStyle(doSize, newStyle, kXMPTrue, fDocument->theTE);
	
	XMPDisposePtr((Ptr)newStyle);
}


void DragPart::CheckMarkMenu(MenuHandle	menu, Str255 checkItem, XMPBoolean	checkIt)
{
	
	XMPSShort  	numMenuItems = CountMItems( menu );
	Str255  	menuItemString;
	
	for (XMPSShort menuItemNum = 1; menuItemNum <= numMenuItems; menuItemNum++)
	{
		GetItem( menu, menuItemNum, menuItemString );
		
		if ( IUEqualString( checkItem, menuItemString) == 0)
		{
			if ( checkIt )
				CheckItem( menu, menuItemNum, kXMPTrue );
			else
				CheckItem( menu, menuItemNum, kXMPFalse );
		}
	}
}


void DragPart::OutlineSizeMenu()
{
	MenuHandle menu = GetMHandle( 142 );
	
	// () Outline the sizes in the font menu that are available
	
	if (RealFont(fLastFont, 9))
		SetItemStyle( menu, 1, outline );
	else
		SetItemStyle( menu, 1, normal );

	if (RealFont(fLastFont, 10))
		SetItemStyle( menu, 2, outline );
	else
		SetItemStyle( menu, 2, normal );

	if (RealFont(fLastFont, 11))
		SetItemStyle( menu, 3, outline );
	else
		SetItemStyle( menu, 3, normal );

	if (RealFont(fLastFont, 12))
		SetItemStyle( menu, 4, outline );
	else
		SetItemStyle( menu, 4, normal );

	if (RealFont(fLastFont, 14))
		SetItemStyle( menu, 5, outline );
	else
		SetItemStyle( menu, 5, normal );

	if (RealFont(fLastFont, 18))
		SetItemStyle( menu, 6, outline );
	else
		SetItemStyle( menu, 6, normal );

	if (RealFont(fLastFont, 24))
		SetItemStyle( menu, 7, outline );
	else
		SetItemStyle( menu, 7, normal );

	if (RealFont(fLastFont, 36))
		SetItemStyle( menu, 8, outline );
	else
		SetItemStyle( menu, 8, normal );
}

//------------------------------------------------------------------------------
// DragPart::Open
//------------------------------------------------------------------------------
XMPID DragPart::Open(XMPFrame* frame)
{		
	XMPWindow* window = kXMPNULL;

	if (frame) // Doing a View As Window
	{
		window = fSession->GetWindowState()->GetWindow(fWindowID);
		if (window)
			window->Select();
		else
		{
			window = this->CreateWindow(frame);
			fWindowID = window->GetID();
			window->Open();
			window->Show();
			window->Select();			
		}
	}
	else
	{
		window = this->CreateWindow(frame);
		fWindowID = window->GetID();
		window->Open();
		window->Show();
		window->Select();			
	}
	return window->GetID();
}

XMPWindow* DragPart::CreateWindow(XMPFrame* sourceFrame)
{
	Rect windRect;
	XMPPlatformWindow platformWindow = kXMPNULL;
	XMPWindow* window = kXMPNULL;
	
	if (sourceFrame)
	{
		SetRect(&windRect, 100, 100, 300, 300);	
	}
	else
	{
		const	XMPSShort		kOnePageWidth = 600;
		// Get the menu bar height, for figuring the top of our windows
		XMPSShort	mbHeight	= 24; //(*(XMPSShort*)MBarHeight);	//get it from lomem
		SetRect(&windRect, 4, mbHeight+16,
				XMPASLMQDGlobals.screenBits.bounds.right - 64,
				XMPASLMQDGlobals.screenBits.bounds.bottom - 4);		
		
		if (windRect.right-windRect.left > kOnePageWidth)
		{
			windRect.right = windRect.left + kOnePageWidth;
		}
	}
	
	platformWindow = NewCWindow(kXMPNULL, &windRect, "\pUntitled", false, zoomDocProc, (WindowPtr)-1L, true, kXMPNULL);
	window =  fSession->GetWindowState()->CreateWindow(platformWindow, 
													 (sourceFrame==kXMPNULL),	// Keeps draft open
													 kXMPTrue,	// Is resizable
													 kXMPFalse,	// Is floating
													 kXMPTrue,	// should save
												 this, kXMPNullTypeToken, kXMPNullTypeToken, sourceFrame);

	return window;
}


//------------------------------------------------------------------------------
// DragPart::MouseEnterFrame
//------------------------------------------------------------------------------
void DragPart::MouseEnter(XMPFacet* facet, XMPPoint where)
{
	XMPUnused(facet);

	this->AdjustCursor(facet->GetWindowContentTransform()->TransformPoint(where));	
}
	
			
//------------------------------------------------------------------------------
// DrawPart::MouseLeave
//------------------------------------------------------------------------------
void DragPart::MouseLeave(XMPFacet* facet)
{
	XMPUnused(facet);

	SetCursor(&(XMPASLMQDGlobals.arrow));
}



//
// Utilities
//

void DragPart::CreateDocument(XMPFrame* frame)
{
	TEStyleHandle theStyleHandle;
	GrafPtr	savePort;
	RgnHandle FrameShape;
	Point xForm;
	Rect scrollRect, destRect, viewRect, FrameBounds;
	
	// For now we can support only one display frame so if fDocument is already
	// present, then the display frame must have been changed.
	if (fDocument)
	{
		fDocument->theFrame = frame;
		fFrameShapeChanged = true;
	}
	else
	{
		XMPFrameFacetIterator* faceti = frame->CreateFacetIterator();

		fDocument = (Document *) XMPNewPtrClear(sizeof(Document));
		fDocument->theWindow = faceti->First()->GetWindow()->GetPlatformWindow();
		fDocument->theFrame = frame;
		
		frame->SetDroppable(kXMPTrue);
	
		GetPort(&savePort);
		SetPort(fDocument->theWindow);
		SetOrigin(0, 0);
			
		FrameShape = fDocument->theFrame->GetFrameShape()->GetQDRegion();
		xForm = this->GetAggregateTranslation(faceti->First()).AsQDPoint();
		FrameBounds = (**FrameShape).rgnBBox;
		OffsetRect(&FrameBounds, xForm.h, xForm.v);
		
		scrollRect = FrameBounds;
		scrollRect.left = scrollRect.right - 16;
		scrollRect.bottom -= 15;
		fDocument->vScroll = NewControl(fDocument->theWindow, &scrollRect, (ConstStr255Param)"", true, 0, 0, 0, 
								scrollBarProc, (long)fDocument);
		
		scrollRect = FrameBounds;
		scrollRect.top = scrollRect.bottom - 16;
		scrollRect.right  -= 15;
		fDocument->hScroll = NewControl(fDocument->theWindow, &scrollRect, (ConstStr255Param)"", true, 0, 0, 0,
								scrollBarProc, (long)fDocument);
		
		viewRect = FrameBounds;
		viewRect.right  -= 16;
		viewRect.bottom -= 16;
		
		destRect = viewRect;
		destRect.top    += TopMargin;
		destRect.left   += LeftMargin;
		destRect.bottom  = 32767;
		destRect.right  -= RightMargin;
		
		fDocument->theTE = TEStylNew(&destRect, &viewRect);
		
		TEAutoView(true, fDocument->theTE);
		
		TEFeatureFlag(teFOutlineHilite, TEBitSet, fDocument->theTE);
	
		if (fTextHandle)
		{
			TEActivate(fDocument->theTE);
			TESetSelect(32767, 32767, fDocument->theTE);
			XMPLockHandle(fTextHandle);
			TEStylInsert(*fTextHandle, XMPGetHandleSize(fTextHandle), fStylHandle, fDocument->theTE);
			XMPUnlockHandle(fTextHandle);
			TECalText(fDocument->theTE);
			TEDeactivate(fDocument->theTE);
			XMPDisposeHandle(fTextHandle);
			fTextHandle = kXMPNULL;
			XMPDisposeHandle((Handle)fStylHandle);
			fStylHandle = kXMPNULL;
		}
		
		fDocument->hiliteRgn = NewRgn();
		theStyleHandle = GetStylHandle(fDocument->theTE);
		(**theStyleHandle).teRefCon = (long)fDocument;
		
		fFrameShapeChanged = true;
	
		SetPort(savePort);
		delete faceti;
		
		// If a destination link was created by Internalize, we can register it now
		if (fDstLink.link != (XMPLink*) kXMPNULL)
			fDstLink.link->RegisterDependent(this, fDstLink.info.change);
	}
}

/*
 *	This routine computes the size and location of the window's scroll bars
 *	and the TextEdit field given the current size of the window. This routine
 *	is called to initialize these components of the window and to update
 *	the position of these components when the window is resized.
 */
void DragPart::PositionDocumentParts()
{
	RgnHandle FrameShape;
	Point xForm;
	Rect theRect, FrameBounds;
	GrafPtr	savePort;
	
	GetPort(&savePort);
	SetPort(fDocument->theWindow);
	SetOrigin(0, 0);
	
	XMPFrameFacetIterator* faceti = fDocument->theFrame->CreateFacetIterator();
	FrameShape = fDocument->theFrame->GetFrameShape()->GetQDRegion();
	xForm = this->GetAggregateTranslation(faceti->First()).AsQDPoint();
	FrameBounds = (**FrameShape).rgnBBox;
	OffsetRect(&FrameBounds, xForm.h, xForm.v);

	theRect = FrameBounds;
	theRect.left = theRect.right - 16;
	theRect.bottom -= 15;
	(**(fDocument->vScroll)).contrlRect = theRect;

	theRect = FrameBounds;
	theRect.top = theRect.bottom - 16;
	theRect.right  -= 15;
	(**(fDocument->hScroll)).contrlRect = theRect;

	theRect = FrameBounds;
	theRect.right  -= 16;
	theRect.bottom -= 16;
	(**(fDocument->theTE)).viewRect = theRect;
	(**(fDocument->theTE)).destRect.right = theRect.right - RightMargin;
	(**(fDocument->theTE)).destRect.top   = theRect.top + TopMargin;
	(**(fDocument->theTE)).destRect.left  = theRect.left + LeftMargin;
	TECalText(fDocument->theTE);

	SetPort(savePort);
}

/*
 *	AdjustScrollBar calculates the current position and maximum value of the
 *	document's scroll bar given the current viewing position and document
 *	text. This routine is called whenever the text is changed or the view
 *	is auto-scrolled.
 */
void DragPart::AdjustScrollBar()
{	
	short		docTop, docBottom, viewTop, viewBottom;
	short		offTop, offBottom;
	RgnHandle	viewRgn;

	docTop = (**(fDocument->theTE)).destRect.top;
	docBottom = (short)(docTop + TEGetHeight(32767, 0, fDocument->theTE));
	viewTop = (**(fDocument->theTE)).viewRect.top;
	viewBottom = (**(fDocument->theTE)).viewRect.bottom;

	offTop = ((viewTop - (docTop - TopMargin)) + ScrollResolution - 1) / ScrollResolution;
	offBottom = (((docBottom + BottomMargin) - viewBottom) + ScrollResolution - 1) / ScrollResolution;
	if (offTop < 0)
		offTop = 0;
	if (offBottom < 0)
		offBottom = 0;

	fDocument->vScrollPos = offTop;

	SetCtlMax(fDocument->vScroll, offTop + offBottom);
	SetCtlValue(fDocument->vScroll, offTop);

	viewRgn = NewRgn();
	RectRgn(viewRgn, &(**(fDocument->theTE)).viewRect);
	SectRgn(viewRgn, fDocument->hiliteRgn, fDocument->hiliteRgn);
	DisposeRgn(viewRgn);
}

void DragPart::AdjustCursor(XMPPoint where)
{	
	RgnHandle	arrowRgn, iBeamRgn, hiliteRgn;
	Rect		theRect;
	Point		thePoint,theLoc;
	GrafPtr		savePort;
	
	GetPort(&savePort);
	SetPort(fDocument->theWindow);
	SetOrigin(0, 0);

	theLoc = where.AsQDPoint();
	LocalToGlobal(&theLoc);
	
	arrowRgn = NewRgn();
	SetRectRgn(arrowRgn, -32767, -32767, 32767, 32767);
	iBeamRgn = NewRgn();
	hiliteRgn = NewRgn();

	theRect = (**(fDocument->theTE)).viewRect;
	LocalToGlobal((Point *)&(theRect.top));
	LocalToGlobal((Point *)&(theRect.bottom));
	RectRgn(iBeamRgn, &theRect);

	CopyRgn(fDocument->hiliteRgn, hiliteRgn);
	SetPt(&thePoint, 0, 0);
	LocalToGlobal(&thePoint);
	OffsetRgn(hiliteRgn, thePoint.h, thePoint.v);

	DiffRgn(arrowRgn, hiliteRgn, arrowRgn);
	DiffRgn(arrowRgn, iBeamRgn, arrowRgn);
	DiffRgn(iBeamRgn, hiliteRgn, iBeamRgn);

	if (PtInRgn(theLoc, iBeamRgn)) {
		SetCursor(*GetCursor(iBeamCursor));
	} else if (PtInRgn(theLoc, hiliteRgn)) {
		SetCursor(&(XMPASLMQDGlobals.arrow));
	} else {
		SetCursor(&(XMPASLMQDGlobals.arrow));
	}
	
	DisposeRgn(iBeamRgn);
	DisposeRgn(hiliteRgn);
	DisposeRgn(arrowRgn);
	SetPort(savePort);
}

short DragPart::DragText(EventRecord *theEvent)
{	
	RgnHandle		dragRgn, tempRgn;
	XMPDragAndDrop *dad;
	XMPStorageUnit *dadsu;
	XMPPart        *destPart;
	Point           xForm;
	
	/*
	 *	Wait in the following loop until the user moves the mouse by at
	 *	least 4 pixels. If the mouse is released before moving beyond this
	 *	minimum threshold, return false.
	 */
	if (!WaitMouseMoved(theEvent->where)) {
		return(false);
	}

	/*
	 *	Set up the drag region for the drag outline
	 */	
	dragRgn = NewRgn();
	CopyRgn(fDocument->hiliteRgn, dragRgn);
	SetPt(&xForm, 0, 0);
	LocalToGlobal(&xForm);
	OffsetRgn(dragRgn, xForm.h, xForm.v);
	tempRgn = NewRgn();
	CopyRgn(dragRgn, tempRgn);
	InsetRgn(tempRgn, 1, 1);
	DiffRgn(dragRgn, tempRgn, dragRgn);
	DisposeRgn(tempRgn);

	/*
	 *	Set up the drag and drop storage unit
	 */
	dad = fSession->GetDragAndDrop();
	dad->Clear();
	dadsu = dad->GetContentStorageUnit();

	this->ExternalizeSelection(dadsu, kXMPCloneCut);

	/*
	 *	Do the drag
	 */
	dad->StartDrag(fDocument->theFrame, "RGNH", dragRgn, &destPart, theEvent);

	DisposeRgn(dragRgn);

	return(true);
}

//------------------------------------------------------------------------------
// DragPart::HasExtension
//------------------------------------------------------------------------------

XMPBoolean	DragPart::HasExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	return !XMPISOStrCompare( (XMPISOStr)extensionName, semtIntf ) ;
}

//------------------------------------------------------------------------------
// DragPart::GetExtension
//------------------------------------------------------------------------------

XMPExtension*	DragPart::GetExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	if (!XMPISOStrCompare( (XMPISOStr)extensionName, semtIntf ))
		return fSemtIntf;
	else
		return kXMPNULL;
}

//------------------------------------------------------------------------------
// DragPart::GetSemanticInterface
//------------------------------------------------------------------------------

XMPSemanticInterface* DragPart::GetSemanticInterface()
{
	return fSemtIntf ;
}

//------------------------------------------------------------------------------
// DragPart::GetTEH
//------------------------------------------------------------------------------

TEHandle DragPart::GetTEH()
{
	return fDocument->theTE ;
}

//------------------------------------------------------------------------------
// DragPart::InvalAllFrames()
//------------------------------------------------------------------------------

void DragPart::InvalAllFrames()
{
	OrderedCollectionIterator dispIter(fDisplayFrames);
	for (XMPFrame* aFrame = (XMPFrame*)dispIter.First();
		 dispIter.IsNotComplete();
		 aFrame = (XMPFrame*)dispIter.Next())
	{
		aFrame->Invalidate( (XMPShape*)kXMPNULL ) ;
	}
}	// InvalAllFrames()

//------------------------------------------------------------------------------
// DragPart::GetDragTextFrame()
//------------------------------------------------------------------------------

XMPFrame* DragPart::GetDragTextFrame()
{
	return fDocument->theFrame ;
}

//------------------------------------------------------------------------------
// DragPart::AdjustMenus()
//------------------------------------------------------------------------------

void DragPart::AdjustMenus(XMPFrame* frame)
{
	XMPUnused(frame);

	this->SyncMenuBar();
}

extern "C" {

WindowOffscreen *DrawOffscreen(WindowPtr theWindow)
{	
	WindowOffscreen		*theOffscreen;
	GWorldPtr			theWorld;
	Rect				globalRect;

	if ((theOffscreen = (WindowOffscreen *) XMPNewPtr(sizeof(WindowOffscreen))) == 0L)
		return(0L);

	SetPort(theWindow);
	GetGWorld(&theOffscreen->windowPort, &theOffscreen->windowDevice);

	globalRect = theWindow->portRect;
	LocalToGlobal((Point *) &globalRect.top);
	LocalToGlobal((Point *) &globalRect.bottom);
	
	if (NewGWorld(&theWorld, 0, &globalRect, 0L, 0L, 0) == noErr)
	{
		SetGWorld(theWorld, 0L);
		if (!LockPixels(theWorld->portPixMap))
		{
			DisposeGWorld(theWorld);
			XMPDisposePtr((Ptr)theOffscreen);
			return(0L);
		}

		CopyBits(&theWindow->portBits,
				 &((GrafPtr) theWorld)->portBits,
				 &theWindow->portRect,
				 &theWorld->portRect,
				 srcCopy, 0L);

		theOffscreen->offscreenWorld = theWorld;
		SetPort(theWindow);
		return(theOffscreen);
	} 
	else 
	{
		XMPDisposePtr((Ptr)theOffscreen);
		return(0L);
	}
}

void DrawOnscreen(WindowOffscreen *theOffscreen)
{
	SetGWorld(theOffscreen->windowPort, theOffscreen->windowDevice);
	CopyBits(&((GrafPtr) theOffscreen->offscreenWorld)->portBits,
			 &((GrafPtr) theOffscreen->windowPort)->portBits,
			 &theOffscreen->offscreenWorld->portRect,
			 &theOffscreen->windowPort->portRect,
			 srcCopy, 0L);
	UnlockPixels(theOffscreen->offscreenWorld->portPixMap);
	DisposeGWorld(theOffscreen->offscreenWorld);
	XMPDisposePtr((Ptr) theOffscreen);
}


/*
 *	DrawCaret draws a caret in a TextEdit field at the given offset. DrawCaret
 *	expects the port to be set to the port that the TextEdit field is in.
 *	A clipping region may be optionally given to clip the drawing to. DrawCaret
 *	inverts the image of the caret onto the screen.
 */
 
void DrawCaret(short offset, TEHandle theTE, RgnHandle clipRgn)
{	
	Point		theLoc;
	short		theLine, lineHeight;
	RgnHandle	oldClip;

	/*
	 *	Get the coordinates and the line of the offset to draw the caret.
	 */

	theLoc  = TEGetPoint(offset, theTE);
	theLine = TEGetLine(offset, theTE);

	/*
	 *	For some reason, TextEdit dosen't return the proper coordinates
	 *	of the last offset in the field if the last character in the record
	 *	is a carriage return. TEGetPoint returns a point that is one line
	 *	higher than expected. The following code fixes this problem.
	 */

	if ((offset == (**theTE).teLength) &&
			(*((char **)(**theTE).hText))[(**theTE).teLength - 1] == 0x0D) {
		theLoc.v += (short)(TEGetHeight(theLine, theLine, theTE));
	}

	/*
	 *	Always invert the caret when drawing.
	 */

	PenMode(patXor);

	/*
	 *	Get the height of the line that the offset points to.
	 */

	lineHeight = (short)(TEGetHeight(theLine, theLine, theTE));

	/*
	 *	If we are given a clip region, use it to draw the caret.
	 */

	if (clipRgn) {
		oldClip = NewRgn();
		GetClip(oldClip);
		SetClip(clipRgn);
	}

	/*
	 *	Draw the appropriate caret image.
	 */

	MoveTo(theLoc.h - 1, theLoc.v - 1);
	Line(0, 1 - lineHeight);

	/*
	 *	Restore the clipping region, if necessary.
	 */

	if (clipRgn) {
		SetClip(oldClip);
		DisposeRgn(oldClip);
	}

	PenNormal();
}

char GetCharAtOffset(short offset, TEHandle theTE)
{
	if (offset < 0)
		return(0x0D);

	return(((char *) *((**theTE).hText))[offset]);
}


Boolean WhiteSpace(char theChar)
{
	return((theChar == ' ') || (theChar == 0x0D));
}


Boolean WhiteSpaceAtOffset(short offset, TEHandle theTE)
{	
	char		theChar;

	if (offset < 0)
		return(true);

	theChar = ((char *) *((**theTE).hText))[offset];
	return((theChar == ' ') || (theChar == 0x0D));
}


void InsertTextAtOffset(short offset, char *theBuf, long size, StScrpHandle theStyl, TEHandle theTE)
{
	if (size == 0)
		return;

	//
	//	If inserting at the end of a word and the selection does not begin with
	//	a space, insert a space before the insertion.
	//

	if (!WhiteSpaceAtOffset(offset - 1, theTE) &&
		 WhiteSpaceAtOffset(offset, theTE) &&
		!WhiteSpace(theBuf[0])) {

		TESetSelect(offset, offset, theTE);
		TEKey(' ', theTE);
		offset++;
	}

	//
	//	If inserting at the beginning of a word and the selection does not end
	//	with a space, insert a space after the insertion.
	//

	if ( WhiteSpaceAtOffset(offset - 1, theTE) &&
		!WhiteSpaceAtOffset(offset, theTE) &&
		!WhiteSpace(theBuf[size - 1])) {

		TESetSelect(offset, offset, theTE);
		TEKey(' ', theTE);
	}

	TESetSelect(offset, offset, theTE);
	TEStylInsert(theBuf, size, theStyl, theTE);
	TESetSelect(offset, offset + size, theTE);
}

/*
 *	TEIsFrontOfLine, given an offset and a TextEdit handle, returns true if
 *	the given offset is at the beginning of a line start.
 */

short TEIsFrontOfLine(short offset, TEHandle theTE)

{	short		line = 0;

	if ((**theTE).teLength == 0)
		return(true);

	if (offset >= (**theTE).teLength)
		return( (*((char **)(**theTE).hText))[(**theTE).teLength - 1] == 0x0d );

	while ((**theTE).lineStarts[line] < offset)
		line++;

	return( (**theTE).lineStarts[line] == offset );
}

/*
 *	TEGetLine, given an offset and a TextEdit handle, returns the line number
 *	of the line that contains the offset.
 */

short TEGetLine(short offset, TEHandle theTE)

{	short		line = 0;

	if (offset > (**theTE).teLength)
		return((**theTE).nLines);

	while ((**theTE).lineStarts[line] < offset)
		line++;
	
	return(line);
}

/*
 *	AdjustDocumentView scrolls the document image by the appropriate amount
 *	if the scroll bar value has changed.
 */
void AdjustDocumentView(Document *theDocument)
{	
	short delta, docTop, docTopLimit;

	delta = (theDocument->vScrollPos - GetCtlValue(theDocument->vScroll)) * ScrollResolution;

	if (delta) {
		if (delta > 0) {
			docTop = (**(theDocument->theTE)).destRect.top;
			docTopLimit = (**(theDocument->theTE)).viewRect.top + TopMargin;
			if (docTop + delta > docTopLimit) {
				delta = docTopLimit - docTop;
			}
		}
		TEScroll(0, delta, theDocument->theTE);
		theDocument->vScrollPos = GetCtlValue(theDocument->vScroll);
	}
}

/*
 *	ScrollProc is the function used in a call to TrackControl for
 *	scrolling a document window.
 */
pascal void ScrollProc(ControlHandle theControl, short theCode)
{
	short		delta = 0, pageDelta;
	Document	*theDocument;
	Rect		viewRect;

	if (theCode == 0)
		return;

	theDocument = (Document *) (**theControl).contrlRfCon;
	viewRect = (**(theDocument->theTE)).viewRect;
	pageDelta = ((viewRect.bottom - viewRect.top) / ScrollResolution) - 1;

	switch(theCode) {
		case inUpButton:
			delta = -1;
			break;
		case inDownButton:
			delta = 1;
			break;
		case inPageUp:
			delta = -pageDelta;
			break;
		case inPageDown:
			delta = pageDelta;
			break;
	}

	SetCtlValue(theControl, GetCtlValue(theControl) + delta);
	AdjustDocumentView((Document *) (**theControl).contrlRfCon);
}


} // extern "C"
