/*
	File:		DragPart.h

	Contains:	DragPart class definition.

	Written by:	Paul Hartenstine

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<43>	  4/4/94	CC		Added private method RevealLink()
									(1154069); include LinkDlgs.h and added
									XMPLinkInfo field for destination links
									(1154127).
		<42>	 3/27/94	T		#1153523 Merge PrtRCR into DragPart
		<41>	 3/17/94	CC		Added internal methods
									SelectionInLinkSource(),
									SelectionInLinkDestination(), and
									UserCategoryString(). (1151329)
		<40>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<39>	 3/14/94	CC		Added Incorporate methods and other
									revisions to support Paste As dialog.
									(1144421)
		<38>	 2/16/94	JA		Stop including Point.h.
		<37>	  2/8/94	JA		Added missing #include.
		<36>	  2/7/94	T		#include "TextEdit.h"		// for TEHandle, to
									make this compile
		<35>	  2/4/94	VL		Added QDOffScreen.h and Point.h to make
									this compile.
		<34>	  2/4/94	T		DONT #include header files for Class
									Definitions
		<33>	  2/4/94	VL		Removed Shape.h.
		<32>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame()
		<31>	  2/1/94	CC		Added XMPCloneKind argument to
									ExternalizeSelection.
		<30>	 1/28/94	CC		Added override of ReleaseAll().
		<29>	 1/27/94	RR		Added CreateWindow
		<28>	 1/25/94	NP		Added AdjustMenus.
		<27>	 1/24/94	CC		Added XMPChangeID parameter to
									LinkUpdated().
		<26>	 1/21/94	CC		Adapted to split of links into XMPLink and
									XMPLinkSource, conform to new link and
									clipboard Lock() method.  Added code to
									write LinkedText value format.
		<25>	 1/20/94	SS		::Open returns a window id
		<24>	 1/15/94	RR		WinState.h->WinStat.h,
									Dispatch.h->Disptch.h
		<23>	 1/14/94	CC		Changed fClipboardGeneration to
									fClipboardTimeStamp.
		<22>	 1/13/94	NP		Mods for new Undo API.
		<21>	 1/11/94	eeh		added GetDragTextFrame
		<20>	 1/11/94	T		Init... changes
		<19>	12/22/93	CC		Added private RemoveLink() method.
		<18>	12/21/93	JBS		rename FrameClosed to CloseDisplayFrame
		<17>	12/15/93	JBS		add FrameClosed
		<16>	12/13/93	T		InitPart changes
		<15>	12/13/93	RR		Remove extra parameter from MouseLeave
		<14>	 12/9/93	eeh		added GetTEH, GetSemanticInterface,
									HasExtension, GetExtension, InvalAllFrames;
									and fSemtIntf
		<13>	 12/2/93	CC		Use Lock() and Unlock() methods of the
									clipboard object instead of the clipboard
									focus.
		<12>	11/24/93	T		modifications to use correct part storage
									model and moved defines to DragDef.h
		<11>	11/18/93	CC		Support for links.
		<10>	11/16/93	SS		Added Facet methods and new methods for
									Undo handling
		 <9>	11/10/93	SS		Added command and clipboard focus methods
		 <8>	 11/8/93	SS		Added ExternalizeSelection method
		 <7>	10/28/93	PH		Add MouseEnter/Leave
		 <6>	10/26/93	SS		Added two new methods to simplify clipping;
									also added invalidation in
									frameshapechanged
		 <5>	10/21/93	PH		Change FulfillPromise call
		 <4>	10/20/93	PH		Add open method
		 <3>	10/14/93	RR		Added sourceFrame parameter to
									AddDisplayFrame
		 <2>	10/12/93	PH		Add fTextHandle and fStylHandle
		 <7>	 10/6/93	JA		Use new fixed-point XMPPoint.
		 <6>	 9/30/93	PH		Add support for multiple drag items
		 <5>	 9/29/93	VL		Added InitializePersistent.
		 <4>	 9/27/93	PH		Remove DragHilite
		 <3>	 9/16/93	RR		Added fMenuBar
		 <2>	 9/10/93	PH		Add support for dragging text
		 <1>	  9/7/93	PH		first checked in
*/

#ifndef _DRAGPART_
#define _DRAGPART_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _DRAGDEF_
#include "DragDef.h"
#endif

#ifndef _LINKDLGS_
#include "LinkDlgs.h"
#endif

#ifndef _SIMPLPRT_
#include "SimplPrt.h"
#endif

#ifndef __TEXTEDIT__
#include "TextEdit.h"		// for TEHandle
#endif

#ifndef __QDOFFSCREEN__
#include "QDOffscreen.h"	// for GWorldPtr, StScrpHandle
#endif

//==============================================================================
// Types used by this interface
//==============================================================================

class OrderedCollection;
class OrderedCollectionIterator;
class XMPFacet;
class XMPFrame;
class XMPMenuBar;
class XMPFocusSet;
class XMPDragItemIterator;
class XMPSemanticInterface;
class XMPShape;
class XMPStorageUnit;
class XMPSession;
class XMPWindow;

//==============================================================================
// Typedefs
//==============================================================================

struct UndoInfo
{
	XMPFrame*		displayFrame;
	XMPCommandID	command;
	XMPULong		start;
	XMPULong		end;
	XMPPtr			text;
	XMPHandle		style;
};

typedef struct UndoInfo UndoInfo;
typedef UndoInfo *UndoInfoPtr;

typedef struct Document 
{
	WindowPtr		theWindow;
	TEHandle		theTE;
	short			docTop;
	RgnHandle		hiliteRgn;
	ControlHandle	vScroll, hScroll;
	short			vScrollPos;
	Handle			undoDragText;
	short			undoSelStart, undoSelEnd;
	short			lastSelStart, lastSelEnd;
	XMPFrame       *theFrame;
	Boolean			frameIsActive, caretShow, cursorInContent, canAccept;
	long			caretTime;
	short			caretOffset, lastOffset, insertPosition;
} Document;

typedef struct WindowOffscreen {
	CGrafPtr		windowPort;
	GDHandle		windowDevice;
	GWorldPtr		offscreenWorld;
} WindowOffscreen;

typedef struct DestOfLink
{
	XMPLink*			link;
	XMPSShort			start;
	XMPSShort			end;
	XMPLinkInfo			info;
} DestOfLink;

typedef struct SrceOfLink
{
	XMPLinkSource*	linkSource;
	XMPSShort		start;
	XMPSShort		end;
} SrceOfLink;

extern "C" {

WindowOffscreen *DrawOffscreen(WindowPtr theWindow);
void DrawOnscreen(WindowOffscreen *theOffscreen);

void DrawCaret(short offset, TEHandle theTE, RgnHandle clipRgn);
void InsertTextAtOffset(short offset, char *theBuf, long size, StScrpHandle theStyl, TEHandle theTE);
Boolean WhiteSpaceAtOffset(short offset, TEHandle theTE);
Boolean WhiteSpace(char theChar);
char GetCharAtOffset(short offset, TEHandle theTE);
short TEIsFrontOfLine(short offset, TEHandle theTE);
short TEGetLine(short offset, TEHandle theTE);

pascal void ScrollProc(ControlHandle theControl, short theCode);
void AdjustDocumentView(Document *theDocument);

}
//==============================================================================
// DragPart
//==============================================================================

class DragPart : public SimplePart
{
public:
	DragPart();
	virtual ~ CDECL DragPart();
	
	XMPVMethod	void InitPart(XMPStorageUnit* storageUnit);
	XMPVMethod	void InitPartFromStorage(XMPStorageUnit* storageUnit);
	XMPVMethod	void ReleaseAll();
			   
	XMPVMethod void		AddDisplayFrame(XMPFrame* newFrame);
	XMPVMethod void        RemoveDisplayFrame(XMPFrame* oldFrame);
	XMPVMethod void        CloseDisplayFrame(XMPFrame* frame);
	XMPVMethod void		Draw(XMPFacet* facet, XMPShape* invalidShape);
	XMPVMethod XMPBoolean 	HandleEvent(XMPEventData event, XMPFrame* frame, XMPFacet* facet);
	XMPVMethod XMPBoolean	HandleMouseDown(XMPFacet* facet, XMPPoint where, XMPEventData event);
	XMPVMethod XMPBoolean	HandleKeyDown(XMPFrame* focusFrame, XMPEventData event);
	XMPVMethod XMPBoolean	HandleMenuEvent(XMPFrame* focusFrame, XMPEventData event);
	
	XMPVMethod void InstallMenus(XMPFrame* aFrame); 
	XMPVMethod void SyncMenuBar();
	XMPVMethod void RemoveMenus(XMPFrame* aFrame); 

	XMPVMethod void CommitRelinquishFocus(XMPTypeToken focus, 
										XMPFrame* currentFrame,
										XMPFrame* proposedFrame);

	XMPVMethod void InvalidateFacet(XMPFacet* facet);

	XMPVMethod void Idle(XMPFrame* frame);
	
	XMPVMethod void Externalize();
	
	XMPVMethod void FrameShapeChanged(XMPFrame* frame);
	
	XMPVMethod void FacetAdded(XMPFacet* facet);
	XMPVMethod void ExternalTransformChanged(XMPFacet* facet);
	
	XMPVMethod XMPID Open(XMPFrame* frame);

	XMPVMethod void MouseEnter(XMPFacet* facet, XMPPoint where);
	XMPVMethod void MouseLeave(XMPFacet* facet);

	XMPVMethod void FulfillPromise(XMPStorageUnitView *promiseSUView);
	XMPVMethod void DropCompleted(XMPPart* destPart, XMPDropResult dropResult);
	XMPVMethod void DragEnter(XMPDragItemIterator* dragInfo, XMPFacet* facet, XMPPoint where);
	XMPVMethod void DragWithin(XMPDragItemIterator* dragInfo, XMPFacet* facet, XMPPoint where);
	XMPVMethod void DragLeave(XMPFacet* facet, XMPPoint where);
	XMPVMethod XMPDropResult Drop(XMPDragItemIterator* dropInfo, XMPFacet* facet, XMPPoint where);

	
	XMPVMethod void UndoAction(XMPActionData action);
	XMPVMethod void RedoAction(XMPActionData action);
	XMPVMethod void DisposeActionState(XMPActionData action, XMPDoneState doneState);
	XMPVMethod void WriteActionState(XMPPtr action, XMPStorageUnitView* storageUnitView);
	XMPVMethod XMPPtr ReadActionState(XMPStorageUnitView* storageUnitView);

	XMPVMethod	XMPLink* CreateLink(XMPPtr data, XMPULong size);
	XMPVMethod void LinkUpdated(XMPLink* updatedLink, XMPChangeID id);
	XMPVMethod void AdjustLinks(
							XMPSShort 	start,
							XMPSShort 	endOld,
							XMPSShort 	endNew,
							XMPChangeID changeID);
	XMPVMethod void LinkChanged(SrceOfLink* srcLink, XMPChangeID changeID);
	XMPVMethod void RevealLink(XMPLinkSource* linkSource);
	
	XMPVMethod void AdjustMenus(XMPFrame* frame);

//private: BY CONVENTION -- needed by DragPart's object accessors

	TEHandle GetTEH() ;
	XMPVMethod XMPSemanticInterface* GetSemanticInterface() ;
	XMPVMethod XMPBoolean		HasExtension(XMPType extensionName);
	XMPExtension*	GetExtension(XMPType extensionName) ;
	XMPVMethod void InvalAllFrames() ;
	XMPFrame* GetDragTextFrame() ;

	XMPMethod XMPWindow* CreateWindow(XMPFrame* sourceFrame);

protected:
	XMPNVMethod	void CommonInitDragPart();
	
	XMPMethod void ActivatingWindow(XMPFrame* frame);
	
	XMPMethod void DeActivatingWindow(XMPFrame* frame);

	XMPMethod void ExternalizeSelection(XMPStorageUnit* su, XMPCloneKind cloneKind);

	XMPMethod XMPBoolean	HandleMouseDownInBorder(XMPFacet* container,
									XMPFacet* facet,
									XMPPoint where,
									XMPEventData event);
	XMPMethod XMPBoolean ActivateFrame(XMPFrame* frame);

	XMPMethod void DeActivateFrame(XMPFrame* frame);
	XMPMethod XMPPoint GetAggregateTranslation(XMPFacet* facet);
	XMPMethod void ExternalShapeToWindowShape(XMPFacet* facet, XMPShape* inShape, XMPShape* outShape);
	XMPMethod XMPPoint InternalPointToWindowPoint(XMPFacet* facet);

	XMPMethod void HandleFontMenu( XMPUShort  item );
	XMPMethod void HandleSizeMenu( XMPUShort  item );
	
	XMPMethod void CheckMarkMenu( MenuHandle	menu, Str255  checkItem, XMPBoolean  checkIt );
	XMPMethod void OutlineSizeMenu();

	XMPMethod void CreateDocument(XMPFrame* frame);
	XMPMethod void PositionDocumentParts();
	XMPMethod void AdjustScrollBar();
	XMPMethod void AdjustCursor(XMPPoint where);
	XMPMethod short DragText(EventRecord *theEvent);
	XMPMethod void ClipThis(XMPFacet* facet);
	XMPMethod void ClipToRoot(XMPFacet* facet);

	XMPMethod XMPBoolean DoCommand(XMPFrame* frame, XMPCommandID command);
	XMPMethod XMPBoolean DoCut(XMPFrame* frame);
	XMPMethod XMPBoolean DoCopy(XMPFrame* frame);
	XMPMethod XMPBoolean DoPaste(XMPFrame* frame, XMPType desiredKind);
	XMPMethod XMPBoolean DoPasteAs(XMPFrame* frame);
	XMPMethod XMPBoolean CanPasteLink();
	XMPMethod XMPBoolean DoPasteLink(XMPFrame* frame, XMPType desiredKind);
	XMPMethod XMPBoolean DoLinkInfo(XMPFrame* frame);
	XMPMethod XMPBoolean DoClear(XMPFrame* frame);
	XMPMethod XMPBoolean DoSelectAll(XMPFrame* frame);
	XMPMethod XMPBoolean DoSampleTwo(XMPFrame* frame);
	XMPMethod XMPBoolean DoSampleOne(XMPFrame* frame);
	
	XMPMethod void UndoRedoCut(UndoInfo* action, XMPUShort selector);
	XMPMethod void UndoRedoPaste(UndoInfo* action, XMPUShort selector);
	XMPMethod void UndoRedoPasteLink(UndoInfo* action, XMPUShort selector);
	XMPMethod void UndoRedoClear(UndoInfo* action, XMPUShort selector);
	XMPMethod void UndoRedoSampleOne(UndoInfo* action, XMPUShort selector);
	XMPMethod void UndoRedoSampleTwo(UndoInfo* action, XMPUShort selector);
	XMPMethod void UndoRedoTyping(UndoInfo* action, XMPUShort selector);
	XMPMethod void UndoRedoStyling(UndoInfo* action, XMPUShort selector);

	XMPMethod void PushUndoAction(XMPFrame* frame, XMPCommandID command, XMPULong start,
								XMPULong end, XMPPtr text, XMPHandle style,
								char* undoText, char* redoText);
								
	XMPMethod void IncorporateTestDragContent(XMPStorageUnit* su);
	XMPMethod void IncorporateTextContent(XMPStorageUnit* su);
	XMPMethod void IncorporateStyledTextContent(XMPStorageUnit* su);

	XMPMethod void ExpandSelectionAroundLink();
	XMPMethod XMPBoolean SelectionInLinkSource();
	XMPMethod XMPBoolean SelectionInLinkDestination();

	XMPMethod void AdjustDstLinks(XMPSShort start, XMPSShort endOld, XMPSShort endNew);
	XMPMethod void AdjustSrcLinks(
							XMPSShort 	start,
							XMPSShort 	endOld,
							XMPSShort 	endNew,
							XMPChangeID changeID);
	XMPMethod void CheckDstLinkDeletion (XMPSShort start, XMPSShort endOld);

	XMPMethod void RemoveSrcLink();
	XMPMethod void RemoveDstLink();
	XMPMethod void RemoveLinkSpec();
	XMPMethod XMPBoolean UserCategoryString(XMPName** name);

	void InstallObjectAccessors();

private:
	XMPBoolean fIsActive;
	XMPFocusSet* fFocusSet;

	Document          *fDocument;
	Handle             fTextHandle;
	StScrpHandle       fStylHandle;
	XMPPoint           fExtTransStore;
	XMPSShort          fLastFont;
	XMPSShort          fLastSize;
	XMPBoolean         fFrameShapeChanged;
	OrderedCollection *fDisplayFrames;
	XMPID				fWindowID;			// ID of View As Window window
	SrceOfLink			fSrcLink;
	DestOfLink			fDstLink;
	XMPDraft*			fDraft;
	
	MenuHandle	fTextMenu;
	MenuHandle	fFontMenu;
	MenuHandle	fSizeMenu;
	XMPMenuBar* fMenuBar;

	XMPChangeID fClipboardChangeID;

	XMPUShort			fUndoCount;
	XMPUShort			fRedoCount;

	XMPStorageUnit*		fTestDragSU;
	XMPSemanticInterface*	fSemtIntf;
};

#endif // _DRAGPART_