/*
	File:		DrawObj.cpp

	Contains:	Object accessor implementation for the DrawPart

	Written by:	Caia Grisar

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<10>	  4/6/94	JBS		1155477
		 <9>	 3/28/94	CG		1153547: include XMPSessM.h instead of
									XMPSessn.h.
		 <8>	 3/26/94	NP		1153509. Messaging not PowerPC ready.
		 <7>	 3/26/94	NP		1153053.
		 <6>	 3/25/94	JA		JPA: GetPlatformTransform -> GetQDOffset
									(1153438). JBS: Changed API of
									CreateEmbeddedFrameIterator, CanvasChanged
									(1153056, 1153059).
		 <5>	 3/25/94	MB		Symantec ASLM fixes. #1150864
		 <4>	 3/25/94	eeh		bug #1153053: Changes for PPC nativity
		 <3>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <2>	 2/16/94	JA		Include new AltPoint.h.
		<21>	  2/8/94	JA		Added missing #includes.
		<20>	  2/8/94	T		Throw -> THROW & some code clean up
		<19>	  2/7/94	JA		Utility.h --> XMPUtils.h
		<18>	  2/7/94	T		fixes to compile with PPC Headers
		<17>	  2/2/94	JBS		new AddDisplayFrame recipie
		<16>	 1/31/94	NP		Fixed coercion handler to conform to new
									interface.
		<15>	 1/28/94	NP		Fixed include statements.
		<14>	 1/17/94	NP		Removed consts from public API.
		<13>	 1/17/94	NP		Semantic Events callbacks API changes.
		<12>	 1/14/94	NP		Removed CreateObjSpecifier (it's already in
									OpenDoc) and fixed some include statements.
		<11>	 1/11/94	eeh		removed "frame" from objspec vocab; work on
									Move event and bounds property (not done)
		<10>	12/20/93	NP		Changed calls to CreateSwapToken.
		 <9>	12/17/93	eeh		object accessors for frames
		 <8>	12/15/93	NP		CreatePartToken -> CreateSwapToken.
		 <7>	 12/9/93	SS		Compiler conditional #include
		 <6>	 12/9/93	eeh		implement CreateNewPart,
									CreatePartObjectSpec, and SetNewBounds,
									helpers, and general support for scripting,
									esp the make event
		 <5>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		 <4>	 12/1/93	CG		bug fix.
		 <3>	 12/1/93	CG		Added CreateNewPart, CreatePartObjectSpec,
									GetPropertyFromPart,GetPartFromPart,
									updated HandleGetData and HandleSetData.
		 <2>	11/30/93	JA		Rearranged #includes for new "frame"
									constant fix.
		 <1>	11/16/93	CG		first checked in
	To Do:
*/

#ifndef _ALTPOINT_
#include "AltPoint.h"			// Use C++ savvy XMPPoint and XMPRect
#endif


#ifndef _PLFMTYPE_
#include "PlfmType.h"
#endif

#include "DrawObj.h"

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _DRAWPART_
#include "DrawPart.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef _NAMRSLVM_
#include "NamRslvr.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _PART_
#include "Part.h"
#endif

#ifndef _PRTPRPAC_
#include "PrtPrpAc.h"
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

#ifndef _XMPSESSM_
#include "XMPSessM.h"
#endif

#ifndef _SEUTILS_
#include "SEUtils.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _XMPUTILS_
#include "XMPUtils.h"
#endif

#ifndef __AEOBJECTS__
#include <AEObjects.h>
#endif

#ifndef __AEPACKOBJECT__
#include <AEPackObject.h>
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef _MSSGINTF_
#include "MssgIntf.h"
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#pragma segment DrawObj

//==============================================================================
// Temporary constants
//==============================================================================

#define cDrawPart 'draw'
#define cDragPart 'drag'
#define cClockPart 'clck'
#define cTestPart 'test'

//==============================================================================
// Semantic Event Handlers and utility Functions
//==============================================================================

// This might be moved into a common header file, but there's something to
// be said for having it right here.

#define XMPOJBECT_ACCESSOR_PARAMS	XMPPart*			thisPart,			\
									DescType			desiredClass,		\
									XMPOSLToken*	container,			\
									DescType			containerClass,		\
									DescType			form,				\
									AEDesc*		selectionData,		\
									XMPOSLToken*		value,				\
									XMPSLong			refCon

static XMPError GetPropertyFromNULL(XMPOJBECT_ACCESSOR_PARAMS);
static XMPError GetPartFromNULL(XMPOJBECT_ACCESSOR_PARAMS);
//static void GetFrameFromNULL(XMPOJBECT_ACCESSOR_PARAMS);
static XMPError GetPropertyFromPart(XMPOJBECT_ACCESSOR_PARAMS);
static XMPError GetWildcardFromPart(XMPOJBECT_ACCESSOR_PARAMS);
//static void GetPartFromPart(XMPOJBECT_ACCESSOR_PARAMS);
//static void GetPropertyFromFrame(XMPOJBECT_ACCESSOR_PARAMS);
static XMPError GetWildcardFromList(XMPOJBECT_ACCESSOR_PARAMS);
//static void GetPropertyFromFrame(XMPOJBECT_ACCESSOR_PARAMS);
//static void GetPartFromFrame(XMPOJBECT_ACCESSOR_PARAMS);

static XMPError HandleSetData(XMPPart*		thisPart,
						AppleEvent*	theAppleEvent,
						AppleEvent*	reply,
						XMPSLong	handlerRefcon);
						
static XMPError HandleGetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon);
					
static XMPError HandleCreate(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon);

static XMPError HandleMove(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon);


static XMPError CoerceList2RGB(XMPPart* thisPart, AEDesc* from, DescType toType,
							XMPSLong /*refcon*/, AEDesc* result);

static XMPError DrawCompare(XMPPart* thePart, DescType oper,
		XMPOSLToken* obj1, XMPOSLToken*	obj2, XMPBoolean* result,
		XMPSLong refCon);

static XMPError DrawCount(XMPPart* thePart, DescType desiredType,
		DescType containerClass, XMPOSLToken* container,
		XMPSLong* result, XMPSLong refCon);
static XMPError DrawDisposeToken(XMPPart* thePart, XMPOSLToken* unneededToken,
		XMPSLong refCon);

//------------------------------------------------------------------------------
// DrawPart::InstallObjectAccessors
//------------------------------------------------------------------------------

void DrawPart::InstallObjectAccessors()
{
	XMPObjectAccessorUPP theAccessorUPP ;

	theAccessorUPP = NewXMPObjectAccessorProc( GetPropertyFromNULL ) ;
	fSemtIntf->InstallObjectAccessor(cProperty, typeNull, theAccessorUPP,
										(XMPSLong)fSession);

	theAccessorUPP = NewXMPObjectAccessorProc( GetPartFromNULL ) ;
	fSemtIntf->InstallObjectAccessor(cPart, typeNull, theAccessorUPP,
										(XMPSLong)fSession);

	theAccessorUPP = NewXMPObjectAccessorProc( GetPropertyFromPart ) ;
 	fSemtIntf->InstallObjectAccessor(cProperty, cPart, theAccessorUPP,
										(XMPSLong)fSession);

	theAccessorUPP = NewXMPObjectAccessorProc( GetWildcardFromPart ) ;
 	fSemtIntf->InstallObjectAccessor(typeWildCard, cPart, theAccessorUPP,
										(XMPSLong)fSession);

	theAccessorUPP = NewXMPObjectAccessorProc( GetWildcardFromList ) ;
 	fSemtIntf->InstallObjectAccessor(typeWildCard, typeAEList,
										theAccessorUPP, (XMPSLong)fSession);


	XMPEventHandlerUPP theEventHandlerUPP ;

	theEventHandlerUPP = NewXMPEventHandlerProc( HandleSetData ) ;
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAESetData,
												theEventHandlerUPP,
												(XMPSLong)fSession);

	theEventHandlerUPP = NewXMPEventHandlerProc( HandleGetData ) ;
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAEGetData,
												theEventHandlerUPP,
												(XMPSLong)fSession);

	theEventHandlerUPP = NewXMPEventHandlerProc( HandleCreate ) ;
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAECreateElement,
												theEventHandlerUPP,
												(XMPSLong)fSession);

	theEventHandlerUPP = NewXMPEventHandlerProc( HandleMove ) ;
	fSemtIntf->InstallEventHandler(kAECoreSuite, kAEMove, theEventHandlerUPP,
												(XMPSLong)fSession);

	XMPCoercionHandlerUPP coercionHndUPP =
			NewXMPDescCoercionHandlerProc( CoerceList2RGB ) ;
	fSemtIntf->InstallCoercionHandler(typeAEList, cRGBColor,
			coercionHndUPP, (XMPSLong)kXMPNULL, kXMPTrue);


	XMPCompareUPP compUPP = NewXMPCompareProc( DrawCompare ) ;
	fSemtIntf->InstallCompareProc(compUPP, (XMPSLong)fSession);

	XMPCountUPP countUPP = NewXMPCountProc( DrawCount ) ;
	fSemtIntf->InstallCountProc(countUPP, (XMPSLong)fSession);

	XMPDisposeTokenUPP disposeTokenUPP
			= NewXMPDisposeTokenProc( DrawDisposeToken ) ;
	fSemtIntf->InstallDisposeTokenProc( disposeTokenUPP, (XMPSLong)fSession);

//	fSemtIntf->InstallObjectAccessor(cPart, cFrame, GetPartFromFrame,
//										(XMPSLong)fSession);
// 	fSemtIntf->InstallObjectAccessor(cProperty, cFrame, GetPropertyFromFrame,
//										(XMPSLong)fSession);
// 	fSemtIntf->InstallObjectAccessor(cFrame, typeNull, GetFrameFromNULL,
//										(XMPSLong)fSession);
}

//------------------------------------------------------------------------------
// DrawPart::CreateNewPart
//	Copied from DrawPart::Embed.  For now theLocation is unused.
//------------------------------------------------------------------------------

// $$$$$ a very temporary hack....
static short ItemFromClass(DescType theClass)
{
	short item;
	
	switch(theClass)
	{
		case cDrawPart : item = 1;
		case cDragPart : item = 2;
		case cClockPart : item = 3;
		case cTestPart : item = 4;
	}
	
	return item;
}

XMPSemanticInterface* DrawPart::GetSemanticInterface()
{
	return fSemtIntf;
}

XMPPart* DrawPart::CreateNewPart(DescType theClass, AEDesc *theLocation)
{
	XMPUnused(theLocation);
	OrderedCollectionIterator iter(this->fDisplayFrames);
	XMPFrame* frame = (XMPFrame*)iter.First();

#ifdef TO_BE_DELETED
	XMPPoint ignoreWhere;
	XMPEventData ignoreEvent;
	
	XMPFrameFacetIterator* fi = frame->CreateFacetIterator();
	for(XMPFacet* facet = fi->First(); fi->IsNotComplete();
			facet = fi->Next())
	{
		XMPFacet* container = facet->GetContainingFacet();
		this->HandleMouseDownInEmbeddedFrame(container, facet, ignoreWhere,
				ignoreEvent);
	}

	switch (fSelection->Count())
	{
		case 0:
			break;

		case 1:
			this->InvalidateSelection(frame);
			this->EmptySelection();
			this->ClipEmbeddedFrames(frame);
			break;
			
		default:
			// !!! handle multiple selection
			break;
	}

#endif // TO_BE_DELETED

	XMPPart* theResult;
	this->Embed(ItemFromClass(theClass), frame, &theResult);
	return theResult;

#ifdef TO_BE_DELETED
XMPUnused(theLocation);
	XMPFrame* embeddedFrame;
	XMPTransform* externalXForm = new XMPTransform;
	XMPSLong count = 1 + fContents->Count();
	XMPStorageUnit* su = this->GetStorageUnit();
		// For now, assume first frame
	OrderedCollectionIterator iter(this->fDisplayFrames);
	XMPFrame* frame = (XMPFrame*)iter.First();

	XMPPart* theNewPart;
	switch((long)theClass)
	{
		case cDrawPart:
			theNewPart = su->GetDraft()->CreatePart(kXMPPartDrawPart);
			break;
		case cDragPart:
			theNewPart = su->GetDraft()->CreatePart(kXMPPartDragPart);
			break;
		case cClockPart:
			theNewPart = su->GetDraft()->CreatePart(kXMPPartClockPart);
			break;
		case cTestPart:
//			theNewPart = su->GetDraft()->CreatePart(kXMPPartTestPart);
			break;
		default:
			break;
	}
	Point ptTemp;
	SetPt(&ptTemp,(short)(count*20),(short)(count*20));
	externalXForm->SetPlatformTransform(kXMPQuickDraw,
										*(XMPPlatformTransform*)&ptTemp);
	embeddedFrame = this->CreateEmbeddedFrame(frame,
											   NULL,
											   externalXForm,
											   theNewPart,
											   0,
											   kXMPFalse);

	XMPFrameFacetIterator* facets = frame->CreateFacetIterator();
	for (XMPFacet* facet = facets->First(); facets->IsNotComplete();
			facet = facets->Next())
	{
		XMPShape* clip = new XMPShape;
		clip->CopyFrom(embeddedFrame->GetFrameShape());
		XMPTransform* xform = new XMPTransform;
		xform->CopyFrom(this->ProxyForFrame(embeddedFrame)->transform);
		facet->CreateEmbeddedFacet(embeddedFrame, clip, xform, kXMPNULL,
				kXMPFrameInFront);
	}
	delete facets;
	this->ClipEmbeddedFrames(frame);
	frame->Invalidate(NULL);
	
	return theNewPart;
#endif // TO_BE_DELETED
}	// CreateNewPart()

//------------------------------------------------------------------------------
// DrawPart::CreatePartObjectSpec
//------------------------------------------------------------------------------

static void CreatePartObjectSpec(DescType keyForm, AEDesc* containerObjSpec,
		AEDesc *objectSpec)
{
	// We'll really return a spec to the FRAME containing the part.
	// This cause you can get to the part from the frame, but not the other
	// way 'round.
	
	AEDesc keyData;
	
	switch(keyForm)
	{
		case formAbsolutePosition :
			long index = 1;			// <eeh> we should figure out where
			THROW_IF_ERROR(AECreateDesc(typeLongInteger, &index, sizeof(index),
					&keyData));
			break;
		default :
			ASSERTM(false, -1, "Can't handle that form!!! (yet)");
	}

	CreateObjSpecifier(cPart, containerObjSpec, keyForm, &keyData, true,
			objectSpec);
}	// CreatePartObjectSpec()



//------------------------------------------------------------------------------
// GetPropertyFromNULL
//------------------------------------------------------------------------------

static XMPError GetPropertyFromNULL(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(desiredClass);
XMPUnused(container);
XMPUnused(containerClass);

	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();
	DescType				propID;
	DrawPartPropAccessor*	accessorObj;
	XMPError	error = noErr;

	TRY

	if (form != formPropertyID)
		THROW(errAEWrongDataType);

	switch(propID = **(DescType**)(selectionData->dataHandle))
	{
//		case pBounds:
		case pColor:
		case pName:
			resolver->CreateToken(value, cProperty, thisPart);
			accessorObj = new DrawPartPropAccessor(propID, (DrawPart*)thisPart);
			ReallocAndStuff2nd4Bytes(value, (XMPULong)accessorObj);
			break;
		default:
			THROW(errAECantSupplyType);
			break;
	}
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetPropertyFromNULL()



//------------------------------------------------------------------------------
// GetPartFromNULL
//------------------------------------------------------------------------------

static XMPError GetPartFromNULL(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
	XMPUnused(desiredClass);
	XMPUnused(container);
	XMPUnused(containerClass);

	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();
	XMPSLong				partCount;
	XMPSLong				theIndex;
	XMPBoolean				allFlag;
	XMPBoolean				zeroFlag;
	XMPError				result;
	XMPUShort 				i;
//	XMPPart* 				theEmbeddedPart;
	XMPFrame* 				frame;
	XMPOSLToken				listItem;
	XMPError				error = noErr;

	TRY

	partCount = ((DrawPart*)thisPart)->CountEmbeddedFrames();
	
	if (partCount != 0)
	{
		switch (form)
		{
			case formAbsolutePosition:
					// Are we being asked for and indexed part
					// or for all parts?
				result = DecodeOrdinal(*selectionData, (long)partCount, 
								(long*)&theIndex, (Boolean*)&allFlag, (Boolean*)&zeroFlag);
				THROW_IF_ERROR(result);
				
				if (allFlag)
				{
					result = AECreateList(kXMPNULL, 0, kXMPFalse, (AEDesc*)value);
					THROW_IF_ERROR(result);
		
						// Get theEmbeddedPart number theIndex
					DrawEmbeddedFramesIterator* iter = 
							(DrawEmbeddedFramesIterator*)thisPart->CreateEmbeddedFramesIterator(kXMPNULL);
					for (frame = iter->First(), i=1; iter->IsNotComplete(); frame = iter->Next(), i++)
					{	
//						theEmbeddedPart = frame ? frame->GetPart() : (XMPPart*)kXMPNULL;
						
						if (frame)
						{
							resolver->CreateToken(&listItem, cPart, thisPart);
							ReallocAndStuff2nd4Bytes(&listItem, (XMPULong)frame);
						
							result = AEPutDesc((AEDesc*)value, i, (AEDesc*)&listItem);
							THROW_IF_ERROR(result);
							
							result = AEDisposeDesc((AEDesc*)&listItem);
							THROW_IF_ERROR(result);
						}
					}
				}
				else
				{
						// Get theEmbeddedPart number theIndex
					DrawEmbeddedFramesIterator* iter = 
							(DrawEmbeddedFramesIterator*)thisPart->CreateEmbeddedFramesIterator(kXMPNULL);
					for (frame = iter->First(), i=1; iter->IsNotComplete(); frame = iter->Next(), i++)
					{	
						if (i == theIndex)
						{
					//		theEmbeddedPart = frame ? frame->GetPart() : (XMPPart*)kXMPNULL;
							break;
						}
					}
					
					if (frame)
					{
						resolver->CreateToken(value, cPart, thisPart);
						ReallocAndStuff2nd4Bytes(value, (XMPULong)frame);
					}
					else
						THROW(errAENoSuchObject);
				}
				break;
			default:
				THROW(errAENoSuchObject);
				break;
		}
	}
	else
		THROW(errAENoSuchObject);
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetPartFromNULL()


//------------------------------------------------------------------------------
// GetPartFromFrame
//------------------------------------------------------------------------------

static void GetPartFromFrame(	XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
	XMPUnused(thisPart);
	XMPUnused(desiredClass);
	XMPUnused(containerClass);
	XMPUnused(form);
	XMPUnused(selectionData);

	// Here I'm ASSUMING Part 1 of frame.  Nothing else will work just now.
	
	
	// A part is represented by its frame, as you can
	// always get the part when you need it.  Leave it that way until
	// the swap (save that the "type" of the token needs to be changed.)
	
	XMPFrame* theFrame = (XMPFrame*)GetSecondFourBytesFromDataHandle(container);

	XMPNameResolver* resolver = ((XMPSession*)refCon)->GetNameResolver();
	resolver->CreateToken(value, cPart, thisPart);
	ReallocAndStuff2nd4Bytes(value, (XMPULong)theFrame);

}	// GetPartFromFrame()


//------------------------------------------------------------------------------
// GetFrameFromNULL
//------------------------------------------------------------------------------

#ifdef TO_BE_DELETED
static void GetFrameFromNULL(	XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
	XMPUnused(desiredClass);
	XMPUnused(container);
	XMPUnused(containerClass);

	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();
	XMPSLong				partCount;
	XMPSLong				theIndex;
	XMPBoolean				allFlag;
	XMPBoolean				zeroFlag;
	XMPError				result;
	XMPUShort 				i;
//	XMPPart* 				theEmbeddedPart;
	XMPFrame* 				frame;
	XMPOSLToken				listItem;

	partCount = ((DrawPart*)thisPart)->CountEmbeddedFrames();
	
	if (partCount != 0)
	{
		switch (form)
		{
			case formAbsolutePosition:
					// Are we being asked for an indexed frame
					// or for all parts?
				result = DecodeOrdinal((AEDesc)*selectionData, (long)partCount, 
								(long*)&theIndex, (Boolean*)&allFlag, (Boolean*)&zeroFlag);
				THROW_IF_ERROR(result);
				
				if (allFlag)
				{
					result = AECreateList(kXMPNULL, 0, kXMPFalse, (AEDesc*)value);
					THROW_IF_ERROR(result);
		
						// Get theEmbeddedPart number theIndex
					DrawEmbeddedFramesIterator* iter = 
							(DrawEmbeddedFramesIterator*)thisPart->CreateEmbeddedFramesIterator(kXMPNULL);
					for (frame = iter->First(), i=1; iter->IsNotComplete(); frame = iter->Next(), i++)
					{	
						if (frame)
						{
							resolver->CreateToken(&listItem, cFrame, thisPart);
							ReallocAndStuff2nd4Bytes(&listItem, (XMPULong)frame);
						
							result = AEPutDesc((AEDesc*)value, i, (AEDesc*)&listItem);
							THROW_IF_ERROR(result);
							
							result = AEDisposeDesc((AEDesc*)&listItem);
							THROW_IF_ERROR(result);
						}
					}
				}
				else
				{
						// Get theEmbeddedPart number theIndex
					DrawEmbeddedFramesIterator* iter = 
							(DrawEmbeddedFramesIterator*)
							thisPart->CreateEmbeddedFramesIterator(kXMPNULL);
					for (frame = iter->First(), i=1; iter->IsNotComplete();
							frame = iter->Next(), i++)
					{	
						if (i == theIndex)
						{
							break;
						}
					}
					
					if (frame)
					{
						resolver->CreateToken(value, cFrame, thisPart);
						ReallocAndStuff2nd4Bytes(value, (XMPULong)frame);
					}
					else
						THROW(errAENoSuchObject);
				}
				break;
			default:
				THROW(errAENoSuchObject);
				break;
		}
	}
	else
		THROW(errAENoSuchObject);
}	// GetFrameFromNULL()
#endif // TO_BE_DELETED


								
//------------------------------------------------------------------------------
// GetPropertyFromPart
//------------------------------------------------------------------------------

static XMPError GetPropertyFromPart(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(desiredClass);
XMPUnused(containerClass);

	XMPSession*			session = (XMPSession*)refCon;
	XMPNameResolver*	resolver = session->GetNameResolver();
	DescType			propID;
	XMPError			error = noErr;

	TRY

	if (form != formPropertyID)
		THROW(errAEWrongDataType);

	XMPFrame* frame = (XMPFrame*)GetSecondFourBytesFromDataHandle(container);

	propID = **(DescType**)(selectionData->dataHandle);
	switch (propID)
	{
 		case pTranslation:
		case pBounds:
		{
			resolver->CreateToken(value, typeEmbededPartProp, thisPart);
			EmbeddedPartPropAccessor* accessorObj =
					new EmbeddedPartPropAccessor(propID, frame,
							(DrawPart*)thisPart);
			ReallocAndStuff2nd4Bytes(value, (XMPULong)accessorObj);
		}
		default:
		{
			break;
		}
	}

	resolver->CreateSwapToken(value, frame);
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetPropertyFromPart()

//------------------------------------------------------------------------------
// GetWildcardFromPart
//------------------------------------------------------------------------------

static XMPError GetWildcardFromPart(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
	XMPUnused(thisPart);
	XMPUnused(desiredClass);
	XMPUnused(containerClass);
	XMPUnused(form);
	XMPUnused(selectionData);

	XMPError	error = noErr;

	TRY

	// we need to just return a switch here: we can't go into the part.
	XMPNameResolver* resolver = ((XMPSession*)refCon)->GetNameResolver();

	// REMEMBER: I don't know what kind of part this is!
	XMPFrame* frame = (XMPFrame*)GetSecondFourBytesFromDataHandle(container);

	resolver->CreateSwapToken(value, frame);
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetWildcardFromPart()

//------------------------------------------------------------------------------
// GetPropertyFromFrame
//------------------------------------------------------------------------------

#ifdef TO_BE_DELETED
static void GetPropertyFromFrame(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(desiredClass);
XMPUnused(containerClass);

	XMPSession*			session = (XMPSession*)refCon;
	XMPNameResolver*	resolver = session->GetNameResolver();
	DescType			propID;
	FramePropAccessor*	accessorObj;

	if (form != formPropertyID)
		THROW(errAEWrongDataType);

	propID = **(DescType**)(selectionData->dataHandle);
	switch (propID)
	{
		case pBounds:
		{
			XMPFrame* theFrame
					= (XMPFrame*)GetSecondFourBytesFromDataHandle(container);
			resolver->CreateToken(value, typeFrameProp, thisPart);
			accessorObj = new FramePropAccessor(propID, theFrame);
			ReallocAndStuff2nd4Bytes(value, (XMPULong)accessorObj);
			break;
		}
		default:
		{
			THROW(errAENoSuchObject);
		}
	}
}	// GetPropertyFromFrame()
#endif // TO_BE_DELETED


//------------------------------------------------------------------------------
// GetPartFromPart
//------------------------------------------------------------------------------

#ifdef TO_BE_DELETED
static void GetPartFromPart(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(thisPart);
XMPUnused(desiredClass);
XMPUnused(container);
XMPUnused(containerClass);
XMPUnused(form);
XMPUnused(selectionData);

	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();

	XMPFrame* frame
			= (XMPFrame*)GetSecondFourBytesFromDataHandle(container);
	resolver->CreateSwapToken(value, frame);
}	// GetPartFromPart()
#endif // TO_BE_DELETED


//------------------------------------------------------------------------------
// GetWildcardFromList
//------------------------------------------------------------------------------

static XMPError GetWildcardFromList(XMPPart*			thisPart,
								DescType			desiredClass,
								XMPOSLToken*	container,
								DescType			containerClass,
								DescType			form,
								AEDesc*		selectionData,
								XMPOSLToken*		value,
								XMPSLong			refCon)
{
XMPUnused(containerClass);

	XMPError	result;
	XMPSLong	itemCount;
	AEKeyword	theAEKeyword;
	XMPOSLToken	thisItem;
	XMPOSLToken	myToken;
	XMPSLong	i;
	XMPError	error = noErr;

	TRY

	XMPSession*				session = (XMPSession*)refCon;
	XMPNameResolver*		resolver = session->GetNameResolver();

	result = AECountItems((AEDesc*)container, &itemCount);
	THROW_IF_ERROR(result);
	
	result = AECreateList(nil,0,false,(AEDesc*)value);
	THROW_IF_ERROR(result);

	for (i=1;i<=itemCount;++i)
	{
		result = AEGetNthDesc((AEDesc*)container,i,typeWildCard, 
											&theAEKeyword, (AEDesc*)&thisItem);
		THROW_IF_ERROR(result);

		resolver->CallObjectAccessor(thisPart, desiredClass, &thisItem, 
						thisItem.GetDescType(),form, (AEDesc*)selectionData, 
						&myToken);

		result = AEPutDesc((AEDesc*)value, 0, (AEDesc*)&myToken);
		THROW_IF_ERROR(result);

		result = AEDisposeDesc((AEDesc*)&thisItem);
		THROW_IF_ERROR(result);

		result = AEDisposeDesc((AEDesc*)&myToken);
		THROW_IF_ERROR(result);
	}
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// GetWildcardFromList()


//------------------------------------------------------------------------------
// HandleSetData
//------------------------------------------------------------------------------

static XMPError HandleSetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
	XMPUnused(thisPart);
	XMPUnused(reply);
	XMPUnused(handlerRefcon);

	AEDesc theData;
	XMPError	error = noErr;
	
	TRY
	
	THROW_IF_ERROR(AEGetKeyDesc(theAppleEvent, keyAEData,
              typeWildCard, &theData));

	AEDesc theToken;
	OSErr err =  AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard,
			&theToken);
	if (err == noErr)
	{
		switch (theToken.descriptorType)
		{
			case typeEmbededPartProp:
			case typeFrameProp:
			case typeProperty:
				PropAccessor* embedPropAccessorObj
						= (PropAccessor*)GetSecondFourBytesFromDataHandle
								((XMPOSLToken*)&theToken);
					embedPropAccessorObj->SetData(&theData);
				delete embedPropAccessorObj;
				break;
			
			default:
				ASSERTM(false, errAECantSupplyType, "You shouldn't be here");
			
		}
	}
	AEDisposeDesc(&theData);
	THROW_IF_ERROR(err);

	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// HandleSetData()


//------------------------------------------------------------------------------
// HandleGetData
//------------------------------------------------------------------------------

static XMPError HandleGetData(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
XMPUnused(thisPart);
XMPUnused(handlerRefcon);

	DescType		reqType;
	AEDesc			theToken;
	Size			theSize;
	AEDesc			objectData;
	XMPError		result;
	XMPError		error;

	TRY

	result = AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard,
								&theToken);
	THROW_IF_ERROR(result);

			//	Next, get the requested return type, if it exists.
	result = AEGetParamPtr(theAppleEvent,keyAERequestedType,typeType,
					&reqType,(Ptr)&reqType,sizeof(reqType),&theSize);
	if (result == errAEDescNotFound)		// Not an error if return type is not found.
	{
		result = noErr;
		reqType = typeWildCard;
	}
	THROW_IF_ERROR(result);

	switch (theToken.descriptorType)
	{
#ifdef TO_BE_DELETED
		case typeEmbededPartProp:
			EmbeddedPartPropAccessor* embedPropAccessorObj
					= (EmbeddedPartPropAccessor*)GetSecondFourBytesFromDataHandle
							((XMPOSLToken*)&theToken);
			embedPropAccessorObj->GetData(&objectData); //, reqType);
			delete embedPropAccessorObj;
			break;
		
#endif // TO_BE_DELETED

		case typeEmbededPartProp:
		case typeFrameProp:
		case typeProperty:
			PropAccessor* propAccessorObj
					= (PropAccessor*)GetSecondFourBytesFromDataHandle
							((XMPOSLToken*)&theToken);
			propAccessorObj->GetData(&objectData); //, reqType);
			delete propAccessorObj;
			break;
		default:
			ASSERTM(false, errAECantSupplyType, "You shouldn't be here");
		
	}
	
	result = AEPutParamDesc(reply, keyAEResult, &objectData);
	THROW_IF_ERROR(result);

//	delete propAccessorObj;

	CATCH_ALL
		error = ErrorCode();
	ENDTRY
	
	return error;
}	// HandleGetData()

//------------------------------------------------------------------------------
// HandleCreate
//------------------------------------------------------------------------------

// This guy translates the name of the user's new part into a class that
// we can actually use.  Or at least I think that's how it will work.

static DescType Partstring2Type(AEDesc* thePartType)
{
	DescType result = cDrawPart;		// default (for now)

	if(thePartType->dataHandle != NULL)
	{
		HLock(thePartType->dataHandle);
		short len = (short)GetHandleSize(thePartType->dataHandle);
		
		char* drawpart = "drawpart";
		char* dragpart = "dragpart";
		char* clockpart = "clockpart";
		char* testpart = "testpart";
		if (IUMagString(*thePartType->dataHandle, drawpart, len,
				strlen(drawpart)) == 0)
			result = cDrawPart;
		else if (IUMagString(*thePartType->dataHandle, dragpart, len,
				strlen(dragpart)) == 0)
			result = cDragPart;
		else if (IUMagString(*thePartType->dataHandle, clockpart, len,
				strlen(clockpart)) == 0)
			result = cClockPart;
		else if (IUMagString(*thePartType->dataHandle, testpart, len,
				strlen(testpart)) == 0)
			result = cTestPart;
			
		AEDisposeDesc(thePartType);
	}
	return result;

}	// Partstring2Type()

static XMPError HandleCreate(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
XMPUnused(thisPart);
XMPUnused(reply);
XMPUnused(handlerRefcon);

	XMPError		result;
	DescType		theType;
	DescType		theClass;
	XMPSize			theSize;
	AEDesc			replyObject;
	XMPPart* 		theNewPart;
	XMPError	error = noErr;

	TRY

		//	First, get the object class -- a required parameter
	result = AEGetParamPtr(theAppleEvent, keyAEObjectClass, typeType, &theType, 
					(Ptr)&theClass, sizeof(DescType), (Size *)&theSize);
	THROW_IF_ERROR(result);

	if(theClass != cPart)
		THROW(errAENoSuchObject);
	
	AEDesc thePartType;
	ThrowIfNotAbsent(AEGetParamDesc(theAppleEvent, typeType, typeChar,
								&thePartType));
	DescType partKind = Partstring2Type(&thePartType);
	
		//	Next, get the insertion location -- a required parameter
	AEDesc theLocation;
	result = AEGetParamDesc(theAppleEvent, keyAEInsertHere, typeInsertionLoc, 
					&theLocation);
	if (result != errAEDescNotFound)
		THROW_IF_ERROR(result);

		// Create the new part
	theNewPart=((DrawPart*)thisPart)->CreateNewPart(partKind, &theLocation);
	
		// have the new part create an object specifier for itself
	AEDesc containerObjSpec;
	THROW_IF_ERROR(AEGetAttributeDesc(theAppleEvent, 'subj',
			typeObjectSpecifier, &containerObjSpec));
	CreatePartObjectSpec(formAbsolutePosition, &containerObjSpec, &replyObject);
	AEDisposeDesc(&containerObjSpec);

		// put the object spec in the result field of reply
	result = AEPutParamDesc(reply, keyAEResult, &replyObject);
	AEDisposeDesc(&replyObject);
	THROW_IF_ERROR(result);

	// <eeh> But I want it to stick around!!!!
	// delete theNewPart;
	// XMPReleaseObject(theNewPart);

	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// HandleCreate()

//------------------------------------------------------------------------------
// HandleMove
//
// ALL THIS HANDLES right now is moving a part's frame[s], though I've
// never tested it on any part with more than one frame!
//------------------------------------------------------------------------------

static void MoveFacetTransform(XMPFacet* facet, Point* desiredLocation,
		XMPPart* part)
{
	XMPTransform* newXForm = new XMPTransform;
	newXForm->CopyFrom(facet->GetExternalTransform());
	Point curOffset = newXForm->GetQDOffset();
	Point scratch = *desiredLocation;
	scratch.h -= curOffset.h;
	scratch.v -= curOffset.v;

	XMPPoint newDelta = *desiredLocation = scratch;
	newXForm->MoveBy(newDelta);
	facet->ChangeGeometry(kXMPNULL, newXForm);
	facet->Invalidate(facet->GetClipShape());
}

static XMPError HandleMove(XMPPart*	thisPart,
					AppleEvent*	theAppleEvent,
					AppleEvent*	reply,
					XMPSLong	handlerRefcon)
{
	XMPUnused(handlerRefcon);
	XMPUnused(reply);

	DrawPart* thisDrawPart = (DrawPart*) thisPart;
	
	// Size theSize;
	// DescType theType;
	XMPError	error = noErr;

	TRY

	AEDesc destination;		// where it's to be moved to
	OSErr result = AEGetParamDesc(theAppleEvent, keyAEInsertHere, typeWildCard,
			&destination);
	THROW_IF_ERROR(result);
	
	XMPOSLToken theToken;		// what I'm supposed to move
	THROW_IF_ERROR(AEGetParamDesc(theAppleEvent, keyDirectObject, typeWildCard,
			(AEDesc*)&theToken));
	WASSERT(((AEDesc*)&theToken)->descriptorType == cPart);

	if (destination.descriptorType == typeObjectSpecifier)	// a path
	{
		XMPOSLToken token;
	
		XMPMacNameResolver* resolver = ((XMPSession*)handlerRefcon)->
				GetNameResolver();
		// start resolution at the shell
		resolver->Resolve(&destination, &token, kXMPAppShell);

		// now replace the location with the token, and pass the
		// event to that part.  should I dispose of the token after
		// putting it in the AppleEvent?
		
		THROW_IF_ERROR(AEPutParamDesc(theAppleEvent, keyAEInsertHere, 
				(AEDesc*)&token));
		XMPPart* embedder;
		XMPFrame* ignoreF;
		resolver->GetContextInfo(&ignoreF, &embedder);
		XMPMessageInterface* msgInterface = ((XMPSession*)handlerRefcon)->
				GetMessageInterface();
		THROW_IF_ERROR(msgInterface->DispatchToEventHandler(theAppleEvent,
				reply, embedder));

	}
	else if (destination.descriptorType == cPart)
	{
		// Here the destination is a token belonging to me.  As the
		// embedding part (the last to deal with the token before the
		// original recipient of the event finished resolving it) I
		// own the destination and can get out of the token what I
		// need.
		
		XMPFrame* frameToMoveTo = (XMPFrame*)
				GetSecondFourBytesFromDataHandle((XMPOSLToken*)&destination);
		XMPFrame* frameToMove
				= (XMPFrame*)GetSecondFourBytesFromDataHandle(&theToken);


		// get some info for the embed call
		XMPShape* frameShape = new XMPShape;
		frameShape->CopyFrom(frameToMove->GetFrameShape());

		XMPTransform* xtrnlXfrm = new XMPTransform;
		XMPFrameFacetIterator* fi = frameToMove->CreateFacetIterator();
		XMPFacet* facet = fi->First();
		xtrnlXfrm->CopyFrom(facet->GetExternalTransform());


		// remove the frame from the old place
		frameToMove->GetContainingFrame()->GetPart()->
				RemoveEmbeddedFrame(frameToMove);

		// embed it in the new

		WASSERT(thisPart == frameToMoveTo->GetPart());

		XMPFrame* newFrame = thisDrawPart->CreateEmbeddedFrame(
												frameToMoveTo,
												frameShape,
												xtrnlXfrm,
												frameToMove->GetPart(),
												kXMPNullTypeToken,
												kXMPNullTypeToken,
												0,
												kXMPFalse);
	}
	else
	{
		// get the point
		AEDesc dPoint;
		result = AECoerceDesc(&destination, typeQDPoint, &dPoint);
		THROW_IF_ERROR(result);
		Point pt = **(Point**)dPoint.dataHandle;
		AEDisposeDesc(&dPoint);

		XMPFrame* frameToMove
				= (XMPFrame*)GetSecondFourBytesFromDataHandle(&theToken);
		XMPFrameFacetIterator* fi = frameToMove->CreateFacetIterator();
		XMPFacet* facet = fi->First();
		MoveFacetTransform(facet, &pt, frameToMove->GetPart());

		Proxy* p = thisDrawPart->ProxyForFrame(frameToMove);

		// !!! this is a single-facet hack
		p->transform->MoveBy(pt);		
		thisDrawPart->UpdateProxyRegion(p);
		thisDrawPart->CreateProxySelectionBorder(p);

		frameToMove->GetContainingFrame()->Invalidate((XMPShape*)kXMPNULL);
	}

	AEDisposeDesc(&destination);

	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}	// HandleMove()

//------------------------------------------------------------------------------
// Coercelist2RGB
//
//	From the AppleScript code and tweaked.
//------------------------------------------------------------------------------

#define QDRGBsize			3

XMPError CoerceList2RGB(XMPPart* thisPart, AEDesc* from, DescType toType,
					XMPSLong refcon, AEDesc* result)
{
XMPUnused(thisPart);
XMPUnused(refcon);

	OSErr err;
	AEKeyword dummyKeyword;
	DescType dummyCode;
	Size itemSize;
	long theData[QDRGBsize];
	unsigned short theRealData[QDRGBsize];
	XMPError	error = noErr;

	TRY

	// the data goes R, G, B
	for (int i=0; i<QDRGBsize; i++) {
		err = AEGetNthPtr(from, i+1, typeLongInteger, &dummyKeyword,
						  &dummyCode, (Ptr) &theData[i], sizeof(theData[0]),
						  &itemSize);
		if ((err != noErr) || (theData[i] < 0) || (theData[i] > USHRT_MAX))
			THROW(errAECoercionFail);
		theRealData[i] = (unsigned short)theData[i];
	}
	THROW_IF_ERROR(AECreateDesc(toType, (Ptr)&theRealData,
								QDRGBsize * sizeof(short), result));
	
	CATCH_ALL
	error = ErrorCode();
	ENDTRY
	
	return error;
}

//------------------------------------------------------------------------------
// DrawCompare
// For now, just handle what we need to handle
//------------------------------------------------------------------------------
static XMPError DrawCompare(	XMPPart*			thePart,
							DescType			oper,
							XMPOSLToken*	obj1,
							XMPOSLToken*	obj2,
							XMPBoolean*			result,
							XMPSLong			refCon)
{
	XMPUnused(thePart);
	XMPUnused(obj1);
	XMPUnused(result);
	XMPUnused(refCon);

	XMPError	error = noErr;

	switch(oper)
	{
		case kAEEquals:
			if (((AEDesc*)obj2)->descriptorType == typeChar)
				break;
		default:
			error = errAEEventNotHandled;
	}
	
	return error;
}

//------------------------------------------------------------------------------
// DrawCount
//------------------------------------------------------------------------------
static XMPError DrawCount(		XMPPart*			thePart,
							DescType			desiredType,
							DescType			containerClass,
							XMPOSLToken*	container,
							XMPSLong*			result,
							XMPSLong			refCon)
{
	XMPUnused(thePart);
	XMPUnused(desiredType);
	XMPUnused(containerClass);
	XMPUnused(container);
	XMPUnused(result);
	XMPUnused(refCon);

	XMPError	error = noErr;

	TRY

	if (containerClass == cPart)
	{
		XMPNameResolver* resolver = ((XMPSession*)refCon)->GetNameResolver();
	
		XMPPart* part
				= (XMPPart*)GetSecondFourBytesFromDataHandle(container);
		// resolver->CreateSwapToken(value, part);
	}
	
	CATCH_ALL
		error = ErrorCode();
	ENDTRY

	return error;
}

//------------------------------------------------------------------------------
// DrawDisposeToken
//------------------------------------------------------------------------------
static XMPError DrawDisposeToken(XMPPart*			thePart,
							XMPOSLToken*	unneededToken,
							XMPSLong		refCon)
{
	XMPUnused(thePart);
	XMPUnused(unneededToken);
	XMPUnused(refCon);
	return errAEEventNotHandled;
}

#undef XMPOJBECT_ACCESSOR_PARAMS
