/*
	File:		DrawPart.cpp

	Contains:	implementation of class DrawPart

	Written by:	Joshua Susser & Tantek elik

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<18>	 4/13/94	SS		#1144419, 1144417: Fixes to support
									laserwriter printing/pagesetup - DrawPart
									now uses FocusLib for drawing/printing
		<17>	 4/12/94	VL		1143573, 1153424: StartDrag() now returns
									XMPDropResult which indicates copy or move
									or fail.
		<16>	  4/7/94	JA		Made some string consts static to fix link
									problems (1153437)
		<15>	  4/6/94	JBS		1155477
		<14>	  4/1/94	T		#1154941: Get OpenDoc source to actually
									compile for PowerPC
		<13>	 3/31/94	eeh		bug #1153999: change use of PStrToIntl.
		<12>	 3/28/94	T		#1145834.  Remove PrtType.h & .cpp from the
									project
		<11>	 3/28/94	CG		1153547: include XMPSessM.h instead of
									XMPSessn.h.
		<10>	 3/27/94	T		#1153526 fix DrawPart to ShowPartFrameInfo
									on itself when nothing is selected
									#1153528 fix DrawPart to name child windows
									using part name
		 <9>	 3/26/94	NP		1153504. InitSemanticInterface.
		 <8>	 3/25/94	CC		Replaced kXMPDropSucceed with kXMPDropCopy
									(1153424); use
									XMPDragAndDrop::GetContentStorageUnit()
									(1153043).
		 <7>	 3/25/94	JA		JPA: GetPlatformTransform -> GetQDOffset
									(1153438). JBS: Changed API of
									CreateEmbeddedFrameIterator, CanvasChanged
									(1153056, 1153059).
		 <6>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <5>	  3/1/94	JA		Made string consts static.
		 <4>	 2/17/94	VL		#1142972, #1144024: Enabled receiving a
									Drop from the Finder.
		 <3>	 2/17/94	JBS		#114???? fix inconsistent facet coords
		 <2>	 2/16/94	JA		Include new AltPoint.h.
		<89>	  2/8/94	JA		Added missing #includes.
		<88>	  2/8/94	T		Throw -> THROW & some code clean up
		<87>	  2/7/94	JA		Utility.h --> XMPUtils.h
		<86>	  2/7/94	T		fixes to compile with PPC Headers
		<85>	  2/3/94	CG		System Session break out.  Added skeleton
									for Copy and Paste.
		<84>	  2/2/94	VL		Removed standin frame stuff. Used
									RemoveEmbeddedFrame in RemoveDisplayFrame.
		<83>	  2/2/94	RR		Added isRoot to CreateFrame
		<82>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame();
									added AttachSourceFrame(); new
									AddDisplayFrame recipie
		<81>	  2/1/94	SS		Support for view/presentation + printing to
									QD printers
		<80>	  2/1/94	CC		Added XMPCloneKind argument to BeginClone;
									commented out call to RemovePart() after a
									drag.
		<79>	 1/27/94	RR		Added CreateWindow. Reworked ::Open
		<78>	 1/27/94	VL		Changed XMPPart::CloneTo to
									XMPPart::CloneInto. Used a stand-in frame
									to ensure that embedded frames can find
									their containing frame after Clone.
		<77>	 1/25/94	RR		Enable Open. ViewAsWindow on active part.
									Call window->Open,Show,Select
		<76>	 1/21/94	RR		Added shouldSave to CreateWindow
		<75>	 1/20/94	SS		::Open returns a window id
		<74>	 1/20/94	SS		Added Frame ViewType support and icons
		<73>	 1/19/94	NP		Added some undo support.
		<72>	 1/15/94	VL		Beefed up the destructor. Delete facet
									collection.
		<71>	 1/15/94	RR		WinState.h->WinStat.h,
									Dispatch.h->Disptch.h
		<70>	 1/15/94	RR		Updated HandleMenuEvent (command no. is no
									longer in event)
		<69>	 1/14/94	NP		Init changes.
		<68>	 1/13/94	JBS		delete facet after removing
		<67>	 1/12/94	RR		Remove window->Release calls and
									RemoveFrame calls.
		<66>	 1/11/94	T		Init... changes
		<65>	12/23/93	RR		Call InitFocusSet
		<64>	12/21/93	VL		Changed XMPStorageUnit::GetValueSize to
									StorageUnit::GetSize.
		<63>	12/21/93	JBS		CreateFrame with viewType & presentation,
									added ViewTypeChanged() &
									PresentationChanged(), renamed
									FrameClosed() to CloseDisplayFrame()
		<62>	12/20/93	VL		Used WindowState to CreateWindow.
		<61>	12/17/93	VL		Fixed parameter to NewCWindow. Set
									ShouldSave flag for Palette to false.
		<60>	12/16/93	T		CreatePart change
		<59>	12/15/93	JBS		fix FrameClosed
		<58>	12/14/93	T		remove call to SimplPrt::Initialize()
									inside CommonInitDrawPart
		<57>	12/14/93	T		renamed CommonInit to CommonInitDrawPart
		<56>	12/13/93	T		InitPart changes
		<55>	12/13/93	RR		Removed accidental test of redispatching.
		<54>	12/13/93	EL		Remove the reference to the embedded
									container when they are removed.
		<53>	 12/9/93	eeh		added params to Embed
		<52>	 12/8/93	JBS		changed references to "qd" to
									XMPASLMQDGlobals
		<51>	 12/8/93	VL		Fixed ViewAsWindow problem by making the
									platform window invisible and calling
									Select and Open on the  XMPWindow.
		<50>	 12/7/93	NP		Changed MouseLeave to use new QDGlobals
									macro.
		<49>	 12/6/93	T		make CFront warnings go away
		<48>	 12/6/93	RR		Added hiding of palette in Suspend/Resume
		<47>	 12/3/93	T		Stop including XMPError.h, it is included
									as ErrorDef.h inside Except.h
		<45>	 12/2/93	RR		For ASLM, use InitCursor instead of
									SetCursor(&qd.arrow)
									Use new events for "embedded clicks"
		<44>	 12/1/93	CG		Added CountEmbeddedFrames.
		<43>	 12/1/93	VL		Added floating palette.
		<42>	11/30/93	LM		Added an XMPFrame parameter to ReadPartInfo
		<41>	11/24/93	RR		Implement Part Activation
		<40>	11/23/93	T		modifications to use the correct part
									storage model
		<39>	11/23/93	JA		Updated for new Shape API.
		<38>	11/19/93	PH		Add cast for CFront
		<37>	11/19/93	RR		Added command numbers for all colors. Added
									checking of the color menus.
		<36>	11/19/93	RR		Use correct event constants. Added some
									skeleton code to HandleEvent for events in
									title bar
		<35>	11/18/93	T		add support for Get Part Info
		<34>	11/18/93	T		call inherited::InitializePersistent()
									::Externalize()
		<33>	11/17/93	SS		Fixed drop placement of parts
		<32>	11/17/93	RR		In ::Open, call SetShouldSave
		<31>	11/16/93	SS		Fixed transformation of drag outline for
									nested frames
		<30>	11/16/93	EL		IDOrderList should be IDOrderedList.
		<29>	11/16/93	SS		Reactivated DragPart
		<28>	11/16/93	EL		Use OpenDoc: instead of XMP.
		<27>	11/16/93	CG		Moved DrawPart Object accessors to
									DrawObj.cp and DrawObj.h.
		<26>	11/16/93	RR		Added AdjustMenus (not yet called), and
									MouseWithin. Removed point from MouseLeave
		<25>	11/12/93	JBS		use Facets
		<24>	 11/2/93	CG		backed out of a check-in for Joshua.
		<22>	 11/1/93	VL		Replaced CloneFrom with BeginClone, CloneTo
									and EndClone. Also, changed Internalize to
									handle bogus references.
		<21>	10/28/93	RR		Added missing break; in switch statement.
		<20>	10/27/93	RR		Added MouseEnterFrame/MouseLeaveFrame
		<19>	10/27/93	SS		Rewrote ClipEmbeddedFrames, cleaned up Draw
									method, changed ordering of internal
									collections to be AddFirst (was AddLast)
		<18>	10/25/93	PH		Fix SUView problems
		<17>	10/22/93	SS		Fixed odd behavior in
									HandleMouseDownCornerResize and fixed
									casting problems for ASLM builds
		<16>	10/21/93	PH		Change FulfillPromise
		<15>	10/21/93	SS		Rewrote "HandleMouseDown", added support
									for edge resize handles, and made all
									handles match HI spec
		<14>	10/21/93	RR		CreateWindow takes new flags
		<13>	10/20/93	PH		Only read/write partinfo when needed
		<12>	10/20/93	SS		Rewrote invalidation code in
									HandleMouseDown, added ClipEmbeddedFrames
									to a couple methods, and fixed a problem
									created by new Get/CopyQDRegion model
		<11>	10/15/93	NP		Cast so CFront compile would work.
		<10>	10/14/93	RR		Implement Open, Improve AddDisplayFrame.
									Use command number for View As Window. Add
									frame parameter to ClipEmbeddedFrames
									Added fDefaultColor, shared by all display
									frames
		 <9>	10/14/93	JA		XMPShape fu: No longer returns copies from
									GetPlatformShape. Deal w/it.
		 <8>	10/12/93	PH		Rewrite drop code again
		 <7>	 10/8/93	T		make most not used warnings go away
		 <6>	 10/8/93	VL		Added Release.
		 <5>	 10/8/93	T		call SetChangedFromPrev when content changes
		 <4>	 10/8/93	T		remove StylPart from Embed Menu, reorder
									Embed menu
		 <3>	 10/6/93	PH		Fix drop xform code
		 <2>	 10/6/93	T		remove obsolete parts (StylPart)
		<95>	 10/6/93	JA		Use new fixed-point XMPPoint.
		<94>	 10/5/93	T		use fFocusSet everywhere instead of
									building up focusSet from scratch every
									time
		<93>	 10/5/93	T		added #ifdef SSREMOVEWORKAROUND until
									su->Remove() is fixed
		<92>	 10/4/93	PH		Fix drop code again
		<91>	 10/1/93	RR		Fixed Itsy witsy bug in HandleEvent
		<90>	 9/30/93	PH		Support new Drag and Drop protocol
		<89>	 9/30/93	RR		Added fFocusSet
		<88>	 9/29/93	T		Gets the selection, menu, kbd foci inside
									AddDisplayFrame if the frame is the root
									frame
		<87>	 9/29/93	VL		Added InitializePersistent.
		<86>	 9/27/93	PH		Fix externalize and tweak Drop code
		<85>	 9/24/93	PH		Use new content properties
		<84>	 9/23/93	JA		Use new convenience methods from Shape and
									Transform.
		<83>	 9/22/93	JA		Updated ASSERTs for new macro.
		<82>	 9/22/93	RR		Release menu bar, instead of deleting it
		<81>	 9/21/93	PH		Use new Frame calls to setDroppable and
									SetDragging
		<80>	 9/20/93	NP		Added RGB coercion handler.
		<79>	 9/16/93	RR		Menu event uses command numbers
		<78>	 9/16/93	JA		Dispose rgns returned by GetPlatformShape.
									Don't use obsolete transform MoveBy method.
		<77>	 9/15/93	RR		Update menu code
		<77>	 9/15/93	RR		Moved GetBaseMenuBar to CommonInit
		<76>	 9/14/93	NP		Factored common init code for Initialize
									and Internalize into CommonInit routine.
		<75>	 9/10/93	RR		Moved GetBaseMenuBar back to constructor
									for now, because Initialize is not called
									during internalization.
		<74>	 9/10/93	PH		Use global coordinates for StartDrag
		<73>	 9/10/93	NP		Added accessors to set color.
		<72>	 9/10/93	RR		Use new menu bar calls
		<71>	  9/7/93	PH		Add code to create a DragPart
		<70>	  9/3/93	NP		Added semantic interface.
		<69>	 8/31/93	PH		Use FrameSHape rather than ClipShape in
									Drop
		<68>	 8/23/93	PH		Add drag handles
		<67>	 8/23/93	PH		Clean up in/externalize
		<66>	 8/20/93	PH		Fix refcounting
		<65>	 8/19/93	PH		Use random color in AddDisplayFrame
		<64>	 8/17/93	PH		Fix ref counting problems in Drop
		<63>	 8/13/93	RR		Use OrdColl.h insteadl of Collect.h
		<62>	 8/10/93	PH		Call release on frame rather than on the
									draft
		<61>	  8/9/93	PH		Create storage unit cursor for drag
		<60>	  8/3/93	PH		Have RemoveEmbeddedFrame call Clip
		<59>	  8/3/93	PH		Call removeFrame in RemoveEmbeddedFrame
		<58>	 7/30/93	SES		Added TestPart to Embed menu
									w/implemetation
		<57>	 7/29/93	VL		Fixed parameters to Get calls on persistent
									objects.
		<56>	 7/28/93	PH		DisposeRgn after StartDrag
		<55>	 7/28/93	PH		Move menu setup to constructor
		<54>	 7/21/93	PH		Fix leak in Draw method
		<53>	 7/20/93	PH		Fix menus
		<52>	 7/16/93	PH		Remove extra values after externalizing
		<51>	 7/15/93	PH		Rewrite Drop method
		<50>	 7/13/93	PH		Always set/restore port in HandleMenuEvent
		<49>	 7/13/93	PH		Fix GetOrderedProxyCollection to init to
									the proper regions.
		<48>	 7/12/93	PH		Externalize and internalize fDisplayFrames
		<47>	 7/12/93	PH		Fix ReadPartInfo
		<46>	  7/8/93	PH		Add SetPort call before changing background
									color
		<45>	 6/29/93	PH		Fix FrameShapeChanged
		<44>	 6/29/93	RCR		#include Draft.h
		<43>	 6/28/93	PH		Register droppable frames!
		<42>	 6/23/93	RCR		Use tokens for foci
		<41>	 6/15/93	RCR		Compare foci using XMPISOStrCompare
		<40>	  6/8/93	RCR		Use XMPISOStrCompare for foci 
		<39>	  6/8/93	RCR		RelinquishFocusSet now takes a frame instead of a part 
		<38>	 5/26/93	RCR		Update active and used shapes in FrameShapeChanged
		<37>	 5/21/93	JBS		update for namesmith changes
		<36>	 5/20/93	RCR		Namesmith
		<35>	 5/13/93	JBS		fix for imaging namesmith changes
		<34>	  5/7/93	JBS		remove some embedded part types
		<33>	  5/4/93	NP		Updated for new interfaces.
		<32>	 4/29/93	JBS		8.3 file names
		<31>	 4/28/93	JBS		8.3 file names
		<26>	 4/16/93	RCR		Added Frame Freeze
		<25>	 4/15/93	T		do intra document inter draw part drag and
									drop
		<24>	 4/13/93	SES		Added "StylPart" to Embed menu
		<21>	 4/12/93	TI		write out frame shape and presentation info
									to drag and drop storageUnit
		<20>	 4/11/93	SES		Fixed embedded frame resizing
		<19>	  4/9/93	SES		Added rubber-banding with frame resizing
		<18>	  4/8/93	JBS		fix dragging
		<17>	  4/7/93	JBS		add Drag&Drop and frame resizing - needs work
		<16>	  4/7/93	JBS		garbage check-in
		<15>	  4/6/93	TI		comment out ExternalizeContent and just use default behaviour
		<14>	  4/6/93	JBS		add SetGrafPortOrigin(); fix drawing
									transformation
		<13>	  4/6/93	JBS		add ClipShapeChanged()
		<12>	  4/5/93	TI		implement Initialize
		<11>	  4/5/93	SES		Added ClockPart to Embed menu
		<10>	  4/3/93	JBS		add Proxy and Selection handling calls
		 <9>	  4/2/93	TI		Read/WritePartInfo only 'ternalizes the RGBColor
		 <8>	  4/1/93	JBS		merge changes - clipping of embedded frames
		 <7>	  4/1/93	TI		implement Read/WritePartInfo, Internalize, Externalize
		 <6>	 3/29/93	SF		(1) Changed sequence of MoveBy and AddDisplayFrame needed for QT
									frame computation. NP (2) removed uninitalized
									"externalTransform" SF.
		 <5>	 3/29/93	NP		Added MoviePart back in.
		 <4>	 3/29/93	RCR		Fixed bug in ::Draw. Shape was not allocated.
		 <3>	 3/29/93	RCR		Help it to compile
		 <2>	 3/26/93	JBS		more changes...
		 <1>	 3/23/93	JBS		first checked in
		 <0>	 3/23/93	JBS		created
*/

#ifndef _ALTPOINT_
#include "AltPoint.h"			// Use C++ savvy XMPPoint and XMPRect
#endif


#ifndef __EVENTS__
#include <Events.h>		// GetMouse, StillDown.
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __ICONS__
#include <Icons.h>
#endif

#ifndef __PICKER__
#include <Picker.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>	// HiWord etc.
#endif

#ifndef __LIMITS__
#include <limits.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>		// Abs
#endif

#ifndef __LIBRARYMANAGERUTILITIES__
#include <LibraryManagerUtilities.h>
#endif

#ifndef __LIBRARYMANAGERCLASSES__
#include <LibraryManagerClasses.h>
#endif

#ifndef _DRAWPART_
#include "DrawPart.h"
#endif

#ifndef _FOCI_
#include "Foci.h"
#endif

#ifndef _FOCUSSET_
#include "FocusSet.h"
#endif

#ifndef _CMDDEFS_
#include "CmdDefs.h"
#endif

#ifndef _ARBITRAT_
#include "Arbitrat.h"
#endif

#ifndef _ISOSTRING_
#include "ISOStr.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _DISPTCH_
#include "Disptch.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _XMPMEM_
#include "XMPMem.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

#ifndef _STDPROPS_
#include "StdProps.h"
#endif

#ifndef _STDTYPES_
#include "StdTypes.h"
#endif

#ifndef _SUVIEW_
#include "SUView.h"
#endif

#ifndef _SUCURSOR_
#include "SUCursor.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _DOCUMENT_
#include "Document.h"
#endif

#ifndef _XMPCTR_
#include "XMPCtr.h"
#endif

#ifndef _STORAGE_
#include "Storage.h"
#endif

#ifndef _XMPSESSM_
#include "XMPSessM.h"
#endif

#ifndef _INFO_
#include "Info.h"
#endif

#ifndef _MENUBAR_
#include "MenuBar.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _WINSTAT_
#include "WinStat.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

#ifndef _NAMRSLVR_
#include "NamRslvr.h"
#endif

#ifndef _SEUTILS_
#include "SEUtils.h"
#endif

#ifndef _PRTPRPAC_
#include "PrtPrpAc.h"
#endif

#ifndef _STDPROPS_
#include "StdProps.h"
#endif

#ifndef _DRAGDROP_
#include "DragDrop.h"
#endif

#ifndef _DGITMITR_
#include "DgItmItr.h"
#endif

#ifndef _STDDEFS_
#include "StdDefs.h"
#endif

#ifndef _CANVAS_
#include "Canvas.h"
#endif

#ifndef _XMPUTILS_
#include "XMPUtils.h"
#endif

#ifndef _PASCLSTR_
#include "PasclStr.h"
#endif

#ifndef _UNDO_
#include "Undo.h"
#endif

#ifndef _CLIPBD_
#include "Clipbd.h"
#endif

#ifndef _FOCUSLIB_
#include "FocusLib.h"
#endif

#ifndef mathRoutinesIncludes
#include <math routines.h>		// See MoveTransformBy
#endif

#pragma segment DrawPart


//==============================================================================
// Local functions
//==============================================================================

XMPBoolean ValueOnClipboard(XMPSession* session);

//==============================================================================
// Constants
//==============================================================================

	const XMPSShort kXMPBorderWidth = 5;
	const XMPSShort kXMPHandleLenMultiplier = 3;

	static const RGBColor rgbGray =	{ 0x7FFF, 0x7FFF, 0x7FFF };
	static const RGBColor rgbRed =		{ 0xBFFF, 0x7FFF, 0x7FFF };
	static const RGBColor rgbGreen =	{ 0x7FFF, 0xBFFF, 0x7FFF };
	static const RGBColor rgbYellow =	{ 0xBFFF, 0xBFFF, 0x7FFF };
	static const RGBColor rgbBlue =	{ 0x7FFF, 0x7FFF, 0xBFFF };
	static const RGBColor rgbMagenta =	{ 0xBFFF, 0x7FFF, 0xBFFF };
	static const RGBColor rgbCyan =	{ 0x7FFF, 0xBFFF, 0xBFFF };
	static const RGBColor rgbWhite =	{ 0xFFFF, 0xFFFF, 0xFFFF };
	static const RGBColor rgbBlack =	{ 0x0000, 0x0000, 0x0000 };

	const XMPSShort kPaletteLeft = 0;
	const XMPSShort kPaletteRight = 50;
	const XMPSShort kPaletteTop = 0;
	const XMPSShort kPaletteBottom = 200;
	
	enum {
		kGray = 0,
		kRed,
		kGreen,
		kYellow,
		kBlue,
		kMagenta,
		kCyan,
		kWhite,
		kNumColors
	};

	const		XMPSShort	kMargin = 10;

const short 	kSuspendResumeMessage = 0x01;	// Resume vs. suspend mask
const short 	kMouseMovedMessage    = 0xFA;	// High byte mouse-moved event message

// $$$$$ STATIC BUILD CRUFT
// $$$$$ EMBED MENU CRUFT.  SHOULD NOT BE HARDCODING OTHER PART KINDS INTO CODE.
static const XMPType kXMPPartTogglePart        = "TogglePart";
static const XMPType kXMPPartButtonPart        = "ButtonPart";
static const XMPType kXMPPartTextPart          = "TextPart";
static const XMPType kXMPPartDrawPart          = "Apple:Kind:TestDraw";
static const XMPType kXMPPartMoviePart         = "MoviePart";
static const XMPType kXMPPartClockPart         = "Apple:Kind:TestClock";
static const XMPType kXMPPartStylPart          = "StylPart";
static const XMPType kXMPPartTestPart          = "TestPart";
static const XMPType kXMPPartDragPart          = "Apple:Kind:TestDrag";
static const XMPType kXMPPartPartsBin          = "Apple:Kind:PartsBin";

//==============================================================================
// Commands
//==============================================================================
	
	const XMPCommandID cXMPGray = 2001;
	const XMPCommandID cXMPRed = 2002;
	const XMPCommandID cXMPGreen = 2003;
	const XMPCommandID cXMPYellow = 2004;
	const XMPCommandID cXMPBlue = 2005;
	const XMPCommandID cXMPMagenta = 2006;
	const XMPCommandID cXMPCyan = 2007;
	const XMPCommandID cXMPWhite = 2008;
	const XMPCommandID cXMPOtherColor = 2009;

	
//==============================================================================
// 'ternalization 
//==============================================================================
		
	static const XMPPropertyName	kXMPPropFrameInfo = "DrawPart:Property:FrameInfo";
	static const XMPPropertyName	kXMPPropMouseDownOffset = "DrawPart:Property:MouseDownOffset";
	
	static const XMPValueType		kQDPoint = "DrawPart:Type:Point";
	static const XMPValueType		kXMPMacTPrintRec = "Macintosh:Type:TPrint Record";
	
	static const XMPType			kDrawPresNormal =  "DrawPart:Presentation:Normal";
	static const XMPType			kDrawPresPalette = "DrawPart:Presentation:Palette";

//==============================================================================
// Function Prototype
//==============================================================================

XMPBoolean RGBEqual(RGBColor a,RGBColor b);	// Function Prototype for correctness

XMPBoolean RGBEqual(RGBColor a,RGBColor b)
{
	return (a.red == b.red &&
			a.green == b.green &&
			a.blue == b.blue);
}

XMPUShort DetermineNumberOfPagesinDoc(XMPFrame* frame, Rect page);

//==============================================================================
// DrawPart
//==============================================================================

//-------------------------------------------------------------------------
// DrawPart::DrawPart
//-------------------------------------------------------------------------

DrawPart::DrawPart()
{
	fDisplayFrames = kXMPNULL;
	fEmbeddedFrames = kXMPNULL;
	fWindowID = 0;

	fFrameGroupIDCounter = 1;

	fContents = kXMPNULL;
	fSelection = kXMPNULL;

	fSelectRgn = kXMPNULL;
	fCornerHandleRgn = kXMPNULL;
	fEdgeHandleRgn = kXMPNULL;

	fEmbedMenu = kXMPNULL;
	fColorMenu = kXMPNULL;
	fFrameMenu = kXMPNULL;
	fMenuBar = kXMPNULL;
	fFocusSet = kXMPNULL;

	fSemtIntf = kXMPNULL;
	fTestDrawSU = kXMPNULL;
	
	fPalette = kXMPNULL;
	fPaletteUp = kXMPFalse;
	fPaletteWasShown = kXMPFalse;
	
	fLargeIcons = kXMPNULL;
	fSmallIcons = kXMPNULL;

	fSession = kXMPNULL;
	
	fPrintRecord = kXMPNULL;
}

//  destructor 

DrawPart::~DrawPart()
{
	if (fContents != kXMPNULL)
		delete fContents;
	if (fSelection != kXMPNULL)
		delete fSelection;
	if (fSelectRgn) DisposeRgn(fSelectRgn);
	if (fCornerHandleRgn) DisposeRgn(fCornerHandleRgn);
	if (fEdgeHandleRgn) DisposeRgn(fEdgeHandleRgn);
	
	if (fMenuBar != kXMPNULL) {	
		fMenuBar->UnregisterCommand(cXMPGray);
		fMenuBar->UnregisterCommand(cXMPRed);
		fMenuBar->UnregisterCommand(cXMPGreen);
		fMenuBar->UnregisterCommand(cXMPYellow);
		fMenuBar->UnregisterCommand(cXMPBlue);
		fMenuBar->UnregisterCommand(cXMPMagenta);
		fMenuBar->UnregisterCommand(cXMPCyan);
		fMenuBar->UnregisterCommand(cXMPWhite);
		fMenuBar->UnregisterCommand(cXMPOtherColor);
		
		fMenuBar->RemoveMenu(4);
		fMenuBar->RemoveMenu(5);
		fMenuBar->RemoveMenu(6);
		
		fMenuBar->Release();
	}
	
	if (fEmbedMenu) DisposeMenu(fEmbedMenu);
	if (fColorMenu) DisposeMenu(fColorMenu);
	if (fFrameMenu) DisposeMenu(fFrameMenu);

	if (fDisplayFrames != kXMPNULL)
		delete fDisplayFrames;		// make sure it's empty first
	if (fEmbeddedFrames != kXMPNULL)
		delete fEmbeddedFrames;		// make sure it's empty first

	if (fSemtIntf != kXMPNULL)	
		delete fSemtIntf;
	
	XMPReleaseObject(fTestDrawSU);
}

//  Initialize 

void DrawPart::InitPart(XMPStorageUnit* storageUnit)	// Override
{
	if (fInitialized)
		return;
	
	SimplePart::InitPart(storageUnit);
	
	this->CommonInitDrawPart();
	
	XMPStorageUnit* msu = this->GetStorageUnit();
	msu->AddProperty(kXMPPropContents)->AddValue(kXMPKindTestDraw);	
	fTestDrawSU = msu->GetDraft()->CreateStorageUnit();

	fTestDrawSU->AddProperty(kXMPPropDisplayFrames)->AddValue(kXMPIDs);			
	fTestDrawSU->AddProperty(kXMPPropEmbeddedFrames)->AddValue(kXMPIDs);			
	fTestDrawSU->AddProperty(kXMPPropFrameGroup)->AddValue(kXMPULong);			
	fTestDrawSU->AddProperty(kXMPPropContents)->AddValue(kXMPIDs);
}

void DrawPart::InitPartFromStorage(XMPStorageUnit* storageUnit)	// Override
{
	if (fInitialized)
		return;
	
	SimplePart::InitPartFromStorage(storageUnit);
	
	this->CommonInitDrawPart();
	

	XMPStorageUnit* su;
 	XMPStorageUnitRef aSURef;
	XMPFrame* frame;
	XMPShape* newShape;
	XMPULong offset, offsetLimit;
	XMPRgnHandle newRegion;
	Point xform;
	XMPTransform* transform;

XMPUnused(newRegion);
XMPVolatile(frame);

	su = this->GetStorageUnit();
	su->Focus(kXMPPropContents,kXMPPosSame,kXMPKindTestDraw,1,kXMPPosFirstSib);
	su->GetValue(sizeof(XMPStorageUnitRef),&aSURef);
	fTestDrawSU = su->GetDraft()->GetStorageUnit(su->GetIDFromStorageUnitRef(aSURef));
	su = fTestDrawSU;

 	su->Focus(kXMPPropDisplayFrames, kXMPPosSame, 0, 1, kXMPPosFirstSib);
	offsetLimit = su->GetSize();
	for (offset = 0; offset < offsetLimit; offset += sizeof(XMPStorageUnitRef))
	{
		su->SetOffset(offset);
		su->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		
		TRY
		
			frame = su->GetDraft()->GetFrame(su->GetIDFromStorageUnitRef(aSURef));
			fDisplayFrames->AddLast((ElementType)frame);
			frame->SetDroppable(kXMPTrue);
			
		CATCH_ALL
		
			frame = kXMPNULL;
	
		ENDTRY
	}

/*
 	su->Focus(kXMPPropEmbeddedFrames, kXMPPosSame, 0, 1, kXMPPosFirstSib);
	offsetLimit = su->GetSize();
	for (offset = 0; offset < offsetLimit; offset += sizeof(XMPStorageUnitRef))
	{
		su->SetOffset(offset);
		su->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		frame = su->GetDraft()->GetFrame(su->GetIDFromStorageUnitRef(aSURef));
		fEmbeddedFrames->AddLast((ElementType)frame);
	}
*/

 	su->Focus(kXMPPropFrameGroup,kXMPPosSame,0,1,kXMPPosFirstSib);
	su->GetValue(sizeof(fFrameGroupIDCounter),(XMPValue)&fFrameGroupIDCounter);
	
 	su->Focus(kXMPPropContents,kXMPPosSame,0,1,kXMPPosFirstSib); 
	offsetLimit = su->GetSize();
	newShape = new XMPShape();
	for (offset = 0; offset < offsetLimit;)
	{
		su->SetOffset(offset);
		su->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		offset += sizeof(XMPStorageUnitRef);
		frame = su->GetDraft()->GetFrame(su->GetIDFromStorageUnitRef(aSURef));
		fEmbeddedFrames->AddLast((ElementType)frame);

		su->SetOffset(offset);
		su->GetValue(sizeof(XMPPlatformTransform), (XMPValue)&xform);
		offset += sizeof(Point);
		transform = new XMPTransform;
		transform->SetQDOffset(xform);
		
		newShape->CopyFrom(frame->GetFrameShape());
		newShape->Transform(transform);
		RgnHandle newRegion = newShape->CopyQDRegion();
		fContents->AddLast((ElementType)(new Proxy(newRegion, frame, transform)));
	}
	delete newShape;
}

void DrawPart::CommonInitDrawPart()
{
	fDisplayFrames = new OrderedCollection;
	fEmbeddedFrames = new OrderedCollection;

	RGBColor random;
	random.red = Random();
	random.green = Random();
	random.blue = Random();
	fDefaultColor = random;

	fContents = new OrderedCollection;
	fSelection = new OrderedCollection;
	
	fSelectRgn = NewRgn();
	fCornerHandleRgn = NewRgn();
	fEdgeHandleRgn = NewRgn();

	fEmbedMenu = NewMenu(4,"\pEmbed");
	fColorMenu = NewMenu(5,"\pBackground");
	fFrameMenu = NewMenu(6,"\pFrame");

	if (!fEmbedMenu || !fColorMenu || !fFrameMenu)
		DebugStr("\pError creating menus");
	
	fSession = this->/*XMPPersistentObject::*/GetStorageUnit()->GetSession();

	AppendMenu(fEmbedMenu,"\pDraw");
	AppendMenu(fEmbedMenu,"\pDrag Text");
	AppendMenu(fEmbedMenu,"\pTime");
	
	AppendMenu(fColorMenu,"\pGray");
	AppendMenu(fColorMenu,"\pRed");
	AppendMenu(fColorMenu,"\pGreen");
	AppendMenu(fColorMenu,"\pYellow");
	AppendMenu(fColorMenu,"\pBlue");
	AppendMenu(fColorMenu,"\pMagenta");
	AppendMenu(fColorMenu,"\pCyan");
	AppendMenu(fColorMenu,"\pWhite");
	AppendMenu(fColorMenu,"\pOther");

	AppendMenu(fFrameMenu,"\pFreeze");
	AppendMenu(fFrameMenu,"\pDefrost");
	
	fMenuBar = fSession->GetWindowState()->CopyBaseMenuBar();

	fMenuBar->AddMenuLast(4, fEmbedMenu, this);
	fMenuBar->AddMenuLast(5, fColorMenu, this);
	fMenuBar->AddMenuLast(6, fFrameMenu, this);
	
	fMenuBar->RegisterCommand(cXMPGray, 5, 1);
	fMenuBar->RegisterCommand(cXMPRed, 5, 2);
	fMenuBar->RegisterCommand(cXMPGreen, 5, 3);
	fMenuBar->RegisterCommand(cXMPYellow, 5, 4);
	fMenuBar->RegisterCommand(cXMPBlue, 5, 5);
	fMenuBar->RegisterCommand(cXMPMagenta, 5, 6);
	fMenuBar->RegisterCommand(cXMPCyan, 5, 7);
	fMenuBar->RegisterCommand(cXMPWhite, 5, 8);
	fMenuBar->RegisterCommand(cXMPOtherColor, 5, 9);
		
	fSelectionFocus = fSession->Tokenize(kXMPSelectionFocus);
	fMenuFocus = fSession->Tokenize(kXMPMenuFocus);
	fKeyFocus = fSession->Tokenize(kXMPKeyFocus);
	
	fFocusSet = new XMPFocusSet();
	fFocusSet->InitFocusSet();
	fFocusSet->Add(fSelectionFocus);
	fFocusSet->Add(fMenuFocus);
	fFocusSet->Add(fKeyFocus);
	
	fSemtIntf = new XMPSemanticInterface();
	fSemtIntf->InitSemanticInterface(this);
	InstallObjectAccessors();
	
	XMPASLMQDGlobals.randSeed = TickCount();
	
	this->LoadIcons();
}

void DrawPart::LoadIcons()
{
#ifdef ASLMBUILD
	XMPSLong		shellLibRefNum;
	XMPVolatile(shellLibRefNum);
	TLibraryFile*	shellLibFile = GetLocalLibraryFile();
	shellLibFile->Preflight(shellLibRefNum);
#endif // ASLMBUILD

	Handle largesuite;
	OSErr osr = NewIconSuite(&largesuite);
	THROW_IF_ERROR(osr,"Error creating icon suite");
	
	Handle icon = GetResource('icl8',90);
	THROW_IF_ERROR(ResError(),"Trouble loading icons (icl8)");
	DetachResource(icon);
	osr = AddIconToSuite(icon,largesuite,'icl8');
	
	icon = GetResource('icl4',90);
	THROW_IF_ERROR(ResError(),"Trouble loading icons (icl4)");
	DetachResource(icon);
	osr = AddIconToSuite(icon,largesuite,'icl4');
	
	icon = GetResource('ICN#',90);
	THROW_IF_ERROR(ResError(),"Trouble loading icons (ICN#)");
	DetachResource(icon);
	osr = AddIconToSuite(icon,largesuite,'ICN#');

	if (largesuite) fLargeIcons = largesuite;

	Handle smallsuite;
	osr = NewIconSuite(&smallsuite);
	THROW_IF_ERROR(osr,"Error creating icon suite");
	
	icon = GetResource('ics8',90);
	THROW_IF_ERROR(ResError(),"Trouble loading icons (ics8)");
	DetachResource(icon);
	osr = AddIconToSuite(icon,smallsuite,'ics8');
	
	icon = GetResource('ics4',90);
	THROW_IF_ERROR(ResError(),"Trouble loading icons (ics4)");
	DetachResource(icon);
	osr = AddIconToSuite(icon,smallsuite,'ics4');
	
	icon = GetResource('ics#',90);
	THROW_IF_ERROR(ResError(),"Trouble loading icons (ics#)");
	DetachResource(icon);
	osr = AddIconToSuite(icon,smallsuite,'ics#');

	if (smallsuite) fSmallIcons = smallsuite;
	
			
#ifdef ASLMBUILD
 	shellLibFile->Postflight(shellLibRefNum);
#endif // ASLMBUILD
}

//  Release 

void DrawPart::Release()
{
	Proxy*		p;
	XMPFrame*	frame;
	
	XMPPersistentObject::Release();
	if (this->GetRefCount() == 0) {
		this->HidePalette();
		frame = (XMPFrame*) fDisplayFrames->First();
		while (frame != kXMPNULL) {
			fDisplayFrames->Remove(frame);
			XMPReleaseObject(frame);
			frame = (XMPFrame*) fDisplayFrames->First();
		}
		frame = (XMPFrame*) fEmbeddedFrames->First();
		while (frame != kXMPNULL) {
			p = ProxyForFrame(frame);
			fContents->Remove(p);
			fEmbeddedFrames->Remove(frame);
			XMPReleaseObject(frame);
			frame = (XMPFrame*) fEmbeddedFrames->First();
		}
		this->GetStorageUnit()->GetDraft()->ReleasePart(this);
	}
}

//-------------------------------------------------------------------------
// From DragAndDrop protocol
//-------------------------------------------------------------------------

//  FullfillPromise 

void DrawPart::FulfillPromise(XMPStorageUnitView *promiseSUView)
{
XMPUnused(promiseSUView);
}

//  NotifyDropComplete 

void DrawPart::DropCompleted(XMPPart* destPart, XMPDropResult dropResult)
{
XMPUnused(destPart);
XMPUnused(dropResult);
}

//  DragEnter 

void DrawPart::DragEnter(XMPDragItemIterator* dragInfo,
							XMPFacet* facet,
							XMPPoint where)
{
XMPUnused(dragInfo);
XMPUnused(facet);
XMPUnused(where);
}

//  DragWithin 

void DrawPart::DragWithin(XMPDragItemIterator* dragInfo,
							XMPFacet* facet,
							XMPPoint where)
{
XMPUnused(dragInfo);
XMPUnused(facet);
XMPUnused(where);
}

//  DragLeave 

void DrawPart::DragLeave(XMPFacet* facet,
						 XMPPoint where)
{
XMPUnused(facet);
XMPUnused(where);
}

//  Drop 

XMPDropResult DrawPart::Drop(XMPDragItemIterator *dropInfo,
							 XMPFacet* facet,
							 XMPPoint where)
{
	XMPDraft       *theDraft = this->GetStorageUnit()->GetDraft();
	XMPULong        attributes;
	DragReference   theDrag;
	Point           delta, dragPoint, dropPoint, pinnedPoint, newXTrans;
	short           mouseDownModifiers, mouseUpModifiers;
	XMPStorageUnit *dropSU, *newSU;
	XMPID           draggedFrameID;
	Proxy          *p;
	XMPFrame       *frameDroppedIn;
	XMPFrame		*newFrame = kXMPNULL;
	XMPPart        *newPart;
	XMPShape	   *clipShape, *newFrameShape;
	XMPTransform   *newExternalXForm;
	XMPBoolean		canHandleDrop = kXMPFalse;
	
	XMPDropResult	dropResult = kXMPDropMove;
			
	// Check to see if we can handle the info in this drag (right now only parts)
	for (dropSU = dropInfo->First(); dropSU; dropSU = dropInfo->Next())
		if (dropSU->Exists(kXMPPropPart, kXMPISOStr, 0))
			canHandleDrop = kXMPTrue;
			
	if (canHandleDrop == kXMPFalse)
			return kXMPDropFail;

	// Get the attributes and other crap for this drag
	attributes = fSession->GetDragAndDrop()->GetDragAttributes();
	theDrag = fSession->GetDragAndDrop()->GetDragReference();
	GetDragModifiers(theDrag, 0L, &mouseDownModifiers, &mouseUpModifiers);
	if ((mouseDownModifiers & optionKey) || (mouseUpModifiers & optionKey))
		dropResult = kXMPDropCopy;
		
	XMPBoolean	notDone = kXMPTrue;
	for (dropSU = dropInfo->First(); dropSU && notDone; dropSU = dropInfo->Next())
	{		
		if ((attributes & kXMPdragIsInSourceFrame) && (dropResult == kXMPDropMove))
		{
			// Get where drag started and tweak our own structures to do the move.
			GetDragOrigin(theDrag, &dragPoint);
			GetDragMouse(theDrag, &dropPoint, &pinnedPoint);
			delta.h = dropPoint.h - dragPoint.h;
			delta.v = dropPoint.v - dragPoint.v;
			dropSU->Focus(kXMPPropFrameInfo, kXMPPosUndefined, kXMPID, 0, kXMPPosFirstSib);
			dropSU->GetValue(sizeof(XMPID), (XMPValue) &draggedFrameID);
			p = this->ProxyForFrameID(draggedFrameID);
			
			// $$$$$ this is a single-facet hack
			p->transform->MoveBy(delta);
			newExternalXForm = new XMPTransform;
			newExternalXForm->CopyFrom(p->transform);
			XMPFrameFacetIterator* facets = p->frame->CreateFacetIterator();
			facets->First()->ChangeGeometry(kXMPNULL, newExternalXForm);
			delete facets;
			
			this->UpdateProxyRegion(p);
			this->CreateProxySelectionBorder(p);
			
			notDone = kXMPFalse;
		}
		else
		{
			// Create a new storage unit for the part
//			newSU = theDraft->CreateStorageUnit();
			
			XMPDraftKey	key;
			
			XMPVolatile(dropSU);
			XMPVolatile(key);
			
			TRY
				key = dropSU->GetDraft()->BeginClone(kXMPClonePaste);				
				newSU = dropSU->CloneTo(key, theDraft, kXMPNULL);
				dropSU->GetDraft()->EndClone(key);
				
			CATCH_ALL
				dropSU->GetDraft()->AbortClone(key);
				RERAISE;
			ENDTRY
			
			if (dropSU->Exists(kXMPPropMouseDownOffset, kQDPoint, 0))
			{
				Point mdOffset = {0,0};
				newXTrans = where.AsQDPoint();
				
				dropSU->Focus(kXMPPropMouseDownOffset, kXMPPosUndefined, kQDPoint, 0, kXMPPosFirstSib);
				dropSU->GetValue(sizeof(Point), (XMPValue) &mdOffset);
				
				SubPt(mdOffset,&newXTrans);
			}
			else
			{
				SetPt(&newXTrans, 20, 20);
			}
			
			newExternalXForm = new XMPTransform;
			newExternalXForm->SetQDOffset(newXTrans);

			if (dropSU->Exists(kXMPPropFrameShape, NULL, 0))
			{
				dropSU->Focus(kXMPPropFrameShape, kXMPPosUndefined, NULL, 1, kXMPPosUndefined);
				newFrameShape = new XMPShape;
				newFrameShape = newFrameShape->ReadShape(dropSU);
			}
			else
			{
				newFrameShape = kXMPNULL;		// use default size {80,80}
			}
			
			newPart = theDraft->GetPart(newSU->GetID());
			newSU->Release();

			frameDroppedIn = facet->GetFrame();
				newFrame = this->CreateEmbeddedFrame(frameDroppedIn,
													 newFrameShape,
													 newExternalXForm,
													 newPart,
													 kXMPNullTypeToken,
													 kXMPNullTypeToken,
													 0,
													 kXMPFalse);

			clipShape = new XMPShape;
			clipShape->CopyFrom(newFrame->GetFrameShape());
			
			newExternalXForm = new XMPTransform;
			newExternalXForm->CopyFrom(this->ProxyForFrame(newFrame)->transform);
			facet->CreateEmbeddedFacet(newFrame, clipShape, newExternalXForm,
										kXMPNULL, kXMPFrameInFront);
			frameDroppedIn->Invalidate(kXMPNULL);
			newPart->Release();
			
			notDone = kXMPFalse;
		}
	}
	this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();

	return dropResult;
}

//-------------------------------------------------------------------------
// From Embedding protocol
//-------------------------------------------------------------------------

//  SetContainerProperties  !!!

	// ********* for containing parts *********

//  GetContainingPartProperties 

XMPStorageUnit* DrawPart::GetContainingPartProperties(XMPFrame* displayFrame)
{
XMPUnused(displayFrame);

	return kXMPNULL;
}

//  CreateEmbeddedFramesIterator 

XMPEmbeddedFramesIterator* DrawPart::CreateEmbeddedFramesIterator(XMPFrame* frame)
{
XMPUnused(frame);

	return new DrawEmbeddedFramesIterator(this);
}

//  CountEmbeddedFrames 

XMPSLong DrawPart::CountEmbeddedFrames()
{
	return GetEmbeddedFrames()->Count();
}

//  GetEmbeddedFrames 
// private - for use by iterator only

OrderedCollection* DrawPart::GetEmbeddedFrames()
{
	return fEmbeddedFrames;
}

//-------------------------------------------------------------------------
// from Frame protocol:
//-------------------------------------------------------------------------

//  AddDisplayFrame 

void DrawPart::AddDisplayFrame(XMPFrame* frame)
{

	if (frame->GetPart() == this)		// frame belongs to me
	{
		OrderedCollectionIterator	displayFramesIter(fDisplayFrames);
		XMPFrame* displayFrame = (XMPFrame*) displayFramesIter.First();
		if (displayFramesIter.IsNotComplete() == kXMPFalse) {
			OrderedCollectionIterator	contentsIter(fContents);
			Proxy*						proxy;
			
			for (proxy = (Proxy*) contentsIter.First();
					contentsIter.IsNotComplete(); 
					proxy = (Proxy*) contentsIter.Next())
			{
				proxy->frame->SetContainingFrame(frame);
			}
		}

		// !!! do something with viewType and partInfo...
		PartInfoRec* pInfo = new PartInfoRec;
		pInfo->bgColor = fDefaultColor;
		//pInfo->bgClipRegion = frame->GetFrameShape()->CopyQDRegion();			// <12> was GetQDRegion()
		if (frame->IsRoot())
			pInfo->fNeedsActivating = kXMPTrue;

		frame->SetPartInfo((XMPInfoType) pInfo);
		fDisplayFrames->AddLast(frame);
		frame->IncrementRefCount();
		frame->SetDroppable(kXMPTrue);
		
		if (frame->GetViewType() == kXMPNullTypeToken)						// if frame view is set don't change it
			frame->SetViewType(fSession->Tokenize(kXMPViewAsFrame));		// if not, make it viewasframe
		if (frame->GetPresentation() == kXMPNullTypeToken)					
			frame->SetPresentation(fSession->Tokenize(kDrawPresNormal));
		
		if (frame->GetContainingFrame() == kXMPNULL)
		{
			//Wrong place. this->ActivateFrame(frame);
		}
	}
	else
		THROW(kXMPErrInvalidFrame);

	// render in frame?
}

//  AttachSourceFrame 

void DrawPart::AttachSourceFrame(XMPFrame* frame, XMPFrame* sourceFrame)
{
	if (fDisplayFrames->Contains(frame)
		 && sourceFrame && fDisplayFrames->Contains(sourceFrame))
	{
		//RCR Note. DrawPart does not currently store embedded
		// frames on a per-containing-frame basis (eg. in the part info
		// of each frame). Since we can't add to the embedded frames list
		// while iterating over it, we make a temporary list here
		
			OrderedCollection* embeddedFrames = new OrderedCollection;
			{
				OrderedCollectionIterator iter(fEmbeddedFrames);
				for (XMPFrame* embeddedFrame = (XMPFrame*) iter.First(); 
						iter.IsNotComplete(); 
						embeddedFrame = (XMPFrame*) iter.Next())
				{
					embeddedFrames->AddLast(embeddedFrame);
				}
			}
			
			OrderedCollectionIterator iter(embeddedFrames);
			for (XMPFrame* embeddedFrame = (XMPFrame*) iter.First(); 
					iter.IsNotComplete(); 
					embeddedFrame = (XMPFrame*) iter.Next())
			{
				if (embeddedFrame->GetContainingFrame() == sourceFrame)
				{
					XMPPart* embeddedPart = embeddedFrame->GetPart();
					XMPShape* newShape = new XMPShape();
					newShape->CopyFrom(embeddedFrame->GetFrameShape());
					XMPTransform* newXForm = new XMPTransform();
					newXForm->CopyFrom(this->ProxyForFrame(embeddedFrame)->transform);
				XMPFrame* newFrame =
					this->CreateEmbeddedFrame(frame, newShape, newXForm, embeddedPart,
												kXMPNullTypeToken, kXMPNullTypeToken,
												0, embeddedFrame->IsOverlaid());
				embeddedPart->AttachSourceFrame(newFrame, embeddedFrame);
				}
			}
			delete embeddedFrames; // Delete the copy
		}
	else
		THROW(kXMPErrInvalidFrame);
}

//  RemoveDisplayFrame 

void DrawPart::RemoveDisplayFrame(XMPFrame* oldFrame)
{
	if (oldFrame != (fPalette ? fPalette->GetRootFrame() : kXMPNULL)) {
		if (fDisplayFrames->Contains(oldFrame))
		{
			OrderedCollection* embeddedFrames = new OrderedCollection;
			{
				OrderedCollectionIterator iter(fEmbeddedFrames);
				for (XMPFrame* embeddedFrame = (XMPFrame*) iter.First(); 
						iter.IsNotComplete(); 
						embeddedFrame = (XMPFrame*) iter.Next())
				{
					embeddedFrames->AddLast(embeddedFrame);
				}
			}
			OrderedCollectionIterator iter(embeddedFrames);
			for (XMPFrame* embeddedFrame = (XMPFrame*) iter.First(); 
					iter.IsNotComplete(); 
					embeddedFrame = (XMPFrame*) iter.Next())
			{
				if (embeddedFrame->GetContainingFrame() == oldFrame)
				{
////				embeddedFrame->Close();
//					fEmbeddedFrames->Remove(embeddedFrame);
////				embeddedFrame->Release();
//					embeddedFrame->Remove();
					this->RemoveEmbeddedFrame(embeddedFrame);
				}
			}
			delete embeddedFrames; // Delete the copy
	
			fSession->GetArbitrator()->RelinquishFocusSet(fFocusSet,oldFrame);
	
			PartInfoRec* pInfo = (PartInfoRec*) oldFrame->GetPartInfo();
			oldFrame->SetPartInfo((XMPInfoType) kXMPNULL);
			delete pInfo;
			fDisplayFrames->Remove(oldFrame);
			oldFrame->Release();
			
			OrderedCollectionIterator dIter(fDisplayFrames);
			XMPFrame*	displayFrame = (XMPFrame*) dIter.First();
			if (dIter.IsNotComplete() == kXMPFalse) {
				this->DestroyPalette();
			}
		}
		else
			THROW(kXMPErrInvalidFrame);
	}

	// any display frames left?
}

//  CloseDisplayFrame 

void DrawPart::CloseDisplayFrame(XMPFrame* frame)
{
	if (fDisplayFrames->Contains(frame))
	{
		OrderedCollection* embeddedFrames = new OrderedCollection;
		{
			OrderedCollectionIterator iter(fEmbeddedFrames);
			for (XMPFrame* embeddedFrame = (XMPFrame*) iter.First(); 
					iter.IsNotComplete(); 
					embeddedFrame = (XMPFrame*) iter.Next())
			{
				embeddedFrames->AddLast(embeddedFrame);
			}
		}
		
		OrderedCollectionIterator iter(embeddedFrames);
		for (XMPFrame* embeddedFrame = (XMPFrame*) iter.First(); 
				iter.IsNotComplete(); 
				embeddedFrame = (XMPFrame*) iter.Next())
		{
			if (embeddedFrame->GetContainingFrame() == frame)
			{
				embeddedFrame->Close();
				fEmbeddedFrames->Remove(embeddedFrame);
				XMPReleaseObject(embeddedFrame);
			}
		}
		delete embeddedFrames; // Delete the copy
		
		fSession->GetArbitrator()->RelinquishFocusSet(fFocusSet, frame);

		PartInfoRec* pInfo = (PartInfoRec*) frame->GetPartInfo();
		frame->SetPartInfo((XMPInfoType) kXMPNULL);
		delete pInfo;
		fDisplayFrames->Remove(frame);
		XMPReleaseObject(frame);
	}
	else
		THROW(kXMPErrInvalidFrame);
}

//  FrameShapeChanged 

void DrawPart::FrameShapeChanged(XMPFrame* frame)
{
	if (fDisplayFrames->Contains(frame))
	{
		/// !!! should leave UsedShape and ActiveShape null to inherit FrameShape
//		XMPShape* scratch;
//		scratch = new XMPShape();
//		scratch->CopyFrom(frame->GetFrameShape());
//		frame->ChangeUsedShape(scratch);
// 		scratch = new XMPShape();
//		scratch->CopyFrom(frame->GetFrameShape());
//		frame->ChangeActiveShape(scratch);
	}
	else
		THROW(kXMPErrInvalidFrame);
}

//  ViewTypeChanged 

void DrawPart::ViewTypeChanged(XMPFrame* frame)
{
	if (fDisplayFrames->Contains(frame))
		{ /* !!! change viewType of frame */ }
	else
		THROW(kXMPErrInvalidFrame);
}

//  PresentationChanged 

void DrawPart::PresentationChanged(XMPFrame* frame)
{
	if (fDisplayFrames->Contains(frame))
		{ /* !!! change presentation of frame */ }
	else
		THROW(kXMPErrInvalidFrame);
}

//  WritePartInfo 

void DrawPart::WritePartInfo(XMPPtr partInfo, XMPStorageUnitView* storageUnitView)
{
	if (partInfo)
	{
		storageUnitView->SetValue(sizeof(RGBColor), (XMPValue)&(((PartInfoRec*)partInfo)->bgColor));
		XMPBoolean needsActivating = ((PartInfoRec*)partInfo)->fNeedsActivating 
									|| ((PartInfoRec*)partInfo)->fIsActive;
		storageUnitView->SetValue(sizeof(XMPBoolean),
									(XMPValue)&needsActivating);
	}
}

//  ReadPartInfo 

XMPPtr DrawPart::ReadPartInfo(XMPFrame* frame, XMPStorageUnitView* storageUnitView)
{
	XMPUnused(frame);
	if (storageUnitView->GetSize())
	{
		PartInfoRec* partInfo = new PartInfoRec;
		
		storageUnitView->GetValue(sizeof(RGBColor),
									(XMPValue)&(partInfo->bgColor));
		XMPBoolean needsActivating;
		storageUnitView->GetValue(sizeof(XMPBoolean),
									(XMPValue)&(needsActivating));
		partInfo->fNeedsActivating = needsActivating;
									
		OrderedCollectionIterator d(fDisplayFrames);
		XMPFrame* dispFrame = (XMPFrame*) d.First();
	
		return partInfo;
	}
	else
		return ((XMPPtr)kXMPNULL);
}

//  Open 

XMPID DrawPart::Open(XMPFrame* frame)
{
	XMPWindow* window = kXMPNULL;

	if (frame) // Doing a View As Window
	{
		window = fSession->GetWindowState()->GetWindow(fWindowID);
		if (window)
			window->Select();
		else
		{
			window = this->CreateWindow(frame);
			fWindowID = window->GetID();
			window->Open();
			window->Show();
			window->Select();			
		}
	}
	else
	{
		window = this->CreateWindow(frame);
		fWindowID = window->GetID();
		window->Open();
		window->Show();
		window->Select();			
	}
	return window->GetID();
}

XMPWindow* DrawPart::CreateWindow(XMPFrame* sourceFrame)
{
	Rect windRect;
	XMPPlatformWindow platformWindow = kXMPNULL;
	XMPWindow* window = kXMPNULL;
	
	if (sourceFrame)
	{
		SetRect(&windRect, 100, 100, 300, 300);	
	}
	else
	{
		const	XMPSShort		kOnePageWidth = 600;
		// Get the menu bar height, for figuring the top of our windows
		XMPSShort	mbHeight	= 24; //(*(XMPSShort*)MBarHeight);	//get it from lomem
		SetRect(&windRect, 4, mbHeight+16,
				XMPASLMQDGlobals.screenBits.bounds.right - 64,
				XMPASLMQDGlobals.screenBits.bounds.bottom - 4);		
		
		if (windRect.right-windRect.left > kOnePageWidth)
		{
			windRect.right = windRect.left + kOnePageWidth;
		}
	}

	Str255 aStr255;
	XMPULong maxsize = 32+sizeof(XMPName);
	XMPName* partName = (XMPName*)XMPNewPtr(maxsize);
	XMPGetPOName(this,partName,&maxsize);
	IntlToPStr(partName,(StringPtr)&aStr255);
	XMPDisposePtr(partName);
	
	platformWindow = NewCWindow(kXMPNULL, &windRect, aStr255, false, zoomDocProc, (WindowPtr)-1L, true, kXMPNULL);
	window =  fSession->GetWindowState()->CreateWindow(platformWindow, 
													 (sourceFrame==kXMPNULL),	// Keeps draft open
													 kXMPTrue,	// Is resizable
													 kXMPFalse,	// Is floating
													 kXMPTrue,  // should save
												 this, kXMPNullTypeToken, kXMPNullTypeToken, sourceFrame);

	return window;
}

	// ********* for containing parts *********

//  CreateEmbeddedFrame 

XMPFrame* DrawPart::CreateEmbeddedFrame(XMPFrame* containingFrame,
										XMPShape* frameShape,
										XMPTransform* externalTransform,
										XMPPart* embedPart,
										XMPTypeToken viewType,
										XMPTypeToken presentation,
										XMPID frameGroupID,
										XMPBoolean isOverlaid)
{
	XMPShape* newShape = NULL;

	// decide what newShape should be - for now, give it what it asked for
	if (frameShape)
	{	
		newShape = frameShape;
	}
	else
	{	// no frameShape specified, use a default size
		XMPRect rect;
		rect.SetInt(0,0,80,80);
		newShape = new XMPShape;
		newShape->SetRectangle(&rect);
	};
	
	// create the new frame
	XMPFrame* newFrame = this->GetStorageUnit()->GetDraft()->
			CreateFrame(containingFrame, newShape, embedPart,
						viewType, presentation, kXMPFalse, isOverlaid);
	fEmbeddedFrames->AddFirst(newFrame);

	// create a proxy to hold the embedded frame
	XMPShape* scratch = new XMPShape;
	scratch->CopyFrom(newShape);
	scratch->Transform(externalTransform);
	RgnHandle scratchRgn = scratch->CopyQDRegion();
	delete scratch;
	
	Proxy* p = new Proxy(scratchRgn, newFrame, externalTransform);  // new for Facets
	// put proxy into contents, possibly adjusting other content
	fContents->AddFirst(p);

	return newFrame;
}
	
//  RemoveEmbeddedFrame 

void DrawPart::RemoveEmbeddedFrame(XMPFrame* embeddedFrame)
{
	if (fEmbeddedFrames->Contains(embeddedFrame))
	{
		XMPFrame* containingFrame = embeddedFrame->GetContainingFrame();
		Proxy* p = ProxyForFrame(embeddedFrame);
		if (p)
		{	
			fContents->Remove(p);
			XMPDeleteObject(p->transform);
			DisposeRgn(p->region);
			delete p;
		}
		fEmbeddedFrames->Remove(embeddedFrame);
		embeddedFrame->Remove();
		//this->GetStorageUnit()->GetDraft()->RemoveFrame(embeddedFrame);
		
		this->ClipEmbeddedFrames(containingFrame);

	}
	else
		THROW(kXMPErrInvalidFrame);
}

//  RequestFrameShape 

XMPShape* DrawPart::RequestFrameShape(XMPFrame* embeddedFrame, XMPShape* frameShape)
{
	if (fEmbeddedFrames->Contains(embeddedFrame))
	{	// find frame proxy and fix cached region
		Proxy* p = ProxyForFrame(embeddedFrame);
		if (p)
		{
			XMPShape* scratch = new XMPShape();
			scratch->CopyFrom(frameShape);
			scratch->Transform(p->transform);
			p->region = scratch->CopyQDRegion();
			delete scratch;

			p->frame->Invalidate(kXMPNULL);
			p->frame->Invalidate(frameShape);

			if (fSelection->Contains(p)) { /* !!! fix highlighting */ };
			
			// fix clipping of obscured frames
			this->ClipEmbeddedFrames(embeddedFrame->GetContainingFrame());
		}
	}
	else
		THROW(kXMPErrInvalidFrame);

	return frameShape;
}

//  UsedShapeChanged 

void DrawPart::UsedShapeChanged(XMPFrame* embeddedFrame)
{
	if (fEmbeddedFrames->Contains(embeddedFrame))
	{
		this->ClipEmbeddedFrames(embeddedFrame->GetContainingFrame());
	}
	else
		THROW(kXMPErrInvalidFrame);
}

//  ProxyForFrame 

Proxy* DrawPart::ProxyForFrame(XMPFrame* frame)
{
	if (fEmbeddedFrames->Contains(frame))
	{
		Proxy* p;
		OrderedCollectionIterator i(fContents);
		for (p = (Proxy*) i.First(); i.IsNotComplete(); p = (Proxy*) i.Next())
			if (p->frame == frame) return p;
		return kXMPNULL;
	}
	else
		THROW(kXMPErrInvalidFrame);

	return kXMPNULL;
}


//  ProxyForFrameID 

Proxy* DrawPart::ProxyForFrameID(XMPID frameID)
{
	XMPFrame* frame = this->GetStorageUnit()->GetDraft()->GetFrame(frameID);
	Proxy* p = this->ProxyForFrame(frame);
	frame->Release();
	return p;
}


//-------------------------------------------------------------------------
// From Facet protocol
//-------------------------------------------------------------------------

//  FacetAdded 

void DrawPart::FacetAdded(XMPFacet* facet)
{
	XMPFrame* dispFrame = facet->GetFrame();
	if (!fDisplayFrames->Contains(dispFrame))
		THROW(kXMPErrInvalidFacet);

	OrderedCollectionIterator frames(fEmbeddedFrames);
	for (XMPFrame* frame = (XMPFrame*) frames.First(); 
			frames.IsNotComplete(); 
			frame = (XMPFrame*) frames.Next())
	{
		if (frame->GetContainingFrame() == dispFrame)
		{
			XMPShape* clip = new XMPShape;
			clip->CopyFrom(frame->GetFrameShape());
			XMPTransform* xform = new XMPTransform;
			xform->CopyFrom(this->ProxyForFrame(frame)->transform);
			facet->CreateEmbeddedFacet(frame,
										clip, xform,
										kXMPNULL, kXMPFrameBehind);
		}
	}
	
	this->ClipEmbeddedFacets(facet);
}

//  FacetRemoved 

void DrawPart::FacetRemoved(XMPFacet* facet)
{
	XMPFrame* dispFrame = facet->GetFrame();
	if (!fDisplayFrames->Contains(dispFrame))
		THROW(kXMPErrInvalidFacet);
	
	OrderedCollection* children = new OrderedCollection;
	XMPFacetIterator* facets = facet->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (XMPFacet* childFacet = facets->First();
		facets->IsNotComplete();
		childFacet = facets->Next())
	{
		children->AddLast(childFacet);
	}
	delete facets;
	OrderedCollectionIterator iter(children);
	for (childFacet = (XMPFacet*)iter.First();
			iter.IsNotComplete();
			childFacet = (XMPFacet*)iter.Next())
	{
		facet->RemoveFacet(childFacet);
		delete childFacet;
	}
	delete children;
}

//  ClipShapeChanged 

void DrawPart::GeometryChanged(XMPFacet* facet)
{
	if (fDisplayFrames->Contains(facet->GetFrame()))
		this->ClipEmbeddedFacets(facet);
	else
		THROW(kXMPErrInvalidFacet);
}

//-------------------------------------------------------------------------
// From Imaging protocol
//-------------------------------------------------------------------------

//  Draw 

void DrawPart::Draw(XMPFacet* facet, XMPShape* invalShape)
{
	XMPBoolean drawing = facet->GetCanvas()->IsDynamic();
	
	XMPUnused(invalShape);

	XMPFrame* displayFrame = facet->GetFrame();
	if (fDisplayFrames->Contains(displayFrame))	
	{
		// transform grafport
		
		CFocus prepareForRendering(facet);

		if (facet->GetFrame()->GetPresentation() == fSession->Tokenize(kDrawPresNormal))
		{
			XMPTypeToken curView = facet->GetFrame()->GetViewType();
			
			if (curView == fSession->Tokenize(kXMPViewAsFrame))
			{
				PartInfoRec* pInfo = (PartInfoRec*) displayFrame->GetPartInfo();
				
				// paint the background
			
				PenNormal();
		
				RGBColor saveRGB;
				GetBackColor(&saveRGB);
				RGBColor bgRGB = pInfo->bgColor;

				RGBBackColor(&bgRGB);
				RgnHandle rgn = displayFrame->GetFrameShape()->GetQDRegion();
				EraseRect(&(**rgn).rgnBBox);
				RGBBackColor(&saveRGB);		// restore old bg color

				// no intrinsic contents, so just highlight the selection
			
				if (displayFrame == fSession->GetArbitrator()->GetFocusOwner(fSelectionFocus))
				{	// drawing active frame - highlight selection
					this->HighlightSelection();
				}
				else
				{	// drawing inactive frame - do nothing for now
				}
			}
			else if (curView == fSession->Tokenize(kXMPViewAsLargeIcon))
			{
				Rect rect32;
				SetRect(&rect32, 0, 0, kXMPLargeIconSize, kXMPLargeIconSize);
				PlotIconSuite(&rect32,atAbsoluteCenter,ttNone,fLargeIcons);
			}
			else if (curView == fSession->Tokenize(kXMPViewAsSmallIcon))
			{
				Rect rect16;
				SetRect(&rect16, 0, 0, kXMPSmallIconSize, kXMPSmallIconSize);
				PlotIconSuite(&rect16,atAbsoluteCenter,ttNone,fSmallIcons);
			}
		}
		else if ((drawing) && (facet->GetFrame()->GetPresentation() == fSession->Tokenize(kDrawPresPalette)))
		{				
			RgnHandle	bRgn = facet->GetFrame()->GetFrameShape()->GetQDRegion();
			Rect		bRect;
			Rect		colorRect;				
			XMPUShort	i;
			RGBColor	bgColor;
			XMPSShort	height;
			OSErr		err = noErr;
	
			RGBBackColor(&rgbBlack);
			EraseRgn(bRgn);
			
			bRect.left = (**bRgn).rgnBBox.left;
			bRect.top = (**bRgn).rgnBBox.top;
			bRect.bottom = (**bRgn).rgnBBox.bottom;
			bRect.right = (**bRgn).rgnBBox.right;
			
			height = ((bRect.bottom - bRect.top) - kMargin - ((kNumColors-1) * kMargin) - kMargin) / kNumColors;
				
			colorRect.left = bRect.left + kMargin;
			colorRect.right = bRect.right - kMargin;
			colorRect.top = bRect.top + kMargin;
			colorRect.bottom = colorRect.top + height;
			
			for (i = 0; i < kNumColors; i++) {
				this->GetBGColor(i, &bgColor);
				RGBBackColor(&bgColor);
				EraseRect(&colorRect);
				colorRect.top = colorRect.bottom + kMargin;
				colorRect.bottom = colorRect.top + height;
			}
			RGBBackColor(&rgbWhite);
		}
	}
	else
	{
		// !!! signal error: invalid frame
	}
}

//  HighlightSelection 

void DrawPart::HighlightSelection()
{
	switch (fSelection->Count())
	{
		case 0:		// no selection
			break;

		case 1:		// single selection
			{	OrderedCollectionIterator i(fSelection);
				this->HighlightProxyBorder((Proxy*) i.First());
			};
			break;

		default:	// multiple selection
			{	Proxy* p;
				OrderedCollectionIterator i(fSelection);
				for (p = (Proxy*) i.First(); i.IsNotComplete(); p = (Proxy*) i.Next())
				{	if (fSelection->Contains(p))
						this->HighlightContentObject(p);
				};
			};
	};
}

//  CreateProxySelectionBorder 

void DrawPart::CreateProxySelectionBorder(Proxy* p)
{

	// -- selection border region --

	XMPRgnHandle border = NewRgn();
	CopyRgn(p->region, border);
	InsetRgn(border, -kXMPBorderWidth, -kXMPBorderWidth);
	DiffRgn(border, p->region, border);

	Rect borderRect = (**border).rgnBBox;

	// -- corner region --

	RgnHandle cornerHdlRgn = NewRgn();
	OpenRgn();

	// top left handle
	MoveTo(borderRect.left, borderRect.top);
	LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top);
	LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top + kXMPBorderWidth);
	LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + kXMPBorderWidth);
	LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.left, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.left, borderRect.top);

	// top right handle
	MoveTo(borderRect.right, borderRect.top);
	LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top);
	LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top + kXMPBorderWidth);
	LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + kXMPBorderWidth);
	LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.right, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.right, borderRect.top);

	// bottom right handle
	MoveTo(borderRect.right, borderRect.bottom);
	LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom);
	LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom - kXMPBorderWidth);
	LineTo(borderRect.right - kXMPBorderWidth, borderRect.bottom - kXMPBorderWidth);
	LineTo(borderRect.right - kXMPBorderWidth, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.right, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.right, borderRect.bottom);

	
	// bottom left handle
	MoveTo(borderRect.left, borderRect.bottom);
	LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom);
	LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom - kXMPBorderWidth);
	LineTo(borderRect.left + kXMPBorderWidth, borderRect.bottom - kXMPBorderWidth);
	LineTo(borderRect.left + kXMPBorderWidth, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.left, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	LineTo(borderRect.left, borderRect.bottom);

	CloseRgn(cornerHdlRgn);

	// -- edge region --

	RgnHandle edgeHdlRgn = NewRgn();
	OpenRgn();

	if ((borderRect.right - borderRect.left) >= (5 * kXMPHandleLenMultiplier * kXMPBorderWidth)) {
	
		short hcenter = (borderRect.right - borderRect.left) / 2;
		short hdist = (kXMPHandleLenMultiplier * kXMPBorderWidth) / 2;
		
		// top middle handle
		MoveTo(borderRect.left + hcenter - hdist, borderRect.top);
		LineTo(borderRect.left + hcenter + hdist, borderRect.top);
		LineTo(borderRect.left + hcenter + hdist, borderRect.top + kXMPBorderWidth);
		LineTo(borderRect.left + hcenter - hdist, borderRect.top + kXMPBorderWidth);
		LineTo(borderRect.left + hcenter - hdist, borderRect.top);
		
		// bottom middle handle
		MoveTo(borderRect.left + hcenter - hdist, borderRect.bottom);
		LineTo(borderRect.left + hcenter + hdist, borderRect.bottom);
		LineTo(borderRect.left + hcenter + hdist, borderRect.bottom - kXMPBorderWidth);
		LineTo(borderRect.left + hcenter - hdist, borderRect.bottom - kXMPBorderWidth);
		LineTo(borderRect.left + hcenter - hdist, borderRect.bottom);
	}
	
	if ((borderRect.bottom - borderRect.top) >= (5 * kXMPHandleLenMultiplier * kXMPBorderWidth)) {
	
		short vcenter = (borderRect.bottom - borderRect.top) / 2;
		short vdist = (kXMPHandleLenMultiplier * kXMPBorderWidth) / 2;
		
		// left middle handle
		MoveTo(borderRect.left, borderRect.top + vcenter - vdist);
		LineTo(borderRect.left, borderRect.top + vcenter + vdist);
		LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + vcenter + vdist);
		LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + vcenter - vdist);
		LineTo(borderRect.left, borderRect.top + vcenter - vdist);
		
		// right middle handle
		MoveTo(borderRect.right, borderRect.top + vcenter - vdist);
		LineTo(borderRect.right, borderRect.top + vcenter + vdist);
		LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + vcenter + vdist);
		LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + vcenter - vdist);
		LineTo(borderRect.right, borderRect.top + vcenter - vdist);
	}
	
	CloseRgn(edgeHdlRgn);

	// clip by obscuring frames

	Proxy* q;
	OrderedCollectionIterator i(fContents);
	XMPBoolean above = true;
	for (q = (Proxy*) i.First(); i.IsNotComplete(); q = (Proxy*) i.Next())
	{	if (p == q) {above = false; continue;}
		if (!above) {continue;}
		DiffRgn(border, q->region, border);
		DiffRgn(cornerHdlRgn, q->region, cornerHdlRgn);
		DiffRgn(edgeHdlRgn, q->region, edgeHdlRgn);
	};


	DisposeRgn(fCornerHandleRgn);
	fCornerHandleRgn = cornerHdlRgn;
	UnionRgn(border, cornerHdlRgn, border);

	DisposeRgn(fEdgeHandleRgn);
	fEdgeHandleRgn = edgeHdlRgn;
	UnionRgn(border, edgeHdlRgn, border);
	
	DisposeRgn(fSelectRgn);
	fSelectRgn = border;
	
	// clip path from obscured frames
	this->ClipEmbeddedFrames(p->frame->GetContainingFrame());
}


//  HighlightProxyBorder 

void DrawPart::HighlightProxyBorder(Proxy* p)
{
XMPUnused(p);

	// draw frame border
#ifdef THINK_CPLUS
		FillRgn(fSelectRgn, XMPASLMQDGlobals.gray);
		FillRgn(fCornerHandleRgn, XMPASLMQDGlobals.white);
		FillRgn(fEdgeHandleRgn, XMPASLMQDGlobals.white);
#else
		FillRgn(fSelectRgn, &(XMPASLMQDGlobals.gray));
		FillRgn(fCornerHandleRgn, &(XMPASLMQDGlobals.white));
		FillRgn(fEdgeHandleRgn, &(XMPASLMQDGlobals.white));
#endif
	
	FrameRgn(fCornerHandleRgn);
	FrameRgn(fEdgeHandleRgn);
	
}

//  HighlightContentObject 

void DrawPart::HighlightContentObject(Proxy* p)
{
XMPUnused(p);
}


//  ClipEmbeddedFrames 

void DrawPart::ClipEmbeddedFrames(XMPFrame* frame)
{
	XMPFrameFacetIterator* facets = frame->CreateFacetIterator();
	for (XMPFacet* facet = facets->First(); facets->IsNotComplete(); facet = facets->Next())
		this->ClipEmbeddedFacets(facet);
	delete facets;
}

//  ClipEmbeddedFacets 

void DrawPart::ClipEmbeddedFacets(XMPFacet* facet)
{
	XMPTransform* wclipTrans = new XMPTransform;

	XMPShape* workingClip = new XMPShape();
	workingClip->CopyFrom(facet->GetClipShape());

	// let selection obscure embedded parts

	RgnHandle tempRgn;
	XMPShape* tempShape = new XMPShape;
	
	if (fSelectRgn && !EmptyRgn(fSelectRgn))
	{
		tempRgn = NewRgn();
		CopyRgn(fSelectRgn, tempRgn);
		tempShape->SetPlatformShape(kXMPQuickDraw, tempRgn);
		workingClip->Subtract(tempShape);
	}

	if (fCornerHandleRgn && !EmptyRgn(fCornerHandleRgn))
	{
		tempRgn = NewRgn();
		CopyRgn(fCornerHandleRgn, tempRgn);
		tempShape->SetPlatformShape(kXMPQuickDraw, tempRgn);
		workingClip->Subtract(tempShape);
	}

	if (fEdgeHandleRgn && !EmptyRgn(fEdgeHandleRgn))
	{
		tempRgn = NewRgn();
		CopyRgn(fEdgeHandleRgn, tempRgn);
		tempShape->SetPlatformShape(kXMPQuickDraw, tempRgn);
		workingClip->Subtract(tempShape);
	}

	delete tempShape;


	wclipTrans->CopyFrom(facet->GetWindowFrameTransform());
	workingClip->Transform(wclipTrans);		// now in window coordinates
	
	XMPFacet* embFacet;	
	XMPFacetIterator* facets = facet->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (embFacet = facets->First();
			facets->IsNotComplete();
			embFacet = facets->Next())
	{
		XMPShape* newClipShape = new XMPShape();
		newClipShape->CopyFrom(embFacet->GetFrame()->GetUsedShape());

		wclipTrans->CopyFrom(embFacet->GetWindowFrameTransform());
		wclipTrans->Invert();
		workingClip->Transform(wclipTrans);		// now in frame coordinates
		
		newClipShape->Intersect(workingClip);
		workingClip->Subtract(newClipShape);

		wclipTrans->Invert();
		workingClip->Transform(wclipTrans);		// now in window coordinates
		
		embFacet->ChangeGeometry(newClipShape, kXMPNULL);	// facet now owns shape storage
	}
	delete facets;
	
	// loop to clip for part content here
	
	delete wclipTrans;
	delete workingClip;
}

//-------------------------------------------------------------------------
// From Memory Management protocol
//-------------------------------------------------------------------------

//  Purge 

XMPSize DrawPart::Purge(XMPSize size)
{
	return this->GetStorageUnit()->Purge(size);
}

//-------------------------------------------------------------------------
// From Part Activation protocol
//-------------------------------------------------------------------------

//  RelinquishFocus 

void DrawPart::CommitRelinquishFocus(XMPTypeToken focus, 
								XMPFrame* currentFrame,
								XMPFrame* proposedFrame)
{
XMPUnused(proposedFrame);

	this->FocusLost(focus, currentFrame);
	this->HidePalette();
}

//  FocusAcquired 

void DrawPart::FocusAcquired(XMPTypeToken focus,
								XMPFrame* newOwner)
{
	if (focus == fSelectionFocus) 
	{
		PartInfoRec* pInfo = (PartInfoRec*) newOwner->GetPartInfo();
		pInfo->fIsActive = kXMPTrue;
		
		this->ShowPalette();
	}
	else if (focus == fMenuFocus) 
		this->InstallMenus(newOwner);
}

//  FocusLost  

void DrawPart::FocusLost(XMPTypeToken focus,
							XMPFrame* oldOwner)
{
	if (focus == fSelectionFocus) 
	{
		PartInfoRec* pInfo = (PartInfoRec*) oldOwner->GetPartInfo();
		pInfo->fIsActive = kXMPFalse;
		this->InvalidateSelection(oldOwner);
		//this->EmptySelection();
		this->ClipEmbeddedFrames(oldOwner);
		
		this->HidePalette();
	}
	else if (focus == fMenuFocus) 
		this->RemoveMenus(oldOwner);
}

//  InvalidateSelection 

void DrawPart::InvalidateSelection(XMPFrame* frame)
{
	XMPShape* selectShape = new XMPShape();
	UnionRgn(fSelectRgn,fCornerHandleRgn,fSelectRgn);
	selectShape->SetQDRegion((RgnHandle)XMPCopyHandle((XMPHandle)fSelectRgn));
	frame->Invalidate(selectShape);
	delete selectShape;
}

//  EmptySelection 

void DrawPart::EmptySelection()
{
	for(XMPULong i = fSelection->Count(); i > 0; i--)
		fSelection->RemoveFirst();
	DisposeRgn(fSelectRgn);
	fSelectRgn = NewRgn();
	DisposeRgn(fCornerHandleRgn);
	fCornerHandleRgn = NewRgn();
	DisposeRgn(fEdgeHandleRgn);
	fEdgeHandleRgn = NewRgn();
}

//-------------------------------------------------------------------------
// From Storage protocol
//-------------------------------------------------------------------------

//  Externalize 

 void DrawPart::Externalize()
 {
	SimplePart::Externalize();
	
	XMPStorageUnit* su = fTestDrawSU;
	XMPStorageUnitRef aSURef;
	XMPULong offset;
	XMPFrame* frame;
	Proxy* proxy;
	OrderedCollectionIterator aIter(fDisplayFrames);
	OrderedCollectionIterator bIter(fEmbeddedFrames);
	OrderedCollectionIterator cIter(fContents);
	Point xform;

	su->Focus(kXMPPropDisplayFrames,kXMPPosUndefined,0,1,kXMPPosFirstSib);
#ifndef SSREMOVEWORKAROUND
	su->Remove();
	su->AddProperty(kXMPPropDisplayFrames);
	su->AddValue(kXMPIDs);
#else
	XMPULong oldValueSize = su->GetSize();;
#endif
	offset = 0;
	for (frame = (XMPFrame*)aIter.First(); aIter.IsNotComplete();
			frame = (XMPFrame*)aIter.Next(), offset+=sizeof(XMPStorageUnitRef))
	{
		frame->Externalize();
		aSURef = su->GetWeakStorageUnitRef(frame->GetStorageUnit());
		su->SetOffset(offset);
		su->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
	}
#ifdef SSREMOVEWORKAROUND
	if (oldValueSize > offset)
		su->DeleteValue(oldValueSize-offset);
#endif

// 	su->Focus(kXMPPropEmbeddedFrames,kXMPPosUndefined,0,1,kXMPPosFirstSib);
//#ifndef SSREMOVEWORKAROUND
//	su->Remove();
//	su->AddProperty(kXMPPropEmbeddedFrames);
//	su->AddValue(kXMPIDs);			
//#else
//	oldValueSize = su->GetSize();;
//#endif
//	offset = 0;
//	for (frame = (XMPFrame*)bIter.First(); bIter.IsNotComplete();
//			frame = (XMPFrame*)bIter.Next(), offset+=sizeof(XMPStorageUnitRef))
//	{
//		frame->Externalize();
//		aSURef = su->GetStrongStorageUnitRef(frame->GetStorageUnit());
//		su->SetOffset(offset);
//		su->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
//	}
//#ifdef SSREMOVEWORKAROUND
//	if (oldValueSize > offset)
//		su->DeleteValue(oldValueSize-offset);
//#endif

 	su->Focus(kXMPPropFrameGroup,kXMPPosUndefined,0,1,kXMPPosFirstSib);
	su->SetValue(sizeof(fFrameGroupIDCounter), (XMPValue)&fFrameGroupIDCounter);
	
 	su->Focus(kXMPPropContents,kXMPPosUndefined,0,1,kXMPPosFirstSib);
#ifndef SSREMOVEWORKAROUND
	su->Remove();
	su->AddProperty(kXMPPropContents);
	su->AddValue(kXMPIDs);
#else
	oldValueSize = su->GetSize();;
#endif
	offset = 0;
	for (proxy = (Proxy*)cIter.First();
			cIter.IsNotComplete(); 
			proxy = (Proxy*)cIter.Next())
	{
		proxy->frame->Externalize();
		aSURef = su->GetStrongStorageUnitRef(proxy->frame->GetStorageUnit());
		su->SetOffset(offset);
		su->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		offset += sizeof(XMPStorageUnitRef);
		
		xform = proxy->transform->GetQDOffset();
		su->SetOffset(offset);
		su->SetValue(sizeof(XMPPlatformTransform), (XMPValue)&xform);
		offset += sizeof(Point);
	}
#ifdef SSREMOVEWORKAROUND
	if (oldValueSize > offset)
		su->DeleteValue(oldValueSize-offset);
#endif

	su = this->GetStorageUnit();
	su->Focus(kXMPPropContents,kXMPPosSame,kXMPKindTestDraw,1,kXMPPosFirstSib);
	aSURef = su->GetStrongStorageUnitRef(fTestDrawSU);
	su->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
 }

//  CloneInto 

void DrawPart::CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU)
{
	XMPStorageUnit*	su = this->GetStorageUnit();
	
	this->Externalize();
	XMPStorageUnit* destTestDrawSU = fTestDrawSU->CloneTo(key, storageUnit->GetDraft(), initiatingFrameSU);	
	
	storageUnit->AddProperty(kXMPPropContents);
	storageUnit->AddValue(kXMPKindTestDraw);
	XMPStorageUnitRef aSURef = storageUnit->GetStrongStorageUnitRef(destTestDrawSU);
	storageUnit->SetValue(sizeof(XMPStorageUnitRef),&aSURef);
		
	OrderedCollectionIterator contentsIter(fContents);
	Proxy* proxy;
	for (proxy = (Proxy*)contentsIter.First();
			contentsIter.IsNotComplete(); 
			proxy = (Proxy*)contentsIter.Next())
	{
		if ((initiatingFrameSU== kXMPNULL) ||
			(proxy->frame->GetContainingFrame()->GetStorageUnit() == initiatingFrameSU)) {
			XMPStorageUnit* toFrameSU = proxy->frame->CloneTo(key, storageUnit->GetDraft());
			toFrameSU->Release();
		}
	}
	destTestDrawSU->Release();
	
	su->CloneInto(key, storageUnit, initiatingFrameSU);
}

//-------------------------------------------------------------------------
// From UI Events protocol
//-------------------------------------------------------------------------


//  HandleEvent  !!!

XMPBoolean DrawPart::HandleEvent(XMPEventData event, XMPFrame* frame,
		XMPFacet* facet)
{
	Point windowPoint;
	GrafPtr curPort;
	XMPBoolean handled = kXMPFalse;
	
	switch (event->what)
	{
		case nullEvent:
			break;
		
		case mouseDown:
			{
				windowPoint = event->where;
				GetPort(&curPort);
				SetPort(facet->GetWindow()->GetPlatformWindow());
				SetOrigin(0,0);
				GlobalToLocal(&windowPoint);
				XMPPoint windowXMPPoint(windowPoint);
				handled = this->HandleMouseDown(facet, windowXMPPoint, event);
				SetPort(curPort);
			}
			break;
			
		case kXMPEvtMouseDownEmbedded:
			{
				windowPoint = event->where;
				GetPort(&curPort);
				SetPort(facet->GetWindow()->GetPlatformWindow());
				SetOrigin(0,0);
				GlobalToLocal(&windowPoint);
				handled = this->HandleMouseDownInEmbeddedFrame(facet, 
										(XMPFacet*)event->message, windowPoint, event);
				SetPort(curPort);
			}
			break;
		
		case kXMPEvtMouseDownBorder:
			{
				windowPoint = event->where;
				GetPort(&curPort);
				SetPort(facet->GetWindow()->GetPlatformWindow());
				SetOrigin(0,0);
				GlobalToLocal(&windowPoint);
				handled = this->HandleMouseDownInEmbeddedFrame(facet, 
										(XMPFacet*)event->message, windowPoint, event);
				SetPort(curPort);
			}
			break;
		
		case kXMPEvtWindow: 
			{
				switch (event->message)
				{
					case inGoAway:
						XMPWindow*	window = facet->GetWindow();
						if (window == fPalette) {
							this->HidePalette();
							fPaletteUp = kXMPFalse;
						}
						break;
					case inZoomIn:
					case inZoomOut:
						SysBeep(2); 
						handled = kXMPTrue;
						break;
					default:
						break;
				}
			}
			break;
		
		case mouseUp:
			break;
			
		case keyDown:
		case autoKey:
			handled = this->HandleKeyDown(frame, event);
			break;
	
		case activateEvt:
			handled = true; // actually ignored by dispatcher
			if ((event->modifiers & activeFlag) != 0)
				this->ActivatingWindow(frame);
			else
				this->DeActivatingWindow(frame);
			break;
		
		case keyUp:
			break;
						
		case osEvt:
			{
				unsigned char 	typeOSEvent;
				
				//  Is it a multifinder event?
				typeOSEvent = (unsigned char) (event->message >> 24) & 0x00FF;
				
				//  Switch on the type of OSEvent that occurred, high byte of message is event type
				switch (typeOSEvent) 
				{ 	
					case kMouseMovedMessage:
						break;
						
					case kSuspendResumeMessage:
						{
							const short kResumeMask = 0x01;	// High byte suspend/resume event 
							XMPBoolean	goingToBackground = (event->message & kResumeMask) == 0;
							XMPWindow*	window = facet->GetWindow();
							if (window && window != fPalette) 
							{
								if (goingToBackground)
								{
									if (fPalette && fPalette->IsShown())
									{
										fPaletteWasShown = kXMPTrue;
										this->HidePalette();
										fPaletteUp = kXMPFalse;
									}
									else
										fPaletteWasShown = kXMPFalse;
								}
								else
								{
									if (fPaletteWasShown)
									{
										fPaletteUp = kXMPTrue;
										this->ShowPalette();
									}
								}
							}
								
						}
						break;
				}
			}

			break;
	
		case kHighLevelEvent:
			break;

		case kXMPEvtMenu:
			handled = this->HandleMenuEvent(frame, event);
			break;
			
		default:
			return kXMPFalse;
	}
	return handled;
}

//  HandleMouseDownDrag 

XMPBoolean DrawPart::HandleMouseDownDrag(XMPFacet* facet, Proxy* selection, XMPEventData event)
{
	XMPBoolean handled = kXMPFalse;
	XMPPart* destPart;
	XMPShape* scratch;
	Point translation = {0,0};
	Point mdOffset = {0,0};
	RgnHandle dragRgn, tempRgn;
	XMPID frameInfoData;
	XMPDragAndDrop*	dad;
	XMPStorageUnit*	unit;
	XMPDropResult dropResult;

	// Build the drag outline
	scratch = new XMPShape();
	scratch->CopyFrom(selection->frame->GetFrameShape());
	scratch->Transform(selection->transform);
	dragRgn = scratch->CopyQDRegion();
	translation = facet->GetWindowFrameTransform()->TransformPoint(translation).AsQDPoint();
	LocalToGlobal(&translation);
	OffsetRgn(dragRgn, translation.h, translation.v);
	delete scratch;
	tempRgn = NewRgn();
	CopyRgn(dragRgn, tempRgn);
	InsetRgn(tempRgn, 1, 1);
	DiffRgn(dragRgn, tempRgn, dragRgn);
	DisposeRgn(tempRgn);
	
	//  Fill DAD SU with Props n' Vals
	
	dad = fSession->GetDragAndDrop();
	dad->Clear();
	unit = dad->GetContentStorageUnit();

	XMPDraft* fromDraft = this->GetStorageUnit()->GetDraft();
	XMPDraftKey	key = fromDraft->BeginClone(kXMPCloneCut);
	selection->frame->GetPart()->CloneInto(key, unit, selection->frame->GetStorageUnit());
	fromDraft->EndClone(key);

	//  save ID of frame being dragged
	
	frameInfoData = selection->frame->GetID();
	unit->AddProperty(kXMPPropFrameInfo)->AddValue(kXMPID);
	unit->SetValue(sizeof(XMPID), (XMPValue) &frameInfoData);
	
	//  (optional) save offset between mousedown pt and topLeft pt of selection
	
	XMPFrameFacetIterator* faceti = selection->frame->CreateFacetIterator();
	Point topLeft = faceti->First()->GetWindowFrameTransform()->GetQDOffset();
	
	mdOffset = event->where;
	GlobalToLocal(&mdOffset);
	
	SubPt(topLeft,&mdOffset);
	
	unit->AddProperty(kXMPPropMouseDownOffset)->AddValue(kQDPoint);
	unit->SetValue(sizeof(Point), (XMPValue) &mdOffset);

	//  (optional) if dragging one frame, save its external transform (not aggregate)
	
	unit->AddProperty(kXMPPropExternalTransform)->AddValue(kQDPoint);
	unit->SetValue(sizeof(Point), (XMPValue) &topLeft);
	
	//  if dragging one frame, write its shape   [jpa]
	
	unit->AddProperty(kXMPPropFrameShape);
	selection->frame->GetFrameShape()->WriteShape(unit);
	
	// 
		
	selection->frame->SetDragging(kXMPTrue);
	
	dropResult = dad->StartDrag(facet->GetFrame(), "RGNH", dragRgn, &destPart, event);
	
	if ( (dropResult == kXMPDropCopy) || (dropResult == kXMPDropMove) )
	{
		if (destPart)
		{
			// The frame was dropped in the same document
			if (destPart == this)
			{
				// If this was a move to the same part
				facet->GetFrame()->Invalidate(NULL);
				selection->frame->SetDragging(kXMPFalse);				
			}
			else if ((dropResult == kXMPDropMove) &&
					(destPart->GetStorageUnit()->GetDraft() == this->GetStorageUnit()->GetDraft()))
			{
				// This was a move to a different part
				XMPPart *oldPart = selection->frame->GetPart();
				oldPart->IncrementRefCount();
				facet->GetFrame()->Invalidate(NULL);
				this->InvalidateSelection(selection->frame);
				this->EmptySelection();
				
				OrderedCollection* tempFacets = new OrderedCollection;
				
				XMPFrameFacetIterator* facets = selection->frame->CreateFacetIterator();
				for (XMPFacet* f1 = facets->First();
						facets->IsNotComplete();
						f1 = facets->Next())
				{
					tempFacets->AddLast(f1);
				}
				delete facets;
				
				OrderedCollectionIterator t(tempFacets);
				for (XMPFacet* f2 = (XMPFacet*)t.First();
						t.IsNotComplete();
						f2 = (XMPFacet*)t.Next())
				{
					f2->GetContainingFacet()->RemoveFacet(f2);
					XMPDeleteObject(f2);
				}
				delete tempFacets;
				
				this->RemoveEmbeddedFrame(selection->frame);
//				this->GetStorageUnit()->GetDraft()->RemovePart(oldPart);
				this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
			}
		}
		else
		{
			// The frame was dropped in a different document
			selection->frame->SetDragging(kXMPFalse);
		}
		
		handled = kXMPTrue;
	}
	else
	{
		selection->frame->SetDragging(kXMPFalse);
	}
	DisposeRgn(dragRgn);
	
	return handled;
}

//  HandleMouseDownEdgeResize 

XMPBoolean DrawPart::HandleMouseDownEdgeResize(XMPFacet* facet,
		Proxy* selection, Point mouse)
{
	enum edge {kNone, kTE, kLE, kBE, kRE};
	edge growEdge = kNone;

	Rect* sr = this->GetSelectionRectLocal(selection);

	if		(mouse.v <= ((*sr).top + (0.1 * (*sr).top))) growEdge = kTE;
	else if (mouse.h <= ((*sr).left + (0.1 * (*sr).left))) growEdge = kLE;
	else if (mouse.v >= ((*sr).bottom - (0.1 * (*sr).bottom))) growEdge = kBE;
	else if (mouse.h >= ((*sr).right -  (0.1 * (*sr).right))) growEdge = kRE;

	this->SetGrafPortOrigin(facet);
	Rect growBox = (*sr);
	Rect oldBox = {0,0,0,0};
	
	PenMode(patXor);
#ifdef THINK_CPLUS
	PenPat(XMPASLMQDGlobals.gray);
#else
	PenPat(&(XMPASLMQDGlobals.gray));
#endif
	
	Point theLoc = {0,0};
	Point lastLoc = {0,0};
	
	while (StillDown())
	{
		GetMouse(&theLoc);
		
		if ((theLoc.h != lastLoc.h) || (theLoc.v != lastLoc.v))
		{
			switch (growEdge) 
			{
				case kTE:
					growBox.top = theLoc.v;
					break;

				case kLE:
					growBox.left = theLoc.h;
					break;

				case kBE:
					growBox.bottom = theLoc.v;
					break;

				case kRE:
					growBox.right = theLoc.h;
					break;
				
				default:
					break;
			}
			FrameRect(&oldBox);
			FrameRect(&growBox);
		}
		
		lastLoc = theLoc;
		oldBox = growBox;
	}
	
	FrameRect(&oldBox);
	
#ifdef THINK_CPLUS
	PenPat(XMPASLMQDGlobals.gray);
#else
	PenPat(&(XMPASLMQDGlobals.gray));
#endif
	PenMode(patCopy);
	
	SetOrigin(0,0);

	Point delta;
	delta.h = theLoc.h - mouse.h; //where.h;
	delta.v = theLoc.v - mouse.v; //where.v;
	Point transOffset = { 0, 0 };

	//JPA: Does this really need to be a copy???
	XMPShape* oldShape = selection->frame->GetFrameShape()->Copy();
	XMPRect newBounds;
	oldShape->GetBoundingBox(&newBounds);

	switch (growEdge)
	{
		case kTE:
			newBounds.bottom -= ff(delta.v);
			transOffset.v += delta.v;
			break;

		case kLE:
			newBounds.right -= ff(delta.h);
			transOffset.h += delta.h;
			break;

		case kBE:
			newBounds.bottom += ff(delta.v);
			break;

		case kRE:
			newBounds.right += ff(delta.h);
			break;

		default:
			break;
	}
	
	XMPShape* newShape = new XMPShape;
	newShape->SetRectangle(&newBounds);
	
	this->InvalEmbedFrameAfterResize(facet, selection, oldShape, newShape, transOffset);
	delete oldShape;
	
	return kXMPTrue;
}

//  GetSelectionRectLocal 

Rect* DrawPart::GetSelectionRectLocal(Proxy* selection)
{
	RgnHandle shapeRgn = selection->frame->GetFrameShape()->GetQDRegion();
	Rect shapeRect = (**shapeRgn).rgnBBox;
	
	Rect* localRect = new Rect;
	SetRect(localRect, shapeRect.left, shapeRect.top,
						shapeRect.right, shapeRect.bottom);
	
	Point offset = selection->transform->GetQDOffset();
	OffsetRect(localRect, offset.h, offset.v);
	
	return localRect;
}


//  HandleMouseDownCornerResize 

XMPBoolean DrawPart::HandleMouseDownCornerResize(XMPFacet* facet, Proxy* selection, Point mouse)
{
	enum corner {kNone, kTL, kTR, kBL, kBR};
	corner growCorner = kNone;

	Rect* sr = this->GetSelectionRectLocal(selection);
	Point center;
	SetPt(&center, ((*sr).right + (*sr).left) / 2, ((*sr).bottom + (*sr).top) / 2);		// center of FrameShape
	
	if		((mouse.v <= center.v) && (mouse.h <= center.h)) growCorner = kTL;
	else if ((mouse.v <  center.v) && (mouse.h >  center.h)) growCorner = kTR;
	else if ((mouse.v >  center.v) && (mouse.h <  center.h)) growCorner = kBL;
	else if ((mouse.v >  center.v) && (mouse.h >  center.h)) growCorner = kBR;
		
	this->SetGrafPortOrigin(facet);
	Rect growBox = (*sr);
	Rect oldBox = {0,0,0,0};
	
	PenMode(patXor);
#ifdef THINK_CPLUS
	PenPat(XMPASLMQDGlobals.gray);
#else
	PenPat(&(XMPASLMQDGlobals.gray));
#endif
	
	Point theLoc = {0,0};
	Point lastLoc = {0,0};
	
	while (StillDown())
	{
		GetMouse(&theLoc);
		
		if ((theLoc.h != lastLoc.h) || (theLoc.v != lastLoc.v))
		{
			switch (growCorner) 
			{
				case kTL:
					growBox.top = theLoc.v;
					growBox.left = theLoc.h;
					break;

				case kTR:
					growBox.top = theLoc.v;
					growBox.right = theLoc.h;
					break;

				case kBL:
					growBox.left = theLoc.h;
					growBox.bottom = theLoc.v;
					break;

				case kBR:
					growBox.bottom = theLoc.v;
					growBox.right = theLoc.h;
					break;
				
				default:
					break;
			}
			
			FrameRect(&oldBox);
			FrameRect(&growBox);
		}
		
		lastLoc = theLoc;
		oldBox = growBox;
	}
	
	FrameRect(&oldBox);
	
#ifdef THINK_CPLUS
	PenPat(XMPASLMQDGlobals.gray);
#else
	PenPat(&(XMPASLMQDGlobals.gray));
#endif
	PenMode(patCopy);
	
	SetOrigin(0,0);

	Point delta;
	delta.h = theLoc.h - mouse.h; //where.h;
	delta.v = theLoc.v - mouse.v; //where.v;
	Point transOffset = { 0, 0 };

	//JPA: Does this need to be a copy??? If so, who disposes it???
	XMPShape* oldShape = selection->frame->GetFrameShape()->Copy();
	XMPRect newBounds;
	oldShape->GetBoundingBox(&newBounds);
	XMPPoint fixDelta = delta;

	switch (growCorner)
	{
		case kTL:
			newBounds.bottom -= fixDelta.y; newBounds.right -= fixDelta.x;
			transOffset.v += delta.v; transOffset.h += delta.h;
			break;

		case kTR:
			newBounds.bottom -= fixDelta.y; newBounds.right += fixDelta.x;
			transOffset.v += delta.v;
			break;

		case kBL:
			newBounds.bottom += fixDelta.y; newBounds.right -= fixDelta.x;
			transOffset.h += delta.h;
			break;

		case kBR:
			newBounds.bottom += fixDelta.y;
			newBounds.right += fixDelta.x;
			break;

		default:
			break;
	}

	XMPShape* newShape = new XMPShape;
	newShape->SetRectangle(&newBounds);

	this->InvalEmbedFrameAfterResize(facet, selection, oldShape, newShape, transOffset);
	delete oldShape;
	
	return kXMPTrue;
}

//  InvalEmbedFrameAfterResize 

void DrawPart::InvalEmbedFrameAfterResize(	XMPFacet* facet, Proxy* selection,
											XMPShape* oldShape, XMPShape* newShape,
											Point transOffset)
{
	// invalidate old frame shape
	oldShape->Transform(selection->transform);
	facet->GetFrame()->Invalidate(oldShape);

	XMPShape* newShapeCopy = new XMPShape;
	newShapeCopy->CopyFrom(newShape);		// save to invalidate later

	selection->frame->ChangeFrameShape(newShape);	// already in frame coords

	// if the left or top side of the frame was adjusted, the transform will be updated
	if ((transOffset.h != 0) || (transOffset.v != 0))
	{
		selection->transform->MoveBy(transOffset);

		// $$$$$ move facets for selected frame - this is slightly hackish
		XMPTransform* newTrans;
		XMPFacetIterator* facets =
			facet->GetWindow()->GetRootFacet()->CreateFacetIterator(kXMPTopDown, kXMPFrontToBack);
		for (XMPFacet* child = facets->First();
				facets->IsNotComplete(); child = facets->Next())
		{
			if (child->GetFrame() == selection->frame)
			{
				newTrans = new XMPTransform;
				newTrans->CopyFrom(selection->transform);
				child->ChangeGeometry(kXMPNULL, newTrans);
				facets->SkipChildren();
			}
		}
		delete facets;
	}

	// invalidate changed areas
	newShapeCopy->Transform(selection->transform);
	facet->GetFrame()->Invalidate(newShapeCopy);
	delete newShapeCopy;

	this->InvalidateSelection(facet->GetFrame());
	this->UpdateProxyRegion(selection);
	this->CreateProxySelectionBorder(selection);
	this->ClipEmbeddedFrames(facet->GetFrame());
	this->InvalidateSelection(facet->GetFrame());
	
	this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
}

//  HandleMouseDown 

XMPBoolean DrawPart::HandleMouseDown(	XMPFacet* facet,
										XMPPoint where,
										XMPEventData event)
{
	if (!facet->GetWindow()->IsActive())
		facet->GetWindow()->Select();
	else
		this->ActivateFrame(facet->GetFrame());

	Point mouse = facet->GetWindowContentTransform()->InvertPoint(where).AsQDPoint();

	if (facet->GetWindow() == fPalette) {
		
		RgnHandle	bRgn = facet->GetFrame()->GetFrameShape()->GetQDRegion();
		Rect		bRect;
		XMPUShort	height;
		XMPUShort	whichColor;
		XMPBoolean	didHitMargin;
		
		bRect.left = (**bRgn).rgnBBox.left;
		bRect.top = (**bRgn).rgnBBox.top;
		bRect.bottom = (**bRgn).rgnBBox.bottom;
		bRect.right = (**bRgn).rgnBBox.right;
		height = ((bRect.bottom - bRect.top) - kMargin - ((kNumColors-1) * kMargin) - kMargin) / kNumColors;
		
		whichColor = mouse.v / (kMargin + height);
		didHitMargin = ((mouse.v  % (kMargin + height)) < kMargin) || 
						(mouse.h < kMargin) || 
						((mouse.h + kMargin) >  bRect.right);
		
		if (didHitMargin == kXMPFalse) {
			OrderedCollectionIterator aIter(fDisplayFrames);
			
			XMPFrame*	displayFrame;
			for (displayFrame = (XMPFrame*) aIter.First(); aIter.IsNotComplete(); displayFrame = (XMPFrame*) aIter.Next())
			{
				XMPFrameFacetIterator*	facets = displayFrame->CreateFacetIterator();
				for (XMPFacet* facet = (XMPFacet*) facets->First();
						facets->IsNotComplete();
						facet = (XMPFacet*) facets->Next())
				{
					if (facet->GetWindow() != fPalette)
					{
						this->SetBGColor(displayFrame, whichColor);
						displayFrame->Invalidate(kXMPNULL);
					}
				}
				delete facets;
			}
		}
	}
	else {
	
		OrderedCollectionIterator i(fSelection);
		Proxy* p = (Proxy*)i.First();
	
		Point qdWhere = where.AsQDPoint();
		
		if (PtInRgn(mouse, fCornerHandleRgn))
			return HandleMouseDownCornerResize(facet, p, mouse);
		else if (PtInRgn(mouse, fEdgeHandleRgn))
			return HandleMouseDownEdgeResize(facet, p, mouse);
		else if (PtInRgn(mouse, fSelectRgn))
			return HandleMouseDownDrag(facet, p, event);
		else
			switch (fSelection->Count())
			{
				case 0:
					break;
			
				case 1:
					this->InvalidateSelection(facet->GetFrame());
					this->EmptySelection();
					this->ClipEmbeddedFrames(facet->GetFrame());
					break;
					
				default:
					// !!! handle multiple selection
					break;
			}
	}

	return kXMPTrue;
}

//  HandleMouseDownInEmbeddedFrame 

XMPBoolean DrawPart::HandleMouseDownInEmbeddedFrame(XMPFacet* container,
													XMPFacet* facet,
													XMPPoint where,
													XMPEventData event)
{
XMPUnused(where);
XMPUnused(event);

	this->ActivateFrame(container->GetFrame());

	Proxy* p = ProxyForFrame(facet->GetFrame());

	switch (fSelection->Count())
	{
		case 0:
			fSelection->AddFirst((ElementType)p);
			this->CreateProxySelectionBorder(p);
			this->ClipEmbeddedFrames(container->GetFrame());
			this->InvalidateSelection(container->GetFrame());
			break;

		case 1:
			fSelection->RemoveFirst();
			fSelection->AddFirst((ElementType)p);
			this->CreateProxySelectionBorder(p);
			this->ClipEmbeddedFrames(container->GetFrame());
			this->InvalidateSelection(container->GetFrame());
			break;
			
		default:
			// !!! handle multiple selection
			break;
	}

	this->InvalidateSelection(container->GetFrame());

	return kXMPTrue;
}

//  HandleKeyDown 

XMPBoolean DrawPart::HandleKeyDown(XMPFrame* focusFrame, XMPEventData event)
{
XMPUnused(focusFrame);
XMPUnused(event);
	// cursor keys? !!!

	return kXMPFalse;
}

//  HandleMenuEvent 

XMPBoolean DrawPart::HandleMenuEvent(XMPFrame* focusFrame, XMPEventData event)
{
	ASSERTM(fMenuBar != kXMPNULL, 0, "Menubar is NULL");
	
	long menuResult = event->message;
	short menu = HiWord(menuResult);
	short item = LoWord(menuResult);
	XMPCommandID command = fMenuBar->GetCommand(menu, item);
	XMPBoolean handled = kXMPFalse;
	
	// Until such time as we clean this up and register all commands:
	if (!fMenuBar->IsCommandSynthetic(command))
	{
		switch (command)
		{
			case kXMPCommandPageSetup:
				handled = this->HandlePageSetup(focusFrame);
				break;
			case kXMPCommandPrint:
				handled = this->HandlePrinting(focusFrame);
				break;
			
			case kXMPCommandCut: 		handled = DoCut(focusFrame); break;
			case kXMPCommandCopy:		handled = DoCopy(focusFrame); break;
			case kXMPCommandPaste:		handled = DoPaste(focusFrame); break;
			case kXMPCommandPasteAs:	handled = DoPasteLink(focusFrame); break;
			case kXMPCommandClear:		handled = DoClear(focusFrame); break;
			case kXMPCommandSelectAll:	handled = DoSelectAll(focusFrame); break;
			
			case cXMPGray:
			case cXMPRed:
			case cXMPGreen:
			case cXMPYellow:
			case cXMPBlue:
			case cXMPMagenta:
			case cXMPCyan:
			case cXMPWhite:
				{	//!!! This whole method do with some cleanup
					// Change SetBGColor to use a command number
					short menu;
					short item;
					fMenuBar->GetMenuAndItem(command, menu, item);
					this->SetBGColor(focusFrame, item-1);
					handled = kXMPTrue;
				}
				break;
			case cXMPOtherColor:
				fPaletteUp = kXMPTrue;
				this->ShowPalette();
				handled = kXMPTrue;
			break;
			case kXMPCommandOpen :
				{
					Proxy* p;
					OrderedCollectionIterator i(fSelection);
					for (p = (Proxy*) i.First(); i.IsNotComplete(); p = (Proxy*) i.Next())
					{	
						XMPFrame* frame = p->frame;
						XMPPart* part
								= frame ? frame->GetPart() : (XMPPart*)kXMPNULL;
						if (part)
							part->Open(frame);
					}
				}
				handled = kXMPTrue;
				break;
				
			case kXMPCommandViewAsWin :
				this->Open(focusFrame);
				handled = kXMPTrue;
				break;
				
			case kXMPCommandGetPartInfo : 
				{
					Proxy* p;
					OrderedCollectionIterator i(fSelection);
					p = (Proxy*) i.First();
					if (p!=kXMPNULL) 
					{
						ShowPartFrameInfo(p->frame);
					}
					else
					{
						ShowPartFrameInfo(focusFrame);
					}
					handled = kXMPTrue;
				}
				break;
			default:
				break;
		}
				
	}
	else
	{
		short menu;
		short item;
		fMenuBar->GetMenuAndItem(command, menu, item);
		
		if (menu == 4)
		{
			switch (fSelection->Count())
			{
				case 0:
					break;
		
				case 1:
					this->InvalidateSelection(focusFrame);
					this->EmptySelection();
					this->ClipEmbeddedFrames(focusFrame);
					break;
					
				default:
					// !!! handle multiple selection
					break;
			}
			this->Embed(item, focusFrame);
			handled = kXMPTrue;
		}
		else if (menu == 5)
		{
			if (item == kNumColors) {
				fPaletteUp = kXMPTrue;
				this->ShowPalette();
				handled = kXMPTrue;
			}
			else {
				this->SetBGColor(focusFrame, item-1);
				focusFrame->Invalidate(NULL);
				handled = kXMPTrue;
			}
		}
		else if (menu == 6)
		{
			if (item == 1)
			{
				Proxy* p;
				OrderedCollectionIterator i(fSelection);
				for (p = (Proxy*) i.First(); i.IsNotComplete(); p = (Proxy*) i.Next())
				{	
					p->frame->SetFrozen(true);
				}
			}
			else if (item == 2)	
			{
				Proxy* p;
				OrderedCollectionIterator i(fSelection);
				for (p = (Proxy*) i.First(); i.IsNotComplete(); p = (Proxy*) i.Next())
				{	
					p->frame->SetFrozen(false);
				}
			}
			handled = kXMPTrue;
		}
	}
	
//	SetPort(curPort);
	HiliteMenu(0);
	return handled;
}

//------------------------------------------------------------------------------
// DrawPart::DoCut
//------------------------------------------------------------------------------
	
XMPBoolean DrawPart::DoCut(XMPFrame* frame)
{
XMPUnused(frame);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DrawPart::DoCopy
//------------------------------------------------------------------------------
	
XMPBoolean DrawPart::DoCopy(XMPFrame* containingFrame)
{
XMPUnused(containingFrame);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DrawPart::DoPaste
//------------------------------------------------------------------------------
	
XMPBoolean DrawPart::DoPaste(XMPFrame* frame)
{
XMPUnused(frame);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DrawPart::DoPasteLink
//------------------------------------------------------------------------------
	
XMPBoolean DrawPart::DoPasteLink(XMPFrame* frame)
{
XMPUnused(frame);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DrawPart::DoClear
//------------------------------------------------------------------------------
	
XMPBoolean DrawPart::DoClear(XMPFrame* frame)
{
XMPUnused(frame);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DrawPart::DoSelectAll
//------------------------------------------------------------------------------
	
XMPBoolean DrawPart::DoSelectAll(XMPFrame* frame)
{
XMPUnused(frame);
	return kXMPFalse;
}

//------------------------------------------------------------------------------
// DrawPart::MouseEnter
//------------------------------------------------------------------------------
	
void DrawPart::MouseEnter(XMPFacet* facet, XMPPoint where)
{
	XMPUnused(facet);
	XMPUnused(where);
	SetCursor(*GetCursor(crossCursor)); // For now. 
}

//------------------------------------------------------------------------------
// DrawPart::MouseWithin
//------------------------------------------------------------------------------
	
void DrawPart::MouseWithin(XMPFacet* facet, XMPPoint where)
{
	XMPUnused(facet);
	XMPUnused(where);
}

//------------------------------------------------------------------------------
// DrawPart::MouseLeave
//------------------------------------------------------------------------------

void DrawPart::MouseLeave(XMPFacet* facet)
{
	XMPUnused(facet);
	SetCursor(&XMPASLMQDGlobals.arrow);
}


//------------------------------------------------------------------------------
// DrawPart::AdjustMenus
//------------------------------------------------------------------------------
	
void DrawPart::AdjustMenus(XMPFrame* frame)
{
	PartInfoRec* pInfo = (PartInfoRec*) frame->GetPartInfo();

	// if this frame is the root frame, enable printing.
	
	if (frame->GetContainingFrame() == kXMPNULL)
	{
		fMenuBar->EnableCommand(kXMPCommandPageSetup, kXMPTrue);
		fMenuBar->EnableCommand(kXMPCommandPrint, kXMPTrue);
	}
	else
	{
		fMenuBar->EnableCommand(kXMPCommandPageSetup, kXMPFalse);
		fMenuBar->EnableCommand(kXMPCommandPrint, kXMPFalse);
	}

	fMenuBar->EnableCommand(kXMPCommandGetPartInfo, kXMPTrue);

	// if a part is selected 
		fMenuBar->EnableCommand(kXMPCommandOpen, kXMPTrue);
		
	fMenuBar->EnableCommand(kXMPCommandViewAsWin, !frame->IsRoot());

	fMenuBar->EnableAndCheckCommand(cXMPGray, kXMPTrue, RGBEqual(pInfo->bgColor,rgbGray));
	fMenuBar->EnableAndCheckCommand(cXMPRed, kXMPTrue, RGBEqual(pInfo->bgColor,rgbRed));
	fMenuBar->EnableAndCheckCommand(cXMPGreen, kXMPTrue, RGBEqual(pInfo->bgColor,rgbGreen));
	fMenuBar->EnableAndCheckCommand(cXMPYellow, kXMPTrue, RGBEqual(pInfo->bgColor,rgbYellow));
	fMenuBar->EnableAndCheckCommand(cXMPBlue, kXMPTrue, RGBEqual(pInfo->bgColor,rgbBlue));
	fMenuBar->EnableAndCheckCommand(cXMPMagenta, kXMPTrue, RGBEqual(pInfo->bgColor,rgbMagenta));
	fMenuBar->EnableAndCheckCommand(cXMPCyan, kXMPTrue, RGBEqual(pInfo->bgColor,rgbCyan));
	fMenuBar->EnableAndCheckCommand(cXMPWhite, kXMPTrue, RGBEqual(pInfo->bgColor,rgbWhite));
	fMenuBar->EnableCommand(cXMPOtherColor, kXMPTrue);

	XMPBoolean hasSelection = fSelection->Count()>0;

	fMenuBar->EnableCommand(kXMPCommandCut, hasSelection);
	fMenuBar->EnableCommand(kXMPCommandCopy, hasSelection);
	fMenuBar->EnableCommand(kXMPCommandClear, hasSelection);

	fMenuBar->EnableCommand(kXMPCommandPaste, ValueOnClipboard(fSession));
	fMenuBar->EnableCommand(kXMPCommandPasteAs, kXMPFalse);
	fMenuBar->EnableCommand(kXMPCommandSelectAll, fEmbeddedFrames->Count()>0);
}

//-------------------------------------------------------------------------
// private utilities
//-------------------------------------------------------------------------

//  ActivateFrame 

void DrawPart::ActivateFrame(XMPFrame* frame)
{
	if (frame != (fPalette ? fPalette->GetRootFrame() : kXMPNULL)) {
		PartInfoRec* pInfo = (PartInfoRec*) frame->GetPartInfo();
		if (!(pInfo->fIsActive))
		{
			XMPBoolean succeeded = false;
								
			succeeded = fSession->GetArbitrator()->RequestFocusSet(fFocusSet,frame);
					
			if (succeeded)
			{
				this->FocusAcquired(fSelectionFocus, frame);
				this->FocusAcquired(fMenuFocus, frame);
				this->FocusAcquired(fKeyFocus, frame);
			}
		}
	}
}

//  DeActivateFrame 

void DrawPart::DeActivateFrame(XMPFrame* frame)
{
	if (frame != (fPalette ? fPalette->GetRootFrame() : kXMPNULL)) {
		fSession->GetArbitrator()->RelinquishFocusSet(fFocusSet,frame);
		this->FocusLost(fSelectionFocus, frame);
		this->FocusLost(fMenuFocus, frame);
		this->FocusLost(fKeyFocus, frame);
	}
}

//  ActivatingWindow 

void DrawPart::ActivatingWindow(XMPFrame* frame)
{
	PartInfoRec* pInfo = (PartInfoRec*) frame->GetPartInfo();
	if (pInfo->fNeedsActivating)
	{
		this->ActivateFrame(frame);
		this->ShowPalette();
		pInfo->fNeedsActivating = kXMPFalse;
	}
}

//  DeActivatingWindow 

void DrawPart::DeActivatingWindow(XMPFrame* frame)
{
	PartInfoRec* pInfo = (PartInfoRec*) frame->GetPartInfo();
	if (frame == fSession->GetArbitrator()->GetFocusOwner(fSelectionFocus))
	{
		pInfo->fNeedsActivating = kXMPTrue;
		//this->DeActivateFrame(frame);
	}
	else
		pInfo->fNeedsActivating = kXMPFalse;
}

//  InstallMenus 

void DrawPart::InstallMenus(XMPFrame* frame)
{
	if (frame && fMenuBar)
		fMenuBar->Display();
}

//  ValueOnClipboard 

XMPBoolean ValueOnClipboard(XMPSession* session)
{
	XMPBoolean		result = kXMPFalse;
	XMPClipboard*	clipboard = session->GetClipboard();
	XMPClipboardKey key;

	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* su = clipboard->GetContentStorageUnit(key);
			result =  su->Exists(kXMPPropContents, kXMPNULL, 0);
		CATCH_ALL
		ENDTRY
		clipboard->Unlock(key);
	}
	return result;
}

//  RemoveMenus 

void DrawPart::RemoveMenus(XMPFrame* frame)
{
XMPUnused(frame);

// Don't need to remove the menus we've added
// Just disable Shell menu items we've enabled
}

//  SetBGColor 

void DrawPart::SetBGColor(XMPFrame* frame, XMPUShort whichColor)
{
	PartInfoRec* pInfo = (PartInfoRec *) frame->GetPartInfo();
	if (!pInfo)
	{	pInfo = new PartInfoRec;
		frame->SetPartInfo((XMPInfoType) pInfo);
	};

	RGBColor newColor = pInfo->bgColor;
	
	switch (whichColor)
	{
		case  kGray: newColor = rgbGray;		break;
		case  kRed: newColor = rgbRed;		break;
		case  kGreen: newColor = rgbGreen;		break;
		case  kYellow: newColor = rgbYellow;	break;
		case  kBlue: newColor = rgbBlue;		break;
		case  kMagenta: newColor = rgbMagenta;	break;
		case  kCyan: newColor = rgbCyan;		break;
		case  kWhite: newColor = rgbWhite;		break;
		default: break;
	}
	UserSetBGColor(frame,newColor);
}

struct SetBGColorRec
{
	SetBGColorRec(RGBColor oldColor, RGBColor newColor, XMPFrame* frame)
					{fOldColor = oldColor; fNewColor = newColor;
						fFrame = frame;}
	RGBColor 	fOldColor;
	RGBColor 	fNewColor;
	XMPFrame*	fFrame;
};

//  UserSetBGColor 

void DrawPart::UserSetBGColor(XMPFrame* frame, RGBColor whichColor)
{
	PartInfoRec* pInfo = (PartInfoRec *) frame->GetPartInfo();
	if (!pInfo)
	{	pInfo = new PartInfoRec;
		frame->SetPartInfo((XMPInfoType) pInfo);
	}
	if (!RGBEqual(whichColor,pInfo->bgColor))
	{
		XMPIText*	undoActionName;
		XMPIText*	redoActionName;
	
		undoActionName = PStrToIntl("\pUndo Set Color", kXMPNULL );
		redoActionName = PStrToIntl("\pRedo Set Color", kXMPNULL );
		SetBGColorRec* undoRec = new SetBGColorRec(pInfo->bgColor, whichColor,
													frame);
		fSession->GetUndo()->AddActionToHistory(this, (XMPActionData)undoRec,
												kXMPSingleAction,
												(XMPName*)undoActionName,
												(XMPName*)redoActionName);
		this->SetBGColor(frame, whichColor);
	}
}

//  SetBGColor 

void DrawPart::SetBGColor(XMPFrame* frame, RGBColor whichColor)
{
	PartInfoRec* pInfo = (PartInfoRec *) frame->GetPartInfo();
	if (!pInfo)
	{	pInfo = new PartInfoRec;
		frame->SetPartInfo((XMPInfoType) pInfo);
	}
	if (!RGBEqual(whichColor,pInfo->bgColor))
	{
		pInfo->bgColor = whichColor;
		this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
		frame->Invalidate(NULL);
	}
}

void DrawPart::GetBGColor(XMPUShort whichColor, RGBColor* newColor)
{
	switch (whichColor)
	{
		case  kGray: *newColor = rgbGray;		break;
		case  kRed: *newColor = rgbRed;		break;
		case  kGreen: *newColor = rgbGreen;		break;
		case  kYellow: *newColor = rgbYellow;	break;
		case  kBlue: *newColor = rgbBlue;		break;
		case  kMagenta: *newColor = rgbMagenta;	break;
		case  kCyan: *newColor = rgbCyan;		break;
		case  kWhite: *newColor = rgbWhite;		break;
		
		default:
			WASSERTM(kXMPFalse, "No such color.");
			break;
	}
}

void DrawPart::SetNewBounds(XMPFrame* containedFrame, Rect* r)
{

	Proxy* p = ProxyForFrame(containedFrame);
	XMPTransform* externalXfrm = p->transform;
	XMPShape* shape = containedFrame->GetFrameShape();
	
	XMPTransform* newXForm = new XMPTransform;
	newXForm->CopyFrom(externalXfrm);
	Point pt = *(Point*)r;
	XMPPoint xmpPt = pt;
	newXForm->MoveBy(xmpPt);				// I don't want to move it by,
											// I want to set it to something!
	p->transform = newXForm;			// is this right?  how else will it
										// know?

	XMPShape* newShape = new XMPShape;
	XMPRect xmpr = *r;
	newShape->SetRectangle(&xmpr);
	
	XMPFrameFacetIterator* fi = containedFrame->CreateFacetIterator();
	for(XMPFacet* facet = fi->First(); fi->IsNotComplete();
			facet = fi->Next())
	{
		facet->ChangeGeometry(kXMPNULL, newXForm);
		facet->Invalidate(facet->GetClipShape());
	}
	containedFrame->ChangeFrameShape(newShape);

#ifdef TO_BE_DELETED
	get proxy(frame)
	get external xform from proxy
	frame has a shape
	
	make new external xform and shape for the frame
	changeexternalxform(proxy->facet())
	changeframeshape
	
	for every facet
		clip embedded facets (facet->GetContainingFacet())
#endif // TO_BE_DELETED
	



#ifdef TO_BE_DELETED
	XMPShape* newShape;
	Point transOffset;
	XMPFrameFacetIterator* facetIterator = frame->CreateFacetIterator();
	for (XMPFacet* facet = facetIterator->First();
		facetIterator->IsNotComplete();
		facet = facetIterator->Next())
	{
		OrderedCollectionIterator i(fSelection);
		Proxy* selection = (Proxy*)i.First();

		Point transOffset = { 0, 0 };

		XMPShape* oldShape = selection->frame->GetFrameShape()->Copy();
		XMPRect newBounds = *r;
//		oldShape->GetBoundingBox(&newBounds);

		XMPShape* newShape = new XMPShape;
		newShape->SetRectangle(&newBounds);

		this->InvalEmbedFrameAfterResize(facet, selection, oldShape,
				newShape, transOffset);
		delete oldShape;
	}
	delete facetIterator;
#endif // TO_BE_DELETED
}	// SetNewBounds()


//  Embed 

void DrawPart::Embed(short item, XMPFrame* frame, XMPPart** newPart)
{
	XMPPart* embeddedPart;
	XMPFrame* embeddedFrame;
	XMPTransform* externalXForm = new XMPTransform;
	XMPSLong count = 1 + fContents->Count();

	XMPStorageUnit* su = this->GetStorageUnit();
	
	switch (item)
	{
		case 1:
			embeddedPart = su->GetDraft()->CreatePart(kXMPPartDrawPart,kXMPNULL);
			break;
		case 2:
			embeddedPart = su->GetDraft()->CreatePart(kXMPPartDragPart,kXMPNULL);
			break;
		case 3:
			embeddedPart = su->GetDraft()->CreatePart(kXMPPartClockPart,kXMPNULL);
			break;
		default:
			break;
	}
	Point ptTemp;
	SetPt(&ptTemp,(short)(count*20),(short)(count*20));
	externalXForm->SetQDOffset(ptTemp);
	embeddedFrame = this->CreateEmbeddedFrame(frame,
											   NULL,
											   externalXForm,
											   embeddedPart,
											   kXMPNullTypeToken,
											   kXMPNullTypeToken,
											   0,
											   kXMPFalse);
											   
	XMPFrameFacetIterator* facets = frame->CreateFacetIterator();
	for (XMPFacet* facet = facets->First(); facets->IsNotComplete(); facet = facets->Next())
	{
		XMPShape* clip = new XMPShape;
		clip->CopyFrom(embeddedFrame->GetFrameShape());
		XMPTransform* xform = new XMPTransform;
		xform->CopyFrom(this->ProxyForFrame(embeddedFrame)->transform);
		facet->CreateEmbeddedFacet(embeddedFrame, clip, xform, kXMPNULL, kXMPFrameInFront);
	}
	delete facets;
	this->ClipEmbeddedFrames(frame);
	frame->Invalidate(NULL);
	
	if (newPart)
		*newPart = embeddedPart;
	else
		XMPReleaseObject(embeddedPart);
}


void DrawPart::UpdateProxyRegion(Proxy* proxy)
{
	XMPShape* scratch = new XMPShape();
	scratch->CopyFrom(proxy->frame->GetFrameShape());
	scratch->Transform(proxy->transform);
	RgnHandle scratchRgn = scratch->GetQDRegion();
	CopyRgn(scratchRgn, proxy->region);
	delete scratch;
}

//------------------------------------------------------------------------------
// Frame Shape Transformation Utilities
//------------------------------------------------------------------------------


#ifdef BOGUSUTILITIES
//  EmbeddedToLocal 

void DrawPart::EmbeddedToLocal(XMPShape* shape, XMPFacet* embeddedFacet)
{
	XMPTransform* xform = new XMPTransform;
	xform->CopyFrom(embeddedFacet->GetExternalTransform());
	xform->Invert();
	shape->Transform(xform);
	delete xform;
}

//  LocalToEmbedded 

void DrawPart::LocalToEmbedded(XMPShape* shape, XMPFacet* embeddedFacet)
{
	shape->Transform(embeddedFacet->GetExternalTransform());
}

//  FrameToWindow 

void DrawPart::FrameToWindow(XMPShape* shape, XMPFacet* facet)
{
	XMPTransform* xform = new XMPTransform;
	xform->CopyFrom(frame->GetWindowFrameTransform());
	xform->Invert();
	shape->Transform(xform);
	delete xform;
}

//  WindowToFrame 

void DrawPart::WindowToFrame(XMPShape* shape, XMPFacet* facet)
{
	shape->Transform(frame->GetAggregateExternalTransform());
}
#endif // BOGUSUTILITIES

//  SetGrafPortOrigin 

void DrawPart::SetGrafPortOrigin(XMPFacet* facet)
{
	XMPTransform* localToGlobal = facet->GetContentTransform();
	XMPPoint offset(0,0);
	offset = localToGlobal->TransformPoint(offset);
	SetOrigin(-offset.IntX(), -offset.IntY());
}

//------------------------------------------------------------------------------
// DrawPart::HasExtension
//------------------------------------------------------------------------------

//  HasExtension 

XMPBoolean	DrawPart::HasExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	if (!XMPISOStrCompare((XMPISOStr)extensionName, semtIntf))
		return kXMPTrue;
	else
		return kXMPFalse;
}

	
//------------------------------------------------------------------------------
// DrawPart::GetExtension
//------------------------------------------------------------------------------


//  GetExtension 

XMPExtension*	DrawPart::GetExtension(XMPType extensionName)
{
	XMPISOStr	semtIntf = "SemanticInterface";
	if (!XMPISOStrCompare((XMPISOStr)extensionName, semtIntf))
		return fSemtIntf;
	else
		return kXMPNULL;
}

//==============================================================================
// DrawEmbeddedFramesIterator
//==============================================================================

XMPFrame*	DrawEmbeddedFramesIterator::First()
{
	return (XMPFrame*) fOCIter->First();
}

XMPFrame*	DrawEmbeddedFramesIterator::Next()
{
	return (XMPFrame*) fOCIter->Next();
}

XMPBoolean	DrawEmbeddedFramesIterator::IsNotComplete()
{
	return fOCIter->IsNotComplete();
}

DrawEmbeddedFramesIterator::DrawEmbeddedFramesIterator(XMPPart* part)
{
	// !!! fix this - no allocation in constructor!
	fOCIter = ((DrawPart*) part)->GetEmbeddedFrames()->CreateIterator();
}

DrawEmbeddedFramesIterator::~DrawEmbeddedFramesIterator()
{
	delete fOCIter;
}

void DrawPart::CreatePalette()
{
	if (fPalette == kXMPNULL) {
		XMPPlatformWindow	platformWindow;
		Rect				windRect;
		
		SetRect(&windRect, kPaletteLeft, kPaletteTop, kPaletteRight, kPaletteBottom);
		platformWindow = NewCWindow(kXMPNULL,
									&windRect,
									"\p",
									false,
									noGrowDocProc,
									(WindowPtr)-1L,
									true,
									kXMPNULL);
		MoveWindow(platformWindow, 100, 100, kXMPFalse);
		fPalette = this->GetStorageUnit()->GetSession()->GetWindowState()->CreateWindow(platformWindow,
									 kXMPFalse,
									 kXMPFalse,	// Is resizable
									 kXMPTrue,	// Is floating
									 kXMPFalse,	// don't save
									 this, 
									 fSession->Tokenize(kXMPViewAsFrame),
									 fSession->Tokenize(kDrawPresPalette),
									 kXMPNULL);
		fPalette->Open();
	}
}

void DrawPart::DestroyPalette()
{
	if (fPalette != kXMPNULL) {
		fPalette->CloseAndRemove();
		fPalette = kXMPNULL;
	}
}

void DrawPart::ShowPalette()
{
	if (fPaletteUp == kXMPTrue) {
		if (fPalette == kXMPNULL)
			this->CreatePalette();
		fPalette->Show();
	}
}

void DrawPart::HidePalette()
{
	if (fPalette != kXMPNULL) {
		fPalette->Hide();
	}
}

void DrawPart::UndoAction(XMPActionData actionState)
{
	// FOR NOW, WE JUST HAVE ONE ACTION THAT CAN BE UNDONE OR REDONE.
	SetBGColorRec*	undoRec = (SetBGColorRec*)actionState;
	
	this->SetBGColor(undoRec->fFrame, undoRec->fOldColor);
}

void DrawPart::RedoAction(XMPActionData actionState)
{
	// FOR NOW, WE JUST HAVE ONE ACTION THAT CAN BE UNDONE OR REDONE.
	SetBGColorRec*	undoRec = (SetBGColorRec*)actionState;
	
	this->SetBGColor(undoRec->fFrame, undoRec->fNewColor);
}

void DrawPart::DisposeActionState(XMPActionData actionState,
									   XMPDoneState doneState)
{
XMPUnused(doneState);
	// FOR NOW, WE JUST HAVE ONE ACTION THAT CAN BE UNDONE OR REDONE.
	delete (SetBGColorRec*)actionState;
}

//------------------------------------------------------------------------------
//  PRINTING  PRINTING  PRINTING  PRINTING  PRINTING  PRINTING  PRINTING 
//------------------------------------------------------------------------------

#pragma segment DrawPartPrinting
void DrawPart::CreateDefPrintRec()
{
	fPrintRecord = (THPrint) XMPNewHandle(sizeof(TPrint));
	if (!fPrintRecord)
	{
		THROW(kXMPErrOutOfMemory);
	}
	
	PrintDefault(fPrintRecord);
}

#pragma segment DrawPartPrinting
XMPBoolean DrawPart::HandlePageSetup(XMPFrame* frame)
{
XMPUnused(frame);
	PrOpen();
	ASSERTM(PrError() == noErr, PrError(), "DrawPart could not open the PageSetup dialog");
	
	if (!fPrintRecord)
	{
		this->CreateDefPrintRec();
	}
	
	if (PrStlDialog(fPrintRecord))
	{
		//  externalize contents to part su
		if (fTestDrawSU->Exists(kXMPPropPrintingInfo,kXMPMacTPrintRec,0))
			fTestDrawSU->Focus(kXMPPropPrintingInfo, kXMPPosSame, 0, 1, kXMPPosFirstSib);
		else
			fTestDrawSU->AddProperty(kXMPPropPrintingInfo)->AddValue(kXMPMacTPrintRec);
			
		XMPULong oldsize = fTestDrawSU->GetSize();
		XMPULong newsize = GetHandleSize((Handle)fPrintRecord);
		
		HLock((Handle)fPrintRecord);
		fTestDrawSU->SetOffset(0);
		fTestDrawSU->SetValue(newsize,(XMPPtr)(*fPrintRecord));
		
		if (oldsize > newsize)
			fTestDrawSU->DeleteValue(oldsize-newsize);
	}
	
	PrClose();
	
	return kXMPTrue;
}

#pragma segment DrawPartPrinting
XMPBoolean DrawPart::HandlePrinting(XMPFrame* frame)
{
	PrOpen();
	ASSERTM(PrError() == noErr, PrError(), "DrawPart could not open the PageSetup dialog");
	
	if (!fPrintRecord)
	{
		this->CreateDefPrintRec();
	}
	
	if (PrJobDialog(fPrintRecord))
	{
		this->PrintDocument(frame);
	}
	
	PrClose();
	
	return kXMPTrue;
}

#pragma segment DrawPartPrinting
XMPFacet* DrawPart::BeginPrinting(XMPFrame* rootFrame, TPPrPort thePrPort)
{	
	XMPTransform* xtransform = new XMPTransform;
	
	Rect pagerect = (**fPrintRecord).prInfo.rPage;
	
	XMPShape* clipshape = new XMPShape;
	XMPRect cliprect(pagerect);
	clipshape->SetRectangle(&cliprect);
	
	XMPFacet* prFacet = new XMPFacet;
	prFacet->InitFacet(rootFrame, clipshape, xtransform);
	
	PrInfoPtr prinfo = (PrInfoPtr) XMPNewPtr(sizeof(PrintingInfo));
	prinfo->printingPort = thePrPort;
	prinfo->printRecord = fPrintRecord;
	
	XMPCanvas* prCanvas = new XMPCanvas;
	prCanvas->InitCanvas(kXMPQuickDraw, (XMPPlatformCanvas)prinfo, kXMPFalse, kXMPFalse);
	
	prFacet->SetCanvas(prCanvas);
	rootFrame->FacetAdded(prFacet);
	
	return prFacet;
}

#pragma segment DrawPartPrinting
void DrawPart::PrintPage(XMPFacet* prFacet, XMPUShort pagenum)
{
	Rect pageSize = (**fPrintRecord).prInfo.rPage;
	XMPRect bbox;
	Rect frect;
	
	prFacet->GetFrame()->GetFrameShape()->GetBoundingBox(&bbox);
	bbox.AsQDRect(frect);
	
	XMPULong height = frect.bottom;
	XMPULong width  = frect.right;
	
	Point 		pt = {0,0};
	XMPUShort 	locator = pagenum-1;
	
	while (locator)
	{
		pt.v += (pageSize.bottom+1);
		locator--;

		if (PtInRect(pt,&frect))
		{
			continue;
		}
		else
		{
			pt.v = 0;
			pt.h += (pageSize.right+1);
		}
	}
	
	XMPTransform* xtransform = new XMPTransform;
	
	xtransform->SetQDOffset(pt);
	
	Rect pagerect = (**fPrintRecord).prInfo.rPage;
	
	XMPShape* clipshape = new XMPShape;
	XMPShape* invalshape = new XMPShape;
	XMPRect cliprect(pagerect);
	clipshape->SetRectangle(&cliprect);
	invalshape->CopyFrom(clipshape);
	clipshape->Transform(xtransform);
	//clipshape->GetBoundingBox(&bbox);	// !!!
	xtransform->Invert();
	prFacet->ChangeGeometry(clipshape, xtransform);
	
	prFacet->Update(invalshape);
}

#pragma segment DrawPartPrinting
void DrawPart::EndPrinting(XMPFacet* prFacet)
{
	XMPCanvas* prCanvas = prFacet->GetCanvas();
	
	prFacet->GetFrame()->FacetRemoved(prFacet);
	
	delete prCanvas;
	delete prFacet;
}


#pragma segment DrawPartPrinting
XMPUShort DetermineNumberOfPagesinDoc(XMPFrame* frame, Rect page)
{
	XMPRect bbox;
	Rect frect;
	
	frame->GetFrameShape()->GetBoundingBox(&bbox);
	bbox.AsQDRect(frect);
	
	XMPSLong height = frect.bottom;
	XMPSLong width  = frect.right;
	
	XMPUShort pages = 0;
	
	XMPSLong wt = width;
	while (wt > 0)
	{
		XMPSLong ht = height;
		while (ht > 0)
		{
			ht -= page.bottom;
			pages++;
		}
		
		wt -= page.right;
	}
	
	return pages;
}

#pragma segment DrawPartPrinting
void DrawPart::PrintDocument(XMPFrame* frame)
{
	GrafPtr		oldPort;
	XMPUShort	copies,
				firstPage,
				lastPage,
				numberOfCopies,
				printmgrsResFile,
 				realNumberOfPagesinDoc,
				pageNumber,
 				PrintError;
//	THPrint		thePrRecHdl;
	TPPrPort	thePrPort;
	TPrStatus	theStatus;
	XMPFacet*	prFacet;

	GetPort(&oldPort);

	PrOpen();

	if (PrError() == noErr)
	{
		/**
		Save the current resource file (i.e. the printer driver's) so
		the driver will not lose its resources upon return from the pIdleProc.
		**/
		printmgrsResFile = CurResFile();

		realNumberOfPagesinDoc = DetermineNumberOfPagesinDoc(frame,(**fPrintRecord).prInfo.rPage);

		numberOfCopies = (**fPrintRecord).prJob.iCopies;

		firstPage = (**fPrintRecord).prJob.iFstPage;
		lastPage = (**fPrintRecord).prJob.iLstPage;

		(**fPrintRecord).prJob.iFstPage = 1;
		(**fPrintRecord).prJob.iLstPage = 9999;

		if (realNumberOfPagesinDoc < lastPage) 
			lastPage = realNumberOfPagesinDoc;

		//PrintingStatusDialog = GetNewDialog(257, nil, (WindowPtr) -1);

		for (copies = 1; copies <= numberOfCopies; copies++)
		{
			// Install a pointer to your pIdle proc in my print record.
			//(**thePrRecHdl).prJob.pIdleProc = checkMyPrintDialogButton;

			UseResFile(printmgrsResFile);
			thePrPort = PrOpenDoc(fPrintRecord, nil, nil);
			
			if (PrError() == noErr)
			{
				TRY
					prFacet = this->BeginPrinting(frame, thePrPort);
					THROW_IF_NULL(prFacet);
					
					pageNumber = firstPage;
					while (pageNumber <= lastPage && PrError() == noErr)
					{
						PrOpenPage(thePrPort, nil);
	
						if (PrError() == noErr) 
						{
							this->PrintPage(prFacet, pageNumber);
						}
	
						PrClosePage(thePrPort);
						pageNumber++;
					}
					
					this->EndPrinting(prFacet);
				CATCH_ALL
					// garbage collection needs to be improved
				ENDTRY
				
			}
			
			PrCloseDoc(thePrPort);
		}
	}

	if (((**fPrintRecord).prJob.bJDocLoop == bSpoolLoop) && (PrError() == noErr))
		PrPicFile(fPrintRecord, nil, nil, nil, &theStatus);
	
	PrintError = PrError();
	
	PrClose();
	
	THROW_IF_ERROR(PrintError);
	
	//if (PrintingStatusDialog != nil)
	//	DisposDialog(PrintingStatusDialog);
	
	SetPort(oldPort);
}

