/*
	File:		DrawPart.h

	Contains:	definition for class DrawPart

	Written by:	Joshua Susser

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<37>	  4/6/94	JBS		1155477, 1155480
		<36>	 3/28/94	T		#1145834.  Remove PrtType.h & .cpp from the
									project
		<35>	 3/25/94	JA		JPA: GetPlatformTransform -> GetQDOffset
									(1153438). JBS:
									CreateEmbeddedFrameIterator, CanvasChanged
									(1153056, 1153059).
		<34>	 3/25/94	MB		Symantec ASLM fixes. #1150864
		<33>	 3/25/94	VL		#1147173: Moved constructor, destructor and
									Init methods to public by convention.
		<32>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<31>	  2/3/94	CG		Added method definitions for Edit menu
									items.
		<30>	  2/2/94	JBS		removed sourceFrame from AddDisplayFrame();
									added AttachSourceFrame()
		<29>	  2/1/94	SS		New methods/field for printing
		<28>	 1/27/94	RR		Added CreateWindow
		<27>	 1/27/94	VL		Changed CloneTo to CloneInto.
		<26>	 1/20/94	SS		::Open returns a window id
		<25>	 1/20/94	SS		Added fields for icon suites
		<24>	 1/19/94	NP		Added Undo support methods.
		<23>	 1/11/94	eeh		made private methods public so event
									handlers can use them.
		<22>	 1/11/94	T		Init... changes
		<21>	12/21/93	JBS		added ViewTypeChanged() &
									PresentationChanged(), renamed
									FrameClosed() to CloseDisplayFrame()
		<20>	12/14/93	T		renamed CommonInit to CommonInitDrawPart
									and made it nonvirtual
		<19>	12/13/93	T		InitPart changes
		<18>	 12/9/93	eeh		added CreateNewPart, CreatePartObjectSpec,
									and SetNewBounds
		<17>	 12/6/93	RR		Added fPaletteWasUp for suspend/resume
									support
		<16>	 12/1/93	CG		Added CountEmbeddedFrames, CreateNewPart,
									CreatePartOjbectSpec.
		<15>	 12/1/93	VL		Added floating window support.
		<14>	11/30/93	LM		Added an XMPFrame parameter to ReadPartInfo
		<13>	11/24/93	RR		Made PartInfoRec a class. Removed
									fIsActive. Added caches for tokenized foci.
		<12>	11/23/93	T		modifications to use the correct part
									storage model
		<11>	11/23/93	JA		Fixed include of DrawDef.h (no <>)
		<10>	11/18/93	CG		Moved kDrawPartID to DrawDef.h
		 <9>	11/16/93	CG		Moved DrawPart Object accessors to
									DrawObj.cp and DrawObj.h.
		 <8>	11/16/93	RR		Added AdjustMenus (not yet called), and
									MouseWithin. Removed point from MouseLeave
		 <7>	11/12/93	JBS		use Facets
		 <6>	10/27/93	RR		Added MouseEnterFrame/MouseLeaveFrame
		 <5>	10/21/93	PH		Change FulfillPromise
		 <4>	10/21/93	SS		Added Methods to improve HandleMouseDown
									behavior/handling
		 <3>	10/14/93	RR		Implement Open, Improve AddDisplayFrame.
									Use command number for View As Window. Add
									frame parameter to ClipEmbeddedFrames
									Added fDefaultColor, shared by all display
									frames
		 <2>	 10/8/93	VL		Added Release.
		<27>	 9/30/93	PH		Support new Drag and Drop protocol
		<26>	 9/30/93	RR		Added fFocusSet
		<25>	 9/29/93	VL		Added InitializePersistent.
		<24>	 9/14/93	NP		Added CommonInit.
		<23>	 9/10/93	NP		Added accessors to set color.
		<22>	 9/10/93	RR		Added fMenuBar
		<21>	  9/3/93	NP		Added semantic interface.
		<20>	 8/23/93	PH		Add drag handles
		<19>	  8/9/93	NP		Removed class id strings.
		<18>	 7/20/93	PH		Fix menus
		<17>	 6/23/93	RCR		Use tokens for foci
		<16>	 5/21/93	JBS		update for XMPPart namesmithing
		<15>	 4/29/93	JBS		8.3 file names
		<14>	 4/28/93	JBS		8.3 file names
		<10>	  4/7/93	JBS		add Drag&Drop routines
		 <9>	  4/6/93	TI		comment out ExternalizeContent and just use default behaviour
		 <8>	  4/6/93	JBS		add SetGrafPortOrigin()
		 <7>	  4/6/93	JBS		add ClipShapeChanged()
		 <6>	  4/5/93	TI		implement Initialize
		 <5>	  4/3/93	JBS		add Proxy and Selection handling calls
		 <4>	  4/1/93	JBS		merge changes - clipping of embedded frames
		 <3>	  4/1/93	TI		implement Read/WritePartInfo, Internalize, Externalize
		 <2>	 3/26/93	JBS		more changes...
		 <1>	 3/23/93	JBS		first checked in
		 <0>	 3/23/93	JBS		created
*/

#ifndef _DRAWPART_
#define _DRAWPART_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _SIMPLPRT_
#include "SimplPrt.h"
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __DRAWDEF__
#include "DrawDef.h"
#endif

#ifndef __PRINTING__
#include <Printing.h>
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class DrawPart;
class DrawEmbeddedFramesIterator;

//==============================================================================
// Classes used by this interface
//==============================================================================

class OrderedCollection;
class OrderedCollectionIterator;
class XMPEmbeddedFramesIterator;
class XMPFacet;
class XMPFrame;
class XMPMenuBar;
class XMPFocusSet;
class XMPSemanticInterface;
class DrawPartPropAccessor;
class XMPDragItemIterator;
class XMPWindow;
class XMPSession;

	//--------------------------------------------------------------------------
	// Class Types
	//--------------------------------------------------------------------------

	struct Proxy {
		XMPRgnHandle region;
		XMPFrame* frame;
		XMPTransform* transform;
		// XMPBoolean isSelected;
		
		Proxy(XMPRgnHandle r, XMPFrame* f, XMPTransform* t)
			{region = r; frame = f; transform = t; /* isSelected = kXMPFalse; */ }
	};

//==============================================================================
// Types
//==============================================================================

	class PartInfoRec 
	{
	public:
		PartInfoRec() { fIsActive = kXMPFalse; fNeedsActivating = kXMPFalse; }
		virtual ~PartInfoRec() { }
		RGBColor	bgColor;
		RgnHandle	bgClipRegion;
		XMPBoolean	fIsActive;	// Does this frame have its focus set?
		XMPBoolean	fNeedsActivating;	// should this be activated when its window is activated?
	};

//==============================================================================
// DrawPart
//==============================================================================

//#define kDrawPartID "appl:drawpart$class,1.0.0"

class DrawPart : public SimplePart
{
	//--------------------------------------------------------------------------
	// Class Friends
	//--------------------------------------------------------------------------

	friend class XMPDraft;
	friend class DrawEmbeddedFramesIterator;
	friend class DrawPartPropAccessor;
//	friend class EmbeddedPartPropAccessor;

public:

	// -Extension-	
	XMPMethod XMPBoolean		HasExtension(XMPType extensionName);
	XMPMethod XMPExtension*	GetExtension(XMPType extensionName);

	// -Drag&Drop-
	XMPMethod void FulfillPromise(XMPStorageUnitView *promiseSUView);
	XMPMethod void DropCompleted(XMPPart* destPart, XMPDropResult dropResult);
	XMPMethod void DragEnter(XMPDragItemIterator* dragInfo, XMPFacet* facet, XMPPoint where);
	XMPMethod void DragWithin(XMPDragItemIterator* dragInfo, XMPFacet* facet, XMPPoint where);
	XMPMethod void DragLeave(XMPFacet* facet, XMPPoint where);
	XMPMethod XMPDropResult Drop(XMPDragItemIterator* dropInfo, XMPFacet* facet, XMPPoint where);

	// -Embedding-
	XMPMethod XMPStorageUnit* GetContainingPartProperties(XMPFrame* displayFrame);
	XMPMethod XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(XMPFrame* frame);
	XMPMethod XMPSLong CountEmbeddedFrames();
	
	// -Frame-
	XMPMethod void AddDisplayFrame(XMPFrame* frame);
	XMPMethod void AttachSourceFrame(XMPFrame* frame, XMPFrame* sourceFrame);
	XMPMethod void RemoveDisplayFrame(XMPFrame* frame);
	XMPMethod void CloseDisplayFrame(XMPFrame* frame);
	XMPMethod void FrameShapeChanged(XMPFrame* frame);
	XMPMethod void ViewTypeChanged(XMPFrame* frame);
	XMPMethod void PresentationChanged(XMPFrame* frame);
	XMPMethod void WritePartInfo(XMPPtr partInfo, XMPStorageUnitView* storageUnitView);
	XMPMethod XMPPtr ReadPartInfo(XMPFrame* frame, XMPStorageUnitView* storageUnitView);
	XMPMethod XMPID Open(XMPFrame* frame); //Override
	XMPMethod XMPFrame* CreateEmbeddedFrame(XMPFrame* containingFrame,
											XMPShape* frameShape,
											XMPTransform* externalTransform,
											XMPPart* embedPart,
											XMPTypeToken viewType,
											XMPTypeToken presentation,
											XMPID frameGroupID,
											XMPBoolean isOverlaid);
	XMPMethod void RemoveEmbeddedFrame(XMPFrame* embeddedFrame);
	XMPMethod XMPShape* RequestFrameShape(XMPFrame* embeddedFrame, XMPShape* frameShape);
	XMPMethod void UsedShapeChanged(XMPFrame* embeddedFrame);
	
	// -Facet-
	XMPMethod void FacetAdded(XMPFacet* facet);
	XMPMethod void FacetRemoved(XMPFacet* facet);
	XMPMethod void GeometryChanged(XMPFacet* facet);

	// -Imaging-
	XMPMethod void Draw(XMPFacet* facet, XMPShape* invalidShape);

	// -Memory Management-
	XMPMethod XMPSize Purge(XMPSize size);

	// -Part Activation-
	XMPMethod void CommitRelinquishFocus(XMPTypeToken focus, 
										  XMPFrame* ownerFrame,
										  XMPFrame* proposedFrame);
	XMPMethod void FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame);
	XMPMethod void FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame);

	// -Storage-
	XMPMethod void Externalize();
	XMPMethod void CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU);

	// -UI Events-
	XMPMethod XMPBoolean HandleEvent(XMPEventData event, XMPFrame* frame, XMPFacet* facet);
	XMPMethod void	MouseEnter(XMPFacet* facet, XMPPoint where);
	XMPMethod void	MouseWithin(XMPFacet* facet, XMPPoint where);
	XMPMethod void	MouseLeave(XMPFacet* facet);
	XMPMethod void	AdjustMenus(XMPFrame* frame);

	// -Undo-
	XMPVMethod void UndoAction(XMPActionData actionState);
	XMPVMethod void RedoAction(XMPActionData actionState);
	XMPVMethod void DisposeActionState(XMPActionData actionState,
									   XMPDoneState doneState);

		//PRIVATE BY CONVENTION -- needed by DrawPart's object accessors
	XMPMethod XMPSemanticInterface* GetSemanticInterface();
	XMPMethod XMPPart* CreateNewPart(DescType theClass, AEDesc *theLocation ) ;
	XMPMethod void SetNewBounds( XMPFrame* frame, Rect* r ) ;
	XMPMethod Proxy* ProxyForFrame(XMPFrame* frame);
	XMPMethod void UpdateProxyRegion(Proxy* proxy);
	XMPMethod void CreateProxySelectionBorder(Proxy* p);
	XMPMethod void InvalidateSelection(XMPFrame* frame);
	XMPMethod void ClipEmbeddedFrames(XMPFrame* frame);
	
protected:

	XMPNVMethod void CommonInitDrawPart();
	XMPMethod void LoadIcons();
	
	XMPMethod void Release();
	XMPMethod void InstallObjectAccessors();
	
	// -Embedding-
	XMPMethod OrderedCollection* GetEmbeddedFrames();
//	XMPMethod Proxy* ProxyForFrame(XMPFrame* frame);
	XMPMethod Proxy* ProxyForFrameID(XMPID frameID);

	// -Imaging-
	XMPMethod void HighlightSelection();
	XMPMethod void HighlightProxyBorder(Proxy* p);
	XMPMethod void HighlightContentObject(Proxy* p);
//	XMPMethod void ClipEmbeddedFrames(XMPFrame* frame);
	XMPMethod void ClipEmbeddedFacets(XMPFacet* facet);

//	XMPMethod void InvalidateSelection(XMPFrame* frame);
	XMPMethod void EmptySelection();
//	XMPMethod void CreateProxySelectionBorder(Proxy* p);

	// -UI Events-
	XMPMethod XMPBoolean HandleMouseDown(XMPFacet* facet,
										 XMPPoint where,
										 XMPEventData event);
	XMPMethod XMPBoolean HandleMouseDownDrag(XMPFacet* facet, Proxy* selection, XMPEventData event);
	XMPMethod XMPBoolean HandleMouseDownEdgeResize(XMPFacet* facet, Proxy* selection, Point mouse);
	XMPMethod XMPBoolean HandleMouseDownCornerResize(XMPFacet* facet, Proxy* selection, Point mouse);
	XMPMethod XMPBoolean HandleMouseDownInEmbeddedFrame(XMPFacet* container,
														XMPFacet* facet,
														XMPPoint where,
														XMPEventData event);
	XMPMethod XMPBoolean HandleKeyDown(XMPFrame* focusFrame, XMPEventData event);
	XMPMethod XMPBoolean HandleMenuEvent(XMPFrame* focusFrame, XMPEventData event);
	XMPMethod XMPBoolean HandlePageSetup(XMPFrame* focusFrame);
	XMPMethod XMPBoolean HandlePrinting(XMPFrame* focusFrame);
	
	// -Printing-
	XMPMethod void	CreateDefPrintRec();
	XMPMethod XMPFacet*	BeginPrinting(XMPFrame* rootFrame, TPPrPort thePrPort);
	XMPMethod void 	PrintPage(XMPFacet* prFacet, XMPUShort page);
	XMPMethod void	EndPrinting(XMPFacet* prFacet);
	XMPMethod void	PrintDocument(XMPFrame* frame);
	
	
	// -Menu Commands-
	XMPMethod XMPBoolean DoCut(XMPFrame* frame);
	XMPMethod XMPBoolean DoCopy(XMPFrame* frame);
	XMPMethod XMPBoolean DoPaste(XMPFrame* frame);
	XMPMethod XMPBoolean DoPasteLink(XMPFrame* frame);
	XMPMethod XMPBoolean DoClear(XMPFrame* frame);
	XMPMethod XMPBoolean DoSelectAll(XMPFrame* frame);

	// -utilities-
	XMPMethod void ActivateFrame(XMPFrame* frame);
	XMPMethod void DeActivateFrame(XMPFrame* frame);
	XMPMethod void ActivatingWindow(XMPFrame* frame);
	XMPMethod void DeActivatingWindow(XMPFrame* frame);
	XMPMethod void InstallMenus(XMPFrame* aFrame);
	XMPMethod void RemoveMenus(XMPFrame* aFrame);
	XMPMethod void SetBGColor(XMPFrame* aFrame, XMPUShort whichColor);
	XMPMethod void UserSetBGColor(XMPFrame* frame, RGBColor whichColor);
	XMPMethod void SetBGColor(XMPFrame* aFrame, RGBColor whichColor);
	XMPMethod void Embed(short item, XMPFrame* frame, XMPPart** newPart=kXMPNULL);
//	XMPMethod void UpdateProxyRegion(Proxy* proxy);
//	XMPMethod void EmbeddedToLocal(XMPShape* shape, XMPFrame* embeddedFrame);
//	XMPMethod void LocalToEmbedded(XMPShape* shape, XMPFrame* embeddedFrame);
//	XMPMethod void FrameToWindow(XMPShape* shape, XMPFrame* frame);
//	XMPMethod void WindowToFrame(XMPShape* shape, XMPFrame* frame);
	XMPMethod void SetGrafPortOrigin(XMPFacet* facet);
	XMPMethod void InvalEmbedFrameAfterResize(XMPFacet* facet, Proxy* selection,
											  XMPShape* oldShape, XMPShape* newShape,
											  Point transOffset);
	XMPMethod Rect* GetSelectionRectLocal(Proxy* selection);

	XMPMethod	void CreatePalette();
	XMPMethod void DestroyPalette();
	XMPMethod	void ShowPalette();
	XMPMethod void HidePalette();
	XMPMethod void GetBGColor(XMPUShort whichColor, RGBColor* newColor);
	XMPMethod XMPWindow* CreateWindow(XMPFrame* sourceFrame);
	
private:

	OrderedCollection*	fDisplayFrames;
	OrderedCollection*	fEmbeddedFrames;
	XMPID				fWindowID;			// ID of View As Window window
	
	XMPULong			fFrameGroupIDCounter;
	Point				fDragStart;
	
	RGBColor			fDefaultColor; // Initial background color of all display frames
	OrderedCollection*	fContents;
	OrderedCollection*	fSelection;
	
	RgnHandle			fSelectRgn;
	RgnHandle			fCornerHandleRgn;
	RgnHandle			fEdgeHandleRgn;
	
	MenuHandle			fEmbedMenu;
	MenuHandle			fColorMenu;
	MenuHandle			fFrameMenu;

	XMPIcon				fLargeIcons;
	XMPIcon				fSmallIcons;

	XMPMenuBar*			fMenuBar;
	XMPFocusSet* 		fFocusSet;
	XMPTypeToken		fSelectionFocus;
	XMPTypeToken		fMenuFocus;
	XMPTypeToken		fKeyFocus;

	XMPSemanticInterface*	fSemtIntf;

	XMPStorageUnit*			fTestDrawSU;
	
	XMPWindow*			fPalette;
	XMPBoolean			fPaletteUp;
	XMPBoolean			fPaletteWasShown;
	
	XMPSession*			fSession;
	
	THPrint				fPrintRecord;	

public:	// private by convention

	// -constructor/destructor-
	DrawPart();
	virtual ~ CDECL DrawPart();
	
	XMPMethod void InitPart(XMPStorageUnit* storageUnit);
	XMPMethod void InitPartFromStorage(XMPStorageUnit* storageUnit);
};


//==============================================================================
// DrawEmbeddedFramesIterator
//==============================================================================

class DrawEmbeddedFramesIterator : public XMPEmbeddedFramesIterator
{
	friend class DrawPart;

public:

	XMPMethod XMPFrame* First();
	XMPMethod XMPFrame* Next();
	XMPMethod XMPBoolean IsNotComplete();

protected:

	DrawEmbeddedFramesIterator(XMPPart* part);
	virtual ~ CDECL DrawEmbeddedFramesIterator();

private:

	OrderedCollectionIterator* fOCIter;
};



#endif // _DRAWPART_
