/*
	File:		FocusLib.cpp

	Contains:	Library routines for focusing (setting up for drawing into a facet)

	Written by:	Jens Alfke

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):
	
		 <4>	 4/13/94	T		jpa: Stop including obsolete PolyIter.h.
									(1145831)
		 <3>	 4/13/94	SS		Fixes for Post
		 <2>	 2/17/94	JA		Include new AltPoly.h.
		 <4>	  2/2/94	JA		Made it work better with offscreen or
									PostScript canvases. Added PS clip stuff.
									Added CFocusWindow.
		 <3>	 12/7/93	NP		Changed references to qd.
		 <2>	11/30/93	JA		Rearranged #includes for new "frame"
									constant fix.
		 <1>	11/23/93	JA		first checked in
*/


#ifndef _ALTPOLY_
#include "AltPoly.h"
#endif


#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _POLYGON_
#include "Polygon.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _CANVAS_
#include "Canvas.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _FOCUSLIB_
#include "FocusLib.h"
#endif

#include <stdio.h>

#ifndef topLeft
#define topLeft(r)		(((Point *) &(r))[0])
#endif

#define bDevLaser			3		// copied from "PrPrivate.a"

#define kPostScriptBegin	190		// Picture-comments for PostScript printing
#define kPostScriptEnd		191
#define kTextIsPostScript	194


// UNDER CONSTRUCTION: These calls allow clipping to arbitrary XMPShapes
// on PostScript printers (remember that the LaserWriter driver cannot
// handle non-rectangular regions!)
// PostScript printing isn't yet implemented (there are other changes that
// need to be made to XMPCanvas and to all parts) so these calls have not
// yet been used or tested.

void
XMPBeginPostScriptClip( XMPShape *shape )
{
	XMPPolygon *poly = shape->CopyPolygon();
	
	PicComment(kPostScriptBegin, 0, kXMPNULL);
	PicComment(kTextIsPostScript, 0, kXMPNULL);
	
	DrawString("\pgrestore");
	DrawString("\pnewpath");
	
	char buf[128];
	XMPContour *cont = poly->FirstContour();
	for( long n=poly->nContours; n>0; n-- ) {
		const XMPPoint *v = cont->vertex;
		long m = cont->nVertices;
		if( m>2 ) {
			sprintf(buf,"%.2f %.2f moveto", v->x/65536.0, v->y/65536.0);
			DrawText(buf,0,strlen(buf));
			while( --m >0 ) {
				v++;
				sprintf(buf,"%.2f %.2f lineto", v->x/65536.0, v->y/65536.0);
				DrawText(buf,0,strlen(buf));
			}
		}
		DrawString("\pclosepath clip");
		cont = cont->NextContour();
	}
	
	delete poly;
	
	DrawString("\pgsave");
	PicComment(kPostScriptEnd, 0,kXMPNULL);
}


void
XMPEndPostScriptClip( )
{
	PicComment(kPostScriptBegin, 0,kXMPNULL);
	PicComment(kTextIsPostScript, 0,kXMPNULL);
	DrawString("\pgrestore");
	PicComment(kPostScriptEnd, 0,kXMPNULL);
}


/*
	void
	Focus( XMPFacet *facet )
	{
		//Focus(facet, kXMPFalse, kXMPNULL,kXMPNULL,kXMPNULL);
		
		CFocus focus(facet);
	}
	
	
	void
	FocusWindow( XMPFacet *facet )
	{
		//Focus(facet, kXMPTrue, kXMPNULL,kXMPNULL,kXMPNULL);
		
		CFocusWindow focus(facet);
	}
*/


CFocus::CFocus( XMPFacet *facet )
{
	Focus(facet, kXMPFalse, &fPort,&fOrigin,&fClip);
}


CFocusWindow::CFocusWindow( XMPFacet *facet )
{
	Focus(facet, kXMPTrue, &fPort,&fOrigin,&fClip);
}


void
CFocus::Focus( XMPFacet *facet, Boolean toWindow,
		GrafPtr *oldPort, Point *oldOrigin, RgnHandle *oldClip )
{
	GrafPtr port;
	XMPShape *newClip;
	Point offset;
	PrInfoPtr prInfo;
	
	XMPBoolean printing = !facet->GetCanvas()->IsDynamic();
	
	if (printing)
	{
		prInfo = (PrInfoPtr) facet->GetCanvas()->GetPlatformCanvas();
		
		short device = (**(prInfo->printRecord)).prStl.wDev;
		fPrintingPostScript = ((device >> 8) == bDevLaser);
	}
	else
		fPrintingPostScript = kXMPFalse;
	
	if( toWindow && !printing ) {
		port = (GrafPtr) facet->GetWindow()->GetPlatformWindow();
		newClip = facet->GetWindowAggregateClipShape();
		offset = facet->GetWindowContentTransform()->GetQDOffset();
	} else {
		if( printing ) 
			port = (GrafPtr) prInfo->printingPort;
		else
			port = (GrafPtr) facet->GetCanvas()->GetPlatformCanvas();
		newClip = facet->GetAggregateClipShape();
		offset = facet->GetContentTransform()->GetQDOffset();
	}
		
	if( oldPort )
		*oldPort = XMPASLMQDGlobals.thePort;
	if( port != XMPASLMQDGlobals.thePort )
		SetPort(port);

	if( oldOrigin )
		*oldOrigin = topLeft(XMPASLMQDGlobals.thePort->portRect);
	if( -offset.h != XMPASLMQDGlobals.thePort->portRect.left || -offset.v != XMPASLMQDGlobals.thePort->portRect.top )
		SetOrigin(-offset.h,-offset.v);
		
	if( fPrintingPostScript ) {
		XMPBeginPostScriptClip(newClip);
		if( oldClip )
			*oldClip = kXMPNULL;
	} else {
		if( oldClip ) {
			*oldClip = XMPASLMQDGlobals.thePort->clipRgn;
			XMPASLMQDGlobals.thePort->clipRgn = newClip->CopyQDRegion();
		} else
			SetClip(newClip->GetQDRegion());
	}
}

void
CFocus::Defocus( )
{
	if( fPort ) {
		if( fPrintingPostScript ) {
			XMPEndPostScriptClip();
		} else {
			if( fClip ) {
				DisposeRgn(XMPASLMQDGlobals.thePort->clipRgn);
				XMPASLMQDGlobals.thePort->clipRgn = fClip;
			} else {
				Rect r;
				SetRect(&r,-32767,-32767,32767,32767);
				ClipRect(&r);
			}
		}
		if( fOrigin.h!=XMPASLMQDGlobals.thePort->portRect.left || fOrigin.v!=XMPASLMQDGlobals.thePort->portRect.top )
			SetOrigin(fOrigin.h, fOrigin.v);
		if( XMPASLMQDGlobals.thePort!=fPort )
			SetPort(fPort);
		fPort = kXMPNULL;		// So nothing happens if this is called again
	}
}
