/*
	File:		PrtPrpAc.cpp

	Contains:	Implemementation of DrawPartPropAccessor class.

	Written by:	Nick Pilch

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	  4/6/94	JBS		1155477
		 <3>	 3/25/94	JA		GetPlatformTransform -> GetQDOffset
									(1153438).
		 <2>	 2/16/94	JA		Include new AltPoint.h.
		<14>	  2/8/94	T		Throw -> THROW & some code clean up
		<13>	  2/7/94	T		fixes to compile with PPC Headers
		<12>	 1/19/94	eeh		removed assert from GetDrawPart; fixed
									G/SetData param types
		<11>	 1/11/94	eeh		bounds property; removing "frame" from
									ospec vocabulary
		<10>	12/17/93	eeh		more work on setting bounds property of
									frame
		 <9>	12/15/93	JBS		use new clipShape calls
		 <8>	 12/9/93	eeh		implement classes AbsDrawPartPropAccessor,
									DragPartPropAccessor and FramePropAccessor
		 <7>	 12/1/93	CG		Added new class EmbeddedPartPropAccessor
									for getting pBounds and pTranslation from
									an embedded part.
		 <6>	11/22/93	SS		Added typecast for lastest headers
		 <5>	11/16/93	CG		Added some code to GetData.
		 <4>	11/12/93	JBS		use Facets
		 <3>	 11/2/93	NP		Cast kXMPNULL to proper type.
		 <2>	 9/14/93	NP		Explicitly set port to frame's window
									before calling invalidate.
		 <1>	 9/10/93	NP		first checked in

	To Do:
*/

#ifndef _ALTPOINT_
#include "AltPoint.h"			// Use C++ savvy XMPPoint and XMPRect
#endif


#ifndef _PRTPRPAC_
#include "PrtPrpAc.h"
#endif

#ifndef _DRAFT_
#include "Draft.h"
#endif

#ifndef _DRAWPART_
#include "DrawPart.h"
#endif

#ifndef _FACET_
#include "Facet.h"
#endif

#ifndef _FRAME_
#include "Frame.h"
#endif

#ifndef _SEUTILS_
#include "SEUtils.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#pragma segment PrtPrpAc

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Local Classes
//==============================================================================

//==============================================================================
// Global Variables
//==============================================================================

//==============================================================================
// Function Prototype
//==============================================================================

//==============================================================================
// AbsDrawPartPropAccessor
//==============================================================================

//------------------------------------------------------------------------------
// AbsDrawPartPropAccessor::AbsDrawPartPropAccessor
//------------------------------------------------------------------------------
AbsDrawPartPropAccessor::AbsDrawPartPropAccessor(DescType property,
		XMPObject* object)
{
	fProperty = property;
	fObject = object;
}

//------------------------------------------------------------------------------
// AbsDrawPartPropAccessor::SetData
//------------------------------------------------------------------------------
void AbsDrawPartPropAccessor::SetData(AEDesc* data)
{
	XMPUnused(data);
	THROW(errAEEventNotHandled);
}

//------------------------------------------------------------------------------
// AbsDrawPartPropAccessor::GetData
//------------------------------------------------------------------------------
void AbsDrawPartPropAccessor::GetData(AEDesc* result)
{
	XMPUnused(result);
	THROW(errAEEventNotHandled);
}

//------------------------------------------------------------------------------
// AbsDrawPartPropAccessor::GetProperty
//------------------------------------------------------------------------------
DescType AbsDrawPartPropAccessor::GetProperty()
{
	return fProperty;
}

//------------------------------------------------------------------------------
// AbsDrawPartPropAccessor::GetObject
//------------------------------------------------------------------------------
XMPObject* AbsDrawPartPropAccessor::GetObject()
{
	return fObject;
}

//==============================================================================
// DrawPartPropAccessor
//==============================================================================

//------------------------------------------------------------------------------
// DrawPartPropAccessor::DrawPartPropAccessor
//------------------------------------------------------------------------------

DrawPartPropAccessor::DrawPartPropAccessor(DescType property, DrawPart* part)
	:AbsDrawPartPropAccessor(property, part)
{
#ifdef TO_BE_DELETED
	fProperty = property;
	fPart = part;
#endif // TO_BE_DELETED
//	fContainedPart = kXMPNULL;
}

//------------------------------------------------------------------------------
// DrawPartPropAccessor::SetData
//------------------------------------------------------------------------------

void DrawPartPropAccessor::SetData(AEDesc* data)
{
//	DescType	gotType;
//	Size		actualSize;
//	XMPError	result;

	switch(this->GetProperty())
	{
		case pColor:
		{
			RGBColor color;
			if (data->descriptorType == typeRGBColor)
			{
				color = **(RGBColor**)data->dataHandle;
			}
			else
			{
				AEDesc colorDesc;
				THROW_IF_ERROR(AECoerceDesc(data, typeRGBColor, &colorDesc));
				color = **(RGBColor**)colorDesc.dataHandle;
				AEDisposeDesc(&colorDesc);
			}

			OrderedCollectionIterator iter(this->GetDrawPart()->fDisplayFrames);
			for (XMPFrame* frame = (XMPFrame*)iter.First()
					;iter.IsNotComplete()
					;frame = (XMPFrame*)iter.Next())
			{
				this->GetDrawPart()->SetBGColor(frame, color);
				frame->Invalidate((XMPShape*)kXMPNULL);
			}
			break;
		}
		default:
			THROW(errAECantSupplyType);
			break;
	}
}	// DrawPartPropAccessor::SetData()

//------------------------------------------------------------------------------
// DrawPartPropAccessor::GetData
//------------------------------------------------------------------------------

void DrawPartPropAccessor::GetData(AEDesc* objectData)
{
	XMPError	result;
	
	switch(this->GetProperty())
	{
		case pName:
		{
			// this is the draw part for now
			unsigned char* name = "\pDrawPart";
			THROW_IF_ERROR(AECreateDesc(typeChar, (Ptr)&name[1], name[0],
					objectData));
		}
			break;

		case pBounds:
		{
			OrderedCollectionIterator iter(this->GetDrawPart()->fDisplayFrames);
			XMPFrame* frame = (XMPFrame*)iter.First();
			XMPShape* shape = frame->GetFrameShape();
		//	if(shape->IsRectangular())
			{
				XMPRect r;
				shape->GetBoundingBox(&r);
				Rect qdR;
				r.AsQDRect(qdR);
				result = AECreateDesc(typeQDRectangle, (Ptr)&qdR, sizeof(qdR),
						objectData);
				THROW_IF_ERROR(result);
			}
		}
			break;

		case pColor:
		{
			OrderedCollectionIterator iter(this->GetDrawPart()->fDisplayFrames);
				// For now, assume first frame
			XMPFrame* frame = (XMPFrame*)iter.First();

			PartInfoRec* pInfo = (PartInfoRec *) frame->GetPartInfo();
			RGBColor theColor = pInfo->bgColor;
			
#ifdef TO_BE_DELETED
			for (XMPFrame* frame = (XMPFrame*)iter.First()
					;iter.IsNotComplete()
					;frame = (XMPFrame*)iter.Next())
			{
				PartInfoRec* pInfo = (PartInfoRec *) frame->GetPartInfo();
				theColor = pInfo->bgColor;
			}
			
#endif // TO_BE_DELETED

			result = AECreateDesc(typeRGBColor, (Ptr)&theColor, sizeof(theColor), objectData);
			THROW_IF_ERROR(result);

			break;
		}
		default:
			THROW(errAECantSupplyType);
			break;
	}
}

//------------------------------------------------------------------------------
// DrawPartPropAccessor::GetDrawPart
//------------------------------------------------------------------------------
DrawPart* DrawPartPropAccessor::GetDrawPart()
	
{
	return (DrawPart*)AbsDrawPartPropAccessor::GetObject();
}


#ifdef TO_BE_DELETED
//------------------------------------------------------------------------------
// DrawPartPropAccessor::SetContainedPart
//------------------------------------------------------------------------------
void DrawPartPropAccessor::SetContainedPart(XMPPart* containedPart)
{
	fContainedPart = containedPart;
}

//------------------------------------------------------------------------------
// DrawPartPropAccessor::GetContainedPart
//------------------------------------------------------------------------------
XMPPart* DrawPartPropAccessor::GetContainedPart()
{
	return fContainedPart;
}
#endif // TO_BE_DELETED

//==============================================================================
// EmbeddedPartPropAccessor
//==============================================================================

//------------------------------------------------------------------------------
// EmbeddedPartPropAccessor::DrawPartPropAccessor
//------------------------------------------------------------------------------

EmbeddedPartPropAccessor::EmbeddedPartPropAccessor(DescType property,
		XMPFrame* frame, DrawPart* me)
	:AbsDrawPartPropAccessor(property, frame)
{
	fDrawPart = me;
}

//------------------------------------------------------------------------------
// EmbeddedPartPropAccessor::GetThisPart
//------------------------------------------------------------------------------

DrawPart* EmbeddedPartPropAccessor::GetThisPart()
{
	return fDrawPart;
}

//------------------------------------------------------------------------------
// EmbeddedPartPropAccessor::SetData
//------------------------------------------------------------------------------

static void MoveFacetTransform(XMPFacet* facet, Point* desiredLocation,
		XMPPart* part)
{
	XMPTransform* newXForm = new XMPTransform;
	newXForm->CopyFrom(facet->GetExternalTransform());
	Point curOffset = newXForm->GetQDOffset();
	Point scratch = *desiredLocation;
	scratch.h -= curOffset.h;
	scratch.v -= curOffset.v;

	XMPPoint newDelta = *desiredLocation = scratch;
	newXForm->MoveBy(newDelta);
	facet->ChangeGeometry(kXMPNULL, newXForm);
	facet->Invalidate(facet->GetClipShape());
}

void EmbeddedPartPropAccessor::SetData(AEDesc* data)
{
	XMPError	result;

	XMPFrame* targetFrame = this->GetFrame();		// the *contained* frame

	switch(this->GetProperty())
	{
		case pTranslation:
		{
		
				// Get the old shape to invalidate
			XMPShape* oldShape = targetFrame->GetFrameShape()->Copy();
			
				// Make a new shape and set bounds
			XMPShape* newShape = new XMPShape();
		
			AEDesc dPoint;
			result = AECoerceDesc(data, typeQDPoint, &dPoint); 
			THROW_IF_ERROR(result);
			Point thePoint = **(Point**)dPoint.dataHandle;
			AEDisposeDesc(&dPoint);

				// Make a new shape and set bounds
			XMPTransform* newTrans = new XMPTransform();
			newTrans->SetQDOffset(thePoint);
			newShape->Transform(newTrans);

			XMPFrameFacetIterator* facetIterator =
					targetFrame->CreateFacetIterator();
			for (XMPFacet* facet = facetIterator->First();
				facetIterator->IsNotComplete();
				facet = facetIterator->Next())
			{
				// invalidate old frame shape
				//oldShape->Transform(selection->transform);
				targetFrame->Invalidate(oldShape);
			
				XMPShape* newShapeCopy = new XMPShape;
				newShapeCopy->CopyFrom(newShape);		// save to invalidate later
			
				targetFrame->ChangeFrameShape(newShape);	// already in frame coords
				facet->ChangeGeometry(kXMPNULL, newTrans);

				// invalidate changed areas
				//newShapeCopy->Transform(selection->transform);
				//facet->GetFrame()->Invalidate(newShapeCopy);
				targetFrame->Invalidate(newShapeCopy);
				delete newShapeCopy;
			
		//		DrawPart* thisPart = (DrawPart*)targetFrame->GetPart();
				this->GetThisPart()->InvalidateSelection(targetFrame);
				//fPart->UpdateProxyRegion(selection);
				//fPart->CreateProxySelectionBorder(selection);
				this->GetThisPart()->ClipEmbeddedFrames(targetFrame);
				this->GetThisPart()->InvalidateSelection(targetFrame);
				
				this->GetThisPart()->GetStorageUnit()->GetDraft()->
						SetChangedFromPrev();
			}	
			delete oldShape;
			break;
		}
		case pBounds:
		{
			// Get the new rect
			AEDesc dRect;
			result = AECoerceDesc(data, typeQDRectangle, &dRect); 
			THROW_IF_ERROR(result);
			Rect usersRect = **(Rect**)dRect.dataHandle;
			AEDisposeDesc(&dRect);
			
			Point pt = *(Point*)&usersRect;

			// make the rect 0-based
			OffsetRect(&usersRect, -usersRect.left, -usersRect.top);
			
			XMPShape* newShape = new XMPShape;
			newShape->CopyFrom(targetFrame->GetFrameShape());
			XMPRect usersXMPRect = usersRect;
			newShape->SetRectangle(&usersXMPRect);
			targetFrame->ChangeFrameShape(newShape);

			XMPFrameFacetIterator* fi = this->GetFrame()->CreateFacetIterator();
			XMPFacet* facet = fi->First();
			MoveFacetTransform(facet, &pt, this->GetFrame()->GetPart());
	
			Proxy* p = this->GetThisPart()->ProxyForFrame(targetFrame);
	
			// !!! this is a single-facet hack
			p->transform->MoveBy(pt);		
			this->GetThisPart()->UpdateProxyRegion(p);
			this->GetThisPart()->CreateProxySelectionBorder(p);
	
			targetFrame->GetContainingFrame()->Invalidate((XMPShape*)kXMPNULL);

			break;

#ifdef TO_BE_DELETED
			XMPFrame* frame = this->GetFrame();
			XMPFrameFacetIterator* fi =
					new XMPFrameFacetIterator(frame);
			XMPFacet* facet = fi->First();
			WASSERT((fi->Next(), !fi->IsNotComplete()));
			XMPTransform* transform = facet->GetExternalTransform();

			// NOTE: this is VERY dangerous, as I'm assuming that I'm contained
			// in a DrawPart.  Won't work when other parts can contain! <eeh>

			XMPFrame* outerFrame = frame->GetContainingFrame();
			DrawPart* outerPart = (DrawPart*)outerFrame->GetPart();
			Proxy* p = outerPart->ProxyForFrame(frame);

			Point newOffset = *(Point*)&usersRect;	//for the new xform
			Point curOffset = p->transform->GetQDOffset();
			
			newOffset.h -= curOffset.h;
			newOffset.v -= curOffset.v;
			short usersRight = usersRect.right - usersRect.left;
			short usersBottom = usersRect.bottom - usersRect.top;
			Rect xformedRect;		// goes into the new shape
			SetRect(&xformedRect, 0, 0, usersRight, usersBottom);

			XMPTransform* newXform = new XMPTransform;
			newXform->CopyFrom(transform);
			XMPPoint xmpPt = newOffset;
			newXform->MoveBy(xmpPt);
			facet->ChangeGeometry(kXMPNULL, newXform);
			
			// !!! this is a single-facet hack
			p->transform->MoveBy(newOffset);
			XMPTransform* newExternalXForm = new XMPTransform;
			newExternalXForm->CopyFrom(p->transform);
			XMPFrameFacetIterator* facets = p->frame->CreateFacetIterator();
			facets->First()->ChangeGeometry(kXMPNULL, newExternalXForm);
			delete facets;
			
			outerPart->UpdateProxyRegion(p);
			outerPart->CreateProxySelectionBorder(p);

			XMPShape* shape = new XMPShape();
			XMPRect rect = xformedRect;
			shape->SetRectangle(&rect);
			frame->ChangeFrameShape(shape);
			facet->Invalidate(facet->GetClipShape());
			facet->ChangeGeometry(facet->GetFrame()->GetFrameShape()->Copy(), kXMPNULL);
			facet->GetContainingFacet()->Invalidate(kXMPNULL);

			break;
#endif // TO_BE_DELETED
		}
		default:
			THROW(errAECantSupplyType);
			break;
	}
}	// EmbeddedPartPropAccessor::SetData()

//------------------------------------------------------------------------------
// EmbeddedPartPropAccessor::GetData
//------------------------------------------------------------------------------

void EmbeddedPartPropAccessor::GetData(AEDesc* objectData)
{
	XMPError	result;
	XMPFrame* frame = this->GetFrame();

	switch(this->GetProperty())
	{
		case pTranslation:
		{
			WASSERT(false);		// should not be here
			Point thePoint = frame->GetInternalTransform()->GetQDOffset();

			result = AECreateDesc(typeQDPoint, (Ptr)&thePoint, sizeof(thePoint), objectData);
			THROW_IF_ERROR(result);

			break;
		}
		case pBounds:
		{
				// make a copy of frame's shape
			XMPShape* shape = new XMPShape();
			shape->CopyFrom(frame->GetFrameShape());

			XMPFrameFacetIterator* fi = new XMPFrameFacetIterator(frame);
			XMPFacet* facet = fi->First();
			WASSERT((fi->Next(), !fi->IsNotComplete()));
			delete fi;
			XMPTransform* transform = facet->GetExternalTransform();
			shape->Transform(facet->GetExternalTransform());

				// get the bounding box of the shape
			XMPRect theBounds;
			shape->GetBoundingBox(&theBounds);
			delete shape;

				// send it back to the user
			Rect qdRect;
			theBounds.AsQDRect(qdRect);
			result = AECreateDesc(typeQDRectangle, (Ptr)&qdRect,
					sizeof(qdRect), objectData);
			THROW_IF_ERROR(result);
			break;
		}
		default:
			THROW(errAECantSupplyType);
			break;
	}
}	// EmbeddedPartPropAccessor::GetData

//------------------------------------------------------------------------------
// EmbeddedPartPropAccessor::GetPart
//------------------------------------------------------------------------------

XMPFrame* EmbeddedPartPropAccessor::GetFrame()
{
	return (XMPFrame*)AbsDrawPartPropAccessor::GetObject();
}

//==============================================================================
// FramePropAccessor
//==============================================================================

//------------------------------------------------------------------------------
// FramePropAccessor::DrawPartPropAccessor
//------------------------------------------------------------------------------

#ifdef TO_BE_DELETED
FramePropAccessor::FramePropAccessor(DescType property, XMPFrame* frame)
	:AbsDrawPartPropAccessor(property, frame)
{
}
#endif // TO_BE_DELETED

//------------------------------------------------------------------------------
// FramePropAccessor::GetFrame
//------------------------------------------------------------------------------
#ifdef TO_BE_DELETED
XMPFrame* FramePropAccessor::GetFrame()
{
	return (XMPFrame*)AbsDrawPartPropAccessor::GetObject();
}
#endif // TO_BE_DELETED

//------------------------------------------------------------------------------
// FramePropAccessor::SetData
//------------------------------------------------------------------------------

#ifdef TO_BE_DELETED
void FramePropAccessor::SetData(AEDesc* data)
{
	DescType	gotType;
	XMPSize		actualSize;
	XMPError	result;
	Point 		thePoint = { 0, 0 };

			
	switch(this->GetProperty())
	{
		case pBounds:
		{
			// Get the new rect
			AEDesc dRect;
			result = AECoerceDesc(data, typeQDRectangle, &dRect); 
			THROW_IF_ERROR(result);
			Rect usersRect = **(Rect**)dRect.dataHandle;
			AEDisposeDesc(&dRect);

			XMPFrameFacetIterator* fi =
					new XMPFrameFacetIterator(this->GetFrame());
			XMPFacet* facet = fi->First();
			WASSERT((fi->Next(), !fi->IsNotComplete()));
			XMPTransform* transform = facet->GetExternalTransform();

			// NOTE: this is VERY dangerous, as I'm assuming that I'm contained
			// in a DrawPart.  Won't work when other parts can contain! <eeh>

			XMPFrame* outerFrame = this->GetFrame()->GetContainingFrame();
			DrawPart* outerPart = (DrawPart*)outerFrame->GetPart();
			Proxy* p = outerPart->ProxyForFrame(this->GetFrame());

			Point newOffset = *(Point*)&usersRect;	//for the new xform
	//		Point curOffset = transform->GetQDOffset();
			Point curOffset = p->transform->GetQDOffset();
			
			newOffset.h -= curOffset.h;
			newOffset.v -= curOffset.v;
			short usersRight = usersRect.right - usersRect.left;
			short usersBottom = usersRect.bottom - usersRect.top;
			Rect xformedRect;		// goes into the new shape
			SetRect(&xformedRect, 0, 0, usersRight, usersBottom);

			XMPTransform* newXform = new XMPTransform;
			newXform->CopyFrom(transform);
			XMPPoint xmpPt = newOffset;
			newXform->MoveBy(xmpPt);
			facet->ChangeGeometry(kXMPNULL, newXform);

			
			// !!! this is a single-facet hack
			p->transform->MoveBy(newOffset);
			XMPTransform* newExternalXForm = new XMPTransform;
			newExternalXForm->CopyFrom(p->transform);
			XMPFrameFacetIterator* facets = p->frame->CreateFacetIterator();
			facets->First()->ChangeGeometry(kXMPNULL, newExternalXForm);
			delete facets;
			
			outerPart->UpdateProxyRegion(p);
			outerPart->CreateProxySelectionBorder(p);

			XMPShape* shape = new XMPShape();
			XMPRect rect = xformedRect;
			shape->SetRectangle(&rect);
			this->GetFrame()->ChangeFrameShape(shape);
			facet->Invalidate(facet->GetClipShape());
			facet->ChangeGeometry(facet->GetFrame()->GetFrameShape()->Copy(), kXMPNULL);
			facet->GetContainingFacet()->Invalidate(kXMPNULL);
			
#ifdef TO_BE_DELETED
			this->GetFrame()->GetPart()->Draw(facet, shape);
			XMPFrame* containerF = this->GetFrame()->GetContainingFrame();
			if (containerF)
			{
				XMPPart* container = containerF->GetPart();
				XMPEmbeddedFramesIterator* fi =
						container->CreateEmbeddedFramesIterator();
				for(XMPFrame* frm = fi->First(); fi->IsNotComplete();
						frm = fi->Next())
				{
					frm->Invalidate(frm->GetUsedShape());
					//container->Draw((XMPFacet*)kXMPNULL,
					//		(XMPShape*)kXMPNULL);
				}
			}
#endif // TO_BE_DELETED


#ifdef TO_BE_DELETED
			// xform it using the current external xform
			XMPFrameFacetIterator* fi =
					new XMPFrameFacetIterator(this->GetFrame());
			XMPFacet* facet = fi->First();
			WASSERT((fi->Next(), !fi->IsNotComplete()));
			XMPShape* shape = new XMPShape();
			XMPRect rect = usersRect;
			shape->SetRectangle(&rect);
			XMPTransform* transform = facet->GetExternalTransform();
			shape->Transform(transform);
			
			XMPRect transformedRect;
			shape->GetBoundingBox(&transformedRect);

			// subtract transformed from incoming to get new pt, and make
			// it into the new transform
			Point pt;
			pt.h = (short)(transformedRect.left - transformedRect.left);
			pt.v = (short)(transformedRect.top - transformedRect.top);
			XMPTransform* newXform = new XMPTransform;
			newXform->CopyFrom(transform);
			XMPPoint xmpPt = pt;
			newXform->MoveBy(xmpPt);
			facet->ChangeGeometry(kXMPNULL, newXform);

			// use xformed rect to set new shape (shape is already set up?)
			// That is, set the facet's shape to be the new one?
			this->GetFrame()->ChangeFrameShape(shape);
			facet->ResetClipShape();
#endif // TO_BE_DELETED

			break;
		}
		default:
			THROW(errAEEventNotHandled);
			break;
	}
}
#endif // TO_BE_DELETED

//------------------------------------------------------------------------------
// FramePropAccessor::GetData
//------------------------------------------------------------------------------

#ifdef TO_BE_DELETED
void FramePropAccessor::GetData(AEDesc* objectData)
{
	switch(this->GetProperty())
	{
		case pBounds:
		{
			XMPShape* theShape = this->GetFrame()->GetFrameShape();
			XMPRect bounds;
			theShape->GetBoundingBox(&bounds);
			Rect qdBounds;
			bounds.AsQDRect(qdBounds);
			THROW_IF_ERROR(AECreateDesc(typeQDRectangle, (Ptr)&qdBounds,
					sizeof(qdBounds), objectData));
			break;

#ifdef TO_BE_DELETED
			XMPRect theBounds;
			XMPFrameFacetIterator* fi =
					new XMPFrameFacetIterator(this->GetEmbeddedFrame());
			XMPFacet* facet = fi->First();
			WASSERT((fi->Next(), !fi->IsNotComplete()));

				// make a copy of frame's shape
			XMPShape* shape = new XMPShape();
			shape->CopyFrom(this->GetEmbeddedFrame()->GetFrameShape());
			
			XMPTransform* transform = facet->GetExternalTransform();
			shape->Transform(transform);
			
				// get the bounding box of the shape
			shape->GetBoundingBox(&theBounds);
			delete shape;

			Rect r;
			theBounds.AsQDRect(r);
			THROW_IF_ERROR(AECreateDesc(typeQDRectangle, (Ptr)&r,
					sizeof(r), objectData));
#endif // TO_BE_DELETED
		}
		default:
			THROW(errAEEventNotHandled);
			break;
	}
}

#endif // TO_BE_DELETED
