/*
	File:		PrtPrpAc.h

	Contains:	Interface to class that handles accessing part properties for
				the OpenDoc shell.

	Written by:	Nick Pilch

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	 1/19/94	eeh		fixed G/SetData param types
		 <5>	 1/11/94	eeh		nuke FramePropAccessor class; mods to
									others
		 <4>	 12/9/93	eeh		new classes AbsDrawPartPropAccessor,
									DragPartPropAccessor and FramePropAccessor
		 <3>	 12/1/93	CG		Added new class EmbeddedPartPropAccessor
									for getting pBounds and pTranslation from
									an embedded part.
		 <2>	11/16/93	CG		Includes DrawPart.h
		 <1>	 9/10/93	NP		first checked in

	To Do:
*/

#ifndef _PRTPRPAC_
#define _PRTPRPAC_

#ifndef _DRAWPART_
#include "DrawPart.h"
#endif

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _PROPACSR_
#include "PropAcsr.h"
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This a simple abstract class to aid in the processing of Apple Events that
	are requests to get or set properties. Since OpenDoc does the resolution
	first, the object accessor routines can allocate one of these classes
	according to the property for which they have been asked to create a token.
	When passed back to the event handler, the event handler need only call the
	the appropriate member function of this class, GetData or SetData, to
	process the event. The event handler must then delete the object.
*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Classes defined in this interface
//==============================================================================

class DrawPartPropAccessor;
class EmbeddedPartPropAccessor;
class DragPartPropAccessor;

//==============================================================================
// Classes used by this interface
//==============================================================================

class DrawPart;
class DragPart;

//==============================================================================
// PropAccessor
//==============================================================================

class AbsDrawPartPropAccessor : public PropAccessor
{
	public:
		AbsDrawPartPropAccessor(DescType property, XMPObject* object);
		XMPVMethod void SetData(AEDesc* data);
		XMPVMethod void GetData(AEDesc* result);
	
		XMPMethod DescType GetProperty();

	protected :
		XMPMethod XMPObject* GetObject() ;

	private:
		XMPObject*	fObject;
		DescType	fProperty;
};

class DrawPartPropAccessor : public AbsDrawPartPropAccessor
{
  public:

	DrawPartPropAccessor(DescType property, DrawPart* part);

	XMPMethod void SetData(AEDesc* data);
	XMPMethod void GetData(AEDesc* result);
	
	XMPMethod DrawPart* GetDrawPart() ;
};

class EmbeddedPartPropAccessor : public AbsDrawPartPropAccessor
{
	public:
		EmbeddedPartPropAccessor( DescType property, XMPFrame* frame,
			DrawPart* me ) ;
		
		XMPMethod void SetData(AEDesc* data);
		XMPMethod void GetData(AEDesc* result);
		
		XMPMethod XMPFrame* GetFrame() ;
		XMPMethod DrawPart* GetThisPart() ;

	private:
		DrawPart* fDrawPart ;
};

#ifdef TO_BE_DELETED
class FramePropAccessor : public AbsDrawPartPropAccessor
{
  public:

	FramePropAccessor(DescType property, XMPFrame* frame);

	XMPMethod void SetData(AEDesc* data);
	XMPMethod void GetData(AppleEvent* reply);

	XMPMethod XMPFrame* GetFrame() ;
};
#endif // TO_BE_DELETED

class DragPartPropAccessor : public PropAccessor
{
  public:

	DragPartPropAccessor(DescType property, DragPart* part);

	XMPVMethod void SetData(AEDesc* data);
	XMPVMethod void GetData(AEDesc* result);

  private:

	DescType	fProperty;
	DragPart*	fPart;
};

#endif // _PRTPRPAC_