/*
	File:		CanvasM.h

	Contains:	Definitions of XMPMacCanvas

	Written by:	Joshua Susser & Sepp Friedrich

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):
			
		 <9>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <8>	  2/8/94	JA		Include <FixMath.h> before GX header to
									avoid conflicts.
		 <7>	  2/7/94	JA		The Makeover Continues!
		 <6>	  2/3/94	JA		Tiger Team Makeover!
		 <5>	 1/24/94	SS		Checkin for Joshua (see below)
		 <4+>	 1/13/94	JBS		update region support, new getters
	   								Init... changes
		 <4>	10/18/93	NP		Added class id string.
		 <3>	 9/20/93	JA		Added QD/GX fields.
		 <10>	  7/12/93	RCR		Help it to compile
		 <1>	  7/6/93	JBS		first checked in
		 <7>	  5/7/93	JBS		add Purge()
		 <6>	  5/3/93	JBS		namesmithing
		 <5>	 4/29/93	JBS		8.3 file names
		 <4>	 4/28/93	JBS		8.3 file names
		 <3>	 3/23/93	SF		subclass of XMPObject
		 <2>	 1/26/93	SF		header files
		 <1>	 1/16/93	SF		first checked in
*/

#ifndef _CANVASM_
#define _CANVASM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"			// base class
#endif

#ifndef _PLFMTYPE_
#include "PlfmType.h"
#endif

#ifndef __FIXMATH__
#include <FixMath.h>			// Must include before GX headers...
#endif
#ifndef graphicsTypesIncludes
#include <graphics types.h>		// Another fine GX header; needed only for gxViewPort.
#endif
						  
//==============================================================================
// Classes used in this interface
//==============================================================================

class XMPPart;
class XMPFacet;
class XMPShape;

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPMacCanvas;

//==============================================================================
// XMPMacCanvas
//==============================================================================

#define kXMPMacCanvasID "appl:xmpmaccanvas$class,1.0.0"

class XMPMacCanvas : public XMPAbsCanvas
{

public:

	//---------------------------------
	// overrides

	XMPVMethod XMPSize Purge(XMPSize size);		// override from XMPObject

	//---------------------------------
	// constructor & destructor

	XMPMacCanvas();
			  
	XMPVMethod ~ CDECL XMPMacCanvas();

	XMPNVMethod void InitCanvas(XMPGraphicsSystem graphicsSystem,
			 	 XMPPlatformCanvas	platformCanvas,
			 	 XMPBoolean 		isDynamic,
			 	 XMPBoolean 		isOffscreen);
			  
	//---------------------------------
	// getters

	XMPMethod XMPGraphicsSystem	GetGraphicsSystem();
	
	XMPMethod XMPPlatformCanvas	GetPlatformCanvas();
	
	XMPMethod XMPPart* GetOwner();
	XMPMethod void SetOwner(XMPPart* owner);
	
	XMPMethod XMPFacet* GetFacet();
	XMPMethod void SetFacet(XMPFacet* facet);
	
	XMPMethod XMPBoolean		IsDynamic();
	
	XMPMethod XMPBoolean		IsOffscreen();
		
	//---------------------------------
	// update region

	XMPMethod XMPShape* GetUpdateShape();

	XMPMethod void ResetUpdateShape();

	XMPMethod void Invalidate(XMPShape* shape);

	XMPMethod void Validate(XMPShape* shape);

	XMPMethod void CheckUpdateShape();
	 
private:

	XMPPart*			fOwner;
	XMPFacet*			fFacet;
	XMPShape*			fUpdateShape;
	XMPBoolean			fIsDynamic; 	
	XMPBoolean			fIsOffscreen;
	XMPGraphicsSystem	fGraphicsSystem;
	union{
		XMPPlatformCanvas	fPlatformCanvas;
		GrafPtr				fQDPort;
		gxViewPort			fGXPort;
	};
};

#endif // _CANVASM_