/*
	File:		ErrorDef.h

	Contains:	Error codes for OpenDoc

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<37>	 3/28/94	CG		1152929: Added kXMPInvalidFileContents
									error.
		<36>	 3/25/94	JA		Added kXMPErrTransformErr,
									kXMPErrInvalidParameter,
									kXMPErrAssertionFailed (1153438)
		<35>	 3/18/94	CC		Added kXMPErrInvalidClipboardKey. (1151855)
		<34>	 3/14/94	NP		Added kXMPNoError.
		<33>	 2/22/94	VL		Added kXMPErrMemoryProblem.
		<32>	  2/7/94	NP		Exemplar -> OpenDoc
		<31>	  2/2/94	CC		Added kXMPErrInvalidCloneKind.
		<30>	 1/31/94	JA		Added kXMPErrShapeTooComplex.
		<29>	 1/26/94	RR		Added Undo errors
		<28>	 1/21/94	CC		Added kXMPErrInvalidLinkKey and
									kXMPErrDocNotSaved.
		<27>	 1/13/94	JBS		added kXMPErrInvalidTransform
		<26>	12/20/93	SS		Added error for Part init failure
		<25>	12/16/93	CC		Added three linking errors.
		<24>	 12/3/93	T		Renamed from XMPError.h to ErrorDef.h
		<23>	11/23/93	PH		Add DragDrop errors
		<22>	11/23/93	JA		Added kXMPErrNoShapeGeometry.
		<21>	11/15/93	CC		Added kXMPErrCorruptLinkSpecValue
		<20>	11/15/93	NP		Added kXMPErrInvalidKey, kXMPErrReadErr and
									kXMPErrWriteErr.
		<19>	11/12/93	JBS		add Facet errors
		<18>	11/10/93	CC		Added errors for links
		<17>	 11/4/93	PH		Add promise error
		<16>	10/14/93	RR		Added a couple
		<15>	 10/7/93	VL		Added a lot of Storage System Error codes.
		<14>	 9/15/93	T		changed 'const's to #defines, and added
									kXMPUndefined.  note: this file is included
									in ShellM.r, so nothing but #defines ok?
		<13>	 9/08/93	jpa		Added kXMPErrInvalidGraphicsSystem & kXMPErrSingularTransform.
		<12>	  9/3/93	JBS		added kXMPErrCannotCreateLayout
		<11>	 8/13/93	VL		Added kXMPErrZeroRefCount.
		<10>	  8/9/93	VL		Added more storage exception codes.
		 <9>	 6/30/93	VL		More error codes for Storage.
		 <8>	 6/10/93	NP		Added kXMPErrPartMissing, XMPError
									kXMPErrExtensionMissing.
		 <7>	  6/4/93	NP		Added kXMPErrKeyAlreadyExists.
		 <6>	  6/2/93	NP		Added XMPErrWrongType.
		 <5>	  6/1/93	VL		Added some Bento errors (starting at 100).
		 <4>	 4/29/93	VL		8.3 Name Change.
		 <3>	 4/26/93	NP		Added kXMPErrKeyDoesNotExist.
		 <2>	 4/19/93	VL		Added kXMPErrBentoErr.
		 <1>	 2/17/93	VL		first checked in

	To Do:
*/

/*
	This include file contains a list of all exception codes that can be
	signeled within OpenDoc. For now, the exception codes are just an 
	enumerated list. In the future, we may want to re-categorize them.
*/

#ifndef _ERRORDEF_
#define _ERRORDEF_

//==============================================================================
// Constants
//==============================================================================
		
#define kXMPNoError							0

#define kXMPErrUndefined					1
#define kXMPErrCannotResize					2
#define kXMPErrInvalidContainer				3
#define kXMPErrInvalidDocument				4
#define kXMPErrInvalidDraft					5
#define kXMPErrInvalidStorageUnit			6
#define kXMPErrCannotEmbed					7
#define kXMPErrInvalidPart					8
#define kXMPErrInvalidFrame					9
#define kXMPErrInvalidWindow				10
#define kXMPErrInvalidWindowState			11
#define kXMPErrInvalidSystemInterface		12
#define kXMPErrInvalidFocusSet				13
#define kXMPErrInvalidFocusModule			14
#define kXMPErrInvalidDispatchModule		15
#define kXMPErrInvalidType					16
#define kXMPErrTypeExists					17
#define kXMPErrInvalidName					18
#define kXMPErrNameExists					19
#define kXMPErrInvalidID					20
#define kXMPErrInvalidToken					21
#define kXMPErrInvalidExtension				22
#define kXMPErrInvalidValue					23
#define kXMPErrInvalidProperty				24
#define kXMPErrInvalidPosCode				25
#define kXMPErrInvalidValueIndex			26
#define kXMPErrKeyDoesNotExist				27
#define kXMPErrKeyAlreadyExists				28
#define kXMPErrWrongType					29
#define kXMPErrPartMissing					30
#define kXMPErrExtensionMissing				31
#define kXMPErrCannotAddProperty			32
#define kXMPErrCannotAddType				33
#define kXMPErrUnfocusedStorageUnit			34
#define kXMPErrInvalidPermissions			35
#define kXMPErrInvalidVersion				36
#define kXMPErrCannotCreateDraftVersion		37
#define kXMPErrInvalidBelowDraft			38
#define kXMPErrDraftNotExists				39
#define kXMPErrContainerExists				40
#define kXMPErrCannotCollapseDrafts			41
#define kXMPErrNonEmptyDraft				42
#define kXMPErrInvalidContainerType			43
#define kXMPErrContainerNotExists			44
#define kXMPErrDraftExistsWithDifferentID	45
#define kXMPErrNoPreviousDraft				46
#define kXMPErrInvalidPropertyName			47
#define kXMPErrInvalidSUCursor				48
#define kXMPErrVersionListInternalized		49
#define kXMPErrNoVersionList				50
#define kXMPErrNoDraftProperties			51
#define kXMPErrInvalidRefCount				52
#define kXMPErrCannotCreateFrame			53
#define kXMPErrCannotCreateWindow			54
#define kXMPErrCannotGetPersistentObject	55
#define kXMPErrOutstandingDraft				56
#define kXMPErrZeroRefCount					57
#define kXMPErrCannotCreateLayout			58
#define kXMPErrInvalidGraphicsSystem		59
#define kXMPErrSingularTransform			60
#define kXMPErrOutOfMemory					61
#define kXMPErrInvalidDraftID				62
#define kXMPErrVersionListUnavailable		63
#define kXMPErrSemaphoreReleased 			64
#define kXMPErrCannotChangePermissions		65
#define	kXMPErrInvalidVersionList			66
#define	kXMPErrNameNotExists				67
#define	kXMPErrCannotGetFrame				68
#define	kXMPErrCannotCreatePart				69
#define	kXMPErrCannotGetPart				70
#define	kXMPErrCannotGetWindow				71
#define	kXMPErrCannotGetLayout				72
#define	kXMPErrInvalidStorageUnitRef		73
#define	kXMPErrInvalidIDList				74
#define	kXMPErrStorageUnitLocked			75
#define	kXMPErrInvalidStorageUnitKey		76
#define	kXMPErrStorageUnitNotLocked			77
#define kXMPErrBentoErr						78
#define kXMPErrBentoNoSession				79
#define kXMPErrBentoInvalidProperty			80
#define kXMPErrBentoInvalidType				81
#define kXMPErrBentoCannotNewObject			82
#define kXMPErrBentoCannotNewValue			83
#define kXMPErrBentoCannotCreateContainer	84
#define kXMPErrBentoCannotOpenContainer		85
#define	kXMPErrBentoInvalidObject 			86
#define kXMPErrNotImplemented				87
#define kXMPErrIteratorOutOfSync 			88
#define kXMPErrFocusAlreadyRegistered		89
#define kXMPErrFocusNotRegistered			90
#define kXMPErrFocusIsExclusive				91
#define kXMPErrFocusIsNonExclusive			92
#define kXMPErrUnknownWindow				93
#define kXMPErrNotPromise					94
#define kXMPErrCannotGetExternalLink		95
#define kXMPErrCannotCreateLink				96
#define kXMPErrNoLinkSpecValue				97
#define kXMPErrUnknownLinkSpecVersion		98
#define kXMPErrInvalidCanvas				99
#define kXMPErrCorruptLinkSpecValue			100
#define kXMPErrInvalidFacet					101
#define kXMPErrInvalidPositionCode			102
#define kXMPErrInvalidKey					103
#define kXMPErrReadErr						104
#define kXMPErrWriteErr						105
#define kXMPErrNoDragManager				106
#define kXMPErrNoDragSystemStorage			107
#define kXMPErrDragItemNotFound				108
#define kXMPErrCannotAllocateDragItem		109
#define kXMPErrUnknownDragImageType			110
#define kXMPErrDragTrackingException		111
#define kXMPErrNoShapeGeometry				112		// Shape has no polygon representation
#define kXMPErrNotExportedLink				113
#define kXMPErrNotImportedLink				114
#define kXMPErrCannotGetLink				115
#define kXMPErrCannotInitializePart			116
#define kXMPErrInvalidTransform				117
#define kXMPErrInvalidLinkKey				118
#define kXMPErrDocNotSaved					119
#define kXMPErrCannotMarkAction				120
#define kXMPErrEmptyStack					121
#define kXMPErrNoBeginAction				122
#define kXMPErrCannotAddAction				123
#define kXMPErrInvalidPlatformWindow 		124
#define kXMPErrShapeTooComplex				125		// Polygon shape is too huge for QuickDraw
#define kXMPErrInvalidCloneKind				126
#define kXMPErrMemoryProblem				127		// bad heap or damaged object
#define kXMPErrInvalidClipboardKey			128
#define kXMPErrTransformErr					129		// Illegal op on xform
#define kXMPErrInvalidParameter				130		// Invalid fn param
#define kXMPErrAssertionFailed				131		// Internal error
#define kXMPErrInvlaidFileContents			132		// bad name space file data

#endif // _ERRORDEF_
