/*
	File:		FocusItr.h

	Contains:	Interface to XMPFocusSetIterator class.

	Written by:	Richard Rodseth

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <7>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <6>	 2/17/94	RR		#1143609 Made destructor public again
		 <5>	 2/14/94	RR		#1143609. Marked init methods private to UI
		 <4>	  2/9/94	NP		Tiger Team cleanup.
		 <3>	  2/7/94	NP		Tiger Team doings.
		 <2>	12/23/93	RR		Added Init method. Remove arguments from
									constructor
		 <1>	 12/8/93	RR		first checked in

	To Do:
*/

#ifndef _FOCUSITR_
#define _FOCUSITR_

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPFocusSetIterator;		// Iterator over the elements in a focus set

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPFocusSet;				
class LinkedListIterator;

//=====================================================================================
// Class XMPFocusSetIterator
//=====================================================================================

#define kXMPFocusSetIteratorID "appl:xmpfocussetiterator$class,1.0.0"

class XMPFocusSetIterator
{
	ASLM_COMPATIBLE

public:
	
	virtual ~ CDECL XMPFocusSetIterator();

	XMPMethod XMPTypeToken First();
	
	XMPMethod XMPTypeToken Next();
	
	XMPMethod XMPBoolean IsNotComplete();

public: // Private to UI subsystem

	XMPFocusSetIterator();
	
	XMPNVMethod void InitFocusSetIterator(XMPFocusSet* focusSet);
	
private:
	
	LinkedListIterator* fIterator;
};

#endif // _FOCUSITR_