/*
	File:		FocusMod.h

	Contains:	Interface to XMPFocusModule class.

	Written by:	Richard Rodseth

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <7>	 2/17/94	RR		#1144732 Added forward declaration of
									XMPFocusOwnerIterator
		 <6>	 2/14/94	RR		#1143609. Marked Begin/Commit/Abort methods
									private to UI
		 <5>	  2/9/94	NP		Tiger Team cleanup.
		 <4>	  2/7/94	NP		Tiger Team doings.
		 <3>	  2/7/94	NP		Tiger Team doings.
		 <2>	12/23/93	RR		Added Init method. Removed arguments from
									constructor
		 <1>	 12/8/93	RR		first checked in

	To Do:
*/

#ifndef _FOCUSMOD_
#define _FOCUSMOD_

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPFocusModule;

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPSession;
class XMPFrame;
class XMPFocusOwnerIterator;

//=====================================================================================
// Class XMPFocusModule
//=====================================================================================

class XMPFocusModule
{
public:

	XMPFocusModule();
	
	XMPVMethod ~XMPFocusModule();
	
	XMPNVMethod void InitFocusModule(XMPSession* session);
	
	XMPVMethod XMPBoolean IsFocusExclusive(XMPTypeToken focus)
		= 0;
	
	XMPVMethod void SetFocusOwnership(XMPTypeToken focus, XMPFrame* frame)
		= 0;

	XMPVMethod void UnsetFocusOwnership(XMPTypeToken focus, XMPFrame* frame)
		= 0;
	
	XMPVMethod XMPFrame* GetFocusOwner(XMPTypeToken focus)
		= 0;
	
	XMPVMethod XMPFocusOwnerIterator* CreateOwnerIterator(XMPTypeToken focus)
		= 0;	

public: // Private to UI subsystem

	XMPMethod XMPBoolean BeginRelinquishFocus(XMPTypeToken focus, 
												XMPFrame* requestingFrame);
							 
	XMPMethod void CommitRelinquishFocus(XMPTypeToken focus,
											XMPFrame* requestingFrame);
							 
	XMPMethod void AbortRelinquishFocus(XMPTypeToken focus,
										  XMPFrame* requestingFrame);

protected:

	XMPSession* fSession;	// Access to globals
};

#endif // _FOCUSMOD_