/*
	File:		LinkSpeM.h

	Contains:	Class definition for XMPMacLinkSpec.

	Written by:	Craig Carper 

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <8>	 3/24/94	CC		Changed WriteToProperty() to
									WriteLinkSpec(), ReadFromProperty() to
									ReadLinkSpec(). (1153049)
		 <7>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <6>	  2/7/94	JA		Removed spurious 'XMPMacLinkSpec::' from
									defn of InitLinkSpec.
		 <5>	 1/11/94	T		Init... changes
		 <4>	 12/8/93	CC		Added	WriteToAppleEvent,
									ReadFromAppleEvent, and accessors for
									fields.
		 <3>	11/23/93	CC		Removed redundant #include
		 <2>	11/19/93	PH		Add virtual keyword and fix for ASLM
		 <1>	11/15/93	CC		first checked in
	To Do:
*/

#ifndef _LINKSPEM_
#define _LINKSPEM_

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef __PROCESSES__
#include "Processes.h"
#endif

#ifndef __APPLEEVENTS__
#include "AppleEvents.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This file describes the Macintosh version of link specs.
*/

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPMacLinkSpec;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPStorageUnit;
class XMPDraft;
class XMPPart;

//==============================================================================
// XMPMacLinkSpec
//==============================================================================

#define kXMPMacLinkSpecID "appl:xmpmaclinkspec$class,1.0.0"

class XMPMacLinkSpec : public XMPAbsLinkSpec
{
	ASLM_COMPATIBLE
	
public:

	// ----------------------------------------
	// Parts may call

	XMPMacLinkSpec();
	
	virtual ~ CDECL XMPMacLinkSpec();
	
	XMPVMethod void WriteLinkSpec(XMPStorageUnit* su);

		// Write this object into the pre-focused storage unit.
		// The argument storage unit must be pre-focused to a property.
	
	XMPVMethod void ReadLinkSpec(XMPStorageUnit* su);
	
		// Initialize this object from the value in the pre-focused storage unit.
		// The argument storage unit must be pre-focused to a property containing
		// a link spec value.

	// ----------------------------------------
	// Private by convention

	XMPNVMethod void InitLinkSpec(
			XMPPart*	part,
			XMPPtr		data,
			XMPULong	size);
	
	XMPVMethod ProcessSerialNumber GetProcessID();

		// Returns the process serial number of the originating part

	XMPVMethod XMPULong GetProcessLaunchDate();

		// Returns the launch date of the originating process

	XMPVMethod XMPPart* GetPart();

		// Returns the originating part
		
	XMPVMethod XMPULong	GetDataSize();

		// Returns the size of the link spec data

	XMPVMethod XMPPtr GetDataPtr();

		// Returns a pointer to the link spec data

	XMPVMethod void WriteToAppleEvent(AppleEvent* theAppleEvent);
	
		// Write this object into the argument apple event.

	XMPVMethod void ReadFromAppleEvent(const AppleEvent* theAppleEvent);
	
		// Initialize this object from the argument apple event.

private:

	void VoidLinkSpec();

	// identification of the process creating this link spec
	// unique during a single boot of the local machine
	ProcessSerialNumber fProcessID;
	XMPULong fProcessLaunchDate;		// for validation of fProcessID

	// identification of the source part - only valid during the current session!
	XMPPart* fPart;	// Session-wide identification for the source part

	// Private data for eventual use by the source part
	XMPULong fDataSize;
	XMPPtr fData;
};

#endif
