/*
	File:		MenuBarM.h

	Contains:	Interface to XMPMacMenuBar class.

	Written by:	Richard Rodseth

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<13>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<12>	  2/9/94	NP		Tiger Team cleanup.
		<11>	  2/7/94	NP		Tiger Team doings.
		<10>	 1/21/94	RR		Updated method visibility
		 <9>	 1/11/94	T		Init... changes
		 <8>	12/22/93	RR		XMPMethod Usage
		 <7>	12/20/93	RR		New init strategy
		 <6>	10/29/93	RR		Added several methods for setting menu
									appearance
		 <5>	10/20/93	eeh		Add EnDisableCommand()
		 <4>	 9/22/93	RR		Derive from RefCntObj.
		 <3>	 9/16/93	CG		Added virtual keyword to XMPMacMenuBar for
									ASLM compatibility.
		 <2>	 9/15/93	RR		Added fCommandTable
		 <1>	  9/9/93	RR		first checked in

	To Do:
*/

// MenuBarM.h

#ifndef _MENUBARM_
#define _MENUBARM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif


//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPMenuBar;

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPSession;
class XMPPart;
class OrderedCollection;
class CommandTable;

//=====================================================================================
// Class XMPMacMenuBar
//=====================================================================================

#define kXMPMacMenuBarID "appl:xmpmacmenubar$class,1.0.0"

class XMPMacMenuBar : public XMPAbsMenuBar
{
public:

	XMPMacMenuBar();
	
	virtual ~ CDECL XMPMacMenuBar();
	
	XMPNVMethod void InitMenuBar(XMPSession* session, XMPPlatformMenuBar menuBar);

	XMPMethod void Release();

	XMPMethod XMPSize Purge(XMPSize size);
	
	XMPMethod void Display();
	
	XMPMethod void AddMenuLast(XMPMenuID menuID, XMPPlatformMenu menu, XMPPart* part);

	XMPMethod void AddMenuBefore(XMPMenuID menuID, XMPPlatformMenu menu, XMPPart* part, XMPMenuID beforeID);

	XMPMethod void AddSubMenu(XMPMenuID menuID, XMPPlatformMenu menu, XMPPart* part);
		
	XMPMethod void RemoveMenu(XMPMenuID menu);	

	XMPMethod XMPPlatformMenu GetMenu(XMPMenuID menu);	

	XMPMethod XMPMenuBar* Copy();	

public: // Macintosh-specific

	XMPMethod void	RegisterCommand(XMPCommandID command,
									XMPMenuID menu,
									XMPMenuItemID menuItem);
									
	XMPMethod void UnregisterCommand(XMPCommandID command);
	
	XMPMethod void UnregisterAll();	
	
	XMPMethod XMPBoolean IsCommandRegistered(XMPCommandID command);

	XMPMethod XMPBoolean IsCommandSynthetic(XMPCommandID command);		
	
	XMPMethod XMPCommandID GetCommand(XMPMenuID menu,
									XMPMenuItemID menuItem);
									
	XMPMethod void GetMenuAndItem(XMPCommandID command,
									XMPMenuID& menu,
									XMPMenuItemID& menuItem);
		
	XMPMethod void EnableCommand(XMPCommandID cmdNumber, XMPBoolean enable);

	XMPMethod void CheckCommand(XMPCommandID cmdNumber, XMPBoolean check);

	XMPMethod void EnableAndCheckCommand(XMPCommandID cmdNumber, XMPBoolean enable, XMPBoolean check) ;

	XMPMethod void SetItemString(XMPCommandID cmdNumber, ConstStr255Param itemString);

	XMPMethod void GetItemString(XMPCommandID cmdNumber, Str255 itemString);

protected:
	
	XMPSession* fSession;

	 XMPPlatformMenuBar fSystemMenus;
	 OrderedCollection* fOtherMenus;	
	 
	 CommandTable* fCommandTable; // a mapping from (menu/item) to command number  
};

#endif // _MENUBARM_