/*
	File:		MssgIntM.h

	Contains:	Mac implementation of messaging system class interface.

	Written by:	Caia Grisar

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<23>	 3/26/94	NP		1137665. Predispatch proc.
		<22>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<21>	  2/9/94	NP		Tiger Team cleanup.
		<20>	  2/7/94	NP		Tiger Team doings.
		<19>	 1/17/94	NP		Removed consts from public API.
		<18>	 1/17/94	NP		Renamed some private methods for clarity.
		<17>	 1/14/94	NP		Init changes.
		<16>	 1/11/94	eeh		made DispatchToEventHandler public (temp)
		<15>	12/20/93	NP		Support for last chance shell handling of
									events.
		<14>	11/30/93	NP		Fooling with private parts
		<13>	 11/5/93	NP		Added private methods.
		<12>	 11/2/93	NP		Moved pascal keyword to different position.
		<11>	10/22/93	NP		Moved DispatchToEventHandler and
									DispatchToCoercionHandler to here as
									private methods.
		<10>	10/18/93	NP		Removed code name.
		 <9>	10/11/93	NP		Mods to stay in sync with MssgIntf.h.
		 <8>	 9/20/93	NP		Fooling with coercion handlers. Added
									private members.
		 <7>	 8/16/93	CG		Removed ResetTimer, SuspendTheCurrentEvent,
									ResumeTheCurrentEvent, GetTheCurrentEvent,
									SetTheCurrentEvent, GetInteractionAllowed,
									SetInteractionAllowed, and InteractWithUser
									from the API.
		 <6>	  8/9/93	NP		Added class id string.
		 <5>	 7/29/93	NP		Global variable change for ASLM.
		 <4>	 7/26/93	CG		Added CreatePartAddrDesc method.
		 <3>	 7/23/93	CG		Added coercion handling.
		 <2>	 7/12/93	CG		Added fSessions and HandleAllSEventsAux.
		 <1>	  7/2/93	CG		first checked in

	To Do:
*/

#ifndef _MSSGINTM_
#define _MSSGINTM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _SEMTINTF_
#include "SemtIntf.h"
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPMacMessageInterface;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPPart;
class XMPShape;
class XMPSETransactionList;
class XMPSession;
class XMPNameResolver;
class OrderedCollection;

//==============================================================================
// XMPMessageInterface
//==============================================================================

#define kXMPMacMessageInterfaceID "appl:xmpmacmessageinterface$class,1.0.0"

class XMPMacMessageInterface : public XMPAbsMessageInterface
{
  public:

	XMPVMethod XMPSShort			CreateEvent(AEEventClass	theAEEventClass,
										AEEventID		theAEEventID,
										AEAddressDesc*	target,
										XMPSLong		transactionID,
										AppleEvent*		result);

	XMPVMethod void CreatePartAddrDesc(AEDesc*		theAddressDesc,
											XMPPart*	thePart);

	XMPVMethod void CreatePartObjSpec(AEDesc* theObjSpec, XMPPart* thePart);

	XMPVMethod void				Send(XMPPart*					thePart,
									AppleEvent*				theAppleEvent,
									AppleEvent*				reply,
									AESendMode				sendMode,
									AESendPriority			sendPriority,
									XMPULong				timeOutInTicks,
									XMPIdleProcPtr			idleProc,
									XMPEventFilterProcPtr	filterProc);

	XMPVMethod XMPBoolean	ProcessSemanticEvent(XMPEventData theEvent);
	
	XMPVMethod XMPSize		Purge(XMPSize howMuch);

	// <eeh> made public for Move event.  Is there another way?
	// <NP> Yes. Call Send.
	OSErr	DispatchToEventHandler(AppleEvent*	message,
											AppleEvent* reply,
											XMPPart*	contextPart);

  public: // Private by convention

	XMPMacMessageInterface();
	XMPVMethod ~ CDECL XMPMacMessageInterface();
	XMPNVMethod void InitMessageInterface(XMPSession* session);
	
	XMPNVMethod OrderedCollection*	GetPreHandlers() {return fPreHandlers;}
	XMPNVMethod void				PreHandlerAdded(XMPPart*			part,
													XMPPreDispatchUPP	proc,
													XMPSLong			refCon);
	XMPNVMethod void				PreHandlerRemoved(XMPPart*			part,
													XMPPreDispatchUPP	proc,
													XMPSLong			refCon);

  private:

	static OSErr pascal	HandleAllSEvents(AppleEvent* message,
											AppleEvent* reply,
											long refCon);
	OSErr				HandleAllSEventsAux(AppleEvent* message,
											AppleEvent* reply);

	static OSErr pascal	HandleReplies(AppleEvent* message,
											AppleEvent* reply,
											long refCon);
	OSErr				HandleRepliesAux(AppleEvent* message,
											AppleEvent* reply);

	static OSErr pascal	HandlePreDispatch(AppleEvent* message,
											AppleEvent* reply,
											long refCon);

	static OSErr pascal HandleAllCoercions(	AEDesc		theAEDesc,
											DescType	toType,
											long		refCon,
											AEDesc*		retDesc);
	OSErr 				HandleAllCoercionsAux(	AEDesc		theAEDesc,
												DescType	toType,
												AEDesc*		retDesc);
	OSErr 	DispatchToCoercionHandler(AEDesc theAEDesc, DescType toType,
										XMPPart* sendee, AEDesc* retDesc);

	OSErr	DispatchEventWithToken(XMPOSLToken*	token,
									AppleEvent*	message,
									AppleEvent*	reply,
									XMPBoolean	usingSubjectAttr);
	XMPBoolean	HandleSpecialEvents(AppleEvent* message,
														AppleEvent* reply,
														OSErr&	error);
//	OSErr	DispatchToEventHandler(AppleEvent*	message,
//											AppleEvent* reply,
//											XMPPart*	contextPart);
	OSErr	DispatchToEventHandlerAux(AppleEvent* message, AppleEvent* reply,
										XMPPart* sendee);
	OSErr	GiveShellATry(AppleEvent* message, AppleEvent* reply, OSErr error);
#if 0
	static pascal OSErr HandleListCoercions(AEDesc		theAEDesc,
											DescType	toType,
											long		handlerRefCon,
											AEDesc*		retDesc);
	OSErr 				HandleListCoercionsAux(	AEDesc		theAEDesc,
												DescType	toType,
												AEDesc*		retDesc);

	static pascal OSErr HandleRecordCoercions(	AEDesc		theAEDesc,
												DescType	toType,
												long		handlerRefCon,
												AEDesc*		retDesc);
	OSErr 				HandleRecordCoercionsAux(	AEDesc		theAEDesc,
													DescType	toType,
													AEDesc*		retDesc);
#endif // 0
  private:
	
	XMPSETransactionList*	fTransactionList;
	XMPSShort				fNextReturnID;
	XMPSession* 			fSession;
	XMPNameResolver*		fNameResolver;
	OrderedCollection*		fPreHandlers;
};

#endif // _MSSGINTM_