/*
	File:		NamRslvM.h

	Contains:	XMPMacNameResolver class.

	Written by:	Caia Grisar

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<20>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<19>	  2/9/94	NP		Tiger Team cleanup.
		<18>	  2/7/94	NP		Tiger Team doings.
		<17>	 1/14/94	NP		Init changes.
		<16>	 1/11/94	eeh		removed XMPPart* param from CreateSwapToken
		<15>	12/20/93	NP		Support for last chance shell handling of
									events.
		<14>	12/15/93	NP		Added GetContextInfo and
									GetContainingFrame.
		<13>	 11/5/93	NP		Added CallObjectAccessor. Removed return
									value from Resolve.
		<12>	 11/2/93	NP		Fiddling with private parts.
		<11>	10/28/93	NP		Added contextTable private field for
									storing contexts.
		<10>	10/22/93	NP		Added private method and field.
		 <9>	10/11/93	NP		Added private method.
		 <8>	 9/20/93	NP		Added parameter to Resolve. Added some
									private members.
		 <7>	 8/19/93	NP		Changed parameters to MakeToken and
									MakePartToken.
		 <6>	 8/18/93	NP		Changed parameters to Resolve.
		 <5>	 8/16/93	NP		Adjusted for latest OSL proposal.
		 <4>	  8/9/93	NP		Added class id string.
		 <3>	 7/28/93	NP		Tweaked interfaces.
		 <2>	 7/21/93	NP		Changed interface to Resolve.
		 <1>	  7/2/93	CG		first checked in

	To Do:
*/

#ifndef _NAMRSLVM_
#define _NAMRSLVM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPMacNameResolver;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPPart;
class OrderedCollection;
class XMPMessageInterface;
class XMPSession;
class XMPContext;
class XMPFrame;
class XMPSemanticInterface;
class XMPAEHashTable;

//==============================================================================
// Structures used by this interface
//==============================================================================

struct OSLContext;

//==============================================================================
// XMPMacNameResolver
//==============================================================================

#define kXMPMacNameResolverID "appl:xmpmacnameresolver$class,1.0.0"

class XMPMacNameResolver : public XMPAbsNameResolver
{
  public:

	XMPVMethod void			Resolve(XMPObjectSpec*			theObject, 
								  	XMPOSLToken*	token,
									XMPPart*		contextPart);

	XMPVMethod void			CreateSwapToken(XMPOSLToken*	token,
													XMPFrame*	frame);

	XMPVMethod void			CreateToken(XMPOSLToken*		token,
										DescType			type,
										XMPPart*			inPart);

	XMPVMethod XMPFrame*	GetContainingFrame();

	XMPVMethod void		GetContextInfo(XMPFrame** frame, XMPPart** part);

	XMPVMethod void				CallObjectAccessor(
												XMPPart*			thePart,
												DescType		desiredClass,
												XMPOSLToken*	containerToken,
											 	DescType		containerClass,
											 	DescType		keyForm,
												AEDesc*			keyData,
												XMPOSLToken*	token);

	XMPVMethod void			DisposeToken(XMPOSLToken* theToken);

	XMPVMethod XMPSize		Purge(XMPSize howMuch);

  public: // private by convention

	XMPMacNameResolver();
	XMPVMethod ~ CDECL XMPMacNameResolver();
	XMPNVMethod void InitNameResolver(XMPSession* session);
	
	XMPSemanticInterface*	GetSemtIntf(XMPPart* part);
	XMPPart*				GetPartFromToken(XMPOSLToken* token);
	XMPPart*				GetCurrentContextPart();
	void					SetCurrentContextPart(XMPPart*);
	void					SetLastContainer(DescType container)
									{fLastContainer = container;}
	void					SetLastObject(DescType object)
									{fLastObject = object;}
	DescType				GetLastContainer() {return fLastContainer;}
	DescType				GetLastObject() {return fLastObject;}
	void					ClearLastSwapFrame() ;

  private:

	OSLContext*		GetContextForPart(XMPPart* part);
	OSLContext*		CreateContext(XMPPart* part);
	void			GetContextFromAnyTokenInList(XMPOSLToken*		token,
									  				OSLContext**	context);
	void			CreateTokenCommon(XMPOSLToken*	token,
													XMPPart*	part);

	OSLContext*			fRootContext;
//	OrderedCollection*	fPartContexts;
	XMPBoolean			fCurrentlyResolving;
	XMPPart*			fCurrentContextPart;
	XMPSession*			fSession;
	XMPAEHashTable*		fContextTable;
	XMPFrame*			fLastSwapFrame;
	DescType			fLastContainer;
	DescType			fLastObject;
};

#endif // _NAMRSLVM_