/*
	File:		NamRslvr.h

	Contains:	XMPNameResolver class.

	Written by:	Nick Pilch

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<19>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<18>	  2/9/94	NP		Tiger Team cleanup.
		<17>	  2/7/94	NP		Tiger Team doings.
		<16>	 1/14/94	NP		Init changes.
		<15>	 1/11/94	eeh		removed XMPPart* param from CreateSwapToken
		<14>	12/20/93	NP		Support for last chance shell handling of
									events.
		<13>	12/15/93	NP		Added GetContextInfo and
									GetContainingFrame.
		<12>	 11/5/93	NP		Added CallObjectAccessor.
		<11>	 9/20/93	NP		Added parameter to Resolve.
		<10>	 8/25/93	NP		Included SemtIntf.h
		 <9>	 8/19/93	NP		Changed parameters to MakeToken and
									MakePartToken.
		 <8>	 8/18/93	NP		Changed parameters to Resolve.
		 <7>	 8/16/93	NP		Adjusted for latest OSL proposal.
		 <6>	 7/28/93	NP		Tweaked interfaces.
		 <5>	 7/21/93	NP		Fooling with.
		 <4>	  7/2/93	CG		Abstract/Concrete breakout.
		 <3>	 4/28/93	NP		File name changes.
		 <2>	 4/23/93	NP		Added Initialize and Purge.
		 <1>	 4/13/93	NP		first checked in

	To Do:
*/

#ifndef _NAMRSLVR_
#define _NAMRSLVR_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPAbsNameResolver;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPPart;
class XMPFrame;

//==============================================================================
// Structures used by this interface
//==============================================================================

class XMPOSLToken;

//==============================================================================
// XMPAbsNameResolver
//==============================================================================

class XMPAbsNameResolver : public XMPObject
{
  public:

	XMPVMethod void			Resolve(XMPObjectSpec*			theObject, 
								 	 XMPOSLToken*	token,
									 XMPPart*		contextPart) 
		= 0;

	XMPVMethod void			CreateSwapToken(XMPOSLToken*	token,
													XMPFrame*	frame)
		= 0;
	
	XMPVMethod void			CreateToken(XMPOSLToken*		token,
										DescType			type,
										XMPPart*			inPart)
		= 0;

	XMPVMethod XMPFrame*	GetContainingFrame()
		= 0;

	XMPVMethod void			GetContextInfo(XMPFrame** frame, XMPPart** part)
		= 0;

	XMPVMethod void			CallObjectAccessor(
												XMPPart*		part,
												DescType		desiredClass,
												XMPOSLToken*	containerToken,
											 	DescType		containerClass,
											 	DescType		keyForm,
												AEDesc*			keyData,
												XMPOSLToken*	token)
		= 0;

	XMPVMethod void			DisposeToken(XMPOSLToken* theToken) 
		= 0;

	XMPVMethod XMPSize		Purge(XMPSize howMuch) 
		= 0;
	
  public: // Private by convention

	XMPAbsNameResolver() {}
	XMPVMethod ~ CDECL XMPAbsNameResolver() {}
};

#ifdef PLATFORM_MACINTOSH
#ifndef _NAMRSLVM_
#include "NamRslvM.h"
#endif
#endif

#endif // _NAMRSLVR_