/*
	File:		PfTypLs.h

	Contains:	Class definition for XMPPlatformTypeList.

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <1>	  4/4/94	CC		first checked in

		------------------- CC ---- Renamed from PfTypSt.h

		 <5>	 3/16/94	CG		#1151186:Added InitPlatformTypeSet().
		 <4>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <3>	 2/15/94	CC		Bug #1142949 - Standardize method names,
									remove Copy method.
		 <2>	  2/9/94	T		made destructor explicitly virtual to
									pacify ASLM
		 <1>	  2/4/94	VL		first checked in

	To Do:

*/

#ifndef _PFTYPLS_
#define _PFTYPLS_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================
class XMPPlatformTypeList;

//=====================================================================================
// Classes used by this interface
//=====================================================================================
class OrderedCollection;

//=====================================================================================
// Class XMPPlatformTypeList
//=====================================================================================

#define kXMPPlatformTypeListID "appl:xmpplatformtypelist$class,1.0.0"

class XMPPlatformTypeList
{
	ASLM_COMPATIBLE

	friend class XMPPlatformTypeListIterator;
	
public:

	XMPPlatformTypeList();
	
	virtual ~ CDECL XMPPlatformTypeList();
		
	XMPMethod void InitPlatformTypeList(XMPPlatformTypeList* typeList);
	
	XMPMethod void AddLast(XMPPlatformType type);
	XMPMethod void Remove(XMPPlatformType type);
	XMPMethod XMPBoolean Contains(XMPPlatformType type);
	XMPMethod XMPULong Count();
		
	XMPVMethod XMPPlatformTypeListIterator* CreatePlatformTypeListIterator();

private:

	OrderedCollection*	fList;
};

#endif	// _PFTYPLS_