/*
	File:		SemtIntM.h

	Contains:	Mac implementation of SemanticInterface class

	Written by:	Caia Grisar

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<23>	  4/4/94	T		#1154941: Get OpenDoc source to actually
									compile for PowerPC.  Fixed
									CallXMPTokenInquiryProc.
		<22>	  4/1/94	T		#1154941: Get OpenDoc source to actually
									compile for PowerPC.  Moved typedefs of
									UPPs to PlfmType.h where Platform Specific
									black box types belong.
		<21>	 3/26/94	NP		1153509. Messaging not PowerPC ready.
		<20>	 3/26/94	NP		1137665. Predispatch proc.
		<19>	 3/25/94	eeh		bug #1153053: goofed last time.
		<18>	 3/24/94	eeh		bug #1153053: Changes for PPC nativity (or
									is it nativeness...?)
		<17>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<16>	  2/9/94	NP		Tiger Team cleanup.
		<15>	  2/7/94	NP		Tiger Team doings.
		<14>	 1/14/94	NP		Init changes.
		<13>	11/30/93	NP		Added token inquiry proc.
		<12>	 11/5/93	NP		Removed CallObjectAccessor to
									XMPNameResolver.
		<11>	10/22/93	NP		Added new private types for implementation.
		<10>	10/18/93	NP		InstallErrDescProc -> InstallGetErrDescProc
									and removed code name.
		 <9>	10/11/93	NP		Mods to keep in sync with SemtIntf.h. Added
									new private instance variables.
		 <8>	 9/20/93	NP		Moved some members around.
		 <7>	 8/19/93	NP		Added context field. Some moving of things.
		 <6>	 8/18/93	NP		Added OSLSupportFlags getters and setters.
		 <5>	  8/9/93	NP		Added class id string.
		 <4>	 7/28/93	NP		Added some temporary code.
		 <3>	 7/27/93	CG		Replaced the hash table
									fSpecialHandlerTable with fields in
									SemtIntf for each of the possible special
									handler proc ptrs.
		 <2>	 7/23/93	CG		Added Coercion handler dispatching.
		 <1>	  7/2/93	CG		first checked in

	To Do:
*/

#ifndef _SEMTINTM_
#define _SEMTINTM_


#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _EXTENSN_
#include "Extensn.h"
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class	XMPMacSemanticInterface;
class	XMPAEHashTable;

//==============================================================================
// Classes used by this interface
//==============================================================================

class	XMPPart;
class	XMPMessageInterface;

//==============================================================================
// XMPMacSemanticInterface
//==============================================================================

#if USESROUTINEDESCRIPTORS
enum {
	uppXMPEventHandlerProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(AppleEvent*)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(AppleEvent*)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPDescCoercionHandlerProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(AEDesc*)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(DescType)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPSLong)))
		 | STACK_ROUTINE_PARAMETER(5, SIZE_CODE(sizeof(AEDesc*)))
};

enum {
	uppXMPPtrCoercionHandlerProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(AppleEvent*)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(AppleEvent*)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPObjectAccessorProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(DescType)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(XMPPtr)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPSize)))
		 | STACK_ROUTINE_PARAMETER(5, SIZE_CODE(sizeof(DescType)))
		 | STACK_ROUTINE_PARAMETER(6, SIZE_CODE(sizeof(XMPSLong)))
		 | STACK_ROUTINE_PARAMETER(7, SIZE_CODE(sizeof(AEDesc*)))
};

enum {
	uppXMPCompareProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(DescType)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(5, SIZE_CODE(sizeof(XMPBoolean*)))
		 | STACK_ROUTINE_PARAMETER(6, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPCountProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(DescType)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(DescType)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(5, SIZE_CODE(sizeof(XMPSLong*)))
		 | STACK_ROUTINE_PARAMETER(6, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPDisposeTokenProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPGetMarkTokenProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(DescType)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(5, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPGetErrDescProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(AEDesc**)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPMarkProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPSLong)))
		 | STACK_ROUTINE_PARAMETER(5, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPAdjustMarksProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(XMPSLong)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(XMPSLong)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(5, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPPreDispatchProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPError)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(AppleEvent*)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(AppleEvent*)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(XMPSLong)))
};

enum {
	uppXMPTokenInquiryProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(XMPPart*)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(XMPOSLToken*)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(XMPSLong)))
};

#define CallXMPEventHandlerProc(userRoutine, thePart, theAppleEvent, reply, handlerRefcon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPEventHandlerProcInfo, (thePart), (theAppleEvent), (reply), (handlerRefcon))
#define NewXMPEventHandlerProc( theHandlerProc )								\
	(XMPEventHandlerUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),	\
			uppXMPEventHandlerProcInfo, GetCurrentISA() )

#define CallXMPDescCoercionHandlerProc(userRoutine, thePart, theAEDesc, toType, handlerRefCon, result)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPDescCoercionHandlerProcInfo, (thePart), (theAEDesc), (toType), (handlerRefCon), (result))
#define NewXMPDescCoercionHandlerProc( theHandlerProc )							\
	(XMPDescCoercionHandlerUPP) NewRoutineDescriptor(						\
			(ProcPtr)(theHandlerProc),										\
			uppXMPDescCoercionHandlerProcInfo, GetCurrentISA() )

#define CallXMPPtrCoercionHandlerProc(userRoutine, thePart, typeCode, dataPtr, dataSize, toType, handlerRefCon, result)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPPtrCoercionHandlerProcInfo, (thePart), (typeCode), (dataPtr), (dataSize), (toType), (handlerRefCon), (result))
#define NewXMPPtrCoercionHandlerProc( theHandlerProc )							\
	(XMPPtrCoercionHandlerUPP) NewRoutineDescriptor(						\
			(ProcPtr)(theHandlerProc),										\
			uppXMPPtrCoercionHandlerProcInfo, GetCurrentISA() )


#define CallXMPObjectAccessorProc(userRoutine, thePart, desiredClass, container, containerClass, form, selectionData, value, handlerRefCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPObjectAccessorProcInfo, (thePart), (desiredClass), (container), (containerClass), (form), (selectionData), (value), (handlerRefCon))
#define NewXMPObjectAccessorProc( theAccessorProc )								\
	(XMPObjectAccessorUPP) NewRoutineDescriptor( (ProcPtr)(theAccessorProc),\
			uppXMPObjectAccessorProcInfo, GetCurrentISA() )

#define CallXMPCompareProc(userRoutine, thePart, oper, obj1, obj2, result, refCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPCompareProcInfo, (thePart), (oper), (obj1), (obj2), (result), (refCon))
#define NewXMPCompareProc( theHandlerProc )										\
	(XMPCompareUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),		\
			uppXMPCompareProcInfo, GetCurrentISA() )

#define CallXMPCountProc(userRoutine, thePart, desiredType, containerClass, container, result, refCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPCountProcInfo, (thePart), (desiredType), (containerClass), (container), (result), (refCon))
#define NewXMPCountProc( theHandlerProc )										\
	(XMPCountUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),			\
			uppXMPCountProcInfo, GetCurrentISA() )

#define CallXMPDisposeTokenProc(userRoutine, thePart, unneededToken, refCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPDisposeTokenProcInfo, (thePart), (unneededToken), (refCon))
#define NewXMPDisposeTokenProc( theHandlerProc )								\
	(XMPDisposeTokenUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),	\
			uppXMPDisposeTokenProcInfo, GetCurrentISA() )

#define CallXMPGetMarkTokenProc(userRoutine, thePart, dContainerToken, containerClass, result, refCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPGetMarkTokenProcInfo, (thePart), (dContainerToken), (containerClass), (result), (refCon))
#define NewXMPGetMarkTokenProc( theHandlerProc )								\
	(XMPGetMarkTokenUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),	\
			uppXMPGetMarkTokenProcInfo, GetCurrentISA() )

#define CallXMPMarkProc(userRoutine, thePart, dToken, markToken, index, refCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPMarkProcInfo, (thePart), (dToken), (markToken), (index), (refCon))
#define NewXMPMarkProc( theHandlerProc )										\
	(XMPMarkUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),			\
			uppXMPMarkProcInfo, GetCurrentISA() )

#define CallXMPAdjustMarksProc(userRoutine, thePart, newStart, newStop, markToken, refCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPAdjustMarksProcInfo, (thePart), (newStart), (newStop), (markToken), (refCon))
#define NewXMPAdjustMarksProc( theHandlerProc )									\
	(XMPAdjustMarksUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),	\
			uppXMPAdjustMarksProcInfo, GetCurrentISA() )

#define CallXMPGetErrDescProc(userRoutine, thePart, errDesc, refCon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPGetErrDescProcInfo, (thePart), (errDesc), (refCon))
#define NewXMPGetErrDescProc( theHandlerProc )									\
	(XMPGetErrDescUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),	\
			uppXMPGetErrDescProcInfo, GetCurrentISA() )

#define CallXMPPreDispatchProc(userRoutine, thePart, theAppleEvent, reply, handlerRefcon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPPreDispatchProcInfo, (thePart), (theAppleEvent), (reply), (handlerRefcon))
#define NewXMPPreDispatchProc( theHandlerProc )								\
	(XMPPreDispatchUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),	\
			uppXMPPreDispatchProcInfo, GetCurrentISA() )

#define CallXMPTokenInquiryProc(userRoutine, token, refCon)		\
		(XMPPart*)CallUniversalProc((UniversalProcPtr)(userRoutine), uppXMPTokenInquiryProcInfo, (token), (refCon))
#define NewXMPTokenInquiryProc( theHandlerProc )								\
	(XMPTokenInquiryUPP) NewRoutineDescriptor( (ProcPtr)(theHandlerProc),	\
			uppXMPTokenInquiryProcInfo, GetCurrentISA() )

#else	// if not USESROUTINEDESCRIPTORS

#define CallXMPEventHandlerProc(userRoutine, thePart, theAppleEvent, reply, handlerRefcon)		\
		(*(userRoutine))((thePart), (theAppleEvent), (reply), (handlerRefcon))
#define NewXMPEventHandlerProc(theHandlerProc)									\
		(XMPEventHandlerUPP)(theHandlerProc)

#define CallXMPObjectAccessorProc(userRoutine, thePart, desiredClass, container, containerClass, form, selectionData, value, handlerRefCon)		\
		(*(userRoutine))((thePart), (desiredClass), (container), (containerClass), (form), (selectionData), (value), (handlerRefCon))
#define NewXMPObjectAccessorProc(theAccessorProc)								\
		(XMPObjectAccessorUPP)(theAccessorProc)

#define CallXMPCompareProc(userRoutine, thePart, oper, obj1, obj2, result, refCon)		\
		(*(userRoutine))((thePart), (oper), (obj1), (obj2), (result), (refCon))
#define NewXMPCompareProc(theAccessorProc)										\
		(XMPCompareUPP)(theAccessorProc)

#define CallXMPCountProc(userRoutine, thePart, desiredType, containerClass, container, result, refCon)		\
		(*(userRoutine))((thePart), (desiredType), (containerClass), (container), (result), (refCon))
#define NewXMPCountProc(theAccessorProc)										\
		(XMPCountUPP)(theAccessorProc)

#define CallXMPGetMarkTokenProc(userRoutine, thePart, dContainerToken, containerClass, result, refCon)		\
		(*(userRoutine))((thePart), (dContainerToken), (containerClass), (result), (refCon))
#define NewXMPGetMarkTokenProc(theAccessorProc)									\
		(XMPGetMarkTokenUPP)(theAccessorProc)

#define CallXMPMarkProc(userRoutine, thePart, dToken, markToken, index, refCon)		\
		(*(userRoutine))((thePart), (dToken), (markToken), (index), (refCon))
#define NewXMPMarkProc(theAccessorProc)											\
		(XMPMarkUPP)(theAccessorProc)

#define CallXMPAdjustMarksProc(userRoutine, thePart, newStart, newStop, markToken, refCon)		\
		(*(userRoutine))((thePart), (newStart), (newStop), (markToken), (refCon))
#define NewXMPAdjustMarksProc(theAccessorProc)									\
		(XMPAdjustMarksUPP)(theAccessorProc)

#define CallXMPDisposeTokenProc(userRoutine, thePart, unneededToken, refCon)		\
		(*(userRoutine))((thePart), (unneededToken), (refCon))
#define NewXMPDisposeTokenProc(theAccessorProc)									\
		(XMPDisposeTokenUPP)(theAccessorProc)

#define CallXMPGetErrDescProc(userRoutine, thePart, errDesc, refCon)		\
		(*(userRoutine))((thePart), (errDesc), (refCon))
#define NewXMPGetErrDescProc(theAccessorProc)									\
		(XMPGetErrDescUPP)(theAccessorProc)

#define CallXMPPreDispatchProc(userRoutine, thePart, theAppleEvent, reply, handlerRefcon)		\
		(*(userRoutine))((thePart), (theAppleEvent), (reply), (handlerRefcon))
#define NewXMPPreDispatchProc(theHandlerProc)									\
		(XMPEventHandlerUPP)(theHandlerProc)

#define CallXMPTokenInquiryProc(userRoutine, token, refCon)		\
		(*(userRoutine))((token), (refCon))
#define NewXMPTokenInquiryProc(theAccessorProc)									\
		(XMPTokenInquiryUPP)(theAccessorProc)


#define CallXMPDescCoercionHandlerProc(userRoutine, thePart, theAEDesc, toType, handlerRefCon, result)		\
		(*(userRoutine))((thePart), (theAEDesc), (toType), (handlerRefCon), (result))
#define NewXMPDescCoercionHandlerProc( theHandlerProc )							\
		(XMPCoercionHandlerUPP)(theHandlerProc)
#define CallXMPPtrCoercionHandlerProc(userRoutine, thePart, typeCode, dataPtr, dataSize, toType, handlerRefCon, result)		\
		(*(userRoutine))((thePart), (typeCode), (dataPtr), (dataSize), (toType), (handlerRefCon), (result))
#define NewXMPPtrCoercionHandlerProc( theHandlerProc )							\
		(XMPCoercionHandlerUPP)(theHandlerProc)


#endif		// USESROUTINEDESCRIPTORS


#define kXMPMacSemanticInterfaceID "appl:xmpmacsemanticinterface$class,1.0.0"

class XMPMacSemanticInterface : public XMPAbsSemanticInterface
{
  public:

	XMPMacSemanticInterface();
	XMPVMethod ~ CDECL XMPMacSemanticInterface();
	XMPNVMethod void InitSemanticInterface(XMPPart* base);

	//--------------------------------------------------------------------------
	// Extension Interface
	//--------------------------------------------------------------------------

	XMPVMethod void Release();

	//--------------------------------------------------------------------------
	// Content Model
	//--------------------------------------------------------------------------

	// object accessor management
		
	XMPVMethod void		InstallObjectAccessor(DescType				desiredClass,
												DescType			containerType,
												XMPObjectAccessorUPP	theAccessor,
												XMPSLong			accessorRefcon);

	XMPVMethod void		GetObjectAccessor(DescType				desiredClass,
										   DescType				containerType,
										   XMPObjectAccessorUPP*	theAccessor,		// return parameter
										   XMPSLong*			accessorRefcon);	// return parameter

	XMPVMethod void			RemoveObjectAccessor(DescType			desiredClass,
											  DescType				containerType,
											  XMPObjectAccessorUPP		theAccessor);

	// callback management

	XMPVMethod void InstallCompareProc(XMPCompareUPP	compareProc,
											XMPSLong		refCon);

	XMPVMethod void InstallCountProc(XMPCountUPP	countProc,
											XMPSLong	refCon);

	XMPVMethod void InstallDisposeTokenProc(XMPDisposeTokenUPP	disposeTokenProc,
											XMPSLong					refCon);

	XMPVMethod void InstallGetMarkTokenProc(XMPGetMarkTokenUPP	getMarkTokenProc,
												XMPSLong				refCon);

	XMPVMethod void InstallMarkProc(XMPMarkUPP	markProc,
											XMPSLong	refCon);

	XMPVMethod void InstallAdjustMarksProc(XMPAdjustMarksUPP	adjustMarksProc,
											XMPSLong				refCon);

	XMPVMethod void InstallGetErrDescProc(XMPGetErrDescUPP	errorDescProc,
											XMPSLong			refCon);

	XMPVMethod void InstallTokenInquiryProc(XMPTokenInquiryUPP	tokenInquiryProc,
											XMPSLong			refCon);

	//--------------------------------------------------------------------------
	// Semantic Events
	//--------------------------------------------------------------------------

	XMPVMethod void	InstallEventHandler(	AEEventClass			theAEEventClass,
											AEEventID				theAEEventID,
											XMPEventHandlerUPP		handler,
											XMPSLong				handlerRefcon);
	
	XMPVMethod void	RemoveEventHandler(AEEventClass					theAEEventClass,
										  AEEventID					theAEEventID,
										  XMPEventHandlerUPP	handler);
	
	XMPVMethod void	GetEventHandler(AEEventClass				theAEEventClass,
									   AEEventID				theAEEventID,
									   XMPEventHandlerUPP*		handler,
									   XMPSLong*				handlerRefcon);
	
	XMPVMethod void	InstallCoercionHandler(DescType					fromType,
											  DescType				toType,
											  XMPCoercionHandlerUPP	handler,
											  XMPSLong				handlerRefcon,
											  XMPBoolean			fromTypeIsDesc);
	
	XMPVMethod void	RemoveCoercionHandler(DescType				fromType,
											 DescType			toType,
											 XMPCoercionHandlerUPP	handler);
	
	XMPVMethod void	GetCoercionHandler(DescType					fromType,
										  DescType				toType,
										  XMPCoercionHandlerUPP*	handler,
										  XMPSLong*				handlerRefcon,
										  XMPBoolean*			fromTypeIsDesc);
	
	XMPVMethod void	InstallSpecialHandler(AEKeyword					functionClass,
												XMPSpecialHandlerUPP	handler,
												XMPSLong				refCon);
	
	XMPVMethod void	RemoveSpecialHandler(AEKeyword					functionClass,
												XMPSpecialHandlerUPP	handler);
	
	XMPVMethod void	GetSpecialHandler(AEKeyword					functionClass,
											XMPSpecialHandlerUPP*	handler,
											XMPSLong*				refCon);

	XMPVMethod void		SetOSLSupportFlags(short flags);

	XMPVMethod short	GetOSLSupportFlags();

  public: // Private by convention

	XMPBoolean	LookupEventHandler(AEEventClass				eventClass,
								   AEEventID				eventID,
								   XMPEventHandlerUPP*		handler,
								   XMPSLong*				handlerRefcon);

	XMPBoolean	LookupCoercionHandler(DescType				fromType,
									   DescType				toType,
									   XMPCoercionHandlerUPP*	handler,
									   XMPSLong*			handlerRefcon,
									   XMPBoolean*			fromTypeIsDesc);

	XMPBoolean	LookupObjectAccessor(DescType				desiredClass,
									   DescType				containerType,
									   XMPObjectAccessorUPP*	theAccessor,
									   XMPSLong*			accessorRefcon);

	OSLContext*	GetOSLContext();

	XMPPart*	CallTokenInquiryProc(XMPOSLToken*	token);

  private:

	void		ThrowProcNotFound(XMPAEHashTable* table);
	void		AddToTable(DescType key1, DescType key2, UniversalProcPtr handler,
								XMPSLong handlerRefCon,
								XMPBoolean fromTypeIsDesc,
								XMPAEHashTable* table);
	void		RetrieveFromTable(DescType key1, DescType key2,
											UniversalProcPtr* handler,
											XMPSLong* handlerRefCon,
											XMPBoolean* fromTypeIsDesc,
											XMPAEHashTable* table);
	void		RemoveFromTable(DescType key1, DescType key2, UniversalProcPtr handler,
									XMPAEHashTable* table);
	XMPBoolean	HandlerLookup(
								DescType		key1,
								DescType		key2,
								XMPAEHashTable*	theAEHashTable,
							  	XMPEntryPtr		theValue);


	struct SIGenericKey
	{
		DescType		key1;
		DescType		key2;
	};
	
	struct SIGenericValue
	{
		UniversalProcPtr	handler;
		XMPSLong		refCon;
		XMPBoolean		fromTypeIsDesc;
	};
	
	struct SIEventHandlerKey
	{
		AEEventClass	eventClass;
		AEEventID		eventID;
	};
	
	struct SIEventHandlerValue
	{
		XMPEventHandlerUPP		handler;
		XMPSLong				refCon;
	};
	
	struct SIObjectAccessorKey
	{
		DescType		desiredClass;
		DescType		containerType;
	};
	
	struct SIObjectAccessorValue
	{
		XMPObjectAccessorUPP	accessor;
		XMPSLong				refCon;
	};
	
	struct SICoercionHandlerKey
	{
		DescType		fromType;
		DescType		toType;
	};

	struct SICoercionHandlerValue
	{
		XMPCoercionHandlerUPP	handler;
		XMPSLong				refCon;
		XMPBoolean				fromTypeIsDesc;
	};

	XMPMessageInterface*	fMessageInterface;

	XMPAEHashTable*	fEventHandlerTable;
	XMPAEHashTable*	fObjectAccessorTable;
	XMPAEHashTable*	fCoercionHandlerTable;

		// Special Handler procs	
	XMPCountUPP 			fCountProcPtr;
	XMPSLong		 		fCountProcPtrRefCon;
	XMPCompareUPP			fCompareProcPtr;
	XMPSLong		 		fCompareProcPtrRefCon;
	XMPDisposeTokenUPP 		fDisposeTokenProcPtr;
	XMPSLong		 		fDisposeTokenProcPtrRefCon;
	XMPGetErrDescUPP		fErrDescProcPtr;
	XMPSLong		 		fErrDescProcPtrRefCon;
	XMPGetMarkTokenUPP 		fGetMarkTokenProcPtr;
	XMPSLong		 		fGetMarkTokenProcPtrRefCon;
	XMPMarkUPP 				fMarkProcPtr;
	XMPSLong		 		fMarkProcPtrRefCon;
	XMPAdjustMarksUPP 		fAdjustMarksProcPtr;
	XMPSLong		 		fAdjustMarksProcPtrRefCon;
	XMPPreDispatchUPP 		fPreDispatchProcPtr;
	XMPSLong		 		fPreDispatchProcPtrRefCon;

	XMPTokenInquiryUPP 		fTokenInquiryProc;
	XMPSLong		 		fTokenInquiryProcRefCon;

	short					fOSLSupportFlags;
};


#endif // _SEMTINTM_