/*
	File:		ShapeM.h

	Contains:	Definition of class XMPMacShape

	Written by:	Joshua Susser & Sepp Friedrich & Jens Alfke

	Copyright:	 1992, 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<12>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<11>	  2/8/94	JA		Include <FixMath.h> before GX header to
									avoid conflicts.
		<10>	  2/7/94	JA		Further tigerings.
		 <9>	  2/3/94	JA		Tiger Team Makeover!
		 <8>	  2/1/94	JA		Added InitShape and removed comments.
		 <7>	 1/31/94	JA		Added GeometryMode stuff. Changed XMPShape*
									return types.
		 <6>	11/23/93	JA		New Shape API (big changes!)
		 <5>	10/14/93	JA		Rewrote theory (no more copying). Modified
									inline methods.
		 <4>	 9/23/93	JA		Added Mac-specific convenience methods.
		 <3>	 9/16/93	JA		Removed ExtractPlatformShape, added new
									private fields. Added IsEmpty. Added
									Read/WritePlatformShape. Removed
									constructor w/params.
		 <2>	  8/9/93	NP		Added class id string.
		 <1>	  7/6/93	JBS		first checked in
		<3+>	 6/30/93	JBS		separate into abstract/concrete
		 <3>	 4/30/93	JBS		8.3 file names
		 <2>	 4/14/93	JBS		clean up; add ExtractPlatformShape()
		 <1>	 4/14/93	JBS		first checked in
*/

#ifndef _SHAPEM_
#define _SHAPEM_

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _XMPOBJ_
#include "XMPObj.h"			// base class
#endif

#ifndef _PLFMTYPE_
#include "PlfmType.h"
#endif

#ifndef __FIXMATH__
#include <FixMath.h>			// Must include before GX headers...
#endif
#ifndef graphicsTypesIncludes
#include <graphics types.h>		// Another fine GX header
#endif


//==============================================================================
// Classes used in this interface
//==============================================================================

class XMPTransform;
class XMPRealShape;

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPMacShape;
class XMPRectShape;

//==============================================================================
// XMPMacShape
//==============================================================================

#define kXMPMacShapeID "appl:xmpmacshape$class,1.0.0"

class XMPMacShape : public XMPAbsShape
{

public:

	//---------------------------------
	// overrides

	XMPVMethod XMPSize Purge(XMPSize size);		// override from XMPObject

	//---------------------------------
	// constructor/destructor

	XMPMacShape();
		// Creates an empty shape.
	
	XMPVMethod ~ CDECL XMPMacShape();

	XMPVMethod void InitShape( );

	//---------------------------------
	// getters/setters
	
	XMPMethod void SetGeometryMode( XMPGeometryMode mode );
	
	XMPMethod XMPGeometryMode GetGeometryMode( );
	
	XMPMethod void GetBoundingBox( XMPRect *bounds );
	
	XMPMethod XMPShape* SetRectangle( XMPRect *rect );
	
	XMPMethod XMPPolygon* CopyPolygon( );
	
	XMPMethod XMPShape* SetPolygon( XMPPolygon* );

	XMPMethod XMPPlatformShape GetPlatformShape( XMPGraphicsSystem );
	
	XMPMethod void SetPlatformShape( XMPGraphicsSystem, XMPPlatformShape );

	//---------------------------------
	// input / output:
	
	XMPMethod void WriteShape( XMPStorageUnit* );
					
	XMPMethod XMPShape* ReadShape( XMPStorageUnit* );

	//---------------------------------
	// comparison/testing functions

	XMPMethod XMPBoolean IsSameAs(XMPShape* compareShape);

	XMPMethod XMPBoolean IsEmpty( );

	XMPMethod XMPBoolean ContainsPoint(XMPPoint point);
	
	XMPMethod XMPBoolean IsRectangular( );

	XMPMethod XMPBoolean HasGeometry( );

	//---------------------------------
	// geometry operations

	XMPMethod XMPShape* Copy( );

	XMPMethod void CopyFrom(XMPShape* sourceShape);

	XMPMethod XMPShape* Transform(XMPTransform* transform);

	XMPMethod XMPShape* InverseTransform(XMPTransform* transform);

	XMPMethod XMPShape* Subtract(XMPShape* diffShape);

	XMPMethod XMPShape* Intersect(XMPShape* sectShape);

	XMPMethod XMPShape* Union(XMPShape* sectShape);

	//---------------------------------
	// Mac-specific API extensions
	
	inline RgnHandle GetQDRegion( )
							{return (RgnHandle)this->GetPlatformShape(kXMPQuickDraw);}
	inline void		 SetQDRegion( RgnHandle rgn )
							{this->SetPlatformShape(kXMPQuickDraw,(XMPPlatformShape)rgn);}
							// This consumes the region!!
	inline gxShape	 GetGXShape( )
							{return (gxShape)this->GetPlatformShape(kXMPQuickDrawGX);}
	inline void		 SetGXShape( gxShape s )
							{this->SetPlatformShape(kXMPQuickDrawGX,(XMPPlatformShape)s);}
	
	XMPMethod RgnHandle CopyQDRegion( );
	
//private-by-convention:

	inline XMPRealShape* GetRealShape( )
							{return fShape;}
				
private:

	XMPRealShape*	fShape;
	XMPGeometryMode	fMode;
};

#endif // _SHAPEM_