/*
	File:		StdTypes.h

	Contains:	Standard Types

	Written by:	OpenDoc Engineering

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<27>	 3/18/94	CC		Added kXMPASCIIText value type constant;
									removed constants for ISO string versions
									of Apple scrap types.  (1151636)
		<26>	  3/1/94	JA		Made string consts extern. (1147295)
		<25>	 2/17/94	VL		#1142972, #1144024: 'hfs' should be 'hfs '.
		<24>	  2/7/94	JA		Minor tweak to kXMPWindowRect type to
									bypass Metrowerks warning.
		<23>	  2/1/94	CC		Added kXMPLinkContent.
		<22>	 1/27/94	VL		Added kXMPKindStandInFrame.
		<21>	 1/21/94	CC		Removed kXMPLinkState; added kXMPLinkSource
									and kXMPChangeID.
		<20>	 1/18/94	CC		Removed kXMPApplePrefix; changed ISO string
									equivalents to Apple scrap types to match
									string format now returned by
									GetISOTypeFromPlatformType().
		<19>	 1/12/94	CG		Moved Category definitions to StdDefs.h.
		<18>	12/16/93	CC		Added kXMPLinkState, kXMPApplealis, and
									kXMPApplesect.
		<17>	12/12/93	T		added kXMPEditor and kXMPStr32
		<16>	 12/8/93	T		added kXMPIconFamily and kXMPIntlText
		<15>	11/23/93	JA		Deleted kXMPRegion, added kXMPPolygon and
									kXMPPoint.
		<14>	11/18/93	CG		Added Text, Graphics and Time Category
									defines
		<13>	11/18/93	T		added kXMPTime_T
		<12>	11/16/93	EL		Change OpenDocType to OpenDoc:Type.
		<11>	11/10/93	CC		Added value type kXMPLinkSpec
									Added kXMPStrongStorageUnitRef and
									kXMPWeakStorageUnitRef
		<10>	 11/9/93	PH		Added kXMPDragDropStorageUnit
		 <9>	10/25/93	RR		Added kXMPStr255
		 <8>	 10/4/93	RR		Added signed short and long types
		 <7>	 9/24/93	PH		Add some Apple types for content
		 <6>	  9/3/93	JBS		added Layout types, fixed some window types
		 <5>	 8/13/93	PH		Name all types with OpenDocType
		 <4>	 8/13/93	VL		Added kXMPStorageUnit, kXMPFrame, kXMPLink,
									kXMPWindow and kXMPPart.
		 <3>	 8/12/93	T		unify type constants
		 <2>	 8/11/93	T		add types for persistent objects
		 <1>	  8/9/93	VL		first checked in

	To Do:
*/

#ifndef _STDTYPES_
#define _STDTYPES_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This file is used to store all the standard types.
*/

//==============================================================================
// Constants
//==============================================================================

// Universal Types
extern const	XMPValueType	kXMPBoolean 	; // = "OpenDoc:Type:Boolean";
extern const	XMPValueType	kXMPValue		; // = "OpenDoc:Type:Bytes";
extern const 	XMPValueType	kXMPUShort 		; // = "OpenDoc:Type:UShort";
extern const 	XMPValueType	kXMPSShort 		; // = "OpenDoc:Type:SShort";
extern const	XMPValueType	kXMPULong		; // = "OpenDoc:Type:ULong";
extern const	XMPValueType	kXMPSLong		; // = "OpenDoc:Type:SLong";
extern const	XMPValueType	kXMPULongs		; // = "OpenDoc:Type:ULongs";

extern const	XMPValueType	kXMPISOStr		; // = "OpenDoc:Type:ISOStr";

extern const	XMPValueType	kXMPASCIIText	; // = "OpenDoc:Type:Text";
extern const	XMPValueType	kXMPIntlText	; // = "OpenDoc:Type:IntlText";
extern const	XMPValueType	kXMPName		; // = "OpenDoc:Type:Name";
extern const	XMPValueType	kXMPNames 		; // = "OpenDoc:Type:Names";
extern const	XMPValueType	kXMPStr32 		; // = "OpenDoc:Type:Str32";
extern const	XMPValueType	kXMPStr255 		; // = "OpenDoc:Type:Str255";

extern const	XMPValueType	kXMPDate		; // = "OpenDoc:Type:Date";
extern const	XMPValueType	kXMPTime_T		; // = "OpenDoc:Type:Time_T";

extern const	XMPValueType	kXMPID			; // = "OpenDoc:Type:ID";
extern const	XMPValueType	kXMPIDs			; // = "OpenDoc:Type:IDs";

extern const	XMPValueType	kXMPPoint		; // = "OpenDoc:Type:Point";
extern const	XMPValueType	kXMPRect		; // = "OpenDoc:Type:Rect";
extern const	XMPValueType	kXMPPolygon		; // = "OpenDoc:Type:Polygon";
extern const	XMPValueType	kXMPIconFamily	; // = "OpenDoc:Type:IconFamily";

extern const	XMPValueType	kXMPPartInfo	; // = "OpenDoc:Type:InfoType";
extern const	XMPValueType	kXMPTransform	; // = "OpenDoc:Type:Transform";

extern const	XMPValueType	kXMPVersionList ; // = "OpenDoc:Type:VersionList";

extern const	XMPValueType	kXMPHighlight	; // = "OpenDoc:Type:Highlight";

extern const	XMPValueType	kXMPEditor		; // = "OpenDoc:Type:Editor";

// Window
extern const	XMPValueType	kXMPWindowRect	; // = (XMPValueType) kXMPRect;
extern const	XMPValueType	kXMPSourceFrame	; // = "OpenDoc:Type:SourceFrame";


// Layout
extern const	XMPValueType	kXMPFrameTree	; // = "OpenDoc:Type:FrameTree";


extern const	XMPType			kXMPFrame					; // = "OpenDoc:Type:Frame";
extern const	XMPType			kXMPPart					; // = "OpenDoc:Type:Part";
extern const	XMPType			kXMPLink					; // = "OpenDoc:Type:Link";
extern const	XMPType			kXMPLinkSource				; // = "OpenDoc:Type:LinkSource";
extern const	XMPType			kXMPLinkContent				; // = "OpenDoc:Type:LinkContent";
extern const	XMPType			kXMPStorageUnit				; // = "OpenDoc:Type:StorageUnit";
extern const	XMPType			kXMPStrongStorageUnitRef	; // = "OpenDoc:Type:StorageUnitRef";
extern const	XMPType			kXMPWeakStorageUnitRef		; // = "OpenDoc:Type:StorageUnitRef";
extern const	XMPType			kXMPWindow					; // = "OpenDoc:Type:Window";
extern const	XMPType			kXMPLayout					; // = "OpenDoc:Type:Layout";
extern const	XMPType			kXMPDragDropStorageUnit		; // = "OpenDoc:Type:DragDropStorageUnit";

extern const	XMPValueType	kXMPLinkSpec				; // = "OpenDoc:Type:LinkSpec";
extern const	XMPValueType	kXMPChangeID				; // = "OpenDoc:Type:ChangeID";

// Drag & Drop and Clipboard

extern const	XMPValueType	kXMPKindStandInFrame		; // = "OpenDoc:Type:StandInFrame";

#endif	// _STDTYPES_
