/*
	File:		Storage.h

	Contains:	Defintion of XMPMacStorageSystem

	Written by:	Vincent Lo, Tantek I. Celik, Paul Hartenstine

	Copyright:	 1992-1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<14>	 4/13/94	VL		1157028: StorageM.h does not include
									CMAPI.h anymore, so GetCMSession returns
									XMPULong.
		<13>	 4/12/94	VL		1154100: StorageM.h should not include
									Storage.h.
		<12>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<11>	  2/4/94	VL		Used XMPCtr.h.
		<10>	  2/4/94	VL		Moved to PPC Header and code cleanup.
		 <9>	  2/3/94	CG		Changed fSession to a XMPBaseSession*.
		 <8>	 1/21/94	CG		Added GetContainerSuite method.
		 <7>	 1/12/94	VL		Init changes.
		 <6>	11/23/93	PH		Move AppleEvent handler stuff to DragDrop
		 <5>	11/23/93	VL		Added fHeap and GetHeap().
		 <4>	11/19/93	PH		Added PromiseAEHandler
		 <3>	 8/10/93	VL		Removed fClosedContainers and
									fProtoContainers.
		 <2>	  8/9/93	NP		Added class id string.
		 <1>	  7/6/93	PH		first checked in

	To Do:
		1) Replace fCMSession with the Name Space thing.
*/

#ifndef _STORAGEM_
#define _STORAGEM_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _XMPCTR_
#include "XMPCtr.h"			// included for XMPContainerID
#endif

//==============================================================================
// Classes used in this interface
//==============================================================================
class	XMPBaseSession;
class	XMPContainer;
class	ContainerList;

//==============================================================================
// Classes defined in this interface
//==============================================================================
class	XMPMacStorageSystem;

//==============================================================================
// XMPMacStorageSystem
//==============================================================================

#define kXMPMacStorageSystemID "appl:xmpmacstoragesystem$class,1.0.0"

class XMPMacStorageSystem : public	XMPAbsStorageSystem	 {

public:

	XMPMacStorageSystem();
	XMPVMethod	~ CDECL XMPMacStorageSystem();
	
	XMPNVMethod	void InitStorageSystem(XMPBaseSession* session, XMPHeap heap);

	XMPMethod	XMPBaseSession*	GetSession();
	
	XMPMethod	XMPContainer*	GetContainer(XMPContainerType containerType,
												XMPContainerID id);
												
	XMPMethod	XMPContainer*	CreateContainer(XMPContainerType containerType,
												XMPContainerID id);
											
	XMPMethod void NeedSpace(XMPSize memSize, XMPBoolean doPurge);

private:

	XMPMethod XMPContainerSuite GetContainerSuite(XMPContainerType containerType);
			
	XMPMethod XMPSize Purge(XMPSize size);		
			
	XMPMethod void GrowHeap(XMPSize memSize);

	XMPBaseSession*		fSession;
	XMPHeap				fHeap;
	ContainerList*		fContainers;

	XMPULong			fCMSession;

	
public: // private by convention

	XMPMethod	XMPAbsStorageSystem*	ReleaseContainer(XMPContainer* container);
	XMPMethod	XMPHeap					GetHeap();
	
	XMPMethod	XMPULong	GetCMSession();
	XMPMethod	void	SetCMSession(XMPULong session);
};


#endif	// _STORAGEM_