/*
	File:		StorageU.h

	Contains:	definition of XMPStorageUnit

	Written by: Vincent Lo, Tantek I. elik

	Copyright:	 1992-1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<43>	 3/25/94	VL		#1147173, #1144732: made this class
									platform independent.
		<42>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<41>	 3/14/94	CC		Added IsValidStorageUnitRef(). (1144253)
		<40>	  3/1/94	JA		Made string const extern. (1147295)
		<39>	  2/8/94	VL		Got rid of friends.
		<38>	  2/4/94	VL		Moved to PPC Header and began code cleanup.
		<37>	 1/27/94	VL		Changed CloneTo which takes XMPDraftKey and
									initiatingFrameSU and returns a
									XMPStorageUnit*. Added CloneInto which also
									takes XMPDraftKey and initiatingFrameSU.
		<36>	 1/12/94	VL		Added GetSession.
		<35>	12/21/93	VL		Changed GetValueSize to GetValue. Added
									GetObjectID.
		<34>	12/16/93	VL		Fixed problems associated with DocSU.
		<33>	12/14/93	VL		Replaced Initialize with InitStorageUnit.
									Added GetCMContainer.
		<32>	11/23/93	VL		Made XMPStorageUnit pool-aware.
		<31>	11/12/93	PH		ResolveAllPromises
		<30>	 11/9/93	PH		Add private by convention calls
		<29>	 11/5/93	PH		Add Get/SetPromiseInfo calls
		<28>	 11/5/93	VL		Changed CopyFrom to CopyTo.
		<27>	 11/4/93	PH		Add new promise api calls
		<26>	 11/1/93	VL		Replaced GetStorageUnitRef with
									GetStrongStorageUnitRef and
									GetWeakStorageUnitRef. Added
									IsStrongStorageUnitRef and
									IsWeakStorageUnitRef. Replaced CloneFrom
									with CloneTo.
		<25>	10/21/93	PH		Add ResolvePromise method
		<24>	10/20/93	PH		Add promise changes
		<23>	10/18/93	CG		Added #define id for ASLM
		<22>	 9/23/93	VL		Added CountValues and CountProperties.
		<21>	 9/14/93	VL		Added Lock and Unlock to be used as
									semaphore.
		<20>	  9/8/93	VL		Added new fields to handle flushing of
									Bento Container.
		<19>	 8/19/93	VL		Added ::RemoveStorageUnitRef.
		<18>	 8/18/93	VL		Added private CleanupAndFail.
		<17>	 8/13/93	CG		Added ASLM define for exported class.
		<16>	 8/12/93	VL		Added Exists.
		<15>	 8/11/93	VL		XMPStorageUnitRefIterator should have
									IsNotComplete (instead of IsComplete).
		<14>	  8/6/93	PH		Storage integration
		<13>	 7/29/93	VL		StorageUnitName should be XMPISOStr.
		<12>	 7/28/93	PH		Added iterator code
		<11>	 7/21/93	VL		Implemented Clone. Added
									XMPStorageUnitRefIterator. Removed Revert.
		<10>	  7/2/93	T		change GetName to return XMPName* not
									XMPName
		 <9>	  7/1/93	VL		Added fID for Prototype.
		 <8>	 6/23/93	T		change XMPPropertyName from XMPName to
									XMPISOStr
		 <7>	 6/22/93	VL		fObject is also fID.
		 <6>	 4/29/93	VL		8.3 Name Change.
		 <5>	  4/9/93	VL		XMPStorageUnit should be subclassed from
									XMPRefCntObject.
		 <4>	  4/7/93	VL		Initialize should return void.
		 <3>	  4/7/93	VL		Added Initialize method.
		 <2>	  4/6/93	VL		Made compilable.
		 <1>	  4/6/93	VL		first checked in

	To Do:
		1) kXMPIndexAll should be XMPValueIndex.
		2) Should we separate this class to StorUM.cp and StorageU.cp?
*/

#ifndef _STORAGEU_
#define _STORAGEU_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

#ifndef _REFCTOBJ_
#include "RefCtObj.h"
#endif

//==============================================================================
// Constants
//==============================================================================

typedef enum {
	kXMPPosUndefined		= -1,
	kXMPPosSame				= 0x00,
	kXMPPosAll				= 1,
	kXMPPosFirstSib			= 2,
	kXMPPosLastSib			= 3,
	kXMPPosNextSib			= 4,
	kXMPPosPrevSib			= 5,
	kXMPPosFirstBelow		= 6,
	kXMPPosLastBelow		= 7,
	kXMPPosFirstAbove 		= 8,
	kXMPPosLastAbove		= 9,
	kXMPPosReserved11 		= 10,
	kXMPPosReserved12		= 11,
	kXMPPosReserved13		= 12,
	kXMPPosReserved14		= 13,
	kXMPPosReserved15		= 14,
	
	kXMPPosMWrap			= 0x10,
	kXMPPosMOtherTypes		= 0x20
} XMPPositionCode;

const	XMPID			kXMPIDAll	= 0;
const	XMPID			kXMPIndexAll= 0;
extern const XMPValueType	kXMPTypeAll; // = 0;
const	XMPID			kXMPIDWild	= 0;

//==============================================================================
// Scalar Types
//==============================================================================

typedef	void*		XMPValue;
typedef	XMPID		XMPValueIndex;
typedef	XMPISOStr	XMPPropertyName;
typedef XMPID		XMPStorageUnitID;
typedef	XMPISOStr	XMPStorageUnitName;
typedef XMPID		XMPStorageUnitRef;
typedef XMPULong	XMPStorageUnitKey;
typedef XMPULong	XMPDraftKey;

//==============================================================================
// Classes defined in this interface
//==============================================================================

class	XMPStorageUnit;

//==============================================================================
// Classes used by this interface
//==============================================================================

class	XMPDraft;
class	XMPSession;
class	XMPPart;
class	XMPStorageUnitCursor;
class	XMPStorageUnitView;
class	XMPStorageUnitRefIterator;

//==============================================================================
// XMPStorageUnit
//==============================================================================

class XMPStorageUnit : public XMPRefCntObject {

public:

	XMPStorageUnit() {}
	
	XMPVMethod	~ CDECL XMPStorageUnit() {}
	
	XMPVMethod	void InitStorageUnit(XMPDraft* draft, XMPStorageUnitID suid)
		= 0;

	XMPVMethod	XMPDraft*		GetDraft()
		= 0;
	
	XMPVMethod	XMPSession*		GetSession()
		= 0;
			
	XMPVMethod	void IncrementRefCount()
		= 0;
			
	XMPVMethod	void Release()
		= 0;

	XMPVMethod XMPSize Purge(XMPSize size)
		= 0;

	XMPVMethod	XMPBoolean	Exists(XMPPropertyName propertyName,
									XMPValueType valueType,
									XMPValueIndex valueIndex)
		= 0;

	XMPVMethod	XMPBoolean	Exists(XMPStorageUnitCursor* cursor)
		= 0;

	XMPVMethod	XMPULong	CountProperties()
		= 0;
		
	XMPVMethod	XMPULong	CountValues()
		= 0;
	
	XMPVMethod	XMPStorageUnit*	Focus(XMPPropertyName propertyName,
										XMPPositionCode propertyPosCode,
										XMPValueType valueType,
										XMPValueIndex valueIndex,
										XMPPositionCode	valuePosCode)
		= 0;

	XMPVMethod	XMPStorageUnit*	Focus(XMPStorageUnitCursor* cursor)
		= 0;
	
	XMPVMethod	XMPStorageUnitCursor*	CreateCursor()
		= 0;
	
	XMPVMethod	XMPStorageUnit*	Externalize()
		= 0;
			
	XMPVMethod 	XMPStorageUnit*	Internalize()
		= 0;
		
	XMPVMethod	XMPID			GetID()
		= 0;
	
	XMPVMethod	XMPStorageUnitName GetName()
		= 0;
		
	XMPVMethod	void SetName(XMPStorageUnitName name)
		= 0;

	XMPVMethod	XMPStorageUnit*	AddProperty(XMPPropertyName propertyName)
		= 0;
	
	XMPVMethod	XMPStorageUnit* AddValue(XMPValueType type)
		= 0;

	XMPVMethod	XMPStorageUnit*	Remove()
		= 0;

	XMPVMethod void			CopyTo(XMPStorageUnit* toSU)
		= 0;
		
	XMPVMethod XMPStorageUnit*	CloneTo(XMPDraftKey key, XMPDraft* destDraft, XMPStorageUnit* initiatingFrame)
		= 0;
		
	XMPVMethod void			CloneInto(XMPDraftKey key, XMPStorageUnit* destStorageUnit, XMPStorageUnit* initiatingFrame)
		= 0;
		
	XMPVMethod	XMPStorageUnitView*	CreateView(XMPStorageUnitCursor* cursor)
		= 0;

	XMPVMethod XMPPropertyName	GetProperty()
		= 0;

	XMPVMethod XMPValueType	GetType()
		= 0;
		
	XMPVMethod void			SetType(XMPValueType valueType)
		= 0;

	XMPVMethod void			SetOffset(XMPULong offset)
		= 0;
		
	XMPVMethod XMPULong		GetOffset()
		= 0;
	
	XMPVMethod XMPULong		GetValue(XMPULong length, XMPValue value)
		= 0;
		
	XMPVMethod void			SetValue(XMPULong length, XMPValue value)
		= 0;
		
	XMPVMethod void			InsertValue(XMPULong length, XMPValue value)
		= 0;
		
	XMPVMethod void			DeleteValue(XMPULong length)
		= 0;
	
	XMPVMethod XMPULong		GetSize()
		= 0;

	XMPVMethod XMPBoolean		IsValidStorageUnitRef(XMPStorageUnitRef aRef)
		= 0;
		
	XMPVMethod XMPStorageUnitRef	GetStrongStorageUnitRef(XMPStorageUnit* embeddedSU)
		= 0;
		
	XMPVMethod XMPStorageUnitRef	GetWeakStorageUnitRef(XMPStorageUnit* embeddedSU)
		= 0;
		
	XMPVMethod XMPBoolean		IsStrongStorageUnitRef(XMPStorageUnitRef ref)
		= 0;
		
	XMPVMethod XMPBoolean		IsWeakStorageUnitRef(XMPStorageUnitRef ref)
		= 0;
		
	XMPVMethod XMPStorageUnit*	RemoveStorageUnitRef(XMPStorageUnitRef aRef)
		= 0;
		
	XMPVMethod XMPStorageUnitID	GetIDFromStorageUnitRef(XMPStorageUnitRef aRef)
		= 0;
	
	XMPVMethod XMPStorageUnitRefIterator* GetStorageUnitRefIterator()
		= 0;
	
	XMPVMethod	XMPULong			GetGenerationNumber()
		= 0;
		
	XMPVMethod	XMPULong			IncrementGenerationNumber()
		= 0;

	XMPVMethod	XMPBoolean	IsPromiseValue()
		= 0;
		
	XMPVMethod	void		SetPromiseValue(XMPValueType valueType,
											XMPULong offset,
											XMPULong length,
											XMPValue value,
											XMPPart *sourcePart)
		= 0;
		
	XMPVMethod	XMPULong	GetPromiseValue(XMPValueType valueType,
											XMPULong offset,
											XMPULong length,
											XMPValue value,
											XMPPart **sourcePart)
		= 0;
		
	XMPVMethod	void		ClearAllPromises()
		= 0;

	XMPVMethod	XMPStorageUnitKey	Lock(XMPStorageUnitKey key)
		= 0;
		
	XMPVMethod	void				Unlock(XMPStorageUnitKey key)
		= 0;
};


#endif	// _STORAGEU_
