/*
	File:		TypeList.h

	Contains:	Class definition for TypeList.

	Written by:	Vincent Lo

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <1>	  4/4/94	CC		first checked in

		------------------- CC ---- Renamed from TypeSet.h

		 <6>	 3/16/94	CG		#1151183: Added InitTypeSet().
		 <5>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <4>	 2/15/94	CG		#1143680 - changed fSet from a LinkList* to
									an OrderedCollection* which can be
									allocated on a specified heap.
		 <3>	 2/15/94	CC		Bug #1142945 - Standardize method names.
		 <2>	  2/9/94	T		made destructor explicitly virtual to
									pacify ASLM
		 <1>	  2/4/94	VL		first checked in

	To Do:

*/

#ifndef _TYPELIST_
#define _TYPELIST_

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================
class XMPTypeList;

//=====================================================================================
// Classes used by this interface
//=====================================================================================
class OrderedCollection;

//=====================================================================================
// Class XMPTypeList
//=====================================================================================

#define kXMPTypeListID "appl:xmptypelist$class,1.0.0"

class XMPTypeList
{
	ASLM_COMPATIBLE

	friend class 	XMPTypeListIterator;

public:

	XMPTypeList();
	virtual ~ CDECL XMPTypeList();
	
	XMPMethod void InitTypeList(XMPTypeList* typeList);
	XMPMethod void InitTypeList(XMPTypeList* typeList, XMPHeap where);
	
	XMPMethod void AddLast(XMPType type);
	XMPMethod void Remove(XMPType type);
	XMPMethod XMPBoolean Contains(XMPType type);
	XMPMethod XMPULong Count();
		
	XMPVMethod XMPTypeListIterator* CreateTypeListIterator();

private:

	OrderedCollection*	fList;	
	XMPHeap 			fHeap;
};

#endif	// _TYPELIST_