/*
	File:		UndoM.h

	Contains:	Class definition for XMPMacUndo

	Written by:	Nick Pilch

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <8>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <7>	  2/9/94	NP		Tiger Team cleanup.
		 <6>	  2/7/94	NP		Tiger Team doings.
		 <5>	 1/19/94	NP		Removed unneeded private field.
		 <4>	 1/14/94	NP		Init changes.
		 <3>	 1/13/94	NP		API Change.
		 <2>	 1/10/94	NP		Small parameter change.
		 <1>	12/14/93	NP		first checked in

	To Do:
*/

#ifndef _UNDOM_
#define _UNDOM_

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class XMPMacUndo;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPSystemUndo;
class XMPPart;

//==============================================================================
// XMPUndo
//==============================================================================

#define kXMPMacUndoID "appl:xmpmacundo$class,1.0.0"

class XMPMacUndo : public XMPAbsUndo
{
  public:

	XMPVMethod XMPSize Purge(XMPSize size);

	XMPVMethod void AddActionToHistory(XMPPart* whichPart, 
									XMPActionData actionData,
									XMPActionType actionType,
									XMPName* undoActionLabel,
									XMPName* redoActionLabel);

	XMPVMethod void Undo();

	XMPVMethod void Redo();

	XMPVMethod void MarkActionHistory();

	XMPVMethod void ClearActionHistory(XMPRespectMarksChoices respectMarks);

	XMPVMethod void ClearRedoHistory();

	XMPVMethod XMPBoolean PeekUndoHistory(XMPPart** part,
								XMPActionData* actionData,
								XMPActionType* actionType,
								XMPName** actionLabel);
	
	XMPVMethod XMPBoolean PeekRedoHistory(XMPPart** part,
								XMPActionData* actionData,
								XMPActionType* actionType,
								XMPName** actionLabel);
	
  public: // Private by convention

	XMPMacUndo();
	XMPVMethod ~ CDECL XMPMacUndo();
	XMPNVMethod void InitUndo();
	
  private:

	XMPSystemUndo*	fSystemUndo; // THIS MAY BE NULL!
};

#endif // _UNDO_