/*
	File:		WinIter.h

	Contains:	Interface to XMPWindowIterator class.

	Written by:	Richard Rodseth

	Copyright:	 1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <7>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <6>	 2/17/94	RR		#1144732 Added forward declaration of
									XMPWindow
		 <5>	 2/14/94	RR		#1143609. Marked init methods private to UI
		 <4>	  2/9/94	NP		Tiger Team cleanup.
		 <3>	 1/10/94	RR		Added Last and Previous
		 <2>	12/23/93	RR		Added InitWindowIter. Remove arguments from
									constructor
		 <1>	 12/8/93	RR		first checked in

	To Do:
*/

#ifndef _WINITER_
#define _WINITER_

#ifndef _XMPOBJ_
#include "XMPObj.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPWindowIterator;				// An iterator for all windows

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPWindowState;
class XMPWindow;
class LinkedListIterator;

//=====================================================================================
// Class XMPWindowIterator
//=====================================================================================

#define kXMPWindowIteratorID "appl:xmpwindowiterator$class,1.0.0"

class XMPWindowIterator
{
	ASLM_COMPATIBLE

public:

	virtual ~ CDECL XMPWindowIterator();
	
	XMPMethod XMPWindow* First();
		
	XMPMethod XMPWindow* Next();

	XMPMethod XMPWindow* Last();
		
	XMPMethod XMPWindow* Previous();
		
	XMPMethod XMPBoolean IsNotComplete();

public: // Private to UI subsystem

	XMPWindowIterator();
				
	XMPNVMethod void InitWindowIterator(XMPWindowState* windowState);

private:

	XMPWindowState* fWindowState;
	LinkedListIterator* fIterator;
};

#endif // _WINITER_