/*
	File:		WindowM.h

	Contains:	XMPMacWindow

	Written by:	Richard Rodseth

	Copyright:	 1992-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<49>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		<48>	 2/14/94	RR		#1143609. Marked Select Mac-specific
		<47>	  2/9/94	NP		Tiger Team cleanup.
		<46>	  2/7/94	NP		Tiger Team doings.
		<45>	  2/1/94	CC		Added ShouldShowLinks(),
									SetShouldShowLinks(), and fShouldShowLinks.
		<44>	 1/25/94	RR		Remove redundant field fWindowWasVisible
		<43>	 1/21/94	RR		Added shouldSave to InitWindow. Misc.
									cleanup.
		<42>	 1/20/94	RR		Removed AOCEObj include
		<41>	 1/19/94	RR		Removed redundant methods for drawing
									active border. Reorganized header.
		<40>	 1/18/94	RR		Remove const from Drag. Reorder methods
		<39>	 1/14/94	RR		Added utility GetWindowPoint (Private to UI
									subsystem)
		<38>	 1/13/94	eeh		moved most AOCE-related methods and
									instance variables (to AOCEObjM.h)
		<37>	 1/12/94	RR		Added CloseAndRemove
		<36>	 1/11/94	T		Init... changes
		<35>	 1/10/94	RR		Added SetWindowTitle
		<34>	12/22/93	RR		Added GetID/SetID
		<33>	12/22/93	RR		XMPMethod Usage
		<32>	12/21/93	JBS		InitWindow, CreateFrame take viewType and
									presentation params
		<31>	12/20/93	VL		XMPWindow is no longer a persistent object.
									Added calls to support saving windows by
									XMPWindowState (GetStorageUnit,
									SetStorageUnit, Externalize,
									InitWindowFromStorage, GetDraft).
		<30>	12/14/93	T		InitWindow changes.  Internalize,
									Initialize went away.
		<29>	 12/8/93	RR		Cleanup XMPMethod and forward declarations.
									Move some methods to public section for
									Vincent
		<28>	 12/8/93	VL		Added fWasWindowVisibile and getter/setter
									for it.
		<27>	 12/6/93	RR		Added SuspendResume
		<26>	 12/1/93	VL		Added floating window support.
		<25>	11/24/93	RR		Added fArbitrator
		<24>	11/18/93	eeh		added AdjustMailerSpace (was global
									function)
		<23>	11/17/93	RR		Added fShouldSave and accessors
		<22>	11/15/93	eeh		added AOCEMailerIsReceived
		<21>	11/12/93	JBS		use Facets
		<20>	10/27/93	eeh		changed AOCEMailerIsActive to -Present
		<19>	10/22/93	RR		GetProcID returns a short, not a long
		<18>	10/21/93	RR		Added fIsRootWindow/IsRootWindow()
		<17>	10/20/93	eeh		added fAOCEInstalled as temp fix for
									non-AOCE machines' menu problems
		<16>	10/12/93	eeh		added UpdateMailerMenus()
		<15>	 10/8/93	VL		Added Release().
		<14>	 10/7/93	RR		Added fRootPart. Added rootpart to
									constructor. Renamed CreateRootFrame to
									CreateLayout
		<13>	 10/7/93	eeh		Added methods for AOCE mailer support.
		<12>	 10/4/93	RR		Add protected methods for internal use
		<11>	 9/29/93	VL		Added InitializePersistent.
		<10>	 9/22/93	RR		Added Open() method
		 <9>	 9/21/93	PH		Remove Remove/RegisterDroppableFrame and
									draginfo stuff
		 <8>	  9/3/93	JBS		updated to use new Layout class
		 <7>	  8/9/93	NP		Added class id string.
		 <6>	 7/28/93	PH		Added code for new drag manager
		 <5>	 7/21/93	T		removed #ifdef PROTOTYPEBUILD stuff
		 <4>	  7/6/93	VL		Used PROTOTYPEBUILD for fDragHandler too.
		 <3>	  7/6/93	VL		Used PROTOTYPEBUILD.
		 <2>	 6/29/93	NP		Removed DragDrop.h include.
		 <4>	 6/29/93	RCR		Separate Abstract/Concrete class
		 <1>	12/18/92	RCR		first checked in

	To Do:
*/

#ifndef _WINDOWM_
#define _WINDOWM_

#ifndef _WINDOW_
#include "Window.h"
#endif

#ifndef _XMPTYPES_
#include "XMPTypes.h"
#endif

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class XMPWindow;

//=====================================================================================
// Classes used by this interface
//=====================================================================================

class XMPFrame;
class XMPPart;
class XMPStorageUnit;
class XMPSession;
class XMPArbitrator;
class XMPDraft;
class AOCEObj;

//=====================================================================================
// XMPWindow
//=====================================================================================

#define kXMPMacWindowID "appl:xmpmacwindow$class,1.0.0"

class XMPMacWindow : public XMPAbsWindow
{
public:
									
	XMPMethod XMPSize Purge(XMPSize size); 
	
	XMPMethod XMPPlatformWindow GetPlatformWindow();
	
	XMPMethod void Resized();	
	
	XMPMethod XMPFacet* GetFacetUnderPoint(XMPPoint aPoint);	
	
	// Open/Close
	// ----------

	XMPMethod void Open();
	
	XMPMethod void Close();
	
	XMPMethod void CloseAndRemove();
	
	XMPMethod void Release();

	// Show/Hide
	// ---------
	
	XMPMethod void Show();
	
	XMPMethod void Hide();
	
	XMPMethod XMPBoolean IsShown();
	
	// Layering
	// --------
	
	XMPMethod XMPBoolean IsActive();
		
	// Attributes
	// ----------

	XMPMethod XMPID	GetID();
	
	XMPMethod XMPBoolean IsRootWindow();

	XMPMethod XMPBoolean IsResizable();
	
	XMPMethod	XMPBoolean	IsFloating();

	XMPMethod XMPBoolean ShouldSave();

	XMPMethod void SetShouldSave(XMPBoolean shouldSave);

	XMPMethod XMPBoolean ShouldShowLinks();
		
	XMPMethod void SetShouldShowLinks(XMPBoolean shouldShowLinks);

	XMPMethod XMPFrame* GetSourceFrame();
	
	XMPMethod XMPFrame* GetRootFrame();
	
	XMPMethod XMPFacet* GetRootFacet();

public: // Macintosh-specific

	XMPMethod	void	Select();
	
public: // Private to UI Subsystem

	XMPMacWindow(); 

	XMPVMethod ~ CDECL XMPMacWindow();	
		// Destructor

	XMPNVMethod void InitWindow(XMPPlatformWindow platformWindow, 
								XMPBoolean isRootWindow,
								XMPBoolean isResizable,
								XMPBoolean isFloating,
								XMPBoolean shouldSave,
								XMPPart* rootPart,
								XMPTypeToken viewType,
								XMPTypeToken presentation,
								XMPFrame* sourceFrame);
			  
		// Constructor. This window becomes a dependent window of "sourceWindow",
		// which means this window is closed if sourceWindow closes.
		// sourceWindow may be NULL.

	XMPNVMethod	void InitWindowFromStorage(XMPStorageUnit* su);

	// These methods are used by the Dispatcher, not by parts

	// AOCE Mailer
	// -----------
	
	XMPMethod XMPBoolean IsAOCEMailerAware();
	XMPMethod void SetAOCEMailerAware(XMPBoolean newState);
	XMPMethod AOCEObj* GetAOCEObj();
	XMPMethod void SetAOCEObj(AOCEObj* theHelper);

	
	XMPMethod void HandleActivateEvent (XMPEventData theEvent);
	
		// Processes an update event for this window
	
	XMPMethod void HandleUpdateEvent (XMPEventData theEvent);
	
		// Processes an update event for this window
		
	XMPMethod void Update();

	XMPMethod	void	Activate();
	
	XMPMethod	void	Deactivate();
	
	XMPMethod	void	Drag(Point startPoint, Rect* draggingBounds);
	
	XMPMethod void SuspendResume(XMPEventData event);
	
	XMPMethod void SetWindowTitle(char* title);
	
	XMPMethod XMPPoint GetWindowPoint(Point globalPoint);		

	XMPMethod void SetWasVisible(XMPBoolean wasVisible);
	
	XMPMethod XMPBoolean GetWasVisible();
	
	XMPMethod void Externalize();
		
	XMPMethod void SetID(XMPID windowID);

	XMPMethod XMPStorageUnit* GetStorageUnit();
	
	XMPMethod void SetStorageUnit(XMPStorageUnit* su);
	
	XMPMethod XMPDraft* GetDraft();

protected:

	XMPMethod void CommonInitWindow();
	XMPMethod void CreateRootFrame(XMPTypeToken viewType, XMPTypeToken presentation);
	XMPMethod void CreateRootFacet();
	XMPMethod void PrepareCanvas();
	XMPMethod void DrawActiveBorder(XMPFrame* frame);
	XMPMethod XMPBoolean HasCloseBox();
	XMPMethod XMPBoolean HasZoomBox();
	XMPMethod XMPBoolean HasGrowBox();
	XMPMethod XMPSShort GetProcID();
	XMPMethod void GetWindowBounds(Rect& bounds);
	XMPMethod XMPSLong GetRefCon();

protected:

	XMPSession*			fSession;
	XMPFrame*			fSourceFrame;		
	XMPPlatformWindow	fPlatformWindow;
	XMPFrame*			fRootFrame;
	XMPFacet*			fRootFacet;
	XMPBoolean			fIsRootWindow;
	XMPBoolean			fShouldSave;
	XMPBoolean			fShouldShowLinks;
	XMPBoolean			fIsResizable;
	XMPBoolean			fIsFloating;
	XMPBoolean			fWasVisible; // Used during Internalization. Windows are created
									 // invisible, then shown all at once, so the 
									 // parts can adjust the title without flicker
	XMPArbitrator*		fArbitrator;
	XMPStorageUnit*		fStorageUnit;
	XMPDraft*			fDraft;
	XMPID				fID;
	XMPPart*			fRootPart;
	XMPBoolean			fIsAOCEMailerAware ;	// can this window even have a mailer?
	AOCEObj*			fMailerHelper ;	
};

#endif // _WINDOWM_