// CAExample.c
// Example usage of Container Application Library
// Author: Joshua Susser


// EventLoop with standard update-based drawing of embedded parts

void ExEventLoop1()
{
	EventRecord	event;
	WindowPtr	window;
	short		windowPart;
	
	while ( RunEventLoop )
	{
		if ( GetNextEvent(kAllEvents, &event) )
		{
			windowPart = FindWindow(event.where, &window);
			
			// must BeginUpdate and start drawing before
			// passing update event to OpenDoc dispatcher,
			// then EndUpdate after.
			
			if event.what == updateEvt
			{
				BeginUpdate(window);
				// draw my contents...
			}
			
			// pass event to OpenDoc dispatcher. only need
			// to handle event if OD couldn't.
			
			if ( CADispatchEvent(&event) = false )
			{
				ExHandleEvent(&event);
			}
			
			if event.what == updateEvt
			{
				EndUpdate(window);
			}
		}
	}
}


// EventLoop with explicit drawing of embedded parts

void ExEventLoop2()
{
	EventRecord	event;
	WindowPtr	window;
	short		windowPart;
	
	while ( RunEventLoop )
	{
		if ( GetNextEvent(kAllEvents, &event) )
		{
			windowPart = FindWindow(event.where, &window);
			
			// update events get handled entirely by the app
			
			if event.what == updateEvt
			{
				BeginUpdate(window);
				
				// draw my contents...
				// ...
				// draw an embedded part
				CADrawFrame(visFrame, invalidRgn);
				// ...
				
				EndUpdate(window);
			}
			else if ( CADispatchEvent(&event) = false )
			{
				// handle the event locally...
			}
		}
	}
}


void ExHandleEvent(EventRecord* event)
{
	WindowPtr	window;
	long		menuSelection;
	
	switch ( FindWindow(event->where, &window) )
	{
		case inMenuBar:
		
			// tell part with menu focus to prepare to display its menus
			CAAdjustMenus();
			
			menuSelection = MenuSelect(event->where);
			
			// let part with menu focus handle menu selection
			if ( CADispatchMenuEvent(menuSelection) == false )
				{ // handle menu locally... }
			break;

		case inContent:
			switch  ( event->what )
			{
				case mouseDown:
					// ...

				case kXMPEvtMouseDownEmbedded:
					// handle mouse down in active border of embedded frame,
					// or frame that was bundled...
					// ...select embedded frame...
			}
			break;
			
		// handle other window parts...
	}
}

// scroll document to offset

void ExScroll(WindowPtr window, Point pt)
{
	GrafPtr savePort;
	GetPort(savePort);
	SetPort(window);
	
	// scroll my contents...
	// ...
	SetOrigin(pt.h, pt.v);
	
	CATransform xform;

	// optional - hide frames scrolled out of view
	CAMakeInvisible(visFrame, ...);

	// change location of scrolled embedded frames
	CAChangeVisible(visFrame, ...);

	// make newly revealed frames visible
	CAMakeVisible(document, frameRef, ...)

	SetPort(savePort);
}