/*
	File:		CASample.c

	Contains:	Sample uses of Container Application Library

	Written by:	Tantek elik

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

	To Do:
*/


#ifndef _CALIB_
#include "CALib.h"
#endif

//==============================================================================
// Global Variables
//==============================================================================

CADocumentRefnum	gDocument;
Boolean				gDirty = false;
char*				gBuffer = NULL;

//==============================================================================
// Document operations
//==============================================================================

//------------------------------------------------------------------------------
// New
//------------------------------------------------------------------------------

void	SampleNew()
{
	// create an FSSpec filespec; for the new document somehow (using a temp file or whatever)
	gDocument = CACreateDocument(&filespec);
}

//------------------------------------------------------------------------------
// Open
//------------------------------------------------------------------------------

void SampleOpen()
{
	// get an FSSpec filespec for the file to be opened
	// either from ODOC or StandardFile
	gDocument = CAOpenDocument(&filespec);
	gBuffer = NewPtrClear(CAGetLength(gDocument));
	CARead(gDocument,gBuffer,CAGetLength(gDocument));
}

//------------------------------------------------------------------------------
// Close
//------------------------------------------------------------------------------

void SampleClose()
{
	if (!(gDirty || CAHasDocumentChanged(gDocument))) ||
		UserWantsToClose())
	{
		CACloseDocument(gDocument);
		DisposPtr(gBuffer);
		gBuffer = NULL;
	}
}

Boolean	SampleUserWantsToClose()
{
	// ask user if they want to Cancel, Discard changes, or Save changes
	
	if (true /* user wanted to Save */)
	{
		SampleSave()
		return true;
	}
	else if (true /* user wanted to Discard changes*/)
	{
		return true;
	}
	else /* user wanted to Cancel */
	{
		return false;
	}
}

//------------------------------------------------------------------------------
// Save
//------------------------------------------------------------------------------

void	SampleSave()
{
	if (gDirty)
	{
		CASize length = GetPtrSize(gBuffer);
		CASetOffset(gDocument, 0)
		CAWrite(gDocument,gBuffer,&length);
	}
	if (gDirty || CAHasDocumentChanged(gDocument))
		CASaveDocument(gDocument);
	
	gDirty = false;
}

//==============================================================================
// Edit Operations
//==============================================================================

//------------------------------------------------------------------------------
// Clear an entire frame
//------------------------------------------------------------------------------

void	SampleClearFrame(CAFrameRef selectedFrameRef)
{
	CADeleteFrameRef(selectedFrameRef,gDocument);
	// remove from gBuffer also
}

//------------------------------------------------------------------------------
// Copy an entire frame
//------------------------------------------------------------------------------

void	SampleCopyFrame(CAFrameRef selectedFrameRef)
{
	CAFrameRef clipFrameRef;
	CADocumentRefnum clipDocument = CAGetClipboardDocument();
	CABeginClone(gDocument, clipDocument);
	clipFrameRef = CACloneFrameRef(selectedFrameRef,gDocument,clipDocument);
	CASetRootFrameRef(clipDocument,clipFrameRef);
	CAEndClone(gDocument, clipDocument);
}

//------------------------------------------------------------------------------
// Cut an entire frame
//------------------------------------------------------------------------------

void	SampleCutFrame(CAFrameRef selectedFrameRef)
{
	SampleCopyFrame(selectedFrameRef);
	SampleClearFrame(selectedFrameRef);
}

//------------------------------------------------------------------------------
// Paste and embed an entire frame
//------------------------------------------------------------------------------

void	SamplePasteFrame()
{
	CADocumentRefnum clipDocument = CAGetClipboardDocument();
	CAFrameRef newFrameRef,clipFrameRef = CAGetRootFrameRef(clipDocument);

	CABeginClone(clipDocument, gDocument);
	newFrameRef = CACloneFrameRef(clipFrameRef,clipDocument,gDocument);
	// write newFrameRef into gBuffer
	CAEndClone(clipDocument, gDocument);
}

