/* @(#) somk/somapi.h 2.3 5/5/93 17:25:21 [8/2/93 08:45:42] */

/*
 * 96F8647, 96F8648 (C) Copyright IBM Corp. 1992, 1993
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 */

/*
 *    SOMAPI.H
 *
 *    Note: the typedefs in this file explicitly show pointers
 *    to objects that support an interface described by IDL. Thus, these are
 *    not IDL typedefs; they are C/C++ typedefs that reflect the implementation
 *    of object references in SOM as pointers to structures in memory.
 */

#ifndef somapi_h
#define somapi_h

/*  SOM Version Numbers  */

#if applec ^ PRAGMA_LIB_EXPORT
#pragma lib_export on
#endif

SOMEXTERN long SOM_MajorVersion;
SOMEXTERN long SOM_MinorVersion;

#if applec ^ PRAGMA_LIB_EXPORT
#pragma lib_export off
#endif

/*----------------------------------------------------------------------
 *  Typedefs for pointers to functions
 */

typedef void	SOMLINK somTD_classInitRoutine(SOMClass *, SOMClass *);
typedef int	SOMLINK somTD_SOMOutCharRoutine(char);
typedef int	SOMLINK somTD_SOMLoadModule(IN zString /* className */,
					IN zString     /* fileName */,
					IN zString     /* functionName */,
					IN integer4    /* majorVersion */,
					IN integer4    /* minorVersion */,
					OUT somToken * /* modHandle */);
typedef int	SOMLINK somTD_SOMDeleteModule(IN somToken /* modHandle */);
typedef zString SOMLINK somTD_SOMClassInitFuncName(void);
typedef void *	SOMLINK somTD_SOMMalloc(IN size_t /* nbytes */);
typedef void *	SOMLINK somTD_SOMCalloc(IN size_t /* element_count */,
					IN size_t /* element_size */);
typedef void *	SOMLINK somTD_SOMRealloc(IN void * /* memory */,
					 IN size_t /* nbytes */);
typedef void	SOMLINK somTD_SOMFree(IN void * /* memory */);
typedef void	SOMLINK somTD_SOMError(IN int	   /* code */,
				       IN zString  /* fileName */,
				       IN int	   /* lineNum */);

/*----------------------------------------------------------------------
 *  Misc. procedures:
 */

/*
 *  Create and initialize the SOM environment
 *
 *  Can be called repeatedly
 *
 *  Will be called automatically when first object (including a class
 *  object) is created, if it has not already been done.
 *
 *  Returns the SOMClassMgrObject
 */

SOMEXTERN SOMClassMgr * SOMLINK somEnvironmentNew (void);

/*----------------------------------------------------------------------
 *  String Manager: stem <somsm>
 */
SOMEXTERN somId SOMLINK somCheckId (somId id);
/* makes sure that the id is registered and in normal form, returns */
/* the id */

SOMEXTERN int SOMLINK somRegisterId(somId id);
/* Same as somCheckId except returns 1 (true) if this is the first */
/* time the string associated with this id has been registered, */
/* returns 0 (false) otherwise */

SOMEXTERN somId SOMLINK somIdFromString (zString aString);

SOMEXTERN zString SOMLINK somStringFromId (somId id);

SOMEXTERN int SOMLINK somCompareIds(somId id1, somId id2);
/* returns true (1) if the two ids are equal, else false (0) */

SOMEXTERN unsigned long SOMLINK somTotalRegIds(void);
/* Returns the total number of ids that have been registered so far, */
/* you can use this to advise the SOM runtime concerning expected */
/* number of ids in later executions of your program, via a call to */
/* somSetExpectedIds defined below */

SOMEXTERN void SOMLINK somSetExpectedIds(unsigned long numIds);
/* Tells the SOM runtime how many unique ids you expect to use during */
/* the execution of your program, this can improve space and time */
/* utilization slightly, this routine must be called before the SOM */
/* environment is created to have any effect */

SOMEXTERN unsigned long SOMLINK somUniqueKey(somId id);
/* Returns the unique key for this id, this key will be the same as the */
/* key in another id if and only if the other id refers to the same */
/* name as this one */

SOMEXTERN void SOMLINK somBeginPersistentIds(void);
/* Tells the id manager that strings for any new ids that are */
/* registered will never be freed or otherwise modified. This allows */
/* the id manager to just use a pointer to the string in the */
/* unregistered id as the master copy of the ids string. Thus saving */
/* space */
/* Under normal use (where ids are static varibles) the string */
/* associated with an id would only be freed if the code module in */
/* which it occured was unloaded */

SOMEXTERN void SOMLINK somEndPersistentIds(void);
/* Tells the id manager that strings for any new ids that are */
/* registered may be freed or otherwise modified.  Therefore the id */
/* manager must copy the strings inorder to remember the name of an */
/* id. */

/*----------------------------------------------------------------------
 *  Class Manager: SOMClassMgr, stem <somcm>
 */

#if applec ^ PRAGMA_LIB_EXPORT
#pragma lib_export on
#endif

/* Global class manager object */
SOMEXTERN SOMClassMgr *SOMClassMgrObject;

#if applec ^ PRAGMA_LIB_EXPORT
#pragma lib_export off
#endif

/*---------------------------------------------------------------------
 *  Basic offset based method resolution, this is used in every method
 *  class that uses offset resolution
 *  It returns the appropriate method procedure for the method
 *  identified by <mdata>, Mdata id the 32 bit value stored in the
 *  class data structure in the entry with the methods name. I.e., if
 *  a object, obj, of class, Foo, has a method, bar, then:
 *	somResolve(obj, FooClassData.bar)
 *  will return the appropriate method procedure for bar.
 *  The way that <mdata> identifies a method and the algorithm used by
 *  somResolve to locate the appropriate method procedure is not part
 *  of the visible SOM architecture and is subject to change in
 *  subsequent releases.
 */
SOMEXTERN somMethodProc * SOMLINK somResolve(SOMObject *obj, somMToken mdata);
SOMEXTERN somMethodProc * SOMLINK somParentResolve(somMethodTabs parentMtabs,
                                         somMToken mToken);
SOMEXTERN somMethodProc * SOMLINK somParentNumResolve(somMethodTabs parentMtabs,
                                         int parentNum,
                                         somMToken mToken);
SOMEXTERN somMethodProc * SOMLINK
    somClassResolve(SOMClass *, somMToken mdata);
SOMEXTERN somMethodProc * SOMLINK somResolveByName(SOMObject *obj, 
                                         char *methodName);
SOMEXTERN void * SOMLINK somDataResolve(SOMObject *obj, somDToken dataId);
SOMEXTERN integer1 SOMLINK somIsObj(somToken obj);

/*----------------------------------------------------------------------
 * Method Stubs -- Signature Support
 *
 *
 * This section defines the structures used to pass method signature
 * ingo to the runtime. This supports selection of generic apply stubs
 * and runtime generation of redispatchstubs when these are needed. The
 * information is registered with the runtime when methods are defined.
 *
 * When calling somAddStaticMethod, if the redispatchStub is -1, then a
 * pointer to a struct of type somApRdInfo is passed as the applyStub.
 * Otherwise, the passed redispatchstub and applystub are taken as given.
 * When calling somAddDynamicMethod, an actual apply stub must be passed.
 * Redispatch stubs for dynamic methods are not available, nor is
 * automated support for dynamic method apply stubs. The following
 * atructures only appropriate in relation to static methods.
 *
 * In SOMr2, somAddStaticMethod can be called with an actual redispatchstub
 * and applystub *ONLY* if the method doesn't return a structure. Recall
 * that no SOMr1 methods returned structures, so SOMr1 binaries obey this
 * restriction. The reason for this rule is that SOMr2 *may* use thunks,
 * and thunks need to know if a structure is returned. We therefore assume
 * that if no signature information is provided for a method through the
 * somAddStaticMethod interface, then the method returns a scalar.
 *
 * If a structure is returned, then a -1 *must* be passed to
 * somAddStaticMethod as a redispatchstub. In any case, if a -1 is passed,
 * then this means that the applystub actually points to a structure of type
 * somApRdInfo. This structure is used to hold and access signature
 * information encoded as follows.
 *
 * If the somApRdInfo pointer is NULL, then, if the runtime was built with
 * SOM_METHOD_STUBS defined, a default signature is assumed (no arguments,
 * and no structure returned); otherwise, the stubs are taken as
 * somDefaultMethod (which produces a runtime error when used) if dynamic
 * stubs are not available. 
 *
 * If the somApRdInfo pointer is not NULL, then the structure it points to can
 * either include (non-null) redispatch and applystubs (the method is then
 * assumed to return a structure), or null stubs followed by information needed
 * to generate necessary stubs dynamically.
 */
   
typedef unsigned int somRdAppType;    /* method signature code -- see def below */
typedef unsigned int somFloatMap[13]; /* float map -- see def below */
typedef struct somMethodInfoStruct {
    somRdAppType       callType;
    long               va_listSize;
    somFloatMap        *float_map;
} somMethodInfo;

typedef struct somApRdInfoStruct {
    somMethodProc *rdStub;
    somMethodProc *apStub;
    somMethodInfo *stubInfo;
/* #if __MAC__ -- user's don't define __MAC__!! */
	integer4	   retAggSize;
/* #endif */
} somApRdInfo;

/*
 * Values for somRdAppType are generated by summing one from column A and one
 * from column B of the following constants:
 */
/* Column A: return type */
#define SOMRdRetsimple     0 /* Return type is a non-float fullword */
#define SOMRdRetfloat      2 /* Return type is (single) float */
#define SOMRdRetdouble     4 /* Return type is double */
#define SOMRdRetlongdouble 6 /* Return type is long double */
#define SOMRdRetaggregate  8 /* Return type is struct or union */
#define SOMRdRetbyte      10 /* Return type is a byte */
#define SOMRdRethalf      12 /* Return type is a (2 byte) halfword */
/* Column B: are there any floating point scalar arguments? */
#define SOMRdNoFloatArgs   0
#define SOMRdFloatArgs     1

/* A somFloatMap is only needed on RS/6000 */
/*
 * This is an array of offsets for up to the first 13 floating point arguments.
 * If there are fewer than 13 floating point arguments, then there will be
 * zero entries following the non-zero entries which represent the float args.
 * A non-zero entry signals either a single- or a double-precision floating point
 * argument. For a double-precision argument, the entry is the stack
 * frame offset. For a single-precision argument the entry is the stack
 * frame offset + 1. For the final floating point argument, add 2 to the
 * code that would otherwise be used.
 */
#define SOMFMSingle 1        /* add to indicate single-precision */
#define SOMFMLast   2        /* add to indicate last floating point arg */


#ifndef SOM_SMD_DEFINED
typedef somToken somSharedMethodData;
#endif

typedef struct somMethodDataStruct {
    somId id;
    uinteger4 type;          /* 0=static, 1=dynamic */
    somId descriptor;        /* for use with IR interfaces */
    somMToken mToken;        /* NULL for dynamic methods */
    somMethodPtr method;     /* depends on resolution context */
    somSharedMethodData *shared;
} somMethodData, *somMethodDataPtr;


SOMEXTERN boolean SOMLINK somApply(SOMObject *somSelf,
				void **retVal,
				somMethodDataPtr mdPtr, 
				va_list ap);

/* This routine replaces direct use of applyStubs in SOMr1. The reason
 * for the replacement is that the SOMr1 style of applyStub is not
 * generally available in SOMr2, which uses a fixed set of applyStubs,
 * according to method information in the somMethodData. In particular,
 * neither the redispatch stub nor the apply stub found in the method
 * data structure are necessarily useful as such. The method somGetRdStub
 * is the way to get a redispatch stub, and the above function is the
 * way to call an apply stub. If an appropriate apply stub for the
 * method indicated by md is available, then this is invoked and TRUE is
 * returned; otherwise FALSE is returned.
 *
 * The va_list passed to somApply *must* include the target object,
 * somSelf, as its first entry, and any single precision floating point
 * arguments being passed to the the method procedure must be
 * represented on the va_list using double precision values. retVal cannot
 * be NULL.
 */


/*---------------------------------------------------------------------
 * somBuildClass is a convenience procedure that automates construction
 * of a new class object. The following structures are required for its use.
 */

/* to specify a new static method */
typedef struct somStaticMethodStruct {
    somMToken *classData;
    somId *methodId;
    somId *methodDescriptor;
    somMethodProc *method;
    somMethodProc *redispatchStub;
    somMethodProc *applyStub;
} somStaticMethod_t;

/* to specify an overridden method */
typedef struct somOverideMethodStruct {
    somId *methodId;
    somMethodProc *method;
} somOverrideMethod_t;

/* to specify non-internal data */
typedef struct somNonInternalDataStruct {
    somDToken *classData;
    char * basisForDataOffset;
} somNonInternalData_t;

/* to specify a "procedure" method */
typedef struct somProcMethodsStruct {
    somMethodProc **classData, *pEntry;
} somProcMethods_t;

/* to specify a varargs function */
typedef struct somVarargsFuncsStruct {
    somMethodProc **classData, *vEntry;
} somVarargsFuncs_t;

/*
 * The address of the class's ClassData structure is passed to
 * somBuildClass, to allow somBuildClass to initialize it. This
 * structure (whose actual typedef is located in somitype.h) should
 * have the external name, <className>ClassData. The tokens array
 * should have (numStaticMethods + numNonInternalData) entries, and
 * the classObject should be NULL (if it is not NULL, then a new
 * class will not be built).
 *
 * typedef struct {
 *   SOMClass *classObject;
 *   somToken tokens[1];
 *   } somClassDataStructure, *somClassDataStructurePtr;
 */

/*
 * The address of the class's auxiliary ClassData structure is passed to
 * somBuildClass, to allow somBuildClass to initialize it. This
 * structure (whose actual typedef is located in somitype.h) should
 * have the external name, <className>CClassData. The wrappers array
 * should have numVarargsFuncs entries. 
 *
 *  typedef struct {
 *   somMethodTabs parentMtab;
 *   somDToken instanceDataToken;
 *   somMethodProc *wrappers[1];
 *   } somCClassDataStructure;
 */

/*
 *  The Static Class Info Structure passed to somBuildClass
 */

typedef struct somStaticClassInfoStruct {
    integer4 layoutVersion; /* this is layout version 2 */
    integer4 numStaticMethods;   /* count of smt entries */
    integer4 numStaticOverrides; /* count of omt entries */
    integer4 numNonInternalData; /* count of nit entries */
    integer4 numProcMethods;     /* count of pmt entries */
    integer4 numVarargsFuncs;    /* count of vft entries */
    integer4 majorVersion;
    integer4 minorVersion;
    integer4 instanceDataSize;
    integer4 maxMethods;
    integer4 numParents;
    somId    classId;
    somId    explicitMetaId;
    integer4 implicitParentMeta;
    somId                  *parents;
    somClassDataStructure  *cds;
    somCClassDataStructure *ccds;
    somStaticMethod_t      *smt;
    somOverrideMethod_t    *omt;
    char                   *nitReferenceBase;
    somNonInternalData_t   *nit;
    somProcMethods_t       *pmt;
    somVarargsFuncs_t      *vft;
    somTP_somClassInitFunc *cif;

    /* end of layout version 1 */

    integer4 dataAlignment; /* the desired byte alignment for instance data */
} somStaticClassInfo, *somStaticClassInfoPtr;

SOMEXTERN SOMClass * SOMLINK somBuildClass (
		    long inherit_vars,
                    somStaticClassInfo *sci,
                    long majorVersion,
                    long minorVersion);

/*---------------------------------------------------------------------
 *  Used by old single-inheritance emitters to make class creation
 *  an atomic operation. Kept for backwards compatability.
 */
SOMEXTERN void SOMLINK somConstructClass (
                    somTD_classInitRoutine *classInitRoutine,
                    SOMClass *parentClass,
                    SOMClass *metaClass,
                    somClassDataStructure *cds);


/*
 * Uses <SOMOutCharRoutine> to output its arguments under control of the ANSI C
 * style format.  Returns the number of characters output.
 */
SOMEXTERN int SOMLINK somPrintf (zString fmt, ...);
/*
 * vprint form of somPrintf
 */
SOMEXTERN int SOMLINK somVprintf (zString fmt, va_list ap);
/*
 * Outputs (via somPrintf) blanks to prefix a line at the indicated level
 */
SOMEXTERN void SOMLINK somPrefixLevel (long level);
/*
 * Combines somPrefixLevel and somPrintf
 */
SOMEXTERN int SOMLINK somLPrintf (int level, zString fmt, ...);

#if applec ^ PRAGMA_LIB_EXPORT
#pragma lib_export on
#endif

/*
 *  Replaceable character output handler.
 *  Points to the character output routine to be used in development
 *  support.  Initialized to <somOutChar>, but may be reset at anytime.
 *  Should return 0 (false) if an error occurs and 1 (true) otherwise.
 */
SOMEXTERN somTD_SOMOutCharRoutine *SOMOutCharRoutine;

/*----------------------------------------------------------------------
 * Pointers to routines used to do dynamic code loading and deleting
 */

SOMEXTERN somTD_SOMLoadModule	  *SOMLoadModule;
SOMEXTERN somTD_SOMDeleteModule   *SOMDeleteModule;
SOMEXTERN somTD_SOMClassInitFuncName *SOMClassInitFuncName;

/*----------------------------------------------------------------------
 *  Replaceable SOM Memory Management Interface
 *
 *  External procedure variables SOMCalloc, SOMFree, SOMMalloc, SOMRealloc
 *  have the same interface as their standard C-library analogs.
 */

SOMEXTERN somTD_SOMCalloc	*SOMCalloc;
SOMEXTERN somTD_SOMFree *SOMFree;
SOMEXTERN somTD_SOMMalloc	*SOMMalloc;
SOMEXTERN somTD_SOMRealloc *SOMRealloc;

/*----------------------------------------------------------------------
 *  Replaceable SOM Error handler
 */

SOMEXTERN somTD_SOMError *SOMError;

#if applec ^ PRAGMA_LIB_EXPORT
#pragma lib_export off
#endif

/*----------------------------------------------------------------------
 * Externals used in the implementation of SOM, but not part of the
 * SOM API.
 */

SOMEXTERN SOMObject * SOMLINK somTestCls(SOMObject *obj, SOMClass *classObj,
				   zString fileName, int lineNumber);
SOMEXTERN void SOMLINK somTest(int condition, int severity, zString fileName,
			    int lineNum, zString msg);
SOMEXTERN void SOMLINK somAssert(int condition, int ecode,
				 zString fileName, int lineNum, zString msg);



#ifdef SOM_NO_DATA_EXPORTS
#ifndef MISOM

SOMEXTERN long GetSOM_MajorVersion(void);
#define SOM_MajorVersion (GetSOM_MajorVersion())

SOMEXTERN long GetSOM_MinorVersion(void);
#define SOM_MinorVersion (GetSOM_MinorVersion())

SOMEXTERN SOMClassMgr **GetSOMClassMgrObject(void);
#define SOMClassMgrObject (*GetSOMClassMgrObject())

SOMEXTERN somTD_SOMOutCharRoutine **GetSOMOutCharRoutine(void);
#define SOMOutCharRoutine (*GetSOMOutCharRoutineGetter())

SOMEXTERN somTD_SOMLoadModule	  **GetSOMLoadModule(void);
#define SOMLoadModule (*GetSOMLoadModule())

SOMEXTERN somTD_SOMDeleteModule   **GetSOMDeleteModule(void);
#define SOMDeleteModule (*GetSOMDeleteModule())

SOMEXTERN somTD_SOMClassInitFuncName **GetSOMClassInitFuncName(void);
#define SOMClassInitFuncName (*GetSOMClassInitFuncName())

#ifndef SOMCalloc
SOMEXTERN somTD_SOMCalloc	**GetSOMCalloc(void);
#define SOMCalloc (*GetSOMCalloc())
#endif

#ifndef SOMFree
SOMEXTERN somTD_SOMFree		**GetSOMFree(void);
#define SOMFree (*GetSOMFree())
#endif

#ifndef SOMMalloc
SOMEXTERN somTD_SOMMalloc	**GetSOMMalloc(void);
#define SOMMalloc (*GetSOMMalloc())
#endif

#ifndef SOMRealloc
SOMEXTERN somTD_SOMRealloc	**GetSOMRealloc(void);
#define SOMRealloc (*GetSOMRealloc())
#endif

SOMEXTERN somTD_SOMError **GetSOMError(void);
#define SOMError (*GetSOMError())

#endif
#endif /* SOM_NO_DATA_EXPORTS */


#endif /* somapi_h */
