/************************************************************

Created: Monday, January 27, 1992 at 10:19:57 AM
    SpeechRecognition.h
	
    C Interface to the PlainTalk Speech Recognition Toolbox

        Copyright Apple Computer, Inc.    1992-1994
        All rights reserved

************************************************************/


#ifndef		__SPEECHRECOGNITION__
#define		__SPEECHRECOGNITION__

#ifndef		__MEMORY__
#include	"Memory.h"
#endif

#ifndef	gestaltSpeechRecognitionVersion

/* will be going into GestaltEqu.h */

#define	gestaltSpeechRecognitionVersion 	'srtb'
#define	gestaltSpeechRecognitionAttr		'srta'

#define	gestaltDesktopSpeechRecognition		(1L<<0)
#define	gestaltTelephoneSpeechRecognition	(1L<<1)

#endif


/* Error Codes [Speech recognition gets -5100 through -5199] */

enum {
	eSRNotAvailable				= -5100,	/* the service requested is not avail or applicable */
	eSRInternalError			= -5101,	/* a system internal or hardware error condition */
	eSRComponentNotFound		= -5102,	/* a needed system resource was not located */
	eSROutOfMemory				= -5103,	/* an out of memory error occurred in MacPlainTalk space */
	eSROutOfTempMemory			= -5104,	/* an out of memory error occurred in Temporary memory space */
	eSRBadParameter				= -5105,	/* an invalid parameter was specified */
	eSRParamOutOfRange			= -5106,	/* when we say 0-100, don't pass in 101. */
	eSRBadSelector				= -5107,	/* an unrecognized selector was specified */
	eSRBufferTooSmall			= -5108,	/* returned from attribute access functions */
	eSRCantTrackClient			= -5109,	/* an error occurred while tracking client process */
	eSRCantGetSRSystem			= -5110,	/* the requested system could not be located/initialized */
	eSRCantSetProperty			= -5111,	/* a non-settable property was specified */
	eSRCantGetProperty			= -5112,	/* a non-gettable property was specified */
	eSRCantSetDuringRecognition	= -5113,	/* the property can't be set while recognition is in progress -- do before or between utterances. */
	eSRAlreadyListening			= -5114,	/* in response to SRStartListening */
	eSRNotListeningState		= -5115,	/* in response to SRStopListening */
	eSRModelMismatch			= -5116,	/* no acoustical models are avail to match request */
	eSRNoClientLanguageModel	= -5117,	/* trying to access a non-specified LanguageModel */
	eSRNoPendingUtterances		= -5118,	/* nothing to continue search on */
	eSRRecognitionAborted		= -5119,	/* an abort error occurred during search */
	eSRRecognitionDone			= -5120,	/* search has finished, but nothing was recognized */
	eSRBadObject				= -5121,	/* expecting an object as specified in API */
	eSRHasNoSubItems			= -5122,	/* SRCountItems or related routine was called on an object without subelements -- e.g. a word -- rather than phrase, path, or LM. */
	eSRSubItemNotFound			= -5123,	/* returned when accessing a non-existent sub item of a container */
	eSRLanguageModelTooBig		= -5124,	/* Cant build language models so big */
	eSRAlreadyReleased			= -5125,	/* this object has already been released before */
	eSRAlreadyFinished			= -5126,	/* the language model can't be finished twice */
	eSRWordNotFound				= -5127,	/* the spelling couldn't be found in lookup(s) */
	eSRNotFinishedWithRejection	= -5128,	/* property not found because the LMObj is not finished with rejection */
	eSRExpansionTooDeep			= -5129,	/* Language model is left recursive or is embedded too many levels */
	eSRTooManyElements			= -5130,	/* Too many elements added to phrase or path or other langauge model object */
	eSRCantAdd					= -5131		/* Can't add given type of object to the base LMObject (e.g.in SRAddLMObject)	*/
};


/* Type Definitions */

typedef unsigned long SpeechObject;
typedef SpeechObject RecognitionSystem;
typedef SpeechObject Recognizer;
typedef SpeechObject SpeechSource;
typedef SpeechSource RecognitionResult;
typedef SpeechObject LMObject;			/* an LMWord, LMPhrase or LanguageModel */
typedef LMObject LanguageModel;
typedef LMObject LMPath;
typedef LMObject LMPhrase;
typedef LMObject LMWord;
typedef unsigned long LMFlags;
typedef unsigned short SpeedSetting;	/* between 0 and 100 */
typedef unsigned short RejectionLevel;	/* between 0 and 100 */
typedef unsigned long UtteranceLength;	/* a length measure in mS */
typedef unsigned long UtteranceCount;	/* the number of queueable utterances */

/* When an event occurs, the user supplied proc will be called with a pointer
	to the param passed in and a flag to indicate conditions such
	as interrupt time or system background time.
*/

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif

struct SRCallBackStruct {
	long		what;				/* one of notification flags */
	long		message;			/* contains RecognitionResult id */
	Recognizer	instance;			/* ID of recognizer being notified */
	OSErr		status;				/* result status of last search */
	short		flags;				/* non-zero if occurs during interrupt */
};
typedef struct SRCallBackStruct SRCallBackStruct;

#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif

/* Call back procedure definition */
typedef pascal void (*SRCallBackProcPtr) (SRCallBackStruct *param);

#ifndef	USESROUTINEDESCRIPTORS
#define	USESROUTINEDESCRIPTORS 0
#endif

#if USESROUTINEDESCRIPTORS
enum {
	uppSRCallBackProcInfo = kPascalStackBased
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(SRCallBackStruct*)))
};

typedef UniversalProcPtr SRCallBackUPP;

#define CallSRCallBackProc(userRoutine, param)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppSRCallBackProcInfo, (param))
#define NewSRCallBackProc(userRoutine)		\
		(SRCallBackUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppSRCallBackProcInfo, GetCurrentISA())
#else
typedef SRCallBackProcPtr SRCallBackUPP;

#define CallSRCallBackProc(userRoutine, param)		\
		(*(userRoutine))((param))
#define NewSRCallBackProc(userRoutine)		\
		(SRCallBackUPP)(userRoutine)
#endif

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif

struct SRCallBackParam {
	SRCallBackUPP		callBack;
	SRCallBackStruct	*param;
};
typedef struct SRCallBackParam SRCallBackParam;

#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif



/* Recognition System Types */
#define kPlainTalkSystem			'ptlk'
#define	kDefaultRecognitionSystemID	((OSType) 0)

/* Recognition System Properties */
#define	kRejectedWord				'rejq'	/* the LMWord used to represent a rejection */
#define kSeparationChars			'spch'	/* separation chars for current dialect setting -- use pointer to SeparationChars struct, defined in Dialect.h */


/* Speech Source Types */
#define	kSRDefaultSpeechSource		((OSType) 0)

/* Speech Source Properties */


/* Notification via Apple Event or Callback */

/* Notification Flags */
#define kNotifyRecognitionBeginning	(1L<<0)	/* recognition can begin. client must now call SRContinueRecognition or SRAbortRecognition */
#define kNotifyRecognitionDone		(1L<<1)	/* recognition has terminated. result (if any) is available. */

/* Apple Event selectors */

/* AppleEvent message class  */
#define kAESpeechSuite				'sprc'

/* AppleEvent message event ids */
#define kAESpeechDone				'srsd'
#define kAESpeechDetected			'srbd'

/* AppleEvent Parameter ids */
#define keyRecognizer  				'krec'
#define keySpeechResult				'kspr'
#define keySpeechStatus				'ksst'

/* AppleEvent Parameter types */
#define typeRecognizer				'trec'
#define typeSpeechResult			'tspr'


/* Recognizer Properties */
#define kSpeedVsAccuracyParam		'sped'	/* SpeedSetting between 0 and 100 */
											/* 0 means more accurate but slower. */
											/* 100 means (much) less accurate but faster. */
#define kSearchStatusParam			'stat'	/* see status flags below */
#define kNotificationParam			'noti'	/* see notification flags below */
#define kCallBackParam				'call'	/* type SRCallBackParam */
#define	kAutoFinishingParam			'afin'	/* automatic finishing applied on LM for search */
#define	kMinUtteranceLength			'minl'	/* type UtteranceLength */
#define	kMaxUtteranceLength			'maxl'	/* type UtteranceLength */
#define	kNumQueuedUtterances		'numq'	/* type UtteranceCount */
#define kForegroundOnly				'fgon'	/* Boolean. Default is true. If true, client recognizer only active when in foreground.	*/
#define kBlockBackground			'blbg'	/* Boolean. Default is false. If true, when client recognizer in foreground, rest of LMs are inactive.	*/

/* Operational Status Flags */
#define kIdleRecognizer				(1L<<0)	/* engine is not active */
#define kSearchInProgress			(1L<<1)	/* search is in progress */
#define kSearchWaitForAllClients	(1L<<2)	/* search is suspended waiting on all clients' input */
#define kMustAbortSearch			(1L<<3) /* something has occurred (sound played, non-speech detected) requiring the search to abort */
#define kPendingSearch				(1L<<4) /* we're about to start searching */

/* Recognition Result Properties */
#define	kTEXTFormat					'TEXT'	/* raw text in user supplied memory */
#define kLMPhraseFormat				'lmph'	/* LMPhrase containing result words */
#define kLMPathFormat				'lmpt'	/* LMPath containing result phrases or words */
#define kLanguageModelFormat		'lmfm'	/* top level LanguageModel for post parse */

/* LMObject Family Properties */
#define kLMName						'name'	/* name of a LanguageModel */
#define kSpelling					'spel'	/* spelling of a LMWord or LMPhrase or LMPath, or name of a LanguageModel */
#define kLMObjType					'lmtp'	/* Returns one of LMObject Types listed below */
#define kRefCon						'refc'	/* 4 bytes of user storage */
#define kEnabled					'enbl'	/* Boolean -- true if LMObject enabled */
#define kPrimitive					'prim'	/* Boolean -- determines what shows up in search result's list of primitives */
#define kFinishingFlags				'finf'	/* pass pointer to LMFlags */
#define kRejectionLevel				'rjct'	/* RejectionLevel between 0 and 100 */

/* LM Object Types -- returned as kLMObjType property of language model objects */
#define kLMType						'lmob'	/* LanguageModel */
#define kPathType					'path'	/* LMPath */
#define kPhraseType					'phra'	/* LMPhrase */
#define kWordType					'word'	/* LMWord */

/* a normal and reasonable rejection level */
#define	kDefaultLMRejectionLevel 	50

/* Finishing Flags - used to make up LMFlags, which is used for kFinishingFlags and kAutoFinishingParam properties */
											/* bit zero is reserved! - remove before shipping */
#define kAddPauses 					(1L<<1)	/* allows user to pause between words */
#define kAddRejection 				(1L<<2)	/* unrecognized words will be rejected */
#define kAddNoise					(1L<<3)	/* allows a some noise at beginning or end of utterance */
#define kMakeOptional				(1L<<4)	/* contents of LM are optional */
#define kMakeRepeatable				(1L<<5)	/* contents of LM may be repeated 0 or more times */
#define kAddKeyword					(1L<<7) /* only used in kAutoFinishingParam property */
#define kAddWordSpotting			(kAddRejection | kMakeRepeatable)
#define kDefaultFinishing			kAddPauses
#define kNoFinishing				0

/*
						NOTES ON USING THE API

	All operations (with the exception of SRGetRecognitionSystem) are
	directed toward an object allocated or begot from New, Get and Read
	type calls.
	
	There is a simple rule in dealing with allocation and disposal:
	
	*	all toolbox allocations are obtained from a RecognitionSystem
	
	*	if you obtain an object via New or Get, then you own a reference 
		to that object and it must be released via SRReleaseObject when
		you no longer need it
	
	*	when you receive a RecognitionResult object via AppleEvent or
		callback, it has essentially been created on your behalf and so
		you are responsible for releasing it as above
	
	*	when you close a RecognitionSystem, all remaining objects which
		were allocated with it will be forcefully released and any
		remaining references to those objects will be invalid.
	
	This translates into a very simple guideline:
		If you allocate it or have it allocated for you, you must release
		it.  If you are only peeking at it, then don't release it.

*/

#if defined (__NOSRTRAPS__)
#define	SRINLINE(hx)	/* eat it up */
#elif defined (THREEWORDINLINE)
#define	SRINLINE(hx)	THREEWORDINLINE(0x303C,hx,0xAA56)
#else
#define	SRINLINE(hx)	= {0x303C,hx,0xAA56}
#endif


#ifdef	__cplusplus
extern "C" {
#endif

/* Opening and Closing of the RecognitionSystem */
pascal OSErr SROpenRecognitionSystem (RecognitionSystem *system, OSType systemID)
 SRINLINE (0x0400);
pascal OSErr SRCloseRecognitionSystem (RecognitionSystem system)
 SRINLINE (0x0201);

/* Accessing Properties of any Speech Object */
pascal OSErr SRSetProperty (SpeechObject srObject, OSType selector,
		const void *property, Size propertyLen)
 SRINLINE (0x0802);
pascal OSErr SRGetProperty (SpeechObject srObject, OSType selector,
		void *property, Size *propertyLen)
 SRINLINE (0x0803);

/* Any object obtained via New or Get type calls must be released */
pascal OSErr SRReleaseObject (SpeechObject srObject)
 SRINLINE (0x0204);

/* Traversing RecognitionResult or LMObjects */
pascal OSErr SRCountItems (SpeechObject container, long *count)
 SRINLINE (0x0405);
pascal OSErr SRGetIndItem (SpeechObject container, SpeechObject *item, long index)
 SRINLINE (0x0606);
pascal OSErr SRSetIndItem (SpeechObject container, SpeechObject item, long index)
 SRINLINE (0x0607);
pascal OSErr SRRemoveIndItem (SpeechObject container, long index)
 SRINLINE (0x0408);

/* Selecting the SpeechSource */
pascal OSErr SRGetSpeechSource (RecognitionSystem system, SpeechSource *source,
		OSType selector)
 SRINLINE (0x0609);

/* Recognizer Instance Functions */
pascal OSErr SRNewRecognizer (RecognitionSystem system, Recognizer *instance,
		OSType sourceID)
 SRINLINE (0x060A);
pascal OSErr SRNewRecognizerFromSource (RecognitionSystem system, Recognizer *instance,
		SpeechSource source)
 SRINLINE (0x060B);
pascal OSErr SRStartListening (Recognizer instance)
 SRINLINE (0x020C);
pascal OSErr SRStopListening (Recognizer instance)
 SRINLINE (0x020D);
pascal OSErr SRSetLanguageModel (Recognizer instance, LanguageModel active)
 SRINLINE (0x040E);
pascal OSErr SRGetLanguageModel (Recognizer instance, LanguageModel *active)
 SRINLINE (0x040F);
pascal OSErr SRContinueRecognition (Recognizer instance)
 SRINLINE (0x0210);
pascal OSErr SRAbortRecognition (Recognizer instance)
 SRINLINE (0x0211);

/* Language Model Building and Manipulation Functions */
pascal OSErr SRNewLanguageModel (RecognitionSystem system, LanguageModel *model,
		const void *name, Size nameLength)
 SRINLINE (0x0812);
pascal OSErr SRNewPath (RecognitionSystem system, LMPath *path)
 SRINLINE (0x0413);
pascal OSErr SRNewPhrase (RecognitionSystem system, LMPhrase *phrase,
		const void *text, Size textLength)
 SRINLINE (0x0814);
pascal OSErr SRNewWord (RecognitionSystem system, LMWord *word,
		const void *text, Size textLength)
 SRINLINE (0x0815);

/* Operations on any object of the LMObject family */
pascal OSErr SRSaveLMObject (LMObject lmObject, short resID)
 SRINLINE (0x0316);
pascal OSErr SRLoadLMObject (RecognitionSystem system,
		LMObject *lmObject, short resID)
 SRINLINE (0x0517);
pascal OSErr SREmptyLMObject (LMObject lmObject)
 SRINLINE (0x0218);
pascal OSErr SRChangeLMObject (LMObject lmObject, const void *text, Size textLength)
 SRINLINE (0x0619);
pascal OSErr SRAddLMObject (LMObject base, LMObject addon)
 SRINLINE (0x041A);
pascal OSErr SRAddText (LMObject base, const void *text,
	Size textLength, long refCon)
 SRINLINE (0x081B);
pascal OSErr SRRemoveLMObject (LMObject base, LMObject toRemove)
 SRINLINE (0x041C);

#ifdef	__cplusplus
}
#endif

#endif	/* __SPEECHRECOGNITION__ */
