{
	FILENAME
		Messages.p

	DESCRIPTION
		This file contains all of the public data structures, constants, and 
		function declarations for the Message Manager.

	COPYRIGHT
		Copyright  Apple Computer, Inc. 1989, 1990, 1991, 1992, 1993
		All rights reserved. 
	
}
{$IFC UNDEFINED UsingIncludes}
{$SETC UsingIncludes := 0}
{$ENDC}

{$IFC NOT UsingIncludes}
    UNIT Messages;
    INTERFACE
{$ENDC}

{$IFC UNDEFINED UsingMessages}
{$SETC UsingMessages := 1}

{$I+}
{$SETC MessagesIncludes := UsingIncludes}
{$SETC UsingIncludes := 1}
{$IFC UNDEFINED UsingTypes}
{$I $$Shell(PInterfaces)Types.p}
{$ENDC}
{$SETC UsingIncludes := MessagesIncludes}

{ Constants }


CONST


{ Message Manager Gestalt Selector... }
	gestaltMessageMgrVersion = 'mess';


{ Message Manager Error Result Codes... }
	messageStopLoopingErr = -5775;
	cantDeleteRunningHandlerErr = -5776;
	noMessageTableErr = -5777;
	dupSignatureErr = -5778;
	messageNotReceivedErr = -5799;


{ Message Manager Class Options... }
	messageClassNoOptions	 	= $00000000;
	messageClassIgnoreResult	= $80000000;


{ Message Manager Useful Constants... }
	topMessageHandler	 = 0;				{ indicates top handler when position is expected }


{ the Message Manager trap }
	 messageManagerTrap = $ABFB;	
	 sendObjectMessageSelector = $702F;
	 sendObjectMessageToSelector = $7030;
	 startObjectMessageAtSelector = $7031;
	 sendMessageSelector = $7032;
	 sendMessageToSelector = $7033;
	 startMessageAtSelector = $7034;
	 forwardSelector = $7035;
	 forwardThisSelector = $7036;

TYPE

MessageClass = ^LONGINT;	{ abstract data type for message class }

MessageHandler = ^LONGINT;	{ abstract data type for message handler }

MessageObject = ^LONGINT;	{ abstract data type for message object }

{ definition of an override table... }


MessageOverride  = PACKED RECORD
	messageID: INTEGER;		{ message being overridden }
	dispatch1: LongInt;		{ first dispatch selector }
	dispatch2: LongInt;		{ second dispatch selector }
	END;

MessageOverrideTable  = PACKED RECORD
	numOverrides: INTEGER;
	overrides: ARRAY[1..1] OF MessageOverride;
	END;
MessageOverrideTablePtr = ^MessageOverrideTable;


Signature =  OSType;
MessageStackMark = ARRAY[0..1] OF Ptr;

MessageHandlerSetupProc = ProcPtr;
MessageClassExceptionProc = ProcPtr;
MessageObjectLoopProc = ProcPtr;
MessageHandlerLoopProc = ProcPtr;
MessageGlobalsInitProc = ProcPtr;

{ ******************************* }
{ Message Handler API Routines... }
{ ******************************* }



FUNCTION GetMessageHandler : MessageHandler;
	INLINE $7027, messageManagerTrap;

FUNCTION CountMessageHandlerInstances: LongInt;
	INLINE $7028, messageManagerTrap;

FUNCTION GetMessageHandlerClassContext: LongInt;
	INLINE $7029, messageManagerTrap;

FUNCTION SetMessageHandlerClassContext (refCon: LongInt): LongInt;
	INLINE $702A, messageManagerTrap;

FUNCTION GetMessageHandlerInstanceContext : LongInt;
	INLINE $702B, messageManagerTrap;

FUNCTION SetMessageHandlerInstanceContext (refCon: LongInt): LongInt;
	INLINE $702C, messageManagerTrap;

FUNCTION NewMessageGlobals (messageGlobalsSize: LongInt; initProc: MessageGlobalsInitProc): OSErr;
	INLINE $702D, messageManagerTrap;

PROCEDURE DisposeMessageGlobals;
	INLINE $702E, messageManagerTrap;
	


{$ENDC}    { UsingMessages }

{$IFC NOT UsingIncludes}
    END.
{$ENDC}
