{ ------------------------------------------------------------------------------

	FILENAME
		PrintingManager.p

	DESCRIPTION
		This file contains all of the public data structures, constants, and 
		function declarations for the QuickDraw GX Printing Manager.

	COPYRIGHT
		Copyright  Apple Computer, Inc. 1989, 1990, 1991, 1992, 1993
		All rights reserved. 

------------------------------------------------------------------------------- }


{$IFC UNDEFINED UsingIncludes}
{$SETC UsingIncludes := 0}
{$ENDC}

{$IFC NOT UsingIncludes}
    UNIT PrintingManager;
    INTERFACE
{$ENDC}

{$IFC UNDEFINED UsingPrintingManager}
{$SETC UsingPrintingManager := 1}

{$I+}
{$SETC PrintingManagerIncludes := UsingIncludes}
{$SETC UsingIncludes := 1}
{$IFC UNDEFINED UsingTypes}
{$I $$Shell(PInterfaces)Types.p}
{$ENDC}
{$SETC UsingIncludes := PrintingManagerIncludes}

{ Constants }


CONST
{ GX Printing Manager & Overall GX Gestalt Selectors... }


	gestaltGXPrintingMgrVersion = 'pmgr';
	gestaltGXVersion 			= 'qdgx';

{ possible values for LoopStatus }
	gxStopLooping				= false;
	gxKeepLooping				= true;
	
{ printing Collection constants }
	gxNoCollectionCategory			= $0000;
	gxOutputDriverCategory			= $0001;
	gxFormattingDriverCategory		= $0002;
	gxDriverVolatileCategory		= $0004;
	
	gxVolatileOutputDriverCategory = gxOutputDriverCategory + gxDriverVolatileCategory;
	gxVolatileFormattingDriverCategory = gxFormattingDriverCategory + gxDriverVolatileCategory;

{ printing Collection tags }
	gxPrintingTagID 				= -28672;
	
{ job Collection tags }
	gxJobTag 						= 'job ';
	gxPrintJobUrgent				= $00000001;
	gxPrintJobAtTime				= $00000002; 
	gxPrintJobASAP					= $00000003;
	gxPrintJobHoldingBit 			= $00001000;
	gxPrintJobHolding				= gxPrintJobHoldingBit + gxPrintJobASAP;
	gxPrintJobHoldingAtTime			= gxPrintJobHoldingBit + gxPrintJobAtTime;
	gxPrintJobHoldingUrgent			= gxPrintJobHoldingBit + gxPrintJobUrgent;

{ ===== Job Alert User constants (jobAlert field) ===== }
	gxNoPrintTimeAlert				=	0;	{ don't alert user when we print }
	gxAlertBefore					=	1;	{ alert user before we print }
	gxAlertAfter					=	2;	{ alert user after we print }
	gxAlertBothTimes				=	3;	{ alert before and after we print }

{ ===== Job Timeout constants (jobTimeout field) ===== }
	gxThirtySeconds					=	1800;	{ 30 seconds in ticks }
	gxTwoMinutes					=	7200;	{ 2 minutes in ticks }

	gxCollationTag 					= 'sort';
	gxCopiesTag 					= 'copy';
	gxPageRangeTag 					= 'rang';
		gxDefaultPageRange				= 0;
		gxReplacePageRange				= 1;
		gxCustomizePageRange			= 2;
	gxQualityTag 					= 'qual';
	gxFileDestinationTag 			= 'dest';
	gxFileLocationTag 				= 'floc';
	gxFileFormatTag 				= 'ffmt';
	gxFileFontsTag				 	= 'incf';
		gxIncludeNoFonts				= 1;
		gxIncludeAllFonts				= 2; 
		gxIncludeNonStandardFonts 		= 3;
	gxPaperFeedTag 					= 'feed';
	gxTrayFeedTag 					= 'tray';
	gxManualFeedTag 				= 'manf';
	gxNormalMappingTag 				= 'nmap';
	gxSpecialMappingTag 			= 'smap';
		gxRedirectPages					= 1;
		gxScalePages					= 2;
		gxTilePages						= 3;
	gxTrayMappingTag 				= 'tmap';
	gxPaperMappingTag 				= 'pmap';
	gxPrintPanelTag 				= 'ppan';
	gxFormatPanelTag 				= 'fpan';
	gxTranslatedDocumentTag 		= 'trns';
	
{ format Collection tags }
	gxPaperTypeLockTag 				= 'ptlk';
	gxOrientationTag 				= 'layo';
		gxPortraitLayout 				= 0;
		gxLandscapeLayout 				= 1;
		gxRotatedPortraitLayout 		= 2;
		gxRotatedLandscapeLayout 		= 3;
	gxScalingTag 					= 'scal';
	gxDirectModeTag 				= 'dirm';
	gxFormatHalftoneTag 			= 'half';
	gxInvertPageTag 				= 'invp';
	gxFlipPageHorizontalTag 		= 'flph';
	gxFlipPageVerticalTag 			= 'flpv';
	gxPreciseBitmapsTag 			= 'pbmp';
	
{ PaperType Collection tags }
	gxBaseTag 						= 'base';
		gxUnknownBase					=	0;
		gxUSLetterBase					=	1;
		gxUSLegalBase					=	2;
		gxA4LetterBase					=	3;
		gxB5LetterBase					=	4;
		gxTabloidBase					=	5;
	gxCreatorTag 					= 'crea';
		gxSysPaperType					=	'sypt';
		gxUserPaperType					=	'uspt';
	gxUnitsTag 						= 'unit';
		gxPicas							=	0;
		gxMMs							=	1;
		gxInches						=	2;
	gxFlagsTag 						= 'flag';
		gxOldPaperTypeFlag				=	$00800000;
		gxNewPaperTypeFlag				=	$00400000;
		gxOldAndNewPaperTypeFlag		=	$00C00000;
		gxDefaultPaperTypeFlag			=	$00100000;
	gxCommentTag 					= 'cmnt';

{ Printer ViewDevice tags }
	gxPenTableTag 					= 'pent';
		gxDeviceUnits 					= 0;
		gxMMUnits 						= 1;
		gxInchesUnits 					= 2;
		gxPenNotLoaded	 				= -1;

{ Dialog related constants }
	gxCancelSelected		= 0;
	gxOKSelected 			= 1;
	gxRevertSelected		= 2;

{ job format mode constants }
	gxGetJobFormatLineConstraintQuery		= 0;
	gxGetJobFormatFontsQuery				= 1;
	gxGetJobFormatFontCommonStylesQuery		= 2;
	gxGetJobFormatFontConstraintQuery		= 3;
	gxSetStyleJobFormatCommonStyleQuery		= 4;

	gxGraphicsJobFormatMode		= 'grph';
	gxTextJobFormatMode			= 'text';
	gxPostScriptJobFormatMode 	= 'post';

	gxConstraintRange = -1; { in numSizes field of PositionConstraintTable to indicate a range in sizes array }

{ Types }


TYPE

gxPrivatePrinterRecord = LONGINT;
gxPrinter = ^gxPrivatePrinterRecord;
gxPrivateJobRecord = LONGINT;
gxJob = ^gxPrivateJobRecord;
gxPrivateFormatRecord = LONGINT;
gxFormat = ^gxPrivateFormatRecord;
gxPrivatePaperTypeRecord = LONGINT;
gxPaperType = ^gxPrivatePaperTypeRecord;
gxPrivatePrintFileRecord = LONGINT;
gxPrintFile = ^gxPrivatePrintFileRecord;

gxLoopStatus = Boolean;

gxViewDeviceProc = ProcPtr;
gxFormatProc = ProcPtr;
gxPaperTypeProc = ProcPtr;
gxPrintingFlattenProc = ProcPtr;

{
These procedures should look like this:

FUNCTION MyViewDeviceProc(theDevice: gxViewDevice; refCon : UNIV Ptr) gxLoopStatus;
FUNCTION MyFormatProc(theFormat: gxFormat; refCon : UNIV Ptr) gxLoopStatus;
FUNCTION MyPaperTypeProc(thePaperType: gxPaperType; refCon : UNIV Ptr) gxLoopStatus;
FUNCTION MyFlattenProc (size : LONGINT; data : UNIV Ptr; refCon : UNIV Ptr) : OSErr;
}

{ printing Collection types }
gxCollectionCategory = INTEGER; { stored in collection items' user attribute bits}

gxJobInfo = PACKED RECORD
	numPages: LongInt;
	priority: LongInt;
	timeToPrint: LongInt;
	jobTimeout: LongInt;			{ in ticks }
	firstPageToPrint: LongInt;		{ start printing from this page }
	jobAlert: INTEGER;
	appName: Str32;
	documentName: Str32;
	userName: Str32;
	END;

gxCollationInfo = PACKED RECORD
	collation: Boolean;
	END;

gxCopiesInfo = PACKED RECORD
	copies: LongInt;
	END;
	
gxSimplePageRangeInfo = PACKED RECORD
	optionChosen: Byte;		{ from options listed above }
	printAll: Boolean;		{ true if user wants to print all pages }
	fromPage: LongInt;		{ for gxDefaultPageRange, current value }
	toPage: LongInt;		{ for gxDefaultPageRange, current value }
	END;
	
gxPageRangeInfo = PACKED RECORD
	simpleRange: gxSimplePageRangeInfo;		{ info which will be returned for GetJobPageRange }
	fromString: Str31;						{ for gxCustomizePageRange, current value  }
	toString: Str31;						{ for gxCustomizePageRange, current value }
	minFromPage: LongInt;					{ for gxDefaultPageRange, we parse with this, ignored if nil }
	maxToPage: LongInt;						{ for gxDefaultPageRange, we parse with this, ignored if nil }
	replaceString: ARRAY [0..0] OF Char;	{ for gxReplacePageRange, string to display }
	END;
	
gxQualityInfo = PACKED RECORD
	disableQuality: Boolean;			{ true to disable standard quality controls }
	defaultQuality: INTEGER;
	currentQuality: INTEGER;
	qualityCount: INTEGER;				{ number of quality menu items in popup menu }
	qualityNames: ARRAY [0..0] OF Char;	{ array of packed pascal strings for popup menu titles }
	END;
	
gxFileDestinationInfo = PACKED RECORD
	toFile: Boolean;
	END;

gxFileLocationInfo = PACKED RECORD
	fileSpec: FSSpec;
	END;

gxFileFormatInfo = PACKED RECORD
	fileFormatName: Str31;
	END;

gxFileFontsInfo = PACKED RECORD
	includeFonts: Byte;
	END;

gxPaperFeedInfo = PACKED RECORD
	autoFeed: Boolean;
	END;

gxTrayIndex = LongInt;
gxTrayFeedInfo = PACKED RECORD
	feedTrayIndex: gxTrayIndex;
	manualFeedThisPage: Boolean;
	END;

gxManualFeedInfo = PACKED RECORD
	numPaperTypeNames: LongInt;
	paperTypeNames: ARRAY [0..0] OF Str31;
	END;
	
gxNormalMappingInfo = PACKED RECORD
	normalPaperMapping: Boolean;
	END;

gxSpecialMappingInfo = PACKED RECORD
	specialMapping: Byte;
	END;

gxTrayMappingInfo = PACKED RECORD
	mapPaperToTray: gxTrayIndex;
	END;

gxPrintPanelInfo = PACKED RECORD
	startPanelName: Str31;
	END;

gxFormatPanelInfo = PACKED RECORD
	startPanelName: Str31;
	END;
	
gxTranslatedDocumentInfo = PACKED RECORD
	translatorInfo: LongInt;
	END;
	
{ format Collection types }

gxPaperTypeLockInfo = PACKED RECORD
	paperTypeLocked: Boolean;
	END;

gxOrientationInfo = PACKED RECORD
	orientation: Byte;
	END;

gxScalingInfo = PACKED RECORD
	horizontalScaleFactor: Fixed;		{ current horizontal scaling factor }
	verticalScaleFactor: Fixed;			{ current vertical scaling factor }
	minScaling: INTEGER;				{ minimum current scaling }
	maxScaling: INTEGER;				{ maximum current scaling }
	END;

gxDirectModeInfo = PACKED RECORD
	directModeOn: Boolean;
	END;

gxFormatHalftoneInfo = PACKED RECORD
	numHalftones: LongInt;					{ how many halftones }
	halftones: ARRAY [0..0] OF gxHalftone; 	{ any number of halftones }
	END;

gxInvertPageInfo = PACKED RECORD
	invert: Boolean;
	END;

gxFlipPageHorizontalInfo = PACKED RECORD
	flipHorizontal: Boolean;
	END;

gxFlipPageVerticalInfo = PACKED RECORD
	flipVertical: Boolean;
	END;

gxPreciseBitmapInfo = PACKED RECORD
	preciseBitmaps: Boolean;
	END;

{ PaperType Collection types }

gxBaseInfo = PACKED RECORD
	baseType: LongInt;
	END;

gxCreatorInfo = PACKED RECORD
	creator: OSType;
	END;

gxUnitsInfo = PACKED RECORD
	units: Byte;
	END;

gxFlagsInfo = PACKED RECORD
	flags: LongInt;
	END;

gxCommentInfo = PACKED RECORD
	comment: Str255;
	END;

{ printer viewDevice tag types }

gxPenTableEntry = PACKED RECORD
	penName: Str31;			{ name of the pen }
	penColor: gxColor;		{ color that is part of the color set }
	penThickness: Fixed;	{ size of the pen }
	penUnits: INTEGER;		{ specifies units in which pen thickness is defined }
	penPosition: INTEGER;	{ pen position in the carousel, -1 (kPenNotLoaded) if not loaded }
	END;

gxPenTable = PACKED RECORD
	numPens: LongInt;						{ number of pen entries in the following array }
	pens: ARRAY [0..0] OF gxPenTableEntry; 	{ array of pen entries }
	END;

{ dialog related types }
gxDialogResult = LongInt;

gxEditMenuRecord = PACKED RECORD
	editMenuID: INTEGER;

	cutItem: INTEGER;
	copyItem: INTEGER;
	pasteItem: INTEGER;
	clearItem: INTEGER;
	undoItem: INTEGER;
	END;

{ job format mode types }
gxQueryType = LongInt;
gxJobFormatMode = OSType;

gxJobFormatModeTable = PACKED RECORD
	numModes: LongInt;						
	modes: ARRAY [0..0] OF gxJobFormatMode; 	
	END;
gxJobFormatModeTablePtr = ^gxJobFormatModeTable;
gxJobFormatModeTableHdl = ^gxJobFormatModeTablePtr;

gxPositionConstraintTable = PACKED RECORD
	phase: gxPoint;						
	offset: gxPoint;						
	numSizes: LongInt;						
	sizes: ARRAY [0..0] OF Fixed; 	
	END;
gxPositionConstraintTablePtr = ^gxPositionConstraintTable;
gxPositionConstraintTableHdl = ^gxPositionConstraintTablePtr;
	
gxStyleNameTable = PACKED RECORD
	numStyleNames: LongInt;						
	styleNames: ARRAY [0..0] OF Str255; 	
	END;
gxStyleNameTablePtr = ^gxStyleNameTable;
gxStyleNameTableHdl = ^gxStyleNameTablePtr;

gxFontTable = PACKED RECORD
	numFonts: LongInt;						
	fonts: ARRAY [0..0] OF gxFont; 	
	END;
gxFontTablePtr = ^gxFontTable;
gxFontTableHdl = ^gxFontTablePtr;

{********************************************************************}
{************************ Public Interfaces *************************}
{********************************************************************}

{ Global Routines (Basic) }

FUNCTION GXInitPrinting : OSErr;
	INLINE $203C,$0000, $0000, $ABFE;
	
FUNCTION GXExitPrinting : OSErr;
	INLINE $203C,$0000, $0001, $ABFE;


{ Job Routines (Basic) }

FUNCTION GXNewJob(VAR theJob: gxJob) : OSErr;
	INLINE $203C,$0000, $0002, $ABFE;

FUNCTION GXDisposeJob(theJob: gxJob) : OSErr;
	INLINE $203C,$0000, $0003, $ABFE;

PROCEDURE GXFlattenJob(theJob: gxJob; theProc: gxPrintingFlattenProc; refCon : UNIV Ptr);
	INLINE $203C,$0000, $0004, $ABFE;

FUNCTION GXUnflattenJob(theJob: gxJob; theProc: gxPrintingFlattenProc; refCon : UNIV Ptr) : gxJob;
	INLINE $203C,$0000, $0005, $ABFE;

FUNCTION GXFlattenJobToHdl(theJob: gxJob; theHandle: Handle) : Handle;
	INLINE $203C,$0000, $0006, $ABFE;

FUNCTION GXUnflattenJobFromHdl(theJob: gxJob; theHandle: Handle) : gxJob;
	INLINE $203C,$0000, $0007, $ABFE;

PROCEDURE GXInstallApplicationOverride(theJob: gxJob; messageID: INTEGER; refCon : UNIV Ptr);
	INLINE $203C,$0000, $0008, $ABFE;


{ Format Routines (Basic) }

FUNCTION GXNewFormat(theJob: gxJob) : gxFormat;
	INLINE $203C,$0000, $0009, $ABFE;

PROCEDURE GXDisposeFormat(theFormat: gxFormat);
	INLINE $203C,$0000, $000A, $ABFE;


{ PaperType Routines (Basic) }

FUNCTION GXNewPaperType(theJob: gxJob; name: Str31; VAR pageSize: gxRectangle; VAR paperSize: gxRectangle) : gxPaperType;
	INLINE $203C,$0000, $000B, $ABFE;

PROCEDURE GXDisposePaperType(thePaperType: gxPaperType);
	INLINE $203C,$0000, $000C, $ABFE;

FUNCTION GXGetNewPaperType(theJob: gxJob; resID: INTEGER) : gxPaperType;
	INLINE $203C,$0000, $000D, $ABFE;


{ Error-Handling Routines (Basic) }

FUNCTION GXGetJobError(theJob: gxJob) : OSErr;
	INLINE $203C,$0000, $000E, $ABFE;

PROCEDURE GXSetJobError(theJob: gxJob; theError: OSErr);
	INLINE $203C,$0000, $000F, $ABFE;


{ Dialog Routines (Basic) }

FUNCTION GXJobDefaultFormatDialog(theJob: gxJob; VAR theEditRecord: gxEditMenuRecord) : gxDialogResult;
	INLINE $203C,$0000, $0010, $ABFE;

FUNCTION GXJobPrintDialog(theJob: gxJob; VAR theEditRecord: gxEditMenuRecord) : gxDialogResult;
	INLINE $203C,$0000, $0011, $ABFE;

FUNCTION GXFormatDialog(theJob: gxJob; VAR theEditRecord: gxEditMenuRecord; title: StringPtr) : gxDialogResult;
	INLINE $203C,$0000, $0012, $ABFE;


{ Format Routines (Basic) }

FUNCTION GXGetJobFormat(theJob: gxJob; whichFormat: LongInt) : gxFormat;
	INLINE $203C,$0000, $0013, $ABFE;

FUNCTION GXGetFormatJob(theFormat: gxFormat) : gxJob;
	INLINE $203C,$0000, $0014, $ABFE;

FUNCTION GXGetFormatPaperType(theFormat: gxFormat) : gxPaperType;
	INLINE $203C,$0000, $0015, $ABFE;

PROCEDURE GXGetFormatDimensions(theFormat: gxFormat; VAR pageSize: gxRectangle; VAR paperSize: gxRectangle);
	INLINE $203C,$0000, $0016, $ABFE;


{ Spooling Routines (Basic) }

PROCEDURE GXGetJobPageRange(theJob: gxJob; VAR firstPage: LongInt; VAR lastPage: LongInt);
	INLINE $203C,$0000, $0017, $ABFE;

PROCEDURE GXStartJob(theJob: gxJob; docName: StringPtr; pageCount: LongInt);
	INLINE $203C,$0000, $0018, $ABFE;

PROCEDURE GXPrintPage(theJob: gxJob; pageNumber: LongInt; theFormat: gxFormat; thePage: gxShape);
	INLINE $203C,$0000, $0019, $ABFE;

FUNCTION GXStartPage(theJob: gxJob; pageNumber: LongInt; theFormat: gxFormat; numViewPorts: LongInt; VAR viewPortList: gxViewPort): Boolean;
	INLINE $203C,$0000, $001A, $ABFE;

PROCEDURE GXFinishPage(theJob: gxJob);
	INLINE $203C,$0000, $001B, $ABFE;

PROCEDURE GXFinishJob(theJob: gxJob);
	INLINE $203C,$0000, $001C, $ABFE;


{ Job Routines (Advanced) }

FUNCTION GXGetJobCollection(theJob: gxJob) : Collection;
	INLINE $203C,$0000, $001D, $ABFE;

FUNCTION GXGetJobRefCon(theJob: gxJob) : LongInt;
	INLINE $203C,$0000, $001E, $ABFE;

PROCEDURE GXSetJobRefCon(theJob: gxJob; refCon: LongInt);
	INLINE $203C,$0000, $001F, $ABFE;

FUNCTION GXCopyJob(srcJob: gxJob; dstJob: gxJob) : gxJob;
	INLINE $203C,$0000, $0020, $ABFE;

PROCEDURE GXSelectJobFormattingPrinter(theJob: gxJob; printerName: Str31);
	INLINE $203C,$0000, $0021, $ABFE;
	
PROCEDURE GXSelectJobOutputPrinter(theJob: gxJob; printerName: Str31);
	INLINE $203C,$0000, $0022, $ABFE;

PROCEDURE GXForEachJobFormatDo(theJob: gxJob; theProc: gxFormatProc; refCon : UNIV Ptr);
	INLINE $203C,$0000, $0023, $ABFE;

FUNCTION GXCountJobFormats(theJob: gxJob) : LongInt;
	INLINE $203C,$0000, $0024, $ABFE;

FUNCTION GXUpdateJob(theJob: gxJob) : Boolean;
	INLINE $203C,$0000, $0025, $ABFE;

PROCEDURE GXConvertPrintRecord(theJob: gxJob; oldPrintRecord: THPrint);
	INLINE $203C,$0000, $0026, $ABFE;


{ Printer Routines (Advanced) }

FUNCTION GXGetJobFormattingPrinter(theJob: gxJob) : gxPrinter;
	INLINE $203C,$0000, $0027, $ABFE;

FUNCTION GXGetJobOutputPrinter(theJob: gxJob) : gxPrinter;
	INLINE $203C,$0000, $0028, $ABFE;

FUNCTION GXGetJobPrinter(theJob: gxJob) : gxPrinter;
	INLINE $203C,$0000, $0029, $ABFE;

FUNCTION GXGetPrinterJob(thePrinter: gxPrinter) : gxJob;
	INLINE $203C,$0000, $002A, $ABFE;

PROCEDURE GXForEachPrinterViewDeviceDo(thePrinter: gxPrinter; theProc: gxViewDeviceProc; refCon : UNIV Ptr);
	INLINE $203C,$0000, $002B, $ABFE;

FUNCTION GXCountPrinterViewDevices(thePrinter: gxPrinter) : LongInt;
	INLINE $203C,$0000, $002C, $ABFE;

FUNCTION GXGetPrinterViewDevice(thePrinter: gxPrinter; whichViewDevice: LongInt) : gxViewDevice;
	INLINE $203C,$0000, $002D, $ABFE;

PROCEDURE GXSelectPrinterViewDevice(thePrinter: gxPrinter; whichViewDevice: LongInt);
	INLINE $203C,$0000, $002E, $ABFE;

PROCEDURE GXGetPrinterName(thePrinter: gxPrinter; VAR printerName: Str31);
	INLINE $203C,$0000, $002F, $ABFE;

FUNCTION GXGetPrinterType(thePrinter: gxPrinter) : OSType;
	INLINE $203C,$0000, $0030, $ABFE;

PROCEDURE GXGetPrinterDriverName(thePrinter: gxPrinter; VAR printerName: Str31);
	INLINE $203C,$0000, $0031, $ABFE;

FUNCTION GXGetPrinterDriverType(thePrinter: gxPrinter) : OSType;
	INLINE $203C,$0000, $0032, $ABFE;


{ Format Routines (Advanced) }

FUNCTION GXGetFormatCollection(theFormat: gxFormat) : Collection;
	INLINE $203C,$0000, $0033, $ABFE;

PROCEDURE GXChangedFormat(theFormat: gxFormat);
	INLINE $203C,$0000, $0034, $ABFE;

FUNCTION GXCopyFormat(srcFormat: gxFormat; dstFormat: gxFormat) : gxFormat;
	INLINE $203C,$0000, $0035, $ABFE;

FUNCTION GXCloneFormat(theFormat: gxFormat) : gxFormat;
	INLINE $203C,$0000, $0036, $ABFE;

FUNCTION GXCountFormatOwners(theFormat: gxFormat) : LongInt;
	INLINE $203C,$0000, $0037, $ABFE;

PROCEDURE GXGetFormatMapping(theFormat: gxFormat; VAR theMapping: gxMapping);
	INLINE $203C,$0000, $0038, $ABFE;

FUNCTION GXGetFormatForm(theFormat: gxFormat; VAR mask: gxShape) : gxShape;
	INLINE $203C,$0000, $0039, $ABFE;

PROCEDURE GXSetFormatForm(theFormat: gxFormat; form: gxShape; mask: gxShape);
	INLINE $203C,$0000, $003A, $ABFE;


{ Job Format Modes Routines (Advanced) }

PROCEDURE GXSetAvailableJobFormatModes(theJob: gxJob; whichModes: gxJobFormatModeTableHdl);
	INLINE $203C,$0000, $003B, $ABFE;

FUNCTION GXGetPreferredJobFormatMode(theJob: gxJob; VAR directOnly: Boolean) : gxJobFormatMode;
	INLINE $203C,$0000, $003C, $ABFE;

FUNCTION GXGetJobFormatMode(theJob: gxJob) : gxJobFormatMode;
	INLINE $203C,$0000, $003D, $ABFE;

PROCEDURE GXSetJobFormatMode(theJob: gxJob; theMode: gxJobFormatMode);
	INLINE $203C,$0000, $003E, $ABFE;

PROCEDURE GXJobFormatModeQuery(theJob: gxJob; theQuery: gxQueryType; srcData: Ptr; dstData: Ptr);
	INLINE $203C,$0000, $003F, $ABFE;


{ Dialog Routines  (Advanced) }

PROCEDURE GXEnableJobScalingPanel(theJob: gxJob; enabled: Boolean);
	INLINE $203C,$0000, $0040, $ABFE;

PROCEDURE GXGetJobPanelDimensions(theJob: gxJob; VAR theSize: Rect);
	INLINE $203C,$0000, $0041, $ABFE;


{ PaperType Routines (Advanced) }

FUNCTION GXCountJobPaperTypes(theJob: gxJob; forFormatDevice: Boolean) : LongInt;
	INLINE $203C,$0000, $0042, $ABFE;

FUNCTION GXGetJobPaperType(theJob: gxJob; whichPaperType: LongInt; forFormatDevice: Boolean; inputPaperType: gxPaperType) : gxPaperType;
	INLINE $203C,$0000, $0043, $ABFE;

PROCEDURE GXForEachJobPaperTypeDo(theJob: gxJob; theProc: gxPaperTypeProc; refCon : UNIV Ptr; forFormatDevice: Boolean);
	INLINE $203C,$0000, $0044, $ABFE;

FUNCTION GXCopyPaperType(srcPaperType: gxPaperType; dstPaperType: gxPaperType) : gxPaperType;
	INLINE $203C,$0000, $0045, $ABFE;

PROCEDURE GXGetPaperTypeName(thePaperType: gxPaperType; VAR theName: Str31);
	INLINE $203C,$0000, $0046, $ABFE;

PROCEDURE GXGetPaperTypeDimensions(thePaperType: gxPaperType; VAR pageSize: gxRectangle; VAR paperSize: gxRectangle);
	INLINE $203C,$0000, $0047, $ABFE;

FUNCTION GXGetPaperTypeJob(thePaperType: gxPaperType) : gxJob;
	INLINE $203C,$0000, $0048, $ABFE;

FUNCTION GXGetPaperTypeCollection(thePaperType: gxPaperType) : Collection;
	INLINE $203C,$0000, $0049, $ABFE;


{ PrintViewer Routines (Advanced) }

FUNCTION GXOpenPrintFile(theJob: gxJob; whichFile: FSSpecPtr; permission: Byte) : gxPrintFile;
	INLINE $203C,$0000, $004A, $ABFE;

PROCEDURE GXClosePrintFile(thePrintFile: gxPrintFile);
	INLINE $203C,$0000, $004B, $ABFE;

FUNCTION GXGetPrintFileJob(thePrintFile: gxPrintFile) : gxJob;
	INLINE $203C,$0000, $004C, $ABFE;

FUNCTION GXCountPrintFilePages(thePrintFile: gxPrintFile) : LongInt;
	INLINE $203C,$0000, $004D, $ABFE;

PROCEDURE GXReadPrintFilePage(thePrintFile: gxPrintFile; pageNumber: LongInt; numViewPorts: LongInt; VAR viewPortList: gxViewPort; VAR theFormat: gxFormat; VAR theShape: gxShape);
	INLINE $203C,$0000, $004E, $ABFE;

PROCEDURE GXReplacePrintFilePage(thePrintFile: gxPrintFile; pageNumber: LongInt; theFormat: gxFormat; theShape: gxShape);
	INLINE $203C,$0000, $004F, $ABFE;

PROCEDURE GXInsertPrintFilePage(thePrintFile: gxPrintFile; atPageNumber: LongInt; theFormat: gxFormat; theShape: gxShape);
	INLINE $203C,$0000, $0050, $ABFE;

PROCEDURE GXDeletePrintFilePageRange(thePrintFile: gxPrintFile; fromPageNumber: LongInt; toPageNumber: LongInt);
	INLINE $203C,$0000, $0051, $ABFE;

PROCEDURE GXSavePrintFile(thePrintFile: gxPrintFile; toWhere: FSSpecPtr);
	INLINE $203C,$0000, $0052, $ABFE;


{ ColorSync Routines (Advanced) }

FUNCTION GXFindPrinterProfile(thePrinter: gxPrinter; searchData: Ptr; index: LongInt; VAR returnedProfile: gxColorProfile) : LongInt;
	INLINE $203C,$0000, $0053, $ABFE;

FUNCTION GXFindFormatProfile(theFormat: gxFormat; searchData: Ptr; index: LongInt; VAR returnedProfile: gxColorProfile) : LongInt;
	INLINE $203C,$0000, $0054, $ABFE;

PROCEDURE GXSetPrinterProfile(thePrinter: gxPrinter; oldProfile: gxColorProfile; newProfile: gxColorProfile);
	INLINE $203C,$0000, $0055, $ABFE;

PROCEDURE GXSetFormatProfile(theFormat: gxFormat; oldProfile: gxColorProfile; newProfile: gxColorProfile);
	INLINE $203C,$0000, $0056, $ABFE;


{$ENDC}    { UsingPrintingManager }

{$IFC NOT UsingIncludes}
    END.
{$ENDC}
