{
  scaler types.h
**
**      These are the data types relevent to the API of a scaler.  Included are the
**  types for managing memory blocks, and for communicating with scalers in the input
**  and output directions.
**
**  Copyright:   1992-93 by Apple Computer, Inc., all rights reserved.
**
**  Change History:
**
**  <1>     03/15/92 ABB    Created

}

{$IFC UNDEFINED UsingIncludes}
{$SETC UsingIncludes := 0}
{$ENDC}

{$IFC NOT UsingIncludes}
    UNIT ScalerTypes;
    INTERFACE
{$ENDC}

{$IFC UNDEFINED UsingScalerTypes}
{$SETC UsingScalerTypes := 1}


{$I+}
{$SETC ScalerTypesIncludes := UsingIncludes}
{$SETC UsingIncludes := 1}
{$IFC UNDEFINED UsingMathTypes}
{$I $$Shell(PInterfaces)MathTypes.p}
{$ENDC}
{$IFC UNDEFINED UsingSfntTypes}
{$I $$Shell(PInterfaces)SfntTypes.p}
{$ENDC}
{$SETC UsingIncludes := ScalerTypesIncludes}

CONST
scaler_no_problem = 0;
scaler_first_error = 1;
scaler_null_context = 1;
scaler_null_input = 2;
scaler_invalid_context = 3;
scaler_invalid_input = 4;
scaler_invalid_font_data = 5;
scaler_new_block_failed = 6;
scaler_bitmap_alloc_failed = 7;
scaler_outline_alloc_failed = 8;
scaler_required_table_missing = 9;
scaler_unsupported_outline_format = 10;
scaler_unsupported_stream_format = 11;
scaler_unsupported_font_format = 12;
scaler_hinting_error = 13;
scaler_scan_error = 14;
scaler_internal_error = 15;
scaler_invalid_matrix = 16;
scaler_fixed_overflow = 17;
scaler_API_version_mismatch = 18;
scaler_no_kerning_applied = 19;
scaler_last_error = 19;
scaler_first_warning = 20;
scaler_no_output = 20;
scaler_fake_metrics = 21;
scaler_fake_linespacing = 22;
scaler_glyph_substitution = 23;
scaler_last_warning = 23;

TYPE
GxScalerError = LongInt;
GxScalerErrorPointer = ^GxScalerError;
GxScalerErrorArray = ARRAY [0..0] OF GxScalerError;


{** ScalerOpen output type **}

GxScalerInfo = RECORD
	format: GxFontFormatTag;
	scalerVersion: Fixed;
	APIVersion: Fixed;
	END;
GxScalerInfoPointer = ^GxScalerInfo;
GxScalerInfoArray = ARRAY [0..0] OF GxScalerInfo;


{** ScalerNewFont output type **}


CONST
gxRequiresLayoutFont = 1;
gxHasNormalLayoutFont = 2;
gxCanReorderFont = 4;
gxCanRearrangeFont = 8;
gxHasOutlinesFont = 16;

TYPE
GxScalerFontFlag = LongInt;
GxScalerFontFlagPointer = ^GxScalerFontFlag;
GxScalerFontFlagArray = ARRAY [0..0] OF GxScalerFontFlag;


GxScalerFontInfo = RECORD
	unitsPerEm: UnsignedLong;
	flags: GxScalerFontFlag;
	numGlyphs: UnsignedLong;
	END;
GxScalerFontInfoPointer = ^GxScalerFontInfo;
GxScalerFontInfoArray = ARRAY [0..0] OF GxScalerFontInfo;


{** ScalerNewTransform input types **}


CONST
gxApplyHintsTransform = 1;
gxExactBitmapTransform = 2;
gxUseThresholdTransform = 4;
gxVerticalTransform = 8;
deviceMetricsTransform = 16;
allScalerTransformFlags = 31;

TYPE
GxScalerTransformFlag = LongInt;
GxScalerTransformFlagPointer = ^GxScalerTransformFlag;
GxScalerTransformFlagArray = ARRAY [0..0] OF GxScalerTransformFlag;


GxScalerTransform = RECORD
	flags: GxScalerTransformFlag;
	pointSize: Fixed;
	fontMatrix: ^GxMapping;
	resolution: GxPoint;
	spotSize: GxPoint;
	END;
GxScalerTransformPointer = ^GxScalerTransform;
GxScalerTransformArray = ARRAY [0..0] OF GxScalerTransform;


{** ScalerNewTransform output type **}

GxScalerTransformInfo = RECORD
	before: GxPoint;
	after: GxPoint;
	caretAngle: GxPoint;
	caretOffset: GxPoint;
	END;
GxScalerTransformInfoPointer = ^GxScalerTransformInfo;
GxScalerTransformInfoArray = ARRAY [0..0] OF GxScalerTransformInfo;


{** ScalerNewGlyph input types **}


CONST
gxNoImageGlyph = 1;

TYPE
GxScalerGlyphFlag = LongInt;
GxScalerGlyphFlagPointer = ^GxScalerGlyphFlag;
GxScalerGlyphFlagArray = ARRAY [0..0] OF GxScalerGlyphFlag;


ScalerOutlineFormat = LongInt;
ScalerOutlineFormatPointer = ^ScalerOutlineFormat;
ScalerOutlineFormatArray = ARRAY [0..0] OF ScalerOutlineFormat;


GxScalerGlyph = RECORD
	glyphIndex: LongInt;
	bandingTop: LongInt;
	bandingBottom: LongInt;
	format: ScalerOutlineFormat;
	flags: GxScalerGlyphFlag;
	END;
GxScalerGlyphPointer = ^GxScalerGlyph;
GxScalerGlyphArray = ARRAY [0..0] OF GxScalerGlyph;


{** ScalerNewGlyph output types **}

GxScalerMetrics = RECORD
	advance, sideBearing, otherSideBearing: GxPoint;
	END;
GxScalerMetricsPointer = ^GxScalerMetrics;
GxScalerMetricsArray = ARRAY [0..0] OF GxScalerMetrics;


GxScalerRectangle = RECORD
	xMin: LongInt;
	yMin: LongInt;
	xMax: LongInt;
	yMax: LongInt;
	END;
GxScalerRectanglePointer = ^GxScalerRectangle;
GxScalerRectangleArray = ARRAY [0..0] OF GxScalerRectangle;


GxScalerBitmap = RECORD
	image: ^SignedByte;
	topLeft: GxPoint;
	bounds: GxScalerRectangle;
	rowBytes: LongInt;
	END;
GxScalerBitmapPointer = ^GxScalerBitmap;
GxScalerBitmapArray = ARRAY [0..0] OF GxScalerBitmap;


{** ScalerKernGlyphs input/output types **}


CONST
gxLineStartKern = 1;
gxLineEndKern = 2;
gxNoCrossKern = 4;
allScalerKerningFlags = 7;

TYPE
ScalerKerningFlag = LongInt;
ScalerKerningFlagPointer = ^ScalerKerningFlag;
ScalerKerningFlagArray = ARRAY [0..0] OF ScalerKerningFlag;



CONST
gxNoStakeNote = 1;
gxCrossStreamResetNote = 2;

TYPE
ScalerKerningNote = UnsignedInteger;
ScalerKerningNotePointer = ^ScalerKerningNote;
ScalerKerningNoteArray = ARRAY [0..0] OF ScalerKerningNote;


GxScalerKerning = RECORD
	numGlyphs: LongInt;
	scaleFactor: Fract;
	flags: ScalerKerningFlag;
	glyphs: ^UnsignedInteger;
	END;
GxScalerKerningPointer = ^GxScalerKerning;
GxScalerKerningArray = ARRAY [0..0] OF GxScalerKerning;


{** ScalerStream input/output types **}


CONST
gxTruetypeStream = 1;
gxType1Stream = 2;
gxType3Stream = 4;
gxType42Stream = 8;
gxType42GXStream = 16;
gxPortableStream = 32;
gxFlattenedStream = 64;

TYPE
ScalerStreamType = LongInt;
ScalerStreamTypePointer = ^ScalerStreamType;
ScalerStreamTypeArray = ARRAY [0..0] OF ScalerStreamType;

{ Possible streamed gxFont formats }


CONST
gxDownloadFlag = 0;
gxASCIIDownloadFlag = 1;
gxFontSizeQueryFlag = 2;
gxEncodingOnlyFlag = 3;

TYPE
GxScalerStreamFlag = LongInt;
GxScalerStreamFlagPointer = ^GxScalerStreamFlag;
GxScalerStreamFlagArray = ARRAY [0..0] OF GxScalerStreamFlag;


GxScalerStream = RECORD
	streamRefCon: ^VOID;
	scalerStreamType: ScalerStreamType;
	flags: GxScalerStreamFlag;
	memorySize: UnsignedLong;
	variationCount: LongInt;
	variations: ^GxFontVariation;
	encoding: ^Integer;
	glyphBits: ^LongInt;
	fontName: ^SignedByte;
	END;
GxScalerStreamPointer = ^GxScalerStream;
GxScalerStreamArray = ARRAY [0..0] OF GxScalerStream;


GxScalerStreamData = RECORD
	hexFlag: LongInt;
	byteCount: LongInt;
	data: ^VOID;
	END;
GxScalerStreamDataPointer = ^GxScalerStreamData;
GxScalerStreamDataArray = ARRAY [0..0] OF GxScalerStreamData;



CONST
gxScalerScratchBlock = -1;
gxScalerOpenBlock = 0;
gxScalerFontBlock = 1;
gxScalerVariationBlock = 2;
gxScalerTransformBlock = 3;
gxScalerGlyphBlock = 4;
gxScalerBlockCount = 5;
gxScalerOutlineBlock = 5;
gxScalerBitmapBlock = 6;

TYPE
GxScalerBlockType = LongInt;
GxScalerBlockTypePointer = ^GxScalerBlockType;
GxScalerBlockTypeArray = ARRAY [0..0] OF GxScalerBlockType;


GxScalerContext = RECORD
	version: Fixed;
	theFont: ^VOID;
	format: GxFontFormatTag;
	GetFontTable: ProcPtr;
	ReleaseFontTable: ProcPtr;
	NewBlock: ProcPtr;
	DisposeBlock: ProcPtr;
	StreamFunction: ProcPtr;
	ScanLineFunction: ProcPtr;
	PostErrorFunction: ProcPtr;
	scalerBlocks: ARRAY [0..5] OF ^VOID;
	ScalerFunction: ProcPtr;
	END;
GxScalerContextPointer = ^GxScalerContext;
GxScalerContextArray = ARRAY [0..0] OF GxScalerContext;



{$ENDC}

{$IFC NOT UsingIncludes}
    END.
{$ENDC}
