/* ------------------------------------------------------------------------------

	FILENAME
		PrintingMessages.h

	DESCRIPTION
		This file defines all the universal printing messages,
		and Printing Manager API functions callable only from
		within a message override.

	COPYRIGHT
		Copyright  Apple Computer, Inc. 1989-1993
		All rights reserved. 
	
------------------------------------------------------------------------------- */


#ifndef __PRINTINGMESSAGES__
#define __PRINTINGMESSAGES__


#ifndef __PRINTINGMANAGER__
	#include <PrintingManager.h>
#endif

#ifndef __PRINTINGRESTYPES__
	#include <PrintingResTypes.h>
#endif

#ifndef __MESSAGEMANAGER__
	#include <Messages.h>
#endif

#ifndef __MENUS__
	#include <Menus.h>
#endif

#ifndef __LISTS__
	#include <Lists.h>
#endif

#ifndef __DIALOGS__
	#include <Dialogs.h>
#endif


/* ------------------------------------------------------------------------------

									Constants and Types

-------------------------------------------------------------------------------- */



/*--------------------------*/
/* abstract data types...   */
/*--------------------------*/



typedef struct gxPrivateFileRecord *gxSpoolFile;



/*---------------------------------------*/
/* dialog panel constants and types...   */
/*---------------------------------------*/



typedef long gxPanelEvent;

enum  {													// dialog panel events
	gxPanelNoEvt 					= (gxPanelEvent) 0,
	gxPanelOpenEvt 				= (gxPanelEvent) 1,			// initialize and draw
	gxPanelCloseEvt 				= (gxPanelEvent) 2,			// your panel is going away ( panel switch, confirm or cancel )
	gxPanelHitEvt					= (gxPanelEvent) 3,			// there's a hit in your panel
	gxPanelActivateEvt			= (gxPanelEvent) 4,			// the dialog window has just been activated
	gxPanelDeactivateEvt 		= (gxPanelEvent) 5,			// the dialog window is about to be deactivated
	gxPanelIconFocusEvt 			= (gxPanelEvent) 6,			// the focus changes from the panel to the icon list
	gxPanelPanelFocusEvt 		= (gxPanelEvent) 7,			// the focus changes from the icon list to the panel
	gxPanelFilterEvt				= (gxPanelEvent) 8,			// this is called to filter every event
	gxPanelCancelEvt				= (gxPanelEvent) 9,			// the user has cancelled the dialog
	gxPanelConfirmEvt				= (gxPanelEvent) 10,			// the user has confirmed the dialog
	gxPanelDialogEvt				= (gxPanelEvent) 11,			// event to be handle by dialoghandler ( you get to see all events )
	gxPanelOtherEvt				= (gxPanelEvent) 12,			// osEvts, etc.
	gxPanelUserWillConfirmEvt	= (gxPanelEvent) 13			// user has selected confirm, time to parse panel interdependencies
};


typedef long gxPanelResult;

enum {										// possible panel responses to dialog handler calls
	gxPanelNoResult			 	= 0,
	gxPanelCancelConfirmation 	= 1		// only valid from panelUserWillConfirmEvt - used to keep the dialog from going away
};


enum {										// values for the evtAction field in PanelInfoRecord
	gxOtherAction 				= 0,				// current item will not change
	gxClosePanelAction		= 1,				// panel will be closed
	gxCancelDialogAction		= 2,				// dialog will be cancelled
	gxConfirmDialogAction	= 3				// dialog will be confirmed
};


struct gxPanelInfoRecord{							// for FilterPanelEvent and HandlePanelEvent messages
	gxPanelEvent 	panelEvt;					// why we're calling
	short 			panelResId;					// 'ppnl' resource id of current panel
	DialogPtr 		pDlg;							// pointer to dialog
	EventRecord 	*theEvent;					// pointer to event
	short 			itemHit;						// actual item number as dialog mgr thinks
	short 			itemCount;					// number of items before your items
	short				evtAction;					// once this event is processed, the action that will result
															// (only meaningful in filtering, used for parsing)
	short				errorStringId;				// STR id of string to put in error alert (0 means no string)
	gxFormat			theFormat;					// the current format (only meaningful in a format dialog)
	void				*refCon;						// refCon passed in PanelSetupRecord
};

typedef struct gxPanelInfoRecord gxPanelInfoRecord;
					

typedef long gxPrintingPanelKind;

enum {
	gxApplicationPanel 	= (gxPrintingPanelKind) 0,
	gxExtensionPanel		= (gxPrintingPanelKind) 1,
	gxDriverPanel			= (gxPrintingPanelKind) 2
};
	

// The gxPanelSetupInfo structure is passed to the dialog handler when the client calls
// to setup panels for the dialog


struct gxPanelSetupRecord{
	gxPrintingPanelKind		panelKind;
	short 						panelResId;
	short 						resourceRefNum;
	void							*refCon;
};

typedef struct gxPanelSetupRecord gxPanelSetupRecord;


typedef long gxParsePageRangeResult;		// returned by gxParsePageRange message

enum {
	gxRangeNotParsed		= (gxParsePageRangeResult) 0,		// default initial value
	gxRangeParsed			= (gxParsePageRangeResult) 1,
	gxRangeBadFromValue	= (gxParsePageRangeResult) 2,
	gxRangeBadToValue		= (gxParsePageRangeResult) 3
};



/*----------------------------------------*/
/* status-related constants and types...  */
/*----------------------------------------*/



// status type ids


enum {
	gxNonFatalError = 1,				// effects icon on spooling dialog
	gxFatalError = 2,					// sends up printing alert on spooling dialog
	gxPrinterReady = 3,				// signals PFE to leave alert mode
	gxUserAttention = 4,				// signals initiation of a modal alert
	gxUserAlert = 5,					// signals initiation of a moveable modal alert
	gxPageTransmission = 6,			// signals page sent to printer, increments page count in strings to user
	gxOpenConnectionStatus = 7,	// signals PFE to begin animation on printer icon
	gxInformationalStatus = 8,		// default status type, no side effects
	gxSpoolingPageStatus = 9,		// signals page spooled, increments page count in spooling dialog
	gxEndStatus = 10,					// signals end of spooling
	gxPercentageStatus = 11			// signals the PFE as to the amount of the job which is currently complete
};


struct gxStatusRecord{							// for status messages
	unsigned short		statusType;			// one of the ids listed above ( nonFatalError, etc. )
	unsigned short		statusId;			// specific status ( out of paper, etc. ) - To be defined
	unsigned short		statusAlertId;		//	printing alert id (if any) for status
	Signature			statusOwner;		// creator type of status owner
	short					statResId;			// id for 'stat' resource
	short					statResIndex;		// index into 'stat' resource for this status
	short					dialogResult;		// id of button string selected on dismissal of printing alert
	unsigned short		bufferLen;			// number of bytes in status buffer - total record size must be <= 512
	char					statusBuffer[1];	// user response from alert
};

typedef struct gxStatusRecord gxStatusRecord;




struct gxDisplayRecord{							// for WriteStatusToDTPWindow message
	Boolean			useText;
	Handle			hPicture;
	Str255			theText;
};

typedef struct gxDisplayRecord gxDisplayRecord;



/*-----------------------------------------------*/
/* paper mapping-related constants and types...  */
/*-----------------------------------------------*/


typedef long gxTrayMapping;

enum {
	gxDefaultTrayMapping		= (gxTrayMapping) 0,
	gxConfiguredTrayMapping	= (gxTrayMapping) 1
};



/* ------------------------------------------------------------------------------

				API Functions callable only from within message overrides

-------------------------------------------------------------------------------- */



#define GXPROTECTEDAPIGLUE(selector) {0x203C, 0x0001, selector, 0xABFE}

#define GXPRINTINGDISPATCH(segID, selector) {0x203C, 0x0001, 0, 0x223C, (segID & 0x0FFF), selector << 2, 0xABFE}


#ifdef __cplusplus
extern "C" {
#endif

#pragma push
#pragma skipping on
#ifndef __powerc
#ifdef applec

	#pragma parameter __D0 GXPrintingDispatch(__D1)
	OSErr GXPrintingDispatch (long selector, ...)
  	  = GXPROTECTEDAPIGLUE(0);

#else

	OSErr GXPrintingDispatch (long selector, ...)
  	  = {0x221F, 0x203C, 0x0001, 0x0000, 0xABFE, 0x598F};

#endif
#endif
#pragma pop

/*
	How to use the GXPRINTINGDISPATCH macro...
	
	If your driver or extension is large, you may want to segment it
	across smaller boundaries than is permitted by the messaging system.
	Without using the Printing Manager's segmentation manager directly,
	the smallest segment you can create consists of the code to override
	a single message.  If you are overriding workhorse messages such as
	RenderPage, you may want to divide up the work among many functions
	distributed across several segments.  Here's how...
	
	The Printing Manager segment scheme involves the construction of a
	single 32-bit dispatch selector, which contains all the information
	necessary for the dispatcher to find a single routine.  It contains the
	segment's resource ID, and the offset within the segment which contains
	the start of the routine.  The GXPRINTINGDISPATCH macro will construct the
	dispatch selector for you, as well as the code to do the dispatch.
	
	Usually, it is convenient to start your segment with a long aligned jump table,
	beginning after the 4 byte header required by the Printing Manager.  The
	macro assumes this is the case and takes a 1-based routine selector from
	which it conmstructs the offset.
	
	For example, if your code is in resource 'pdvr' (print driver), ID=2
	at offset=12 (third routine in segment), you would declare your
	routine as follows:
	
	OSErr MyRenderingRoutine (long param1, Ptr param2)
		= GXPRINTINGDISPATCH(2, 3);
		
	Remember, ALL segment dispatches must return OSErr.  If your routine
	does not generate errors, you must still declare it to return OSErr
	and have the routine itself return noErr.
	
	An alternative way to call across segments is to call the GXPrintingDispatch
	function directly.  You must construct the 32-bit selector yourself and pass
	it as the first parameter.  This is usually not preferable since you don't get
	type-checking unless you declare a prototype as shown above, and your code
	isn't as easy to read.
	
	So given the above prototype, there are two ways to call the function:
	
		anErr = MyRenderingRoutine(p1, p2);			// free type checking!
		
	or:
	
		#define kMyRenderRoutineSelector 0x0002000C
		anErr = GXPrintingDispatch(kMyRenderRoutineSelector, p1, p2);		// no type-checking!
	
	
	Both have the same effect.
*/


gxJob GXGetJob (void)
    = GXPROTECTEDAPIGLUE(1);

short GXGetMessageHandlerResFile (void)
	= GXPROTECTEDAPIGLUE(2);
	
Boolean GXSpoolingAborted (void)
	= GXPROTECTEDAPIGLUE(3);
	
OSErr GXJobIdle (void)
	= GXPROTECTEDAPIGLUE(4);

#ifdef applec
OSErr GXReportStatus (short statusID, unsigned short statusIndex)
	= GXPROTECTEDAPIGLUE(5);
#else
OSErr GXReportStatus (long statusID, unsigned long statusIndex)
	= GXPROTECTEDAPIGLUE(5);
#endif

OSErr GXAlertTheUser (gxStatusRecord *)
	= GXPROTECTEDAPIGLUE(6);

OSErr GXSetupDialogPanel (gxPanelSetupRecord *)
	= GXPROTECTEDAPIGLUE(7);

OSErr GXCountTrays (gxTrayIndex *numTrays)
	= GXPROTECTEDAPIGLUE(8);
	
OSErr GXGetTrayName (gxTrayIndex trayNumber, Str31 trayName)
	= GXPROTECTEDAPIGLUE(9);

OSErr GXSetTrayPaperType (gxTrayIndex whichTray, gxPaperType)
	= GXPROTECTEDAPIGLUE(10);

OSErr GXGetTrayPaperType (gxTrayIndex whichTray, gxPaperType)
	= GXPROTECTEDAPIGLUE(11);

OSErr GXGetTrayMapping (gxTrayMapping *trayMapping)
	= GXPROTECTEDAPIGLUE(12);

void GXCleanupStartJob (void)
	= GXPROTECTEDAPIGLUE(13);

void GXCleanupStartPage (void)
	= GXPROTECTEDAPIGLUE(14);

void GXCleanupOpenConnection (void)
	= GXPROTECTEDAPIGLUE(15);

void GXCleanupStartSendPage (void)
	= GXPROTECTEDAPIGLUE(16);



/* ------------------------------------------------------------------------------

					Constants and types for Universal Printing Messages

-------------------------------------------------------------------------------- */



enum {											// options for gxCreateSpoolFile message
	gxNoCreateOptions	= 0x00000000,		// just create the file
	gxInhibitAlias		= 0x00000001,		// do not create an alias in the PMD folder
	gxInhibitUniqueName	= 0x00000002,	// do not append to the filename to make it unique
	gxResolveBitmapAlias	= 0x00000004	// resolve bitmap aliases and duplicate data in file
};


enum {										// options for gxCloseSpoolFile message
	gxNoCloseOptions	= 0x00000000,		// just close the file
	gxDeleteOnClose	= 0x00000001,		// delete the file rather than closing it
	gxUpdateJobData	= 0x00000002,		// write current job information into file prior to closing
	gxMakeRemoteFile	= 0x00000004		// mark job as a remote file
};


enum { 											// options for gxCreateImageFile message
	gxNoImageFile 		= 0x00000000,				// don't create image file
	gxMakeImageFile 	= 0x00000001,				// create an image file
	gxEachPlane			= 0x00000002,				// only save up planes before rewinding
	gxEachPage			= 0x00000004,				// save up entire pages before rewinding
	gxEntireFile		= gxEachPlane + gxEachPage	// save up the entire file before rewinding
};


enum {										// options for gxBufferData message
	gxNoBufferOptions	= 0x00000000, 
	gxMakeBufferHex		= 0x00000001, 
	gxDontSplitBuffer	= 0x00000002
};


struct gxPrintingBuffer{							// for gxDumpBuffer and gxFreeBuffer messages
	long			size;							//	size of buffer in bytes
	long			userData;					//	client assigned id for the buffer
	char			data[1];						// array of size bytes
};

typedef struct gxPrintingBuffer gxPrintingBuffer;


struct gxPageInfoRecord{							// for gxRenderPage message
	long		docPageNum;						// number of page being printed
	long		copyNum;							// copy number being printed
	Boolean	formatChanged;					// true if format changed from last page
	Boolean	pageChanged;					// true if page contents changed from last page
	long		internalUse;					// private
};

typedef struct gxPageInfoRecord gxPageInfoRecord;



/* ------------------------------------------------------------------------------

								Universal Printing Messages

-------------------------------------------------------------------------------- */


#define GXUNIVSENDGLUE(selector) SendMessageGlue(0x0000, selector)
#define GXFORWARDGLUE ForwardThisMessageGlue


#define Send_GXFetchTaggedDriverData(tag, id, pHandle) Send_GXFetchTaggedData(tag, id, pHandle, 'drvr')
#define Forward_GXFetchTaggedDriverData(tag, id, pHandle) Forward_GXFetchTaggedData(tag, id, pHandle, 'drvr')

#pragma procname GXJobIdle
typedef OSErr (*GXJobIdleProcPtr)(void);

OSErr Send_GXJobIdle (void)
    = GXUNIVSENDGLUE(gxJobIdle);
OSErr Forward_GXJobIdle (void)
    = GXFORWARDGLUE;

#pragma procname GXJobStatus
typedef OSErr (*GXJobStatusProcPtr)(gxStatusRecord *pStatus);

OSErr Send_GXJobStatus (gxStatusRecord *pStatus)
    = GXUNIVSENDGLUE(gxJobStatus);
OSErr Forward_GXJobStatus (gxStatusRecord *pStatus)
    = GXFORWARDGLUE;

#pragma procname GXPrintingEvent
typedef OSErr (*GXPrintingEventProcPtr)(EventRecord *eventPtr, Boolean filterEvent);

OSErr Send_GXPrintingEvent (EventRecord *, Boolean filterEvent)
    = GXUNIVSENDGLUE(gxPrintingEvent);
OSErr Forward_GXPrintingEvent (EventRecord *, Boolean filterEvent)
    = GXFORWARDGLUE;


#pragma procname GXJobDefaultFormatDialog
typedef OSErr (*GXJobDefaultFormatDialogProcPtr)(gxDialogResult *dlgResult);

OSErr Send_GXJobDefaultFormatDialog (gxDialogResult *dlgResult)
    = GXUNIVSENDGLUE(gxJobDefaultFormatDialog);
OSErr Forward_GXJobDefaultFormatDialog (gxDialogResult *dlgResult)
    = GXFORWARDGLUE;
	
#pragma procname GXFormatDialog
typedef OSErr (*GXFormatDialogProcPtr)(gxFormat theFormat, StringPtr title, gxDialogResult *dlgResult);

OSErr Send_GXFormatDialog (gxFormat theFormat, StringPtr title, gxDialogResult *dlgResult)
    = GXUNIVSENDGLUE(gxFormatDialog);
OSErr Forward_GXFormatDialog (gxFormat theFormat, StringPtr title, gxDialogResult *dlgResult)
    = GXFORWARDGLUE;
	
#pragma procname GXJobPrintDialog
typedef OSErr (*GXJobPrintDialogProcPtr)(gxDialogResult *dlgResult);

OSErr Send_GXJobPrintDialog (gxDialogResult *dlgResult)
    = GXUNIVSENDGLUE(gxJobPrintDialog);
OSErr Forward_GXJobPrintDialog (gxDialogResult *dlgResult)
    = GXFORWARDGLUE;

#pragma procname GXFilterPanelEvent
typedef OSErr (*GXFilterPanelEventProcPtr)(gxPanelInfoRecord *pHitInfo, Boolean *returnImmed);

OSErr Send_GXFilterPanelEvent (gxPanelInfoRecord *pHitInfo, Boolean *returnImmed)
	= GXUNIVSENDGLUE(gxFilterPanelEvent);

#pragma procname GXHandlePanelEvent
typedef OSErr (*GXHandlePanelEventProcPtr)(gxPanelInfoRecord *pHitInfo, gxPanelResult *panelResponse);

OSErr Send_GXHandlePanelEvent (gxPanelInfoRecord *pHitInfo, gxPanelResult *panelResponse)
	= GXUNIVSENDGLUE(gxHandlePanelEvent);

#pragma procname GXParsePageRange
typedef OSErr (*GXParsePageRangeProcPtr)(StringPtr fromString, StringPtr toString, gxParsePageRangeResult *result);

OSErr Send_GXParsePageRange (StringPtr fromString, StringPtr toString, gxParsePageRangeResult *result)
	= GXUNIVSENDGLUE(gxParsePageRange);
OSErr Forward_GXParsePageRange (StringPtr fromString, StringPtr toString, gxParsePageRangeResult *result)
	= GXFORWARDGLUE;
	
	
#pragma procname GXDefaultJob
typedef OSErr (*GXDefaultJobProcPtr)(void);

OSErr Send_GXDefaultJob (void)
    = GXUNIVSENDGLUE(gxDefaultJob);
OSErr Forward_GXDefaultJob (void)
    = GXFORWARDGLUE;
	
#pragma procname GXDefaultFormat
typedef OSErr (*GXDefaultFormatProcPtr)(gxFormat theFormat);

OSErr Send_GXDefaultFormat (gxFormat theFormat)
    = GXUNIVSENDGLUE(gxDefaultFormat);
OSErr Forward_GXDefaultFormat (gxFormat theFormat)
    = GXFORWARDGLUE;

#pragma procname GXDefaultPaperType
typedef OSErr (*GXDefaultPaperTypeProcPtr)(gxPaperType thePaperType);

OSErr Send_GXDefaultPaperType (gxPaperType thePaperType)
    = GXUNIVSENDGLUE(gxDefaultPaperType);
OSErr Forward_GXDefaultPaperType (gxPaperType thePaperType)
    = GXFORWARDGLUE;
	
#pragma procname GXDefaultPrinter
typedef OSErr (*GXDefaultPrinterProcPtr)(gxPrinter thePrinter);

OSErr Send_GXDefaultPrinter (gxPrinter thePrinter)
    = GXUNIVSENDGLUE(gxDefaultPrinter);
OSErr Forward_GXDefaultPrinter (gxPrinter thePrinter)
    = GXFORWARDGLUE;
	

#pragma procname GXCreateSpoolFile
typedef OSErr (*GXCreateSpoolFileProcPtr)(FSSpecPtr pFileSpec, long createOptions, gxSpoolFile *theSpoolFile);

OSErr Send_GXCreateSpoolFile (FSSpecPtr pFileSpec, long createOptions, gxSpoolFile *theSpoolFile)
    = GXUNIVSENDGLUE(gxCreateSpoolFile);
OSErr Forward_GXCreateSpoolFile (FSSpecPtr pFileSpec, long createOptions, gxSpoolFile *theSpoolFile)
    = GXFORWARDGLUE;
	
#pragma procname GXSpoolPage
typedef OSErr (*GXSpoolPageProcPtr)(gxSpoolFile theSpoolFile, gxFormat theFormat, gxShape thePage);

OSErr Send_GXSpoolPage (gxSpoolFile theSpoolFile, gxFormat theFormat, gxShape thePage)
    = GXUNIVSENDGLUE(gxSpoolPage);
OSErr Forward_GXSpoolPage (gxSpoolFile theSpoolFile, gxFormat theFormat, gxShape thePage)
    = GXFORWARDGLUE;

#pragma procname GXSpoolData
typedef OSErr (*GXSpoolDataProcPtr)(gxSpoolFile theSpoolFile, Ptr data, long *length);

OSErr Send_GXSpoolData (gxSpoolFile theSpoolFile, Ptr data, long *length)
    = GXUNIVSENDGLUE(gxSpoolData);
OSErr Forward_GXSpoolData (gxSpoolFile theSpoolFile, Ptr data, long *length)
    = GXFORWARDGLUE;

#ifdef applec
#pragma procname GXSpoolResource
typedef OSErr (*GXSpoolResourceProcPtr)(gxSpoolFile theSpoolFile, Handle theResource, ResType theType, short id);

OSErr Send_GXSpoolResource (gxSpoolFile theSpoolFile, Handle theResource, ResType theType, short id)
    = GXUNIVSENDGLUE(gxSpoolResource);
OSErr Forward_GXSpoolResource (gxSpoolFile theSpoolFile, Handle theResource, ResType theType, short id)
    = GXFORWARDGLUE;
#else
#pragma procname GXSpoolResource
typedef OSErr (*GXSpoolResourceProcPtr)(gxSpoolFile theSpoolFile, Handle theResource, ResType theType, long id);

OSErr Send_GXSpoolResource (gxSpoolFile theSpoolFile, Handle theResource, ResType theType, long id)
    = GXUNIVSENDGLUE(gxSpoolResource);
OSErr Forward_GXSpoolResource (gxSpoolFile theSpoolFile, Handle theResource, ResType theType, long id)
    = GXFORWARDGLUE;
#endif

#pragma procname GXCompleteSpoolFile
typedef OSErr (*GXCompleteSpoolFileProcPtr)(gxSpoolFile theSpoolFile);

OSErr Send_GXCompleteSpoolFile (gxSpoolFile theSpoolFile)
    = GXUNIVSENDGLUE(gxCompleteSpoolFile);
OSErr Forward_GXCompleteSpoolFile (gxSpoolFile theSpoolFile)
    = GXFORWARDGLUE;


#pragma procname GXCountPages
typedef OSErr (*GXCountPagesProcPtr)(gxSpoolFile theSpoolFile, long *numPages);

OSErr Send_GXCountPages (gxSpoolFile theSpoolFile, long *numPages)
    = GXUNIVSENDGLUE(gxCountPages);
OSErr Forward_GXCountPages (gxSpoolFile theSpoolFile, long *numPages)
    = GXFORWARDGLUE;

#pragma procname GXDespoolPage
typedef OSErr (*GXDespoolPageProcPtr)(gxSpoolFile theSpoolFile, long numPages, gxFormat theFormat, gxShape *thePage,  Boolean *formatChanged);

OSErr Send_GXDespoolPage (gxSpoolFile theSpoolFile, long numPages, gxFormat theFormat, gxShape *thePage,  Boolean *formatChanged)
    = GXUNIVSENDGLUE(gxDespoolPage);
OSErr Forward_GXDespoolPage (gxSpoolFile theSpoolFile, long numPages, gxFormat theFormat, gxShape *thePage,  Boolean *formatChanged)
    = GXFORWARDGLUE;

#pragma procname GXDespoolData
typedef OSErr (*GXDespoolDataProcPtr)(gxSpoolFile theSpoolFile, Ptr data, long *length);

OSErr Send_GXDespoolData (gxSpoolFile theSpoolFile, Ptr data, long *length)
    = GXUNIVSENDGLUE(gxDespoolData);
OSErr Forward_GXDespoolData (gxSpoolFile theSpoolFile, Ptr data, long *length)
    = GXFORWARDGLUE;

#ifdef applec
#pragma procname GXDespoolResource
typedef OSErr (*GXDespoolResourceProcPtr)(gxSpoolFile theSpoolFile, ResType theType, short id, Handle *theResource);

OSErr Send_GXDespoolResource (gxSpoolFile theSpoolFile, ResType theType, short id, Handle *theResource)
    = GXUNIVSENDGLUE(gxDespoolResource);
OSErr Forward_GXDespoolResource (gxSpoolFile theSpoolFile, ResType theType, short id, Handle *theResource)
    = GXFORWARDGLUE;
#else
#pragma procname GXDespoolResource
typedef OSErr (*GXDespoolResourceProcPtr)(gxSpoolFile theSpoolFile, ResType theType, long id, Handle *theResource);

OSErr Send_GXDespoolResource (gxSpoolFile theSpoolFile, ResType theType, long id, Handle *theResource)
    = GXUNIVSENDGLUE(gxDespoolResource);
OSErr Forward_GXDespoolResource (gxSpoolFile theSpoolFile, ResType theType, long id, Handle *theResource)
    = GXFORWARDGLUE;
#endif

#pragma procname GXCloseSpoolFile
typedef OSErr (*GXCloseSpoolFileProcPtr)(gxSpoolFile theSpoolFile, long closeOptions);

OSErr Send_GXCloseSpoolFile (gxSpoolFile theSpoolFile, long closeOptions)
    = GXUNIVSENDGLUE(gxCloseSpoolFile);
OSErr Forward_GXCloseSpoolFile (gxSpoolFile theSpoolFile, long closeOptions)
    = GXFORWARDGLUE;
	
	
#pragma procname GXStartJob
typedef OSErr (*GXStartJobProcPtr)(StringPtr docName, long pageCount);

OSErr Send_GXStartJob (StringPtr docName, long pageCount)
    = GXUNIVSENDGLUE(gxStartJob);
OSErr Forward_GXStartJob (StringPtr docName, long pageCount)
    = GXFORWARDGLUE;
	
#pragma procname GXFinishJob
typedef OSErr (*GXFinishJobProcPtr)(void);

OSErr Send_GXFinishJob (void)
    = GXUNIVSENDGLUE(gxFinishJob);
OSErr Forward_GXFinishJob (void)
    = GXFORWARDGLUE;
	
#pragma procname GXStartPage
typedef OSErr (*GXStartPageProcPtr)(gxFormat theFormat, long numViewPorts, gxViewPort *viewPortList);

OSErr Send_GXStartPage (gxFormat theFormat, long numViewPorts, gxViewPort *viewPortList)
	= GXUNIVSENDGLUE(gxStartPage);
OSErr Forward_GXStartPage (gxFormat theFormat, long numViewPorts, gxViewPort *viewPortList)
	= GXFORWARDGLUE;

#pragma procname GXFinishPage
typedef OSErr (*GXFinishPageProcPtr)(void);

OSErr Send_GXFinishPage (void)
	= GXUNIVSENDGLUE(gxFinishPage);
OSErr Forward_GXFinishPage (void)
	= GXFORWARDGLUE;
	
#pragma procname GXPrintPage
typedef OSErr (*GXPrintPageProcPtr)(gxFormat theFormat, gxShape thePage);

OSErr Send_GXPrintPage (gxFormat theFormat, gxShape thePage)
	= GXUNIVSENDGLUE(gxPrintPage);
OSErr Forward_GXPrintPage (gxFormat theFormat, gxShape thePage)
	= GXFORWARDGLUE;
	

#pragma procname GXSetupImageData
typedef OSErr (*GXSetupImageDataProcPtr)(void *imageData);

OSErr Send_GXSetupImageData (void *imageData)
    = GXUNIVSENDGLUE(gxSetupImageData);
OSErr Forward_GXSetupImageData (void *imageData)
    = GXFORWARDGLUE;

#pragma procname GXImageJob
typedef OSErr (*GXImageJobProcPtr)(gxSpoolFile, long *closeOptions);

OSErr Send_GXImageJob(gxSpoolFile, long *closeOptions)
    = GXUNIVSENDGLUE(gxImageJob);
OSErr Forward_GXImageJob (gxSpoolFile, long *closeOptions)
    = GXFORWARDGLUE;

#pragma procname GXImageDocument
typedef OSErr (*GXImageDocumentProcPtr)(gxSpoolFile theSpoolFile, void *imageData);

OSErr Send_GXImageDocument (gxSpoolFile theSpoolFile, void *imageData)
    = GXUNIVSENDGLUE(gxImageDocument);
OSErr Forward_GXImageDocument (gxSpoolFile theSpoolFile, void *imageData)
    = GXFORWARDGLUE;

#pragma procname GXImagePage
typedef OSErr (*GXImagePageProcPtr)(gxSpoolFile theSpoolFile, long pageNumber, gxFormat theFormat, void *imageData);

OSErr Send_GXImagePage (gxSpoolFile theSpoolFile, long pageNumber, gxFormat theFormat, void *imageData)
    = GXUNIVSENDGLUE(gxImagePage);
OSErr Forward_GXImagePage (gxSpoolFile theSpoolFile, long pageNumber, gxFormat theFormat, void *imageData)
    = GXFORWARDGLUE;

#pragma procname GXRenderPage
typedef OSErr (*GXRenderPageProcPtr)(gxFormat theFormat, gxShape thePage, gxPageInfoRecord *, void *imageData);

OSErr Send_GXRenderPage (gxFormat theFormat, gxShape thePage, gxPageInfoRecord *, void *imageData)
    = GXUNIVSENDGLUE(gxRenderPage);
OSErr Forward_GXRenderPage (gxFormat theFormat, gxShape thePage, gxPageInfoRecord *, void *imageData)
    = GXFORWARDGLUE;


#pragma procname GXCreateImageFile
typedef OSErr (*GXCreateImageFileProcPtr)(FSSpecPtr pFileSpec, long imageFileOptions, long *theImageFile);

OSErr Send_GXCreateImageFile (FSSpecPtr pFileSpec, long imageFileOptions, long *theImageFile)
    = GXUNIVSENDGLUE(gxCreateImageFile);
OSErr Forward_GXCreateImageFile (FSSpecPtr pFileSpec, long imageFileOptions, long *theImageFile)
    = GXFORWARDGLUE;


#pragma procname GXOpenConnection
typedef OSErr (*GXOpenConnectionProcPtr)(void);

OSErr Send_GXOpenConnection (void)
    = GXUNIVSENDGLUE(gxOpenConnection);
OSErr Forward_GXOpenConnection (void)
    = GXFORWARDGLUE;

#pragma procname GXCloseConnection
typedef OSErr (*GXCloseConnectionProcPtr)(void);

OSErr Send_GXCloseConnection (void)
    = GXUNIVSENDGLUE(gxCloseConnection);
OSErr Forward_GXCloseConnection (void)
    = GXFORWARDGLUE;

#pragma procname GXStartSendPage
typedef OSErr (*GXStartSendPageProcPtr)(gxFormat theFormat);

OSErr Send_GXStartSendPage (gxFormat theFormat)
    = GXUNIVSENDGLUE(gxStartSendPage);
OSErr Forward_GXStartSendPage (gxFormat theFormat)
    = GXFORWARDGLUE;

#pragma procname GXFinishSendPage
typedef OSErr (*GXFinishSendPageProcPtr)(void);

OSErr Send_GXFinishSendPage (void)
    = GXUNIVSENDGLUE(gxFinishSendPage);
OSErr Forward_GXFinishSendPage (void)
    = GXFORWARDGLUE;


#pragma procname GXWriteData
typedef OSErr (*GXWriteDataProcPtr)(Ptr data, long length);

OSErr Send_GXWriteData (Ptr data, long length)
    = GXUNIVSENDGLUE(gxWriteData);
OSErr Forward_GXWriteData (Ptr data, long length)
    = GXFORWARDGLUE;

#pragma procname GXBufferData
typedef OSErr (*GXBufferDataProcPtr)(Ptr data, long length, long bufferOptions);

OSErr Send_GXBufferData (Ptr data, long length, long bufferOptions)
    = GXUNIVSENDGLUE(gxBufferData);
OSErr Forward_GXBufferData (Ptr data, long length, long bufferOptions)
    = GXFORWARDGLUE;

#pragma procname GXDumpBuffer
typedef OSErr (*GXDumpBufferProcPtr)(gxPrintingBuffer *theBuffer);

OSErr Send_GXDumpBuffer (gxPrintingBuffer *theBuffer)
    = GXUNIVSENDGLUE(gxDumpBuffer);
OSErr Forward_GXDumpBuffer (gxPrintingBuffer *theBuffer)
    = GXFORWARDGLUE;

#pragma procname GXFreeBuffer
typedef OSErr (*GXFreeBufferProcPtr)(gxPrintingBuffer *theBuffer);

OSErr Send_GXFreeBuffer (gxPrintingBuffer *theBuffer)
    = GXUNIVSENDGLUE(gxFreeBuffer);
OSErr Forward_GXFreeBuffer (gxPrintingBuffer *theBuffer)
    = GXFORWARDGLUE;


#ifdef applec
#pragma procname GXCheckStatus
typedef OSErr (*GXCheckStatusProcPtr)(Ptr data, long length, short statusType, Signature owner);

OSErr Send_GXCheckStatus (Ptr data, long length, short statusType, Signature owner)
    = GXUNIVSENDGLUE(gxCheckStatus);
OSErr Forward_GXCheckStatus (Ptr data, long length, short statusType, Signature owner)
    = GXFORWARDGLUE;
#else
#pragma procname GXCheckStatus
typedef OSErr (*GXCheckStatusProcPtr)(Ptr data, long length, long statusType, Signature owner);

OSErr Send_GXCheckStatus (Ptr data, long length, long statusType, Signature owner)
    = GXUNIVSENDGLUE(gxCheckStatus);
OSErr Forward_GXCheckStatus (Ptr data, long length, long statusType, Signature owner)
    = GXFORWARDGLUE;
#endif

#pragma procname GXGetDeviceStatus
typedef OSErr (*GXGetDeviceStatusProcPtr)(Ptr cmdData, long cmdSize, Ptr responseData, long *responseSize, Str255 termination);

OSErr Send_GXGetDeviceStatus (Ptr cmdData, long cmdSize, Ptr responseData, long *responseSize, Str255 termination)
    = GXUNIVSENDGLUE(gxGetDeviceStatus);
OSErr Forward_GXGetDeviceStatus (Ptr cmdData, long cmdSize, Ptr responseData, long *responseSize, Str255 termination)
    = GXFORWARDGLUE;


#ifdef applec
#pragma procname GXFetchTaggedData
typedef OSErr (*GXFetchTaggedDataProcPtr)(ResType theType, short id, Handle *, Signature owner);

OSErr Send_GXFetchTaggedData (ResType theType, short id, Handle *, Signature owner)
	= GXUNIVSENDGLUE(gxFetchTaggedData);
OSErr Forward_GXFetchTaggedData (ResType theType, short id, Handle *, Signature owner)
	= GXFORWARDGLUE;
#else
#pragma procname GXFetchTaggedData
typedef OSErr (*GXFetchTaggedDataProcPtr)(ResType theType, long id, Handle *, Signature owner);

OSErr Send_GXFetchTaggedData (ResType theType, long id, Handle *, Signature owner)
	= GXUNIVSENDGLUE(gxFetchTaggedData);
OSErr Forward_GXFetchTaggedData (ResType theType, long id, Handle *, Signature owner)
	= GXFORWARDGLUE;
#endif


#pragma procname GXGetDTPMenuList
typedef OSErr (*GXGetDTPMenuListProcPtr)(MenuHandle menuHdl);

OSErr	Send_GXGetDTPMenuList (MenuHandle menuHdl)
	= GXUNIVSENDGLUE(gxGetDTPMenuList);
OSErr	Forward_GXGetDTPMenuList (MenuHandle menuHdl)
	= GXFORWARDGLUE;

#ifdef applec
#pragma procname GXDTPMenuSelect
typedef OSErr (*GXDTPMenuSelectProcPtr)(short id);

OSErr	Send_GXDTPMenuSelect (short id)
	= GXUNIVSENDGLUE(gxDTPMenuSelect);
OSErr	Forward_GXDTPMenuSelect (short id)
	= GXFORWARDGLUE;
#else
#pragma procname GXDTPMenuSelect
typedef OSErr (*GXDTPMenuSelectProcPtr)(long id);

OSErr	Send_GXDTPMenuSelect (long id)
	= GXUNIVSENDGLUE(gxDTPMenuSelect);
OSErr	Forward_GXDTPMenuSelect (long id)
	= GXFORWARDGLUE;
#endif

#pragma procname GXHandleAlertFilter
typedef OSErr (*GXHandleAlertFilterProcPtr)(gxJob theJob, gxStatusRecord *pStatusRec, DialogPtr pDialog, EventRecord *theEvent, short *itemHit, Boolean *returnImmed);

OSErr	Send_GXHandleAlertFilter (gxJob theJob, gxStatusRecord *pStatusRec, DialogPtr pDialog, EventRecord *theEvent, short *itemHit, Boolean *returnImmed)
	= GXUNIVSENDGLUE(gxHandleAlertFilter);
OSErr	Forward_GXHandleAlertFilter (gxJob theJob, gxStatusRecord *pStatusRec, DialogPtr pDialog, EventRecord *theEvent, short *itemHit, Boolean *returnImmed)
	= GXFORWARDGLUE;
	

#pragma procname GXJobFormatModeQuery
typedef OSErr (*GXJobFormatModeQueryProcPtr)(gxQueryType theQuery, void *srcData, void *dstData);

OSErr Send_GXJobFormatModeQuery (gxQueryType theQuery, void *srcData, void *dstData)
	= GXUNIVSENDGLUE(gxJobFormatModeQuery);
OSErr Forward_GXJobFormatModeQuery (gxQueryType theQuery, void *srcData, void *dstData)
	= GXFORWARDGLUE;


#pragma procname GXWriteStatusToDTPWindow
typedef OSErr (*GXWriteStatusToDTPWindowProcPtr)(gxStatusRecord *pStatusRec, gxDisplayRecord *pDisplay);

OSErr Send_GXWriteStatusToDTPWindow (gxStatusRecord *pStatusRec, gxDisplayRecord *pDisplay)
	= GXUNIVSENDGLUE(gxWriteStatusToDTPWindow);
OSErr Forward_GXWriteStatusToDTPWindow (gxStatusRecord *pStatusRec, gxDisplayRecord *pDisplay)
	= GXFORWARDGLUE;

#pragma procname GXInitializeStatusAlert
typedef OSErr (*GXInitializeStatusAlertProcPtr)(gxStatusRecord *pStatusRec, DialogPtr *pDialog);

OSErr Send_GXInitializeStatusAlert (gxStatusRecord *pStatusRec, DialogPtr *pDialog)
	= GXUNIVSENDGLUE(gxInitializeStatusAlert);
OSErr Forward_GXInitializeStatusAlert (gxStatusRecord *pStatusRec, DialogPtr *pDialog)
	= GXFORWARDGLUE;

#pragma procname GXHandleAlertStatus
typedef OSErr (*GXHandleAlertStatusProcPtr)(gxStatusRecord *pStatusRec);

OSErr Send_GXHandleAlertStatus (gxStatusRecord *pStatusRec)
	= GXUNIVSENDGLUE(gxHandleAlertStatus);
OSErr Forward_GXHandleAlertStatus (gxStatusRecord *pStatusRec)
	= GXFORWARDGLUE;
	
#pragma procname GXHandleAlertEvent
typedef OSErr (*GXHandleAlertEventProcPtr)(gxStatusRecord *pStatusRec, DialogPtr pDialog, EventRecord *theEvent, short *response);

OSErr Send_GXHandleAlertEvent (gxStatusRecord *pStatusRec, DialogPtr pDialog, EventRecord *theEvent, short *response)
	= GXUNIVSENDGLUE(gxHandleAlertEvent);
OSErr Forward_GXHandleAlertEvent (gxStatusRecord *pStatusRec, DialogPtr pDialog, EventRecord *theEvent, short *response)
	= GXFORWARDGLUE;
		

#pragma procname GXCleanupStartJob
typedef void (*GXCleanupStartJobProcPtr)(void);

void Send_GXCleanupStartJob (void)
	= GXUNIVSENDGLUE(gxCleanupStartJob);
void Forward_GXCleanupStartJob (void)
	= GXFORWARDGLUE;

#pragma procname GXCleanupStartPage
typedef void (*GXCleanupStartPageProcPtr)(void);

void Send_GXCleanupStartPage (void)
	= GXUNIVSENDGLUE(gxCleanupStartPage);
void Forward_GXCleanupStartPage (void)
	= GXFORWARDGLUE;

#pragma procname GXCleanupOpenConnection
typedef void (*GXCleanupOpenConnectionProcPtr)(void);

void Send_GXCleanupOpenConnection (void)
	= GXUNIVSENDGLUE(gxCleanupOpenConnection);
void Forward_GXCleanupOpenConnection (void)
	= GXFORWARDGLUE;

#pragma procname GXCleanupStartSendPage
typedef void (*GXCleanupStartSendPageProcPtr)(void);

void Send_GXCleanupStartSendPage (void)
	= GXUNIVSENDGLUE(gxCleanupStartSendPage);
void Forward_GXCleanupStartSendPage (void)
	= GXFORWARDGLUE;


#pragma procname GXDefaultDesktopPrinter
typedef OSErr (*GXDefaultDesktopPrinterProcPtr)(Str31);

OSErr Send_GXDefaultDesktopPrinter (Str31 dtpName)
	= GXUNIVSENDGLUE(gxDefaultDesktopPrinter);
OSErr Forward_GXDefaultDesktopPrinter (Str31 dtpName)
	= GXFORWARDGLUE;
	
#pragma procname GXCaptureOutputDevice
typedef OSErr (*GXCaptureOutputDeviceProcPtr)(Boolean capture);

OSErr	Send_GXCaptureOutputDevice (Boolean capture)
	= GXUNIVSENDGLUE(gxCaptureOutputDevice);
OSErr	Forward_GXCaptureOutputDevice (Boolean capture)
	= GXFORWARDGLUE;


#pragma procname GXOpenConnectionRetry
typedef OSErr (*GXOpenConnectionRetryProcPtr)(ResType theType, void *, Boolean *retryopenPtr, OSErr anErr);

OSErr Send_GXOpenConnectionRetry (ResType theType, void *, Boolean *retryopenPtr, OSErr anErr)
	= GXUNIVSENDGLUE(gxOpenConnectionRetry);
OSErr Forward_GXOpenConnectionRetry (ResType theType, void *, Boolean *retryopenPtr, OSErr anErr)
	= GXFORWARDGLUE;
	
#pragma procname GXExamineSpoolFile
typedef OSErr (*GXExamineSpoolFileProcPtr)(gxSpoolFile theSpoolFile);

OSErr Send_GXExamineSpoolFile (gxSpoolFile theSpoolFile)
	= GXUNIVSENDGLUE(gxExamineSpoolFile);
OSErr Forward_GXExamineSpoolFile (gxSpoolFile theSpoolFile)
	= GXFORWARDGLUE;

#pragma procname GXFinishSendPlane
typedef OSErr (*GXFinishSendPlaneProcPtr)(void);

OSErr Send_GXFinishSendPlane (void)
	= GXUNIVSENDGLUE(gxFinishSendPlane);
OSErr Forward_GXFinishSendPlane (void)
	= GXFORWARDGLUE;

#pragma procname GXDoesPaperFit
typedef OSErr (*GXDoesPaperFitProcPtr)( gxTrayIndex whichTray, gxPaperType paper, Boolean *fits );

OSErr Send_GXDoesPaperFit( gxTrayIndex whichTray, gxPaperType paper, Boolean *fits )
	= GXUNIVSENDGLUE(gxDoesPaperFit);
OSErr Forward_GXDoesPaperFit( gxTrayIndex whichTray, gxPaperType paper, Boolean *fits )
	= GXFORWARDGLUE;
	
#ifdef applec
#pragma procname GXChooserMessage
typedef OSErr (*GXChooserMessageProcPtr)(short message, short caller, StringPtr objName,
									StringPtr zoneName, ListHandle theList, long p2);

OSErr Send_GXChooserMessage (short message, short caller, StringPtr objName,
									StringPtr zoneName, ListHandle theList, long p2)
	= GXUNIVSENDGLUE(gxChooserMessage);
OSErr Forward_GXChooserMessage (short message, short caller, StringPtr objName,
										StringPtr zoneName, ListHandle theList, long p2)
	= GXFORWARDGLUE;
#else
#pragma procname GXChooserMessage
typedef OSErr (*GXChooserMessageProcPtr)(long message, long caller, StringPtr objName,
									StringPtr zoneName, ListHandle theList, long p2);

OSErr Send_GXChooserMessage (long message, long caller, StringPtr objName,
									StringPtr zoneName, ListHandle theList, long p2)
	= GXUNIVSENDGLUE(gxChooserMessage);
OSErr Forward_GXChooserMessage (long message, long caller, StringPtr objName,
										StringPtr zoneName, ListHandle theList, long p2)
	= GXFORWARDGLUE;
#endif


#pragma procname GXFindPrinterProfile
typedef OSErr (*GXFindPrinterProfileProcPtr)(gxPrinter thePrinter, void *searchData, long index, gxColorProfile *returnedProfile, long *numProfiles);

OSErr Send_GXFindPrinterProfile (gxPrinter thePrinter, void *searchData, long index, gxColorProfile *returnedProfile, long *numProfiles)
	= GXUNIVSENDGLUE(gxFindPrinterProfile);
OSErr Forward_GXFindPrinterProfile (gxPrinter thePrinter, void *searchData, long index, gxColorProfile *returnedProfile, long *numProfiles)
	= GXFORWARDGLUE;
	
#pragma procname GXFindFormatProfile
typedef OSErr (*GXFindFormatProfileProcPtr)(gxFormat theFormat, void *searchData, long index, gxColorProfile *returnedProfile, long *numProfiles);

OSErr Send_GXFindFormatProfile (gxFormat theFormat, void *searchData, long index, gxColorProfile *returnedProfile, long *numProfiles)
	= GXUNIVSENDGLUE(gxFindFormatProfile);
OSErr Forward_GXFindFormatProfile (gxFormat theFormat, void *searchData, long index, gxColorProfile *returnedProfile, long *numProfiles)
	= GXFORWARDGLUE;
	
#pragma procname GXSetPrinterProfile
typedef OSErr (*GXSetPrinterProfileProcPtr)(gxPrinter thePrinter, gxColorProfile oldProfile, gxColorProfile newProfile);

OSErr Send_GXSetPrinterProfile (gxPrinter thePrinter, gxColorProfile oldProfile, gxColorProfile newProfile)
	= GXUNIVSENDGLUE(gxSetPrinterProfile);
OSErr Forward_GXSetPrinterProfile (gxPrinter thePrinter, gxColorProfile oldProfile, gxColorProfile newProfile)
	= GXFORWARDGLUE;

#pragma procname GXSetFormatProfile
typedef OSErr (*GXSetFormatProfileProcPtr)(gxFormat theFormat, gxColorProfile oldProfile, gxColorProfile newProfile);

OSErr Send_GXSetFormatProfile (gxFormat theFormat, gxColorProfile oldProfile, gxColorProfile newProfile)
	= GXUNIVSENDGLUE(gxSetFormatProfile);
OSErr Forward_GXSetFormatProfile (gxFormat theFormat, gxColorProfile oldProfile, gxColorProfile newProfile)
	= GXFORWARDGLUE;

#ifdef __cplusplus
}
#endif


#endif
