/* graphics:
	public object and constant definitions
	by Cary Clark, Georgiann Delaney, Herb Derby, Michael Fairman, Pablo Fernicola, Dave Good, Josh Horwich, Barton House, Robert Johnson, Keith McGreggor, Mike Reed, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1994 Apple Computer, Inc.  All rights reserved.	*/

#pragma once

#ifndef graphicsTypesIncludes
	#define graphicsTypesIncludes

	#ifndef mathTypesIncludes
		#include "math types.h"
	#endif

	#ifndef gxAnyNumber
		#define gxAnyNumber 1
	#endif

	#ifndef nil
		#define nil 0L
	#endif
	
/* graphics objects */

		typedef struct gxPrivateShapeRecord *gxShape;
		typedef struct gxPrivateStyleRecord *gxStyle;
		typedef struct gxPrivateInkRecord *gxInk;
		typedef struct gxPrivateTransformRecord *gxTransform;
		typedef struct gxPrivateTagRecord *gxTag;
		typedef struct gxPrivateColorSetRecord *gxColorSet;
		typedef struct gxPrivateProfileRecord *gxColorProfile;
		typedef struct gxPrivateGraphicsClientRecord *gxGraphicsClient;
		typedef struct gxPrivateViewGroupRecord *gxViewGroup;
		typedef struct gxPrivateViewPortRecord *gxViewPort;
		typedef struct gxPrivateViewDeviceRecord *gxViewDevice;

	typedef long gxColorSpace;

/* gxShape enumerations */

	enum gxShapeTypes {
		gxEmptyType = 1,
		gxPointType,
		gxLineType,
		gxCurveType,
		gxRectangleType,
		gxPolygonType,
		gxPathType,
		gxBitmapType,
		gxTextType,
		gxGlyphType,
		gxLayoutType,
		gxFullType,
		gxPictureType
	};

	typedef long gxShapeType;

	enum gxShapeFills {
		gxNoFill,
		gxOpenFrameFill,
		gxFrameFill = gxOpenFrameFill,
		gxClosedFrameFill,
		gxHollowFill = gxClosedFrameFill,
		gxEvenOddFill,
		gxSolidFill = gxEvenOddFill,
		gxWindingFill,
		gxInverseEvenOddFill,
		gxInverseSolidFill = gxInverseEvenOddFill,
		gxInverseFill = gxInverseEvenOddFill,
		gxInverseWindingFill
	};

	typedef long gxShapeFill;

	enum gxShapeAttributes {
		gxNoAttributes,
		gxDirectShape			= 0x0001,
		gxRemoteShape			= 0x0002,
		gxCachedShape			= 0x0004,
		gxLockedShape			= 0x0008,
		gxGroupShape			= 0x0010,
		gxMapTransformShape  	= 0x0020,
		gxUniqueItemsShape		= 0x0040,
		gxIgnorePlatformShape 	= 0x0080,
		gxNoMetricsGridShape  	= 0x0100,
		gxDiskShape			= 0x0200,
		gxMemoryShape		= 0x0400
	};

	typedef long gxShapeAttribute;

/* gxShape editing enumerations */

	enum gxEditShapeFlags {
		gxBreakNeitherEdit			= 0,
		gxBreakLeftEdit			= 0x0001,
		gxBreakRightEdit			= 0x0002,
		gxRemoveDuplicatePointsEdit	= 0x0004		/* if the new first (or last) gxPoint exactly matches the gxPoint before it in */
											/* the same contour, then remove it) */
	};

	typedef long gxEditShapeFlag;

	#define gxSelectToEnd -1
	#define gxSetToNil	(void *)(-1)

	enum gxContourDirections {
		gxCounterclockwiseDirection,
		gxClockwiseDirection
	};
	
	typedef long gxContourDirection;

/* gxShape structures */

/* The type 'gxPoint' is defined in "math types.h" */
/*
	struct gxPoint {
		Fixed		x;
		Fixed  		y;
	};
*/

	struct gxLine {
		struct gxPoint	first;
		struct gxPoint	last;
	} ;
	
	struct gxCurve {
		struct gxPoint	first;
		struct gxPoint	control;
		struct gxPoint	last;
	};

	struct gxRectangle {
		Fixed		left;
		Fixed		top;
		Fixed		right;
		Fixed		bottom;
	};

	struct gxPolygon {
		long			vectors;
		struct gxPoint	vector[gxAnyNumber];
	};

	struct gxPolygons {
		long			contours;
		struct gxPolygon contour[gxAnyNumber];
	};

	struct gxPath {
		long			vectors;
		long			controlBits[gxAnyNumber];
		struct gxPoint	vector[gxAnyNumber];
	};

	struct gxPaths {
		long			contours;
		struct gxPath	contour[gxAnyNumber];
	};

	struct gxBitmap {
		char			*image;		/* pointer to pixels */
		long			width;		/* width in pixels */
		long			height;		/* height in pixels */
		long			rowBytes;	/* width in bytes */
		long			pixelSize;		/* physical bits per pixel */
		gxColorSpace  	space;
		gxColorSet	set;
		gxColorProfile	profile;
	};
	
	struct gxLongRectangle {
		long			left;
		long			top;
		long			right;
		long			bottom;
	};

	#ifndef __cplusplus
		typedef struct gxLine gxLine;
		typedef struct gxCurve gxCurve;
		typedef struct gxRectangle gxRectangle;
		typedef struct gxPolygon gxPolygon;
		typedef struct gxPolygons gxPolygons;
		typedef struct gxPath gxPath;
		typedef struct gxPaths gxPaths;
		typedef struct gxBitmap gxBitmap;
		typedef struct gxLongRectangle gxLongRectangle;
	#endif

/* gxStyle enumerations */

	enum gxStyleAttributes {
		gxCenterFrameStyle			= 0,
		gxSourceGridStyle 			= 0x0001,
		gxDeviceGridStyle 			= 0x0002,
		gxInsideFrameStyle			= 0x0004,
		gxOutsideFrameStyle		= 0x0008,
		gxAutoInsetStyle			= 0x0010
	};

	typedef long gxStyleAttribute;

	enum gxDashAttributes {
		gxBendDash				= 0x0001,
		gxBreakDash				= 0x0002,
		gxClipDash				= 0x0004,
		gxLevelDash				= 0x0008,
		gxAutoAdvanceDash 			= 0x0010
	};
	
	typedef long gxDashAttribute;
	
	enum gxPatternAttributes {
		gxPortAlignPattern			= 0x0001,
		gxPortMapPattern			= 0x0002
	};
	
	typedef long gxPatternAttribute;

	enum gxJoinAttributes {
		gxSharpJoin				= 0x0000,
		gxCurveJoin				= 0x0001,
		gxLevelJoin				= 0x0002
	};
	
	typedef long gxJoinAttribute;

	enum gxCapAttributes {
		gxLevelStartCap			= 0x0001,
		gxLevelEndCap				= 0x0002
	};
	
	typedef long gxCapAttribute;

	enum gxTextAttributes {
		gxAutoAdvanceText 			= 0x0001,
		gxNoContourGridText		= 0x0002,
		gxNoMetricsGridText		= 0x0004,
		gxAnchorPointsText			= 0x0008,
		gxVerticalText				= 0x0010,
		gxNoOpticalScaleText  		= 0x0020
	};
	
	typedef long gxTextAttribute;	

	#define gxLeftJustify 0
	#define gxCenterJustify (fract1/2)
	#define gxRightJustify fract1
	#define gxFillJustify -1

	enum gxLayerFlags {
		gxUnderlineAdvanceLayer		= 0x0001,	/* a gxLine is drawn through the advances */
		gxSkipWhiteSpaceLayer		= 0x0002,	/* except characters describing white space */
		gxUnderlineIntervalLayer		= 0x0004,	/* (+ gxStringLayer) a gxLine is drawn through the gaps between advances */
		gxUnderlineContinuationLayer	= 0x0008,	/* (+ gxStringLayer) join this underline with another face */
		gxWhiteLayer				= 0x0010,	/* the layer draws to white instead of black */
		gxClipLayer				= 0x0020,	/* the characters define a clip */
		gxStringLayer				= 0x0040		/* all characters in run are combined */
	};

	typedef long gxLayerFlag;

/* gxStyle structures */

	struct gxFaceLayer {
		gxShapeFill		outlineFill;			/* outline framed or filled */
		gxLayerFlag		flags;				/* various additional effects */
		gxStyle			outlineStyle;			/* outline */
		gxTransform		outlineTransform;		/* italic, condense, extend */
		gxPoint			boldOutset;			/* bold */
	};

	struct gxTextFace {
		long				faceLayers;			/* layer to implement shadow */
		gxMapping			advanceMapping;		/* algorithmic change to advance width */
		struct gxFaceLayer	faceLayer[gxAnyNumber]; /* zero or more face layers describing the face */
	};

	struct gxJoinRecord {
		gxJoinAttribute	attributes;
		gxShape			join;
		Fixed			miter;
	};

	struct gxDashRecord {
		gxDashAttribute	attributes;
		gxShape			dash;				/* similar to pattern, except rotated to gxLine slope */
		Fixed			advance;				/* specifies repeating frequency of dash */
		fract				phase;  				/* specifies offset into the gxPath to start dashing */
		Fixed			scale;  				/* specifies height of dash to be mapped to penWidth */
	};

	struct gxPatternRecord {
		gxPatternAttribute	attributes;
		gxShape			pattern;
		gxPoint			u;
		gxPoint			v;
	};

	struct gxCapRecord {
		gxCapAttribute		attributes;
		gxShape			startCap;
		gxShape			endCap;
	};

	#ifndef __cplusplus
		typedef struct gxFaceLayer gxFaceLayer;
		typedef struct gxTextFace gxTextFace;
		typedef struct gxJoinRecord gxJoinRecord;
		typedef struct gxDashRecord gxDashRecord;
		typedef struct gxPatternRecord gxPatternRecord;
		typedef struct gxCapRecord gxCapRecord;
	#endif

/* gxInk enumerations */

	enum gxInkAttributes {
		gxPortAlignDitherInk		= 0x0001,
		gxForceDitherInk		= 0x0002,
		gxSuppressDitherInk		= 0x0004,
		gxSuppressHalftoneInk	= 0x0008
	};

	typedef long gxInkAttribute;

	enum gxComponentModes {
		gxNoMode = 0,
		gxCopyMode,
		gxAddMode,
		gxBlendMode,
		gxMigrateMode,
		gxMinimumMode,
		gxMaximumMode,
		gxHighlightMode,
		gxAndMode,
		gxOrMode,
		gxXorMode,
		gxRampAndMode,
		gxRampOrMode,
		gxRampXorMode,
		gxOverMode,				/* Alpha channel modes				*/
		gxAtopMode,				/* Note: In England = Beta channel modes		*/
		gxExcludeMode,
		gxFadeMode
	};

	typedef unsigned char gxComponentMode;

	enum gxTransferFlags {
		gxRejectSourceTransfer		= 0x0001,	/* at least one component must be out of range		*/
		gxRejectDeviceTransfer		= 0x0002,	/* at least one component must be out of range		*/
		gxSingleComponentTransfer	= 0x0004		/* duplicate gxTransferComponent[0] for all components in transfer */
	};
	
	typedef long gxTransferFlag;
	
	enum gxComponentFlags {
		gxOverResultComponent 		= 0x01,	/* & result gxColor with 0xFFFF before clamping	*/
		gxReverseComponent		= 0x02	/* reverse source and device before moding		*/
	};

	typedef unsigned char gxComponentFlag;

/* gxInk structures */

	struct gxTransferComponent {
		gxComponentMode	mode;			/* how the component is operated upon				*/
		gxComponentFlag	flags;			/* flags for each component						*/
		gxColorValue		sourceMinimum;
		gxColorValue		sourceMaximum;	/* input filter range								*/
		gxColorValue		deviceMinimum;
		gxColorValue		deviceMaximum;	/* output filter range							*/
		gxColorValue		clampMinimum;
		gxColorValue		clampMaximum;	/* output clamping range							*/
		gxColorValue		operand;			/* ratio for blend, step for migrate, gxColor for highlight	*/
	} ;

	struct gxTransferMode {
		gxColorSpace		space;			/* the gxColor-space the transfer mode is to operate in	*/
		gxColorSet		set;
		gxColorProfile		profile;
		Fixed			sourceMatrix[5][4];
		Fixed			deviceMatrix[5][4];
		Fixed			resultMatrix[5][4];
		gxTransferFlag		flags;
		struct gxTransferComponent component[4];  /* how each component is operated upon			*/
	};

	#ifndef __cplusplus
		typedef struct gxTransferComponent gxTransferComponent;
		typedef struct gxTransferMode gxTransferMode;
	#endif
	
/* gxColor space enumerations */

	enum gxColorPackingTypes {
		gxNoColorPacking		= 0x0000,	/* 16 bits per channel */
		gxAlphaSpace			= 0x0080,	/* space includes alpha channel */
		gxWord5ColorPacking	= 0x0500,	/* 5 bits per channel, right-justified */
		gxLong8ColorPacking		= 0x0800,	/* 8 bits per channel, right-justified */
		gxLong10ColorPacking  	= 0x0a00,	/* 10 bits per channel, right-justified */
		gxAlphaFirstPacking		= 0x1000		/* alpha channel is the first field in the packed space */
	};
	
	enum gxColorSpaces {
		gxNoSpace = 0,
		gxRGBSpace,
		gxCMYKSpace,
		gxHSVSpace,
		gxHLSSpace,
		gxYXYSpace,
		gxXYZSpace,
		gxLUVSpace,
		gxLABSpace,
		gxYIQSpace,
		gxNTSCSpace		= gxYIQSpace,
		gxPALSpace		= gxYIQSpace,
		gxGraySpace,
		gxIndexedSpace,
		gxRGBASpace		= gxRGBSpace + gxAlphaSpace,
		gxGrayASpace  	= gxGraySpace + gxAlphaSpace,
		gxRGB16Space  	= gxWord5ColorPacking + gxRGBSpace,
		gxRGB32Space  	= gxLong8ColorPacking + gxRGBSpace,
		gxARGB32Space 	= gxLong8ColorPacking + gxAlphaFirstPacking + gxRGBASpace,
		gxCMYK32Space 	= gxLong8ColorPacking + gxCMYKSpace,
		gxHSV32Space  	= gxLong10ColorPacking + gxHSVSpace,
		gxHLS32Space  	= gxLong10ColorPacking + gxHLSSpace,
		gxYXY32Space  	= gxLong10ColorPacking + gxYXYSpace,
		gxXYZ32Space  	= gxLong10ColorPacking + gxXYZSpace,
		gxLUV32Space  	= gxLong10ColorPacking + gxLUVSpace,
		gxLAB32Space  	= gxLong10ColorPacking + gxLABSpace,
		gxYIQ32Space  		= gxLong10ColorPacking + gxYIQSpace,
		gxNTSC32Space	= gxYIQ32Space,
		gxPAL32Space  	= gxYIQ32Space
	};

/* gxColor space structures */

	struct gxRGBColor {
		gxColorValue 	red;
		gxColorValue 	green;
		gxColorValue 	blue;
	};

	struct gxRGBAColor {
		gxColorValue 	red;
		gxColorValue 	green;
		gxColorValue 	blue;
		gxColorValue 	alpha;
	};

	struct gxHSVColor {
		gxColorValue 	hue;
		gxColorValue 	saturation;
		gxColorValue 	value;
	};

	struct gxHLSColor {
		gxColorValue 	hue;
		gxColorValue 	lightness;
		gxColorValue 	saturation;
	};

	struct gxCMYKColor {
		gxColorValue 	cyan;
		gxColorValue 	magenta;
		gxColorValue 	yellow;
		gxColorValue 	black;
	};

	struct gxXYZColor {
		gxColorValue	x;
		gxColorValue	y;
		gxColorValue	z;
	};
	
	struct gxYXYColor {
		gxColorValue	capY;
		gxColorValue	x;
		gxColorValue	y;
	};
	
	struct gxLUVColor {
		gxColorValue	l;
		gxColorValue	u;
		gxColorValue	v;
	};
	
	struct gxLABColor {
		gxColorValue	l;
		gxColorValue	a;
		gxColorValue	b;
	};

	struct gxYIQColor {
		gxColorValue 	y;
		gxColorValue 	i;
		gxColorValue 	q;
	};

	struct gxGrayAColor {
		gxColorValue 	gray;
		gxColorValue 	alpha;
	};

	typedef long gxColorIndex;

	struct gxIndexedColor {
		gxColorIndex  	index;
		gxColorSet	set;
	};

	struct gxColor {
		gxColorSpace 	space;
		gxColorProfile	profile;
		union {
			struct gxCMYKColor		cmyk;
			struct gxRGBColor		rgb;
			struct gxRGBAColor		rgba;
			struct gxHSVColor		hsv;
			struct gxHLSColor		hls;
			struct gxXYZColor		xyz;
			struct gxYXYColor		yxy;
			struct gxLUVColor		luv;
			struct gxLABColor		lab;
			struct gxYIQColor		yiq;
			gxColorValue			gray;
			struct gxGrayAColor		graya;
			unsigned short			pixel16;
			unsigned long			pixel32;
			struct gxIndexedColor	indexed;
			gxColorValue			component[4];
		} element;
	};

	#ifndef __cplusplus
		typedef struct gxRGBColor gxRGBColor;
		typedef struct gxRGBAColor gxRGBAColor;
		typedef struct gxHSVColor gxHSVColor;
		typedef struct gxHLSColor gxHLSColor;
		typedef struct gxCMYKColor gxCMYKColor;
		typedef struct gxXYZColor gxXYZColor;
		typedef struct gxYXYColor gxYXYColor;
		typedef struct gxLUVColor gxLUVColor;
		typedef struct gxLABColor gxLABColor;
		typedef struct gxYIQColor gxYIQColor;
		typedef struct gxGrayAColor gxGrayAColor;
		typedef struct gxIndexedColor gxIndexedColor;
		typedef struct gxColor gxColor;
	#endif
	
/* gxColorSet structures */

	union gxSetColor {
		gxCMYKColor	cmyk;
		gxRGBColor	rgb;
		gxRGBAColor	rgba;
		gxHSVColor	hsv;
		gxHLSColor	hls;
		gxXYZColor	xyz;
		gxYXYColor	yxy;
		gxLUVColor	luv;
		gxLABColor	lab;
		gxYIQColor	yiq;
		gxColorValue	gray;
		gxGrayAColor  graya;
		unsigned short  pixel16;
		unsigned long	pixel32;
		gxColorValue	component[4];
	};

	#ifndef __cplusplus
		typedef union gxSetColor gxSetColor;
	#endif

/* gxTransform enumerations */

	enum gxShapeParts {				/* parts of a gxShape considered in hit testing: */
		gxNoPart			= 0,			/* (in order of evaluation) */
		gxBoundsPart		= 0x0001,
		gxGeometryPart	= 0x0002,
		gxPenPart			= 0x0004,
		gxCornerPointPart	= 0x0008,
		gxControlPointPart	= 0x0010,
		gxEdgePart		= 0x0020,
		gxJoinPart		= 0x0040,
		gxStartCapPart		= 0x0080,
		gxEndCapPart		= 0x0100,
		gxDashPart		= 0x0200,
		gxPatternPart		= 0x0400,
		gxGlyphBoundsPart	= gxJoinPart,
		gxGlyphFirstPart	= gxStartCapPart,
		gxGlyphLastPart	= gxEndCapPart,
		gxSideBearingPart	= gxDashPart,
		gxAnyPart		= gxBoundsPart | gxGeometryPart | gxPenPart | gxCornerPointPart | gxControlPointPart |
							gxEdgePart | gxJoinPart | gxStartCapPart | gxEndCapPart | gxDashPart | gxPatternPart
	};

	typedef long gxShapePart;

/* gxTransform structures */

	struct gxHitTestInfo {
		gxShapePart	what;			/* which part of gxShape */
		long			index;			/* control gxPoint index */
		Fixed		distance;			/* how far from gxPoint or outside of area click was */

		/* these fields are only set by GXHitTestPicture */
		gxShape		which;
		gxShape		containerPicture;	/* picture which contains gxShape hit */
		long			containerIndex;		/* the index within that picture  */
		long			totalIndex;		/* the total index within the root picture */
	};

	#ifndef __cplusplus
		typedef struct gxHitTestInfo gxHitTestInfo;
	#endif
	
/* gxViewPort enumerations */

	enum gxPortAttributes {
		gxGrayPort			= 0x0001,
		gxAlwaysGridPort		= 0x0002,
		gxEnableMatchPort		= 0x0004
	};

	typedef long gxPortAttribute;

/* gxViewDevice enumerations */

	enum gxDeviceAttributes {
		gxDirectDevice			= 0x01,		/* for the device gxBitmap baseAddr pointer */
		gxRemoteDevice		= 0x02,
		gxInactiveDevice		= 0x04
	};

	typedef long gxDeviceAttribute;

	enum gxDotTypes {
		gxRoundDot = 1,
		gxSpiralDot,
		gxSquareDot,
		gxLineDot,
		gxEllipticDot,
		gxTriangleDot,
		gxDispersedDot
	};

	typedef long gxDotType;

/* gxViewPort structures */

	enum gxTintTypes {
		gxNoTint,
		gxLuminanceTint,		/* use the luminance of the gxColor */
		gxAverageTint,			/* add all the components and divide by the number of components */
		gxMixtureTint,			/* find the closest gxColor on the axis between the foreground and background */
		gxComponent1Tint,		/* use the value of the first component of the gxColor */
		gxComponent2Tint,		/* ... etc. */
		gxComponent3Tint,
		gxComponent4Tint
	};

	typedef long gxTintType;

	struct gxHalftone {
		Fixed		angle;
		Fixed		frequency;
		gxDotType	method;
		gxTintType	tinting;
		gxColor		dotColor;
		gxColor		backgroundColor;
		gxColorSpace	tintSpace;
	};

	#ifndef __cplusplus
		typedef struct gxHalftone gxHalftone;
	#endif
	
/* gxViewGroup enumerations */

	#define gxAllViewDevices			((gxViewGroup) 0)
	#define gxScreenViewDevices		((gxViewGroup) 1)

/* graphics stream constants and structures */

	enum gxSpoolCommands {
		gxOpenReadSpool	= 1,
		gxOpenWriteSpool,
		gxReadSpool,
		gxWriteSpool,
		gxCloseSpool
	};
	
	typedef long gxSpoolCommand;
	
	typedef unsigned char gxGraphicsOpcode;

	#ifdef __cplusplus
	extern "C" {
	#endif

	#pragma procname gxSpool
	typedef long (*gxSpoolProcPtr)(gxSpoolCommand command, struct gxSpoolBlock *block);

	struct gxSpoolBlock {
	/* these fields are read only */
		gxSpoolProcPtr		spoolProcedure;	
		void				*buffer;			/* source/destination pointer to data */
		long				bufferSize;		/* how many bytes for the system to read (flatten) / write (unflatten) */
	/* these fields are written to (but are not read from) */
		long				count;			/* how many bytes for the caller to read (unflatten) /write (flatten) */
		long				operationSize;		/* operation size (including operand byte) */
		long				operationOffset;	/* the data offset, if any, within the current operation */
		gxGraphicsOpcode	lastTypeOpcode; 	/* type of last created object */
		gxGraphicsOpcode	currentOperation;	/* operation emitted by flatten, or intrepreted by last unflatten */
		gxGraphicsOpcode	currentOperand;	/* e.g., gxTransformTypeOpcode, gxInkTagOpcode */
		unsigned char		compressed;		/* one of: gxTwoBitCompressionValues */
	};

	#ifdef __cplusplus
	}
	#endif

	#ifndef __cplusplus
		typedef struct gxSpoolBlock gxSpoolBlock;
	#endif
	
	enum gxFlattenFlags {
		gxFontListFlatten		= 0x01,	/* if set, generate a gxTag containing list of each gxFont referenced */
		gxFontGlyphsFlatten		= 0x02,	/* if set, generate a gxTag containing the list of glyphs referenced inside the gxFont */
		gxFontVariationsFlatten	= 0x04,	/* if set, append the gxTag with separate [variation] coordinates */
		gxBitmapAliasFlatten	= 0x08	/* if set, open bitmap alias files and flatten out their image data */
	};
	
	typedef long gxFlattenFlag;

/* gxGraphicsClient constants */

	#define gxGraphicsSystemClient		(gxGraphicsClient)(-1)

	enum gxClientAttributes {
		gxStaticHeapClient		= 0x0001
	};

	typedef long gxClientAttribute;

/* graphics patching constants */

	#define gxOriginalGraphicsFunction		-1
	#define gxOriginalGraphicsIdentifier		'grfx'
	
	struct gxBitmapDataSourceAlias {
		unsigned long			fileOffset;			/* file offset (in bytes) of top-left pixel */
		unsigned long			aliasRecordSize;	/* size of alias record below */
		unsigned char			aliasRecord[gxAnyNumber];	/* the actual alias record data */
	};
	
	#define	gxBitmapFileAliasTagType		'bfil'
	#define	gxBitmapFileAliasImageValue	0x00000001

#endif /* of graphic types includes */
