/* graphics libraries:  
    offscreen support library interfaces
    by Cary Clark, Michael Fairman, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink
    Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.
*/

#pragma once
#ifndef offscreenLibraryIncludes
    #define offscreenLibraryIncludes
    
    #ifndef graphicsTypesIncludes
        #include "graphics types.h"
    #endif

	#ifdef __cplusplus
	extern "C" {
	#endif

    struct offscreen {
        gxShape		draw;	/* a gxBitmap gxShape which, when drawn, will transfer the offscreen to the display */
        gxTransform	xform;	/* a gxTransform that will cause shapes owning it to draw offscreen. */
        gxViewDevice	device;	/* the offscreen gxViewDevice whose gxColorSpace, etc. you may change */
        gxViewPort	port;		/* the offscreen gxViewPort which may be placed in any gxTransform's gxViewPort list */
        gxViewGroup	group;	/* the global space in which the gxViewPort and gxViewDevice exist */
    };
    
    typedef struct viewPortBufferRecord **viewPortBuffer;

	#ifndef __cplusplus
		typedef struct offscreen offscreen;
	#endif
	
    /* (see the .c file for routine explanations) */
    void CreateOffscreen(offscreen *target, gxShape bitmapShape);
    void DisposeOffscreen(offscreen *target);
    void CopyToBitmaps(gxShape target, gxShape source);

    viewPortBuffer NewViewPortBuffer(gxViewPort originalPort);
    void DisposeViewPortBuffer(viewPortBuffer target);
    boolean ValidViewPortBuffer(viewPortBuffer target);
    boolean UpdateViewPortBuffer(viewPortBuffer target);
    gxViewPort GetViewPortBufferViewPort(viewPortBuffer source);
    gxShape GetViewPortBufferShape(viewPortBuffer source);

	#ifdef __cplusplus
	}
	#endif
#endif
