/* graphics libraries:
	layout library interface: Routines to manipulate Selection objects
  by Dave Opstad, Eric Mader
	Copyright 1991 - 1993 Apple Computer, Inc.  All rights reserved. */

#pragma once
#ifndef selectionLibraryIncludes
	#define selectionLibraryIncludes
	
	#ifndef layoutTypesIncludes
	#include "layout types.h"
	#endif
	
	#ifndef graphicsTypesIncludes
	#include "graphics types.h"
	#endif
	
	#ifdef __cplusplus
	extern "C" {
	#endif
	
	/* Constants and Enumerations */
	
	#define selectionExtremeEdge -1
	
	enum SelectionTypes {
	  emptySelection,
	  simpleCaret,
	  simpleRange,
	  discontiguousRange
	};
	typedef unsigned short SelectionType;
	
	enum SelectionMatches {
	  notInSelection,
	  partlyInSelection,
	  fullyInSelection,
	  equalToSelection
	};
	typedef unsigned short SelectionMatch;
	
	/* Types and Structures */
	
	typedef long SelectionOffset;
	
	struct SelectionOffsetRange {
	  SelectionOffset minOffset;  /* earliest char */
	  SelectionOffset maxOffset;  /* latest char */
	};
	
	struct SelectionRanges {
	  long													rangeCount;
	  struct SelectionOffsetRange		ranges[gxAnyNumber];
	};
	
	struct CaretPiece {
	  SelectionOffset								offset;
	  short													leadingEdge;
	};
	
	struct Selection {
	 	SelectionType									type;
	  union {
	    struct CaretPiece						caret;
	    struct SelectionRanges			range;
		} data;
	};
	
	typedef struct Selection* SelectionPtr, ** SelectionHandle;
	
	#ifndef __cplusplus
		typedef struct SelectionOffsetRange SelectionOffsetRange;
		typedef struct SelectionRanges SelectionRanges;
		typedef struct CaretPiece CaretPiece;
		typedef struct Selection Selection;
	#endif
	
	/* Routines */
	
	SelectionHandle NewEmptySelection(void);
	SelectionHandle NewCaretSelection(SelectionOffset offset, long leadingEdge);
	SelectionHandle NewRangeSelection(SelectionOffsetRange *range);
	
	SelectionHandle NewFullSelection(void);
	SelectionHandle NewStartSelection(SelectionOffset toOffset);
	SelectionHandle NewEndSelection(SelectionOffset fromOffset);
	
	void SetEmptySelection(SelectionHandle selection);
	void SetCaretSelection(SelectionHandle selection, SelectionOffset offset, long leadingEdge);
	void SetRangeSelection(SelectionHandle selection, SelectionOffsetRange *range);
	
	void SetFullSelection(SelectionHandle selection);
	void SetStartSelection(SelectionHandle selection, SelectionOffset toOffset);
	void SetEndSelection(SelectionHandle selection, SelectionOffset fromOffset);
	
	void DisposeSelection(SelectionHandle selection);
	
	SelectionType GetSelectionType(SelectionHandle selection);
	
	SelectionOffset GetCaretSelection(SelectionHandle selection, boolean *leadingEdge);
	long GetRangeSelection(SelectionHandle selection, SelectionRanges *selectionRanges);
	
	SelectionMatch RangeInSelection(SelectionHandle selection, SelectionOffsetRange *range);
	
	void InvertSelection(SelectionHandle dest);
	void ShiftSelection(SelectionHandle dest, SelectionOffset delta);
	
	void UnionSelection(SelectionHandle dest, SelectionHandle source);
	void SectSelection(SelectionHandle dest, SelectionHandle source);
	void XorSelection(SelectionHandle dest, SelectionHandle source);
	void DiffSelection(SelectionHandle dest, SelectionHandle source);
	
	gxShape GetLayoutSelection(
	  gxShape           layout,
	  SelectionHandle selection,
	  SelectionOffset lineStart,
	  gxHighlightType   highlightType,
	  gxCaretType       caretType);
	
	SelectionHandle NewDiscontiguousSelection(SelectionRanges *ranges);
	
	#ifdef __cplusplus
	}
	#endif
#endif
