/* graphics:
	font file structures
	by Cary Clark, Georgiann Delaney, Herb Derby, Michael Fairman, Pablo Fernicola, Dave Good, Josh Horwich, Barton House, Robert Johnson, Keith McGreggor, Mike Reed, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1994 Apple Computer, Inc.  All rights reserved.	*/

#pragma once

#ifndef sfntTypesIncludes
	#define sfntTypesIncludes
	
	#ifndef mathTypesIncludes
		#include "math types.h"
	#endif
	
	#ifndef gxAnyNumber
		#define gxAnyNumber 1
	#endif
	
	#ifndef gxSharedFontTypeDefines
		
		typedef long gxFontTableTag;
		typedef long gxFontVariationTag;
		typedef long gxFontFormatTag;
		typedef long gxFontStorageTag;
		typedef gxFontVariationTag gxFontDescriptorTag;
	
		struct gxFontVariation {
			gxFontVariationTag		name;
			Fixed				value;
		};
		typedef struct gxFontVariation gxFontDescriptor;
	
		struct gxFontFeatureSetting {
			unsigned short			setting;
			unsigned short			nameID;
		};
	#endif
	
	struct sfntDirectoryEntry {
		gxFontTableTag				tableTag;
		unsigned long				checkSum;
		unsigned long				offset;
		unsigned long				length;
	};
	
	/* The search fields limits numOffsets to 4096. */
	
	struct sfntDirectory {
		gxFontFormatTag			format;
		unsigned short				numOffsets;			/* number of tables */
		unsigned short				searchRange;			/* (max2 <= numOffsets)*16 */
		unsigned short				entrySelector;			/* log2(max2 <= numOffsets) */
		unsigned short				rangeShift;			/* numOffsets*16-searchRange*/
		struct sfntDirectoryEntry		table[gxAnyNumber];	/* table[numOffsets] */
	};
	
	#define sizeof_sfntDirectory		12
	
	/* Cmap - character id to glyph id gxMapping */
	
	#define cmapFontTableTag			0x636d6170			/* 'cmap' */
	
	struct sfntCMapSubHeader {
		unsigned short				format;
		unsigned short				length;
		unsigned short				languageID;			/* base-1 */
	};
	
	#define sizeof_sfntCMapSubHeader	6
	
	struct sfntCMapEncoding {
		unsigned short				platformID;			/* base-0 */
		unsigned short				scriptID;				/* base-0 */
		unsigned long				offset;
	};
	
	#define sizeof_sfntCMapEncoding 8
	
	struct sfntCMapHeader {
		unsigned short				version;
		unsigned short				numTables;
		struct sfntCMapEncoding		encoding[gxAnyNumber];
	};
	
	#define sizeof_sfntCMapHeader		4
	
	/* Name table */
	
	#define nameFontTableTag			0x6e616d65			/* 'name' */
	
	struct sfntNameRecord {
		unsigned short				platformID;			/* base-0 */
		unsigned short				scriptID;				/* base-0 */
		unsigned short				languageID;			/* base-0 */
		unsigned short				nameID;				/* base-0 */
		unsigned short				length;
		unsigned short				offset;
	};
	
	#define sizeof_sfntNameRecord		12
	
	struct sfntNameHeader {
		unsigned short				format;
		unsigned short				count;
		unsigned short				stringOffset;
		struct sfntNameRecord		record[gxAnyNumber];
	};
	
	#define sizeof_sfntNameHeader	6
	
	
	/* Fvar table - gxFont variations */
	
	#define variationFontTableTag		0x66766172			/* 'fvar' */
	
	/* These define each gxFont variation */
	
	struct sfntVariationAxis {
		gxFontVariationTag			axisTag;
		Fixed					minValue;
		Fixed					defaultValue;
		Fixed					maxValue;
		short					flags;
		short					nameID;
	};
	
	#define sizeof_sfntVariationAxis	20
	
	/* These are named locations in gxStyle-space for the user */
	
	struct sfntInstance {
		short					nameID;
		short					flags;
		Fixed						coord[gxAnyNumber];	/* [axisCount] */
		/* room to grow since the header carries a tupleSize field */
	};
	
	#define sizeof_sfntInstance		4
	
	struct sfntVariationHeader {
		Fixed					version;				/* 1.0 Fixed */
		unsigned short				offsetToData;			/* to first axis = 16*/
		unsigned short				countSizePairs;			/* axis+inst = 2 */
		unsigned short				axisCount, axisSize;
		unsigned short				instanceCount;
		unsigned short				instanceSize;
		/* other <count,size> pairs */
		struct sfntVariationAxis		axis[gxAnyNumber];		/* [axisCount] */
		struct sfntInstance			instance[gxAnyNumber];	/* [instanceCount] */
		/* other arrays of data */
	};
	
	#define sizeof_sfntVariationHeader	16
	
	/* Fdsc table - gxFont descriptor */
	
	#define descriptorFontTableTag		0x66647363			/* 'fdsc' */
	
	struct sfntDescriptorHeader {
		Fixed					version;				/* 1.0 in Fixed */
		long						descriptorCount;
		gxFontDescriptor			descriptor[gxAnyNumber];
	};
	
	#define sizeof_sfntDescriptorHeader	8
	
	/* Feat Table - layout feature table */
	
	#define featureFontTableTag		0x66656174				/* 'feat' */
	
	struct sfntFeatureName {
		unsigned short				featureType;
		unsigned short				settingCount;
		long						offsetToSettings;
		unsigned short				featureFlags;
		unsigned short				nameID;
	};
	
	struct sfntFontRunFeature {
		unsigned short			featureType;
		unsigned short			setting;
	};

	struct sfntFeatureHeader {
		long						version;				/* 1.0 */
		unsigned short				featureNameCount;
		unsigned short				featureSetCount;
		long						reserved;				/* set to 0 */
		struct sfntFeatureName		names[gxAnyNumber];
		struct gxFontFeatureSetting	settings[gxAnyNumber];
		struct sfntFontRunFeature		runs[gxAnyNumber];
	};
	
	/* OS/2 Table */
	
	#define os2FontTableTag			0x4f532f32			/* 'OS/2' */
	
	/*  Special invalid glyph ID value, useful as a sentinel value, for example */
	#define nonGlyphID				(65535)
	
	/*** need to put os/2 structure definition here */

	#ifndef __cplusplus
		#ifndef gxSharedFontTypeDefines
			typedef struct gxFontVariation gxFontVariation;
			typedef struct gxFontFeatureSetting gxFontFeatureSetting;
		#endif
		typedef struct sfntDirectoryEntry sfntDirectoryEntry;
		typedef struct sfntDirectory sfntDirectory;
		typedef struct sfntCMapSubHeader sfntCMapSubHeader;
		typedef struct sfntCMapEncoding sfntCMapEncoding;
		typedef struct sfntCMapHeader sfntCMapHeader;
		typedef struct sfntNameRecord sfntNameRecord;
		typedef struct sfntNameHeader sfntNameHeader;
		typedef struct sfntVariationAxis sfntVariationAxis;
		typedef struct sfntInstance sfntInstance;
		typedef struct sfntVariationHeader sfntVariationHeader;
		typedef struct sfntDescriptorHeader sfntDescriptorHeader;
		typedef struct sfntFeatureName sfntFeatureName;
		typedef struct sfntFeatureHeader sfntFeatureHeader;
	#endif
	
	#ifndef gxSharedFontTypeDefines
		#define gxSharedFontTypeDefines
	#endif
#endif
