/* graphics:
	data structures and types for graphics synonyms stored in tags
	by Cary Clark, Georgiann Delaney, Herb Derby, Michael Fairman, Pablo Fernicola, Dave Good, Josh Horwich, Barton House, Robert Johnson, Dan Lipton, Keith McGreggor, Mike Reed, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1994 Apple Computer, Inc.  All rights reserved.	*/

#pragma once

#ifndef gxSynonymTags
	#define gxSynonymTags

	#ifndef mathTypesIncludes
		#include "math types.h"
	#endif

	#ifndef gxAnyNumber
		#define gxAnyNumber 1
	#endif

	#ifndef graphicsTypesIncludes
		#include "graphics types.h"
	#endif

	#ifndef fontTypesIncludes
		#include "font types.h"
	#endif


	#define gxPostScriptTag		0x706f7374	/* 'post' */

	#define gxPostControlTag		0x70736374   /* 'psct' */

	enum gxPsStateFlags {
		gxNoSave = 1				/* don't do save-restore around PS data */
	};

	struct gxPostControl {
		long		flags;			/* PostScript state flags */
	};
	
	typedef struct gxPostControl gxPostControl;

	#define gxDashSynonymTag 0x73647368   	/* 'sdsh' */

	struct gxDashSynonym{
		long		size;						/* number of elements in array */
		Fixed	dashLength[gxAnyNumber];	/* Array of dash lengths */
	};
	
	typedef struct gxDashSynonym gxDashSynonym;

	#define gxLineCapSynonymTag 0x6c636170	/* 'lcap' */

	enum gxLineCaps {
		gxButtCap,
		gxRoundCap,
		gxSquareCap,
		gxTriangleCap
	};

	typedef long gxLineCapSynonym;			/* gxLine cap type */

	#define gxCubicSynonymTag 0x63756278  	/* 'cubx' */
	
	enum gxCubicSynonym {
	
		gxIgnoreFlag 	=	0x0000,	/* Ignore this word, get next one */
		gxLineToFlag 	=	0x0001,	/* Draw a gxLine to gxPoint following this flag */
		gxCurveToFlag 	=   	0x0002,	/* Draw a gxCurve through the 3 points following this flag */
		gxMoveToFlag 	=	0x0003,	/* Start a new contour at the gxPoint following this flag */
		gxClosePathFlag = 	0x0004	/* Close the contour */
		
	};
	
	typedef enum gxCubicSynonym gxCubicSynonym;
	
	#define gxCubicInstructionMask 0x000F	/* low four bits are gxPoint instructions */
	
	typedef short gxCubicSynonymFlags;		/* Low four bits are instruction (moveto, lineto, curveto, closepath) */

	#define gxPatternSynonymTag 0x7074726E	/* 'ptrn' */

	enum gxPatterns {
		gxHatch,
		gxCrossHatch
	};

	struct gxPatternSynonym{
		long		patternType;		/* one of the gxPatterns: gxHatch or gxCrossHatch */
		Fixed	angle;			/* angle at which pattern is drawn */
		Fixed	spacing;			/* distance between two parallel pattern lines */
		Fixed	thickness;			/* thickness of the pattern */
		gxPoint	anchorPoint;		/* gxPoint with with respect to which pattern position is calculated */
	};
	
	typedef struct gxPatternSynonym gxPatternSynonym;

#endif
