
/************************************************************

 ISDNToolIntf.h
 C Interface to the Apple ISDN Telephone Tool


  Copyright Apple Computer, Inc. 1990-1993
  All rights reserved

************************************************************/


/* Record and Playback parameter block for TELOtherFunction */

#ifndef __ISDNTOOLINTF__
#define __ISDNTOOLINTF__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __TELEPHONES__
#include <Telephones.h>
#endif


typedef pascal void (*RPCompletionProc)(struct TELISDNOtherPB *pb);

typedef struct TELISDNOtherPB {
	short				csCode;
	Ptr					bufferPtr;
	long				bufferLength;
	Ptr					netRecordBufferPtr;
	long				netRecordBufferLength;
	TELCAHandle			hTELCA;
	short				async;
	RPCompletionProc	completionRtn;
	TELErr				result;
	short				reserved0;
	short				reserved1;
	long				reserved2;
	long				reserved3;
} TELISDNOtherPB;


enum
{

/* CSCodes */
	telISDNLocalRecord			= 1,
	telISDNLocalPlayback		= 2,
	telISDNNetworkRecord		= 3,
	telISDNNetworkPlayback		= 4,
	telISDNLocalStop			= 5,
	telISDNNetworkStop			= 6,
	telISDNBeginRPSession		= 7,
	telISDNEndRPSession			= 8,

/* Result Codes */
	isdnRPnoError				= 0,		/* no error during RP */
	isdnRPLocalDisconnect		= 1,		/* local user disconnected during RP */
	isdnRPRemoteDisconnect		= 2,		/* remote user disconnected during RP */
	isdnRPLocalStop				= 3,		/* local side stopped local record/playback via TELISDNLocalStop*/
	isdnRPNetworkStop			= 4,		/* local side stopped network record/playback via TELISDNNetworkStop */
	isdnRPLocalInterruption		= 5,		/* local side interrupted network rec/play by picking up phone */
	isdnRPBusy					= 6,		/* session is in use */
	isdnRPNoLocalSession		= 7,		/* local stop didn't work because no local session was in use */
	isdnRPNoNetworkSession		= 8,		/* network stop didn't work because no network session was in use */

/* Error Codes */
	isdnError					= (-1001),	/* some error resulted in stopping of playback */
	isdnBadBufferLength			= (-1002),	/* bad buffer length specified in bufflen */
	isdnBadNetBufferLength		= (-1003),	/* bad buffer length specified in netRecordBufferLength */
	isdnBadBufferSpecified		= (-1004),	/* bad buffer specified in buffPtr */
	isdnBadNetBufferSpecified	= (-1005),	/* bad buffer specified in netRecordBufferPtr */
	isdnBadcsCode				= (-1006),	/* bad csCode specified */
	isdnCannotLoadLocalRPTask	= (-1007),	/* could not load local Rec/Play task to card */
	isdnCannotLoadNetworkRPTask = (-1008),	/* could not load network Rec/Play task to card */
	isdnCardNotRunning			= (-1009),	/* ISDN card is not running */
	isdnCANotInValidState		= (-1010),	/* CA is not in valid state (active or alerting) for network R/P */

/* Error Codes that can be returned in termOtherMsg */
	isdnFailure					= (-51),	/* board response bad or missing		*/
	isdnBadData					= (-52),	/* invalid message data	sent by tool	*/
	isdnNoMemory				= (-53),	/* no buffers available	for tool to card communication	*/
	isdnNoCode					= (-55),	/* isdn card not downloaded				*/
	isdnSmallBuffer				= (-57),	/* read buffer too small				*/
	isdnLostVFIMsg				= (-58),	/* VFI queue overflow					*/
	isdnBadSlot					= (-59),	/* selected slot has no card			*/
	isdnRestart					= (-60),	/* isdn code restarted					*/
	isdnBusyVFI					= (-61),	/* R/P or NR/P in progress				*/
	isdnBadVFICmd				= (-62),	/* unrecognized VFI command type		*/
	isdnReqExcl					= (-63),	/* VFI req has exclusive parms			*/
	isdnReqUnavailable			= (-64),	/* VFI req unavailable					*/
	isdnReqState				= (-65),	/* VFI req can't be executed now		*/
	isdnBadProc					= (-66)		/* bad receive notify request			*/

};


#endif